@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2015, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,20 +27,18 @@
 #include <time.h>
 #include <iostream>
 #include <locale>
-#include <memory>
 
 using namespace std;
 
 ostream &operator<<(ostream &os, const Datetime &) {
   const char format[] = "%Y-%m-%d %X";
-  const time_t t(time(nullptr));
-  const tm tm(*localtime(&t));
-
-  const size_t date_length{50};
-  const std::unique_ptr<char[]> date{new char[date_length]};
-  strftime(date.get(), date_length, format, &tm);
-
-  os << date.get() << " ";
+  time_t t(time(nullptr));
+  tm tm(*localtime(&t));
+  std::locale loc(cout.getloc());
+  ostringstream sout;
+  const std::time_put<char> &tput = std::use_facet<std::time_put<char>>(loc);
+  tput.put(sout.rdbuf(), sout, '\0', &tm, &format[0], &format[11]);
+  os << sout.str() << " ";
   return os;
 }
 
@@ -50,7 +47,7 @@ ostream &operator<<(ostream &os, const Gen_spaces &gen) {
 }
 
 int Log::Log_buff::sync() {
-  const string sout(str());
+  string sout(str());
   if (m_enabled && sout.length() > 0) {
     m_os << Datetime() << "[" << m_logc << "]"
          << Gen_spaces(8 - m_logc.length()) << sout;
