@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2012, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2012, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -39,7 +38,7 @@
 #include <stdlib.h>
 #include <sys/types.h>
 
-#include "client/include/client_priv.h"
+#include "client/client_priv.h"
 #include "my_aes.h"
 #include "my_byteorder.h"
 #include "my_compiler.h"
@@ -51,7 +50,6 @@
 #include "my_rnd.h"
 #include "mysql/service_mysql_alloc.h"
 #include "mysys/my_default_priv.h"
-#include "nulls.h"
 #include "print_version.h"
 #include "welcome_copyright_notice.h"
 
@@ -135,9 +133,9 @@ struct my_command_data {
 
 /* mysql_config_editor utility options. */
 static struct my_option my_program_long_options[] = {
-#ifdef NDEBUG
-    {"debug", '#', "This is a non-debug version. Catch this and exit.", nullptr,
-     nullptr, nullptr, GET_DISABLED, OPT_ARG, 0, 0, 0, nullptr, 0, nullptr},
+#ifdef DBUG_OFF
+    {"debug", '#', "This is a non-debug version. Catch this and exit.", 0, 0, 0,
+     GET_DISABLED, OPT_ARG, 0, 0, 0, 0, 0, 0},
 #else
     {"debug", '#', "Output debug log. Often this is 'd:t:o,filename'.", nullptr,
      nullptr, nullptr, GET_STR, OPT_ARG, 0, 0, 0, nullptr, 0, nullptr},
@@ -239,10 +237,9 @@ static struct my_option my_help_command_options[] = {
      0, nullptr, 0, nullptr}};
 
 extern "C" {
-static bool my_program_get_one_option(int optid,
-                                      const struct my_option *opt
-                                      [[maybe_unused]],
-                                      char *argument [[maybe_unused]]) {
+static bool my_program_get_one_option(
+    int optid, const struct my_option *opt MY_ATTRIBUTE((unused)),
+    char *argument MY_ATTRIBUTE((unused))) {
   switch (optid) {
     case '#':
       DBUG_PUSH(argument ? argument : "d:t:o,/tmp/mysql_config_editor.trace");
@@ -499,8 +496,8 @@ static int set_command(void) {
 
   DYNAMIC_STRING file_buf, path_buf;
 
-  init_dynamic_string(&path_buf, "", MY_LINE_MAX);
-  init_dynamic_string(&file_buf, "", file_size);
+  init_dynamic_string(&path_buf, "", MY_LINE_MAX, MY_LINE_MAX);
+  init_dynamic_string(&file_buf, "", file_size, 3 * MY_LINE_MAX);
 
   if (tty_password) opt_password = get_tty_password(NullS);
 
@@ -518,24 +515,24 @@ static int set_command(void) {
   if (opt_user) /* --user */
   {
     dynstr_append(&path_buf, "\nuser = ");
-    dynstr_append_quoted(&path_buf, "\"", 1, opt_user, NullS);
+    dynstr_append(&path_buf, opt_user);
   }
 
   if (opt_password) /* --password */
   {
     dynstr_append(&path_buf, "\npassword = ");
-    dynstr_append_quoted(&path_buf, "\"", 1, opt_password, NullS);
+    dynstr_append(&path_buf, opt_password);
   }
 
   if (opt_host) /* --host */
   {
     dynstr_append(&path_buf, "\nhost = ");
-    dynstr_append_quoted(&path_buf, "\"", 1, opt_host, NullS);
+    dynstr_append(&path_buf, opt_host);
   }
 
   if (opt_socket) {
     dynstr_append(&path_buf, "\nsocket = ");
-    dynstr_append_quoted(&path_buf, "\"", 1, opt_socket, NullS);
+    dynstr_append(&path_buf, opt_socket);
   }
 
   if (opt_port) {
@@ -582,8 +579,8 @@ static int remove_command(void) {
 
   DYNAMIC_STRING file_buf, path_buf;
 
-  init_dynamic_string(&path_buf, "", MY_LINE_MAX);
-  init_dynamic_string(&file_buf, "", file_size);
+  init_dynamic_string(&path_buf, "", MY_LINE_MAX, MY_LINE_MAX);
+  init_dynamic_string(&file_buf, "", file_size, 3 * MY_LINE_MAX);
 
   if (file_size) {
     if (read_and_decrypt_file(&file_buf) == -1) goto error;
@@ -629,7 +626,7 @@ static int print_command(void) {
   DBUG_TRACE;
   DYNAMIC_STRING file_buf;
 
-  init_dynamic_string(&file_buf, "", file_size);
+  init_dynamic_string(&file_buf, "", file_size, 3 * MY_LINE_MAX);
 
   if (file_size) {
     if (read_and_decrypt_file(&file_buf) == -1) goto error;
@@ -1005,7 +1002,7 @@ static char *locate_login_path(DYNAMIC_STRING *file_buf,
   char *addr = nullptr;
   DYNAMIC_STRING dy_path_name;
 
-  init_dynamic_string(&dy_path_name, "", 512);
+  init_dynamic_string(&dy_path_name, "", 512, 512);
 
   dynstr_append(&dy_path_name, "\n[");
   dynstr_append(&dy_path_name, path_name);
