@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2000, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -33,9 +32,8 @@
 #include <sys/types.h>
 #include <time.h>
 
-#include "client/include/client_priv.h"
+#include "client/client_priv.h"
 #include "compression.h"
-#include "m_string.h"
 #include "my_alloc.h"
 #include "my_dbug.h"
 #include "my_default.h"
@@ -44,11 +42,8 @@
 #include "my_macros.h"
 #include "my_systime.h"
 #include "mysql/service_mysql_alloc.h"
-#include "mysql/strings/int2str.h"
 #include "mysql_version.h"
-#include "nulls.h"
 #include "print_version.h"
-#include "strxmov.h"
 #include "thr_cond.h"
 #include "thr_mutex.h"
 #include "typelib.h"
@@ -60,21 +55,23 @@ native_mutex_t init_mutex;
 native_mutex_t counter_mutex;
 native_cond_t count_threshold;
 
-static int db_error_with_table(MYSQL *mysql, char *table);
-static int db_error(MYSQL *mysql);
+static void db_error_with_table(MYSQL *mysql, char *table);
+static void db_error(MYSQL *mysql);
 static char *field_escape(char *to, const char *from, uint length);
 static char *add_load_option(char *ptr, const char *object,
                              const char *statement);
 
 static bool verbose = false, lock_tables = false, ignore_errors = false,
             opt_delete = false, replace = false, silent = false, ignore = false,
-            opt_compress = false, opt_low_priority = false;
+            opt_compress = false, opt_low_priority = false,
+            tty_password = false;
 static bool debug_info_flag = false, debug_check_flag = false;
 static uint opt_use_threads = 0, opt_local_file = 0, my_end_arg = 0;
-static char *current_user = nullptr, *current_host = nullptr,
-            *current_db = nullptr, *fields_terminated = nullptr,
-            *lines_terminated = nullptr, *enclosed = nullptr,
-            *opt_enclosed = nullptr, *escaped = nullptr, *opt_columns = nullptr;
+static char *opt_password = nullptr, *current_user = nullptr,
+            *current_host = nullptr, *current_db = nullptr,
+            *fields_terminated = nullptr, *lines_terminated = nullptr,
+            *enclosed = nullptr, *opt_enclosed = nullptr, *escaped = nullptr,
+            *opt_columns = nullptr;
 static const char *default_charset = MYSQL_AUTODETECT_CHARSET_NAME;
 static uint opt_enable_cleartext_plugin = 0;
 static bool using_opt_enable_cleartext_plugin = false;
@@ -86,11 +83,11 @@ static longlong opt_ignore_lines = -1;
 static uint opt_zstd_compress_level = default_zstd_compression_level;
 static char *opt_compress_algorithm = nullptr;
 
-#include "client/include/caching_sha2_passwordopt-vars.h"
-#include "client/include/multi_factor_passwordopt-vars.h"
-#include "client/include/sslopt-vars.h"
+#include "caching_sha2_passwordopt-vars.h"
+#include "sslopt-vars.h"
+
 #if defined(_WIN32)
-static char *shared_memory_base_name = nullptr;
+static char *shared_memory_base_name = 0;
 #endif
 
 static struct my_option my_long_options[] = {
@@ -112,15 +109,15 @@ static struct my_option my_long_options[] = {
     {"compress", 'C', "Use compression in server/client protocol.",
      &opt_compress, &opt_compress, nullptr, GET_BOOL, NO_ARG, 0, 0, 0, nullptr,
      0, nullptr},
-#ifdef NDEBUG
-    {"debug", '#', "This is a non-debug version. Catch this and exit.", nullptr,
-     nullptr, nullptr, GET_DISABLED, OPT_ARG, 0, 0, 0, nullptr, 0, nullptr},
+#ifdef DBUG_OFF
+    {"debug", '#', "This is a non-debug version. Catch this and exit.", 0, 0, 0,
+     GET_DISABLED, OPT_ARG, 0, 0, 0, 0, 0, 0},
     {"debug-check", OPT_DEBUG_CHECK,
-     "This is a non-debug version. Catch this and exit.", nullptr, nullptr,
-     nullptr, GET_DISABLED, NO_ARG, 0, 0, 0, nullptr, 0, nullptr},
+     "This is a non-debug version. Catch this and exit.", 0, 0, 0, GET_DISABLED,
+     NO_ARG, 0, 0, 0, 0, 0, 0},
     {"debug-info", OPT_DEBUG_INFO,
-     "This is a non-debug version. Catch this and exit.", nullptr, nullptr,
-     nullptr, GET_DISABLED, NO_ARG, 0, 0, 0, nullptr, 0, nullptr},
+     "This is a non-debug version. Catch this and exit.", 0, 0, 0, GET_DISABLED,
+     NO_ARG, 0, 0, 0, 0, 0, 0},
 #else
     {"debug", '#', "Output debug log. Often this is 'd:t:o,filename'.", nullptr,
      nullptr, nullptr, GET_STR, OPT_ARG, 0, 0, 0, nullptr, 0, nullptr},
@@ -181,10 +178,14 @@ static struct my_option my_long_options[] = {
      "Use LOW_PRIORITY when updating the table.", &opt_low_priority,
      &opt_low_priority, nullptr, GET_BOOL, NO_ARG, 0, 0, 0, nullptr, 0,
      nullptr},
-#include "client/include/multi_factor_passwordopt-longopts.h"
+    {"password", 'p',
+     "Password to use when connecting to server. If password is not given it's "
+     "asked from the tty.",
+     nullptr, nullptr, nullptr, GET_PASSWORD, OPT_ARG, 0, 0, 0, nullptr, 0,
+     nullptr},
 #ifdef _WIN32
-    {"pipe", 'W', "Use named pipes to connect to server.", nullptr, nullptr,
-     nullptr, GET_NO_ARG, NO_ARG, 0, 0, 0, nullptr, 0, nullptr},
+    {"pipe", 'W', "Use named pipes to connect to server.", 0, 0, 0, GET_NO_ARG,
+     NO_ARG, 0, 0, 0, 0, 0, 0},
 #endif
     {"plugin_dir", OPT_PLUGIN_DIR, "Directory for client-side plugins.",
      &opt_plugin_dir, &opt_plugin_dir, nullptr, GET_STR, REQUIRED_ARG, 0, 0, 0,
@@ -207,16 +208,16 @@ static struct my_option my_long_options[] = {
 #if defined(_WIN32)
     {"shared-memory-base-name", OPT_SHARED_MEMORY_BASE_NAME,
      "Base name of shared memory.", &shared_memory_base_name,
-     &shared_memory_base_name, nullptr, GET_STR_ALLOC, REQUIRED_ARG, 0, 0, 0,
-     nullptr, 0, nullptr},
+     &shared_memory_base_name, 0, GET_STR_ALLOC, REQUIRED_ARG, 0, 0, 0, 0, 0,
+     0},
 #endif
     {"silent", 's', "Be more silent.", &silent, &silent, nullptr, GET_BOOL,
      NO_ARG, 0, 0, 0, nullptr, 0, nullptr},
     {"socket", 'S', "The socket file to use for connection.",
      &opt_mysql_unix_port, &opt_mysql_unix_port, nullptr, GET_STR, REQUIRED_ARG,
      0, 0, 0, nullptr, 0, nullptr},
-#include "client/include/caching_sha2_passwordopt-longopts.h"
-#include "client/include/sslopt-longopts.h"
+#include "caching_sha2_passwordopt-longopts.h"
+#include "sslopt-longopts.h"
 
     {"use-threads", OPT_USE_THREADS,
      "Load files in parallel. The argument is the number "
@@ -267,7 +268,24 @@ extern "C" {
 static bool get_one_option(int optid, const struct my_option *opt,
                            char *argument) {
   switch (optid) {
-    PARSE_COMMAND_LINE_PASSWORD_OPTION;
+    case 'p':
+      if (argument == disabled_my_option) {
+        // Don't require password
+        static char empty_password[] = {'\0'};
+        DBUG_ASSERT(empty_password[0] ==
+                    '\0');  // Check that it has not been overwritten
+        argument = empty_password;
+      }
+      if (argument) {
+        char *start = argument;
+        my_free(opt_password);
+        opt_password = my_strdup(PSI_NOT_INSTRUMENTED, argument, MYF(MY_FAE));
+        while (*argument) *argument++ = 'x'; /* Destroy argument */
+        if (*start) start[1] = 0;            /* Cut length of argument */
+        tty_password = false;
+      } else
+        tty_password = true;
+      break;
 #ifdef _WIN32
     case 'W':
       opt_protocol = MYSQL_PROTOCOL_PIPE;
@@ -285,7 +303,7 @@ static bool get_one_option(int optid, const struct my_option *opt,
       DBUG_PUSH(argument ? argument : "d:t:o");
       debug_check_flag = true;
       break;
-#include "client/include/sslopt-case.h"
+#include "sslopt-case.h"
 
     case 'V':
       print_version();
@@ -294,9 +312,6 @@ static bool get_one_option(int optid, const struct my_option *opt,
     case '?':
       usage();
       exit(0);
-    case 'C':
-      CLIENT_WARN_DEPRECATED("--compress", "--compression-algorithms");
-      break;
   }
   return false;
 }
@@ -328,13 +343,14 @@ static int get_options(int *argc, char ***argv) {
   }
   current_db = *((*argv)++);
   (*argc)--;
+  if (tty_password) opt_password = get_tty_password(NullS);
   return (0);
 }
 
 static int write_to_table(char *filename, MYSQL *mysql) {
   char tablename[FN_REFLEN], hard_path[FN_REFLEN],
       escaped_name[FN_REFLEN * 2 + 1], sql_statement[FN_REFLEN * 16 + 256],
-      escaped_tablename[FN_REFLEN * 2 + 1], *end;
+      *end, *pos;
   DBUG_TRACE;
   DBUG_PRINT("enter", ("filename: %s", filename));
 
@@ -344,16 +360,14 @@ static int write_to_table(char *filename, MYSQL *mysql) {
   else
     my_load_path(hard_path, filename, nullptr); /* filename includes the path */
 
-  mysql_real_escape_string_quote(mysql, escaped_tablename, tablename,
-                                 (unsigned long)strlen(tablename), '`');
-
   if (opt_delete) {
     if (verbose)
       fprintf(stdout, "Deleting the old data from table %s\n", tablename);
-    snprintf(sql_statement, FN_REFLEN * 16 + 256, "DELETE FROM `%s`",
-             escaped_tablename);
-    if (mysql_query(mysql, sql_statement))
-      return db_error_with_table(mysql, tablename);
+    snprintf(sql_statement, FN_REFLEN * 16 + 256, "DELETE FROM %s", tablename);
+    if (mysql_query(mysql, sql_statement)) {
+      db_error_with_table(mysql, tablename);
+      return 1;
+    }
   }
   to_unix_path(hard_path);
   if (verbose) {
@@ -364,8 +378,6 @@ static int write_to_table(char *filename, MYSQL *mysql) {
       fprintf(stdout, "Loading data from SERVER file: %s into %s\n", hard_path,
               tablename);
   }
-  if (opt_local_file)
-    mysql_options(mysql, MYSQL_OPT_LOAD_DATA_LOCAL_DIR, filename);
   mysql_real_escape_string_quote(mysql, escaped_name, hard_path,
                                  (unsigned long)strlen(hard_path), '\'');
   sprintf(sql_statement, "LOAD DATA %s %s INFILE '%s'",
@@ -375,7 +387,11 @@ static int write_to_table(char *filename, MYSQL *mysql) {
   if (replace) end = my_stpcpy(end, " REPLACE");
   if (ignore) end = my_stpcpy(end, " IGNORE");
   end = my_stpcpy(end, " INTO TABLE `");
-  end = my_stpcpy(end, escaped_tablename);
+  /* Turn any ` into `` in table name. */
+  for (pos = tablename; *pos; pos++) {
+    if (*pos == '`') *end++ = '`';
+    *end++ = *pos;
+  }
   end = my_stpcpy(end, "`");
 
   if (fields_terminated || enclosed || opt_enclosed || escaped)
@@ -393,8 +409,10 @@ static int write_to_table(char *filename, MYSQL *mysql) {
     end = my_stpcpy(my_stpcpy(my_stpcpy(end, " ("), opt_columns), ")");
   *end = '\0';
 
-  if (mysql_query(mysql, sql_statement))
-    return db_error_with_table(mysql, tablename);
+  if (mysql_query(mysql, sql_statement)) {
+    db_error_with_table(mysql, tablename);
+    return 1;
+  }
   if (!silent) {
     if (mysql_info(mysql)) /* If NULL-pointer, print nothing */
     {
@@ -404,24 +422,23 @@ static int write_to_table(char *filename, MYSQL *mysql) {
   return 0;
 }
 
-static int lock_table(MYSQL *mysql, int tablecount, char **raw_tablename) {
+static void lock_table(MYSQL *mysql, int tablecount, char **raw_tablename) {
   DYNAMIC_STRING query;
   int i;
   char tablename[FN_REFLEN];
 
   if (verbose) fprintf(stdout, "Locking tables for write\n");
-  init_dynamic_string(&query, "LOCK TABLES ", 256);
+  init_dynamic_string(&query, "LOCK TABLES ", 256, 1024);
   for (i = 0; i < tablecount; i++) {
     fn_format(tablename, raw_tablename[i], "", "", 1 | 2);
     dynstr_append(&query, tablename);
     dynstr_append(&query, " WRITE,");
   }
   if (mysql_real_query(mysql, query.str, (ulong)(query.length - 1)))
-    return db_error(mysql); /* We shall continue here, if --force was given */
-  return 0;
+    db_error(mysql); /* We shall countinue here, if --force was given */
 }
 
-static MYSQL *db_connect(char *host, char *database, char *user, char *) {
+static MYSQL *db_connect(char *host, char *database, char *user, char *passwd) {
   MYSQL *mysql;
   if (verbose) fprintf(stdout, "Connecting to %s\n", host ? host : "localhost");
   if (opt_use_threads && !lock_tables) {
@@ -442,6 +459,8 @@ static MYSQL *db_connect(char *host, char *database, char *user, char *) {
   mysql_options(mysql, MYSQL_OPT_ZSTD_COMPRESSION_LEVEL,
                 &opt_zstd_compress_level);
 
+  if (opt_local_file)
+    mysql_options(mysql, MYSQL_OPT_LOCAL_INFILE, (char *)&opt_local_file);
   if (SSL_SET_OPTIONS(mysql)) {
     fprintf(stderr, "%s", SSL_SET_OPTIONS_ERROR);
     return nullptr;
@@ -471,26 +490,16 @@ static MYSQL *db_connect(char *host, char *database, char *user, char *) {
                  "mysqlimport");
   set_server_public_key(mysql);
   set_get_server_public_key_option(mysql);
-
-  set_password_options(mysql);
-  if (!(mysql_real_connect(mysql, host, user, nullptr, database, opt_mysql_port,
+  if (!(mysql_real_connect(mysql, host, user, passwd, database, opt_mysql_port,
                            opt_mysql_unix_port, 0))) {
     ignore_errors = false; /* NO RETURN FROM db_error */
     db_error(mysql);
-    if (mysql) mysql_close(mysql);
-    return nullptr;
-  }
-  if (ssl_client_check_post_connect_ssl_setup(
-          mysql, [](const char *err) { fprintf(stderr, "%s\n", err); })) {
-    if (mysql) mysql_close(mysql);
-    return nullptr;
   }
+  mysql->reconnect = false;
   if (verbose) fprintf(stdout, "Selecting database %s\n", database);
   if (mysql_select_db(mysql, database)) {
     ignore_errors = false;
     db_error(mysql);
-    if (mysql) mysql_close(mysql);
-    return nullptr;
   }
   return mysql;
 }
@@ -498,24 +507,25 @@ static MYSQL *db_connect(char *host, char *database, char *user, char *) {
 static void db_disconnect(char *host, MYSQL *mysql) {
   if (verbose)
     fprintf(stdout, "Disconnecting from %s\n", host ? host : "localhost");
+  mysql_close(mysql);
+}
+
+static void safe_exit(int error, MYSQL *mysql) {
+  if (ignore_errors) return;
   if (mysql) mysql_close(mysql);
+  exit(error);
 }
 
-static int safe_exit(int error) {
-  if (ignore_errors) return 0;
-  return error;
-}
-
-static int db_error_with_table(MYSQL *mysql, char *table) {
+static void db_error_with_table(MYSQL *mysql, char *table) {
   my_printf_error(0, "Error: %d, %s, when using table: %s", MYF(0),
                   mysql_errno(mysql), mysql_error(mysql), table);
-  return safe_exit(1);
+  safe_exit(1, mysql);
 }
 
-static int db_error(MYSQL *mysql) {
+static void db_error(MYSQL *mysql) {
   my_printf_error(0, "Error: %d %s", MYF(0), mysql_errno(mysql),
                   mysql_error(mysql));
-  return safe_exit(1);
+  safe_exit(1, mysql);
 }
 
 static char *add_load_option(char *ptr, const char *object,
@@ -551,7 +561,7 @@ static char *field_escape(char *to, const char *from, uint length) {
       end_backslashes ^= 1; /* find odd number of backslashes */
     else {
       if (*from == '\'' && !end_backslashes)
-        *to++ = *from; /* We want a duplicate of "'" for MySQL */
+        *to++ = *from; /* We want a dublicate of "'" for MySQL */
       end_backslashes = 0;
     }
   }
@@ -570,28 +580,25 @@ static void *worker_thread(void *arg) {
 
   if (mysql_thread_init()) goto error;
 
-  if (!(mysql = db_connect(current_host, current_db, current_user,
-                           opt_password[0]))) {
+  if (!(mysql =
+            db_connect(current_host, current_db, current_user, opt_password))) {
     goto error;
   }
 
-  if (mysql_query(mysql, "/*!40101 set @@character_set_database=binary */;") &&
-      (error = db_error(mysql))) {
-    if (exitcode == 0) exitcode = error;
-    /* We shall continue here, if --force was given */
+  if (mysql_query(mysql, "/*!40101 set @@character_set_database=binary */;")) {
+    db_error(mysql); /* We shall countinue here, if --force was given */
     goto error;
   }
 
   /*
     We are not currently catching the error here.
   */
-  if ((error = write_to_table(raw_table_name, mysql))) {
+  if ((error = write_to_table(raw_table_name, mysql)))
     if (exitcode == 0) exitcode = error;
-    goto error;
-  }
 
 error:
   if (mysql) db_disconnect(current_host, mysql);
+
   native_mutex_lock(&counter_mutex);
   counter--;
   native_cond_signal(&count_threshold);
@@ -605,7 +612,7 @@ error:
 int main(int argc, char **argv) {
   int error = 0;
   MY_INIT(argv[0]);
-  MYSQL *mysql = nullptr;
+
   my_getopt_use_args_separator = true;
   MEM_ROOT alloc{PSI_NOT_INSTRUMENTED, 512};
   if (load_defaults("my", load_default_groups, &argc, &argv, &alloc)) return 1;
@@ -636,10 +643,8 @@ int main(int argc, char **argv) {
 
     if (!(worker_threads = (my_thread_handle *)my_malloc(
               PSI_NOT_INSTRUMENTED, table_count * sizeof(*worker_threads),
-              MYF(0)))) {
-      exitcode = -2;
-      goto end;
-    }
+              MYF(0))))
+      return -2;
 
     for (counter = 0; *argv != nullptr; argv++) /* Loop through tables */
     {
@@ -688,33 +693,25 @@ int main(int argc, char **argv) {
 
     my_free(worker_threads);
   } else {
+    MYSQL *mysql = nullptr;
     if (!(mysql = db_connect(current_host, current_db, current_user,
-                             opt_password[0]))) {
-      exitcode = 1;
-      goto end;
+                             opt_password))) {
+      return (1); /* purecov: deadcode */
     }
 
     if (mysql_query(mysql,
-                    "/*!40101 set @@character_set_database=binary */;") &&
-        (error = db_error(mysql))) {
-      if (exitcode == 0) exitcode = error;
-      /* We shall continue here, if --force was given */
-      goto end;
+                    "/*!40101 set @@character_set_database=binary */;")) {
+      db_error(mysql); /* We shall countinue here, if --force was given */
+      return (1);
     }
 
-    if (lock_tables && (error = lock_table(mysql, argc, argv))) {
-      if (exitcode == 0) exitcode = error;
-      goto end;
-    }
+    if (lock_tables) lock_table(mysql, argc, argv);
     for (; *argv != nullptr; argv++)
-      if ((error = write_to_table(*argv, mysql))) {
+      if ((error = write_to_table(*argv, mysql)))
         if (exitcode == 0) exitcode = error;
-        break;
-      }
+    db_disconnect(current_host, mysql);
   }
-end:
-  db_disconnect(current_host, mysql);
-  free_passwords();
+  my_free(opt_password);
 #if defined(_WIN32)
   my_free(shared_memory_base_name);
 #endif
