@@ -1,16 +1,15 @@
-/* Copyright (c) 2018, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2018, 2020, Oracle and/or its affiliates. All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
 as published by the Free Software Foundation.
 
-This program is designed to work with certain software (including
+This program is also distributed with certain software (including
 but not limited to OpenSSL) that is licensed under separate terms,
 as designated in a particular file or component or in included license
 documentation.  The authors of MySQL hereby grant you an additional
 permission to link the program and your derivative works with the
-separately licensed software that they have either included with
-the program or referenced in the documentation.
+separately licensed software that they have included with MySQL.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -32,8 +31,8 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
   will receive this event.
 */
 
-#include <assert.h>
 #include <ctype.h>
+#include <my_dbug.h>
 #include <mysql/components/component_implementation.h>
 #include <mysql/components/my_service.h>
 #include <mysql/components/service_implementation.h>
@@ -42,6 +41,7 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 #include <mysql/components/services/udf_registration.h>
 #include <mysql/service_plugin_registry.h>
 #include <mysql_com.h>
+#include "template_utils.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -52,7 +52,6 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 #include <map>
 #include <memory>
 #include <string>
-#include "my_compiler.h"
 #include "template_utils.h"
 
 REQUIRES_SERVICE_PLACEHOLDER(mysql_udf_metadata);
@@ -69,7 +68,7 @@ class IError_handler {
   /**
     Virtual destructor.
   */
-  virtual ~IError_handler() = default;
+  virtual ~IError_handler() {}
   /**
     Error reporting method.
 
@@ -93,7 +92,8 @@ typedef bool (*validate_function)(IError_handler &handler, const char *arg,
   Check, whether the argument is not null pointer.
 */
 static bool not_null(IError_handler &handler, const char *arg,
-                     unsigned long length [[maybe_unused]], size_t arg_pos) {
+                     unsigned long length MY_ATTRIBUTE((unused)),
+                     size_t arg_pos) {
   if (arg == nullptr) {
     handler.error("Argument cannot be NULL [%d].", arg_pos);
     return false;
@@ -153,7 +153,7 @@ Arg_def audit_log_primary_args_def[] = {
     {audit_log_primary_args, array_elements(audit_log_primary_args)}};
 
 /**
-  Optional arguments definition (key, value).
+  Optional arguments defition (key, value).
 */
 Arg_def audit_log_extra_args_def[] = {
     {audit_log_key_value_string_args,
@@ -191,13 +191,12 @@ char *collation_name = const_cast<char *>(collation);
   @param [in, out]  args      UDF arguments structure
   @param [out]      handler   Error handler
 
-  @retval false Set the charset of all arguments successfully
+  @retval false Set the charset of all arguments successully
   @retval true  Otherwise
 */
 static bool set_args_charset_info(UDF_ARGS *args, IError_handler &handler) {
   for (size_t index = 0; index < args->arg_count; ++index) {
-    if (args->arg_type[index] == STRING_RESULT &&
-        mysql_service_mysql_udf_metadata->argument_set(
+    if (mysql_service_mysql_udf_metadata->argument_set(
             args, "collation", index, pointer_cast<void *>(collation_name))) {
       handler.error("Could not set the %s collation of argument '%d'.",
                     collation_name, index);
@@ -208,7 +207,7 @@ static bool set_args_charset_info(UDF_ARGS *args, IError_handler &handler) {
 }
 
 /**
-  Sets the charset info of the return value to utf8mb4.
+  Sets the charset info of the return value to utf8.
 
   @param [in, out]  initid    A pointer to the UDF_INIT structure
   @param [out]      handler   Error handler that keeps the error message
@@ -252,7 +251,7 @@ static int arg_check(IError_handler &handler, unsigned int arg_count,
   bool res[2];
   bool result = false;
 
-  assert(array_elements(res) >= arg_def_size);
+  DBUG_ASSERT(array_elements(res) >= arg_def_size);
 
   /*
     Check, whether provided argument count matches expected argument count.
@@ -263,7 +262,7 @@ static int arg_check(IError_handler &handler, unsigned int arg_count,
       result = true;
 
   /*
-    At least one argument count was matched against definition.
+    At least one argument count was matched againts definition.
   */
   if (result == false) {
     handler.error("Invalid argument count.");
@@ -397,11 +396,11 @@ class String_error_handler : public IError_handler {
 
     @param message Message to be copied.
   */
-  void error(const char *message, ...) override
+  virtual void error(const char *message, ...)
       MY_ATTRIBUTE((format(printf, 2, 3))) {
     va_list va;
     va_start(va, message);
-    const int copied = vsnprintf(m_buffer, m_size - 1, message, va);
+    int copied = vsnprintf(m_buffer, m_size - 1, message, va);
     va_end(va);
     m_buffer[copied] = '\0';
 
@@ -426,10 +425,10 @@ class String_error_handler : public IError_handler {
 /**
   UDF function itself.
 */
-static char *emit(UDF_INIT *initid [[maybe_unused]], UDF_ARGS *args,
+static char *emit(UDF_INIT *initid MY_ATTRIBUTE((unused)), UDF_ARGS *args,
                   char *result, unsigned long *length,
-                  unsigned char *null_value [[maybe_unused]],
-                  unsigned char *error [[maybe_unused]]) {
+                  unsigned char *null_value MY_ATTRIBUTE((unused)),
+                  unsigned char *error MY_ATTRIBUTE((unused))) {
   /*
     Store the error as the result of the UDF.
   */
@@ -477,9 +476,9 @@ static char *emit(UDF_INIT *initid [[maybe_unused]], UDF_ARGS *args,
                        arg_lengths, false)) < 0)
       return result;
 
-    const std::string key(*arguments, *arg_lengths);
+    std::string key(*arguments, *arg_lengths);
 
-    const std::map<std::string, mysql_event_message_key_value_t>::const_iterator
+    std::map<std::string, mysql_event_message_key_value_t>::const_iterator
         iter = key_values.find(key);
     if (iter != key_values.end()) {
       handler.error("Duplicated key [%d].", args->arg_count - arg_count);
@@ -519,7 +518,7 @@ static char *emit(UDF_INIT *initid [[maybe_unused]], UDF_ARGS *args,
   /**
     Allocate array that is used by the audit api service.
   */
-  const std::unique_ptr<mysql_event_message_key_value_t[]> key_value_map(
+  std::unique_ptr<mysql_event_message_key_value_t[]> key_value_map(
       key_values.size() > 0
           ? new mysql_event_message_key_value_t[key_values.size()]
           : nullptr);
