@@ -1,4 +1,4 @@
-/* Copyright (c) 2020, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2020, Oracle and/or its affiliates. All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
@@ -26,7 +26,6 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 #include <mysql/components/services/mysql_psi_system_service.h>
 #include <mysql/components/services/mysql_runtime_error.h>
 #include <mysql/components/services/mysql_rwlock_service.h>
-#include <mysql/components/services/psi_memory_service.h>
 
 #include <mysql/components/minimal_chassis.h>
 #include <mysql/components/services/dynamic_loader.h>
@@ -35,7 +34,6 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 #include "dynamic_loader_scheme_file_imp.h"
 #include "minimal_chassis_runtime_error_imp.h"
 #include "registry_imp.h"
-#include "registry_no_lock_imp.h"
 
 extern SERVICE_TYPE(registry) imp_mysql_minimal_chassis_registry;
 
@@ -47,10 +45,6 @@ extern SERVICE_TYPE(mysql_psi_system_v1)
     SERVICE_IMPLEMENTATION(mysql_minimal_chassis, mysql_psi_system_v1);
 REQUIRES_SERVICE_PLACEHOLDER(mysql_psi_system_v1);
 
-extern SERVICE_TYPE(psi_memory_v2)
-    SERVICE_IMPLEMENTATION(mysql_minimal_chassis, psi_memory_v2);
-REQUIRES_SERVICE_PLACEHOLDER(psi_memory_v2);
-
 REQUIRES_SERVICE_PLACEHOLDER(mysql_runtime_error);
 my_h_service h_err_service;
 
@@ -58,21 +52,10 @@ BEGIN_SERVICE_IMPLEMENTATION(mysql_minimal_chassis, registry)
 mysql_registry_imp::acquire, mysql_registry_imp::acquire_related,
     mysql_registry_imp::release END_SERVICE_IMPLEMENTATION();
 
-BEGIN_SERVICE_IMPLEMENTATION(mysql_minimal_chassis_no_lock, registry)
-mysql_registry_no_lock_imp::acquire,
-    mysql_registry_no_lock_imp::acquire_related,
-    mysql_registry_no_lock_imp::release END_SERVICE_IMPLEMENTATION();
-
 BEGIN_SERVICE_IMPLEMENTATION(mysql_minimal_chassis, registry_registration)
 mysql_registry_imp::register_service, mysql_registry_imp::unregister,
     mysql_registry_imp::set_default END_SERVICE_IMPLEMENTATION();
 
-BEGIN_SERVICE_IMPLEMENTATION(mysql_minimal_chassis_no_lock,
-                             registry_registration)
-mysql_registry_no_lock_imp::register_service,
-    mysql_registry_no_lock_imp::unregister,
-    mysql_registry_no_lock_imp::set_default END_SERVICE_IMPLEMENTATION();
-
 BEGIN_SERVICE_IMPLEMENTATION(mysql_minimal_chassis, registry_query)
 mysql_registry_imp::iterator_create, mysql_registry_imp::iterator_get,
     mysql_registry_imp::iterator_next, mysql_registry_imp::iterator_is_valid,
@@ -121,9 +104,7 @@ mysql_runtime_error_imp::emit END_SERVICE_IMPLEMENTATION();
 
 BEGIN_COMPONENT_PROVIDES(mysql_minimal_chassis)
 PROVIDES_SERVICE(mysql_minimal_chassis, registry),
-    PROVIDES_SERVICE(mysql_minimal_chassis_no_lock, registry),
     PROVIDES_SERVICE(mysql_minimal_chassis, registry_registration),
-    PROVIDES_SERVICE(mysql_minimal_chassis_no_lock, registry_registration),
     PROVIDES_SERVICE(mysql_minimal_chassis, registry_query),
     PROVIDES_SERVICE(mysql_minimal_chassis, registry_metadata_enumerate),
     PROVIDES_SERVICE(mysql_minimal_chassis, registry_metadata_query),
@@ -135,7 +116,6 @@ PROVIDES_SERVICE(mysql_minimal_chassis, registry),
     PROVIDES_SERVICE(mysql_minimal_chassis, mysql_runtime_error),
     PROVIDES_SERVICE(mysql_minimal_chassis, mysql_rwlock_v1),
     PROVIDES_SERVICE(mysql_minimal_chassis, mysql_psi_system_v1),
-    PROVIDES_SERVICE(mysql_minimal_chassis, psi_memory_v2),
     END_COMPONENT_PROVIDES();
 
 BEGIN_COMPONENT_REQUIRES_WITHOUT_REGISTRY(mysql_minimal_chassis)
@@ -148,7 +128,7 @@ METADATA("mysql.author", "Oracle Corporation"),
 DECLARE_COMPONENT(mysql_minimal_chassis, "mysql_minimal_chassis")
 /* There are no initialization/deinitialization functions, they will not be
   called as this component is not a regular one. */
-nullptr, nullptr END_DECLARE_COMPONENT();
+NULL, NULL END_DECLARE_COMPONENT();
 
 /**
   This is the entry function for minimal_chassis static library, which has to be
@@ -176,7 +156,7 @@ bool minimal_chassis_init(SERVICE_TYPE_NO_CONST(registry) * *registry,
     as well as other main bootstrap dynamic loader service implementations. */
   for (int inx = 0;
        mysql_component_mysql_minimal_chassis.provides[inx].implementation !=
-       nullptr;
+       NULL;
        ++inx) {
     if (imp_mysql_minimal_chassis_registry_registration.register_service(
             mysql_component_mysql_minimal_chassis.provides[inx].name,
@@ -187,7 +167,7 @@ bool minimal_chassis_init(SERVICE_TYPE_NO_CONST(registry) * *registry,
     }
   }
 
-  if (registry != nullptr) {
+  if (registry != NULL) {
     my_h_service registry_handle;
     if (imp_mysql_minimal_chassis_registry.acquire("registry",
                                                    &registry_handle)) {
@@ -210,9 +190,8 @@ bool minimal_chassis_init(SERVICE_TYPE_NO_CONST(registry) * *registry,
 
   mysql_dynamic_loader_scheme_file_imp::init();
 
-  if (comp_ref != nullptr) {
-    for (int inx = 0; comp_ref->provides[inx].implementation != nullptr;
-         ++inx) {
+  if (comp_ref != NULL) {
+    for (int inx = 0; comp_ref->provides[inx].implementation != NULL; ++inx) {
       if (imp_mysql_minimal_chassis_registry_registration.register_service(
               comp_ref->provides[inx].name,
               reinterpret_cast<my_h_service>(
@@ -248,9 +227,8 @@ bool minimal_chassis_deinit(SERVICE_TYPE_NO_CONST(registry) * registry,
   imp_mysql_minimal_chassis_registry.release(h_err_service);
   mysql_dynamic_loader_scheme_file_imp::deinit();
 
-  if (comp_ref != nullptr) {
-    for (int inx = 0; comp_ref->provides[inx].implementation != nullptr;
-         ++inx) {
+  if (comp_ref != NULL) {
+    for (int inx = 0; comp_ref->provides[inx].implementation != NULL; ++inx) {
       if (imp_mysql_minimal_chassis_registry_registration.unregister(
               comp_ref->provides[inx].name)) {
         return true;
@@ -258,14 +236,14 @@ bool minimal_chassis_deinit(SERVICE_TYPE_NO_CONST(registry) * registry,
     }
   }
 
-  if (registry != nullptr) {
+  if (registry != NULL) {
     imp_mysql_minimal_chassis_registry.release(
         reinterpret_cast<my_h_service>(registry));
-    registry = nullptr;
+    registry = NULL;
   }
   for (int inx = 0;
        mysql_component_mysql_minimal_chassis.provides[inx].implementation !=
-       nullptr;
+       NULL;
        ++inx) {
     if (imp_mysql_minimal_chassis_registry_registration.unregister(
             mysql_component_mysql_minimal_chassis.provides[inx].name)) {
@@ -281,11 +259,11 @@ bool minimal_chassis_deinit(SERVICE_TYPE_NO_CONST(registry) * registry,
   flag.
   The global services are mysql_runtime_error, mysql_psi_system_v1 and
   mysql_rwlock_v1.
-  If the use_related is ON then the globals are loaded with minimal chassis
+  If the use_related is ON then the globals are leaded with minimal chassis
   service implementations else they are loaded with the default service
   implementations
 
-  @param use_related Used to decide which service implementation to load
+  @param use_related Used to decide which service implementaion to load
          for globals.
 */
 void minimal_chassis_services_refresh(bool use_related) {
