@@ -1,18 +1,17 @@
 /************************************************************************
                       Mysql Enterprise Backup
- Copyright (c) 2019, 2024, Oracle and/or its affiliates.
+ Copyright (c) 2019, 2020, Oracle and/or its affiliates. All rights reserved.
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2.0,
  as published by the Free Software Foundation.
 
- This program is designed to work with certain software (including
+ This program is also distributed with certain software (including
  but not limited to OpenSSL) that is licensed under separate terms,
  as designated in a particular file or component or in included license
  documentation.  The authors of MySQL hereby grant you an additional
  permission to link the program and your derivative works with the
- separately licensed software that they have either included with
- the program or referenced in the documentation.
+ separately licensed software that they have included with MySQL.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,25 +24,25 @@
  *************************************************************************/
 
 #include "backup_page_tracker.h"
-
-#include <algorithm>
+#include "backup_comp_constants.h"
 #if defined _MSC_VER
 #include <direct.h>
 #endif
 #include <fcntl.h>
 #include <sys/stat.h>
-
-#include "backup_comp_constants.h"
+#include <algorithm>
+#include "m_string.h"
+#include "my_io.h"
 #include "mysql/plugin.h"
 #include "mysqld_error.h"
 
 // defined in mysqlbackup component definition
 extern char *mysqlbackup_backup_id;
 
-// Page track system variables
-bool Backup_page_tracker::m_receive_changed_page_data = false;
-char *Backup_page_tracker::m_changed_pages_file = nullptr;
+// page track system variables
 uchar *Backup_page_tracker::m_changed_pages_buf = nullptr;
+static std::string changed_pages_file;
+bool Backup_page_tracker::m_receive_changed_page_data = false;
 std::list<udf_data_t *> Backup_page_tracker::m_udf_list;
 
 bool Backup_page_tracker::backup_id_update() {
@@ -51,23 +50,11 @@ bool Backup_page_tracker::backup_id_update() {
   if (Backup_page_tracker::m_receive_changed_page_data)
     Backup_page_tracker::m_receive_changed_page_data = false;
 
-  // Delete the existing page tracker file
-  if (m_changed_pages_file != nullptr) {
-    remove(m_changed_pages_file);
-    free(m_changed_pages_file);
-    m_changed_pages_file = nullptr;
-  }
-
+  // delete the existing page tracker file
+  remove(changed_pages_file.c_str());
   return true;
 }
 
-void Backup_page_tracker::deinit() {
-  if (m_changed_pages_file != nullptr) {
-    free(m_changed_pages_file);
-    m_changed_pages_file = nullptr;
-  }
-}
-
 /**
    Make a list of the UDFs exposed by mysqlbackup page_tracking.
 */
@@ -96,12 +83,6 @@ void Backup_page_tracker::initialize_udf_list() {
       reinterpret_cast<Udf_func_any>(page_track_get_changed_pages),
       reinterpret_cast<Udf_func_init>(page_track_get_changed_pages_init),
       reinterpret_cast<Udf_func_deinit>(page_track_get_changed_pages_deinit)));
-
-  m_udf_list.push_back(new udf_data_t(
-      Backup_comp_constants::udf_page_track_purge_up_to, INT_RESULT,
-      reinterpret_cast<Udf_func_any>(page_track_purge_up_to),
-      reinterpret_cast<Udf_func_init>(page_track_purge_up_to_init),
-      reinterpret_cast<Udf_func_deinit>(page_track_purge_up_to_deinit)));
 }
 
 /**
@@ -112,38 +93,48 @@ void Backup_page_tracker::initialize_udf_list() {
    @retval non-zero failure
 */
 mysql_service_status_t Backup_page_tracker::register_udfs() {
-  if (!m_udf_list.empty()) {
-    std::string msg{"UDF list for mysqlbackup_component is not empty."};
-    LogErr(ERROR_LEVEL, ER_MYSQLBACKUP_MSG, msg.c_str());
-    return (1);
-  }
-
-  // Initialize the UDF list
   initialize_udf_list();
-
+  std::list<udf_data_t *> success_list;
   for (auto udf : m_udf_list) {
-    if (udf->m_is_registered) {
-      std::string msg{udf->m_name + " is already registered."};
-      LogErr(ERROR_LEVEL, ER_MYSQLBACKUP_MSG, msg.c_str());
-      // un-register the already registered UDFs
-      unregister_udfs();
-      return (1);
-    }
-
     if (mysql_service_udf_registration->udf_register(
             udf->m_name.c_str(), udf->m_return_type, udf->m_function,
             udf->m_init_function, udf->m_deinit_function)) {
-      std::string msg{udf->m_name + " register failed."};
-      LogErr(ERROR_LEVEL, ER_MYSQLBACKUP_MSG, msg.c_str());
-      // un-register the already registered UDFs
-      unregister_udfs();
+      LogErr(ERROR_LEVEL, ER_MYSQLBACKUP_MSG,
+             std::string(udf->m_name + " registration failed.").c_str());
+      // un-register already successful UDFs
+      unregister_udfs(success_list);
       return (1);
     }
+    // add the udf into success list
+    success_list.push_back(udf);
+  }
+  return (0);
+}
 
-    // UDF is registered successfully
-    udf->m_is_registered = true;
+/**
+   Un-Register a given list of UDFs
+
+   @return Status of UDF un-registration
+   @retval 0 success
+   @retval non-zero failure
+*/
+mysql_service_status_t Backup_page_tracker::unregister_udfs(
+    std::list<udf_data_t *> list) {
+  int was_present;
+  std::list<udf_data_t *> fail_list;
+
+  for (auto udf : list) {
+    if (mysql_service_udf_registration->udf_unregister(udf->m_name.c_str(),
+                                                       &was_present) ||
+        !was_present) {
+      LogErr(ERROR_LEVEL, ER_MYSQLBACKUP_MSG,
+             std::string(udf->m_name + " un-register failed").c_str());
+      fail_list.push_back(udf);
+    }
+    delete (udf);
   }
 
+  if (!fail_list.empty()) return (1);
   return (0);
 }
 
@@ -155,33 +146,7 @@ mysql_service_status_t Backup_page_tracker::register_udfs() {
    @retval non-zero failure
 */
 mysql_service_status_t Backup_page_tracker::unregister_udfs() {
-  mysql_service_status_t fail_status{0};
-
-  for (auto udf : m_udf_list) {
-    int was_present;
-    if (mysql_service_udf_registration->udf_unregister(udf->m_name.c_str(),
-                                                       &was_present) &&
-        was_present) {
-      if (udf->m_is_registered) {
-        std::string msg{udf->m_name + " unregister failed."};
-        LogErr(ERROR_LEVEL, ER_MYSQLBACKUP_MSG, msg.c_str());
-        fail_status = 1;
-      }
-    } else {
-      // UDF is un-registered successfully
-      udf->m_is_registered = false;
-    }
-  }
-
-  if (fail_status == 0) {
-    // UDFs are un-registered successfully, clear the UDF list.
-    while (!m_udf_list.empty()) {
-      delete (m_udf_list.back());
-      m_udf_list.pop_back();
-    }
-  }
-
-  return fail_status;
+  return (unregister_udfs(m_udf_list));
 }
 
 /**
@@ -199,8 +164,8 @@ bool Backup_page_tracker::set_page_tracking_init(UDF_INIT *, UDF_ARGS *,
 /**
    Callback method for initialization of UDF "mysqlbackup_page_track_set".
 */
-void Backup_page_tracker::set_page_tracking_deinit(UDF_INIT *initid
-                                                   [[maybe_unused]]) {}
+void Backup_page_tracker::set_page_tracking_deinit(
+    UDF_INIT *initid MY_ATTRIBUTE((unused))) {}
 
 /**
   UDF for "mysqlbackup_page_track_set"
@@ -252,8 +217,8 @@ bool Backup_page_tracker::page_track_get_start_lsn_init(UDF_INIT *, UDF_ARGS *,
    Callback method for initialization of UDF
    "mysqlbackup_page_track_get_start_lsn"
 */
-void Backup_page_tracker::page_track_get_start_lsn_deinit(UDF_INIT *initid
-                                                          [[maybe_unused]]) {}
+void Backup_page_tracker::page_track_get_start_lsn_deinit(
+    UDF_INIT *initid MY_ATTRIBUTE((unused))) {}
 
 /**
   UDF for "mysqlbackup_page_track_get_start_lsn"
@@ -297,7 +262,7 @@ bool Backup_page_tracker::page_track_get_changed_page_count_init(UDF_INIT *,
    "mysqlbackup_page_track_get_changed_page_count".
 */
 void Backup_page_tracker::page_track_get_changed_page_count_deinit(
-    UDF_INIT *initid [[maybe_unused]]) {}
+    UDF_INIT *initid MY_ATTRIBUTE((unused))) {}
 
 /**
   UDF for "mysqlbackup_page_track_get_changed_page_count"
@@ -347,10 +312,9 @@ bool Backup_page_tracker::page_track_get_changed_pages_init(UDF_INIT *,
    Callback method for initialization of UDF
    "mysqlbackup_page_track_get_changed_pages".
 */
-void Backup_page_tracker::page_track_get_changed_pages_deinit(UDF_INIT *initid [
-    [maybe_unused]]) {
+void Backup_page_tracker::page_track_get_changed_pages_deinit(
+    UDF_INIT *initid MY_ATTRIBUTE((unused))) {
   free(m_changed_pages_buf);
-  m_changed_pages_buf = nullptr;
 }
 
 /**
@@ -376,7 +340,6 @@ long long Backup_page_tracker::page_track_get_changed_pages(UDF_INIT *,
   if (!mysqlbackup_backup_id) {
     return (-1);
   }
-
   // Not expecting anything other than digits in the backupid.
   // Make sure no elements of a relative path are there if the
   // above rule is relaxed
@@ -392,8 +355,7 @@ long long Backup_page_tracker::page_track_get_changed_pages(UDF_INIT *,
   if (var_len == 0) return 2;
 
   std::string changed_pages_file_dir =
-      mysqlbackup_backupdir +
-      std::string(Backup_comp_constants::backup_scratch_dir);
+      mysqlbackup_backupdir + Backup_comp_constants::backup_scratch_dir;
 
 #if defined _MSC_VER
   _mkdir(changed_pages_file_dir.c_str());
@@ -403,14 +365,10 @@ long long Backup_page_tracker::page_track_get_changed_pages(UDF_INIT *,
   }
 #endif
 
-  free(m_changed_pages_file);
-  m_changed_pages_file =
-      strdup((changed_pages_file_dir + FN_LIBCHAR + backupid +
-              Backup_comp_constants::change_file_extension)
-                 .c_str());
-
-  // If file already exists return error
-  FILE *fd = fopen(m_changed_pages_file, "r");
+  changed_pages_file = changed_pages_file_dir + FN_LIBCHAR + backupid +
+                       Backup_comp_constants::change_file_extension;
+  // if file already exists return error
+  FILE *fd = fopen(changed_pages_file.c_str(), "r");
   if (fd) {
     fclose(fd);
     return (-1);
@@ -430,67 +388,6 @@ long long Backup_page_tracker::page_track_get_changed_pages(UDF_INIT *,
   return (status);
 }
 
-/**
-  Callback function for initialization of UDF
-  "mysqlbackup_page_track_purge_up_to".
-
-  @return Status of initialization
-  @retval false on success
-  @retval true on failure
-*/
-bool Backup_page_tracker::page_track_purge_up_to_init(UDF_INIT *,
-                                                      UDF_ARGS *args,
-                                                      char *message) {
-  if (args->arg_count != 1) {
-    snprintf(message, MYSQL_ERRMSG_SIZE, "Invalid number of arguments.");
-    return true;
-  }
-  if (args->arg_type[0] != INT_RESULT) {
-    snprintf(message, MYSQL_ERRMSG_SIZE, "Invalid argument type.");
-    return true;
-  }
-  return false;
-}
-
-/**
-  Callback method for deinitialization of UDF
-  "mysqlbackup_page_track_purge_up_to".
-*/
-void Backup_page_tracker::page_track_purge_up_to_deinit(UDF_INIT *) {}
-
-/**
-  UDF for "mysqlbackup_page_track_purge_up_to"
-  See include/mysql/udf_registration_types.h
-
-  The function takes the following parameter:
-  lsn           the lsn up to which the page-track data shall be purged.
-
-  The lsn is fixed down to the last start-lsn.
-
-  @return Status
-  @retval lsn up to which page-track data was purged on success
-  @retval -1 on failure
-*/
-long long Backup_page_tracker::page_track_purge_up_to(UDF_INIT *,
-                                                      UDF_ARGS *args,
-                                                      unsigned char *,
-                                                      unsigned char *) {
-  MYSQL_THD thd;
-  if (mysql_service_mysql_current_thread_reader->get(&thd)) {
-    mysql_error_service_printf(ER_MYSQLBACKUP_CLIENT_MSG, MYF(0),
-                               "Cannot get current thread handle");
-    return -1;
-  }
-
-  uint64_t lsn = *((long long *)args->args[0]);
-  int retval =
-      mysql_service_mysql_page_track->purge(thd, PAGE_TRACK_SE_INNODB, &lsn);
-  if (retval != 0) {
-    return -1;
-  }
-  return lsn;
-}
-
 /**
    Callback method from InnoDB page-tracking to return the changed pages.
 
@@ -504,20 +401,17 @@ long long Backup_page_tracker::page_track_purge_up_to(UDF_INIT *,
    @retval 0 success
    @retval non-zero failure
 */
-int page_track_callback(MYSQL_THD opaque_thd [[maybe_unused]],
+int page_track_callback(MYSQL_THD opaque_thd MY_ATTRIBUTE((unused)),
                         const uchar *buffer,
-                        size_t buffer_length [[maybe_unused]], int page_count,
-                        void *context [[maybe_unused]]) {
-  // Append to the disk file in binary mode
-  FILE *fd = fopen(Backup_page_tracker::m_changed_pages_file, "ab");
+                        size_t buffer_length MY_ATTRIBUTE((unused)),
+                        int page_count, void *context MY_ATTRIBUTE((unused))) {
+  // append to the disk file in binary mode
+  FILE *fd = fopen(changed_pages_file.c_str(), "ab");
   if (!fd) {
-    std::string msg{std::string("[page-track] Cannot open '") +
-                    Backup_page_tracker::m_changed_pages_file +
-                    "': " + strerror(errno) + "\n"};
     LogEvent()
         .type(LOG_TYPE_ERROR)
         .prio(ERROR_LEVEL)
-        .lookup(ER_MYSQLBACKUP_MSG, msg.c_str());
+        .lookup(ER_MYSQLBACKUP_MSG, "[page-track] File open failed.");
     return (1);
   }
 
@@ -527,19 +421,16 @@ int page_track_callback(MYSQL_THD opaque_thd [[maybe_unused]],
 
   // write failed
   if (write_count != data_size) {
-    std::string msg{std::string("[page-track] Cannot write '") +
-                    Backup_page_tracker::m_changed_pages_file +
-                    "': " + strerror(errno) + "\n"};
     LogEvent()
         .type(LOG_TYPE_ERROR)
         .prio(ERROR_LEVEL)
-        .lookup(ER_MYSQLBACKUP_MSG, msg.c_str());
+        .lookup(ER_MYSQLBACKUP_MSG, "[page-track] Writing to file failed.");
     return (1);
   }
 
   // on-going backup interrupted, stop receiving the changed page data
   if (!Backup_page_tracker::m_receive_changed_page_data)
-    return (2);  // interrupt an ongoing transfer
+    return (2);  // interupt an on going transfer
   else
     return (0);
 }
