@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2001, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2001, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -30,11 +30,10 @@
 
 #define MY_BIT_NONE (~(uint)0)
 
-#include <assert.h>
-#include <limits.h>
 #include <string.h>
 #include <sys/types.h>
 
+#include "my_dbug.h"
 #include "my_inttypes.h"
 
 typedef uint32 my_bitmap_map;
@@ -53,7 +52,6 @@ extern bool bitmap_is_prefix(const MY_BITMAP *map, uint prefix_size);
 extern bool bitmap_is_set_all(const MY_BITMAP *map);
 extern bool bitmap_is_subset(const MY_BITMAP *map1, const MY_BITMAP *map2);
 extern bool bitmap_is_overlapping(const MY_BITMAP *map1, const MY_BITMAP *map2);
-extern bool bitmap_is_valid(const MY_BITMAP *map);
 extern bool bitmap_test_and_set(MY_BITMAP *map, uint bitmap_bit);
 extern uint bitmap_set_next(MY_BITMAP *map);
 extern uint bitmap_get_first(const MY_BITMAP *map);
@@ -69,30 +67,28 @@ extern void bitmap_union(MY_BITMAP *map, const MY_BITMAP *map2);
 extern void bitmap_xor(MY_BITMAP *map, const MY_BITMAP *map2);
 extern void bitmap_invert(MY_BITMAP *map);
 extern void bitmap_copy(MY_BITMAP *map, const MY_BITMAP *map2);
-extern uint bitmap_n_copy(MY_BITMAP *dst, const MY_BITMAP *src,
-                          uint max_bits_to_copy = UINT_MAX);
 
 #define bitmap_buffer_size(bits) (((bits) + 31) / 32) * 4
 #define no_bytes_in_map(map) (((map)->n_bits + 7) / 8)
 #define no_words_in_map(map) (((map)->n_bits + 31) / 32)
 
 static inline void bitmap_set_bit(MY_BITMAP *map, uint bit) {
-  assert(bit < map->n_bits);
+  DBUG_ASSERT(bit < map->n_bits);
   ((uchar *)map->bitmap)[bit / 8] |= (1 << (bit & 7));
 }
 
 static inline void bitmap_flip_bit(MY_BITMAP *map, uint bit) {
-  assert(bit < map->n_bits);
+  DBUG_ASSERT(bit < map->n_bits);
   ((uchar *)map->bitmap)[bit / 8] ^= (1 << (bit & 7));
 }
 
 static inline void bitmap_clear_bit(MY_BITMAP *map, uint bit) {
-  assert(bit < map->n_bits);
+  DBUG_ASSERT(bit < map->n_bits);
   ((uchar *)map->bitmap)[bit / 8] &= ~(1 << (bit & 7));
 }
 
 static inline bool bitmap_is_set(const MY_BITMAP *map, uint bit) {
-  assert(bit < map->n_bits);
+  DBUG_ASSERT(bit < map->n_bits);
   return ((uchar *)map->bitmap)[bit / 8] & (1 << (bit & 7));
 }
 
@@ -105,8 +101,8 @@ static inline bool bitmap_is_set(const MY_BITMAP *map, uint bit) {
    @retval false The bitmaps differ.
  */
 static inline bool bitmap_cmp(const MY_BITMAP *map1, const MY_BITMAP *map2) {
-  assert(map1->n_bits > 0);
-  assert(map2->n_bits > 0);
+  DBUG_ASSERT(map1->n_bits > 0);
+  DBUG_ASSERT(map2->n_bits > 0);
 
   if (memcmp(map1->bitmap, map2->bitmap, 4 * (no_words_in_map(map1) - 1)) != 0)
     return false;
