@@ -1,4 +1,4 @@
-/* Copyright (c) 2007, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2007, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -29,12 +29,9 @@
 
 #include "mysql/mysql_lex_string.h"
 #include "plugin.h"
-
 #ifndef MYSQL_ABI_CHECK
 #include "m_string.h"
 #endif
-
-#include <mysql/components/services/bits/plugin_audit_connection_types.h>
 #include "my_command.h"
 #include "my_sqlcommand.h"
 #include "plugin_audit_message_types.h"
@@ -145,6 +142,22 @@ struct mysql_event_general {
   MYSQL_LEX_CSTRING general_ip;
 };
 
+/**
+  @enum mysql_event_connection_subclass_t
+
+  Events for MYSQL_AUDIT_CONNECTION_CLASS event class.
+*/
+typedef enum {
+  /** occurs after authentication phase is completed. */
+  MYSQL_AUDIT_CONNECTION_CONNECT = 1 << 0,
+  /** occurs after connection is terminated. */
+  MYSQL_AUDIT_CONNECTION_DISCONNECT = 1 << 1,
+  /** occurs after COM_CHANGE_USER RPC is completed. */
+  MYSQL_AUDIT_CONNECTION_CHANGE_USER = 1 << 2,
+  /** occurs before authentication. */
+  MYSQL_AUDIT_CONNECTION_PRE_AUTHENTICATE = 1 << 3
+} mysql_event_connection_subclass_t;
+
 #define MYSQL_AUDIT_CONNECTION_ALL                                      \
   (MYSQL_AUDIT_CONNECTION_CONNECT | MYSQL_AUDIT_CONNECTION_DISCONNECT | \
    MYSQL_AUDIT_CONNECTION_CHANGE_USER |                                 \
@@ -222,7 +235,7 @@ struct mysql_event_parse {
   /** input: the original query text */
   MYSQL_LEX_CSTRING query;
 
-  /** output: returns the null-terminated rewritten query allocated by
+  /** output: returns the null-terminated rewriten query allocated by
    * my_malloc() */
   MYSQL_LEX_CSTRING *rewritten_query;
 };
