@@ -1,7 +1,7 @@
 #ifndef SQL_COMMON_INCLUDED
 #define SQL_COMMON_INCLUDED
 
-/* Copyright (c) 2003, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -23,6 +23,8 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
+#define SQL_COMMON_INCLUDED
+
 /**
   @file include/sql_common.h
 */
@@ -40,18 +42,13 @@
 #ifdef MYSQL_SERVER
 #include "mysql_com_server.h"
 #endif
-
-struct CHARSET_INFO;
 struct MEM_ROOT;
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-#ifdef IMPORT_UNKNOWN_SQLSTATE
-__declspec(dllimport)
-#endif
-    extern const char *unknown_sqlstate;
+extern const char *unknown_sqlstate;
 extern const char *cant_connect_sqlstate;
 extern const char *not_error_sqlstate;
 
@@ -95,7 +92,6 @@ struct STATE_INFO {
 */
 
 struct st_mysql_trace_info;
-struct mysql_async_connect;
 
 struct MYSQL_EXTENSION {
   struct st_mysql_trace_info *trace_data;
@@ -105,21 +101,12 @@ struct MYSQL_EXTENSION {
 #ifdef MYSQL_SERVER
   // Used by replication to pass around compression context data.
   NET_SERVER *server_extn;
-  /* used by mysql_init() api for mysql command service related information */
-  void *mcs_extn;
 #endif
-  struct {
-    uint n_params;
-    char **names;
-    MYSQL_BIND *bind;
-  } bind_info;
 };
 
 /* "Constructor/destructor" for MYSQL extension structure. */
 MYSQL_EXTENSION *mysql_extension_init(MYSQL *);
 void mysql_extension_free(MYSQL_EXTENSION *);
-/* cleanup for the MYSQL extension bind structure */
-void mysql_extension_bind_free(MYSQL_EXTENSION *ext);
 /*
   Note: Allocated extension structure is freed in mysql_close_free()
   called by mysql_close().
@@ -131,40 +118,11 @@ void mysql_extension_bind_free(MYSQL_EXTENSION *ext);
 
 #define ASYNC_DATA(M) \
   (NULL != (M) ? (MYSQL_EXTENSION_PTR(M)->mysql_async_context) : NULL)
-/**
-  Sets the MYSQL_EXTENSION::server_extn attribute by the use of NET_SERVER which
-  contains information about compression context and compression attributes.
-  This attribute needs to be set each time mysql_real_connect() is called to
-  make a connection. When a connection attempt fails or when a connection is
-  closed, as part of the MYSQL handle cleanup, mysql_close_free() is called and
-  that will free MYSQL_EXTENSION::server_extn.
-
-  mysql_close_free() will free all the memory allocated in the MYSQL handle but
-  preserves MYSQL::options. These options are later free'd by
-  mysql_close_free_options() unless the client flag CLIENT_REMEMBER_OPTIONS is
-  set.
-
-  @param mysql  The MYSQL handle
-  @param extn   The NET_SERVER handle that contains compression context info.
-*/
 #ifdef MYSQL_SERVER
 inline void mysql_extension_set_server_extn(MYSQL *mysql, NET_SERVER *extn) {
   MYSQL_EXTENSION_PTR(mysql)->server_extn = extn;
 }
-#endif /* MYSQL_SERVER*/
-
-/*
-  Maximum allowed authentication plugins for a given user account.
-*/
-#define MAX_AUTHENTICATION_FACTOR 3
-/*
-  Placeholder to save plugin name and password of 1st, 2nd and 3rd
-  factor authentication methods.
-*/
-struct client_authentication_info {
-  char *plugin_name;
-  char *password;
-};
+#endif
 
 struct st_mysql_options_extention {
   char *plugin_dir;
@@ -186,17 +144,9 @@ struct st_mysql_options_extention {
   unsigned int total_configured_compression_algorithms;
   unsigned int zstd_compression_level;
   bool connection_compressed;
-  char *load_data_dir;
-  struct client_authentication_info client_auth_info[MAX_AUTHENTICATION_FACTOR];
-  void *ssl_session_data;   /** the session serialization to use */
-  char *tls_sni_servername; /* TLS sni server name */
 };
 
-#define MYSQL_OPTIONS_EXTENSION_PTR(H, what) \
-  ((H)->options.extension ? (H)->options.extension->what : nullptr)
-
 struct MYSQL_METHODS {
-  MYSQL *(*connect_method)(mysql_async_connect *connect_args);
   bool (*read_query_result)(MYSQL *mysql);
   bool (*advanced_command)(MYSQL *mysql, enum enum_server_command command,
                            const unsigned char *header, size_t header_length,
@@ -205,12 +155,12 @@ struct MYSQL_METHODS {
   MYSQL_DATA *(*read_rows)(MYSQL *mysql, MYSQL_FIELD *mysql_fields,
                            unsigned int fields);
   MYSQL_RES *(*use_result)(MYSQL *mysql);
-  MYSQL_ROW (*fetch_row)(MYSQL_RES *);
   void (*fetch_lengths)(unsigned long *to, MYSQL_ROW column,
                         unsigned int field_count);
   void (*flush_use_result)(MYSQL *mysql, bool flush_all_results);
   int (*read_change_user_result)(MYSQL *mysql);
-#if (!defined(MYSQL_SERVER) && !defined(MYSQL_COMPONENT)) || defined(XTRABACKUP)
+#if !defined(MYSQL_SERVER) || defined(XTRABACKUP)
+  MYSQL_FIELD *(*list_fields)(MYSQL *mysql);
   bool (*read_prepare_result)(MYSQL *mysql, MYSQL_STMT *stmt);
   int (*stmt_execute)(MYSQL_STMT *stmt);
   int (*read_binary_rows)(MYSQL_STMT *stmt);
@@ -237,32 +187,33 @@ struct MYSQL_METHODS {
                                                                ulong *res);
 };
 
-#define simple_command(mysql, command, arg, length, skip_check)               \
-  ((mysql)->methods                                                           \
-       ? (*(mysql)->methods->advanced_command)(mysql, command, nullptr, 0,    \
-                                               arg, length, skip_check, NULL) \
-       : (set_mysql_error(mysql, CR_COMMANDS_OUT_OF_SYNC, unknown_sqlstate),  \
+#define simple_command(mysql, command, arg, length, skip_check)              \
+  ((mysql)->methods                                                          \
+       ? (*(mysql)->methods->advanced_command)(mysql, command, 0, 0, arg,    \
+                                               length, skip_check, NULL)     \
+       : (set_mysql_error(mysql, CR_COMMANDS_OUT_OF_SYNC, unknown_sqlstate), \
           1))
 #define simple_command_nonblocking(mysql, command, arg, length, skip_check, \
                                    error)                                   \
   (*(mysql)->methods->advanced_command_nonblocking)(                        \
-      mysql, command, nullptr, 0, arg, length, skip_check, nullptr, error)
+      mysql, command, 0, 0, arg, length, skip_check, NULL, error)
 
 #define stmt_command(mysql, command, arg, length, stmt)                      \
   ((mysql)->methods                                                          \
-       ? (*(mysql)->methods->advanced_command)(mysql, command, nullptr, 0,   \
-                                               arg, length, 1, stmt)         \
+       ? (*(mysql)->methods->advanced_command)(mysql, command, 0, 0, arg,    \
+                                               length, 1, stmt)              \
        : (set_mysql_error(mysql, CR_COMMANDS_OUT_OF_SYNC, unknown_sqlstate), \
           1))
 
 extern CHARSET_INFO *default_client_charset_info;
+MYSQL_FIELD *unpack_fields(MYSQL *mysql, MYSQL_ROWS *data, MEM_ROOT *alloc,
+                           uint fields, bool default_value,
+                           uint server_capabilities);
 MYSQL_FIELD *cli_read_metadata_ex(MYSQL *mysql, MEM_ROOT *alloc,
                                   unsigned long field_count,
                                   unsigned int fields);
 MYSQL_FIELD *cli_read_metadata(MYSQL *mysql, unsigned long field_count,
                                unsigned int fields);
-MYSQL_RES *use_result(MYSQL *mysql);
-MYSQL *connect_helper(mysql_async_connect *ctx);
 void free_rows(MYSQL_DATA *cur);
 void free_old_query(MYSQL *mysql);
 void end_server(MYSQL *mysql);
@@ -281,10 +232,8 @@ unsigned long cli_safe_read_with_ok(MYSQL *mysql, bool parse_ok,
                                     bool *is_data_packet);
 void net_clear_error(NET *net);
 void set_stmt_errmsg(MYSQL_STMT *stmt, NET *net);
-void set_stmt_error(MYSQL_STMT *stmt, int errcode, const char *sqlstate);
-void set_stmt_extended_error(MYSQL_STMT *stmt, int errcode,
-                             const char *sqlstate, const char *format, ...)
-    MY_ATTRIBUTE((format(printf, 4, 5)));
+void set_stmt_error(MYSQL_STMT *stmt, int errcode, const char *sqlstate,
+                    const char *err);
 void set_mysql_error(MYSQL *mysql, int errcode, const char *sqlstate);
 void set_mysql_extended_error(MYSQL *mysql, int errcode, const char *sqlstate,
                               const char *format, ...)
@@ -307,21 +256,10 @@ extern bool libmysql_cleartext_plugin_enabled;
 int is_file_or_dir_world_writable(const char *filepath);
 void read_ok_ex(MYSQL *mysql, unsigned long len);
 
-bool fix_param_bind(MYSQL_BIND *param, uint idx);
-bool mysql_int_serialize_param_data(
-    NET *net, unsigned int param_count, MYSQL_BIND *params, const char **names,
-    unsigned long n_param_sets, uchar **ret_data, ulong *ret_length,
-    uchar send_types_to_server, bool send_named_params,
-    bool send_parameter_set_count, bool send_parameter_count_when_zero);
-
 #ifdef __cplusplus
 }
 #endif
 
 #define protocol_41(A) ((A)->server_capabilities & CLIENT_PROTOCOL_41)
 
-/*
-  AuthNextFactor packet is defined to have value 2 in 1st byte.
-*/
-#define AUTH_NEXT_FACTOR_PACKETTYPE 2
 #endif /* SQL_COMMON_INCLUDED */
