@@ -1,7 +1,7 @@
 #ifndef SQL_STRING_INCLUDED
 #define SQL_STRING_INCLUDED
 
-/* Copyright (c) 2000, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -29,26 +29,24 @@
   See in particular the comment on String before you use anything from here.
 */
 
-#include <assert.h>
 #include <stdint.h>
 #include <string.h>
 #include <sys/types.h>
 #include <new>
 #include <string>
-#include <string_view>
 
 #include "lex_string.h"
+#include "m_ctype.h"   // my_convert
+#include "m_string.h"  // LEX_CSTRING
 #include "memory_debugging.h"
 #include "my_alloc.h"
 #include "my_compiler.h"
-#include "my_sys.h"
-
+#include "my_dbug.h"
 #include "my_inttypes.h"
-#include "mysql/components/services/bits/psi_bits.h"
 #include "mysql/mysql_lex_string.h"  // LEX_STRING
+#include "mysql/psi/psi_base.h"
 #include "mysql/psi/psi_memory.h"
 #include "mysql/service_mysql_alloc.h"  // my_free
-#include "mysql/strings/m_ctype.h"      // my_convert
 
 struct MEM_ROOT;
 
@@ -68,7 +66,7 @@ extern PSI_memory_key key_memory_String_value;
   pass it and its descendants (such as Name_string) into functions
   using call-by-value evaluation.
 
-  Don't add new members or virtual methods to this class!
+  Don't add new members or virual methods into this class!
 */
 class Simple_cstring {
  private:
@@ -81,9 +79,9 @@ class Simple_cstring {
   */
   void set(const char *str_arg, size_t length_arg) {
     // NULL is allowed only with length==0
-    assert(str_arg || length_arg == 0);
+    DBUG_ASSERT(str_arg || length_arg == 0);
     // For non-NULL, make sure length_arg is in sync with '\0' terminator.
-    assert(!str_arg || str_arg[length_arg] == '\0');
+    DBUG_ASSERT(!str_arg || str_arg[length_arg] == '\0');
     m_str = str_arg;
     m_length = length_arg;
   }
@@ -219,8 +217,8 @@ class String {
     str.m_is_alloced = false;
   }
   static void *operator new(size_t size, MEM_ROOT *mem_root,
-                            const std::nothrow_t &arg
-                            [[maybe_unused]] = std::nothrow) noexcept {
+                            const std::nothrow_t &arg MY_ATTRIBUTE((unused)) =
+                                std::nothrow) noexcept {
     return mem_root->Alloc(size);
   }
   static void operator delete(void *ptr_arg, size_t size) {
@@ -230,7 +228,7 @@ class String {
   }
 
   static void operator delete(
-      void *, MEM_ROOT *, const std::nothrow_t &) noexcept { /* never called */
+      void *, MEM_ROOT *, const std::nothrow_t &)noexcept { /* never called */
   }
 
   ~String() { mem_free(); }
@@ -248,8 +246,8 @@ class String {
   const char *ptr() const { return m_ptr; }
   char *ptr() { return m_ptr; }
   char *c_ptr() {
-    assert(!m_is_alloced || !m_ptr || !m_alloced_length ||
-           (m_alloced_length >= (m_length + 1)));
+    DBUG_ASSERT(!m_is_alloced || !m_ptr || !m_alloced_length ||
+                (m_alloced_length >= (m_length + 1)));
 
     /*
       Should be safe, but in case valgrind complains on this line, it means
@@ -277,7 +275,7 @@ class String {
   }
 
   void set(String &str, size_t offset, size_t arg_length) {
-    assert(&str != this);
+    DBUG_ASSERT(&str != this);
     mem_free();
     m_ptr = str.ptr() + offset;
     m_length = arg_length;
@@ -368,9 +366,9 @@ class String {
     m_ptr[m_length] = '\0';
   }
 
-  void mem_claim(bool claim) {
+  void mem_claim() {
     if (m_is_alloced) {
-      my_claim(m_ptr, claim);
+      my_claim(m_ptr);
     }
   }
 
@@ -418,7 +416,7 @@ class String {
         It is forbidden to do assignments like
         some_string = substring_of_that_string
        */
-      assert(!s.uses_buffer_owned_by(this));
+      DBUG_ASSERT(!s.uses_buffer_owned_by(this));
       mem_free();
       m_ptr = s.m_ptr;
       m_length = s.m_length;
@@ -434,7 +432,7 @@ class String {
         It is forbidden to do assignments like
         some_string = substring_of_that_string
        */
-      assert(!s.uses_buffer_owned_by(this));
+      DBUG_ASSERT(!s.uses_buffer_owned_by(this));
       mem_free();
       m_ptr = s.m_ptr;
       m_length = s.m_length;
@@ -455,9 +453,9 @@ class String {
     @param s - a String object to steal buffer from.
   */
   void takeover(String &s) {
-    assert(this != &s);
+    DBUG_ASSERT(this != &s);
     // Make sure buffers of the two Strings do not overlap
-    assert(!s.uses_buffer_owned_by(this));
+    DBUG_ASSERT(!s.uses_buffer_owned_by(this));
     mem_free();
     m_ptr = s.m_ptr;
     m_length = s.m_length;
@@ -470,12 +468,10 @@ class String {
     s.m_is_alloced = false;
   }
 
-  bool copy();                 // Alloc string if not allocated
+  bool copy();                 // Alloc string if not alloced
   bool copy(const String &s);  // Allocate new string
+  // Allocate new string
   bool copy(const char *s, size_t arg_length, const CHARSET_INFO *cs);
-  bool copy(const char *s, size_t arg_length, const CHARSET_INFO *from_cs,
-            const CHARSET_INFO *to_cs, uint *errors);
-
   static bool needs_conversion(size_t arg_length, const CHARSET_INFO *cs_from,
                                const CHARSET_INFO *cs_to, size_t *offset);
   bool needs_conversion(const CHARSET_INFO *cs_to) const {
@@ -495,8 +491,10 @@ class String {
                     const CHARSET_INFO *cs);
   bool set_or_copy_aligned(const char *s, size_t arg_length,
                            const CHARSET_INFO *cs);
+  bool copy(const char *s, size_t arg_length, const CHARSET_INFO *csfrom,
+            const CHARSET_INFO *csto, uint *errors);
   bool append(const String &s);
-  bool append(std::string_view s) { return append(s.data(), s.size()); }
+  bool append(const char *s);
   bool append(LEX_STRING *ls) { return append(ls->str, ls->length); }
   bool append(Simple_cstring str) { return append(str.ptr(), str.length()); }
   bool append(const char *s, size_t arg_length);
@@ -527,7 +525,7 @@ class String {
   */
   int strrstr(const String &search, size_t offset = 0) const;
   /**
-   * Returns substring of given characters length, starting at given character
+   * Returns substring of given characters lenght, starting at given character
    * offset. Note that parameter indexes are character indexes and not byte
    * indexes.
    */
@@ -553,27 +551,24 @@ class String {
   size_t charpos(size_t i, size_t offset = 0) const;
 
   bool reserve(size_t space_needed) {
-    if (m_alloced_length < m_length + space_needed) {
-      return mem_realloc(m_length + space_needed);
-    }
-    return false;
+    return mem_realloc(m_length + space_needed);
   }
   bool reserve(size_t space_needed, size_t grow_by);
 
   /* Inline (general) functions used by the protocol functions */
 
   char *prep_append(size_t arg_length, size_t step_alloc) {
-    const size_t new_length = arg_length + m_length;
+    size_t new_length = arg_length + m_length;
     if (new_length > m_alloced_length) {
       if (mem_realloc(new_length + step_alloc)) return nullptr;
     }
-    const size_t old_length = m_length;
+    size_t old_length = m_length;
     m_length += arg_length;
     return m_ptr + old_length; /* Area to use */
   }
 
   bool append(const char *s, size_t arg_length, size_t step_alloc) {
-    const size_t new_length = arg_length + m_length;
+    size_t new_length = arg_length + m_length;
     if (new_length > m_alloced_length &&
         mem_realloc_exp(new_length + step_alloc))
       return true;
@@ -608,35 +603,6 @@ class String {
   char *dup(MEM_ROOT *root) const;
 };
 
-/**
-  Checks that the source string can be just copied to the destination string
-  without conversion.
-
-  @param arg_length     Length of string to copy.
-  @param from_cs        Character set to copy from
-  @param to_cs          Character set to copy to
-  @param[out] offset	Returns number of unaligned characters.
-
-  @returns true if conversion is required, false otherwise.
-
-  @note
-  to_cs may be nullptr for "no conversion" if the system variable
-  character_set_results is NULL.
-*/
-
-inline bool String::needs_conversion(size_t arg_length,
-                                     const CHARSET_INFO *from_cs,
-                                     const CHARSET_INFO *to_cs,
-                                     size_t *offset) {
-  *offset = 0;
-  if (to_cs == nullptr || (to_cs == &my_charset_bin) || from_cs == to_cs ||
-      my_charset_same(from_cs, to_cs) ||
-      ((from_cs == &my_charset_bin) &&
-       (0 == (*offset = (arg_length % to_cs->mbminlen)))))
-    return false;
-  return true;
-}
-
 static inline void swap(String &a, String &b) noexcept { a.swap(b); }
 
 static inline std::string to_string(const String &str) {
