@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -24,6 +24,7 @@
   @file include/sslopt-longopts.h
 */
 
+#if defined(HAVE_OPENSSL)
 #if !defined(MYSQL_SERVER) || defined(XTRABACKUP)
 {"ssl-mode",
  OPT_SSL_MODE,
@@ -158,11 +159,7 @@
     {"tls-version",
      OPT_TLS_VERSION,
      "TLS version to use, "
-#ifdef HAVE_TLSv13
-     "permitted values are: TLSv1.2, TLSv1.3",
-#else
-     "permitted values are: TLSv1.2",
-#endif
+     "permitted values are: TLSv1, TLSv1.1, TLSv1.2, TLSv1.3",
      &opt_tls_version,
      &opt_tls_version,
      nullptr,
@@ -203,46 +200,3 @@
      nullptr,
      0,
      nullptr},
-    {"ssl-session-data",
-     OPT_SSL_SESSION_DATA,
-     "Session data file to use to enable ssl session reuse",
-     &opt_ssl_session_data,
-     &opt_ssl_session_data,
-     nullptr,
-     GET_STR,
-     REQUIRED_ARG,
-     0,
-     0,
-     0,
-     nullptr,
-     0,
-     nullptr},
-    {"ssl-session-data-continue-on-failed-reuse",
-     OPT_SSL_SESSION_DATA_CONTINUE_ON_FAILED_REUSE,
-     "If set to ON, this option will allow connection to succeed even if "
-     "session data cannot be reused.",
-     &opt_ssl_session_data_continue_on_failed_reuse,
-     &opt_ssl_session_data_continue_on_failed_reuse,
-     nullptr,
-     GET_BOOL,
-     OPT_ARG,
-     0,
-     0,
-     0,
-     nullptr,
-     0,
-     nullptr},
-    {"tls-sni-servername",
-     OPT_TLS_SNI_SERVERNAME,
-     "The SNI server name to pass to server",
-     &opt_tls_sni_servername,
-     &opt_tls_sni_servername,
-     nullptr,
-     GET_STR,
-     REQUIRED_ARG,
-     0,
-     0,
-     0,
-     nullptr,
-     0,
-     nullptr},
