@@ -1,4 +1,4 @@
-/* Copyright (c) 2019, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -23,8 +23,8 @@
 #ifndef CODECS_BINARY_INCLUDED
 #define CODECS_BINARY_INCLUDED
 
+#include "base.h"
 #include "libbinlogevents/include/binary_log.h"
-#include "libbinlogevents/include/codecs/base.h"
 
 namespace binary_log {
 namespace codecs {
@@ -34,19 +34,23 @@ namespace binary {
   This is the abstract and base class for binary log BINARY codecs.
  */
 class Base_codec : public binary_log::codecs::Codec {
+ public:
+  static const unsigned short UINT_64T_MIN_SIZE = 1;
+  static const unsigned short UINT_64T_MAX_SIZE = 1;
+
  protected:
   Event_reader *m_reader;
   inline Event_reader &reader() { return *m_reader; }
 
  public:
-  std::pair<std::size_t, bool> decode(const unsigned char *from,
-                                      std::size_t size,
-                                      Binary_log_event &to) const override = 0;
-  std::pair<std::size_t, bool> encode(const Binary_log_event &from,
-                                      unsigned char *to,
-                                      std::size_t size) const override = 0;
+  virtual std::pair<std::size_t, bool> decode(const unsigned char *from,
+                                              std::size_t size,
+                                              Binary_log_event &to) const = 0;
+  virtual std::pair<std::size_t, bool> encode(const Binary_log_event &from,
+                                              unsigned char *to,
+                                              std::size_t size) const = 0;
 
-  ~Base_codec() override = default;
+  virtual ~Base_codec() {}
 };
 
 /**
@@ -54,26 +58,7 @@ class Base_codec : public binary_log::codecs::Codec {
  */
 class Transaction_payload : public Base_codec {
  public:
-  /**
-    The on-the-wire fields
-   */
-  enum fields {
-    /** Marks the end of the payload header. */
-    OTW_PAYLOAD_HEADER_END_MARK = 0,
-
-    /** The payload field */
-    OTW_PAYLOAD_SIZE_FIELD = 1,
-
-    /** The compression type field */
-    OTW_PAYLOAD_COMPRESSION_TYPE_FIELD = 2,
-
-    /** The uncompressed size field */
-    OTW_PAYLOAD_UNCOMPRESSED_SIZE_FIELD = 3,
-
-    /** Other fields are appended here. */
-  };
-
-  Transaction_payload() = default;
+  Transaction_payload() {}
   /**
      This member function shall decode the contents of the buffer provided and
      fill in the event referenced. Note that the event provided needs to be of
@@ -86,9 +71,9 @@ class Transaction_payload : public Base_codec {
      @return a pair containing the amount of bytes decoded and whether there was
              an error or not. False if no error, true otherwise.
    */
-  std::pair<std::size_t, bool> decode(const unsigned char *from,
-                                      std::size_t size,
-                                      Binary_log_event &to) const override;
+  virtual std::pair<std::size_t, bool> decode(const unsigned char *from,
+                                              std::size_t size,
+                                              Binary_log_event &to) const;
 
   /**
      This member function shall encode the contents of the event referenced and
@@ -102,64 +87,9 @@ class Transaction_payload : public Base_codec {
      @return a pair containing the amount of bytes encoded and whether there was
              an error or not.
   */
-  std::pair<std::size_t, bool> encode(const Binary_log_event &from,
-                                      unsigned char *to,
-                                      std::size_t size) const override;
-};
-
-/**
-  Binary codec for the heartbeat log event.
- */
-class Heartbeat : public Base_codec {
- public:
-  /**
-    The on-the-wire fields
-   */
-  enum fields {
-    /** Marks the end of the fields. */
-    OTW_HB_HEADER_END_MARK = 0,
-
-    /** The log file name */
-    OTW_HB_LOG_FILENAME_FIELD = 1,
-
-    /** The log position field */
-    OTW_HB_LOG_POSITION_FIELD = 2,
-
-    /** Other fields are appended here. */
-  };
-
-  Heartbeat() {}
-  /**
-     This member function shall decode the contents of the buffer provided and
-     fill in the event referenced. Note that the event provided needs to be of
-     type HEARTBEAT_EVENT_V2.
-
-     @param from the buffer to decode
-     @param size the size of the buffer to decode.
-     @param to the event to store the decoded information into.
-
-     @return a pair containing the amount of bytes decoded and whether there was
-             an error or not. False if no error, true otherwise.
-   */
-  std::pair<std::size_t, bool> decode(const unsigned char *from,
-                                      std::size_t size,
-                                      Binary_log_event &to) const override;
-
-  /**
-     This member function shall encode the contents of the event referenced and
-     store the result in the buffer provided. Note that the event referenced
-     needs to be of type HEARTBEAT_EVENT_V2.
-
-     @param from the event to encode.
-     @param to the buffer where to store the encoded event.
-     @param size the size of the buffer.
-
-     @return a pair containing the amount of bytes encoded and whether there was
-             an error or not.
-  */
-  std::pair<std::size_t, bool> encode(const Binary_log_event &from,
-                                      unsigned char *to,
-                                      std::size_t size) const override;
+  virtual std::pair<std::size_t, bool> encode(const Binary_log_event &from,
+                                              unsigned char *to,
+                                              std::size_t size) const;
 };
 
 }  // namespace binary
