@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -28,7 +28,7 @@
 #include "m_string.h"
 #include "my_byteorder.h"
 
-#if !defined(NDEBUG) && defined(HAVE_MYSYS)
+#ifndef DBUG_OFF
 /* Event parser state names */
 static const char *event_parser_state_names[] = {"None", "GTID", "DDL", "DML",
                                                  "Error"};
@@ -40,7 +40,7 @@ static const char *event_parser_state_names[] = {"None", "GTID", "DDL", "DML",
   -----------------------------------------
 */
 
-Transaction_boundary_parser::~Transaction_boundary_parser() = default;
+Transaction_boundary_parser::~Transaction_boundary_parser() {}
 
 /**
    Reset the transaction boundary parser.
@@ -119,9 +119,7 @@ bool Transaction_boundary_parser::check_row_logging_constraints(
             !native_strncasecmp(log_event_info.query,
                                 STRING_WITH_LEN("XA PREPARE")) ||
             !native_strncasecmp(log_event_info.query,
-                                STRING_WITH_LEN("SAVEPOINT")) ||
-            !native_strncasecmp(log_event_info.query,
-                                STRING_WITH_LEN("CREATE TABLE")))
+                                STRING_WITH_LEN("SAVEPOINT")))
           return false;
       }
       return true;
@@ -245,7 +243,6 @@ Transaction_boundary_parser::get_event_boundary_type(
     case binary_log::ROTATE_EVENT:
     case binary_log::FORMAT_DESCRIPTION_EVENT:
     case binary_log::HEARTBEAT_LOG_EVENT:
-    case binary_log::HEARTBEAT_LOG_EVENT_V2:
     case binary_log::PREVIOUS_GTIDS_LOG_EVENT:
     case binary_log::STOP_EVENT:
     case binary_log::SLAVE_EVENT:
