@@ -1,16 +1,15 @@
-/* Copyright (c) 2011, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2011, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    Without limiting anything contained in the foregoing, this file,
    which is part of C Driver for MySQL (Connector/C), is also subject to the
@@ -33,8 +32,6 @@
 // First include (the generated) my_config.h, to get correct platform defines.
 #include "my_config.h"
 
-#include <memory>
-
 #include <sys/types.h>
 
 #include <openssl/evp.h>
@@ -48,10 +45,17 @@
 
 #include "crypt_genhash_impl.h"
 #include "m_string.h"
-#include "my_inttypes.h"
+
+#ifdef HAVE_ALLOCA_H
+#include <alloca.h>
+#endif
 
 #include <errno.h>
 
+#ifdef _WIN32
+#include <malloc.h>
+#endif
+
 #define DIGEST_CTX EVP_MD_CTX
 #define DIGEST_LEN SHA256_DIGEST_LENGTH
 
@@ -179,7 +183,7 @@ static uint getrounds(const char *s) {
    @param [in,out] salt_begin  As input, pointer to start of crypt passwd,
                            as output, pointer to first byte of the salt
    @param [in,out] salt_end    As input, pointer to the last byte in passwd,
-                           as output, pointer to the byte immediately following
+                           as output, pointer to the byte immediatly following
   the salt ($)
 
    @return The size of the salt identified
@@ -323,8 +327,7 @@ char *my_crypt_genhash(char *ctbuffer, size_t ctbufflen, const char *plaintext,
   DIGESTFinal(DP, ctxDP);
 
   /* 16. */
-  std::unique_ptr<char[]> PPbuf(new char[plaintext_len]);
-  Pp = P = PPbuf.get();
+  Pp = P = (char *)alloca(plaintext_len);
   for (i = plaintext_len; i >= MIXCHARS; i -= MIXCHARS) {
     Pp = (char *)(memcpy(Pp, DP, MIXCHARS)) + MIXCHARS;
   }
@@ -336,8 +339,7 @@ char *my_crypt_genhash(char *ctbuffer, size_t ctbufflen, const char *plaintext,
   DIGESTFinal(DS, ctxDS);
 
   /* 20. */
-  std::unique_ptr<char[]> SSbuf(new char[salt_len]);
-  Sp = S = SSbuf.get();
+  Sp = S = (char *)alloca(salt_len);
   for (i = salt_len; i >= MIXCHARS; i -= MIXCHARS) {
     Sp = (char *)(memcpy(Sp, DS, MIXCHARS)) + MIXCHARS;
   }
@@ -416,7 +418,7 @@ char *my_crypt_genhash(char *ctbuffer, size_t ctbufflen, const char *plaintext,
 }
 
 /**
-  Generate a random string using ASCII characters but avoid separator character.
+  Generate a random string using ASCII characters but avoid seperator character.
   Stdlib rand and srand are used to produce pseudo random numbers between
   with about 7 bit worth of entropty between 1-127.
 */
@@ -440,13 +442,3 @@ void xor_string(char *to, int to_len, char *pattern, int pattern_len) {
     ++loop;
   }
 }
-
-// extern "C" since it is an (undocumented) part of the libmysql ABI.
-extern "C" void my_make_scrambled_password(char *to, const char *password,
-                                           size_t pass_len) {
-  char salt[CRYPT_SALT_LENGTH + 1];
-
-  generate_user_salt(salt, CRYPT_SALT_LENGTH + 1);
-  my_crypt_genhash(to, CRYPT_MAX_PASSWORD_SIZE, password, pass_len, salt,
-                   nullptr);
-}
