@@ -1,16 +1,15 @@
-/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    Without limiting anything contained in the foregoing, this file,
    which is part of C Driver for MySQL (Connector/C), is also subject to the
@@ -146,17 +145,17 @@ uchar *zstd_compress_alloc(mysql_zstd_compress_context *comp_ctx,
 /**
   Uncompress a zstd compressed data.
 
-  @param      comp_ctx    Pointer to compression context.
-  @param      packet      Packet with zstd compressed data.
-  @param      len         Length of zstd compressed packet.
-  @param[out] complen     Length of uncompressed packet.
+  @param comp_ctx         Pointer to compression context.
+  @param packet           Packet with zstd compressed data.
+  @param len              Length of zstd compressed packet.
+  @param complen [out]    Length of uncompressed packet.
 
   @return true on error else false.
 */
 
 static bool zstd_uncompress(mysql_zstd_compress_context *comp_ctx,
                             uchar *packet, size_t len, size_t *complen) {
-  assert(comp_ctx != nullptr);
+  DBUG_ASSERT(comp_ctx != nullptr);
   size_t zstd_res;
   void *compbuf;
 
@@ -236,9 +235,9 @@ static uchar *zlib_compress_alloc(mysql_zlib_compress_context *comp_ctx,
 /**
   Uncompress a zlib compressed data.
 
-  @param      packet      Packet which zstd compressed data.
-  @param      len         Length of zstd compressed packet.
-  @param[out] complen     Length of uncompressed packet.
+  @param packet           Packet which zstd compressed data.
+  @param len              Length of zstd compressed packet.
+  @param complen [out]    Length of uncompressed packet.
 
   @return true on error else false.
 */
@@ -264,17 +263,18 @@ static bool zlib_uncompress(uchar *packet, size_t len, size_t *complen) {
   return false;
 }
 
-/**
+/*
    This replaces the packet with a compressed packet
 
-   @param [in] comp_ctx     Compress context
-   @param [in, out] packet  Data to compress. This is replaced with the
-                            compressed data.
-   @param [in] len          Length of data to compress at 'packet'
-   @param [out] complen     Compressed packet length. 0, if packet was not
-                            compressed
-   @retval 1   error. 'len' is not changed
-   @retval 0   ok.    'len' contains the size of the compressed packet
+   SYNOPSIS
+     my_compress()
+     packet   Data to compress. This is is replaced with the compressed data.
+     len      Length of data to compress at 'packet'
+     complen  out: 0 if packet was not compressed
+
+   RETURN
+     1   error. 'len' is not changed'
+     0   ok.  In this case 'len' contains the size of the compressed packet
 */
 
 bool my_compress(mysql_compress_context *comp_ctx, uchar *packet, size_t *len,
@@ -304,18 +304,18 @@ uchar *my_compress_alloc(mysql_compress_context *comp_ctx, const uchar *packet,
     return nullptr;
   }
 
-  assert(comp_ctx->algorithm == enum_compression_algorithm::MYSQL_ZLIB);
+  DBUG_ASSERT(comp_ctx->algorithm == enum_compression_algorithm::MYSQL_ZLIB);
   return zlib_compress_alloc(&comp_ctx->u.zlib_ctx, packet, len, complen);
 }
 
-/**
+/*
   Uncompress packet
 
   @param comp_ctx      Pointer to compression context.
   @param packet        Compressed data. This is is replaced with the original
                        data.
   @param len           Length of compressed data
-  @param[out] complen  Length of the packet buffer after uncompression (must be
+  @param complen [out] Length of the packet buffer after uncompression (must be
                        enough for the original data)
 
   @return true on error else false on success
@@ -324,7 +324,7 @@ uchar *my_compress_alloc(mysql_compress_context *comp_ctx, const uchar *packet,
 bool my_uncompress(mysql_compress_context *comp_ctx, uchar *packet, size_t len,
                    size_t *complen) {
   DBUG_ENTER("my_uncompress");
-  assert(comp_ctx != nullptr);
+  DBUG_ASSERT(comp_ctx != nullptr);
 
   if (*complen) /* If compressed */
   {
@@ -356,7 +356,7 @@ unsigned int mysql_default_compression_level(
     case MYSQL_ZSTD:
       return 3;
     default:
-      assert(0);  // should not reach here.
-      return 0;   // To make compiler happy.
+      DBUG_ASSERT(0);  // should not reach here.
+      return 0;        // To make compiler happy.
   }
 }
