@@ -1,16 +1,15 @@
-/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    Without limiting anything contained in the foregoing, this file,
    which is part of C Driver for MySQL (Connector/C), is also subject to the
@@ -28,7 +27,7 @@
 
 /**
   @file mysys/thr_lock.cc
-Read and write locks for Posix threads. Every thread must acquire
+Read and write locks for Posix threads. All tread must acquire
 all locks it needs through thr_multi_lock() to avoid dead-locks.
 A lock consists of a master lock (THR_LOCK), and lock instances
 (THR_LOCK_DATA).
@@ -92,7 +91,6 @@ lock at the same time as multiple read locks.
 #include <unistd.h>
 #endif
 
-#include "m_string.h"
 #include "my_compiler.h"
 #include "my_dbug.h"
 #include "my_inttypes.h"
@@ -209,7 +207,7 @@ static void check_locks(THR_LOCK *lock, const char *where,
       for (data = lock->read.data; data; data = data->next) {
         if ((int)data->type == (int)TL_READ_NO_INSERT) count++;
         /* Protect against infinite loop. */
-        assert(count <= lock->read_no_write_count);
+        DBUG_ASSERT(count <= lock->read_no_write_count);
       }
       if (count != lock->read_no_write_count) {
         found_errors++;
@@ -368,8 +366,8 @@ static enum enum_thr_lock_result wait_for_lock(struct st_lock_list *wait,
     One can use this to signal when a thread is going to wait for a lock.
     See debug_sync.cc.
 
-    Beware of waiting for a signal here. The lock has acquired its mutex.
-    While waiting on a signal here, the locking thread could not acquire
+    Beware of waiting for a signal here. The lock has aquired its mutex.
+    While waiting on a signal here, the locking thread could not aquire
     the mutex to release the lock. One could lock up the table
     completely.
 
@@ -378,8 +376,8 @@ static enum enum_thr_lock_result wait_for_lock(struct st_lock_list *wait,
     if not, it calls wait_for_lock(). Here it unlocks the table lock
     while waiting on a condition. The sync point is located before this
     wait for condition. If we have a waiting action here, we hold the
-    table locks mutex all the time. Any attempt to look at the table
-    lock from another thread blocks it immediately on lock->mutex. This
+    the table locks mutex all the time. Any attempt to look at the table
+    lock by another thread blocks it immediately on lock->mutex. This
     can easily become an unexpected and unobvious blockage. So be
     warned: Do not request a WAIT_FOR action for the 'wait_for_lock'
     sync point unless you really know what you do.
@@ -416,7 +414,7 @@ static enum enum_thr_lock_result wait_for_lock(struct st_lock_list *wait,
 
   set_timespec(&wait_timeout, lock_wait_timeout);
   while (!is_killed_hook(nullptr) || in_wait_list) {
-    const int rc =
+    int rc =
         mysql_cond_timedwait(data->cond, &data->lock->mutex, &wait_timeout);
     /*
       We must break the wait if one of the following occurs:
@@ -518,8 +516,8 @@ enum enum_thr_lock_result thr_lock(THR_LOCK_DATA *data, THR_LOCK_INFO *owner,
            |\  = READ_WITH_SHARED_LOCKS
            \   = READ
 
-        + = Request can be satisfied.
-        - = Request cannot be satisfied.
+        + = Request can be satisified.
+        - = Request cannot be satisified.
 
         READ_NO_INSERT and WRITE_ALLOW_WRITE should in principle
         be incompatible. Before this could have caused starvation of
@@ -617,10 +615,10 @@ enum enum_thr_lock_result thr_lock(THR_LOCK_DATA *data, THR_LOCK_INFO *owner,
         tries to update t1, is an example of statement which requests two
         different types of write lock on the same table).
       */
-      assert(!has_old_lock(lock->write.data, data->owner) ||
-             ((lock_type <= lock->write.data->type ||
-               (lock_type == TL_WRITE &&
-                lock->write.data->type == TL_WRITE_LOW_PRIORITY))));
+      DBUG_ASSERT(!has_old_lock(lock->write.data, data->owner) ||
+                  ((lock_type <= lock->write.data->type ||
+                    (lock_type == TL_WRITE &&
+                     lock->write.data->type == TL_WRITE_LOW_PRIORITY))));
 
       if ((lock_type == TL_WRITE_ALLOW_WRITE && !lock->write_wait.data &&
            lock->write.data->type == TL_WRITE_ALLOW_WRITE) ||
@@ -733,7 +731,7 @@ static inline void free_all_read_locks(THR_LOCK *lock,
 
 void thr_unlock(THR_LOCK_DATA *data) {
   THR_LOCK *lock = data->lock;
-  const enum thr_lock_type lock_type = data->type;
+  enum thr_lock_type lock_type = data->type;
   DBUG_TRACE;
   DBUG_PRINT("lock", ("data: %p  thread: 0x%x  lock: %p", data,
                       data->owner->thread_id, lock));
@@ -870,7 +868,7 @@ end:
 
 /*
 ** Get all locks in a specific order to avoid dead-locks
-** Sort according to lock position and put write_locks before read_locks if
+** Sort acording to lock position and put write_locks before read_locks if
 ** lock on same lock.
 */
 
@@ -904,7 +902,7 @@ enum enum_thr_lock_result thr_multi_lock(THR_LOCK_DATA **data, uint count,
   if (count > 1) sort_locks(data, count);
   /* lock everything */
   for (pos = data, end = data + count; pos < end; pos++) {
-    const enum enum_thr_lock_result result =
+    enum enum_thr_lock_result result =
         thr_lock(*pos, owner, (*pos)->type, lock_wait_timeout);
     if (result != THR_LOCK_SUCCESS) { /* Aborted */
       thr_multi_unlock(data, (uint)(pos - data));
@@ -999,11 +997,8 @@ void thr_multi_unlock(THR_LOCK_DATA **data, uint count) {
     if ((*pos)->type != TL_UNLOCK)
       thr_unlock(*pos);
     else {
-      DBUG_PRINT("lock", ("Free lock: data: %p  lock: %p", *pos, (*pos)->lock));
-      if ((*pos)->owner) {
-        DBUG_PRINT("lock",
-                   ("Free lock: thread: 0x%x", (*pos)->owner->thread_id));
-      }
+      DBUG_PRINT("lock", ("Free lock: data: %p  thread: 0x%x  lock: %p", *pos,
+                          (*pos)->owner->thread_id, (*pos)->lock));
     }
   }
 }
@@ -1176,15 +1171,17 @@ static ulong sum = 0;
 
 /* The following functions is for WRITE_CONCURRENT_INSERT */
 
-static void test_get_status(void *param [[maybe_unused]],
-                            int concurrent_insert [[maybe_unused]]) {}
+static void test_get_status(void *param MY_ATTRIBUTE((unused)),
+                            int concurrent_insert MY_ATTRIBUTE((unused))) {}
 
-static void test_update_status(void *param [[maybe_unused]]) {}
+static void test_update_status(void *param MY_ATTRIBUTE((unused))) {}
 
-static void test_copy_status(void *to [[maybe_unused]],
-                             void *from [[maybe_unused]]) {}
+static void test_copy_status(void *to MY_ATTRIBUTE((unused)),
+                             void *from MY_ATTRIBUTE((unused))) {}
 
-static bool test_check_status(void *param [[maybe_unused]]) { return false; }
+static bool test_check_status(void *param MY_ATTRIBUTE((unused))) {
+  return false;
+}
 
 static void *test_thread(void *arg) {
   int i, j, param = *((int *)arg);
@@ -1192,7 +1189,7 @@ static void *test_thread(void *arg) {
   THR_LOCK_INFO lock_info;
   THR_LOCK_DATA *multi_locks[MAX_LOCK_COUNT];
   my_thread_id id;
-  mysql_cond_t COND_thr_lock{};
+  mysql_cond_t COND_thr_lock;
 
   id = param + 1; /* Main thread uses value 0. */
   mysql_cond_init(0, &COND_thr_lock);
