@@ -1,16 +1,15 @@
-/* Copyright (c) 2010, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,16 +27,12 @@
 #include <sys/types.h>
 
 #include "lex_string.h"
-#include "m_string.h"
+#include "m_ctype.h"
 #include "my_compiler.h"
 #include "my_inttypes.h"
 #include "my_macros.h"
 #include "my_sys.h"
 #include "mysql/psi/mysql_mutex.h"
-#include "mysql/strings/m_ctype.h"
-#include "nulls.h"
-#include "string_with_len.h"
-#include "strxnmov.h"
 #include "thr_mutex.h"
 
 /** Event strings. */
@@ -162,12 +157,6 @@ static SHOW_VAR simple_status[] = {
 
     {nullptr, nullptr, SHOW_UNDEF, SHOW_SCOPE_GLOBAL}};
 
-static void increment_counter(volatile int *counter) {
-  int value = *counter;
-  int new_value = value + 1;
-  *counter = new_value;
-}
-
 /*
   Define plugin variables.
 */
@@ -221,7 +210,7 @@ static MYSQL_THDVAR_STR(event_record,
     1                    failure (cannot happen)
 */
 
-static int audit_null_plugin_init(void *arg [[maybe_unused]]) {
+static int audit_null_plugin_init(void *arg MY_ATTRIBUTE((unused))) {
   SHOW_VAR *var;
 
   for (var = simple_status; var->value != nullptr; var++) {
@@ -250,8 +239,7 @@ static int audit_null_plugin_init(void *arg [[maybe_unused]]) {
 
 */
 
-static int audit_null_plugin_deinit(void *arg [[maybe_unused]]) {
-  assert(arg);
+static int audit_null_plugin_deinit(void *arg MY_ATTRIBUTE((unused))) {
   if (g_plugin_installed == true) {
     my_free((void *)(g_record_buffer));
 
@@ -332,8 +320,8 @@ static char *add_event(const char *var, LEX_CSTRING event, const char *data,
 static void process_event_record(MYSQL_THD thd, LEX_CSTRING event_name,
                                  const char *data, size_t data_length) {
   char *record_str = THDVAR(thd, event_record_def);
-  const LEX_CSTRING record_begin = get_token(&record_str);
-  const LEX_CSTRING record_end = get_token(&record_str);
+  LEX_CSTRING record_begin = get_token(&record_str);
+  LEX_CSTRING record_end = get_token(&record_str);
 
   if (record_str == nullptr) {
     return;
@@ -404,15 +392,15 @@ static void process_event_record(MYSQL_THD thd, LEX_CSTRING event_name,
 
 static int process_command(MYSQL_THD thd, LEX_CSTRING event_command,
                            bool consume_event) {
-  const LEX_CSTRING abort_ret_command = {STRING_WITH_LEN("ABORT_RET")};
+  LEX_CSTRING abort_ret_command = {STRING_WITH_LEN("ABORT_RET")};
 
   if (!my_charset_latin1.coll->strnncoll(
           &my_charset_latin1, (const uchar *)event_command.str,
           event_command.length, (const uchar *)abort_ret_command.str,
           abort_ret_command.length, false)) {
-    const int ret_code = (int)THDVAR(thd, abort_value);
+    int ret_code = (int)THDVAR(thd, abort_value);
     const char *err_message = (const char *)THDVAR(thd, abort_message);
-    const LEX_CSTRING status = {STRING_WITH_LEN("EVENT-ORDER-ABORT")};
+    LEX_CSTRING status = {STRING_WITH_LEN("EVENT-ORDER-ABORT")};
 
     char *order_str = THDVAR(thd, event_order_check);
 
@@ -451,19 +439,19 @@ static int audit_null_notify(MYSQL_THD thd, mysql_event_class_t event_class,
       0,
   };
   int buffer_data = 0;
-  const unsigned long event_subclass =
+  unsigned long event_subclass =
       static_cast<unsigned long>(*static_cast<const int *>(event));
   char *order_str = THDVAR(thd, event_order_check);
-  const int event_order_started = (int)THDVAR(thd, event_order_started);
-  const int exact_check = (int)THDVAR(thd, event_order_check_exact);
-  const LEX_CSTRING event_name = event_to_str(event_class, event_subclass);
-  const LEX_CSTRING event_token = get_token(&order_str);
-  const LEX_CSTRING event_data = get_token(&order_str);
+  int event_order_started = (int)THDVAR(thd, event_order_started);
+  int exact_check = (int)THDVAR(thd, event_order_check_exact);
+  LEX_CSTRING event_name = event_to_str(event_class, event_subclass);
+  LEX_CSTRING event_token = get_token(&order_str);
+  LEX_CSTRING event_data = get_token(&order_str);
   LEX_CSTRING event_command = get_token(&order_str);
   bool consume_event = true;
 
   /* prone to races, oh well */
-  increment_counter(&number_of_calls);
+  number_of_calls++;
 
   if (event_class == MYSQL_AUDIT_GENERAL_CLASS) {
     const struct mysql_event_general *event_general =
@@ -471,16 +459,16 @@ static int audit_null_notify(MYSQL_THD thd, mysql_event_class_t event_class,
 
     switch (event_general->event_subclass) {
       case MYSQL_AUDIT_GENERAL_LOG:
-        increment_counter(&number_of_calls_general_log);
+        number_of_calls_general_log++;
         break;
       case MYSQL_AUDIT_GENERAL_ERROR:
-        increment_counter(&number_of_calls_general_error);
+        number_of_calls_general_error++;
         break;
       case MYSQL_AUDIT_GENERAL_RESULT:
-        increment_counter(&number_of_calls_general_result);
+        number_of_calls_general_result++;
         break;
       case MYSQL_AUDIT_GENERAL_STATUS:
-        increment_counter(&number_of_calls_general_status);
+        number_of_calls_general_status++;
         break;
       default:
         break;
@@ -491,16 +479,16 @@ static int audit_null_notify(MYSQL_THD thd, mysql_event_class_t event_class,
 
     switch (event_connection->event_subclass) {
       case MYSQL_AUDIT_CONNECTION_CONNECT:
-        increment_counter(&number_of_calls_connection_connect);
+        number_of_calls_connection_connect++;
         break;
       case MYSQL_AUDIT_CONNECTION_DISCONNECT:
-        increment_counter(&number_of_calls_connection_disconnect);
+        number_of_calls_connection_disconnect++;
         break;
       case MYSQL_AUDIT_CONNECTION_CHANGE_USER:
-        increment_counter(&number_of_calls_connection_change_user);
+        number_of_calls_connection_change_user++;
         break;
       case MYSQL_AUDIT_CONNECTION_PRE_AUTHENTICATE:
-        increment_counter(&number_of_calls_connection_pre_authenticate);
+        number_of_calls_connection_pre_authenticate++;
         break;
       default:
         break;
@@ -511,10 +499,10 @@ static int audit_null_notify(MYSQL_THD thd, mysql_event_class_t event_class,
 
     switch (event_parse->event_subclass) {
       case MYSQL_AUDIT_PARSE_PREPARSE:
-        increment_counter(&number_of_calls_parse_preparse);
+        number_of_calls_parse_preparse++;
         break;
       case MYSQL_AUDIT_PARSE_POSTPARSE:
-        increment_counter(&number_of_calls_parse_postparse);
+        number_of_calls_parse_postparse++;
         break;
       default:
         break;
@@ -566,24 +554,24 @@ static int audit_null_notify(MYSQL_THD thd, mysql_event_class_t event_class,
   else if (event_class == MYSQL_AUDIT_SERVER_STARTUP_CLASS) {
     /* const struct mysql_event_server_startup *event_startup=
        (const struct mysql_event_server_startup *) event; */
-    increment_counter(&number_of_calls_server_startup);
+    number_of_calls_server_startup++;
   } else if (event_class == MYSQL_AUDIT_SERVER_SHUTDOWN_CLASS) {
     /* const struct mysql_event_server_shutdown *event_startup=
        (const struct mysql_event_server_shutdown *) event; */
-    increment_counter(&number_of_calls_server_shutdown);
+    number_of_calls_server_shutdown++;
   } else if (event_class == MYSQL_AUDIT_COMMAND_CLASS) {
-    const struct mysql_event_command *local_event_command =
+    const struct mysql_event_command *event_command =
         (const struct mysql_event_command *)event;
 
     buffer_data =
-        sprintf(buffer, "command_id=\"%d\"", local_event_command->command_id);
+        sprintf(buffer, "command_id=\"%d\"", event_command->command_id);
 
-    switch (local_event_command->event_subclass) {
+    switch (event_command->event_subclass) {
       case MYSQL_AUDIT_COMMAND_START:
-        increment_counter(&number_of_calls_command_start);
+        number_of_calls_command_start++;
         break;
       case MYSQL_AUDIT_COMMAND_END:
-        increment_counter(&number_of_calls_command_end);
+        number_of_calls_command_end++;
         break;
       default:
         break;
@@ -597,16 +585,16 @@ static int audit_null_notify(MYSQL_THD thd, mysql_event_class_t event_class,
 
     switch (event_query->event_subclass) {
       case MYSQL_AUDIT_QUERY_START:
-        increment_counter(&number_of_calls_query_start);
+        number_of_calls_query_start++;
         break;
       case MYSQL_AUDIT_QUERY_NESTED_START:
-        increment_counter(&number_of_calls_query_nested_start);
+        number_of_calls_query_nested_start++;
         break;
       case MYSQL_AUDIT_QUERY_STATUS_END:
-        increment_counter(&number_of_calls_query_status_end);
+        number_of_calls_query_status_end++;
         break;
       case MYSQL_AUDIT_QUERY_NESTED_STATUS_END:
-        increment_counter(&number_of_calls_query_nested_status_end);
+        number_of_calls_query_nested_status_end++;
         break;
       default:
         break;
@@ -621,16 +609,16 @@ static int audit_null_notify(MYSQL_THD thd, mysql_event_class_t event_class,
 
     switch (event_table->event_subclass) {
       case MYSQL_AUDIT_TABLE_ACCESS_INSERT:
-        increment_counter(&number_of_calls_table_access_insert);
+        number_of_calls_table_access_insert++;
         break;
       case MYSQL_AUDIT_TABLE_ACCESS_DELETE:
-        increment_counter(&number_of_calls_table_access_delete);
+        number_of_calls_table_access_delete++;
         break;
       case MYSQL_AUDIT_TABLE_ACCESS_UPDATE:
-        increment_counter(&number_of_calls_table_access_update);
+        number_of_calls_table_access_update++;
         break;
       case MYSQL_AUDIT_TABLE_ACCESS_READ:
-        increment_counter(&number_of_calls_table_access_read);
+        number_of_calls_table_access_read++;
         break;
       default:
         break;
@@ -657,10 +645,10 @@ static int audit_null_notify(MYSQL_THD thd, mysql_event_class_t event_class,
 
     switch (event_gvar->event_subclass) {
       case MYSQL_AUDIT_GLOBAL_VARIABLE_GET:
-        increment_counter(&number_of_calls_global_variable_get);
+        number_of_calls_global_variable_get++;
         break;
       case MYSQL_AUDIT_GLOBAL_VARIABLE_SET:
-        increment_counter(&number_of_calls_global_variable_set);
+        number_of_calls_global_variable_set++;
         break;
       default:
         break;
@@ -707,10 +695,10 @@ static int audit_null_notify(MYSQL_THD thd, mysql_event_class_t event_class,
 
     switch (evt->event_subclass) {
       case MYSQL_AUDIT_MESSAGE_INTERNAL:
-        increment_counter(&number_of_calls_message_internal);
+        number_of_calls_message_internal++;
         break;
       case MYSQL_AUDIT_MESSAGE_USER:
-        increment_counter(&number_of_calls_message_user);
+        number_of_calls_message_user++;
         break;
       default:
         break;
@@ -740,7 +728,7 @@ static int audit_null_notify(MYSQL_THD thd, mysql_event_class_t event_class,
       return 1;
     }
   } else {
-    const LEX_CSTRING ignore = {STRING_WITH_LEN("<IGNORE>")};
+    LEX_CSTRING ignore = {STRING_WITH_LEN("<IGNORE>")};
 
     /* When we are not in the event order check, check if the specified
        data corresponds to the actual event data. */
@@ -756,19 +744,18 @@ static int audit_null_notify(MYSQL_THD thd, mysql_event_class_t event_class,
         char invalid_data_buffer[sizeof(buffer)] = {
             0,
         };
-        const LEX_CSTRING status = {
-            STRING_WITH_LEN("EVENT-ORDER-INVALID-DATA")};
+        LEX_CSTRING status = {STRING_WITH_LEN("EVENT-ORDER-INVALID-DATA")};
 
-        char *local_order_str = THDVAR(thd, event_order_check);
+        char *order_str = THDVAR(thd, event_order_check);
 
-        memmove(local_order_str, status.str, status.length + 1);
+        memmove(order_str, status.str, status.length + 1);
 
         strxnmov(invalid_data_buffer, sizeof(invalid_data_buffer),
                  "Invalid data for '", event_name.str, "' -> ", buffer, NullS);
         my_message(ER_AUDIT_API_ABORT, invalid_data_buffer, MYF(0));
 
         THDVAR(thd, event_order_started) = 0;
-        THDVAR(thd, event_order_check) = local_order_str;
+        THDVAR(thd, event_order_check) = order_str;
 
         return 1;
       }
@@ -778,7 +765,7 @@ static int audit_null_notify(MYSQL_THD thd, mysql_event_class_t event_class,
       event_command.length = 0;
     } else {
       LEX_STRING order_cstr;
-      const ulong consume = THDVAR(thd, event_order_check_consume_ignore_count);
+      ulong consume = THDVAR(thd, event_order_check_consume_ignore_count);
       lex_string_set(&order_cstr, THDVAR(thd, event_order_check));
 
       THDVAR(thd, event_order_started) = 1;
@@ -799,7 +786,7 @@ static int audit_null_notify(MYSQL_THD thd, mysql_event_class_t event_class,
         lex_string_set(&order_cstr, order_cstr.str);
 
         if (order_cstr.length == 0) {
-          const LEX_CSTRING status = {STRING_WITH_LEN("EVENT-ORDER-OK")};
+          LEX_CSTRING status = {STRING_WITH_LEN("EVENT-ORDER-OK")};
 
           memmove(order_cstr.str, status.str, status.length + 1);
 
