@@ -1,4 +1,4 @@
-/* Copyright (c) 2014, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -44,12 +44,11 @@
 
 // Define the applier packet types
 #define ACTION_PACKET_TYPE 2
+#define VIEW_CHANGE_PACKET_TYPE 3
 #define SINGLE_PRIMARY_PACKET_TYPE 4
 #define SYNC_BEFORE_EXECUTION_PACKET_TYPE 5
 #define TRANSACTION_PREPARED_PACKET_TYPE 6
 #define LEAVING_MEMBERS_PACKET_TYPE 7
-#define RECOVERY_METADATA_PROCESSING_PACKET_TYPE 8
-#define ERROR_PACKET_TYPE 9  ///< Make the applier fail
 
 // Define the applier return error codes
 #define APPLIER_GTID_CHECK_TIMEOUT_ERROR -1
@@ -79,35 +78,29 @@ class Action_packet : public Packet {
   Action_packet(enum_packet_action action)
       : Packet(ACTION_PACKET_TYPE), packet_action(action) {}
 
-  ~Action_packet() override = default;
+  ~Action_packet() {}
 
   enum_packet_action packet_action;
 };
 
 /**
-  @class Recovery_metadata_processing_packets
-  A packet to send Metadata related processing.
+  @class View_change_packet
+  A packet to send view change related info to the applier
 */
-class Recovery_metadata_processing_packets : public Packet {
+class View_change_packet : public Packet {
  public:
   /**
     Create a new data packet with associated data.
+
+    @param  view_id_arg    The view id associated to this view
   */
-  Recovery_metadata_processing_packets()
-      : Packet(RECOVERY_METADATA_PROCESSING_PACKET_TYPE) {}
+  View_change_packet(std::string &view_id_arg)
+      : Packet(VIEW_CHANGE_PACKET_TYPE), view_id(view_id_arg) {}
 
-  virtual ~Recovery_metadata_processing_packets() override = default;
+  ~View_change_packet() {}
 
-  /*
-    List of view-id of which metadata is received.
-  */
-  std::vector<std::string> m_view_id_to_be_deleted;
-
-  /* List of member that left the group. */
-  std::vector<Gcs_member_identifier> m_member_left_the_group;
-
-  /* A flag that indicates all the recovery metadata should be cleared. */
-  bool m_current_member_leaving_the_group{false};
+  std::string view_id;
+  std::vector<std::string> group_executed_set;
 };
 
 /**
@@ -126,7 +119,7 @@ class Single_primary_action_packet : public Packet {
   Single_primary_action_packet(enum enum_action action_arg)
       : Packet(SINGLE_PRIMARY_PACKET_TYPE), action(action_arg) {}
 
-  ~Single_primary_action_packet() override = default;
+  virtual ~Single_primary_action_packet() {}
 
   enum enum_action action;
 };
@@ -161,40 +154,32 @@ class Transaction_prepared_action_packet : public Packet {
   /**
     Create a new transaction prepared action.
 
-    @param  tsid             the prepared transaction tsid
-    @param  is_tsid_specified information on whether tsid is specified
+    @param  sid              the prepared transaction sid
     @param  gno              the prepared transaction gno
     @param  gcs_member_id    the member id that did prepare the
                              transaction
   */
-  Transaction_prepared_action_packet(const gr::Gtid_tsid &tsid,
-                                     bool is_tsid_specified, rpl_gno gno,
+  Transaction_prepared_action_packet(const rpl_sid *sid, rpl_gno gno,
                                      const Gcs_member_identifier &gcs_member_id)
       : Packet(TRANSACTION_PREPARED_PACKET_TYPE),
-        m_tsid_specified(is_tsid_specified),
+        m_sid_specified(sid != nullptr ? true : false),
         m_gno(gno),
         m_gcs_member_id(gcs_member_id.get_member_id()) {
-    if (m_tsid_specified) {
-      m_tsid = tsid;
+    if (sid != nullptr) {
+      m_sid.copy_from(*sid);
     }
   }
 
-  ~Transaction_prepared_action_packet() override = default;
+  virtual ~Transaction_prepared_action_packet() {}
 
-  const bool m_tsid_specified;
+  const bool m_sid_specified;
   const rpl_gno m_gno;
   const Gcs_member_identifier m_gcs_member_id;
 
-  /// @brief tsid accessor
-  /// @return tsid const reference
-  const gr::Gtid_tsid &get_tsid() const { return m_tsid; }
-
-  /// @brief returns information on whether TSID is specified for this trx
-  /// @return information on whether TSID is specified for this trx
-  bool is_tsid_specified() const { return m_tsid_specified; }
+  const rpl_sid *get_sid() { return m_sid_specified ? &m_sid : nullptr; }
 
  private:
-  gr::Gtid_tsid m_tsid;
+  rpl_sid m_sid;
 };
 
 /**
@@ -216,7 +201,7 @@ class Sync_before_execution_action_packet : public Packet {
         m_thread_id(thread_id),
         m_gcs_member_id(gcs_member_id.get_member_id()) {}
 
-  ~Sync_before_execution_action_packet() override = default;
+  virtual ~Sync_before_execution_action_packet() {}
 
   const my_thread_id m_thread_id;
   const Gcs_member_identifier m_gcs_member_id;
@@ -240,32 +225,11 @@ class Leaving_members_action_packet : public Packet {
       : Packet(LEAVING_MEMBERS_PACKET_TYPE),
         m_leaving_members(leaving_members) {}
 
-  ~Leaving_members_action_packet() override = default;
+  virtual ~Leaving_members_action_packet() {}
 
   const std::vector<Gcs_member_identifier> m_leaving_members;
 };
 
-/// @class Error_action_packet
-/// A packet to inform the applier it should fail.
-/// It should include a message about the failure
-class Error_action_packet : public Packet {
- public:
-  /// Create a new error packet.
-  /// @param  error_message  the error that will make the applier stop
-  Error_action_packet(const char *error_message)
-      : Packet(ERROR_PACKET_TYPE), m_error_message(error_message) {}
-
-  ~Error_action_packet() override = default;
-
-  /// @brief Returns the error message for the failure
-  /// @return the error message
-  const char *get_error_message() { return m_error_message; }
-
- private:
-  /// The error message for the failure process
-  const char *m_error_message;
-};
-
 typedef enum enum_applier_state {
   APPLIER_STATE_ON = 1,
   APPLIER_STATE_OFF,
@@ -274,7 +238,7 @@ typedef enum enum_applier_state {
 
 class Applier_module_interface {
  public:
-  virtual ~Applier_module_interface() = default;
+  virtual ~Applier_module_interface() {}
   virtual Certification_handler *get_certification_handler() = 0;
   virtual int wait_for_applier_complete_suspension(
       bool *abort_flag, bool wait_for_execution = true) = 0;
@@ -292,10 +256,7 @@ class Applier_module_interface {
   virtual size_t get_message_queue_size() = 0;
   virtual Member_applier_state get_applier_status() = 0;
   virtual void add_suspension_packet() = 0;
-  virtual void add_packet(Packet *packet) = 0;
   virtual void add_view_change_packet(View_change_packet *packet) = 0;
-  virtual void add_metadata_processing_packet(
-      Recovery_metadata_processing_packets *packet) = 0;
   virtual void add_single_primary_action_packet(
       Single_primary_action_packet *packet) = 0;
   virtual void add_transaction_prepared_action_packet(
@@ -306,8 +267,7 @@ class Applier_module_interface {
       Leaving_members_action_packet *packet) = 0;
   virtual int handle(const uchar *data, ulong len,
                      enum_group_replication_consistency_level consistency_level,
-                     std::list<Gcs_member_identifier> *online_members,
-                     PSI_memory_key key) = 0;
+                     std::list<Gcs_member_identifier> *online_members) = 0;
   virtual int handle_pipeline_action(Pipeline_action *action) = 0;
   virtual Flow_control_module *get_flow_control_module() = 0;
   virtual void run_flow_control_step() = 0;
@@ -321,7 +281,7 @@ class Applier_module_interface {
 class Applier_module : public Applier_module_interface {
  public:
   Applier_module();
-  ~Applier_module() override;
+  ~Applier_module();
 
   /**
     Initializes and launches the applier thread
@@ -354,7 +314,7 @@ class Applier_module : public Applier_module_interface {
       @retval !=0    yes
   */
   bool is_applier_thread_aborted() {
-    return (applier_aborted || applier_thd == nullptr || applier_thd->killed);
+    return (applier_aborted || applier_thd->killed);
   }
 
   /**
@@ -374,6 +334,7 @@ class Applier_module : public Applier_module_interface {
     @param[in] stop_timeout               the timeout when waiting on shutdown
     @param[in] group_sidno                the group configured sidno
     @param[in] gtid_assignment_block_size the group gtid assignment block size
+    @param[in] shared_stop_lock           the lock used to block transactions
 
     @return the operation status
       @retval 0      OK
@@ -381,7 +342,8 @@ class Applier_module : public Applier_module_interface {
   */
   int setup_applier_module(Handler_pipeline_type pipeline_type, bool reset_logs,
                            ulong stop_timeout, rpl_sidno group_sidno,
-                           ulonglong gtid_assignment_block_size);
+                           ulonglong gtid_assignment_block_size,
+                           Shared_writelock *shared_stop_lock);
 
   /**
     Configure the applier pipeline handlers
@@ -399,7 +361,7 @@ class Applier_module : public Applier_module_interface {
       @retval 0      OK
       @retval !=0    Error
   */
-  int purge_applier_queue_and_restart_applier_module() override;
+  virtual int purge_applier_queue_and_restart_applier_module();
 
   /**
     Runs the applier thread process, reading events and processing them.
@@ -421,7 +383,6 @@ class Applier_module : public Applier_module_interface {
     @param[in]  consistency_level  the transaction consistency level
     @param[in]  online_members     the ONLINE members when the transaction
                                    message was delivered
-    @param[in]  key       the memory instrument key
 
     @return the operation status
       @retval 0      OK
@@ -429,10 +390,9 @@ class Applier_module : public Applier_module_interface {
   */
   int handle(const uchar *data, ulong len,
              enum_group_replication_consistency_level consistency_level,
-             std::list<Gcs_member_identifier> *online_members,
-             PSI_memory_key key) override {
+             std::list<Gcs_member_identifier> *online_members) {
     this->incoming->push(
-        new Data_packet(data, len, key, consistency_level, online_members));
+        new Data_packet(data, len, consistency_level, online_members));
     return 0;
   }
 
@@ -445,7 +405,7 @@ class Applier_module : public Applier_module_interface {
       @retval 0      OK
       @retval !=0    Error executing the action
   */
-  int handle_pipeline_action(Pipeline_action *action) override {
+  int handle_pipeline_action(Pipeline_action *action) {
     return this->pipeline->handle_action(action);
   }
 
@@ -493,18 +453,6 @@ class Applier_module : public Applier_module_interface {
   */
   void inform_of_applier_stop(char *channel_name, bool aborted);
 
-  /**
-   Check whether to ignore applier errors during stop or not.
-   Errors put the members into ERROR state.
-   If errors are ignored member will stay in ONLINE state.
-   During clone, applier errors are ignored, since data will come from clone.
-
-    @param[in]  ignore_errors  if true ignore applier errors during stop
-  */
-  void ignore_errors_during_stop(bool ignore_errors) {
-    m_ignore_applier_errors_during_stop = ignore_errors;
-  }
-
   // Packet based interface methods
 
   /**
@@ -513,7 +461,7 @@ class Applier_module : public Applier_module_interface {
 
     @note This will happen only after all the previous packets are processed.
   */
-  void add_suspension_packet() override {
+  virtual void add_suspension_packet() {
     this->incoming->push(new Action_packet(SUSPENSION_PACKET));
   }
 
@@ -528,10 +476,6 @@ class Applier_module : public Applier_module_interface {
     this->incoming->push(new Action_packet(TERMINATION_PACKET));
   }
 
-  /// @brief Generic add packet method
-  /// @param packet the packet to be queued in the applier
-  void add_packet(Packet *packet) override { incoming->push(packet); }
-
   /**
     Queues a view change packet into the applier.
     This packets contain the new view id and they mark the exact frontier
@@ -541,12 +485,7 @@ class Applier_module : public Applier_module_interface {
 
     @param[in]  packet              The view change packet to be queued
   */
-  void add_view_change_packet(View_change_packet *packet) override {
-    incoming->push(packet);
-  }
-
-  void add_metadata_processing_packet(
-      Recovery_metadata_processing_packets *packet) override {
+  virtual void add_view_change_packet(View_change_packet *packet) {
     incoming->push(packet);
   }
 
@@ -557,8 +496,7 @@ class Applier_module : public Applier_module_interface {
 
     @param[in]  packet              The packet to be queued
   */
-  void add_single_primary_action_packet(
-      Single_primary_action_packet *packet) override {
+  void add_single_primary_action_packet(Single_primary_action_packet *packet) {
     incoming->push(packet);
   }
 
@@ -570,7 +508,7 @@ class Applier_module : public Applier_module_interface {
     @param[in]  packet              The packet to be queued
   */
   void add_transaction_prepared_action_packet(
-      Transaction_prepared_action_packet *packet) override {
+      Transaction_prepared_action_packet *packet) {
     incoming->push(packet);
   }
 
@@ -582,7 +520,7 @@ class Applier_module : public Applier_module_interface {
     @param[in]  packet              The packet to be queued
   */
   void add_sync_before_execution_action_packet(
-      Sync_before_execution_action_packet *packet) override {
+      Sync_before_execution_action_packet *packet) {
     incoming->push(packet);
   }
 
@@ -594,7 +532,7 @@ class Applier_module : public Applier_module_interface {
     @param[in]  packet              The packet to be queued
   */
   void add_leaving_members_action_packet(
-      Leaving_members_action_packet *packet) override {
+      Leaving_members_action_packet *packet) {
     incoming->push(packet);
   }
 
@@ -606,11 +544,11 @@ class Applier_module : public Applier_module_interface {
   /**
    Awakes the applier module
   */
-  void awake_applier_module() override {
+  virtual void awake_applier_module() {
     mysql_mutex_lock(&suspend_lock);
     suspended = false;
-    mysql_cond_broadcast(&suspend_cond);
     mysql_mutex_unlock(&suspend_lock);
+    mysql_cond_broadcast(&suspend_cond);
   }
 
   /**
@@ -626,14 +564,14 @@ class Applier_module : public Applier_module_interface {
      @retval 0      OK
      @retval !=0    Error when accessing the applier module status
   */
-  int wait_for_applier_complete_suspension(
-      bool *abort_flag, bool wait_for_execution = true) override;
+  virtual int wait_for_applier_complete_suspension(
+      bool *abort_flag, bool wait_for_execution = true);
 
   /**
    Interrupts the current applier waiting process either for it's suspension
    or it's wait for the consumption of pre suspension events
   */
-  void interrupt_applier_suspension_wait() override;
+  virtual void interrupt_applier_suspension_wait();
 
   /**
     Checks if the applier, and its workers when parallel applier is
@@ -665,8 +603,8 @@ class Applier_module : public Applier_module_interface {
       @retval -1     A timeout occurred
       @retval -2     An error occurred
   */
-  int wait_for_applier_event_execution(
-      double timeout, bool check_and_purge_partial_transactions) override;
+  virtual int wait_for_applier_event_execution(
+      double timeout, bool check_and_purge_partial_transactions);
 
   /**
     Waits for the execution of all current events by part of the current SQL
@@ -690,9 +628,9 @@ class Applier_module : public Applier_module_interface {
       @retval false    All transactions were executed
       @retval true     An error occurred
   */
-  bool wait_for_current_events_execution(
+  virtual bool wait_for_current_events_execution(
       std::shared_ptr<Continuation> checkpoint_condition, bool *abort_flag,
-      bool update_THD_status = true) override;
+      bool update_THD_status = true);
 
   /**
     Returns the retrieved gtid set for the applier channel
@@ -702,7 +640,7 @@ class Applier_module : public Applier_module_interface {
     @retval true there was an error.
     @retval false the operation has succeeded.
   */
-  bool get_retrieved_gtid_set(std::string &retrieved_set) override;
+  virtual bool get_retrieved_gtid_set(std::string &retrieved_set);
 
   /**
     Waits for the execution of all events in the given set by the current SQL
@@ -719,9 +657,9 @@ class Applier_module : public Applier_module_interface {
       @retval -1     A timeout occurred
       @retval -2     An error occurred
   */
-  int wait_for_applier_event_execution(std::string &retrieved_set,
-                                       double timeout,
-                                       bool update_THD_status = true) override;
+  virtual int wait_for_applier_event_execution(std::string &retrieved_set,
+                                               double timeout,
+                                               bool update_THD_status = true);
 
   /**
     Returns the handler instance in the applier module responsible for
@@ -733,16 +671,16 @@ class Applier_module : public Applier_module_interface {
       @retval !=NULL The certification handler
       @retval NULL   No certification handler present
   */
-  Certification_handler *get_certification_handler() override;
+  virtual Certification_handler *get_certification_handler();
 
   /**
     Returns the applier module's queue size.
 
     @return the size of the queue
   */
-  size_t get_message_queue_size() override { return incoming->size(); }
+  virtual size_t get_message_queue_size() { return incoming->size(); }
 
-  Member_applier_state get_applier_status() override {
+  virtual Member_applier_state get_applier_status() {
     if (applier_thd_state.is_running())
       return APPLIER_STATE_ON;
     else if (suspended)         /* purecov: inspected */
@@ -751,21 +689,20 @@ class Applier_module : public Applier_module_interface {
       return APPLIER_ERROR; /* purecov: inspected */
   }
 
-  Pipeline_stats_member_collector *get_pipeline_stats_member_collector()
-      override {
+  Pipeline_stats_member_collector *get_pipeline_stats_member_collector() {
     return &pipeline_stats_member_collector;
   }
 
-  Flow_control_module *get_flow_control_module() override {
+  Flow_control_module *get_flow_control_module() {
     return &flow_control_module;
   }
 
-  void run_flow_control_step() override {
+  virtual void run_flow_control_step() {
     flow_control_module.flow_control_step(&pipeline_stats_member_collector);
   }
 
-  bool queue_and_wait_on_queue_checkpoint(
-      std::shared_ptr<Continuation> checkpoint_condition) override;
+  virtual bool queue_and_wait_on_queue_checkpoint(
+      std::shared_ptr<Continuation> checkpoint_condition);
 
  private:
   // Applier packet handlers
@@ -796,18 +733,6 @@ class Applier_module : public Applier_module_interface {
                                Format_description_log_event *fde_evt,
                                Continuation *cont);
 
-  /**
-    Apply a Recovery metadata processing information received from the GCS.
-
-    @param metadata_processing_packet Information of member left the group
-
-    @return the operation status
-      @retval 0      OK
-      @retval !=0    Error when injecting event
-  */
-  int apply_metadata_processing_packet(
-      Recovery_metadata_processing_packets *metadata_processing_packet);
-
   /**
     Apply a Data packet received by the applier.
     It executes some certification operations and queues a View Change Event
@@ -945,8 +870,6 @@ class Applier_module : public Applier_module_interface {
   int applier_error;
   /* Applier killed status */
   bool applier_killed_status;
-  /* Ignore applier errors during stop. */
-  bool m_ignore_applier_errors_during_stop{false};
 
   // condition and lock used to suspend/awake the applier module
   /* The lock for suspending/wait for the awake of  the applier module */
@@ -955,10 +878,15 @@ class Applier_module : public Applier_module_interface {
   mysql_cond_t suspend_cond;
   /* Suspend flag that informs if the applier is suspended */
   bool suspended;
+  /* Suspend wait flag used when waiting for the applier to suspend */
+  bool waiting_for_applier_suspension;
 
   /* The condition for signaling the applier suspension*/
   mysql_cond_t suspension_waiting_condition;
 
+  /* The stop lock used when killing transaction/stopping server*/
+  Shared_writelock *shared_stop_write_lock;
+
   /* The incoming event queue */
   Synchronized_queue<Packet *> *incoming;
 
