@@ -1,16 +1,15 @@
-/* Copyright (c) 2014, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,44 +23,23 @@
 #ifndef CERTIFIER_INCLUDE
 #define CERTIFIER_INCLUDE
 
-#include <assert.h>
 #include <mysql/group_replication_priv.h>
-#include <atomic>
 #include <list>
 #include <map>
 #include <string>
 #include <unordered_map>
 #include <vector>
 
+#include "my_dbug.h"
 #include "my_inttypes.h"
-#include "plugin/group_replication/include/certification/certified_gtid.h"
-#include "plugin/group_replication/include/certification/gtid_generator.h"
-#include "plugin/group_replication/include/certification_result.h"
 #include "plugin/group_replication/include/certifier_stats_interface.h"
 #include "plugin/group_replication/include/gcs_plugin_messages.h"
-#include "plugin/group_replication/include/gr_compression.h"
-#include "plugin/group_replication/include/gr_decompression.h"
 #include "plugin/group_replication/include/member_info.h"
 #include "plugin/group_replication/include/pipeline_interfaces.h"
-#include "plugin/group_replication/include/plugin_messages/recovery_metadata_message.h"
 #include "plugin/group_replication/include/plugin_utils.h"
 #include "plugin/group_replication/libmysqlgcs/include/mysql/gcs/gcs_communication_interface.h"
 #include "plugin/group_replication/libmysqlgcs/include/mysql/gcs/gcs_control_interface.h"
 
-#include "plugin/group_replication/generated/protobuf_lite/replication_group_recovery_metadata.pb.h"
-
-#include "mysql/utils/return_status.h"
-
-/**
-  While sending Recovery Metadata the Certification Information is divided into
-  several small packets of MAX_COMPRESSED_PACKET_SIZE before sending it to
-  group for Recovery.
-  The compressed packet size is choosen as 10MB so that multiple threads can
-  process (serialize and compress or unserialize and decompress) packets
-  simultaneously without consuming too much memory.
-*/
-#define MAX_COMPRESSED_PACKET_SIZE 10485760
-
 /**
   This class extends Gtid_set to include a reference counter.
 
@@ -74,12 +52,12 @@
 */
 class Gtid_set_ref : public Gtid_set {
  public:
-  Gtid_set_ref(Tsid_map *tsid_map, int64 parallel_applier_sequence_number)
-      : Gtid_set(tsid_map),
+  Gtid_set_ref(Sid_map *sid_map, int64 parallel_applier_sequence_number)
+      : Gtid_set(sid_map),
         reference_counter(0),
         parallel_applier_sequence_number(parallel_applier_sequence_number) {}
 
-  virtual ~Gtid_set_ref() = default;
+  virtual ~Gtid_set_ref() {}
 
   /**
     Increment the number of references by one.
@@ -94,7 +72,7 @@ class Gtid_set_ref : public Gtid_set {
     @return the number of references
   */
   size_t unlink() {
-    assert(reference_counter > 0);
+    DBUG_ASSERT(reference_counter > 0);
     return --reference_counter;
   }
 
@@ -129,6 +107,8 @@ class Gtid_set_ref : public Gtid_set {
   negatively certified. Otherwise, this transaction is marked
   certified and goes into applier.
 */
+typedef std::unordered_map<std::string, Gtid_set_ref *> Certification_info;
+
 class Certifier_broadcast_thread {
  public:
   /**
@@ -193,7 +173,7 @@ class Certifier_broadcast_thread {
 
 class Certifier_interface : public Certifier_stats {
  public:
-  ~Certifier_interface() override = default;
+  virtual ~Certifier_interface() {}
   virtual void handle_view_change() = 0;
   virtual int handle_certifier_data(
       const uchar *data, ulong len,
@@ -201,34 +181,19 @@ class Certifier_interface : public Certifier_stats {
 
   virtual void get_certification_info(
       std::map<std::string, std::string> *cert_info) = 0;
-  virtual bool get_certification_info_recovery_metadata(
-      Recovery_metadata_message *recovery_metadata_message) = 0;
   virtual int set_certification_info(
       std::map<std::string, std::string> *cert_info) = 0;
-  virtual bool set_certification_info_recovery_metadata(
-      Recovery_metadata_message *recovery_metadata_message) = 0;
-  virtual bool initialize_server_gtid_set_after_distributed_recovery() = 0;
-  virtual void garbage_collect(Gtid_set *executed_gtid_set = nullptr,
-                               bool on_member_join = false) = 0;
+  virtual bool set_group_stable_transactions_set(
+      Gtid_set *executed_gtid_set) = 0;
   virtual void enable_conflict_detection() = 0;
   virtual void disable_conflict_detection() = 0;
   virtual bool is_conflict_detection_enable() = 0;
-  virtual ulonglong get_certification_info_size() override = 0;
 };
 
 class Certifier : public Certifier_interface {
  public:
-  typedef std::unordered_map<
-      std::string, Gtid_set_ref *, std::hash<std::string>,
-      std::equal_to<std::string>,
-      Malloc_allocator<std::pair<const std::string, Gtid_set_ref *>>>
-      Certification_info;
-
-  typedef protobuf_replication_group_recovery_metadata::
-      CertificationInformationMap ProtoCertificationInformationMap;
-
   Certifier();
-  ~Certifier() override;
+  virtual ~Certifier();
 
   /**
     Key used to store errors in the certification info
@@ -259,7 +224,7 @@ class Certifier : public Certifier_interface {
   /**
     Handle view changes on certifier.
    */
-  void handle_view_change() override;
+  virtual void handle_view_change();
 
   /**
     Queues the packet coming from the reader for future processing.
@@ -272,9 +237,8 @@ class Certifier : public Certifier_interface {
       @retval 0      OK
       @retval !=0    Error on queue
   */
-  int handle_certifier_data(
-      const uchar *data, ulong len,
-      const Gcs_member_identifier &gcs_member_id) override;
+  virtual int handle_certifier_data(const uchar *data, ulong len,
+                                    const Gcs_member_identifier &gcs_member_id);
 
   /**
     This member function SHALL certify the set of items against transactions
@@ -282,7 +246,8 @@ class Certifier : public Certifier_interface {
 
     @param snapshot_version   The incoming transaction snapshot version.
     @param write_set          The incoming transaction write set.
-    @param is_gtid_specified  True in case GTID is specified for this trx
+    @param generate_group_id  Flag that indicates if transaction group id
+                              must be generated.
     @param member_uuid        The UUID of the member from which this
                               transaction originates.
     @param gle                The incoming transaction global identifier
@@ -296,10 +261,10 @@ class Certifier : public Certifier_interface {
     @retval  0                negatively certified;
     @retval -1                error.
    */
-  gr::Certified_gtid certify(Gtid_set *snapshot_version,
-                             std::list<const char *> *write_set,
-                             bool is_gtid_specified, const char *member_uuid,
-                             Gtid_log_event *gle, bool local_transaction);
+  rpl_gno certify(Gtid_set *snapshot_version,
+                  std::list<const char *> *write_set, bool generate_group_id,
+                  const char *member_uuid, Gtid_log_event *gle,
+                  bool local_transaction);
 
   /**
     Returns the transactions in stable set in text format, that is, the set of
@@ -315,8 +280,7 @@ class Certifier : public Certifier_interface {
       @retval 0         OK
       @retval !=0       Out of memory error
    */
-  int get_group_stable_transactions_set_string(char **buffer,
-                                               size_t *length) override;
+  int get_group_stable_transactions_set_string(char **buffer, size_t *length);
 
   /**
     Retrieves the current certification info.
@@ -326,23 +290,8 @@ class Certifier : public Certifier_interface {
 
      @param[out] cert_info        a pointer to retrieve the certification info
   */
-  void get_certification_info(
-      std::map<std::string, std::string> *cert_info) override;
-
-  /**
-    Retrieves the current certification info.
-
-    @note if concurrent access is introduce to these variables,
-    locking is needed in this method
-
-    @param[out] recovery_metadata_message  Retrieves the metadata message
-
-    @return the operation status
-      @retval false      OK
-      @retval true       Error
-  */
-  bool get_certification_info_recovery_metadata(
-      Recovery_metadata_message *recovery_metadata_message) override;
+  virtual void get_certification_info(
+      std::map<std::string, std::string> *cert_info);
 
   /**
     Sets the certification info according to the given value.
@@ -355,84 +304,107 @@ class Certifier : public Certifier_interface {
     @retval  > 0  Error during setting certification info.
     @retval  = 0  Everything went fine.
   */
-  int set_certification_info(
-      std::map<std::string, std::string> *cert_info) override;
-
-  /**
-    The received certification info from Recovery Metadata is decoded,
-    compressed and added to the member certification info for certification.
-
-    @note if concurrent access is introduce to these variables,
-    locking is needed in this method
-
-    @param[in]  recovery_metadata_message  the pointer to
-                                           Recovery_metadata_message.
-
-    @return the operation status
-      @retval false      OK
-      @retval true       Error
-  */
-  bool set_certification_info_recovery_metadata(
-      Recovery_metadata_message *recovery_metadata_message) override;
-  /**
-    Initializes the gtid_executed set.
-
-    @return the operation status
-      @retval false      OK
-      @retval true       Error
-  */
-  bool initialize_server_gtid_set_after_distributed_recovery() override;
+  virtual int set_certification_info(
+      std::map<std::string, std::string> *cert_info);
 
   /**
     Get the number of postively certified transactions by the certifier
     */
-  ulonglong get_positive_certified() override;
+  ulonglong get_positive_certified();
 
   /**
     Get method to retrieve the number of negatively certified transactions.
     */
-  ulonglong get_negative_certified() override;
+  ulonglong get_negative_certified();
 
   /**
     Get method to retrieve the certification db size.
     */
-  ulonglong get_certification_info_size() override;
+  ulonglong get_certification_info_size();
 
   /**
     Get method to retrieve the last conflict free transaction.
 
     @param[out] value The last conflict free transaction
     */
-  void get_last_conflict_free_transaction(std::string *value) override;
+  void get_last_conflict_free_transaction(std::string *value);
 
   /**
-    Generate group GTID for a view change log event.
-    @return Generated gtid and Gtid generation result
-    @see Return_status
+    Get method to retrieve the size of the members.
   */
-  std::pair<Gtid, mysql::utils::Return_status>
-  generate_view_change_group_gtid();
+  size_t get_members_size();
+
+  /**
+    Generate group GNO for a view change log event.
+
+    @retval  >0         view change GNO
+    @retval  otherwise  Error on GNO generation
+  */
+  rpl_gno generate_view_change_group_gno();
+
+  /**
+    Public method to add the given gno value to the group_gtid_executed set
+    which is used to support skip gtid functionality.
+
+    @param[in] gno  The gno of the transaction which will be added to the
+                    group_gtid executed GTID set. The sidno used for this
+    transaction will be the group_sidno. The gno here belongs specifically to
+    the group UUID.
+    @param[in] local If the gtid value is local or comes from a remote server
+
+    @retval  1  error during addition.
+    @retval  0  success.
+  */
+  int add_group_gtid_to_group_gtid_executed(rpl_gno gno, bool local);
 
   /**
     Public method to add the given GTID value in the group_gtid_executed set
     which is used to support skip gtid functionality.
 
-    @param[in] gtid GTID to be added
+    @param[in] gle  The gtid value that needs to the added in the
+                    group_gtid_executed GTID set.
+    @param[in] local If the gtid value is local or comes from a remote server
 
     @retval  1  error during addition.
     @retval  0  success.
   */
-  int add_gtid_to_group_gtid_executed(const Gtid &gtid);
+  int add_specified_gtid_to_group_gtid_executed(Gtid_log_event *gle,
+                                                bool local);
+
+  /**
+    This member function shall add transactions to the stable set
+
+    @param executed_gtid_set  The GTID set of the transactions to be added
+                              to the stable set.
+
+    @note when set, the stable set will cause the garbage collection
+          process to be invoked
+
+    @retval False  if adds successfully,
+    @retval True   otherwise.
+   */
+  bool set_group_stable_transactions_set(Gtid_set *executed_gtid_set);
+
+  /**
+    Method to get a string that represents the last local certified GTID
+
+    @param[out] local_gtid_certified_string  The last local GTID transaction
+    string
+
+    @retval 0    if there is no GTID / the string is empty
+    @retval !=0  the size of the string
+  */
+  size_t get_local_certified_gtid(std::string &local_gtid_certified_string);
 
   /**
     Enables conflict detection.
   */
-  void enable_conflict_detection() override;
+  void enable_conflict_detection();
 
   /**
     Disables conflict detection.
   */
-  void disable_conflict_detection() override;
+  void disable_conflict_detection();
 
   /**
     Check if conflict detection is enable.
@@ -440,22 +412,7 @@ class Certifier : public Certifier_interface {
     @retval True   conflict detection is enable
     @retval False  otherwise
   */
-  bool is_conflict_detection_enable() override;
-
-  /**
-    Compute GTID intervals.
-  */
-  void gtid_intervals_computation();
-
-  /**
-    Validates if garbage collect should run against the intersection of the
-    received transactions stable sets.
-
-    @param executed_gtid_set intersection gtid set
-    @param on_member_join    call due to member joining
-   */
-  void garbage_collect(Gtid_set *executed_gtid_set = nullptr,
-                       bool on_member_join = false) override;
+  bool is_conflict_detection_enable();
 
  private:
   /**
@@ -467,22 +424,13 @@ class Certifier : public Certifier_interface {
   /**
     Is certifier initialized.
   */
-  std::atomic<bool> initialized{false};
+  bool initialized;
 
   /**
     Variable to store the sidno used for transactions which will be logged
     with the group_uuid.
   */
-  rpl_sidno group_gtid_tsid_map_group_sidno;
-
-  /**
-    The sidno used for view log events as seen by the group sid map
-    */
-  rpl_sidno views_sidno_group_representation;
-  /**
-    The sidno used for view log events as seen by the server sid map
-    */
-  rpl_sidno views_sidno_server_representation;
+  rpl_sidno group_gtid_sid_map_group_sidno;
 
   /**
     Method to initialize the group_gtid_executed gtid set with the server gtid
@@ -497,6 +445,20 @@ class Certifier : public Certifier_interface {
   */
   int initialize_server_gtid_set(bool get_server_gtid_retrieved = false);
 
+  /**
+    This function computes the available GTID intervals from group
+    UUID and stores them on group_available_gtid_intervals.
+  */
+  void compute_group_available_gtid_intervals();
+
+  /**
+    This function reserves a block of GTIDs from the
+    group_available_gtid_intervals list.
+
+    @retval Gtid_set::Interval which is the interval os GTIDs attributed
+  */
+  Gtid_set::Interval reserve_gtid_block(longlong block_size);
+
   /**
     This function updates parallel applier indexes.
     It must be called for each remote transaction.
@@ -529,144 +491,46 @@ class Certifier : public Certifier_interface {
 
     @param[in] gno  rpl_gno part of the executing gtid of the ongoing
                     transaction.
+    @param[in] local_transaction if the GTID belongs to a local transaction
   */
-  void add_to_group_gtid_executed_internal(rpl_sidno sidno, rpl_gno gno);
+  void add_to_group_gtid_executed_internal(rpl_sidno sidno, rpl_gno gno,
+                                           bool local_transaction);
 
-  /// @brief Returns group_executed_gtid_set or group_extracted_gtid_set while
-  /// certifying already applied transactions from the donor
-  /// @returns Pointer to the 'correct' group_gtid_set
-  const Gtid_set *get_group_gtid_set() const;
+  /**
+    This method is used to get the next valid GNO for the
+    transaction that is being executed. It checks the already used
+    up GNOs and based on that chooses the next possible value.
+    This method will consult group_available_gtid_intervals to
+    assign GTIDs in blocks according to gtid_assignment_block_size.
 
-  /// @brief Returns group_executed_gtid_set or group_extracted_gtid_set while
-  /// certifying already applied transactions from the donor
-  /// @returns Pointer to the 'correct' group_gtid_set
-  Gtid_set *get_group_gtid_set();
+    @param member_uuid        The UUID of the member from which this
+                              transaction originates. It will be NULL
+                              on View_change_log_event.
 
-  /// @brief This function determines three sidnos for a specific TSID
-  /// based on information obtained from the Gtid_log_event.
-  /// @param gle Gtid_log_event from which tsid will be extracted
-  /// @param is_gtid_specified True in case GTID is specified
-  /// @param snapshot_gtid_set Snapshot GTIDs
-  /// @param group_gtid_set Current GTID set
-  /// @return A tuple of:
-  ///   - group_sidno Sidno relative to the group sid map
-  ///   - gtid_snapshot_sidno Sidno relative to the snapshot sid map
-  ///   - gtid_global_sidno Sidno relative to the global sid map
-  ///   - return status
-  /// @details
-  /// We need to ensure that group sidno does exist on snapshot
-  /// version due to the following scenario:
-  ///   1) Member joins the group.
-  ///   2) Goes through recovery procedure, view change is queued to
-  ///       apply, member is marked ONLINE. This requires
-  ///         --group_replication_recovery_complete_at=TRANSACTIONS_CERTIFIED
-  ///       to happen.
-  ///   3) Despite the view change log event is still being applied,
-  ///       since the member is already ONLINE it can execute
-  ///       transactions. The first transaction from this member will
-  ///       not include any group GTID, since no group transaction is
-  ///       yet applied.
-  ///   4) As a result of this sequence snapshot_version will not
-  ///       contain any group GTID and the below instruction
-  ///         snapshot_version->_add_gtid(group_sidno, result);
-  ///       would fail because of that
-  std::tuple<rpl_sidno, rpl_sidno, rpl_sidno, mysql::utils::Return_status>
-  extract_sidno(Gtid_log_event &gle, bool is_gtid_specified,
-                Gtid_set &snapshot_gtid_set, Gtid_set &group_gtid_set);
+    @retval >0                The GNO to be used.
+    @retval -1                Error: GNOs exhausted for group UUID.
+  */
+  rpl_gno get_group_next_available_gtid(const char *member_uuid);
 
-  /// @brief Internal helper method for ending certification, determination
-  /// of final GTID values after certification according to certification result
-  /// @param[in] gtid_server_sidno SIDNO for transaction GTID as represented in
-  /// the server (global sid map)
-  /// @param[in] gtid_group_sidno SIDNO for transaction GTID as represented in
-  /// the group
-  /// @param[in] generated_gno GNO generated for the transaction
-  /// @param[in] is_gtid_specified True if GTID was specified
-  /// @param[in] local_transaction True in case this transaction originates
-  /// from the this server
-  /// @param[in] certification_result Determined certification result
-  gr::Certified_gtid end_certification_result(
-      const rpl_sidno &gtid_server_sidno, const rpl_sidno &gtid_group_sidno,
-      const rpl_gno &generated_gno, bool is_gtid_specified,
-      bool local_transaction,
-      const gr::Certification_result &certification_result);
+  /**
+    Generate the candidate GNO for the current transaction.
+    The candidate will be on the interval [start, end] or a error
+    be returned.
+    This method will consult group_gtid_executed to avoid generate
+    the same value twice.
 
-  /// @brief Adds the transaction's write set to certification info.
-  /// @param[out] transaction_last_committed The transaction's logical
-  /// timestamps used for MTS
-  /// @param[in,out] snapshot_version   The incoming transaction snapshot
-  /// version.
-  /// @param[in, out] write_set          The incoming transaction write set.
-  /// @param[in] local_transaction True in case this transaction originates
-  /// from the this server
-  gr::Certification_result add_writeset_to_certification_info(
-      int64 &transaction_last_committed, Gtid_set *snapshot_version,
-      std::list<const char *> *write_set, bool local_transaction);
+    @param start              The first possible value for the GNO
+    @param end                The last possible value for the GNO
 
-  /// @brief Updates parallel applier indexes in GLE
-  /// @param gle Gle currently processed
-  /// @param has_write_set True in case transaction write set is not empty
-  /// @param has_write_set_large_size True in case number of write sets in
-  /// transactions is greater than
-  /// group_replication_preemptive_garbage_collection_rows_threshold
-  /// @param transaction_last_committed The transaction's logical timestamps
-  /// used for MTS
-  void update_transaction_dependency_timestamps(
-      Gtid_log_event &gle, bool has_write_set, bool has_write_set_large_size,
-      int64 transaction_last_committed);
+    @retval >0                The GNO to be used.
+    @retval -1                Error: GNOs exhausted for group UUID.
+    @retval -2                Error: generated GNO is bigger than end.
+  */
+  rpl_gno get_group_next_available_gtid_candidate(rpl_gno start,
+                                                  rpl_gno end) const;
 
   bool inline is_initialized() { return initialized; }
 
-  /**
-    This shall serialize the certification info stored in protobuf map format,
-    and then compress provided serialized string. The compressed payload is
-    stored into multiple buffer containers of the output list.
-
-    @param[in]  cert_info        the certification info stored in protobuf map.
-    @param[out] uncompresssed_buffer the buffer for uncompressed data.
-    @param[out] compressor_list  the certification info in compressed form
-                                 splitted into multiple container of list.
-    @param[in] compression_type the type of compression used
-
-    @return the operation status
-      @retval false      OK
-      @retval true       Error
-  */
-  bool compress_packet(ProtoCertificationInformationMap &cert_info,
-                       unsigned char **uncompresssed_buffer,
-                       std::vector<GR_compress *> &compressor_list,
-                       GR_compress::enum_compression_type compression_type);
-
-  /**
-    Sets the certification info according to the given value.
-    This shall uncompress and then convert uncompressed string into the protobuf
-    map format storing certification info. This certification info is added to
-    certifier's certification info.
-
-    @note if concurrent access is introduce to these variables,
-    locking is needed in this method
-
-    @param[in]  compression_type  the compression type
-    @param[in]  buffer         the compressed  certification info retrieved from
-                               recovery procedure.
-    @param[in]  buffer_length  the size of the compressed retrieved
-                               certification info.
-    @param[in]  uncompressed_buffer_length the size of the uncompressed
-                                           certification info before it was
-                                           compressed.
-
-    @return the operation status
-      @retval false      OK
-      @retval true       Error
-  */
-  bool set_certification_info_part(
-      GR_compress::enum_compression_type compression_type,
-      const unsigned char *buffer, unsigned long long buffer_length,
-      unsigned long long uncompressed_buffer_length);
-
-  /**
-    Empties certification info.
-  */
   void clear_certification_info();
 
   /**
@@ -683,14 +547,14 @@ class Certifier : public Certifier_interface {
     Certification database.
   */
   Certification_info certification_info;
-  Tsid_map *certification_info_tsid_map;
+  Sid_map *certification_info_sid_map;
 
   ulonglong positive_cert;
   ulonglong negative_cert;
   int64 parallel_applier_last_committed_global;
   int64 parallel_applier_sequence_number;
 
-#if !defined(NDEBUG)
+#if !defined(DBUG_OFF)
   bool certifier_garbage_collection_block;
   bool same_member_message_discarded;
 #endif
@@ -701,7 +565,7 @@ class Certifier : public Certifier_interface {
     Stable set and garbage collector variables.
   */
   Checkable_rwlock *stable_gtid_set_lock;
-  Tsid_map *stable_tsid_map;
+  Sid_map *stable_sid_map;
   Gtid_set *stable_gtid_set;
   Synchronized_queue<Data_packet *> *incoming;
 
@@ -754,7 +618,7 @@ class Certifier : public Certifier_interface {
   /*
     Sid map to store the GTIDs that are executed in the group.
   */
-  Tsid_map *group_gtid_tsid_map;
+  Sid_map *group_gtid_sid_map;
 
   /*
     A Gtid_set containing the already executed for the group.
@@ -769,9 +633,26 @@ class Certifier : public Certifier_interface {
   */
   Gtid_set *group_gtid_extracted;
 
-  /// Object responsible for generation of the GTIDs for transactions with
-  /// gtid_next equal to AUTOMATIC (tagged/untagged)
-  gr::Gtid_generator gtid_generator;
+  /**
+    The group GTID assignment block size.
+  */
+  uint64 gtid_assignment_block_size;
+
+  /**
+    List of free GTID intervals in group
+  */
+  std::list<Gtid_set::Interval> group_available_gtid_intervals;
+
+  /**
+    Extends the above to allow GTIDs to be assigned in blocks per member.
+  */
+  std::map<std::string, Gtid_set::Interval> member_gtids;
+  ulonglong gtids_assigned_in_blocks_counter;
+
+  /**
+    Last local known GTID
+  */
+  Gtid last_local_gtid;
 
   /**
     Conflict detection is performed when:
@@ -814,6 +695,22 @@ class Certifier : public Certifier_interface {
   */
   Gtid_set *get_certified_write_set_snapshot_version(const char *item);
 
+  /**
+    Computes intersection between all sets received, so that we
+    have the already applied transactions on all servers.
+
+    @return the operation status
+      @retval 0      OK
+      @retval !=0    Error
+  */
+  int stable_set_handle();
+
+  /**
+    Removes the intersection of the received transactions stable
+    sets from certification database.
+   */
+  void garbage_collect();
+
   /**
     Clear incoming queue.
   */
@@ -824,51 +721,6 @@ class Certifier : public Certifier_interface {
     certified transaction on a particular group member.
   */
   void update_certified_transaction_count(bool result, bool local_transaction);
-
-  /*
-    The first remote transaction certified does need to reset
-    replication_group_applier channel previous transaction
-    sequence_number.
-  */
-  bool is_first_remote_transaction_certified{true};
-
-  /**
-    Removes the intersection of the received transactions stable
-    sets from certification database.
-
-    @param intersection_gtid_set intersection gtid set
-    @param preemptive            is a preemptive run
-   */
-  void garbage_collect_internal(Gtid_set *intersection_gtid_set,
-                                bool preemptive = false);
-
-  /**
-    Computes intersection between all sets received, so that we
-    have the already applied transactions on all servers.
-
-    @return the operation status
-      @retval false  it did not run garbage_collect
-      @retval true   it did run garbage_collect
-  */
-  bool intersect_members_gtid_executed_and_garbage_collect();
-
-  enum enum_update_status {
-    // stable set successfully updated
-    STABLE_SET_UPDATED,
-    // stable set already contains set
-    STABLE_SET_ALREADY_CONTAINED,
-    // not able to update due error
-    STABLE_SET_ERROR
-  };
-
-  /**
-   * Update stable set with set if not already contained.
-   *
-   * @param set Gtid to add to stable set
-   *
-   * @return status of operation
-   */
-  enum enum_update_status update_stable_set(const Gtid_set &set);
 };
 
 /*
@@ -885,18 +737,15 @@ class Gtid_Executed_Message : public Plugin_gcs_message {
     // Length of the payload item: variable
     PIT_GTID_EXECUTED = 1,
 
-    // Length of the payload item: 8 bytes
-    PIT_SENT_TIMESTAMP = 2,
-
     // No valid type codes can appear after this one.
-    PIT_MAX = 3
+    PIT_MAX = 2
   };
 
   /**
    Gtid_Executed_Message constructor
    */
   Gtid_Executed_Message();
-  ~Gtid_Executed_Message() override;
+  virtual ~Gtid_Executed_Message();
 
   /**
     Appends Gtid executed information in a raw format
@@ -906,25 +755,12 @@ class Gtid_Executed_Message : public Plugin_gcs_message {
    */
   void append_gtid_executed(uchar *gtid_data, size_t len);
 
-  /**
-    Return the time at which the message contained in the buffer was sent.
-    @see Metrics_handler::get_current_time()
-
-    @param[in] buffer            the buffer to decode from.
-    @param[in] length            the buffer length
-
-    @return the time on which the message was sent.
-  */
-  static uint64_t get_sent_timestamp(const unsigned char *buffer,
-                                     size_t length);
-
  protected:
   /*
    Implementation of the template methods of Gcs_plugin_message
    */
-  void encode_payload(std::vector<unsigned char> *buffer) const override;
-  void decode_payload(const unsigned char *buffer,
-                      const unsigned char *) override;
+  void encode_payload(std::vector<unsigned char> *buffer) const;
+  void decode_payload(const unsigned char *buffer, const unsigned char *);
 
  private:
   std::vector<uchar> data;
