@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -34,13 +34,6 @@
 #include "plugin/group_replication/include/plugin_observers/channel_observation_manager.h"
 #include "plugin/group_replication/include/replication_threads_api.h"
 
-typedef enum st_state_transfer_status {
-  STATE_TRANSFER_OK,            // OK
-  STATE_TRANSFER_STOP,          // Fail to stop replica threads
-  STATE_TRANSFER_PURGE,         // Fail to purge replica threads
-  STATE_TRANSFER_NO_CONNECTION  // No connection to donor
-} State_transfer_status;
-
 class Recovery_state_transfer {
  public:
   /**
@@ -60,19 +53,9 @@ class Recovery_state_transfer {
   /**
     Initialize the state transfer class and reset the class flags
 
-    @param view_id         The view id to use for the recovery.
-    @param is_vcle_enabled  the flag determine if View_change_log_event
-                             is enabled.
-
+    @param rec_view_id  The view id to use on this round
   */
-  void initialize(const std::string &view_id, bool is_vcle_enabled);
-
-  /**
-    Set the gtids to use for the recovery until condition.
-
-    @param after_gtids  the gtids to use for the recovery until condition.
-  */
-  void set_until_condition_after_gtids(const std::string &after_gtids);
+  void initialize(const std::string &rec_view_id);
 
   /** Abort the state transfer */
   void abort_state_transfer();
@@ -303,8 +286,7 @@ class Recovery_state_transfer {
       @retval 0      OK
       @retval !=0    Recovery state transfer failed
    */
-  State_transfer_status state_transfer(
-      Plugin_stage_monitor_handler &stage_handler);
+  int state_transfer(Plugin_stage_monitor_handler &stage_handler);
 
  private:
   /**
@@ -324,19 +306,6 @@ class Recovery_state_transfer {
   /** Method that sets the failover status to true and awakes recovery */
   void donor_failover();
 
-  /**
-    Verify member has after_gtids received from replication donor already
-    present in member's gtid_executed.
-
-    @param local_gtid_executed_string the local member gtid_executed string
-
-    @return the operation status
-      @retval true     after_gtids there in member's gtid_executed.
-      @retval false    after_gtids not there in member's gtid_executed.
-  */
-  bool verify_member_has_after_gtids_present(
-      std::string &local_gtid_executed_string);
-
   /**
     Establish a master/slave connection to the selected donor.
 
@@ -350,14 +319,11 @@ class Recovery_state_transfer {
     Initializes the structures for the donor connection threads.
     Recovery channel is always purged.
 
-    @param hostname hostname of current selected donor
-    @param port port of current selected donor
-
     @return the operation status
       @retval 0      OK
       @retval !=0    Error
   */
-  int initialize_donor_connection(std::string hostname, uint port);
+  int initialize_donor_connection();
 
   /**
     Initializes the connection parameters for the donor connection.
@@ -382,11 +348,10 @@ class Recovery_state_transfer {
     @param purge_logs  purge recovery logs
 
     @return the operation status
-      @retval STATE_TRANSFER_OK      OK
-      @retval !=STATE_TRANSFER_OK    Error
+      @retval 0      OK
+      @retval !=0    Error
   */
-  State_transfer_status terminate_recovery_slave_threads(
-      bool purge_logs = true);
+  int terminate_recovery_slave_threads(bool purge_logs = true);
 
   /**
     Purges relay logs and the master info object
@@ -401,48 +366,26 @@ class Recovery_state_transfer {
   int purge_recovery_slave_threads_repos();
 
  private:
-  /**
-    Verifies if after_gtids sent by sender is successfully applied when recovery
-    is using CHANNEL_UNTIL_APPLIER_AFTER_GTIDS for start replica until condition
-
-    @retval false after_gtids is successfully applied.
-    @retval true  Error in applying after_gtids.
-  */
-  bool verify_after_gtids_applied();
-
-  /** The recovery channel name */
-  std::string m_recovery_channel_name;
-
   /* The member uuid*/
   std::string member_uuid;
-
   /* The associated view id for the current recovery session */
   std::string view_id;
 
-  /* The gtids used by the replication until condition for the recovery. */
-  std::string m_after_gtids;
-
-  /*
-    The replication until condition that can be applied to
-    channels for the recovery.
-  */
-  enum_channel_until_condition m_until_condition;
-
   /* The selected donor member*/
   Group_member_info *selected_donor;
   /* The selected donor member hostname */
   std::string selected_donor_hostname;
   /* Vector with group members info*/
-  Group_member_info_list *group_members;
+  std::vector<Group_member_info *> *group_members;
   /* Member with suitable donors for use on recovery*/
-  Group_member_info_list suitable_donors;
+  std::vector<Group_member_info *> suitable_donors;
 
   /* Retry count on donor connections*/
   long donor_connection_retry_count;
 
   /* Recovery abort flag */
   bool recovery_aborted;
-  /*  Flag that signals when the donor transferred all its data */
+  /*  Flag that signals when the donor transfered all it's data */
   bool donor_transfer_finished;
   /* Are we successfully connected to a donor*/
   bool connected_to_donor;
