@@ -1,16 +1,15 @@
-/* Copyright (c) 2015, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,8 +29,6 @@
 #include "my_inttypes.h"
 #include "plugin/group_replication/include/sql_service/sql_resultset.h"
 
-struct CHARSET_INFO;
-
 class Sql_service_context_base {
  public:
   /** The sql service callbacks that will call the below virtual methods*/
@@ -41,9 +38,9 @@ class Sql_service_context_base {
     Sql_service_context_base constructor
     resets all variables
   */
-  Sql_service_context_base() = default;
+  Sql_service_context_base() {}
 
-  virtual ~Sql_service_context_base() = default;
+  virtual ~Sql_service_context_base() {}
 
   /** Getting metadata **/
   /**
@@ -248,12 +245,6 @@ class Sql_service_context_base {
   */
   virtual void shutdown(int flag) = 0;
 
-  /**
-   Check if the connection is still alive.
-   It should always return true unless the protocol closed the connection.
-  */
-  virtual bool connection_alive() = 0;
-
  private:
   static int sql_start_result_metadata(void *ctx, uint num_cols, uint flags,
                                        const CHARSET_INFO *resultcs) {
@@ -347,10 +338,6 @@ class Sql_service_context_base {
   static void sql_shutdown(void *ctx, int flag) {
     return ((Sql_service_context_base *)ctx)->shutdown(flag);
   }
-
-  static bool sql_connection_alive(void *ctx) {
-    return ((Sql_service_context_base *)ctx)->connection_alive();
-  }
 };
 
 #endif  // SQL_SERVICE_CONTEXT_BASE_INCLUDE
