@@ -1,16 +1,15 @@
-/* Copyright (c) 2015, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -43,18 +42,15 @@
 #include "plugin/group_replication/libmysqlgcs/src/bindings/xcom/gcs_xcom_networking.h"
 #include "plugin/group_replication/libmysqlgcs/src/bindings/xcom/gcs_xcom_notification.h"
 #include "plugin/group_replication/libmysqlgcs/src/bindings/xcom/gcs_xcom_utils.h"
-#include "plugin/group_replication/libmysqlgcs/src/bindings/xcom/xcom/network/include/network_management_interface.h"
 #include "plugin/group_replication/libmysqlgcs/src/bindings/xcom/xcom/site_struct.h"
 #include "plugin/group_replication/libmysqlgcs/src/bindings/xcom/xcom/sock_probe.h"
 #include "plugin/group_replication/libmysqlgcs/src/bindings/xcom/xcom/synode_no.h"
-#include "xdr_gen/xcom_vp.h"
+#include "plugin/group_replication/libmysqlgcs/src/bindings/xcom/xcom/xcom_ssl_transport.h"
 
 using std::map;
 using std::string;
 using std::vector;
 
-extern uint32_t get_my_xcom_id();
-
 Gcs_interface *Gcs_xcom_interface::interface_reference_singleton = nullptr;
 
 Gcs_xcom_config::Gcs_xcom_config()
@@ -99,36 +95,6 @@ bool Gcs_xcom_config::same_event_horizon(
   return (event_horizon == event_horizon_);
 }
 
-bool Gcs_xcom_config::same_xcom_nodes_v3(
-    Gcs_xcom_nodes const &xcom_nodes) const {
-  bool are_same_nodes = (xcom_nodes.get_size() == xcom_nodes_.get_size());
-  if (are_same_nodes) {
-    for (auto const &node : xcom_nodes.get_nodes()) {
-      Gcs_xcom_node_information const *existing_node =
-          xcom_nodes_.get_node(node.get_member_id());
-      /* Take UUID into account to distinguish between reincarnations of the
-         same node. */
-      bool const node_already_existed =
-          (existing_node != nullptr &&
-           existing_node->get_member_uuid().actual_value ==
-               node.get_member_uuid().actual_value);
-      are_same_nodes = are_same_nodes && node_already_existed;
-    }
-  }
-  MYSQL_GCS_TRACE_EXECUTE(
-      MYSQL_GCS_LOG_TRACE(
-          "Received global view: previous node set: (same_xcom_nodes=%d)",
-          are_same_nodes);
-      for (auto const &node
-           : xcom_nodes.get_nodes()) {
-        MYSQL_GCS_LOG_TRACE("My node_id is %d peer_ %d address: %s flag: %s",
-                            xcom_nodes.get_node_no(), node.get_node_no(),
-                            node.get_member_id().get_member_id().c_str(),
-                            node.is_alive() ? "Active" : "Failed");
-      });
-  return are_same_nodes;
-}
-
 static Gcs_xcom_config last_accepted_xcom_config;
 
 int xcom_local_port = 0;
@@ -143,12 +109,10 @@ static Gcs_xcom_proxy *s_xcom_proxy;
 */
 static Gcs_xcom_engine *gcs_engine;
 
-void cb_xcom_receive_data(synode_no message_id, synode_no origin,
-                          site_def const *site, node_set nodes, u_int size,
+void cb_xcom_receive_data(synode_no message_id, node_set nodes, u_int size,
                           synode_no last_removed, char *data);
-void do_cb_xcom_receive_data(synode_no message_id, synode_no origin,
-                             Gcs_xcom_nodes *xcom_nodes, synode_no last_removed,
-                             u_int size, char *data);
+void do_cb_xcom_receive_data(synode_no message_id, Gcs_xcom_nodes *xcom_nodes,
+                             synode_no last_removed, u_int size, char *data);
 void cb_xcom_receive_local_view(synode_no config_id, node_set nodes);
 void do_cb_xcom_receive_local_view(synode_no config_id,
                                    Gcs_xcom_nodes *xcom_nodes,
@@ -168,9 +132,7 @@ void do_cb_xcom_expel();
 
 synode_no cb_xcom_get_app_snap(blob *gcs_snap);
 int cb_xcom_get_should_exit();
-void cb_xcom_handle_app_snap(blob *store_snap [[maybe_unused]],
-                             synode_no start [[maybe_unused]],
-                             synode_no end [[maybe_unused]]);
+void cb_xcom_handle_app_snap(blob *gcs_snap);
 int cb_xcom_socket_accept(int fd, site_def const *xcom_config);
 
 xcom_input_request_ptr cb_xcom_input_try_pop();
@@ -208,8 +170,7 @@ void Gcs_xcom_interface::cleanup() {
 }
 
 void Gcs_xcom_interface::cleanup_thread_ssl_resources() {
-  ::get_network_management_interface()
-      ->delayed_cleanup_secure_connections_context();
+  ::xcom_cleanup_ssl();
 }
 
 Gcs_xcom_interface::Gcs_xcom_interface()
@@ -225,16 +186,17 @@ Gcs_xcom_interface::Gcs_xcom_interface()
       m_default_sink(nullptr),
       m_default_logger(nullptr),
       m_default_debugger(nullptr),
-      m_ip_allowlist(),
+      m_ip_whitelist(),
       m_ssl_init_state(-1),
       m_wait_for_ssl_init_cond(),
-      m_wait_for_ssl_init_mutex(),
-      m_netns_manager(nullptr) {
+      m_wait_for_ssl_init_mutex() {
   // Initialize random seed
   srand(static_cast<unsigned int>(time(nullptr)));
+
+  My_xp_util::init_time();
 }
 
-Gcs_xcom_interface::~Gcs_xcom_interface() = default;
+Gcs_xcom_interface::~Gcs_xcom_interface() {}
 
 enum_gcs_error Gcs_xcom_interface::initialize_logging(
     const std::string *debug_file, const std::string *debug_path) {
@@ -246,7 +208,7 @@ enum_gcs_error Gcs_xcom_interface::initialize_logging(
         new Gcs_async_buffer(new Gcs_file_sink(*debug_file, *debug_path));
   else
 #endif /* XCOM_STANDALONE */
-    /* purecov: begin inspected */
+       /* purecov: begin inspected */
     m_default_sink = new Gcs_async_buffer(new Gcs_output_sink());
   /* purecov: end */
 
@@ -313,7 +275,7 @@ enum_gcs_error Gcs_xcom_interface::finalize_logging() {
 
 enum_gcs_error Gcs_xcom_interface::initialize(
     const Gcs_interface_parameters &interface_params) {
-  const std::string *ip_allowlist_str = nullptr;
+  const std::string *ip_whitelist_str = nullptr;
   Gcs_interface_parameters validated_params;
 
   if (is_initialized()) return GCS_OK;
@@ -351,10 +313,10 @@ enum_gcs_error Gcs_xcom_interface::initialize(
   */
   Gcs_xcom_utils::init_net();
 
-  // validate allowlist
-  ip_allowlist_str = validated_params.get_parameter("ip_allowlist");
+  // validate whitelist
+  ip_whitelist_str = validated_params.get_parameter("ip_whitelist");
 
-  if (ip_allowlist_str && !m_ip_allowlist.is_valid(*ip_allowlist_str)) goto err;
+  if (ip_whitelist_str && !m_ip_whitelist.is_valid(*ip_whitelist_str)) goto err;
 
   /*
     ---------------------------------------------------------------
@@ -368,8 +330,7 @@ enum_gcs_error Gcs_xcom_interface::initialize(
     Perform syntax checks
     ---------------------------------------------------------------
   */
-  if (!is_parameters_syntax_correct(validated_params, m_netns_manager))
-    goto err;
+  if (!is_parameters_syntax_correct(validated_params)) goto err;
 
   /*
     ---------------------------------------------------------------
@@ -392,15 +353,10 @@ enum_gcs_error Gcs_xcom_interface::initialize(
     Proceed with initialization
     ---------------------------------------------------------------
   */
-  // Initalize statistic storage
-  m_stats_mgr = new Gcs_xcom_statistics_manager_interface_impl();
-  m_xcom_stats_storage = new Gcs_xcom_statistics_storage_impl(m_stats_mgr);
 
   // initialize xcom's data structures to pass configuration
   // from the application
   m_gcs_xcom_app_cfg.init();
-  m_gcs_xcom_app_cfg.set_network_namespace_manager(m_netns_manager);
-  m_gcs_xcom_app_cfg.set_statists_storage_implementation(m_xcom_stats_storage);
   this->clean_group_interfaces();
   m_socket_util = new My_xp_socket_util_impl();
 
@@ -443,7 +399,6 @@ enum_gcs_error Gcs_xcom_interface::configure(
   map<std::string, gcs_xcom_group_interfaces *>::const_iterator
       registered_group;
   Gcs_xcom_control *xcom_control = nullptr;
-  const std::string *ip_allowlist_str{nullptr};
 
   // Error! Interface still not initialized or finalize has already been invoked
   if (!is_initialized()) return GCS_NOK;
@@ -463,24 +418,25 @@ enum_gcs_error Gcs_xcom_interface::configure(
     Perform syntax checks
     ---------------------------------------------------------------
    */
-  if (!is_parameters_syntax_correct(validated_params, m_netns_manager))
-    return GCS_NOK;
+  if (!is_parameters_syntax_correct(validated_params)) return GCS_NOK;
 
-  // validate allowlist
-  std::string *ip_allowlist_reconfigure_str = const_cast<std::string *>(
-      interface_params.get_parameter("reconfigure_ip_allowlist"));
+  // validate whitelist
+  std::string *ip_whitelist_reconfigure_str = const_cast<std::string *>(
+      interface_params.get_parameter("reconfigure_ip_whitelist"));
 
-  bool should_configure_allowlist = false;
-  if (ip_allowlist_reconfigure_str) {
-    should_configure_allowlist =
-        ip_allowlist_reconfigure_str->compare("on") == 0 ||
-        ip_allowlist_reconfigure_str->compare("true") == 0;
+  bool should_configure_whitelist = true;
+  if (ip_whitelist_reconfigure_str) {
+    should_configure_whitelist =
+        ip_whitelist_reconfigure_str->compare("on") == 0 ||
+        ip_whitelist_reconfigure_str->compare("true") == 0;
   }
 
-  if (should_configure_allowlist) {
-    ip_allowlist_str = validated_params.get_parameter("ip_allowlist");
-    if (!ip_allowlist_str || !m_ip_allowlist.is_valid(*ip_allowlist_str)) {
-      MYSQL_GCS_LOG_ERROR("The ip_allowlist parameter is not valid")
+  if (should_configure_whitelist) {
+    const std::string *ip_whitelist_str =
+        validated_params.get_parameter("ip_whitelist");
+
+    if (!ip_whitelist_str || !m_ip_whitelist.is_valid(*ip_whitelist_str)) {
+      MYSQL_GCS_LOG_ERROR("The ip_whitelist parameter is not valid")
       return GCS_NOK;
     }
   }
@@ -590,13 +546,6 @@ enum_gcs_error Gcs_xcom_interface::configure(
     reconfigured |= true;
   }
 
-  // ip_allowlist
-  if (should_configure_allowlist &&
-      GCS_OK == m_ip_allowlist.configure(*ip_allowlist_str)) {
-    // configure allowlist
-    reconfigured |= true;
-  }
-
 end:
   if (error == GCS_NOK || !reconfigured) {
     MYSQL_GCS_LOG_ERROR("Error while configuring the member.")
@@ -612,7 +561,6 @@ void cleanup_xcom() {
   s_xcom_proxy->xcom_destroy_ssl();
   s_xcom_proxy->xcom_set_ssl_mode(0 /* SSL_DISABLED */);
   s_xcom_proxy->xcom_set_ssl_fips_mode(0 /* SSL_FIPS_MODE_OFF */);
-  s_xcom_proxy->finalize_network_manager();
 }
 
 void Gcs_xcom_interface::finalize_xcom() {
@@ -653,18 +601,9 @@ void Gcs_xcom_interface::make_gcs_leave_group_on_error() {
   }
 }
 
-void Gcs_xcom_interface::announce_finalize_to_view_control() {
-  for (const auto &group : m_group_interfaces) {
-    group.second->vce->finalize();
-  }
-}
-
 enum_gcs_error Gcs_xcom_interface::finalize() {
   if (!is_initialized()) return GCS_NOK;
 
-  // Announces finalize() to all Control interfaces
-  announce_finalize_to_view_control();
-
   // Finalize and delete the engine
   gcs_engine->finalize(cleanup_xcom);
   delete gcs_engine;
@@ -689,13 +628,6 @@ enum_gcs_error Gcs_xcom_interface::finalize() {
   delete m_socket_util;
   m_socket_util = nullptr;
 
-  delete m_stats_mgr;
-  m_stats_mgr = nullptr;
-
-  delete m_xcom_stats_storage;
-  m_xcom_stats_storage = nullptr;
-
-  ::get_network_management_interface()->remove_all_network_provider();
   Gcs_xcom_utils::deinit_net();
 
   // de-initialize data structures to pass configs to xcom
@@ -749,8 +681,7 @@ gcs_xcom_group_interfaces *Gcs_xcom_interface::get_group_interfaces(
     const Gcs_group_identifier &group_identifier) {
   if (!is_initialized()) return nullptr;
 
-  // Try and retrieve already instantiated group interfaces for a certain
-  // group
+  // Try and retrieve already instantiated group interfaces for a certain group
   map<std::string, gcs_xcom_group_interfaces *>::const_iterator
       registered_group;
   registered_group = m_group_interfaces.find(group_identifier.get_group_id());
@@ -772,34 +703,27 @@ gcs_xcom_group_interfaces *Gcs_xcom_interface::get_group_interfaces(
     group_interface = new gcs_xcom_group_interfaces();
     m_group_interfaces[group_identifier.get_group_id()] = group_interface;
 
-    Gcs_xcom_statistics *stats = new Gcs_xcom_statistics(m_stats_mgr);
+    Gcs_xcom_statistics *stats = new Gcs_xcom_statistics();
 
     group_interface->statistics_interface = stats;
 
     Gcs_xcom_view_change_control_interface *vce =
         new Gcs_xcom_view_change_control();
 
-    std::unique_ptr<Network_provider_management_interface>
-        net_manager_for_communication = ::get_network_management_interface();
-
     auto *xcom_communication = new Gcs_xcom_communication(
-        m_stats_mgr, s_xcom_proxy, vce, gcs_engine, group_identifier,
-        std::move(net_manager_for_communication));
+        stats, s_xcom_proxy, vce, gcs_engine, group_identifier);
     group_interface->communication_interface = xcom_communication;
 
     Gcs_xcom_state_exchange_interface *se =
         new Gcs_xcom_state_exchange(group_interface->communication_interface);
 
     Gcs_xcom_group_management *xcom_group_management =
-        new Gcs_xcom_group_management(s_xcom_proxy, group_identifier, vce);
+        new Gcs_xcom_group_management(s_xcom_proxy, group_identifier);
     group_interface->management_interface = xcom_group_management;
 
-    std::unique_ptr<Network_provider_operations_interface>
-        net_manager_for_control = ::get_network_operations_interface();
     Gcs_xcom_control *xcom_control = new Gcs_xcom_control(
         m_node_address, m_xcom_peers, group_identifier, s_xcom_proxy,
-        xcom_group_management, gcs_engine, se, vce, m_boot, m_socket_util,
-        std::move(net_manager_for_control), m_stats_mgr);
+        xcom_group_management, gcs_engine, se, vce, m_boot, m_socket_util);
     group_interface->control_interface = xcom_control;
 
     xcom_control->set_join_behavior(
@@ -828,30 +752,6 @@ enum_gcs_error Gcs_xcom_interface::set_logger(Logger_interface *logger) {
   return Gcs_log_manager::initialize(logger);
 }
 
-enum_gcs_error Gcs_xcom_interface::setup_runtime_resources(
-    Gcs_interface_runtime_requirements &reqs) {
-  std::unique_ptr<Network_provider_management_interface> mgmtn_if =
-      ::get_network_management_interface();
-
-  if (reqs.provider != nullptr) mgmtn_if->add_network_provider(reqs.provider);
-
-  if (reqs.namespace_manager != nullptr)
-    m_netns_manager = reqs.namespace_manager;
-
-  return GCS_OK;
-}
-
-enum_gcs_error Gcs_xcom_interface::cleanup_runtime_resources(
-    Gcs_interface_runtime_requirements &reqs) {
-  std::unique_ptr<Network_provider_management_interface> mgmtn_if =
-      ::get_network_management_interface();
-
-  if (reqs.provider != nullptr)
-    mgmtn_if->remove_network_provider(reqs.provider->get_communication_stack());
-
-  return GCS_OK;
-}
-
 void Gcs_xcom_interface::clean_group_interfaces() {
   map<string, gcs_xcom_group_interfaces *>::iterator group_if;
   for (group_if = m_group_interfaces.begin();
@@ -889,7 +789,7 @@ void start_ssl() {
 
 void Gcs_xcom_interface::initialize_ssl() {
   m_wait_for_ssl_init_mutex.lock();
-  m_ssl_init_state = s_xcom_proxy->xcom_init_ssl();
+  m_ssl_init_state = (s_xcom_proxy->xcom_init_ssl() ? 1 : 0);
   m_wait_for_ssl_init_cond.broadcast();
   m_wait_for_ssl_init_mutex.unlock();
 }
@@ -942,12 +842,10 @@ bool Gcs_xcom_interface::initialize_xcom(
       interface_params.get_parameter("bootstrap_group");
   const std::string *poll_spin_loops_str =
       interface_params.get_parameter("poll_spin_loops");
-  const std::string *ip_allowlist_str =
-      interface_params.get_parameter("ip_allowlist");
+  const std::string *ip_whitelist_str =
+      interface_params.get_parameter("ip_whitelist");
   const std::string *xcom_cache_size_str =
       interface_params.get_parameter("xcom_cache_size");
-  const std::string *comm_stack_str =
-      interface_params.get_parameter("communication_stack");
 
   set_xcom_group_information(*group_name);
 
@@ -979,8 +877,8 @@ bool Gcs_xcom_interface::initialize_xcom(
                         xcom_cache_size_str->c_str());
   }
 
-  // configure allowlist
-  if (ip_allowlist_str) m_ip_allowlist.configure(*ip_allowlist_str);
+  // configure whitelist
+  if (ip_whitelist_str) m_ip_whitelist.configure(*ip_whitelist_str);
 
   // Register XCom callbacks
   ::set_xcom_data_receiver(cb_xcom_receive_data);
@@ -989,12 +887,14 @@ bool Gcs_xcom_interface::initialize_xcom(
   ::set_port_matcher(cb_xcom_match_port);
   ::set_app_snap_handler(cb_xcom_handle_app_snap);
   ::set_should_exit_getter(cb_xcom_get_should_exit);
+  ::set_app_snap_getter(cb_xcom_get_app_snap);
   ::set_xcom_run_cb(cb_xcom_ready);
   ::set_xcom_comms_cb(cb_xcom_comms);
   ::set_xcom_exit_cb(cb_xcom_exit);
   ::set_xcom_expel_cb(cb_xcom_expel);
   ::set_xcom_socket_accept_cb(cb_xcom_socket_accept);
   ::set_xcom_input_try_pop_cb(cb_xcom_input_try_pop);
+
   const std::string *wait_time_str =
       interface_params.get_parameter("wait_time");
 
@@ -1012,20 +912,11 @@ bool Gcs_xcom_interface::initialize_xcom(
   gcs_engine = new Gcs_xcom_engine();
   gcs_engine->initialize(nullptr);
 
-  /*Setup Network and SSL related*/
-  // Initialize XCom's Network Provider Manager
-  enum_transport_protocol comm_stack =
-      static_cast<enum_transport_protocol>(std::atoi(comm_stack_str->c_str()));
-  s_xcom_proxy->initialize_network_manager();
-  s_xcom_proxy->set_network_manager_active_provider(comm_stack);
-
-  // Initialize SSL related data
   const std::string *ssl_mode_str = interface_params.get_parameter("ssl_mode");
   const std::string *ssl_fips_mode_str =
       interface_params.get_parameter("ssl_fips_mode");
-  int ssl_mode_int = 0;
   if (ssl_mode_str) {
-    ssl_mode_int = s_xcom_proxy->xcom_get_ssl_mode(ssl_mode_str->c_str());
+    int ssl_mode_int = s_xcom_proxy->xcom_get_ssl_mode(ssl_mode_str->c_str());
     if (ssl_mode_int == -1) /* INVALID_SSL_MODE */
     {
       MYSQL_GCS_LOG_ERROR(
@@ -1068,8 +959,7 @@ bool Gcs_xcom_interface::initialize_xcom(
     const std::string *tls_ciphersuites =
         interface_params.get_parameter("tls_ciphersuites");
 
-    ssl_parameters ssl_configuration = {
-        ssl_mode_int,
+    Gcs_xcom_proxy::ssl_parameters ssl_configuration = {
         server_key_file ? server_key_file->c_str() : nullptr,
         server_cert_file ? server_cert_file->c_str() : nullptr,
         client_key_file ? client_key_file->c_str() : nullptr,
@@ -1078,13 +968,13 @@ bool Gcs_xcom_interface::initialize_xcom(
         ca_path ? ca_path->c_str() : nullptr,
         crl_file ? crl_file->c_str() : nullptr,
         crl_path ? crl_path->c_str() : nullptr,
-        cipher ? cipher->c_str() : nullptr};
-    tls_parameters tls_configuration = {
+        cipher ? cipher->c_str() : nullptr,
+    };
+    Gcs_xcom_proxy::tls_parameters tls_configuration = {
         tls_version ? tls_version->c_str() : nullptr,
         tls_ciphersuites ? tls_ciphersuites->c_str() : nullptr};
-
     s_xcom_proxy->xcom_set_ssl_parameters(ssl_configuration, tls_configuration);
-    /*s_xcom_proxy->xcom_init_ssl();*/
+
     m_wait_for_ssl_init_mutex.lock();
     gcs_engine->push(new Initialize_notification(start_ssl));
     while (m_ssl_init_state < 0) {
@@ -1093,13 +983,12 @@ bool Gcs_xcom_interface::initialize_xcom(
     }
     m_wait_for_ssl_init_mutex.unlock();
 
-    if (m_ssl_init_state) {
+    if (!m_ssl_init_state) {
       MYSQL_GCS_LOG_ERROR("Error starting SSL in the group communication"
                           << " engine.")
       m_ssl_init_state = -1;
       goto error;
     }
-
     m_ssl_init_state = -1;
   } else {
     MYSQL_GCS_LOG_INFO("SSL was not enabled");
@@ -1121,9 +1010,6 @@ error:
 
   clean_group_references();
 
-  // Finalize Network Manager
-  s_xcom_proxy->finalize_network_manager();
-
   /*
     If this method created the proxy object it should also
     delete it.
@@ -1267,8 +1153,6 @@ enum_gcs_error Gcs_xcom_interface::configure_message_stages(
   pipeline.register_stage<Gcs_message_stage_lz4>(compression_enabled, compression_threshold);
   pipeline.register_stage<Gcs_message_stage_lz4_v2>(compression_enabled, compression_threshold);
   pipeline.register_stage<Gcs_message_stage_split_v2>(fragmentation_enabled, fragmentation_threshold);
-  pipeline.register_stage<Gcs_message_stage_lz4_v3>(compression_enabled, compression_threshold);
-  pipeline.register_stage<Gcs_message_stage_split_v3>(fragmentation_enabled, fragmentation_threshold);
 
   error = pipeline.register_pipeline({
     {
@@ -1283,13 +1167,6 @@ enum_gcs_error Gcs_xcom_interface::configure_message_stages(
         Stage_code::ST_LZ4_V2,
         Stage_code::ST_SPLIT_V2
       }
-    },
-    {
-      Gcs_protocol_version::V3,
-      {
-        Stage_code::ST_LZ4_V3,
-        Stage_code::ST_SPLIT_V3
-      }
     }
   });
   // clang-format on
@@ -1331,8 +1208,7 @@ enum_gcs_error Gcs_xcom_interface::configure_suspicions_mgr(
         atoi(suspicions_processing_period_ptr->c_str())));
     ret = GCS_OK;
     MYSQL_GCS_LOG_TRACE(
-        "::configure_suspicions_mgr():: Set suspicions processing period to "
-        "%s "
+        "::configure_suspicions_mgr():: Set suspicions processing period to %s "
         "seconds",
         suspicions_processing_period_ptr->c_str());
   }
@@ -1342,13 +1218,14 @@ enum_gcs_error Gcs_xcom_interface::configure_suspicions_mgr(
   return ret;
 }
 
-Gcs_ip_allowlist &Gcs_xcom_interface::get_ip_allowlist() {
-  return m_ip_allowlist;
+const Gcs_ip_whitelist &Gcs_xcom_interface::get_ip_whitelist() {
+  return m_ip_whitelist;
 }
 
-void cb_xcom_receive_data(synode_no message_id, synode_no origin,
-                          site_def const *site, node_set nodes, u_int size,
+void cb_xcom_receive_data(synode_no message_id, node_set nodes, u_int size,
                           synode_no last_removed, char *data) {
+  const site_def *site = find_site_def(message_id);
+
   if (site->nodeno == VOID_NODE_NO) {
     free_node_set(&nodes);
     free(data);
@@ -1360,20 +1237,17 @@ void cb_xcom_receive_data(synode_no message_id, synode_no origin,
   free_node_set(&nodes);
 
   Gcs_xcom_notification *notification =
-      new Data_notification(do_cb_xcom_receive_data, message_id, origin,
-                            xcom_nodes, last_removed, size, data);
+      new Data_notification(do_cb_xcom_receive_data, message_id, xcom_nodes,
+                            last_removed, size, data);
   bool scheduled = gcs_engine->push(notification);
   if (!scheduled) {
     MYSQL_GCS_LOG_DEBUG(
-        "xcom_id %x Tried to enqueue a message but the member is about to "
-        "stop.",
-        get_my_xcom_id())
+        "Tried to enqueue a message but the member is about to stop.")
     free(data);
     delete xcom_nodes;
     delete notification;
   } else {
-    MYSQL_GCS_LOG_TRACE("xcom_id %x Scheduled message notification: %p",
-                        get_my_xcom_id(), notification)
+    MYSQL_GCS_LOG_TRACE("Scheduled message notification: %p", notification)
   }
 }
 
@@ -1401,7 +1275,7 @@ static inline void do_cb_xcom_receive_data_user(
                                               std::move(xcom_nodes));
 }
 
-void do_cb_xcom_receive_data(synode_no message_id, synode_no origin,
+void do_cb_xcom_receive_data(synode_no message_id,
                              Gcs_xcom_nodes *xcom_nodes_raw_ptr,
                              synode_no cache_last_removed, u_int size,
                              char *data_raw_ptr) {
@@ -1452,7 +1326,7 @@ void do_cb_xcom_receive_data(synode_no message_id, synode_no origin,
     happen because global view messages are delivered periodically after
     communication channels have been established.
 
-    When a global view message is received and can be successfully processed,
+    When a global view message is received and can be successfuly processed,
     the node can start receiving data messages. Note though that it does not
     mean that the application has received a view change notification and
     and can start receiving data messages. Recall that the state exchange
@@ -1460,8 +1334,8 @@ void do_cb_xcom_receive_data(synode_no message_id, synode_no origin,
     after that it can start receiving data messages.
 
     It is important to clean up last_accepted_xcom_config when the node leaves
-    or joins the cluster otherwise, it may receive messages before a global
-    view message is delivered.
+    or joins the cluster otherwise, it may receive messages before a global view
+    message is delivered.
   */
   bool const received_initial_global_view =
       last_accepted_xcom_config.has_view();
@@ -1472,20 +1346,19 @@ void do_cb_xcom_receive_data(synode_no message_id, synode_no origin,
   }
 
   MYSQL_GCS_LOG_TRACE(
-      "xcom_id %x xcom_receive_data_internal:: xcom_receive_data My node_id is "
-      "%d "
+      "xcom_receive_data_internal:: xcom_receive_data My node_id is %d "
       "message_id.group= %u message_id.msgno= %llu message_id.node= %d",
-      get_my_xcom_id(), xcom_nodes->get_node_no(), message_id.group_id,
+      xcom_nodes->get_node_no(), message_id.group_id,
       static_cast<long long unsigned>(message_id.msgno), message_id.node);
 
   Gcs_xcom_communication *xcom_communication =
       static_cast<Gcs_xcom_communication *>(
           intf->get_communication_session(*destination));
-  assert(xcom_communication != nullptr);
+  DBUG_ASSERT(xcom_communication != nullptr);
 
-  auto packet = Gcs_packet::make_incoming_packet(
-      std::move(data), size, message_id, origin,
-      xcom_communication->get_msg_pipeline());
+  auto packet =
+      Gcs_packet::make_incoming_packet(std::move(data), size, message_id,
+                                       xcom_communication->get_msg_pipeline());
 
   switch (packet.get_cargo_type()) {
     case Cargo_type::CT_INTERNAL_STATE_EXCHANGE:
@@ -1504,71 +1377,6 @@ void do_cb_xcom_receive_data(synode_no message_id, synode_no origin,
   }
 }
 
-static bool must_filter_xcom_view_v1(synode_no config_id,
-                                     Gcs_xcom_nodes const &xcom_nodes,
-                                     xcom_event_horizon event_horizon) {
-  bool const already_processed =
-      (last_accepted_xcom_config.has_view() &&
-       last_accepted_xcom_config.same_view(config_id));
-  bool const same_xcom_nodes =
-      last_accepted_xcom_config.same_xcom_nodes(xcom_nodes);
-  bool const different_event_horizons =
-      !last_accepted_xcom_config.same_event_horizon(event_horizon);
-  bool const event_horizon_reconfiguration =
-      (same_xcom_nodes && different_event_horizons);
-
-  bool filter_xcom_view = already_processed || event_horizon_reconfiguration;
-
-  MYSQL_GCS_TRACE_EXECUTE(if (filter_xcom_view) {
-    if (filter_xcom_view) {
-      MYSQL_GCS_LOG_TRACE(
-          "Received a global view we already processed: { group=%" PRIu32
-          " msgno=%" PRIu64 " node=%" PRIu32 " }",
-          config_id.group_id, config_id.msgno, config_id.node);
-    } else {
-      MYSQL_GCS_LOG_TRACE(
-          "Received a global view due to an event horizon reconfiguration: { "
-          "same_xcom_nodes=%d different_event_horizons=%d }",
-          same_xcom_nodes, different_event_horizons);
-    }
-  });
-
-  return filter_xcom_view;
-}
-
-static bool must_filter_xcom_view_v3(Gcs_xcom_nodes const &xcom_nodes) {
-  bool filter_xcom_view =
-      last_accepted_xcom_config.same_xcom_nodes_v3(xcom_nodes);
-
-  MYSQL_GCS_TRACE_EXECUTE(if (filter_xcom_view) {
-    MYSQL_GCS_LOG_TRACE(
-        "Received a global view with no membership changes: { "
-        "filter_xcom_view=%d }",
-        filter_xcom_view);
-  });
-
-  return filter_xcom_view;
-}
-
-static bool must_filter_xcom_view(synode_no config_id,
-                                  Gcs_xcom_nodes const &xcom_nodes,
-                                  xcom_event_horizon event_horizon,
-                                  Gcs_protocol_version protocol) {
-  switch (protocol) {
-    case Gcs_protocol_version::V1:
-    case Gcs_protocol_version::V2:
-      return must_filter_xcom_view_v1(config_id, xcom_nodes, event_horizon);
-    case Gcs_protocol_version::HIGHEST_KNOWN:
-      return must_filter_xcom_view_v3(xcom_nodes);
-    case Gcs_protocol_version::UNKNOWN:
-    case Gcs_protocol_version::V4:
-    case Gcs_protocol_version::V5:
-      assert(false && "supposedly unreachable code");
-  }
-  assert(false && "supposedly unreachable code");
-  return true;
-}
-
 void cb_xcom_receive_global_view(synode_no config_id, synode_no message_id,
                                  node_set nodes,
                                  xcom_event_horizon event_horizon) {
@@ -1637,21 +1445,20 @@ void do_cb_xcom_receive_global_view(synode_no config_id, synode_no message_id,
           xcom_nodes->get_nodes();
       std::vector<Gcs_xcom_node_information>::const_iterator nodes_it;
       MYSQL_GCS_LOG_TRACE(
-          "xcom_id %x Received global view: My node_id is %d "
+          "Received global view: My node_id is %d "
           "config_id.group= %u config_id.msgno= %llu config_id.node=%d "
           "message_id.group= %u message_id.msgno= %llu message_id.node= %d",
-          get_my_xcom_id(), node_no, config_id.group_id,
+          node_no, config_id.group_id,
           static_cast<long long unsigned>(config_id.msgno), config_id.node,
           message_id.group_id,
           static_cast<long long unsigned>(message_id.msgno), message_id.node)
 
-          MYSQL_GCS_LOG_TRACE("xcom_id %x Received global view: node set:",
-                              get_my_xcom_id())
+          MYSQL_GCS_LOG_TRACE("Received global view: node set:")
 
               for (nodes_it = nodes.begin(); nodes_it != nodes.end();
                    ++nodes_it){MYSQL_GCS_LOG_TRACE(
-                  "xcom_id %x My node_id is %d peer_ %d address: %s flag: %s",
-                  get_my_xcom_id(), node_no, (*nodes_it).get_node_no(),
+                  "My node_id is %d peer_ %d address: %s flag: %s", node_no,
+                  (*nodes_it).get_node_no(),
                   (*nodes_it).get_member_id().get_member_id().c_str(),
                   ((*nodes_it).is_alive() ? "Active" : "Failed"))})
 
@@ -1668,49 +1475,50 @@ void do_cb_xcom_receive_global_view(synode_no config_id, synode_no message_id,
     must be called because it is responsible for removing from the system
     those members that are faulty.
 
-    do_not_deliver_to_client = true prevents
-    control_if->xcom_receive_global_view(message_id, xcom_nodes,
-                                         do_not_deliver_to_client)
+    same_view = true prevents
+    control_if->xcom_receive_global_view(message_id, xcom_nodes, same_view)
     from firing a new view upwards.
-    The logic to filter out a view depends on the GCS protocol version:
-
-    GCS PROTOCOL <= V2
-    ------------------
     We want to filter out a view if:
-      a) we have already processed it, or
-      b) it is due to an event horizon reconfiguration.
+
+    a) we have already processed it, or
+    b) it is due to an event horizon reconfiguration.
 
     We identify situation (a) by comparing the incoming XCom view's config
     synod against the config synod of the last XCom view scheduled for
-    delivery. We conclude we're in situation (b) if the incoming XCom view's
-    membership is the same as the last XCom view scheduled for delivery *and*
-    their event horizons are different.
-
-    This approach is, unfortunately, insufficient. If we reconfigure the XCom's
-    event horizon to the same value it currently has, we get an XCom view where
-    the membership is the same as the last XCom view scheduled for delivery
-    *but* their event horizons are the same. We do not filter such a view, but
-    we want to. The next GCS protocol version fixes this.
-
-    GCS PROTOCOL >= V3
-    ------------------
-    We want to filter out a view if the view is *not* about a membership change.
-    In other words, we only want to let a view bubble upwards if it's about a
-    new member joining, or an existing member leaving.
-    So we filter an incoming XCom view if the incoming XCom view's membership is
-    the same as the last XCom view scheduled for delivery.
+    delivery. We identify situation (b) if the incoming XCom view's membership
+    is the same as the last XCom view scheduled for delivery *and* their event
+    horizons are different.
   */
-  auto *xcom_communication_if = static_cast<Gcs_xcom_communication *>(
-      intf->get_communication_session(*destination));
-  Gcs_protocol_version const protocol =
-      xcom_communication_if->get_protocol_version();
+  // Situation (a).
+  bool const already_processed =
+      (last_accepted_xcom_config.has_view() &&
+       last_accepted_xcom_config.same_view(config_id));
+  // Situation (b).
+  bool const same_xcom_nodes =
+      last_accepted_xcom_config.same_xcom_nodes(*xcom_nodes);
+  bool const different_event_horizons =
+      !last_accepted_xcom_config.same_event_horizon(event_horizon);
+  bool const event_horizon_reconfiguration =
+      (same_xcom_nodes && different_event_horizons);
 
-  bool do_not_deliver_to_client =
-      must_filter_xcom_view(config_id, *xcom_nodes, event_horizon, protocol);
+  bool const same_view = already_processed || event_horizon_reconfiguration;
+
+  MYSQL_GCS_TRACE_EXECUTE(if (same_view) {
+    if (already_processed) {
+      MYSQL_GCS_LOG_TRACE(
+          "Received a global view we already processed: { group=%" PRIu32
+          " msgno=%" PRIu64 " node=%" PRIu32 " }",
+          config_id.group_id, config_id.msgno, config_id.node);
+    } else {
+      MYSQL_GCS_LOG_TRACE(
+          "Received a global view due to an event horizon reconfiguration: { "
+          "same_xcom_nodes=%d different_event_horizons=%d }",
+          same_xcom_nodes, different_event_horizons);
+    }
+  });
 
   if (!(xcom_control_if->xcom_receive_global_view(
-          config_id, message_id, xcom_nodes, do_not_deliver_to_client,
-          max_synode))) {
+          config_id, message_id, xcom_nodes, same_view, max_synode))) {
     // Copy node set and config id if the view is not rejected...
     last_accepted_xcom_config.update(config_id, *xcom_nodes, event_horizon);
   } else {
@@ -1783,11 +1591,9 @@ end:
   delete xcom_nodes;
 }
 
-void cb_xcom_handle_app_snap(blob *store_snap [[maybe_unused]],
-                             synode_no start [[maybe_unused]],
-                             synode_no end [[maybe_unused]]) {}
+void cb_xcom_handle_app_snap(blob *gcs_snap MY_ATTRIBUTE((unused))) {}
 
-synode_no cb_xcom_get_app_snap(blob *gcs_snap [[maybe_unused]]) {
+synode_no cb_xcom_get_app_snap(blob *gcs_snap MY_ATTRIBUTE((unused))) {
   return null_synode;
 }
 
@@ -1798,7 +1604,7 @@ int cb_xcom_get_should_exit() {
     return 0;
 }
 
-void cb_xcom_ready(int status [[maybe_unused]]) {
+void cb_xcom_ready(int status MY_ATTRIBUTE((unused))) {
   if (s_xcom_proxy) s_xcom_proxy->xcom_signal_ready();
 }
 
@@ -1806,7 +1612,7 @@ void cb_xcom_comms(int status) {
   if (s_xcom_proxy) s_xcom_proxy->xcom_signal_comms_status_changed(status);
 }
 
-void cb_xcom_exit(int status [[maybe_unused]]) {
+void cb_xcom_exit(int status MY_ATTRIBUTE((unused))) {
   last_accepted_xcom_config.reset();
   if (s_xcom_proxy) s_xcom_proxy->xcom_signal_exit();
 }
@@ -1815,7 +1621,7 @@ void cb_xcom_exit(int status [[maybe_unused]]) {
   Callback function used by XCom to signal that a node has left the group
   because of a `die_op` or a view where the node does not belong to.
  */
-void cb_xcom_expel(int status [[maybe_unused]]) {
+void cb_xcom_expel(int status MY_ATTRIBUTE((unused))) {
   Gcs_xcom_notification *notification =
       new Expel_notification(do_cb_xcom_expel);
   bool scheduled = gcs_engine->push(notification);
@@ -1887,7 +1693,7 @@ int cb_xcom_socket_accept(int fd, site_def const *xcom_config) {
   Gcs_xcom_interface *intf =
       static_cast<Gcs_xcom_interface *>(Gcs_xcom_interface::get_interface());
 
-  Gcs_ip_allowlist &wl = intf->get_ip_allowlist();
+  const Gcs_ip_whitelist &wl = intf->get_ip_whitelist();
 
   return wl.shall_block(fd, xcom_config) ? 0 : 1;
 }
