@@ -1,16 +1,15 @@
-/* Copyright (c) 2015, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -87,7 +86,6 @@ uint64_t Xcom_member_state::get_encode_snapshot_size() const {
 
   return snapshot_size;
 }
-extern uint32_t get_my_xcom_id();
 
 bool Xcom_member_state::encode_header(uchar *buffer,
                                       uint64_t *buffer_len) const {
@@ -99,8 +97,7 @@ bool Xcom_member_state::encode_header(uchar *buffer,
   uint64_t encoded_size = get_encode_header_size();
   unsigned char *slider = buffer;
 
-  MYSQL_GCS_LOG_TRACE("xcom_id %x Encoding header for exchangeable data.",
-                      get_my_xcom_id())
+  MYSQL_GCS_LOG_TRACE("Encoding header for exchangeable data.")
 
   if (buffer == nullptr || buffer_len == nullptr) {
     MYSQL_GCS_LOG_ERROR(
@@ -147,11 +144,8 @@ bool Xcom_member_state::encode_header(uchar *buffer,
   slider += WIRE_XCOM_NODE_ID_SIZE;
   assert(static_cast<uint64_t>(slider - buffer) == encoded_size);
 
-  MYSQL_GCS_LOG_TRACE(
-      "xcom_id %x Encoded header for exchangeable data: (header)=%llu view_id "
-      "%s",
-      get_my_xcom_id(), static_cast<long long unsigned>(encoded_size),
-      m_view_id->get_representation().c_str());
+  MYSQL_GCS_LOG_TRACE("Encoded header for exchangeable data: (header)=%llu",
+                      static_cast<long long unsigned>(encoded_size));
 
   return false;
 }
@@ -165,8 +159,7 @@ bool Xcom_member_state::encode_snapshot(uchar *buffer,
   /* There is no snapshot information on protocol V1. */
   if (m_version == Gcs_protocol_version::V1) goto end;
 
-  MYSQL_GCS_LOG_TRACE("xcom_id %x Encoding snapshot for exchangeable data.",
-                      get_my_xcom_id())
+  MYSQL_GCS_LOG_TRACE("Encoding snapshot for exchangeable data.")
 
   if (buffer == nullptr || buffer_len == nullptr) {
     MYSQL_GCS_LOG_ERROR(
@@ -596,6 +589,7 @@ enum_gcs_error Gcs_xcom_state_exchange::broadcast_state(
    However, it will send an empty message anyway.
    */
   if (exchangeable_data_len > 0) {
+    uint64_t slider_total_len = 0;
     uint64_t slider_len = 0;
     for (auto it = exchangeable_data.begin(); it != it_ends; ++it) {
       auto &msg_data = (*it);
@@ -607,6 +601,7 @@ enum_gcs_error Gcs_xcom_state_exchange::broadcast_state(
             static_cast<long long unsigned>(slider_len));
         msg_data->encode(slider, &slider_len);
         slider += slider_len;
+        slider_total_len += slider_len;
       }
     }
   }
@@ -753,7 +748,7 @@ Gcs_xcom_state_exchange::compute_incompatible_members() {
 std::pair<bool, Gcs_protocol_version>
 Gcs_xcom_state_exchange::members_announce_same_version() const {
   /* Validate preconditions. */
-  assert(m_member_versions.size() > 1);
+  DBUG_ASSERT(m_member_versions.size() > 1);
 
   bool constexpr SAME_VERSION = true;
   bool constexpr DIFFERENT_VERSIONS = false;
@@ -845,7 +840,7 @@ bool Gcs_xcom_state_exchange::incompatible_with_group() const {
           "again.");
       goto end;
     } else {
-      assert(group_version != Gcs_protocol_version::UNKNOWN);
+      DBUG_ASSERT(group_version != Gcs_protocol_version::UNKNOWN);
     }
 
     /*
@@ -855,11 +850,12 @@ bool Gcs_xcom_state_exchange::incompatible_with_group() const {
     bool const supports_protocol =
         (group_version <= Gcs_protocol_version::HIGHEST_KNOWN);
     if (supports_protocol) {
-#ifndef NDEBUG
+#ifndef DBUG_OFF
       bool const failed =
 #endif
           pipeline.set_version(group_version);
-      assert(!failed && "Setting the pipeline version should not have failed");
+      DBUG_ASSERT(!failed &&
+                  "Setting the pipeline version should not have failed");
       MYSQL_GCS_LOG_INFO("This server adjusted its communication protocol to "
                          << gcs_protocol_to_mysql_version(group_version)
                          << " in order to join the group.");
@@ -872,7 +868,7 @@ bool Gcs_xcom_state_exchange::incompatible_with_group() const {
       goto end;
     }
   } else {
-    assert(m_member_versions.begin()->first == m_local_information);
+    DBUG_ASSERT(m_member_versions.begin()->first == m_local_information);
   }
 
   result = COMPATIBLE;
@@ -893,11 +889,11 @@ Gcs_xcom_state_exchange::compute_incompatible_joiners() {
 
   /* Compute the set of incompatible joiners. */
   for (Gcs_member_identifier const *joiner_id : m_ms_joined) {
-    assert(m_member_versions.find(*joiner_id) != m_member_versions.end());
+    DBUG_ASSERT(m_member_versions.find(*joiner_id) != m_member_versions.end());
     Gcs_protocol_version const &joiner_version = m_member_versions[*joiner_id];
 
-    assert(m_member_max_versions.find(*joiner_id) !=
-           m_member_max_versions.end());
+    DBUG_ASSERT(m_member_max_versions.find(*joiner_id) !=
+                m_member_max_versions.end());
     Gcs_protocol_version const &joiner_max_version =
         m_member_max_versions[*joiner_id];
 
@@ -971,7 +967,7 @@ bool Gcs_xcom_state_exchange::process_recovery_state() {
    */
   bool const only_i_exist = (m_member_states.size() == 1);
   if (only_i_exist) {
-    assert(m_member_states.begin()->first == m_local_information);
+    DBUG_ASSERT(m_member_states.begin()->first == m_local_information);
     successful = true;
     goto end;
   }
@@ -1071,8 +1067,6 @@ Gcs_xcom_view_identifier *Gcs_xcom_state_exchange::get_new_view_id() {
     }
   });
 
-  MYSQL_GCS_LOG_TRACE("get_new_view_id returns view_id %s",
-                      view_id->get_representation().c_str());
   return view_id;
 }
 
@@ -1085,8 +1079,7 @@ Gcs_xcom_view_change_control::Gcs_xcom_view_change_control()
       m_joining_leaving_mutex(),
       m_current_view(nullptr),
       m_current_view_mutex(),
-      m_belongs_to_group(false),
-      m_finalized(false) {
+      m_belongs_to_group(false) {
   m_wait_for_view_cond.init(
       key_GCS_COND_Gcs_xcom_view_change_control_m_wait_for_view_cond);
   m_wait_for_view_mutex.init(
@@ -1226,7 +1219,3 @@ bool Gcs_xcom_view_change_control::is_joining() {
 
   return retval;
 }
-
-void Gcs_xcom_view_change_control::finalize() { m_finalized.store(true); }
-
-bool Gcs_xcom_view_change_control::is_finalized() { return m_finalized.load(); }
