@@ -1,16 +1,15 @@
-/* Copyright (c) 2015, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -369,7 +368,7 @@ class Xcom_member_state {
 */
 class Gcs_xcom_state_exchange_interface {
  public:
-  virtual ~Gcs_xcom_state_exchange_interface() = default;
+  virtual ~Gcs_xcom_state_exchange_interface() {}
 
   /**
     Accomplishes all necessary initialization steps.
@@ -385,7 +384,7 @@ class Gcs_xcom_state_exchange_interface {
   virtual void reset() = 0;
 
   /**
-    Has the same behavior as the reset but additionally flushes buffered
+    Has the same behavior as the reset but additionaly flushes buffered
     messages.
   */
 
@@ -519,16 +518,16 @@ class Gcs_xcom_state_exchange : public Gcs_xcom_state_exchange_interface {
 
   explicit Gcs_xcom_state_exchange(Gcs_communication_interface *comm);
 
-  ~Gcs_xcom_state_exchange() override;
+  virtual ~Gcs_xcom_state_exchange();
 
   // Implementation of gcs_xcom_state_exchange_interface
-  void init() override;
+  void init();
 
-  void reset() override;
+  void reset();
 
-  void reset_with_flush() override;
+  void reset_with_flush();
 
-  void end() override;
+  void end();
 
   bool state_exchange(
       synode_no configuration_id, std::vector<Gcs_member_identifier *> &total,
@@ -537,38 +536,32 @@ class Gcs_xcom_state_exchange : public Gcs_xcom_state_exchange_interface {
       std::vector<std::unique_ptr<Gcs_message_data>> &exchangeable_data,
       Gcs_view *current_view, std::string *group,
       const Gcs_member_identifier &local_info,
-      const Gcs_xcom_nodes &xcom_nodes) override;
+      const Gcs_xcom_nodes &xcom_nodes);
 
   bool process_member_state(
       Xcom_member_state *ms_info, const Gcs_member_identifier &p_id,
       Gcs_protocol_version maximum_supported_protocol_version,
-      Gcs_protocol_version used_protocol_version) override;
+      Gcs_protocol_version used_protocol_version);
 
-  std::vector<Gcs_xcom_node_information> compute_incompatible_members()
-      override;
+  std::vector<Gcs_xcom_node_information> compute_incompatible_members();
 
-  bool process_recovery_state() override;
+  bool process_recovery_state();
 
-  Gcs_xcom_view_identifier *get_new_view_id() override;
+  Gcs_xcom_view_identifier *get_new_view_id();
 
-  std::set<Gcs_member_identifier *> *get_joined() override {
-    return &m_ms_joined;
-  }
+  std::set<Gcs_member_identifier *> *get_joined() { return &m_ms_joined; }
 
-  std::set<Gcs_member_identifier *> *get_left() override { return &m_ms_left; }
+  std::set<Gcs_member_identifier *> *get_left() { return &m_ms_left; }
 
-  std::set<Gcs_member_identifier *> *get_total() override {
-    return &m_ms_total;
-  }
+  std::set<Gcs_member_identifier *> *get_total() { return &m_ms_total; }
 
-  std::map<Gcs_member_identifier, Xcom_member_state *> *get_member_states()
-      override {
+  std::map<Gcs_member_identifier, Xcom_member_state *> *get_member_states() {
     return &m_member_states;
   }
 
-  std::string *get_group() override { return m_group_name; }
+  std::string *get_group() { return m_group_name; }
 
-  void compute_maximum_supported_protocol_version() override;
+  void compute_maximum_supported_protocol_version();
 
  private:
   /**
@@ -715,7 +708,7 @@ class Gcs_xcom_state_exchange : public Gcs_xcom_state_exchange_interface {
 */
 class Gcs_xcom_view_change_control_interface {
  public:
-  virtual ~Gcs_xcom_view_change_control_interface() = default;
+  virtual ~Gcs_xcom_view_change_control_interface() {}
 
   virtual void start_view_exchange() = 0;
   virtual void end_view_exchange() = 0;
@@ -739,10 +732,6 @@ class Gcs_xcom_view_change_control_interface {
   virtual void set_belongs_to_group(bool belong) = 0;
   virtual void set_unsafe_current_view(Gcs_view *current_view) = 0;
   virtual Gcs_view *get_unsafe_current_view() = 0;
-
-  // Keep track if GCS as a whole has been ordered to finalize;
-  virtual void finalize() = 0;
-  virtual bool is_finalized() = 0;
 };
 
 /*
@@ -754,30 +743,27 @@ class Gcs_xcom_view_change_control
     : public Gcs_xcom_view_change_control_interface {
  public:
   explicit Gcs_xcom_view_change_control();
-  ~Gcs_xcom_view_change_control() override;
+  virtual ~Gcs_xcom_view_change_control();
 
-  void start_view_exchange() override;
-  void end_view_exchange() override;
-  void wait_for_view_change_end() override;
-  bool is_view_changing() override;
+  void start_view_exchange();
+  void end_view_exchange();
+  void wait_for_view_change_end();
+  bool is_view_changing();
 
-  bool start_leave() override;
-  void end_leave() override;
-  bool is_leaving() override;
+  bool start_leave();
+  void end_leave();
+  bool is_leaving();
 
-  bool start_join() override;
-  void end_join() override;
-  bool is_joining() override;
+  bool start_join();
+  void end_join();
+  bool is_joining();
 
-  void set_current_view(Gcs_view *current_view) override;
-  Gcs_view *get_current_view() override;
-  bool belongs_to_group() override;
-  void set_belongs_to_group(bool belong) override;
-  void set_unsafe_current_view(Gcs_view *current_view) override;
-  Gcs_view *get_unsafe_current_view() override;
-
-  void finalize() override;
-  bool is_finalized() override;
+  void set_current_view(Gcs_view *current_view);
+  Gcs_view *get_current_view();
+  bool belongs_to_group();
+  void set_belongs_to_group(bool belong);
+  void set_unsafe_current_view(Gcs_view *current_view);
+  Gcs_view *get_unsafe_current_view();
 
  private:
   bool m_view_changing;
@@ -804,10 +790,6 @@ class Gcs_xcom_view_change_control
   */
   bool m_belongs_to_group;
 
-  /*
-   */
-  std::atomic<bool> m_finalized;
-
   /*
     Disabling the copy constructor and assignment operator.
   */
