@@ -1,16 +1,15 @@
-/* Copyright (c) 2017, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,23 +23,30 @@
 #ifndef ERR_DUMP_H
 #define ERR_DUMP_H
 
+#ifdef __cplusplus
+extern "C" {
+#endif
 #include <string.h>
 
-#include "xcom/result.h"
-#include "xcom/task_debug.h"
+#include "plugin/group_replication/libmysqlgcs/src/bindings/xcom/xcom/gcs_debug.h"
+#include "plugin/group_replication/libmysqlgcs/src/bindings/xcom/xcom/result.h"
 
 static inline void task_dump_err(int err) {
   if (err) {
 #ifndef XCOM_WITHOUT_OPENSSL
     if (is_ssl_err(err)) {
-      IFDBG(D_BUG, FN; NDBG(from_ssl_err(err), d));
+      MAY_DBG(FN; NDBG(to_ssl_err(err), d));
     } else {
 #endif
-      IFDBG(D_BUG, FN; NDBG(from_errno(err), d); STREXP(strerror(err)));
+      MAY_DBG(FN; NDBG(to_errno(err), d); STREXP(strerror(err)));
 #ifndef XCOM_WITHOUT_OPENSSL
     }
 #endif
   }
 }
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
