@@ -1,16 +1,15 @@
-/* Copyright (c) 2015, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,10 +23,11 @@
 #ifndef XCOM_TRANSPORT_H
 #define XCOM_TRANSPORT_H
 
-#include "xcom/server_struct.h"
-#include "xcom/site_struct.h"
-#include "xcom/xcom_common.h"
-#include "xdr_gen/xcom_vp.h"
+#include "plugin/group_replication/libmysqlgcs/src/bindings/xcom/xcom/xcom_common.h"
+
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 #define XDR_INT_SIZE 4
 #define MSG_HDR_SIZE (3 * XDR_INT_SIZE)
@@ -63,11 +63,6 @@
 #endif
 #define MSG_PTR(buf) &((buf)[MSG_HDR_SIZE])
 
-extern xcom_proto const my_min_xcom_version; /* The minimum protocol version I
-                                                am able to understand */
-extern xcom_proto const
-    my_xcom_version; /* The maximum protocol version I am able to understand */
-
 /* Transport level message types */
 enum x_msg_type {
   x_normal = 0,       /* Normal message */
@@ -123,20 +118,17 @@ int send_to_all_site(site_def const *s, pax_msg *p, const char *dbg);
 int send_to_others(site_def const *s, pax_msg *p, const char *dbg);
 int send_to_someone(site_def const *s, pax_msg *p, const char *dbg);
 int send_to_self_site(site_def const *s, pax_msg *p);
-int send_to_all_except_self(site_def const *s, pax_msg *p, const char *dbg);
 
-void wakeup_sender();
 int sender_task(task_arg arg);
 int local_sender_task(task_arg arg);
 int shutdown_servers();
 int srv_ref(server *s);
 int srv_unref(server *s);
 int tcp_reaper_task(task_arg arg);
-int tcp_reconnection_task(task_arg arg);
-int incoming_connection_task(task_arg arg);
+int tcp_server(task_arg arg);
 uint32_t crc32c_hash(char *buf, char *end);
-int apply_xdr(void *buff, uint32_t bufflen, xdrproc_t xdrfunc, void *xdrdata,
-              enum xdr_op op);
+int apply_xdr(xcom_proto x_proto, void *buff, uint32_t bufflen,
+              xdrproc_t xdrfunc, void *xdrdata, enum xdr_op op);
 void init_crc32c();
 void init_xcom_transport(xcom_port listen_port);
 void reset_srv_buf(srv_buf *sb);
@@ -146,6 +138,7 @@ int send_server_msg(site_def const *s, node_no i, pax_msg *p);
 double server_active(site_def const *s, node_no i);
 void update_servers(site_def *s, cargo_type operation);
 void garbage_collect_servers();
+int client_task(task_arg arg);
 int send_msg(server *s, node_no from, node_no to, uint32_t group_id,
              pax_msg *p);
 /**
@@ -162,31 +155,9 @@ void shutdown_connection(connection_descriptor *con);
 void reset_connection(connection_descriptor *con);
 void close_connection(connection_descriptor *con);
 
-int close_open_connection(connection_descriptor *conn);
-connection_descriptor *open_new_connection(
-    const char *server, xcom_port port,
-    int connection_timeout = Network_provider::default_connection_timeout(),
-    network_provider_dynamic_log_level log_level =
-        network_provider_dynamic_log_level::PROVIDED);
-connection_descriptor *open_new_local_connection(const char *server,
-                                                 xcom_port port);
-
-/**
- * @brief Tests connectivity to another node under the current configuration
- * setup.
- *
- * This function verifies that this node is able to successfully reach to the
- * other node. It is used to do a sanity check at when a new member is about
- * to be added to the list of participants and to check local node
- * connectivity, in order to validate configurations.
- *
- * @return false In case of error.
- * @return true If it succeeds to connect to a node.
- */
-bool is_able_to_connect_to_node(const char *server, const xcom_port port);
-
 #ifndef XCOM_WITHOUT_OPENSSL
 void ssl_free_con(connection_descriptor *con);
+void ssl_shutdown_con(connection_descriptor *con);
 #endif
 
 char const *xcom_proto_name(xcom_proto proto_vers);
@@ -209,8 +180,6 @@ int deserialize_msg(pax_msg *p, xcom_proto x_proto, char *buf, uint32_t buflen);
 xcom_proto common_xcom_version(site_def const *site);
 xcom_proto get_latest_common_proto();
 xcom_proto set_latest_common_proto(xcom_proto x_proto);
-extern linkage connect_wait;
-extern int connect_tcp(char *server, xcom_port port, int *ret);
 
 /**
  * @brief Returns the version from which nodes are able to speak IPv6
@@ -231,7 +200,7 @@ xcom_proto minimum_ipv6_version();
  * @param port the resulting port
  * @return int true (1) in case of parse error
  */
-int get_ip_and_port(char const *address, char ip[IP_MAX_SIZE], xcom_port *port);
+int get_ip_and_port(char *address, char ip[IP_MAX_SIZE], xcom_port *port);
 
 /**
  * @brief Checks if an incoming node is eligible to enter the group
@@ -239,7 +208,7 @@ int get_ip_and_port(char const *address, char ip[IP_MAX_SIZE], xcom_port *port);
  * This function checks if a new node entering the group is able to be part of
  * it.
  * This is needed duw to downgrade procedures to server versions that do not
- * speak IPv6. One will check if:
+ * speak IPv6. One wil check if:
  * - Our server is being contacted by a server that has a lower version than the
  * IPv6 baseline
  * - Check if the current configuration is all reachable by an IPv4 node
@@ -253,7 +222,11 @@ int is_new_node_eligible_for_ipv6(xcom_proto incoming_proto,
                                   const site_def *current_site_def);
 
 #define INITIAL_CONNECT_WAIT 0.1
-#define MAX_CONNECT_WAIT 10.0
-#define CONNECT_WAIT_INCREASE 1.0
+#define MAX_CONNECT_WAIT 1.0
+#define CONNECT_WAIT_INCREASE 1.1
+
+#ifdef __cplusplus
+}
+#endif
 
 #endif
