@@ -1,16 +1,15 @@
-/* Copyright (c) 2018, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,48 +25,42 @@
 #include <mysql/components/services/log_builtins.h>
 #include "plugin/group_replication/include/consistency_manager.h"
 #include "plugin/group_replication/include/plugin.h"
-#include "plugin/group_replication/include/plugin_handlers/metrics_handler.h"
 #include "plugin/group_replication/include/plugin_messages/sync_before_execution_message.h"
 #include "plugin/group_replication/include/plugin_messages/transaction_prepared_message.h"
 #include "plugin/group_replication/include/plugin_psi.h"
-#include "string_with_len.h"
 
 Transaction_consistency_info::Transaction_consistency_info(
-    my_thread_id thread_id, bool local_transaction, const gr::Gtid_tsid &tsid,
-    bool is_tsid_specified, rpl_sidno sidno, rpl_gno gno,
+    my_thread_id thread_id, bool local_transaction, const rpl_sid *sid,
+    rpl_sidno sidno, rpl_gno gno,
     enum_group_replication_consistency_level consistency_level,
-    Members_list *members_that_must_prepare_the_transaction)
+    std::list<Gcs_member_identifier> *members_that_must_prepare_the_transaction)
     : m_thread_id(thread_id),
       m_local_transaction(local_transaction),
-      m_tsid_specified(is_tsid_specified),
-      m_tsid(tsid),
+      m_sid_specified(sid != nullptr ? true : false),
       m_sidno(sidno),
       m_gno(gno),
       m_consistency_level(consistency_level),
       m_members_that_must_prepare_the_transaction(
           members_that_must_prepare_the_transaction),
       m_transaction_prepared_locally(local_transaction),
-      m_transaction_prepared_remotely(false),
-      m_begin_timestamp(Metrics_handler::get_current_time()) {
+      m_transaction_prepared_remotely(false) {
   DBUG_TRACE;
-  assert(m_consistency_level >= GROUP_REPLICATION_CONSISTENCY_AFTER);
-  assert(nullptr != m_members_that_must_prepare_the_transaction);
+  DBUG_ASSERT(m_consistency_level >= GROUP_REPLICATION_CONSISTENCY_AFTER);
+  DBUG_ASSERT(nullptr != m_members_that_must_prepare_the_transaction);
   DBUG_PRINT(
       "info",
-      ("thread_id: %u; local_transaction: %d; gtid: %d:%" PRId64
-       "; tsid_specified: "
+      ("thread_id: %u; local_transaction: %d; gtid: %d:%llu; sid_specified: "
        "%d; consistency_level: %d; "
        "transaction_prepared_locally: %d; transaction_prepared_remotely: %d",
-       m_thread_id, m_local_transaction, m_sidno, m_gno, m_tsid_specified,
+       m_thread_id, m_local_transaction, m_sidno, m_gno, m_sid_specified,
        m_consistency_level, m_transaction_prepared_locally,
        m_transaction_prepared_remotely));
 
-  m_members_that_must_prepare_the_transaction_lock = std::make_unique<
-      Checkable_rwlock>(
-#ifdef HAVE_PSI_INTERFACE
-      key_GR_RWLOCK_transaction_consistency_info_members_that_must_prepare_the_transaction
-#endif
-  );
+  if (nullptr != sid) {
+    m_sid.copy_from(*sid);
+  } else {
+    m_sid.clear();
+  }
 }
 
 Transaction_consistency_info::~Transaction_consistency_info() {
@@ -96,14 +89,10 @@ Transaction_consistency_info::get_consistency_level() {
 }
 
 bool Transaction_consistency_info::is_a_single_member_group() {
-  Checkable_rwlock::Guard g(*m_members_that_must_prepare_the_transaction_lock,
-                            Checkable_rwlock::READ_LOCK);
   return 0 == m_members_that_must_prepare_the_transaction->size();
 }
 
 bool Transaction_consistency_info::is_the_transaction_prepared_remotely() {
-  Checkable_rwlock::Guard g(*m_members_that_must_prepare_the_transaction_lock,
-                            Checkable_rwlock::READ_LOCK);
   return m_transaction_prepared_remotely ||
          m_members_that_must_prepare_the_transaction->empty();
 }
@@ -112,7 +101,7 @@ int Transaction_consistency_info::after_applier_prepare(
     my_thread_id thread_id,
     Group_member_info::Group_member_status member_status) {
   DBUG_TRACE;
-  assert(m_consistency_level >= GROUP_REPLICATION_CONSISTENCY_AFTER);
+  DBUG_ASSERT(m_consistency_level >= GROUP_REPLICATION_CONSISTENCY_AFTER);
   /*
     Update thread_id with applier worker id in order to allow it to
     be resumed once all prepared messages are delivered.
@@ -122,23 +111,16 @@ int Transaction_consistency_info::after_applier_prepare(
 
   DBUG_PRINT(
       "info",
-      ("thread_id: %u; local_transaction: %d; gtid: %d:%" PRId64
-       "; sid_specified: "
+      ("thread_id: %u; local_transaction: %d; gtid: %d:%llu; sid_specified: "
        "%d; consistency_level: %d; "
        "transaction_prepared_locally: %d; transaction_prepared_remotely: %d; "
        "member_status: %d",
-       m_thread_id, m_local_transaction, m_sidno, m_gno, m_tsid_specified,
+       m_thread_id, m_local_transaction, m_sidno, m_gno, m_sid_specified,
        m_consistency_level, m_transaction_prepared_locally,
        m_transaction_prepared_remotely, member_status));
 
-  m_members_that_must_prepare_the_transaction_lock->rdlock();
-  const bool needs_to_acknowledge =
-      std::find(m_members_that_must_prepare_the_transaction->begin(),
-                m_members_that_must_prepare_the_transaction->end(),
-                local_member_info->get_gcs_member_id()) !=
-      m_members_that_must_prepare_the_transaction->end();
-  m_members_that_must_prepare_the_transaction_lock->unlock();
-  if (!needs_to_acknowledge) {
+  // Only ONLINE members do acknowledge transactions prepare.
+  if (Group_member_info::MEMBER_ONLINE != member_status) {
     return 0;
   }
 
@@ -149,7 +131,7 @@ int Transaction_consistency_info::after_applier_prepare(
             "signal.after_before_message_send_after_applier_prepare_waiting "
             "wait_for "
             "signal.after_before_message_send_after_applier_prepare_continue";
-        assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
+        DBUG_ASSERT(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
       };);
 
   DBUG_EXECUTE_IF(
@@ -159,16 +141,16 @@ int Transaction_consistency_info::after_applier_prepare(
             "signal.after_supress_message_send_after_applier_prepare_waiting "
             "wait_for "
             "signal.after_supress_message_send_after_applier_prepare_continue";
-        assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
+        DBUG_ASSERT(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
         return 0;
       };);
 
-  Transaction_prepared_message message(m_tsid, m_tsid_specified, m_gno);
+  Transaction_prepared_message message((m_sid_specified ? &m_sid : nullptr),
+                                       m_gno);
   if (gcs_module->send_message(message)) {
     /* purecov: begin inspected */
-    const std::string tsid_str = m_tsid.to_string();
     LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_SEND_TRX_PREPARED_MESSAGE_FAILED,
-                 tsid_str.c_str(), m_gno, m_thread_id);
+                 m_sidno, m_gno, m_thread_id);
     return 1;
     /* purecov: end */
   }
@@ -181,40 +163,28 @@ int Transaction_consistency_info::handle_remote_prepare(
   DBUG_TRACE;
   DBUG_PRINT(
       "info",
-      ("thread_id: %u; local_transaction: %d; gtid: %d:%" PRId64
-       "; sid_specified: "
+      ("thread_id: %u; local_transaction: %d; gtid: %d:%llu; sid_specified: "
        "%d; consistency_level: %d; "
        "transaction_prepared_locally: %d; transaction_prepared_remotely: %d",
-       m_thread_id, m_local_transaction, m_sidno, m_gno, m_tsid_specified,
+       m_thread_id, m_local_transaction, m_sidno, m_gno, m_sid_specified,
        m_consistency_level, m_transaction_prepared_locally,
        m_transaction_prepared_remotely));
 
-  m_members_that_must_prepare_the_transaction_lock->wrlock();
   m_members_that_must_prepare_the_transaction->remove(gcs_member_id);
-  const bool members_that_must_prepare_the_transaction_empty =
-      m_members_that_must_prepare_the_transaction->empty();
-  m_members_that_must_prepare_the_transaction_lock->unlock();
 
-  if (members_that_must_prepare_the_transaction_empty) {
+  if (m_members_that_must_prepare_the_transaction->empty()) {
     m_transaction_prepared_remotely = true;
 
     if (m_transaction_prepared_locally) {
       if (transactions_latch->releaseTicket(m_thread_id)) {
         /* purecov: begin inspected */
-        const std::string tsid_str = m_tsid.to_string();
         LogPluginErr(ERROR_LEVEL,
                      ER_GRP_RPL_RELEASE_COMMIT_AFTER_GROUP_PREPARE_FAILED,
-                     tsid_str.c_str(), m_gno, m_thread_id);
+                     m_sidno, m_gno, m_thread_id);
         return CONSISTENCY_INFO_OUTCOME_ERROR;
         /* purecov: end */
       }
 
-      if (m_local_transaction) {
-        const auto end_timestamp = Metrics_handler::get_current_time();
-        metrics_handler->add_transaction_consistency_after_termination(
-            m_begin_timestamp, end_timestamp);
-      }
-
       return CONSISTENCY_INFO_OUTCOME_COMMIT;
     }
   }
@@ -236,42 +206,19 @@ int Transaction_consistency_info::handle_member_leave(
 
   DBUG_PRINT(
       "info",
-      ("thread_id: %u; local_transaction: %d; gtid: %d:%" PRId64
-       "; tsid_specified: "
+      ("thread_id: %u; local_transaction: %d; gtid: %d:%llu; sid_specified: "
        "%d; consistency_level: %d; "
        "transaction_prepared_locally: %d; transaction_prepared_remotely: %d; "
        "error: %d",
-       m_thread_id, m_local_transaction, m_sidno, m_gno, m_tsid_specified,
+       m_thread_id, m_local_transaction, m_sidno, m_gno, m_sid_specified,
        m_consistency_level, m_transaction_prepared_locally,
        m_transaction_prepared_remotely, error));
 
   return error;
 }
 
-uint64_t Transaction_consistency_info::get_begin_timestamp() const {
-  return m_begin_timestamp;
-}
-
-std::string Transaction_consistency_info::get_tsid_string() const {
-  assert(!m_tsid.to_string().empty());
-  return m_tsid.to_string();
-}
-
 Transaction_consistency_manager::Transaction_consistency_manager()
-    : m_map(
-          Malloc_allocator<std::pair<const Transaction_consistency_manager_key,
-                                     Transaction_consistency_info *>>(
-              key_consistent_transactions)),
-      m_prepared_transactions_on_my_applier(
-          Malloc_allocator<Transaction_consistency_manager_key>(
-              key_consistent_transactions_prepared)),
-      m_new_transactions_waiting(
-          Malloc_allocator<my_thread_id>(key_consistent_transactions_waiting)),
-      m_delayed_view_change_events(
-          Malloc_allocator<Transaction_consistency_manager_pevent_pair>(
-              key_consistent_transactions_delayed_view_change)),
-      m_plugin_stopping(true),
-      m_primary_election_active(false) {
+    : m_plugin_stopping(true) {
   m_map_lock = new Checkable_rwlock(
 #ifdef HAVE_PSI_INTERFACE
       key_GR_RWLOCK_transaction_consistency_manager_map
@@ -286,10 +233,10 @@ Transaction_consistency_manager::Transaction_consistency_manager()
 }
 
 Transaction_consistency_manager::~Transaction_consistency_manager() {
-  assert(m_map.empty());
-  assert(m_prepared_transactions_on_my_applier.empty());
-  assert(m_new_transactions_waiting.empty());
-  assert(m_delayed_view_change_events.empty());
+  DBUG_ASSERT(m_map.empty());
+  DBUG_ASSERT(m_prepared_transactions_on_my_applier.empty());
+  DBUG_ASSERT(m_new_transactions_waiting.empty());
+  DBUG_ASSERT(m_delayed_view_change_events.empty());
   delete m_map_lock;
   delete m_prepared_transactions_on_my_applier_lock;
 }
@@ -297,6 +244,11 @@ Transaction_consistency_manager::~Transaction_consistency_manager() {
 void Transaction_consistency_manager::clear() {
   DBUG_TRACE;
   m_map_lock->wrlock();
+  for (typename Transaction_consistency_manager_map::iterator it =
+           m_map.begin();
+       it != m_map.end(); ++it) {
+    delete it->second; /* purecov: inspected */
+  }
   m_map.clear();
   m_map_lock->unlock();
 
@@ -305,8 +257,7 @@ void Transaction_consistency_manager::clear() {
   m_new_transactions_waiting.clear();
 
   while (!m_delayed_view_change_events.empty()) {
-    auto element = m_delayed_view_change_events.front();
-    delete element.first;
+    delete m_delayed_view_change_events.front();
     m_delayed_view_change_events.pop_front();
   }
   m_delayed_view_change_events.clear();
@@ -314,24 +265,22 @@ void Transaction_consistency_manager::clear() {
 }
 
 int Transaction_consistency_manager::after_certification(
-    std::unique_ptr<Transaction_consistency_info> transaction_info) {
+    Transaction_consistency_info *transaction_info) {
   DBUG_TRACE;
-  assert(transaction_info->get_consistency_level() >=
-         GROUP_REPLICATION_CONSISTENCY_AFTER);
+  DBUG_ASSERT(transaction_info->get_consistency_level() >=
+              GROUP_REPLICATION_CONSISTENCY_AFTER);
   int error = 0;
   Transaction_consistency_manager_key key(transaction_info->get_sidno(),
                                           transaction_info->get_gno());
-
   m_map_lock->wrlock();
 
   typename Transaction_consistency_manager_map::iterator it = m_map.find(key);
   if (it != m_map.end()) {
     /* purecov: begin inspected */
-    const std::string tsid_str = transaction_info->get_tsid_string();
+    m_map_lock->unlock();
     LogPluginErr(ERROR_LEVEL,
                  ER_GRP_RPL_TRX_ALREADY_EXISTS_ON_TCM_ON_AFTER_CERTIFICATION,
-                 tsid_str.c_str(), transaction_info->get_gno());
-    m_map_lock->unlock();
+                 transaction_info->get_sidno(), transaction_info->get_gno());
     return 1;
     /* purecov: end */
   }
@@ -340,46 +289,26 @@ int Transaction_consistency_manager::after_certification(
   if (transaction_info->is_local_transaction() &&
       transaction_info->is_a_single_member_group()) {
     transactions_latch->releaseTicket(transaction_info->get_thread_id());
-    const auto end_timestamp = Metrics_handler::get_current_time();
-    metrics_handler->add_transaction_consistency_after_termination(
-        transaction_info->get_begin_timestamp(), end_timestamp);
-
+    delete transaction_info;
     m_map_lock->unlock();
     return 0;
   }
 
-  DBUG_PRINT("info",
-             ("gtid: %d:%" PRId64 "; consistency_level: %d; ",
-              transaction_info->get_sidno(), transaction_info->get_gno(),
-              transaction_info->get_consistency_level()));
-
-  if (transaction_info->is_local_transaction()) m_last_local_transaction = key;
-
-  const std::string tsid_string = transaction_info->get_tsid_string();
-  const rpl_gno gno = transaction_info->get_gno();
   std::pair<typename Transaction_consistency_manager_map::iterator, bool> ret =
-      m_map.insert(Transaction_consistency_manager_pair(
-          key, std::move(transaction_info)));
-
+      m_map.insert(Transaction_consistency_manager_pair(key, transaction_info));
   if (ret.second == false) {
     /* purecov: begin inspected */
     LogPluginErr(ERROR_LEVEL,
                  ER_GRP_RPL_FAILED_TO_INSERT_TRX_ON_TCM_ON_AFTER_CERTIFICATION,
-                 tsid_string.c_str(), gno);
+                 transaction_info->get_sidno(), transaction_info->get_gno());
     error = 1;
     /* purecov: end */
   }
 
-  DBUG_EXECUTE_IF("group_replication_consistency_manager_after_certification", {
-    const char act[] =
-        "now signal "
-        "signal.group_replication_consistency_manager_after_certification_"
-        "reached "
-        "wait_for "
-        "signal.group_replication_consistency_manager_after_certification_"
-        "continue";
-    assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
-  };);
+  DBUG_PRINT(
+      "info",
+      ("gtid: %d:%llu; consistency_level: %d; ", transaction_info->get_sidno(),
+       transaction_info->get_gno(), transaction_info->get_consistency_level()));
 
   m_map_lock->unlock();
   return error;
@@ -400,8 +329,7 @@ int Transaction_consistency_manager::after_applier_prepare(
     return 0;
   }
 
-  auto &transaction_info = it->second;
-  const std::string tsid_string = transaction_info->get_tsid_string();
+  Transaction_consistency_info *transaction_info = it->second;
   const bool transaction_prepared_remotely =
       transaction_info->is_the_transaction_prepared_remotely();
 
@@ -410,16 +338,16 @@ int Transaction_consistency_manager::after_applier_prepare(
     /* purecov: begin inspected */
     LogPluginErr(ERROR_LEVEL,
                  ER_GRP_RPL_REGISTER_TRX_TO_WAIT_FOR_GROUP_PREPARE_FAILED,
-                 tsid_string.c_str(), gno, thread_id);
+                 sidno, gno, thread_id);
     m_map_lock->unlock();
     return 1;
     /* purecov: end */
   }
 
-  DBUG_PRINT("info",
-             ("gtid: %d:%" PRId64 "; consistency_level: %d; ",
-              transaction_info->get_sidno(), transaction_info->get_gno(),
-              transaction_info->get_consistency_level()));
+  DBUG_PRINT(
+      "info",
+      ("gtid: %d:%llu; consistency_level: %d; ", transaction_info->get_sidno(),
+       transaction_info->get_gno(), transaction_info->get_consistency_level()));
 
   // Mark the transaction as prepared for consistent reads.
   m_prepared_transactions_on_my_applier_lock->wrlock();
@@ -440,15 +368,17 @@ int Transaction_consistency_manager::after_applier_prepare(
     const char act[] =
         "now signal signal.after_applier_prepare_waiting wait_for "
         "signal.after_applier_prepare_continue";
-    assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
+    DBUG_ASSERT(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
   };);
 
   if (!transaction_prepared_remotely &&
       transactions_latch->waitTicket(thread_id)) {
+    /* purecov: begin inspected */
     LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_TRX_WAIT_FOR_GROUP_PREPARE_FAILED,
-                 tsid_string.c_str(), gno, thread_id);
+                 sidno, gno, thread_id);
     error = 1;
     goto end;
+    /* purecov: end */
   }
 
   // Remove transaction from map
@@ -456,6 +386,7 @@ int Transaction_consistency_manager::after_applier_prepare(
     m_map_lock->wrlock();
     typename Transaction_consistency_manager_map::iterator it = m_map.find(key);
     if (it != m_map.end()) {
+      delete it->second;
       m_map.erase(it);
     }
     m_map_lock->unlock();
@@ -474,18 +405,18 @@ end:
 }
 
 int Transaction_consistency_manager::handle_remote_prepare(
-    const gr::Gtid_tsid &tsid, bool is_tsid_specified, rpl_gno gno,
+    const rpl_sid *sid, rpl_gno gno,
     const Gcs_member_identifier &gcs_member_id) {
   DBUG_TRACE;
   rpl_sidno sidno = 0;
 
-  if (is_tsid_specified) {
+  if (sid != nullptr) {
     /*
      This transaction has a UUID different from the group name,
      thence we need to fetch the corresponding sidno from the
-     global tsid_map.
+     global sid_map.
     */
-    sidno = get_sidno_from_global_tsid_map(tsid);
+    sidno = get_sidno_from_global_sid_map(*sid);
     if (sidno <= 0) {
       /* purecov: begin inspected */
       LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_FAILED_TO_GENERATE_SIDNO_FOR_GRP);
@@ -495,7 +426,7 @@ int Transaction_consistency_manager::handle_remote_prepare(
   } else {
     /*
       This transaction has the group name as UUID, so we can skip
-      a lock on global tsid_map and use the cached group sidno.
+      a lock on global sid_map and use the cached group sidno.
     */
     sidno = get_group_sidno();
   }
@@ -519,55 +450,22 @@ int Transaction_consistency_manager::handle_remote_prepare(
     }
 
     /* purecov: begin inspected */
-    const gr::Gtid_tsid tsid = get_tsid_from_global_tsid_map(sidno);
-    assert(!tsid.to_string().empty());
     LogPluginErr(ERROR_LEVEL,
                  ER_GRP_RPL_TRX_DOES_NOT_EXIST_ON_TCM_ON_HANDLE_REMOTE_PREPARE,
-                 tsid.to_string().c_str(), gno);
+                 sidno, gno);
     m_map_lock->unlock();
     return 1;
     /* purecov: end */
   }
 
-  auto &transaction_info = it->second;
+  Transaction_consistency_info *transaction_info = it->second;
 
-  DBUG_PRINT("info",
-             ("gtid: %d:%" PRId64 "; consistency_level: %d; ",
-              transaction_info->get_sidno(), transaction_info->get_gno(),
-              transaction_info->get_consistency_level()));
+  DBUG_PRINT(
+      "info",
+      ("gtid: %d:%llu; consistency_level: %d; ", transaction_info->get_sidno(),
+       transaction_info->get_gno(), transaction_info->get_consistency_level()));
 
   int result = transaction_info->handle_remote_prepare(gcs_member_id);
-
-  if (transaction_info->is_transaction_prepared_locally() &&
-      transaction_info->is_the_transaction_prepared_remotely()) {
-    auto it = m_delayed_view_change_events.begin();
-    while (it != m_delayed_view_change_events.end()) {
-      Transaction_consistency_manager_key view_key = it->second;
-      /*
-        Check if there is pending view change processing post the current
-        transaction. If so, process all view changes which were queued post the
-        current transaction.
-      */
-      if (view_key == key) {
-        Pipeline_event *pevent = it->first;
-        Continuation cont;
-        pevent->set_delayed_view_change_resumed();
-        int error = applier_module->inject_event_into_pipeline(pevent, &cont);
-        if (!cont.is_transaction_discarded()) {
-          delete pevent;
-        }
-        m_delayed_view_change_events.erase(it++);
-        if (error) {
-          LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_FAILED_TO_LOG_VIEW_CHANGE);
-          m_map_lock->unlock();
-          return 1;
-        }
-      } else {
-        ++it;
-      }
-    }
-  }
-
   if (CONSISTENCY_INFO_OUTCOME_ERROR == result) {
     /* purecov: begin inspected */
     m_map_lock->unlock();
@@ -581,6 +479,7 @@ int Transaction_consistency_manager::handle_remote_prepare(
     m_map_lock->wrlock();
     typename Transaction_consistency_manager_map::iterator it = m_map.find(key);
     if (it != m_map.end()) {
+      delete it->second;
       m_map.erase(it);
     }
     m_map_lock->unlock();
@@ -601,9 +500,10 @@ int Transaction_consistency_manager::handle_member_leave(
 
   typename Transaction_consistency_manager_map::iterator it = m_map.begin();
   while (it != m_map.end()) {
-    auto &transaction_info = it->second;
+    Transaction_consistency_info *transaction_info = it->second;
     int result = transaction_info->handle_member_leave(leaving_members);
     if (CONSISTENCY_INFO_OUTCOME_COMMIT == result) {
+      delete it->second;
       m_map.erase(it++);
     } else {
       ++it;
@@ -617,7 +517,7 @@ int Transaction_consistency_manager::handle_member_leave(
 int Transaction_consistency_manager::after_commit(my_thread_id, rpl_sidno sidno,
                                                   rpl_gno gno) {
   DBUG_TRACE;
-  DBUG_PRINT("info", ("gtid: %d:%" PRId64, sidno, gno));
+  DBUG_PRINT("info", ("gtid: %d:%llu", sidno, gno));
   int error = 0;
 
   // Only acquire a write lock if really needed.
@@ -635,7 +535,7 @@ int Transaction_consistency_manager::after_commit(my_thread_id, rpl_sidno sidno,
 
 int Transaction_consistency_manager::before_transaction_begin(
     my_thread_id thread_id, ulong gr_consistency_level, ulong timeout,
-    enum_rpl_channel_type rpl_channel_type, const THD *thd) {
+    enum_rpl_channel_type rpl_channel_type) {
   DBUG_TRACE;
   int error = 0;
 
@@ -660,10 +560,12 @@ int Transaction_consistency_manager::before_transaction_begin(
 
   if (GROUP_REPLICATION_CONSISTENCY_BEFORE == consistency_level ||
       GROUP_REPLICATION_CONSISTENCY_BEFORE_AND_AFTER == consistency_level) {
-    error = transaction_begin_sync_before_execution(
-        thread_id, consistency_level, timeout, thd);
+    error = transaction_begin_sync_before_execution(thread_id,
+                                                    consistency_level, timeout);
     if (error) {
+      /* purecov: begin inspected */
       return error;
+      /* purecov: end */
     }
   }
 
@@ -674,25 +576,18 @@ int Transaction_consistency_manager::before_transaction_begin(
     /* purecov: end */
   }
 
-  if (m_primary_election_active) {
-    if (consistency_level ==
-            GROUP_REPLICATION_CONSISTENCY_BEFORE_ON_PRIMARY_FAILOVER ||
-        consistency_level == GROUP_REPLICATION_CONSISTENCY_AFTER) {
-      return m_hold_transactions.wait_until_primary_failover_complete(timeout);
-    }
-  }
-
   return 0;
 }
 
 int Transaction_consistency_manager::transaction_begin_sync_before_execution(
     my_thread_id thread_id,
-    enum_group_replication_consistency_level consistency_level [[maybe_unused]],
-    ulong timeout, const THD *thd) const {
+    enum_group_replication_consistency_level consistency_level
+        MY_ATTRIBUTE((unused)),
+    ulong timeout) const {
   DBUG_TRACE;
-  int error{0};
-  assert(GROUP_REPLICATION_CONSISTENCY_BEFORE == consistency_level ||
-         GROUP_REPLICATION_CONSISTENCY_BEFORE_AND_AFTER == consistency_level);
+  DBUG_ASSERT(GROUP_REPLICATION_CONSISTENCY_BEFORE == consistency_level ||
+              GROUP_REPLICATION_CONSISTENCY_BEFORE_AND_AFTER ==
+                  consistency_level);
   DBUG_PRINT("info", ("thread_id: %d; consistency_level: %d", thread_id,
                       consistency_level));
 
@@ -700,8 +595,6 @@ int Transaction_consistency_manager::transaction_begin_sync_before_execution(
     return ER_GRP_TRX_CONSISTENCY_BEGIN_NOT_ALLOWED;
   }
 
-  const auto begin_timestamp = Metrics_handler::get_current_time();
-
   if (transactions_latch->registerTicket(thread_id)) {
     /* purecov: begin inspected */
     LogPluginErr(
@@ -714,14 +607,12 @@ int Transaction_consistency_manager::transaction_begin_sync_before_execution(
 
   // send message
   Sync_before_execution_message message(thread_id);
-  if (gcs_module->send_message(message, false, thd)) {
-    // sent message failed so ticket aren't needed, it won't receive
-    // notifications
-    transactions_latch->releaseTicket(thread_id);
-    transactions_latch->waitTicket(thread_id);
+  if (gcs_module->send_message(message)) {
+    /* purecov: begin inspected */
     LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_SEND_TRX_SYNC_BEFORE_EXECUTION_FAILED,
                  thread_id);
     return ER_GRP_TRX_CONSISTENCY_BEFORE;
+    /* purecov: end */
   }
 
   DBUG_PRINT("info", ("waiting for Sync_before_execution_message"));
@@ -735,34 +626,18 @@ int Transaction_consistency_manager::transaction_begin_sync_before_execution(
     /* purecov: end */
   }
 
-  std::string applier_retrieved_gtids;
-  Replication_thread_api applier_channel("group_replication_applier");
-  if (applier_channel.get_retrieved_gtid_set(applier_retrieved_gtids)) {
-    LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_GTID_SET_EXTRACT_ERROR);
-    return ER_GRP_TRX_CONSISTENCY_BEFORE;
-  }
+  DBUG_PRINT("info", ("waiting for channel_wait_until_apply_queue_applied()"));
 
-  DBUG_PRINT("info", ("waiting for wait_for_gtid_set_committed()"));
-
-  /*
-    We want to keep the current thd stage info of
-    "Executing hook on transaction begin.", thence we disable
-    `update_thd_status`.
-  */
-  if (wait_for_gtid_set_committed(applier_retrieved_gtids.c_str(), timeout,
-                                  false /* update_thd_status */)) {
+  if (channel_wait_until_apply_queue_applied("group_replication_applier",
+                                             timeout) < 0) {
     /* purecov: begin inspected */
     LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_TRX_WAIT_FOR_GROUP_GTID_EXECUTED,
                  thread_id);
-    error = ER_GRP_TRX_CONSISTENCY_BEFORE;
+    return ER_GRP_TRX_CONSISTENCY_BEFORE;
     /* purecov: end */
   }
 
-  const auto end_timestamp = Metrics_handler::get_current_time();
-  metrics_handler->add_transaction_consistency_before_begin(begin_timestamp,
-                                                            end_timestamp);
-
-  return error;
+  return 0;
 }
 
 int Transaction_consistency_manager::handle_sync_before_execution_message(
@@ -787,7 +662,6 @@ int Transaction_consistency_manager::
     transaction_begin_sync_prepared_transactions(my_thread_id thread_id,
                                                  ulong timeout) {
   DBUG_TRACE;
-  int error{0};
   Transaction_consistency_manager_key key(0, 0);
 
   // Take a read lock to check queue size.
@@ -813,8 +687,6 @@ int Transaction_consistency_manager::
 
   DBUG_PRINT("info", ("thread_id: %d", thread_id));
 
-  const auto begin_timestamp = Metrics_handler::get_current_time();
-
   if (transactions_latch->registerTicket(thread_id)) {
     /* purecov: begin inspected */
     LogPluginErr(ERROR_LEVEL,
@@ -839,15 +711,11 @@ int Transaction_consistency_manager::
     remove_prepared_transaction(key);
     LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_WAIT_FOR_DEPENDENCIES_FAILED,
                  thread_id);
-    error = ER_GRP_TRX_CONSISTENCY_AFTER_ON_TRX_BEGIN;
+    return ER_GRP_TRX_CONSISTENCY_AFTER_ON_TRX_BEGIN;
     /* purecov: end */
   }
 
-  const uint64_t end_timestamp = Metrics_handler::get_current_time();
-  metrics_handler->add_transaction_consistency_after_sync(begin_timestamp,
-                                                          end_timestamp);
-
-  return error;
+  return 0;
 }
 
 bool Transaction_consistency_manager::has_local_prepared_transactions() {
@@ -858,7 +726,7 @@ bool Transaction_consistency_manager::has_local_prepared_transactions() {
   for (typename Transaction_consistency_manager_map::iterator it =
            m_map.begin();
        it != m_map.end(); it++) {
-    auto &transaction_info = it->second;
+    Transaction_consistency_info *transaction_info = it->second;
 
     if (transaction_info->is_local_transaction() &&
         transaction_info->is_transaction_prepared_locally()) {
@@ -874,13 +742,13 @@ bool Transaction_consistency_manager::has_local_prepared_transactions() {
 int Transaction_consistency_manager::schedule_view_change_event(
     Pipeline_event *pevent) {
   DBUG_TRACE;
-#ifndef NDEBUG
-  m_map_lock->rdlock();
-  assert(!m_map.empty());
-  m_map_lock->unlock();
-#endif
-  m_delayed_view_change_events.push_back(
-      std::make_pair(pevent, m_last_local_transaction));
+  Transaction_consistency_manager_key key(-1, -1);
+
+  m_prepared_transactions_on_my_applier_lock->wrlock();
+  m_prepared_transactions_on_my_applier.push_back(key);
+  m_delayed_view_change_events.push_back(pevent);
+  m_prepared_transactions_on_my_applier_lock->unlock();
+
   return 0;
 }
 
@@ -889,7 +757,7 @@ int Transaction_consistency_manager::remove_prepared_transaction(
   DBUG_TRACE;
   int error = 0;
 
-  DBUG_PRINT("info", ("gtid: %d:%" PRId64, key.first, key.second));
+  DBUG_PRINT("info", ("gtid: %d:%llu", key.first, key.second));
 
   m_prepared_transactions_on_my_applier_lock->wrlock();
   if (key.first > 0 && key.second > 0) {
@@ -901,7 +769,7 @@ int Transaction_consistency_manager::remove_prepared_transaction(
         m_prepared_transactions_on_my_applier.front();
 
     if (0 == next_prepared.first && 0 == next_prepared.second) {
-      assert(!m_new_transactions_waiting.empty());
+      DBUG_ASSERT(!m_new_transactions_waiting.empty());
       // This is a new transaction waiting, lets wake it up.
       m_prepared_transactions_on_my_applier.pop_front();
       my_thread_id waiting_thread_id = m_new_transactions_waiting.front();
@@ -912,15 +780,29 @@ int Transaction_consistency_manager::remove_prepared_transaction(
 
       if (transactions_latch->releaseTicket(waiting_thread_id)) {
         /* purecov: begin inspected */
-        const gr::Gtid_tsid tsid = get_tsid_from_global_tsid_map(key.first);
-        assert(!tsid.to_string().empty());
         LogPluginErr(
             ERROR_LEVEL,
             ER_GRP_RPL_RELEASE_BEGIN_TRX_AFTER_DEPENDENCIES_COMMIT_FAILED,
-            tsid.to_string().c_str(), key.second, waiting_thread_id);
+            key.first, key.second, waiting_thread_id);
         error = 1;
         /* purecov: end */
       }
+    } else if (-1 == next_prepared.first && -1 == next_prepared.second) {
+      DBUG_ASSERT(!m_delayed_view_change_events.empty());
+      /*
+        This is a view change that was waiting for prepared
+        transactions completion, now it is time to log it.
+      */
+      m_prepared_transactions_on_my_applier.pop_front();
+      Pipeline_event *pevent = m_delayed_view_change_events.front();
+      m_delayed_view_change_events.pop_front();
+
+      Continuation cont;
+      int error = applier_module->inject_event_into_pipeline(pevent, &cont);
+      delete pevent;
+      if (error) {
+        abort_plugin_process("unable to log the View_change_log_event");
+      }
     } else {
       break;
     }
@@ -947,15 +829,6 @@ void Transaction_consistency_manager::unregister_transaction_observer() {
   group_transaction_observation_manager->unregister_transaction_observer(this);
 }
 
-void Transaction_consistency_manager::enable_primary_election_checks() {
-  m_hold_transactions.enable();
-  m_primary_election_active = true;
-}
-
-void Transaction_consistency_manager::disable_primary_election_checks() {
-  m_primary_election_active = false;
-  m_hold_transactions.disable();
-}
 /*
   These methods are necessary to fulfil the Group_transaction_listener
   interface.
