@@ -1,16 +1,15 @@
-/* Copyright (c) 2018, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,25 +21,17 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 #include "plugin/group_replication/include/group_actions/primary_election_action.h"
+#include <plugin/group_replication/include/plugin_handlers/persistent_variables_handler.h>
 #include "plugin/group_replication/include/plugin.h"
 #include "plugin/group_replication/include/plugin_handlers/server_ongoing_transactions_handler.h"
 #include "plugin/group_replication/include/plugin_messages/group_action_message.h"
-#include "plugin/group_replication/include/services/system_variable/set_system_variable.h"
-#include "string_with_len.h"
 #include "template_utils.h"
 
 Primary_election_action::Primary_election_action()
-    : Primary_election_action(std::string(""), 0) {
-  if (local_member_info && local_member_info->in_primary_mode()) {
-    action_execution_mode = PRIMARY_ELECTION_ACTION_PRIMARY_SWITCH;
-  } else {
-    action_execution_mode = PRIMARY_ELECTION_ACTION_MODE_SWITCH;
-  }
-}
+    : Primary_election_action(std::string(""), 0) {}
 
-Primary_election_action::Primary_election_action(
-    std::string primary_uuid_arg, my_thread_id thread_id,
-    int32 transaction_wait_timeout_arg)
+Primary_election_action::Primary_election_action(std::string primary_uuid_arg,
+                                                 my_thread_id thread_id)
     : action_execution_mode(PRIMARY_ELECTION_ACTION_END),
       current_action_phase(PRIMARY_NO_PHASE),
       single_election_action_aborted(false),
@@ -53,53 +44,26 @@ Primary_election_action::Primary_election_action(
       is_primary(false),
       invoking_thread_id(thread_id),
       is_primary_election_invoked(false),
-      is_transaction_queue_applied(false),
-      m_transaction_wait_timeout(transaction_wait_timeout_arg) {
+      is_primary_elected(false),
+      primary_changed(false),
+      is_transaction_queue_applied(false) {
   mysql_mutex_init(key_GR_LOCK_primary_election_action_phase, &phase_lock,
                    MY_MUTEX_INIT_FAST);
   mysql_mutex_init(key_GR_LOCK_primary_election_action_notification,
                    &notification_lock, MY_MUTEX_INIT_FAST);
   mysql_cond_init(key_GR_COND_primary_election_action_notification,
                   &notification_cond);
-  if (local_member_info && local_member_info->in_primary_mode()) {
-    action_execution_mode = PRIMARY_ELECTION_ACTION_PRIMARY_SWITCH;
-  } else {
-    action_execution_mode = PRIMARY_ELECTION_ACTION_MODE_SWITCH;
-  }
 }
 
 Primary_election_action::~Primary_election_action() {
   mysql_mutex_destroy(&phase_lock);
   mysql_mutex_destroy(&notification_lock);
   mysql_cond_destroy(&notification_cond);
-  /**
-    Assert has been used since object has not been destroyed at the exit point
-    code. If this ever occurs, the scenario needs to be analyzed to understand
-    why object was not deleted on the exit points of the primary election.
-  */
-  assert(transaction_monitor_thread == nullptr);
-  /**
-    delete transaction_monitor_thread is present at exit points when a change of
-    primary is finishing. However if delete is not triggered at those exit
-    points then object should be destroyed here. This code is added for safety.
-  */
-  stop_transaction_monitor_thread();
-}
-
-bool Primary_election_action::stop_transaction_monitor_thread() {
-  bool ret = false;
-  if (transaction_monitor_thread) {
-    ret = transaction_monitor_thread->terminate();
-    delete transaction_monitor_thread;
-    transaction_monitor_thread = nullptr;
-  }
-  return ret;
 }
 
 void Primary_election_action::get_action_message(
     Group_action_message **message) {
-  *message = new Group_action_message(appointed_primary_uuid,
-                                      m_transaction_wait_timeout);
+  *message = new Group_action_message(appointed_primary_uuid);
 }
 
 int Primary_election_action::process_action_message(
@@ -138,19 +102,11 @@ int Primary_election_action::process_action_message(
       /* purecov: end */
     }
 
-    Group_member_info appointed_primary_info;
-    if (group_member_mgr->get_group_member_info(appointed_primary_uuid,
-                                                appointed_primary_info)) {
-      std::string error_message{
-          "The appointed primary member is no more member of the group."};
-      execution_message_area.set_execution_message(
-          Group_action_diagnostics::GROUP_ACTION_LOG_ERROR, error_message);
-      validation_handler.terminates_validation_structures();
-      return 1;
-    } else {
-      appointed_primary_gcs_id.assign(
-          appointed_primary_info.get_gcs_member_id().get_member_id());
-    }
+    Group_member_info *appointed_primary_info =
+        group_member_mgr->get_group_member_info(appointed_primary_uuid);
+    appointed_primary_gcs_id.assign(
+        appointed_primary_info->get_gcs_member_id().get_member_id());
+    delete appointed_primary_info;
   }
 
   std::string error_message;
@@ -163,41 +119,29 @@ int Primary_election_action::process_action_message(
     return 1;
   }
 
-  if (PRIMARY_ELECTION_ACTION_PRIMARY_SWITCH == action_execution_mode) {
-    if (local_member_info->get_role() ==
-            Group_member_info::MEMBER_ROLE_PRIMARY &&
-        message.get_transaction_monitor_timeout() != -1) {
-      /*
-       Here Transaction_monitor_thread object is created but thread is not
-       started.
-       @note Transaction_monitor_thread thread start is delayed because
-       validation of running channels has not been done yet.
-       @note If we need to start the Transaction_monitor_thread thread now, we
-       will have to check if there is any running channel on this member. It is
-       not future safe, since validations may increase in future.
-       @refer validate_election for duplicate post validation conditions
-      */
-      transaction_monitor_thread = new Transaction_monitor_thread(
-          message.get_transaction_monitor_timeout());
-    }
-
-    Group_member_info primary_info;
-    if (!group_member_mgr->get_primary_member_info(primary_info)) {
+  if (local_member_info && local_member_info->in_primary_mode()) {
+    action_execution_mode = PRIMARY_ELECTION_ACTION_PRIMARY_SWITCH;
+    Group_member_info *primary_info =
+        group_member_mgr->get_primary_member_info();
+    if (primary_info != nullptr) {
       invoking_member_gcs_id.assign(
-          primary_info.get_gcs_member_id().get_member_id());
+          primary_info->get_gcs_member_id().get_member_id());
       is_primary = invoking_member_gcs_id ==
                    local_member_info->get_gcs_member_id().get_member_id();
-      old_primary_uuid.assign(primary_info.get_uuid());
+      old_primary_uuid.assign(primary_info->get_uuid());
+      delete primary_info;
     }
+  } else {
+    action_execution_mode = PRIMARY_ELECTION_ACTION_MODE_SWITCH;
   }
 
   /*
-    If there is no old primary to invoke the election then select
-    1. The action invocation member
-    2. If not there, then the first member in the group (after sort).
+   * If there is no old primary to invoke the election then select
+     1. The action invocation member
+     2. If not there, then the first member in the group (after sort).
   */
   if (invoking_member_gcs_id.empty()) {
-    Group_member_info_list *all_members_info =
+    std::vector<Group_member_info *> *all_members_info =
         group_member_mgr->get_all_members();
     std::sort(all_members_info->begin(), all_members_info->end());
 
@@ -220,7 +164,7 @@ int Primary_election_action::process_action_message(
     delete all_members_info;
   }
 
-  m_execution_status = PRIMARY_ELECTION_INIT;
+  is_primary_elected = false;
   is_transaction_queue_applied = false;
   change_action_phase(PRIMARY_VALIDATION_PHASE);
   group_events_observation_manager->register_group_event_observer(this);
@@ -230,7 +174,7 @@ int Primary_election_action::process_action_message(
 
 Group_action::enum_action_execution_result
 Primary_election_action::execute_action(
-    bool, Plugin_stage_monitor_handler *stage_handler, Notification_context *) {
+    bool, Plugin_stage_monitor_handler *stage_handler) {
   bool mode_is_set = false;
   bool action_terminated = false;
   int error = 0;
@@ -248,9 +192,10 @@ Primary_election_action::execute_action(
   if (validation_handler.prepare_election()) {
     /* purecov: begin inspected */
     error = 1;
-    error_msg =
-        " This operation ended in error as it was not possible to share "
-        "information for the election process.";
+    execution_message_area.set_execution_message(
+        Group_action_diagnostics::GROUP_ACTION_LOG_ERROR,
+        "This operation ended in error as it was not possible to share "
+        "information for the election process.");
     goto end;
     /* purecov: end */
   }
@@ -268,19 +213,10 @@ Primary_election_action::execute_action(
       goto end;
     }
   }
-  if (transaction_monitor_thread != nullptr) {
-    if (transaction_monitor_thread->start()) {
-      error = 1;
-      error_msg =
-          " This operation ended in error as it was not possible to stop the "
-          "ongoing transactions.";
-      goto end;
-    }
-  }
 
   DBUG_EXECUTE_IF("group_replication_block_primary_action_validation", {
     const char act[] = "now wait_for signal.primary_action_continue";
-    assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
+    DBUG_ASSERT(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
   });
 
   stage_handler->set_completed_work(1);
@@ -303,9 +239,10 @@ Primary_election_action::execute_action(
                   &single_election_action_aborted, invoking_thread_id)) {
         /* purecov: begin inspected */
         error = 2;
-        error_msg =
-            " This operation ended in error as it was not possible to wait for "
-            "the execution of server running transactions.";
+        execution_message_area.set_execution_message(
+            Group_action_diagnostics::GROUP_ACTION_LOG_ERROR,
+            "This operation ended in error as it was not possible to wait for "
+            "the execution of server running transactions.");
         goto end;
         /* purecov: end */
       }
@@ -340,8 +277,7 @@ Primary_election_action::execute_action(
   stage_handler->set_stage(stage_key, __FILE__, __LINE__, 2, 0);
 
   mysql_mutex_lock(&notification_lock);
-  while (PRIMARY_ELECTION_INIT == m_execution_status &&
-         !single_election_action_aborted) {
+  while (!is_primary_elected && !single_election_action_aborted) {
     DBUG_PRINT("sleep", ("Waiting for the primary to be elected."));
     mysql_cond_wait(&notification_cond, &notification_lock);
   }
@@ -349,8 +285,7 @@ Primary_election_action::execute_action(
 
   stage_handler->set_completed_work(1);
 
-  if (PRIMARY_ELECTION_END_ERROR == m_execution_status ||
-      PRIMARY_ELECTION_INIT == m_execution_status) {
+  if (!primary_changed) {
     goto end;
   }
 
@@ -399,19 +334,10 @@ end:
   }
 
   group_events_observation_manager->unregister_group_event_observer(this);
-  /**
-    Observers have been de-registered so transaction_monitor_thread should have
-    been destroyed by now in after_view_change or after_primary_election. If
-    not, then destroy the object now(during UDF failure it has been observed
-    intermittently object was not destroyed in after_view_change or
-    after_primary_election).
-  */
-  stop_transaction_monitor_thread();
 
   error += error_on_primary_election;
-  log_result_execution(error,
-                       single_election_action_aborted && !action_terminated,
-                       mode_is_set, error_msg);
+  log_result_execution(
+      error, single_election_action_aborted && !action_terminated, mode_is_set);
 
   // Don't abort if it already finished
   if ((!single_election_action_aborted && !error) || action_terminated)
@@ -428,12 +354,6 @@ end:
 
 bool Primary_election_action::stop_action_execution(bool killed) {
   mysql_mutex_lock(&notification_lock);
-  /**
-    When a UDF group_replication_set_as_primary is killed using 'KILL
-    connection_id' code reaches here.
-  */
-  stop_transaction_monitor_thread();
-
   action_killed = killed;
   single_election_action_aborted = true;
   mysql_cond_broadcast(&notification_cond);
@@ -441,6 +361,17 @@ bool Primary_election_action::stop_action_execution(bool killed) {
   return false;
 }
 
+const char *Primary_election_action::get_action_name() {
+  switch (action_execution_mode) {
+    case PRIMARY_ELECTION_ACTION_PRIMARY_SWITCH:
+      return "Primary election change";
+    case PRIMARY_ELECTION_ACTION_MODE_SWITCH:
+      return "Change to single primary mode";
+    default:
+      return "Single primary related change"; /* purecov: inspected */
+  }
+}
+
 void Primary_election_action::change_action_phase(
     enum_primary_election_phase phase) {
   mysql_mutex_lock(&phase_lock);
@@ -470,7 +401,6 @@ int Primary_election_action::after_view_change(
     bool *skip_primary_election, enum_primary_election_mode *election_mode_out,
     std::string &proposed_primary) {
   if (is_leaving) {
-    stop_transaction_monitor_thread(); /* purecov: inspected */
     return 0;
   }
 
@@ -542,7 +472,7 @@ int Primary_election_action::after_view_change(
   if (is_old_primary_leaving && current_action_phase < PRIMARY_ELECTION_PHASE) {
     *skip_primary_election = true;
 
-    Group_member_info_list *all_members_info =
+    std::vector<Group_member_info *> *all_members_info =
         group_member_mgr->get_all_members();
     std::sort(all_members_info->begin(), all_members_info->end(),
               Group_member_info::comparator_group_member_uuid);
@@ -554,7 +484,7 @@ int Primary_election_action::after_view_change(
           new_invoking_member->get_gcs_member_id().get_member_id());
     } else {
       /* purecov: begin inspected */
-      assert(proposed_primary.empty());
+      DBUG_ASSERT(proposed_primary.empty());
       *skip_primary_election = false;
       if (PRIMARY_ELECTION_ACTION_PRIMARY_SWITCH == action_execution_mode) {
         proposed_primary.assign(appointed_primary_uuid);
@@ -586,11 +516,11 @@ int Primary_election_action::after_view_change(
     election methods.
   */
   if (current_action_phase == PRIMARY_ELECTION_PHASE) {
-    Group_member_info member_info;
-    const bool member_info_not_found =
-        group_member_mgr->get_primary_member_info(member_info);
-    if (member_info_not_found || is_appointed_primary_leaving) {
-      assert(appointed_primary_gcs_id.empty() || is_appointed_primary_leaving);
+    Group_member_info *member_info =
+        group_member_mgr->get_primary_member_info();
+    if (member_info == nullptr || is_appointed_primary_leaving) {
+      DBUG_ASSERT(appointed_primary_gcs_id.empty() ||
+                  is_appointed_primary_leaving);
       *skip_primary_election = false;
       std::string new_primary("");
       if (PRIMARY_ELECTION_ACTION_PRIMARY_SWITCH == action_execution_mode) {
@@ -634,6 +564,7 @@ int Primary_election_action::after_view_change(
       }
       appointed_primary_gcs_id.clear();
     }
+    delete member_info;
   }
 
   /*
@@ -652,18 +583,16 @@ int Primary_election_action::after_view_change(
   return 0;
 }
 
-int Primary_election_action::after_primary_election(
-    std::string elected_uuid,
-    enum_primary_election_primary_change_status primary_change_status,
-    enum_primary_election_mode primary_election_mode, int error) {
+int Primary_election_action::after_primary_election(std::string elected_uuid,
+                                                    bool did_primary_change,
+                                                    enum_primary_election_mode,
+                                                    int error) {
   // We are leaving the group but we can speed up the process
   if (error == PRIMARY_ELECTION_PROCESS_ERROR) {
     error_on_primary_election = true; /* purecov: inspected */
     stop_action_execution(false);     /* purecov: inspected */
   }
-  if (UNSAFE_OLD_PRIMARY == primary_election_mode) {
-    stop_transaction_monitor_thread();
-  }
+
   // No candidates? Just abort
   if (error == PRIMARY_ELECTION_NO_CANDIDATES_ERROR) {
     /* purecov: begin inspected */
@@ -673,16 +602,15 @@ int Primary_election_action::after_primary_election(
     mysql_mutex_unlock(&notification_lock);
     /* purecov: end */
   }
-  if (enum_primary_election_primary_change_status::PRIMARY_DID_CHANGE ==
-          primary_change_status ||
-      enum_primary_election_primary_change_status::
-              PRIMARY_DID_NOT_CHANGE_PRIMARY_LEFT_FORCE_ELECTION_END ==
-          primary_change_status) {
+
+  if (did_primary_change || (!appointed_primary_uuid.empty() &&
+                             elected_uuid == appointed_primary_uuid)) {
     mysql_mutex_lock(&notification_lock);
 
-    m_execution_status = PRIMARY_ELECTION_END_ELECTION;
+    is_primary_elected = true;
     // Set this also to true for election invoked on member leaves
     is_primary_election_invoked = true;
+    primary_changed = did_primary_change;
 
     /*
       Note that for all elections types besides DEAD_OLD_PRIMARY this observer
@@ -731,23 +659,32 @@ int Primary_election_action::before_message_handling(
 }
 
 bool Primary_election_action::persist_variable_values() {
-  int error = 0;
-  Set_system_variable set_system_variable;
+  Sql_service_command_interface *sql_command_interface =
+      new Sql_service_command_interface();
+  long error = 0;
+  std::string var_name, var_value;
 
-  if ((error =
-           set_system_variable
-               .set_persist_only_group_replication_enforce_update_everywhere_checks(
-                   false))) {
+  if ((error = sql_command_interface->establish_session_connection(
+           PSESSION_USE_THREAD, GROUPREPL_USER, get_plugin_pointer())))
     goto end; /* purecov: inspected */
-  }
 
-  if ((error =
-           set_system_variable
-               .set_persist_only_group_replication_single_primary_mode(true))) {
+  var_name.assign("group_replication_enforce_update_everywhere_checks");
+  var_value.assign("OFF");
+
+  if ((error = set_persist_only_variable(var_name, var_value,
+                                         sql_command_interface)))
+    goto end; /* purecov: inspected */
+
+  var_name.assign("group_replication_single_primary_mode");
+  var_value.assign("ON");
+
+  if ((error = set_persist_only_variable(var_name, var_value,
+                                         sql_command_interface)))
+
     goto end; /* purecov: inspected */
-  }
 
 end:
+  delete sql_command_interface;
   if (error) {
     execution_message_area.set_warning_message(
         "It was not possible to persist the configuration values for this "
@@ -758,8 +695,7 @@ end:
 }
 
 void Primary_election_action::log_result_execution(bool error, bool aborted,
-                                                   bool mode_changed,
-                                                   std::string &error_msg) {
+                                                   bool mode_changed) {
   if (error) {
     execution_message_area.set_execution_message(
         Group_action_diagnostics::GROUP_ACTION_LOG_ERROR,
@@ -771,8 +707,6 @@ void Primary_election_action::log_result_execution(bool error, bool aborted,
           "primary mode, but the configuration was not "
           "persisted."); /* purecov: inspected */
     }
-    if (!error_msg.empty())
-      execution_message_area.append_execution_message(error_msg);
     return; /* purecov: inspected */
   }
 
