@@ -1,16 +1,15 @@
-/* Copyright (c) 2014, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -35,35 +34,6 @@ using std::map;
 using std::string;
 using std::vector;
 
-Group_member_info::Group_member_info(PSI_mutex_key psi_mutex_key_arg)
-    : Plugin_gcs_message(CT_MEMBER_INFO_MESSAGE),
-      hostname(""),
-      port(0),
-      uuid(""),
-      status(MEMBER_OFFLINE),
-      write_set_extraction_algorithm(0),
-      gtid_assignment_block_size(0),
-      unreachable(false),
-      role(MEMBER_ROLE_SECONDARY),
-      configuration_flags(0),
-      conflict_detection_enable(true),
-      member_weight(0),
-      lower_case_table_names(false),
-      default_table_encryption(false),
-      group_action_running(false),
-      primary_election_running(false),
-      recovery_endpoints("DEFAULT"),
-      m_view_change_uuid("AUTOMATIC"),
-      m_allow_single_leader(false),
-      m_preemptive_garbage_collection(PREEMPTIVE_GARBAGE_COLLECTION_DEFAULT),
-#ifndef NDEBUG
-      skip_encode_default_table_encryption(false),
-      m_skip_encode_view_change_uuid(false),
-#endif
-      psi_mutex_key(psi_mutex_key_arg) {
-  mysql_mutex_init(psi_mutex_key, &update_lock, MY_MUTEX_INIT_FAST);
-}
-
 Group_member_info::Group_member_info(
     const char *hostname_arg, uint port_arg, const char *uuid_arg,
     int write_set_extraction_algorithm_arg,
@@ -74,8 +44,6 @@ Group_member_info::Group_member_info(
     Group_member_info::Group_member_role role_arg, bool in_single_primary_mode,
     bool has_enforces_update_everywhere_checks, uint member_weight_arg,
     uint lower_case_table_names_arg, bool default_table_encryption_arg,
-    const char *recovery_endpoints_arg, const char *view_change_uuid_arg,
-    bool allow_single_leader, bool preemptive_garbage_collection,
     PSI_mutex_key psi_mutex_key_arg)
     : Plugin_gcs_message(CT_MEMBER_INFO_MESSAGE),
       hostname(hostname_arg),
@@ -93,15 +61,8 @@ Group_member_info::Group_member_info(
       default_table_encryption(default_table_encryption_arg),
       group_action_running(false),
       primary_election_running(false),
-      recovery_endpoints(recovery_endpoints_arg ? recovery_endpoints_arg
-                                                : "DEFAULT"),
-      m_view_change_uuid(view_change_uuid_arg ? view_change_uuid_arg
-                                              : "AUTOMATIC"),
-      m_allow_single_leader(allow_single_leader),
-      m_preemptive_garbage_collection(preemptive_garbage_collection),
-#ifndef NDEBUG
+#ifndef DBUG_OFF
       skip_encode_default_table_encryption(false),
-      m_skip_encode_view_change_uuid(false),
 #endif
       psi_mutex_key(psi_mutex_key_arg) {
   mysql_mutex_init(psi_mutex_key, &update_lock, MY_MUTEX_INIT_FAST);
@@ -137,17 +98,8 @@ Group_member_info::Group_member_info(Group_member_info &other)
       default_table_encryption(other.get_default_table_encryption()),
       group_action_running(other.is_group_action_running()),
       primary_election_running(other.is_primary_election_running()),
-      recovery_endpoints(other.get_recovery_endpoints()),
-      m_view_change_uuid(other.get_view_change_uuid()),
-      m_allow_single_leader(other.get_allow_single_leader()),
-      m_group_action_running_name(other.get_group_action_running_name()),
-      m_group_action_running_description(
-          other.get_group_action_running_description()),
-      m_preemptive_garbage_collection(
-          other.get_preemptive_garbage_collection()),
-#ifndef NDEBUG
+#ifndef DBUG_OFF
       skip_encode_default_table_encryption(false),
-      m_skip_encode_view_change_uuid(false),
 #endif
       psi_mutex_key(other.psi_mutex_key) {
   mysql_mutex_init(psi_mutex_key, &update_lock, MY_MUTEX_INIT_FAST);
@@ -166,13 +118,8 @@ Group_member_info::Group_member_info(const uchar *data, size_t len,
       default_table_encryption(false),
       group_action_running(false),
       primary_election_running(false),
-      recovery_endpoints("DEFAULT"),
-      m_view_change_uuid("AUTOMATIC"),
-      m_allow_single_leader(false),
-      m_preemptive_garbage_collection(PREEMPTIVE_GARBAGE_COLLECTION_DEFAULT),
-#ifndef NDEBUG
+#ifndef DBUG_OFF
       skip_encode_default_table_encryption(false),
-      m_skip_encode_view_change_uuid(false),
 #endif
       psi_mutex_key(psi_mutex_key_arg) {
   mysql_mutex_init(psi_mutex_key, &update_lock, MY_MUTEX_INIT_FAST);
@@ -186,7 +133,7 @@ Group_member_info::~Group_member_info() {
 }
 
 void Group_member_info::update(
-    const char *hostname_arg, uint port_arg, const char *uuid_arg,
+    char *hostname_arg, uint port_arg, char *uuid_arg,
     int write_set_extraction_algorithm_arg,
     const std::string &gcs_member_id_arg,
     Group_member_info::Group_member_status status_arg,
@@ -194,9 +141,7 @@ void Group_member_info::update(
     ulonglong gtid_assignment_block_size_arg,
     Group_member_info::Group_member_role role_arg, bool in_single_primary_mode,
     bool has_enforces_update_everywhere_checks, uint member_weight_arg,
-    uint lower_case_table_names_arg, bool default_table_encryption_arg,
-    const char *recovery_endpoints_arg, const char *view_change_uuid_arg,
-    bool allow_single_leader, bool preemptive_garbage_collection) {
+    uint lower_case_table_names_arg, bool default_table_encryption_arg) {
   MUTEX_LOCK(lock, &update_lock);
 
   hostname.assign(hostname_arg);
@@ -229,51 +174,6 @@ void Group_member_info::update(
   /* Handle enforce_update_everywhere_checks */
   if (has_enforces_update_everywhere_checks)
     configuration_flags |= CNF_ENFORCE_UPDATE_EVERYWHERE_CHECKS_F;
-
-  recovery_endpoints.assign(recovery_endpoints_arg);
-
-  m_view_change_uuid.assign(view_change_uuid_arg);
-  m_allow_single_leader = allow_single_leader;
-  m_preemptive_garbage_collection = preemptive_garbage_collection;
-}
-
-void Group_member_info::update(Group_member_info &other) {
-  MUTEX_LOCK(lock, &update_lock);
-  hostname.assign(other.get_hostname());
-  port = other.get_port();
-  uuid.assign(other.get_uuid());
-  status = other.get_recovery_status();
-  delete gcs_member_id;
-  gcs_member_id =
-      new Gcs_member_identifier(other.get_gcs_member_id().get_member_id());
-  delete member_version;
-  member_version = new Member_version(other.get_member_version().get_version());
-  executed_gtid_set.assign(other.get_gtid_executed());
-  purged_gtid_set.assign(other.get_gtid_purged());
-  retrieved_gtid_set.assign(other.get_gtid_retrieved());
-  write_set_extraction_algorithm = other.get_write_set_extraction_algorithm();
-  gtid_assignment_block_size = other.get_gtid_assignment_block_size();
-  unreachable = other.is_unreachable();
-  role = other.get_role();
-  configuration_flags = other.get_configuration_flags();
-  conflict_detection_enable = other.is_conflict_detection_enabled();
-  member_weight = other.get_member_weight();
-  lower_case_table_names = other.get_lower_case_table_names();
-  default_table_encryption = other.get_default_table_encryption();
-  group_action_running = other.is_group_action_running();
-  primary_election_running = other.is_primary_election_running();
-  recovery_endpoints.assign(other.get_recovery_endpoints());
-  m_view_change_uuid.assign(other.get_view_change_uuid());
-  m_allow_single_leader = other.get_allow_single_leader();
-  m_group_action_running_name.assign(other.get_group_action_running_name());
-  m_group_action_running_description.assign(
-      other.get_group_action_running_description());
-  m_preemptive_garbage_collection = other.get_preemptive_garbage_collection();
-#ifndef NDEBUG
-  skip_encode_default_table_encryption =
-      other.skip_encode_default_table_encryption;
-  m_skip_encode_view_change_uuid = other.m_skip_encode_view_change_uuid;
-#endif
 }
 
 /*
@@ -338,7 +238,7 @@ void Group_member_info::encode_payload(
 
   uint16 lower_case_table_names_aux =
       static_cast<uint16>(lower_case_table_names);
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   if (lower_case_table_names != SKIP_ENCODING_LOWER_CASE_TABLE_NAMES)
 #endif
     encode_payload_item_int2(buffer, PIT_LOWER_CASE_TABLE_NAME,
@@ -357,7 +257,7 @@ void Group_member_info::encode_payload(
                            is_election_running_aux);
 
   char default_table_encryption_aux = default_table_encryption ? '1' : '0';
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   if (!skip_encode_default_table_encryption)
 #endif
     encode_payload_item_char(buffer, PIT_DEFAULT_TABLE_ENCRYPTION,
@@ -365,35 +265,6 @@ void Group_member_info::encode_payload(
 
   encode_payload_item_string(buffer, PIT_PURGED_GTID, purged_gtid_set.c_str(),
                              purged_gtid_set.length());
-
-  encode_payload_item_string(buffer, PIT_RECOVERY_ENDPOINTS,
-                             recovery_endpoints.c_str(),
-                             recovery_endpoints.length());
-
-#ifndef NDEBUG
-  if (!m_skip_encode_view_change_uuid)
-#endif
-    encode_payload_item_string(buffer, PIT_VIEW_CHANGE_UUID,
-                               m_view_change_uuid.c_str(),
-                               m_view_change_uuid.length());
-
-  char allow_single_leader_aux = m_allow_single_leader ? '1' : '0';
-  encode_payload_item_char(buffer, PIT_ALLOW_SINGLE_LEADER,
-                           allow_single_leader_aux);
-
-  if (group_action_running) {
-    encode_payload_item_string(buffer, PIT_GROUP_ACTION_RUNNING_NAME,
-                               m_group_action_running_name.c_str(),
-                               m_group_action_running_name.length());
-    encode_payload_item_string(buffer, PIT_GROUP_ACTION_RUNNING_DESCRIPTION,
-                               m_group_action_running_description.c_str(),
-                               m_group_action_running_description.length());
-  }
-
-  char preemptive_garbage_collection_aux =
-      m_preemptive_garbage_collection ? '1' : '0';
-  encode_payload_item_char(buffer, PIT_PREEMPTIVE_GARBAGE_COLLECTION,
-                           preemptive_garbage_collection_aux);
 }
 
 void Group_member_info::decode_payload(const unsigned char *buffer,
@@ -466,6 +337,7 @@ void Group_member_info::decode_payload(const unsigned char *buffer,
       case PIT_CONFLICT_DETECTION_ENABLE:
         if (slider + payload_item_length <= end) {
           unsigned char conflict_detection_enable_aux = *slider;
+          slider += payload_item_length;
           conflict_detection_enable =
               (conflict_detection_enable_aux == '1') ? true : false;
         }
@@ -474,6 +346,7 @@ void Group_member_info::decode_payload(const unsigned char *buffer,
       case PIT_MEMBER_WEIGHT:
         if (slider + payload_item_length <= end) {
           uint16 member_weight_aux = uint2korr(slider);
+          slider += payload_item_length;
           member_weight = (uint)member_weight_aux;
         }
         break;
@@ -481,6 +354,7 @@ void Group_member_info::decode_payload(const unsigned char *buffer,
       case PIT_LOWER_CASE_TABLE_NAME:
         if (slider + payload_item_length <= end) {
           uint16 lower_case_table_names_aux = uint2korr(slider);
+          slider += payload_item_length;
           lower_case_table_names =
               static_cast<uint>(lower_case_table_names_aux);
         }
@@ -489,6 +363,7 @@ void Group_member_info::decode_payload(const unsigned char *buffer,
       case PIT_GROUP_ACTION_RUNNING:
         if (slider + payload_item_length <= end) {
           unsigned char is_action_running_aux = *slider;
+          slider += payload_item_length;
           group_action_running = (is_action_running_aux == '1') ? true : false;
         }
         break;
@@ -496,6 +371,7 @@ void Group_member_info::decode_payload(const unsigned char *buffer,
       case PIT_PRIMARY_ELECTION_RUNNING:
         if (slider + payload_item_length <= end) {
           unsigned char is_election_running_aux = *slider;
+          slider += payload_item_length;
           primary_election_running =
               (is_election_running_aux == '1') ? true : false;
         }
@@ -510,6 +386,7 @@ void Group_member_info::decode_payload(const unsigned char *buffer,
       case PIT_DEFAULT_TABLE_ENCRYPTION:
         if (slider + payload_item_length <= end) {
           unsigned char default_table_encryption_aux = *slider;
+          slider += payload_item_length;
           default_table_encryption =
               (default_table_encryption_aux == '1') ? true : false;
         }
@@ -518,71 +395,25 @@ void Group_member_info::decode_payload(const unsigned char *buffer,
         if (slider + payload_item_length <= end) {
           purged_gtid_set.assign(reinterpret_cast<const char *>(slider),
                                  static_cast<size_t>(payload_item_length));
-        }
-        break;
-      case PIT_RECOVERY_ENDPOINTS:
-        if (slider + payload_item_length <= end) {
-          recovery_endpoints.assign(reinterpret_cast<const char *>(slider),
-                                    static_cast<size_t>(payload_item_length));
-        }
-        break;
-      case PIT_VIEW_CHANGE_UUID:
-        if (slider + payload_item_length <= end) {
-          m_view_change_uuid.assign(reinterpret_cast<const char *>(slider),
-                                    static_cast<size_t>(payload_item_length));
-        }
-        break;
-
-      case PIT_ALLOW_SINGLE_LEADER:
-        if (slider + payload_item_length <= end) {
-          unsigned char allow_single_leader_aux = *slider;
-          m_allow_single_leader =
-              (allow_single_leader_aux == '1') ? true : false;
-        }
-        break;
-      case PIT_GROUP_ACTION_RUNNING_NAME:
-        if (slider + payload_item_length <= end) {
-          m_group_action_running_name.assign(
-              reinterpret_cast<const char *>(slider),
-              static_cast<size_t>(payload_item_length));
-        }
-        break;
-      case PIT_GROUP_ACTION_RUNNING_DESCRIPTION:
-        if (slider + payload_item_length <= end) {
-          m_group_action_running_description.assign(
-              reinterpret_cast<const char *>(slider),
-              static_cast<size_t>(payload_item_length));
-        }
-        break;
-      case PIT_PREEMPTIVE_GARBAGE_COLLECTION:
-        if (slider + payload_item_length <= end) {
-          unsigned char preemptive_garbage_collection_aux = *slider;
-          m_preemptive_garbage_collection =
-              (preemptive_garbage_collection_aux == '1') ? true : false;
+          slider += payload_item_length;
         }
         break;
     }
-
-    // Seek to next payload item.
-    slider += payload_item_length;
   }
 }
 
 string Group_member_info::get_hostname() {
   MUTEX_LOCK(lock, &update_lock);
-  assert(!hostname.empty());
   return hostname;
 }
 
 uint Group_member_info::get_port() {
   MUTEX_LOCK(lock, &update_lock);
-  assert(port > 0);
   return port;
 }
 
 string Group_member_info::get_uuid() {
   MUTEX_LOCK(lock, &update_lock);
-  assert(!uuid.empty());
   return uuid;
 }
 
@@ -616,7 +447,6 @@ const char *Group_member_info::get_member_role_string() {
 
 Gcs_member_identifier Group_member_info::get_gcs_member_id() {
   MUTEX_LOCK(lock, &update_lock);
-  assert(!gcs_member_id->get_member_id().empty());
   return *gcs_member_id;
 }
 
@@ -664,19 +494,6 @@ uint Group_member_info::get_write_set_extraction_algorithm() {
   return write_set_extraction_algorithm;
 }
 
-const char *Group_member_info::get_write_set_extraction_algorithm_name() {
-  switch (get_write_set_extraction_algorithm()) {
-    case HASH_ALGORITHM_OFF:
-      return "OFF";
-    case HASH_ALGORITHM_MURMUR32:
-      return "MURMUR32";
-    case HASH_ALGORITHM_XXHASH64:
-      return "XXHASH64";
-    default:
-      return "UNKNOWN ALGORITHM";
-  }
-}
-
 ulonglong Group_member_info::get_gtid_assignment_block_size() {
   MUTEX_LOCK(lock, &update_lock);
   return gtid_assignment_block_size;
@@ -783,24 +600,6 @@ void Group_member_info::set_is_group_action_running(bool is_running) {
   group_action_running = is_running;
 }
 
-const std::string &Group_member_info::get_group_action_running_name() {
-  return m_group_action_running_name;
-}
-
-void Group_member_info::set_group_action_running_name(
-    const std::string &group_action_running_name) {
-  m_group_action_running_name = group_action_running_name;
-}
-
-const std::string &Group_member_info::get_group_action_running_description() {
-  return m_group_action_running_description;
-}
-
-void Group_member_info::set_group_action_running_description(
-    const std::string &group_action_running_description) {
-  m_group_action_running_description = group_action_running_description;
-}
-
 bool Group_member_info::is_primary_election_running() {
   MUTEX_LOCK(lock, &update_lock);
   return primary_election_running;
@@ -874,36 +673,6 @@ bool Group_member_info::comparator_group_member_version(Group_member_info *m1,
   return m2->has_greater_version(m1);
 }
 
-string Group_member_info::get_recovery_endpoints() {
-  MUTEX_LOCK(lock, &update_lock);
-  return recovery_endpoints;
-}
-
-void Group_member_info::set_recovery_endpoints(const char *endpoints) {
-  MUTEX_LOCK(lock, &update_lock);
-  recovery_endpoints.assign(endpoints);
-}
-
-string Group_member_info::get_view_change_uuid() {
-  MUTEX_LOCK(lock, &update_lock);
-  return m_view_change_uuid;
-}
-
-bool Group_member_info::get_allow_single_leader() {
-  MUTEX_LOCK(lock, &update_lock);
-  return m_allow_single_leader;
-}
-
-void Group_member_info::set_view_change_uuid(const char *view_change_cnf) {
-  MUTEX_LOCK(lock, &update_lock);
-  m_view_change_uuid.assign(view_change_cnf);
-}
-
-bool Group_member_info::get_preemptive_garbage_collection() {
-  MUTEX_LOCK(lock, &update_lock);
-  return m_preemptive_garbage_collection;
-}
-
 bool Group_member_info::comparator_group_member_uuid(Group_member_info *m1,
                                                      Group_member_info *m2) {
   return m1->has_lower_uuid(m2);
@@ -942,11 +711,7 @@ bool Group_member_info::has_greater_weight(Group_member_info *other) {
 
 Group_member_info_manager::Group_member_info_manager(
     Group_member_info *local_member_info, PSI_mutex_key psi_mutex_key) {
-  members = new std::map<
-      std::string, Group_member_info *, std::less<std::string>,
-      Malloc_allocator<std::pair<const std::string, Group_member_info *>>>(
-      Malloc_allocator<std::pair<const std::string, Group_member_info *>>(
-          key_group_member_info));
+  members = new map<string, Group_member_info *>();
   this->local_member_info = local_member_info;
 
   mysql_mutex_init(psi_mutex_key, &update_lock, MY_MUTEX_INIT_FAST);
@@ -964,21 +729,6 @@ size_t Group_member_info_manager::get_number_of_members() {
   return members->size();
 }
 
-size_t Group_member_info_manager::get_number_of_members_online() {
-  size_t number = 0;
-  mysql_mutex_lock(&update_lock);
-
-  for (auto it = members->begin(); it != members->end(); it++) {
-    if ((*it).second->get_recovery_status() ==
-        Group_member_info::MEMBER_ONLINE) {
-      number++;
-    }
-  }
-
-  mysql_mutex_unlock(&update_lock);
-  return number;
-}
-
 bool Group_member_info_manager::is_member_info_present(
     const std::string &uuid) {
   bool found = false;
@@ -993,23 +743,34 @@ bool Group_member_info_manager::is_member_info_present(
   return found;
 }
 
-bool Group_member_info_manager::get_group_member_info(
-    const string &uuid, Group_member_info &member_info_arg) {
-  MUTEX_LOCK(lock, &update_lock);
+Group_member_info *Group_member_info_manager::get_group_member_info(
+    const string &uuid) {
+  Group_member_info *member = nullptr;
+  mysql_mutex_lock(&update_lock);
+
+  map<string, Group_member_info *>::iterator it;
+
+  it = members->find(uuid);
 
-  map<string, Group_member_info *>::iterator it = members->find(uuid);
   if (it != members->end()) {
-    member_info_arg.update(*it->second);
-    return false;
+    member = (*it).second;
   }
 
-  return true;
+  Group_member_info *member_copy = nullptr;
+  if (member != nullptr) {
+    member_copy = new Group_member_info(*member);
+  }
+
+  mysql_mutex_unlock(&update_lock);
+
+  return member_copy;
 }
 
-bool Group_member_info_manager::get_group_member_info_by_index(
-    int idx, Group_member_info &member_info_arg) {
+Group_member_info *Group_member_info_manager::get_group_member_info_by_index(
+    int idx) {
   Group_member_info *member = nullptr;
-  MUTEX_LOCK(lock, &update_lock);
+
+  mysql_mutex_lock(&update_lock);
 
   map<string, Group_member_info *>::iterator it;
   if (idx < (int)members->size()) {
@@ -1019,12 +780,13 @@ bool Group_member_info_manager::get_group_member_info_by_index(
     }
   }
 
+  Group_member_info *member_copy = nullptr;
   if (member != nullptr) {
-    member_info_arg.update(*member);
-    return false;
+    member_copy = new Group_member_info(*member);
   }
+  mysql_mutex_unlock(&update_lock);
 
-  return true;
+  return member_copy;
 }
 
 Member_version Group_member_info_manager::get_group_lowest_online_version() {
@@ -1048,74 +810,28 @@ Member_version Group_member_info_manager::get_group_lowest_online_version() {
 }
 
 Group_member_info *
-Group_member_info_manager::get_group_member_info_by_member_id_internal(
-    const Gcs_member_identifier &id) {
-  mysql_mutex_assert_owner(&update_lock);
+Group_member_info_manager::get_group_member_info_by_member_id(
+    Gcs_member_identifier idx) {
   Group_member_info *member = nullptr;
 
+  mysql_mutex_lock(&update_lock);
+
   map<string, Group_member_info *>::iterator it;
   for (it = members->begin(); it != members->end(); it++) {
-    if ((*it).second->get_gcs_member_id() == id) {
+    if ((*it).second->get_gcs_member_id() == idx) {
       member = (*it).second;
       break;
     }
   }
 
+  mysql_mutex_unlock(&update_lock);
   return member;
 }
 
-bool Group_member_info_manager::get_group_member_info_by_member_id(
-    const Gcs_member_identifier &id, Group_member_info &member_info_arg) {
-  MUTEX_LOCK(lock, &update_lock);
-
-  Group_member_info *member = get_group_member_info_by_member_id_internal(id);
-  if (member != nullptr) {
-    member_info_arg.update(*member);
-    return false;
-  }
-
-  return true;
-}
-
-std::pair<bool, std::string>
-Group_member_info_manager::get_group_member_uuid_from_member_id(
-    const Gcs_member_identifier &id) {
-  std::pair<bool, std::string> result{true, ""};
+vector<Group_member_info *> *Group_member_info_manager::get_all_members() {
   mysql_mutex_lock(&update_lock);
 
-  Group_member_info *member = get_group_member_info_by_member_id_internal(id);
-  if (member != nullptr) {
-    result.first = false;
-    result.second = member->get_uuid();
-  }
-
-  mysql_mutex_unlock(&update_lock);
-  return result;
-}
-
-Group_member_info::Group_member_status
-Group_member_info_manager::get_group_member_status_by_member_id(
-    const Gcs_member_identifier &id) {
-  Group_member_info::Group_member_status status = Group_member_info::MEMBER_END;
-  Group_member_info *member = nullptr;
-  mysql_mutex_lock(&update_lock);
-
-  member = get_group_member_info_by_member_id_internal(id);
-  if (nullptr != member) {
-    status = member->get_recovery_status();
-  }
-
-  mysql_mutex_unlock(&update_lock);
-  return status;
-}
-
-Group_member_info_list *Group_member_info_manager::get_all_members() {
-  mysql_mutex_lock(&update_lock);
-
-  Group_member_info_list *all_members =
-      new std::vector<Group_member_info *,
-                      Malloc_allocator<Group_member_info *>>(
-          Malloc_allocator<Group_member_info *>(key_group_member_info));
+  vector<Group_member_info *> *all_members = new vector<Group_member_info *>();
   map<string, Group_member_info *>::iterator it;
   for (it = members->begin(); it != members->end(); it++) {
     Group_member_info *member_copy = new Group_member_info(*(*it).second);
@@ -1174,12 +890,13 @@ void Group_member_info_manager::update(Group_member_info *update_local_member) {
   mysql_mutex_unlock(&update_lock);
 }
 
-void Group_member_info_manager::update(Group_member_info_list *new_members) {
+void Group_member_info_manager::update(
+    vector<Group_member_info *> *new_members) {
   mysql_mutex_lock(&update_lock);
 
   this->clear_members();
 
-  Group_member_info_list_iterator new_members_it;
+  vector<Group_member_info *>::iterator new_members_it;
   for (new_members_it = new_members->begin();
        new_members_it != new_members->end(); new_members_it++) {
     // If this bears the local member to be updated
@@ -1220,25 +937,6 @@ void Group_member_info_manager::update_member_status(
   mysql_mutex_unlock(&update_lock);
 }
 
-void Group_member_info_manager::set_member_unreachable(
-    const std::string &uuid) {
-  MUTEX_LOCK(lock, &update_lock);
-
-  auto it = members->find(uuid);
-  if (it != members->end()) {
-    (*it).second->set_unreachable();
-  }
-}
-
-void Group_member_info_manager::set_member_reachable(const std::string &uuid) {
-  MUTEX_LOCK(lock, &update_lock);
-
-  auto it = members->find(uuid);
-  if (it != members->end()) {
-    (*it).second->set_reachable();
-  }
-}
-
 void Group_member_info_manager::update_gtid_sets(const string &uuid,
                                                  string &gtid_executed,
                                                  std::string &purged_gtids,
@@ -1367,9 +1065,9 @@ void Group_member_info_manager::encode(vector<uchar> *to_encode) {
   delete group_info_message;
 }
 
-Group_member_info_list *Group_member_info_manager::decode(
+vector<Group_member_info *> *Group_member_info_manager::decode(
     const uchar *to_decode, size_t length) {
-  Group_member_info_list *decoded_members = nullptr;
+  vector<Group_member_info *> *decoded_members = nullptr;
 
   Group_member_info_manager_message *group_info_message =
       new Group_member_info_manager_message();
@@ -1394,7 +1092,7 @@ bool Group_member_info_manager::get_primary_member_uuid(
   for (it = members->begin(); it != members->end(); it++) {
     Group_member_info *info = (*it).second;
     if (info->get_role() == Group_member_info::MEMBER_ROLE_PRIMARY) {
-      assert(primary_member_uuid.empty());
+      DBUG_ASSERT(primary_member_uuid.empty());
       primary_member_uuid = info->get_uuid();
     }
   }
@@ -1408,21 +1106,22 @@ bool Group_member_info_manager::get_primary_member_uuid(
   return true;
 }
 
-bool Group_member_info_manager::get_primary_member_info(
-    Group_member_info &member_info_arg) {
-  MUTEX_LOCK(lock, &update_lock);
-
+Group_member_info *Group_member_info_manager::get_primary_member_info() {
+  mysql_mutex_lock(&update_lock);
   map<string, Group_member_info *>::iterator it;
+
+  Group_member_info *member_copy = nullptr;
   for (it = members->begin(); it != members->end(); it++) {
     Group_member_info *info = (*it).second;
     if (info->get_role() == Group_member_info::MEMBER_ROLE_PRIMARY) {
-      assert(info->in_primary_mode());
-      member_info_arg.update(*info);
-      return false;
+      member_copy = new Group_member_info(*info);
     }
   }
+  mysql_mutex_unlock(&update_lock);
 
-  return true;
+  DBUG_ASSERT(member_copy == nullptr || member_copy->in_primary_mode());
+
+  return member_copy;
 }
 
 bool Group_member_info_manager::is_majority_unreachable() {
@@ -1499,9 +1198,7 @@ std::string Group_member_info_manager::get_string_current_view_active_hosts()
 Group_member_info_manager_message::Group_member_info_manager_message()
     : Plugin_gcs_message(CT_MEMBER_INFO_MANAGER_MESSAGE) {
   DBUG_TRACE;
-  members = new std::vector<Group_member_info *,
-                            Malloc_allocator<Group_member_info *>>(
-      Malloc_allocator<Group_member_info *>(key_group_member_info));
+  members = new vector<Group_member_info *>();
 }
 
 Group_member_info_manager_message::Group_member_info_manager_message(
@@ -1515,9 +1212,7 @@ Group_member_info_manager_message::Group_member_info_manager_message(
     Group_member_info *member_info)
     : Plugin_gcs_message(CT_MEMBER_INFO_MANAGER_MESSAGE), members(nullptr) {
   DBUG_TRACE;
-  members = new std::vector<Group_member_info *,
-                            Malloc_allocator<Group_member_info *>>(
-      Malloc_allocator<Group_member_info *>(key_group_member_info));
+  members = new vector<Group_member_info *>();
   members->push_back(member_info);
 }
 
@@ -1529,21 +1224,19 @@ Group_member_info_manager_message::~Group_member_info_manager_message() {
 
 void Group_member_info_manager_message::clear_members() {
   DBUG_TRACE;
-  Group_member_info_list_iterator it;
+  std::vector<Group_member_info *>::iterator it;
   for (it = members->begin(); it != members->end(); it++) {
     delete (*it);
   }
   members->clear();
 }
 
-Group_member_info_list *Group_member_info_manager_message::get_all_members() {
+std::vector<Group_member_info *>
+    *Group_member_info_manager_message::get_all_members() {
   DBUG_TRACE;
-  Group_member_info_list *all_members =
-      new std::vector<Group_member_info *,
-                      Malloc_allocator<Group_member_info *>>(
-          Malloc_allocator<Group_member_info *>(key_group_member_info));
+  vector<Group_member_info *> *all_members = new vector<Group_member_info *>();
 
-  Group_member_info_list_iterator it;
+  std::vector<Group_member_info *>::iterator it;
   for (it = members->begin(); it != members->end(); it++) {
     Group_member_info *member_copy = new Group_member_info(*(*it));
     all_members->push_back(member_copy);
@@ -1559,7 +1252,7 @@ void Group_member_info_manager_message::encode_payload(
   uint16 number_of_members = (uint16)members->size();
   encode_payload_item_int2(buffer, PIT_MEMBERS_NUMBER, number_of_members);
 
-  Group_member_info_list_iterator it;
+  std::vector<Group_member_info *>::iterator it;
   for (it = members->begin(); it != members->end(); it++) {
     std::vector<uchar> encoded_member;
     (*it)->encode(&encoded_member);
@@ -1597,71 +1290,3 @@ void Group_member_info_manager_message::decode_payload(
     slider += payload_item_length;
   }
 }
-
-void Group_member_info_manager_message::
-    add_member_actions_serialized_configuration(
-        std::vector<unsigned char> *buffer,
-        const std::string &member_actions_serialized_configuration) const {
-  DBUG_TRACE;
-
-  encode_payload_item_type_and_length(
-      buffer, PIT_MEMBER_ACTIONS,
-      member_actions_serialized_configuration.size());
-  buffer->insert(buffer->end(), member_actions_serialized_configuration.begin(),
-                 member_actions_serialized_configuration.end());
-}
-
-bool Group_member_info_manager_message::get_pit_data(
-    const enum_payload_item_type pit, const unsigned char *buffer,
-    size_t length, const unsigned char **pit_data, size_t *pit_length) {
-  DBUG_TRACE;
-  const unsigned char *slider = buffer;
-  const unsigned char *end = buffer + length;
-  uint16 payload_item_type = 0;
-  unsigned long long payload_item_length = 0;
-
-  decode_header(&slider);
-
-  uint16 number_of_members = 0;
-  decode_payload_item_int2(&slider, &payload_item_type, &number_of_members);
-
-  for (uint16 i = 0; i < number_of_members; i++) {
-    decode_payload_item_type_and_length(&slider, &payload_item_type,
-                                        &payload_item_length);
-    slider += payload_item_length;
-  }
-
-  while (slider + Plugin_gcs_message::WIRE_PAYLOAD_ITEM_HEADER_SIZE <= end) {
-    // Read payload item header to find payload item length.
-    decode_payload_item_type_and_length(&slider, &payload_item_type,
-                                        &payload_item_length);
-
-    if (pit == payload_item_type) {
-      if (slider + payload_item_length <= end) {
-        *pit_data = slider;
-        *pit_length = payload_item_length;
-        return false;
-      }
-    }
-
-    // Seek to next payload item.
-    slider += payload_item_length;
-  }
-
-  return true;
-}
-
-void Group_member_info_manager_message::
-    add_replication_failover_channels_serialized_configuration(
-        std::vector<unsigned char> *buffer,
-        const std::string
-            &replication_failover_channels_serialized_configuration) const {
-  DBUG_TRACE;
-
-  encode_payload_item_type_and_length(
-      buffer, PIT_RPL_FAILOVER_CONFIGURATION,
-      replication_failover_channels_serialized_configuration.size());
-  buffer->insert(buffer->end(),
-                 replication_failover_channels_serialized_configuration.begin(),
-                 replication_failover_channels_serialized_configuration.end());
-}
