@@ -1,4 +1,4 @@
-/* Copyright (c) 2013, 2022, Oracle and/or its affiliates.
+/* Copyright (c) 2013, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -67,12 +67,11 @@ int add_write_set(Transaction_context_log_event *tcle,
     uint64 const tmp_str_sz =
         base64_needed_encoded_length((uint64)BUFFER_READ_PKE);
     char *write_set_value =
-        (char *)my_malloc(key_write_set_encoded, tmp_str_sz, MYF(MY_WME));
+        (char *)my_malloc(PSI_NOT_INSTRUMENTED, tmp_str_sz, MYF(MY_WME));
     if (!write_set_value) {
       /* purecov: begin inspected */
       LogPluginErr(ERROR_LEVEL,
                    ER_GRP_RPL_OOM_FAILED_TO_GENERATE_IDENTIFICATION_HASH);
-      my_free(write_set_value);
       return 1;
       /* purecov: end */
     }
@@ -81,7 +80,6 @@ int add_write_set(Transaction_context_log_event *tcle,
       /* purecov: begin inspected */
       LogPluginErr(ERROR_LEVEL,
                    ER_GRP_RPL_WRITE_IDENT_HASH_BASE64_ENCODING_FAILED);
-      my_free(write_set_value);
       return 1;
       /* purecov: end */
     }
@@ -121,6 +119,12 @@ int group_replication_trans_before_dml(Trans_param *param, int &out) {
     return 0;
   }
 
+  if ((out += (param->trans_ctx_info.binlog_checksum_options !=
+               binary_log::BINLOG_CHECKSUM_ALG_OFF))) {
+    LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_BINLOG_CHECKSUM_SET);
+    return 0;
+  }
+
   if ((out += (param->trans_ctx_info.transaction_write_set_extraction ==
                HASH_ALGORITHM_OFF))) {
     /* purecov: begin inspected */
@@ -169,22 +173,13 @@ int group_replication_trans_before_commit(Trans_param *param) {
   int error = 0;
   const int pre_wait_error = 1;
   const int post_wait_error = 2;
-  int64 sequence_number = 1;
 
   DBUG_EXECUTE_IF("group_replication_force_error_on_before_commit_listener",
                   return 1;);
 
   DBUG_EXECUTE_IF("group_replication_before_commit_hook_wait", {
-    const char act[] =
-        "now signal signal.group_replication_before_commit_hook_wait_reached "
-        "wait_for continue_commit";
-    assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
-  });
-
-  DBUG_EXECUTE_IF("group_replication_pause_on_before_commit_hook", {
-    // DBUG_SYNC are hold by same MDL lock test is using
-    const uint sleep_time_seconds = VIEW_MODIFICATION_TIMEOUT * 1.5;
-    my_sleep(sleep_time_seconds * 1000000);
+    const char act[] = "now wait_for continue_commit";
+    DBUG_ASSERT(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
   });
 
   /*
@@ -195,24 +190,17 @@ int group_replication_trans_before_commit(Trans_param *param) {
   */
   Replication_thread_api channel_interface;
   if (GR_APPLIER_CHANNEL == param->rpl_channel_type) {
-    // If plugin is not initialized, there is nothing to do.
-    if (nullptr == local_member_info) {
-      return 0;
-    }
-
     // If plugin is stopping, there is no point in update the statistics.
     bool fail_to_lock = shared_plugin_stop_lock->try_grab_read_lock();
     if (!fail_to_lock) {
       const Group_member_info::Group_member_status member_status =
           local_member_info->get_recovery_status();
-      if (Group_member_info::MEMBER_ONLINE == member_status ||
-          Group_member_info::MEMBER_IN_RECOVERY == member_status) {
+      if (Group_member_info::MEMBER_ONLINE == member_status) {
         applier_module->get_pipeline_stats_member_collector()
             ->decrement_transactions_waiting_apply();
         applier_module->get_pipeline_stats_member_collector()
             ->increment_transactions_applied();
-      }
-      if (Group_member_info::MEMBER_IN_RECOVERY == member_status) {
+      } else if (Group_member_info::MEMBER_IN_RECOVERY == member_status) {
         applier_module->get_pipeline_stats_member_collector()
             ->increment_transactions_applied_during_recovery();
       }
@@ -234,10 +222,7 @@ int group_replication_trans_before_commit(Trans_param *param) {
     return 0;
   }
 
-  if (shared_plugin_stop_lock->try_grab_read_lock()) {
-    /* If plugin is stopping, rollback the transaction immediatly. */
-    return 1;
-  }
+  shared_plugin_stop_lock->grab_read_lock();
 
   if (is_plugin_waiting_to_set_server_read_mode()) {
     LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_CANNOT_EXECUTE_TRANS_WHILE_STOPPING);
@@ -251,6 +236,7 @@ int group_replication_trans_before_commit(Trans_param *param) {
     return 0;
   }
 
+  DBUG_ASSERT(applier_module != nullptr && recovery_module != nullptr);
   Group_member_info::Group_member_status member_status =
       local_member_info->get_recovery_status();
 
@@ -276,7 +262,6 @@ int group_replication_trans_before_commit(Trans_param *param) {
     /* purecov: end */
   }
 
-  assert(applier_module != nullptr && recovery_module != nullptr);
   // Transaction information.
   const ulong transaction_size_limit = get_transaction_size_limit();
   my_off_t transaction_size = 0;
@@ -313,7 +298,6 @@ int group_replication_trans_before_commit(Trans_param *param) {
   my_off_t cache_log_position = 0;
   const my_off_t trx_cache_log_position = param->trx_cache_log->length();
   const my_off_t stmt_cache_log_position = param->stmt_cache_log->length();
-  unsigned long long immediate_commit_timestamp = 0;
 
   if (trx_cache_log_position > 0 && stmt_cache_log_position == 0) {
     cache_log = param->trx_cache_log;
@@ -362,8 +346,7 @@ int group_replication_trans_before_commit(Trans_param *param) {
       a transaction may have not write set at all because it didn't
       change any data, it will just persist that GTID as applied.
     */
-    if ((write_set == nullptr) && (!is_gtid_specified) &&
-        (!param->is_create_table_as_query_block)) {
+    if ((write_set == nullptr) && (!is_gtid_specified)) {
       LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_FAILED_TO_EXTRACT_TRANS_WRITE_SET,
                    param->thread_id);
       error = pre_wait_error;
@@ -381,7 +364,7 @@ int group_replication_trans_before_commit(Trans_param *param) {
         /* purecov: end */
       }
       cleanup_transaction_write_set(write_set);
-      assert(is_gtid_specified || (tcle->get_write_set()->size() > 0));
+      DBUG_ASSERT(is_gtid_specified || (tcle->get_write_set()->size() > 0));
     } else {
       /*
         For empty transactions we should set the GTID may_have_sbr_stmts. See
@@ -389,32 +372,30 @@ int group_replication_trans_before_commit(Trans_param *param) {
       */
       may_have_sbr_stmts = true;
     }
+  }
 
-    DBUG_EXECUTE_IF("group_replication_after_add_write_set", {
-      const char act[] =
-          "now signal signal.group_replication_after_add_write_set_reached "
-          "wait_for signal.group_replication_after_add_write_set_continue";
-      assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
-    });
+  /*
+    The BEFORE consistency can be used on groups with members that
+    do not support GROUP_REPLICATION_CONSISTENCY_BEFORE. In order to
+    allow that, after the wait is done on the transaction begin on
+    the local member, we broadcast the transaction as a normal
+    transaction that all versions do understand.
+  */
+  if (consistency_level < GROUP_REPLICATION_CONSISTENCY_AFTER) {
+    transaction_msg = new Transaction_message();
+  } else {
+    transaction_msg = new Transaction_with_guarantee_message(consistency_level);
+  }
 
-    /*
-      'CREATE TABLE ... AS SELECT' is considered a DML, though in reality it
-      is DDL + DML, which write-sets do not capture all dependencies.
-      Example:
-        CREATE TABLE test.t1 (c1 INT NOT NULL PRIMARY KEY);
-        INSERT INTO test.t1 VALUES (99);
-        CREATE TABLE test.t2 (c1 INT NOT NULL PRIMARY KEY) AS SELECT * FROM
-      test.t1; The 'CREATE TABLE ... AS SELECT' does depend on the INSERT on t1,
-      though these transactions have non intersecting write-sets, the INSERT
-      will refer to table t1 and the 'CREATE TABLE ... AS SELECT' to table t2.
-      As such we need to say that 'CREATE TABLE ... AS SELECT' can only be
-      executed on parallel applier after all precedent transactions like any
-      other DDL.
-    */
-    if (param->is_create_table_as_query_block) {
-      sequence_number = 0;
-      may_have_sbr_stmts = true;
-    }
+  // serialize transaction context into a transaction message.
+  // There is a chance you encounter an OOM in Transaction_message::write()
+  // here, so we take care accordingly.
+  try {
+    binary_event_serialize(tcle, transaction_msg);
+  } catch (const std::bad_alloc &) {
+    LogPluginErr(ERROR_LEVEL, ER_OUT_OF_RESOURCES);
+    error = pre_wait_error;
+    goto err;
   }
 
   if (*(param->original_commit_timestamp) == UNDEFINED_COMMIT_TIMESTAMP) {
@@ -423,11 +404,8 @@ int group_replication_trans_before_commit(Trans_param *param) {
      variable so that it won't be re-defined when this GTID is written to the
      binlog
     */
-    *(param->original_commit_timestamp) = immediate_commit_timestamp =
-        my_micro_time();
-  } else {  // the transaction did not originate in this server
-    immediate_commit_timestamp = my_micro_time();
-  }
+    *(param->original_commit_timestamp) = my_micro_time();
+  }  // otherwise the transaction did not originate in this server
 
   *(param->immediate_server_version) = do_server_version_int(::server_version);
   if (*(param->original_server_version) == UNDEFINED_SERVER_VERSION) {
@@ -443,23 +421,27 @@ int group_replication_trans_before_commit(Trans_param *param) {
 
   // Notice the GTID of atomic DDL is written to the trans cache as well.
   gle = new Gtid_log_event(
-      param->server_id, is_dml || param->is_atomic_ddl, 0, sequence_number,
-      may_have_sbr_stmts, *(param->original_commit_timestamp),
-      immediate_commit_timestamp, gtid_specification,
-      *(param->original_server_version), *(param->immediate_server_version));
+      param->server_id, is_dml || param->is_atomic_ddl, 0, 1,
+      may_have_sbr_stmts, *(param->original_commit_timestamp), 0,
+      gtid_specification, *(param->original_server_version),
+      *(param->immediate_server_version));
   /*
     GR does not support event checksumming. If GR start to support event
     checksumming, the calculation below should take the checksum payload into
     account.
   */
   gle->set_trx_length_by_cache_size(cache_log_position);
+  // There is a chance you encounter an OOM in Transaction_message::write()
+  // here, so we take care accordingly.
+  try {
+    binary_event_serialize(gle, transaction_msg);
+  } catch (const std::bad_alloc &) {
+    LogPluginErr(ERROR_LEVEL, ER_OUT_OF_RESOURCES);
+    error = pre_wait_error;
+    goto err;
+  }
 
-  /*
-    Only proceed to message serialization and send if the transaction
-    size does not exceed the limit.
-  */
-  transaction_size =
-      cache_log_position + tcle->get_event_length() + gle->get_event_length();
+  transaction_size = cache_log_position + transaction_msg->length();
   if (is_dml && transaction_size_limit &&
       transaction_size > transaction_size_limit) {
     LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_TRANS_SIZE_EXCEEDS_LIMIT,
@@ -468,44 +450,17 @@ int group_replication_trans_before_commit(Trans_param *param) {
     goto err;
   }
 
-  /*
-    Serialize transaction context, Gtid and binlog cache into
-    transaction message.
-    There is a chance we encounter an OOM, thence consider it.
-  */
+  // Copy binlog cache content to buffer.
+  // There is a chance you encounter an OOM in Transaction_message::write()
+  // here, so we take care accordingly.
   try {
-    /*
-      The BEFORE consistency can be used on groups with members that
-      do not support GROUP_REPLICATION_CONSISTENCY_BEFORE. In order to
-      allow that, after the wait is done on the transaction begin on
-      the local member, we broadcast the transaction as a normal
-      transaction that all versions do understand.
-    */
-    if (consistency_level < GROUP_REPLICATION_CONSISTENCY_AFTER) {
-      transaction_msg = new Transaction_message(transaction_size);
-    } else {
-      transaction_msg = new Transaction_with_guarantee_message(
-          transaction_size, consistency_level);
-    }
-
-    if (binary_event_serialize(tcle, transaction_msg) ||
-        binary_event_serialize(gle, transaction_msg)) {
-      LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_WRITE_TO_TRANSACTION_MESSAGE_FAILED,
-                   param->thread_id);
-      error = pre_wait_error;
-      goto err;
-    }
-    /* Release memory as soon as possible. */
-    delete tcle;
-    tcle = nullptr;
-    delete gle;
-    gle = nullptr;
-
     if (cache_log->copy_to(transaction_msg)) {
+      /* purecov: begin inspected */
       LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_WRITE_TO_TRANSACTION_MESSAGE_FAILED,
                    param->thread_id);
       error = pre_wait_error;
       goto err;
+      /* purecov: end */
     }
   } catch (const std::bad_alloc &) {
     LogPluginErr(ERROR_LEVEL, ER_OUT_OF_RESOURCES);
@@ -523,17 +478,15 @@ int group_replication_trans_before_commit(Trans_param *param) {
     /* purecov: end */
   }
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   DBUG_EXECUTE_IF("test_basic_CRUD_operations_sql_service_interface", {
     DBUG_SET("-d,test_basic_CRUD_operations_sql_service_interface");
-    assert(!sql_command_check());
+    DBUG_ASSERT(!sql_command_check());
   };);
 
   DBUG_EXECUTE_IF("group_replication_before_message_broadcast", {
-    const char act[] =
-        "now signal signal.group_replication_before_message_broadcast_reached "
-        "wait_for waiting";
-    assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
+    const char act[] = "now wait_for waiting";
+    DBUG_ASSERT(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
   });
 #endif
 
@@ -544,11 +497,7 @@ int group_replication_trans_before_commit(Trans_param *param) {
   applier_module->get_flow_control_module()->do_wait();
 
   // Broadcast the Transaction Message
-  send_error = gcs_module->send_transaction_message(*transaction_msg);
-
-  /* Release memory as soon as possible. */
-  delete transaction_msg;
-  transaction_msg = nullptr;
+  send_error = gcs_module->send_message(*transaction_msg);
 
   if (send_error == GCS_MESSAGE_TOO_BIG) {
     /* purecov: begin inspected */
@@ -594,10 +543,8 @@ err:
   }
 
   DBUG_EXECUTE_IF("group_replication_after_before_commit_hook", {
-    const char act[] =
-        "now SIGNAL signal.group_replication_after_before_commit_hook_reached "
-        "WAIT_FOR signal.group_replication_after_before_commit_hook_continue";
-    assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
+    const char act[] = "now wait_for signal.commit_continue";
+    DBUG_ASSERT(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
   });
   return error;
 }
@@ -676,14 +623,6 @@ int group_replication_trans_begin(Trans_param *param, int &out) {
   }
 
   group_transaction_observation_manager->read_lock_observer_list();
-
-  DBUG_EXECUTE_IF("group_replication_wait_on_observer_trans", {
-    const char act[] =
-        "now signal signal.group_replication_wait_on_observer_trans_waiting "
-        "wait_for signal.group_replication_wait_on_observer_trans_continue";
-    assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
-  });
-
   std::list<Group_transaction_listener *> *transaction_observers =
       group_transaction_observation_manager->get_all_observers();
   for (Group_transaction_listener *transaction_observer :
