@@ -1,4 +1,4 @@
-/* Copyright (c) 2018, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -21,10 +21,9 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 #include "plugin/group_replication/include/plugin_handlers/primary_election_invocation_handler.h"
+#include "plugin/group_replication/include/hold_transactions.h"
 #include "plugin/group_replication/include/plugin.h"
-#include "plugin/group_replication/include/plugin_handlers/member_actions_handler.h"
 #include "plugin/group_replication/include/plugin_handlers/primary_election_utils.h"
-#include "plugin/group_replication/include/services/system_variable/get_system_variable.h"
 
 Primary_election_handler::Primary_election_handler(
     ulong components_stop_timeout)
@@ -95,7 +94,7 @@ int Primary_election_handler::execute_primary_election(
   bool has_primary_changed;
   bool in_primary_mode;
   Group_member_info *primary_member_info = nullptr;
-  Group_member_info_list *all_members_info =
+  std::vector<Group_member_info *> *all_members_info =
       group_member_mgr->get_all_members();
 
   bool appointed_uuid = !primary_uuid.empty();
@@ -108,11 +107,8 @@ int Primary_election_handler::execute_primary_election(
       } else {
         // If the requested primary is not there, ignore the request.
         LogPluginErr(WARNING_LEVEL, ER_GRP_RPL_APPOINTED_PRIMARY_NOT_PRESENT);
-        group_events_observation_manager->after_primary_election(
-            "",
-            enum_primary_election_primary_change_status::
-                PRIMARY_DID_NOT_CHANGE_NO_CANDIDATE,
-            mode);
+        group_events_observation_manager->after_primary_election("", false,
+                                                                 mode);
         goto end;
       }
       /* purecov: end */
@@ -133,11 +129,8 @@ int Primary_election_handler::execute_primary_election(
                    ER_GRP_RPL_NO_SUITABLE_PRIMARY_MEM); /* purecov: inspected */
     }
     group_events_observation_manager->after_primary_election(
-        "",
-        enum_primary_election_primary_change_status::
-            PRIMARY_DID_NOT_CHANGE_NO_CANDIDATE,
-        mode, PRIMARY_ELECTION_NO_CANDIDATES_ERROR);
-    if (enable_server_read_mode()) {
+        "", false, mode, PRIMARY_ELECTION_NO_CANDIDATES_ERROR);
+    if (enable_server_read_mode(PSESSION_DEDICATED_THREAD)) {
       LogPluginErr(WARNING_LEVEL,
                    ER_GRP_RPL_ENABLE_READ_ONLY_FAILED); /* purecov: inspected */
     }
@@ -188,13 +181,13 @@ int Primary_election_handler::execute_primary_election(
             "Enabling conflict detection until the new primary applies all "
             "relay logs.");
 
-      LogPluginErr(SYSTEM_LEVEL, ER_GRP_RPL_NEW_PRIMARY_ELECTED,
+      LogPluginErr(INFORMATION_LEVEL, ER_GRP_RPL_NEW_PRIMARY_ELECTED,
                    primary_member_info->get_hostname().c_str(),
                    primary_member_info->get_port(), message.c_str());
       internal_primary_election(primary_uuid, mode);
     } else {
       // retain the old message
-      LogPluginErr(SYSTEM_LEVEL, ER_GRP_RPL_NEW_PRIMARY_ELECTED,
+      LogPluginErr(INFORMATION_LEVEL, ER_GRP_RPL_NEW_PRIMARY_ELECTED,
                    primary_member_info->get_hostname().c_str(),
                    primary_member_info->get_port(),
                    "Enabling conflict detection until the new primary applies "
@@ -202,14 +195,12 @@ int Primary_election_handler::execute_primary_election(
       legacy_primary_election(primary_uuid);
     }
   } else {
-    group_events_observation_manager->after_primary_election(
-        "", enum_primary_election_primary_change_status::PRIMARY_DID_NOT_CHANGE,
-        mode);
+    group_events_observation_manager->after_primary_election("", false, mode);
   }
 
 end:
   // clean the members
-  Group_member_info_list_iterator it;
+  std::vector<Group_member_info *>::iterator it;
   for (it = all_members_info->begin(); it != all_members_info->end(); it++) {
     delete (*it);
   }
@@ -222,9 +213,16 @@ void Primary_election_handler::print_gtid_info_in_log() {
   Replication_thread_api applier_channel("group_replication_applier");
   std::string applier_retrieved_gtids;
   std::string server_executed_gtids;
-  Get_system_variable *get_system_variable = new Get_system_variable();
-
-  if (get_system_variable->get_global_gtid_executed(server_executed_gtids)) {
+  Sql_service_command_interface *sql_command_interface =
+      new Sql_service_command_interface();
+  if (sql_command_interface->establish_session_connection(
+          PSESSION_DEDICATED_THREAD, GROUPREPL_USER, get_plugin_pointer())) {
+    /* purecov: begin inspected */
+    LogPluginErr(WARNING_LEVEL, ER_GRP_RPL_CONN_INTERNAL_PLUGIN_FAIL);
+    goto err;
+    /* purecov: end */
+  }
+  if (sql_command_interface->get_server_gtid_executed(server_executed_gtids)) {
     /* purecov: begin inspected */
     LogPluginErr(WARNING_LEVEL, ER_GRP_RPL_GTID_EXECUTED_EXTRACT_ERROR);
     goto err;
@@ -243,7 +241,7 @@ void Primary_election_handler::print_gtid_info_in_log() {
                "applier channel received_transaction_set",
                applier_retrieved_gtids.c_str());
 err:
-  delete get_system_variable;
+  delete sql_command_interface;
 }
 
 int Primary_election_handler::internal_primary_election(
@@ -252,22 +250,24 @@ int Primary_election_handler::internal_primary_election(
     secondary_election_handler.terminate_election_process();
   }
 
-  assert(!primary_election_handler.is_election_process_running() ||
-         primary_election_handler.is_election_process_terminating());
+  DBUG_ASSERT(!primary_election_handler.is_election_process_running() ||
+              primary_election_handler.is_election_process_terminating());
 
   /** Wait for an old process to end*/
   if (primary_election_handler.is_election_process_terminating())
     primary_election_handler
         .wait_on_election_process_termination(); /* purecov: inspected */
 
-  Group_member_info_list *members_info = group_member_mgr->get_all_members();
+  std::vector<Group_member_info *> *members_info =
+      group_member_mgr->get_all_members();
 
   /* Declare at this point that all members are in primary mode for switch
    * cases*/
   group_member_mgr->update_primary_member_flag(true);
 
   if (!local_member_info->get_uuid().compare(primary_to_elect)) {
-    notify_election_running();
+    hold_transactions->enable();
+    register_transaction_observer();
     primary_election_handler.launch_primary_election_process(
         mode, primary_to_elect, members_info);
   } else {
@@ -300,10 +300,13 @@ int Primary_election_handler::legacy_primary_election(
   applier_module->add_single_primary_action_packet(single_primary_action);
 
   if (is_primary_local) {
-    member_actions_handler->trigger_actions(
-        Member_actions::AFTER_PRIMARY_ELECTION);
+    if (disable_server_read_mode(PSESSION_DEDICATED_THREAD)) {
+      LogPluginErr(
+          WARNING_LEVEL,
+          ER_GRP_RPL_DISABLE_READ_ONLY_FAILED); /* purecov: inspected */
+    }
   } else {
-    if (enable_server_read_mode()) {
+    if (enable_server_read_mode(PSESSION_DEDICATED_THREAD)) {
       LogPluginErr(WARNING_LEVEL,
                    ER_GRP_RPL_ENABLE_READ_ONLY_FAILED); /* purecov: inspected */
     }
@@ -319,32 +322,31 @@ int Primary_election_handler::legacy_primary_election(
     internal_primary_election(primary_uuid, LEGACY_ELECTION_PRIMARY);
   } else {
     set_election_running(false);
-    LogPluginErr(SYSTEM_LEVEL, ER_GRP_RPL_SRV_SECONDARY_MEM,
+    LogPluginErr(INFORMATION_LEVEL, ER_GRP_RPL_SRV_SECONDARY_MEM,
                  primary_member_info->get_hostname().c_str(),
                  primary_member_info->get_port());
   }
 
-  group_events_observation_manager->after_primary_election(
-      primary_uuid,
-      enum_primary_election_primary_change_status::PRIMARY_DID_CHANGE,
-      DEAD_OLD_PRIMARY);
+  group_events_observation_manager->after_primary_election(primary_uuid, true,
+                                                           DEAD_OLD_PRIMARY);
   delete primary_member_info;
 
   return 0;
 }
 
 bool Primary_election_handler::pick_primary_member(
-    std::string &primary_uuid, Group_member_info_list *all_members_info) {
+    std::string &primary_uuid,
+    std::vector<Group_member_info *> *all_members_info) {
   DBUG_TRACE;
 
   bool am_i_leaving = true;
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   int n = 0;
 #endif
   Group_member_info *the_primary = nullptr;
 
-  Group_member_info_list_iterator it;
-  Group_member_info_list_iterator lowest_version_end;
+  std::vector<Group_member_info *>::iterator it;
+  std::vector<Group_member_info *>::iterator lowest_version_end;
 
   /* sort members based on member_version and get first iterator position
      where member version differs
@@ -363,15 +365,15 @@ bool Primary_election_handler::pick_primary_member(
    2. Check if I am leaving the group or not.
    */
   for (it = all_members_info->begin(); it != all_members_info->end(); it++) {
-#ifndef NDEBUG
-    assert(n <= 1);
+#ifndef DBUG_OFF
+    DBUG_ASSERT(n <= 1);
 #endif
 
     Group_member_info *member = *it;
     if (local_member_info->in_primary_mode() && the_primary == nullptr &&
         member->get_role() == Group_member_info::MEMBER_ROLE_PRIMARY) {
       the_primary = member;
-#ifndef NDEBUG
+#ifndef DBUG_OFF
       n++;
 #endif
     }
@@ -402,7 +404,7 @@ bool Primary_election_handler::pick_primary_member(
            it != lowest_version_end && the_primary == nullptr; it++) {
         Group_member_info *member_info = *it;
 
-        assert(member_info);
+        DBUG_ASSERT(member_info);
         if (member_info && member_info->get_recovery_status() ==
                                Group_member_info::MEMBER_ONLINE)
           the_primary = member_info;
@@ -416,9 +418,10 @@ bool Primary_election_handler::pick_primary_member(
   return false;
 }
 
-Group_member_info_list_iterator sort_and_get_lowest_version_member_position(
-    Group_member_info_list *all_members_info) {
-  Group_member_info_list_iterator it;
+std::vector<Group_member_info *>::iterator
+sort_and_get_lowest_version_member_position(
+    std::vector<Group_member_info *> *all_members_info) {
+  std::vector<Group_member_info *>::iterator it;
 
   // sort in ascending order of lower member version
   std::sort(all_members_info->begin(), all_members_info->end(),
@@ -427,7 +430,8 @@ Group_member_info_list_iterator sort_and_get_lowest_version_member_position(
   /* if vector contains only single version then leader should be picked from
      all members
    */
-  Group_member_info_list_iterator lowest_version_end = all_members_info->end();
+  std::vector<Group_member_info *>::iterator lowest_version_end =
+      all_members_info->end();
 
   /* first member will have lowest version as members are already
      sorted above using member_version.
@@ -482,8 +486,8 @@ Group_member_info_list_iterator sort_and_get_lowest_version_member_position(
 }
 
 void sort_members_for_election(
-    Group_member_info_list *all_members_info,
-    Group_member_info_list_iterator lowest_version_end) {
+    std::vector<Group_member_info *> *all_members_info,
+    std::vector<Group_member_info *>::iterator lowest_version_end) {
   Group_member_info *first_member = *(all_members_info->begin());
   Member_version lowest_version = first_member->get_member_version();
 
@@ -496,10 +500,52 @@ void sort_members_for_election(
               Group_member_info::comparator_group_member_uuid);
 }
 
-void Primary_election_handler::notify_election_running() {
-  transaction_consistency_manager->enable_primary_election_checks();
+void Primary_election_handler::register_transaction_observer() {
+  group_transaction_observation_manager->register_transaction_observer(this);
 }
 
-void Primary_election_handler::notify_election_end() {
-  transaction_consistency_manager->disable_primary_election_checks();
+void Primary_election_handler::unregister_transaction_observer() {
+  group_transaction_observation_manager->unregister_transaction_observer(this);
 }
+
+int Primary_election_handler::before_transaction_begin(
+    my_thread_id, ulong gr_consistency, ulong hold_timeout,
+    enum_rpl_channel_type channel_type) {
+  DBUG_TRACE;
+
+  if (GR_RECOVERY_CHANNEL == channel_type ||
+      GR_APPLIER_CHANNEL == channel_type) {
+    return 0;
+  }
+
+  const enum_group_replication_consistency_level consistency_level =
+      static_cast<enum_group_replication_consistency_level>(gr_consistency);
+
+  if (consistency_level ==
+          GROUP_REPLICATION_CONSISTENCY_BEFORE_ON_PRIMARY_FAILOVER ||
+      consistency_level == GROUP_REPLICATION_CONSISTENCY_AFTER) {
+    return hold_transactions->wait_until_primary_failover_complete(
+        hold_timeout);
+  }
+
+  return 0;
+}
+
+/*
+  These methods are necessary to fulfil the Group_transaction_listener
+  interface.
+*/
+/* purecov: begin inspected */
+int Primary_election_handler::before_commit(
+    my_thread_id, Group_transaction_listener::enum_transaction_origin) {
+  return 0;
+}
+int Primary_election_handler::before_rollback(
+    my_thread_id, Group_transaction_listener::enum_transaction_origin) {
+  return 0;
+}
+int Primary_election_handler::after_rollback(my_thread_id) { return 0; }
+int Primary_election_handler::after_commit(my_thread_id, rpl_sidno, rpl_gno) {
+  return 0;
+}
+/* purecov: end */
