@@ -1,16 +1,14 @@
-/* Copyright (c) 2019, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2019, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
-   but not limited to OpenSSL) that is licensed under separate terms,
+   This program is also distributed with certain software (including
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -21,15 +19,15 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
-#include "plugin/group_replication/include/plugin_handlers/remote_clone_handler.h"
+#include <random>
+
 #include "plugin/group_replication/include/leave_group_on_failure.h"
 #include "plugin/group_replication/include/plugin.h"
-#include "plugin/group_replication/include/plugin_variables/recovery_endpoints.h"
-#include "string_with_len.h"
+#include "plugin/group_replication/include/plugin_handlers/remote_clone_handler.h"
 
-void *Remote_clone_handler::launch_thread(void *arg) {
+[[noreturn]] void *Remote_clone_handler::launch_thread(void *arg) {
   Remote_clone_handler *thd = static_cast<Remote_clone_handler *>(arg);
-  thd->clone_thread_handle();  // Does not return.
+  thd->clone_thread_handle();
 }
 
 Remote_clone_handler::Remote_clone_handler(ulonglong threshold,
@@ -102,9 +100,9 @@ int Remote_clone_handler::after_view_change(
   return 0;
 }
 
-int Remote_clone_handler::after_primary_election(
-    std::string, enum_primary_election_primary_change_status,
-    enum_primary_election_mode, int) {
+int Remote_clone_handler::after_primary_election(std::string, bool,
+                                                 enum_primary_election_mode,
+                                                 int) {
   return 0;
 }
 
@@ -151,23 +149,23 @@ Remote_clone_handler::check_clone_plugin_presence() {
 }
 
 int Remote_clone_handler::extract_donor_info(
-    std::tuple<uint, uint, uint, bool> *donor_info) {
+    std::tuple<uint, uint, uint, ulonglong> *donor_info) {
   int error = 0;
 
   uint valid_clone_donors = 0;
   uint valid_recovery_donors = 0;
   uint valid_recovering_donors = 0;
-  bool clone_activation_threshold_breach = false;
+  ulonglong number_gtids_missing = 0;
 
-  Group_member_info_list *all_members_info =
+  std::vector<Group_member_info *> *all_members_info =
       group_member_mgr->get_all_members();
 
-  Tsid_map local_tsid_map(nullptr);
-  Tsid_map group_tsid_map(nullptr);
-  Gtid_set local_member_set(&local_tsid_map, nullptr);
-  Gtid_set group_set(&group_tsid_map, nullptr);
-  Tsid_map purged_tsid_map(nullptr);
-  Gtid_set purged_set(&purged_tsid_map, nullptr);
+  Sid_map local_sid_map(nullptr);
+  Sid_map group_sid_map(nullptr);
+  Gtid_set local_member_set(&local_sid_map, nullptr);
+  Gtid_set group_set(&group_sid_map, nullptr);
+  Sid_map purged_sid_map(nullptr);
+  Gtid_set purged_set(&purged_sid_map, nullptr);
 
   if (local_member_set.add_gtid_text(
           local_member_info->get_gtid_executed().c_str()) != RETURN_STATUS_OK ||
@@ -183,36 +181,41 @@ int Remote_clone_handler::extract_donor_info(
 
   for (Group_member_info *member : *all_members_info) {
     std::string m_uuid = member->get_uuid();
-    const bool not_self = m_uuid.compare(local_member_info->get_uuid());
-    const bool is_online =
+    bool is_online =
         member->get_recovery_status() == Group_member_info::MEMBER_ONLINE;
-    const bool valid_donor_version =
-        (member->get_member_version().get_version() >=
-         CLONE_GR_SUPPORT_VERSION);
+    bool not_self = m_uuid.compare(local_member_info->get_uuid());
+    // We can only clone from our own version.
+    bool supports_clone =
+        member->get_member_version().get_version() >=
+            CLONE_GR_SUPPORT_VERSION &&
+        member->get_member_version().get_version() ==
+            local_member_info->get_member_version().get_version();
 
     std::string member_exec_set_str = member->get_gtid_executed();
     std::string applier_ret_set_str = member->get_gtid_retrieved();
 
-    if (is_online && not_self && valid_donor_version) {
-      valid_clone_donors++;
+    if (is_online) {
+      if (not_self) {
+        // Check if it support cloning looking at the server version
+        if (supports_clone) valid_clone_donors++;
 
-      if (group_set.add_gtid_text(member_exec_set_str.c_str()) !=
-              RETURN_STATUS_OK ||
-          group_set.add_gtid_text(applier_ret_set_str.c_str()) !=
-              RETURN_STATUS_OK) {
-        /* purecov: begin inspected */
-        LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_LOCAL_GTID_SETS_PROCESS_ERROR);
-        error = 1;
-        goto cleaning;
-        /* purecov: end */
+        if (group_set.add_gtid_text(member_exec_set_str.c_str()) !=
+                RETURN_STATUS_OK ||
+            group_set.add_gtid_text(applier_ret_set_str.c_str()) !=
+                RETURN_STATUS_OK) {
+          /* purecov: begin inspected */
+          LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_LOCAL_GTID_SETS_PROCESS_ERROR);
+          error = 1;
+          goto cleaning;
+          /* purecov: end */
+        }
       }
     }
   }
 
-  // Check clone activation threshold breach
+  // Calculate the number of missing gtids
   group_set.remove_gtid_set(&local_member_set);
-  clone_activation_threshold_breach =
-      group_set.is_size_greater_than_or_equal(m_clone_activation_threshold);
+  number_gtids_missing = group_set.get_gtid_number();
 
   // Before deciding calculate also the number of valid recovery donors
   for (Group_member_info *member : *all_members_info) {
@@ -230,7 +233,7 @@ int Remote_clone_handler::extract_donor_info(
 
     if (not_self) {
       if (is_online || is_recovering) {
-        purged_set.clear_set_and_tsid_map();
+        purged_set.clear_set_and_sid_map();
         if (purged_set.add_gtid_text(member_purged_set_str.c_str()) !=
             RETURN_STATUS_OK) {
           /* purecov: begin inspected */
@@ -256,7 +259,7 @@ cleaning:
   std::get<0>(*donor_info) = valid_clone_donors;
   std::get<1>(*donor_info) = valid_recovery_donors;
   std::get<2>(*donor_info) = valid_recovering_donors;
-  std::get<3>(*donor_info) = clone_activation_threshold_breach;
+  std::get<3>(*donor_info) = number_gtids_missing;
 
   // clean the members
   for (Group_member_info *member : *all_members_info) {
@@ -271,7 +274,7 @@ Remote_clone_handler::enum_clone_check_result
 Remote_clone_handler::check_clone_preconditions() {
   Remote_clone_handler::enum_clone_check_result result = NO_RECOVERY_POSSIBLE;
 
-  std::tuple<uint, uint, uint, bool> donor_info(0, 0, 0, false);
+  std::tuple<uint, uint, uint, ulonglong> donor_info(0, 0, 0, 0);
   if (extract_donor_info(&donor_info)) {
     return CHECK_ERROR; /* purecov: inspected */
   }
@@ -279,10 +282,10 @@ Remote_clone_handler::check_clone_preconditions() {
   uint valid_clone_donors = std::get<0>(donor_info);
   uint valid_recovery_donors = std::get<1>(donor_info);
   uint valid_recovering_donors = std::get<2>(donor_info);
-  bool clone_activation_threshold_breach = std::get<3>(donor_info);
+  ulonglong number_gtids_missing = std::get<3>(donor_info);
   ulonglong threshold = m_clone_activation_threshold;
 
-  if (clone_activation_threshold_breach && valid_clone_donors > 0) {
+  if (number_gtids_missing >= threshold && valid_clone_donors > 0) {
     result = DO_CLONE;
     LogPluginErr(WARNING_LEVEL, ER_GRP_RPL_RECOVERY_STRAT_CLONE_THRESHOLD,
                  threshold);
@@ -340,22 +343,26 @@ end:
 
 void Remote_clone_handler::get_clone_donors(
     std::list<Group_member_info *> &suitable_donors) {
-  Group_member_info_list *all_members_info =
+  std::vector<Group_member_info *> *all_members_info =
       group_member_mgr->get_all_members();
   if (all_members_info->size() > 1) {
-    vector_random_shuffle(all_members_info);
+    std::random_device rng;
+    std::mt19937 urng(rng());
+    std::shuffle(all_members_info->begin(), all_members_info->end(), urng);
   }
 
   for (Group_member_info *member : *all_members_info) {
     std::string m_uuid = member->get_uuid();
-    const bool not_self = m_uuid.compare(local_member_info->get_uuid());
-    const bool is_online =
+    bool is_online =
         member->get_recovery_status() == Group_member_info::MEMBER_ONLINE;
-    const bool valid_donor_version =
-        (member->get_member_version().get_version() >=
-         CLONE_GR_SUPPORT_VERSION);
+    bool not_self = m_uuid.compare(local_member_info->get_uuid());
+    bool supports_clone =
+        member->get_member_version().get_version() >=
+            CLONE_GR_SUPPORT_VERSION &&
+        member->get_member_version().get_version() ==
+            local_member_info->get_member_version().get_version();
 
-    if (is_online && not_self && valid_donor_version) {
+    if (is_online && not_self && supports_clone) {
       suitable_donors.push_back(member);
     } else {
       delete member;
@@ -393,21 +400,23 @@ int Remote_clone_handler::set_clone_ssl_options(
   return error;
 }
 
-int Remote_clone_handler::fallback_to_recovery_or_leave(bool critical_error) {
+int Remote_clone_handler::fallback_to_recovery_or_leave(
+    Sql_service_command_interface *sql_command_interface, bool critical_error) {
   // Do nothing if the server is shutting down.
   // The stop process will leave the group
   if (get_server_shutdown_status()) return 0;
 
   Replication_thread_api applier_channel("group_replication_applier");
   if (!critical_error && !applier_channel.is_applier_thread_running() &&
-      applier_channel.start_threads(false, true, nullptr, false)) {
+      applier_channel.start_threads(false, true, NULL, false)) {
     abort_plugin_process(
         "The plugin was not able to start the group_replication_applier "
         "channel.");
     return 1;
   }
   // If it failed to (re)connect to the server or the set read only query
-  if (enable_server_read_mode()) {
+  if (!sql_command_interface->is_session_valid() ||
+      sql_command_interface->set_super_read_only()) {
     abort_plugin_process(
         "Cannot re-enable the super read only after clone failure.");
     return 1;
@@ -418,7 +427,7 @@ int Remote_clone_handler::fallback_to_recovery_or_leave(bool critical_error) {
    Since cloning can be time consuming valid members may have left
    or joined in the meanwhile.
   */
-  std::tuple<uint, uint, uint, bool> donor_info(0, 0, 0, false);
+  std::tuple<uint, uint, uint, ulonglong> donor_info(0, 0, 0, 0);
   if (extract_donor_info(&donor_info)) {
     critical_error = true; /* purecov: inspected */
   } else {
@@ -439,9 +448,9 @@ int Remote_clone_handler::fallback_to_recovery_or_leave(bool critical_error) {
     leave_group_on_failure::mask leave_actions;
     leave_actions.set(leave_group_on_failure::SKIP_SET_READ_ONLY, true);
     leave_actions.set(leave_group_on_failure::HANDLE_EXIT_STATE_ACTION, true);
-    leave_group_on_failure::leave(leave_actions,
-                                  ER_GRP_RPL_RECOVERY_STRAT_NO_FALLBACK,
-                                  nullptr, exit_state_action_abort_log_message);
+    leave_group_on_failure::leave(
+        leave_actions, ER_GRP_RPL_RECOVERY_STRAT_NO_FALLBACK,
+        PSESSION_INIT_THREAD, nullptr, exit_state_action_abort_log_message);
     return 1;
   }
 }
@@ -477,20 +486,6 @@ int Remote_clone_handler::run_clone_query(
     std::string &port, std::string &username, std::string &password,
     bool use_ssl) {
   int error = 0;
-
-#ifndef NDEBUG
-  DBUG_EXECUTE_IF("gr_run_clone_query_fail_once", {
-    const char act[] =
-        "now signal signal.run_clone_query_waiting wait_for "
-        "signal.run_clone_query_continue";
-    assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
-
-    DBUG_SET("-d,gr_run_clone_query_fail_once");
-
-    return 1;
-  });
-#endif /* NDEBUG */
-
   mysql_mutex_lock(&m_clone_query_lock);
   m_clone_query_session_id =
       sql_command_interface->get_sql_service_interface()->get_session_id();
@@ -519,7 +514,7 @@ int Remote_clone_handler::kill_clone_query() {
   mysql_mutex_lock(&m_clone_query_lock);
 
   if (m_clone_query_status == CLONE_QUERY_EXECUTING) {
-    assert(m_clone_query_session_id != 0);
+    DBUG_ASSERT(m_clone_query_session_id != 0);
     Sql_service_command_interface *sql_command_interface =
         new Sql_service_command_interface();
     error = sql_command_interface->establish_session_connection(
@@ -640,20 +635,20 @@ bool Remote_clone_handler::evaluate_error_code(int) {
   return false;
 }
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
 void Remote_clone_handler::gr_clone_debug_point() {
   DBUG_EXECUTE_IF("gr_clone_process_before_execution", {
     const char act[] =
         "now signal signal.gr_clone_thd_paused wait_for "
         "signal.gr_clone_thd_continue";
-    assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
+    DBUG_ASSERT(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
   });
   DBUG_EXECUTE_IF("gr_clone_before_applier_stop", {
     const char act[] = "now wait_for applier_stopped";
-    assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
+    DBUG_ASSERT(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
   });
 }
-#endif /* NDEBUG */
+#endif /* DBUG_OFF */
 
 [[noreturn]] void Remote_clone_handler::clone_thread_handle() {
   int error = 0;
@@ -716,7 +711,7 @@ void Remote_clone_handler::gr_clone_debug_point() {
 
   /* The clone operation does not work with read mode so we have to disable it
    * here */
-  if (disable_server_read_mode()) {
+  if (sql_command_interface->reset_read_only()) {
     /* purecov: begin inspected */
     LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_CLONE_PROCESS_PREPARE_ERROR,
                  "Could not disable the server read only mode for cloning.");
@@ -753,12 +748,11 @@ void Remote_clone_handler::gr_clone_debug_point() {
       /* purecov: end */
     }
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   gr_clone_debug_point();
-#endif /* NDEBUG */
+#endif /* DBUG_OFF */
   // Ignore any channel stop error and confirm channel is stopped or not.
   // Since we will clone next.
-  applier_module->ignore_errors_during_stop(true);
   applier_channel.stop_threads(false, true);
   if (applier_channel.is_applier_thread_running()) {
     /* purecov: begin inspected */
@@ -778,9 +772,9 @@ void Remote_clone_handler::gr_clone_debug_point() {
   stage_handler.set_stage(info_GR_STAGE_clone_execute.m_key, __FILE__, __LINE__,
                           number_servers, number_attempts);
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   gr_clone_debug_point();
-#endif /* NDEBUG */
+#endif /* DBUG_OFF */
 
   while (!empty_donor_list && !m_being_terminated) {
     stage_handler.set_completed_work(number_attempts);
@@ -788,15 +782,13 @@ void Remote_clone_handler::gr_clone_debug_point() {
 
     std::string hostname("");
     std::string port("");
-    std::vector<std::pair<std::string, uint>> endpoints;
 
     mysql_mutex_lock(&m_donor_list_lock);
     empty_donor_list = m_suitable_donors.empty();
     if (!empty_donor_list) {
       Group_member_info *member = m_suitable_donors.front();
-      Donor_recovery_endpoints donor_endpoints;
-      endpoints = donor_endpoints.get_endpoints(member);
-
+      hostname.assign(member->get_hostname());
+      port.assign(std::to_string(member->get_port()));
       // define the current donor
       delete m_current_donor_address;
       m_current_donor_address =
@@ -809,56 +801,45 @@ void Remote_clone_handler::gr_clone_debug_point() {
     }
     mysql_mutex_unlock(&m_donor_list_lock);
 
-    // No valid donor in the list
-    if (endpoints.size() == 0) {
-      error = 1;
-      continue;
+    // No valid donor remains in the list
+    if (hostname.empty()) continue;
+
+    /* Update the allowed donor list */
+    if ((error = update_donor_list(sql_command_interface, hostname, port))) {
+      break; /* purecov: inspected */
     }
 
-    for (auto endpoint : endpoints) {
-      hostname.assign(endpoint.first);
-      port.assign(std::to_string(endpoint.second));
+    if (m_being_terminated) goto thd_end;
 
-      /* Update the allowed donor list */
-      if ((error = update_donor_list(sql_command_interface, hostname, port))) {
-        continue; /* purecov: inspected */
+    terminate_wait_on_start_process(true);
+
+    error = run_clone_query(sql_command_interface, hostname, port, username,
+                            password, use_ssl);
+
+    // Even on critical errors we continue as another clone can fix the issue
+    if (!critical_error) critical_error = evaluate_error_code(error);
+
+    // On ER_RESTART_SERVER_FAILED it makes no sense to retry
+    if (error == ER_RESTART_SERVER_FAILED) goto thd_end;
+
+    if (error && !m_being_terminated) {
+      if (evaluate_server_connection(sql_command_interface)) {
+        /* purecov: begin inspected */
+        // This is a bad sign, might as well quit
+        critical_error = true;
+        goto thd_end;
+        /* purecov: end */
       }
 
-      if (m_being_terminated) goto thd_end;
-
-      terminate_wait_on_start_process(WAIT_ON_START_PROCESS_ABORT_ON_CLONE);
-
-      error = run_clone_query(sql_command_interface, hostname, port, username,
-                              password, use_ssl);
-
-      // Even on critical errors we continue as another clone can fix the issue
-      if (!critical_error) critical_error = evaluate_error_code(error);
-
-      // On ER_RESTART_SERVER_FAILED it makes no sense to retry
-      if (error == ER_RESTART_SERVER_FAILED) goto thd_end;
-
-      if (error && !m_being_terminated) {
-        if (evaluate_server_connection(sql_command_interface)) {
-          /* purecov: begin inspected */
-          // This is a bad sign, might as well quit
-          critical_error = true;
-          goto thd_end;
-          /* purecov: end */
-        }
-
-        /*
-          If we reach this point and we are now alone it means no alternatives
-          remain for recovery. A fallback to distributed recovery would be
-          interpreted as a solo member recovery.
-          */
-        if (group_member_mgr->get_number_of_members() == 1) {
-          critical_error = true;
-          goto thd_end;
-        }
+      /*
+       If we reach this point and we are now alone it means no alternatives
+       remain for recovery. A fallback to distributed recovery would be
+       interpreted as a solo member recovery.
+      */
+      if (group_member_mgr->get_number_of_members() == 1) {
+        critical_error = true;
+        goto thd_end;
       }
-
-      // try till there is a recovery endpoint that succeeds
-      if (!error) break;
     }
 
     // try till there is a clone command that succeeds
@@ -868,10 +849,9 @@ void Remote_clone_handler::gr_clone_debug_point() {
 thd_end:
 
   declare_plugin_cloning(false);
-  applier_module->ignore_errors_during_stop(false);
 
   if (error && !m_being_terminated) {
-    fallback_to_recovery_or_leave(critical_error);
+    fallback_to_recovery_or_leave(sql_command_interface, critical_error);
   }
 
   delete sql_command_interface;
@@ -887,10 +867,10 @@ thd_end:
   delete thd;
   m_clone_thd = nullptr;
   thd = nullptr;
-  my_thread_end();
   m_clone_process_thd_state.set_terminated();
   mysql_cond_broadcast(&m_run_cond);
   mysql_mutex_unlock(&m_run_lock);
 
+  my_thread_end();
   my_thread_exit(nullptr);
 }
