@@ -1,4 +1,4 @@
-/* Copyright (c) 2016, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -77,12 +77,13 @@ void log_primary_member_details() {
        Group_member_info::MEMBER_ROLE_SECONDARY)) {
     std::string primary_member_uuid;
     group_member_mgr->get_primary_member_uuid(primary_member_uuid);
-    Group_member_info primary_member_info;
-    if (!group_member_mgr->get_group_member_info(primary_member_uuid,
-                                                 primary_member_info)) {
-      LogPluginErr(SYSTEM_LEVEL, ER_GRP_RPL_SRV_SECONDARY_MEM,
-                   primary_member_info.get_hostname().c_str(),
-                   primary_member_info.get_port());
+    Group_member_info *primary_member_info =
+        group_member_mgr->get_group_member_info(primary_member_uuid);
+    if (primary_member_info != nullptr) {
+      LogPluginErr(INFORMATION_LEVEL, ER_GRP_RPL_SERVER_WORKING_AS_SECONDARY,
+                   primary_member_info->get_hostname().c_str(),
+                   primary_member_info->get_port());
+      delete primary_member_info;
     }
   }
 }
@@ -99,7 +100,7 @@ void plugin_escape_string(std::string &string_to_escape) {
   size_t length = string_to_escape.length();
   char *end_string =
       (char *)my_malloc(PSI_NOT_INSTRUMENTED, 2 * length + 1, MYF(0));
-  escape_string_for_mysql(&my_charset_utf8mb3_general_ci, end_string,
+  escape_string_for_mysql(&my_charset_utf8_general_ci, end_string,
                           2 * length + 1, string_to_escape.c_str(), length);
   string_to_escape.assign(end_string);
   my_free(end_string);
