@@ -1,16 +1,15 @@
-/* Copyright (c) 2015, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,28 +27,19 @@
 #include "my_dbug.h"
 #include "my_systime.h"
 #include "mysql/components/services/log_builtins.h"
-#include "plugin/group_replication/include/compatibility_module.h"
 #include "plugin/group_replication/include/plugin.h"
-#include "plugin/group_replication/include/plugin_handlers/recovery_metadata.h"
 #include "plugin/group_replication/include/plugin_psi.h"
 #include "plugin/group_replication/include/plugin_server_include.h"
-#include "plugin/group_replication/include/plugin_variables/recovery_endpoints.h"
 #include "plugin/group_replication/include/recovery_channel_state_observer.h"
 #include "plugin/group_replication/include/recovery_state_transfer.h"
-#include "plugin/group_replication/include/services/system_variable/get_system_variable.h"
-#include "string_with_len.h"
 
 using std::string;
 
 Recovery_state_transfer::Recovery_state_transfer(
     char *recovery_channel_name, const string &member_uuid,
     Channel_observation_manager *channel_obsr_mngr)
-    : m_recovery_channel_name(recovery_channel_name),
-      m_until_condition(CHANNEL_UNTIL_VIEW_ID),
-      selected_donor(nullptr),
+    : selected_donor(nullptr),
       group_members(nullptr),
-      suitable_donors(
-          Malloc_allocator<Group_member_info *>(key_group_member_info)),
       donor_connection_retry_count(0),
       recovery_aborted(false),
       donor_transfer_finished(false),
@@ -88,7 +78,8 @@ Recovery_state_transfer::Recovery_state_transfer(
 
 Recovery_state_transfer::~Recovery_state_transfer() {
   if (group_members != nullptr) {
-    Group_member_info_list_iterator member_it = group_members->begin();
+    std::vector<Group_member_info *>::iterator member_it =
+        group_members->begin();
     while (member_it != group_members->end()) {
       delete (*member_it);
       ++member_it;
@@ -96,15 +87,12 @@ Recovery_state_transfer::~Recovery_state_transfer() {
   }
   delete group_members;
   delete recovery_channel_observer;
-  delete selected_donor;
-  selected_donor = nullptr;
   mysql_mutex_destroy(&recovery_lock);
   mysql_cond_destroy(&recovery_condition);
   mysql_mutex_destroy(&donor_selection_lock);
 }
 
-void Recovery_state_transfer::initialize(const string &view_id,
-                                         bool is_vcle_enabled) {
+void Recovery_state_transfer::initialize(const string &rec_view_id) {
   DBUG_TRACE;
 
   // reset the recovery aborted flag
@@ -117,69 +105,30 @@ void Recovery_state_transfer::initialize(const string &view_id,
   donor_channel_thread_error = false;
   // reset the retry count
   donor_connection_retry_count = 0;
-  this->view_id.assign(view_id);
-  if (is_vcle_enabled) {
-    m_until_condition = CHANNEL_UNTIL_VIEW_ID;
-  } else {
-    m_until_condition = CHANNEL_UNTIL_APPLIER_AFTER_GTIDS;
-  }
-}
 
-void Recovery_state_transfer::set_until_condition_after_gtids(
-    const std::string &after_gtids) {
-  m_after_gtids.assign(after_gtids);
+  this->view_id.clear();
+  this->view_id.append(rec_view_id);
 }
 
 void Recovery_state_transfer::inform_of_applier_stop(my_thread_id thread_id,
-                                                     bool aborted) {
+                                                     bool) {
   DBUG_TRACE;
-  bool error{false};
 
   /*
     This method doesn't take any locks as it could lead to dead locks between
     the connection process and this method that can be invoked in that context.
+    Since this only affects the recovery loop and the flag is reset at each
     connection, no major concurrency issues should exist.
   */
 
   // Act if:
-  if (!donor_transfer_finished &&  // we don't have all the data yet
-      !recovery_aborted &&         // recovery was not aborted
+  if (
+      // we don't have all the data yet
+      !donor_transfer_finished &&
+      // recovery was not aborted
+      !recovery_aborted &&
       // the signal belongs to the recovery donor channel thread
       donor_connection_interface.is_own_event_applier(thread_id)) {
-    /*
-      if recovery is using CHANNEL_UNTIL_APPLIER_AFTER_GTIDS for start replica
-      until condition then,
-      - if aborted == true :
-        which could be due to some error or sql thread than we should not call
-        end_state_transfer() for successful completion, we should set error so
-        that recovery_thd can error out.
-      - else if aborted != true :
-        then we call end_state_transfer() for successful completion.
-    */
-    if (m_until_condition == CHANNEL_UNTIL_APPLIER_AFTER_GTIDS) {
-      if (aborted) {
-        error = true;
-      } else {
-        DBUG_EXECUTE_IF(
-            "group_replication_recovery_after_gtids_applier_stop_error_out",
-            { error = true; });
-        std::string local_gtid_executed_string;
-        if (!error &&
-            verify_member_has_after_gtids_present(local_gtid_executed_string)) {
-          LogPluginErr(INFORMATION_LEVEL,
-                       ER_GROUP_REPLICATION_RECOVERY_STOPPED_GTID_PRESENT,
-                       m_after_gtids.c_str());
-          end_state_transfer();
-        } else {
-          error = true;
-        }
-      }
-    } else {
-      error = true;
-    }
-  }
-
-  if (error) {
     mysql_mutex_lock(&recovery_lock);
     donor_channel_thread_error = true;
     mysql_cond_broadcast(&recovery_condition);
@@ -212,7 +161,6 @@ void Recovery_state_transfer::inform_of_receiver_stop(my_thread_id thread_id) {
 void Recovery_state_transfer::initialize_group_info() {
   DBUG_TRACE;
 
-  delete selected_donor;
   selected_donor = nullptr;
   selected_donor_hostname.clear();
   // Update the group member info
@@ -224,7 +172,7 @@ void Recovery_state_transfer::initialize_group_info() {
 void Recovery_state_transfer::update_group_membership(bool update_donor) {
   DBUG_TRACE;
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   mysql_mutex_assert_owner(&donor_selection_lock);
 #endif
 
@@ -235,7 +183,8 @@ void Recovery_state_transfer::update_group_membership(bool update_donor) {
   }
 
   if (group_members != nullptr) {
-    Group_member_info_list_iterator member_it = group_members->begin();
+    std::vector<Group_member_info *>::iterator member_it =
+        group_members->begin();
     while (member_it != group_members->end()) {
       delete (*member_it);
       ++member_it;
@@ -288,7 +237,10 @@ int Recovery_state_transfer::update_recovery_process(bool did_members_left) {
     current_donor_uuid.assign(selected_donor->get_uuid());
     current_donor_hostname.assign(selected_donor->get_hostname());
     current_donor_port = selected_donor->get_port();
-    donor_left = !group_member_mgr->is_member_info_present(current_donor_uuid);
+    Group_member_info *current_donor =
+        group_member_mgr->get_group_member_info(current_donor_uuid);
+    donor_left = (current_donor == nullptr);
+    delete current_donor;
   }
 
   /*
@@ -304,7 +256,6 @@ int Recovery_state_transfer::update_recovery_process(bool did_members_left) {
   */
   if (donor_left) {
     // The selected donor no longer holds a meaning after deleting the group
-    delete selected_donor;
     selected_donor = nullptr;
     if (connected_to_donor) {
       /*
@@ -356,7 +307,7 @@ int Recovery_state_transfer::check_recovery_thread_status() {
   // if some of the threads are running
   if (donor_connection_interface.is_receiver_thread_running() ||
       donor_connection_interface.is_applier_thread_running()) {
-    return terminate_recovery_slave_threads() != STATE_TRANSFER_OK;
+    return terminate_recovery_slave_threads(); /* purecov: inspected */
   }
   return 0;
 }
@@ -371,30 +322,20 @@ void Recovery_state_transfer::build_donor_list(string *selected_donor_uuid) {
 
   suitable_donors.clear();
 
-  Group_member_info_list_iterator member_it = group_members->begin();
+  std::vector<Group_member_info *>::iterator member_it = group_members->begin();
 
   while (member_it != group_members->end()) {
     Group_member_info *member = *member_it;
-    const string m_uuid(member->get_uuid());
-    const bool is_online =
+    // is online and it's not me
+    string m_uuid(member->get_uuid());
+    bool is_online =
         member->get_recovery_status() == Group_member_info::MEMBER_ONLINE;
-    const bool not_self = m_uuid.compare(member_uuid);
+    bool not_self = m_uuid.compare(member_uuid);
     bool valid_donor = false;
 
-    // is online and it's not me
     if (is_online && not_self) {
-      const Member_version local_member_version =
-          local_member_info->get_member_version();
-      const Member_version donor_member_version = member->get_member_version();
-      std::set<Member_version> local_and_donor_member_versions;
-      local_and_donor_member_versions.insert(local_member_version);
-      local_and_donor_member_versions.insert(donor_member_version);
-
-      if (donor_member_version <= local_member_version) {
-        suitable_donors.push_back(member);
-        valid_donor = true;
-      } else if (Compatibility_module::do_all_versions_belong_to_the_same_lts(
-                     local_and_donor_member_versions)) {
+      if (member->get_member_version() <=
+          local_member_info->get_member_version()) {
         suitable_donors.push_back(member);
         valid_donor = true;
       } else if (get_allow_local_lower_version_join()) {
@@ -407,18 +348,16 @@ void Recovery_state_transfer::build_donor_list(string *selected_donor_uuid) {
     // reference
     if (selected_donor_uuid != nullptr &&
         !m_uuid.compare(*selected_donor_uuid) && valid_donor) {
-      if (selected_donor != nullptr) {
-        selected_donor->update(*member);
-      } else {
-        selected_donor = new Group_member_info(*member);
-      }
+      selected_donor = member;
     }
 
     ++member_it;
   }
 
   if (suitable_donors.size() > 1) {
-    vector_random_shuffle(&suitable_donors);
+    std::random_device rng;
+    std::mt19937 urng(rng());
+    std::shuffle(suitable_donors.begin(), suitable_donors.end(), urng);
   }
 
   // no need for errors if no donors exist, we thrown it in the connection
@@ -439,7 +378,7 @@ int Recovery_state_transfer::establish_donor_connection() {
         const char act[] =
             "now signal signal.connection_attempt_3 wait_for "
             "signal.reset_recovery_retry_count_done";
-        assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
+        DBUG_ASSERT(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
       }
     };);
     // max number of retries reached, abort
@@ -490,34 +429,18 @@ int Recovery_state_transfer::establish_donor_connection() {
     donor_channel_thread_error = false;
 
     // Get the last element and delete it
-    if (selected_donor != nullptr) {
-      selected_donor->update(*suitable_donors.back());
-    } else {
-      selected_donor = new Group_member_info(*suitable_donors.back());
-    }
+    selected_donor = suitable_donors.back();
     suitable_donors.pop_back();
     // increment the number of tries
     donor_connection_retry_count++;
 
-    Donor_recovery_endpoints donor_endpoints;
-    std::vector<std::pair<std::string, uint>> endpoints;
-    endpoints = donor_endpoints.get_endpoints(selected_donor);
+    if ((error = initialize_donor_connection())) {
+      LogPluginErr(ERROR_LEVEL,
+                   ER_GRP_RPL_CONFIG_RECOVERY); /* purecov: inspected */
+    }
 
-    for (auto endpoint : endpoints) {
-      auto hostname = endpoint.first;
-      uint port = endpoint.second;
-      if ((error = initialize_donor_connection(hostname, port))) {
-        LogPluginErr(ERROR_LEVEL,
-                     ER_GRP_RPL_CONFIG_RECOVERY); /* purecov: inspected */
-      }
-
-      if (!error && !recovery_aborted) {
-        error = start_recovery_donor_threads();
-      }
-
-      if (!error) {
-        break;
-      }
+    if (!error && !recovery_aborted) {
+      error = start_recovery_donor_threads();
     }
 
     if (!error) {
@@ -538,8 +461,7 @@ int Recovery_state_transfer::establish_donor_connection() {
   return error;
 }
 
-int Recovery_state_transfer::initialize_donor_connection(std::string hostname,
-                                                         uint port) {
+int Recovery_state_transfer::initialize_donor_connection() {
   DBUG_TRACE;
 
   int error = 0;
@@ -554,114 +476,51 @@ int Recovery_state_transfer::initialize_donor_connection(std::string hostname,
     attached to this object, more precisely to
     selected_donor_hostname class member.
   */
-
-  selected_donor_hostname.assign(hostname);
+  selected_donor_hostname.assign(selected_donor->get_hostname());
+  char *hostname = const_cast<char *>(selected_donor_hostname.c_str());
+  uint port = selected_donor->get_port();
 
   error = donor_connection_interface.initialize_channel(
-      const_cast<char *>(hostname.c_str()), port, /*user*/ nullptr,
-      /*pass*/ nullptr, recovery_use_ssl, recovery_ssl_ca, recovery_ssl_capath,
-      recovery_ssl_cert, recovery_ssl_cipher, recovery_ssl_key,
-      recovery_ssl_crl, recovery_ssl_crlpath, recovery_ssl_verify_server_cert,
-      DEFAULT_THREAD_PRIORITY, /*retry_count*/ 1, /*preserve_logs*/ false,
+      hostname, port, nullptr, nullptr, recovery_use_ssl, recovery_ssl_ca,
+      recovery_ssl_capath, recovery_ssl_cert, recovery_ssl_cipher,
+      recovery_ssl_key, recovery_ssl_crl, recovery_ssl_crlpath,
+      recovery_ssl_verify_server_cert, DEFAULT_THREAD_PRIORITY, 1, false,
       recovery_public_key_path, recovery_get_public_key,
       recovery_compression_algorithm, recovery_zstd_compression_level,
       recovery_tls_version,
-      recovery_tls_ciphersuites_null ? nullptr : recovery_tls_ciphersuites,
-      /*ignore_ws_mem_limit*/ true, /*allow_drop_write_set*/ true);
+      recovery_tls_ciphersuites_null ? nullptr : recovery_tls_ciphersuites);
 
   if (!error) {
     LogPluginErr(INFORMATION_LEVEL, ER_GRP_RPL_ESTABLISHING_CONN_GRP_REC_DONOR,
-                 selected_donor->get_uuid().c_str(), hostname.c_str(), port);
+                 selected_donor->get_uuid().c_str(), hostname, port);
   } else {
     LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_CREATE_GRP_RPL_REC_CHANNEL,
-                 selected_donor->get_uuid().c_str(), hostname.c_str(), port);
+                 selected_donor->get_uuid().c_str(), hostname,
+                 port); /* purecov: inspected */
   }
 
   return error;
 }
 
-bool Recovery_state_transfer::verify_member_has_after_gtids_present(
-    std::string &local_gtid_executed_string) {
-  Get_system_variable get_system_variable;
-  if (get_system_variable.get_global_gtid_executed(
-          local_gtid_executed_string)) {
-    LogPluginErr(ERROR_LEVEL,
-                 ER_GROUP_REPLICATION_RECOVERY_FETCHING_GTID_EXECUTED_SET);
-    return false;
-  }
-
-  Tsid_map local_tsid_map(nullptr);
-  Gtid_set local_gtid_executed(&local_tsid_map, nullptr);
-  if (local_gtid_executed.add_gtid_text(local_gtid_executed_string.c_str()) !=
-      RETURN_STATUS_OK) {
-    LogPluginErr(ERROR_LEVEL,
-                 ER_GROUP_REPLICATION_RECOVERY_ERROR_ADD_GTID_EXECUTED);
-    return false;
-  }
-
-  Tsid_map donor_tsid_map(nullptr);
-  Gtid_set donor_gtid_executed(&donor_tsid_map, nullptr);
-  if (donor_gtid_executed.add_gtid_text(m_after_gtids.c_str()) !=
-      RETURN_STATUS_OK) {
-    LogPluginErr(ERROR_LEVEL,
-                 ER_GROUP_REPLICATION_RECOVERY_ERROR_ADD_GTID_EXECUTED);
-    return false;
-  }
-
-  if (donor_gtid_executed.is_subset(&local_gtid_executed)) {
-    return true;
-  }
-
-  return false;
-}
-
 int Recovery_state_transfer::start_recovery_donor_threads() {
   DBUG_TRACE;
-  int error = 1;
-  std::string local_gtid_executed_string;
 
-  /*
-    If recovery is without VCLE which is START REPLICA SQL_AFTER_GTIDS UNTIL
-    clause, then joiner is recovered till Gtid_set sent in Recovery Metadata by
-    sender. But before starting recovery threads it's checked if local member
-    already has those gtid_set present. If present then recovery is skipped.
-  */
-  if (m_until_condition == CHANNEL_UNTIL_APPLIER_AFTER_GTIDS) {
-    if (verify_member_has_after_gtids_present(local_gtid_executed_string)) {
-      LogPluginErr(INFORMATION_LEVEL,
-                   ER_GROUP_REPLICATION_RECOVERY_SKIPPED_GTID_PRESENT,
-                   local_gtid_executed_string.c_str(), m_after_gtids.c_str());
-      end_state_transfer();
-      return 0;
-    } else {
-      // recovery using START REPLICA SQL_AFTER_GTIDS UNTIL clause.
-      error = donor_connection_interface.start_threads(
-          true, true, &m_after_gtids, true, m_until_condition);
-    }
-    /*
-      If recovery is using VCLE which is START REPLICA SQL_VIEW_ID UNTIL clause.
-    */
-  } else if (m_until_condition == CHANNEL_UNTIL_VIEW_ID) {
-    error = donor_connection_interface.start_threads(true, true, &view_id, true,
-                                                     m_until_condition);
-  }
+  int error =
+      donor_connection_interface.start_threads(true, true, &view_id, true);
 
   if (!error) {
     DBUG_EXECUTE_IF("pause_after_io_thread_stop_hook", {
       const char act[] =
           "now "
           "WAIT_FOR reached_stopping_io_thread";
-      assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
+      DBUG_ASSERT(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
     };);
     DBUG_EXECUTE_IF("pause_after_sql_thread_stop_hook", {
       const char act[] =
           "now "
           "WAIT_FOR reached_stopping_sql_thread";
-      assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
+      DBUG_ASSERT(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
     };);
-    DBUG_EXECUTE_IF(
-        "group_replication_recovery_sleep_before_wait_for_connection",
-        { my_sleep(1000000); });
 
     /*
       Register a channel observer to detect SQL/IO thread stops
@@ -686,60 +545,9 @@ int Recovery_state_transfer::start_recovery_donor_threads() {
   bool is_applier_stopped =
       !donor_connection_interface.is_applier_thread_running();
 
-  if (m_until_condition == CHANNEL_UNTIL_APPLIER_AFTER_GTIDS) {
-    /*
-      If applier is stopping we should give some time for it to
-      stop, so that we can compare the after gtids set.
-    */
-    const unsigned long sleep_time{100000UL};  // 100 milliseconds
-    constexpr const unsigned long max_channel_stopped_wait_timeout{30UL *
-                                                                   1000000UL};
-    unsigned long channel_stopped_wait_timeout{0};
-    while (!error && !recovery_aborted && !donor_channel_thread_error &&
-           !on_failover && !donor_transfer_finished &&
-           donor_connection_interface.is_applier_thread_stopping() &&
-           channel_stopped_wait_timeout < max_channel_stopped_wait_timeout) {
-      my_sleep(sleep_time);
-      channel_stopped_wait_timeout += sleep_time;
-    }
-
-    /*
-      Check if replication applier thread has stopped:
-      Verify after_gtids received from recovery metadata
-      donor is present on member.
-       - If present, recovery has completed.
-       - If not present, set error.
-    */
-    if (!donor_transfer_finished &&
-        !donor_connection_interface.is_applier_thread_running()) {
-      if (verify_member_has_after_gtids_present(local_gtid_executed_string)) {
-        LogPluginErr(INFORMATION_LEVEL,
-                     ER_GROUP_REPLICATION_RECOVERY_STOPPED_GTID_PRESENT,
-                     m_after_gtids.c_str());
-        end_state_transfer();
-        error = 0;
-      } else {
-        if (!error) {
-          error = 1;
-        }
-        channel_observation_manager->unregister_channel_observer(
-            recovery_channel_observer);
-      }
-    }
-
-    if (!error && !donor_transfer_finished &&
-        (donor_connection_interface.is_receiver_thread_stopping() ||
-         !donor_connection_interface.is_receiver_thread_running() ||
-         donor_connection_interface.is_applier_thread_stopping() ||
-         !donor_connection_interface.is_applier_thread_running())) {
-      error = 1;
-      channel_observation_manager->unregister_channel_observer(
-          recovery_channel_observer);
-    }
-
-  } else if (!error && !donor_transfer_finished &&
-             (is_receiver_stopping || is_receiver_stopped ||
-              is_applier_stopping || is_applier_stopped)) {
+  if (!error && !donor_transfer_finished &&
+      (is_receiver_stopping || is_receiver_stopped || is_applier_stopping ||
+       is_applier_stopped)) {
     error = 1;
     channel_observation_manager->unregister_channel_observer(
         recovery_channel_observer);
@@ -747,11 +555,11 @@ int Recovery_state_transfer::start_recovery_donor_threads() {
 
   DBUG_EXECUTE_IF("pause_after_io_thread_stop_hook", {
     const char act[] = "now SIGNAL continue_to_stop_io_thread";
-    assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
+    DBUG_ASSERT(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
   };);
   DBUG_EXECUTE_IF("pause_after_sql_thread_stop_hook", {
     const char act[] = "now SIGNAL continue_to_stop_sql_thread";
-    assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
+    DBUG_ASSERT(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
   };);
 
   if (error) {
@@ -769,22 +577,21 @@ int Recovery_state_transfer::start_recovery_donor_threads() {
   return error;
 }
 
-State_transfer_status Recovery_state_transfer::terminate_recovery_slave_threads(
-    bool purge_logs) {
+int Recovery_state_transfer::terminate_recovery_slave_threads(bool purge_logs) {
   DBUG_TRACE;
 
   LogPluginErr(INFORMATION_LEVEL, ER_GRP_RPL_DONOR_CONN_TERMINATION);
 
-  State_transfer_status error = STATE_TRANSFER_OK;
+  int error = 0;
 
   // If the threads never started, the method just returns
-  if (donor_connection_interface.stop_threads(true, true)) {
-    error = STATE_TRANSFER_STOP;
-    LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_STOPPING_GRP_REC);
+  if ((error = donor_connection_interface.stop_threads(true, true))) {
+    LogPluginErr(ERROR_LEVEL,
+                 ER_GRP_RPL_STOPPING_GRP_REC); /* purecov: inspected */
   } else {
     if (purge_logs) {
       // If there is no repository in place nothing happens
-      if (purge_recovery_slave_threads_repos()) error = STATE_TRANSFER_PURGE;
+      error = purge_recovery_slave_threads_repos();
     }
   }
 
@@ -802,26 +609,19 @@ int Recovery_state_transfer::purge_recovery_slave_threads_repos() {
     /* purecov: end */
   }
   error = donor_connection_interface.initialize_channel(
-      /*host*/ const_cast<char *>("<NULL>"), /*port*/ 0, /*user*/ nullptr,
-      /*pass*/ nullptr, /*use_ssl*/ false, /*ssl_ca*/ nullptr,
-      /*ssl_capath*/ nullptr, /*ssl_cert*/ nullptr, /*ssl_cipher*/ nullptr,
-      /*ssl_key*/ nullptr, /*ssl_crl*/ nullptr, /*ssl_crlpath*/ nullptr,
-      /*ssl_verify*/ false,
-      /*priority*/ DEFAULT_THREAD_PRIORITY,
-      /*retry_count*/ 1,
-      /*preserve_logs*/ false, /*public_key_path*/ nullptr,
-      /*get_public_key*/ false, /*compression_alg*/ nullptr,
-      /*compression_level*/ 0, /*tls_version*/ nullptr, /*tls_cipher*/ nullptr,
-      /*ignore_ws_mem_limit*/ true, /*allow_drop_write_set*/ true);
+      const_cast<char *>("<NULL>"), 0, nullptr, nullptr, false, nullptr,
+      nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, false,
+      DEFAULT_THREAD_PRIORITY, 1, false, nullptr, false, nullptr, 0, nullptr,
+      nullptr);
 
   return error;
 }
 
-State_transfer_status Recovery_state_transfer::state_transfer(
+int Recovery_state_transfer::state_transfer(
     Plugin_stage_monitor_handler &stage_handler) {
   DBUG_TRACE;
 
-  State_transfer_status error = STATE_TRANSFER_OK;
+  int error = 0;
 
   while (!donor_transfer_finished && !recovery_aborted) {
     /*
@@ -854,12 +654,12 @@ State_transfer_status Recovery_state_transfer::state_transfer(
           recovery_channel_observer);
 
       // Stop the threads before reconfiguring the connection
-      if (donor_connection_interface.stop_threads(true, true)) {
+      if ((error = donor_connection_interface.stop_threads(true, true))) {
         /* purecov: begin inspected */
         LogPluginErr(ERROR_LEVEL,
                      ER_GRP_RPL_UNABLE_TO_KILL_CONN_REC_DONOR_FAILOVER);
         // if we can't stop, abort recovery
-        return STATE_TRANSFER_STOP;
+        return error;
         /* purecov: end */
       }
     }
@@ -868,8 +668,7 @@ State_transfer_status Recovery_state_transfer::state_transfer(
                             __FILE__, __LINE__, 0, 0);
     if (!recovery_aborted) {
       // if the connection to the donor failed, abort recovery
-      if (establish_donor_connection()) {
-        error = STATE_TRANSFER_NO_CONNECTION;
+      if ((error = establish_donor_connection())) {
         break;
       }
     }
@@ -877,12 +676,7 @@ State_transfer_status Recovery_state_transfer::state_transfer(
                             __FILE__, __LINE__, 0, 0);
 
     /*
-      donor_transfer_finished    -> When recovery is using CHANNEL_UNTIL_VIEW_ID
-                                 set by the set_retrieved_cert_info method.
-                                 When recovery is using
-                                 CHANNEL_UNTIL_APPLIER_AFTER_GTIDS for start
-                                 replica until condition set by
-                                 inform_of_applier_stop method.
+      donor_transfer_finished    -> set by the set_retrieved_cert_info method.
                                  lock: recovery_lock
       recovery_aborted           -> set when stopping recovery
                                  lock: run_lock
@@ -904,17 +698,8 @@ State_transfer_status Recovery_state_transfer::state_transfer(
 
   channel_observation_manager->unregister_channel_observer(
       recovery_channel_observer);
-
   // do not purge logs if an error occur, keep the diagnose on SLAVE STATUS
-  bool purge_relay_logs = (error == STATE_TRANSFER_OK);
-  DBUG_EXECUTE_IF("gr_recovery_skip_purge_logs", { purge_relay_logs = false; });
-  State_transfer_status stop_error =
-      terminate_recovery_slave_threads(purge_relay_logs);
-
-  // If a connection error exists, we preserve it
-  if (error == STATE_TRANSFER_OK && stop_error != STATE_TRANSFER_OK)
-    error = stop_error;
-
+  terminate_recovery_slave_threads(!error);
   connected_to_donor = false;
 
   return error;
