@@ -1,4 +1,4 @@
-/* Copyright (c) 2014, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -22,7 +22,6 @@
 
 #include <sstream>
 
-#include "plugin/group_replication/include/plugin.h"
 #include "plugin/group_replication/include/replication_threads_api.h"
 
 #include <mysql/components/services/log_builtins.h>
@@ -31,14 +30,6 @@
 
 using std::string;
 
-Replication_thread_api::Replication_thread_api(const char *channel_interface)
-    : stop_wait_timeout(get_components_stop_timeout_var()),
-      interface_channel(channel_interface) {}
-
-Replication_thread_api::Replication_thread_api()
-    : stop_wait_timeout(get_components_stop_timeout_var()),
-      interface_channel(nullptr) {}
-
 int Replication_thread_api::initialize_channel(
     char *hostname, uint port, char *user, char *password, bool use_ssl,
     char *ssl_ca, char *ssl_capath, char *ssl_cert, char *ssl_cipher,
@@ -46,8 +37,7 @@ int Replication_thread_api::initialize_channel(
     bool ssl_verify_server_cert, int priority, int retry_count,
     bool preserve_logs, char *public_key_path, bool get_public_key,
     char *compression_algorithm, uint zstd_compression_level, char *tls_version,
-    char *tls_ciphersuites, bool ignore_ws_mem_limit,
-    bool allow_drop_write_set) {
+    char *tls_ciphersuites) {
   DBUG_TRACE;
   int error = 0;
 
@@ -66,10 +56,6 @@ int Replication_thread_api::initialize_channel(
   if (priority == GROUP_REPLICATION_APPLIER_THREAD_PRIORITY) {
     info.thd_tx_priority = GROUP_REPLICATION_APPLIER_THREAD_PRIORITY;
   }
-
-  info.m_ignore_write_set_memory_limit = ignore_ws_mem_limit;
-  info.m_allow_drop_write_set = allow_drop_write_set;
-
   info.type = GROUP_REPLICATION_CHANNEL;
 
   info.retry_count = retry_count;
@@ -111,26 +97,22 @@ int Replication_thread_api::initialize_channel(
   return error;
 }
 
-int Replication_thread_api::start_threads(
-    bool start_receiver, bool start_applier, string *value,
-    bool wait_for_connection, enum_channel_until_condition until_condition) {
+int Replication_thread_api::start_threads(bool start_receiver,
+                                          bool start_applier, string *view_id,
+                                          bool wait_for_connection) {
   DBUG_TRACE;
 
   Channel_connection_info info;
   initialize_channel_connection_info(&info);
 
-  char *cvalue = nullptr;
+  char *cview_id = nullptr;
 
-  if (value) {
-    cvalue = new char[value->size() + 1];
-    memcpy(cvalue, value->c_str(), value->size() + 1);
+  if (view_id) {
+    cview_id = new char[view_id->size() + 1];
+    memcpy(cview_id, view_id->c_str(), view_id->size() + 1);
 
-    info.until_condition = until_condition;
-    if (until_condition == CHANNEL_UNTIL_VIEW_ID) {
-      info.view_id = cvalue;
-    } else if (until_condition == CHANNEL_UNTIL_APPLIER_AFTER_GTIDS) {
-      info.gtid = cvalue;
-    }
+    info.until_condition = CHANNEL_UNTIL_VIEW_ID;
+    info.view_id = cview_id;
   }
 
   int thread_mask = 0;
@@ -141,11 +123,11 @@ int Replication_thread_api::start_threads(
     thread_mask |= CHANNEL_RECEIVER_THREAD;
   }
 
-  int error = channel_start(interface_channel, &info, thread_mask,
-                            wait_for_connection, true);
+  int error =
+      channel_start(interface_channel, &info, thread_mask, wait_for_connection);
 
-  if (cvalue != nullptr) {
-    delete[] cvalue;
+  if (view_id) {
+    delete[] cview_id;
   }
 
   return error;
@@ -237,9 +219,6 @@ int Replication_thread_api::wait_for_gtid_execution(std::string &retrieved_set,
                                                     bool update_THD_status) {
   DBUG_TRACE;
 
-  DBUG_EXECUTE_IF("group_replication_wait_for_gtid_execution_force_error",
-                  { return REPLICATION_THREAD_WAIT_NO_INFO_ERROR; });
-
   int error = channel_wait_until_transactions_applied(
       interface_channel, retrieved_set.c_str(), timeout, update_THD_status);
   return error;
@@ -334,31 +313,23 @@ bool Replication_thread_api::get_retrieved_gtid_set(std::string &retrieved_set,
 bool Replication_thread_api::get_channel_credentials(std::string &username,
                                                      std::string &password,
                                                      const char *channel_name) {
-  DBUG_TRACE;
+  DBUG_ENTER("Replication_thread_api::get_channel_credentials");
+
   const char *name = channel_name ? channel_name : interface_channel;
+  const char *user_arg = nullptr;
+  char user_pass[MAX_PASSWORD_LENGTH + 1];
+  char *user_pass_pointer = user_pass;
+  size_t password_size = sizeof(user_pass);
 
   int error;
-  error = channel_get_credentials(name, username, password);
-  if (error) {
-    username.clear();
-    password.clear();
+  error = channel_get_credentials(name, &user_arg, &user_pass_pointer,
+                                  &password_size);
+  if (!error) {
+    username.assign(user_arg);
+    password.assign(user_pass, password_size);
   }
 
-  return (error != 0);
-}
-
-bool Replication_thread_api::get_channel_network_namespace(
-    std::string &net_ns, const char *channel_name) {
-  DBUG_TRACE;
-  const char *name = channel_name ? channel_name : interface_channel;
-
-  int error;
-  error = channel_get_network_namespace(name, net_ns);
-  if (error) {
-    net_ns.clear();
-  }
-
-  return (error != 0);
+  DBUG_RETURN((error != 0));
 }
 
 bool Replication_thread_api::is_partial_transaction_on_relay_log() {
@@ -388,15 +359,3 @@ int Replication_thread_api::rpl_binlog_dump_thread_kill() {
   DBUG_TRACE;
   return binlog_dump_thread_kill();
 }
-
-int Replication_thread_api::delete_credential(const char *channel_name) {
-  DBUG_TRACE;
-  return channel_delete_credentials(channel_name);
-}
-
-bool Replication_thread_api::
-    is_any_channel_using_uuid_for_assign_gtids_to_anonymous_transaction(
-        const char *group_name) {
-  DBUG_TRACE;
-  return channel_has_same_uuid_as_group_name(group_name);
-}
