@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2022, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -42,7 +42,7 @@ Sql_service_command_interface::~Sql_service_command_interface() {
 int Sql_service_command_interface::establish_session_connection(
     enum_plugin_con_isolation isolation_param, const char *user,
     void *plugin_pointer) {
-  assert(m_server_interface == nullptr);
+  DBUG_ASSERT(m_server_interface == nullptr);
 
   int error = 0;
   connection_thread_isolation = isolation_param;
@@ -99,7 +99,7 @@ bool Sql_service_command_interface::is_session_valid() {
 
 bool Sql_service_command_interface::is_session_killed() {
   DBUG_ENTER("Sql_service_command_interface::is_session_killed");
-  assert(m_server_interface != nullptr);
+  DBUG_ASSERT(m_server_interface != nullptr);
   if (m_server_interface->is_session_killed(m_server_interface->get_session()))
     DBUG_RETURN(true);
   DBUG_RETURN(false);
@@ -153,20 +153,20 @@ long Sql_service_commands::internal_set_super_read_only(
   DBUG_EXECUTE_IF("group_replication_read_mode_error", { return 1; });
   DBUG_EXECUTE_IF("group_replication_skip_read_mode", { return 0; });
 
-  assert(sql_interface != nullptr);
+  DBUG_ASSERT(sql_interface != nullptr);
 
   Sql_resultset rset;
   long srv_err = sql_interface->execute_query("SET GLOBAL super_read_only= 1;");
-  LogPluginErr(SYSTEM_LEVEL, ER_GRP_RPL_SUPER_READ_ON);
-#ifndef NDEBUG
   if (srv_err == 0) {
+#ifndef DBUG_OFF
     long err;
     err =
         sql_interface->execute_query("SELECT @@GLOBAL.super_read_only", &rset);
 
-    assert(err || (!err && rset.get_rows() > 0 && rset.getLong(0) == 1));
-  }
+    DBUG_ASSERT(err || (!err && rset.get_rows() > 0 && rset.getLong(0) == 1));
+    LogPluginErr(INFORMATION_LEVEL, ER_GRP_RPL_SUPER_READ_ON);
 #endif
+  }
 
   return srv_err;
 }
@@ -191,20 +191,20 @@ long Sql_service_commands::internal_reset_super_read_only(
     Sql_service_interface *sql_interface, void *) {
   DBUG_TRACE;
 
-  assert(sql_interface != nullptr);
+  DBUG_ASSERT(sql_interface != nullptr);
 
   Sql_resultset rset;
 
   const char *query = "SET GLOBAL super_read_only= 0";
   long srv_err = sql_interface->execute_query(query);
-  LogPluginErr(SYSTEM_LEVEL, ER_GRP_RPL_SUPER_READ_OFF);
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   if (srv_err == 0) {
     long err;
     query = "SELECT @@GLOBAL.super_read_only";
     err = sql_interface->execute_query(query, &rset);
 
-    assert(!err && rset.get_rows() > 0 && rset.getLong(0) == 0);
+    DBUG_ASSERT(!err && rset.get_rows() > 0 && rset.getLong(0) == 0);
+    LogPluginErr(INFORMATION_LEVEL, ER_GRP_RPL_SUPER_READ_OFF);
   }
 #endif
   return srv_err;
@@ -229,21 +229,21 @@ long Sql_service_commands::internal_reset_read_only(
     Sql_service_interface *sql_interface, void *) {
   DBUG_TRACE;
 
-  assert(sql_interface != nullptr);
+  DBUG_ASSERT(sql_interface != nullptr);
 
   Sql_resultset rset;
 
   const char *query = "SET GLOBAL read_only= 0";
   long srv_err = sql_interface->execute_query(query);
-  LogPluginErr(SYSTEM_LEVEL, ER_GRP_RPL_SUPER_READ_OFF);
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   if (srv_err == 0) {
     long err;
     query = "SELECT @@GLOBAL.read_only";
     err = sql_interface->execute_query(query, &rset);
 
-    assert(!err && rset.get_rows() > 0 && rset.getLong(0) == 0);
+    DBUG_ASSERT(!err && rset.get_rows() > 0 && rset.getLong(0) == 0);
+    LogPluginErr(INFORMATION_LEVEL, ER_GRP_RPL_SUPER_READ_OFF);
   }
 #endif
 
@@ -254,7 +254,7 @@ long Sql_service_commands::internal_kill_session(
     Sql_service_interface *sql_interface, void *session_id) {
   DBUG_TRACE;
 
-  assert(sql_interface != nullptr);
+  DBUG_ASSERT(sql_interface != nullptr);
 
   Sql_resultset rset;
   long srv_err = 0;
@@ -311,7 +311,7 @@ long Sql_service_commands::internal_get_server_super_read_only(
     Sql_service_interface *sql_interface, void *) {
   DBUG_TRACE;
 
-  assert(sql_interface != nullptr);
+  DBUG_ASSERT(sql_interface != nullptr);
 
   Sql_resultset rset;
   long server_super_read_only = -1;
@@ -345,7 +345,7 @@ long Sql_service_commands::internal_get_server_read_only(
     Sql_service_interface *sql_interface, void *) {
   DBUG_TRACE;
 
-  assert(sql_interface != nullptr);
+  DBUG_ASSERT(sql_interface != nullptr);
 
   Sql_resultset rset;
   longlong server_read_only = -1;
@@ -358,13 +358,86 @@ long Sql_service_commands::internal_get_server_read_only(
   return server_read_only;
 }
 
+int Sql_service_command_interface::get_server_gtid_executed(
+    string &gtid_executed) {
+  DBUG_TRACE;
+  long error = 0;
+
+  if (connection_thread_isolation != PSESSION_DEDICATED_THREAD) {
+    error = sql_service_commands.internal_get_server_gtid_executed(
+        m_server_interface, static_cast<void *>(&gtid_executed));
+  } else {
+    m_plugin_session_thread->set_return_pointer(
+        static_cast<void *>(&gtid_executed));
+    m_plugin_session_thread->queue_new_method_for_application(
+        &Sql_service_commands::internal_get_server_gtid_executed);
+    error = m_plugin_session_thread->wait_for_method_execution();
+  }
+
+  return error;
+}
+
+long Sql_service_commands::internal_get_server_gtid_executed(
+    Sql_service_interface *sql_interface, void *gtid_executed_arg) {
+  DBUG_TRACE;
+
+  DBUG_ASSERT(sql_interface != nullptr);
+
+  std::string *gtid_executed = static_cast<std::string *>(gtid_executed_arg);
+
+  Sql_resultset rset;
+  long srv_err =
+      sql_interface->execute_query("SELECT @@GLOBAL.gtid_executed", &rset);
+  if (srv_err == 0 && rset.get_rows() > 0) {
+    gtid_executed->assign(rset.getString(0));
+    return 0;
+  }
+  return 1;
+}
+
+int Sql_service_command_interface::get_server_gtid_purged(string &gtid_purged) {
+  DBUG_TRACE;
+  long error = 0;
+
+  if (connection_thread_isolation != PSESSION_DEDICATED_THREAD) {
+    error = sql_service_commands.internal_get_server_gtid_purged(
+        m_server_interface, static_cast<void *>(&gtid_purged));
+  } else {
+    m_plugin_session_thread->set_return_pointer(
+        static_cast<void *>(&gtid_purged));
+    m_plugin_session_thread->queue_new_method_for_application(
+        &Sql_service_commands::internal_get_server_gtid_purged);
+    error = m_plugin_session_thread->wait_for_method_execution();
+  }
+
+  return error;
+}
+
+long Sql_service_commands::internal_get_server_gtid_purged(
+    Sql_service_interface *sql_interface, void *gtid_purged_arg) {
+  DBUG_TRACE;
+
+  DBUG_ASSERT(sql_interface != nullptr);
+
+  std::string *gtid_purged = static_cast<std::string *>(gtid_purged_arg);
+
+  Sql_resultset rset;
+  long srv_err =
+      sql_interface->execute_query("SELECT @@GLOBAL.GTID_PURGED", &rset);
+  if (srv_err == 0 && rset.get_rows() > 0) {
+    gtid_purged->assign(rset.getString(0));
+    return 0;
+  }
+  return 1;
+}
+
 long Sql_service_command_interface::wait_for_server_gtid_executed(
     std::string &gtid_executed, int timeout) {
   DBUG_TRACE;
   long error = 0;
 
   /* No support for this method on thread isolation mode */
-  assert(connection_thread_isolation != PSESSION_DEDICATED_THREAD);
+  DBUG_ASSERT(connection_thread_isolation != PSESSION_DEDICATED_THREAD);
 
   if (connection_thread_isolation != PSESSION_DEDICATED_THREAD) {
     error = sql_service_commands.internal_wait_for_server_gtid_executed(
@@ -379,7 +452,7 @@ long Sql_service_commands::internal_wait_for_server_gtid_executed(
     int timeout) {
   DBUG_TRACE;
 
-  assert(sql_interface != nullptr);
+  DBUG_ASSERT(sql_interface != nullptr);
 
   DBUG_EXECUTE_IF("sql_int_wait_for_gtid_executed_no_timeout",
                   { timeout = 0; });
@@ -414,7 +487,7 @@ long Sql_service_commands::internal_set_persist_only_variable(
   std::pair<std::string, std::string> *variable_args =
       (std::pair<std::string, std::string> *)var_args;
 
-  assert(sql_interface != nullptr);
+  DBUG_ASSERT(sql_interface != nullptr);
 
   std::string query = "SET PERSIST_ONLY ";
   query.append(variable_args->first);
@@ -482,7 +555,7 @@ long Sql_service_commands::internal_clone_server(
     Sql_service_interface *sql_interface, void *var_args) {
   DBUG_ENTER("Sql_service_commands::internal_clone_server");
 
-  assert(sql_interface != nullptr);
+  DBUG_ASSERT(sql_interface != nullptr);
 
   std::tuple<std::string, std::string, std::string, std::string, bool,
              std::string *> *variable_args =
@@ -575,7 +648,7 @@ long Sql_service_commands::internal_execute_query(
     Sql_service_interface *sql_interface, void *var_args) {
   DBUG_ENTER("Sql_service_commands::internal_execute_query");
 
-  assert(sql_interface != nullptr);
+  DBUG_ASSERT(sql_interface != nullptr);
 
   std::pair<std::string, std::string *> *variable_args =
       static_cast<std::pair<std::string, std::string *> *>(var_args);
@@ -634,7 +707,7 @@ long Sql_service_commands::internal_execute_conditional_query(
     Sql_service_interface *sql_interface, void *var_args) {
   DBUG_ENTER("Sql_service_commands::internal_execute_conditional_query");
 
-  assert(sql_interface != nullptr);
+  DBUG_ASSERT(sql_interface != nullptr);
 
   std::tuple<std::string, bool *, std::string *> *variable_args =
       static_cast<std::tuple<std::string, bool *, std::string *> *>(var_args);
@@ -708,8 +781,7 @@ Session_plugin_thread::Session_plugin_thread(
   mysql_mutex_init(key_GR_LOCK_session_thread_method_exec, &m_method_lock,
                    MY_MUTEX_INIT_FAST);
   mysql_cond_init(key_GR_COND_session_thread_method_exec, &m_method_cond);
-  this->incoming_methods =
-      new Synchronized_queue<st_session_method *>(key_sql_service_command_data);
+  this->incoming_methods = new Synchronized_queue<st_session_method *>();
 }
 
 Session_plugin_thread::~Session_plugin_thread() {
@@ -733,7 +805,7 @@ void Session_plugin_thread::queue_new_method_for_application(
     bool terminate) {
   st_session_method *method_to_execute;
   method_to_execute = (st_session_method *)my_malloc(
-      key_sql_service_command_data, sizeof(st_session_method), MYF(0));
+      PSI_NOT_INSTRUMENTED, sizeof(st_session_method), MYF(0));
   method_to_execute->method = method;
   method_to_execute->terminated = terminate;
   m_method_execution_completed = false;
@@ -804,7 +876,7 @@ int Session_plugin_thread::terminate_session_thread() {
 
     struct timespec abstime;
     set_timespec(&abstime, 1);
-#ifndef NDEBUG
+#ifndef DBUG_OFF
     int error =
 #endif
         mysql_cond_timedwait(&m_run_cond, &m_run_lock, &abstime);
@@ -815,10 +887,10 @@ int Session_plugin_thread::terminate_session_thread() {
       mysql_mutex_unlock(&m_run_lock);
       return 1;
     }
-    assert(error == ETIMEDOUT || error == 0);
+    DBUG_ASSERT(error == ETIMEDOUT || error == 0);
   }
 
-  assert(!m_session_thread_state.is_running());
+  DBUG_ASSERT(!m_session_thread_state.is_running());
 
   while (!this->incoming_methods->empty()) {
     st_session_method *method = nullptr;
@@ -851,19 +923,6 @@ int Session_plugin_thread::session_thread_handler() {
   if (m_session_thread_error) goto end;
 
   while (!m_session_thread_terminate) {
-    DBUG_EXECUTE_IF("group_replication_session_plugin_handler_before_pop", {
-      st_session_method *m = nullptr;
-      this->incoming_methods->front(&m);
-      const char act[] =
-          "now signal "
-          "signal.group_replication_session_plugin_handler_before_pop_"
-          "reached "
-          "wait_for "
-          "signal.group_replication_session_plugin_handler_before_pop_"
-          "continue";
-      assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
-    });
-
     this->incoming_methods->pop(&method);
 
     if (method->terminated) {
