@@ -1,16 +1,15 @@
-/* Copyright (c) 2015, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -32,8 +31,6 @@
 #include "my_dbug.h"
 #include "my_systime.h"  // my_sleep()
 
-struct CHARSET_INFO;
-
 static SERVICE_TYPE_NO_CONST(mysql_admin_session) * admin_session_factory;
 
 /* Sql_service_interface constructor */
@@ -75,22 +72,16 @@ int Sql_service_interface::open_session() {
     return 1; /* purecov: inspected */
   }
 
-  if (configure_session()) {
-    srv_session_close(m_session);
-    m_session = nullptr;
-    return 1;
-  }
-
   return 0;
 }
 
 int Sql_service_interface::open_thread_session(void *plugin_ptr) {
-  assert(plugin_ptr != nullptr);
+  DBUG_ASSERT(plugin_ptr != nullptr);
 
   m_session = nullptr;
   /* open a server session after server is in operating state */
   if (!wait_for_session_server(SESSION_WAIT_TIMEOUT)) {
-    /* initialize new thread to be used with server session */
+    /* initalize new thread to be used with server session */
     if (srv_session_init_thread(plugin_ptr)) {
       /* purecov: begin inspected */
       LogPluginErr(ERROR_LEVEL,
@@ -108,24 +99,10 @@ int Sql_service_interface::open_thread_session(void *plugin_ptr) {
     return 1; /* purecov: inspected */
   }
 
-  if (configure_session()) {
-    srv_session_close(m_session);
-    m_session = nullptr;
-    srv_session_deinit_thread();
-    return 1;
-  }
-
   m_plugin = plugin_ptr;
   return 0;
 }
 
-long Sql_service_interface::configure_session() {
-  DBUG_TRACE;
-  assert(m_session != nullptr);
-
-  return execute_query("SET SESSION group_replication_consistency= EVENTUAL;");
-}
-
 long Sql_service_interface::execute_internal(
     Sql_resultset *rset, enum cs_text_or_binary cs_txt_bin,
     const CHARSET_INFO *cs_charset, COM_DATA cmd,
@@ -191,11 +168,10 @@ long Sql_service_interface::execute_internal(
 
 long Sql_service_interface::execute_query(std::string sql_string) {
   DBUG_TRACE;
-  assert(sql_string.length() <= UINT_MAX);
+  DBUG_ASSERT(sql_string.length() <= UINT_MAX);
   COM_DATA cmd;
   Sql_resultset rset;
 
-  memset(&cmd, 0, sizeof(cmd));
   cmd.com_query.query = sql_string.c_str();
   cmd.com_query.length = static_cast<unsigned int>(sql_string.length());
 
@@ -209,9 +185,8 @@ long Sql_service_interface::execute_query(std::string sql_string,
                                           enum cs_text_or_binary cs_txt_or_bin,
                                           const CHARSET_INFO *cs_charset) {
   DBUG_TRACE;
-  assert(sql_string.length() <= UINT_MAX);
+  DBUG_ASSERT(sql_string.length() <= UINT_MAX);
   COM_DATA cmd;
-  memset(&cmd, 0, sizeof(cmd));
   cmd.com_query.query = sql_string.c_str();
   cmd.com_query.length = static_cast<unsigned int>(sql_string.length());
 
