@@ -1,16 +1,15 @@
-/* Copyright (c) 2016, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,12 +22,12 @@
 
 #include "plugin/keyring/common/keys_container.h"
 
-#include <assert.h>
 #include <stddef.h>
 #include <algorithm>
 
 #include <mysqld_error.h>
-#include "sql/mysqld_cs.h"
+#include "my_dbug.h"
+#include "plugin/keyring/common/system_keys_container.h"
 
 using std::string;
 using std::unique_ptr;
@@ -52,6 +51,7 @@ bool Keys_container::init(IKeyring_io *keyring_io,
   this->keyring_io = keyring_io;
   this->keyring_storage_url = keyring_storage_url;
   keys_hash->clear();
+  system_keys_container.reset(new System_keys_container(logger));
   if (keyring_io->init(&this->keyring_storage_url) ||
       load_keys_from_keyring_storage()) {
     keys_hash->clear();
@@ -71,13 +71,13 @@ std::string Keys_container::get_keyring_storage_url() {
 
 void Keys_container::store_keys_metadata(IKey *key) {
   /* if key metadata not present store it */
-  const Key_metadata km(key->get_key_id(), key->get_user_id());
+  Key_metadata km(key->get_key_id(), key->get_user_id());
   keys_metadata.push_back(km);
 }
 
 bool Keys_container::store_key_in_hash(IKey *key) {
   // TODO: This can be written more succinctly with C++17's try_emplace.
-  const string signature = *key->get_key_signature();
+  string signature = *key->get_key_signature();
   if (keys_hash->count(signature) != 0)
     return true;
   else {
@@ -88,16 +88,22 @@ bool Keys_container::store_key_in_hash(IKey *key) {
 }
 
 bool Keys_container::store_key(IKey *key) {
-  if (flush_to_backup() || store_key_in_hash(key)) return true;
+  if (system_keys_container->rotate_key_id_if_system_key_without_version(key) ||
+      flush_to_backup() || store_key_in_hash(key))
+    return true;
   if (flush_to_storage(key, STORE_KEY)) {
     remove_key_from_hash(key);
     return true;
   }
+  system_keys_container->store_or_update_if_system_key_with_version(key);
   return false;
 }
 
 IKey *Keys_container::get_key_from_hash(IKey *key) {
-  return find_or_nullptr(*keys_hash, *key->get_key_signature());
+  IKey *system_key =
+      system_keys_container->get_latest_key_if_system_key_without_version(key);
+  return system_key ? system_key
+                    : find_or_nullptr(*keys_hash, *key->get_key_signature());
 }
 
 void Keys_container::allocate_and_set_data_for_key(
@@ -110,8 +116,8 @@ void Keys_container::allocate_and_set_data_for_key(
 }
 
 IKey *Keys_container::fetch_key(IKey *key) {
-  assert(key->get_key_data() == nullptr);
-  assert(key->get_key_type_as_string()->empty());
+  DBUG_ASSERT(key->get_key_data() == nullptr);
+  DBUG_ASSERT(key->get_key_type_as_string()->empty());
 
   IKey *fetched_key = get_key_from_hash(key);
 
@@ -126,7 +132,7 @@ IKey *Keys_container::fetch_key(IKey *key) {
 }
 
 bool Keys_container::remove_keys_metadata(IKey *key) {
-  const Key_metadata src(key->get_key_id(), key->get_user_id());
+  Key_metadata src(key->get_key_id(), key->get_user_id());
   auto it =
       std::find_if(keys_metadata.begin(), keys_metadata.end(),
                    [src](Key_metadata const &dest) {
@@ -150,8 +156,10 @@ bool Keys_container::remove_key_from_hash(IKey *key) {
 
 bool Keys_container::remove_key(IKey *key) {
   IKey *fetched_key_to_delete = get_key_from_hash(key);
-  if (fetched_key_to_delete == nullptr || flush_to_backup() ||
-      remove_key_from_hash(fetched_key_to_delete))
+  // removing system keys is forbidden
+  if (fetched_key_to_delete == nullptr ||
+      system_keys_container->is_system_key(fetched_key_to_delete) ||
+      flush_to_backup() || remove_key_from_hash(fetched_key_to_delete))
     return true;
   if (flush_to_storage(fetched_key_to_delete, REMOVE_KEY)) {
     // reinsert the key
@@ -179,6 +187,8 @@ bool Keys_container::load_keys_from_keyring_storage() {
         delete key_loaded;
         break;
       }
+      system_keys_container->store_or_update_if_system_key_with_version(
+          key_loaded);
       key_loaded = nullptr;
     }
     delete serialized_keys;
