@@ -17,33 +17,25 @@
 #ifndef MYSQL_VAULT_CURL_H
 #define MYSQL_VAULT_CURL_H
 
-#include <my_global.h>
-#include <boost/core/noncopyable.hpp>
 #include <curl/curl.h>
+#include <boost/core/noncopyable.hpp>
 #include <sstream>
-#include "vault_key.h"
 #include "i_vault_curl.h"
-#include "logger.h"
+#include "plugin/keyring/common/i_keyring_key.h"
+#include "plugin/keyring/common/logger.h"
+#include "plugin/keyring/common/secure_string.h"
 #include "vault_credentials.h"
-#include "secure_string.h"
-#include "i_keyring_key.h"
+#include "vault_key.h"
 
-namespace keyring
-{
+namespace keyring {
 
-class Vault_curl : public IVault_curl, private boost::noncopyable
-{
-public:
-  Vault_curl(ILogger *logger, uint timeout)
-    : logger(logger)
-    , list(NULL)
-    , timeout(timeout)
-  {}
+class Vault_curl final : public IVault_curl, private boost::noncopyable {
+ public:
+  Vault_curl(ILogger *logger, uint timeout) noexcept
+      : logger(logger), list(nullptr), timeout(timeout) {}
 
-  ~Vault_curl()
-  {
-    if (list != NULL)
-      curl_slist_free_all(list);
+  ~Vault_curl() {
+    if (list != nullptr) curl_slist_free_all(list);
   }
 
   virtual bool init(const Vault_credentials &vault_credentials);
@@ -51,28 +43,25 @@ public:
   virtual bool write_key(const Vault_key &key, Secure_string *response);
   virtual bool read_key(const Vault_key &key, Secure_string *response);
   virtual bool delete_key(const Vault_key &key, Secure_string *response);
-  virtual void set_timeout(uint timeout)
-  {
-    this->timeout = timeout; 
-  }
-
-private:
+  virtual void set_timeout(uint timeout) noexcept { this->timeout = timeout; }
 
+ private:
   bool setup_curl_session(CURL *curl);
   std::string get_error_from_curl(CURLcode curl_code);
-  bool encode_key_signature(const Vault_key &key, Secure_string *encoded_key_signature);
+  bool encode_key_signature(const Vault_key &key,
+                            Secure_string *encoded_key_signature);
   bool get_key_url(const Vault_key &key, Secure_string *key_url);
 
   ILogger *logger;
   Secure_string token_header;
   Secure_string vault_url;
-  char curl_errbuf[CURL_ERROR_SIZE]; //error from CURL
+  char curl_errbuf[CURL_ERROR_SIZE];  // error from CURL
   Secure_ostringstream read_data_ss;
   struct curl_slist *list;
   Secure_string vault_ca;
   uint timeout;
 };
 
-} //namespace keyring
+}  // namespace keyring
 
-#endif //MYSQL_VAULT_CURL_H
+#endif  // MYSQL_VAULT_CURL_H
