@@ -1,16 +1,15 @@
-/* Copyright (c) 2014, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -37,15 +36,12 @@
 #include <mysqld_error.h>
 #include <sys/types.h>
 #include "plugin/replication_observers_example/gr_message_service_example.h"
-#include "plugin/replication_observers_example/src/binlog/service/iterator/tests/pfs.h"
-#include "plugin/replication_observers_example/src/binlog/service/iterator/tests/status_vars.h"
 
-#include <mysql/components/services/ongoing_transaction_query_service.h>
+#include <include/mysql/components/services/ongoing_transaction_query_service.h>
 #include "my_dbug.h"
 #include "my_inttypes.h"
 #include "sql/current_thd.h"
 #include "sql/sql_class.h"
-#include "string_with_len.h"
 
 static MYSQL_PLUGIN plugin_info_ptr;
 static SERVICE_TYPE(registry) *reg_srv = nullptr;
@@ -204,7 +200,8 @@ static void dump_transaction_calls() {
 /*
   Transaction lifecycle events observers.
 */
-static int trans_before_dml(Trans_param *, int &out_val [[maybe_unused]]) {
+static int trans_before_dml(Trans_param *,
+                            int &out_val MY_ATTRIBUTE((unused))) {
   trans_before_dml_call++;
 
   DBUG_EXECUTE_IF("cause_failure_in_before_dml_hook", out_val = 1;);
@@ -235,17 +232,16 @@ typedef enum enum_before_commit_test_cases {
   INVALID_CERTIFICATION_OUTCOME
 } before_commit_test_cases;
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
 static int before_commit_tests(Trans_param *param,
                                before_commit_test_cases test_case) {
-  mysql::gtid::Tsid fake_tsid;
+  rpl_sid fake_sid;
   rpl_sidno fake_sidno;
   rpl_gno fake_gno;
 
   Transaction_termination_ctx transaction_termination_ctx;
   memset(&transaction_termination_ctx, 0, sizeof(transaction_termination_ctx));
   transaction_termination_ctx.m_thread_id = param->thread_id;
-  [[maybe_unused]] std::size_t tsid_chars = 0;
 
   switch (test_case) {
     case NEGATIVE_CERTIFICATION:
@@ -256,9 +252,9 @@ static int before_commit_tests(Trans_param *param,
       break;
 
     case POSITIVE_CERTIFICATION_WITH_GTID:
-      std::ignore =
-          fake_tsid.from_cstring("aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa");
-      fake_sidno = get_sidno_from_global_tsid_map(fake_tsid);
+      fake_sid.parse("aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa",
+                     binary_log::Uuid::TEXT_LENGTH);
+      fake_sidno = get_sidno_from_global_sid_map(fake_sid);
       fake_gno = get_last_executed_gno(fake_sidno);
       fake_gno++;
 
@@ -297,7 +293,7 @@ static int before_commit_tests(Trans_param *param,
 }
 #endif
 
-static int trans_before_commit(Trans_param *param [[maybe_unused]]) {
+static int trans_before_commit(Trans_param *param MY_ATTRIBUTE((unused))) {
   trans_before_commit_call++;
 
   DBUG_EXECUTE_IF("force_error_on_before_commit_listener", return 1;);
@@ -327,9 +323,9 @@ static int trans_before_rollback(Trans_param *) {
 }
 
 static int trans_after_commit(Trans_param *) {
-  DBUG_EXECUTE_IF("bgc_after_after_commit_stage", {
+  DBUG_EXECUTE_IF("group_replication_before_commit_hook_wait", {
     const char act[] = "now wait_for continue_commit";
-    assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
+    DBUG_ASSERT(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
   });
 
   trans_after_commit_call++;
@@ -337,7 +333,7 @@ static int trans_after_commit(Trans_param *) {
   return 0;
 }
 
-static int trans_after_rollback(Trans_param *param [[maybe_unused]]) {
+static int trans_after_rollback(Trans_param *param MY_ATTRIBUTE((unused))) {
   trans_after_rollback_call++;
 
   DBUG_EXECUTE_IF("validate_replication_observers_plugin_server_requirements",
@@ -346,8 +342,8 @@ static int trans_after_rollback(Trans_param *param [[maybe_unused]]) {
   return 0;
 }
 
-static int trans_begin(Trans_param *param [[maybe_unused]],
-                       int &out_val [[maybe_unused]]) {
+static int trans_begin(Trans_param *param MY_ATTRIBUTE((unused)),
+                       int &out_val MY_ATTRIBUTE((unused))) {
   trans_begin_call++;
 
   return 0;
@@ -502,15 +498,14 @@ int validate_plugin_server_requirements(Trans_param *param) {
   /*
     Instantiate a Gtid_log_event without a THD parameter.
   */
-  mysql::gtid::Tsid fake_tsid;
-  std::ignore = fake_tsid.from_cstring("aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa");
-  const rpl_sidno fake_sidno = get_sidno_from_global_tsid_map(fake_tsid);
-  const rpl_gno fake_gno = get_last_executed_gno(fake_sidno) + 1;
+  rpl_sid fake_sid;
+  fake_sid.parse("aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa",
+                 binary_log::Uuid::TEXT_LENGTH);
+  rpl_sidno fake_sidno = get_sidno_from_global_sid_map(fake_sid);
+  rpl_gno fake_gno = get_last_executed_gno(fake_sidno) + 1;
 
-  const Gtid gtid = {fake_sidno, fake_gno};
-  mysql::gtid::Tag_plain empty_tag;
-  empty_tag.clear();
-  Gtid_specification gtid_spec = {ASSIGNED_GTID, gtid, empty_tag};
+  Gtid gtid = {fake_sidno, fake_gno};
+  Gtid_specification gtid_spec = {ASSIGNED_GTID, gtid};
   Gtid_log_event *gle =
       new Gtid_log_event(param->server_id, true, 0, 1, true, 0, 0, gtid_spec,
                          UNKNOWN_SERVER_VERSION, UNKNOWN_SERVER_VERSION);
@@ -527,8 +522,7 @@ int validate_plugin_server_requirements(Trans_param *param) {
   /*
     Instantiate a anonymous Gtid_log_event without a THD parameter.
   */
-  const Gtid_specification anonymous_gtid_spec = {ANONYMOUS_GTID, gtid,
-                                                  empty_tag};
+  Gtid_specification anonymous_gtid_spec = {ANONYMOUS_GTID, gtid};
   gle = new Gtid_log_event(param->server_id, true, 0, 1, true, 0, 0,
                            anonymous_gtid_spec, UNKNOWN_SERVER_VERSION,
                            UNKNOWN_SERVER_VERSION);
@@ -550,7 +544,7 @@ int validate_plugin_server_requirements(Trans_param *param) {
 
   if (tcle->is_valid()) {
     Gtid_set *snapshot_version = tcle->get_snapshot_version();
-    const size_t snapshot_version_len = snapshot_version->get_encoded_length();
+    size_t snapshot_version_len = snapshot_version->get_encoded_length();
     uchar *snapshot_version_buf =
         (uchar *)my_malloc(PSI_NOT_INSTRUMENTED, snapshot_version_len, MYF(0));
     snapshot_version->encode(snapshot_version_buf);
@@ -586,22 +580,22 @@ int validate_plugin_server_requirements(Trans_param *param) {
   my_thread_attr_t *thread_attr = get_connection_attrib();
 
   char *hostname, *uuid;
-  uint port, admin_port;
+  uint port;
   unsigned int server_version;
 
-  get_server_parameters(&hostname, &port, &uuid, &server_version, &admin_port);
+  get_server_parameters(&hostname, &port, &uuid, &server_version);
 
   Trans_context_info startup_pre_reqs;
-  get_server_startup_prerequirements(startup_pre_reqs);
+  get_server_startup_prerequirements(startup_pre_reqs, false);
 
   // check the server is initialized by checking if the default channel exists
-  const bool server_engine_ready = channel_is_active("", CHANNEL_NO_THD);
+  bool server_engine_ready = channel_is_active("", CHANNEL_NO_THD);
 
   uchar *encoded_gtid_executed = nullptr;
   size_t length;
   get_server_encoded_gtid_executed(&encoded_gtid_executed, &length);
 
-#if !defined(NDEBUG)
+#if !defined(DBUG_OFF)
   char *encoded_gtid_executed_string =
       encoded_gtid_set_to_string(encoded_gtid_executed, length);
 #endif
@@ -609,7 +603,7 @@ int validate_plugin_server_requirements(Trans_param *param) {
   if (thread_attr != nullptr && hostname != nullptr && uuid != nullptr &&
       port > 0 && startup_pre_reqs.gtid_mode == 3 && server_engine_ready &&
       encoded_gtid_executed != nullptr
-#if !defined(NDEBUG)
+#if !defined(DBUG_OFF)
       && encoded_gtid_executed_string != nullptr
 #endif
   )
@@ -621,7 +615,7 @@ int validate_plugin_server_requirements(Trans_param *param) {
         "requirements:"
         " failed to invoke group_replication_priv.h exported functions");
 
-#if !defined(NDEBUG)
+#if !defined(DBUG_OFF)
   my_free(encoded_gtid_executed_string);
 #endif
   my_free(encoded_gtid_executed);
@@ -638,143 +632,148 @@ int validate_plugin_server_requirements(Trans_param *param) {
 }
 
 int test_channel_service_interface_initialization() {
-  const int error = initialize_channel_service_interface();
-  assert(error);
+  int error = initialize_channel_service_interface();
+  DBUG_ASSERT(error);
   return error;
 }
 
 int test_channel_service_interface() {
   // The initialization method should return OK
   int error = initialize_channel_service_interface();
-  assert(!error);
+  DBUG_ASSERT(!error);
 
   // Test channel creation
   char interface_channel[] = "example_channel";
   Channel_creation_info info;
   initialize_channel_creation_info(&info);
   error = channel_create(interface_channel, &info);
-  assert(!error);
+  DBUG_ASSERT(!error);
 
   // Assert the channel exists
   bool exists = channel_is_active(interface_channel, CHANNEL_NO_THD);
-  assert(exists);
+  DBUG_ASSERT(exists);
 
   // Check that a non existing channel is declared as such
   char dummy_channel[] = "dummy_channel";
   exists = channel_is_active(dummy_channel, CHANNEL_NO_THD);
-  assert(!exists);
+  DBUG_ASSERT(!exists);
 
   // Test that we cannot create a empty named channel (the default channel)
   char empty_interface_channel[] = "";
   initialize_channel_creation_info(&info);
   error = channel_create(empty_interface_channel, &info);
-  assert(error == RPL_CHANNEL_SERVICE_DEFAULT_CHANNEL_CREATION_ERROR);
+  DBUG_ASSERT(error == RPL_CHANNEL_SERVICE_DEFAULT_CHANNEL_CREATION_ERROR);
 
   // Start the applier thread (since it does not need an external server)
   Channel_connection_info connection_info;
   initialize_channel_connection_info(&connection_info);
   error = channel_start(interface_channel, &connection_info,
                         CHANNEL_APPLIER_THREAD, true);
-  assert(!error);
+  DBUG_ASSERT(!error);
 
   // Assert that the applier thread is running
   bool running = channel_is_active(interface_channel, CHANNEL_APPLIER_THREAD);
-  assert(running);
+  DBUG_ASSERT(running);
 
   // Wait for execution of events (none in this case so it should return OK)
   error = channel_wait_until_apply_queue_applied(interface_channel, 100000);
-  assert(!error);
+  DBUG_ASSERT(!error);
 
   // Get the last delivered gno (should be 0)
-  mysql::gtid::Tsid fake_tsid;
-  std::ignore = fake_tsid.from_cstring("aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa");
-  const rpl_sidno fake_sidno = get_sidno_from_global_tsid_map(fake_tsid);
+  rpl_sid fake_sid;
+  fake_sid.parse("aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa",
+                 binary_log::Uuid::TEXT_LENGTH);
+  rpl_sidno fake_sidno = get_sidno_from_global_sid_map(fake_sid);
   rpl_gno gno = channel_get_last_delivered_gno(interface_channel, fake_sidno);
-  assert(gno == 0);
+  DBUG_ASSERT(gno == 0);
 
   // Check that for non existing channels it returns the corresponding error
   gno = channel_get_last_delivered_gno(dummy_channel, fake_sidno);
-  assert(gno == RPL_CHANNEL_SERVICE_CHANNEL_DOES_NOT_EXISTS_ERROR);
+  DBUG_ASSERT(gno == RPL_CHANNEL_SERVICE_CHANNEL_DOES_NOT_EXISTS_ERROR);
 
   // Extract the applier id
   long unsigned int *applier_id = nullptr;
   channel_get_thread_id(interface_channel, CHANNEL_APPLIER_THREAD, &applier_id);
-  assert(*applier_id > 0);
+  DBUG_ASSERT(*applier_id > 0);
   my_free(applier_id);
 
-  assert(binlog_relay_applier_stop_call == 0);
+  DBUG_ASSERT(binlog_relay_applier_stop_call == 0);
 
   // Stop the channel applier
   error = channel_stop(interface_channel, 3, 10000);
-  assert(!error);
+  DBUG_ASSERT(!error);
   // Repeat the stop to check it goes ok
   error = channel_stop(interface_channel, 3, 10000);
-  assert(!error);
+  DBUG_ASSERT(!error);
 
-  assert(binlog_relay_applier_stop_call > 0);
-  assert(!thread_aborted);
+  DBUG_ASSERT(binlog_relay_applier_stop_call > 0);
+  DBUG_ASSERT(!thread_aborted);
 
   // Assert that the applier thread is not running
   running = channel_is_active(interface_channel, CHANNEL_APPLIER_THREAD);
-  assert(!running);
+  DBUG_ASSERT(!running);
 
   // Purge the channel and assert all is OK
   error = channel_purge_queue(interface_channel, true);
-  assert(!error);
+  DBUG_ASSERT(!error);
 
   // Assert the channel is not there.
   exists = channel_is_active(interface_channel, CHANNEL_NO_THD);
-  assert(!exists);
+  DBUG_ASSERT(!exists);
 
   // Check that a queue in an empty channel will fail.
   char empty_event[] = "";
   error = channel_queue_packet(dummy_channel, empty_event, 0);
-  assert(error);
+  DBUG_ASSERT(error);
 
   // Test a multi thread channel
   info.channel_mts_parallel_type = CHANNEL_MTS_PARALLEL_TYPE_LOGICAL_CLOCK;
   info.channel_mts_parallel_workers = 3;
 
   error = channel_create(interface_channel, &info);
-  assert(!error);
+  DBUG_ASSERT(!error);
 
   // Assert the channel exists
   exists = channel_is_active(interface_channel, CHANNEL_NO_THD);
-  assert(exists);
+  DBUG_ASSERT(exists);
 
   error = channel_start(interface_channel, &connection_info,
                         CHANNEL_APPLIER_THREAD, true);
-  assert(!error);
+  DBUG_ASSERT(!error);
 
   // Extract the applier ids
   applier_id = nullptr;
-  const int num_appliers = channel_get_thread_id(
-      interface_channel, CHANNEL_APPLIER_THREAD, &applier_id);
-  assert(num_appliers == 4);
+  int num_appliers = channel_get_thread_id(interface_channel,
+                                           CHANNEL_APPLIER_THREAD, &applier_id);
+  DBUG_ASSERT(num_appliers == 4);
 
   unsigned long thread_id = 0;
   for (int i = 0; i < num_appliers; i++) {
     thread_id = applier_id[i];
-    assert(thread_id > 0);
+    DBUG_ASSERT(thread_id > 0);
   }
   my_free(applier_id);
 
   // Stop the channel applier
   error = channel_stop(interface_channel, 3, 10000);
-  assert(!error);
+  DBUG_ASSERT(!error);
 
   // Purge the channel and assert all is OK
   error = channel_purge_queue(interface_channel, true);
-  assert(!error);
+  DBUG_ASSERT(!error);
 
   // Assert the channel is not there.
   exists = channel_is_active(interface_channel, CHANNEL_NO_THD);
-  assert(!exists);
+  DBUG_ASSERT(!exists);
 
   // Test the method to extract credentials - first a non existing channel
-  std::string username, password;
-  error = channel_get_credentials(dummy_channel, username, password);
-  assert(error == RPL_CHANNEL_SERVICE_CHANNEL_DOES_NOT_EXISTS_ERROR);
+  const char *user_arg = nullptr;
+  char user_pass[MAX_PASSWORD_LENGTH + 1];
+  char *user_pass_pointer = user_pass;
+  size_t password_size = sizeof(user_pass);
+  error = channel_get_credentials(dummy_channel, &user_arg, &user_pass_pointer,
+                                  &password_size);
+  DBUG_ASSERT(error == RPL_CHANNEL_SERVICE_CHANNEL_DOES_NOT_EXISTS_ERROR);
 
   // Now get channel credentials, after setting them
 
@@ -784,12 +783,13 @@ int test_channel_service_interface() {
   info.user = dummy_user;
   info.password = dummy_pass;
   error = channel_create(interface_channel, &info);
-  assert(!error);
+  DBUG_ASSERT(!error);
 
-  error = channel_get_credentials(interface_channel, username, password);
-  assert(!error);
-  assert(strcmp(dummy_user, username.c_str()) == 0);
-  assert(strcmp(dummy_pass, password.c_str()) == 0);
+  error = channel_get_credentials(interface_channel, &user_arg,
+                                  &user_pass_pointer, &password_size);
+  DBUG_ASSERT(!error);
+  DBUG_ASSERT(strcmp(dummy_user, user_arg) == 0);
+  DBUG_ASSERT(strcmp(dummy_pass, user_pass_pointer) == 0);
 
   return (error && exists && running && gno && num_appliers && thread_id);
 }
@@ -797,60 +797,60 @@ int test_channel_service_interface() {
 int test_channel_service_interface_io_thread() {
   // The initialization method should return OK
   int error = initialize_channel_service_interface();
-  assert(!error);
+  DBUG_ASSERT(!error);
 
   char interface_channel[] = "example_channel";
 
   // Assert the channel exists
-  const bool exists = channel_is_active(interface_channel, CHANNEL_NO_THD);
-  assert(exists);
+  bool exists = channel_is_active(interface_channel, CHANNEL_NO_THD);
+  DBUG_ASSERT(exists);
 
   // Assert that the receiver is running
   bool running = channel_is_active(interface_channel, CHANNEL_RECEIVER_THREAD);
-  assert(running);
+  DBUG_ASSERT(running);
 
   // Extract the receiver id
   long unsigned int *thread_id = nullptr;
-  const int num_threads = channel_get_thread_id(
-      interface_channel, CHANNEL_RECEIVER_THREAD, &thread_id);
-  assert(num_threads == 1);
-  assert(*thread_id > 0);
+  int num_threads = channel_get_thread_id(interface_channel,
+                                          CHANNEL_RECEIVER_THREAD, &thread_id);
+  DBUG_ASSERT(num_threads == 1);
+  DBUG_ASSERT(*thread_id > 0);
   my_free(thread_id);
 
   // Get the I/O thread retrieved GTID set
   char *retrieved_gtid_set;
   error =
       channel_get_retrieved_gtid_set(interface_channel, &retrieved_gtid_set);
-  assert(!error);
-  assert(strlen(retrieved_gtid_set) > 0);
+  DBUG_ASSERT(!error);
+  DBUG_ASSERT(strlen(retrieved_gtid_set) > 0);
   my_free(retrieved_gtid_set);
 
   // Check that the applier thread is waiting for events to be queued.
-  const int is_waiting = channel_is_applier_waiting(interface_channel);
-  assert(is_waiting == 1);
+  int is_waiting = channel_is_applier_waiting(interface_channel);
+  DBUG_ASSERT(is_waiting == 1);
 
   // Stop the channel
   error = channel_stop(interface_channel, 3, 10000);
-  assert(!error);
+  DBUG_ASSERT(!error);
 
   // Assert that the receiver thread is not running
   running = channel_is_active(interface_channel, CHANNEL_RECEIVER_THREAD);
-  assert(!running);
+  DBUG_ASSERT(!running);
 
   return (error && exists && running && num_threads && is_waiting);
 }
 
 bool test_channel_service_interface_is_io_stopping() {
   // The initialization method should return OK
-  bool error = initialize_channel_service_interface();
-  assert(!error);
+  int error = initialize_channel_service_interface();
+  DBUG_ASSERT(!error);
 
   // Initialize the channel to be used with the channel service interface
   char interface_channel[] = "example_channel";
   Channel_creation_info info;
   initialize_channel_creation_info(&info);
   error = channel_create(interface_channel, &info);
-  assert(!error);
+  DBUG_ASSERT(!error);
 
   // Reset the I/O stop counter
   binlog_relay_thread_stop_call = 0;
@@ -858,140 +858,140 @@ bool test_channel_service_interface_is_io_stopping() {
   // Unregister the thread stop hook
   error = unregister_binlog_relay_io_observer(&relay_io_observer,
                                               (void *)plugin_info_ptr);
-  assert(!error);
+  DBUG_ASSERT(!error);
 
   // Start the I/O thread
   Channel_connection_info connection_info;
   initialize_channel_connection_info(&connection_info);
   error = channel_start(interface_channel, &connection_info,
                         CHANNEL_RECEIVER_THREAD, true);
-  assert(!error);
+  DBUG_ASSERT(!error);
 
   // Assert the channel exists
-  const bool exists = channel_is_active(interface_channel, CHANNEL_NO_THD);
-  assert(exists);
+  bool exists = channel_is_active(interface_channel, CHANNEL_NO_THD);
+  DBUG_ASSERT(exists);
 
   // Wait until I/O thread reached the error and is going to stop
   DBUG_EXECUTE_IF("pause_after_io_thread_stop_hook", {
     const char act[] =
         "now "
         "WAIT_FOR reached_stopping_io_thread";
-    assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
+    DBUG_ASSERT(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
   };);
 
   // Register the thread stop hook again
   error = register_binlog_relay_io_observer(&relay_io_observer,
                                             (void *)plugin_info_ptr);
-  assert(!error);
+  DBUG_ASSERT(!error);
 
   // Assert that the receiver is stopping
-  const bool io_stopping =
+  bool io_stopping =
       channel_is_stopping(interface_channel, CHANNEL_RECEIVER_THREAD);
-  assert(io_stopping);
+  DBUG_ASSERT(io_stopping);
 
   // Assert that the receiver is running
-  const bool io_running =
+  bool io_running =
       channel_is_active(interface_channel, CHANNEL_RECEIVER_THREAD);
-  assert(io_running);
+  DBUG_ASSERT(io_running);
 
   // Signal to make the MTR test case to start monitoring the I/O thread
   DBUG_EXECUTE_IF("pause_after_io_thread_stop_hook", {
     const char act[] =
         "now "
         "SIGNAL reached_io_thread_started";
-    assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
+    DBUG_ASSERT(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
   };);
 
   DBUG_EXECUTE_IF("pause_after_io_thread_stop_hook", {
     const char act[] = "now SIGNAL continue_to_stop_io_thread";
-    assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
+    DBUG_ASSERT(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
   };);
 
   // The plug-in has missed the stop
-  assert(binlog_relay_thread_stop_call == 0);
+  DBUG_ASSERT(binlog_relay_thread_stop_call == 0);
 
   return (error | exists | io_stopping | io_running);
 }
 
 bool test_channel_service_interface_is_sql_stopping() {
   // The initialization method should return OK
-  bool error = initialize_channel_service_interface();
-  assert(!error);
+  int error = initialize_channel_service_interface();
+  DBUG_ASSERT(!error);
 
   // Initialize the channel to be used with the channel service interface
   char interface_channel[] = "example_channel";
   Channel_creation_info info;
   initialize_channel_creation_info(&info);
   error = channel_create(interface_channel, &info);
-  assert(!error);
+  DBUG_ASSERT(!error);
 
   // Assert the channel exists
-  const bool exists = channel_is_active(interface_channel, CHANNEL_NO_THD);
-  assert(exists);
+  bool exists = channel_is_active(interface_channel, CHANNEL_NO_THD);
+  DBUG_ASSERT(exists);
 
   // Unregister the thread stop hook
   error = unregister_binlog_relay_io_observer(&relay_io_observer,
                                               (void *)plugin_info_ptr);
-  assert(!error);
+  DBUG_ASSERT(!error);
 
   // Start the I/O thread
   Channel_connection_info connection_info;
   initialize_channel_connection_info(&connection_info);
   error = channel_start(interface_channel, &connection_info,
                         CHANNEL_RECEIVER_THREAD, true);
-  assert(!error);
+  DBUG_ASSERT(!error);
 
   // Start the SQL thread
   error = channel_start(interface_channel, &connection_info,
                         CHANNEL_APPLIER_THREAD, true);
-  assert(!error);
+  DBUG_ASSERT(!error);
 
   // Wait until SQL thread reached the error and is going to stop
   DBUG_EXECUTE_IF("pause_after_sql_thread_stop_hook", {
     const char act[] =
         "now "
         "WAIT_FOR reached_stopping_sql_thread";
-    assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
+    DBUG_ASSERT(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
   };);
 
   // Register the thread stop hook again
   error = register_binlog_relay_io_observer(&relay_io_observer,
                                             (void *)plugin_info_ptr);
-  assert(!error);
+  DBUG_ASSERT(!error);
 
   // Assert that the applier is stopping
-  const bool sql_stopping =
+  bool sql_stopping =
       channel_is_stopping(interface_channel, CHANNEL_APPLIER_THREAD);
-  assert(sql_stopping);
+  DBUG_ASSERT(sql_stopping);
 
   // Assert that the applier is running
-  const bool sql_running =
+  bool sql_running =
       channel_is_active(interface_channel, CHANNEL_APPLIER_THREAD);
-  assert(sql_running);
+  DBUG_ASSERT(sql_running);
 
   // Signal to make the MTR test case to start monitoring the SQL thread
   DBUG_EXECUTE_IF("pause_after_sql_thread_stop_hook", {
     const char act[] =
         "now "
         "SIGNAL reached_sql_thread_started";
-    assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
+    DBUG_ASSERT(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
   };);
 
   DBUG_EXECUTE_IF("pause_after_sql_thread_stop_hook", {
     const char act[] = "now SIGNAL continue_to_stop_sql_thread";
-    assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
+    DBUG_ASSERT(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
   };);
 
   // The plug-in has missed the stop
-  assert(binlog_relay_applier_stop_call == 0);
+  DBUG_ASSERT(binlog_relay_applier_stop_call == 0);
 
   return (error | exists | sql_stopping | sql_running);
 }
 
 bool test_channel_service_interface_relay_log_renamed() {
   // The initialization method should return OK
-  bool error = initialize_channel_service_interface();
-  assert(!error);
+  int error = initialize_channel_service_interface();
+  DBUG_ASSERT(!error);
 
   // Initialize the channel to be used with the channel service interface
   char interface_channel[] = "example_channel";
@@ -1003,11 +1003,11 @@ bool test_channel_service_interface_relay_log_renamed() {
   info.hostname = channel_hostname;
   info.user = channel_user;
   error = channel_create(interface_channel, &info);
-  assert(!error);
+  DBUG_ASSERT(!error);
 
   // Assert the channel exists
-  const bool exists = channel_is_active(interface_channel, CHANNEL_NO_THD);
-  assert(exists);
+  bool exists = channel_is_active(interface_channel, CHANNEL_NO_THD);
+  DBUG_ASSERT(exists);
 
   // Start the SQL thread
   Channel_connection_info connection_info;
@@ -1018,14 +1018,14 @@ bool test_channel_service_interface_relay_log_renamed() {
   if (error) {
     THD *thd = current_thd;
     thd->clear_error();
-#if !defined(NDEBUG)
+#if !defined(DBUG_OFF)
     const char act[] = "now SIGNAL reached_sql_thread_startup_failed";
-    assert(!debug_sync_set_action(thd, STRING_WITH_LEN(act)));
+    DBUG_ASSERT(!debug_sync_set_action(thd, STRING_WITH_LEN(act)));
 #endif
   } else {
-#if !defined(NDEBUG)
+#if !defined(DBUG_OFF)
     const char act[] = "now SIGNAL reached_sql_thread_started";
-    assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
+    DBUG_ASSERT(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
 #endif
   }
 
@@ -1037,14 +1037,15 @@ bool test_server_count_transactions() {
   my_service<SERVICE_TYPE(mysql_ongoing_transactions_query)> service(
       "mysql_ongoing_transactions_query", reg_srv);
 
-  assert(service.is_valid());
+  DBUG_ASSERT(service.is_valid());
 
-  unsigned long *ids = nullptr;
+  unsigned long *ids = NULL;
   unsigned long size = 0;
   bool error = service->get_ongoing_server_transactions(&ids, &size);
-  assert(!error);
+  DBUG_ASSERT(!error);
+  fprintf(stderr, "[DEBUG:] Counting transactions! %lu \n", size);
 
-  assert(size == 3);
+  DBUG_ASSERT(size == 3);
 
   my_free(ids);
 
@@ -1105,22 +1106,6 @@ static int replication_observers_example_plugin_init(MYSQL_PLUGIN plugin_info) {
     return 1;
   }
 
-  if (binlog::service::iterators::tests::register_pfs_tables()) {
-    /* purecov: begin inspected */
-    LogPluginErr(ERROR_LEVEL, ER_LOG_PRINTF_MSG, "Failure on init PFS tables");
-    deinit_logging_service_for_plugin(&reg_srv, &log_bi, &log_bs);
-    return 1;
-    /* purecov: end */
-  }
-
-  if (binlog::service::iterators::tests::register_status_variables()) {
-    /* purecov: begin inspected */
-    LogPluginErr(ERROR_LEVEL, ER_LOG_PRINTF_MSG, "Failure on init STATS VARS");
-    deinit_logging_service_for_plugin(&reg_srv, &log_bi, &log_bs);
-    return 1;
-    /* purecov: end */
-  }
-
   LogPluginErr(INFORMATION_LEVEL, ER_LOG_PRINTF_MSG,
                "replication_observers_example_plugin: init finished");
 
@@ -1150,9 +1135,6 @@ static int replication_observers_example_plugin_deinit(void *p) {
   dump_transaction_calls();
   dump_binlog_relay_calls();
 
-  binlog::service::iterators::tests::unregister_status_variables();
-  binlog::service::iterators::tests::unregister_pfs_tables();
-
   if (unregister_server_state_observer(&server_state_observer, p)) {
     LogPluginErr(ERROR_LEVEL, ER_LOG_PRINTF_MSG,
                  "Failure in unregistering the server state observers");
