@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -33,7 +33,7 @@
 #include "my_io.h"
 #include "my_psi_config.h"
 #include "my_sys.h"  // my_write, my_malloc
-#include "mysql/components/services/bits/psi_thread_bits.h"
+#include "mysql/components/services/psi_thread_bits.h"
 #include "mysql/psi/mysql_thread.h"
 #include "mysql_com.h"
 #include "sql_string.h" /* STRING_PSI_MEMORY_KEY */
@@ -44,8 +44,6 @@
 #include <mysql/components/services/log_builtins.h>
 #include <mysqld_error.h>
 
-struct CHARSET_INFO;
-
 static const char *log_filename = "test_session_info";
 
 static File outfile;
@@ -188,7 +186,7 @@ static int sql_start_result_metadata(void *ctx, uint num_cols, uint,
   DBUG_TRACE;
   DBUG_PRINT("info", ("resultcs->number: %d", resultcs->number));
   DBUG_PRINT("info", ("resultcs->csname: %s", resultcs->csname));
-  DBUG_PRINT("info", ("resultcs->name: %s", resultcs->m_coll_name));
+  DBUG_PRINT("info", ("resultcs->name: %s", resultcs->name));
   pctx->num_cols = num_cols;
   pctx->resultcs = resultcs;
   pctx->current_col = 0;
@@ -264,8 +262,8 @@ static ulong sql_get_client_capabilities(void *) {
 static int sql_get_null(void *ctx) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   DBUG_TRACE;
-  const uint row = pctx->num_rows;
-  const uint col = pctx->current_col;
+  uint row = pctx->num_rows;
+  uint col = pctx->current_col;
   pctx->current_col++;
 
   memcpy(pctx->sql_str_value[row][col], "[NULL]", sizeof("[NULL]"));
@@ -277,13 +275,12 @@ static int sql_get_null(void *ctx) {
 static int sql_get_integer(void *ctx, longlong value) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   DBUG_TRACE;
-  const uint row = pctx->num_rows;
-  const uint col = pctx->current_col;
+  uint row = pctx->num_rows;
+  uint col = pctx->current_col;
   pctx->current_col++;
 
-  const size_t len =
-      snprintf(pctx->sql_str_value[row][col],
-               sizeof(pctx->sql_str_value[row][col]), "%lld", value);
+  size_t len = snprintf(pctx->sql_str_value[row][col],
+                        sizeof(pctx->sql_str_value[row][col]), "%lld", value);
   pctx->sql_str_len[row][col] = len;
   pctx->sql_int_value[row][col] = value;
 
@@ -293,13 +290,13 @@ static int sql_get_integer(void *ctx, longlong value) {
 static int sql_get_longlong(void *ctx, longlong value, uint is_unsigned) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   DBUG_TRACE;
-  const uint row = pctx->num_rows;
-  const uint col = pctx->current_col;
+  uint row = pctx->num_rows;
+  uint col = pctx->current_col;
   pctx->current_col++;
 
-  const size_t len = snprintf(pctx->sql_str_value[row][col],
-                              sizeof(pctx->sql_str_value[row][col]),
-                              is_unsigned ? "%llu" : "%lld", value);
+  size_t len = snprintf(pctx->sql_str_value[row][col],
+                        sizeof(pctx->sql_str_value[row][col]),
+                        is_unsigned ? "%llu" : "%lld", value);
 
   pctx->sql_str_len[row][col] = len;
   pctx->sql_longlong_value[row][col] = value;
@@ -311,14 +308,14 @@ static int sql_get_longlong(void *ctx, longlong value, uint is_unsigned) {
 static int sql_get_decimal(void *ctx, const decimal_t *value) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   DBUG_TRACE;
-  const uint row = pctx->num_rows;
-  const uint col = pctx->current_col;
+  uint row = pctx->num_rows;
+  uint col = pctx->current_col;
   pctx->current_col++;
 
-  const size_t len = snprintf(
-      pctx->sql_str_value[row][col], sizeof(pctx->sql_str_value[row][col]),
-      "%s%d.%d(%d)[%s]", value->sign ? "+" : "-", value->intg, value->frac,
-      value->len, (char *)value->buf);
+  size_t len = snprintf(pctx->sql_str_value[row][col],
+                        sizeof(pctx->sql_str_value[row][col]),
+                        "%s%d.%d(%d)[%s]", value->sign ? "+" : "-", value->intg,
+                        value->frac, value->len, (char *)value->buf);
   pctx->sql_str_len[row][col] = len;
   pctx->sql_decimal_value[row][col].intg = value->intg;
   pctx->sql_decimal_value[row][col].frac = value->frac;
@@ -334,13 +331,12 @@ static int sql_get_decimal(void *ctx, const decimal_t *value) {
 static int sql_get_double(void *ctx, double value, uint32 decimals) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   DBUG_TRACE;
-  const uint row = pctx->num_rows;
-  const uint col = pctx->current_col;
+  uint row = pctx->num_rows;
+  uint col = pctx->current_col;
   pctx->current_col++;
 
-  const size_t len =
-      snprintf(pctx->sql_str_value[row][col],
-               sizeof(pctx->sql_str_value[row][col]), "%3.7g", value);
+  size_t len = snprintf(pctx->sql_str_value[row][col],
+                        sizeof(pctx->sql_str_value[row][col]), "%3.7g", value);
 
   pctx->sql_str_len[row][col] = len;
 
@@ -353,11 +349,11 @@ static int sql_get_double(void *ctx, double value, uint32 decimals) {
 static int sql_get_date(void *ctx, const MYSQL_TIME *value) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   DBUG_TRACE;
-  const uint row = pctx->num_rows;
-  const uint col = pctx->current_col;
+  uint row = pctx->num_rows;
+  uint col = pctx->current_col;
   pctx->current_col++;
 
-  const size_t len =
+  size_t len =
       snprintf(pctx->sql_str_value[row][col],
                sizeof(pctx->sql_str_value[row][col]), "%s%4d-%02d-%02d",
                value->neg ? "-" : "", value->year, value->month, value->day);
@@ -379,11 +375,11 @@ static int sql_get_date(void *ctx, const MYSQL_TIME *value) {
 static int sql_get_time(void *ctx, const MYSQL_TIME *value, uint decimals) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   DBUG_TRACE;
-  const uint row = pctx->num_rows;
-  const uint col = pctx->current_col;
+  uint row = pctx->num_rows;
+  uint col = pctx->current_col;
   pctx->current_col++;
 
-  const size_t len = snprintf(
+  size_t len = snprintf(
       pctx->sql_str_value[row][col], sizeof(pctx->sql_str_value[row][col]),
       "%s%02d:%02d:%02d", value->neg ? "-" : "",
       value->day ? (value->day * 24 + value->hour) : value->hour, value->minute,
@@ -408,11 +404,11 @@ static int sql_get_time(void *ctx, const MYSQL_TIME *value, uint decimals) {
 static int sql_get_datetime(void *ctx, const MYSQL_TIME *value, uint decimals) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   DBUG_TRACE;
-  const uint row = pctx->num_rows;
-  const uint col = pctx->current_col;
+  uint row = pctx->num_rows;
+  uint col = pctx->current_col;
   pctx->current_col++;
 
-  const size_t len = snprintf(
+  size_t len = snprintf(
       pctx->sql_str_value[row][col], sizeof(pctx->sql_str_value[row][col]),
       "%s%4d-%02d-%02d %02d:%02d:%02d", value->neg ? "-" : "", value->year,
       value->month, value->day, value->hour, value->minute, value->second);
@@ -437,8 +433,8 @@ static int sql_get_string(void *ctx, const char *const value, size_t length,
                           const CHARSET_INFO *const) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   DBUG_TRACE;
-  const uint row = pctx->num_rows;
-  const uint col = pctx->current_col;
+  uint row = pctx->num_rows;
+  uint col = pctx->current_col;
   pctx->current_col++;
 
   strncpy(pctx->sql_str_value[row][col], value, length);
@@ -497,7 +493,6 @@ const struct st_command_service_cbs sql_cbs = {
     sql_handle_ok,
     sql_handle_error,
     sql_shutdown,
-    nullptr,
 };
 
 static void get_data_str(void *ctx) {
@@ -539,18 +534,17 @@ static void handle_error(void *ctx) {
   exec_test_cmd((s), (q), (p), (ctx), (err), __FUNCTION__, __LINE__)
 
 static void exec_test_cmd(MYSQL_SESSION session, const char *query,
-                          void *p [[maybe_unused]], void *ctx,
+                          void *p MY_ATTRIBUTE((unused)), void *ctx,
                           bool expect_error, const char *func, uint line) {
   char buffer[STRING_BUFFER_SIZE];
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   pctx->reset();
-  memset(&cmd, 0, sizeof(cmd));
   cmd.com_query.query = query;
   cmd.com_query.length = strlen(cmd.com_query.query);
   WRITE_VAL("%s\n", query);
-  const int fail = command_service_run_command(
-      session, COM_QUERY, &cmd, &my_charset_utf8mb3_general_ci, &sql_cbs,
-      CS_TEXT_REPRESENTATION, ctx);
+  int fail = command_service_run_command(session, COM_QUERY, &cmd,
+                                         &my_charset_utf8_general_ci, &sql_cbs,
+                                         CS_TEXT_REPRESENTATION, ctx);
   if (fail) {
     srv_session_close(session);
     if (!expect_error)
@@ -573,8 +567,8 @@ static void exec_test_cmd(MYSQL_SESSION session, const char *query,
   WRITE_STR("\n");
 }
 
-static void test_com_init_db(void *p [[maybe_unused]], MYSQL_SESSION st_session,
-                             const char *db_name) {
+static void test_com_init_db(void *p MY_ATTRIBUTE((unused)),
+                             MYSQL_SESSION st_session, const char *db_name) {
   char buffer[STRING_BUFFER_SIZE];
   DBUG_TRACE;
 
@@ -588,9 +582,9 @@ static void test_com_init_db(void *p [[maybe_unused]], MYSQL_SESSION st_session,
   cmd.com_init_db.db_name = db_name;
   cmd.com_init_db.length = strlen(db_name);
 
-  const int fail = command_service_run_command(
-      st_session, COM_INIT_DB, &cmd, &my_charset_utf8mb3_general_ci, &sql_cbs,
-      CS_TEXT_REPRESENTATION, plugin_ctx);
+  int fail = command_service_run_command(st_session, COM_INIT_DB, &cmd,
+                                         &my_charset_utf8_general_ci, &sql_cbs,
+                                         CS_TEXT_REPRESENTATION, plugin_ctx);
 
   if (fail) {
     LogPluginErr(ERROR_LEVEL, ER_LOG_PRINTF_MSG, "run_statement code: %d\n",
@@ -615,7 +609,7 @@ static void test_sql(void *p) {
 
   /* Opening session 1 */
   WRITE_STR("Opening Session 1\n");
-  session_1 = srv_session_open(nullptr, plugin_ctx);
+  session_1 = srv_session_open(NULL, plugin_ctx);
   if (!session_1)
     LogPluginErr(ERROR_LEVEL, ER_LOG_PRINTF_MSG, "Opening Session 1 failed");
   else
@@ -623,7 +617,7 @@ static void test_sql(void *p) {
 
   /* Opening session 2 */
   WRITE_STR("Opening Session 2\n");
-  session_2 = srv_session_open(nullptr, plugin_ctx);
+  session_2 = srv_session_open(NULL, plugin_ctx);
   if (!session_2)
     LogPluginErr(ERROR_LEVEL, ER_LOG_PRINTF_MSG, "Opening Session 2 failed");
   else
@@ -636,8 +630,8 @@ static void test_sql(void *p) {
       "Session 1 : srv_session_info_get_thd and "
       "srv_session_info_get_session_id\n");
   WRITE_SEP();
-  const my_thread_id session_1_id = srv_session_info_get_session_id(session_1);
-  const my_thread_id session_2_id = srv_session_info_get_session_id(session_2);
+  my_thread_id session_1_id = srv_session_info_get_session_id(session_1);
+  my_thread_id session_2_id = srv_session_info_get_session_id(session_2);
   MYSQL_THD thd = srv_session_info_get_thd(session_1);
   unsigned long thd_id = thd_get_thread_id(thd);
   if (thd_id != session_1_id) {
@@ -884,7 +878,6 @@ static void test_sql(void *p) {
                    "INFORMATION_SCHEMA.PROCESSLIST WHERE info LIKE 'PLUGIN%' "
                    "ORDER BY id",
                    p, plugin_ctx, true);
-  session_1 = nullptr;  // session_1 is closed in EXEC_TEST_CMD_EX.
   WRITE_SEP();
   WRITE_VAL("srv_session_info_killed(Session 1) : %d\n",
             srv_session_info_killed(session_1));
@@ -931,13 +924,12 @@ static void test_sql(void *p) {
   snprintf(buffer_query, sizeof(buffer_query), "KILL QUERY %i /*session_2_id*/",
            session_2_id);
   WRITE_VAL("%s\n", buffer_query);
-  memset(&cmd, 0, sizeof(cmd));
   cmd.com_query.query = buffer_query;
   cmd.com_query.length = strlen(buffer_query);
 
-  int fail = command_service_run_command(
-      session_2, COM_QUERY, &cmd, &my_charset_utf8mb3_general_ci, &sql_cbs,
-      CS_TEXT_REPRESENTATION, plugin_ctx);
+  int fail = command_service_run_command(session_2, COM_QUERY, &cmd,
+                                         &my_charset_utf8_general_ci, &sql_cbs,
+                                         CS_TEXT_REPRESENTATION, plugin_ctx);
 
   if (fail) {
     LogPluginErr(ERROR_LEVEL, ER_LOG_PRINTF_MSG, "run_statement code: %d\n",
@@ -952,12 +944,11 @@ static void test_sql(void *p) {
   snprintf(buffer_query, sizeof(buffer_query),
            "KILL CONNECTION %i  /*session_2_id*/", session_2_id);
   WRITE_VAL("%s\n", buffer_query);
-  memset(&cmd, 0, sizeof(cmd));
   cmd.com_query.query = buffer_query;
   cmd.com_query.length = strlen(buffer_query);
 
   fail = command_service_run_command(session_2, COM_QUERY, &cmd,
-                                     &my_charset_utf8mb3_general_ci, &sql_cbs,
+                                     &my_charset_utf8_general_ci, &sql_cbs,
                                      CS_TEXT_REPRESENTATION, plugin_ctx);
 
   if (fail) {
@@ -989,7 +980,7 @@ static void test_sql(void *p) {
       srv_session_info_set_connection_type(session_2, VIO_TYPE_SHARED_MEMORY));
   WRITE_STR("\n");
 
-  session_3 = srv_session_open(nullptr, plugin_ctx);
+  session_3 = srv_session_open(NULL, plugin_ctx);
   if (!session_3)
     LogPluginErr(ERROR_LEVEL, ER_LOG_PRINTF_MSG, "Opening Session 3 failed");
   else {
@@ -1056,8 +1047,7 @@ static void create_log_file(const char *log_name) {
 #ifdef HAVE_PSI_INTERFACE
 static PSI_thread_key key_thread_session_info = PSI_NOT_INSTRUMENTED;
 static PSI_thread_info session_info_threads[] = {
-    {&key_thread_session_info, "session_info", "session_info", 0, 0,
-     PSI_DOCUMENT_ME}};
+    {&key_thread_session_info, "session_info", 0, 0, PSI_DOCUMENT_ME}};
 #endif  // HAVE_PSI_INTERFACE
 
 static void test_in_spawned_thread(void *p, void (*test_function)(void *)) {
@@ -1108,7 +1098,7 @@ static int test_sql_service_plugin_init(void *p) {
   return 0;
 }
 
-static int test_sql_service_plugin_deinit(void *p [[maybe_unused]]) {
+static int test_sql_service_plugin_deinit(void *p MY_ATTRIBUTE((unused))) {
   DBUG_TRACE;
   LogPluginErr(INFORMATION_LEVEL, ER_LOG_PRINTF_MSG, "Uninstallation.");
   deinit_logging_service_for_plugin(&reg_srv, &log_bi, &log_bs);
