@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2022, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -431,14 +431,13 @@ static void handle_error() {
     WRITE_VAL("affected rows: %d\n", sql_affected_rows);
     WRITE_VAL("server status: %d\n", sql_server_status);
     WRITE_VAL("warn count: %d\n", sql_warn_count);
-    //           WRITE_VAL("message: %s\n",msg);
+    //           WRITE_VAL("messsage: %s\n",msg);
   }
 }
 
 static void exec_test_cmd(MYSQL_SESSION session, const char *test_cmd,
                           void *p) {
   WRITE_VAL("%s\n", test_cmd);
-  memset(&cmd, 0, sizeof(cmd));
   cmd.com_query.query = (char *)test_cmd;
   cmd.com_query.length = strlen(cmd.com_query.query);
   fail = command_service_run_command(session, select_prot, COM_QUERY, &cmd,
@@ -499,7 +498,6 @@ int test_sql(void *p) {
   if (session_ret) {
     LogPluginErr(ERROR_LEVEL, ER_LOG_PRINTF_MSG, "Close session_1 failed.");
   }
-  session_1 = nullptr;
   /* 4. statement */
   WRITE_STR(
       "-----------------------------------------------------------------\n");
@@ -550,7 +548,7 @@ static int test_sql_service_plugin_init(void *p) {
   return 0;
 }
 
-static int test_sql_service_plugin_deinit(void *p [[maybe_unused]]) {
+static int test_sql_service_plugin_deinit(void *p MY_ATTRIBUTE((unused))) {
   DBUG_TRACE;
   LogPluginErr(INFORMATION_LEVEL, ER_LOG_PRINTF_MSG, "Uninstallation.");
   deinit_logging_service_for_plugin(&reg_srv, &log_bi, &log_bs);
