@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -184,7 +184,7 @@ static int sql_start_result_metadata(void *ctx, uint num_cols, uint,
   DBUG_TRACE;
   DBUG_PRINT("info", ("resultcs->number: %d", resultcs->number));
   DBUG_PRINT("info", ("resultcs->csname: %s", resultcs->csname));
-  DBUG_PRINT("info", ("resultcs->m_coll_name: %s", resultcs->m_coll_name));
+  DBUG_PRINT("info", ("resultcs->name: %s", resultcs->name));
   pctx->num_cols = num_cols;
   pctx->resultcs = resultcs;
   pctx->current_col = 0;
@@ -491,7 +491,6 @@ const struct st_command_service_cbs sql_cbs = {
     sql_handle_ok,
     sql_handle_error,
     sql_shutdown,
-    nullptr,
 };
 
 static const char *fieldtype2str(enum enum_field_types type) {
@@ -634,17 +633,16 @@ static void handle_error(struct st_plugin_ctx *pctx) {
 }
 
 static void exec_test_cmd(MYSQL_SESSION session, const char *test_cmd,
-                          void *p [[maybe_unused]], void *ctx) {
+                          void *p MY_ATTRIBUTE((unused)), void *ctx) {
   WRITE_VAL("%s\n", test_cmd);
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   COM_DATA cmd;
   pctx->reset();
-  memset(&cmd, 0, sizeof(cmd));
   cmd.com_query.query = test_cmd;
   cmd.com_query.length = strlen(cmd.com_query.query);
   int fail = command_service_run_command(session, COM_QUERY, &cmd,
-                                         &my_charset_utf8mb3_general_ci,
-                                         &sql_cbs, CS_TEXT_REPRESENTATION, ctx);
+                                         &my_charset_utf8_general_ci, &sql_cbs,
+                                         CS_TEXT_REPRESENTATION, ctx);
   if (fail) {
     if (!srv_session_close(session))
       LogPluginErrMsg(ERROR_LEVEL, ER_LOG_PRINTF_MSG,
@@ -715,7 +713,6 @@ static void test_sql(void *p) {
   WRITE_STR("\nClose Session 1\n");
   if (srv_session_close(session_1))
     LogPluginErr(ERROR_LEVEL, ER_LOG_PRINTF_MSG, "Close session_1 failed.");
-  session_1 = nullptr;
 
   /* Statement 4 */
   WRITE_SEP();
@@ -733,7 +730,6 @@ static void test_sql(void *p) {
   WRITE_STR("\nClose Session 2\n");
   if (srv_session_close(session_2))
     LogPluginErr(ERROR_LEVEL, ER_LOG_PRINTF_MSG, "Close session_2 failed.");
-  session_2 = nullptr;
 
   /* Close session 2 */
   WRITE_STR("\nClose Session 2 again\n");
@@ -742,7 +738,7 @@ static void test_sql(void *p) {
   delete plugin_ctx;
 }
 
-static void test_session_only_open(void *p [[maybe_unused]]) {
+static void test_session_only_open(void *p MY_ATTRIBUTE((unused))) {
   DBUG_TRACE;
 
   MYSQL_SESSION sessions[MAX_SESSIONS];
@@ -839,7 +835,7 @@ static int test_session_service_plugin_init(void *p) {
   return 0;
 }
 
-static int test_session_service_plugin_deinit(void *p [[maybe_unused]]) {
+static int test_session_service_plugin_deinit(void *p MY_ATTRIBUTE((unused))) {
   DBUG_TRACE;
   LogPluginErr(INFORMATION_LEVEL, ER_LOG_PRINTF_MSG, "Uninstallation.");
   deinit_logging_service_for_plugin(&reg_srv, &log_bi, &log_bs);
