@@ -1,16 +1,15 @@
-/* Copyright (c) 2018, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2018, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -43,9 +42,7 @@
 #include "mysql_com.h"
 #include "template_utils.h"
 
-static constexpr int STRING_BUFFER = 256 * 2;
-
-struct CHARSET_INFO;
+#define STRING_BUFFER 256
 
 static File outfile;
 
@@ -149,7 +146,7 @@ static int sql_start_result_metadata(void *ctx, uint num_cols, uint,
   DBUG_TRACE;
   DBUG_PRINT("info", ("resultcs->number: %d", resultcs->number));
   DBUG_PRINT("info", ("resultcs->csname: %s", resultcs->csname));
-  DBUG_PRINT("info", ("resultcs->m_coll_name: %s", resultcs->m_coll_name));
+  DBUG_PRINT("info", ("resultcs->name: %s", resultcs->name));
   pctx->num_cols = num_cols;
   pctx->resultcs = resultcs;
   pctx->current_col = 0;
@@ -225,8 +222,8 @@ static ulong sql_get_client_capabilities(void *) {
 static int sql_get_null(void *ctx) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   DBUG_TRACE;
-  const uint row = pctx->num_rows;
-  const uint col = pctx->current_col;
+  uint row = pctx->num_rows;
+  uint col = pctx->current_col;
   pctx->current_col++;
 
   memcpy(pctx->sql_str_value[row][col], "[NULL]", sizeof("[NULL]"));
@@ -238,13 +235,12 @@ static int sql_get_null(void *ctx) {
 static int sql_get_integer(void *ctx, longlong value) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   DBUG_TRACE;
-  const uint row = pctx->num_rows;
-  const uint col = pctx->current_col;
+  uint row = pctx->num_rows;
+  uint col = pctx->current_col;
   pctx->current_col++;
 
-  const size_t len =
-      snprintf(pctx->sql_str_value[row][col],
-               sizeof(pctx->sql_str_value[row][col]), "%lld", value);
+  size_t len = snprintf(pctx->sql_str_value[row][col],
+                        sizeof(pctx->sql_str_value[row][col]), "%lld", value);
   pctx->sql_str_len[row][col] = len;
 
   return false;
@@ -253,13 +249,13 @@ static int sql_get_integer(void *ctx, longlong value) {
 static int sql_get_longlong(void *ctx, longlong value, uint is_unsigned) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   DBUG_TRACE;
-  const uint row = pctx->num_rows;
-  const uint col = pctx->current_col;
+  uint row = pctx->num_rows;
+  uint col = pctx->current_col;
   pctx->current_col++;
 
-  const size_t len = snprintf(pctx->sql_str_value[row][col],
-                              sizeof(pctx->sql_str_value[row][col]),
-                              is_unsigned ? "%llu" : "%lld", value);
+  size_t len = snprintf(pctx->sql_str_value[row][col],
+                        sizeof(pctx->sql_str_value[row][col]),
+                        is_unsigned ? "%llu" : "%lld", value);
 
   pctx->sql_str_len[row][col] = len;
 
@@ -269,14 +265,14 @@ static int sql_get_longlong(void *ctx, longlong value, uint is_unsigned) {
 static int sql_get_decimal(void *ctx, const decimal_t *value) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   DBUG_TRACE;
-  const uint row = pctx->num_rows;
-  const uint col = pctx->current_col;
+  uint row = pctx->num_rows;
+  uint col = pctx->current_col;
   pctx->current_col++;
 
-  const size_t len = snprintf(
-      pctx->sql_str_value[row][col], sizeof(pctx->sql_str_value[row][col]),
-      "%s%d.%d(%d)[%s]", value->sign ? "+" : "-", value->intg, value->frac,
-      value->len, (char *)value->buf);
+  size_t len = snprintf(pctx->sql_str_value[row][col],
+                        sizeof(pctx->sql_str_value[row][col]),
+                        "%s%d.%d(%d)[%s]", value->sign ? "+" : "-", value->intg,
+                        value->frac, value->len, (char *)value->buf);
   pctx->sql_str_len[row][col] = len;
 
   return false;
@@ -285,13 +281,12 @@ static int sql_get_decimal(void *ctx, const decimal_t *value) {
 static int sql_get_double(void *ctx, double value, uint32 /*decimals*/) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   DBUG_TRACE;
-  const uint row = pctx->num_rows;
-  const uint col = pctx->current_col;
+  uint row = pctx->num_rows;
+  uint col = pctx->current_col;
   pctx->current_col++;
 
-  const size_t len =
-      snprintf(pctx->sql_str_value[row][col],
-               sizeof(pctx->sql_str_value[row][col]), "%3.7g", value);
+  size_t len = snprintf(pctx->sql_str_value[row][col],
+                        sizeof(pctx->sql_str_value[row][col]), "%3.7g", value);
 
   pctx->sql_str_len[row][col] = len;
 
@@ -301,11 +296,11 @@ static int sql_get_double(void *ctx, double value, uint32 /*decimals*/) {
 static int sql_get_date(void *ctx, const MYSQL_TIME *value) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   DBUG_TRACE;
-  const uint row = pctx->num_rows;
-  const uint col = pctx->current_col;
+  uint row = pctx->num_rows;
+  uint col = pctx->current_col;
   pctx->current_col++;
 
-  const size_t len =
+  size_t len =
       snprintf(pctx->sql_str_value[row][col],
                sizeof(pctx->sql_str_value[row][col]), "%s%4d-%02d-%02d",
                value->neg ? "-" : "", value->year, value->month, value->day);
@@ -317,11 +312,11 @@ static int sql_get_date(void *ctx, const MYSQL_TIME *value) {
 static int sql_get_time(void *ctx, const MYSQL_TIME *value, uint /*decimals*/) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   DBUG_TRACE;
-  const uint row = pctx->num_rows;
-  const uint col = pctx->current_col;
+  uint row = pctx->num_rows;
+  uint col = pctx->current_col;
   pctx->current_col++;
 
-  const size_t len = snprintf(
+  size_t len = snprintf(
       pctx->sql_str_value[row][col], sizeof(pctx->sql_str_value[row][col]),
       "%s%02d:%02d:%02d", value->neg ? "-" : "",
       value->day ? (value->day * 24 + value->hour) : value->hour, value->minute,
@@ -336,11 +331,11 @@ static int sql_get_datetime(void *ctx, const MYSQL_TIME *value,
                             uint /*decimals*/) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   DBUG_TRACE;
-  const uint row = pctx->num_rows;
-  const uint col = pctx->current_col;
+  uint row = pctx->num_rows;
+  uint col = pctx->current_col;
   pctx->current_col++;
 
-  const size_t len = snprintf(
+  size_t len = snprintf(
       pctx->sql_str_value[row][col], sizeof(pctx->sql_str_value[row][col]),
       "%s%4d-%02d-%02d %02d:%02d:%02d", value->neg ? "-" : "", value->year,
       value->month, value->day, value->hour, value->minute, value->second);
@@ -354,8 +349,8 @@ static int sql_get_string(void *ctx, const char *const value, size_t length,
                           const CHARSET_INFO *const) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   DBUG_TRACE;
-  const uint row = pctx->num_rows;
-  const uint col = pctx->current_col;
+  uint row = pctx->num_rows;
+  uint col = pctx->current_col;
   pctx->current_col++;
 
   strncpy(pctx->sql_str_value[row][col], value, length);
@@ -414,7 +409,6 @@ const struct st_command_service_cbs sql_cbs = {
     sql_handle_ok,
     sql_handle_error,
     sql_shutdown,
-    nullptr,
 };
 
 static const char *fieldtype2str(enum enum_field_types type) {
@@ -502,11 +496,10 @@ static void query_execute(MYSQL_SESSION session, st_plugin_ctx *pctx,
   pctx->reset();
 
   COM_DATA cmd;
-  memset(&cmd, 0, sizeof(cmd));
   cmd.com_query.query = query.c_str();
   cmd.com_query.length = query.size();
   if (command_service_run_command(session, COM_QUERY, &cmd,
-                                  &my_charset_utf8mb3_general_ci, &sql_cbs,
+                                  &my_charset_utf8_general_ci, &sql_cbs,
                                   CS_TEXT_REPRESENTATION, pctx)) {
     LogPluginErr(ERROR_LEVEL, ER_LOG_PRINTF_MSG, "fail query execution - %d:%s",
                  pctx->sql_errno, pctx->err_msg);
@@ -570,7 +563,7 @@ static void ensure_api_not_null(const char *function, void *result) {
 static void reset_connection(MYSQL_SESSION st_session, st_plugin_ctx *pctx) {
   COM_DATA cmd;
   ENSURE_API_OK(command_service_run_command(
-      st_session, COM_RESET_CONNECTION, &cmd, &my_charset_utf8mb3_general_ci,
+      st_session, COM_RESET_CONNECTION, &cmd, &my_charset_utf8_general_ci,
       &sql_cbs, CS_TEXT_REPRESENTATION, pctx));
 }
 
@@ -589,7 +582,7 @@ static void test_com_reset_connection(void *p) {
   MYSQL_SESSION st_session;
   ENSURE_API_NOT_NULL(st_session = srv_session_open(session_error_cb, p));
 
-  const my_thread_id session_id = srv_session_info_get_session_id(st_session);
+  my_thread_id session_id = srv_session_info_get_session_id(st_session);
 
   std::unique_ptr<st_plugin_ctx> ctx(new st_plugin_ctx());
   query_execute(st_session, ctx.get(), "set @secret = 123");
@@ -609,9 +602,9 @@ static void test_com_reset_connection_from_another_session(void *p) {
   WRITE_STR("COM_RESET_CONNECTION from another session\n");
 
   MYSQL_SESSION st_session;
-  ENSURE_API_NOT_NULL(st_session = srv_session_open(nullptr, p));
+  ENSURE_API_NOT_NULL(st_session = srv_session_open(NULL, p));
 
-  const my_thread_id session_id = srv_session_info_get_session_id(st_session);
+  my_thread_id session_id = srv_session_info_get_session_id(st_session);
 
   std::unique_ptr<st_plugin_ctx> ctx(new st_plugin_ctx());
   query_execute(st_session, ctx.get(), "set @another_secret = 456");
@@ -710,7 +703,7 @@ static int test_sql_service_plugin_init(void *p) {
   return 0;
 }
 
-static int test_sql_service_plugin_deinit(void *p [[maybe_unused]]) {
+static int test_sql_service_plugin_deinit(void *p MY_ATTRIBUTE((unused))) {
   DBUG_TRACE;
   LogPluginErr(INFORMATION_LEVEL, ER_LOG_PRINTF_MSG, "Uninstallation.");
 
