@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2023, Oracle and/or its affiliates.
+ * Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -137,14 +137,6 @@ class Query_sequencer : public Query_instances {
   Instance_id m_last_instance{0};
 };
 
-inline size_t message_byte_size(const XProtocol::Message &msg) {
-#if (defined(GOOGLE_PROTOBUF_VERSION) && GOOGLE_PROTOBUF_VERSION > 3000000)
-  return msg.ByteSizeLong();
-#else
-  return msg.ByteSize();
-#endif
-}
-
 }  // namespace details
 
 Protocol_impl::Protocol_impl(std::shared_ptr<Context> context,
@@ -325,10 +317,14 @@ XError Protocol_impl::authenticate_mysql41(const std::string &user,
     XError operator()(
         const std::string &user, const std::string &pass, const std::string &db,
         const Mysqlx::Session::AuthenticateContinue &auth_continue) {
+      std::string data;
       std::string password_hash;
+
+      Mysqlx::Session::AuthenticateContinue auth_continue_response;
+
       if (pass.length()) {
-        password_hash =
-            password_hasher::scramble(auth_continue.auth_data(), pass);
+        password_hash = password_hasher::scramble(
+            auth_continue.auth_data().c_str(), pass.c_str());
         password_hash = password_hasher::get_password_from_salt(password_hash);
 
         if (password_hash.empty()) {
@@ -336,12 +332,9 @@ XError Protocol_impl::authenticate_mysql41(const std::string &user,
         }
       }
 
-      std::string data;
       data.append(db).push_back('\0');    // authz
       data.append(user).push_back('\0');  // authc
       data.append(password_hash);         // pass
-
-      Mysqlx::Session::AuthenticateContinue auth_continue_response;
       auth_continue_response.set_auth_data(data);
 
       return m_protocol->send(auth_continue_response);
@@ -437,7 +430,7 @@ XError Protocol_impl::send(const Header_message_type_id mid,
   };
   /*
     Use dummy, otherwise g++ 4.4 reports: unused variable 'dummy'
-    [[maybe_unused]] did not work, so we must use it.
+    MY_ATTRIBUTE((unused)) did not work, so we must use it.
   */
   dummy = 0;
 
@@ -467,7 +460,7 @@ bool Protocol_impl::send_impl(const Client_message_type_id mid,
   const Header_message_type_id header_mesage_id = mid;
   const int header_message_type_size = sizeof(Header_message_type_id);
   const std::size_t header_whole_message_size =
-      details::message_byte_size(msg) + header_message_type_size;
+      msg.ByteSize() + header_message_type_size;
 
   cos.WriteLittleEndian32(header_whole_message_size);
   cos.WriteRaw(&header_mesage_id, header_message_type_size);
@@ -869,11 +862,11 @@ XError Protocol_impl::send_compressed_multiple_frames(
   DBUG_TRACE;
 
   std::string compressed_messages;
-  size_t total_size = 0;
+  int total_size = 0;
 
   {
     for (const auto &message : messages)
-      total_size += details::message_byte_size(*message.second) + 5;
+      total_size += message.second->ByteSize() + 5;
 
     auto algo = m_compression->compression_algorithm();
     if (algo) algo->set_pledged_source_size(total_size);
@@ -895,7 +888,7 @@ XError Protocol_impl::send_compressed_multiple_frames(
 
       dispatch_send_message(msg_id, *msg);
 
-      cos.WriteLittleEndian32(details::message_byte_size(*msg) + 1);
+      cos.WriteLittleEndian32(msg->ByteSize() + 1);
       cos.WriteRaw(&header_msg_id, 1);
       msg->SerializeToCodedStream(&cos);
     }
@@ -1064,11 +1057,6 @@ XProtocol::Message *Protocol_impl::read_compressed(Server_message_type_id *mid,
   return message.release();
 }
 
-void Protocol_impl::reset_buffering() {
-  m_connection_input_stream.reset(
-      new Connection_input_stream(m_connection.get()));
-}
-
 XProtocol::Message *Protocol_impl::recv_message_with_header(
     Server_message_type_id *mid, XError *out_error) {
   DBUG_TRACE;
