@@ -1,17 +1,16 @@
 /*
- * Copyright (c) 2015, 2024, Oracle and/or its affiliates.
+ * Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is designed to work with certain software (including
+ * This program is also distributed with certain software (including
  * but not limited to OpenSSL) that is licensed under separate terms,
  * as designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have either included with
- * the program or referenced in the documentation.
+ * separately licensed software that they have included with MySQL.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -31,6 +30,8 @@
 
 #include "my_dbug.h"  // NOLINT(build/include_subdir)
 
+#include "plugin/x/ngs/include/ngs/notice_descriptor.h"
+#include "plugin/x/ngs/include/ngs/protocol/column_info_builder.h"
 #include "plugin/x/protocol/encoders/encoding_xrow.h"
 #include "plugin/x/src/admin_cmd_arguments.h"
 #include "plugin/x/src/admin_cmd_index.h"
@@ -42,9 +43,6 @@
 #include "plugin/x/src/interface/server.h"
 #include "plugin/x/src/interface/session.h"
 #include "plugin/x/src/mysql_function_names.h"
-#include "plugin/x/src/ngs/client_list.h"
-#include "plugin/x/src/ngs/notice_descriptor.h"
-#include "plugin/x/src/ngs/protocol/column_info_builder.h"
 #include "plugin/x/src/query_string_builder.h"
 #include "plugin/x/src/sql_data_result.h"
 #include "plugin/x/src/xpl_error.h"
@@ -312,8 +310,7 @@ ngs::Error_code Admin_command_handler::kill_client(Command_arguments *args) {
  *                     if not exists
  *   - validation: object, optional - validation schema options
  *     - schema: object|string, optional - json validation document
- *     - level: string, optional - level of validation {"STRICT"|"OFF"};
- *       default "STRICT"
+ *     - level: string - level of validation {"STRICT"|"OFF"}
  */
 ngs::Error_code Admin_command_handler::create_collection(
     Command_arguments *args) {
@@ -381,16 +378,16 @@ ngs::Error_code Admin_command_handler::drop_collection_index(
 
 namespace {
 
-static const std::array<const char *const, 4> k_fixed_notice_names = {
-    "account_expired",
-    "generated_insert_id",
-    "rows_affected",
-    "produced_message",
-};
+static const char *const fixed_notice_names[] = {
+    "account_expired", "generated_insert_id", "rows_affected",
+    "produced_message"};
+static const char *const *fixed_notice_names_end =
+    &fixed_notice_names[0] +
+    sizeof(fixed_notice_names) / sizeof(fixed_notice_names[0]);
 
 inline bool is_fixed_notice_name(const std::string &notice) {
-  return std::find(k_fixed_notice_names.begin(), k_fixed_notice_names.end(),
-                   notice) != k_fixed_notice_names.end();
+  return std::find(fixed_notice_names, fixed_notice_names_end, notice) !=
+         fixed_notice_names_end;
 }
 
 inline void add_notice_row(iface::Protocol_encoder *proto,
@@ -512,7 +509,7 @@ ngs::Error_code Admin_command_handler::list_notices(Command_arguments *args) {
                    notice_config.is_notice_enabled(notice_type) ? 1 : 0);
   }
 
-  for (const auto notice : k_fixed_notice_names) {
+  for (const auto notice : fixed_notice_names) {
     add_notice_row(&proto, notice, 1);
   }
 
@@ -646,7 +643,7 @@ ngs::Error_code Admin_command_handler::modify_collection_options(
  * Required arguments:
  * - name: string - name of collection
  * - schema: string - name of collection's schema
- * - options: string, list - collection options to fetch
+ * - options: object - collection options to fetch
  */
 ngs::Error_code Admin_command_handler::get_collection_options(
     Command_arguments *args) {
