@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016, 2023, Oracle and/or its affiliates.
+ * Copyright (c) 2016, 2020, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -28,7 +28,6 @@
 #include <fcntl.h>
 #include <stdlib.h>
 
-#include "m_string.h"
 #include "my_config.h"  // NOLINT(build/include_subdir)
 #include "my_io.h"      // NOLINT(build/include_subdir)
 
@@ -179,7 +178,6 @@ class Unixsocket_creator {
     return false;
 #else
     char buffer[8];
-    char *pid_begin = buffer;
     const char x_prefix = 'X';
     const pid_t cur_pid = m_system_interface->get_pid();
     const std::string lock_filename =
@@ -244,12 +242,15 @@ class Unixsocket_creator {
       }
       buffer[len] = '\0';
 
-      if (x_prefix == pid_begin[0]) {
-        ++pid_begin;
+      if (x_prefix != buffer[0]) {
+        error_message = "lock file wasn't allocated by X Plugin ";
+        error_message += lock_filename;
+
+        return false;
       }
 
       pid_t parent_pid = m_system_interface->get_ppid();
-      pid_t read_pid = atoi(pid_begin);
+      pid_t read_pid = atoi(buffer + 1);
 
       if (read_pid <= 0) {
         error_message = "invalid PID in UNIX socket lock file ";
@@ -281,7 +282,9 @@ class Unixsocket_creator {
       }
     }
 
-    snprintf(buffer, sizeof(buffer), "%d\n", static_cast<int>(cur_pid));
+    // The "X" should fail legacy UNIX socket lock-file allocation
+    snprintf(buffer, sizeof(buffer), "%c%d\n", x_prefix,
+             static_cast<int>(cur_pid));
     if (lockfile_fd->write(buffer, strlen(buffer)) !=
         static_cast<signed>(strlen(buffer))) {
       error_message = String_formatter()
@@ -341,13 +344,30 @@ Listener_unix_socket::~Listener_unix_socket() {
   close_listener();
 }
 
-const Listener_unix_socket::Sync_variable_state &
-Listener_unix_socket::get_state() const {
+Listener_unix_socket::Sync_variable_state &Listener_unix_socket::get_state() {
   return m_state;
 }
 
-std::string Listener_unix_socket::get_configuration_variable() const {
-  return MYSQLX_SYSTEM_VARIABLE_PREFIX("socket");
+std::string Listener_unix_socket::get_name_and_configuration() const {
+  std::string result = "socket: '";
+
+  result += m_unix_socket_path;
+  result += "'";
+
+  return result;
+}
+
+std::vector<std::string> Listener_unix_socket::get_configuration_variables()
+    const {
+  std::vector<std::string> result;
+
+  result.push_back(MYSQLX_SYSTEM_VARIABLE_PREFIX("socket"));
+
+  return result;
+}
+
+std::string Listener_unix_socket::get_last_error() const {
+  return m_last_error;
 }
 
 bool Listener_unix_socket::setup_listener(On_connection on_connection) {
@@ -395,7 +415,6 @@ void Listener_unix_socket::close_listener() {
 
 void Listener_unix_socket::pre_loop() {
   if (m_unix_socket) m_unix_socket->set_socket_thread_owner();
-  m_state.set(xpl::iface::Listener::State::k_running);
 }
 
 void Listener_unix_socket::loop() {}
@@ -418,17 +437,4 @@ void Listener_unix_socket::report_properties(On_report_properties on_prop) {
   }
 }
 
-bool Listener_unix_socket::report_status() const {
-  const std::string msg = "socket: '" + m_unix_socket_path + "'";
-
-  if (m_state.is(State::k_prepared)) {
-    log_info(ER_XPLUGIN_LISTENER_STATUS_MSG, msg.c_str());
-    return true;
-  }
-
-  log_error(ER_XPLUGIN_LISTENER_SETUP_FAILED, msg.c_str(),
-            m_last_error.c_str());
-  return false;
-}
-
 }  // namespace xpl
