@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2019, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2019, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,7 +27,6 @@
 
 #include <string>
 
-#include <mysql/psi/psi_metric.h>
 #include "mysql/plugin.h"
 #include "plugin/x/src/server/server_properties.h"
 
@@ -37,11 +35,9 @@ namespace xpl {
 class Plugin_status_variables {
  public:
   static struct SHOW_VAR m_plugin_status_variables[];
-  static PSI_meter_info_v1 m_xpl_meter[];
   static ngs::Server_properties m_properties;
 
   static std::string get_property(const ngs::Server_property_ids id);
-  static size_t get_meter_count();
 };
 
 }  // namespace xpl
