@@ -1,17 +1,16 @@
 /*
- * Copyright (c) 2015, 2024, Oracle and/or its affiliates.
+ * Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is designed to work with certain software (including
+ * This program is also distributed with certain software (including
  * but not limited to OpenSSL) that is licensed under separate terms,
  * as designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have either included with
- * the program or referenced in the documentation.
+ * separately licensed software that they have included with MySQL.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,8 +23,8 @@
  */
 
 #include "plugin/x/src/xpl_performance_schema.h"
-#include "plugin/x/src/ngs/memory.h"
-#include "template_utils.h"
+
+#include "plugin/x/ngs/include/ngs/memory.h"
 
 #ifdef HAVE_PSI_INTERFACE
 
@@ -33,10 +32,8 @@ PSI_thread_key KEY_thread_x_acceptor;
 PSI_thread_key KEY_thread_x_worker;
 
 static PSI_thread_info all_x_threads[] = {
-    {&KEY_thread_x_acceptor, "acceptor_network", "xpl_accept",
-     PSI_FLAG_AUTO_SEQNUM, 0, PSI_DOCUMENT_ME},
-    {&KEY_thread_x_worker, "worker", "xpl_worker",
-     PSI_FLAG_USER | PSI_FLAG_AUTO_SEQNUM, 0, PSI_DOCUMENT_ME},
+    {&KEY_thread_x_acceptor, "acceptor_network", 0, 0, PSI_DOCUMENT_ME},
+    {&KEY_thread_x_worker, "worker", PSI_FLAG_USER, 0, PSI_DOCUMENT_ME},
 };
 
 PSI_mutex_key KEY_mutex_x_lock_list_access;
