@@ -1,17 +1,16 @@
 /*
- * Copyright (c) 2017, 2024, Oracle and/or its affiliates.
+ * Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is designed to work with certain software (including
+ * This program is also distributed with certain software (including
  * but not limited to OpenSSL) that is licensed under separate terms,
  * as designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have either included with
- * the program or referenced in the documentation.
+ * separately licensed software that they have included with MySQL.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -34,14 +33,22 @@
 
 #include "plugin/x/tests/driver/processor/variable_names.h"
 
+google::protobuf::LogHandler *g_lh = nullptr;
+
 Connection_manager::Connection_manager(const Connection_options &co,
                                        Variable_container *variables,
-                                       const Console &console_with_flow_history,
                                        const Console &console)
     : m_default_connection_options(co),
       m_variables(variables),
-      m_console_with_flow_history(console_with_flow_history),
       m_console(console) {
+  g_lh = google::protobuf::SetLogHandler([](google::protobuf::LogLevel level,
+                                            const char *filename, int line,
+                                            const std::string &message) {
+    if (g_lh) g_lh(level, filename, line, message);
+    DBUG_LOG("debug",
+             "Protobuf error (level:" << level << ", filename:" << filename
+                                      << ":" << line << ", text:" << message);
+  });
   m_variables->make_special_variable(
       k_variable_option_user,
       new Variable_dynamic_string(m_default_connection_options.user));
@@ -102,9 +109,8 @@ Connection_manager::Connection_manager(const Connection_options &co,
   m_variables->make_special_variable(k_variable_option_compression_level,
                                      new Variable_string_readonly(level));
 
-  m_active_holder.reset(
-      new Session_holder(xcl::create_session(), m_console_with_flow_history,
-                         m_console, m_default_connection_options));
+  m_active_holder.reset(new Session_holder(xcl::create_session(), m_console,
+                                           m_default_connection_options));
 
   m_session_holders[""] = m_active_holder;
 }
@@ -215,8 +221,8 @@ void Connection_manager::create(const std::string &name,
   }
 
   auto session = xcl::create_session();
-  std::shared_ptr<Session_holder> holder{new Session_holder(
-      std::move(session), m_console_with_flow_history, m_console, co)};
+  std::shared_ptr<Session_holder> holder{
+      new Session_holder(std::move(session), m_console, co)};
 
   xcl::XError error = holder->connect(is_raw_connection);
 
@@ -255,65 +261,64 @@ bool Connection_manager::is_default_active() {
 
 void Connection_manager::close_active(const bool shutdown,
                                       const bool be_quiet) {
-  if (!m_active_holder) {
-    if (!shutdown) throw std::runtime_error("no active session");
-    return;
-  }
+  if (m_active_holder) {
+    if (m_active_session_name.empty() && !shutdown)
+      throw std::runtime_error("cannot close default session");
+    try {
+      if (!m_active_session_name.empty() && !be_quiet)
+        m_console.print("closing session ", m_active_session_name, "\n");
 
-  if (m_active_session_name.empty() && !shutdown) {
-    throw std::runtime_error("cannot close default session");
-  }
-
-  try {
-    if (!m_active_session_name.empty() && !be_quiet)
-      m_console.print("closing session ", m_active_session_name, "\n");
-
-    if (active_xconnection()->state().is_connected()) {
-      // send a close message and wait for the corresponding Ok message
-      active_xprotocol()->send(Mysqlx::Connection::Close());
-      xcl::XProtocol::Server_message_type_id msgid;
-      xcl::XError error;
-      Message_ptr msg{active_xprotocol()->recv_single_message(&msgid, &error)};
-
-      if (error) throw error;
-
-      if (!be_quiet) m_console.print(*msg);
-      if (Mysqlx::ServerMessages::OK != msgid)
-        throw xcl::XError(CR_COMMANDS_OUT_OF_SYNC,
-                          "Disconnect was expecting Mysqlx.Ok(bye!), but "
-                          "got the one above (one or more calls to -->recv "
-                          "are probably missing)");
-
-      std::string text = static_cast<Mysqlx::Ok *>(msg.get())->msg();
-      if (text != "bye!" && text != "tchau!")
-        throw xcl::XError(CR_COMMANDS_OUT_OF_SYNC,
-                          "Disconnect was expecting Mysqlx.Ok(bye!), but "
-                          "got the one above (one or more calls to -->recv "
-                          "are probably missing)");
-
-      if (!m_default_connection_options.dont_wait_for_disconnect) {
+      if (active_xconnection()->state().is_connected()) {
+        // send a close message and wait for the corresponding Ok message
+        active_xprotocol()->send(Mysqlx::Connection::Close());
+        xcl::XProtocol::Server_message_type_id msgid;
+        xcl::XError error;
         Message_ptr msg{
             active_xprotocol()->recv_single_message(&msgid, &error)};
 
-        if (!error && !be_quiet) {
-          m_console.print_error("Was expecting closure but got message:", *msg);
-        }
-      }
+        if (error) throw error;
 
+        if (!be_quiet) m_console.print(*msg);
+        if (Mysqlx::ServerMessages::OK != msgid)
+          throw xcl::XError(CR_COMMANDS_OUT_OF_SYNC,
+                            "Disconnect was expecting Mysqlx.Ok(bye!), but "
+                            "got the one above (one or more calls to -->recv "
+                            "are probably missing)");
+
+        std::string text = static_cast<Mysqlx::Ok *>(msg.get())->msg();
+        if (text != "bye!" && text != "tchau!")
+          throw xcl::XError(CR_COMMANDS_OUT_OF_SYNC,
+                            "Disconnect was expecting Mysqlx.Ok(bye!), but "
+                            "got the one above (one or more calls to -->recv "
+                            "are probably missing)");
+
+        if (!m_default_connection_options.dont_wait_for_disconnect) {
+          Message_ptr msg{
+              active_xprotocol()->recv_single_message(&msgid, &error)};
+
+          if (!error && !be_quiet) {
+            m_console.print_error("Was expecting closure but got message:",
+                                  *msg);
+          }
+        }
+
+        active_xconnection()->close();
+      }
+      m_session_holders.erase(m_active_session_name);
+      if (!shutdown) set_active("", be_quiet);
+    } catch (const std::exception &error) {
       active_xconnection()->close();
+      m_session_holders.erase(m_active_session_name);
+      if (!shutdown) set_active("", be_quiet);
+      throw error;
+    } catch (const xcl::XError &error) {
+      active_xconnection()->close();
+      m_session_holders.erase(m_active_session_name);
+      if (!shutdown) set_active("", be_quiet);
+      throw error;
     }
-    m_session_holders.erase(m_active_session_name);
-    if (!shutdown) set_active("", be_quiet);
-  } catch (const std::exception &error) {
-    active_xconnection()->close();
-    m_session_holders.erase(m_active_session_name);
-    if (!shutdown) set_active("", be_quiet);
-    throw error;
-  } catch (const xcl::XError &error) {
-    active_xconnection()->close();
-    m_session_holders.erase(m_active_session_name);
-    if (!shutdown) set_active("", be_quiet);
-    throw error;
+  } else {
+    if (!shutdown) throw std::runtime_error("no active session");
   }
 }
 
@@ -321,14 +326,10 @@ void Connection_manager::set_active(const std::string &name,
                                     const bool be_quiet) {
   if (m_session_holders.count(name) == 0) {
     std::string slist;
-    bool first = true;
     for (auto it = m_session_holders.begin(); it != m_session_holders.end();
-         ++it) {
-      if (!first) slist.append(", ");
-      slist.append(it->first);
-      first = false;
-    }
-
+         ++it)
+      slist.append(it->first).append(", ");
+    if (!slist.empty()) slist.resize(slist.length() - 2);
     throw std::runtime_error("no session named '" + name + "': " + slist);
   }
   m_active_holder = m_session_holders[name];
