@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2015, 2023, Oracle and/or its affiliates.
+  Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -31,16 +31,14 @@
 #include <ostream>
 #include <stdexcept>
 #include <string>
-#include <string_view>
-#include <system_error>
-#include <vector>
 
 #ifndef _WIN32
 #include <fcntl.h>
 #endif
 
-#include "mysql/harness/access_rights.h"
-#include "mysql/harness/stdx/expected.h"
+#ifdef _WIN32
+#include <aclapi.h>
+#endif
 
 namespace mysql_harness {
 
@@ -117,12 +115,12 @@ class HARNESS_EXPORT Path {
    * Construct a path
    *
    * @param path Non-empty string denoting the path.
-   * @throws std::invalid_argument
    */
-  Path(std::string path);
+  /** @overload */                // throws std::invalid_argument
+  Path(const std::string &path);  // NOLINT(runtime/explicit)
 
-  Path(std::string_view path) : Path(std::string(path)) {}
-  Path(const char *path) : Path(std::string(path)) {}
+  /** @overload */         // throws std::invalid_argument
+  Path(const char *path);  // NOLINT(runtime/explicit)
 
   /**
    * Create a path from directory, basename, and extension.
@@ -222,7 +220,7 @@ class HARNESS_EXPORT Path {
    * Append a path component to the current path.
    *
    * This function will append a path component to the path using the
-   * appropriate directory separator.
+   * apropriate directory separator.
    *
    * @param other Path component to append to the path.
    */
@@ -241,6 +239,9 @@ class HARNESS_EXPORT Path {
    */
   Path join(const Path &other) const;
 
+  /** @overload */
+  Path join(const char *other) const { return join(Path(other)); }
+
   /**
    * Returns the canonical form of the path, resolving relative paths.
    */
@@ -274,7 +275,7 @@ class HARNESS_EXPORT Path {
   /**
    * Directory separator string.
    *
-   * @note This is platform-dependent and defined in the appropriate
+   * @note This is platform-dependent and defined in the apropriate
    * source file.
    */
   static const char *const directory_separator;
@@ -282,7 +283,7 @@ class HARNESS_EXPORT Path {
   /**
    * Root directory string.
    *
-   * @note This is platform-dependent and defined in the appropriate
+   * @note This is platform-dependent and defined in the apropriate
    * source file.
    */
   static const char *const root_directory;
@@ -318,16 +319,12 @@ class HARNESS_EXPORT Directory : public Path {
    *
    * A directory iterator is an input iterator.
    */
-  class HARNESS_EXPORT DirectoryIterator {
+  using DirectoryIteratorBase = std::iterator<std::input_iterator_tag, Path>;
+
+  class HARNESS_EXPORT DirectoryIterator : public DirectoryIteratorBase {
     friend class Directory;
 
    public:
-    using value_type = Path;
-    using iterator_category = std::input_iterator_tag;
-    using difference_type = std::ptrdiff_t;
-    using pointer = value_type *;
-    using reference = value_type &;
-
     DirectoryIterator(const Path &path,
                       const std::string &pattern = std::string());
 
@@ -417,15 +414,6 @@ class HARNESS_EXPORT Directory : public Path {
    */
   DirectoryIterator begin();
 
-  DirectoryIterator begin() const { return cbegin(); }
-
-  /**
-   * Constant iterator to first entry.
-   *
-   * @return Returns a constant iterator pointing to the first entry.
-   */
-  DirectoryIterator cbegin() const;
-
   /**
    * Iterator past-the-end of entries.
    *
@@ -433,33 +421,6 @@ class HARNESS_EXPORT Directory : public Path {
    */
   DirectoryIterator end();
 
-  DirectoryIterator end() const { return cend(); }
-
-  /**
-   * Constant iterator past-the-end of entries.
-   *
-   * @return Returns a constant iterator pointing *past-the-end* of the entries.
-   */
-  DirectoryIterator cend() const;
-
-  /**
-   * Check if the directory is empty.
-   *
-   * @retval true Directory is empty.
-   * @retval false Directory is no empty.
-   */
-  bool is_empty() const;
-
-  /**
-   * Recursively list all paths in a directory.
-   *
-   * Recursively create a list of relative paths from a directory. Path will
-   * be relative to the given directory. Empty directories are also listed.
-   *
-   * @return Recursive list of paths from a directory.
-   */
-  std::vector<Path> list_recursive() const;
-
   /**
    * Iterate over entries matching a glob.
    */
@@ -478,11 +439,10 @@ class HARNESS_EXPORT Directory : public Path {
  *
  * @param dir path of the directory to be removed; this directory must be empty
  *
- * @return void on success, error_code on failure
+ * @return 0 on success, -1 on error and sets errno
  */
 HARNESS_EXPORT
-stdx::expected<void, std::error_code> delete_dir(
-    const std::string &dir) noexcept;
+int delete_dir(const std::string &dir) noexcept;
 
 /** @brief Removes a file.
  *
@@ -490,11 +450,10 @@ stdx::expected<void, std::error_code> delete_dir(
  *
  * @param path of the file to be removed
  *
- * @return void on success, error_code on failure
+ * @return 0 on success, -1 on error
  */
 HARNESS_EXPORT
-stdx::expected<void, std::error_code> delete_file(
-    const std::string &path) noexcept;
+int delete_file(const std::string &path) noexcept;
 
 /** @brief Removes directory and all its contents.
  *
@@ -502,11 +461,10 @@ stdx::expected<void, std::error_code> delete_file(
  *
  * @param dir path of the directory to be removed
  *
- * @return void on success, error_code on failure
+ * @return 0 on success, -1 on error
  */
 HARNESS_EXPORT
-stdx::expected<void, std::error_code> delete_dir_recursive(
-    const std::string &dir) noexcept;
+int delete_dir_recursive(const std::string &dir) noexcept;
 
 /** @brief Creates a temporary directory with partially-random name and returns
  * its path.
@@ -537,6 +495,30 @@ std::string get_plugin_dir(const std::string &runtime_dir);
 HARNESS_EXPORT
 std::string get_tests_data_dir(const std::string &runtime_dir);
 
+#ifdef _WIN32
+/**
+ * Deleter for smart pointers pointing to objects allocated with `std::malloc`.
+ */
+template <typename T>
+class StdFreeDeleter {
+ public:
+  void operator()(T *ptr) { std::free(ptr); }
+};
+
+/**
+ * Retrieves file's DACL security descriptor.
+ *
+ * @param[in] file_name File name.
+ *
+ * @return File's DACL security descriptor.
+ *
+ * @throw std::exception Failed to retrieve security descriptor.
+ */
+HARNESS_EXPORT std::unique_ptr<SECURITY_DESCRIPTOR, decltype(&free)>
+get_security_descriptor(const std::string &file_name);
+
+#endif
+
 #ifndef _WIN32
 using perm_mode = mode_t;
 HARNESS_EXPORT
@@ -551,7 +533,7 @@ extern const perm_mode kStrictDirectoryPerm;
  * *
  * @param dir       name (or path) of the directory to create
  * @param mode      permission mode for the created directory
- * @param recursive if true then imitate unix `mkdir -p` recursively
+ * @param recursive if true then immitated unix `mkdir -p` recursively
  *                  creating parent directories if needed
  * @retval 0 operation succeeded
  * @retval -1 operation failed because of wrong parameters
