@@ -1,17 +1,16 @@
 /*
-  Copyright (c) 2015, 2024, Oracle and/or its affiliates.
+  Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -103,7 +102,7 @@
  *   1. shutdown signal is received by the harness
  *   2. one of the plugins exits with error
  *
- * When one of these two events occurs, harness progresses to the
+ * When one of these two events occurrs, harness progresses to the
  * next step.
  *
  *
@@ -568,7 +567,7 @@ threads should exit before their parent function finishes running.
 NOTE: WL#9558 HLD version of this section additionally discusses design,
       rationale for the approach chosen, etc; look there if interested.
 
-When plugin functions encounter an error, they are expected to signal it via
+When plugin functions enounter an error, they are expected to signal it via
 set_error(). What happens next, depends on the case, but in all four
 cases the error will be logged automatically by the harness. Also, the first
 error passed from any plugin will be saved until the end of life-cycle
@@ -615,6 +614,8 @@ end). Actions taken for each plugin function are as follows:
 #ifndef MYSQL_HARNESS_LOADER_INCLUDED
 #define MYSQL_HARNESS_LOADER_INCLUDED
 
+#include "router_config.h"
+
 #include "config_parser.h"
 #include "filesystem.h"
 #include "mysql/harness/dynamic_loader.h"
@@ -641,7 +642,59 @@ end). Actions taken for each plugin function are as follows:
 #include <tuple>
 
 #ifdef FRIEND_TEST
+// TODO replace by #include after merge:
+// #include "../../../mysqlrouter/utils.h"  // DECLARE_TEST
+/** @brief Declare test (class)
+ *
+ * When using FRIEND_TEST() on classes that are not in the same namespace
+ * as the test, the test (class) needs to be forward-declared. This marco
+ * eases this.
+ *
+ * @note We need this for unit tests, BUT on the TESTED code side (not in unit
+ * test code)
+ */
+#define DECLARE_TEST(test_case_name, test_name) \
+  class test_case_name##_##test_name##_Test
+
 class TestLoader;
+class LifecycleTest;
+
+DECLARE_TEST(TestStart, StartLogger);
+DECLARE_TEST(LifecycleTest, Simple_None);
+DECLARE_TEST(LifecycleTest, Simple_AllFunctions);
+DECLARE_TEST(LifecycleTest, Simple_Init);
+DECLARE_TEST(LifecycleTest, Simple_StartStop);
+DECLARE_TEST(LifecycleTest, Simple_StartStopBlocking);
+DECLARE_TEST(LifecycleTest, Simple_Start);
+DECLARE_TEST(LifecycleTest, Simple_Stop);
+DECLARE_TEST(LifecycleTest, Simple_Deinit);
+DECLARE_TEST(LifecycleTest, ThreeInstances_NoError);
+DECLARE_TEST(LifecycleTest, BothLifecycles_NoError);
+DECLARE_TEST(LifecycleTest, OneInstance_NothingPersists_NoError);
+DECLARE_TEST(LifecycleTest, OneInstance_NothingPersists_StopFails);
+DECLARE_TEST(LifecycleTest, ThreeInstances_InitFails);
+DECLARE_TEST(LifecycleTest, BothLifecycles_InitFails);
+DECLARE_TEST(LifecycleTest, ThreeInstances_Start1Fails);
+DECLARE_TEST(LifecycleTest, ThreeInstances_Start2Fails);
+DECLARE_TEST(LifecycleTest, ThreeInstances_Start3Fails);
+DECLARE_TEST(LifecycleTest, ThreeInstances_2StartsFail);
+DECLARE_TEST(LifecycleTest, ThreeInstances_StopFails);
+DECLARE_TEST(LifecycleTest, ThreeInstances_DeinintFails);
+DECLARE_TEST(LifecycleTest, ThreeInstances_StartStopDeinitFail);
+DECLARE_TEST(LifecycleTest, NoInstances);
+DECLARE_TEST(LifecycleTest, EmptyErrorMessage);
+DECLARE_TEST(LifecycleTest, send_signals);
+DECLARE_TEST(LifecycleTest, send_signals2);
+DECLARE_TEST(LifecycleTest, wait_for_stop);
+DECLARE_TEST(LifecycleTest, InitThrows);
+DECLARE_TEST(LifecycleTest, StartThrows);
+DECLARE_TEST(LifecycleTest, StopThrows);
+DECLARE_TEST(LifecycleTest, DeinitThrows);
+DECLARE_TEST(LifecycleTest, InitThrowsWeird);
+DECLARE_TEST(LifecycleTest, StartThrowsWeird);
+DECLARE_TEST(LifecycleTest, StopThrowsWeird);
+DECLARE_TEST(LifecycleTest, DeinitThrowsWeird);
+DECLARE_TEST(LoaderReadTest, Loading);
 #endif
 
 namespace mysql_harness {
@@ -694,8 +747,8 @@ class HARNESS_EXPORT PluginFuncEnv {
   void set_running() noexcept;
   void clear_running() noexcept;
   bool is_running() const noexcept;
-  bool wait_for_stop(
-      uint32_t milliseconds) const noexcept;  // 0 = infinite wait
+  bool wait_for_stop(uint32_t milliseconds) const
+      noexcept;  // 0 = infinite wait
 
   // error handling
   // (see also corresponding Harness API functions in plugin.h for more info)
@@ -753,8 +806,8 @@ class HARNESS_EXPORT Loader {
    * @param program Name of our program
    * @param config Router configuration
    */
-  Loader(std::string program, LoaderConfig &config)
-      : config_(config), program_(std::move(program)) {}
+  Loader(const std::string &program, LoaderConfig &config)
+      : config_(config), program_(program) {}
 
   Loader(const Loader &) = delete;
   Loader &operator=(const Loader &) = delete;
@@ -795,87 +848,6 @@ class HARNESS_EXPORT Loader {
    */
   LoaderConfig &get_config() { return config_; }
 
-  /**
-   * service names to wait on.
-   *
-   * add a service name and call on_service_ready() when the service ready().
-   *
-   * @see on_service_ready()
-   */
-  std::vector<std::string> &waitable_services() { return waitable_services_; }
-
-  /**
-   * service names to wait on.
-   *
-   * @see on_service_ready()
-   */
-  const std::vector<std::string> &waitable_services() const {
-    return waitable_services_;
-  }
-
-  /**
-   * set a function that's called after all plugins have been started.
-   *
-   * @see after_all_finished()
-   */
-  void after_all_started(std::function<void()> &&func) {
-    after_all_started_ = std::move(func);
-  }
-
-  /**
-   * set a function that's called after the first plugin exited.
-   *
-   * @see after_all_started()
-   */
-  void after_first_finished(std::function<void()> &&func) {
-    after_first_finished_ = std::move(func);
-  }
-
-  /**
-   * Register global configuration options supported by the application. Will be
-   * used by the Loader to verify the [DEFAULT] options in the configuration.
-   *
-   * @param options array of global options supported by the applications
-   */
-  template <size_t N>
-  void register_supported_app_options(
-      const std::array<std::string_view, N> &options) {
-    supported_app_options_.clear();
-    for (const auto &option : options) {
-      supported_app_options_.emplace_back(std::string(option));
-    }
-  }
-
-  /**
-   * Register a callback that the Loader will call when exposing of the whole
-   * configuration is requested. This callback is supposed to expose the
-   * application-level configration options (not plugin specific ones).
-   *
-   * @param clb callback to register that takes 2 parameters
-   *            - bool indicating if the requested configuration is the initial
-   *              one (if true) or default (false)
-   *            - object representing default section of the current
-   *              configuration
-   */
-  void register_expose_app_config_callback(
-      std::function<void(const bool, const ConfigSection &)> clb) {
-    expose_app_config_clb_ = clb;
-  }
-
-  /**
-   * Load all the configured plugins.
-   */
-  void load_all();  // throws bad_plugin on load error
-
-  /**
-   * Request the application and all the configured plugins to expose their
-   * configuration in the DyncamicConfig object.
-   *
-   * @param initial if true initial configuration is to exposed, default
-   *                configuration otherwise
-   */
-  void expose_config_all(const bool initial);
-
  private:
   enum class Status { UNVISITED, ONGOING, VISITED };
 
@@ -931,6 +903,7 @@ class HARNESS_EXPORT Loader {
   // PluginFuncEnv will always exist when plugin stop() function is called.
 
   // start() calls these, indents reflect call hierarchy
+  void load_all();  // throws bad_plugin on load error
   void setup_info();
   std::exception_ptr
   run();  // returns first exception returned from below harness functions
@@ -1028,51 +1001,65 @@ class HARNESS_EXPORT Loader {
   std::string program_;
   AppInfo appinfo_;
 
-  void spawn_signal_handler_thread();
-
-  std::mutex signal_thread_ready_m_;
-  std::condition_variable signal_thread_ready_cond_;
-  bool signal_thread_ready_{false};
-  std::thread signal_thread_;
-
-  std::vector<std::string> supported_app_options_;
-  /**
-   * Checks if all the options in the configuration fed to the Loader are
-   * supported.
-   *
-   * @throws std::runtime_error if there is unsupported option in the
-   * configuration
-   */
-  void check_config_options_supported();
-
-  /**
-   * Checks if all the options in the section [DEFAULT] in the configuration fed
-   * to the Loader are supported.
-   *
-   * @throws std::runtime_error if there is unsupported option in the [DEFAULT]
-   * section of the configuration
-   */
-  void check_default_config_options_supported();
-
-  // service names that need to be waited on.
-  //
-  // @see on_service_ready()
-  std::vector<std::string> waitable_services_;
-
-  // called after "start_all()" succeeded.
-  std::function<void()> after_all_started_;
-
-  // called after "main_loop()" exited.
-  std::function<void()> after_first_finished_;
-
-  // called as a part of expose_config_all()
-  std::function<void(const bool, const ConfigSection &)> expose_app_config_clb_;
-
 #ifdef FRIEND_TEST
   friend class ::TestLoader;
+  friend class ::LifecycleTest;
+
+  FRIEND_TEST(::TestStart, StartLogger);
+  FRIEND_TEST(::LifecycleTest, Simple_None);
+  FRIEND_TEST(::LifecycleTest, Simple_AllFunctions);
+  FRIEND_TEST(::LifecycleTest, Simple_Init);
+  FRIEND_TEST(::LifecycleTest, Simple_StartStop);
+  FRIEND_TEST(::LifecycleTest, Simple_StartStopBlocking);
+  FRIEND_TEST(::LifecycleTest, Simple_Start);
+  FRIEND_TEST(::LifecycleTest, Simple_Stop);
+  FRIEND_TEST(::LifecycleTest, Simple_Deinit);
+  FRIEND_TEST(::LifecycleTest, ThreeInstances_NoError);
+  FRIEND_TEST(::LifecycleTest, BothLifecycles_NoError);
+  FRIEND_TEST(::LifecycleTest, OneInstance_NothingPersists_NoError);
+  FRIEND_TEST(::LifecycleTest, OneInstance_NothingPersists_StopFails);
+  FRIEND_TEST(::LifecycleTest, ThreeInstances_InitFails);
+  FRIEND_TEST(::LifecycleTest, BothLifecycles_InitFails);
+  FRIEND_TEST(::LifecycleTest, ThreeInstances_Start1Fails);
+  FRIEND_TEST(::LifecycleTest, ThreeInstances_Start2Fails);
+  FRIEND_TEST(::LifecycleTest, ThreeInstances_Start3Fails);
+  FRIEND_TEST(::LifecycleTest, ThreeInstances_2StartsFail);
+  FRIEND_TEST(::LifecycleTest, ThreeInstances_StopFails);
+  FRIEND_TEST(::LifecycleTest, ThreeInstances_DeinintFails);
+  FRIEND_TEST(::LifecycleTest, ThreeInstances_StartStopDeinitFail);
+  FRIEND_TEST(::LifecycleTest, NoInstances);
+  FRIEND_TEST(::LifecycleTest, EmptyErrorMessage);
+  FRIEND_TEST(::LifecycleTest, send_signals);
+  FRIEND_TEST(::LifecycleTest, send_signals2);
+  FRIEND_TEST(::LifecycleTest, wait_for_stop);
+  FRIEND_TEST(::LifecycleTest, InitThrows);
+  FRIEND_TEST(::LifecycleTest, StartThrows);
+  FRIEND_TEST(::LifecycleTest, StopThrows);
+  FRIEND_TEST(::LifecycleTest, DeinitThrows);
+  FRIEND_TEST(::LifecycleTest, InitThrowsWeird);
+  FRIEND_TEST(::LifecycleTest, StartThrowsWeird);
+  FRIEND_TEST(::LifecycleTest, StopThrowsWeird);
+  FRIEND_TEST(::LifecycleTest, DeinitThrowsWeird);
+  FRIEND_TEST(::LoaderReadTest, Loading);
 #endif
+
 };  // class Loader
 
 }  // namespace mysql_harness
 
+HARNESS_EXPORT
+void request_application_shutdown();
+
+#ifdef _WIN32
+HARNESS_EXPORT
+void register_ctrl_c_handler();
+#endif
+
+#ifdef FRIEND_TEST
+namespace unittest_backdoor {
+HARNESS_EXPORT
+void set_shutdown_pending(bool shutdown_pending);
+}  // namespace unittest_backdoor
+#endif
+
 #endif /* MYSQL_HARNESS_LOADER_INCLUDED */
