@@ -1,17 +1,16 @@
 /*
-  Copyright (c) 2015, 2024, Oracle and/or its affiliates.
+  Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -36,7 +35,6 @@
 #include <cstring>
 #include <exception>
 #include <map>
-#include <mutex>
 #include <sstream>
 #include <stdexcept>
 #include <string>
@@ -49,31 +47,21 @@
 #include <unistd.h>
 #endif
 
-#ifdef HAVE_SYS_PRCTL_H
-#include <sys/prctl.h>
-#endif
-
 ////////////////////////////////////////
 // Package include files
 #include "builtin_plugins.h"
+#include "common.h"  // mysql_harness::rename_thread()
 #include "designator.h"
 #include "dim.h"
 #include "exception.h"
 #include "harness_assert.h"
-#include "my_thread.h"  // my_thread_self_setname
-#include "mysql/harness/dynamic_config.h"
+#include "my_stacktrace.h"
 #include "mysql/harness/dynamic_loader.h"
 #include "mysql/harness/filesystem.h"
 #include "mysql/harness/logging/logging.h"
 #include "mysql/harness/logging/registry.h"
 #include "mysql/harness/plugin.h"
-#include "mysql/harness/process_state_component.h"
-#include "mysql/harness/sd_notify.h"
-#include "mysql/harness/stdx/monitor.h"
-#include "mysql/harness/supported_config_options.h"
-#include "mysql/harness/utility/string.h"  // join
-#include "scope_guard.h"
-#include "utilities.h"  // make_range
+#include "utilities.h"
 IMPORT_LOG_FUNCTIONS()
 
 #include "my_compiler.h"
@@ -85,9 +73,11 @@ using mysql_harness::utility::reverse;
 using mysql_harness::Config;
 using mysql_harness::Path;
 
-using namespace std::chrono_literals;
+using std::ostringstream;
 
-static std::atomic<size_t> num_of_non_ready_services{0};
+#if !defined(_WIN32)
+#define USE_POSIX_SIGNALS
+#endif
 
 /**
  * @defgroup Loader Plugin loader
@@ -101,6 +91,213 @@ static std::atomic<size_t> num_of_non_ready_services{0};
 //
 ////////////////////////////////////////////////////////////////////////////////
 
+std::mutex we_might_shutdown_cond_mutex;
+std::condition_variable we_might_shutdown_cond;
+
+enum ShutdownReason { SHUTDOWN_NONE, SHUTDOWN_REQUESTED, SHUTDOWN_FATAL_ERROR };
+
+// set when the Router receives a signal to shut down or some fatal error
+// condition occurred
+static std::atomic<ShutdownReason> g_shutdown_pending{SHUTDOWN_NONE};
+
+// the thread that is setting the g_shutdown_pending to SHUTDOWN_FATAL_ERROR is
+// supposed to set this error message so that it bubbles up and ends up on the
+// console
+static std::string shutdown_fatal_error_message;
+
+std::mutex log_reopen_cond_mutex;
+std::condition_variable log_reopen_cond;
+static std::atomic<bool> g_log_reopen_requested{false};
+
+/**
+ * request application shutdown.
+ *
+ * @throws std::system_error same as std::unique_lock::lock does
+ */
+static void request_application_shutdown(const ShutdownReason reason) {
+  {
+    std::unique_lock<std::mutex> lk(we_might_shutdown_cond_mutex);
+    std::unique_lock<std::mutex> lk2(log_reopen_cond_mutex);
+    g_shutdown_pending = reason;
+  }
+
+  we_might_shutdown_cond.notify_one();
+  // let's wake the log_reopen_thread too
+  log_reopen_cond.notify_one();
+}
+
+/**
+ * request application shutdown.
+ *
+ * @throws std::system_error same as std::unique_lock::lock does
+ */
+void request_application_shutdown() {
+  request_application_shutdown(SHUTDOWN_REQUESTED);
+}
+
+#ifdef USE_POSIX_SIGNALS
+/**
+ * notify a "log_reopen" is requested.
+ *
+ * @throws std::system_error same as std::unique_lock::lock does
+ */
+static void request_log_reopen() {
+  {
+    std::unique_lock<std::mutex> lk(log_reopen_cond_mutex);
+    g_log_reopen_requested = true;
+  }
+  log_reopen_cond.notify_one();
+}
+#endif
+
+namespace {
+#ifdef USE_POSIX_SIGNALS
+const std::array<int, 6> g_fatal_signals{SIGSEGV, SIGABRT, SIGBUS,
+                                         SIGILL,  SIGFPE,  SIGTRAP};
+#endif
+}  // namespace
+
+static void block_all_nonfatal_signals() {
+#ifdef USE_POSIX_SIGNALS
+  sigset_t ss;
+  sigfillset(&ss);
+  // we can't block those signals globally and rely on our handler thread, as
+  // these are only received by the offending thread itself.
+  // see "man signal" for more details
+  for (const auto &sig : g_fatal_signals) {
+    sigdelset(&ss, sig);
+  }
+  if (0 != pthread_sigmask(SIG_SETMASK, &ss, nullptr)) {
+    throw std::runtime_error("pthread_sigmask() failed: " +
+                             std::string(std::strerror(errno)));
+  }
+#endif
+}
+
+#if !defined(__has_feature)
+#define __has_feature(x) 0
+#endif
+
+// GCC defines __SANITIZE_ADDRESS
+// clang has __has_feature and 'address_sanitizer'
+#if defined(__SANITIZE_ADDRESS__) || (__has_feature(address_sanitizer))
+#define HAS_FEATURE_ASAN
+#endif
+
+static void register_fatal_signal_handler() {
+  // enable a crash handler on POSIX systems if not built with ASAN
+#if defined(USE_POSIX_SIGNALS) && !defined(HAS_FEATURE_ASAN)
+#if defined(HAVE_STACKTRACE)
+  my_init_stacktrace();
+#endif  // HAVE_STACKTRACE
+
+  struct sigaction sa;
+  (void)sigemptyset(&sa.sa_mask);
+  sa.sa_flags = SA_RESETHAND;
+  sa.sa_handler = [](int sig) {
+    my_safe_printf_stderr("Application got fatal signal: %d\n", sig);
+#ifdef HAVE_STACKTRACE
+    my_print_stacktrace(nullptr, 0);
+#endif  // HAVE_STACKTRACE
+  };
+
+  for (const auto &sig : g_fatal_signals) {
+    (void)sigaction(sig, &sa, nullptr);
+  }
+#endif
+}
+
+static void start_and_detach_signal_handler_thread() {
+#ifdef USE_POSIX_SIGNALS
+  std::promise<void> signal_handler_thread_setup_done;
+
+  std::thread signal_thread([&signal_handler_thread_setup_done] {
+    mysql_harness::rename_thread("sig handler");
+
+    sigset_t ss;
+    sigemptyset(&ss);
+    sigaddset(&ss, SIGINT);
+    sigaddset(&ss, SIGTERM);
+    sigaddset(&ss, SIGHUP);
+
+    signal_handler_thread_setup_done.set_value();
+    int sig = 0;
+
+    while (true) {
+      if (0 == sigwait(&ss, &sig)) {
+        if (sig == SIGHUP) {
+          request_log_reopen();
+        } else {
+          harness_assert(sig == SIGINT || sig == SIGTERM);
+          request_application_shutdown();
+          return;
+        }
+      } else {
+        // man sigwait() says, it should only fail if we provided invalid
+        // signals.
+        harness_assert_this_should_not_execute();
+      }
+    }
+  });
+
+  // wait until the signal handler is setup
+  signal_handler_thread_setup_done.get_future().wait();
+
+  // let the signal handler thread be independent of the rest of the app
+  signal_thread.detach();
+#endif
+}
+
+static void log_reopen_thread_function() {
+  auto &logging_registry = mysql_harness::DIM::instance().get_LoggingRegistry();
+  bool request_shutdown{false};
+  {
+    std::unique_lock<std::mutex> lk(log_reopen_cond_mutex);
+    log_reopen_cond.wait(lk, [&] {
+      if (g_shutdown_pending) {
+        return true;
+      }
+      if (g_log_reopen_requested) {
+        g_log_reopen_requested = false;
+        try {
+          logging_registry.flush_all_loggers();
+        } catch (const std::exception &e) {
+          // we have to postpone the shutdown request as it locks the
+          // mutex that we currently hold
+          request_shutdown = true;
+          shutdown_fatal_error_message = e.what();
+          return true;
+        }
+      }
+      return false;
+    });
+  }
+
+  if (request_shutdown) {
+    request_application_shutdown(SHUTDOWN_FATAL_ERROR);
+  }
+}
+
+#ifdef _WIN32
+static BOOL WINAPI ctrl_c_handler(DWORD ctrl_type) {
+  if (ctrl_type == CTRL_C_EVENT || ctrl_type == CTRL_BREAK_EVENT) {
+    // user presed Ctrl+C or we got Ctrl+Break request
+    request_application_shutdown();
+    return TRUE;  // don't pass this event to further handlers
+  } else {
+    // some other event
+    return FALSE;  // let the default Windows handler deal with it
+  }
+}
+
+void register_ctrl_c_handler() {
+  if (!SetConsoleCtrlHandler(ctrl_c_handler, TRUE)) {
+    std::cerr << "Could not install Ctrl+C handler, exiting.\n";
+    exit(1);
+  }
+}
+#endif
+
 namespace mysql_harness {
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -147,7 +344,7 @@ bool PluginFuncEnv::is_running() const noexcept {
 }
 
 bool PluginFuncEnv::wait_for_stop(uint32_t milliseconds) const noexcept {
-  auto pred = [this]() noexcept -> bool { return !running_; };
+  auto pred = [this]() noexcept->bool { return !running_; };
 
   std::unique_lock<std::mutex> lock(mutex_);
   if (milliseconds)  // 0 = wait forever
@@ -314,7 +511,7 @@ void PluginThreads::wait_all_stopped(std::exception_ptr &first_exc) {
 //
 ////////////////////////////////////////////////////////////////////////////////
 
-Loader::~Loader() = default;
+Loader::~Loader() {}
 
 Loader::PluginInfo::PluginInfo(const std::string &folder,
                                const std::string &libname) {
@@ -403,7 +600,7 @@ const Plugin *Loader::load_from(const std::string &plugin_name,
   auto plugin = info.plugin();
   if ((plugin->abi_version & 0xFF00) != (PLUGIN_ABI_VERSION & 0xFF00) ||
       (plugin->abi_version & 0xFF) > (PLUGIN_ABI_VERSION & 0xFF)) {
-    std::ostringstream buffer;
+    ostringstream buffer;
     buffer.setf(std::ios::hex, std::ios::basefield);
     buffer.setf(std::ios::showbase);
     buffer << "Bad ABI version - plugin version: " << plugin->abi_version
@@ -415,8 +612,7 @@ const Plugin *Loader::load_from(const std::string &plugin_name,
   // the user might have added these by accident (for example, he
   // assumed that the array was NULL-terminated) and they can safely
   // be ignored instead of raising an error.
-  for (auto req :
-       make_range(plugin->requires_plugins, plugin->requires_length)) {
+  for (auto req : make_range(plugin->requires, plugin->requires_length)) {
     if (req != nullptr) {
       // Parse the designator to extract the plugin and constraints.
       Designator designator(req);
@@ -451,20 +647,18 @@ const Plugin *Loader::load_from(const std::string &plugin_name,
   // If all went well, we register the plugin and return a
   // pointer to it.
   plugins_.emplace(plugin_name, std::move(info));
-
+  log_debug("  plugin '%s' loaded ok", plugin_name.c_str());
   return plugin;
 }
 
 const Plugin *Loader::load(const std::string &plugin_name,
                            const std::string &key) {
-  log_debug("  loading '%s'.", plugin_name.c_str());
+  log_debug("  plugin '%s:%s' loading", plugin_name.c_str(), key.c_str());
 
   if (BuiltinPlugins::instance().has(plugin_name)) {
     Plugin *plugin = BuiltinPlugins::instance().get_plugin(plugin_name);
-    // if plugin isn't registered yet, add it
-    if (plugins_.find(plugin_name) == plugins_.end()) {
-      plugins_.emplace(plugin_name, plugin);
-    }
+    PluginInfo info(plugin);
+    plugins_.emplace(plugin_name, std::move(info));
     return plugin;
   } else {
     ConfigSection &plugin =
@@ -476,38 +670,7 @@ const Plugin *Loader::load(const std::string &plugin_name,
 }
 
 const Plugin *Loader::load(const std::string &plugin_name) {
-  log_debug("  loading '%s'.", plugin_name.c_str());
-
-  if (BuiltinPlugins::instance().has(plugin_name)) {
-    Plugin *plugin = BuiltinPlugins::instance().get_plugin(plugin_name);
-    if (plugins_.find(plugin_name) == plugins_.end()) {
-      plugins_.emplace(plugin_name, plugin);
-
-      // add config-section for builtin plugins, in case it isn't there yet
-      // as the the "start()" function otherwise isn't called by load_all()
-      if (!config_.has_any(plugin_name)) {
-        config_.add(plugin_name);
-      }
-    }
-    return plugin;
-  }
-
-  if (!config_.has_any(plugin_name)) {
-    // if no section for the plugin exists, try to load it anyway with an empty
-    // key-less section
-    //
-    // in case the plugin fails to load with bad_plugin, return bad_section to
-    // be consistent with existing behaviour
-    config_.add(plugin_name).add("library", plugin_name);
-
-    try {
-      return load_from(plugin_name, plugin_name);  // throws bad_plugin
-    } catch (const bad_plugin &) {
-      std::ostringstream buffer;
-      buffer << "Section name '" << plugin_name << "' does not exist";
-      throw bad_section(buffer.str());
-    }
-  }
+  log_debug("  plugin '%s' loading", plugin_name.c_str());
 
   Config::SectionList plugins = config_.get(plugin_name);  // throws bad_section
   if (plugins.size() > 1) {
@@ -530,14 +693,14 @@ const Plugin *Loader::load(const std::string &plugin_name) {
 
 void Loader::start() {
   // unload plugins on exit
-  Scope_guard exit_guard([this]() { unload_all(); });
+  std::shared_ptr<void> exit_guard(nullptr, [this](void *) { unload_all(); });
 
   // check if there is anything to load; if not we currently treat is as an
   // error, not letting the user to run "idle" router that would close right
   // away
   if (external_plugins_to_load_count() == 0) {
     throw std::runtime_error(
-        "Error: The service is not configured to load or start any plugin. "
+        "Error: MySQL Router not configured to load or start any plugin. "
         "Exiting.");
   }
 
@@ -563,18 +726,11 @@ size_t Loader::external_plugins_to_load_count() {
 }
 
 void Loader::load_all() {
+  log_debug("Loading all plugins.");
+
   std::string section_name;
   std::string section_key;
 
-  std::vector<std::string> loadable_plugins;
-  for (auto const &section : available()) {
-    std::tie(section_name, section_key) = section;
-
-    loadable_plugins.push_back(section_name);
-  }
-  log_debug("Loading plugins: %s.",
-            mysql_harness::join(loadable_plugins, ", ").c_str());
-
   for (auto const &section : available()) {
     try {
       std::tie(section_name, section_key) = section;
@@ -606,39 +762,74 @@ T value_or(T a, T b) {
   return a ? a : b;
 }
 
+class LogReopenThread {
+ public:
+  /**
+   * @throws std::system_error if out of threads
+   */
+  LogReopenThread() : reopen_thr_{log_reopen_thread_function} {}
+
+  /**
+   * stop the log_reopen_thread_function.
+   *
+   * @throws std::system_error from request_application_shutdown()
+   */
+  void stop() { request_application_shutdown(); }
+
+  /**
+   * join the log_reopen thread.
+   *
+   * @throws std::system_error same as std::thread::join
+   */
+  void join() { reopen_thr_.join(); }
+
+  /**
+   * destruct the thread.
+   *
+   * Same as std::thread it may call std::terminate in case the thread isn't
+   * joined yet, but joinable.
+   *
+   * In casing join() fails as best-effort, a log-message is attempted to be
+   * written.
+   */
+  ~LogReopenThread() {
+    // if it didn't throw in the constructor, it is joinable and we have to
+    // signal its shutdown
+    if (reopen_thr_.joinable()) {
+      try {
+        // if stop throws ... the join will block
+        stop();
+
+        // if join throws, log it and expect std::thread::~thread to call
+        // std::terminate
+        join();
+      } catch (const std::exception &e) {
+        try {
+          log_error("~LogReopenThread failed to join its thread: %s", e.what());
+        } catch (...) {
+          // ignore it, we did our best to tell the user why std::terminate will
+          // be called in a bit
+        }
+      }
+    }
+  }
+
+ private:
+  std::thread reopen_thr_;
+};
+
 std::exception_ptr Loader::run() {
   // initialize plugins
   std::exception_ptr first_eptr = init_all();
 
-  if (!first_eptr) {
-    try {
-      check_config_options_supported();
-    } catch (std::exception &) {
-      first_eptr = std::current_exception();
-    }
-  }
-
-  if (!first_eptr) {
-    try {
-      expose_config_all(/*initial*/ true);
-    } catch (const std::exception &e) {
-      log_warning(
-          "Failed storing plugins initial configuration in DynamicConfig: %s",
-          e.what());
-    }
-  }
-
   // run plugins if initialization didn't fail
   if (!first_eptr) {
     try {
       start_all();  // if start() throws, exception is forwarded to
                     // main_loop()
 
-      Scope_guard exit_guard([&]() {
-        if (after_first_finished_) after_first_finished_();
-      });
-
-      if (after_all_started_) after_all_started_();
+      // may throw std::system_error
+      LogReopenThread log_reopen_thread;
 
       first_eptr = main_loop();
     } catch (const std::exception &e) {
@@ -647,7 +838,7 @@ std::exception_ptr Loader::run() {
     }
   }
 
-  // not strict requirement, just good measure (they're no longer needed at
+  // not strict requiremnt, just good measure (they're no longer needed at
   // this point)
   assert(plugin_start_env_.empty());
 
@@ -683,10 +874,10 @@ static void call_plugin_function(PluginFuncEnv *env, std::exception_ptr &eptr,
                                  void (*fptr)(PluginFuncEnv *),
                                  const char *fnc_name, const char *plugin_name,
                                  const char *plugin_key = nullptr) noexcept {
-  auto handle_plugin_exception = [](std::exception_ptr &first_eptr,
-                                    const std::string &func_name,
-                                    const char *plug_name, const char *plug_key,
-                                    const std::exception *e) noexcept -> void {
+  auto handle_plugin_exception =
+      [](std::exception_ptr & first_eptr, const std::string &func_name,
+         const char *plug_name, const char *plug_key,
+         const std::exception *e) noexcept->void {
     // Plugins are not allowed to throw, so let's alert the devs. But in
     // production, we want to be robust and try to handle this gracefully
     assert(0);
@@ -718,10 +909,11 @@ static void call_plugin_function(PluginFuncEnv *env, std::exception_ptr &eptr,
 
     // error handling
     if (env->exit_ok()) {
-      if (plugin_key && strlen(plugin_key) > 0)
-        log_debug("  %s '%s:%s' succeeded.", fnc_name, plugin_name, plugin_key);
+      if (plugin_key)
+        log_debug("  plugin '%s:%s' %s exit ok", plugin_name, plugin_key,
+                  fnc_name);
       else
-        log_debug("  %s '%s' succeeded.", fnc_name, plugin_name);
+        log_debug("  plugin '%s' %s exit ok", plugin_name, fnc_name);
     } else {
       std::string message;
       if (!eptr) {
@@ -729,12 +921,11 @@ static void call_plugin_function(PluginFuncEnv *env, std::exception_ptr &eptr,
       } else {
         std::tie(message, std::ignore) = env->pop_error();
       }
-
-      if (plugin_key && strlen(plugin_key) > 0)
-        log_error("  %s '%s:%s' failed: %s", fnc_name, plugin_name, plugin_key,
-                  message.c_str());
+      if (plugin_key)
+        log_error("  plugin '%s:%s' %s failed: %s", plugin_name, plugin_key,
+                  fnc_name, message.c_str());
       else
-        log_error("  %s '%s' failed: %s", fnc_name, plugin_name,
+        log_error("  plugin '%s' %s failed: %s", plugin_name, fnc_name,
                   message.c_str());
     }
 
@@ -747,30 +938,28 @@ static void call_plugin_function(PluginFuncEnv *env, std::exception_ptr &eptr,
 
 // returns first exception triggered by init()
 std::exception_ptr Loader::init_all() {
+  log_debug("Initializing all plugins.");
+
   if (!topsort()) throw std::logic_error("Circular dependencies in plugins");
   order_.reverse();  // we need reverse-topo order for non-built-in plugins
 
-  // build is list of plugins that have an init function, in init-order.
-  std::vector<std::string> plugin_names;
-  for (const auto &plugin_name : order_) {
-    PluginInfo &info = plugins_.at(plugin_name);
-
-    if (info.plugin()->init) {
-      plugin_names.push_back(plugin_name);
+  // we put the built-in plugins at the beginning
+  for (const std::pair<const std::string, PluginInfo> &plugin : plugins_) {
+    if (BuiltinPlugins::instance().has(plugin.first)) {
+      order_.push_front(plugin.first);
     }
   }
 
-  log_debug("Initializing plugins: %s.",
-            mysql_harness::join(plugin_names, ", ").c_str());
-
   for (auto it = order_.begin(); it != order_.end(); ++it) {
     const std::string &plugin_name = *it;
     PluginInfo &info = plugins_.at(plugin_name);
 
     if (!info.plugin()->init) {
+      log_debug("  plugin '%s' doesn't implement init()", plugin_name.c_str());
       continue;
     }
 
+    log_debug("  plugin '%s' initializing", plugin_name.c_str());
     PluginFuncEnv env(&appinfo_, nullptr);
 
     std::exception_ptr eptr;
@@ -788,50 +977,21 @@ std::exception_ptr Loader::init_all() {
   return nullptr;
 }
 
-static std::string section_to_string(const ConfigSection *section) {
-  if (section->key.empty()) return section->name;
-
-  return section->name + ":" + section->key;
-}
-
 // forwards first exception triggered by start() to main_loop()
 void Loader::start_all() {
-  std::vector<std::string> startable_sections;
+  log_debug("Starting all plugins.");
 
-  for (const ConfigSection *section : config_.sections()) {
-    const auto &plugin = plugins_.at(section->name);
+  // block non-fatal signal handling for all threads
+  //
+  // - no other thread than the signal-handler thread should receive signals
+  // - syscalls should not get interrupted by signals either
+  //
+  // on windows, this is a no-op
+  block_all_nonfatal_signals();
 
-    const bool has_start = plugin.plugin()->start;
-    const bool declares_readiness = plugin.plugin()->declares_readiness;
-
-    if (has_start || declares_readiness) {
-      const auto section_name = section_to_string(section);
-      if (has_start) {
-        startable_sections.push_back(section_name);
-      }
-
-      if (declares_readiness) {
-        waitable_services_.push_back(section_name);
-      }
-    }
-  }
-
-  if (!startable_sections.empty()) {
-    log_debug("Starting: %s.",
-              mysql_harness::join(startable_sections, ", ").c_str());
-  }
-
-  // if there are no services that we should wait for let's declare the
-  // readiness right away
-  if (waitable_services_.empty()) {
-    log_debug("Service ready!");
-    notify_ready();
-  } else {
-    log_debug("Waiting for readiness of: %s",
-              mysql_harness::join(waitable_services_, ", ").c_str());
-
-    num_of_non_ready_services = waitable_services_.size();
-  }
+  // for the fatal signals we want to have a handler that prints the stack-trace
+  // if possible
+  register_fatal_signal_handler();
 
   try {
     // start all the plugins (call plugin's start() function)
@@ -840,6 +1000,9 @@ void Loader::start_all() {
       void (*fptr)(PluginFuncEnv *) = plugin.plugin()->start;
 
       if (!fptr) {
+        log_debug("  plugin '%s:%s' doesn't implement start()",
+                  section->name.c_str(), section->key.c_str());
+
         // create a env object for later
         assert(plugin_start_env_.count(section) == 0);
         plugin_start_env_[section] =
@@ -853,6 +1016,9 @@ void Loader::start_all() {
 
       // plugin start() will run in this new thread
       std::thread plugin_thread([fptr, section, &env_promise, this]() {
+        log_debug("  plugin '%s:%s' starting", section->name.c_str(),
+                  section->key.c_str());
+
         // init env object and unblock harness thread
         std::shared_ptr<PluginFuncEnv> this_thread_env =
             std::make_shared<PluginFuncEnv>(nullptr, section, true);
@@ -863,13 +1029,11 @@ void Loader::start_all() {
         call_plugin_function(this_thread_env.get(), eptr, fptr, "start",
                              section->name.c_str(), section->key.c_str());
 
-        // notify the signal-handler's cond-var about the plugin's exit-status
-        ProcessStateComponent::get_instance()
-            .shutdown_pending()
-            .serialize_with_cv([this, &eptr](auto /* pending */, auto &cv) {
-              plugin_threads_.push_exit_status(std::move(eptr));
-              cv.notify_one();
-            });
+        {
+          std::lock_guard<std::mutex> lock(we_might_shutdown_cond_mutex);
+          plugin_threads_.push_exit_status(std::move(eptr));
+        }
+        we_might_shutdown_cond.notify_one();
       });
 
       // we could combine the thread creation with emplace_back
@@ -891,6 +1055,16 @@ void Loader::start_all() {
   } catch (const std::system_error &e) {
     throw std::system_error(e.code(), "starting plugin-threads failed");
   }
+
+  try {
+    // We wait with this until after we launch all plugin threads, to avoid
+    // a potential race if a signal was received while plugins were still
+    // launching.
+    start_and_detach_signal_handler_thread();
+  } catch (const std::system_error &e) {
+    // should we unblock the signals again?
+    throw std::system_error(e.code(), "starting signal-handler-thread failed");
+  }
 }
 
 /**
@@ -907,38 +1081,44 @@ void Loader::start_all() {
  * @retval nullptr if no exception was returned
  */
 std::exception_ptr Loader::main_loop() {
-  notify_status("running");
+  // RouterRoutingTest::RoutingPluginCantSpawnMoreThreads is waiting for this
+  // log-message to appear in the log to get a predictible test-scenario.
+  //
+  // Changing or moving this message, will break that test.
+  log_debug("Running.");
 
   std::exception_ptr first_eptr;
-
   // wait for a reason to shutdown
-  ProcessStateComponent::get_instance().shutdown_pending().wait(
-      [&first_eptr, this](auto pending) {
-        // external shutdown
-        if (pending.reason() == ShutdownPending::Reason::REQUESTED) return true;
+  {
+    std::unique_lock<std::mutex> lk(we_might_shutdown_cond_mutex);
 
-        // shutdown due to a fatal error originating from Loader and its callees
-        // (but NOT from plugins)
-        if (pending.reason() == ShutdownPending::Reason::FATAL_ERROR) {
-          // there is a request to shut down due to a fatal error; generate an
-          // exception with requested message so that it bubbles up and ends up
-          // on the console as an error message
-          try {
-            throw std::runtime_error(pending.message());
-          } catch (const std::exception &) {
-            first_eptr = std::current_exception();  // capture
-          }
-          return true;
+    we_might_shutdown_cond.wait(lk, [&first_eptr, this] {
+      // external shutdown
+      if (g_shutdown_pending == SHUTDOWN_REQUESTED) return true;
+
+      // shutdown due to a fatal error originating from Loader and its callees
+      // (but NOT from plugins)
+      if (g_shutdown_pending == SHUTDOWN_FATAL_ERROR) {
+        // there is a request to shut down due to a fatal error; generate an
+        // exception with requested message so that it bubbles up and ends up on
+        // the console as an error message
+        try {
+          throw std::runtime_error(shutdown_fatal_error_message);
+        } catch (const std::exception &) {
+          first_eptr = std::current_exception();  // capture
         }
+        return true;
+      }
 
-        plugin_threads_.try_stopped(first_eptr);
-        if (first_eptr) return true;
+      plugin_threads_.try_stopped(first_eptr);
+      if (first_eptr) return true;
 
-        // all plugins stop successfully
-        if (plugin_threads_.running() == 0) return true;
+      // all plugins stop successfully
+      if (plugin_threads_.running() == 0) return true;
 
-        return false;
-      });
+      return false;
+    });
+  }
 
   return value_or(first_eptr, stop_and_wait_all());
 }
@@ -972,23 +1152,7 @@ std::exception_ptr Loader::stop_and_wait_all() {
 std::exception_ptr Loader::stop_all() {
   // This function runs exactly once - it will be called even if all plugins
   // exit by themselves (thus there's nothing to stop).
-  std::vector<std::string> stoppable_sections;
-
-  for (const ConfigSection *section : config_.sections()) {
-    PluginInfo &plugin = plugins_.at(section->name);
-
-    if (plugin.plugin()->stop) {
-      stoppable_sections.push_back(section_to_string(section));
-    }
-  }
-
-  if (stoppable_sections.empty()) {
-    log_debug("Shutting down.");
-  } else {
-    log_debug("Shutting down. Signaling stop to: %s.",
-              mysql_harness::join(stoppable_sections, ", ").c_str());
-  }
-  notify_stopping();
+  log_debug("Shutting down. Stopping all plugins.");
 
   // iterate over all plugin instances
   std::exception_ptr first_eptr;
@@ -1002,7 +1166,14 @@ std::exception_ptr Loader::stop_all() {
     // flag plugin::start() to exit (if one exists and it's running)
     plugin_start_env_[section]->clear_running();
 
-    if (!fptr) continue;
+    if (!fptr) {
+      log_debug("  plugin '%s:%s' doesn't implement stop()",
+                section->name.c_str(), section->key.c_str());
+      continue;
+    }
+
+    log_debug("  plugin '%s:%s' stopping", section->name.c_str(),
+              section->key.c_str());
 
     PluginFuncEnv stop_env(nullptr, section);
     call_plugin_function(&stop_env, first_eptr, fptr, "stop",
@@ -1015,37 +1186,25 @@ std::exception_ptr Loader::stop_all() {
 
 // returns first exception triggered by deinit()
 std::exception_ptr Loader::deinit_all() {
+  log_debug("Deinitializing all plugins.");
+
   // we could just reverse order_ and that would work too,
   // but by leaving it intact it's easier to unit-test it
   std::list<std::string> deinit_order = order_;
   deinit_order.reverse();
 
-  {
-    std::vector<std::string> deinitable_sections;
-    for (const std::string &plugin_name : deinit_order) {
-      const PluginInfo &info = plugins_.at(plugin_name);
-
-      if (info.plugin()->deinit) {
-        deinitable_sections.push_back(plugin_name);
-      }
-    }
-
-    if (!deinitable_sections.empty()) {
-      log_debug("Deinitializing plugins: %s.",
-                mysql_harness::join(deinitable_sections, ", ").c_str());
-    }
-  }
-
   // call deinit() on all plugins that support the call
   std::exception_ptr first_eptr;
-
   for (const std::string &plugin_name : deinit_order) {
     const PluginInfo &info = plugins_.at(plugin_name);
 
     if (!info.plugin()->deinit) {
+      log_debug("  plugin '%s' doesn't implement deinit()",
+                plugin_name.c_str());
       continue;
     }
 
+    log_debug("  plugin '%s' deinitializing", plugin_name.c_str());
     PluginFuncEnv env(&appinfo_, nullptr);
 
     call_plugin_function(&env, first_eptr, info.plugin()->deinit, "deinit",
@@ -1059,9 +1218,13 @@ bool Loader::topsort() {
   std::map<std::string, Loader::Status> status;
   std::list<std::string> order;
 
+  // for the non-builtin plugins do the sorting that takes their dependencies
+  // into account
   for (std::pair<const std::string, PluginInfo> &plugin : plugins_) {
-    bool succeeded = visit(plugin.first, &status, &order);
-    if (!succeeded) return false;
+    if (!BuiltinPlugins::instance().has(plugin.first)) {
+      bool succeeded = visit(plugin.first, &status, &order);
+      if (!succeeded) return false;
+    }
   }
 
   order_.swap(order);
@@ -1085,7 +1248,7 @@ bool Loader::visit(const std::string &designator,
       (*status)[info.plugin] = Status::ONGOING;
       if (const Plugin *plugin = plugins_.at(info.plugin).plugin()) {
         for (auto required :
-             make_range(plugin->requires_plugins, plugin->requires_length)) {
+             make_range(plugin->requires, plugin->requires_length)) {
           assert(required != nullptr);
           bool succeeded = visit(required, status, order);
           if (!succeeded) return false;
@@ -1099,112 +1262,13 @@ bool Loader::visit(const std::string &designator,
   return true;
 }
 
-static void report_unsupported_option(const std::string &section,
-                                      const std::string &option,
-                                      const bool error_out) {
-  const std::string msg =
-      "option '" + section + "." + option + "' is not supported";
-  if (error_out) {
-    throw std::runtime_error(msg);
-  } else {
-    log_warning("%s", msg.c_str());
-  }
-}
-
-void Loader::check_config_options_supported() {
-  check_default_config_options_supported();
-
-  const bool error_out = config_.error_on_unsupported_option;
-
-  for (const ConfigSection *section : config_.sections()) {
-    const auto &plugin = plugins_.at(section->name).plugin();
-    for (const auto &option : section->get_options()) {
-      if (option.first == "library") continue;
-
-      bool is_supported{false};
-      for (auto supported_option : make_range(
-               plugin->supported_options, plugin->supported_options_length)) {
-        if (supported_option != nullptr) {
-          if (option.first == supported_option) {
-            is_supported = true;
-            break;
-          }
-        }
-      }
-
-      if (!is_supported) {
-        report_unsupported_option(section->name, option.first, error_out);
-      }
-    }
-  }
-}
-
-void Loader::check_default_config_options_supported() {
-  const auto &default_section = config_.get_default_section();
-  const bool error_out = config_.error_on_unsupported_option;
-
-  for (const auto &option : default_section.get_options()) {
-    // this option is supported by the Loader
-    if (mysql_harness::str_in_collection(loader_supported_options,
-                                         option.first)) {
-      continue;
-    }
-
-    // this option is supported by the App{
-    if (mysql_harness::str_in_collection(supported_app_options_,
-                                         option.first)) {
-      continue;
-    }
-
-    bool option_supported{false};
-    for (const mysql_harness::ConfigSection *section : config_.sections()) {
-      const auto &plugin = plugins_.at(section->name).plugin();
-      for (auto supported_option : make_range(
-               plugin->supported_options, plugin->supported_options_length)) {
-        if (supported_option != nullptr) {
-          // this option is supported by configured plugin
-          if (option.first == supported_option) option_supported = true;
-        }
-      }
-      if (option_supported) {
-        // go to the outer loop to check next option
-        break;
-      }
-      // else check next plugin section
-    }
-
-    if (!option_supported) {
-      report_unsupported_option("DEFAULT", option.first, error_out);
-    }
-  }
-}
-
-void Loader::expose_config_all(const bool initial) {
-  // expose "common" application options
-  if (expose_app_config_clb_) {
-    expose_app_config_clb_(initial, config_.get_default_section());
-  }
-
-  // let each plugin expose its options
-  PluginFuncEnv env(&appinfo_, nullptr);
-  for (const ConfigSection *section : config_.sections()) {
-    const auto &plugin = plugins_.at(section->name).plugin();
-    if (plugin->expose_configuration) {
-      plugin->expose_configuration(&env, section->key.c_str(), initial);
-    }
-  }
-}
-
-void on_service_ready(const std::string &name) {
-  log_debug("  ready '%s'", name.c_str());
-  if (--num_of_non_ready_services == 0) {
-    log_debug("Ready, signaling notify socket");
-    notify_ready();
-  }
-}
-
-void on_service_ready(PluginFuncEnv *plugin_env) {
-  return on_service_ready(section_to_string(get_config_section(plugin_env)));
-}
-
 }  // namespace mysql_harness
+
+// unit test access - DON'T USE IN PRODUCTION CODE!
+// (unfortunately we cannot guard this with #ifdef FRIEND_TEST)
+namespace unittest_backdoor {
+HARNESS_EXPORT
+void set_shutdown_pending(bool shutdown_pending) {
+  g_shutdown_pending = shutdown_pending ? SHUTDOWN_REQUESTED : SHUTDOWN_NONE;
+}
+}  // namespace unittest_backdoor
