@@ -1,17 +1,16 @@
 /*
-  Copyright (c) 2016, 2024, Oracle and/or its affiliates.
+  Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -41,7 +40,6 @@
 #include <condition_variable>
 #include <cstdarg>
 #include <cstdlib>
-#include <cstring>
 #include <mutex>
 #include <stdexcept>
 #include <thread>
@@ -50,6 +48,7 @@
 #include "mysql/harness/config_parser.h"
 #include "mysql/harness/logging/logging.h"
 #include "mysql/harness/plugin.h"
+#include "router_config.h"
 
 #include "my_compiler.h"
 
@@ -59,6 +58,14 @@ namespace mysql_harness {
 class PluginFuncEnv;
 }
 
+using mysql_harness::AppInfo;
+using mysql_harness::ARCHITECTURE_DESCRIPTOR;
+using mysql_harness::ConfigSection;
+using mysql_harness::kRuntimeError;
+using mysql_harness::Plugin;
+using mysql_harness::PLUGIN_ABI_VERSION;
+using mysql_harness::PluginFuncEnv;
+
 const int kExitCheckInterval = 1;
 const int kExitOnStopShortTimeout = 100;
 const int kExitOnStopLongTimeout = 60 * 1000;
@@ -156,7 +163,7 @@ std::mutex g_strategies_mtx;
 
 // called at the earliest opportunity, needs to run only once
 // (since last reset)
-void init_exit_strategies(const mysql_harness::ConfigSection *section) {
+void init_exit_strategies(const ConfigSection *section) {
   std::lock_guard<std::mutex> lock(g_strategies_mtx);
 
   // running more than once doesn't change anything, just wastes cycles
@@ -196,7 +203,7 @@ void init_exit_strategies(const mysql_harness::ConfigSection *section) {
   // clang-format on
 
   // process configuration
-  for (const char *func : {"init", "start", "stop", "deinit"}) {
+  for (const std::string &func : {"init", "start", "stop", "deinit"}) {
     if (section->has(func)) {
       const std::string &line = section->get(func);
 
@@ -216,10 +223,10 @@ void init_exit_strategies(const mysql_harness::ConfigSection *section) {
       } else if (line.find("error") != std::string::npos) {
         g_strategies[section->key].exit_type[func] = ET_ERROR;
       } else if (line.find("exitonstop_s") != std::string::npos &&
-                 strcmp(func, "start") == 0) {
+                 func == "start") {
         g_strategies[section->key].exit_type[func] = ET_EXIT_ON_STOP_SYNC;
       } else if (line.find("exitonstop") != std::string::npos &&
-                 strcmp(func, "start") == 0) {
+                 func == "start") {
         g_strategies[section->key].exit_type[func] = ET_EXIT_ON_STOP;
       } else if (line.find("exit") != std::string::npos) {
         g_strategies[section->key].exit_type[func] = ET_EXIT;
@@ -231,8 +238,7 @@ void init_exit_strategies(const mysql_harness::ConfigSection *section) {
   }
 }
 
-void execute_exit_strategy(const std::string &func,
-                           mysql_harness::PluginFuncEnv *env) {
+void execute_exit_strategy(const std::string &func, PluginFuncEnv *env) {
   // init() and deinit() are called only once per plugin (not per plugin
   // instance), but we need an instance name for our logic to work, therefore
   // we pick the first plugin instance in such case
@@ -273,15 +279,14 @@ void execute_exit_strategy(const std::string &func,
     case ET_ERROR:
       log_info(notify, key_for_log, "  lifecycle:%s %s():ERROR", key_for_log,
                func.c_str());
-      set_error(env, mysql_harness::kRuntimeError,
-                "lifecycle:%s %s(): I'm returning error!", key_for_log,
-                func.c_str());
+      set_error(env, kRuntimeError, "lifecycle:%s %s(): I'm returning error!",
+                key_for_log, func.c_str());
       return;
 
     case ET_ERROR_EMPTY:
       log_info(notify, key_for_log, "  lifecycle:%s %s():ERROR_EMPTY",
                key_for_log, func.c_str());
-      set_error(env, mysql_harness::kRuntimeError, nullptr);
+      set_error(env, kRuntimeError, nullptr);
       return;
 
     case ET_EXIT_ON_STOP_SHORT_TIMEOUT:
@@ -356,18 +361,19 @@ void execute_exit_strategy(const std::string &func,
 #define LIFECYCLE_API
 #endif
 
-static std::array<const char *, 2> requires_plugins = {
+static std::array<const char *, 2> requires = {
     "routertestplugin_magic (>>1.0)",
     "routertestplugin_lifecycle3",
 };
 
-static void init(mysql_harness::PluginFuncEnv *env) {
-  const auto *info = get_app_info(env);
+static void init(PluginFuncEnv *env) {
+  const AppInfo *info = get_app_info(env);
 
   // init() and deinit() are called only once per plugin (not per plugin
   // instance), but we need an instance name for our logic to work, therefore
   // we pick the first plugin instance in such case
-  const auto *section = info->config->get("routertestplugin_lifecycle").front();
+  const ConfigSection *section =
+      info->config->get("routertestplugin_lifecycle").front();
 
   // only 3 predefined instances are supported
   harness_assert(section->key == "instance1" || section->key == "instance2" ||
@@ -379,8 +385,8 @@ static void init(mysql_harness::PluginFuncEnv *env) {
   execute_exit_strategy("init", env);
 }
 
-static void start(mysql_harness::PluginFuncEnv *env) {
-  const auto *section = get_config_section(env);
+static void start(PluginFuncEnv *env) {
+  const ConfigSection *section = get_config_section(env);
 
   log_info(true, section->key, "lifecycle:%s start():begin",
            section->key.c_str());
@@ -389,8 +395,8 @@ static void start(mysql_harness::PluginFuncEnv *env) {
   execute_exit_strategy("start", env);
 }
 
-static void stop(mysql_harness::PluginFuncEnv *env) {
-  const auto *section = get_config_section(env);
+static void stop(PluginFuncEnv *env) {
+  const ConfigSection *section = get_config_section(env);
 
   log_info(false, section->key, "lifecycle:%s stop():begin",
            section->key.c_str());
@@ -399,13 +405,14 @@ static void stop(mysql_harness::PluginFuncEnv *env) {
   execute_exit_strategy("stop", env);
 }
 
-static void deinit(mysql_harness::PluginFuncEnv *env) {
-  const auto *info = get_app_info(env);
+static void deinit(PluginFuncEnv *env) {
+  const AppInfo *info = get_app_info(env);
 
   // init() and deinit() are called only once per plugin (not per plugin
   // instance), but we need an instance name for our logic to work, therefore
   // we pick the first plugin instance in such case
-  const auto *section = info->config->get("routertestplugin_lifecycle").front();
+  const ConfigSection *section =
+      info->config->get("routertestplugin_lifecycle").front();
 
   // only 3 predefined instances are supported
   harness_assert(section->key == "instance1" || section->key == "instance2" ||
@@ -418,23 +425,19 @@ static void deinit(mysql_harness::PluginFuncEnv *env) {
 }
 
 extern "C" {
-LIFECYCLE_API mysql_harness::Plugin harness_plugin_routertestplugin_lifecycle =
-    {
-        mysql_harness::PLUGIN_ABI_VERSION,       // abi-version
-        mysql_harness::ARCHITECTURE_DESCRIPTOR,  // arch
-        "Lifecycle test plugin",                 // name
-        VERSION_NUMBER(1, 0, 0),
-        // requires_plugins
-        requires_plugins.size(), requires_plugins.data(),
-        // conflicts
-        0, nullptr,
-        init,    // init
-        deinit,  // deinit
-        start,   // start
-        stop,    // stop
-        false,   // declares_readiness
-        0, nullptr,
-        nullptr,  // expose_configuration
+LIFECYCLE_API Plugin harness_plugin_routertestplugin_lifecycle = {
+    PLUGIN_ABI_VERSION,
+    ARCHITECTURE_DESCRIPTOR,
+    "Lifecycle test plugin",
+    VERSION_NUMBER(1, 0, 0),
+    requires.size(),
+    requires.data(),
+    0,        // \_ conflicts
+    nullptr,  // /
+    init,     // init
+    deinit,   // deinit
+    start,    // start
+    stop,     // stop
 };
 
 LIFECYCLE_API void lifecycle_init(int flags) {
@@ -452,7 +455,7 @@ LIFECYCLE_API void lifecycle_init(int flags) {
     std::lock_guard<std::mutex> lock(g_strategies_mtx);
     g_strategies.clear();
 
-    for (const char *key : {"instance1", "instance2", "instance3"}) {
+    for (const std::string &key : {"instance1", "instance2", "instance3"}) {
       g_strategies[key].strategy_set = false;  // optimisation,
     }                                          // doesn't affect behavior
   }
