@@ -1,17 +1,16 @@
 /*
-  Copyright (c) 2015, 2024, Oracle and/or its affiliates.
+  Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,7 +28,6 @@
 ////////////////////////////////////////
 // Standard include files
 #include <algorithm>
-#include <map>
 #include <stdexcept>
 #include <string>
 #include <vector>
@@ -39,16 +37,22 @@
 #include <gmock/gmock.h>
 #include <gtest/gtest.h>
 
-#include "mysql/harness/filesystem.h"  // Path
+#include "mysql/harness/filesystem.h"
 #include "mysql/harness/loader.h"
 #include "mysql/harness/plugin.h"
 
-#include "exception.h"                   // bad_plugin
-#include "mysql/harness/string_utils.h"  // split_string
+#include "dim.h"
+#include "exception.h"
+#include "mysql/harness/string_utils.h"
+#include "utilities.h"
+
+////////////////////////////////////////
+// Test plugin include files
+#include "magic.h"
 
 ////////////////////////////////////////
 // Test system include files
-#include "test/helpers.h"  // EXPECT_SECTION_AVAILABLE
+#include "test/helpers.h"
 
 using mysql_harness::bad_section;
 using mysql_harness::Loader;
@@ -59,34 +63,25 @@ using testing::UnorderedElementsAre;
 
 Path g_here;
 
-class TestLoader : public Loader {
- public:
-  using Loader::Loader;
-
-  const Plugin *load(const std::string &plugin_name) {
-    return Loader::load(plugin_name);
-  }
-
-  const Plugin *load(const std::string &plugin_name, const std::string &key) {
-    return Loader::load(plugin_name, key);
-  }
-};
-
 class LoaderTest : public ::testing::TestWithParam<const char *> {
  protected:
   void SetUp() override {
+    std::map<std::string, std::string> params;
+    params["program"] = "harness";
     test_data_dir_ = mysql_harness::get_tests_data_dir(g_here.str());
+    params["prefix"] = test_data_dir_;
 
-    config_ = std::make_unique<mysql_harness::LoaderConfig>(
-        std::map<std::string, std::string>{{
-            {"program", "harness"},
-            {"prefix", test_data_dir_},
-        }},
-        std::vector<std::string>(), mysql_harness::Config::allow_keys);
-    loader = std::make_unique<TestLoader>("harness", *config_);
+    config_.reset(new mysql_harness::LoaderConfig(
+        params, std::vector<std::string>(), mysql_harness::Config::allow_keys));
+    loader = new Loader("harness", *config_);
   }
 
-  std::unique_ptr<TestLoader> loader;
+  void TearDown() override {
+    delete loader;
+    loader = nullptr;
+  }
+
+  Loader *loader;
   std::unique_ptr<mysql_harness::LoaderConfig> config_;
   std::string test_data_dir_;
 };
@@ -103,51 +98,43 @@ TEST_P(LoaderReadTest, Available) {
   auto lst = loader->available();
   EXPECT_EQ(5U, lst.size());
 
-  EXPECT_SECTION_AVAILABLE("routertestplugin_example", loader.get());
-  EXPECT_SECTION_AVAILABLE("routertestplugin_magic", loader.get());
+  EXPECT_SECTION_AVAILABLE("routertestplugin_example", loader);
+  EXPECT_SECTION_AVAILABLE("routertestplugin_magic", loader);
 }
 
-TEST_P(LoaderReadTest, load_non_existant_fails) {
-  // Test that loading something non-existent works
+TEST_P(LoaderReadTest, Loading) {
+  // These should fail, for different reasons
+
+  // Test that loading something non-existant works
   EXPECT_THROW(loader->load("nonexistant-plugin"), bad_section);
-}
 
-TEST_P(LoaderReadTest, load_missing_dep_fails) {
   // Dependent plugin does not exist
   EXPECT_THROW(loader->load("routertestplugin_bad_one"), bad_section);
-}
 
-TEST_P(LoaderReadTest, load_wrong_version) {
   // Wrong version of dependent sections
   EXPECT_THROW(loader->load("routertestplugin_bad_two"), bad_plugin);
-}
 
-TEST_P(LoaderReadTest, load_example_succeeds) {
   // These should all be OK.
   const Plugin *ext1 = loader->load("routertestplugin_example", "one");
   EXPECT_NE(ext1, nullptr);
   EXPECT_STREQ("An example plugin", ext1->brief);
-}
 
-TEST_P(LoaderReadTest, load_example_section_two_succeeds) {
   const Plugin *ext2 = loader->load("routertestplugin_example", "two");
   EXPECT_NE(ext2, nullptr);
   EXPECT_STREQ("An example plugin", ext2->brief);
-}
 
-TEST_P(LoaderReadTest, load_magic_succeeds) {
   const Plugin *ext3 = loader->load("routertestplugin_magic");
   EXPECT_NE(ext3, nullptr);
   EXPECT_STREQ("A magic plugin", ext3->brief);
 }
 
-static const char *good_cfgs[] = {
+const char *good_cfgs[] = {
     "tests-good-1.cfg",
     "tests-good-2.cfg",
 };
 
-INSTANTIATE_TEST_SUITE_P(TestLoaderGood, LoaderReadTest,
-                         ::testing::ValuesIn(good_cfgs));
+INSTANTIATE_TEST_CASE_P(TestLoaderGood, LoaderReadTest,
+                        ::testing::ValuesIn(good_cfgs));
 
 TEST_P(LoaderTest, BadSection) {
   EXPECT_THROW(loader->get_config().read(Path(test_data_dir_).join(GetParam())),
@@ -178,16 +165,14 @@ TEST(TestStart, StartLogger) {
 #endif
 
 TEST(TestStart, StartFailure) {
+  std::map<std::string, std::string> params;
+  params["program"] = "harness";
   std::string test_data_dir = mysql_harness::get_tests_data_dir(g_here.str());
+  params["prefix"] = test_data_dir;
 
-  mysql_harness::LoaderConfig config(std::map<std::string, std::string>{{
-                                         {"program", "harness"},
-                                         {"prefix", test_data_dir},
-                                     }},
-                                     std::vector<std::string>(),
+  mysql_harness::LoaderConfig config(params, std::vector<std::string>(),
                                      mysql_harness::Config::allow_keys);
   config.read(Path(test_data_dir).join("tests-start-1.cfg"));
-
   mysql_harness::Loader loader("harness", config);
   try {
     loader.start();
@@ -205,8 +190,8 @@ const char *bad_cfgs[] = {
     "tests-bad-3.cfg",
 };
 
-INSTANTIATE_TEST_SUITE_P(TestLoaderBad, LoaderTest,
-                         ::testing::ValuesIn(bad_cfgs));
+INSTANTIATE_TEST_CASE_P(TestLoaderBad, LoaderTest,
+                        ::testing::ValuesIn(bad_cfgs));
 
 /*
    @test arch-descriptor has 3 slashes
