@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2016, 2023, Oracle and/or its affiliates.
+  Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -29,6 +29,12 @@
 #include <ctime>
 #include <stdexcept>
 
+#ifdef _WIN32
+#include <process.h>  // getpid()
+#else
+#include <unistd.h>  // unlink
+#endif
+
 ////////////////////////////////////////
 // Third-party include'files
 #include <gmock/gmock.h>
@@ -36,6 +42,8 @@
 
 ////////////////////////////////////////
 // Internal interfaces
+#include "../src/utilities.h"  // string_format()
+#include "common.h"
 #include "dim.h"
 #include "include/magic.h"
 #include "mysql/harness/filesystem.h"
@@ -43,12 +51,7 @@
 #include "mysql/harness/logging/handler.h"
 #include "mysql/harness/logging/logging.h"
 #include "mysql/harness/logging/registry.h"
-#include "mysql/harness/stdx/filesystem.h"
-#include "mysql/harness/stdx/process.h"
-#include "mysql/harness/utility/string.h"  // string_format()
-#include "scope_guard.h"
 #include "test/helpers.h"
-#include "test/temp_directory.h"
 
 using mysql_harness::Path;
 using mysql_harness::logging::FileHandler;
@@ -65,7 +68,11 @@ using mysql_harness::logging::LogTimestampPrecision;
 using mysql_harness::logging::Record;
 using mysql_harness::logging::StreamHandler;
 
+#if GTEST_HAS_COMBINE
+// only available if the system has <tr1/tuple> [if not gtest's own, minimal
+// tr1/tuple is used.
 using testing::Combine;
+#endif
 using testing::ContainsRegex;
 using testing::EndsWith;
 using testing::Eq;
@@ -232,7 +239,7 @@ TEST_F(LoggingLowLevelTest, test_logger_update) {
     EXPECT_EQ(3u, g_registry->get_logger("foo").get_handler_names().size());
   }
 
-  // detaching non-existent handlers is optionally verified by detach_handler().
+  // detaching non-existant handlers is optionally verified by detach_handler().
   // It is not something that concerns update_logger(), since it only sees
   // the Logger object after all the handlers that were supposed to be detached,
   // got detached
@@ -267,11 +274,10 @@ TEST_F(LoggingLowLevelTest, test_logger_update) {
 TEST(FunctionalTest, ThisMustRunAsFirst) { init_test_logger(); }
 
 TEST(FunctionalTest, LogFromUnregisteredModule) {
-  // Test a scenario when no domain logger has been added yet.
-  //
-  // Logging should fall back to using application ("main") logger's
-  // configuration, which is always added by the application (init_log() in
-  // main(), in our case), but use the "log domain"
+  // Test a scenario when no domain logger has been added yet. Logging should
+  // fall back to using application ("main") logger, which is always added by
+  // the application (init_log() in main(), in our case), albeit with an extra
+  // error message preceding it.
 
   std::stringstream buffer;
   auto handler = std::make_shared<StreamHandler>(buffer);
@@ -281,14 +287,19 @@ TEST(FunctionalTest, LogFromUnregisteredModule) {
   log_info("Test message from an unregistered module");
   std::string log = buffer.str();
 
-  // log message should be something like:
-  // 2017-04-12 14:05:31 my_domain INFO [7ffff7fd5780] Test message from an
+  // log message should be something like (2 lines):
+  // 2017-04-12 14:05:31 main ERROR [7ffff7fd5780] Module 'my_domain' not
+  // registered with logger - logging the following message as 'main' instead
+  // 2017-04-12 14:05:31 main INFO [7ffff7fd5780] Test message from an
   // unregistered module
-  EXPECT_THAT(log, ::testing::Not(::testing::HasSubstr(" main ERROR")));
-
-  EXPECT_THAT(log, ::testing::HasSubstr(" my_domain INFO"));
-  EXPECT_THAT(
-      log, ::testing::HasSubstr(" Test message from an unregistered module\n"));
+  EXPECT_NE(log.npos, log.find(" main ERROR"));
+  EXPECT_NE(log.npos,
+            log.find(" Module 'my_domain' not registered with logger - logging "
+                     "the following message as 'main' instead\n"));
+  size_t first_endl = log.find('\n');
+  EXPECT_NE(log.npos, log.find(" main INFO", first_endl));
+  EXPECT_NE(log.npos, log.find(" Test message from an unregistered module\n",
+                               first_endl));
 
   // clean up
   g_registry->remove_handler(StreamHandler::kDefaultName);
@@ -297,7 +308,7 @@ TEST(FunctionalTest, LogFromUnregisteredModule) {
 TEST(FunctionalTest, LogOnDanglingHandlerReference) {
   // NOTE: "a_gonner" and "z_stayer" are named like that to ensure that
   // iterating over the container (std::set<std::string>) inside
-  // Logger::handler() will process "a_gonner" first. std::set makes guarantee
+  // Logger::handler() will process "a_gonner" first. std::set makes guarrantee
   // that iterating over its elements will be in ascending element order, which
   // means alphabetical order in case of std::string. By having those two named
   // like that, we additionally verify that logging to a valid handler will
@@ -320,12 +331,12 @@ TEST(FunctionalTest, LogOnDanglingHandlerReference) {
   // now remove first handler
   g_registry->remove_handler("a_gonner");
 
-  // and try to log with the logger still holding a reference to it.
+  // and try to log with the logger still holding a referece to it.
   // Logger::handle() should deal with it properly - it should log
   // to all (still existing) handlers ("z_stayer" in this case).
   EXPECT_NO_THROW(
-      l.handle(Record{LogLevel::kWarning, stdx::this_process::get_id(),
-                      kDefaultTimepoint, "my_logger", "Test message"}));
+      l.handle(Record{LogLevel::kWarning, getpid(), kDefaultTimepoint,
+                      "my_logger", "Test message"}));
   std::string log = buffer.str();
 
   // log message should be something like:
@@ -365,8 +376,8 @@ TEST_F(LoggingTest, StreamHandler) {
 
   // A bunch of casts to int for tellp to avoid C2666 in MSVC
   ASSERT_THAT((int)buffer.tellp(), Eq(0));
-  logger.handle(Record{LogLevel::kInfo, stdx::this_process::get_id(),
-                       kDefaultTimepoint, "my_module", "Message"});
+  logger.handle(Record{LogLevel::kInfo, getpid(), kDefaultTimepoint,
+                       "my_module", "Message"});
   EXPECT_THAT((int)buffer.tellp(), Gt(0));
 
   // message should be logged after applying format (timestamp, etc)
@@ -384,20 +395,17 @@ TEST_F(LoggingTest, FileHandler) {
 
   // We do not use mktemp or friends since we want this to work on
   // Windows as well.
-  Path log_file(g_here.join(
-      "log4-" + std::to_string(stdx::this_process::get_id()) + ".log"));
-  Scope_guard exit_guard([&]() {
-    std::error_code ec;
-    stdx::filesystem::remove(log_file.str(), ec);
-  });
+  Path log_file(g_here.join("log4-" + std::to_string(getpid()) + ".log"));
+  std::shared_ptr<void> exit_guard(nullptr,
+                                   [&](void *) { unlink(log_file.c_str()); });
 
   g_registry->add_handler("TestFileHandler",
                           std::make_shared<FileHandler>(log_file));
   logger.attach_handler("TestFileHandler");
 
   // Log one record
-  logger.handle(Record{LogLevel::kInfo, stdx::this_process::get_id(),
-                       kDefaultTimepoint, "my_module", "Message"});
+  logger.handle(Record{LogLevel::kInfo, getpid(), kDefaultTimepoint,
+                       "my_module", "Message"});
 
   // Open and read the entire file into memory.
   std::vector<std::string> lines;
@@ -422,95 +430,13 @@ TEST_F(LoggingTest, FileHandler) {
   g_registry->remove_handler("TestFileHandler");
 }
 
-TEST_F(LoggingTest, FileHandlerRotate) {
-  // Check that the FileHandler can rotate to supplied filename
-
-  // We do not use mktemp or friends since we want this to work on
-  // Windows as well.
-  Path log_file(g_here.join(
-      "log4-" + std::to_string(stdx::this_process::get_id()) + ".log"));
-  Path renamed_log_file(g_here.join(
-      "rotated-log4-" + std::to_string(stdx::this_process::get_id()) + ".log"));
-  Scope_guard exit_guard([&]() {
-    std::error_code ec;
-    stdx::filesystem::remove(log_file.str(), ec);
-    stdx::filesystem::remove(renamed_log_file.str(), ec);
-  });
-
-  g_registry->add_handler("TestFileHandler",
-                          std::make_shared<FileHandler>(log_file));
-  logger.attach_handler("TestFileHandler");
-
-  // Log one record
-  logger.handle(Record{LogLevel::kInfo, stdx::this_process::get_id(),
-                       kDefaultTimepoint, "my_module", "Message"});
-
-  // Verify only the original logfile exists
-  ASSERT_TRUE(log_file.exists());
-
-  // Open and read the entire file into memory.
-  std::vector<std::string> lines;
-  {
-    std::ifstream ifs_log(log_file.str());
-    std::string line;
-    while (std::getline(ifs_log, line)) lines.push_back(line);
-  }
-
-  // We do the assertion here to ensure that we can do as many tests
-  // as possible and report issues.
-  ASSERT_THAT(lines.size(), Ge(1));
-
-  // Check basic properties for the first line.
-  EXPECT_THAT(lines.size(), Eq(1));
-
-  // Message should be logged after applying format (timestamp, etc)
-  EXPECT_THAT(lines.at(0),
-              ContainsRegex(kDateRegex + " my_module INFO.*Message"));
-
-  // Rotate existing file to old filename
-  g_registry->flush_all_loggers(renamed_log_file.str());
-
-  // Verify the renamed file exists
-  ASSERT_TRUE(renamed_log_file.exists());
-
-  // Log one record after rotation
-  logger.handle(Record{LogLevel::kInfo, stdx::this_process::get_id(),
-                       kDefaultTimepoint, "my_module", "Another message"});
-
-  // Verify the original log file once again gets logged to
-  ASSERT_TRUE(log_file.exists());
-
-  // Open and read the new file into memory after rotation
-  std::vector<std::string> lines2;
-  {
-    std::ifstream ifs_log(log_file.str());
-    std::string line;
-    while (std::getline(ifs_log, line)) lines2.push_back(line);
-  }
-
-  // We do the assertion here to ensure that we can do as many tests
-  // as possible and report issues.
-  ASSERT_THAT(lines2.size(), Ge(1));
-
-  // Check basic properties for the first line.
-  EXPECT_THAT(lines2.size(), Eq(1));
-
-  // Message should be logged after rotation and applying format (timestamp,
-  // etc)
-  EXPECT_THAT(lines2.at(0),
-              ContainsRegex(kDateRegex + " my_module INFO.*Another message"));
-
-  // clean up
-  g_registry->remove_handler("TestFileHandler");
-}
-
 /**
  * @test
  *      Verify if no exception is throw when file can be opened for writing.
  */
 TEST_F(LoggingTest, DontThrowIfOpenedLogFileForWriting) {
-  TempDirectory tmp_dir;
-  Path dir_path(tmp_dir.name());
+  std::string tmp_dir = mysql_harness::get_tmp_dir("logging");
+  Path dir_path(tmp_dir);
   Path file_path(dir_path.join("test_file.log").str());
 
   ASSERT_TRUE(dir_path.exists());
@@ -526,12 +452,12 @@ TEST_F(LoggingTest, DontThrowIfOpenedLogFileForWriting) {
  *       create file in directory.
  *
  * There is component test: RouterLoggingTest, bad_logging_folder
- * that uses file as a directory to verify scenario when file cannot
+ * that uses file as a directory to veryfy scenario when file cannot
  * be created in directory.
  */
 TEST_F(LoggingTest, FileHandlerThrowsNoPermissionToCreateFileInDirectory) {
-  TempDirectory tmp_dir;
-  Path dir_path(tmp_dir.name());
+  std::string tmp_dir = mysql_harness::get_tmp_dir("logging");
+  Path dir_path(tmp_dir);
   Path file_path(dir_path.join("test_file.log").str());
 
   ASSERT_TRUE(dir_path.exists());
@@ -553,8 +479,8 @@ TEST_F(LoggingTest, FileHandlerThrowsNoPermissionToCreateFileInDirectory) {
  */
 TEST_F(LoggingTest,
        FileHandlerThrowsFileExistsButCannotOpenToWriteReadOnlyFile) {
-  TempDirectory tmp_dir;
-  Path dir_path(tmp_dir.name());
+  std::string tmp_dir = mysql_harness::get_tmp_dir("logging");
+  Path dir_path(tmp_dir);
   Path file_path(dir_path.join("test_file.log").str());
 
   // create empty log file
@@ -592,8 +518,8 @@ TEST_F(LoggingTest, HandlerWithDisabledFormatting) {
 
   // A bunch of casts to int for tellp to avoid C2666 in MSVC
   ASSERT_THAT((int)buffer.tellp(), Eq(0));
-  logger.handle(Record{LogLevel::kInfo, stdx::this_process::get_id(),
-                       kDefaultTimepoint, "my_module", "Message"});
+  logger.handle(Record{LogLevel::kInfo, getpid(), kDefaultTimepoint,
+                       "my_module", "Message"});
   EXPECT_THAT((int)buffer.tellp(), Gt(0));
 
   // message should be logged verbatim
@@ -613,7 +539,7 @@ TEST_F(LoggingTest, Messages) {
   std::chrono::time_point<std::chrono::system_clock> now =
       std::chrono::system_clock::now();
 
-  const auto pid = stdx::this_process::get_id();
+  auto pid = getpid();
 
   auto check_message = [this, &buffer, now, pid](const std::string &message,
                                                  LogLevel level,
@@ -664,7 +590,7 @@ TEST_F(LoggingTest, TimestampPrecision) {
   const auto nsec_part = std::chrono::duration_cast<std::chrono::nanoseconds>(
       now - std::chrono::system_clock::from_time_t(cur));
 
-  const auto pid = stdx::this_process::get_id();
+  auto pid = getpid();
 
   auto check_precision = [this, &buffer, now, pid, cur_localtime, nsec_part](
                              const std::string &message,
@@ -725,6 +651,7 @@ TEST_F(LoggingTest, TimestampPrecision) {
   g_registry->remove_handler("TestStreamHandler");
 }
 
+#if GTEST_HAS_COMBINE
 class LogLevelTest : public LoggingTest,
                      public WithParamInterface<std::tuple<LogLevel, LogLevel>> {
 };
@@ -743,7 +670,7 @@ TEST_P(LogLevelTest, Level) {
   std::chrono::time_point<std::chrono::system_clock> now =
       std::chrono::system_clock::now();
 
-  const auto pid = stdx::this_process::get_id();
+  auto pid = getpid();
 
   // Set the log level of the logger.
   logger.set_level(logger_level);
@@ -783,9 +710,9 @@ const LogLevel all_levels[]{
     LogLevel::kFatal, LogLevel::kSystem, LogLevel::kError, LogLevel::kWarning,
     LogLevel::kInfo,  LogLevel::kNote,   LogLevel::kDebug};
 
-INSTANTIATE_TEST_SUITE_P(CheckLogLevel, LogLevelTest,
-                         Combine(ValuesIn(all_levels), ValuesIn(all_levels)));
-
+INSTANTIATE_TEST_CASE_P(CheckLogLevel, LogLevelTest,
+                        Combine(ValuesIn(all_levels), ValuesIn(all_levels)));
+#endif
 ////////////////////////////////////////////////////////////////
 // Tests of the functional interface to the logger.
 ////////////////////////////////////////////////////////////////
@@ -850,64 +777,7 @@ void expect_log(void (*func)(const char *, ...)
   EXPECT_THAT(log, HasSubstr(MYSQL_ROUTER_LOG_DOMAIN));
 }
 
-// Macros for log expectance at different log levels
-#define EXPECT_LOG_LEVEL_DEBUG(buffer)        \
-  expect_log(log_system, buffer, "SYSTEM");   \
-  expect_log(log_error, buffer, "ERROR");     \
-  expect_log(log_warning, buffer, "WARNING"); \
-  expect_log(log_info, buffer, "INFO");       \
-  expect_log(log_note, buffer, "NOTE");       \
-  expect_log(log_debug, buffer, "DEBUG");
-
-#define EXPECT_LOG_LEVEL_NOTE(buffer)         \
-  expect_log(log_system, buffer, "SYSTEM");   \
-  expect_log(log_error, buffer, "ERROR");     \
-  expect_log(log_warning, buffer, "WARNING"); \
-  expect_log(log_info, buffer, "INFO");       \
-  expect_log(log_note, buffer, "NOTE");       \
-  expect_no_log(log_debug, buffer);
-
-#define EXPECT_LOG_LEVEL_INFO(buffer)         \
-  expect_log(log_system, buffer, "SYSTEM");   \
-  expect_log(log_error, buffer, "ERROR");     \
-  expect_log(log_warning, buffer, "WARNING"); \
-  expect_log(log_info, buffer, "INFO");       \
-  expect_no_log(log_note, buffer);            \
-  expect_no_log(log_debug, buffer);
-
-#define EXPECT_LOG_LEVEL_WARNING(buffer)      \
-  expect_log(log_system, buffer, "SYSTEM");   \
-  expect_log(log_error, buffer, "ERROR");     \
-  expect_log(log_warning, buffer, "WARNING"); \
-  expect_no_log(log_info, buffer);            \
-  expect_no_log(log_note, buffer);            \
-  expect_no_log(log_debug, buffer);
-
-#define EXPECT_LOG_LEVEL_ERROR(buffer)      \
-  expect_log(log_system, buffer, "SYSTEM"); \
-  expect_log(log_error, buffer, "ERROR");   \
-  expect_no_log(log_warning, buffer);       \
-  expect_no_log(log_info, buffer);          \
-  expect_no_log(log_note, buffer);          \
-  expect_no_log(log_debug, buffer);
-
-#define EXPECT_LOG_LEVEL_SYSTEM(buffer)     \
-  expect_log(log_system, buffer, "SYSTEM"); \
-  expect_no_log(log_error, buffer);         \
-  expect_no_log(log_warning, buffer);       \
-  expect_no_log(log_info, buffer);          \
-  expect_no_log(log_note, buffer);          \
-  expect_no_log(log_debug, buffer);
-
-#define EXPECT_LOG_LEVEL_NOT_SET(buffer) \
-  expect_no_log(log_system, buffer);     \
-  expect_no_log(log_error, buffer);      \
-  expect_no_log(log_warning, buffer);    \
-  expect_no_log(log_info, buffer);       \
-  expect_no_log(log_note, buffer);       \
-  expect_no_log(log_debug, buffer);
-
-TEST(FunctionalTest, Loggers) {
+TEST(FunctionalTest, Handlers) {
   // The loader creates these modules during start, so tests of the
   // logger that involves the loader are inside the loader unit
   // test. Here we instead call these functions directly.
@@ -919,71 +789,54 @@ TEST(FunctionalTest, Loggers) {
   attach_handler_to_all_loggers(*g_registry, StreamHandler::kDefaultName);
 
   set_log_level_for_all_loggers(*g_registry, LogLevel::kDebug);
-  EXPECT_LOG_LEVEL_DEBUG(buffer);
+  expect_log(log_system, buffer, "SYSTEM");
+  expect_log(log_error, buffer, "ERROR");
+  expect_log(log_warning, buffer, "WARNING");
+  expect_log(log_info, buffer, "INFO");
+  expect_log(log_note, buffer, "NOTE");
+  expect_log(log_debug, buffer, "DEBUG");
 
   set_log_level_for_all_loggers(*g_registry, LogLevel::kNote);
-  EXPECT_LOG_LEVEL_NOTE(buffer);
+  expect_log(log_system, buffer, "SYSTEM");
+  expect_log(log_error, buffer, "ERROR");
+  expect_log(log_warning, buffer, "WARNING");
+  expect_log(log_info, buffer, "INFO");
+  expect_log(log_note, buffer, "NOTE");
+  expect_no_log(log_debug, buffer);
 
   set_log_level_for_all_loggers(*g_registry, LogLevel::kError);
-  EXPECT_LOG_LEVEL_ERROR(buffer);
+  expect_log(log_system, buffer, "SYSTEM");
+  expect_log(log_error, buffer, "ERROR");
+  expect_no_log(log_warning, buffer);
+  expect_no_log(log_info, buffer);
+  expect_no_log(log_note, buffer);
+  expect_no_log(log_debug, buffer);
 
   set_log_level_for_all_loggers(*g_registry, LogLevel::kWarning);
-  EXPECT_LOG_LEVEL_WARNING(buffer);
+  expect_log(log_system, buffer, "SYSTEM");
+  expect_log(log_error, buffer, "ERROR");
+  expect_log(log_warning, buffer, "WARNING");
+  expect_no_log(log_info, buffer);
+  expect_no_log(log_note, buffer);
+  expect_no_log(log_debug, buffer);
 
   set_log_level_for_all_loggers(*g_registry, LogLevel::kSystem);
-  EXPECT_LOG_LEVEL_SYSTEM(buffer);
+  expect_log(log_system, buffer, "SYSTEM");
+  expect_no_log(log_error, buffer);
+  expect_no_log(log_warning, buffer);
+  expect_no_log(log_info, buffer);
+  expect_no_log(log_note, buffer);
+  expect_no_log(log_debug, buffer);
 
   // Check that nothing is logged when the handler is unregistered.
   g_registry->remove_handler(StreamHandler::kDefaultName);
   set_log_level_for_all_loggers(*g_registry, LogLevel::kNotSet);
-  EXPECT_LOG_LEVEL_NOT_SET(buffer);
-
-  ASSERT_NO_THROW(g_registry->remove_logger(MYSQL_ROUTER_LOG_DOMAIN));
-}
-
-TEST(FunctionalTest, Handlers) {
-  // The loader creates these modules during start, so tests of the
-  // logger that involves the loader are inside the loader unit
-  // test. Here we instead call these functions directly.
-  ASSERT_NO_THROW(g_registry->create_logger(MYSQL_ROUTER_LOG_DOMAIN));
-
-  std::stringstream buffer;
-  std::stringstream buffer2;
-  auto handler = std::make_shared<StreamHandler>(buffer);
-  auto handler2 = std::make_shared<StreamHandler>(buffer2);
-  g_registry->add_handler("stream1", handler);
-  g_registry->add_handler("stream2", handler2);
-  attach_handler_to_all_loggers(*g_registry, "stream1");
-  attach_handler_to_all_loggers(*g_registry, "stream2");
-
-  set_log_level_for_all_handlers(*g_registry, LogLevel::kDebug);
-  EXPECT_LOG_LEVEL_DEBUG(buffer);
-  EXPECT_LOG_LEVEL_DEBUG(buffer2);
-
-  set_log_level_for_all_handlers(*g_registry, LogLevel::kNote);
-  EXPECT_LOG_LEVEL_NOTE(buffer);
-  EXPECT_LOG_LEVEL_NOTE(buffer2);
-
-  set_log_level_for_all_handlers(*g_registry, LogLevel::kError);
-  EXPECT_LOG_LEVEL_ERROR(buffer);
-  EXPECT_LOG_LEVEL_ERROR(buffer2);
-
-  set_log_level_for_all_handlers(*g_registry, LogLevel::kWarning);
-  EXPECT_LOG_LEVEL_WARNING(buffer);
-  EXPECT_LOG_LEVEL_WARNING(buffer2);
-
-  set_log_level_for_all_handlers(*g_registry, LogLevel::kSystem);
-  EXPECT_LOG_LEVEL_SYSTEM(buffer);
-  EXPECT_LOG_LEVEL_SYSTEM(buffer2);
-
-  // Check that nothing is logged when the handlers are unregistered.
-  g_registry->remove_handler("stream1");
-  g_registry->remove_handler("stream2");
-  set_log_level_for_all_handlers(*g_registry, LogLevel::kNotSet);
-  EXPECT_LOG_LEVEL_NOT_SET(buffer);
-  EXPECT_LOG_LEVEL_NOT_SET(buffer2);
-
-  ASSERT_NO_THROW(g_registry->remove_logger(MYSQL_ROUTER_LOG_DOMAIN));
+  expect_no_log(log_system, buffer);
+  expect_no_log(log_error, buffer);
+  expect_no_log(log_warning, buffer);
+  expect_no_log(log_info, buffer);
+  expect_no_log(log_note, buffer);
+  expect_no_log(log_debug, buffer);
 }
 
 int main(int argc, char *argv[]) {
