@@ -1,17 +1,16 @@
 /*
-  Copyright (c) 2018, 2024, Oracle and/or its affiliates.
+  Copyright (c) 2018, 2020, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,7 +29,6 @@
  * :  http_auth_backend
  */
 
-#include <array>
 #include <future>
 #include <mutex>
 #include <thread>
@@ -38,21 +36,15 @@
 #include <sys/types.h>
 
 // Harness interface include files
-#include "mysql/harness/config_option.h"
 #include "mysql/harness/config_parser.h"
-#include "mysql/harness/dynamic_config.h"
 #include "mysql/harness/loader.h"
 #include "mysql/harness/logging/logging.h"
 #include "mysql/harness/plugin.h"
-#include "mysql/harness/plugin_config.h"
-#include "mysql/harness/section_config_exposer.h"
-#include "mysql/harness/utility/string.h"
 
-#include "mysqlrouter/component/http_auth_backend_component.h"
+#include "mysqlrouter/http_auth_backend_component.h"
 #include "mysqlrouter/http_auth_backend_export.h"
-#include "mysqlrouter/http_constants.h"
 #include "mysqlrouter/metadata_cache.h"
-#include "mysqlrouter/supported_http_options.h"
+#include "mysqlrouter/plugin_config.h"
 
 #include "http_auth_backend.h"
 #include "http_auth_backend_metadata_cache.h"
@@ -60,30 +52,27 @@
 IMPORT_LOG_FUNCTIONS()
 
 static constexpr const char kSectionName[]{"http_auth_backend"};
-static std::vector<std::string> registered_backends;
 
-using StringOption = mysql_harness::StringOption;
+using mysql_harness::ARCHITECTURE_DESCRIPTOR;
+using mysql_harness::Plugin;
+using mysql_harness::PLUGIN_ABI_VERSION;
+using mysql_harness::PluginFuncEnv;
 
-#define GET_OPTION_CHECKED(option, section, name, value)                       \
-  static_assert(                                                               \
-      mysql_harness::str_in_collection(http_backend_supported_options, name)); \
-  option = get_option(section, name, value);
 namespace {
-class HtpasswdPluginConfig : public mysql_harness::BasePluginConfig {
+class HtpasswdPluginConfig : public mysqlrouter::BasePluginConfig {
  public:
   std::string filename;
 
   explicit HtpasswdPluginConfig(const mysql_harness::ConfigSection *section)
-      : mysql_harness::BasePluginConfig(section) {
-    GET_OPTION_CHECKED(filename, section, "filename", StringOption{});
-  }
+      : mysqlrouter::BasePluginConfig(section),
+        filename(get_option_string(section, "filename")) {}
 
-  std::string get_default(std::string_view option) const override {
+  std::string get_default(const std::string &option) const override {
     if (option == "filename") return "users";
     return std::string();
   }
 
-  bool is_required(std::string_view /* option */) const override {
+  bool is_required(const std::string & /* option */) const override {
     return false;
   }
 };
@@ -114,21 +103,20 @@ class HttpAuthBackendFactory {
   }
 };
 
-class PluginConfig : public mysql_harness::BasePluginConfig {
+class PluginConfig : public mysqlrouter::BasePluginConfig {
  public:
   std::string backend;
   std::string filename;
 
   explicit PluginConfig(const mysql_harness::ConfigSection *section)
-      : mysql_harness::BasePluginConfig(section) {
-    GET_OPTION_CHECKED(backend, section, "backend", StringOption{});
+      : mysqlrouter::BasePluginConfig(section),
+        backend(get_option_string(section, "backend")) {}
+
+  std::string get_default(const std::string & /* option */) const override {
+    return std::string();
   }
 
-  std::string get_default(std::string_view /* option */) const override {
-    return {};
-  }
-
-  bool is_required(std::string_view option) const override {
+  bool is_required(const std::string &option) const override {
     if (option == "backend") return true;
     return false;
   }
@@ -137,7 +125,7 @@ class PluginConfig : public mysql_harness::BasePluginConfig {
 
 std::shared_ptr<HttpAuthBackendComponent::value_type> auth_backends;
 
-static void init(mysql_harness::PluginFuncEnv *env) {
+static void init(PluginFuncEnv *env) {
   const mysql_harness::AppInfo *info = get_app_info(env);
 
   if (nullptr == info->config) {
@@ -145,8 +133,7 @@ static void init(mysql_harness::PluginFuncEnv *env) {
   }
 
   try {
-    auto &auth_backend_component = HttpAuthBackendComponent::get_instance();
-
+    auth_backends = std::make_shared<HttpAuthBackendComponent::value_type>();
     for (const mysql_harness::ConfigSection *section :
          info->config->sections()) {
       if (section->name != kSectionName) {
@@ -161,13 +148,11 @@ static void init(mysql_harness::PluginFuncEnv *env) {
       }
 
       PluginConfig config(section);
-      const std::string backend_name = section->key;
-      auth_backend_component.add_backend(
-          backend_name,
-          HttpAuthBackendFactory::create(config.backend, section));
 
-      registered_backends.push_back(backend_name);
+      auth_backends->insert({section->key, HttpAuthBackendFactory::create(
+                                               config.backend, section)});
     }
+    HttpAuthBackendComponent::get_instance().init(auth_backends);
   } catch (const std::invalid_argument &exc) {
     set_error(env, mysql_harness::kConfigInvalidArgument, "%s", exc.what());
   } catch (const std::exception &exc) {
@@ -177,7 +162,7 @@ static void init(mysql_harness::PluginFuncEnv *env) {
   }
 }
 
-static void start(mysql_harness::PluginFuncEnv *env) {
+static void start(PluginFuncEnv *env) {
   const mysql_harness::ConfigSection *section = get_config_section(env);
 
   PluginConfig config(section);
@@ -208,86 +193,19 @@ static void start(mysql_harness::PluginFuncEnv *env) {
   }
 }
 
-static void deinit(mysql_harness::PluginFuncEnv *) {
-  auto &auth_backend_component = HttpAuthBackendComponent::get_instance();
-
-  for (const auto &backend : registered_backends) {
-    auth_backend_component.remove_backend(backend);
-  }
-
-  registered_backends.clear();
-}
-
-static const std::array<const char *, 2> required = {{
-    "logger",
-    "router_protobuf",
-}};
-
-namespace {
-
-class HttpAuthBackendConfigExposer
-    : public mysql_harness::SectionConfigExposer {
- public:
-  using DC = mysql_harness::DynamicConfig;
-  HttpAuthBackendConfigExposer(
-      const bool initial, const PluginConfig &plugin_config,
-      const mysql_harness::ConfigSection &default_section,
-      const std::string &key)
-      : mysql_harness::SectionConfigExposer(
-            initial, default_section,
-            DC::SectionId{"http_authentication_backends", key}),
-        plugin_config_(plugin_config) {}
-
-  void expose() override {
-    expose_option("backend", plugin_config_.backend,
-                  std::string(kHttpAuthPluginDefaultBackend));
-    expose_option("filename", plugin_config_.filename, "");
-  }
-
- private:
-  const PluginConfig &plugin_config_;
-};
-
-}  // namespace
-
-static void expose_configuration(mysql_harness::PluginFuncEnv *env,
-                                 const char *key, bool initial) {
-  const mysql_harness::AppInfo *info = get_app_info(env);
-
-  if (!info->config) return;
-
-  for (const mysql_harness::ConfigSection *section : info->config->sections()) {
-    if (section->name != kSectionName || section->key != key) {
-      continue;
-    }
-
-    PluginConfig config(section);
-    HttpAuthBackendConfigExposer(initial, config,
-                                 info->config->get_default_section(), key)
-        .expose();
-  }
-}
-
 extern "C" {
-mysql_harness::Plugin HTTP_AUTH_BACKEND_EXPORT
-    harness_plugin_http_auth_backend = {
-        mysql_harness::PLUGIN_ABI_VERSION,       // abi-version
-        mysql_harness::ARCHITECTURE_DESCRIPTOR,  // arch
-        "HTTP_AUTH_BACKEND",                     // name
-        VERSION_NUMBER(0, 0, 1),
-        // requires
-        required.size(),
-        required.data(),
-        // conflicts
-        0,
-        nullptr,
-        init,     // init
-        deinit,   // deinit
-        start,    // start
-        nullptr,  // stop
-        false,    // declares_readiness
-        http_backend_supported_options.size(),
-        http_backend_supported_options.data(),
-        expose_configuration,
+Plugin HTTP_AUTH_BACKEND_EXPORT harness_plugin_http_auth_backend = {
+    PLUGIN_ABI_VERSION,
+    ARCHITECTURE_DESCRIPTOR,
+    "HTTP_AUTH_BACKEND",
+    VERSION_NUMBER(0, 0, 1),
+    0,
+    nullptr,  // requires
+    0,
+    nullptr,  // conflicts
+    init,     // init
+    nullptr,  // deinit
+    start,    // start
+    nullptr,  // stop
 };
 }
