@@ -1,17 +1,16 @@
 /*
-  Copyright (c) 2018, 2024, Oracle and/or its affiliates.
+  Copyright (c) 2018, 2020, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,15 +22,16 @@
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-#include "mysqlrouter/component/http_auth_method_basic.h"
+#include "http_auth_method_basic.h"
 
 #include <algorithm>  // std::find
-#include <iterator>   // std::back_inserter
 #include <string>
-#include <system_error>  // std::errc
 #include <vector>
 
-#include "mysqlrouter/base64.h"
+#include "../src/base64.h"
+
+// instantiate the constexpr members
+constexpr const char HttpAuthMethodBasic::kMethodName[];
 
 HttpAuthMethodBasic::AuthData HttpAuthMethodBasic::decode_authorization(
     const std::string &http_auth_data, std::error_code &ec) {
