@@ -1,17 +1,16 @@
 /*
-  Copyright (c) 2019, 2024, Oracle and/or its affiliates.
+  Copyright (c) 2019, 2020, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -31,18 +30,14 @@
 #include <string>
 
 #include "mysql/harness/config_parser.h"
-#include "mysql/harness/dynamic_config.h"
 #include "mysql/harness/loader.h"
-#include "mysql/harness/logging/logging.h"
 #include "mysql/harness/plugin.h"
-#include "mysql/harness/plugin_config.h"
-#include "mysql/harness/section_config_exposer.h"
 #include "mysql/harness/utility/string.h"  // ::join()
 
-#include "mysqlrouter/component/http_server_component.h"
-#include "mysqlrouter/http_constants.h"
+#include "mysqlrouter/plugin_config.h"
+
+#include "mysqlrouter/http_server_component.h"
 #include "mysqlrouter/rest_api_component.h"
-#include "mysqlrouter/supported_rest_options.h"
 
 #include "rest_clusters_list.h"
 #include "rest_clusters_nodes.h"
@@ -53,34 +48,26 @@ IMPORT_LOG_FUNCTIONS()
 
 using namespace std::string_literals;
 
-static constexpr char kSectionName[]{"rest_metadata_cache"};
-static constexpr char kRequireRealm[]{"require_realm"};
+static const char kSectionName[]{"rest_metadata_cache"};
+static const char kRequireRealm[]{"require_realm"};
 
 // one shared setting
 std::string require_realm_metadata_cache;
 
-using StringOption = mysql_harness::StringOption;
-
-#define GET_OPTION_CHECKED(option, section, name, value)                      \
-  static_assert(                                                              \
-      mysql_harness::str_in_collection(rest_plugin_supported_options, name)); \
-  option = get_option(section, name, value);
-
-class RestMetadataCachePluginConfig : public mysql_harness::BasePluginConfig {
+class RestMetadataCachePluginConfig : public mysqlrouter::BasePluginConfig {
  public:
   std::string require_realm;
 
   explicit RestMetadataCachePluginConfig(
       const mysql_harness::ConfigSection *section)
-      : mysql_harness::BasePluginConfig(section) {
-    GET_OPTION_CHECKED(require_realm, section, kRequireRealm, StringOption{});
-  }
+      : mysqlrouter::BasePluginConfig(section),
+        require_realm(get_option_string(section, kRequireRealm)) {}
 
-  std::string get_default(std::string_view /* option */) const override {
+  std::string get_default(const std::string & /* option */) const override {
     return {};
   }
 
-  bool is_required(std::string_view option) const override {
+  bool is_required(const std::string &option) const override {
     if (option == kRequireRealm) return true;
     return false;
   }
@@ -843,8 +830,6 @@ static void start(mysql_harness::PluginFuncEnv *env) {
       //                        std::make_unique<RestClustersNodes>(require_realm_metadata_cache)},
   }};
 
-  mysql_harness::on_service_ready(env);
-
   wait_for_stop(env, 0);
 
   // in case rest_api never initialized, ensure the rest_api_component doesn't
@@ -859,72 +844,25 @@ static void start(mysql_harness::PluginFuncEnv *env) {
 #define DLLEXPORT
 #endif
 
-static const std::array<const char *, 2> required = {{
-    "logger",
+const char *rest_metadata_plugin_requires[] = {
     // "metadata_cache",
     "rest_api",
-}};
-
-namespace {
-
-class RestMetadataCacheConfigExposer
-    : public mysql_harness::SectionConfigExposer {
- public:
-  using DC = mysql_harness::DynamicConfig;
-  RestMetadataCacheConfigExposer(
-      const bool initial, const RestMetadataCachePluginConfig &plugin_config,
-      const mysql_harness::ConfigSection &default_section)
-      : mysql_harness::SectionConfigExposer(
-            initial, default_section,
-            DC::SectionId{"rest_configs", kSectionName}),
-        plugin_config_(plugin_config) {}
-
-  void expose() override {
-    expose_option("require_realm", plugin_config_.require_realm,
-                  std::string(kHttpDefaultAuthRealmName));
-  }
-
- private:
-  const RestMetadataCachePluginConfig &plugin_config_;
 };
 
-}  // namespace
-
-static void expose_configuration(mysql_harness::PluginFuncEnv *env,
-                                 const char * /*key*/, bool initial) {
-  const mysql_harness::AppInfo *info = get_app_info(env);
-
-  if (!info->config) return;
-
-  for (const mysql_harness::ConfigSection *section : info->config->sections()) {
-    if (section->name == kSectionName) {
-      RestMetadataCachePluginConfig config{section};
-      RestMetadataCacheConfigExposer(initial, config,
-                                     info->config->get_default_section())
-          .expose();
-    }
-  }
-}
-
 extern "C" {
 mysql_harness::Plugin DLLEXPORT harness_plugin_rest_metadata_cache = {
     mysql_harness::PLUGIN_ABI_VERSION,
     mysql_harness::ARCHITECTURE_DESCRIPTOR,
     "REST_METADATA_CACHE",
     VERSION_NUMBER(0, 0, 1),
-    // requires
-    required.size(),
-    required.data(),
-    // conflicts
+    sizeof(rest_metadata_plugin_requires) /
+        sizeof(rest_metadata_plugin_requires[0]),
+    rest_metadata_plugin_requires,  // requires
     0,
-    nullptr,
+    nullptr,  // conflicts
     init,     // init
     nullptr,  // deinit
     start,    // start
     nullptr,  // stop
-    true,     // declares_readiness
-    rest_plugin_supported_options.size(),
-    rest_plugin_supported_options.data(),
-    expose_configuration,
 };
 }
