@@ -1,17 +1,16 @@
 /*
-  Copyright (c) 2019, 2024, Oracle and/or its affiliates.
+  Copyright (c) 2019, 2020, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -31,19 +30,15 @@
 #include <string>
 
 #include "mysql/harness/config_parser.h"
-#include "mysql/harness/dynamic_config.h"
 #include "mysql/harness/loader.h"
-#include "mysql/harness/logging/logging.h"
 #include "mysql/harness/plugin.h"
-#include "mysql/harness/plugin_config.h"
-#include "mysql/harness/section_config_exposer.h"
 #include "mysql/harness/utility/string.h"  // ::join()
 
-#include "mysqlrouter/component/http_server_component.h"
-#include "mysqlrouter/http_constants.h"
+#include "mysqlrouter/plugin_config.h"
+
+#include "mysqlrouter/http_server_component.h"
 #include "mysqlrouter/rest_api_component.h"
 #include "mysqlrouter/rest_routing_export.h"
-#include "mysqlrouter/supported_rest_options.h"
 
 #include "rest_routing_blocked_hosts.h"
 #include "rest_routing_config.h"
@@ -51,7 +46,6 @@
 #include "rest_routing_destinations.h"
 #include "rest_routing_health.h"
 #include "rest_routing_list.h"
-#include "rest_routing_routes_status.h"
 #include "rest_routing_status.h"
 IMPORT_LOG_FUNCTIONS()
 
@@ -63,27 +57,19 @@ static const char kRequireRealm[]{"require_realm"};
 // one shared setting
 std::string require_realm_routing;
 
-using StringOption = mysql_harness::StringOption;
-
-#define GET_OPTION_CHECKED(option, section, name, value)                      \
-  static_assert(                                                              \
-      mysql_harness::str_in_collection(rest_plugin_supported_options, name)); \
-  option = get_option(section, name, value);
-
-class RestRoutingPluginConfig : public mysql_harness::BasePluginConfig {
+class RestRoutingPluginConfig : public mysqlrouter::BasePluginConfig {
  public:
   std::string require_realm;
 
   explicit RestRoutingPluginConfig(const mysql_harness::ConfigSection *section)
-      : mysql_harness::BasePluginConfig(section) {
-    GET_OPTION_CHECKED(require_realm, section, "require_realm", StringOption{});
-  }
+      : mysqlrouter::BasePluginConfig(section),
+        require_realm(get_option_string(section, kRequireRealm)) {}
 
-  std::string get_default(std::string_view /* option */) const override {
+  std::string get_default(const std::string & /* option */) const override {
     return {};
   }
 
-  bool is_required(std::string_view option) const override {
+  bool is_required(const std::string &option) const override {
     if (option == kRequireRealm) return true;
     return false;
   }
@@ -156,13 +142,6 @@ using JsonValue = RestApiComponent::JsonValue;
 
 #define STR(s) \
   { s, strlen(s), rapidjson::kPointerInvalidIndex }
-
-static const std::array<JsonPointer::Token, 2> routing_status_path_tokens{
-    {STR("paths"), STR("/routing/status")}};
-
-static const std::array<JsonPointer::Token, 2> routing_status_def_tokens{
-    {STR("definitions"), STR("RoutingGlobalStatus")}};
-
 static const std::array<JsonPointer::Token, 2> route_name_param_tokens{
     {STR("parameters"), STR("routeNameParam")}};
 
@@ -253,22 +232,6 @@ static void spec_adder(RestApiComponent::JsonDocument &spec_doc) {
             allocator);
   }
 
-  // /definitions/RoutingGlobalStatus
-  const RestApiComponent::JsonPointer routing_status_def_ptr(
-      routing_status_def_tokens.data(), routing_status_def_tokens.size());
-
-  routing_status_def_ptr.Set(
-      spec_doc,
-      JsonValue(rapidjson::kObjectType)
-          .AddMember("totalMaxConnections",
-                     "number of total connections allowed", allocator)
-          .AddMember("currentMaxConnections",
-                     "number of current total connections", allocator),
-      allocator);
-
-  std::string routing_status_def_ptr_str =
-      json_pointer_stringfy(routing_status_def_ptr);
-
   // /parameters/routeNameParam
   const RestApiComponent::JsonPointer route_name_param_ptr(
       route_name_param_tokens.data(), route_name_param_tokens.size());
@@ -694,51 +657,6 @@ static void spec_adder(RestApiComponent::JsonDocument &spec_doc) {
   std::string routes_connection_list_def_ptr_str =
       json_pointer_stringfy(routes_connection_list_def_ptr);
 
-  // /paths/routingStatus
-  {
-    JsonPointer ptr(routing_status_path_tokens.data(),
-                    routing_status_path_tokens.size());
-
-    ptr.Set(
-        spec_doc,
-        JsonValue(rapidjson::kObjectType)
-            .AddMember(
-                "get",
-                JsonValue(rapidjson::kObjectType)
-                    .AddMember("tags",
-                               JsonValue(rapidjson::kArrayType)
-                                   .PushBack("routing", allocator),
-                               allocator)
-                    .AddMember("description",
-                               "Get status of the routing plugin", allocator)
-                    .AddMember(
-                        "responses",
-                        JsonValue(rapidjson::kObjectType)
-                            .AddMember(
-                                "200",
-                                JsonValue(rapidjson::kObjectType)
-                                    .AddMember("description",
-                                               "status of the routing plugin",
-                                               allocator)
-                                    .AddMember(
-                                        "schema",
-                                        JsonValue(rapidjson::kObjectType)
-                                            .AddMember(
-                                                "$ref",
-                                                JsonValue(
-                                                    routing_status_def_ptr_str
-                                                        .data(),
-                                                    routing_status_def_ptr_str
-                                                        .size(),
-                                                    allocator),
-                                                allocator),
-                                        allocator),
-                                allocator),
-                        allocator),
-                allocator),
-        allocator);
-  }
-
   // /paths/routesConfig
   {
     JsonPointer ptr(routes_config_path_tokens.data(),
@@ -1176,9 +1094,7 @@ static void start(mysql_harness::PluginFuncEnv *env) {
 
   const bool spec_adder_executed = rest_api_srv.try_process_spec(spec_adder);
 
-  std::array<RestApiComponentPath, 8> paths{{
-      {rest_api_srv, RestRoutingStatus::path_regex,
-       std::make_unique<RestRoutingStatus>(require_realm_routing)},
+  std::array<RestApiComponentPath, 7> paths{{
       {rest_api_srv, RestRoutingList::path_regex,
        std::make_unique<RestRoutingList>(require_realm_routing)},
       {rest_api_srv, RestRoutingBlockedHosts::path_regex,
@@ -1187,16 +1103,14 @@ static void start(mysql_harness::PluginFuncEnv *env) {
        std::make_unique<RestRoutingDestinations>(require_realm_routing)},
       {rest_api_srv, RestRoutingConfig::path_regex,
        std::make_unique<RestRoutingConfig>(require_realm_routing)},
-      {rest_api_srv, RestRoutingRoutesStatus::path_regex,
-       std::make_unique<RestRoutingRoutesStatus>(require_realm_routing)},
+      {rest_api_srv, RestRoutingStatus::path_regex,
+       std::make_unique<RestRoutingStatus>(require_realm_routing)},
       {rest_api_srv, RestRoutingHealth::path_regex,
        std::make_unique<RestRoutingHealth>(require_realm_routing)},
       {rest_api_srv, RestRoutingConnections::path_regex,
        std::make_unique<RestRoutingConnections>(require_realm_routing)},
   }};
 
-  mysql_harness::on_service_ready(env);
-
   wait_for_stop(env, 0);
 
   // in case rest_api never initialized, ensure the rest_api_component doesn't
@@ -1204,71 +1118,25 @@ static void start(mysql_harness::PluginFuncEnv *env) {
   if (!spec_adder_executed) rest_api_srv.remove_process_spec(spec_adder);
 }
 
-static std::array<const char *, 2> required = {{
-    "logger",
+const char *rest_routing_plugin_requires[] = {
     // "routing",
     "rest_api",
-}};
-
-namespace {
-
-class RestRoutingConfigExposer : public mysql_harness::SectionConfigExposer {
- public:
-  using DC = mysql_harness::DynamicConfig;
-  RestRoutingConfigExposer(const bool initial,
-                           const RestRoutingPluginConfig &plugin_config,
-                           const mysql_harness::ConfigSection &default_section)
-      : mysql_harness::SectionConfigExposer(
-            initial, default_section,
-            DC::SectionId{"rest_configs", kSectionName}),
-        plugin_config_(plugin_config) {}
-
-  void expose() override {
-    expose_option("require_realm", plugin_config_.require_realm,
-                  std::string(kHttpDefaultAuthRealmName));
-  }
-
- private:
-  const RestRoutingPluginConfig &plugin_config_;
 };
 
-}  // namespace
-
-static void expose_configuration(mysql_harness::PluginFuncEnv *env,
-                                 const char * /*key*/, bool initial) {
-  const mysql_harness::AppInfo *info = get_app_info(env);
-
-  if (!info->config) return;
-
-  for (const mysql_harness::ConfigSection *section : info->config->sections()) {
-    if (section->name == kSectionName) {
-      RestRoutingPluginConfig config{section};
-      RestRoutingConfigExposer(initial, config,
-                               info->config->get_default_section())
-          .expose();
-    }
-  }
-}
-
 extern "C" {
 mysql_harness::Plugin REST_ROUTING_EXPORT harness_plugin_rest_routing = {
-    mysql_harness::PLUGIN_ABI_VERSION,       // abi-version
-    mysql_harness::ARCHITECTURE_DESCRIPTOR,  // arch
-    "REST_ROUTING",                          // name
+    mysql_harness::PLUGIN_ABI_VERSION,
+    mysql_harness::ARCHITECTURE_DESCRIPTOR,
+    "REST_ROUTING",
     VERSION_NUMBER(0, 0, 1),
-    // requires
-    required.size(),
-    required.data(),
-    // conflicts
+    sizeof(rest_routing_plugin_requires) /
+        sizeof(rest_routing_plugin_requires[0]),
+    rest_routing_plugin_requires,  // requires
     0,
-    nullptr,
+    nullptr,  // conflicts
     init,     // init
     nullptr,  // deinit
     start,    // start
     nullptr,  // stop
-    true,     // declares_readiness
-    rest_plugin_supported_options.size(),
-    rest_plugin_supported_options.data(),
-    expose_configuration,
 };
 }
