@@ -1,17 +1,16 @@
 /*
-  Copyright (c) 2016, 2024, Oracle and/or its affiliates.
+  Copyright (c) 2016, 2020, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,8 +25,6 @@
 #ifndef ROUTER_CLUSTER_METADATA_INCLUDED
 #define ROUTER_CLUSTER_METADATA_INCLUDED
 
-#include "mysqlrouter/router_export.h"
-
 #include <stdexcept>
 
 #include "config_generator.h"
@@ -39,23 +36,11 @@ namespace mysqlrouter {
 
 struct ClusterInfo {
   std::vector<std::string> metadata_servers;
-  std::string cluster_id;
-  // GR name for GR cluster
-  std::string cluster_type_specific_id;
-  // name of the cluster (or clusterset in case of the clusterset)
-  std::string name;
-  // whether this cluster is a primary cluster in case it is a member of a
-  // ClusterSet
-  bool is_primary{false};
-
-  std::string get_cluster_type_specific_id() const {
-    return cluster_type_specific_id.empty() ? cluster_id
-                                            : cluster_type_specific_id;
-  }
+  std::string metadata_cluster_id;
+  std::string metadata_cluster_name;
+  std::string metadata_replicaset;
 };
 
-using OptionsMap = std::map<std::string, std::string>;
-
 class ClusterMetadata {
  public:
   ClusterMetadata(const MetadataSchemaVersion &schema_version,
@@ -103,14 +88,16 @@ class ClusterMetadata {
       const std::string &router_name, const bool overwrite,
       const std::string &hostname_override = "") = 0;
 
-  virtual void update_router_info(
-      const uint32_t router_id, const std::string &cluster_id,
-      const std::string &target_cluster, const std::string &rw_endpoint,
-      const std::string &ro_endpoint, const std::string &rw_split_endpoint,
-      const std::string &rw_x_endpoint, const std::string &ro_x_endpoint,
-      const std::string &username) = 0;
+  virtual void update_router_info(const uint32_t router_id,
+                                  const std::string &cluster_id,
+                                  const std::string &rw_endpoint,
+                                  const std::string &ro_endpoint,
+                                  const std::string &rw_x_endpoint,
+                                  const std::string &ro_x_endpoint,
+                                  const std::string &username) = 0;
 
-  virtual std::vector<std::string> get_routing_mode_queries() = 0;
+  virtual std::vector<std::string> get_routing_mode_queries(
+      const std::string &cluster_name) = 0;
 
   /** @brief Verify that host is a valid metadata server
    *
@@ -120,7 +107,7 @@ class ClusterMetadata {
    * @throws std::out_of_range
    * @throws std::logic_error
    *
-   * checks that the server
+   *  * checks that the server
    *
    * - has the metadata in the correct version
    * - contains metadata for the group it's in (in case of GR cluster)
@@ -142,8 +129,6 @@ class ClusterMetadata {
 
   virtual ClusterInfo fetch_metadata_servers() = 0;
 
-  virtual InstanceType fetch_current_instance_type() = 0;
-
   virtual std::vector<std::string> get_grant_statements(
       const std::string &new_accounts) const = 0;
 
@@ -152,11 +137,6 @@ class ClusterMetadata {
 
   MySQLSession &get_session() { return *mysql_; }
 
-  virtual uint64_t get_view_id(
-      const std::string & /*cluster_type_specific_id*/) {
-    return 0;
-  }
-
  protected:
   // throws MySQLSession::Error, std::out_of_range, std::logic_error
   virtual uint64_t query_cluster_count() = 0;
@@ -185,6 +165,47 @@ class ClusterMetadataGR : public ClusterMetadata {
       override;
 };
 
+class ClusterMetadataGRV1 : public ClusterMetadataGR {
+ public:
+  ClusterMetadataGRV1(const MetadataSchemaVersion &schema_version,
+                      MySQLSession *mysql,
+                      mysql_harness::SocketOperationsBase *sockops =
+                          mysql_harness::SocketOperations::instance())
+      : ClusterMetadataGR(schema_version, mysql, sockops) {}
+
+  ~ClusterMetadataGRV1() override = default;
+
+  mysqlrouter::ClusterType get_type() override {
+    return mysqlrouter::ClusterType::GR_V1;
+  }
+
+  ClusterInfo fetch_metadata_servers() override;
+
+  std::vector<std::string> get_routing_mode_queries(
+      const std::string &cluster_name) override;
+
+  void verify_router_id_is_ours(
+      const uint32_t router_id,
+      const std::string &hostname_override = "") override;
+
+  void update_router_info(const uint32_t router_id,
+                          const std::string &cluster_id,
+                          const std::string &rw_endpoint,
+                          const std::string &ro_endpoint,
+                          const std::string &rw_x_endpoint,
+                          const std::string &ro_x_endpoint,
+                          const std::string &username) override;
+
+  uint32_t register_router(const std::string &router_name, const bool overwrite,
+                           const std::string &hostname_override = "") override;
+
+  std::vector<std::string> get_grant_statements(
+      const std::string &new_accounts) const override;
+
+ protected:
+  uint64_t query_cluster_count() override;
+};
+
 class ClusterMetadataGRV2 : public ClusterMetadataGR {
  public:
   ClusterMetadataGRV2(const MetadataSchemaVersion &schema_version,
@@ -201,19 +222,19 @@ class ClusterMetadataGRV2 : public ClusterMetadataGR {
 
   ClusterInfo fetch_metadata_servers() override;
 
-  std::vector<std::string> get_routing_mode_queries() override;
-
-  InstanceType fetch_current_instance_type() override;
+  std::vector<std::string> get_routing_mode_queries(
+      const std::string &cluster_name) override;
 
   void verify_router_id_is_ours(
       uint32_t router_id, const std::string &hostname_override = "") override;
 
-  void update_router_info(
-      const uint32_t router_id, const std::string &cluster_id,
-      const std::string &target_cluster, const std::string &rw_endpoint,
-      const std::string &ro_endpoint, const std::string &rw_split_endpoint,
-      const std::string &rw_x_endpoint, const std::string &ro_x_endpoint,
-      const std::string &username) override;
+  void update_router_info(const uint32_t router_id,
+                          const std::string &cluster_id,
+                          const std::string &rw_endpoint,
+                          const std::string &ro_endpoint,
+                          const std::string &rw_x_endpoint,
+                          const std::string &ro_x_endpoint,
+                          const std::string &username) override;
 
   uint32_t register_router(const std::string &router_name, const bool overwrite,
                            const std::string &hostname_override = "") override;
@@ -225,52 +246,6 @@ class ClusterMetadataGRV2 : public ClusterMetadataGR {
   uint64_t query_cluster_count() override;
 };
 
-class ClusterMetadataGRInClusterSet : public ClusterMetadataGRV2 {
- public:
-  ClusterMetadataGRInClusterSet(
-      const MetadataSchemaVersion &schema_version, MySQLSession *mysql,
-      const OptionsMap & /*options*/,
-      mysql_harness::SocketOperationsBase *sockops =
-          mysql_harness::SocketOperations::instance());
-
-  ~ClusterMetadataGRInClusterSet() override = default;
-
-  mysqlrouter::ClusterType get_type() override {
-    return mysqlrouter::ClusterType::GR_CS;
-  }
-
-  // nothing specific to check for ClusterSet
-  void require_metadata_is_ok() override {}
-
-  ClusterInfo fetch_metadata_servers() override;
-
-  std::vector<std::tuple<std::string, unsigned long>> fetch_cluster_hosts()
-      override;
-
-  enum class TargetClusterType {
-    // target should be the cluster on which we bootstrap
-    targetClusterCurrent,
-    // target should be the Priamry Cluster
-    targetClusterPrimary,
-    // target should be the Cluster with the given name
-    targetClusterByName
-  };
-
-  std::string get_cluster_type_specific_id() override;
-  uint64_t get_view_id(const std::string &cluster_type_specific_id) override;
-
-  void update_router_info(
-      const uint32_t router_id, const std::string &cluster_id,
-      const std::string &target_cluster, const std::string &rw_endpoint,
-      const std::string &ro_endpoint, const std::string &rw_split_endpoint,
-      const std::string &rw_x_endpoint, const std::string &ro_x_endpoint,
-      const std::string &username) override;
-
- protected:
-  TargetClusterType target_cluster_type_;
-  std::string target_cluster_name_;
-};
-
 class ClusterMetadataAR : public ClusterMetadata {
  public:
   ClusterMetadataAR(const MetadataSchemaVersion &schema_version,
@@ -279,7 +254,7 @@ class ClusterMetadataAR : public ClusterMetadata {
                         mysql_harness::SocketOperations::instance())
       : ClusterMetadata(schema_version, mysql, sockops) {}
 
-  ~ClusterMetadataAR() override = default;
+  virtual ~ClusterMetadataAR() override = default;
 
   mysqlrouter::ClusterType get_type() override {
     return mysqlrouter::ClusterType::RS_V2;
@@ -291,26 +266,23 @@ class ClusterMetadataAR : public ClusterMetadata {
 
   ClusterInfo fetch_metadata_servers() override;
 
-  InstanceType fetch_current_instance_type() override {
-    return InstanceType::AsyncMember;
-  }
-
   std::string get_cluster_type_specific_id() override;
 
-  uint64_t get_view_id(
-      const std::string & /*cluster_type_specific_id*/) override;
+  unsigned int get_view_id();
 
-  std::vector<std::string> get_routing_mode_queries() override;
+  std::vector<std::string> get_routing_mode_queries(
+      const std::string &cluster_name) override;
 
   void verify_router_id_is_ours(
       uint32_t router_id, const std::string &hostname_override = "") override;
 
-  void update_router_info(
-      const uint32_t router_id, const std::string &cluster_id,
-      const std::string &target_cluster, const std::string &rw_endpoint,
-      const std::string &ro_endpoint, const std::string &rw_split_endpoint,
-      const std::string &rw_x_endpoint, const std::string &ro_x_endpoint,
-      const std::string &username) override;
+  void update_router_info(const uint32_t router_id,
+                          const std::string &cluster_id,
+                          const std::string &rw_endpoint,
+                          const std::string &ro_endpoint,
+                          const std::string &rw_x_endpoint,
+                          const std::string &ro_x_endpoint,
+                          const std::string &username) override;
 
   uint32_t register_router(const std::string &router_name, const bool overwrite,
                            const std::string &hostname_override = "") override;
@@ -325,11 +297,12 @@ class ClusterMetadataAR : public ClusterMetadata {
   uint64_t query_cluster_count() override;
 };
 
-std::unique_ptr<ClusterMetadata> ROUTER_LIB_EXPORT
-create_metadata(const MetadataSchemaVersion &schema_version,
-                MySQLSession *mysql, const OptionsMap &options = {},
-                mysql_harness::SocketOperationsBase *sockops =
-                    mysql_harness::SocketOperations::instance());
+MetadataSchemaVersion get_metadata_schema_version(MySQLSession *mysql);
+
+std::unique_ptr<ClusterMetadata> create_metadata(
+    const MetadataSchemaVersion &schema_version, MySQLSession *mysql,
+    mysql_harness::SocketOperationsBase *sockops =
+        mysql_harness::SocketOperations::instance());
 
 }  // namespace mysqlrouter
 
