@@ -1,17 +1,16 @@
 /*
-  Copyright (c) 2015, 2024, Oracle and/or its affiliates.
+  Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,8 +25,6 @@
 #ifndef ROUTER_MYSQL_ROUTER_INCLUDED
 #define ROUTER_MYSQL_ROUTER_INCLUDED
 
-#include "mysqlrouter/router_export.h"
-
 /** @file
  * @brief Defining the main class MySQLRouter
  *
@@ -37,21 +34,17 @@
 
 #include "mysql/harness/arg_handler.h"
 #include "mysql/harness/loader.h"
-#include "mysql/harness/signal_handler.h"
 #include "mysqlrouter/keyring_info.h"
-#include "mysqlrouter/sys_user_operations.h"
+#include "mysqlrouter/utils.h"
+#include "router_config.h"
 
 #include <cstdint>
-#include <iostream>
 #include <stdexcept>
 #include <vector>
 
 static const size_t kHelpScreenWidth = 72;
 static const size_t kHelpScreenIndent = 8;
 
-static const std::string kSystemRouterName = "system";
-static const std::string kDefaultSystemUserName = "";
-
 class ConfigFiles;
 
 /** @class MySQLRouter
@@ -65,7 +58,7 @@ class ConfigFiles;
  *  explicit location was given, the application exits.
  *
  *  The class depends on MySQL Harness to, among other things, load the
- *  configuration and initialize all request plugins.
+ *  configuration and initalize all request plugins.
  *
  *  Example usage:
  *
@@ -106,19 +99,17 @@ class MySQLRouter {
    *
    * Example usage:
    *
-   *     MySQLRouter router(argv[0],
+   *     MySQLRouter router(Path(argv[0]).dirname(),
    *                        vector<string>({argv + 1, argv + argc}));
    *     router.start();
    *
-   * @param program_name path of the started executable
+   * @param origin Directory where executable is located
    * @param arguments a vector of strings
    * @param out_stream output stream representing "stdout"
    * @param err_stream output stream representing "stderr"
+   * @param sys_user_operations .oO( ... )
    */
-#ifndef _WIN32
-  /// @param sys_user_operations system operations which provide chown, ...
-#endif
-  MySQLRouter(const std::string &program_name,
+  MySQLRouter(const mysql_harness::Path &origin,
               const std::vector<std::string> &arguments,
               std::ostream &out_stream = std::cout,
               std::ostream &err_stream = std::cerr
@@ -146,10 +137,8 @@ class MySQLRouter {
    * @param argv pointer to first command line argument
    * @param out_stream output stream representing "stdout"
    * @param err_stream output stream representing "stderr"
+   * @param sys_user_operations .oO( ... )
    */
-#ifndef _WIN32
-  /// @param sys_user_operations system operations which provide chown, ...
-#endif
   MySQLRouter(const int argc, char **argv, std::ostream &out_stream,
               std::ostream &err_stream
 #ifndef _WIN32
@@ -165,7 +154,7 @@ class MySQLRouter {
    *
    * Initializes main logger, according to options in the configuration.
    *
-   * @param config Configuration to be used to initialize logger
+   * @param config Configuaration to be used to initialize logger
    * @param raw_mode If true, all messages are logged raw; if false, messages
    *        are subject formatting
    * @param use_os_log If true, Windows EventLog will be used instead of STDERR;
@@ -292,6 +281,31 @@ class MySQLRouter {
     return extra_config_files_;
   }
 
+  /** @brief Returns predefined (computed) default paths
+   *
+   * Returns a map of predefined default paths, which are computed based on
+   * `origin` argument. This argument serves as base directory for any
+   * predefined relative paths. The returned map consists of absolue paths.
+   *
+   * @param origin Base directory which will be prepended to any relative
+   *        predefined directories
+   *
+   * @throws std::invalid_argument (std::logic_error) if `origin` is empty
+   */
+  static std::map<std::string, std::string> get_default_paths(
+      const mysql_harness::Path &origin);
+
+  /** @brief Returns absolute path to mysqlrouter.exe currently running
+   *
+   * @param argv0 1th element of `argv` array passed to `main()` (i.e.
+   * `argv[0]`)
+   *
+   * @throws std::runtime_error, ...?
+   *
+   * @note argv0 is currently ignored on Windows platforms
+   */
+  static std::string find_full_path(const std::string &argv0);
+
 #if !defined(_MSC_VER) && !defined(UNIT_TESTS)
   // MSVC produces different symbols for private vs public methods, which mean
   // the #define private public trick for unit-testing private methods doesn't
@@ -321,11 +335,9 @@ class MySQLRouter {
    * use it.
    * @endinternal
    *
-   * @param program_name path to the executable.
    * @param arguments command line arguments as vector of strings
    */
-  virtual void init(const std::string &program_name,
-                    const std::vector<std::string> &arguments);
+  virtual void init(const std::vector<std::string> &arguments);
 
   /** @brief Prepares a command line option
    *
@@ -400,16 +412,6 @@ class MySQLRouter {
    */
   void assert_not_bootstrap_mode(const std::string &option_name) const;
 
-  /**
-   * @brief verify that option given by user is an integer value in the given
-   * range.
-   *
-   * @throw std::out_of_range - option not in [min, max] range
-   * @throw std::invalid_argument - not a valid integer
-   */
-  void assert_option_value_in_range(const std::string &option_value,
-                                    const int min, const int max) const;
-
   /** @brief Shows command line usage and option description
    *
    * Shows command line usage and all available options together with their
@@ -453,9 +455,7 @@ class MySQLRouter {
    */
   void set_default_config_files(const char *locations) noexcept;
 
-  void bootstrap(const std::string &program_name,
-                 const std::string &metadata_server_uri,
-                 const std::string &plugin_folder = "");
+  void bootstrap(const std::string &metadata_server_uri);
 
   /*
    * @brief returns id of the router.
@@ -512,7 +512,6 @@ class MySQLRouter {
   std::vector<std::string> config_files_;
   /** @brief PID file location **/
   std::string pid_file_path_;
-  bool pid_file_created_{false};
 
   /** @brief CmdArgHandler object handling command line arguments **/
   CmdArgHandler arg_handler_;
@@ -556,8 +555,6 @@ class MySQLRouter {
   std::ostream &out_stream_;
   std::ostream &err_stream_;
 
-  bool core_file_{false};
-
 #ifndef _WIN32
   /** @brief Value of the --user parameter given on the command line if router
    * is launched in bootstrap mode **/
@@ -575,11 +572,12 @@ class MySQLRouter {
   mysqlrouter::SysUserOperationsBase *sys_user_operations_;
 #endif
 
-  mysql_harness::SignalHandler signal_handler_;
-
 #ifdef FRIEND_TEST
   FRIEND_TEST(Bug24909259, PasswordPrompt_plain);
   FRIEND_TEST(Bug24909259, PasswordPrompt_keyed);
+  FRIEND_TEST(ConfigGeneratorTest, ssl_stage1_cmdline_arg_parse);
+  FRIEND_TEST(ConfigGeneratorTest, ssl_stage2_bootstrap_connection);
+  FRIEND_TEST(ConfigGeneratorTest, ssl_stage3_create_config);
 #endif
 };
 
@@ -588,10 +586,4 @@ class silent_exception : public std::exception {
   silent_exception() : std::exception() {}
 };
 
-/** @brief Expose the configured application-level options (the ones
- * from [DEFAULT] section that are not specific to any particular plugin and are
- * relevant from the Cluster metadata perspective) **/
-void expose_router_configuration(const bool initial,
-                                 const mysql_harness::ConfigSection &section);
-
 #endif  // ROUTER_MYSQL_ROUTER_INCLUDED
