@@ -1,17 +1,16 @@
 /*
-  Copyright (c) 2015, 2024, Oracle and/or its affiliates.
+  Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,9 +25,6 @@
 #ifndef ROUTING_MYSQLROUTING_INCLUDED
 #define ROUTING_MYSQLROUTING_INCLUDED
 
-#include "mysqlrouter/routing_export.h"
-#include "mysqlrouter/routing_plugin_export.h"
-
 /** @file
  * @brief Defining the class MySQLRouting
  *
@@ -39,11 +35,9 @@
 
 #include <array>
 #include <atomic>
-#include <chrono>
 #include <iostream>
 #include <map>
 #include <memory>
-#include <mutex>
 #include <stdexcept>
 
 #ifndef _WIN32
@@ -63,104 +57,27 @@
 #include "connection_container.h"
 #include "context.h"
 #include "destination.h"
-#include "destination_ssl_context.h"
 #include "mysql/harness/filesystem.h"
-#include "mysql/harness/net_ts/internet.h"
-#include "mysql/harness/net_ts/io_context.h"
-#include "mysql/harness/net_ts/local.h"
 #include "mysql/harness/plugin.h"
-#include "mysql/harness/stdx/monitor.h"
 #include "mysql_router_thread.h"
-#include "mysql_routing_base.h"
-#include "mysqlrouter/base_protocol.h"
-#include "mysqlrouter/io_thread.h"
+#include "mysqlrouter/mysql_protocol.h"
 #include "mysqlrouter/routing.h"
-#include "mysqlrouter/routing_component.h"
+#include "mysqlrouter/routing_export.h"
 #include "mysqlrouter/uri.h"
 #include "plugin_config.h"
-#include "socket_container.h"
+#include "protocol/base_protocol.h"
+#include "router_config.h"
 #include "tcp_address.h"
+#include "utils.h"
 
 namespace mysql_harness {
 class PluginFuncEnv;
-}  // namespace mysql_harness
+}
 
-struct Nothing {};
-class MySQLRouting;
+using mysqlrouter::URI;
+using std::string;
 
-class ROUTING_EXPORT AcceptingEndpoint {
- public:
-  AcceptingEndpoint(net::io_context &io_ctx,
-                    const std::string &parent_routing_name)
-      : io_ctx_(io_ctx), parent_routing_name_(parent_routing_name) {}
-
-  AcceptingEndpoint(const AcceptingEndpoint &) = delete;
-
-  virtual stdx::expected<void, std::error_code> setup() = 0;
-  virtual stdx::expected<void, std::error_code> cancel() = 0;
-  virtual bool is_open() const = 0;
-
-  virtual void start(MySQLRouting *r, std::list<IoThread> &,
-                     WaitableMonitor<Nothing> &waitable) = 0;
-
-  virtual std::string name() = 0;
-
-  virtual ~AcceptingEndpoint() {}
-
- protected:
-  net::io_context &io_ctx_;
-  // used when the acceptor logs
-  std::string parent_routing_name_;
-};
-
-class ROUTING_EXPORT AcceptingEndpointTcpSocket : public AcceptingEndpoint {
- public:
-  AcceptingEndpointTcpSocket(net::io_context &io_ctx,
-                             const std::string &parent_routing_name,
-                             const std::string &address, uint16_t port);
-
-  stdx::expected<void, std::error_code> setup() override;
-  stdx::expected<void, std::error_code> cancel() override;
-  bool is_open() const override;
-
-  void start(MySQLRouting *r, std::list<IoThread> &,
-             WaitableMonitor<Nothing> &waitable) override;
-
-  std::string name() override;
-
- private:
-  net::ip::tcp::acceptor service_;
-  net::ip::tcp::endpoint service_endpoint_;
-
-  std::string address_;
-  uint16_t port_;
-};
-
-#ifndef _WIN32
-class ROUTING_EXPORT AcceptingEndpointUnixSocket : public AcceptingEndpoint {
- public:
-  AcceptingEndpointUnixSocket(net::io_context &io_ctx,
-                              const std::string &parent_routing_name,
-                              const std::string &socket_name);
-
-  stdx::expected<void, std::error_code> setup() override;
-  stdx::expected<void, std::error_code> cancel() override;
-  bool is_open() const override;
-
-  void start(MySQLRouting *r, std::list<IoThread> &,
-             WaitableMonitor<Nothing> &waitable) override;
-
-  std::string name() override;
-
- private:
-  local::stream_protocol::acceptor service_;
-  local::stream_protocol::endpoint service_endpoint_;
-
-  std::string socket_name_;
-};
-#endif
-
-/** @class MySQLRouting
+/** @class MySQLRoutering
  *  @brief Manage Connections from clients to MySQL servers
  *
  *  The class MySQLRouter is used to start a service listening on a particular
@@ -169,8 +86,8 @@ class ROUTING_EXPORT AcceptingEndpointUnixSocket : public AcceptingEndpoint {
  *
  *  Connection routing will not analyze or parse any MySQL package (except from
  *  those in the handshake phase to be able to discover invalid connection
- *  error) nor will it do any authentication. It will not handle errors from the
- *  MySQL server and not automatically recover. The client communicate through
+ * error) nor will it do any authentication. It will not handle errors from the
+ * MySQL server and not automatically recover. The client communicate through
  *  MySQL Router just like it would directly connecting.
  *
  *  The MySQL Server is chosen from a given list of hosts or IP addresses
@@ -182,41 +99,66 @@ class ROUTING_EXPORT AcceptingEndpointUnixSocket : public AcceptingEndpoint {
  *
  *  Example usage: bind to all IP addresses and use TCP Port 7001
  *
- *  @code
- *      MySQLRouting r(conf, ioctx);
- *      r.destination_connect_timeout = std::chrono::seconds(1);
- *      r.set_destinations_from_csv("10.0.10.5;10.0.11.6");
- *      r.run();
- *  @endcode
+ *   MySQLRouting r(routing::AccessMode::kReadWrite, "0.0.0.0", 7001);
+ *   r.destination_connect_timeout = std::chrono::seconds(1);
+ *   r.set_destinations_from_csv("10.0.10.5;10.0.11.6");
+ *   r.start();
  *
  *  The above example will, when MySQL running on 10.0.10.5 is not available,
  *  use 10.0.11.6 to setup the connection routing.
  *
  */
-class ROUTING_EXPORT MySQLRouting : public MySQLRoutingBase {
+class MySQLRouting {
  public:
   /** @brief Default constructor
    *
-   * @param routing_config routing configuration
-   * @param io_ctx IO context
+   * @param routing_strategy routing strategy
+   * @param port TCP port for listening for incoming connections
+   * @param protocol protocol for the routing
+   * @param access_mode access mode of the servers
+   * @param bind_address bind_address Bind to particular IP address
+   * @param named_socket Bind to Unix socket/Windows named pipe
    * @param route_name Name of connection routing (can be empty string)
-   * @param client_ssl_ctx SSL context of the client side
-   * @param dest_ssl_ctx SSL contexts of the destinations
+   * @param max_connections Maximum allowed active connections
+   * @param destination_connect_timeout Timeout trying to connect destination
+   * server
+   * @param max_connect_errors Maximum connect or handshake errors per host
+   * @param connect_timeout Timeout waiting for handshake response
+   * @param net_buffer_length send/receive buffer size
+   * @param routing_sock_ops object handling the operations on network sockets
+   * @param thread_stack_size memory in kilobytes allocated for thread's stack
    */
-  MySQLRouting(const RoutingConfig &routing_config, net::io_context &io_ctx,
-               const std::string &route_name = {},
-               TlsServerContext *client_ssl_ctx = nullptr,
-               DestinationTlsContext *dest_ssl_ctx = nullptr);
+  MySQLRouting(
+      routing::RoutingStrategy routing_strategy, uint16_t port,
+      const Protocol::Type protocol,
+      const routing::AccessMode access_mode = routing::AccessMode::kUndefined,
+      const string &bind_address = string{"0.0.0.0"},
+      const mysql_harness::Path &named_socket = mysql_harness::Path(),
+      const string &route_name = string{},
+      int max_connections = routing::kDefaultMaxConnections,
+      std::chrono::milliseconds destination_connect_timeout =
+          routing::kDefaultDestinationConnectionTimeout,
+      unsigned long long max_connect_errors = routing::kDefaultMaxConnectErrors,
+      std::chrono::milliseconds connect_timeout =
+          routing::kDefaultClientConnectTimeout,
+      unsigned int net_buffer_length = routing::kDefaultNetBufferLength,
+      routing::RoutingSockOpsInterface *routing_sock_ops =
+          routing::RoutingSockOps::instance(
+              mysql_harness::SocketOperations::instance()),
+      size_t thread_stack_size = mysql_harness::kDefaultStackSizeInKiloBytes);
 
-  /** @brief Runs the service and accept incoming connections
+  ~MySQLRouting();
+
+  /** @brief Starts the service and accept incoming connections
    *
-   * Runs the connection routing service and starts accepting incoming
-   * MySQL client connections.
+   * Starts the connection routing service and start accepting incoming
+   * MySQL client connections. Each connection will be further handled
+   * in a separate thread.
    *
    * @throw std::runtime_error on errors.
    *
    */
-  void run(mysql_harness::PluginFuncEnv *env);
+  void start(mysql_harness::PluginFuncEnv *env);
 
   /** @brief Sets the destinations from URI
    *
@@ -269,60 +211,44 @@ class ROUTING_EXPORT MySQLRouting : public MySQLRoutingBase {
    *
    * @return Maximum as int
    */
-  int get_max_connections() const noexcept override { return max_connections_; }
+  int get_max_connections() const noexcept { return max_connections_; }
 
   /**
-   * create new connection to MySQL Server than can handle client's
-   * traffic and adds it to connection container.
+   * @brief create new connection to MySQL Server than can handle client's
+   * traffic and adds it to connection container. Every connection runs in it's
+   * own thread of execution.
    *
-   * @param client_socket socket used to transfer data to/from client
-   * @param client_endpoint endpoint of client
+   * @param client_socket socket used to send/receive data to/from client
+   * @param client_addr address of client
    */
-  template <class ClientProtocol>
-  void create_connection(
-      typename ClientProtocol::socket client_socket,
-      const typename ClientProtocol::endpoint &client_endpoint);
+  void create_connection(int client_socket,
+                         const sockaddr_storage &client_addr);
 
-  routing::RoutingStrategy get_routing_strategy() const override;
+  routing::RoutingStrategy get_routing_strategy() const;
 
-  std::vector<mysql_harness::TCPAddress> get_destinations() const override;
+  routing::AccessMode get_mode() const;
 
-  std::vector<MySQLRoutingAPI::ConnData> get_connections() override;
+  std::vector<mysql_harness::TCPAddress> get_destinations() const;
 
-  MySQLRoutingConnectionBase *get_connection(const std::string &) override;
-
-  RouteDestination *destinations() { return destination_.get(); }
-
-  void disconnect_all();
-
-  /**
-   * Stop accepting new connections on a listening socket.
-   */
-  void stop_socket_acceptors() override;
-
-  /**
-   * Check if we are accepting connections on a routing socket.
-   *
-   * @retval true if we are accepting connections, false otherwise
-   */
-  bool is_accepting_connections() const override;
-
-  /**
-   * Start accepting new connections on a listening socket
-   *
-   * @returns std::string on errors.
-   */
-  stdx::expected<void, std::string> start_accepting_connections() override;
-
-  /**
-   * Start accepting new connections on a listening socket after it has been
-   * quarantined for lack of valid destinations
-   *
-   * @returns std::string on errors.
-   */
-  stdx::expected<void, std::string> restart_accepting_connections() override;
+  std::vector<MySQLRoutingAPI::ConnData> get_connections();
 
  private:
+  /** @brief Sets up the TCP service
+   *
+   * Sets up the TCP service binding to IP addresses and TCP port.
+   *
+   * @throw std::runtime_error on errors.
+   */
+  void setup_tcp_service();
+
+  /** @brief Sets up the named socket service
+   *
+   * Sets up the named socket service creating a socket file on UNIX systems.
+   *
+   * @throw std::runtime_error on errors.
+   */
+  void setup_named_socket_service();
+
   /** @brief Sets unix socket permissions so that the socket is accessible
    *         to all users (no-op on Windows)
    * @param socket_file path to socket file
@@ -331,32 +257,25 @@ class ROUTING_EXPORT MySQLRouting : public MySQLRoutingBase {
    */
   static void set_unix_socket_permissions(const char *socket_file);
 
-  stdx::expected<void, std::string> run_acceptor(
-      mysql_harness::PluginFuncEnv *env);
-
  public:
-  MySQLRoutingContext &get_context() override { return context_; }
-
-  bool is_running() const override { return is_running_; }
+  MySQLRoutingContext &get_context() { return context_; }
 
  private:
-  /** Monitor for notifying socket acceptor */
-  WaitableMonitor<Nothing> acceptor_waitable_{Nothing{}};
+  void start_acceptor(mysql_harness::PluginFuncEnv *env);
 
   /** @brief wrapper for data used by all connections */
   MySQLRoutingContext context_;
 
-  net::io_context &io_ctx_;
+  /** @brief object handling the operations on network sockets */
+  routing::RoutingSockOpsInterface *routing_sock_ops_;
 
   /** @brief Destination object to use when getting next connection */
   std::unique_ptr<RouteDestination> destination_;
 
-  bool is_destination_standalone_{false};
-
   /** @brief Routing strategy to use when getting next destination */
   routing::RoutingStrategy routing_strategy_;
 
-  /** @brief access_mode of the servers in the routing */
+  /** @brief Access mode of the servers in the routing */
   routing::AccessMode access_mode_;
 
   /** @brief Maximum active connections
@@ -367,19 +286,17 @@ class ROUTING_EXPORT MySQLRouting : public MySQLRoutingBase {
    */
   int max_connections_;
 
+  /** @brief Socket descriptor of the TCP service */
+  routing::native_handle_type service_tcp_;
+  /** @brief Socket descriptor of the named socket service */
+  routing::native_handle_type service_named_socket_;
+
   /** @brief used to unregister from subscription on allowed nodes changes */
   AllowedNodesChangeCallbacksListIterator allowed_nodes_list_iterator_;
 
   /** @brief container for connections */
   ConnectionContainer connection_container_;
 
-  /** Information if the routing plugin is still running. */
-  std::atomic<bool> is_running_{true};
-
-  /** Used when the accepting port is been reopened and it failed, to schedule
-   * another retry for standalone-destination(s) route. */
-  net::steady_timer accept_port_reopen_retry_timer_{io_ctx_};
-
 #ifdef FRIEND_TEST
   FRIEND_TEST(RoutingTests, bug_24841281);
   FRIEND_TEST(RoutingTests, get_routing_thread_name);
@@ -395,12 +312,10 @@ class ROUTING_EXPORT MySQLRouting : public MySQLRoutingBase {
   FRIEND_TEST(TestSetupNamedSocketService, unix_socket_permissions_failure);
 #endif
 #endif
-
-  std::vector<std::unique_ptr<AcceptingEndpoint>> accepting_endpoints_;
 };
 
 extern "C" {
-extern mysql_harness::Plugin ROUTING_PLUGIN_EXPORT harness_plugin_routing;
+extern mysql_harness::Plugin ROUTING_EXPORT harness_plugin_routing;
 }
 
 #endif  // ROUTING_MYSQLROUTING_INCLUDED
