@@ -1,17 +1,16 @@
 /*
- Copyright (c) 2018, 2024, Oracle and/or its affiliates.
+ Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2.0,
  as published by the Free Software Foundation.
 
- This program is designed to work with certain software (including
+ This program is also distributed with certain software (including
  but not limited to OpenSSL) that is licensed under separate terms,
  as designated in a particular file or component or in included license
  documentation.  The authors of MySQL hereby grant you an additional
  permission to link the program and your derivative works with the
- separately licensed software that they have either included with
- the program or referenced in the documentation.
+ separately licensed software that they have included with MySQL.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,7 +25,6 @@
 #include <fstream>
 #include <stdexcept>
 #include <system_error>
-#include <thread>
 
 #ifdef _WIN32
 #include <windows.h>
@@ -34,22 +32,15 @@
 #include <unistd.h>
 #endif
 
-#include <gmock/gmock-matchers.h>
-#include <gtest/gtest.h>
-
-#include "config_builder.h"
 #include "dim.h"
+#include "gmock/gmock.h"
 #include "keyring/keyring_manager.h"
-#include "mock_server_testutils.h"
-#include "mysql/harness/string_utils.h"  // split_string
 #include "mysqlrouter/keyring_info.h"
-#include "mysqlrouter/utils.h"  // copy_file
-#include "process_manager.h"
 #include "random_generator.h"
 #include "router_component_test.h"
-#include "router_component_testutils.h"
 #include "script_generator.h"
 #include "tcp_port_pool.h"
+#include "utils.h"
 
 /**
  * @file
@@ -72,10 +63,11 @@ MATCHER_P(FileContentNotEqual, master_key, "") {
   return file_content.str() != master_key;
 }
 
-class MasterKeyReaderWriterTest : public RouterComponentBootstrapTest {
+class MasterKeyReaderWriterTest : public RouterComponentTest {
  protected:
   void SetUp() override {
     RouterComponentTest::SetUp();
+    logging_folder = Path(tmp_dir_.name()).join("log").str();
 
     mysql_harness::DIM &dim = mysql_harness::DIM::instance();
     // RandomGenerator
@@ -94,41 +86,25 @@ class MasterKeyReaderWriterTest : public RouterComponentBootstrapTest {
     }
   }
 
-  static std::pair<std::string, std::map<std::string, std::string>>
-  metadata_cache_section() {
-    return {"metadata_cache:test",
-            {{"router_id", "1"},
-             {"user", "mysql_router1_user"},
-             {"metadata_cluster", "test"},
-             {"ttl", "500"}}};
+  std::string get_metadata_cache_section(unsigned server_port) {
+    return "[metadata_cache:test]\n"
+           "router_id=1\n"
+           "bootstrap_server_addresses=mysql://localhost:" +
+           std::to_string(server_port) +
+           "\n"
+           "user=mysql_router1_user\n"
+           "metadata_cluster=test\n"
+           "ttl=500\n\n";
   }
 
-  static std::string get_metadata_cache_section() {
-    auto section = metadata_cache_section();
-
-    return mysql_harness::ConfigBuilder::build_section(section.first,
-                                                       section.second);
-  }
-
-  static std::pair<std::string, std::map<std::string, std::string>>
-  metadata_cache_routing_section(const std::string &role,
-                                 const std::string &strategy,
-                                 uint16_t router_port) {
-    return {"routing:test_default",
-            {{"bind_port", std::to_string(router_port)},
-             {"destinations", "metadata-cache://test/default?role=" + role},
-             {"protocol", "classic"},
-             {"routing_strategy", strategy}}};
-  }
-
-  static std::string get_metadata_cache_routing_section(
-      const std::string &role, const std::string &strategy,
-      uint16_t router_port) {
-    auto section = metadata_cache_routing_section(role, strategy, router_port);
-
-    return mysql_harness::ConfigBuilder::build_section(section.first,
-                                                       section.second) +
-           "\n";
+  std::string get_metadata_cache_routing_section(const std::string &role,
+                                                 const std::string &strategy,
+                                                 unsigned router_port) {
+    return "[routing:test_default]\n"
+           "bind_port=" +
+           std::to_string(router_port) + "\n" +
+           "destinations=metadata-cache://test/default?role=" + role + "\n" +
+           "protocol=classic\n" + "routing_strategy=" + strategy + "\n";
   }
 
   KeyringInfo init_keyring() {
@@ -160,6 +136,7 @@ class MasterKeyReaderWriterTest : public RouterComponentBootstrapTest {
     ScriptGenerator script_generator(ProcessManager::get_origin(),
                                      tmp_dir_.name());
     std::map<std::string, std::string> default_section = get_DEFAULT_defaults();
+    default_section["logging_folder"] = logging_folder;
     default_section["keyring_path"] =
         Path(tmp_dir_.name()).join("keyring").str();
 
@@ -186,6 +163,7 @@ class MasterKeyReaderWriterTest : public RouterComponentBootstrapTest {
                                      tmp_dir_.name());
 
     auto default_section = get_DEFAULT_defaults();
+    default_section["logging_folder"] = logging_folder;
     default_section["keyring_path"] =
         Path(tmp_dir_.name()).join("keyring").str();
     default_section["master_key_reader"] =
@@ -195,8 +173,10 @@ class MasterKeyReaderWriterTest : public RouterComponentBootstrapTest {
     return default_section;
   }
 
+  TcpPortPool port_pool_;
   TempDirectory tmp_dir_;
   TempDirectory bootstrap_dir_;
+  std::string logging_folder;
   std::string master_key_;
 };
 
@@ -208,29 +188,32 @@ class MasterKeyReaderWriterTest : public RouterComponentBootstrapTest {
 TEST_F(MasterKeyReaderWriterTest,
        NoMasterKeyFileWhenBootstrapPassWithMasterKeyReader) {
   auto server_port = port_pool_.get_next_available();
-  auto http_port = port_pool_.get_next_available();
-  auto &server_mock =
-      launch_mysql_server_mock(get_data_dir().join("bootstrap_gr.js").str(),
-                               server_port, EXIT_SUCCESS, false, http_port);
-  set_mock_metadata(http_port, "00000000-0000-0000-0000-0000000000g1",
-                    classic_ports_to_gr_nodes({server_port}), 0, {server_port});
+  auto &server_mock = launch_mysql_server_mock(
+      get_data_dir().join("bootstrap_gr.js").str(), server_port, false);
+  ASSERT_NO_FATAL_FAILURE(check_port_ready(server_mock, server_port));
 
   ScriptGenerator script_generator(ProcessManager::get_origin(),
                                    tmp_dir_.name());
 
   // launch the router in bootstrap mode
-  auto &router = launch_router_for_bootstrap({
+  auto &router = launch_router({
       "--bootstrap=127.0.0.1:" + std::to_string(server_port),
+      "--report-host",
+      "dont.query.dns",
       "--directory=" + bootstrap_dir_.name(),
       "--force",
       "--master-key-reader=" + script_generator.get_reader_script(),
       "--master-key-writer=" + script_generator.get_writer_script(),
   });
 
-  // check if the bootstrapping was successful
-  ASSERT_NO_FATAL_FAILURE(check_exit_code(router, EXIT_SUCCESS, 30000ms));
+  // add login hook
+  router.register_response("Please enter MySQL password for root: ",
+                           "fake-pass\n");
+
+  // check if the bootstraping was successful
+  check_exit_code(router, EXIT_SUCCESS, 30000ms);
   EXPECT_TRUE(router.expect_output(
-      "MySQL Router configured for the InnoDB Cluster 'test'"))
+      "MySQL Router configured for the InnoDB Cluster 'mycluster'"))
       << router.get_full_output() << std::endl
       << "server: " << server_mock.get_full_output();
 
@@ -258,28 +241,33 @@ TEST_F(MasterKeyReaderWriterTest,
 TEST_F(MasterKeyReaderWriterTest,
        CheckConfigFileWhenBootstrapPassWithMasterKeyReader) {
   auto server_port = port_pool_.get_next_available();
-  auto http_port = port_pool_.get_next_available();
-  auto &server_mock =
-      launch_mysql_server_mock(get_data_dir().join("bootstrap_gr.js").str(),
-                               server_port, EXIT_SUCCESS, false, http_port);
-  set_mock_metadata(http_port, "00000000-0000-0000-0000-0000000000g1",
-                    classic_ports_to_gr_nodes({server_port}), 0, {server_port});
+  auto &server_mock = launch_mysql_server_mock(
+      get_data_dir().join("bootstrap_gr.js").str(), server_port, false);
+  ASSERT_NO_FATAL_FAILURE(check_port_ready(server_mock, server_port));
 
   ScriptGenerator script_generator(ProcessManager::get_origin(),
                                    tmp_dir_.name());
 
   // launch the router in bootstrap mode
-  auto &router = launch_router_for_bootstrap(
-      {"--directory=" + bootstrap_dir_.name(), "--force",
-       "--master-key-reader=" + script_generator.get_reader_script(),
-       "--master-key-writer=" + script_generator.get_writer_script(),
-       "--bootstrap=127.0.0.1:" + std::to_string(server_port)});
+  auto &router = launch_router({
+      "--directory=" + bootstrap_dir_.name(),
+      "--force",
+      "--master-key-reader=" + script_generator.get_reader_script(),
+      "--master-key-writer=" + script_generator.get_writer_script(),
+      "--report-host",
+      "dont.query.dns",
+      "--bootstrap=127.0.0.1:" + std::to_string(server_port),
+  });
 
-  // check if the bootstrapping was successful
-  ASSERT_NO_FATAL_FAILURE(check_exit_code(router, EXIT_SUCCESS, 30000ms));
+  // add login hook
+  router.register_response("Please enter MySQL password for root: ",
+                           "fake-pass\n");
+
+  // check if the bootstraping was successful
+  check_exit_code(router, EXIT_SUCCESS, 30000ms);
   EXPECT_TRUE(
       router.expect_output("MySQL Router configured for the "
-                           "InnoDB Cluster 'test'"))
+                           "InnoDB Cluster 'mycluster'"))
       << router.get_full_output() << std::endl
       << "server: " << server_mock.get_full_output();
 
@@ -311,25 +299,24 @@ TEST_F(MasterKeyReaderWriterTest,
 /**
  * @test
  *       verify that when --master-key-reader option is used, but specified
- * reader cannot be executed, then bootstrap fails and appropriate error
- * message is printed to standard output.
+ * reader cannot be executed, then bootstrap fails and appropriate error message
+ * is printed to standard output.
  */
 TEST_F(MasterKeyReaderWriterTest, BootstrapFailsWhenCannotRunMasterKeyReader) {
   auto server_port = port_pool_.get_next_available();
-  auto http_port = port_pool_.get_next_available();
-  auto &server_mock =
-      launch_mysql_server_mock(get_data_dir().join("bootstrap_gr.js").str(),
-                               server_port, EXIT_SUCCESS, false, http_port);
-  set_mock_metadata(http_port, "00000000-0000-0000-0000-0000000000g1",
-                    classic_ports_to_gr_nodes({server_port}), 0, {server_port});
+  auto &server_mock = launch_mysql_server_mock(
+      get_data_dir().join("bootstrap_gr.js").str(), server_port, false);
+  ASSERT_NO_FATAL_FAILURE(check_port_ready(server_mock, server_port));
 
   ScriptGenerator script_generator(ProcessManager::get_origin(),
                                    tmp_dir_.name());
 
   // launch the router in bootstrap mode
-  auto &router = launch_router_for_bootstrap(
+  auto &router = launch_router(
       {
           "--bootstrap=127.0.0.1:" + std::to_string(server_port),
+          "--report-host",
+          "dont.query.dns",
           "--directory=" + bootstrap_dir_.name(),
           "--force",
           "--master-key-reader=" + script_generator.get_fake_reader_script(),
@@ -337,8 +324,12 @@ TEST_F(MasterKeyReaderWriterTest, BootstrapFailsWhenCannotRunMasterKeyReader) {
       },
       EXIT_FAILURE);
 
-  // check if the bootstrapping failed
-  ASSERT_NO_FATAL_FAILURE(check_exit_code(router, EXIT_FAILURE));
+  // add login hook
+  router.register_response("Please enter MySQL password for root: ",
+                           "fake-pass\n");
+
+  // check if the bootstraping failed
+  check_exit_code(router, EXIT_FAILURE);
   EXPECT_TRUE(router.expect_output(
       "Error: Cannot fetch master key file using master key reader"))
       << router.get_full_output() << std::endl
@@ -353,20 +344,19 @@ TEST_F(MasterKeyReaderWriterTest, BootstrapFailsWhenCannotRunMasterKeyReader) {
  */
 TEST_F(MasterKeyReaderWriterTest, BootstrapFailsWhenCannotRunMasterKeyWriter) {
   auto server_port = port_pool_.get_next_available();
-  auto http_port = port_pool_.get_next_available();
-  auto &server_mock =
-      launch_mysql_server_mock(get_data_dir().join("bootstrap_gr.js").str(),
-                               server_port, EXIT_SUCCESS, false, http_port);
-  set_mock_metadata(http_port, "00000000-0000-0000-0000-0000000000g1",
-                    classic_ports_to_gr_nodes({server_port}), 0, {server_port});
+  auto &server_mock = launch_mysql_server_mock(
+      get_data_dir().join("bootstrap_gr.js").str(), server_port, false);
+  ASSERT_NO_FATAL_FAILURE(check_port_ready(server_mock, server_port));
 
   ScriptGenerator script_generator(ProcessManager::get_origin(),
                                    tmp_dir_.name());
 
   // launch the router in bootstrap mode
-  auto &router = launch_router_for_bootstrap(
+  auto &router = launch_router(
       {
           "--bootstrap=127.0.0.1:" + std::to_string(server_port),
+          "--report-host",
+          "dont.query.dns",
           "--directory=" + bootstrap_dir_.name(),
           "--force",
           "--master-key-reader=" + script_generator.get_reader_script(),
@@ -374,8 +364,12 @@ TEST_F(MasterKeyReaderWriterTest, BootstrapFailsWhenCannotRunMasterKeyWriter) {
       },
       EXIT_FAILURE);
 
-  // check if the bootstrapping failed
-  ASSERT_NO_FATAL_FAILURE(check_exit_code(router, EXIT_FAILURE));
+  // add login hook
+  router.register_response("Please enter MySQL password for root: ",
+                           "fake-pass\n");
+
+  // check if the bootstraping failed
+  check_exit_code(router, EXIT_FAILURE);
   EXPECT_TRUE(router.expect_output(
       "Error: Cannot write master key file using master key writer"))
       << router.get_full_output() << std::endl
@@ -395,37 +389,39 @@ TEST_F(MasterKeyReaderWriterTest, KeyringFileRestoredWhenBootstrapFails) {
   write_to_file(keyring_path, "keyring file content");
 
   auto server_port = port_pool_.get_next_available();
-  auto http_port = port_pool_.get_next_available();
-  launch_mysql_server_mock(get_data_dir().join("bootstrap_gr.js").str(),
-                           server_port, EXIT_SUCCESS, false, http_port);
-  set_mock_metadata(http_port, "00000000-0000-0000-0000-0000000000g1",
-                    classic_ports_to_gr_nodes({server_port}), 0, {server_port});
+  auto &server_mock = launch_mysql_server_mock(
+      get_data_dir().join("bootstrap_gr.js").str(), server_port, false);
+  ASSERT_NO_FATAL_FAILURE(check_port_ready(server_mock, server_port));
 
   ScriptGenerator script_generator(ProcessManager::get_origin(),
                                    tmp_dir_.name());
 
   // launch the router in bootstrap mode
-  auto &router = launch_router_for_bootstrap(
+  auto &router = launch_router(
       {
           "--bootstrap=127.0.0.1:" + std::to_string(server_port),
-          "--conf-set-option=DEFAULT.plugin_folder=" +
-              ProcessManager::get_plugin_dir().str(),
           "--directory=" + bootstrap_dir_.name(),
           "--force",
           "--master-key-reader=" + script_generator.get_fake_reader_script(),
           "--master-key-writer=" + script_generator.get_fake_writer_script(),
+          "--report-host",
+          "dont.query.dns",
       },
       EXIT_FAILURE);
 
-  // check if the bootstrapping failed
-  ASSERT_NO_FATAL_FAILURE(check_exit_code(router, EXIT_FAILURE));
+  // add login hook
+  router.register_response("Please enter MySQL password for root: ",
+                           "fake-pass\n");
+
+  // check if the bootstraping failed
+  check_exit_code(router, EXIT_FAILURE);
   ASSERT_THAT(keyring_path.str(), FileContentEqual("keyring file content"));
 }
 
 /**
  * @test
- *       verify that if --master-key-reader option is used and bootstrap
- * fails, then original master key is restored.
+ *       verify that if --master-key-reader option is used and bootstrap fails,
+ *       then original master key is restored.
  */
 TEST_F(MasterKeyReaderWriterTest, MasterKeyRestoredWhenBootstrapFails) {
   // create file that is used by master-key-reader and master-key-writer
@@ -437,11 +433,9 @@ TEST_F(MasterKeyReaderWriterTest, MasterKeyRestoredWhenBootstrapFails) {
                                    tmp_dir_.name());
 
   // launch the router in bootstrap mode
-  auto &router = launch_router_for_bootstrap(
+  auto &router = launch_router(
       {
           "--bootstrap=127.0.0.1:" + std::to_string(server_port),
-          "--conf-set-option=DEFAULT.plugin_folder=" +
-              ProcessManager::get_plugin_dir().str(),
           "--connect-timeout=1",
           "--directory=" + bootstrap_dir_.name(),
           "--force",
@@ -450,8 +444,12 @@ TEST_F(MasterKeyReaderWriterTest, MasterKeyRestoredWhenBootstrapFails) {
       },
       EXIT_FAILURE);
 
-  // check if the bootstrapping failed
-  ASSERT_NO_FATAL_FAILURE(check_exit_code(router, EXIT_FAILURE));
+  // add login hook
+  router.register_response("Please enter MySQL password for root: ",
+                           "fake-pass\n");
+
+  // check if the bootstraping failed
+  check_exit_code(router, EXIT_FAILURE);
   ASSERT_THAT(master_key_path.str(), FileContentEqual(""));
 }
 
@@ -466,30 +464,33 @@ TEST_F(MasterKeyReaderWriterTest,
   write_to_file(Path(tmp_dir_.name()).join("master_key"), "");
 
   auto server_port = port_pool_.get_next_available();
-  auto http_port = port_pool_.get_next_available();
-  auto &server_mock =
-      launch_mysql_server_mock(get_data_dir().join("bootstrap_gr.js").str(),
-                               server_port, EXIT_SUCCESS, false, http_port);
-  set_mock_metadata(http_port, "00000000-0000-0000-0000-0000000000g1",
-                    classic_ports_to_gr_nodes({server_port}), 0, {server_port});
+  auto &server_mock = launch_mysql_server_mock(
+      get_data_dir().join("bootstrap_gr.js").str(), server_port, false);
+  ASSERT_NO_FATAL_FAILURE(check_port_ready(server_mock, server_port));
 
   ScriptGenerator script_generator(ProcessManager::get_origin(),
                                    tmp_dir_.name());
 
   // launch the router in bootstrap mode
-  auto &router = launch_router_for_bootstrap({
+  auto &router = launch_router({
       "--bootstrap=127.0.0.1:" + std::to_string(server_port),
+      "--report-host",
+      "dont.query.dns",
       "--directory=" + bootstrap_dir_.name(),
       "--force",
       "--master-key-reader=" + script_generator.get_reader_script(),
       "--master-key-writer=" + script_generator.get_writer_script(),
   });
 
-  // check if the bootstrapping was successful
-  ASSERT_NO_FATAL_FAILURE(check_exit_code(router, EXIT_SUCCESS, 30000ms));
+  // add login hook
+  router.register_response("Please enter MySQL password for root: ",
+                           "fake-pass\n");
+
+  // check if the bootstraping was successful
+  check_exit_code(router, EXIT_SUCCESS, 30000ms);
   EXPECT_TRUE(
       router.expect_output("MySQL Router configured for the "
-                           "InnoDB Cluster 'test'"))
+                           "InnoDB Cluster 'mycluster'"))
       << router.get_full_output() << std::endl
       << "server: " << server_mock.get_full_output();
 
@@ -502,33 +503,37 @@ TEST_F(MasterKeyReaderWriterTest,
 /*
  * @test
  *       verify that if master key exists and is not empty and bootstrap pass,
- * then original master key is not overridden.
+ * then original master key is not overriden.
  */
 TEST_F(MasterKeyReaderWriterTest,
        DontWriteMasterKeyAtBootstrapIfMasterKeyAlreadyExists) {
   write_to_file(Path(tmp_dir_.name()).join("master_key"), "master key value");
 
   auto server_port = port_pool_.get_next_available();
-  auto http_port = port_pool_.get_next_available();
-  launch_mysql_server_mock(get_data_dir().join("bootstrap_gr.js").str(),
-                           server_port, EXIT_SUCCESS, false, http_port);
-  set_mock_metadata(http_port, "00000000-0000-0000-0000-0000000000g1",
-                    classic_ports_to_gr_nodes({server_port}), 0, {server_port});
+  auto &server_mock = launch_mysql_server_mock(
+      get_data_dir().join("bootstrap_gr.js").str(), server_port, false);
+  ASSERT_NO_FATAL_FAILURE(check_port_ready(server_mock, server_port));
 
   ScriptGenerator script_generator(ProcessManager::get_origin(),
                                    tmp_dir_.name());
 
   // launch the router in bootstrap mode
-  auto &router = launch_router_for_bootstrap({
+  auto &router = launch_router({
       "--bootstrap=127.0.0.1:" + std::to_string(server_port),
+      "--report-host",
+      "dont.query.dns",
       "--directory=" + bootstrap_dir_.name(),
       "--force",
       "--master-key-reader=" + script_generator.get_reader_script(),
       "--master-key-writer=" + script_generator.get_writer_script(),
   });
 
-  // check if the bootstrapping failed
-  ASSERT_NO_FATAL_FAILURE(check_exit_code(router));
+  // add login hook
+  router.register_response("Please enter MySQL password for root: ",
+                           "fake-pass\n");
+
+  // check if the bootstraping failed
+  check_exit_code(router);
   ASSERT_THAT(Path(tmp_dir_.name()).join("master_key").str(),
               FileContentEqual("master key value"));
 }
@@ -540,102 +545,86 @@ TEST_F(MasterKeyReaderWriterTest,
  *
  */
 TEST_F(MasterKeyReaderWriterTest, ConnectToMetadataServerPass) {
-  const auto server_port = port_pool_.get_next_available();
-  const auto router_port = port_pool_.get_next_available();
+  auto server_port = port_pool_.get_next_available();
+  auto router_port = port_pool_.get_next_available();
+  std::string metadata_cache_section = get_metadata_cache_section(server_port);
+  std::string routing_section =
+      get_metadata_cache_routing_section("PRIMARY", "round-robin", router_port);
+
+  init_keyring();
 
   // launch server
-  /*auto &server =*/launch_mysql_server_mock(
-      get_data_dir().join("metadata_dynamic_nodes_v2_gr.js").str(),
-      server_port);
+  auto &server = launch_mysql_server_mock(
+      get_data_dir().join("metadata_dynamic_nodes.js").str(), server_port,
+      false);
+  ASSERT_NO_FATAL_FAILURE(check_port_ready(server, server_port));
 
+  auto default_section_map = get_default_section_map();
   // launch the router with metadata-cache configuration
   TempDirectory conf_dir("conf");
+  auto &router = ProcessManager::launch_router({
+      "-c",
+      create_config_file(conf_dir.name(),
+                         "[logger]\nlevel = DEBUG\n" + metadata_cache_section +
+                             routing_section,
+                         &default_section_map),
+  });
 
-  auto writer = config_writer(conf_dir.name())
-                    .section(metadata_cache_section())
-                    .section(metadata_cache_routing_section(
-                        "PRIMARY", "round-robin", router_port));
+  // in windows waiting for the router's keyring reader takes about 2seconds,
+  // and we need to do 3 rounds
+  ASSERT_NO_FATAL_FAILURE(check_port_ready(router, router_port, 10000ms));
 
-  // set master-key-reader/writer in DEFAULT section
-  auto &default_section = writer.sections().at("DEFAULT");
+  auto matcher = [&](const std::string &line) -> bool {
+    return line.find("Connected with metadata server running on") != line.npos;
+  };
 
-  const auto keyring_info = init_keyring();
-
-  default_section["keyring_path"] = keyring_info.get_keyring_file();
-  default_section["master_key_reader"] = keyring_info.get_master_key_reader();
-  default_section["master_key_writer"] = keyring_info.get_master_key_writer();
-
-  const auto state_file = create_state_file(
-      get_test_temp_dir_name(),
-      create_state_file_content("uuid", "", {server_port}, 0));
-  default_section["dynamic_state"] = state_file;
-
-  auto &router =
-      router_spawner()
-          // ERROR: 1273 Syntax Error at: ROLLBACK ...
-          // it will never get READY.
-          .wait_for_sync_point(ProcessManager::Spawner::SyncPoint::RUNNING)
-          .spawn({"-c", writer.write()});
-
-  EXPECT_TRUE(wait_log_contains(
-      router, "Connected with metadata server running on", 10s));
+  EXPECT_TRUE(find_in_file(logging_folder + "/mysqlrouter.log", matcher,
+                           std::chrono::milliseconds(10000)));
 }
 
 /**
  * @test
  *       verify that when master key returned by master-key-reader is correct
- *       and then launching the router succeeds, then master-key is not
- * written to log files.
+ *       and then launching the router succeeds, then master-key is not written
+ *       to log files.
  *
  */
 TEST_F(MasterKeyReaderWriterTest,
        NoMasterKeyInLogsWhenConnectToMetadataServerPass) {
-  const auto server_port = port_pool_.get_next_available();
-  const auto router_port = port_pool_.get_next_available();
+  unsigned server_port = port_pool_.get_next_available();
+  unsigned router_port = port_pool_.get_next_available();
+  std::string metadata_cache_section = get_metadata_cache_section(server_port);
+  std::string routing_section =
+      get_metadata_cache_routing_section("PRIMARY", "round-robin", router_port);
+
+  init_keyring();
 
   // launch server
   auto &server = launch_mysql_server_mock(
-      get_data_dir().join("metadata_dynamic_nodes.js").str(), server_port);
+      get_data_dir().join("metadata_dynamic_nodes.js").str(), server_port,
+      false);
   ASSERT_NO_FATAL_FAILURE(check_port_ready(server, server_port, 10000ms));
 
-  TempDirectory conf_dir("conf");
-
-  auto writer = config_writer(conf_dir.name())
-                    .section(metadata_cache_section())
-                    .section(metadata_cache_routing_section(
-                        "PRIMARY", "round-robin", router_port));
-
-  // set master-key-reader/writer in DEFAULT section
-  auto &default_section = writer.sections().at("DEFAULT");
-
-  const auto keyring_info = init_keyring();
-
-  default_section["keyring_path"] = keyring_info.get_keyring_file();
-  default_section["master_key_reader"] = keyring_info.get_master_key_reader();
-  default_section["master_key_writer"] = keyring_info.get_master_key_writer();
-
-  const auto state_file = create_state_file(
-      get_test_temp_dir_name(),
-      create_state_file_content("uuid", "", {server_port}, 0));
-  default_section["dynamic_state"] = state_file;
-
+  auto default_section_map = get_default_section_map();
   // launch the router with metadata-cache configuration
+  TempDirectory conf_dir("conf");
+  auto &router = ProcessManager::launch_router(
+      {"-c", create_config_file(conf_dir.name(),
+                                "[logger]\nlevel = DEBUG\n" +
+                                    metadata_cache_section + routing_section,
+                                &default_section_map)});
 
-  auto &router =
-      router_spawner()
-          // ERROR: 1273 Syntax Error at: ROLLBACK ...
-          // it will never get READY.
-          .wait_for_sync_point(ProcessManager::Spawner::SyncPoint::RUNNING)
-          .spawn({"-c", writer.write()});
+  // in windows waiting for the router's keyring reader takes about 2seconds,
+  // and we need to do 3 rounds
+  ASSERT_NO_FATAL_FAILURE(check_port_ready(router, router_port, 10000ms));
 
-  // wait a bit for log to generate.
-  std::this_thread::sleep_for(1s);
+  auto matcher = [&, this](const std::string &line) -> bool {
+    return line.find(master_key_) != line.npos;
+  };
 
-  auto log_lines =
-      mysql_harness::split_string(router.get_logfile_content(), '\n');
-  EXPECT_THAT(
-      log_lines,
-      ::testing::Not(::testing::Contains(::testing::HasSubstr(master_key_))));
+  EXPECT_FALSE(find_in_file(logging_folder + "/mysqlrouter.log", matcher,
+                            std::chrono::milliseconds(1000)))
+      << router.get_full_output();
 }
 
 /**
@@ -646,7 +635,7 @@ TEST_F(MasterKeyReaderWriterTest,
 TEST_F(MasterKeyReaderWriterTest, CannotLaunchRouterWhenNoMasterKeyReader) {
   auto server_port = port_pool_.get_next_available();
   auto router_port = port_pool_.get_next_available();
-  std::string metadata_cache_section = get_metadata_cache_section();
+  std::string metadata_cache_section = get_metadata_cache_section(server_port);
   std::string routing_section =
       get_metadata_cache_routing_section("PRIMARY", "round-robin", router_port);
 
@@ -661,16 +650,17 @@ TEST_F(MasterKeyReaderWriterTest, CannotLaunchRouterWhenNoMasterKeyReader) {
   auto default_section_map = get_default_section_map(true, true);
   // launch the router with metadata-cache configuration
   TempDirectory conf_dir("conf");
-  auto &router = launch_router(
+  auto &router = ProcessManager::launch_router(
       {
           "-c",
           create_config_file(conf_dir.name(),
-                             metadata_cache_section + routing_section,
+                             "[logger]\nlevel = DEBUG\n" +
+                                 metadata_cache_section + routing_section,
                              &default_section_map),
       },
-      EXIT_FAILURE, true, false, -1s);
+      EXIT_FAILURE);
 
-  ASSERT_NO_FATAL_FAILURE(check_exit_code(router, EXIT_FAILURE));
+  check_exit_code(router, EXIT_FAILURE);
 }
 
 /**
@@ -681,7 +671,7 @@ TEST_F(MasterKeyReaderWriterTest, CannotLaunchRouterWhenNoMasterKeyReader) {
 TEST_F(MasterKeyReaderWriterTest, CannotLaunchRouterWhenMasterKeyIncorrect) {
   auto server_port = port_pool_.get_next_available();
   auto router_port = port_pool_.get_next_available();
-  std::string metadata_cache_section = get_metadata_cache_section();
+  std::string metadata_cache_section = get_metadata_cache_section(server_port);
   std::string routing_section =
       get_metadata_cache_routing_section("PRIMARY", "round-robin", router_port);
 
@@ -697,24 +687,24 @@ TEST_F(MasterKeyReaderWriterTest, CannotLaunchRouterWhenMasterKeyIncorrect) {
       get_incorrect_master_key_default_section_map();
   // launch the router with metadata-cache configuration
   TempDirectory conf_dir("conf");
-  auto &router = launch_router(
+  auto &router = ProcessManager::launch_router(
       {"-c", create_config_file(conf_dir.name(),
-                                metadata_cache_section + routing_section,
+                                "[logger]\nlevel = DEBUG\n" +
+                                    metadata_cache_section + routing_section,
                                 &incorrect_master_key_default_section_map)},
-      EXIT_FAILURE, true, false, -1s);
+      EXIT_FAILURE);
 
-  ASSERT_NO_FATAL_FAILURE(check_exit_code(router, EXIT_FAILURE));
+  check_exit_code(router, EXIT_FAILURE);
 }
 /*
- * These tests are executed only for STANDALONE layout and are not executed
- * for Windows. Bootstrap for layouts different than STANDALONE use
- * directories to which tests don't have access (see install_layout.cmake).
+ * These tests are executed only for STANDALONE layout and are not executed for
+ * Windows. Bootstrap for layouts different than STANDALONE use directories to
+ * which tests don't have access (see install_layout.cmake).
  */
 Path g_origin_path;
 #ifndef SKIP_BOOTSTRAP_SYSTEM_DEPLOYMENT_TESTS
 
-class MasterKeyReaderWriterSystemDeploymentTest
-    : public MasterKeyReaderWriterTest {
+class MasterKeyReaderWriterSystemDeploymentTest : public RouterComponentTest {
  protected:
   void SetUp() override {
     // this test modifies the origin path so we need to restore it
@@ -778,17 +768,15 @@ class MasterKeyReaderWriterSystemDeploymentTest
   auto &run_server_mock() {
     const std::string json_stmts = get_data_dir().join("bootstrap_gr.js").str();
     server_port_ = port_pool_.get_next_available();
-    const auto http_port = port_pool_.get_next_available();
 
     // launch mock server and wait for it to start accepting connections
-    auto &server_mock = launch_mysql_server_mock(
-        json_stmts, server_port_, EXIT_SUCCESS, false, http_port);
-    set_mock_metadata(http_port, "00000000-0000-0000-0000-0000000000g1",
-                      classic_ports_to_gr_nodes({server_port_}), 0,
-                      {server_port_});
+    auto &server_mock = launch_mysql_server_mock(json_stmts, server_port_);
     return server_mock;
   }
 
+  TcpPortPool port_pool_;
+
+  TempDirectory tmp_dir_;
   std::string exec_file_;
   std::string config_file_;
 #ifdef __APPLE__
@@ -812,17 +800,24 @@ TEST_F(MasterKeyReaderWriterSystemDeploymentTest, BootstrapPass) {
                                    tmp_dir_.name());
 
   // launch the router in bootstrap mode
-  auto &router = launch_router_for_bootstrap({
+  auto &router = launch_router({
       "--bootstrap=127.0.0.1:" + std::to_string(server_port_),
+      "--report-host",
+      "dont.query.dns",
       "--master-key-reader=" + script_generator.get_reader_script(),
       "--master-key-writer=" + script_generator.get_writer_script(),
   });
 
-  // check if the bootstrapping was successful
-  ASSERT_NO_FATAL_FAILURE(check_exit_code(router, EXIT_SUCCESS));
+  // add login hook
+  router.register_response("Please enter MySQL password for root: ",
+                           "fake-pass\n");
 
-  EXPECT_TRUE(router.expect_output(
-      "MySQL Router configured for the InnoDB Cluster 'test'"))
+  // check if the bootstraping was successful
+  check_exit_code(router, EXIT_SUCCESS);
+
+  EXPECT_TRUE(
+      router.expect_output("MySQL Router configured for the "
+                           "InnoDB Cluster 'mycluster'"))
       << router.get_full_output() << std::endl
       << "server: " << server_mock.get_full_output();
 
@@ -833,9 +828,9 @@ TEST_F(MasterKeyReaderWriterSystemDeploymentTest, BootstrapPass) {
 
 /**
  * @test
- *       verify if bootstrap with --master-key-reader and with system
- * deployment layout, but specified reader cannot be executed, then bootstrap
- * fails and appropriate error message is printed to standard output.
+ *       verify if bootstrap with --master-key-reader and with system deployment
+ * layout, but specified reader cannot be executed, then bootstrap fails and
+ * appropriate error message is printed to standard output.
  */
 TEST_F(MasterKeyReaderWriterSystemDeploymentTest,
        BootstrapFailsWhenCannotRunMasterKeyReader) {
@@ -846,16 +841,22 @@ TEST_F(MasterKeyReaderWriterSystemDeploymentTest,
                                    tmp_dir_.name());
 
   // launch the router in bootstrap mode
-  auto &router = launch_router_for_bootstrap(
+  auto &router = launch_router(
       {
           "--bootstrap=127.0.0.1:" + std::to_string(server_port_),
+          "--report-host",
+          "dont.query.dns",
           "--master-key-reader=" + script_generator.get_fake_reader_script(),
           "--master-key-writer=" + script_generator.get_writer_script(),
       },
       EXIT_FAILURE);
 
-  // check if the bootstrapping failed
-  ASSERT_NO_FATAL_FAILURE(check_exit_code(router, EXIT_FAILURE));
+  // add login hook
+  router.register_response("Please enter MySQL password for root: ",
+                           "fake-pass\n");
+
+  // check if the bootstraping failed
+  check_exit_code(router, EXIT_FAILURE);
 
   EXPECT_TRUE(router.expect_output(
       "Error: Cannot fetch master key file using master key reader"))
@@ -878,16 +879,22 @@ TEST_F(MasterKeyReaderWriterSystemDeploymentTest,
                                    tmp_dir_.name());
 
   // launch the router in bootstrap mode
-  auto &router = launch_router_for_bootstrap(
+  auto &router = launch_router(
       {
           "--bootstrap=127.0.0.1:" + std::to_string(server_port_),
+          "--report-host",
+          "dont.query.dns",
           "--master-key-reader=" + script_generator.get_reader_script(),
           "--master-key-writer=" + script_generator.get_fake_writer_script(),
       },
       EXIT_FAILURE);
 
-  // check if the bootstrapping failed
-  ASSERT_NO_FATAL_FAILURE(check_exit_code(router, EXIT_FAILURE));
+  // add login hook
+  router.register_response("Please enter MySQL password for root: ",
+                           "fake-pass\n");
+
+  // check if the bootstraping failed
+  check_exit_code(router, EXIT_FAILURE);
 
   EXPECT_TRUE(router.expect_output(
       "Error: Cannot write master key file using master key writer"))
@@ -918,27 +925,31 @@ TEST_F(MasterKeyReaderWriterSystemDeploymentTest,
                                    tmp_dir_.name());
 
   // launch the router in bootstrap mode
-  auto &router = launch_router_for_bootstrap(
+  auto &router = launch_router(
       {
           "--bootstrap=127.0.0.1:" + std::to_string(server_port_),
-          "--conf-set-option=DEFAULT.plugin_folder=" +
-              ProcessManager::get_plugin_dir().str(),
           "--connect-timeout=1",
           "--master-key-reader=" + script_generator.get_fake_reader_script(),
           "--master-key-writer=" + script_generator.get_fake_writer_script(),
+          "--report-host",
+          "dont.query.dns",
       },
       EXIT_FAILURE);
 
-  // check if the bootstrapping failed
-  ASSERT_NO_FATAL_FAILURE(check_exit_code(router, EXIT_FAILURE));
+  // add login hook
+  router.register_response("Please enter MySQL password for root: ",
+                           "fake-pass\n");
+
+  // check if the bootstraping failed
+  check_exit_code(router, EXIT_FAILURE);
 
   ASSERT_THAT(keyring_path.str(), FileContentEqual("keyring file content"));
 }
 
 /**
  * @test
- *       verify bootstrap with --master-key-reader and system deployment
- * layout and bootstrap fails, then original master key is restored.
+ *       verify bootstrap with --master-key-reader and system deployment layout
+ *       and bootstrap fails, then original master key is restored.
  */
 TEST_F(MasterKeyReaderWriterSystemDeploymentTest,
        MasterKeyRestoredWhenBootstrapFails) {
@@ -951,19 +962,21 @@ TEST_F(MasterKeyReaderWriterSystemDeploymentTest,
                                    tmp_dir_.name());
 
   // launch the router in bootstrap mode
-  auto &router = launch_router_for_bootstrap(
+  auto &router = launch_router(
       {
           "--bootstrap=127.0.0.1:" + std::to_string(server_port),
-          "--conf-set-option=DEFAULT.plugin_folder=" +
-              ProcessManager::get_plugin_dir().str(),
           "--connect-timeout=1",
           "--master-key-reader=" + script_generator.get_reader_script(),
           "--master-key-writer=" + script_generator.get_writer_script(),
       },
       EXIT_FAILURE);
 
-  // check if the bootstrapping failed
-  ASSERT_NO_FATAL_FAILURE(check_exit_code(router, EXIT_FAILURE));
+  // add login hook
+  router.register_response("Please enter MySQL password for root: ",
+                           "fake-pass\n");
+
+  // check if the bootstraping failed
+  check_exit_code(router, EXIT_FAILURE);
 
   ASSERT_THAT(master_key_path.str(), FileContentEqual(""));
 }
