@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2020, 2023, Oracle and/or its affiliates.
+Copyright (c) 2020, Oracle and/or its affiliates. All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
@@ -31,8 +31,8 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 #include "keyring/keyring_manager.h"
 #include "mock_server_rest_client.h"
 #include "mock_server_testutils.h"
+#include "mysql_session.h"
 #include "mysqlrouter/cluster_metadata.h"
-#include "mysqlrouter/mysql_session.h"
 #include "rest_api_testutils.h"
 #include "router_component_test.h"
 #include "tcp_port_pool.h"
@@ -141,10 +141,8 @@ class MetadataHttpAuthTest : public RouterComponentTest {
     return result;
   }
 
-  auto &launch_router(
-      const std::string &metadata_cache_section,
-      const int expected_errorcode = EXIT_SUCCESS,
-      const std::chrono::milliseconds wait_for_notify_ready = 30s) {
+  auto &launch_router(const std::string &metadata_cache_section,
+                      const int expected_errorcode = EXIT_SUCCESS) {
     const std::string &temp_test_dir_str = temp_test_dir.name();
 
     const auto &routing_section =
@@ -167,32 +165,32 @@ class MetadataHttpAuthTest : public RouterComponentTest {
     mysql_harness::flush_keyring();
     mysql_harness::reset_keyring();
 
+    // enable debug logs for better diagnostics in case of failure
+    const std::string logger_section =
+        "[logger]\nlevel = DEBUG\ntimestamp_precision=millisecond\n";
+
     // launch the router with metadata-cache configuration
     auto default_section = get_DEFAULT_defaults();
     default_section["keyring_path"] = keyring_file;
     default_section["master_key_path"] = masterkey_file;
     default_section["dynamic_state"] = state_file;
-    const std::string conf_file = create_config_file(
-        temp_test_dir_str,
-        metadata_cache_section + routing_section + rest_section,
-        &default_section);
+    const std::string conf_file =
+        create_config_file(temp_test_dir_str,
+                           logger_section + metadata_cache_section +
+                               routing_section + rest_section,
+                           &default_section);
 
-    return ProcessManager::launch_router(
-        {"-c", conf_file}, expected_errorcode, /*catch_stderr=*/true,
-        /*with_sudo=*/false, wait_for_notify_ready);
+    return ProcessManager::launch_router({"-c", conf_file}, expected_errorcode);
   }
 
-  void set_mock_metadata(const std::vector<Auth_data> &auth_data_collection,
-                         const uint16_t http_port, const std::string &gr_id,
-                         const uint16_t cluster_node_port,
-                         const bool error_on_md_query = false,
-                         const uint64_t view_id = 0,
-                         const mysqlrouter::MetadataSchemaVersion md_version = {
-                             2, 0, 3}) const {
+  void set_mock_metadata(
+      const std::vector<Auth_data> &auth_data_collection,
+      const uint16_t http_port, const std::string &gr_id,
+      const uint16_t cluster_node_port, const bool error_on_md_query = false,
+      const unsigned primary_id = 0, const unsigned view_id = 0,
+      const mysqlrouter::MetadataSchemaVersion md_version = {2, 0, 3}) const {
     auto json_doc = mock_GR_metadata_as_json(
-        gr_id, classic_ports_to_gr_nodes({cluster_node_port}), 0,
-        classic_ports_to_cluster_nodes({cluster_node_port}), view_id,
-        error_on_md_query);
+        gr_id, {cluster_node_port}, primary_id, view_id, error_on_md_query);
 
     JsonAllocator allocator;
     JsonValue nodes(rapidjson::kArrayType);
@@ -224,8 +222,7 @@ class MetadataHttpAuthTest : public RouterComponentTest {
     metadata_version_node.PushBack(md_version.major, allocator);
     metadata_version_node.PushBack(md_version.minor, allocator);
     metadata_version_node.PushBack(md_version.patch, allocator);
-    json_doc.AddMember("metadata_schema_version", metadata_version_node,
-                       allocator);
+    json_doc.AddMember("metadata_version", metadata_version_node, allocator);
 
     const auto json_str = json_to_string(json_doc);
 
@@ -270,6 +267,10 @@ class MetadataHttpAuthTest : public RouterComponentTest {
 
     cluster_node = &ProcessManager::launch_mysql_server_mock(
         trace_file, cluster_node_port, EXIT_SUCCESS, false, cluster_http_port);
+    ASSERT_NO_FATAL_FAILURE(check_port_ready(*cluster_node, cluster_node_port));
+    ASSERT_TRUE(
+        MockServerRestClient(cluster_http_port).wait_for_rest_endpoint_ready())
+        << cluster_node->get_full_output();
 
     router_port = port_pool_.get_next_available();
 
@@ -290,7 +291,7 @@ class MetadataHttpAuthTest : public RouterComponentTest {
   static const std::chrono::milliseconds kAuthCacheRefreshRate;
   static const std::string cluster_id;
   TempDirectory temp_test_dir;
-  uint64_t view_id = 1;
+  unsigned view_id = 1;
 
   ProcessWrapper *cluster_node;
   uint16_t cluster_node_port;
@@ -299,6 +300,8 @@ class MetadataHttpAuthTest : public RouterComponentTest {
   uint16_t router_port;
 
   std::string uri;
+
+  TcpPortPool port_pool_;
 };
 
 const std::chrono::milliseconds MetadataHttpAuthTest::kTTL = 200ms;
@@ -346,11 +349,12 @@ class BasicMetadataHttpAuthTest
 
 TEST_F(BasicMetadataHttpAuthTest, MetadataHttpAuthDefaultConfig) {
   set_mock_metadata({{kTestUser1, ""}}, cluster_http_port, cluster_id,
-                    cluster_node_port, false, view_id);
+                    cluster_node_port, false, 0, view_id);
 
   SCOPED_TRACE("// Launch the router with the initial state file");
-  launch_router(kMetadataCacheSectionBase);
+  ASSERT_NO_FATAL_FAILURE(launch_router(kMetadataCacheSectionBase));
 
+  wait_for_port_ready(router_port);
   ASSERT_TRUE(wait_for_rest_endpoint_ready(uri, http_server_port));
   EXPECT_GT(wait_for_rest_auth_query(2, cluster_http_port), 0);
 
@@ -366,11 +370,12 @@ TEST_F(BasicMetadataHttpAuthTest, MetadataHttpAuthDefaultConfig) {
 
 TEST_F(BasicMetadataHttpAuthTest, UnsupportedMetadataSchemaVersion) {
   set_mock_metadata({{kTestUser1, ""}}, cluster_http_port, cluster_id,
-                    cluster_node_port, false, view_id, {1, 0, 0});
+                    cluster_node_port, false, 0, view_id, {1, 0, 0});
 
   SCOPED_TRACE("// Launch the router with the initial state file");
   launch_router(kMetadataCacheSectionBase);
 
+  wait_for_port_ready(router_port);
   ASSERT_TRUE(wait_for_rest_endpoint_ready(uri, http_server_port));
 
   IOContext io_ctx;
@@ -387,13 +392,14 @@ TEST_P(BasicMetadataHttpAuthTest, BasicMetadataHttpAuth) {
   set_mock_metadata(
       {{GetParam().cached_info.credentials, GetParam().cached_info.privileges,
         GetParam().cached_info.auth_method}},
-      cluster_http_port, cluster_id, cluster_node_port, false, view_id);
+      cluster_http_port, cluster_id, cluster_node_port, false, 0, view_id);
 
   SCOPED_TRACE("// Launch the router with the initial state file");
   const std::string metadata_cache_section =
       get_metadata_cache_section(kTTL, kAuthCacheTTL, kAuthCacheRefreshRate);
   launch_router(metadata_cache_section);
 
+  wait_for_port_ready(router_port);
   ASSERT_TRUE(wait_for_rest_endpoint_ready(uri, http_server_port));
   EXPECT_GT(wait_for_rest_auth_query(2, cluster_http_port), 0);
 
@@ -407,7 +413,7 @@ TEST_P(BasicMetadataHttpAuthTest, BasicMetadataHttpAuth) {
                                        GetParam().http_response.type));
 }
 
-INSTANTIATE_TEST_SUITE_P(
+INSTANTIATE_TEST_CASE_P(
     BasicMetadataHttpAuth, BasicMetadataHttpAuthTest,
     ::testing::Values(
         // matching user and password
@@ -484,29 +490,21 @@ class FileAuthBackendWithMetadataAuthSettings : public MetadataHttpAuthTest {
 };
 
 TEST_F(FileAuthBackendWithMetadataAuthSettings, MixedBackendSettings) {
-  ProcessWrapper::OutputResponder responder{
-      [](const std::string &line) -> std::string {
-        if (line == "Please enter password: ")
-          return std::string(kRestApiPassword) + "\n";
-
-        return "";
-      }};
-
   auto &cmd = launch_command(
       ProcessManager::get_origin().join("mysqlrouter_passwd").str(),
-      {"set", passwd_file.str(), kRestApiUsername}, EXIT_SUCCESS, true, -1ms,
-      responder);
+      {"set", passwd_file.str(), kRestApiUsername}, EXIT_SUCCESS, true);
+  cmd.register_response("Please enter password", "password\n");
   EXPECT_EQ(cmd.wait_for_exit(), 0) << cmd.get_full_output();
 
   set_mock_metadata({}, cluster_http_port, cluster_id, cluster_node_port, false,
-                    view_id);
+                    0, view_id);
 
   const std::string metadata_cache_section =
       get_metadata_cache_section(kTTL, kAuthCacheTTL, kAuthCacheRefreshRate);
   // It should be possible to launch router with backend=file and with
   // additional metadata_cache auth settings
-  launch_router(metadata_cache_section);
-  ASSERT_TRUE(wait_for_port_ready(router_port));
+  ASSERT_NO_FATAL_FAILURE(launch_router(metadata_cache_section));
+  ASSERT_NO_FATAL_FAILURE(wait_for_port_ready(router_port));
 }
 
 class InvalidMetadataHttpAuthTimersTest
@@ -515,15 +513,16 @@ class InvalidMetadataHttpAuthTimersTest
 
 TEST_P(InvalidMetadataHttpAuthTimersTest, InvalidMetadataHttpAuthTimers) {
   set_mock_metadata({{kTestUser1, ""}}, cluster_http_port, cluster_id,
-                    cluster_node_port, false, view_id);
+                    cluster_node_port, false, 0, view_id);
 
   SCOPED_TRACE("// Launch the router with the initial state file");
   auto &router =
-      launch_router(kMetadataCacheSectionBase + GetParam(), EXIT_FAILURE, -1s);
+      launch_router(kMetadataCacheSectionBase + GetParam(), EXIT_FAILURE);
   check_exit_code(router, EXIT_FAILURE);
+  EXPECT_THAT(router.exit_code(), testing::Ne(0));
 }
 
-INSTANTIATE_TEST_SUITE_P(
+INSTANTIATE_TEST_CASE_P(
     InvalidMetadataHttpAuthTimers, InvalidMetadataHttpAuthTimersTest,
     ::testing::Values(
         std::string{"auth_cache_ttl=2.5\nauth_cache_refresh_interval=2.51\n"},
@@ -544,14 +543,15 @@ class ValidMetadataHttpAuthTimersTest
 
 TEST_P(ValidMetadataHttpAuthTimersTest, ValidMetadataHttpAuthTimers) {
   set_mock_metadata({{kTestUser1, ""}}, cluster_http_port, cluster_id,
-                    cluster_node_port, false, view_id);
+                    cluster_node_port, false, 0, view_id);
 
   SCOPED_TRACE("// Launch the router with the initial state file");
-  launch_router(kMetadataCacheSectionBase + "ttl=0.001\n" + GetParam());
-  ASSERT_TRUE(wait_for_port_ready(router_port));
+  ASSERT_NO_FATAL_FAILURE(
+      launch_router(kMetadataCacheSectionBase + "ttl=0.001\n" + GetParam()));
+  ASSERT_NO_FATAL_FAILURE(wait_for_port_ready(router_port));
 }
 
-INSTANTIATE_TEST_SUITE_P(
+INSTANTIATE_TEST_CASE_P(
     ValidMetadataHttpAuthTimers, ValidMetadataHttpAuthTimersTest,
     ::testing::Values(
         std::string{
@@ -568,12 +568,12 @@ class MetadataHttpAuthTestCustomTimers
 
 TEST_P(MetadataHttpAuthTestCustomTimers, MetadataHttpAuthCustomTimers) {
   set_mock_metadata({{kTestUser1, ""}}, cluster_http_port, cluster_id,
-                    cluster_node_port, false, view_id);
+                    cluster_node_port, false, 0, view_id);
 
   SCOPED_TRACE("// Launch the router with the initial state file");
   launch_router(kMetadataCacheSectionBase + GetParam());
 
-  ASSERT_TRUE(wait_for_port_ready(router_port));
+  wait_for_port_ready(router_port);
   ASSERT_TRUE(wait_for_rest_endpoint_ready(uri, http_server_port));
   EXPECT_GT(wait_for_rest_auth_query(2, cluster_http_port), 0);
 
@@ -587,7 +587,7 @@ TEST_P(MetadataHttpAuthTestCustomTimers, MetadataHttpAuthCustomTimers) {
                                        kContentTypeJson));
 }
 
-INSTANTIATE_TEST_SUITE_P(
+INSTANTIATE_TEST_CASE_P(
     MetadataHttpAuthCustomTimers, MetadataHttpAuthTestCustomTimers,
     ::testing::Values(
         std::string{"auth_cache_ttl=3600\nauth_cache_refresh_interval=2\n"},
@@ -598,7 +598,7 @@ INSTANTIATE_TEST_SUITE_P(
 
 TEST_F(MetadataHttpAuthTest, ExpiredAuthCacheTTL) {
   set_mock_metadata({{kTestUser1, ""}}, cluster_http_port, cluster_id,
-                    cluster_node_port, false, view_id);
+                    cluster_node_port, false, 0, view_id);
 
   std::chrono::milliseconds cache_ttl = kAuthCacheRefreshRate * 4;
   SCOPED_TRACE("// Launch the router with the initial state file");
@@ -621,7 +621,14 @@ TEST_F(MetadataHttpAuthTest, ExpiredAuthCacheTTL) {
   const bool fail_on_md_query = true;
   // Start to fail metadata cache updates
   set_mock_metadata({{kTestUser1, ""}}, cluster_http_port, cluster_id,
-                    cluster_node_port, fail_on_md_query, view_id);
+                    cluster_node_port, fail_on_md_query, 0, view_id);
+
+  // auth_cache_ttl is not yet expired
+  EXPECT_GT(wait_for_rest_auth_query(2, cluster_http_port), 0);
+
+  ASSERT_NO_FATAL_FAILURE(request_json(rest_client, uri, HttpMethod::Get,
+                                       HttpStatusCode::Ok, json_doc,
+                                       kContentTypeJson));
 
   // wait long enough for the auth cache to expire
   std::this_thread::sleep_for(cache_ttl);
@@ -644,7 +651,7 @@ class MetadataAuthCacheUpdate
 
 TEST_P(MetadataAuthCacheUpdate, AuthCacheUpdate) {
   set_mock_metadata(GetParam().first_auth_cache_data_set, cluster_http_port,
-                    cluster_id, cluster_node_port, false, view_id);
+                    cluster_id, cluster_node_port, false, 0, view_id);
 
   SCOPED_TRACE("// Launch the router with the initial state file");
   const std::string metadata_cache_section =
@@ -665,7 +672,7 @@ TEST_P(MetadataAuthCacheUpdate, AuthCacheUpdate) {
 
   // Update authentication metadata
   set_mock_metadata(GetParam().second_auth_cache_data_set, cluster_http_port,
-                    cluster_id, cluster_node_port, false, view_id);
+                    cluster_id, cluster_node_port, false, 0, view_id);
 
   // auth_cache is updated
   EXPECT_GT(wait_for_rest_auth_query(2, cluster_http_port), 0);
@@ -675,7 +682,7 @@ TEST_P(MetadataAuthCacheUpdate, AuthCacheUpdate) {
       json_doc, GetParam().second_http_response.type));
 }
 
-INSTANTIATE_TEST_SUITE_P(
+INSTANTIATE_TEST_CASE_P(
     AuthCacheUpdate, MetadataAuthCacheUpdate,
     ::testing::Values(
         // add user
