@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2019, 2023, Oracle and/or its affiliates.
+  Copyright (c) 2019, 2020, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -22,27 +22,22 @@
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-#include <chrono>
-#include <cstdlib>  // setenv
-#include <fstream>
-#include <memory>  // unique_ptr
-#include <string>
-#include <thread>
-
-#include <gmock/gmock.h>
-#include <gtest/gtest.h>
-
-#include "mysql/harness/filesystem.h"
-#include "process_wrapper.h"
+#include "gmock/gmock.h"
 #include "router_component_test.h"
+
+#include "mysqlrouter/utils.h"
+#include "process_wrapper.h"
 #include "tcp_port_pool.h"
 
+#include <chrono>
+#include <thread>
+
 using namespace std::chrono_literals;
 
 #define MY_WAIT_US_NOASSERT(cond, usec) \
   {                                     \
     int n = 0;                          \
-    int max = (usec) / 1000;            \
+    int max = usec / 1000;              \
     do {                                \
       std::this_thread::sleep_for(1ms); \
       n++;                              \
@@ -80,7 +75,7 @@ class RouterPidfileTest : public RouterComponentTest {
     params.at("logging_folder") = logging_folder.name();
     params.at("runtime_folder") = runtime_folder.name();
 
-    // define the additional keepalive section
+    // define the additonal keepalive section
     keepalive = "[keepalive]\ninterval = 10\n";
 
     // load keepalive with 10sec duration
@@ -99,6 +94,9 @@ class RouterPidfileTest : public RouterComponentTest {
 
   void start_router() {
     router = &ProcessManager::launch_router(router_cmdline);
+    // make sure to get past the setup of the signal handler, otherwise
+    // ProcessManager will complain about the "signal 15"
+    wait_log_contains(*router, "Starting all plugins", 5s);
   }
 
   void stop_router() {
@@ -206,7 +204,7 @@ TEST_F(RouterPidfileTest, PidFileRemovedAtExit) {
   ASSERT_TRUE(mysql_harness::Path(fullpath).exists());
 
   // verify clean shutdown exitcode
-  EXPECT_NO_FATAL_FAILURE(stop_router());
+  stop_router();
 
   // check pidfile removed
   ASSERT_FALSE(mysql_harness::Path(fullpath).exists());
@@ -230,18 +228,17 @@ class RouterPidfileOptionTest : public RouterPidfileTest {};
  *      TS_FR04_01
  */
 TEST_F(RouterPidfileOptionTest, PidFileNone) {
-  SCOPED_TRACE("// start router");
-  EXPECT_NO_FATAL_FAILURE(start_router());
+  start_router();
 
-  SCOPED_TRACE("// check default pidfile does NOT exist");
+  // check default pidfile does NOT exist
   mysql_harness::Path fullpath =
       mysql_harness::Path(runtime_folder.name()).join(pidfile.c_str());
   ASSERT_FALSE(fullpath.exists());
 
-  SCOPED_TRACE("// verify clean shutdown exitcode");
-  EXPECT_NO_FATAL_FAILURE(stop_router());
+  // verify clean shutdown exitcode
+  stop_router();
 
-  SCOPED_TRACE("// expect NO PID output");
+  // expect NO PID output
   EXPECT_FALSE(router->expect_output("PID .* written to '.*'", true));
 }
 
@@ -254,10 +251,9 @@ TEST_F(RouterPidfileOptionTest, PidFileOptionTwiceWithoutValue) {
   router_cmdline.emplace_back("--pid-file");
   router_cmdline.emplace_back("--pid-file");
 
-  auto &router = ProcessManager::launch_router(router_cmdline, EXIT_FAILURE,
-                                               true, false, -1s);
+  auto &router = ProcessManager::launch_router(router_cmdline, EXIT_FAILURE);
 
-  EXPECT_NO_FATAL_FAILURE(check_exit_code(router, EXIT_FAILURE, 1s));
+  check_exit_code(router, EXIT_FAILURE, 1s);
 
   // expect error
   EXPECT_TRUE(router.expect_output(
@@ -274,9 +270,9 @@ TEST_F(RouterPidfileOptionTest, PidFileOptionTwice) {
   router_cmdline.emplace_back("--pid-file=" + pidfile_tmp.str());
   router_cmdline.emplace_back("--pid-file=" + pidfile.str());
 
-  auto &router = ProcessManager::launch_router(router_cmdline, EXIT_FAILURE,
-                                               true, false, -1s);
-  EXPECT_NO_FATAL_FAILURE(check_exit_code(router, EXIT_FAILURE, 1s));
+  auto &router = ProcessManager::launch_router(router_cmdline, EXIT_FAILURE);
+
+  check_exit_code(router, EXIT_FAILURE, 1s);
 
   // expect error
   EXPECT_TRUE(router.expect_output(
@@ -299,9 +295,9 @@ TEST_F(RouterPidfileOptionTest, PidFileOptionCfgTwice) {
                                  "mysqlrouter.conf", extra_params);
   router_cmdline = {"-c", conf_file};
 
-  auto &router = ProcessManager::launch_router(router_cmdline, EXIT_FAILURE,
-                                               true, false, -1s);
-  EXPECT_NO_FATAL_FAILURE(check_exit_code(router, EXIT_FAILURE, 1s));
+  auto &router = ProcessManager::launch_router(router_cmdline, EXIT_FAILURE);
+
+  check_exit_code(router, EXIT_FAILURE, 1s);
 
   // expect error
   EXPECT_TRUE(router.expect_output(
@@ -333,7 +329,7 @@ TEST_F(RouterPidfileOptionTest, PidFileOptionEnvWhitespace) {
   ASSERT_TRUE(mysql_harness::Path(fullpath).exists());
 
   // verify clean shutdown exitcode
-  EXPECT_NO_FATAL_FAILURE(stop_router());
+  stop_router();
 
   // unset ROUTER_PID env
   UnsetEnvRouterPid();
@@ -344,14 +340,14 @@ TEST_F(RouterPidfileOptionTest, PidFileOptionEnvWhitespace) {
 
 /**
  * @test
- *      --pid-file option on command line - successful cases
+ *      --pid-file option on command line - succcessful cases
  */
 struct PidFileOptionParams {
   std::string filename;
   bool tmpdir_prefix;
 
-  PidFileOptionParams(std::string filename_, bool tmpdir_prefix_)
-      : filename(std::move(filename_)), tmpdir_prefix(tmpdir_prefix_) {}
+  PidFileOptionParams(const std::string &filename_, bool tmpdir_prefix_)
+      : filename(filename_), tmpdir_prefix(tmpdir_prefix_) {}
 };
 
 class RouterPidfileOptionValueTest
@@ -391,13 +387,13 @@ TEST_P(RouterPidfileOptionValueTest, PidFileOptionValueTest) {
   ASSERT_TRUE(mysql_harness::Path(fullpath).exists());
 
   // verify clean shutdown exitcode
-  EXPECT_NO_FATAL_FAILURE(stop_router());
+  stop_router();
 
   // expect PID output
   EXPECT_TRUE(router->expect_output("PID .* written to '.*'", true));
 }
 
-INSTANTIATE_TEST_SUITE_P(
+INSTANTIATE_TEST_CASE_P(
     PidFileOptionValueTest, RouterPidfileOptionValueTest,
     ::testing::Values(
         // absolute path pidfile value : TS_FR01_01 /  TS_FR01_02 (O)
@@ -405,7 +401,7 @@ INSTANTIATE_TEST_SUITE_P(
         // shell does, so this should be identical to the quoted TS_FR01_02 (O)
         // case.
         PidFileOptionParams("mysqlrouter.pid", true),
-        // relative with subfolders and filename : TS_FR05_01 (O)
+        // realtive with subfolders and filename : TS_FR05_01 (O)
         PidFileOptionParams(
             mysql_harness::Path(FOO).join(BAR).join(PIDFILE).c_str(), false),
         // optional filename : TS_FR05_03
@@ -423,7 +419,7 @@ INSTANTIATE_TEST_SUITE_P(
                                 .join(PIDFILE)
                                 .c_str(),
                             false),
-        // twice relative to relative filename : TS_FR05_02
+        // twice realtive to relative filename : TS_FR05_02
         PidFileOptionParams(mysql_harness::Path(FOO)
                                 .join("..")
                                 .join(FOO)
@@ -434,11 +430,11 @@ INSTANTIATE_TEST_SUITE_P(
                             false)));
 
 #ifndef _WIN32
-INSTANTIATE_TEST_SUITE_P(PidFileOptionValueTestUnix,
-                         RouterPidfileOptionValueTest,
-                         ::testing::Values(
-                             // whitespace : TS_FR01_03 (O)
-                             PidFileOptionParams(" ", false)));
+INSTANTIATE_TEST_CASE_P(PidFileOptionValueTestUnix,
+                        RouterPidfileOptionValueTest,
+                        ::testing::Values(
+                            // whitespace : TS_FR01_03 (O)
+                            PidFileOptionParams(" ", false)));
 #endif
 
 /**
@@ -449,8 +445,9 @@ struct PidFileOptionErrorParams {
   std::string filename;
   std::string pattern;
 
-  PidFileOptionErrorParams(std::string filename_, std::string pattern_)
-      : filename(std::move(filename_)), pattern(std::move(pattern_)) {}
+  PidFileOptionErrorParams(const std::string &filename_,
+                           const std::string &pattern_)
+      : filename(filename_), pattern(pattern_) {}
 };
 
 class RouterPidfileOptionValueTestError
@@ -463,15 +460,15 @@ TEST_P(RouterPidfileOptionValueTestError, PidFileOptionValueTestError) {
   // start router with parameterized value for --pid-file, and expect error
   router_cmdline.emplace_back("--pid-file=" + test_params.filename);
 
-  auto &router = ProcessManager::launch_router(router_cmdline, EXIT_FAILURE,
-                                               true, false, -1s);
-  EXPECT_NO_FATAL_FAILURE(check_exit_code(router, EXIT_FAILURE, 1s));
+  auto &router = ProcessManager::launch_router(router_cmdline, EXIT_FAILURE);
+
+  check_exit_code(router, EXIT_FAILURE, 1s);
 
   // expect error
   EXPECT_TRUE(router.expect_output(test_params.pattern, true));
 }
 
-INSTANTIATE_TEST_SUITE_P(
+INSTANTIATE_TEST_CASE_P(
     PidFileOptionValueTestError, RouterPidfileOptionValueTestError,
     ::testing::Values(
         // empty value : TS_FR10_01 / TS_FR10_02
@@ -483,16 +480,16 @@ INSTANTIATE_TEST_SUITE_P(
         // readonly dir : TS_FR11_01 (M)
         PidFileOptionErrorParams(
             mysql_harness::Path(FOO).join(READONLY_FOLDER).c_str(),
-            "^Error: Failed writing PID to .*/foo/readonly':.*"),
+            "^Error: Failed writing PID to .*/foo/readonly:.*"),
         // readonly file : TS_FR11_02 (M)
         PidFileOptionErrorParams(
             mysql_harness::Path(FOO).join(READONLY_FILE).c_str(),
-            "^Error: Failed writing PID to .*/foo/readonly.pid':.*"),
+            "^Error: Failed writing PID to .*/foo/readonly.pid:.*"),
         // nonexisting dir : TS_FR11_03 (M)
         PidFileOptionErrorParams(
             mysql_harness::Path(FOO).join(NONEXISTING).join(PIDFILE).c_str(),
             "^Error: Failed writing PID to "
-            ".*/foo/nonexisting/mysqlrouter.pid':.*")));
+            ".*/foo/nonexisting/mysqlrouter.pid:.*")));
 
 /**
  * @test
@@ -501,8 +498,7 @@ INSTANTIATE_TEST_SUITE_P(
 struct PidFileOptionCfgParams {
   std::string filename;
 
-  PidFileOptionCfgParams(std::string filename_)
-      : filename(std::move(filename_)) {}
+  PidFileOptionCfgParams(const std::string &filename_) : filename(filename_) {}
 };
 
 class RouterPidfileOptionCfgValueTest
@@ -538,22 +534,22 @@ TEST_P(RouterPidfileOptionCfgValueTest, PidFileOptionCfgValueTest) {
   ASSERT_TRUE(mysql_harness::Path(fullpath).exists());
 
   // verify clean shutdown exitcode
-  EXPECT_NO_FATAL_FAILURE(stop_router());
+  stop_router();
 
   // expect PID output
   EXPECT_TRUE(router->expect_output("PID .* written to '.*'", true));
 }
 
-INSTANTIATE_TEST_SUITE_P(
-    PidFileOptionCfgValueTest, RouterPidfileOptionCfgValueTest,
-    ::testing::Values(
-        // path with whitespace : TS_FR01_06
-        PidFileOptionCfgParams(mysql_harness::Path(FOO)
-                                   .join(WHITESPACE_FOLDER)
-                                   .join(PIDFILE_WHITESPACE)
-                                   .c_str()),
-        // non-empty filename of 2 quotes : TS_FR01_05
-        PidFileOptionCfgParams("''")));
+INSTANTIATE_TEST_CASE_P(PidFileOptionCfgValueTest,
+                        RouterPidfileOptionCfgValueTest,
+                        ::testing::Values(
+                            // path with whitespace : TS_FR01_06
+                            PidFileOptionCfgParams(mysql_harness::Path(FOO)
+                                                       .join(WHITESPACE_FOLDER)
+                                                       .join(PIDFILE_WHITESPACE)
+                                                       .c_str()),
+                            // non-empty filename of 2 quotes : TS_FR01_05
+                            PidFileOptionCfgParams("''")));
 
 /**
  * @test
@@ -562,8 +558,8 @@ INSTANTIATE_TEST_SUITE_P(
 struct PidFileOptionCfgErrorParams {
   std::string filename;
 
-  PidFileOptionCfgErrorParams(std::string filename_)
-      : filename(std::move(filename_)) {}
+  PidFileOptionCfgErrorParams(const std::string &filename_)
+      : filename(filename_) {}
 };
 
 class RouterPidfileOptionCfgValueTestError
@@ -581,20 +577,20 @@ TEST_P(RouterPidfileOptionCfgValueTestError, PidFileOptionCfgValueTestError) {
   router_cmdline = {"-c", conf_file};
 
   // start router with config file, and expect error
-  auto &router = ProcessManager::launch_router(router_cmdline, EXIT_FAILURE,
-                                               true, false, -1s);
-  EXPECT_NO_FATAL_FAILURE(check_exit_code(router, EXIT_FAILURE, 1s));
+  auto &router = ProcessManager::launch_router(router_cmdline, EXIT_FAILURE);
+
+  check_exit_code(router, EXIT_FAILURE, 1s);
 
   // expect error
   EXPECT_TRUE(
       router.expect_output("^Error: PID filename '.*' is illegal", true));
 }
 
-INSTANTIATE_TEST_SUITE_P(PidFileOptionCfgValueTestError,
-                         RouterPidfileOptionCfgValueTestError,
-                         ::testing::Values(
-                             // empty value : TS_FR08_01
-                             PidFileOptionCfgErrorParams("")));
+INSTANTIATE_TEST_CASE_P(PidFileOptionCfgValueTestError,
+                        RouterPidfileOptionCfgValueTestError,
+                        ::testing::Values(
+                            // empty value : TS_FR08_01
+                            PidFileOptionCfgErrorParams("")));
 
 #ifndef _WIN32
 /**
@@ -607,8 +603,8 @@ INSTANTIATE_TEST_SUITE_P(PidFileOptionCfgValueTestError,
 struct PidFileOptionEnvErrorParams {
   std::string filename;
 
-  PidFileOptionEnvErrorParams(std::string filename_)
-      : filename(std::move(filename_)) {}
+  PidFileOptionEnvErrorParams(const std::string &filename_)
+      : filename(filename_) {}
 };
 
 class RouterPidfileOptionEnvValueTestError
@@ -622,9 +618,9 @@ TEST_P(RouterPidfileOptionEnvValueTestError, PidFileOptionEnvValueTestError) {
   SetEnvRouterPid(test_params.filename.c_str());
 
   // start router with default config file, and expect error
-  auto &router = ProcessManager::launch_router(router_cmdline, EXIT_FAILURE,
-                                               true, false, -1s);
-  EXPECT_NO_FATAL_FAILURE(check_exit_code(router, EXIT_FAILURE, 1s));
+  auto &router = ProcessManager::launch_router(router_cmdline, EXIT_FAILURE);
+
+  check_exit_code(router, EXIT_FAILURE, 1s);
 
   // unset ROUTER_PID env
   UnsetEnvRouterPid();
@@ -634,11 +630,11 @@ TEST_P(RouterPidfileOptionEnvValueTestError, PidFileOptionEnvValueTestError) {
       router.expect_output("^Error: PID filename '.*' is illegal", true));
 }
 
-INSTANTIATE_TEST_SUITE_P(PidFileOptionEnvValueTestError,
-                         RouterPidfileOptionEnvValueTestError,
-                         ::testing::Values(
-                             // empty value : TS_FR09_01 (M), TS_FR09_02 (M)
-                             PidFileOptionEnvErrorParams("")));
+INSTANTIATE_TEST_CASE_P(PidFileOptionEnvValueTestError,
+                        RouterPidfileOptionEnvValueTestError,
+                        ::testing::Values(
+                            // empty value : TS_FR09_01 (M), TS_FR09_02 (M)
+                            PidFileOptionEnvErrorParams("")));
 #endif
 
 /**
@@ -674,29 +670,23 @@ TEST_P(RouterPidfileOptionSupremacyTest, PidFileOptionSupremacyTest) {
   auto test_params = GetParam();
 
   // setup the 3 possible pidfile names and resulting full path
-  const std::array<std::string, 3> pidfile_names = {"opt.pid", "cfg.pid",
-                                                    "env.pid"};
-
-  std::array<mysql_harness::Path, pidfile_names.size()> rtpf;
-  {
-    const auto runtime_folder_name = runtime_folder.name();
-    auto it = rtpf.begin();
-    for (const auto &pidfile_name : pidfile_names) {
-      *it++ = mysql_harness::Path(runtime_folder_name).join(pidfile_name);
-    }
+  std::string pidfile[3] = {"opt.pid", "cfg.pid", "env.pid"};
+  mysql_harness::Path rtpf[3];
+  for (int i = 0; i < 3; i++) {
+    rtpf[i] = mysql_harness::Path(runtime_folder.name()).join(pidfile[i]);
   }
 
   // setup according to test parameters
   if (test_params.used & OPT) {
-    router_cmdline.emplace_back("--pid-file=" + pidfile_names[0]);
+    router_cmdline.emplace_back("--pid-file=" + pidfile[0]);
   }
   if (test_params.used & CFG) {
-    params["pid_file"] = pidfile_names[1];
+    params["pid_file"] = pidfile[1];
     // load keepalive with 10sec duration
     conf_file = create_config_file(conf_folder.name(), keepalive, &params);
   }
   if (test_params.used & ENV) {
-    SetEnvRouterPid(pidfile_names[2].c_str());
+    SetEnvRouterPid(pidfile[2].c_str());
   }
 
   // start router with given parameters
@@ -716,7 +706,7 @@ TEST_P(RouterPidfileOptionSupremacyTest, PidFileOptionSupremacyTest) {
   }
 
   // verify clean shutdown exitcode
-  EXPECT_NO_FATAL_FAILURE(stop_router());
+  stop_router();
 
   if (test_params.used & ENV) {
     // unset ROUTER_PID env
@@ -727,17 +717,17 @@ TEST_P(RouterPidfileOptionSupremacyTest, PidFileOptionSupremacyTest) {
   EXPECT_TRUE(router->expect_output("PID .* written to '.*'", true));
 }
 
-INSTANTIATE_TEST_SUITE_P(PidFileOptionSupremacyTest,
-                         RouterPidfileOptionSupremacyTest,
-                         ::testing::Values(
-                             // --pid-file > pid_file > ROUTER_PID : TS_FR02_01
-                             PidFileOptionSupremacyParams(OPT | CFG | ENV),
-                             // --pid-file > pid_file : TS_FR02_02
-                             PidFileOptionSupremacyParams(OPT | CFG),
-                             // --pid-file > ROUTER_PID : TS_FR02_03
-                             PidFileOptionSupremacyParams(OPT | ENV),
-                             // pid_file > ROUTER_PID : TS_FR03_01
-                             PidFileOptionSupremacyParams(CFG | ENV)));
+INSTANTIATE_TEST_CASE_P(PidFileOptionSupremacyTest,
+                        RouterPidfileOptionSupremacyTest,
+                        ::testing::Values(
+                            // --pid-file > pid_file > ROUTER_PID : TS_FR02_01
+                            PidFileOptionSupremacyParams(OPT | CFG | ENV),
+                            // --pid-file > pid_file : TS_FR02_02
+                            PidFileOptionSupremacyParams(OPT | CFG),
+                            // --pid-file > ROUTER_PID : TS_FR02_03
+                            PidFileOptionSupremacyParams(OPT | ENV),
+                            // pid_file > ROUTER_PID : TS_FR03_01
+                            PidFileOptionSupremacyParams(CFG | ENV)));
 
 /**
  * @test
@@ -747,9 +737,9 @@ struct PidFileOptionSupremacyCornerCaseParams {
   std::string extra_params;
   std::string pattern;
 
-  PidFileOptionSupremacyCornerCaseParams(std::string extra_params_,
-                                         std::string pattern_)
-      : extra_params(std::move(extra_params_)), pattern(std::move(pattern_)) {}
+  PidFileOptionSupremacyCornerCaseParams(const std::string &extra_params_,
+                                         const std::string &pattern_)
+      : extra_params(extra_params_), pattern(pattern_) {}
 };
 
 class RouterPidfileOptionSupremacyCornerCaseTest
@@ -770,21 +760,21 @@ TEST_P(RouterPidfileOptionSupremacyCornerCaseTest,
 
   router_cmdline = {"-c", conf_file};
 
-  auto &router = ProcessManager::launch_router(router_cmdline, EXIT_FAILURE,
-                                               true, false, -1s);
-  EXPECT_NO_FATAL_FAILURE(check_exit_code(router, EXIT_FAILURE, 1s));
+  auto &router = ProcessManager::launch_router(router_cmdline, EXIT_FAILURE);
+
+  check_exit_code(router, EXIT_FAILURE, 1s);
 
   // expect error
   EXPECT_TRUE(router.expect_output(test_params.pattern, true));
 }
 
-INSTANTIATE_TEST_SUITE_P(PidFileOptionSupremacyCornerCaseTest,
-                         RouterPidfileOptionSupremacyCornerCaseTest,
-                         ::testing::Values(
-                             // empty value : TS_FR03_02
-                             PidFileOptionSupremacyCornerCaseParams(
-                                 "pid_file = ",
-                                 "^Error: PID filename '.*' is illegal.")));
+INSTANTIATE_TEST_CASE_P(PidFileOptionSupremacyCornerCaseTest,
+                        RouterPidfileOptionSupremacyCornerCaseTest,
+                        ::testing::Values(
+                            // empty value : TS_FR03_02
+                            PidFileOptionSupremacyCornerCaseParams(
+                                "pid_file = ",
+                                "^Error: PID filename '.*' is illegal.")));
 
 /**
  * @test
@@ -807,13 +797,10 @@ TEST_P(RouterPidfileOptionExistsTest, PidFileOptionExistsTest) {
   // Create an already existing pidfile
   mysql_harness::Path fullpath =
       mysql_harness::Path(runtime_folder.name()).join(pidfile.c_str());
-  std::ofstream alreadyexists(fullpath.str());
+  std::ofstream alreadyexists(fullpath.c_str());
   alreadyexists << "PidFileOptionExistsTest already existing file" << std::endl;
   alreadyexists.close();
 
-  // pid-file still exists
-  ASSERT_TRUE(fullpath.exists()) << fullpath.str();
-
   if (test_params.used & ENV) {
     // set ROUTER_PID and and expect error
     SetEnvRouterPid(pidfile.c_str());
@@ -830,9 +817,12 @@ TEST_P(RouterPidfileOptionExistsTest, PidFileOptionExistsTest) {
     router_cmdline.emplace_back("--pid-file=" + pidfile.str());
   }
 
-  auto &router = ProcessManager::launch_router(router_cmdline, EXIT_FAILURE,
-                                               true, false, -1s);
-  EXPECT_NO_FATAL_FAILURE(check_exit_code(router, EXIT_FAILURE, 1s));
+  auto &router = ProcessManager::launch_router(router_cmdline, EXIT_FAILURE);
+
+  check_exit_code(router, EXIT_FAILURE, 1s);
+
+  // Remove the already existing pidfile
+  remove(fullpath.c_str());
 
   if (test_params.used & ENV) {
     // unset ROUTER_PID env
@@ -842,19 +832,16 @@ TEST_P(RouterPidfileOptionExistsTest, PidFileOptionExistsTest) {
   // expect error
   EXPECT_TRUE(router.expect_output(
       "^Error: PID file .* found. Already running?", true));
-
-  // pid-file still exists
-  EXPECT_TRUE(fullpath.exists()) << fullpath.str();
 }
 
-INSTANTIATE_TEST_SUITE_P(PidFileOptionExistsTest, RouterPidfileOptionExistsTest,
-                         ::testing::Values(
-                             // Start when --pid-file file exists : TS_FR12_01
-                             PidFileExistsParams(OPT),
-                             // Start when pid_file file exists : TS_FR12_02
-                             PidFileExistsParams(CFG),
-                             // Start when ROUTER_PID file exists : TS_FR12_03
-                             PidFileExistsParams(ENV)));
+INSTANTIATE_TEST_CASE_P(PidFileOptionExistsTest, RouterPidfileOptionExistsTest,
+                        ::testing::Values(
+                            // Start when --pid-file file exists : TS_FR12_01
+                            PidFileExistsParams(OPT),
+                            // Start when pid_file file exists : TS_FR12_02
+                            PidFileExistsParams(CFG),
+                            // Start when ROUTER_PID file exists : TS_FR12_03
+                            PidFileExistsParams(ENV)));
 
 int main(int argc, char *argv[]) {
   init_windows_sockets();
