@@ -1,17 +1,16 @@
 /*
-  Copyright (c) 2017, 2024, Oracle and/or its affiliates.
+  Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,22 +25,23 @@
 #include <thread>
 
 #ifdef RAPIDJSON_NO_SIZETYPEDEFINE
+// if we build within the server, it will set RAPIDJSON_NO_SIZETYPEDEFINE
+// globally and require to include my_rapidjson_size_t.h
 #include "my_rapidjson_size_t.h"
 #endif
 
-#include <gmock/gmock.h>
 #include <rapidjson/document.h>
-
 #include "dim.h"
+#include "gmock/gmock.h"
 #include "mock_server_rest_client.h"
 #include "mysql/harness/logging/registry.h"
-#include "mysqlrouter/mysql_session.h"
-#include "mysqlrouter/rest_client.h"
+#include "mysql_session.h"
 #include "rest_api_testutils.h"
 #include "router_component_test.h"
-#include "router_component_testutils.h"
 #include "tcp_port_pool.h"
 
+#include "mysqlrouter/rest_client.h"
+
 Path g_origin_path;
 
 static constexpr const char kMockServerConnectionsRestUri[] =
@@ -56,7 +56,10 @@ using JsonDocument =
 using JsonValue =
     rapidjson::GenericValue<rapidjson::UTF8<>, rapidjson::CrtAllocator>;
 
-class RestMockServerTest : public RouterComponentTest {};
+class RestMockServerTest : public RouterComponentTest {
+ protected:
+  TcpPortPool port_pool_;
+};
 
 /**
  * base class.
@@ -134,15 +137,15 @@ TEST_F(RestMockServerRestServerMockTest, get_globals_empty) {
                    << std::to_string(http_port_)
                    << " failed (early): " << req.error_msg();
 
-  ASSERT_GT(req.get_response_code(), 0)
+  ASSERT_GT(req.get_response_code(), 0u)
       << "HTTP Request to " << http_hostname << ":"
       << std::to_string(http_port_) << " failed: " << req.error_msg();
 
-  EXPECT_EQ(req.get_response_code(), 200);
-  EXPECT_THAT(req.get_input_headers().find_cstr("Content-Type"),
+  EXPECT_EQ(req.get_response_code(), 200u);
+  EXPECT_THAT(req.get_input_headers().get("Content-Type"),
               ::testing::StrEq("application/json"));
 
-  auto &resp_body = req.get_input_buffer();
+  auto resp_body = req.get_input_buffer();
   EXPECT_GT(resp_body.length(), 0u);
   auto resp_body_content = resp_body.pop_front(resp_body.length());
 
@@ -219,13 +222,13 @@ TEST_F(RestMockServerRestServerMockTest, handshake_exec_time_via_global) {
                    << std::to_string(http_port_)
                    << " failed (early): " << req.error_msg();
 
-  ASSERT_GT(req.get_response_code(), 0)
+  ASSERT_GT(req.get_response_code(), 0u)
       << "HTTP Request to " << http_hostname << ":"
       << std::to_string(http_port_) << " failed: " << req.error_msg();
 
-  EXPECT_EQ(req.get_response_code(), 204);
+  EXPECT_EQ(req.get_response_code(), 204u);
 
-  auto &resp_body = req.get_input_buffer();
+  auto resp_body = req.get_input_buffer();
   EXPECT_EQ(resp_body.length(), 0u);
 
   SCOPED_TRACE("// slow connect");
@@ -278,16 +281,16 @@ TEST_F(RestMockServerRestServerMockTest, unknown_url_fails) {
                    << std::to_string(http_port_)
                    << " failed (early): " << req.error_msg() << std::endl;
 
-  ASSERT_GT(req.get_response_code(), 0)
+  ASSERT_GT(req.get_response_code(), 0u)
       << "HTTP Request to " << http_hostname << ":"
       << std::to_string(http_port_) << " failed: " << req.error_msg()
       << std::endl;
 
-  EXPECT_EQ(req.get_response_code(), 404);
-  EXPECT_THAT(req.get_input_headers().find_cstr("Content-Type"),
+  EXPECT_EQ(req.get_response_code(), 404u);
+  EXPECT_THAT(req.get_input_headers().get("Content-Type"),
               ::testing::StrEq("text/html"));
 
-  auto &resp_body = req.get_input_buffer();
+  auto resp_body = req.get_input_buffer();
   EXPECT_GT(resp_body.length(), 0u);
   auto resp_body_content = resp_body.pop_front(resp_body.length());
 }
@@ -316,13 +319,13 @@ TEST_F(RestMockServerRestServerMockTest, put_globals_no_json) {
                    << std::to_string(http_port_)
                    << " failed (early): " << req.error_msg() << std::endl;
 
-  ASSERT_GT(req.get_response_code(), 0)
+  ASSERT_GT(req.get_response_code(), 0u)
       << "HTTP Request to " << http_hostname << ":"
       << std::to_string(http_port_) << " failed: " << req.error_msg();
 
-  EXPECT_EQ(req.get_response_code(), 415);
+  EXPECT_EQ(req.get_response_code(), 415u);
 
-  auto &resp_body = req.get_input_buffer();
+  auto resp_body = req.get_input_buffer();
   EXPECT_EQ(resp_body.length(), 0u);
 }
 
@@ -352,13 +355,13 @@ TEST_F(RestMockServerRestServerMockTest, put_root_fails) {
                    << std::to_string(http_port_)
                    << " failed (early): " << req.error_msg();
 
-  ASSERT_GT(req.get_response_code(), 0)
+  ASSERT_GT(req.get_response_code(), 0u)
       << "HTTP Request to " << http_hostname << ":"
       << std::to_string(http_port_) << " failed: " << req.error_msg();
 
-  EXPECT_EQ(req.get_response_code(), 404);
+  EXPECT_EQ(req.get_response_code(), 404u);
 
-  auto &resp_body = req.get_input_buffer();
+  auto resp_body = req.get_input_buffer();
   EXPECT_NE(resp_body.length(), 0u);
 }
 
@@ -398,7 +401,7 @@ TEST_P(RestMockServerRequirePTest, require) {
   });
 }
 
-INSTANTIATE_TEST_SUITE_P(
+INSTANTIATE_TEST_CASE_P(
     js_require_paths, RestMockServerRequirePTest,
     ::testing::Values(std::make_tuple("direct", "direct"),
                       std::make_tuple("dir-with-indexjs", "dir-with-index.js"),
@@ -406,7 +409,7 @@ INSTANTIATE_TEST_SUITE_P(
                                       "dir-with-package.json")));
 
 /**
- * ensure require() only loads and evaluates modules once.
+ * ensure require() only loads and evalutes modules once.
  *
  * js_test_require.js requires the same module twice which exposes
  * a counter function.
@@ -479,7 +482,7 @@ TEST_F(RestMockServerNestingTest, nesting) {
 class RestMockServerMethodsTest
     : public RestMockServerRestServerMockTest,
       public ::testing::WithParamInterface<
-          std::tuple<unsigned int, std::string, int>> {};
+          std::tuple<unsigned int, std::string, unsigned int>> {};
 
 /**
  * ensure OPTIONS, HEAD and others work.
@@ -508,14 +511,14 @@ TEST_P(RestMockServerMethodsTest, methods_avail) {
                    << std::to_string(http_port_)
                    << " failed (early): " << req.error_msg();
 
-  ASSERT_GT(req.get_response_code(), 0)
+  ASSERT_GT(req.get_response_code(), 0u)
       << "HTTP Request to " << http_hostname << ":"
       << std::to_string(http_port_) << " failed: " << req.error_msg();
 
   EXPECT_EQ(std::get<2>(GetParam()), req.get_response_code());
 }
 
-INSTANTIATE_TEST_SUITE_P(
+INSTANTIATE_TEST_CASE_P(
     api__v1__mock_server__globals, RestMockServerMethodsTest,
     ::testing::Values(
         std::make_tuple(HttpMethod::Get, kMockServerGlobalsRestUri,
@@ -528,10 +531,12 @@ INSTANTIATE_TEST_SUITE_P(
                         HttpStatusCode::MethodNotAllowed),
         std::make_tuple(HttpMethod::Options, kMockServerGlobalsRestUri,
                         HttpStatusCode::MethodNotAllowed),
+        std::make_tuple(HttpMethod::Connect, kMockServerGlobalsRestUri,
+                        HttpStatusCode::MethodNotAllowed),
         std::make_tuple(HttpMethod::Head, kMockServerGlobalsRestUri,
                         HttpStatusCode::MethodNotAllowed)));
 
-INSTANTIATE_TEST_SUITE_P(
+INSTANTIATE_TEST_CASE_P(
     api__v1__mock_server__connections, RestMockServerMethodsTest,
     ::testing::Values(
         std::make_tuple(HttpMethod::Get, kMockServerConnectionsRestUri,
@@ -544,12 +549,10 @@ INSTANTIATE_TEST_SUITE_P(
                         HttpStatusCode::MethodNotAllowed),
         std::make_tuple(HttpMethod::Options, kMockServerConnectionsRestUri,
                         HttpStatusCode::MethodNotAllowed),
+        std::make_tuple(HttpMethod::Connect, kMockServerConnectionsRestUri,
+                        HttpStatusCode::MethodNotAllowed),
         std::make_tuple(HttpMethod::Head, kMockServerConnectionsRestUri,
-                        HttpStatusCode::MethodNotAllowed)),
-    [](const auto &info) {
-      return http_method_to_string(std::get<0>(info.param)) + "_" +
-             std::to_string(std::get<2>(info.param));
-    });
+                        HttpStatusCode::MethodNotAllowed)));
 
 /**
  * test storing globals in mock_server via REST bridge.
@@ -577,20 +580,20 @@ TEST_F(RestMockServerRestServerMockTest, put_globals_ok) {
                    << std::to_string(http_port_)
                    << " failed (early): " << req.error_msg();
 
-  ASSERT_GT(req.get_response_code(), 0)
+  ASSERT_GT(req.get_response_code(), 0u)
       << "HTTP Request to " << http_hostname << ":"
       << std::to_string(http_port_) << " failed: " << req.error_msg();
 
-  EXPECT_EQ(req.get_response_code(), 204);
+  EXPECT_EQ(req.get_response_code(), 204u);
 
-  auto &resp_body = req.get_input_buffer();
+  auto resp_body = req.get_input_buffer();
   EXPECT_EQ(resp_body.length(), 0u);
 }
 
 class RestMockServerRequestTest
     : public RestMockServerRestServerMockTest,
       public ::testing::WithParamInterface<
-          std::tuple<int, const char *, const char *, int>> {};
+          std::tuple<int, const char *, const char *, unsigned int>> {};
 
 /**
  * ensure valid and invalid JSON results in the correct behaviour.
@@ -622,7 +625,7 @@ TEST_P(RestMockServerRequestTest, request) {
                    << std::to_string(http_port_)
                    << " failed (early): " << req.error_msg();
 
-  ASSERT_GT(req.get_response_code(), 0)
+  ASSERT_GT(req.get_response_code(), 0u)
       << "HTTP Request to " << http_hostname << ":"
       << std::to_string(http_port_) << " failed: " << req.error_msg();
 
@@ -639,15 +642,15 @@ TEST_P(RestMockServerRequestTest, request) {
                          << std::to_string(http_port_)
                          << " failed (early): " << get_req.error_msg();
 
-    ASSERT_GT(get_req.get_response_code(), 0)
+    ASSERT_GT(get_req.get_response_code(), 0u)
         << "HTTP Request to " << http_hostname << ":"
         << std::to_string(http_port_) << " failed: " << get_req.error_msg();
 
-    EXPECT_EQ(get_req.get_response_code(), 200);
-    EXPECT_THAT(get_req.get_input_headers().find_cstr("Content-Type"),
+    EXPECT_EQ(get_req.get_response_code(), 200u);
+    EXPECT_THAT(get_req.get_input_headers().get("Content-Type"),
                 ::testing::StrEq("application/json"));
 
-    auto &get_resp_body = get_req.get_input_buffer();
+    auto get_resp_body = get_req.get_input_buffer();
     EXPECT_GT(get_resp_body.length(), 0u);
     auto get_resp_body_content =
         get_resp_body.pop_front(get_resp_body.length());
@@ -665,7 +668,7 @@ TEST_P(RestMockServerRequestTest, request) {
   }
 }
 
-INSTANTIATE_TEST_SUITE_P(
+INSTANTIATE_TEST_CASE_P(
     api__v1__mock_server__globals, RestMockServerRequestTest,
     ::testing::Values(
         // parse error
@@ -717,13 +720,13 @@ TEST_F(RestMockServerRestServerMockTest, put_globals_and_read_back) {
                        << std::to_string(http_port_)
                        << " failed (early): " << put_req.error_msg();
 
-  ASSERT_GT(put_req.get_response_code(), 0)
+  ASSERT_GT(put_req.get_response_code(), 0u)
       << "HTTP Request to " << http_hostname << ":"
       << std::to_string(http_port_) << " failed: " << put_req.error_msg();
 
-  EXPECT_EQ(put_req.get_response_code(), 204);
+  EXPECT_EQ(put_req.get_response_code(), 204u);
 
-  auto &put_resp_body = put_req.get_input_buffer();
+  auto put_resp_body = put_req.get_input_buffer();
   EXPECT_EQ(put_resp_body.length(), 0u);
 
   // GET request
@@ -734,15 +737,15 @@ TEST_F(RestMockServerRestServerMockTest, put_globals_and_read_back) {
                        << std::to_string(http_port_)
                        << " failed (early): " << get_req.error_msg();
 
-  ASSERT_GT(get_req.get_response_code(), 0)
+  ASSERT_GT(get_req.get_response_code(), 0u)
       << "HTTP Request to " << http_hostname << ":"
       << std::to_string(http_port_) << " failed: " << get_req.error_msg();
 
-  EXPECT_EQ(get_req.get_response_code(), 200);
-  EXPECT_THAT(get_req.get_input_headers().find_cstr("Content-Type"),
+  EXPECT_EQ(get_req.get_response_code(), 200u);
+  EXPECT_THAT(get_req.get_input_headers().get("Content-Type"),
               ::testing::StrEq("application/json"));
 
-  auto &get_resp_body = get_req.get_input_buffer();
+  auto get_resp_body = get_req.get_input_buffer();
   EXPECT_GT(get_resp_body.length(), 0u);
   auto get_resp_body_content = get_resp_body.pop_front(get_resp_body.length());
 
@@ -803,17 +806,19 @@ TEST_F(RestMockServerRestServerMockTest, delete_all_connections) {
                    << std::to_string(http_port_)
                    << " failed (early): " << req.error_msg();
 
-  ASSERT_GT(req.get_response_code(), 0)
+  ASSERT_GT(req.get_response_code(), 0u)
       << "HTTP Request to " << http_hostname << ":"
       << std::to_string(http_port_) << " failed: " << req.error_msg();
 
-  EXPECT_EQ(req.get_response_code(), 200);
+  EXPECT_EQ(req.get_response_code(), 200u);
 
-  auto &resp_body = req.get_input_buffer();
+  auto resp_body = req.get_input_buffer();
   EXPECT_EQ(resp_body.length(), 0u);
 
   SCOPED_TRACE("// check connection is killed");
-  wait_connection_dropped(client);
+  EXPECT_THROW_LIKE(client.query_one("select @@port"),
+                    mysqlrouter::MySQLSession::Error,
+                    "Lost connection to MySQL server during query");
 }
 
 TEST_F(RestMockServerRestServerMockTest, auth_succeeds_require_user_and_pass) {
@@ -843,13 +848,13 @@ TEST_F(RestMockServerRestServerMockTest, auth_succeeds_require_user_and_pass) {
                    << std::to_string(http_port_)
                    << " failed (early): " << req.error_msg();
 
-  ASSERT_GT(req.get_response_code(), 0)
+  ASSERT_GT(req.get_response_code(), 0u)
       << "HTTP Request to " << http_hostname << ":"
       << std::to_string(http_port_) << " failed: " << req.error_msg();
 
-  EXPECT_EQ(req.get_response_code(), 204);
+  EXPECT_EQ(req.get_response_code(), 204u);
 
-  auto &resp_body = req.get_input_buffer();
+  auto resp_body = req.get_input_buffer();
   EXPECT_EQ(resp_body.length(), 0u);
 
   // mysql query
@@ -884,13 +889,13 @@ TEST_F(RestMockServerRestServerMockTest, auth_succeeds_require_user) {
                    << std::to_string(http_port_)
                    << " failed (early): " << req.error_msg();
 
-  ASSERT_GT(req.get_response_code(), 0)
+  ASSERT_GT(req.get_response_code(), 0u)
       << "HTTP Request to " << http_hostname << ":"
       << std::to_string(http_port_) << " failed: " << req.error_msg();
 
-  EXPECT_EQ(req.get_response_code(), 204);
+  EXPECT_EQ(req.get_response_code(), 204u);
 
-  auto &resp_body = req.get_input_buffer();
+  auto resp_body = req.get_input_buffer();
   EXPECT_EQ(resp_body.length(), 0u);
 
   // mysql query
@@ -928,11 +933,11 @@ TEST_F(RestMockServerRestServerMockTest, auth_fails_wrong_password) {
                    << std::to_string(http_port_)
                    << " failed (early): " << req.error_msg();
 
-  ASSERT_GT(req.get_response_code(), 0)
+  ASSERT_GT(req.get_response_code(), 0u)
       << "HTTP Request to " << http_hostname << ":"
       << std::to_string(http_port_) << " failed: " << req.error_msg();
 
-  EXPECT_EQ(req.get_response_code(), 204);
+  EXPECT_EQ(req.get_response_code(), 204u);
 
   // mysql query
   mysqlrouter::MySQLSession client;
@@ -980,13 +985,13 @@ TEST_F(RestMockServerRestServerMockTest, auth_fails_empty_password) {
                    << std::to_string(http_port_)
                    << " failed (early): " << req.error_msg();
 
-  ASSERT_GT(req.get_response_code(), 0)
+  ASSERT_GT(req.get_response_code(), 0u)
       << "HTTP Request to " << http_hostname << ":"
       << std::to_string(http_port_) << " failed: " << req.error_msg();
 
-  EXPECT_EQ(req.get_response_code(), 204);
+  EXPECT_EQ(req.get_response_code(), 204u);
 
-  auto &resp_body = req.get_input_buffer();
+  auto resp_body = req.get_input_buffer();
   EXPECT_EQ(resp_body.length(), 0u);
 
   // mysql query
@@ -1033,14 +1038,14 @@ TEST_F(RestMockServerRestServerMockTest, auth_fails_wrong_username) {
                    << std::to_string(http_port_)
                    << " failed (early): " << req.error_msg();
 
-  ASSERT_GT(req.get_response_code(), 0)
+  ASSERT_GT(req.get_response_code(), 0u)
       << "HTTP Request to " << http_hostname << ":"
       << std::to_string(http_port_) << " failed: " << req.error_msg();
 
-  EXPECT_EQ(req.get_response_code(), 204);
+  EXPECT_EQ(req.get_response_code(), 204u);
 
-  auto &resp_body = req.get_input_buffer();
-  EXPECT_EQ(resp_body.length(), 0);
+  auto resp_body = req.get_input_buffer();
+  EXPECT_EQ(resp_body.length(), 0u);
 
   // mysql query
   mysqlrouter::MySQLSession client;
@@ -1117,7 +1122,7 @@ TEST_P(RestMockServerConnectThrowsTest, js_test_stmts_is_string) {
       mysqlrouter::MySQLSession::Error, std::get<1>(GetParam()));
 }
 
-INSTANTIATE_TEST_SUITE_P(
+INSTANTIATE_TEST_CASE_P(
     ScriptsFails, RestMockServerConnectThrowsTest,
     ::testing::Values(
         std::make_tuple("js_test_parse_error.js",
@@ -1170,7 +1175,7 @@ TEST_P(RestMockServerScriptsThrowsTest, scripts_throws) {
                     mysqlrouter::MySQLSession::Error, std::get<1>(GetParam()));
 }
 
-INSTANTIATE_TEST_SUITE_P(
+INSTANTIATE_TEST_CASE_P(
     ScriptsFails, RestMockServerScriptsThrowsTest,
     ::testing::Values(
         std::make_tuple("js_test_stmts_result_has_negative_int.js",
@@ -1181,7 +1186,8 @@ INSTANTIATE_TEST_SUITE_P(
         std::make_tuple("js_test_stmts_result_has_repeat.js",
                         "repeat is not supported"),  // WL11861 TS-1_5
         std::make_tuple("js_test_stmts_is_empty.js",
-                        "Unknown statement. (end of stmts)")),
+                        "executing statement failed: Unsupported command in "
+                        "handle_statement()")),
     [](const ::testing::TestParamInfo<std::tuple<const char *, const char *>>
            &info) -> std::string {
       return sanitize_param_name(std::get<0>(info.param));
@@ -1217,7 +1223,7 @@ TEST_P(RestMockServerScriptsWorkTest, scripts_work) {
   ASSERT_NO_THROW(client.execute("select @@port"));
 }
 
-INSTANTIATE_TEST_SUITE_P(
+INSTANTIATE_TEST_CASE_P(
     ScriptsWork, RestMockServerScriptsWorkTest,
     ::testing::Values("simple-client.js", "js_test_handshake_is_empty.js",
                       "js_test_handshake_greeting_is_empty.js",
