@@ -1,17 +1,16 @@
 /*
-  Copyright (c) 2019, 2024, Oracle and/or its affiliates.
+  Copyright (c) 2019, 2020, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,6 +28,8 @@
 #include <gmock/gmock.h>
 
 #ifdef RAPIDJSON_NO_SIZETYPEDEFINE
+// if we build within the server, it will set RAPIDJSON_NO_SIZETYPEDEFINE
+// globally and require to include my_rapidjson_size_t.h
 #include "my_rapidjson_size_t.h"
 #endif
 
@@ -40,8 +41,9 @@
 
 #include "config_builder.h"
 #include "dim.h"
+#include "mysql/harness/logging/registry.h"
 #include "mysql/harness/utility/string.h"  // ::join
-#include "mysqlrouter/mysql_session.h"
+#include "mysql_session.h"
 #include "process_launcher.h"
 #include "rest_api_testutils.h"
 #include "router_component_test.h"
@@ -73,13 +75,14 @@ TEST_P(RestRouterApiTest, ensure_openapi) {
 
   const std::string userfile = create_password_file();
   auto config_sections = get_restapi_config("rest_router", userfile, true);
+  // rest_api section is always needed
+  config_sections.push_back(ConfigBuilder::build_section("rest_api", {}));
 
   const std::string conf_file{create_config_file(
       conf_dir_.name(), mysql_harness::join(config_sections, "\n"))};
   ProcessWrapper &http_server{launch_router({"-c", conf_file})};
 
-  ASSERT_NO_FATAL_FAILURE(
-      fetch_and_validate_schema_and_resource(GetParam(), http_server));
+  fetch_and_validate_schema_and_resource(GetParam(), http_server);
 }
 
 // ****************************************************************************
@@ -151,7 +154,7 @@ static const RestApiTestParams rest_api_valid_methods[]{
      kRouterSwaggerPaths},
 };
 
-INSTANTIATE_TEST_SUITE_P(
+INSTANTIATE_TEST_CASE_P(
     ValidMethods, RestRouterApiTest,
     ::testing::ValuesIn(rest_api_valid_methods),
     [](const ::testing::TestParamInfo<RestApiTestParams> &info) {
@@ -177,7 +180,7 @@ static const RestApiTestParams rest_api_valid_methods_invalid_auth_params[]{
      kRouterSwaggerPaths},
 };
 
-INSTANTIATE_TEST_SUITE_P(
+INSTANTIATE_TEST_CASE_P(
     ValidMethodsInvalidAuth, RestRouterApiTest,
     ::testing::ValuesIn(rest_api_valid_methods_invalid_auth_params),
     [](const ::testing::TestParamInfo<RestApiTestParams> &info) {
@@ -192,15 +195,15 @@ static const RestApiTestParams rest_api_invalid_methods_params[]{
     {"router_status_invalid_methods",
      std::string(rest_api_basepath) + "/router/status", "/router/status",
      HttpMethod::Post | HttpMethod::Delete | HttpMethod::Patch |
-         HttpMethod::Head | HttpMethod::Trace | HttpMethod::Options,
+         HttpMethod::Head | HttpMethod::Trace | HttpMethod::Options |
+         HttpMethod::Connect,
      HttpStatusCode::MethodNotAllowed, kContentTypeJsonProblem,
      kRestApiUsername, kRestApiPassword,
      /*request_authentication =*/true,
-     RestApiComponentTest::get_json_method_not_allowed_verifiers(),
-     kRouterSwaggerPaths},
+     RestApiComponentTest::kProblemJsonMethodNotAllowed, kRouterSwaggerPaths},
 };
 
-INSTANTIATE_TEST_SUITE_P(
+INSTANTIATE_TEST_CASE_P(
     InvalidMethods, RestRouterApiTest,
     ::testing::ValuesIn(rest_api_invalid_methods_params),
     [](const ::testing::TestParamInfo<RestApiTestParams> &info) {
@@ -216,15 +219,23 @@ INSTANTIATE_TEST_SUITE_P(
  * [http_server] enabled but not the [rest_api] plugin.
  *
  */
-TEST_F(RestRouterApiTest, rest_api_section_missing_works) {
+TEST_F(RestRouterApiTest, rest_api_secion_missing) {
   const std::string userfile = create_password_file();
   auto config_sections = get_restapi_config("rest_router", userfile,
                                             /*request_authentication=*/true);
 
   const std::string conf_file{create_config_file(
       conf_dir_.name(), mysql_harness::join(config_sections, "\n"))};
+  auto &router = launch_router({"-c", conf_file}, EXIT_FAILURE);
 
-  launch_router({"-c", conf_file}, EXIT_SUCCESS);
+  const auto wait_for_process_exit_timeout{10000ms};
+  check_exit_code(router, EXIT_FAILURE, wait_for_process_exit_timeout);
+
+  const std::string router_output = router.get_full_output();
+  EXPECT_NE(router_output.find("Plugin 'rest_router' needs plugin 'rest_api' "
+                               "which is missing in the configuration"),
+            router_output.npos)
+      << router_output;
 }
 
 /**
@@ -237,16 +248,18 @@ TEST_F(RestRouterApiTest, rest_router_section_twice) {
   auto config_sections = get_restapi_config("rest_router", userfile,
                                             /*request_authentication=*/true);
 
+  // [rest_api] is always required
+  config_sections.push_back(ConfigBuilder::build_section("rest_api", {}));
+
   // force [rest_router] twice in the config
-  config_sections.push_back(
-      mysql_harness::ConfigBuilder::build_section("rest_router", {}));
+  config_sections.push_back(ConfigBuilder::build_section("rest_router", {}));
 
   const std::string conf_file{create_config_file(
       conf_dir_.name(), mysql_harness::join(config_sections, "\n"))};
-  auto &router =
-      launch_router({"-c", conf_file}, EXIT_FAILURE, true, false, -1s);
+  auto &router = launch_router({"-c", conf_file}, EXIT_FAILURE);
 
-  check_exit_code(router, EXIT_FAILURE, 10s);
+  const auto wait_for_process_exit_timeout{10000ms};
+  check_exit_code(router, EXIT_FAILURE, wait_for_process_exit_timeout);
 
   const std::string router_output = router.get_full_output();
   EXPECT_NE(router_output.find(
@@ -266,17 +279,22 @@ TEST_F(RestRouterApiTest, rest_router_section_has_key) {
   auto config_sections = get_restapi_config("rest_router:A", userfile,
                                             /*request_authentication=*/true);
 
+  // [rest_api] is always required
+  config_sections.push_back(ConfigBuilder::build_section("rest_api", {}));
+
   const std::string conf_file{create_config_file(
       conf_dir_.name(), mysql_harness::join(config_sections, "\n"))};
-  auto &router =
-      launch_router({"-c", conf_file}, EXIT_FAILURE, true, false, -1s);
+  auto &router = launch_router({"-c", conf_file}, EXIT_FAILURE);
 
-  check_exit_code(router, EXIT_FAILURE, 10s);
+  const auto wait_for_process_exit_timeout{10000ms};
+  check_exit_code(router, EXIT_FAILURE, wait_for_process_exit_timeout);
 
-  const std::string router_output = router.get_logfile_content();
-  EXPECT_THAT(router_output,
-              ::testing::HasSubstr("  init 'rest_router' failed: [rest_router] "
-                                   "section does not expect a key, found 'A'"));
+  const std::string router_output = router.get_full_logfile();
+  EXPECT_NE(
+      router_output.find("plugin 'rest_router' init failed: [rest_router] "
+                         "section does not expect a key, found 'A'"),
+      router_output.npos)
+      << router_output;
 }
 
 /**
@@ -289,17 +307,21 @@ TEST_F(RestRouterApiTest, router_api_no_auth) {
   auto config_sections = get_restapi_config("rest_router", userfile,
                                             /*request_authentication=*/false);
 
+  // [rest_api] is always required
+  config_sections.push_back(ConfigBuilder::build_section("rest_api", {}));
+
   const std::string conf_file{create_config_file(
       conf_dir_.name(), mysql_harness::join(config_sections, "\n"))};
-  auto &router =
-      launch_router({"-c", conf_file}, EXIT_FAILURE, true, false, -1s);
+  auto &router = launch_router({"-c", conf_file}, EXIT_FAILURE);
 
-  check_exit_code(router, EXIT_FAILURE, 10s);
+  const auto wait_for_process_exit_timeout{10000ms};
+  check_exit_code(router, EXIT_FAILURE, wait_for_process_exit_timeout);
 
-  const std::string router_output = router.get_logfile_content();
-  EXPECT_THAT(router_output, ::testing::HasSubstr(
-                                 "  init 'rest_router' failed: option "
-                                 "require_realm in [rest_router] is required"));
+  const std::string router_output = router.get_full_logfile();
+  EXPECT_NE(router_output.find("plugin 'rest_router' init failed: option "
+                               "require_realm in [rest_router] is required"),
+            router_output.npos)
+      << router_output;
 }
 
 /**
@@ -311,19 +333,22 @@ TEST_F(RestRouterApiTest, invalid_realm) {
   auto config_sections = get_restapi_config(
       "rest_router", userfile, /*request_authentication=*/true, "invalidrealm");
 
+  // [rest_api] is always required
+  config_sections.push_back(ConfigBuilder::build_section("rest_api", {}));
+
   const std::string conf_file{create_config_file(
       conf_dir_.name(), mysql_harness::join(config_sections, "\n"))};
-  auto &router =
-      launch_router({"-c", conf_file}, EXIT_FAILURE, true, false, -1s);
+  auto &router = launch_router({"-c", conf_file}, EXIT_FAILURE);
 
-  check_exit_code(router, EXIT_FAILURE, 10s);
+  const auto wait_for_process_exit_timeout{10000ms};
+  check_exit_code(router, EXIT_FAILURE, wait_for_process_exit_timeout);
 
-  const std::string router_output = router.get_logfile_content();
-  EXPECT_THAT(
-      router_output,
-      ::testing::HasSubstr(
-          "Configuration error: The option 'require_realm=invalidrealm' "
-          "in [rest_router] does not match any http_auth_realm."));
+  const std::string router_output = router.get_full_logfile();
+  EXPECT_NE(router_output.find(
+                "Configuration error: The option 'require_realm=invalidrealm' "
+                "in [rest_router] does not match any http_auth_realm."),
+            router_output.npos)
+      << router_output;
 }
 
 int main(int argc, char *argv[]) {
