@@ -1,17 +1,16 @@
 /*
-  Copyright (c) 2019, 2024, Oracle and/or its affiliates.
+  Copyright (c) 2019, 2020, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,9 +26,9 @@
 #include <thread>
 
 #include <gmock/gmock.h>
-#include <gtest/gtest.h>
-
 #ifdef RAPIDJSON_NO_SIZETYPEDEFINE
+// if we build within the server, it will set RAPIDJSON_NO_SIZETYPEDEFINE
+// globally and require to include my_rapidjson_size_t.h
 #include "my_rapidjson_size_t.h"
 #endif
 #include <rapidjson/document.h>
@@ -43,15 +42,12 @@
 #include "mock_server_testutils.h"
 #include "mysql/harness/logging/registry.h"
 #include "mysql/harness/utility/string.h"  // ::join
-#include "mysqlrouter/mysql_session.h"
+#include "mysql_session.h"
 #include "mysqlrouter/rest_client.h"
 #include "rest_api_testutils.h"
 #include "router_component_test.h"
-#include "router_component_testutils.h"  // make_bad_connection
-#include "tcp_address.h"
 #include "tcp_port_pool.h"
-#include "test/helpers.h"
-#include "test/temp_directory.h"
+#include "temp_dir.h"
 
 using namespace std::string_literals;
 
@@ -135,13 +131,14 @@ TEST_P(RestRoutingApiTest, ensure_openapi) {
 
   auto config_sections = get_restapi_config("rest_routing", userfile,
                                             GetParam().request_authentication);
+  config_sections.push_back(ConfigBuilder::build_section("rest_api", {}));
   size_t i = 0;
   for (const auto &route_name : route_names) {
     // let's make "_" route a metadata cache one, all other are static
     const std::string destinations =
         (route_name == "_") ? "metadata-cache://test/default?role=PRIMARY"
                             : "127.0.0.1:" + std::to_string(mock_port_);
-    config_sections.push_back(mysql_harness::ConfigBuilder::build_section(
+    config_sections.push_back(ConfigBuilder::build_section(
         std::string("routing") + (route_name.empty() ? "" : ":") + route_name,
         {
             {"bind_port", std::to_string(routing_ports_[i])},
@@ -159,40 +156,36 @@ TEST_P(RestRoutingApiTest, ensure_openapi) {
   // create a "dead" metadata-cache referenced by the routing "_" to check
   // route/health isActive == 0
   const std::string keyring_username = "mysql_router1_user";
-  config_sections.push_back(mysql_harness::ConfigBuilder::build_section(
-      "metadata_cache:test", {
-                                 {"router_id", "1"},
-                                 {"user", keyring_username},
-                                 {"metadata_cluster", "test"},
-                                 //"ttl", "0.5"
-                             }));
+  config_sections.push_back(ConfigBuilder::build_section(
+      "metadata_cache:test",
+      {
+          {"router_id", "3"},
+          {"user", keyring_username},
+          {"metadata_cluster", "test"},
+          {"bootstrap_server_addresses", "mysql://does-not-exist"},
+          //"ttl", "0.5"
+      }));
 
   std::map<std::string, std::string> default_section = get_DEFAULT_defaults();
   init_keyring(default_section, conf_dir_.name());
 
-  // 198.51.100.0/24 is a reserved address block, it could not be
-  // connected to. https://tools.ietf.org/html/rfc5737#section-4
-  const auto state_file = create_state_file(
-      get_test_temp_dir_name(),
-      create_state_file_content(
-          {mysql_harness::TCPAddress("198.51.100.1", 3060)}, "uuid", "", 0));
-  default_section["dynamic_state"] = state_file;
-
   const std::string conf_file{create_config_file(
-      conf_dir_.name(), mysql_harness::join(config_sections, ""),
-      &default_section, "mysqlrouter.conf", "connect_timeout=1")};
+      conf_dir_.name(), mysql_harness::join(config_sections, "\n"),
+      &default_section)};
 
   SCOPED_TRACE("// starting router");
-  ProcessWrapper &http_server =
-      launch_router({"-c", conf_file}, EXIT_SUCCESS, true, false, -1s);
+  ProcessWrapper &http_server = launch_router({"-c", conf_file});
 
   // doesn't really matter which file we use here, we are not going to do any
   // queries
   const std::string json_stmts = get_data_dir().join("bootstrap_gr.js").str();
 
   SCOPED_TRACE("// launch the server mock");
-  launch_mysql_server_mock(json_stmts, mock_port_, EXIT_SUCCESS, false);
+  auto &server_mock =
+      launch_mysql_server_mock(json_stmts, mock_port_, EXIT_SUCCESS, false);
 
+  SCOPED_TRACE("// checking port is ready");
+  ASSERT_NO_FATAL_FAILURE(check_port_ready(server_mock, mock_port_, 5000ms));
   // wait for route being available if we expect it to be and plan to do some
   // connections to it (which are routes: "ro" and "Aaz")
   for (size_t i = 3; i < kRoutesQty; ++i) {
@@ -220,39 +213,24 @@ TEST_P(RestRoutingApiTest, ensure_openapi) {
   // call wait_port_ready a few times on "123" to trigger blocked client
   // on that route (we set max_connect_errors to 2)
   for (size_t i = 0; i < 3; ++i) {
-    ASSERT_NO_FATAL_FAILURE(make_bad_connection(routing_ports_[2]));
+    ASSERT_TRUE(wait_for_port_ready(routing_ports_[2], 500ms));
   }
 
-  // wait until we see that the Router has blocked the host
-  EXPECT_TRUE(wait_log_contains(http_server, "blocking client host", 5s));
+  // wait a bit until the routing plugin really closed the sockets
+  //
+  // the routing plugin has a server-greeting-timeout of 100ms
+  // add a few more on top for our close-and-forget in wait_for_port_ready()
+  std::this_thread::sleep_for(200ms);
 
   EXPECT_NO_FATAL_FAILURE(
       fetch_and_validate_schema_and_resource(GetParam(), http_server));
 }
 
-static const RestApiComponentTest::json_verifiers_t get_expected_status_fields(
-    const int expected_max_total_connections,
-    const int expected_current_total_connections) {
-  return {
-      {"/maxTotalConnections",
-       [=](const JsonValue *value) {
-         ASSERT_TRUE(value != nullptr);
-         ASSERT_TRUE(value->IsInt());
-         ASSERT_EQ(value->GetInt(), expected_max_total_connections);
-       }},
-      {"/currentTotalConnections",
-       [=](const JsonValue *value) {
-         ASSERT_TRUE(value != nullptr);
-         ASSERT_TRUE(value->IsInt());
-         ASSERT_EQ(value->GetInt(), expected_current_total_connections);
-       }},
-  };
-}
-
-static const RestApiComponentTest::json_verifiers_t
-get_expected_routes_status_fields(const int expected_active_connections,
-                                  const int expected_total_connections,
-                                  const int expected_blocked_hosts) {
+static const std::vector<
+    std::pair<std::string, RestApiTestParams::value_check_func>>
+get_expected_status_fields(const int expected_active_connections,
+                           const int expected_total_connections,
+                           const int expected_blocked_hosts) {
   return {
       {"/activeConnections",
        [=](const JsonValue *value) {
@@ -275,7 +253,8 @@ get_expected_routes_status_fields(const int expected_active_connections,
   };
 }
 
-static const RestApiComponentTest::json_verifiers_t
+static const std::vector<
+    std::pair<std::string, RestApiTestParams::value_check_func>>
 get_expected_config_fields() {
   return {
       {"/bindAddress",
@@ -328,8 +307,9 @@ get_expected_config_fields() {
   };
 }
 
-static const RestApiComponentTest::json_verifiers_t get_expected_health_fields(
-    const bool expected_alive) {
+static const std::vector<
+    std::pair<std::string, RestApiTestParams::value_check_func>>
+get_expected_health_fields(const bool expected_alive) {
   return {
       {"/isAlive",
        [=](const JsonValue *value) {
@@ -339,98 +319,102 @@ static const RestApiComponentTest::json_verifiers_t get_expected_health_fields(
   };
 }
 
-static const RestApiComponentTest::json_verifiers_t
+static const std::vector<
+    std::pair<std::string, RestApiTestParams::value_check_func>>
 get_expected_destinations_fields(int expected_destinations_num) {
-  RestApiComponentTest::json_verifiers_t result{
-      {"/items",
-       [=](const JsonValue *value) {
-         ASSERT_NE(value, nullptr);
-         ASSERT_TRUE(value->IsArray());
-         ASSERT_EQ(value->GetArray().Size(), expected_destinations_num);
-       }},
-  };
+  std::vector<std::pair<std::string, RestApiTestParams::value_check_func>>
+      result{
+          {"/items",
+           [=](const JsonValue *value) {
+             ASSERT_NE(value, nullptr);
+             ASSERT_TRUE(value->IsArray());
+             ASSERT_EQ(value->GetArray().Size(), expected_destinations_num);
+           }},
+      };
 
   for (int i = 0; i < expected_destinations_num; ++i) {
-    result.emplace_back("/items/0/address", [](const JsonValue *value) {
-      ASSERT_TRUE(value != nullptr);
-      ASSERT_TRUE(value->IsString());
-      ASSERT_STREQ(value->GetString(), "127.0.0.1");
-    });
-    result.emplace_back("/items/0/port", [](const JsonValue *value) {
-      ASSERT_NE(value, nullptr);
-      ASSERT_GT(value->GetInt(), 0);
-    });
+    result.push_back({"/items/0/address", [](const JsonValue *value) {
+                        ASSERT_TRUE(value != nullptr);
+                        ASSERT_TRUE(value->IsString());
+                        ASSERT_STREQ(value->GetString(), "127.0.0.1");
+                      }});
+    result.push_back({"/items/0/port", [](const JsonValue *value) {
+                        ASSERT_NE(value, nullptr);
+                        ASSERT_GT(value->GetInt(), 0);
+                      }});
   }
 
   return result;
 }
 
-static RestApiComponentTest::json_verifiers_t get_expected_blocked_hosts_fields(
-    const int expected_blocked_hosts) {
-  RestApiComponentTest::json_verifiers_t result{
-      {"/items", [=](const JsonValue *value) {
-         ASSERT_NE(value, nullptr);
-         ASSERT_TRUE(value->IsArray());
-         ASSERT_EQ(value->GetArray().Size(), expected_blocked_hosts);
-       }}};
+static const std::vector<
+    std::pair<std::string, RestApiTestParams::value_check_func>>
+get_expected_blocked_hosts_fields(const int expected_blocked_hosts) {
+  std::vector<std::pair<std::string, RestApiTestParams::value_check_func>>
+      result{{"/items", [=](const JsonValue *value) {
+                ASSERT_NE(value, nullptr);
+                ASSERT_TRUE(value->IsArray());
+                ASSERT_EQ(value->GetArray().Size(), expected_blocked_hosts);
+              }}};
 
   for (int i = 0; i < expected_blocked_hosts; ++i) {
-    result.emplace_back(
-        "/items/" + std::to_string(i), [](const JsonValue *value) {
-          ASSERT_NE(value, nullptr);
-          ASSERT_THAT(value->GetString(), ::testing::StartsWith("127.0.0.1"));
-        });
+    result.push_back(
+        {"/items/" + std::to_string(i), [](const JsonValue *value) {
+           ASSERT_NE(value, nullptr);
+           ASSERT_THAT(value->GetString(), ::testing::StartsWith("127.0.0.1"));
+         }});
   }
 
   return result;
 }
 
-static const RestApiComponentTest::json_verifiers_t
+static const std::vector<
+    std::pair<std::string, RestApiTestParams::value_check_func>>
 get_expected_connections_fields_fields(const int expected_connection_qty) {
-  RestApiComponentTest::json_verifiers_t result{
-      {"/items", [=](const JsonValue *value) {
-         ASSERT_NE(value, nullptr);
-         ASSERT_TRUE(value->IsArray());
-         // -1 means that we don't really know how many connections are
-         // there, we did wait_for_port_ready on a socket and this can
-         // still be accounted for
-         if (expected_connection_qty >= 0) {
-           ASSERT_EQ(value->GetArray().Size(), expected_connection_qty);
-         }
-       }}};
+  std::vector<std::pair<std::string, RestApiTestParams::value_check_func>>
+      result{{"/items", [=](const JsonValue *value) {
+                ASSERT_NE(value, nullptr);
+                ASSERT_TRUE(value->IsArray());
+                // -1 means that we don't really know how many connections are
+                // there, we did wait_for_port_ready on a socket and this can
+                // still be accounted for
+                if (expected_connection_qty >= 0) {
+                  ASSERT_EQ(value->GetArray().Size(), expected_connection_qty);
+                }
+              }}};
 
   for (int i = 0; i < expected_connection_qty; ++i) {
-    result.emplace_back("/items/" + std::to_string(i) + "/bytesToServer",
-                        [](const JsonValue *value) {
-                          ASSERT_NE(value, nullptr);
-                          ASSERT_GT(value->GetUint64(), 0);
-                        });
+    result.push_back({"/items/" + std::to_string(i) + "/bytesToServer",
+                      [](const JsonValue *value) {
+                        ASSERT_NE(value, nullptr);
+                        ASSERT_GT(value->GetUint64(), 0);
+                      }});
 
-    result.emplace_back("/items/" + std::to_string(i) + "/sourceAddress",
-                        [](const JsonValue *value) {
-                          ASSERT_NE(value, nullptr);
-                          ASSERT_TRUE(value->IsString());
-                          ASSERT_THAT(value->GetString(),
-                                      ::testing::StartsWith("127.0.0.1"));
-                        });
+    result.push_back({"/items/" + std::to_string(i) + "/sourceAddress",
+                      [](const JsonValue *value) {
+                        ASSERT_NE(value, nullptr);
+                        ASSERT_TRUE(value->IsString());
+                        ASSERT_THAT(value->GetString(),
+                                    ::testing::StartsWith("127.0.0.1"));
+                      }});
 
-    result.emplace_back("/items/" + std::to_string(i) + "/destinationAddress",
-                        [](const JsonValue *value) {
-                          ASSERT_NE(value, nullptr);
-                          ASSERT_TRUE(value->IsString());
-                          ASSERT_THAT(value->GetString(),
-                                      ::testing::StartsWith("127.0.0.1"));
-                        });
+    result.push_back({"/items/" + std::to_string(i) + "/destinationAddress",
+                      [](const JsonValue *value) {
+                        ASSERT_NE(value, nullptr);
+                        ASSERT_TRUE(value->IsString());
+                        ASSERT_THAT(value->GetString(),
+                                    ::testing::StartsWith("127.0.0.1"));
+                      }});
 
-    result.emplace_back(
-        "/items/" + std::to_string(i) + "/timeConnectedToServer",
-        [](const JsonValue *value) {
-          ASSERT_NE(value, nullptr);
-          ASSERT_TRUE(value->IsString());
+    result.push_back(
+        {"/items/" + std::to_string(i) + "/timeConnectedToServer",
+         [](const JsonValue *value) {
+           ASSERT_NE(value, nullptr);
+           ASSERT_TRUE(value->IsString());
 
-          ASSERT_TRUE(pattern_found(value->GetString(), kTimestampPattern))
-              << value->GetString();
-        });
+           ASSERT_TRUE(pattern_found(value->GetString(), kTimestampPattern))
+               << value->GetString();
+         }});
   }
 
   return result;
@@ -442,51 +426,41 @@ get_expected_connections_fields_fields(const int expected_connection_qty) {
 // ****************************************************************************
 
 static const RestApiTestParams rest_api_valid_methods[]{
-    {"routing_status", std::string(rest_api_basepath) + "/routing/status",
-     "/routing/status", HttpMethod::Get, HttpStatusCode::Ok, kContentTypeJson,
-     kRestApiUsername, kRestApiPassword,
-     /*request_authentication =*/true,
-     get_expected_status_fields(
-         /*expected_max_total_connections=*/512,
-         /*expected_current_total_connections=*/3 + 1),
-     kRoutingSwaggerPaths},
-    {"routing_routes_status_ro",
-     std::string(rest_api_basepath) + "/routes/ro/status",
+    {"routing_status_ro", std::string(rest_api_basepath) + "/routes/ro/status",
      "/routes/{routeName}/status", HttpMethod::Get, HttpStatusCode::Ok,
      kContentTypeJson, kRestApiUsername, kRestApiPassword,
      /*request_authentication =*/true,
-     get_expected_routes_status_fields(/*expected_active_connections=*/3,
-                                       /*expected_total_connections=*/3,
-                                       /*expected_blocked_hosts*=*/0),
+     get_expected_status_fields(/*expected_active_connections=*/3,
+                                /*expected_total_connections=*/3,
+                                /*expected_blocked_hosts*=*/0),
      kRoutingSwaggerPaths},
-    {"routing_routes_status__",
-     std::string(rest_api_basepath) + "/routes/_/status",
+    {"routing_status__", std::string(rest_api_basepath) + "/routes/_/status",
      "/routes/{routeName}/status", HttpMethod::Get, HttpStatusCode::Ok,
      kContentTypeJson, kRestApiUsername, kRestApiPassword,
      /*request_authentication =*/true,
-     get_expected_routes_status_fields(/*expected_active_connections=*/0,
-                                       /*expected_total_connections=*/0,
-                                       /*expected_blocked_hosts*=*/0),
+     get_expected_status_fields(/*expected_active_connections=*/0,
+                                /*expected_total_connections=*/0,
+                                /*expected_blocked_hosts*=*/0),
      kRoutingSwaggerPaths},
-    {"routing_routes_status_Aaz",
+    {"routing_status_Aaz",
      std::string(rest_api_basepath) + "/routes/Aaz/status",
      "/routes/{routeName}/status", HttpMethod::Get, HttpStatusCode::Ok,
      kContentTypeJson, kRestApiUsername, kRestApiPassword,
      /*request_authentication =*/true,
-     get_expected_routes_status_fields(/*expected_active_connections=*/1,
-                                       /*expected_total_connections=*/1,
-                                       /*expected_blocked_hosts*=*/0),
+     get_expected_status_fields(/*expected_active_connections=*/1,
+                                /*expected_total_connections=*/1,
+                                /*expected_blocked_hosts*=*/0),
      kRoutingSwaggerPaths},
-    {"routing_routes_status_123",
+    {"routing_status_123",
      std::string(rest_api_basepath) + "/routes/123/status",
      "/routes/{routeName}/status", HttpMethod::Get, HttpStatusCode::Ok,
      kContentTypeJson, kRestApiUsername, kRestApiPassword,
      /*request_authentication =*/true,
-     get_expected_routes_status_fields(/*expected_active_connections=*/0,
-                                       /*expected_total_connections=*/3,
-                                       /*expected_blocked_hosts*=*/1),
+     get_expected_status_fields(/*expected_active_connections=*/0,
+                                /*expected_total_connections=*/3,
+                                /*expected_blocked_hosts*=*/1),
      kRoutingSwaggerPaths},
-    {"routing_routes_status_nonexistent",
+    {"routing_status_nonexistent",
      std::string(rest_api_basepath) + "/routes/nonexistent/status",
      "/routes/{routeName}/status",
      HttpMethod::Get,
@@ -497,7 +471,7 @@ static const RestApiTestParams rest_api_valid_methods[]{
      /*request_authentication =*/true,
      {},
      kRoutingSwaggerPaths},
-    {"routing_routes_status_params",
+    {"routing_status_params",
      std::string(rest_api_basepath) + "/routes/123/status?someparam",
      "/routes/{routeName}/status",
      HttpMethod::Get,
@@ -769,7 +743,7 @@ static const RestApiTestParams rest_api_valid_methods[]{
      kRoutingSwaggerPaths},
 };
 
-INSTANTIATE_TEST_SUITE_P(
+INSTANTIATE_TEST_CASE_P(
     ValidMethods, RestRoutingApiTest,
     ::testing::ValuesIn(rest_api_valid_methods),
     [](const ::testing::TestParamInfo<RestApiTestParams> &info) {
@@ -861,7 +835,7 @@ static const RestApiTestParams rest_api_valid_methods_invalid_auth_params[]{
      kRoutingSwaggerPaths},
 };
 
-INSTANTIATE_TEST_SUITE_P(
+INSTANTIATE_TEST_CASE_P(
     ValidMethodsInvalidAuth, RestRoutingApiTest,
     ::testing::ValuesIn(rest_api_valid_methods_invalid_auth_params),
     [](const ::testing::TestParamInfo<RestApiTestParams> &info) {
@@ -877,74 +851,74 @@ static const RestApiTestParams rest_api_invalid_methods_params[]{
      std::string(rest_api_basepath) + "/routes/ro/status",
      "/routes/{routeName}/status",
      HttpMethod::Post | HttpMethod::Delete | HttpMethod::Patch |
-         HttpMethod::Trace | HttpMethod::Options | HttpMethod::Head,
+         HttpMethod::Trace | HttpMethod::Options | HttpMethod::Connect |
+         HttpMethod::Head,
      HttpStatusCode::MethodNotAllowed, kContentTypeJsonProblem,
      kRestApiUsername, kRestApiPassword,
      /*request_authentication =*/true,
-     RestApiComponentTest::get_json_method_not_allowed_verifiers(),
-     kRoutingSwaggerPaths},
+     RestApiComponentTest::kProblemJsonMethodNotAllowed, kRoutingSwaggerPaths},
     {"routes_invalid_methods", std::string(rest_api_basepath) + "/routes",
      "/routes",
      HttpMethod::Post | HttpMethod::Delete | HttpMethod::Patch |
-         HttpMethod::Trace | HttpMethod::Options | HttpMethod::Head,
+         HttpMethod::Trace | HttpMethod::Options | HttpMethod::Connect |
+         HttpMethod::Head,
      HttpStatusCode::MethodNotAllowed, kContentTypeJsonProblem,
      kRestApiUsername, kRestApiPassword,
      /*request_authentication =*/true,
-     RestApiComponentTest::get_json_method_not_allowed_verifiers(),
-     kRoutingSwaggerPaths},
+     RestApiComponentTest::kProblemJsonMethodNotAllowed, kRoutingSwaggerPaths},
     {"routes_config_invalid_methods",
      std::string(rest_api_basepath) + "/routes/ro/config",
      "/routes/{routeName}/config",
      HttpMethod::Post | HttpMethod::Delete | HttpMethod::Patch |
-         HttpMethod::Head | HttpMethod::Trace | HttpMethod::Options,
+         HttpMethod::Head | HttpMethod::Trace | HttpMethod::Options |
+         HttpMethod::Connect,
      HttpStatusCode::MethodNotAllowed, kContentTypeJsonProblem,
      kRestApiUsername, kRestApiPassword,
      /*request_authentication =*/true,
-     RestApiComponentTest::get_json_method_not_allowed_verifiers(),
-     kRoutingSwaggerPaths},
+     RestApiComponentTest::kProblemJsonMethodNotAllowed, kRoutingSwaggerPaths},
     {"routes_health_invalid_methods",
      std::string(rest_api_basepath) + "/routes/ro/health",
      "/routes/{routeName}/health",
      HttpMethod::Post | HttpMethod::Delete | HttpMethod::Patch |
-         HttpMethod::Head | HttpMethod::Trace | HttpMethod::Options,
+         HttpMethod::Head | HttpMethod::Trace | HttpMethod::Options |
+         HttpMethod::Connect,
      HttpStatusCode::MethodNotAllowed, kContentTypeJsonProblem,
      kRestApiUsername, kRestApiPassword,
      /*request_authentication =*/true,
-     RestApiComponentTest::get_json_method_not_allowed_verifiers(),
-     kRoutingSwaggerPaths},
+     RestApiComponentTest::kProblemJsonMethodNotAllowed, kRoutingSwaggerPaths},
     {"routes_destinations_invalid_methods",
      std::string(rest_api_basepath) + "/routes/ro/destinations",
      "/routes/{routeName}/destinations",
      HttpMethod::Post | HttpMethod::Delete | HttpMethod::Patch |
-         HttpMethod::Head | HttpMethod::Trace | HttpMethod::Options,
+         HttpMethod::Head | HttpMethod::Trace | HttpMethod::Options |
+         HttpMethod::Connect,
      HttpStatusCode::MethodNotAllowed, kContentTypeJsonProblem,
      kRestApiUsername, kRestApiPassword,
      /*request_authentication =*/true,
-     RestApiComponentTest::get_json_method_not_allowed_verifiers(),
-     kRoutingSwaggerPaths},
+     RestApiComponentTest::kProblemJsonMethodNotAllowed, kRoutingSwaggerPaths},
     {"routes_blockedhosts_invalid_methods",
      std::string(rest_api_basepath) + "/routes/ro/blockedHosts",
      "/routes/{routeName}/blockedHosts",
      HttpMethod::Post | HttpMethod::Delete | HttpMethod::Patch |
-         HttpMethod::Head | HttpMethod::Trace | HttpMethod::Options,
+         HttpMethod::Head | HttpMethod::Trace | HttpMethod::Options |
+         HttpMethod::Connect,
      HttpStatusCode::MethodNotAllowed, kContentTypeJsonProblem,
      kRestApiUsername, kRestApiPassword,
      /*request_authentication =*/true,
-     RestApiComponentTest::get_json_method_not_allowed_verifiers(),
-     kRoutingSwaggerPaths},
+     RestApiComponentTest::kProblemJsonMethodNotAllowed, kRoutingSwaggerPaths},
     {"routes_connections_invalid_methods",
      std::string(rest_api_basepath) + "/routes/ro/connections",
      "/routes/{routeName}/connections",
      HttpMethod::Post | HttpMethod::Delete | HttpMethod::Patch |
-         HttpMethod::Head | HttpMethod::Trace | HttpMethod::Options,
+         HttpMethod::Head | HttpMethod::Trace | HttpMethod::Options |
+         HttpMethod::Connect,
      HttpStatusCode::MethodNotAllowed, kContentTypeJsonProblem,
      kRestApiUsername, kRestApiPassword,
      /*request_authentication =*/true,
-     RestApiComponentTest::get_json_method_not_allowed_verifiers(),
-     kRoutingSwaggerPaths},
+     RestApiComponentTest::kProblemJsonMethodNotAllowed, kRoutingSwaggerPaths},
 };
 
-INSTANTIATE_TEST_SUITE_P(
+INSTANTIATE_TEST_CASE_P(
     InvalidMethods, RestRoutingApiTest,
     ::testing::ValuesIn(rest_api_invalid_methods_params),
     [](const ::testing::TestParamInfo<RestApiTestParams> &info) {
@@ -965,18 +939,21 @@ TEST_F(RestRoutingApiTest, routing_api_no_auth) {
   auto config_sections = get_restapi_config("rest_routing", userfile,
                                             /*request_authentication=*/false);
 
+  // [rest_api] is always required
+  config_sections.push_back(ConfigBuilder::build_section("rest_api", {}));
+
   const std::string conf_file{create_config_file(
       conf_dir_.name(), mysql_harness::join(config_sections, "\n"))};
-  auto &router =
-      launch_router({"-c", conf_file}, EXIT_FAILURE, true, false, -1s);
+  auto &router = launch_router({"-c", conf_file}, EXIT_FAILURE);
 
-  check_exit_code(router, EXIT_FAILURE, 10s);
+  const auto wait_for_process_exit_timeout{10000ms};
+  check_exit_code(router, EXIT_FAILURE, wait_for_process_exit_timeout);
 
-  const std::string router_output = router.get_logfile_content();
-  EXPECT_THAT(
-      router_output,
-      ::testing::HasSubstr("  init 'rest_routing' failed: option "
-                           "require_realm in [rest_routing] is required"));
+  const std::string router_output = router.get_full_logfile();
+  EXPECT_NE(router_output.find("plugin 'rest_routing' init failed: option "
+                               "require_realm in [rest_routing] is required"),
+            router_output.npos)
+      << router_output;
 }
 
 /**
@@ -989,19 +966,21 @@ TEST_F(RestRoutingApiTest, invalid_realm) {
       get_restapi_config("rest_routing", userfile,
                          /*request_authentication=*/true, "invalidrealm");
 
+  // [rest_api] is always required
+  config_sections.push_back(ConfigBuilder::build_section("rest_api", {}));
+
   const std::string conf_file{create_config_file(
       conf_dir_.name(), mysql_harness::join(config_sections, "\n"))};
-  auto &router =
-      launch_router({"-c", conf_file}, EXIT_FAILURE, true, false, -1s);
+  auto &router = launch_router({"-c", conf_file}, EXIT_FAILURE);
 
-  check_exit_code(router, EXIT_FAILURE, 10s);
+  const auto wait_for_process_exit_timeout{10000ms};
+  check_exit_code(router, EXIT_FAILURE, wait_for_process_exit_timeout);
 
-  const std::string router_output = router.get_logfile_content();
-  EXPECT_THAT(
-      router_output,
-      ::testing::HasSubstr(
-          "Configuration error: The option 'require_realm=invalidrealm' "
-          "in [rest_routing] does not match any http_auth_realm."))
+  const std::string router_output = router.get_full_logfile();
+  EXPECT_NE(router_output.find(
+                "Configuration error: The option 'require_realm=invalidrealm' "
+                "in [rest_routing] does not match any http_auth_realm."),
+            router_output.npos)
       << router_output;
 }
 
@@ -1009,14 +988,24 @@ TEST_F(RestRoutingApiTest, invalid_realm) {
  * @test Start router with the REST routing API plugin [rest_routing] and
  * [http_plugin] enabled but not the [rest_api] plugin.
  */
-TEST_F(RestRoutingApiTest, routing_api_no_rest_api_works) {
+TEST_F(RestRoutingApiTest, routing_api_no_rest_api) {
   const std::string userfile = create_password_file();
   auto config_sections = get_restapi_config("rest_routing", userfile,
-                                            /*request_authentication=*/true);
+                                            /*request_authentication=*/false);
 
   const std::string conf_file{create_config_file(
       conf_dir_.name(), mysql_harness::join(config_sections, "\n"))};
-  launch_router({"-c", conf_file}, EXIT_SUCCESS);
+  auto &router = launch_router({"-c", conf_file}, EXIT_FAILURE);
+
+  const auto wait_for_process_exit_timeout{10000ms};
+  check_exit_code(router, EXIT_FAILURE, wait_for_process_exit_timeout);
+
+  const std::string router_output = router.get_full_output();
+  EXPECT_NE(router_output.find("Plugin 'rest_routing' needs plugin "
+                               "'rest_api' which is missing in the "
+                               "configuration"),
+            router_output.npos)
+      << router_output;
 }
 
 /**
@@ -1029,21 +1018,23 @@ TEST_F(RestRoutingApiTest, rest_routing_section_twice) {
   auto config_sections = get_restapi_config("rest_routing", userfile,
                                             /*request_authentication=*/true);
 
+  // [rest_api] is always required
+  config_sections.push_back(ConfigBuilder::build_section("rest_api", {}));
+
   // force [rest_routing] twice in the config
-  config_sections.push_back(
-      mysql_harness::ConfigBuilder::build_section("rest_routing", {}));
+  config_sections.push_back(ConfigBuilder::build_section("rest_routing", {}));
 
   const std::string conf_file{create_config_file(
       conf_dir_.name(), mysql_harness::join(config_sections, "\n"))};
-  auto &router =
-      launch_router({"-c", conf_file}, EXIT_FAILURE, true, false, -1s);
+  auto &router = launch_router({"-c", conf_file}, EXIT_FAILURE);
 
-  check_exit_code(router, EXIT_FAILURE, 10s);
+  const auto wait_for_process_exit_timeout{10000ms};
+  check_exit_code(router, EXIT_FAILURE, wait_for_process_exit_timeout);
 
   const std::string router_output = router.get_full_output();
-  EXPECT_THAT(router_output,
-              ::testing::HasSubstr(
-                  "Configuration error: Section 'rest_routing' already exists"))
+  EXPECT_NE(router_output.find(
+                "Configuration error: Section 'rest_routing' already exists"),
+            router_output.npos)
       << router_output;
 }
 
@@ -1057,20 +1048,40 @@ TEST_F(RestRoutingApiTest, rest_routing_section_has_key) {
   auto config_sections = get_restapi_config("rest_routing:A", userfile,
                                             /*request_authentication=*/true);
 
+  // [rest_api] is always required
+  config_sections.push_back(ConfigBuilder::build_section("rest_api", {}));
+
   const std::string conf_file{create_config_file(
       conf_dir_.name(), mysql_harness::join(config_sections, "\n"))};
-  auto &router =
-      launch_router({"-c", conf_file}, EXIT_FAILURE, true, false, -1s);
+  auto &router = launch_router({"-c", conf_file}, EXIT_FAILURE);
 
-  check_exit_code(router, EXIT_FAILURE, 10s);
+  const auto wait_for_process_exit_timeout{10000ms};
+  check_exit_code(router, EXIT_FAILURE, wait_for_process_exit_timeout);
 
-  const std::string router_output = router.get_logfile_content();
-  EXPECT_THAT(router_output, ::testing::HasSubstr(
-                                 "  init 'rest_routing' failed: [rest_routing] "
-                                 "section does not expect a key, found 'A'"))
+  const std::string router_output = router.get_full_logfile();
+  EXPECT_NE(
+      router_output.find("plugin 'rest_routing' init failed: [rest_routing] "
+                         "section does not expect a key, found 'A'"),
+      router_output.npos)
       << router_output;
 }
 
+static std::string get_server_addr_list(const std::vector<uint16_t> &ports) {
+  std::string result;
+  bool use_comma = false;
+
+  for (const auto &port : ports) {
+    if (use_comma) {
+      result += ",";
+    } else {
+      use_comma = true;
+    }
+    result += "mysql://localhost:" + std::to_string(port);
+  }
+
+  return result;
+}
+
 class RestRoutingApiTestCluster : public RestRoutingApiTest {};
 
 /**
@@ -1089,7 +1100,7 @@ TEST_P(RestRoutingApiTestCluster, ensure_openapi_cluster) {
   std::vector<uint16_t> node_classic_ports;
   uint16_t first_node_http_port{0};
   const std::string json_metadata =
-      get_data_dir().join("metadata_dynamic_nodes_v2_gr.js").str();
+      get_data_dir().join("metadata_dynamic_nodes.js").str();
   for (size_t i = 0; i < 3; ++i) {
     node_classic_ports.push_back(port_pool_.get_next_available());
     if (i == 0) first_node_http_port = port_pool_.get_next_available();
@@ -1097,14 +1108,12 @@ TEST_P(RestRoutingApiTestCluster, ensure_openapi_cluster) {
     nodes.push_back(&launch_mysql_server_mock(
         json_metadata, node_classic_ports[i], EXIT_SUCCESS, false,
         i == 0 ? first_node_http_port : 0));
+    ASSERT_NO_FATAL_FAILURE(check_port_ready(*nodes[i], node_classic_ports[i]));
   }
 
   ASSERT_TRUE(MockServerRestClient(first_node_http_port)
                   .wait_for_rest_endpoint_ready());
-
-  set_mock_metadata(first_node_http_port, "uuid",
-                    classic_ports_to_gr_nodes(node_classic_ports), 0,
-                    classic_ports_to_cluster_nodes(node_classic_ports));
+  set_mock_metadata(first_node_http_port, "", node_classic_ports);
 
   SCOPED_TRACE("// start the router with rest_routing enabled");
   for (size_t i = 0; i < 2; ++i) {
@@ -1115,13 +1124,14 @@ TEST_P(RestRoutingApiTestCluster, ensure_openapi_cluster) {
 
   auto config_sections = get_restapi_config("rest_routing", userfile,
                                             GetParam().request_authentication);
+  config_sections.push_back(ConfigBuilder::build_section("rest_api", {}));
 
   size_t i = 0;
   for (const auto &route_name : route_names) {
     const std::string role = (i == 0) ? "PRIMARY" : "SECONDARY";
     const std::string destinations =
         "metadata-cache://test/default?role=" + role;
-    config_sections.push_back(mysql_harness::ConfigBuilder::build_section(
+    config_sections.push_back(ConfigBuilder::build_section(
         "routing:"s + route_name,
         {
             {"bind_port", std::to_string(routing_ports_[i])},
@@ -1136,28 +1146,29 @@ TEST_P(RestRoutingApiTestCluster, ensure_openapi_cluster) {
     ++i;
   }
 
+  config_sections.push_back(
+      ConfigBuilder::build_section("logger", {
+                                                 {"level", "debug"},
+                                             }));
+
   const std::string keyring_username = "mysql_router1_user";
-  config_sections.push_back(mysql_harness::ConfigBuilder::build_section(
+  config_sections.push_back(ConfigBuilder::build_section(
       "metadata_cache:test", {
-                                 {"router_id", "1"},
+                                 {"router_id", "3"},
                                  {"user", keyring_username},
                                  {"metadata_cluster", "test"},
+                                 {"bootstrap_server_addresses",
+                                  get_server_addr_list(node_classic_ports)},
                              }));
 
   auto default_section = get_DEFAULT_defaults();
   init_keyring(default_section, conf_dir_.name());
 
-  const auto state_file = create_state_file(
-      get_test_temp_dir_name(),
-      create_state_file_content("uuid", "", node_classic_ports, 0));
-  default_section["dynamic_state"] = state_file;
-
   const std::string conf_file{create_config_file(
-      conf_dir_.name(), mysql_harness::join(config_sections, ""),
+      conf_dir_.name(), mysql_harness::join(config_sections, "\n"),
       &default_section)};
 
-  ProcessWrapper &http_server =
-      launch_router({"-c", conf_file}, EXIT_SUCCESS, true, false, -1s);
+  ProcessWrapper &http_server = launch_router({"-c", conf_file});
 
   // wait for both (rw and ro) routes being available
   for (size_t i = 0; i < 2; ++i) {
@@ -1179,37 +1190,29 @@ TEST_P(RestRoutingApiTestCluster, ensure_openapi_cluster) {
   EXPECT_NO_THROW(client_rw_2.connect("127.0.0.1", routing_ports_[1], "root",
                                       "fake-pass", "", ""));
 
-  ASSERT_NO_FATAL_FAILURE(
+  EXPECT_NO_FATAL_FAILURE(
       fetch_and_validate_schema_and_resource(GetParam(), http_server));
 }
 
 static const RestApiTestParams rest_api_valid_methods_params_cluster[]{
-    {"routing_status", std::string(rest_api_basepath) + "/routing/status",
-     "/routing/status", HttpMethod::Get, HttpStatusCode::Ok, kContentTypeJson,
-     kRestApiUsername, kRestApiPassword,
-     /*request_authentication =*/true,
-     get_expected_status_fields(
-         /*expected_max_total_connections=*/512,
-         /*expected_current_total_connections=*/2 + 1),
-     kRoutingSwaggerPaths},
-    {"routing_routes_rw_status",
+    {"routing_rw_status",
      std::string(rest_api_basepath) + "/routes/cluster_rw/status",
      "/routes/{routeName}/status", HttpMethod::Get, HttpStatusCode::Ok,
      kContentTypeJson, kRestApiUsername, kRestApiPassword,
      /*request_authentication =*/true,
-     get_expected_routes_status_fields(/*expected_active_connections=*/1,
-                                       /*expected_total_connections=*/1,
-                                       /*expected_blocked_hosts*=*/0),
+     get_expected_status_fields(/*expected_active_connections=*/1,
+                                /*expected_total_connections=*/1,
+                                /*expected_blocked_hosts*=*/0),
      kRoutingSwaggerPaths},
 
-    {"routing_routes_ro_status",
+    {"routing_ro_status",
      std::string(rest_api_basepath) + "/routes/cluster_ro/status",
      "/routes/{routeName}/status", HttpMethod::Get, HttpStatusCode::Ok,
      kContentTypeJson, kRestApiUsername, kRestApiPassword,
      /*request_authentication =*/true,
-     get_expected_routes_status_fields(/*expected_active_connections=*/2,
-                                       /*expected_total_connections=*/2,
-                                       /*expected_blocked_hosts*=*/0),
+     get_expected_status_fields(/*expected_active_connections=*/2,
+                                /*expected_total_connections=*/2,
+                                /*expected_blocked_hosts*=*/0),
      kRoutingSwaggerPaths},
 
     {"cluster_routes",
@@ -1308,7 +1311,7 @@ static const RestApiTestParams rest_api_valid_methods_params_cluster[]{
      kRoutingSwaggerPaths},
 };
 
-INSTANTIATE_TEST_SUITE_P(
+INSTANTIATE_TEST_CASE_P(
     ValidMethodsCluster, RestRoutingApiTestCluster,
     ::testing::ValuesIn(rest_api_valid_methods_params_cluster),
     [](const ::testing::TestParamInfo<RestApiTestParams> &info) {
