@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2018, 2023, Oracle and/or its affiliates.
+Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
@@ -22,42 +22,38 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-#include <chrono>
-#include <cstdint>
-#include <fstream>
-#include <stdexcept>
-#include <thread>
-
 #ifdef RAPIDJSON_NO_SIZETYPEDEFINE
+// if we build within the server, it will set RAPIDJSON_NO_SIZETYPEDEFINE
+// globally and require to include my_rapidjson_size_t.h
 #include "my_rapidjson_size_t.h"
 #endif
 
-#include <gmock/gmock-matchers.h>
-#include <gtest/gtest.h>
 #include <rapidjson/document.h>
 #include <rapidjson/error/en.h>
 #include <rapidjson/filereadstream.h>
 #include <rapidjson/prettywriter.h>
 #include <rapidjson/schema.h>
 #include <rapidjson/stringbuffer.h>
-
-#include "config_builder.h"
+#include "gmock/gmock.h"
 #include "keyring/keyring_manager.h"
 #include "mock_server_rest_client.h"
 #include "mock_server_testutils.h"
-#include "mysql/harness/string_utils.h"  // split_string
+#include "mysql_session.h"
 #include "mysqlrouter/cluster_metadata.h"
-#include "mysqlrouter/mysql_session.h"
 #include "mysqlrouter/rest_client.h"
-#include "process_manager.h"
 #include "router_component_system_layout.h"
 #include "router_component_test.h"
 #include "router_component_testutils.h"
-#include "router_test_helpers.h"
 #include "tcp_port_pool.h"
 
+#include <chrono>
+#include <fstream>
+#include <stdexcept>
+#include <thread>
+
 using mysqlrouter::ClusterType;
 using mysqlrouter::MySQLSession;
+using ::testing::PrintToString;
 using namespace std::chrono_literals;
 
 Path g_origin_path;
@@ -72,10 +68,10 @@ using JsonDocument =
 using JsonStringBuffer =
     rapidjson::GenericStringBuffer<rapidjson::UTF8<>, rapidjson::CrtAllocator>;
 
-constexpr auto kTTL = 100ms;
+constexpr auto kTTL = std::chrono::milliseconds(100);
 }  // namespace
 
-class StateFileTest : public RouterComponentBootstrapTest {
+class StateFileTest : public RouterComponentTest {
  protected:
   void SetUp() override {
     RouterComponentTest::SetUp();
@@ -83,71 +79,53 @@ class StateFileTest : public RouterComponentBootstrapTest {
     ProcessManager::set_origin(g_origin_path);
   }
 
-  std::pair<std::string, std::map<std::string, std::string>>
-  metadata_cache_section(uint16_t metadata_server_port = 0,
-                         const std::chrono::milliseconds ttl = kTTL,
-                         ClusterType cluster_type = ClusterType::GR_V2) {
-    std::map<std::string, std::string> options{
-        {"cluster_type", (cluster_type == ClusterType::RS_V2) ? "rs" : "gr"},
-        {"router_id", "1"},
-        {"user", "mysql_router1_user"},
-        {"metadata_cluster", "test"},
-        {"connect_timeout", "1"},
-        {"ttl", std::to_string(std::chrono::duration<double>(ttl).count())},
-    };
-
-    if (metadata_server_port != 0) {
-      options["bootstrap_server_addresses"] =
-          "mysql://localhost:" + std::to_string(metadata_server_port);
-    };
-
-    return {"metadata_cache:test", options};
-  }
-
   std::string get_metadata_cache_section(
       uint16_t metadata_server_port = 0,
       const std::chrono::milliseconds ttl = kTTL,
       ClusterType cluster_type = ClusterType::GR_V2) {
-    auto section =
-        metadata_cache_section(metadata_server_port, ttl, cluster_type);
-    return mysql_harness::ConfigBuilder::build_section(section.first,
-                                                       section.second) +
-           "\n";
-  }
+    auto ttl_str = std::to_string(std::chrono::duration<double>(ttl).count());
+    const std::string cluster_type_str =
+        (cluster_type == ClusterType::RS_V2) ? "rs" : "gr";
 
-  std::pair<std::string, std::map<std::string, std::string>>
-  metadata_cache_routing_section(uint16_t router_port, const std::string &role,
-                                 const std::string &strategy,
-                                 const std::string &mode = "") {
-    std::map<std::string, std::string> options{
-        {"bind_port", std::to_string(router_port)},
-        {"destinations", "metadata-cache://test/default?role=" + role},
-        {"protocol", "classic"},
-    };
-
-    if (!strategy.empty()) options["routing_strategy"] = strategy;
-    if (!mode.empty()) options["mode"] = mode;
-
-    return {"routing:test_default", options};
+    return "[metadata_cache:test]\n"
+           "cluster_type=" +
+           cluster_type_str +
+           "\n"
+           "router_id=1\n" +
+           ((metadata_server_port == 0)
+                ? ""
+                : "bootstrap_server_addresses=mysql://localhost:" +
+                      std::to_string(metadata_server_port) + "\n") +
+           "user=mysql_router1_user\n"
+           "metadata_cluster=test\n"
+           "connect_timeout=1\n"
+           "ttl=" +
+           ttl_str + "\n\n";
   }
 
   std::string get_metadata_cache_routing_section(uint16_t router_port,
                                                  const std::string &role,
                                                  const std::string &strategy,
                                                  const std::string &mode = "") {
-    auto section =
-        metadata_cache_routing_section(router_port, role, strategy, mode);
-    return mysql_harness::ConfigBuilder::build_section(section.first,
-                                                       section.second) +
-           "\n";
+    std::string result =
+        "[routing:test_default]\n"
+        "bind_port=" +
+        std::to_string(router_port) + "\n" +
+        "destinations=metadata-cache://test/default?role=" + role + "\n" +
+        "protocol=classic\n";
+
+    if (!strategy.empty())
+      result += std::string("routing_strategy=" + strategy + "\n");
+    if (!mode.empty()) result += std::string("mode=" + mode + "\n");
+
+    return result;
   }
 
   auto &launch_router(const std::string &temp_test_dir,
                       const std::string &metadata_cache_section,
                       const std::string &routing_section,
                       const std::string &state_file_path,
-                      const int expected_errorcode = EXIT_SUCCESS,
-                      std::chrono::milliseconds wait_for_notify = 5s) {
+                      const int expected_errorcode = EXIT_SUCCESS) {
     const std::string masterkey_file =
         Path(temp_test_dir).join("master.key").str();
     const std::string keyring_file = Path(temp_test_dir).join("keyring").str();
@@ -157,31 +135,30 @@ class StateFileTest : public RouterComponentBootstrapTest {
     mysql_harness::flush_keyring();
     mysql_harness::reset_keyring();
 
+    // enable debug logs for better diagnostics in case of failure
+    std::string logger_section = "[logger]\nlevel = DEBUG\n";
+
     // launch the router with metadata-cache configuration
     auto default_section = get_DEFAULT_defaults();
     default_section["keyring_path"] = keyring_file;
     default_section["master_key_path"] = masterkey_file;
     default_section["dynamic_state"] = state_file_path;
     const std::string conf_file = create_config_file(
-        temp_test_dir, metadata_cache_section + routing_section,
+        temp_test_dir,
+        logger_section + metadata_cache_section + routing_section,
         &default_section);
     auto &router = ProcessManager::launch_router(
         {"-c", conf_file}, expected_errorcode, /*catch_stderr=*/true,
-        /*with_sudo=*/false,
-        /*wait_for_notify_ready=*/wait_for_notify);
+        /*with_sudo=*/false);
     return router;
   }
 
   bool wait_log_file_contains(ProcessWrapper &router,
                               const std::string &expected_entry,
                               std::chrono::milliseconds max_wait_time) {
-    auto kRetrySleep = 100ms;
-    if (getenv("WITH_VALGRIND")) {
-      max_wait_time *= 50;
-      kRetrySleep *= 10;
-    }
+    const auto kRetrySleep = 100ms;
     do {
-      const auto log_content = router.get_logfile_content();
+      const auto log_content = router.get_full_logfile();
       if (log_content.find(expected_entry) != std::string::npos) return true;
 
       std::this_thread::sleep_for(kRetrySleep);
@@ -190,6 +167,71 @@ class StateFileTest : public RouterComponentBootstrapTest {
     } while (true);
   }
 
+  bool wait_state_file_contains(
+      const std::string &state_file, const std::string &expected_gr_name,
+      const std::vector<std::string> expected_gr_nodes,
+      std::chrono::milliseconds max_wait_time) {
+    bool result = false;
+    const auto kRetrySleep = 100ms;
+    do {
+      result =
+          check_state_file(state_file, expected_gr_name, expected_gr_nodes);
+      if (result) return true;
+
+      std::this_thread::sleep_for(kRetrySleep);
+      if (max_wait_time <= kRetrySleep) return false;
+      max_wait_time -= kRetrySleep;
+    } while (true);
+  }
+
+#define CHECK_COND(c) \
+  if (!(c)) return false
+
+  bool check_state_file(const std::string &state_file,
+                        const std::string &expected_gr_name,
+                        const std::vector<std::string> &expected_gr_nodes) {
+    const std::string state_file_content =
+        get_file_output(state_file, /*throw_on_error =*/false);
+    JsonDocument json_doc;
+    json_doc.Parse(state_file_content.c_str());
+    const std::string kExpectedVersion = "1.0.0";
+
+    CHECK_COND(json_doc.HasMember("version"));
+    CHECK_COND(json_doc["version"].IsString());
+    CHECK_COND(kExpectedVersion == json_doc["version"].GetString());
+
+    CHECK_COND(json_doc.HasMember("metadata-cache"));
+    CHECK_COND(json_doc["metadata-cache"].IsObject());
+
+    auto metadata_cache_section = json_doc["metadata-cache"].GetObject();
+
+    CHECK_COND(metadata_cache_section.HasMember("group-replication-id"));
+    CHECK_COND(metadata_cache_section["group-replication-id"].IsString());
+    CHECK_COND(expected_gr_name ==
+               metadata_cache_section["group-replication-id"].GetString());
+
+    CHECK_COND(metadata_cache_section.HasMember("cluster-metadata-servers"));
+    CHECK_COND(metadata_cache_section["cluster-metadata-servers"].IsArray());
+    auto cluster_nodes =
+        metadata_cache_section["cluster-metadata-servers"].GetArray();
+    CHECK_COND(expected_gr_nodes.size() == cluster_nodes.Size());
+    for (unsigned i = 0; i < cluster_nodes.Size(); ++i) {
+      CHECK_COND(cluster_nodes[i].IsString());
+      CHECK_COND(expected_gr_nodes[i] == cluster_nodes[i].GetString());
+    }
+
+    // check that we have write access to the file
+    // just append it with an empty line, that will not break it
+    try {
+      std::ofstream ofs(state_file, std::ios::app);
+      ofs << "\n";
+    } catch (...) {
+      return false;
+    }
+
+    return true;
+  }
+
   std::string create_state_file_content(
       const std::string &cluster_id,
       const std::vector<uint16_t> &metadata_servers_ports,
@@ -213,12 +255,16 @@ class StateFileTest : public RouterComponentBootstrapTest {
 
     return result;
   }
+
+  TcpPortPool port_pool_;
 };
 
 //////////////////////////////////////////////////////////////////////////
 
 class StateFileDynamicChangesTest : public StateFileTest {
  protected:
+  void SetUp() override { StateFileTest::SetUp(); }
+
   void kill_server(ProcessWrapper *server) { EXPECT_NO_THROW(server->kill()); }
 };
 
@@ -291,11 +337,11 @@ TEST_P(StateFileMetadataServersChangedInRuntimeTest,
                     .wait_for_rest_endpoint_ready());
 
     SCOPED_TRACE(
-        "// Make our metadata server to return single node as a cluster "
+        "// Make our metadata server to return single node as a replicaset "
         "member (meaning single metadata server)");
     set_mock_metadata(cluster_http_ports[i], kGroupId,
-                      classic_ports_to_gr_nodes({cluster_nodes_ports[i]}), i,
-                      {cluster_nodes_ports[i]}, 0, false, node_host);
+                      std::vector<uint16_t>{cluster_nodes_ports[i]}, 0, 0,
+                      false, node_host);
   }
 
   SCOPED_TRACE("// Create a router state file with a single metadata server");
@@ -321,27 +367,31 @@ TEST_P(StateFileMetadataServersChangedInRuntimeTest,
       "// Check our state file content, it should not change yet, there is "
       "single metadata server reported as initially");
 
-  check_state_file(state_file, param.cluster_type, kGroupId,
-                   {cluster_nodes_ports[0]}, 0, node_host);
+  EXPECT_TRUE(wait_state_file_contains(
+      state_file, kGroupId,
+      {"mysql://" + node_host + ":" + std::to_string(cluster_nodes_ports[0])},
+      10 * kTTL))
+      << get_file_output(state_file);
 
   SCOPED_TRACE(
       "// Now change the response from the metadata server to return 3 gr "
       "nodes (metadata servers)");
   for (unsigned i = 0; i < CLUSTER_NODES; ++i) {
-    set_mock_metadata(cluster_http_ports[i], kGroupId,
-                      classic_ports_to_gr_nodes(cluster_nodes_ports), i,
-                      classic_ports_to_cluster_nodes(cluster_nodes_ports), 0,
-                      false, node_host);
+    set_mock_metadata(cluster_http_ports[i], kGroupId, cluster_nodes_ports, 0,
+                      0, false, node_host);
   }
 
   SCOPED_TRACE(
       "// Check our state file content, it should now contain 3 metadata "
       "servers");
 
-  check_state_file(
-      state_file, param.cluster_type, kGroupId,
-      {cluster_nodes_ports[0], cluster_nodes_ports[1], cluster_nodes_ports[2]},
-      0, node_host);
+  EXPECT_TRUE(wait_state_file_contains(
+      state_file, kGroupId,
+      {"mysql://" + node_host + ":" + std::to_string(cluster_nodes_ports[0]),
+       "mysql://" + node_host + ":" + std::to_string(cluster_nodes_ports[1]),
+       "mysql://" + node_host + ":" + std::to_string(cluster_nodes_ports[2])},
+      std::chrono::milliseconds(10 * kTTL)))
+      << get_file_output(state_file);
 
   ///////////////////////////////////////////////////
 
@@ -361,14 +411,10 @@ TEST_P(StateFileMetadataServersChangedInRuntimeTest,
       "// Instrument the second and third metadata servers to return 2 "
       "servers: second and third");
   set_mock_metadata(cluster_http_ports[1], kGroupId,
-                    classic_ports_to_gr_nodes(
-                        {cluster_nodes_ports[1], cluster_nodes_ports[2]}),
-                    1, {cluster_nodes_ports[1], cluster_nodes_ports[2]}, 0,
+                    {cluster_nodes_ports[1], cluster_nodes_ports[2]}, 0, 0,
                     false, node_host);
   set_mock_metadata(cluster_http_ports[2], kGroupId,
-                    classic_ports_to_gr_nodes(
-                        {cluster_nodes_ports[1], cluster_nodes_ports[2]}),
-                    2, {cluster_nodes_ports[1], cluster_nodes_ports[2]}, 0,
+                    {cluster_nodes_ports[1], cluster_nodes_ports[2]}, 0, 0,
                     false, node_host);
 
   SCOPED_TRACE("// Kill first metada server");
@@ -377,12 +423,15 @@ TEST_P(StateFileMetadataServersChangedInRuntimeTest,
   SCOPED_TRACE(
       "// Check our state file content, it should now contain 2 metadata "
       "servers reported by the second metadata server");
-  check_state_file(state_file, param.cluster_type, kGroupId,
-                   {cluster_nodes_ports[1], cluster_nodes_ports[2]}, 0,
-                   node_host, 10000ms);
+  EXPECT_TRUE(wait_state_file_contains(
+      state_file, kGroupId,
+      {"mysql://" + node_host + ":" + std::to_string(cluster_nodes_ports[1]),
+       "mysql://" + node_host + ":" + std::to_string(cluster_nodes_ports[2])},
+      10000ms))
+      << get_file_output(state_file);
 }
 
-INSTANTIATE_TEST_SUITE_P(
+INSTANTIATE_TEST_CASE_P(
     MetadataServersChangedInRuntime,
     StateFileMetadataServersChangedInRuntimeTest,
     ::testing::Values(
@@ -427,12 +476,11 @@ TEST_P(StateFileMetadataServersInaccessibleTest, MetadataServersInaccessible) {
       MockServerRestClient(cluster_http_port).wait_for_rest_endpoint_ready());
 
   SCOPED_TRACE(
-      "// Make our metadata server return single node as a cluster "
+      "// Make our metadata server return single node as a replicaset "
       "member (meaning single metadata server)");
 
   set_mock_metadata(cluster_http_port, kGroupId,
-                    classic_ports_to_gr_nodes({cluster_node_port}), 0,
-                    classic_ports_to_cluster_nodes({cluster_node_port}));
+                    std::vector<uint16_t>{cluster_node_port});
 
   SCOPED_TRACE("// Create a router state file with a single metadata server");
   const std::string state_file = create_state_file(
@@ -458,15 +506,13 @@ TEST_P(StateFileMetadataServersInaccessibleTest, MetadataServersInaccessible) {
       "// Check our state file content, it should still contain out metadata "
       "server");
 
-  check_state_file(state_file, param.cluster_type, kGroupId,
-                   {cluster_node_port}, 0, "127.0.0.1", 10s);
-
-  router.send_shutdown_event();
-
-  EXPECT_EQ(0, router.wait_for_exit());
+  EXPECT_TRUE(wait_state_file_contains(
+      state_file, kGroupId,
+      {"mysql://127.0.0.1:" + std::to_string(cluster_node_port)}, 10000ms))
+      << get_file_output(state_file);
 }
 
-INSTANTIATE_TEST_SUITE_P(
+INSTANTIATE_TEST_CASE_P(
     MetadataServersInaccessible, StateFileMetadataServersInaccessibleTest,
     ::testing::Values(StateFileTestParam{"gr", "metadata_dynamic_nodes.js",
                                          ClusterType::GR_V1},
@@ -501,18 +547,20 @@ TEST_P(StateFileGroupReplicationIdDiffersTest, GroupReplicationIdDiffers) {
   auto cluster_node_port = port_pool_.get_next_available();
   auto cluster_http_port = port_pool_.get_next_available();
 
-  SCOPED_TRACE("// Launch server mock that will act as our metadata server");
+  SCOPED_TRACE("// Launch  server mock that will act as our metadata server");
   const auto trace_file = get_data_dir().join(param.trace_file).str();
-  /*auto &cluster_node =*/ProcessManager::launch_mysql_server_mock(
+  auto &cluster_node = ProcessManager::launch_mysql_server_mock(
       trace_file, cluster_node_port, EXIT_SUCCESS, false, cluster_http_port);
+  ASSERT_NO_FATAL_FAILURE(check_port_ready(cluster_node, cluster_node_port));
+  ASSERT_TRUE(
+      MockServerRestClient(cluster_http_port).wait_for_rest_endpoint_ready());
 
   SCOPED_TRACE(
-      "// Make our metadata server to return single node as a cluster "
+      "// Make our metadata server to return single node as a replicaset "
       "member (meaning single metadata server)");
 
   set_mock_metadata(cluster_http_port, kClusterFileGroupId,
-                    classic_ports_to_gr_nodes({cluster_node_port}), 0,
-                    {cluster_node_port});
+                    std::vector<uint16_t>{cluster_node_port});
 
   SCOPED_TRACE(
       "// Create a router state file with a single metadata server and "
@@ -534,20 +582,23 @@ TEST_P(StateFileGroupReplicationIdDiffersTest, GroupReplicationIdDiffers) {
 
   SCOPED_TRACE("// Launch ther router with the initial state file");
   auto &router = launch_router(temp_test_dir.name(), metadata_cache_section,
-                               routing_section, state_file, EXIT_SUCCESS, -1s);
+                               routing_section, state_file);
 
   SCOPED_TRACE(
       "// Check our state file content, it should not change. "
       "We did not found the data for our replication group on any of the "
       "servers so we do not update the metadata srever list.");
 
-  check_state_file(state_file, param.cluster_type, kStateFileGroupId,
-                   {cluster_node_port});
+  EXPECT_TRUE(wait_state_file_contains(
+      state_file, kStateFileGroupId,
+      {"mysql://127.0.0.1:" + std::to_string(cluster_node_port)},
+      std::chrono::milliseconds(10 * kTTL)))
+      << get_file_output(state_file);
 
   SCOPED_TRACE("// We expect an error in the logfile");
   EXPECT_TRUE(wait_log_file_contains(
       router, "Failed fetching metadata from any of the 1 metadata servers",
-      5s));
+      10 * kTTL));
 
   // now try to connect to the router port, we expect error 2003
   std::string out_port_unused;
@@ -555,7 +606,7 @@ TEST_P(StateFileGroupReplicationIdDiffersTest, GroupReplicationIdDiffers) {
                                                 /*should_fail=*/true));
 }
 
-INSTANTIATE_TEST_SUITE_P(
+INSTANTIATE_TEST_CASE_P(
     GroupReplicationIdDiffers, StateFileGroupReplicationIdDiffersTest,
     ::testing::Values(StateFileTestParam{"gr", "metadata_dynamic_nodes.js",
                                          ClusterType::GR_V1},
@@ -613,17 +664,13 @@ TEST_P(StateFileSplitBrainScenarioTest, SplitBrainScenario) {
                                    cluster_node_ports[1].first};
   for (unsigned i = 0; i < 1; i++) {
     const auto port_http = cluster_node_ports[i].second;
-    set_mock_metadata(port_http, kClusterGroupId,
-                      classic_ports_to_gr_nodes(fist_group), i,
-                      classic_ports_to_cluster_nodes(fist_group));
+    set_mock_metadata(port_http, kClusterGroupId, fist_group);
   }
 
   std::vector<uint16_t> second_group{cluster_node_ports[2].first};
   for (unsigned i = 2; i < kNodesNum; i++) {
     const auto port_http = cluster_node_ports[i].second;
-    set_mock_metadata(port_http, kMockServerGlobalsRestUri,
-                      classic_ports_to_gr_nodes(second_group), i - 2,
-                      classic_ports_to_cluster_nodes(second_group));
+    set_mock_metadata(port_http, kMockServerGlobalsRestUri, second_group);
   }
 
   SCOPED_TRACE(
@@ -653,11 +700,16 @@ TEST_P(StateFileSplitBrainScenarioTest, SplitBrainScenario) {
       "// Check our state file content, it should now contain only the nodes "
       "from the first group.");
 
-  std::vector<uint16_t> node_ports;
+  std::vector<std::string> expected_gr_nodes;
   for (unsigned i = 0; i < 2; ++i) {
-    node_ports.push_back(cluster_node_ports[i].first);
+    expected_gr_nodes.push_back("mysql://127.0.0.1:" +
+                                std::to_string(cluster_node_ports[i].first));
   }
-  check_state_file(state_file, param.cluster_type, kClusterGroupId, node_ports);
+
+  EXPECT_TRUE(wait_state_file_contains(state_file, kClusterGroupId,
+                                       expected_gr_nodes,
+                                       std::chrono::milliseconds(10 * kTTL)))
+      << get_file_output(state_file);
 
   SCOPED_TRACE(
       "// Try to connect to the router port, we expect first port from the "
@@ -669,7 +721,7 @@ TEST_P(StateFileSplitBrainScenarioTest, SplitBrainScenario) {
                port_connected.c_str());
 }
 
-INSTANTIATE_TEST_SUITE_P(
+INSTANTIATE_TEST_CASE_P(
     SplitBrainScenario, StateFileSplitBrainScenarioTest,
     ::testing::Values(StateFileTestParam{"gr", "metadata_dynamic_nodes.js",
                                          ClusterType::GR_V1},
@@ -680,8 +732,9 @@ INSTANTIATE_TEST_SUITE_P(
 
 /**
  * @test
- *      Verify that in case of empty metada-server-address list in the state
- * file the Router logs proper error and exits.
+ *      Verify that in case of empty metada-server-addess list in the state file
+ * the Router logs proper error and accepts (and errors out) the client
+ * connections.
  */
 TEST_F(StateFileDynamicChangesTest, EmptyMetadataServersList) {
   constexpr const char kGroupId[] = "3a0be5af-0022-11e8-9655-0800279e6a88";
@@ -704,18 +757,22 @@ TEST_F(StateFileDynamicChangesTest, EmptyMetadataServersList) {
 
   SCOPED_TRACE("// Launch ther router with the initial state file");
   auto &router = launch_router(temp_test_dir.name(), metadata_cache_section,
-                               routing_section, state_file, EXIT_FAILURE, -1s);
+                               routing_section, state_file, EXIT_FAILURE);
 
-  // wait for shutdown before checking the logfile.
-  EXPECT_EQ(router.wait_for_exit(), EXIT_FAILURE);
+  wait_for_port_ready(router_port);
 
   // proper error should get logged
   EXPECT_TRUE(wait_log_file_contains(
       router,
-      "'bootstrap_server_addresses' in the configuration file is empty "
+      "'bootstrap_server_addresses' is the configuration file is empty "
       "or not set and list of 'cluster-metadata-servers' in "
       "'dynamic_config'-file is empty, too.",
       3 * kTTL));
+
+  // now try to connect to the router port, we expect error 2003
+  std::string out_port_unused;
+  EXPECT_NO_THROW(connect_client_and_query_port(router_port, out_port_unused,
+                                                /*should_fail=*/true));
 }
 
 //////////////////////////////////////////////////////////////////////////
@@ -746,59 +803,52 @@ class StateFileSchemaTest
     : public StateFileTest,
       public ::testing::WithParamInterface<StateFileSchemaTestParams> {
  protected:
-  void SetUp() override { StateFileTest::SetUp(); }
+  virtual void SetUp() { StateFileTest::SetUp(); }
 };
 
 /**
  * @test
  *      Verify that the proper error gets logged and the Router shuts down in
- * case of various configuration mismatches.
+ * case of various configuration mimatches.
  */
 TEST_P(StateFileSchemaTest, ParametrizedStateFileSchemaTest) {
   auto test_params = GetParam();
 
   TempDirectory temp_test_dir;
 
+  const uint16_t md_server_port =
+      test_params.use_static_server_list ? port_pool_.get_next_available() : 0;
   const uint16_t router_port = port_pool_.get_next_available();
 
+  // launch the router with static metadata-cache configuration and
+  // dynamic state file configured via test parameter
+  const std::string metadata_cache_section = get_metadata_cache_section(
+      md_server_port, kTTL, test_params.cluster_type);
+  const std::string routing_section = get_metadata_cache_routing_section(
+      router_port, "PRIMARY", "first-available");
+
   const std::string state_file =
       test_params.create_state_file_from_content
           ? create_state_file(temp_test_dir.name(),
                               test_params.state_file_content)
           : test_params.state_file_path;
 
-  auto writer =
-      config_writer(temp_test_dir.name())
-          .section(metadata_cache_section(test_params.use_static_server_list
-                                              ? port_pool_.get_next_available()
-                                              : 0,
-                                          kTTL, test_params.cluster_type))
-          .section(metadata_cache_routing_section(router_port, "PRIMARY",
-                                                  "first-available"));
-
-  auto &default_section = writer.sections()["DEFAULT"];
-
-  init_keyring(default_section, temp_test_dir.name());
-
-  default_section["dynamic_state"] = state_file;
-
-  auto &router =
-      router_spawner()
-          .expected_exit_code(EXIT_FAILURE)
-          .wait_for_sync_point(ProcessManager::Spawner::SyncPoint::RUNNING)
-          .spawn({"-c", writer.write()});
+  auto &router = launch_router(temp_test_dir.name(), metadata_cache_section,
+                               routing_section, state_file, EXIT_FAILURE);
 
   // the router should close with non-0 return value
-  ASSERT_NO_FATAL_FAILURE(check_exit_code(router, EXIT_FAILURE));
+  check_exit_code(router, EXIT_FAILURE);
+  EXPECT_THAT(router.exit_code(), testing::Ne(0));
 
   // proper log should get logged
-  auto log_content = router.get_logfile_content();
+  auto log_content = router.get_full_logfile();
   for (const auto &expeted_in_log : test_params.expected_errors_in_log) {
-    EXPECT_TRUE(log_content.find(expeted_in_log) != std::string::npos);
+    EXPECT_TRUE(log_content.find(expeted_in_log) != std::string::npos)
+        << log_content << "\n";
   }
 }
 
-INSTANTIATE_TEST_SUITE_P(
+INSTANTIATE_TEST_CASE_P(
     StateFileTests, StateFileSchemaTest,
     ::testing::Values(
         // state file does not exits
@@ -906,7 +956,7 @@ INSTANTIATE_TEST_SUITE_P(
             "}",
             // clang-format on
             {"Unsupported state file version, "
-             "expected: 1.1.0, found: 2.0.0"}},
+             "expected: 1.0.0, found: 2.0.0"}},
 
         // major version does not match (AR cluster)
         StateFileSchemaTestParams{
@@ -923,13 +973,13 @@ INSTANTIATE_TEST_SUITE_P(
             "}",
             // clang-format on
             {"Unsupported state file version, "
-             "expected: 1.1.0, found: 2.0.0"}, true, "", false,
+             "expected: 1.0.0, found: 2.0.0"}, true, "", false,
              ClusterType::RS_V2},
 
         // minor version does not match
         StateFileSchemaTestParams{            // clang-format off
         "{"
-          "\"version\": \"1.2.0\","
+          "\"version\": \"1.1.0\","
           "\"metadata-cache\": {"
             "\"group-replication-id\": \"3a0be5af-994c-11e8-9655-0800279e6a88\","
             "\"cluster-metadata-servers\": ["
@@ -940,7 +990,7 @@ INSTANTIATE_TEST_SUITE_P(
         "}",
         // clang-format on
         {"Unsupported state file version, "
-         "expected: 1.1.0, found: 1.2.0"}},
+         "expected: 1.0.0, found: 1.1.0"}},
 
         // both bootstrap_server_addresses and dynamic_state configured
         StateFileSchemaTestParams{
@@ -962,23 +1012,22 @@ INSTANTIATE_TEST_SUITE_P(
         true /*use static bootstrap_server_addresses in static conf. file*/},
 
         // group-replication-id filed missing
-        // no longer required
-//        StateFileSchemaTestParams{
-//        // clang-format off
-//        "{"
-//          "\"version\": \"1.0.0\","
-//          "\"metadata-cache\": {"
-//            "\"cluster-metadata-servers\": ["
-//              "\"mysql://localhost:5000\","
-//              "\"mysql://127.0.0.1:5001\""
-//            "]"
-//          "}"
-//        "}",
-//        // clang-format on
-//        {"JSON file failed validation against JSON schema: Failed schema "
-//         "directive: #/properties/metadata-cache",
-//         "Failed schema keyword:   required",
-//         "Failure location in validated document: #/metadata-cache"}},
+        StateFileSchemaTestParams{
+        // clang-format off
+        "{"
+          "\"version\": \"1.0.0\","
+          "\"metadata-cache\": {"
+            "\"cluster-metadata-servers\": ["
+              "\"mysql://localhost:5000\","
+              "\"mysql://127.0.0.1:5001\""
+            "]"
+          "}"
+        "}",
+        // clang-format on
+        {"JSON file failed validation against JSON schema: Failed schema "
+         "directive: #/properties/metadata-cache",
+         "Failed schema keyword:   required",
+         "Failure location in validated document: #/metadata-cache"}},
 
         // cluster-metadata-servers filed missing (GR cluster)
         StateFileSchemaTestParams{
@@ -1042,7 +1091,7 @@ class StateFileAccessRightsTest
     : public StateFileTest,
       public ::testing::WithParamInterface<StateFileAccessRightsTestParams> {
  protected:
-  void SetUp() override { StateFileTest::SetUp(); }
+  virtual void SetUp() { StateFileTest::SetUp(); }
 };
 
 /**
@@ -1059,6 +1108,9 @@ TEST_P(StateFileAccessRightsTest, ParametrizedStateFileSchemaTest) {
 
   // launch the router with static metadata-cache configuration and
   // dynamic state file configured via test parameter
+  const std::string metadata_cache_section = get_metadata_cache_section();
+  const std::string routing_section = get_metadata_cache_routing_section(
+      router_port, "PRIMARY", "first-available");
 
   const std::string state_file = create_state_file(
       temp_test_dir.name(), create_state_file_content("000-000", {10000}));
@@ -1067,31 +1119,25 @@ TEST_P(StateFileAccessRightsTest, ParametrizedStateFileSchemaTest) {
   if (test_params.write_access) file_mode |= S_IWUSR;
   chmod(state_file.c_str(), file_mode);
 
-  auto writer = config_writer(temp_test_dir.name())
-                    .section(metadata_cache_section())
-                    .section(metadata_cache_routing_section(
-                        router_port, "PRIMARY", "first-available"));
-
-  auto &default_section = writer.sections()["DEFAULT"];
-
-  init_keyring(default_section, temp_test_dir.name());
-
-  default_section["dynamic_state"] = state_file;
-
-  auto &router =
-      router_spawner()
-          .expected_exit_code(EXIT_FAILURE)
-          .wait_for_sync_point(ProcessManager::Spawner::SyncPoint::NONE)
-          .spawn({"-c", writer.write()});
+  auto &router = launch_router(temp_test_dir.name(), metadata_cache_section,
+                               routing_section, state_file, EXIT_FAILURE);
 
   // the router should close with non-0 return value
-  ASSERT_NO_FATAL_FAILURE(check_exit_code(router, EXIT_FAILURE));
+  check_exit_code(router, EXIT_FAILURE);
+  EXPECT_THAT(router.exit_code(), testing::Ne(0));
 
   // proper error should get logged
-  EXPECT_TRUE(wait_log_file_contains(router, test_params.expected_error, 1ms));
+  const bool found = find_in_file(
+      get_logging_dir().str() + "/mysqlrouter.log",
+      [&](const std::string &line) -> bool {
+        return pattern_found(line, test_params.expected_error);
+      },
+      std::chrono::milliseconds(1));
+
+  EXPECT_TRUE(found);
 }
 
-INSTANTIATE_TEST_SUITE_P(
+INSTANTIATE_TEST_CASE_P(
     StateFileTests, StateFileAccessRightsTest,
     ::testing::Values(
         // no read, nor write access
@@ -1108,7 +1154,7 @@ INSTANTIATE_TEST_SUITE_P(
 ////////////////////////////////////////////
 
 class StateFileDirectoryBootstrapTest : public StateFileTest {
-  void SetUp() override { StateFileTest::SetUp(); }
+  virtual void SetUp() { StateFileTest::SetUp(); }
 };
 
 /**
@@ -1131,25 +1177,35 @@ TEST_F(StateFileDirectoryBootstrapTest, DirectoryBootstrapTest) {
   std::vector<std::string> router_cmdline{
       "--bootstrap=localhost:" + std::to_string(metadata_server_port), "-d",
       temp_test_dir.name()};
-  auto &router = launch_router_for_bootstrap(router_cmdline, EXIT_SUCCESS);
+  auto &router = ProcessManager::launch_router(router_cmdline);
+  router.register_response("Please enter MySQL password for root: ",
+                           "fake-pass\n");
 
-  ASSERT_NO_FATAL_FAILURE(check_exit_code(router, EXIT_SUCCESS));
+  check_exit_code(router, EXIT_SUCCESS, 1s);
 
-  // check the state file that was produced, if it contains
+  // check the state file that was produced, if it constains
   // what the bootstrap server has reported
   const std::string state_file = temp_test_dir.name() + "/data/state.json";
-  check_state_file(state_file, ClusterType::GR_V1, "cluster-specific-id",
-                   {5500, 5510, 5520});
+  EXPECT_TRUE(
+      check_state_file(state_file, "cluster-specific-id",
+                       {"mysql://localhost:5500", "mysql://localhost:5510",
+                        "mysql://localhost:5520"}))
+      << get_file_output(state_file);
 
   // check that static file has a proper reference to the dynamic file
-  const std::string conf_content =
-      get_file_output("mysqlrouter.conf", temp_test_dir.name());
-  const std::vector<std::string> lines =
-      mysql_harness::split_string(conf_content, '\n');
+  const std::string static_conf = temp_test_dir.name() + "/mysqlrouter.conf";
+  const std::string expected_entry =
+      std::string("dynamic_state=") + Path(state_file).real_path().str();
+  const bool found = find_in_file(
+      static_conf,
+      [&](const std::string &line) -> bool {
+        return pattern_found(line, expected_entry);
+      },
+      std::chrono::milliseconds(1));
 
-  ASSERT_THAT(lines,
-              ::testing::Contains(::testing::HasSubstr(
-                  "dynamic_state=" + Path(state_file).real_path().str())));
+  EXPECT_TRUE(found) << "Did not found: " << expected_entry << "\n"
+                     << get_file_output("mysqlrouter.conf",
+                                        temp_test_dir.name());
 }
 
 /*
@@ -1161,14 +1217,14 @@ TEST_F(StateFileDirectoryBootstrapTest, DirectoryBootstrapTest) {
 
 class StateFileSystemBootstrapTest : public StateFileTest,
                                      public RouterSystemLayout {
-  void SetUp() override {
+  virtual void SetUp() {
     StateFileTest::SetUp();
     RouterSystemLayout::init_system_layout_dir(get_mysqlrouter_exec(),
                                                ProcessManager::get_origin());
     set_mysqlrouter_exec(Path(exec_file_));
   }
 
-  void TearDown() override { RouterSystemLayout::cleanup_system_layout(); }
+  virtual void TearDown() { RouterSystemLayout::cleanup_system_layout(); }
 };
 
 /**
@@ -1188,17 +1244,22 @@ TEST_F(StateFileSystemBootstrapTest, SystemBootstrapTest) {
   SCOPED_TRACE("// Bootstrap against our metadata server");
   std::vector<std::string> router_cmdline{"--bootstrap=localhost:" +
                                           std::to_string(metadata_server_port)};
-  auto &router = launch_router_for_bootstrap(router_cmdline, EXIT_SUCCESS);
+  auto &router = ProcessManager::launch_router(router_cmdline);
+  router.register_response("Please enter MySQL password for root: ",
+                           "fake-pass\n");
 
-  ASSERT_NO_FATAL_FAILURE(check_exit_code(router, EXIT_SUCCESS));
+  check_exit_code(router, EXIT_SUCCESS, 1s);
 
-  // check the state file that was produced, if it contains
+  // check the state file that was produced, if it constains
   // what the bootstrap server has reported
   const std::string state_file =
       RouterSystemLayout::tmp_dir_ + "/stage/var/lib/mysqlrouter/state.json";
 
-  check_state_file(state_file, ClusterType::GR_V1, "cluster-specific-id",
-                   {5500, 5510, 5520});
+  EXPECT_TRUE(
+      check_state_file(state_file, "cluster-specific-id",
+                       {"mysql://localhost:5500", "mysql://localhost:5510",
+                        "mysql://localhost:5520"}))
+      << get_file_output(state_file);
 }
 
 #endif  // SKIP_BOOTSTRAP_SYSTEM_DEPLOYMENT_TESTS
