@@ -1,4 +1,4 @@
-/* Copyright (c) 2010, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -47,6 +47,8 @@
 #include <sys/types.h>
 
 #include "errmsg.h"
+#include "m_ctype.h"
+#include "m_string.h"
 #include "my_alloc.h"
 #include "my_dbug.h"
 #include "my_inttypes.h"
@@ -59,10 +61,7 @@
 #include "mysql/psi/mysql_memory.h"
 #include "mysql/psi/mysql_mutex.h"
 #include "mysql/service_mysql_alloc.h"
-#include "mysql/strings/m_ctype.h"
-#include "nulls.h"
 #include "sql_common.h"
-#include "strxnmov.h"
 #include "template_utils.h"
 
 #ifdef HAVE_DLFCN_H
@@ -73,9 +72,6 @@
 #include <mysql/plugin_trace.h>
 #endif
 
-#include <mysql/plugin_client_telemetry.h>
-struct st_mysql_client_plugin_TELEMETRY *client_telemetry_plugin;
-
 PSI_memory_key key_memory_root;
 PSI_memory_key key_memory_load_env_plugins;
 
@@ -119,7 +115,6 @@ static uint plugin_version[MYSQL_CLIENT_MAX_PLUGINS] = {
     0, /* these two are taken by Connector/C */
     MYSQL_CLIENT_AUTHENTICATION_PLUGIN_INTERFACE_VERSION,
     MYSQL_CLIENT_TRACE_PLUGIN_INTERFACE_VERSION,
-    MYSQL_CLIENT_TELEMETRY_PLUGIN_INTERFACE_VERSION,
 };
 
 /*
@@ -155,8 +150,8 @@ static int is_not_initialized(MYSQL *mysql, const char *name) {
 static struct st_mysql_client_plugin *find_plugin(const char *name, int type) {
   struct st_client_plugin_int *p;
 
-  assert(initialized);
-  assert(type >= 0 && type < MYSQL_CLIENT_MAX_PLUGINS);
+  DBUG_ASSERT(initialized);
+  DBUG_ASSERT(type >= 0 && type < MYSQL_CLIENT_MAX_PLUGINS);
   if (type < 0 || type >= MYSQL_CLIENT_MAX_PLUGINS) return nullptr;
 
   for (p = plugin_list[type]; p; p = p->next) {
@@ -184,7 +179,7 @@ static struct st_mysql_client_plugin *do_add_plugin(
   struct st_client_plugin_int plugin_int, *p;
   char errbuf[1024];
 
-  assert(initialized);
+  DBUG_ASSERT(initialized);
 
   plugin_int.plugin = plugin;
   plugin_int.dlhandle = dlhandle;
@@ -203,7 +198,7 @@ static struct st_mysql_client_plugin *do_add_plugin(
 #if defined(CLIENT_PROTOCOL_TRACING) && !defined(MYSQL_SERVER)
   /*
     If we try to load a protocol trace plugin but one is already
-    loaded (global trace_plugin pointer is not nullptr) then we ignore
+    loaded (global trace_plugin pointer is not NULL) then we ignore
     the new trace plugin and give error. This is done before the
     new plugin gets initialized.
   */
@@ -213,13 +208,6 @@ static struct st_mysql_client_plugin *do_add_plugin(
   }
 #endif
 
-  if (plugin->type == MYSQL_CLIENT_TELEMETRY_PLUGIN &&
-      nullptr != client_telemetry_plugin) {
-    errmsg =
-        "Can not load another telemetry plugin while one is already loaded";
-    goto err1;
-  }
-
   /* Call the plugin initialization function, if any */
   if (plugin->init && plugin->init(errbuf, sizeof(errbuf), argc, args)) {
     errmsg = errbuf;
@@ -243,7 +231,7 @@ static struct st_mysql_client_plugin *do_add_plugin(
 #if defined(CLIENT_PROTOCOL_TRACING) && !defined(MYSQL_SERVER)
   /*
     If loaded plugin is a protocol trace one, then set the global
-    trace_plugin pointer to point at it. When trace_plugin is not nullptr,
+    trace_plugin pointer to point at it. When trace_plugin is not NULL,
     each new connection will be traced using the plugin pointed by it
     (see MYSQL_TRACE_STAGE() macro in libmysql/mysql_trace.h).
   */
@@ -252,10 +240,6 @@ static struct st_mysql_client_plugin *do_add_plugin(
   }
 #endif
 
-  if (plugin->type == MYSQL_CLIENT_TELEMETRY_PLUGIN) {
-    client_telemetry_plugin = (struct st_mysql_client_plugin_TELEMETRY *)plugin;
-  }
-
   return plugin;
 
 err2:
@@ -313,11 +297,10 @@ static void load_env_plugins(MYSQL *mysql) {
   free_env = plugs = my_strdup(key_memory_load_env_plugins, s, MYF(MY_WME));
 
   do {
-    s = strchr(plugs, ';');
-    if (s != nullptr) *s = '\0';
+    if ((s = strchr(plugs, ';'))) *s = '\0';
     mysql_load_plugin(mysql, plugs, -1, 0);
-    if (s != nullptr) plugs = s + 1;
-  } while (s != nullptr);
+    plugs = s + 1;
+  } while (s);
 
   my_free(free_env);
 }
@@ -347,7 +330,7 @@ int mysql_client_plugin_init() {
 
   mysql_mutex_init(key_mutex_LOCK_load_client_plugin, &LOCK_load_client_plugin,
                    MY_MUTEX_INIT_SLOW);
-  ::new ((void *)&mem_root) MEM_ROOT(key_memory_root, 128);
+  init_alloc_root(key_memory_root, &mem_root, 128, 128);
 
   memset(&plugin_list, 0, sizeof(plugin_list));
 
@@ -386,7 +369,7 @@ void mysql_client_plugin_deinit() {
 
   memset(&plugin_list, 0, sizeof(plugin_list));
   initialized = false;
-  mem_root.Clear();
+  free_root(&mem_root, MYF(0));
   mysql_mutex_destroy(&LOCK_load_client_plugin);
 }
 
@@ -423,7 +406,7 @@ struct st_mysql_client_plugin *mysql_load_plugin_v(MYSQL *mysql,
   struct st_mysql_client_plugin *plugin;
   const char *plugindir;
   const CHARSET_INFO *cs = nullptr;
-  const size_t len = (name ? strlen(name) : 0);
+  size_t len = (name ? strlen(name) : 0);
   int well_formed_error;
   size_t res = 0;
 #ifdef _WIN32
@@ -484,14 +467,7 @@ struct st_mysql_client_plugin *mysql_load_plugin_v(MYSQL *mysql,
 
   DBUG_PRINT("info", ("dlopeninig %s", dlpath));
   /* Open new dll handle */
-#if defined(HAVE_ASAN) || defined(HAVE_LSAN)
-  // Do not unload the shared object during dlclose().
-  // LeakSanitizer needs this in order to match entries in lsan.supp
-  if (!(dlhandle = dlopen(dlpath, RTLD_NOW | RTLD_NODELETE)))
-#else
-  if (!(dlhandle = dlopen(dlpath, RTLD_NOW)))
-#endif
-  {
+  if (!(dlhandle = dlopen(dlpath, RTLD_NOW))) {
 #if defined(__APPLE__)
     /* Apple supports plugins with .so also, so try this as well */
     strxnmov(dlpath, sizeof(dlpath) - 1, plugindir, "/", name, ".so", NullS);
@@ -501,8 +477,8 @@ struct st_mysql_client_plugin *mysql_load_plugin_v(MYSQL *mysql,
 #ifdef _WIN32
     /* There should be no win32 calls between failed dlopen() and GetLastError()
      */
-    if (FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, nullptr, GetLastError(), 0,
-                      win_errormsg, 2048, nullptr))
+    if (FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, 0, GetLastError(), 0,
+                      win_errormsg, 2048, NULL))
       errmsg = win_errormsg;
     else
       errmsg = "";
@@ -600,12 +576,3 @@ int mysql_plugin_options(struct st_mysql_client_plugin *plugin,
   if (!plugin || !plugin->options) return 1;
   return plugin->options(option, value);
 }
-
-/* see <mysql/client_plugin.h> for a full description */
-int mysql_plugin_get_option(struct st_mysql_client_plugin *plugin,
-                            const char *option, void *value) {
-  DBUG_TRACE;
-  /* does the plugin support options call? */
-  if (!plugin || !plugin->get_options) return 1;
-  return plugin->get_options(option, value);
-}
