@@ -1,16 +1,15 @@
-/* Copyright (c) 2014, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,14 +29,13 @@
 
 #include "sql/aggregate_check.h"
 
-#include <assert.h>
-#include <cstdio>
-#include <initializer_list>
+#include "my_config.h"
+
+#include <stdio.h>
 #include <utility>
 
-#include "mem_root_deque.h"
 #include "my_base.h"
-
+#include "my_dbug.h"
 #include "my_sys.h"
 #include "mysqld_error.h"
 #include "sql/derror.h"
@@ -48,11 +46,10 @@
 #include "sql/nested_join.h"
 #include "sql/opt_trace.h"
 #include "sql/opt_trace_context.h"
-#include "sql/parser_yystype.h"
+#include "sql/parse_tree_nodes.h"
 #include "sql/sql_base.h"
 #include "sql/sql_class.h"
 #include "sql/sql_const.h"
-#include "sql/sql_executor.h"
 #include "sql/sql_lex.h"
 #include "sql/sql_list.h"
 #include "sql/table.h"
@@ -66,8 +63,8 @@
   used_tables()
 
   - When we are looking for items to validate, we must enter scalar/row
-  subqueries; if we find an item of our Query_block inside such subquery, for
-  example an Item_field with depended_from equal to our Query_block, we
+  subqueries; if we find an item of our SELECT_LEX inside such subquery, for
+  example an Item_field with depended_from equal to our SELECT_LEX, we
   must use used_tables_for_level(). Example: when validating t1.a in
   select (select t1.a from t1 as t2 limit 1) from t1 group by t1.pk;
   we need t1.a's map in the grouped query; used_tables() would return
@@ -114,10 +111,9 @@ bool Distinct_check::check_query(THD *thd) {
        ++number_in_list, order = order->next) {
     if (order->in_field_list)  // is in SELECT list
       continue;
-    assert((*order->item)->fixed);
+    DBUG_ASSERT((*order->item)->fixed);
     uint counter;
     enum_resolution_type resolution;
-    Item **res;
     /*
       Search if this expression is equal to one in the SELECT
       list. setup_order()/find_order_in_list() has already done so, but not
@@ -147,11 +143,12 @@ bool Distinct_check::check_query(THD *thd) {
       differ due to white space....).
       Subqueries in ORDER BY are non-standard anyway.
     */
-    if (find_item_in_list(thd, *order->item, &select->fields, &res, &counter,
-                          &resolution)) {
-      return true; /* purecov: inspected */
-    }
-    if (res != nullptr)  // is in SELECT list
+    Item **const res =
+        find_item_in_list(thd, *order->item, select->item_list, &counter,
+                          REPORT_EXCEPT_NOT_FOUND, &resolution);
+    if (res == nullptr)  // Other error than "not found", my_error() was called
+      return true;       /* purecov: inspected */
+    if (res != not_found_item)  // is in SELECT list
       continue;
     /*
       [numbers refer to the function's comment]
@@ -184,13 +181,13 @@ bool Group_check::check_query(THD *thd) {
   uint number_in_list = 1;
   const char *place = "SELECT list";
 
-  for (Item *sel_expr : select->visible_fields()) {
-    if (check_expression(thd, sel_expr, true)) goto err;
+  for (Item &sel_expr : select->item_list) {
+    if (check_expression(thd, &sel_expr, true)) goto err;
     ++number_in_list;
   }
 
   // Aggregate without GROUP BY has no ORDER BY at this stage
-  assert(!(select->is_implicitly_grouped() && select->is_ordered()));
+  DBUG_ASSERT(!(select->is_implicitly_grouped() && select->is_ordered()));
   // Validate ORDER BY list
   if (order) {
     number_in_list = 1;
@@ -210,13 +207,6 @@ bool Group_check::check_query(THD *thd) {
     if (check_expression(thd, select->having_cond(), false)) goto err;
   }
 
-  // Validate QUALIFY condition
-  if (select->qualify_cond()) {
-    number_in_list = 1;
-    place = "QUALIFY clause";
-    if (check_expression(thd, select->qualify_cond(), false)) goto err;
-  }
-
   // Validate windows' ORDER BY and PARTITION BY clauses.
   char buff[STRING_BUFFER_USUAL_SIZE];
   {
@@ -273,23 +263,21 @@ err:
    list.
 */
 bool Group_check::check_expression(THD *thd, Item *expr, bool in_select_list) {
-  assert(!is_child());
+  DBUG_ASSERT(!is_child());
   if (!in_select_list) {
     uint counter;
     enum_resolution_type resolution;
-    Item **res;
-    // Check if this expression is equal to one in the SELECT list.
-    if (find_item_in_list(thd, expr, &select->fields, &res, &counter,
-                          &resolution)) {
-      return true; /* purecov: inspected */
-    }
-    if (res != nullptr) {  // in SELECT list, which has already been validated.
+    // Search if this expression is equal to one in the SELECT list.
+    Item **const res = find_item_in_list(thd, expr, select->item_list, &counter,
+                                         REPORT_EXCEPT_NOT_FOUND, &resolution);
+    if (res == nullptr)  // Other error than "not found", my_error() was called
+      return true;       /* purecov: inspected */
+    if (res != not_found_item) {
+      // is in SELECT list, which has already been validated.
       return false;
     }
   }
 
-  expr = unwrap_rollup_group(expr);
-
   for (ORDER *grp = select->group_list.first; grp; grp = grp->next) {
     if ((*grp->item)->eq(expr, false))
       return false;  // Expression is in GROUP BY so is ok
@@ -370,7 +358,7 @@ bool Group_check::is_fd_on_source(Item *item) {
     }
   }
 
-  if (select->is_non_primitive_grouped()) {
+  if (select->olap != UNSPECIFIED_OLAP_TYPE) {
     /*
       - the syntactical transformation of ROLLUP is to make a union of
       queries, and in each such query, some group column references are
@@ -402,7 +390,7 @@ bool Group_check::is_fd_on_source(Item *item) {
         @todo make table_ref non-NULL for gcols, then use it for 'tl'.
         Do the same in Item_field::used_tables_for_level().
       */
-      Table_ref *const tl = item_field->field->table->pos_in_table_list;
+      TABLE_LIST *const tl = item_field->field->table->pos_in_table_list;
 
       if (tested_map_for_keys & tl->map()) continue;
       tested_map_for_keys |= tl->map();
@@ -449,7 +437,7 @@ bool Group_check::is_fd_on_source(Item *item) {
     // Build En+2
     uint last_fd = fd.size();
 
-    find_fd_in_joined_table(select->m_current_table_nest);  // [OUTER] JOIN ON
+    find_fd_in_joined_table(select->join_list);  // [OUTER] JOIN ON
 
     if (select->where_cond())  // WHERE
       find_fd_in_cond(select->where_cond(), 0, false);
@@ -542,7 +530,7 @@ void Group_check::add_to_fd(Item *item, bool local_column,
   item = item->real_item();  // for merged view containing mat table
   if (item->type() == Item::FIELD_ITEM) {
     Item_field *const item_field = (Item_field *)item;
-    Table_ref *const tl = item_field->field->table->pos_in_table_list;
+    TABLE_LIST *const tl = item_field->field->table->pos_in_table_list;
     if (tl->uses_materialization() &&  // materialized table
         !tl->is_table_function())      // there's no underlying query expr
       add_to_source_of_mat_table(item_field, tl);
@@ -561,7 +549,7 @@ void Group_check::find_group_in_fd(Item *item) {
   if (select->is_grouped()) {
     /*
       See if we now have all of query expression's GROUP BY list; an
-      implicitly grouped query has an empty group list.
+      implicitely grouped query has an empty group list.
     */
     bool missing = false;
     int j = 0;
@@ -590,15 +578,11 @@ void Group_check::find_group_in_fd(Item *item) {
    @returns the idx-th expression in the SELECT list of our query block.
 */
 Item *Group_check::select_expression(uint idx) {
-  for (Item *item : select->visible_fields()) {
-    if (idx == 0) {
-      return item;
-    } else {
-      --idx;
-    }
-  }
-  assert(false);
-  return nullptr;
+  List_iterator<Item> it_select_list_of_subq(*select->get_item_list());
+  Item *expr_under = nullptr;
+  for (uint k = 0; k <= idx; k++) expr_under = it_select_list_of_subq++;
+  DBUG_ASSERT(expr_under);
+  return expr_under;
 }
 
 /**
@@ -618,33 +602,30 @@ Item *Group_check::select_expression(uint idx) {
    @param  tl          mat table
 */
 void Group_check::add_to_source_of_mat_table(Item_field *item_field,
-                                             Table_ref *tl) {
-  Query_expression *const mat_query_expression = tl->derived_query_expression();
+                                             TABLE_LIST *tl) {
+  SELECT_LEX_UNIT *const mat_unit = tl->derived_unit();
   // Query expression underlying 'tl':
-  Query_block *const mat_query_block =
-      mat_query_expression->first_query_block();
-  if (mat_query_expression->is_set_operation() ||
-      mat_query_block->is_non_primitive_grouped())
-    return;  // If UNION, EXCEPT, INTERSECT or ROLLUP, no FD
+  SELECT_LEX *const mat_select = mat_unit->first_select();
+  if (mat_unit->is_union() || mat_select->olap != UNSPECIFIED_OLAP_TYPE)
+    return;  // If UNION or ROLLUP, no FD
   // Grab Group_check for this subquery.
   Group_check *mat_gc = nullptr;
   uint j;
   for (j = 0; j < mat_tables.size(); j++) {
     mat_gc = mat_tables.at(j);
-    if (mat_gc->select == mat_query_block) break;
+    if (mat_gc->select == mat_select) break;
   }
   if (j == mat_tables.size())  // not found, create it
   {
-    mat_gc = new (m_root) Group_check(mat_query_block, m_root, tl);
+    mat_gc = new (m_root) Group_check(mat_select, m_root, tl);
     mat_tables.push_back(mat_gc);
   }
-  // Find underlying expression of item_field, in SELECT list of mat_query_block
+  // Find underlying expression of item_field, in SELECT list of mat_select
   Item *const expr_under =
-      mat_gc->select_expression(item_field->field->field_index());
+      mat_gc->select_expression(item_field->field->field_index);
 
   // non-nullability of tl's column in tl, is equal to that of expr_under.
-  if (expr_under && !expr_under->is_nullable())
-    mat_gc->non_null_in_source = true;
+  if (expr_under && !expr_under->maybe_null) mat_gc->non_null_in_source = true;
 
   mat_gc->add_to_fd(expr_under, mat_gc->local_column(expr_under));
 
@@ -653,12 +634,12 @@ void Group_check::add_to_source_of_mat_table(Item_field *item_field,
        mat_gc->non_null_in_source))                     // (3)
   {
     /*
-      (1): In mat_gc, all GROUP BY expressions of mat_query_block are dependent
-      on source columns. Thus, all SELECT list expressions are, too (otherwise,
-      the validation of mat_query_block has or will fail). So, in our
-      Group_check, intersect(En, tl.*) -> tl.* . This FD needs to propagate in
-      our Group_check all the way up to the result of the WHERE clause. It does,
-      if:
+      (1): In mat_gc, all GROUP BY expressions of mat_select are dependent on
+      source columns. Thus, all SELECT list expressions are, too (otherwise,
+      the validation of mat_select has or will fail). So, in our Group_check,
+      intersect(En, tl.*) -> tl.* .
+      This FD needs to propagate in our Group_check all the way up to the
+      result of the WHERE clause. It does, if:
       - either there is no weak side above this table (2) (so NFFD is not
       needed).
       - or intersect(En, tl.*) contains a non-nullable column (3) (then
@@ -693,7 +674,7 @@ bool Group_check::is_in_fd(Item *item) {
     return group_in_fd == ~0ULL;
   }
 
-  assert(local_column(item));
+  DBUG_ASSERT(local_column(item));
   Used_tables ut(select);
   (void)item->walk(&Item::used_tables_for_level, enum_walk::POSTFIX,
                    pointer_cast<uchar *>(&ut));
@@ -752,16 +733,16 @@ bool Group_check::is_in_fd_of_underlying(Item_ident *item) {
       we have this->fd={v1.a}, and we search if v1.b is FD on v1.a. We'll look
       if t1.a*2 is FD on t1.a.
     */
-    assert(static_cast<const Item_ref *>(item)->ref_type() ==
-           Item_ref::VIEW_REF);
+    DBUG_ASSERT(static_cast<const Item_ref *>(item)->ref_type() ==
+                Item_ref::VIEW_REF);
     /*
-      Refuse non-deterministic expressions because:
+      Refuse RAND_TABLE_BIT because:
       - FDs in a view are those of the underlying query expression.
       - For FDs in a query expression, expressions in the SELECT list must be
       deterministic.
       Same is true for materialized tables further down.
     */
-    if (item->is_non_deterministic()) return false;
+    if (item->used_tables() & RAND_TABLE_BIT) return false;
 
     Item *const real_it = item->real_item();
     Used_tables ut(select);
@@ -771,8 +752,8 @@ bool Group_check::is_in_fd_of_underlying(Item_ident *item) {
       todo When we eliminate all uses of cached_table, we can probably add a
       derived_table_ref field to Item_view_ref objects and use it here.
     */
-    Table_ref *const tl = item->cached_table;
-    assert(tl->is_view_or_derived());
+    TABLE_LIST *const tl = item->cached_table;
+    DBUG_ASSERT(tl->is_view_or_derived());
     /*
       We might find expression-based FDs in the result of the view's query
       expression; but if this view is on the weak side of an outer join,
@@ -804,12 +785,12 @@ bool Group_check::is_in_fd_of_underlying(Item_ident *item) {
       @todo make table_ref non-NULL for gcols, then use it for 'tl'.
       Do the same in Item_field::used_tables_for_level().
     */
-    Table_ref *const tl = item_field->field->table->pos_in_table_list;
+    TABLE_LIST *const tl = item_field->field->table->pos_in_table_list;
     if (item_field->field->is_gcol())  // Generated column
     {
-      assert(!tl->uses_materialization());
+      DBUG_ASSERT(!tl->uses_materialization());
       Item *const expr = item_field->field->gcol_info->expr_item;
-      assert(expr->fixed);
+      DBUG_ASSERT(expr->fixed);
       Used_tables ut(select);
       item_field->used_tables_for_level(pointer_cast<uchar *>(&ut));
       const bool weak_side_upwards = tl->is_inner_table_of_outer_join();
@@ -821,11 +802,10 @@ bool Group_check::is_in_fd_of_underlying(Item_ident *item) {
       }
     } else if (tl->uses_materialization() &&  // Materialized derived table
                !tl->is_table_function()) {
-      Query_block *const mat_query_block =
-          tl->derived_query_expression()->first_query_block();
+      SELECT_LEX *const mat_select = tl->derived_unit()->first_select();
       uint j;
       for (j = 0; j < mat_tables.size(); j++) {
-        if (mat_tables.at(j)->select == mat_query_block) break;
+        if (mat_tables.at(j)->select == mat_select) break;
       }
       if (j < mat_tables.size())  // if false, we know nothing about this table
       {
@@ -836,7 +816,7 @@ bool Group_check::is_in_fd_of_underlying(Item_ident *item) {
           Search if the expression inside 'item' is FD on them.
         */
         Item *const expr_under =
-            mat_gc->select_expression(item_field->field->field_index());
+            mat_gc->select_expression(item_field->field->field_index);
         /*
           expr_under is the expression underlying 'item'.
           (1) and (4) it is a deterministic expression of mat_gc source
@@ -849,7 +829,7 @@ bool Group_check::is_in_fd_of_underlying(Item_ident *item) {
           - or intersect(En, tl.*) contains a non-nullable column (3) (then
           the FD is NFFD).
         */
-        if (!expr_under->is_non_deterministic() &&            // (1)
+        if (!(expr_under->used_tables() & RAND_TABLE_BIT) &&  // (1)
             (!(mat_gc->table->map() & select->outer_join) ||  // (2)
              mat_gc->non_null_in_source) &&                   // (3)
             !expr_under->walk(&Item::aggregate_check_group,   // (4)
@@ -938,11 +918,10 @@ void Group_check::analyze_conjunct(Item *cond, Item *conjunct,
     conjunct = cnj->arguments()[0];
     if (conjunct->type() != Item::FUNC_ITEM) return;
     cnj = static_cast<const Item_func *>(conjunct);  // Dive in NOT's argument.
-
     /*
       We intentionally keep not_null_tables of the NOT, as we're interested in
       what makes the NOT not true, not what makes NOT's argument not true.
-     */
+    */
   }
   for (Item **parg = cnj->arguments(),
             **parg_end = parg + cnj->argument_count();
@@ -1031,7 +1010,7 @@ void Group_check::analyze_scalar_eq(Item *cond, Item *left_item,
 
   if ((left_is_column && (strong_tables & left_tables) &&
        is_in_fd(left_item)) ||
-      left_item->const_item() || left_item->is_outer_reference()) {
+      left_item->const_item() || (OUTER_REF_TABLE_BIT & left_tables)) {
     // strong_or_literal_or_outer_ref= right_item
 
     if (!weak_tables) {
@@ -1041,11 +1020,11 @@ void Group_check::analyze_scalar_eq(Item *cond, Item *left_item,
         This may be constant=right_item and thus not be an NFFD, but WHERE is
         exterior to join nests so propagation is not needed.
       */
-      assert(!weak_side_upwards);  // cannot be inner join
+      DBUG_ASSERT(!weak_side_upwards);  // cannot be inner join
       add_to_fd(right_item, true);
     } else {
       // Outer join. So condition must be deterministic.
-      if (cond->is_non_deterministic()) return;
+      if (cond->used_tables() & RAND_TABLE_BIT) return;
 
       /*
         FD will have DJS as source columns, where DJS is the set of strong
@@ -1109,8 +1088,8 @@ void Group_check::find_fd_in_cond(Item *cond, table_map weak_tables,
    @param  join_list  members of the join nest
 */
 void Group_check::find_fd_in_joined_table(
-    mem_root_deque<Table_ref *> *join_list) {
-  for (const Table_ref *table : *join_list) {
+    mem_root_deque<TABLE_LIST *> *join_list) {
+  for (const TABLE_LIST *table : *join_list) {
     if (table->is_sj_or_aj_nest()) {
       /*
         We can ignore this nest as:
@@ -1130,7 +1109,7 @@ void Group_check::find_fd_in_joined_table(
     table_map used_tables;
     NESTED_JOIN *nested_join = table->nested_join;
     if (nested_join) {
-      find_fd_in_joined_table(&nested_join->m_tables);
+      find_fd_in_joined_table(&nested_join->join_list);
       used_tables = nested_join->used_tables;
     } else
       used_tables = table->map();
@@ -1140,7 +1119,7 @@ void Group_check::find_fd_in_joined_table(
         outer join nest or to WHERE. So this assertion should hold.
         Thus, used_tables are weak tables.
       */
-      assert(table->outer_join);
+      DBUG_ASSERT(table->outer_join);
       /*
         We might find equality-based FDs in the result of this outer join; but
         if this outer join is itself on the weak side of a parent outer join,
@@ -1158,7 +1137,7 @@ void Group_check::to_opt_trace(THD *thd) {
   if (fd.empty() && !whole_tables_fd) return;
   Opt_trace_context *ctx = &thd->opt_trace;
   if (!ctx->is_started()) return;
-  const Opt_trace_object trace_wrapper(ctx);
+  Opt_trace_object trace_wrapper(ctx);
   Opt_trace_object trace_fds(ctx, "functional_dependencies_of_GROUP_columns");
   to_opt_trace2(ctx, &trace_fds);
 }
@@ -1184,7 +1163,7 @@ void Group_check::to_opt_trace2(Opt_trace_context *ctx,
       parent->add("all_group_expressions", true);
   }
   if (!mat_tables.empty()) {
-    const Opt_trace_array array(ctx, "searched_in_materialized_tables");
+    Opt_trace_array array(ctx, "searched_in_materialized_tables");
     for (uint j = 0; j < mat_tables.size(); j++) {
       Opt_trace_object trace_wrapper(ctx);
       mat_tables.at(j)->to_opt_trace2(ctx, &trace_wrapper);
@@ -1216,10 +1195,8 @@ bool Group_check::do_ident_check(Item_ident *i, table_map tm,
   switch (type) {
     case CHECK_GROUP:
       if (i->type() == Item::FIELD_ITEM &&
-          down_cast<Item_field *>(i)->table_ref->m_was_scalar_subquery) {
-        // The table has exactly one row, thus this column is FD
+          down_cast<Item_field *>(i)->table_ref->m_was_scalar_subquery)
         return false;
-      }
       if (!is_fd_on_source(i)) {
         // It is not FD on source columns:
         if (!is_child()) failed_ident = i;
