@@ -1,16 +1,15 @@
-/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -35,7 +34,6 @@
 #include "sql/auth/auth_common.h"
 #include "sql/auth/dynamic_privilege_table.h"
 #include "sql/auth/partitioned_rwlock.h"
-#include "sql/auth/sql_mfa.h" /* I_multi_factor_auth */
 #include "sql/auth/user_table.h"
 #include "sql/sql_audit.h"
 #include "sql/table.h"
@@ -140,8 +138,7 @@ void acl_update_user(const char *user, const char *host, enum SSL_type ssl_type,
                      const MYSQL_TIME &password_change_time,
                      const LEX_ALTER &password_life, Restrictions &restrictions,
                      acl_table::Pod_user_what_to_update &what_to_update,
-                     uint failed_login_attempts, int password_lock_time,
-                     const I_multi_factor_auth *mfa);
+                     uint failed_login_attempts, int password_lock_time);
 void acl_users_add_one(const char *user, const char *host,
                        enum SSL_type ssl_type, const char *ssl_cipher,
                        const char *x509_issuer, const char *x509_subject,
@@ -151,8 +148,7 @@ void acl_users_add_one(const char *user, const char *host,
                        const MYSQL_TIME &password_change_time,
                        const LEX_ALTER &password_life, bool add_role_vertex,
                        Restrictions &restrictions, uint failed_login_attempts,
-                       int password_lock_time, const I_multi_factor_auth *mfa,
-                       THD *thd [[maybe_unused]]);
+                       int password_lock_time, THD *thd MY_ATTRIBUTE((unused)));
 void acl_insert_user(THD *thd, const char *user, const char *host,
                      enum SSL_type ssl_type, const char *ssl_cipher,
                      const char *x509_issuer, const char *x509_subject,
@@ -160,8 +156,7 @@ void acl_insert_user(THD *thd, const char *user, const char *host,
                      const LEX_CSTRING &plugin, const LEX_CSTRING &auth,
                      const MYSQL_TIME &password_change_time,
                      const LEX_ALTER &password_life, Restrictions &restrictions,
-                     uint failed_login_attempts, int password_lock_time,
-                     const I_multi_factor_auth *mfa);
+                     uint failed_login_attempts, int password_lock_time);
 void acl_update_proxy_user(ACL_PROXY_USER *new_value, bool is_revoke);
 void acl_update_db(const char *user, const char *host, const char *db,
                    ulong privileges);
@@ -178,13 +173,11 @@ bool acl_reload(THD *thd, bool mdl_locked);
 bool grant_reload(THD *thd, bool mdl_locked);
 void clean_user_cache();
 bool set_user_salt(ACL_USER *acl_user);
-void append_auth_id(const THD *thd, ACL_USER *acl_user, String *str);
 
 /* sql_user_table */
 ulong get_access(TABLE *form, uint fieldnr, uint *next_field);
 int replace_db_table(THD *thd, TABLE *table, const char *db,
-                     const LEX_USER &combo, ulong rights, bool revoke_grant,
-                     bool all_current_privileges);
+                     const LEX_USER &combo, ulong rights, bool revoke_grant);
 int replace_proxies_priv_table(THD *thd, TABLE *table, const LEX_USER *user,
                                const LEX_USER *proxied_user,
                                bool with_grant_arg, bool revoke_grant);
@@ -197,16 +190,18 @@ int replace_table_table(THD *thd, GRANT_TABLE *grant_table,
                             *deleted_grant_table,
                         TABLE *table, const LEX_USER &combo, const char *db,
                         const char *table_name, ulong rights, ulong col_rights,
-                        bool revoke_grant, bool all_current_privileges);
+                        bool revoke_grant);
 int replace_routine_table(THD *thd, GRANT_NAME *grant_name, TABLE *table,
                           const LEX_USER &combo, const char *db,
                           const char *routine_name, bool is_proc, ulong rights,
-                          bool revoke_grant, bool all_current_privileges);
-int open_grant_tables(THD *thd, Table_ref *tables, bool *transactional_tables);
-void acl_tables_setup_for_read(Table_ref *tables);
+                          bool revoke_grant);
+int open_grant_tables(THD *thd, TABLE_LIST *tables, bool *transactional_tables);
+void grant_tables_setup_for_open(
+    TABLE_LIST *tables, thr_lock_type lock_type = TL_WRITE,
+    enum_mdl_type mdl_type = MDL_SHARED_NO_READ_WRITE);
 
 void acl_print_ha_error(int handler_error);
-bool check_engine_type_for_acl_table(Table_ref *tables, bool report_error);
+bool check_engine_type_for_acl_table(TABLE_LIST *tables, bool report_error);
 bool log_and_commit_acl_ddl(THD *thd, bool transactional_tables,
                             std::set<LEX_USER *> *extra_users = nullptr,
                             Rewrite_params *rewrite_params = nullptr,
@@ -257,7 +252,7 @@ void revoke_dynamic_privileges_from_auth_id(
     const Role_id &id, const std::vector<std::string> &priv_list);
 bool operator==(const Role_id &a, const Auth_id_ref &b);
 bool operator==(const Auth_id_ref &a, const Role_id &b);
-bool operator==(const std::pair<const Role_id, Role_id> &a,
+bool operator==(const std::pair<const Role_id, const Role_id> &a,
                 const Auth_id_ref &b);
 bool operator==(const Role_id &a, const Role_id &b);
 bool operator==(std::pair<const Role_id, std::pair<std::string, bool>> &a,
@@ -273,7 +268,8 @@ struct role_id_hash {
   }
 };
 
-typedef std::unordered_multimap<Role_id, Role_id, role_id_hash> Default_roles;
+typedef std::unordered_multimap<const Role_id, const Role_id, role_id_hash>
+    Default_roles;
 typedef std::map<std::string, bool> Dynamic_privileges;
 
 void get_privilege_access_maps(
@@ -305,16 +301,15 @@ bool roles_rename_authid(THD *thd, TABLE *edge_table, TABLE *defaults_table,
                          LEX_USER *user_from, LEX_USER *user_to);
 bool set_and_validate_user_attributes(
     THD *thd, LEX_USER *Str, acl_table::Pod_user_what_to_update &what_to_set,
-    bool is_privileged_user, bool is_role, Table_ref *history_table,
-    bool *history_check_done, const char *cmd, Userhostpassword_list &,
-    I_multi_factor_auth **mfa = nullptr, bool if_not_exists = false);
+    bool is_privileged_user, bool is_role, TABLE_LIST *history_table,
+    bool *history_check_done, const char *cmd, Userhostpassword_list &);
 typedef std::pair<std::string, bool> Grant_privilege;
-typedef std::unordered_multimap<Role_id, Grant_privilege, role_id_hash>
+typedef std::unordered_multimap<const Role_id, Grant_privilege, role_id_hash>
     User_to_dynamic_privileges_map;
 User_to_dynamic_privileges_map *get_dynamic_privileges_map();
 User_to_dynamic_privileges_map *swap_dynamic_privileges_map(
     User_to_dynamic_privileges_map *map);
-bool populate_roles_caches(THD *thd, Table_ref *tablelst);
+bool populate_roles_caches(THD *thd, TABLE_LIST *tablelst);
 void grant_role(ACL_USER *role, const ACL_USER *user, bool with_admin_opt);
 void get_mandatory_roles(std::vector<Role_id> *mandatory_roles);
 extern std::vector<Role_id> *g_mandatory_roles;
@@ -337,13 +332,4 @@ bool alter_user_set_default_roles_all(THD *thd, TABLE *def_role_table,
 */
 bool check_system_user_privilege(THD *thd, List<LEX_USER> list);
 
-bool read_user_application_user_metadata_from_table(LEX_CSTRING user,
-                                                    LEX_CSTRING host,
-                                                    String *metadata_str,
-                                                    TABLE *table,
-                                                    bool mode_no_backslash);
-bool is_expected_or_transient_error(THD *thd);
-bool report_missing_user_grant_message(THD *thd, bool user_exists,
-                                       const char *user, const char *host,
-                                       const char *object_name, int err_code);
 #endif /* AUTH_INTERNAL_INCLUDED */
