@@ -1,16 +1,15 @@
-/* Copyright (c) 2017, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
 as published by the Free Software Foundation.
 
-This program is designed to work with certain software (including
+This program is also distributed with certain software (including
 but not limited to OpenSSL) that is licensed under separate terms,
 as designated in a particular file or component or in included license
 documentation.  The authors of MySQL hereby grant you an additional
 permission to link the program and your derivative works with the
-separately licensed software that they have either included with
-the program or referenced in the documentation.
+separately licensed software that they have included with MySQL.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,6 +26,7 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 #include <string>
 
 #include "lex_string.h"
+#include "m_ctype.h"
 #include "my_alloc.h"
 #include "my_base.h"
 #include "my_dbug.h"
@@ -34,13 +34,12 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 #include "my_sys.h"
 #include "mysql/components/my_service.h"
 #include "mysql/components/service.h"
-#include "mysql/components/services/bits/psi_bits.h"
 #include "mysql/components/services/dynamic_privilege.h"
 #include "mysql/components/services/log_shared.h"
 #include "mysql/components/services/registry.h"
 #include "mysql/mysql_lex_string.h"
+#include "mysql/psi/psi_base.h"
 #include "mysql/service_plugin_registry.h"
-#include "mysql/strings/m_ctype.h"
 #include "mysqld_error.h"
 #include "sql/auth/auth_common.h"
 #include "sql/auth/auth_internal.h"
@@ -49,11 +48,10 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 #include "sql/current_thd.h"
 #include "sql/field.h"
 #include "sql/handler.h"
-#include "sql/iterators/row_iterator.h"
 #include "sql/key.h"
-#include "sql/mysqld.h"  // srv_registry
+#include "sql/records.h"
+#include "sql/row_iterator.h"
 #include "sql/sql_const.h"
-#include "sql/sql_executor.h"
 #include "sql/table.h"
 
 class THD;
@@ -64,7 +62,6 @@ class THD;
 #define MYSQL_DYNAMIC_PRIV_FIELD_GRANT 3
 
 Dynamic_privilege_register g_dynamic_privilege_register;
-Dynamic_privilege_register g_dynamic_privilege_deprecations;
 
 /**
   This function returns a pointer to a global variable allocated on the heap.
@@ -75,25 +72,6 @@ Dynamic_privilege_register *get_dynamic_privilege_register(void) {
   return &g_dynamic_privilege_register;
 }
 
-bool is_dynamic_privilege_defined(const std::string &str) {
-  return g_dynamic_privilege_register.find(str) !=
-         g_dynamic_privilege_register.end();
-}
-
-/**
-  This function returns a pointer to a global variable allocated on the heap.
-  @return A pointer to the dynamic privilege deprecation register.
-*/
-
-Dynamic_privilege_register *get_dynamic_privilege_deprecations(void) {
-  return &g_dynamic_privilege_deprecations;
-}
-
-bool is_dynamic_privilege_deprecated(const std::string &str) {
-  return g_dynamic_privilege_deprecations.find(str) !=
-         g_dynamic_privilege_deprecations.end();
-}
-
 /**
   Given an open table handler this function refresh the list of dynamic
   privilege grants by reading the dynamic_privilege table.
@@ -107,10 +85,10 @@ bool is_dynamic_privilege_deprecated(const std::string &str) {
     @retval false Success
 */
 
-bool populate_dynamic_privilege_caches(THD *thd, Table_ref *tablelst) {
+bool populate_dynamic_privilege_caches(THD *thd, TABLE_LIST *tablelst) {
   DBUG_TRACE;
   bool error = false;
-  assert(assert_acl_cache_write_lock(thd));
+  DBUG_ASSERT(assert_acl_cache_write_lock(thd));
   Acl_table_intact table_intact(thd);
 
   if (table_intact.check(tablelst[0].table, ACL_TABLES::TABLE_DYNAMIC_PRIV))
@@ -119,27 +97,29 @@ bool populate_dynamic_privilege_caches(THD *thd, Table_ref *tablelst) {
   TABLE *table = tablelst[0].table;
   table->use_all_columns();
   unique_ptr_destroy_only<RowIterator> iterator =
-      init_table_iterator(thd, table, /*ignore_not_found_rows=*/false,
-                          /*count_examined_rows=*/false);
+      init_table_iterator(thd, table, nullptr, false,
+                          /*ignore_not_found_rows=*/false);
   if (iterator == nullptr) {
     my_error(ER_TABLE_CORRUPT, MYF(0), table->s->db.str,
              table->s->table_name.str);
     return true;
   }
   int read_rec_errcode;
-  MEM_ROOT tmp_mem(PSI_NOT_INSTRUMENTED, 256);
+  MEM_ROOT tmp_mem;
   char percentile_character[2] = {'%', '\0'};
   char empty_str = '\0';
   /*
     We need the the dynamic privilege register in order to register any unknown
     privilege identifiers.
   */
+  SERVICE_TYPE(registry) *r = mysql_plugin_registry_acquire();
   {
     my_service<SERVICE_TYPE(dynamic_privilege_register)> service(
-        "dynamic_privilege_register.mysql_server", srv_registry);
+        "dynamic_privilege_register.mysql_server", r);
     if (!service.is_valid()) {
       return true;
     }
+    init_alloc_root(PSI_NOT_INSTRUMENTED, &tmp_mem, 256, 0);
     while (!error && !(read_rec_errcode = iterator->Read())) {
       char *host =
           get_field(&tmp_mem, table->field[MYSQL_DYNAMIC_PRIV_FIELD_HOST]);
@@ -151,19 +131,12 @@ bool populate_dynamic_privilege_caches(THD *thd, Table_ref *tablelst) {
           get_field(&tmp_mem, table->field[MYSQL_DYNAMIC_PRIV_FIELD_PRIV]);
       char *with_grant_option = get_field(
           &tmp_mem, table->field[MYSQL_DYNAMIC_PRIV_FIELD_WITH_GRANT_OPTION]);
-      if (priv == nullptr) {
-        LogErr(WARNING_LEVEL, ER_EMPTY_PRIVILEGE_NAME_IGNORED);
-        continue;  // skip invalid privilege
-      }
-
+      if (priv == nullptr) priv = &empty_str;
       my_caseup_str(system_charset_info, priv);
       LEX_CSTRING str_priv = {priv, strlen(priv)};
-      const LEX_CSTRING str_user = {user, strlen(user)};
-      const LEX_CSTRING str_host = {host, strlen(host)};
+      LEX_CSTRING str_user = {user, strlen(user)};
+      LEX_CSTRING str_host = {host, strlen(host)};
       Update_dynamic_privilege_table no_update;
-      if (is_dynamic_privilege_deprecated(priv))
-        LogErr(WARNING_LEVEL, ER_WARN_DEPRECATED_DYNAMIC_PRIV_FOR_USER, priv,
-               user ? user : "", host ? host : "");
       if (grant_dynamic_privilege(str_priv, str_user, str_host,
                                   (*with_grant_option == 'Y' ? true : false),
                                   no_update)) {
@@ -194,6 +167,7 @@ bool populate_dynamic_privilege_caches(THD *thd, Table_ref *tablelst) {
     */
     get_global_acl_cache()->increase_version();
   }  // exit scope
+  mysql_plugin_registry_release(r);
   return error;
 }
 
@@ -275,19 +249,3 @@ bool iterate_all_dynamic_privileges(THD *thd,
   }
   return false;
 }
-
-bool iterate_all_dynamic_non_deprecated_privileges(
-    THD *thd, std::function<bool(const char *)> action) {
-  Acl_cache_lock_guard acl_cache_lock(thd, Acl_cache_lock_mode::READ_MODE);
-  if (!acl_cache_lock.lock()) return true;
-  Dynamic_privilege_register::iterator it =
-      get_dynamic_privilege_register()->begin();
-  Dynamic_privilege_register::iterator end_it =
-      get_dynamic_privilege_register()->end();
-  while (it != end_it) {
-    if (!is_dynamic_privilege_deprecated(*it) && action(it->c_str()))
-      return true;
-    ++it;
-  }
-  return false;
-}
