@@ -1,16 +1,15 @@
-/* Copyright (c) 2018, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2018, 2020, Oracle and/or its affiliates. All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
 as published by the Free Software Foundation.
 
-This program is designed to work with certain software (including
+This program is also distributed with certain software (including
 but not limited to OpenSSL) that is licensed under separate terms,
 as designated in a particular file or component or in included license
 documentation.  The authors of MySQL hereby grant you an additional
 permission to link the program and your derivative works with the
-separately licensed software that they have either included with
-the program or referenced in the documentation.
+separately licensed software that they have included with MySQL.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,11 +28,11 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 #include "mysql/components/services/log_shared.h"
 #include "mysqld_error.h"
 #include "partial_revokes.h"
-#include "sql-common/json_dom.h"
 #include "sql/auth/auth_acls.h"
 #include "sql/auth/auth_common.h"
 #include "sql/auth/auth_internal.h"
 #include "sql/auth/sql_auth_cache.h"
+#include "sql/json_dom.h"
 #include "sql/mysqld.h"
 #include "sql/sql_class.h"
 #include "sql/sql_lex.h"
@@ -46,38 +45,50 @@ const std::string Restrictions("Restrictions");
 
 /**
   Abstract restriction constructor
+
+  @param [in] mem_root MEM_ROOT handle to be used to store restrictions.
+                       Can be nullptr.
 */
-Abstract_restrictions::Abstract_restrictions() = default;
+Abstract_restrictions::Abstract_restrictions(MEM_ROOT *mem_root)
+    : m_mem_root_base(mem_root) {}
 
 /** Abstract restriction destructor */
-Abstract_restrictions::~Abstract_restrictions() = default;
+Abstract_restrictions::~Abstract_restrictions() {}
 
 /**
   DB Restrictions constructor
+
+  @param [in] mem_root MEM_ROOT handle. Can be nullptr.
+
 */
-DB_restrictions::DB_restrictions() : Abstract_restrictions() {}
+DB_restrictions::DB_restrictions(MEM_ROOT *mem_root)
+    : Abstract_restrictions(mem_root),
+      m_restrictions(system_charset_info ? system_charset_info
+                                         : &my_charset_utf8_general_ci,
+                     m_mem_root_base.get_mem_root()) {}
 
 /**
   Copy constructor for DB Restrictions
 
-  @param [in] other Source DB restrictions
+  @param [in] restrictions Source DB restrictions
 */
-DB_restrictions::DB_restrictions(const DB_restrictions &other) {
-  copy_restrictions(other);
+DB_restrictions::DB_restrictions(const DB_restrictions &restrictions)
+    : DB_restrictions(nullptr) {
+  add(restrictions);
 }
-
 /** Destructor */
-DB_restrictions::~DB_restrictions() { clear(); }
+DB_restrictions::~DB_restrictions() { m_restrictions.clear(); }
 
 /**
   Assignment operator
 
-  @param [in] other Source DB restrictions
+  @param [in] restrictions Source DB restrictions
 */
-DB_restrictions &DB_restrictions::operator=(const DB_restrictions &other) {
-  if (this != &other) {
-    clear();
-    copy_restrictions(other);
+DB_restrictions &DB_restrictions::operator=(
+    const DB_restrictions &restrictions) {
+  if (this != &restrictions) {
+    this->clear();
+    this->add(restrictions);
   }
   return *this;
 }
@@ -89,8 +100,8 @@ DB_restrictions &DB_restrictions::operator=(const DB_restrictions &other) {
 */
 DB_restrictions &DB_restrictions::operator=(DB_restrictions &&restrictions) {
   if (this != &restrictions) {
-    clear();
-    add(restrictions);
+    this->clear();
+    this->add(restrictions);
   }
   return *this;
 }
@@ -104,9 +115,7 @@ DB_restrictions &DB_restrictions::operator=(DB_restrictions &&restrictions) {
 
 */
 bool DB_restrictions::operator==(const DB_restrictions &restrictions) const {
-  if (is_empty() && restrictions.is_empty()) return true;
-  if (is_empty() || restrictions.is_empty()) return false;
-  return (*m_restrictions == *restrictions.m_restrictions);
+  return (m_restrictions == restrictions.m_restrictions);
 }
 
 /**
@@ -117,13 +126,12 @@ bool DB_restrictions::operator==(const DB_restrictions &restrictions) const {
 */
 void DB_restrictions::add(const std::string &db_name,
                           const ulong revoke_privs) {
-  auto restrictions = create_restrictions_if_needed();
-  if (restrictions->find(db_name) != restrictions->end()) {
+  if (m_restrictions.find(db_name) != m_restrictions.end()) {
     /* Partial revokes already exists on this DB so update them */
-    (*restrictions)[db_name] |= (revoke_privs);
+    m_restrictions[db_name] |= (revoke_privs);
   } else {
     /* Partial revokes do not exist on this DB so add them */
-    restrictions->emplace(db_name, revoke_privs);
+    m_restrictions.emplace(db_name, revoke_privs);
   }
 }
 
@@ -133,10 +141,7 @@ void DB_restrictions::add(const std::string &db_name,
   @param [in] restrictions List of <database, privileges>
 */
 void DB_restrictions::add(const DB_restrictions &restrictions) {
-  if (!restrictions.m_restrictions) {
-    return;
-  }
-  for (auto &itr : *(restrictions.m_restrictions))
+  for (auto &itr : restrictions.m_restrictions)
     add(itr.first.c_str(), itr.second);
 }
 
@@ -202,11 +207,9 @@ bool DB_restrictions::add(const Json_object &json_object) {
 */
 void DB_restrictions::remove(const std::string &db_name,
                              const ulong revoke_privs) {
-  if (m_restrictions != nullptr) {
-    auto rest_itr = m_restrictions->find(db_name);
-    if (rest_itr != m_restrictions->end()) {
-      remove(revoke_privs, rest_itr->second);
-    }
+  auto rest_itr = m_restrictions.find(db_name);
+  if (rest_itr != m_restrictions.end()) {
+    remove(revoke_privs, rest_itr->second);
   }
 }
 
@@ -220,15 +223,13 @@ void DB_restrictions::remove(const std::string &db_name,
   @param [in] revoke_privs Privileges to be removed
 */
 void DB_restrictions::remove(const ulong revoke_privs) {
-  if (m_restrictions != nullptr) {
-    for (auto rest_itr = m_restrictions->begin();
-         rest_itr != m_restrictions->end();) {
-      remove(revoke_privs, rest_itr->second);
-      if (rest_itr->second == 0)
-        rest_itr = m_restrictions->erase(rest_itr);
-      else
-        ++rest_itr;
-    }
+  for (auto rest_itr = m_restrictions.begin();
+       rest_itr != m_restrictions.end();) {
+    remove(revoke_privs, rest_itr->second);
+    if (rest_itr->second == 0)
+      rest_itr = m_restrictions.erase(rest_itr);
+    else
+      ++rest_itr;
   }
 }
 
@@ -240,7 +241,7 @@ void DB_restrictions::remove(const ulong revoke_privs) {
 */
 void DB_restrictions::remove(const ulong remove_restrictions,
                              ulong &restriction_mask) const noexcept {
-  const ulong mask = restriction_mask ^ remove_restrictions;
+  ulong mask = restriction_mask ^ remove_restrictions;
   restriction_mask = restriction_mask & mask;
 }
 
@@ -255,36 +256,25 @@ void DB_restrictions::remove(const ulong remove_restrictions,
     @retval false Entry not found. Do not rely on access.
 */
 bool DB_restrictions::find(const std::string &db_name, ulong &access) const {
-  if (!is_empty()) {
-    const auto &itr = m_restrictions->find(db_name);
-    if (itr != m_restrictions->end()) {
-      access = itr->second;
-      return true;
-    }
+  const auto &itr = m_restrictions.find(db_name);
+  if (itr != m_restrictions.end()) {
+    access = itr->second;
+    return true;
   }
   return false;
 }
 
-/**
-  Status function to check if restriction list is empty.
-  m_restrictions may not be initialized yet, that also indicates that the
-  restrictions are empty.
-*/
-bool DB_restrictions::is_empty() const {
-  return (m_restrictions != nullptr ? m_restrictions->empty() : true);
-}
+/** Status function to check if restriction list is empty */
+bool DB_restrictions::is_empty() const { return m_restrictions.empty(); }
+
+/** Status function to check if restriction list is non-empty */
+bool DB_restrictions::is_not_empty() const { return !is_empty(); }
 
 /** Status function to get number of entries in restriction list */
-size_t DB_restrictions::size() const {
-  if (is_empty()) return 0;
-  return m_restrictions->size();
-}
+size_t DB_restrictions::size() const { return m_restrictions.size(); }
 
 /** Clear restriction list */
-void DB_restrictions::clear() {
-  delete m_restrictions;
-  m_restrictions = nullptr;
-}
+void DB_restrictions::clear() { m_restrictions.clear(); }
 
 /**
   Serializer. Converts restriction list to JSON format.
@@ -292,8 +282,7 @@ void DB_restrictions::clear() {
   This is used while storing restriction list in ACL table.
 */
 void DB_restrictions::get_as_json(Json_array &restrictions_array) const {
-  assert(m_restrictions != nullptr);
-  for (auto &revocations_itr : *m_restrictions) {
+  for (auto &revocations_itr : m_restrictions) {
     Json_array privileges;
     Json_object revocations_obj;
     Json_string db_name(revocations_itr.first.c_str());
@@ -358,7 +347,7 @@ bool DB_restrictions::has_more_restrictions(const DB_restrictions &other,
   A factory method that creates objects from Restrictions_aggregator
   hierarchy.
 
-  Creates an object if --partial_revokes system variable is ON.
+  Creates an object iif --partial_revokes system variable is ON.
   It also records the CURRENT_USER in the binlog so that partial_revokes can
   be executed on slave with context of current user
 
@@ -382,7 +371,7 @@ Restrictions_aggregator_factory::create(THD *thd, const ACL_USER *acl_user,
   */
   if (mysqld_partial_revokes() == false || acl_user == nullptr)
     return aggregator;
-  const enum_sql_command command = thd->lex->sql_command;
+  enum_sql_command command = thd->lex->sql_command;
   const Security_context *security_context = thd->security_context();
   /* Fetch grantor Auth_id */
   const Auth_id grantor = fetch_grantor(security_context);
@@ -390,12 +379,12 @@ Restrictions_aggregator_factory::create(THD *thd, const ACL_USER *acl_user,
   const Auth_id grantee = fetch_grantee(acl_user);
   /* Fetch access information of grantor */
   ulong grantor_global_access;
-  Restrictions grantor_restrictions;
+  Restrictions grantor_restrictions(nullptr);
   fetch_grantor_access(security_context, db, grantor_global_access,
                        grantor_restrictions);
-  /* Fetch access information of grantee */
+  /* Fetch access infomation of grantee */
   ulong grantee_global_access;
-  Restrictions grantee_restrictions;
+  Restrictions grantee_restrictions(nullptr);
   fetch_grantee_access(acl_user, grantee_global_access, grantee_restrictions);
   if (db) {
     /* Fetch DB privileges of grantor */
@@ -418,7 +407,7 @@ Restrictions_aggregator_factory::create(THD *thd, const ACL_USER *acl_user,
             std::move(security_context)));
         break;
       default:
-        assert(false);
+        DBUG_ASSERT(false);
         break;
     }
   } else {
@@ -442,11 +431,11 @@ Restrictions_aggregator_factory::create(THD *thd, const ACL_USER *acl_user,
             std::move(security_context)));
         break;
       default:
-        assert(false);
+        DBUG_ASSERT(false);
         break;
     }
   }
-  assert(aggregator);
+  DBUG_ASSERT(aggregator);
   return aggregator;
 }
 
@@ -463,7 +452,7 @@ Restrictions_aggregator_factory::create(
   aggregator.reset(new DB_restrictions_aggregator_set_role(
       grantor, grantee, grantor_access, grantee_access, grantor_db_restrictions,
       grantee_db_restrictions, required_access, db_map));
-  assert(aggregator);
+  DBUG_ASSERT(aggregator);
   return aggregator;
 }
 
@@ -479,7 +468,7 @@ Auth_id Restrictions_aggregator_factory::fetch_grantor(
   // Fetch the grantor auth_id from security context on master
   grantor_user = sctx->priv_user();
   grantor_host = sctx->priv_host();
-  const Auth_id grantor(grantor_user, grantor_host);
+  Auth_id grantor(grantor_user, grantor_host);
   return grantor;
 }
 
@@ -493,11 +482,10 @@ Auth_id Restrictions_aggregator_factory::fetch_grantor(
 Auth_id Restrictions_aggregator_factory::fetch_grantee(
     const ACL_USER *acl_user) {
   // Fetch the grantee auth_id
-  const std::string grantee_user(acl_user->user != nullptr ? acl_user->user
-                                                           : "");
-  const std::string grantee_host(
+  std::string grantee_user(acl_user->user != nullptr ? acl_user->user : "");
+  std::string grantee_host(
       acl_user->host.get_host() != nullptr ? acl_user->host.get_host() : "");
-  const Auth_id grantee(grantee_user, grantee_host);
+  Auth_id grantee(grantee_user, grantee_host);
   return grantee;
 }
 
@@ -551,7 +539,7 @@ void Restrictions_aggregator_factory::fetch_grantor_access(
 
 void Restrictions_aggregator_factory::fetch_grantee_access(
     const ACL_USER *grantee, ulong &global_access, Restrictions &restrictions) {
-  assert(assert_acl_cache_read_lock(current_thd));
+  DBUG_ASSERT(assert_acl_cache_read_lock(current_thd));
   global_access = grantee->access;
   restrictions = acl_restrictions->find_restrictions(grantee);
 }
@@ -577,11 +565,11 @@ Restrictions_aggregator::Restrictions_aggregator(
       m_requested_access(requested_access),
       m_status(Status::No_op) {
   // partial_revokes system variable is ON
-  assert(mysqld_partial_revokes());
+  DBUG_ASSERT(mysqld_partial_revokes());
 }
 
 /** Destructor */
-Restrictions_aggregator::~Restrictions_aggregator() = default;
+Restrictions_aggregator::~Restrictions_aggregator() {}
 
 /**
   Constructor for database level restrictions aggregator
@@ -619,7 +607,7 @@ DB_restrictions_aggregator::DB_restrictions_aggregator(
   grantor & grantee's restrictions, global access and grantee access.
 
   We also perform dynamic cast here once and call method of respective derived
-  classes. This way, derived classes do not have to override aggregate and
+  classes. This way, dervied classes do not have to override aggregate and
   perform similar dynamic casting before proceeding.
 
   @param [out] restrictions Aggreatated restrictions for grantee
@@ -631,7 +619,7 @@ DB_restrictions_aggregator::DB_restrictions_aggregator(
 bool DB_restrictions_aggregator::generate(Abstract_restrictions &restrictions) {
   DB_restrictions *db_restrictions =
       dynamic_cast<DB_restrictions *>(&restrictions);
-  assert(db_restrictions);
+  DBUG_ASSERT(db_restrictions);
   m_status = validate();
   if (m_status == Status::Validated) {
     aggregate(*db_restrictions);
@@ -644,7 +632,7 @@ bool DB_restrictions_aggregator::generate(Abstract_restrictions &restrictions) {
 /**
   Get list of privileges that are not restricted through restriction list
 
-  @param [out] rights Bitmask of privileges to be processed further
+  @param [out] rights Bitmask of privileges to be processed futher
 
   @returns Any more privilegs remaining?
     @retval false No privileges to be processed further
@@ -690,7 +678,7 @@ bool DB_restrictions_aggregator::check_db_access_and_restrictions_collision(
     const std::string &db_name) noexcept {
   if (grantee_db_access & grantee_restrictions) {
     // find out the least significant bit(lsb)
-    const ulong lsb = grantee_db_access & ~(grantee_db_access - 1);
+    ulong lsb = grantee_db_access & ~(grantee_db_access - 1);
     // find out the position of the lsb
     size_t index = static_cast<size_t>(std::log2(lsb));
     my_error(ER_PARTIAL_REVOKE_AND_DB_GRANT_BOTH_EXISTS, MYF(0),
@@ -714,7 +702,7 @@ void DB_restrictions_aggregator::set_if_db_level_operation(
     const ulong requested_access, const ulong restrictions_mask) noexcept {
   /*
     DB level privilegss are filtered as following -
-    1. If restriction_mask is 0 then there is nothing to filter from the
+    1. If restriction_mask is 0 then there is nothin to filter from the
        requested_access.
     2. If restriction_mask has some access restrictions then remove only
        those access in the requested_access.
@@ -744,55 +732,38 @@ void DB_restrictions_aggregator::set_if_db_level_operation(
                                 is to be done.
   @param  [in]    db_map        DB_access_map used to fetch grantee's db access
                                 for SET ROLE
-  @param  [out]   restrictions  Fills the parameter with the generated
+  @param  [out]   restrictions  Fills the paramter with the generated
                                 DB_restrictions.
 
 */
 void DB_restrictions_aggregator::aggregate_restrictions(
     SQL_OP sql_op, const Db_access_map *db_map, DB_restrictions &restrictions) {
-  if (!m_grantor_rl.is_empty()) {
+  if (m_grantor_rl.is_not_empty()) {
     if (test_all_bits(m_grantee_global_access, m_requested_access) &&
         m_grantee_rl.is_empty()) {
       /*
-        If grantee does not have any restrictions but grantor has, that
-        implies grantee already has more privileges, therefore don't do
-        any further processing. Keep the empty restrictions list.
+        If grantee does not have any restrictions but grantor has, that implies
+        grantee already has more privileges, therefore retain its restrictions
       */
       restrictions = m_grantee_rl;
     } else {
       ulong restrictions_mask;
-      DB_restrictions grantee_rl = m_grantee_rl;
       for (const auto &grantor_rl_itr : m_grantor_rl()) {
         ulong grantee_revokes;
-        if (grantee_rl.find(grantor_rl_itr.first, grantee_revokes)) {
+        if (m_grantee_rl.find(grantor_rl_itr.first.c_str(), grantee_revokes)) {
           /*
             If grantor and grantee both have restriction list then aggregration
             of them will be OR operation of the following :
-             - Grantee's global privileges negates grantor's restrictions
-               (consider relevant restrictions in context of privileges
-                specified in statement i.e. requested_access)
-             - Grantor and Grantee both might have same restrictions as well
-               as global privileges. Remaining restrictions after negation
-               by Grantor and Grantee's global privileges.
+              - Grantor's global privileges negates grantee's restrictions
+              - Grantee's global privileges negates grantor's restrictions
+              - Grantor and grantee both might have same restrictions as well
+                as global privileges
           */
-          ulong rm1 = ((grantor_rl_itr.second & m_requested_access) &
-                       ~m_grantee_global_access);
-          ulong rm2 = (grantor_rl_itr.second & m_grantee_global_access &
-                       grantee_revokes & m_grantor_global_access);
-          restrictions_mask = rm1 | rm2;
-
-          /*
-            Retain restrictions that are not applicable w.r.t.
-            requested operation in the restrictions list.
-            For instance :
-            Grantee may have restrictions SELECT, INSERT on db,
-            the GRANT statement has only SELECT on db, then
-            retain INSERT on db.
-          */
-          ulong remove_mask = grantee_revokes & m_requested_access;
-          grantee_rl.remove(grantor_rl_itr.first, remove_mask);
-          /* Remove the schemas without any restrictions remaining */
-          grantee_rl.remove(0);
+          ulong rm1 = (grantor_rl_itr.second & ~m_grantee_global_access);
+          ulong rm2 = (grantee_revokes & ~m_grantor_global_access);
+          ulong rm3 = (grantor_rl_itr.second & m_grantee_global_access &
+                       m_grantor_global_access & grantee_revokes);
+          restrictions_mask = rm1 | rm2 | rm3;
         } else {
           /*
             Filter restrictions relevant to access from the grantor's
@@ -802,7 +773,8 @@ void DB_restrictions_aggregator::aggregate_restrictions(
 
           ulong grantee_db_access = 0;
           if (sql_op == SQL_OP::GLOBAL_GRANT) {
-            grantee_db_access = get_grantee_db_access(grantor_rl_itr.first);
+            grantee_db_access =
+                get_grantee_db_access(grantor_rl_itr.first.c_str());
           } else if (sql_op == SQL_OP::SET_ROLE && db_map != nullptr) {
             const std::string key(grantor_rl_itr.first.c_str(),
                                   grantor_rl_itr.first.length());
@@ -816,31 +788,16 @@ void DB_restrictions_aggregator::aggregate_restrictions(
           */
           restrictions_mask &= ~grantee_db_access;
 
-          /* Remove any global privilege that grantee already has */
+          /*
+            Remove any global privilege that grantee already has
+          */
           restrictions_mask &= ~m_grantee_global_access;
         }
         if (restrictions_mask)
-          restrictions.add(grantor_rl_itr.first, restrictions_mask);
-      }
-      /*
-       We are left with two types of grantee's restrictions to process.
-       (1) Grantee's schemas that did not match grantor's restrictions but
-           restrictions on these schemas could be negated due to grantor's
-           global privileges.
-       (2) Restrictions in grantee's schemas which are not relevant with
-           the requested access. Grantee must retain these restrictions.
-      */
-      if (!grantee_rl.is_empty()) {
-        for (auto &grantee_rl_itr : grantee_rl()) {
-          grantee_rl_itr.second &=
-              ~(m_grantor_global_access & m_requested_access);
-        }
-        /* Remove the schemas without any restrictions remaining */
-        grantee_rl.remove(0);
-        restrictions.add(grantee_rl);
+          restrictions.add(grantor_rl_itr.first.c_str(), restrictions_mask);
       }
     }
-  } else if (!m_grantee_rl.is_empty()) {
+  } else if (m_grantee_rl.is_not_empty()) {
     restrictions = m_grantee_rl;
     for (auto &grantee_rl_itr : restrictions()) {
       restrictions.remove(grantee_rl_itr.first.c_str(), m_requested_access);
@@ -867,10 +824,16 @@ void DB_restrictions_aggregator::aggregate_restrictions(
 */
 ulong DB_restrictions_aggregator::get_grantee_db_access(
     const std::string &db_name) const {
-  return Security_context::check_db_level_access(
-      current_thd, m_sctx, m_grantee.host().c_str(),
-      m_sctx ? m_sctx->ip().str : m_grantee.host().c_str(),
-      m_grantee.user().c_str(), db_name.c_str(), db_name.length());
+  ulong db_access;
+  if (m_sctx && m_sctx->get_num_active_roles() > 0) {
+    LEX_CSTRING db = {db_name.c_str(), db_name.length()};
+    db_access = m_sctx->db_acl(db, false);
+  } else {
+    db_access = acl_get(current_thd, m_grantee.host().c_str(),
+                        m_sctx ? m_sctx->ip().str : m_grantee.host().c_str(),
+                        m_grantee.user().c_str(), db_name.c_str(), false);
+  }
+  return db_access;
 }
 
 /**
@@ -886,7 +849,7 @@ ulong DB_restrictions_aggregator::get_grantee_db_access(
 void DB_restrictions_aggregator::get_grantee_db_access(
     const std::string &db_name, ulong &access) const {
   if (m_sctx && m_sctx->get_num_active_roles() > 0) {
-    const LEX_CSTRING db = {db_name.c_str(), db_name.length()};
+    LEX_CSTRING db = {db_name.c_str(), db_name.length()};
     access = m_sctx->db_acl(db, false);
   }
 }
@@ -920,9 +883,9 @@ DB_restrictions_aggregator_set_role::DB_restrictions_aggregator_set_role(
   -  Checks possible descrepancy between DB access being granted and
      existing restrictions.
 
-  @returns Status   Moves the object in the appropriate status.
+  @returns Status   Moves the object in the appropiate status.
                     For instance :
-                    - Validated, if validation was performed successfully
+                    - Validated, if validation was performed successfuly
                     - NO_op, if there is no aggregation of privileges required.
                     - Error, if an error encountered
 */
@@ -949,14 +912,14 @@ DB_restrictions_aggregator_set_role::validate() {
   - else if grantee has restrictions
     - Remove the restrictions on which global grant is requested.
 
-  @param  [out]  db_restrictions  Fills the parameter with the generated
+  @param  [out]  db_restrictions  Fills the paramter with the generated
                                   DB_restrictions
 */
 void DB_restrictions_aggregator_set_role::aggregate(
     DB_restrictions &db_restrictions) {
-  assert(m_status == Status::Validated);
+  DBUG_ASSERT(m_status == Status::Validated);
 
-  if (!m_grantee_rl.is_empty()) {
+  if (m_grantee_rl.is_not_empty()) {
     /*
       At this point, we already have aggregated DB privileges and grantee's
       restrictions. Therefore, negate the restrictions with the DB privileges.
@@ -1015,32 +978,30 @@ DB_restrictions_aggregator_global_grant::
   -  Checks possible descrepancy between DB access being granted and
      existing restrictions.
 
-  @returns Status   Moves the object in the appropriate status.
+  @returns Status   Moves the object in the appropiate status.
                     For instance :
-                    - Validated, if validation was performed successfully
+                    - Validated, if validation was performed successfuly
                     - NO_op, if there is no aggregation of privileges required.
                     - Error, if an error encountered
 */
 Restrictions_aggregator::Status
 DB_restrictions_aggregator_global_grant::validate() {
-  if (!m_grantee_rl.is_empty()) {
-    for (auto &grantee_rl_itr : m_grantee_rl()) {
-      ulong grantee_db_access =
-          get_grantee_db_access(grantee_rl_itr.first.c_str());
-      /*
-        Remove the restrictions from grantor's restrictions for which
-        grantee already has DB level privileges.
-      */
-      if (check_db_access_and_restrictions_collision(
-              grantee_db_access, grantee_rl_itr.second,
-              grantee_rl_itr.first.c_str())) {
-        return (m_status = Status::Error);
-        break;
-      }
+  for (auto &grantee_rl_itr : m_grantee_rl()) {
+    ulong grantee_db_access =
+        get_grantee_db_access(grantee_rl_itr.first.c_str());
+    /*
+      Remove the restrictions from grantor's restrictions for which
+      grantee already has DB level privileges.
+    */
+    if (check_db_access_and_restrictions_collision(
+            grantee_db_access, grantee_rl_itr.second,
+            grantee_rl_itr.first.c_str())) {
+      return (m_status = Status::Error);
+      break;
     }
   }
   /*
-    It must be only Global Grant, if grantor and grantee both does not have any
+    It must be only Global Grant, if grantor and grantee both doesnot have any
     restrictions attached.
   */
   if (m_grantor_rl.is_empty() && m_grantee_rl.is_empty())
@@ -1060,12 +1021,12 @@ DB_restrictions_aggregator_global_grant::validate() {
   - else if grantee has restrictions
     - Remove the restrictions on which global grant is requested.
 
-  @param  [out]  restrictions  Fills the parameter with the generated
+  @param  [out]  restrictions  Fills the paramter with the generated
                                DB_restrictions
 */
 void DB_restrictions_aggregator_global_grant::aggregate(
     DB_restrictions &restrictions) {
-  assert(m_status == Status::Validated);
+  DBUG_ASSERT(m_status == Status::Validated);
   aggregate_restrictions(SQL_OP::GLOBAL_GRANT, nullptr, restrictions);
   m_status = Status::Aggregated;
 }
@@ -1098,9 +1059,9 @@ DB_restrictions_aggregator_global_revoke::
   Evaluates the restrictions list of grantor and grantee, as well as requested
   privilege.
 
-  @returns Status   Moves the object in the appropriate status.
+  @returns Status   Moves the object in the appropiate status.
                     For instance :
-                    - Validated, if validation was performed successfully
+                    - Validated, if validation was performed successfuly
                     - NO_op, if there is no aggregation of privileges required.
                     - Error, if an error encountered
 */
@@ -1114,9 +1075,9 @@ DB_restrictions_aggregator_global_revoke::validate() {
       revoke from grantee.
     */
     return (m_status = Status::No_op);
-  } else if (!m_grantee_rl.is_empty()) {
+  } else if (m_grantee_rl.is_not_empty()) {
     return validate_if_grantee_rl_not_empty();
-  } else if (!m_grantor_rl.is_empty()) {
+  } else if (m_grantor_rl.is_not_empty()) {
     for (auto &itr : m_grantor_rl.get()) {
       /*
         Grantor cannot revoke the privileges from grantee which are in
@@ -1134,18 +1095,25 @@ DB_restrictions_aggregator_global_revoke::validate() {
 }
 
 /**
+  Generates DB_restrictions based on the requested access, grantor and
+  grantee's DB_restrictions in the ACL cache.
   - If grantee has the restriction list
     - Remove only requested restrictions from restriction_list
+  - Else clear the DB_restrictions paramater
 
-  @param  [out]  restrictions  Fills the parater with the aggregated
-                               DB_restrictions
+  @param  [out]  restrictions  Fills the paramter with the aggregated
+                                  DB_restrictions
 */
 void DB_restrictions_aggregator_global_revoke::aggregate(
     DB_restrictions &restrictions) {
-  assert(m_status == Status::Validated);
-  restrictions = m_grantee_rl;
+  DBUG_ASSERT(m_status == Status::Validated);
   if (test_all_bits(m_grantee_global_access, m_requested_access)) {
-    restrictions.remove(m_requested_access);
+    if (m_grantee_rl.is_not_empty()) {
+      restrictions = m_grantee_rl;
+      restrictions.remove(m_requested_access);
+    } else {
+      restrictions.clear();
+    }
   }
   m_status = Status::Aggregated;
 }
@@ -1157,9 +1125,9 @@ void DB_restrictions_aggregator_global_revoke::aggregate(
   - Check if grantee has same DB level privilege as well as
     restriction list on the  database
 
-  @returns  Status  Moves the object in the appropriate status.
+  @returns  Status  Moves the object in the appropiate status.
                     For instance :
-                    - Validated, if validation was performed successfully
+                    - Validated, if validation was performed successfuly
                     - NO_op, if there is no aggregation of privileges required.
                     - Error, if an error encountered
 */
@@ -1224,9 +1192,9 @@ Restrictions_aggregator::Status
 DB_restrictions_aggregator_global_revoke_all::validate() {
   if (m_grantor_rl == m_grantee_rl) {
     return (m_status = Status::No_op);
-  } else if (!m_grantee_rl.is_empty()) {
+  } else if (m_grantee_rl.is_not_empty()) {
     return validate_if_grantee_rl_not_empty();
-  } else if (!m_grantor_rl.is_empty()) {
+  } else if (m_grantor_rl.is_not_empty()) {
     for (auto &itr : m_grantor_rl.get()) {
       /*
         Grantor cannot revoke the privileges from grantee which are in
@@ -1250,7 +1218,7 @@ DB_restrictions_aggregator_global_revoke_all::validate() {
 */
 void DB_restrictions_aggregator_global_revoke_all::aggregate(
     DB_restrictions &restrictions) {
-  assert(m_status == Status::Validated);
+  DBUG_ASSERT(m_status == Status::Validated);
   restrictions.clear();
   m_status = Status::Aggregated;
 }
@@ -1335,7 +1303,7 @@ void DB_restrictions_aggregator_db_grant::aggregate(
        should be recorded in mysql.db table. Hence, we call
        set_if_db_level_operation().
   */
-  assert(m_status == Status::Validated);
+  DBUG_ASSERT(m_status == Status::Validated);
   restrictions = m_grantee_rl;
   ulong grantee_db_revokes = 0;
 
@@ -1422,7 +1390,7 @@ DB_restrictions_aggregator_db_revoke::validate() {
 */
 void DB_restrictions_aggregator_db_revoke::aggregate(
     DB_restrictions &restrictions) {
-  assert(m_status == Status::Validated);
+  DBUG_ASSERT(m_status == Status::Validated);
   restrictions = m_grantee_rl;
   if ((test_all_bits(m_grantee_db_access, m_requested_access))) {
     /*
@@ -1451,14 +1419,26 @@ void DB_restrictions_aggregator_db_revoke::aggregate(
 
 /**
   Constructor for Restrictions
+
+  @param [in] mem_root MEM_ROOT to be used to store restrictions
 */
-Restrictions::Restrictions() : m_db_restrictions() {}
+Restrictions::Restrictions(MEM_ROOT *mem_root) : m_db_restrictions(mem_root) {}
 
 /** Destructor */
 Restrictions ::~Restrictions() { m_db_restrictions.clear(); }
 
+/**
+  Copy constructor for Restrictions
+
+  @param [in] restrictions Restrictions to be copied
+*/
+Restrictions::Restrictions(const Restrictions &restrictions)
+    : m_db_restrictions(restrictions.m_db_restrictions) {}
+
 /**
   Move constructor for Restrictions
+
+  @param [in] restrictions
 */
 Restrictions::Restrictions(Restrictions &&restrictions)
     : m_db_restrictions(restrictions.m_db_restrictions) {}
