@@ -1,4 +1,4 @@
-/* Copyright (c) 2018, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
@@ -26,14 +26,15 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 #include <map>
 #include <memory>
 #include <set>
-#include <unordered_map>
 
 #include "map_helpers.h"
 #include "memory_debugging.h"
+#include "my_alloc.h"
 #include "my_inttypes.h"
 #include "my_sqlcommand.h"
 #include "sql/auth/auth_common.h"
 #include "sql/auth/auth_utility.h"
+#include "sql/mem_root_allocator.h"
 
 // Forward declarations
 class THD;
@@ -41,9 +42,10 @@ class ACL_USER;
 class Json_array;
 class Json_object;
 class Restrictions_aggregator;
+extern MEM_ROOT global_acl_memory;
 
 // Alias declarations
-using db_revocations = std::unordered_map<std::string, ulong>;
+using db_revocations = mem_root_collation_unordered_map<std::string, ulong>;
 using Db_access_map = std::map<std::string, unsigned long>;
 
 /**
@@ -51,11 +53,15 @@ using Db_access_map = std::map<std::string, unsigned long>;
 */
 class Abstract_restrictions {
  public:
-  explicit Abstract_restrictions();
+  explicit Abstract_restrictions(MEM_ROOT *mem_root);
   virtual ~Abstract_restrictions();
   virtual bool is_empty() const = 0;
   virtual size_t size() const = 0;
   virtual void clear() = 0;
+
+ protected:
+  /** MEM_ROOT manager */
+  Mem_root_base m_mem_root_base;
 };
 
 /**
@@ -73,8 +79,8 @@ class Abstract_restrictions {
 */
 class DB_restrictions final : public Abstract_restrictions {
  public:
-  DB_restrictions();
-  ~DB_restrictions() override;
+  DB_restrictions(MEM_ROOT *mem_root);
+  virtual ~DB_restrictions() override;
 
   db_revocations &operator()(void) { return db_restrictions(); }
   DB_restrictions(const DB_restrictions &restrictions);
@@ -100,8 +106,8 @@ class DB_restrictions final : public Abstract_restrictions {
 
  private:
   db_revocations &db_restrictions() { return m_restrictions; }
-  void remove(const ulong remove_restrictions,
-              ulong &restrictions_mask) const noexcept;
+  void remove(const ulong remove_restrictions, ulong &restrictions_mask) const
+      noexcept;
 
  private:
   /** Database restrictions */
@@ -116,9 +122,9 @@ class DB_restrictions final : public Abstract_restrictions {
 */
 class Restrictions {
  public:
-  explicit Restrictions();
+  explicit Restrictions(MEM_ROOT *mem_root);
 
-  Restrictions(const Restrictions &) = default;
+  Restrictions(const Restrictions &);
   Restrictions(Restrictions &&);
   Restrictions &operator=(const Restrictions &);
   Restrictions &operator=(Restrictions &&);
@@ -139,11 +145,11 @@ class Restrictions {
 /**
   Factory class that solely creates an object of type Restrictions_aggregator.
 
-  - The concrete implementations of Restrictions_aggregator cannot be created
+  - The concrete implemenations of Restrictions_aggregator cannot be created
     directly since their constructors are private. This class is declared as
     friend in those concrete implementations.
   - It also records the CURRENT_USER in the binlog so that partial_revokes can
-    be executed on the replica with context of current user
+    be executed on slave with context of current user
 */
 class Restrictions_aggregator_factory {
  public:
@@ -194,7 +200,7 @@ class Restrictions_aggregator {
  public:
   virtual ~Restrictions_aggregator();
 
-  /* interface methods which derived classes have to implement */
+  /* interface methods which derived classes havee to implement */
   virtual bool generate(Abstract_restrictions &restrictions) = 0;
   virtual bool find_if_require_next_level_operation(ulong &rights) const = 0;
 
