@@ -1,15 +1,14 @@
-/* Copyright (c) 2014, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2020, Oracle and/or its affiliates. All rights reserved.
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,14 +29,13 @@
 #include <utility>
 #include <vector>
 
-#include "mf_wcomp.h"
+#include "m_ctype.h"
 #include "my_dbug.h"
 #include "my_inttypes.h"
 #include "my_sys.h"
-#include "mysql/components/services/bits/psi_bits.h"
 #include "mysql/mysql_lex_string.h"
+#include "mysql/psi/psi_base.h"
 #include "mysql/service_mysql_alloc.h"
-#include "mysql/strings/m_ctype.h"
 #include "mysqld_error.h"
 #include "sql/auth/auth_acls.h"
 #include "sql/auth/auth_common.h"
@@ -48,20 +46,23 @@
 #include "sql/mysqld.h"
 #include "sql/sql_class.h"
 #include "sql/table.h"
-#include "string_with_len.h"
-#include "strmake.h"
 
 extern bool initialized;
 
 Security_context::Security_context(THD *thd /*= nullptr */)
-    : m_restrictions(), m_thd(thd) {
+    : m_restrictions(nullptr), m_thd(thd) {
+  init();
+}
+
+Security_context::Security_context(MEM_ROOT *mem_root, THD *thd /* = nullptr*/)
+    : m_restrictions(mem_root), m_thd(thd) {
   init();
 }
 
 Security_context::~Security_context() { destroy(); }
 
 Security_context::Security_context(const Security_context &src_sctx)
-    : m_restrictions(), m_thd(nullptr) {
+    : m_restrictions(nullptr), m_thd(nullptr) {
   copy_security_ctx(src_sctx);
 }
 
@@ -90,12 +91,12 @@ void Security_context::init() {
   m_master_access = 0;
   m_db_access = NO_ACCESS;
   m_acl_map = nullptr;
+  m_map_checkout_count = 0;
   m_password_expired = false;
   m_is_locked = false;
   m_is_skip_grants_user = false;
   m_has_drop_policy = false;
   m_executed_drop_policy = false;
-  m_registration_sandbox_mode = false;
 }
 
 void Security_context::logout() {
@@ -107,8 +108,8 @@ void Security_context::logout() {
     get_global_acl_cache()->return_acl_map(m_acl_map);
     m_acl_map = nullptr;
     clear_active_roles();
+    clear_db_restrictions();
   }
-  clear_db_restrictions();
 }
 
 bool Security_context::has_drop_policy(void) { return m_has_drop_policy; }
@@ -162,7 +163,6 @@ void Security_context::destroy() {
   m_password_expired = false;
   m_is_skip_grants_user = false;
   clear_db_restrictions();
-  m_registration_sandbox_mode = false;
 }
 
 /**
@@ -185,12 +185,11 @@ void Security_context::skip_grants(const char *user /*= "skip-grants user"*/,
   m_is_skip_grants_user = true;
 
   /*
-    If the security context is tied up to to the THD object and it is
-    current security context in THD, then set the flags to true.
+    If the security context is tied upto to the THD object and it is
+    current security context in THD then set the flag to true.
   */
   if (m_thd && m_thd->security_context() == this) {
     m_thd->set_system_user(true);
-    m_thd->set_connection_admin(true);
   }
 }
 
@@ -229,14 +228,14 @@ void Security_context::copy_security_ctx(const Security_context &src_sctx) {
   Initialize this security context from the passed in credentials
   and activate it in the current thread.
 
-  @param       thd           Thread handle.
-  @param       definer_user  user part of a 'definer' value.
-  @param       definer_host  host part of a 'definer' value.
-  @param       db            Database name.
-  @param[out]  backup        Save a pointer to the current security context
-                             in the thread. In case of success it points to the
-                             saved old context, otherwise it points to NULL.
-  @param       force         Force context switch
+  @param       thd
+  @param       definer_user
+  @param       definer_host
+  @param       db
+  @param[out]  backup  Save a pointer to the current security context
+                       in the thread. In case of success it points to the
+                       saved old context, otherwise it points to NULL.
+  @param       force   Force context switch
 
 
   @note The Security_context_factory should be used as a replacement to this
@@ -268,12 +267,12 @@ void Security_context::copy_security_ctx(const Security_context &src_sctx) {
   For the main security context, the memory for user/host/ip is
   allocated on system heap, and the THD class frees this memory in
   its destructor. The only case when contents of the main security
-  context may change during its life time is when someone issued a
+  context may change during its life time is when someone issued
   CHANGE USER command.
   Memory management of a "temporary" security context is
   responsibility of the module that creates it.
 
-  @retval true  There is no user with the given credentials. The error
+  @retval true  there is no user with the given credentials. The erro
                 is reported in the thread.
   @retval false success
 */
@@ -285,7 +284,7 @@ bool Security_context::change_security_context(
 
   DBUG_TRACE;
 
-  assert(definer_user.str && definer_host.str);
+  DBUG_ASSERT(definer_user.str && definer_host.str);
 
   *backup = nullptr;
   needs_change =
@@ -353,9 +352,9 @@ int Security_context::activate_role(LEX_CSTRING role, LEX_CSTRING role_host,
                        create_authid_from(role, role_host));
   /* silently ignore requests of activating an already active role */
   if (res != m_active_roles.end()) return 0;
-  const LEX_CSTRING dup_role = {
+  LEX_CSTRING dup_role = {
       my_strdup(PSI_NOT_INSTRUMENTED, role.str, MYF(MY_WME)), role.length};
-  const LEX_CSTRING dup_role_host = {
+  LEX_CSTRING dup_role_host = {
       my_strdup(PSI_NOT_INSTRUMENTED, role_host.str, MYF(MY_WME)),
       role_host.length};
   if (validate_access && !check_if_granted_role(priv_user(), priv_host(),
@@ -399,6 +398,7 @@ void Security_context::checkout_access_maps(void) {
   }
 
   if (m_active_roles.size() == 0) return;
+  ++m_map_checkout_count;
   Auth_id_ref uid;
   uid.first.str = this->m_priv_user;
   uid.first.length = this->m_priv_user_length;
@@ -474,115 +474,44 @@ void Security_context::get_active_roles(THD *thd, List<LEX_USER> &list) {
   }
 }
 
-/**
-  Get grant information for given database
-
-  Cached database access is split into two containers:
-  1. Database names without wildcards
-  2. Database names with wildcards
-
-  First we perform the exact name comprison.
-  If that returns the result, all good.
-
-  Otherwise, we take a look at each db in second list
-  and compare incoming database name against it.
-  If patial_revokes is OFF, use_pattern_scan flag is
-  passed to wild_compare. This would allow incoming
-  database name like db1name to match against wild card
-  db entry db_name/db%name.
-
-  @note  This function should not be used outside ACL subsystem code (sql/auth).
-  Use check_db_level_access() instead.
-
-  @param [in] db               Name of the database
-  @param [in] use_pattern_scan Flag to treat database name as pattern
-
-  @returns Access granted to user for given database
-*/
-
 ulong Security_context::db_acl(LEX_CSTRING db, bool use_pattern_scan) const {
   DBUG_TRACE;
   if (m_acl_map == nullptr || db.length == 0) return 0;
 
-  const std::string key(db.str, db.length);
+  std::string key(db.str, db.length);
   Db_access_map::iterator found_acl_it = m_acl_map->db_acls()->find(key);
   if (found_acl_it == m_acl_map->db_acls()->end()) {
-    Db_access_map::iterator it = m_acl_map->db_wild_acls()->begin();
-    ulong access = 0;
-    for (; it != m_acl_map->db_wild_acls()->end(); ++it) {
-      /*
-        Do the usual string comparison if partial_revokes is ON,
-        otherwise do the wildcard grant comparison
-      */
-      if (mysqld_partial_revokes()
-              ? (my_strcasecmp(system_charset_info, db.str,
-                               it->first.c_str()) == 0)
-              : (wild_compare(db.str, db.length, it->first.c_str(),
-                              it->first.size(), use_pattern_scan)) == 0) {
-        DBUG_PRINT("info", ("Found matching db pattern %s for key %s",
-                            it->first.c_str(), key.c_str()));
-        access |= it->second;
-        return filter_access(access, key);
+    if (use_pattern_scan) {
+      Db_access_map::iterator it = m_acl_map->db_wild_acls()->begin();
+      ulong access = 0;
+      for (; it != m_acl_map->db_wild_acls()->end(); ++it) {
+        /*
+          Do the usual string comparision if partial_revokes is ON,
+          otherwise do the wildcard grant comparision
+        */
+        if (mysqld_partial_revokes()
+                ? (my_strcasecmp(system_charset_info, db.str,
+                                 it->first.c_str()) == 0)
+                : (wild_case_compare(system_charset_info, db.str, db.length,
+                                     it->first.c_str(),
+                                     it->first.size()) == 0)) {
+          DBUG_PRINT("info", ("Found matching db pattern %s for key %s",
+                              it->first.c_str(), key.c_str()));
+          access |= it->second;
+        }
       }
+      return filter_access(access, key);
+    } else {
+      DBUG_PRINT("info", ("Db %s not found in cache (no pattern matching)",
+                          key.c_str()));
+      return 0;
     }
-    DBUG_PRINT("info", ("Db %s not found in cache", key.c_str()));
-    return 0;
   } else {
     DBUG_PRINT("info", ("Found exact match for db %s", key.c_str()));
     return filter_access(found_acl_it->second, key);
   }
 }
 
-/**
-  Checks if any database level privileges are granted to the current session
-  either directly or through active roles.
-
-  @param [in] thd           Thread handler
-  @param [in] sctx          Security context
-  @param [in] host          Host name
-  @param [in] ip            Ip
-  @param [in] user          User name
-  @param [in] db            Database name
-  @param [in] db_len        Database name length
-  @param [in] db_is_pattern Flag to treat db name as pattern
-
-  @returns DB level privileges granted
-*/
-
-ulong Security_context::check_db_level_access(
-    THD *thd, const Security_context *sctx, const char *host, const char *ip,
-    const char *user, const char *db, size_t db_len, bool db_is_pattern) {
-  ulong db_access;
-  if (sctx && sctx->get_num_active_roles()) {
-    db_access = sctx->db_acl({db, db_len}, db_is_pattern);
-    DBUG_PRINT("info", ("check_access using db-level privilege for %s. "
-                        "ACL: %lu",
-                        db, db_access));
-  } else {
-    db_access = acl_get(thd, host, ip, user, db, db_is_pattern);
-  }
-  return db_access;
-}
-
-/**
-  Checks if any database level privileges are granted to the current session
-  either directly or through active roles.
-
-  @param [in] thd           Thread handler
-  @param [in] db            Database name
-  @param [in] db_len        Database name length
-  @param [in] db_is_pattern Flag to treat db name as pattern
-
-  @returns DB level privileges granted
-*/
-
-ulong Security_context::check_db_level_access(THD *thd, const char *db,
-                                              size_t db_len,
-                                              bool db_is_pattern) const {
-  return check_db_level_access(thd, this, m_host.ptr(), m_ip.ptr(), m_priv_user,
-                               db, db_len, db_is_pattern);
-}
-
 ulong Security_context::procedure_acl(LEX_CSTRING db,
                                       LEX_CSTRING procedure_name) {
   if (m_acl_map == nullptr)
@@ -592,9 +521,7 @@ ulong Security_context::procedure_acl(LEX_CSTRING db,
     String q_name;
     append_identifier(&q_name, db.str, db.length);
     q_name.append(".");
-    std::string name(procedure_name.str, procedure_name.length);
-    my_casedn_str(files_charset_info, &name[0]);
-    append_identifier(&q_name, name.c_str(), name.length());
+    append_identifier(&q_name, procedure_name.str, procedure_name.length);
     it = m_acl_map->sp_acls()->find(q_name.c_ptr());
     if (it == m_acl_map->sp_acls()->end()) return 0;
     return filter_access(it->second, q_name.c_ptr());
@@ -608,9 +535,7 @@ ulong Security_context::function_acl(LEX_CSTRING db, LEX_CSTRING func_name) {
     String q_name;
     append_identifier(&q_name, db.str, db.length);
     q_name.append(".");
-    std::string name(func_name.str, func_name.length);
-    my_casedn_str(files_charset_info, &name[0]);
-    append_identifier(&q_name, name.c_str(), name.length());
+    append_identifier(&q_name, func_name.str, func_name.length);
     SP_access_map::iterator it;
     it = m_acl_map->func_acls()->find(q_name.c_ptr());
     if (it == m_acl_map->func_acls()->end()) return 0;
@@ -634,7 +559,7 @@ Grant_table_aggregate Security_context::table_and_column_acls(
 
 ulong Security_context::table_acl(LEX_CSTRING db, LEX_CSTRING table) {
   if (m_acl_map == nullptr) return 0;
-  const Grant_table_aggregate aggr = table_and_column_acls(db, table);
+  Grant_table_aggregate aggr = table_and_column_acls(db, table);
   return filter_access(aggr.table_access, db.str ? db.str : "");
 }
 
@@ -692,29 +617,24 @@ bool Security_context::any_table_acl(const LEX_CSTRING &db) {
   @param [in] priv      privilege to check
   @param [in] priv_len  length of privilege
 
-  @returns  pair@<has_privilege, has_with_grant_option@>
-    @retval "<true, true>"  has required privilege with grant option
-    @retval "<true, false>" has required privilege without grant option
-    @retval "<false, false>" does not have the required privilege
+  @returns  pair/<has_privilege, has_with_grant_option/>
+    @retval /<true, true/>  has required privilege with grant option
+    @retval /<true, false/> has required privilege without grant option
+    @retval /<false, false/> does not have the required privilege
 */
 std::pair<bool, bool> Security_context::has_global_grant(const char *priv,
                                                          size_t priv_len) {
   /* server started with --skip-grant-tables */
   if (!initialized || m_is_skip_grants_user) return std::make_pair(true, true);
 
-  const std::string privilege(priv, priv_len);
+  std::string privilege(priv, priv_len);
 
   if (m_acl_map == nullptr) {
-    THD *thd = m_thd ? m_thd : current_thd;
-    if (thd == nullptr) {
-      DBUG_PRINT("error", ("Security Context must have valid THD handle to"
-                           " probe grants.\n"));
-      return {false, false};
-    }
-    Acl_cache_lock_guard acl_cache_lock(thd, Acl_cache_lock_mode::READ_MODE);
+    Acl_cache_lock_guard acl_cache_lock(current_thd,
+                                        Acl_cache_lock_mode::READ_MODE);
     if (!acl_cache_lock.lock(false)) return std::make_pair(false, false);
-    const Role_id key(&m_priv_user[0], m_priv_user_length, &m_priv_host[0],
-                      m_priv_host_length);
+    Role_id key(&m_priv_user[0], m_priv_user_length, &m_priv_host[0],
+                m_priv_host_length);
     User_to_dynamic_privileges_map::iterator it, it_end;
     std::tie(it, it_end) = get_dynamic_privileges_map()->equal_range(key);
     it = std::find(it, it_end, privilege);
@@ -744,10 +664,10 @@ std::pair<bool, bool> Security_context::has_global_grant(const char *priv,
                                   roles granted to it irrespective the roles are
                                   active or not.
 
-  @returns  pair@<has_privilege, has_with_grant_option@>
-    @retval "<true, true>"  has required privilege with grant option
-    @retval "<true, false>" has required privilege without grant option
-    @retval "<false, false>" does not have the required privilege, OR
+  @returns  pair/<has_privilege, has_with_grant_option/>
+    @retval /<true, true/>  has required privilege with grant option
+    @retval /<true, false/> has required privilege without grant option
+    @retval /<false, false/> does not have the required privilege, OR
                              auth_id does not exist.
 */
 std::pair<bool, bool> Security_context::has_global_grant(
@@ -784,18 +704,14 @@ std::pair<bool, bool> Security_context::has_global_grant(
                                     auth_id.
                                     true  - consider as privilege exists
                                     false - consider as privilege do not exist
-  @param  [in]  throw_error         Flag to decide if error needs to be thrown
-  or not.
 
-  @retval true    auth_id has the privilege but the current_auth does not, also
-                  throws error.
+  @retval true    auth_id has the privilege but the current_auth does not
   @retval false   Otherwise
 */
 bool Security_context::can_operate_with(const Auth_id &auth_id,
                                         const std::string &privilege,
                                         bool cumulative /*= false */,
-                                        bool ignore_if_nonextant /*= true */,
-                                        bool throw_error /* = true */) {
+                                        bool ignore_if_nonextant /*= true */) {
   DBUG_TRACE;
   Acl_cache_lock_guard acl_cache_lock(current_thd,
                                       Acl_cache_lock_mode::READ_MODE);
@@ -810,8 +726,7 @@ bool Security_context::can_operate_with(const Auth_id &auth_id,
     if (ignore_if_nonextant)
       return false;
     else {
-      if (throw_error)
-        my_error(ER_USER_DOES_NOT_EXIST, MYF(0), auth_id.auth_str().c_str());
+      my_error(ER_USER_DOES_NOT_EXIST, MYF(0), auth_id.auth_str().c_str());
       return true;
     }
   }
@@ -823,8 +738,7 @@ bool Security_context::can_operate_with(const Auth_id &auth_id,
                       : true;
   }
   if (is_mismatch) {
-    if (throw_error)
-      my_error(ER_SPECIFIC_ACCESS_DENIED_ERROR, MYF(0), privilege.c_str());
+    my_error(ER_SPECIFIC_ACCESS_DENIED_ERROR, MYF(0), privilege.c_str());
   }
   return is_mismatch;
 }
@@ -925,7 +839,7 @@ void Security_context::set_host_ptr(const char *host_arg,
                                     const size_t host_arg_length) {
   DBUG_TRACE;
 
-  assert(host_arg != nullptr);
+  DBUG_ASSERT(host_arg != nullptr);
 
   if (host_arg == m_host.ptr()) return;
 
@@ -1164,7 +1078,7 @@ void Security_context::init_restrictions(const Restrictions &restrictions) {
 
 bool Security_context::is_access_restricted_on_db(
     ulong want_access, const std::string &db_name) const {
-  const ulong filtered_access = filter_access(want_access, db_name);
+  ulong filtered_access = filter_access(want_access, db_name);
   return (filtered_access != want_access);
 }
 
@@ -1181,7 +1095,7 @@ ulong Security_context::filter_access(const ulong access,
                                       const std::string &db_name) const {
   ulong access_mask = access;
   auto &db_restrictions = m_restrictions.db();
-  if (!db_restrictions.is_empty()) {
+  if (db_restrictions.is_not_empty()) {
     ulong restrictions_mask;
     if (db_restrictions.find(db_name, restrictions_mask))
       access_mask = (access_mask & restrictions_mask) ^ access;
@@ -1201,15 +1115,15 @@ ulong Security_context::filter_access(const ulong access,
                           true  - privileges granted directly or coming through
                                   roles granted to it irrespective the roles are
                                   active or not.
-  @returns  pair@<has_privilege, has_with_grant_option@>
-    @retval "<true, true>"   has required privilege with grant option
-    @retval "<true, false>"  has required privilege without grant option
-    @retval "<false, false>" does not have the required privilege
+  @returns  pair/<has_privilege, has_with_grant_option/>
+    @retval /<true, true/>   has required privilege with grant option
+    @retval /<true, false/>  has required privilege without grant option
+    @retval /<false, false/> does not have the required privilege
 */
 std::pair<bool, bool> Security_context::fetch_global_grant(
     const ACL_USER &acl_user, const std::string &privilege,
     bool cumulative /*= false */) {
-  assert(assert_acl_cache_read_lock(current_thd));
+  DBUG_ASSERT(assert_acl_cache_read_lock(current_thd));
   std::pair<bool, bool> has_privilege{false, false};
   Security_context sctx;
 
@@ -1237,12 +1151,12 @@ std::pair<bool, bool> Security_context::fetch_global_grant(
   @param [in,out] tables Table list object
 
   @returns access information
-  @retval true Success
+  @retval true Sucess
   @retval false Failure
  */
-bool Security_context::has_table_access(ulong priv, Table_ref *tables) {
+bool Security_context::has_table_access(ulong priv, TABLE_LIST *tables) {
   DBUG_TRACE;
-  assert(tables != nullptr);
+  DBUG_ASSERT(tables != nullptr);
   TABLE const *table = tables->table;
   LEX_CSTRING db, table_name;
   db.str = table->s->db.str;
@@ -1258,7 +1172,7 @@ bool Security_context::has_table_access(ulong priv, Table_ref *tables) {
     acls = db_acl(db);
     if (priv & acls) return true;
 
-    const Grant_table_aggregate aggr = table_and_column_acls(db, table_name);
+    Grant_table_aggregate aggr = table_and_column_acls(db, table_name);
     acls = aggr.table_access | aggr.cols;
     if (priv & acls) return true;
   } else {
@@ -1286,7 +1200,7 @@ bool Security_context::has_table_access(ulong priv, Table_ref *tables) {
  */
 bool Security_context::is_table_blocked(ulong priv, TABLE const *table) {
   DBUG_TRACE;
-  assert(table != nullptr);
+  DBUG_ASSERT(table != nullptr);
   LEX_CSTRING db, table_name;
   db.str = table->s->db.str;
   db.length = table->s->db.length;
@@ -1295,7 +1209,7 @@ bool Security_context::is_table_blocked(ulong priv, TABLE const *table) {
   table_name.length = strlen(table->alias);
 
   /* Table privs */
-  Table_ref tables;
+  TABLE_LIST tables;
   tables.table = const_cast<TABLE *>(table);
   tables.db = db.str;
   tables.db_length = db.length;
@@ -1314,13 +1228,13 @@ bool Security_context::is_table_blocked(ulong priv, TABLE const *table) {
   @param [in] columns List of column names to check
 
   @returns access information
-  @retval true Success
+  @retval true Sucess
   @retval false Failure
  */
 bool Security_context::has_column_access(ulong priv, TABLE const *table,
                                          std::vector<std::string> columns) {
   DBUG_TRACE;
-  assert(table != nullptr);
+  DBUG_ASSERT(table != nullptr);
   LEX_CSTRING db, table_name;
   db.str = table->s->db.str;
   db.length = table->s->db.length;
@@ -1329,7 +1243,7 @@ bool Security_context::has_column_access(ulong priv, TABLE const *table,
   table_name.length = strlen(table->alias);
 
   /* Table privs */
-  Table_ref tables;
+  TABLE_LIST tables;
   tables.table = const_cast<TABLE *>(table);
   tables.db = db.str;
   tables.db_length = db.length;
