@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2013, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -67,16 +66,6 @@ class Connection_acceptor {
     }
   }
 
-  /**
-     Spawn admin connection handler to accept admin connections from clients if
-     create-admin-listener-thread is specified by user on commandline.
-
-    @return true unable to spawn admin connection handler thread else false.
-  */
-  bool check_and_spawn_admin_connection_handler_thread() const {
-    return m_listener->check_and_spawn_admin_connection_handler_thread();
-  }
-
   /**
     Close the listener.
   */
