@@ -1,16 +1,15 @@
-/* Copyright (c) 2018, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,24 +22,21 @@
 
 #include "sql/create_field.h"
 
-#include "m_string.h"
-#include "mysql/strings/dtoa.h"
-#include "sql-common/my_decimal.h"
 #include "sql/derror.h"
 #include "sql/field.h"
 #include "sql/item.h"
+#include "sql/my_decimal.h"
 #include "sql/sql_class.h"
 #include "sql_string.h"
 #include "template_utils.h"
 
 #include <cmath>
-#include <optional>
 
-static constexpr const size_t MAX_BIT_FIELD_LENGTH{64};
-/** YYYYMMDDHHMMSS */
-static constexpr const size_t MAX_DATETIME_COMPRESSED_WIDTH{14};
-
-struct CHARSET_INFO;
+// Definition of static constexpr data members in Create_field.
+constexpr size_t Create_field::TINYBLOB_MAX_SIZE_IN_BYTES;
+constexpr size_t Create_field::BLOB_MAX_SIZE_IN_BYTES;
+constexpr size_t Create_field::MEDIUMBLOB_MAX_SIZE_IN_BYTES;
+constexpr size_t Create_field::LONGBLOB_MAX_SIZE_IN_BYTES;
 
 /**
     Constructs a column definition from an object representing an actual
@@ -67,7 +63,7 @@ Create_field::Create_field(Field *old_field, Field *orig_field)
       comment(old_field->comment),
       sql_type(old_field->real_type()),
       decimals(old_field->decimals()),
-      flags(old_field->all_flags()),
+      flags(old_field->flags),
       auto_flags(old_field->auto_flags),
       charset(old_field->charset()),  // May be NULL ptr
       is_explicit_collation(false),
@@ -83,8 +79,6 @@ Create_field::Create_field(Field *old_field, Field *orig_field)
       stored_in_db(old_field->stored_in_db),
       m_default_val_expr(old_field->m_default_val_expr),
       is_array(old_field->is_array()),
-      m_engine_attribute(old_field->m_engine_attribute),
-      m_secondary_engine_attribute(old_field->m_secondary_engine_attribute),
       m_max_display_width_in_codepoints(old_field->char_length()) {
   switch (sql_type) {
     case MYSQL_TYPE_TINY_BLOB:
@@ -108,9 +102,6 @@ Create_field::Create_field(Field *old_field, Field *orig_field)
     case MYSQL_TYPE_YEAR:
       m_max_display_width_in_codepoints = 4;  // set default value
       break;
-    case MYSQL_TYPE_LONGLONG:
-      is_unsigned = old_field->is_unsigned();
-      break;
     default:
       break;
   }
@@ -133,21 +124,21 @@ Create_field::Create_field(Field *old_field, Field *orig_field)
       buffer pointer.
   */
   if (!(flags & (NO_DEFAULT_VALUE_FLAG | BLOB_FLAG)) &&
-      old_field->field_ptr() != nullptr && orig_field != nullptr) {
+      old_field->ptr != nullptr && orig_field != nullptr) {
+    bool default_now = false;
     if (real_type_with_now_as_default(sql_type)) {
       // The SQL type of the new field allows a function default:
-      const bool default_now =
-          orig_field->has_insert_default_datetime_value_expression();
+      default_now = orig_field->has_insert_default_datetime_value_expression();
       auto_flags = default_now ? Field::DEFAULT_NOW : Field::NONE;
       if (orig_field->has_update_default_datetime_value_expression())
         auto_flags |= Field::ON_UPDATE_NOW;
       if (orig_field->has_insert_default_general_value_expression())
         auto_flags |= Field::GENERATED_FROM_EXPRESSION;
     }
-
-    if (orig_field->has_insert_default_constant_expression()) {
+    if (!default_now)  // Give a constant default
+    {
       /* Get the value from default_values */
-      const ptrdiff_t diff = orig_field->table->default_values_offset();
+      ptrdiff_t diff = orig_field->table->default_values_offset();
       orig_field->move_field_offset(diff);  // Points now at default_values
       if (!orig_field->is_real_null()) {
         StringBuffer<MAX_FIELD_WIDTH> tmp(charset);
@@ -198,18 +189,18 @@ bool Create_field::init(
     THD *thd, const char *fld_name, enum_field_types fld_type,
     const char *display_width_in_codepoints, const char *fld_decimals,
     uint fld_type_modifier, Item *fld_default_value, Item *fld_on_update_value,
-    const LEX_CSTRING *fld_comment, const char *fld_change,
+    LEX_CSTRING *fld_comment, const char *fld_change,
     List<String> *fld_interval_list, const CHARSET_INFO *fld_charset,
     bool has_explicit_collation, uint fld_geom_type,
     Value_generator *fld_gcol_info, Value_generator *fld_default_val_expr,
-    std::optional<gis::srid_t> srid, dd::Column::enum_hidden_type hidden,
+    Nullable<gis::srid_t> srid, dd::Column::enum_hidden_type hidden,
     bool is_array_arg) {
   uint sign_len, allowed_type_modifier = 0;
   ulong max_field_charlength = MAX_FIELD_CHARLENGTH;
 
   DBUG_TRACE;
 
-  assert(!(has_explicit_collation && fld_charset == nullptr));
+  DBUG_ASSERT(!(has_explicit_collation && fld_charset == nullptr));
 
   field = nullptr;
   field_name = fld_name;
@@ -263,7 +254,7 @@ bool Create_field::init(
   change = fld_change;
   interval = nullptr;
   geom_type = static_cast<Field::geometry_type>(fld_geom_type);
-  interval_list.clear();
+  interval_list.empty();
 
   comment = *fld_comment;
   gcol_info = fld_gcol_info;
@@ -279,7 +270,7 @@ bool Create_field::init(
   // Initialize data for a virtual field or default value expression
   if (gcol_info || m_default_val_expr) {
     if (gcol_info) {
-      assert(gcol_info->expr_item);
+      DBUG_ASSERT(gcol_info->expr_item);
       stored_in_db = gcol_info->get_field_stored();
 
       /*
@@ -322,8 +313,7 @@ bool Create_field::init(
     const ulonglong ull_length =
         my_strtoull(display_width_in_codepoints, nullptr, 10);
     if ((errno != 0) || (ull_length > MAX_FIELD_BLOBLENGTH)) {
-      my_error(ER_TOO_BIG_DISPLAYWIDTH, MYF(0), fld_name,
-               static_cast<unsigned long>(MAX_FIELD_BLOBLENGTH));
+      my_error(ER_TOO_BIG_DISPLAYWIDTH, MYF(0), fld_name, MAX_FIELD_BLOBLENGTH);
       return true;
     }
     m_max_display_width_in_codepoints = static_cast<size_t>(ull_length);
@@ -362,8 +352,6 @@ bool Create_field::init(
       allowed_type_modifier = AUTO_INCREMENT_FLAG;
       break;
     case MYSQL_TYPE_NULL:
-    case MYSQL_TYPE_INVALID:
-    case MYSQL_TYPE_BOOL:
       break;
     case MYSQL_TYPE_NEWDECIMAL: {
       ulong precision = static_cast<ulong>(m_max_display_width_in_codepoints);
@@ -436,8 +424,9 @@ bool Create_field::init(
       break;
     case MYSQL_TYPE_FLOAT:
       /* change FLOAT(precision) to FLOAT or DOUBLE */
+      allowed_type_modifier = AUTO_INCREMENT_FLAG;
       if (display_width_in_codepoints && !fld_decimals) {
-        const size_t tmp_length = m_max_display_width_in_codepoints;
+        size_t tmp_length = m_max_display_width_in_codepoints;
         if (tmp_length > PRECISION_FOR_DOUBLE) {
           my_error(ER_WRONG_FIELD_SPEC, MYF(0), fld_name);
           return true;
@@ -460,6 +449,7 @@ bool Create_field::init(
       }
       break;
     case MYSQL_TYPE_DOUBLE:
+      allowed_type_modifier = AUTO_INCREMENT_FLAG;
       if (!display_width_in_codepoints && !fld_decimals) {
         m_max_display_width_in_codepoints = DBL_DIG + 7;
         decimals = DECIMAL_NOT_SPECIFIED;
@@ -473,7 +463,7 @@ bool Create_field::init(
     case MYSQL_TYPE_TIMESTAMP:
       /* Add flags for TIMESTAMP for 4.0 MYD and 4.0 InnoDB compatibility */
       flags |= ZEROFILL_FLAG | UNSIGNED_FLAG;
-      [[fallthrough]];
+      /* Fall through */
     case MYSQL_TYPE_TIMESTAMP2:
       if (display_width_in_codepoints == nullptr) {
         m_max_display_width_in_codepoints =
@@ -484,6 +474,7 @@ bool Create_field::init(
           and 19 as length of 4.1 compatible representation.  Silently
           shrink it to MAX_DATETIME_COMPRESSED_WIDTH.
         */
+        DBUG_ASSERT(MAX_DATETIME_COMPRESSED_WIDTH < UINT_MAX);
         if (m_max_display_width_in_codepoints !=
             UINT_MAX) /* avoid overflow; is safe because of min() */
           m_max_display_width_in_codepoints =
@@ -502,7 +493,7 @@ bool Create_field::init(
     case MYSQL_TYPE_DATE:
       /* Old date type. */
       sql_type = MYSQL_TYPE_NEWDATE;
-      [[fallthrough]];
+      /* fall through */
     case MYSQL_TYPE_NEWDATE:
       m_max_display_width_in_codepoints = MAX_DATE_WIDTH;
       break;
@@ -524,7 +515,7 @@ bool Create_field::init(
       break;
     }
     case MYSQL_TYPE_VAR_STRING:
-      assert(0); /* Impossible. */
+      DBUG_ASSERT(0); /* Impossible. */
       break;
     case MYSQL_TYPE_BIT: {
       if (!display_width_in_codepoints) {
@@ -540,7 +531,7 @@ bool Create_field::init(
     }
     case MYSQL_TYPE_DECIMAL:
     default:
-      assert(0); /* Was obsolete */
+      DBUG_ASSERT(0); /* Was obsolete */
   }
 
   if (!(flags & BLOB_FLAG) &&
@@ -568,9 +559,9 @@ bool Create_field::init(
     can't have AUTO_INCREMENT and DEFAULT/ON UPDATE CURRENT_TIMESTAMP at the
     same time.
   */
-  assert(!((auto_flags & (Field::DEFAULT_NOW | Field::ON_UPDATE_NOW |
-                          Field::GENERATED_FROM_EXPRESSION)) != 0 &&
-           (auto_flags & Field::NEXT_NUMBER) != 0));
+  DBUG_ASSERT(!((auto_flags & (Field::DEFAULT_NOW | Field::ON_UPDATE_NOW |
+                               Field::GENERATED_FROM_EXPRESSION)) != 0 &&
+                (auto_flags & Field::NEXT_NUMBER) != 0));
 
   return false; /* success */
 }
@@ -597,7 +588,7 @@ void Create_field::init_for_tmp_table(enum_field_types sql_type_arg,
   DBUG_PRINT("enter", ("sql_type: %d, length: %u", sql_type_arg, length_arg));
 
   /* Init members needed for correct execution of make_field(). */
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   const uint32 FIELDFLAG_MAX_DEC = 31;
 #endif
 
@@ -611,11 +602,11 @@ void Create_field::init_for_tmp_table(enum_field_types sql_type_arg,
       break;
 
     case MYSQL_TYPE_NEWDECIMAL:
-      assert(decimals_arg <= DECIMAL_MAX_SCALE);
+      DBUG_ASSERT(decimals_arg <= DECIMAL_MAX_SCALE);
     case MYSQL_TYPE_DECIMAL:
     case MYSQL_TYPE_FLOAT:
     case MYSQL_TYPE_DOUBLE:
-      assert(decimals_arg <= FIELDFLAG_MAX_DEC);
+      DBUG_ASSERT(decimals_arg <= FIELDFLAG_MAX_DEC);
       decimals = decimals_arg;
       break;
 
@@ -637,8 +628,8 @@ void Create_field::init_for_tmp_table(enum_field_types sql_type_arg,
 
 size_t Create_field::max_display_width_in_codepoints() const {
   if (sql_type == MYSQL_TYPE_ENUM || sql_type == MYSQL_TYPE_SET) {
-    assert(interval != nullptr);
-    assert(charset != nullptr);
+    DBUG_ASSERT(interval != nullptr);
+    DBUG_ASSERT(charset != nullptr);
 
     size_t max_display_width_in_codepoints = 0;
     for (size_t i = 0; i < interval->count; i++) {
@@ -664,19 +655,19 @@ size_t Create_field::max_display_width_in_codepoints() const {
     return std::min(max_display_width_in_codepoints,
                     static_cast<size_t>(MAX_FIELD_WIDTH - 1));
   } else if (sql_type == MYSQL_TYPE_TINY_BLOB) {
-    return Field::MAX_TINY_BLOB_WIDTH / charset->mbmaxlen;
+    return TINYBLOB_MAX_SIZE_IN_BYTES / charset->mbmaxlen;
   } else if (sql_type == MYSQL_TYPE_BLOB && !explicit_display_width()) {
     // For BLOB and TEXT, the user can give a display width explicitly in CREATE
     // TABLE (BLOB(25), TEXT(25)) where the expected behavior is that the server
     // will find the smallest possible BLOB/TEXT type that will fit the given
     // display width. If the user has given an explicit display width, return
     // that instead of the max BLOB size.
-    return Field::MAX_SHORT_BLOB_WIDTH / charset->mbmaxlen;
+    return BLOB_MAX_SIZE_IN_BYTES / charset->mbmaxlen;
   } else if (sql_type == MYSQL_TYPE_MEDIUM_BLOB) {
-    return Field::MAX_MEDIUM_BLOB_WIDTH / charset->mbmaxlen;
+    return MEDIUMBLOB_MAX_SIZE_IN_BYTES / charset->mbmaxlen;
   } else if (sql_type == MYSQL_TYPE_LONG_BLOB || sql_type == MYSQL_TYPE_JSON ||
              sql_type == MYSQL_TYPE_GEOMETRY) {
-    return Field::MAX_LONG_BLOB_WIDTH / charset->mbmaxlen;
+    return LONGBLOB_MAX_SIZE_IN_BYTES / charset->mbmaxlen;
   } else {
     return m_max_display_width_in_codepoints;
   }
@@ -690,36 +681,35 @@ size_t Create_field::max_display_width_in_bytes() const {
   // max_display_width_in_codepoints() will return 1073741823 (truncated from
   // 1073741823.75), and multiplying that by four again will give 4294967292
   // which is the wrong result.
-  assert(charset != nullptr);
+  DBUG_ASSERT(charset != nullptr);
   if (is_numeric_type(sql_type) || is_temporal_real_type(sql_type) ||
       sql_type == MYSQL_TYPE_YEAR || sql_type == MYSQL_TYPE_BIT) {
     // Numeric types, temporal types, YEAR or BIT are never multi-byte.
     return max_display_width_in_codepoints();
   } else if (sql_type == MYSQL_TYPE_TINY_BLOB) {
-    return Field::MAX_TINY_BLOB_WIDTH;
+    return TINYBLOB_MAX_SIZE_IN_BYTES;
   } else if (sql_type == MYSQL_TYPE_BLOB && !explicit_display_width()) {
     // For BLOB and TEXT, the user can give a display width (BLOB(25), TEXT(25))
     // where the expected behavior is that the server will find the smallest
     // possible BLOB/TEXT type that will fit the given display width. If the
     // user has given an explicit display width, return that instead of the
     // max BLOB size.
-    return Field::MAX_SHORT_BLOB_WIDTH;
+    return BLOB_MAX_SIZE_IN_BYTES;
   } else if (sql_type == MYSQL_TYPE_MEDIUM_BLOB) {
-    return Field::MAX_MEDIUM_BLOB_WIDTH;
+    return MEDIUMBLOB_MAX_SIZE_IN_BYTES;
   } else if (sql_type == MYSQL_TYPE_LONG_BLOB || sql_type == MYSQL_TYPE_JSON ||
              sql_type == MYSQL_TYPE_GEOMETRY) {
-    return Field::MAX_LONG_BLOB_WIDTH;
+    return LONGBLOB_MAX_SIZE_IN_BYTES;
   } else {
     // If the user has given a display width to the TEXT type where the display
     // width is 2^32-1, the below computation will exceed
-    // MAX_LONG_BLOB_WIDTH if the character set is multi-byte. So we must
+    // LONGBLOB_MAX_SIZE_IN_BYTES if the character set is multi-byte. So we must
     // ensure that we never return a value greater than
-    // MAX_LONG_BLOB_WIDTH.
-    const std::int64_t display_width =
-        max_display_width_in_codepoints() *
-        static_cast<std::int64_t>(charset->mbmaxlen);
+    // LONGBLOB_MAX_SIZE_IN_BYTES.
+    std::int64_t display_width = max_display_width_in_codepoints() *
+                                 static_cast<std::int64_t>(charset->mbmaxlen);
     return static_cast<size_t>(std::min(
-        display_width, static_cast<std::int64_t>(Field::MAX_LONG_BLOB_WIDTH)));
+        display_width, static_cast<std::int64_t>(LONGBLOB_MAX_SIZE_IN_BYTES)));
   }
 }
 
@@ -736,7 +726,7 @@ size_t Create_field::pack_length(bool dont_override) const {
                                                       : interval->count);
     }
     case MYSQL_TYPE_NEWDECIMAL: {
-      assert(decimals <= DECIMAL_MAX_SCALE);
+      DBUG_ASSERT(decimals <= DECIMAL_MAX_SCALE);
       uint precision = my_decimal_length_to_precision(
           max_display_width_in_bytes(), decimals, (flags & UNSIGNED_FLAG));
       precision = std::min(precision, static_cast<uint>(DECIMAL_MAX_PRECISION));
@@ -788,12 +778,3 @@ size_t Create_field::key_length() const {
 bool is_field_for_functional_index(const Create_field *create_field) {
   return create_field->hidden == dd::Column::enum_hidden_type::HT_HIDDEN_SQL;
 }
-
-bool is_hidden_by_system(const Create_field *create_field) {
-  return create_field->hidden == dd::Column::enum_hidden_type::HT_HIDDEN_SE ||
-         create_field->hidden == dd::Column::enum_hidden_type::HT_HIDDEN_SQL;
-}
-
-bool is_hidden_by_user(const Create_field *create_field) {
-  return create_field->hidden == dd::Column::enum_hidden_type::HT_HIDDEN_USER;
-}
