@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -32,7 +32,6 @@
 #include <stdint.h>
 #include <sys/types.h>
 #include <algorithm>
-#include <functional>
 #include <string>
 #include <vector>
 
@@ -57,7 +56,6 @@
 #include "sql/dd/types/abstract_table.h"
 #include "sql/dd/types/column.h"      // dd::Column
 #include "sql/dd/types/index.h"       // dd::Index
-#include "sql/dd/types/partition.h"   // dd::Partition
 #include "sql/dd/types/schema.h"      // dd::Schema
 #include "sql/dd/types/table.h"       // dd::Table
 #include "sql/dd/types/tablespace.h"  // dd::Tablespace
@@ -103,7 +101,7 @@ const dd::String_type empty_ = "";
 
 char *generic_buf_handle(Byte_buffer *buf, size_t sz) {
   if (buf->reserve(sz)) {
-    assert(false);
+    DBUG_ASSERT(false);
     return nullptr;
   }
   return &(*(buf->begin()));
@@ -185,8 +183,9 @@ String_type generic_serialize(THD *thd, const char *dd_object_type,
   return (wctx.error() ? empty_ : String_type(buf.GetString(), buf.GetSize()));
 }
 
-const String_type &lookup_tablespace_name(Sdi_wcontext *wctx [[maybe_unused]],
-                                          dd::Object_id id [[maybe_unused]]) {
+const String_type &lookup_tablespace_name(
+    Sdi_wcontext *wctx MY_ATTRIBUTE((unused)),
+    dd::Object_id id MY_ATTRIBUTE((unused))) {
   if (wctx->thd() == nullptr || id == INVALID_OBJECT_ID) {
     return empty_;
   }
@@ -217,7 +216,7 @@ const String_type &lookup_tablespace_name(Sdi_wcontext *wctx [[maybe_unused]],
     wctx->set_error();
     return empty_;
   }
-  assert(tsp != nullptr);
+  DBUG_ASSERT(tsp != nullptr);
 
   return tsp->name();
 }
@@ -276,7 +275,7 @@ class Sdi_rcontext {
 
 template <typename T>
 void generic_track_object(dd_vector<T *> *tvp, T *t) {
-  assert(t->ordinal_position() > 0);
+  DBUG_ASSERT(t->ordinal_position() > 0);
   uint opx = t->ordinal_position() - 1;
   dd_vector<T *> &tv = *tvp;
 
@@ -350,7 +349,7 @@ bool lookup_tablespace_ref(Sdi_rcontext *sdictx, const String_type &name,
                                         name, idp);
 }
 
-/** @{ */  // sdi_cc_internal
+/** @} */  // sdi_cc_internal
 
 /**
   @defgroup sdi_api SDI API
@@ -370,60 +369,11 @@ Sdi_type serialize(const Tablespace &tablespace) {
                            nullptr);
 }
 
-/**
-   Default checker which implements the traditional (strict)
-   compatibility check: MYSQL_VERSION less than or equal, dd_version
-   equal, and sdi_version equal.
-*/
-bool CheckDefaultCompatibility(const RJ_Document &doc __attribute__((unused))) {
-  assert(doc.HasMember("mysqld_version_id"));
-#if defined(XTRABACKUP)
-  assert(doc["mysqld_version_id"].IsUint64());
-  assert(doc.HasMember("dd_version"));
-  assert(doc["dd_version"].IsUint());
-  assert(doc.HasMember("sdi_version"));
-  assert(doc["sdi_version"].IsUint64());
-#else
-  const RJ_Value &mysqld_version_id = doc["mysqld_version_id"];
-  assert(mysqld_version_id.IsUint64());
-  if (mysqld_version_id.GetUint64() > std::uint64_t(MYSQL_VERSION_ID)) {
-    // Cannot deserialize SDIs from newer versions.
-    my_error(ER_IMP_INCOMPATIBLE_MYSQLD_VERSION, MYF(0),
-             mysqld_version_id.GetUint64(), std::uint64_t(MYSQL_VERSION_ID));
-    return true;
-  }
-
-  assert(doc.HasMember("dd_version"));
-  const RJ_Value &dd_version_val = doc["dd_version"];
-  assert(dd_version_val.IsUint());
-  uint dd_version = dd_version_val.GetUint();
-  if (dd_version != Dictionary_impl::get_target_dd_version()) {
-    // Incompatible change
-    my_error(ER_IMP_INCOMPATIBLE_DD_VERSION, MYF(0), dd_version,
-             Dictionary_impl::get_target_dd_version());
-    return true;
-  }
-
-  assert(doc.HasMember("sdi_version"));
-  const RJ_Value &sdi_version_val = doc["sdi_version"];
-  assert(sdi_version_val.IsUint64());
-  std::uint64_t sdi_version_ = sdi_version_val.GetUint64();
-  if (sdi_version_ != SDI_VERSION) {
-    // Incompatible change
-    my_error(ER_IMP_INCOMPATIBLE_SDI_VERSION, MYF(0), sdi_version_,
-             SDI_VERSION);
-    return true;
-  }
-#endif /* !XTRABACKUP */
-  return false;
-}
-
 template <class Dd_type>
-bool generic_deserialize(THD *thd, const Sdi_type &sdi,
-                         const String_type &object_type_name [[maybe_unused]],
-                         Dd_type *dst,
-                         const SdiCompatibilityChecker &comp_checker,
-                         String_type *schema_name_from_sdi) {
+bool generic_deserialize(
+    THD *thd, const Sdi_type &sdi,
+    const String_type &object_type_name MY_ATTRIBUTE((unused)), Dd_type *dst,
+    String_type *schema_name_from_sdi = nullptr) {
   RJ_Document doc;
   doc.Parse<0>(sdi.c_str());
   if (doc.HasParseError()) {
@@ -432,24 +382,58 @@ bool generic_deserialize(THD *thd, const Sdi_type &sdi,
     return true;
   }
 
-  if (comp_checker(doc)) {
-    return checked_return(true);
+  if (doc.HasMember("mysqld_version_id")) {
+#if !defined(XTRABACKUP)
+    RJ_Value &mysqld_version_id = doc["mysqld_version_id"];
+    DBUG_ASSERT(mysqld_version_id.IsUint64());
+    if (mysqld_version_id.GetUint64() > std::uint64_t(MYSQL_VERSION_ID)) {
+      // Cannot deserialize SDIs from newer versions. Required?
+      my_error(ER_IMP_INCOMPATIBLE_MYSQLD_VERSION, MYF(0),
+               mysqld_version_id.GetUint64(), std::uint64_t(MYSQL_VERSION_ID));
+      return true;
+    }
+#endif
+  } else {
+    DBUG_ASSERT(false);
   }
 
-  assert(doc.HasMember("dd_object_type"));
+  DBUG_ASSERT(doc.HasMember("dd_version"));
+  RJ_Value &dd_version_val = doc["dd_version"];
+  DBUG_ASSERT(dd_version_val.IsUint());
+  uint dd_version = dd_version_val.GetUint();
+#if !defined(XTRABACKUP)
+  if (dd_version != Dictionary_impl::get_target_dd_version()) {
+    // Incompatible change
+    my_error(ER_IMP_INCOMPATIBLE_DD_VERSION, MYF(0), dd_version,
+             Dictionary_impl::get_target_dd_version());
+    return true;
+  }
+#endif
+
+  DBUG_ASSERT(doc.HasMember("sdi_version"));
+  RJ_Value &sdi_version_val = doc["sdi_version"];
+  DBUG_ASSERT(sdi_version_val.IsUint64());
+  std::uint64_t sdi_version_ = sdi_version_val.GetUint64();
+#if !defined(XTRABACKUP)
+  if (sdi_version_ != SDI_VERSION) {
+    // Incompatible change
+    my_error(ER_IMP_INCOMPATIBLE_SDI_VERSION, MYF(0), sdi_version_,
+             SDI_VERSION);
+    return true;
+  }
+#endif
+
+  DBUG_ASSERT(doc.HasMember("dd_object_type"));
   RJ_Value &dd_object_type_val = doc["dd_object_type"];
-  assert(dd_object_type_val.IsString());
+  DBUG_ASSERT(dd_object_type_val.IsString());
   String_type dd_object_type(dd_object_type_val.GetString());
-  assert(dd_object_type == object_type_name);
+  DBUG_ASSERT(dd_object_type == object_type_name);
 
-  assert(doc.HasMember("dd_object"));
+  DBUG_ASSERT(doc.HasMember("dd_object"));
   RJ_Value &dd_object_val = doc["dd_object"];
-  assert(dd_object_val.IsObject());
+  DBUG_ASSERT(dd_object_val.IsObject());
 
-  assert(doc.HasMember("dd_version"));
-  assert(doc.HasMember("sdi_version"));
-  Sdi_rcontext rctx(thd, doc["dd_version"].GetUint(),
-                    doc["sdi_version"].GetUint());
+  Sdi_rcontext rctx(thd, dd_version, sdi_version_);
   if (dst->deserialize(&rctx, dd_object_val)) {
     return checked_return(true);
   }
@@ -460,52 +444,13 @@ bool generic_deserialize(THD *thd, const Sdi_type &sdi,
   return false;
 }
 
-/**
-  Deserialize a dd::Table object.
-
-  Populates the dd::Table object provided with data from sdi string.
-  Note! Additional objects are dynamically allocated and added to the
-  top-level Schema object, which assumes ownership.
-
-  @param thd thread context
-  @param sdi  serialized representation of schema (as a json string)
-  @param dst_table empty top-level object
-  @param comp_checker callable which will be used to determine if the SDI is
-         compatible
-  @param deser_schema_name name of schema containing the table
-
-  @return error status
-    @retval false if successful
-    @retval true otherwise
-*/
 bool deserialize(THD *thd, const Sdi_type &sdi, Table *dst_table,
-                 SdiCompatibilityChecker comp_checker,
                  String_type *deser_schema_name) {
-  return generic_deserialize(thd, sdi, "Table", dst_table, comp_checker,
-                             deser_schema_name);
+  return generic_deserialize(thd, sdi, "Table", dst_table, deser_schema_name);
 }
 
-/**
-  Deserialize a dd::Tablespace object.
-
-  Populates the dd::Tablespace object provided with data from sdi string.
-  Note! Additional objects are dynamically allocated and added to the
-  top-level Tablespace object, which assumes ownership.
-
-  @param thd thread context
-  @param sdi  serialized representation of schema (as a json string)
-  @param dst_tablespace empty top-level object
-  @param comp_checker callable which will be used to determine if the SDI is
-         compatible.
-
-  @return error status
-    @retval false if successful
-    @retval true otherwise
-*/
-bool deserialize(THD *thd, const Sdi_type &sdi, Tablespace *dst_tablespace,
-                 SdiCompatibilityChecker comp_checker) {
-  return generic_deserialize(thd, sdi, "Tablespace", dst_tablespace,
-                             comp_checker, nullptr);
+bool deserialize(THD *thd, const Sdi_type &sdi, Tablespace *dst_tablespace) {
+  return generic_deserialize(thd, sdi, "Tablespace", dst_tablespace);
 }
 
 namespace {
@@ -513,7 +458,7 @@ namespace {
   Templated convenience wrapper which first attempts to resolve the
   handlerton using the data dictionary object's engine() string.
 
-  @param thd    thread context
+  @param thd
   @param ddt    Data dictionary object
 
   @return handlerton pointer for this object
@@ -534,7 +479,7 @@ static handlerton *resolve_hton(THD *thd, const DDT &ddt) {
   Covenience function for acquiring the schema and invoking a closure
   which uses the schema object.
 
-  @param thd thread context
+  @param thd
   @param key key to use when acquiring Schema object
   @param clos closure to invoke with the Schema object
   @return error status
@@ -579,7 +524,7 @@ bool equal_prefix_chars(CHAR_IT &&begin1, CHAR_IT &&end1, CHAR_IT &&begin2,
     }
     if (rem_bytes == 0) {
       rem_bytes = my_mbcharlen(csi, static_cast<uchar>(*begin1));
-      assert(rem_bytes > 0);
+      DBUG_ASSERT(rem_bytes > 0);
     }
     --rem_bytes;
 
@@ -677,27 +622,9 @@ bool drop_after_update(THD *thd, const Table *old_tp, const Table *new_tp) {
   });
 }
 
-namespace {
-template <typename DDT>
-bool drop_all_impl(THD *thd, const DDT *tp) {
-  const DDT &t = ptr_as_cref(tp);
-  const handlerton &hton = ptr_as_cref(resolve_hton(thd, t));
-  assert(hton.sdi_get != nullptr && hton.sdi_delete != nullptr);
-
-  return checked_return(sdi_tablespace::drop_all_sdi(thd, hton, t));
-}
-}  // namespace
-
-bool drop_all_for_table(THD *thd, const Table *tp) {
-  return drop_all_impl(thd, tp);
-}
-
-bool drop_all_for_part(THD *thd, const Partition *pp) {
-  return drop_all_impl(thd, pp);
-}
 }  // namespace sdi
 }  // namespace dd
-/** @{ */  // end of group sdi_api
+/** @} */  // end of group sdi_api
 
 /**
   @defgroup sdi_ut SDI Unit-testing API
@@ -706,6 +633,7 @@ bool drop_all_for_part(THD *thd, const Partition *pp) {
   Special functions used by unit tests but which are not available in
   the normal api.
 
+  @{
 */
 
 /**
