@@ -1,4 +1,4 @@
-/* Copyright (c) 2018, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2018, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -52,7 +52,7 @@ static std::string to_string(const gis::Unit_Type unit_type) {
   if (unit_type == gis::Unit_Type::kLinear) {
     return "LINEAR";
   } else {
-    assert(false);
+    DBUG_ASSERT(false);
     return "";
   }
 }
