@@ -1,4 +1,4 @@
-/* Copyright (c) 2014, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -22,7 +22,6 @@
 
 #include "sql/dd/impl/types/column_impl.h"
 
-#include <assert.h>
 #include <stddef.h>
 #include <memory>
 #include <set>
@@ -34,15 +33,16 @@
 #include <rapidjson/document.h>
 #include <rapidjson/prettywriter.h>
 
+#include "m_string.h"
+#include "my_dbug.h"
 #include "my_inttypes.h"
 #include "my_sys.h"
-#include "mysqld_error.h"                         // ER_*
-#include "sql/dd/impl/bootstrap/bootstrap_ctx.h"  // dd::bootstrap::DD_bootstrap_ctx
-#include "sql/dd/impl/properties_impl.h"          // Properties_impl
-#include "sql/dd/impl/raw/raw_record.h"           // Raw_record
-#include "sql/dd/impl/sdi_impl.h"                 // sdi read/write functions
+#include "mysqld_error.h"                 // ER_*
+#include "sql/dd/impl/properties_impl.h"  // Properties_impl
+#include "sql/dd/impl/raw/raw_record.h"   // Raw_record
+#include "sql/dd/impl/sdi_impl.h"         // sdi read/write functions
 #include "sql/dd/impl/tables/column_type_elements.h"  // Column_type_elements
-#include "sql/dd/impl/tables/columns.h"               // Columns
+#include "sql/dd/impl/tables/columns.h"               // Colummns
 #include "sql/dd/impl/transaction_impl.h"  // Open_dictionary_tables_ctx
 #include "sql/dd/impl/types/abstract_table_impl.h"       // Abstract_table_impl
 #include "sql/dd/impl/types/column_type_element_impl.h"  // Column_type_element_impl
@@ -51,7 +51,6 @@
 #include "sql/dd/types/column_type_element.h"  // Column_type_element
 #include "sql/dd/types/object_table.h"
 #include "sql/dd/types/weak_object.h"
-#include "string_with_len.h"
 
 using dd::tables::Column_type_elements;
 using dd::tables::Columns;
@@ -63,9 +62,8 @@ class Sdi_rcontext;
 class Sdi_wcontext;
 
 static const std::set<String_type> default_valid_option_keys = {
-    "column_format", "geom_type", "interval_count",
-    "not_secondary", "storage",   "treat_bit_as_char",
-    "is_array",      "gipk" /* generated implicit primary key column */};
+    "column_format", "geom_type",         "interval_count", "not_secondary",
+    "storage",       "treat_bit_as_char", "is_array"};
 
 ///////////////////////////////////////////////////////////////////////////
 // Column_impl implementation.
@@ -123,7 +121,7 @@ Column_impl::Column_impl(Abstract_table_impl *table)
       m_is_explicit_collation(false),
       m_column_key(CK_NONE) {}
 
-Column_impl::~Column_impl() = default;
+Column_impl::~Column_impl() {}
 
 ///////////////////////////////////////////////////////////////////////////
 
@@ -234,14 +232,6 @@ bool Column_impl::restore_attributes(const Raw_record &r) {
   set_options(r.read_str(Columns::FIELD_OPTIONS, ""));
   set_se_private_data(r.read_str(Columns::FIELD_SE_PRIVATE_DATA, ""));
 
-  // m_engine_attribute and m_secondary_engine_attribute added in 80021
-  if (!bootstrap::DD_bootstrap_ctx::instance().is_dd_upgrade_from_before(
-          bootstrap::DD_VERSION_80021)) {
-    m_engine_attribute = r.read_str(Columns::FIELD_ENGINE_ATTRIBUTE, "");
-    m_secondary_engine_attribute =
-        r.read_str(Columns::FIELD_SECONDARY_ENGINE_ATTRIBUTE, "");
-  }
-
   set_default_option(r.read_str(Columns::FIELD_DEFAULT_OPTION, ""));
   set_update_option(r.read_str(Columns::FIELD_UPDATE_OPTION, ""));
 
@@ -271,18 +261,6 @@ bool Column_impl::store_attributes(Raw_record *r) {
   // attribute doesn't make sense for the type ? E.g. "unsigned" for
   // VARCHAR column
 
-  // Store engine_attribute and secondary_engine_attribute only if we're
-  // not upgrading
-  if (!bootstrap::DD_bootstrap_ctx::instance().is_dd_upgrade_from_before(
-          bootstrap::DD_VERSION_80021) &&
-      (r->store(Columns::FIELD_ENGINE_ATTRIBUTE, m_engine_attribute,
-                m_engine_attribute.empty()) ||
-       r->store(Columns::FIELD_SECONDARY_ENGINE_ATTRIBUTE,
-                m_secondary_engine_attribute,
-                m_secondary_engine_attribute.empty()))) {
-    return true;
-  }
-
   return store_id(r, Columns::FIELD_ID) || store_name(r, Columns::FIELD_NAME) ||
          r->store(Columns::FIELD_TABLE_ID, m_table->id()) ||
          r->store(Columns::FIELD_ORDINAL_POSITION, m_ordinal_position) ||
@@ -328,9 +306,8 @@ bool Column_impl::store_attributes(Raw_record *r) {
 }
 
 ///////////////////////////////////////////////////////////////////////////
-static_assert(Columns::NUMBER_OF_FIELDS == 32,
-              "Columns definition has changed, check if serialize() and "
-              "deserialize() need to be updated!!");
+static_assert(Columns::FIELD_SE_PRIVATE_DATA == 25,
+              "Columns definition has changed, review (de)ser memfuns!");
 void Column_impl::serialize(Sdi_wcontext *wctx, Sdi_writer *w) const {
   w->StartObject();
   Entity_object_impl::serialize(wctx, w);
@@ -368,16 +345,11 @@ void Column_impl::serialize(Sdi_wcontext *wctx, Sdi_writer *w) const {
         STRING_WITH_LEN("generation_expression_utf8"));
   write_properties(w, m_options, STRING_WITH_LEN("options"));
   write_properties(w, m_se_private_data, STRING_WITH_LEN("se_private_data"));
-  write(w, m_engine_attribute, STRING_WITH_LEN("engine_attribute"));
-  write(w, m_secondary_engine_attribute,
-        STRING_WITH_LEN("secondary_engine_attribute"));
-
   write_enum(w, m_column_key, STRING_WITH_LEN("column_key"));
   write(w, m_column_type_utf8, STRING_WITH_LEN("column_type_utf8"));
   serialize_each(wctx, w, m_elements, STRING_WITH_LEN("elements"));
   write(w, m_collation_id, STRING_WITH_LEN("collation_id"));
   write(w, m_is_explicit_collation, STRING_WITH_LEN("is_explicit_collation"));
-
   w->EndObject();
 }
 
@@ -411,8 +383,6 @@ bool Column_impl::deserialize(Sdi_rcontext *rctx, const RJ_Value &val) {
   read(&m_generation_expression_utf8, val, "generation_expression_utf8");
   read_properties(&m_options, val, "options");
   read_properties(&m_se_private_data, val, "se_private_data");
-  read(&m_engine_attribute, val, "engine_attribute");
-  read(&m_secondary_engine_attribute, val, "secondary_engine_attribute");
   read_enum(&m_column_key, val, "column_key");
   read(&m_column_type_utf8, val, "column_type_utf8");
   read_enum(&m_hidden, val, "hidden");
@@ -470,9 +440,6 @@ void Column_impl::debug_print(String_type &outb) const {
      << "m_generation_expression_utf8: " << m_generation_expression_utf8 << "; "
      << "m_hidden: " << static_cast<int>(m_hidden) << "; "
      << "m_options: " << m_options.raw_string() << "; "
-     << "m_se_private_data: " << m_se_private_data.raw_string() << "; "
-     << "m_engine_attribute: " << m_engine_attribute << "; "
-     << "m_secondary_engine_attribute: " << m_secondary_engine_attribute << "; "
      << "m_column_key: " << m_column_key << "; "
      << "m_column_type_utf8: " << m_column_type_utf8 << "; "
      << "m_srs_id_null: " << !m_srs_id.has_value() << "; ";
@@ -501,7 +468,8 @@ void Column_impl::debug_print(String_type &outb) const {
 ///////////////////////////////////////////////////////////////////////////
 
 Column_type_element *Column_impl::add_element() {
-  assert(type() == enum_column_types::ENUM || type() == enum_column_types::SET);
+  DBUG_ASSERT(type() == enum_column_types::ENUM ||
+              type() == enum_column_types::SET);
 
   Column_type_element_impl *e =
       new (std::nothrow) Column_type_element_impl(this);
@@ -540,8 +508,6 @@ Column_impl::Column_impl(const Column_impl &src, Abstract_table_impl *parent)
       m_generation_expression_utf8(src.m_generation_expression_utf8),
       m_options(src.m_options),
       m_se_private_data(src.m_se_private_data),
-      m_engine_attribute(src.m_engine_attribute),
-      m_secondary_engine_attribute(src.m_secondary_engine_attribute),
       m_table(parent),
       m_elements(),
       m_column_type_utf8(src.m_column_type_utf8),
