@@ -1,16 +1,15 @@
-/* Copyright (c) 2014, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,7 +22,6 @@
 
 #include "sql/dd/impl/types/partition_impl.h"
 
-#include <assert.h>
 #include <stddef.h>
 #include <set>
 #include <sstream>
@@ -34,6 +32,8 @@
 #include <rapidjson/document.h>
 #include <rapidjson/prettywriter.h>
 
+#include "m_string.h"
+#include "my_dbug.h"
 #include "my_inttypes.h"
 #include "my_sys.h"
 #include "mysqld_error.h"                         // ER_*
@@ -51,7 +51,6 @@
 #include "sql/dd/types/partition_index.h"
 #include "sql/dd/types/partition_value.h"
 #include "sql/dd/types/weak_object.h"
-#include "string_with_len.h"
 
 using dd::tables::Index_partitions;
 using dd::tables::Table_partition_values;
@@ -115,7 +114,7 @@ Partition_impl::Partition_impl(Table_impl *table, Partition_impl *parent)
   m_table->add_leaf_partition(this);
 }
 
-Partition_impl::~Partition_impl() = default;
+Partition_impl::~Partition_impl() {}
 
 ///////////////////////////////////////////////////////////////////////////
 
@@ -253,9 +252,9 @@ bool Partition_impl::store_attributes(Raw_record *r) {
 }
 
 ///////////////////////////////////////////////////////////////////////////
-static_assert(Table_partitions::NUMBER_OF_FIELDS == 12,
-              "Table_partitions definition has changed, check if serialize() "
-              "and deserialize() need to be updated!");
+static_assert(
+    Table_partitions::FIELD_TABLESPACE_ID == 11,
+    "Table_partitions definition has changed. Review (de)ser memfuns!");
 void Partition_impl::serialize(Sdi_wcontext *wctx, Sdi_writer *w) const {
   w->StartObject();
   Entity_object_impl::serialize(wctx, w);
@@ -377,7 +376,7 @@ Partition_index *Partition_impl::add_index(Index *idx) {
 
 Partition *Partition_impl::add_subpartition() {
   /// Support just one level of sub partitions.
-  assert(!parent());
+  DBUG_ASSERT(!parent());
 
   Partition_impl *p =
       new (std::nothrow) Partition_impl(&this->table_impl(), this);
