@@ -1,16 +1,15 @@
-/* Copyright (c) 2014, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,9 +21,7 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 #include "sql/dd/impl/types/table_impl.h"
-#include "string_with_len.h"
 
-#include <assert.h>
 #include <string.h>
 #include <set>
 #include <sstream>
@@ -36,9 +33,8 @@
 #include <rapidjson/prettywriter.h>
 
 #include "m_string.h"
-
+#include "my_dbug.h"
 #include "my_sys.h"
-#include "mysql/strings/m_ctype.h"
 #include "mysqld_error.h"                         // ER_*
 #include "sql/current_thd.h"                      // current_thd
 #include "sql/dd/impl/bootstrap/bootstrap_ctx.h"  // dd::bootstrap::DD_bootstrap_ctx
@@ -136,14 +132,14 @@ bool Table_impl::load_foreign_key_parents(Open_dictionary_tables_ctx *otx) {
 
   // 1. Read the parent's schema name based on schema_id.
   Raw_table *schema_table = otx->get_table<dd::Schema>();
-  assert(schema_table);
+  DBUG_ASSERT(schema_table);
   Primary_id_key schema_pk(schema_id());
 
   std::unique_ptr<Raw_record_set> schema_rs;
   if (schema_table->open_record_set(&schema_pk, schema_rs)) return true;
 
   Raw_record *schema_rec = schema_rs->current_record();
-  assert(schema_rec);
+  DBUG_ASSERT(schema_rec);
   if (schema_rec == nullptr) return true;
 
   // 2. Build a key for searching the FK table.
@@ -157,7 +153,7 @@ bool Table_impl::load_foreign_key_parents(Open_dictionary_tables_ctx *otx) {
 
   // 3. Get the FK record set where this table is parent.
   Raw_table *foreign_key_table = otx->get_table<dd::Foreign_key>();
-  assert(foreign_key_table);
+  DBUG_ASSERT(foreign_key_table);
 
   std::unique_ptr<Raw_record_set> child_fk_rs;
   if (foreign_key_table->open_record_set(&parent_ref_key, child_fk_rs))
@@ -169,13 +165,13 @@ bool Table_impl::load_foreign_key_parents(Open_dictionary_tables_ctx *otx) {
     Primary_id_key child_pk(
         child_fk_rec->read_int(tables::Foreign_keys::FIELD_TABLE_ID));
     Raw_table *tables_table = otx->get_table<dd::Table>();
-    assert(tables_table);
+    DBUG_ASSERT(tables_table);
 
     std::unique_ptr<Raw_record_set> child_table_rs;
     if (tables_table->open_record_set(&child_pk, child_table_rs)) return true;
 
     Raw_record *child_table = child_table_rs->current_record();
-    assert(child_table);
+    DBUG_ASSERT(child_table);
     if (child_table == nullptr) return true;
 
     /*
@@ -197,7 +193,7 @@ bool Table_impl::load_foreign_key_parents(Open_dictionary_tables_ctx *otx) {
     if (schema_table->open_record_set(&schema_pk, schema_rs)) return true;
 
     schema_rec = schema_rs->current_record();
-    assert(schema_rec);
+    DBUG_ASSERT(schema_rec);
     if (schema_rec == nullptr) return true;
 
     // 6. Collect the relevant information.
@@ -238,7 +234,7 @@ bool Table_impl::reload_foreign_key_parents(THD *thd) {
   // Register and open tables.
   trx.otx.register_tables<dd::Table>();
   if (trx.otx.open_tables()) {
-    assert(thd->is_system_thread() || thd->killed || thd->is_error());
+    DBUG_ASSERT(thd->is_system_thread() || thd->killed || thd->is_error());
     return true;
   }
 
@@ -433,14 +429,6 @@ bool Table_impl::restore_attributes(const Raw_record &r) {
 
   set_se_private_data(r.read_str(Tables::FIELD_SE_PRIVATE_DATA, ""));
 
-  // m_engine_attribute and m_secondary_engine_attribute added in 80021
-  if (!bootstrap::DD_bootstrap_ctx::instance().is_dd_upgrade_from_before(
-          bootstrap::DD_VERSION_80021)) {
-    m_engine_attribute = r.read_str(Tables::FIELD_ENGINE_ATTRIBUTE, "");
-    m_secondary_engine_attribute =
-        r.read_str(Tables::FIELD_SECONDARY_ENGINE_ATTRIBUTE, "");
-  }
-
   m_engine = r.read_str(Tables::FIELD_ENGINE);
 
   // m_last_checked_for_upgrade_version added in 80012
@@ -486,7 +474,7 @@ bool Table_impl::store_attributes(Raw_record *r) {
   //
 
   // Temporary table definitions are never persisted.
-  assert(!m_is_temporary);
+  DBUG_ASSERT(!m_is_temporary);
 
   // Store last_checked_for_upgrade_version_id only if we're not upgrading
   if (!bootstrap::DD_bootstrap_ctx::instance().is_dd_upgrade_from_before(
@@ -496,18 +484,6 @@ bool Table_impl::store_attributes(Raw_record *r) {
     return true;
   }
 
-  // Store engine_attribute and secondary_engine_attribute only if
-  // we're not upgrading
-  if (!bootstrap::DD_bootstrap_ctx::instance().is_dd_upgrade_from_before(
-          bootstrap::DD_VERSION_80021) &&
-      (r->store(Tables::FIELD_ENGINE_ATTRIBUTE, m_engine_attribute,
-                m_engine_attribute.empty()) ||
-       r->store(Tables::FIELD_SECONDARY_ENGINE_ATTRIBUTE,
-                m_secondary_engine_attribute,
-                m_secondary_engine_attribute.empty()))) {
-    return true;
-  }
-
   // Store field values
   return Abstract_table_impl::store_attributes(r) ||
          r->store(Tables::FIELD_ENGINE, m_engine) ||
@@ -541,12 +517,10 @@ bool Table_impl::store_attributes(Raw_record *r) {
 }
 
 ///////////////////////////////////////////////////////////////////////////
-static_assert(Tables::NUMBER_OF_FIELDS == 37,
-              "Tables definition has changed, check if serialize() and "
-              "deserialize() need to be updated!");
+
 void Table_impl::serialize(Sdi_wcontext *wctx, Sdi_writer *w) const {
   // Temporary table definitions are never persisted.
-  assert(!m_is_temporary);
+  DBUG_ASSERT(!m_is_temporary);
 
   w->StartObject();
   Abstract_table_impl::serialize(wctx, w);
@@ -556,9 +530,6 @@ void Table_impl::serialize(Sdi_wcontext *wctx, Sdi_writer *w) const {
         STRING_WITH_LEN("last_checked_for_upgrade_version_id"));
   write(w, m_comment, STRING_WITH_LEN("comment"));
   write_properties(w, m_se_private_data, STRING_WITH_LEN("se_private_data"));
-  write(w, m_engine_attribute, STRING_WITH_LEN("engine_attribute"));
-  write(w, m_secondary_engine_attribute,
-        STRING_WITH_LEN("secondary_engine_attribute"));
   write_enum(w, m_row_format, STRING_WITH_LEN("row_format"));
   write_enum(w, m_partition_type, STRING_WITH_LEN("partition_type"));
   write(w, m_partition_expression, STRING_WITH_LEN("partition_expression"));
@@ -594,8 +565,6 @@ bool Table_impl::deserialize(Sdi_rcontext *rctx, const RJ_Value &val) {
        "last_checked_for_upgrade_version_id");
   read(&m_comment, val, "comment");
   read_properties(&m_se_private_data, val, "se_private_data");
-  read(&m_engine_attribute, val, "engine_attribute");
-  read(&m_secondary_engine_attribute, val, "secondary_engine_attribute");
   read_enum(&m_row_format, val, "row_format");
   read_enum(&m_partition_type, val, "partition_type");
   read(&m_partition_expression, val, "partition_expression");
@@ -608,7 +577,7 @@ bool Table_impl::deserialize(Sdi_rcontext *rctx, const RJ_Value &val) {
 
   // Note! Deserialization of ordinal position cross-referenced
   // objects (i.e. Index and Column) must happen before deserializing
-  // objects which reference these objects:
+  // objects which refrence these objects:
   // Foreign_key_element -> Column,
   // Foreign_key         -> Index,
   // Index_element       -> Column,
@@ -650,8 +619,6 @@ void Table_impl::debug_print(String_type &outb) const {
      << "m_comment: " << m_comment << "; "
      << "m_se_private_data " << m_se_private_data.raw_string() << "; "
      << "m_se_private_id: {OID: " << m_se_private_id << "}; "
-     << "m_engine_attribute: " << m_engine_attribute << "; "
-     << "m_secondary_engine_attribute: " << m_secondary_engine_attribute << "; "
      << "m_row_format: " << m_row_format << "; "
      << "m_is_temporary: " << m_is_temporary << "; "
      << "m_tablespace: {OID: " << m_tablespace_id << "}; "
@@ -863,8 +830,8 @@ const Trigger *Table_impl::get_trigger(const char *name) const {
 Trigger *Table_impl::add_trigger_following(const Trigger *trigger,
                                            Trigger::enum_action_timing at,
                                            Trigger::enum_event_type et) {
-  assert(trigger != nullptr && trigger->action_timing() == at &&
-         trigger->event_type() == et);
+  DBUG_ASSERT(trigger != nullptr && trigger->action_timing() == at &&
+              trigger->event_type() == et);
 
   // Allocate new Trigger object.
   Trigger_impl *new_trigger = create_trigger();
@@ -890,8 +857,8 @@ Trigger *Table_impl::add_trigger_following(const Trigger *trigger,
 Trigger *Table_impl::add_trigger_preceding(const Trigger *trigger,
                                            Trigger::enum_action_timing at,
                                            Trigger::enum_event_type et) {
-  assert(trigger != nullptr && trigger->action_timing() == at &&
-         trigger->event_type() == et);
+  DBUG_ASSERT(trigger != nullptr && trigger->action_timing() == at &&
+              trigger->event_type() == et);
 
   Trigger_impl *new_trigger = create_trigger();
   if (new_trigger == nullptr) return nullptr;
@@ -911,7 +878,7 @@ Trigger *Table_impl::add_trigger_preceding(const Trigger *trigger,
 ///////////////////////////////////////////////////////////////////////////
 
 void Table_impl::copy_triggers(const Table *tab_obj) {
-  assert(tab_obj != nullptr);
+  DBUG_ASSERT(tab_obj != nullptr);
 
   for (const Trigger *trig : tab_obj->triggers()) {
     /*
@@ -949,7 +916,7 @@ void Table_impl::copy_triggers(const Table *tab_obj) {
     */
     Trigger_impl *new_trigger =
         new Trigger_impl(*dynamic_cast<const Trigger_impl *>(trig), this);
-    assert(new_trigger != nullptr);
+    DBUG_ASSERT(new_trigger != nullptr);
 
     new_trigger->set_id(INVALID_OBJECT_ID);
 
@@ -964,7 +931,7 @@ void Table_impl::drop_all_triggers() { m_triggers.remove_all(); }
 ///////////////////////////////////////////////////////////////////////////
 
 void Table_impl::drop_trigger(const Trigger *trigger) {
-  assert(trigger != nullptr);
+  DBUG_ASSERT(trigger != nullptr);
   dd::Trigger::enum_action_timing at = trigger->action_timing();
   dd::Trigger::enum_event_type et = trigger->event_type();
 
@@ -1035,8 +1002,6 @@ Table_impl::Table_impl(const Table_impl &src)
       m_last_checked_for_upgrade_version_id{
           src.m_last_checked_for_upgrade_version_id},
       m_se_private_data(src.m_se_private_data),
-      m_engine_attribute(src.m_engine_attribute),
-      m_secondary_engine_attribute(src.m_secondary_engine_attribute),
       m_row_format(src.m_row_format),
       m_is_temporary(src.m_is_temporary),
       m_partition_type(src.m_partition_type),
