@@ -1,16 +1,15 @@
-/* Copyright (c) 2014, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -59,24 +58,24 @@ class Tablespace_file_impl : public Weak_object_impl, public Tablespace_file {
   Tablespace_file_impl(const Tablespace_file_impl &src,
                        Tablespace_impl *parent);
 
-  ~Tablespace_file_impl() override = default;
+  virtual ~Tablespace_file_impl() {}
 
  public:
-  const Object_table &object_table() const override;
+  virtual const Object_table &object_table() const;
 
-  bool store(Open_dictionary_tables_ctx *otx) override;
+  virtual bool store(Open_dictionary_tables_ctx *otx);
 
-  bool validate() const override;
+  virtual bool validate() const;
 
-  bool store_attributes(Raw_record *r) override;
+  virtual bool store_attributes(Raw_record *r);
 
-  bool restore_attributes(const Raw_record &r) override;
+  virtual bool restore_attributes(const Raw_record &r);
 
-  void serialize(Sdi_wcontext *wctx, Sdi_writer *w) const override;
+  void serialize(Sdi_wcontext *wctx, Sdi_writer *w) const;
 
-  bool deserialize(Sdi_rcontext *rctx, const RJ_Value &val) override;
+  bool deserialize(Sdi_rcontext *rctx, const RJ_Value &val);
 
-  void debug_print(String_type &outb) const override;
+  virtual void debug_print(String_type &outb) const;
 
   void set_ordinal_position(uint ordinal_position) {
     m_ordinal_position = ordinal_position;
@@ -89,15 +88,15 @@ class Tablespace_file_impl : public Weak_object_impl, public Tablespace_file {
   // ordinal_position.
   /////////////////////////////////////////////////////////////////////////
 
-  uint ordinal_position() const override { return m_ordinal_position; }
+  virtual uint ordinal_position() const { return m_ordinal_position; }
 
   /////////////////////////////////////////////////////////////////////////
   // filename.
   /////////////////////////////////////////////////////////////////////////
 
-  const String_type &filename() const override { return m_filename; }
+  virtual const String_type &filename() const { return m_filename; }
 
-  void set_filename(const String_type &filename) override {
+  virtual void set_filename(const String_type &filename) {
     m_filename = filename;
   }
 
@@ -105,13 +104,13 @@ class Tablespace_file_impl : public Weak_object_impl, public Tablespace_file {
   // se_private_data.
   /////////////////////////////////////////////////////////////////////////
 
-  const Properties &se_private_data() const override {
+  virtual const Properties &se_private_data() const {
     return m_se_private_data;
   }
 
-  Properties &se_private_data() override { return m_se_private_data; }
+  virtual Properties &se_private_data() { return m_se_private_data; }
 
-  bool set_se_private_data(const String_type &se_private_data_raw) override {
+  virtual bool set_se_private_data(const String_type &se_private_data_raw) {
     return m_se_private_data.insert_values(se_private_data_raw);
   }
 
@@ -119,9 +118,9 @@ class Tablespace_file_impl : public Weak_object_impl, public Tablespace_file {
   // tablespace.
   /////////////////////////////////////////////////////////////////////////
 
-  const Tablespace &tablespace() const override;
+  virtual const Tablespace &tablespace() const;
 
-  Tablespace &tablespace() override;
+  virtual Tablespace &tablespace();
 
  public:
   static Tablespace_file_impl *restore_item(Tablespace_impl *ts) {
@@ -134,8 +133,8 @@ class Tablespace_file_impl : public Weak_object_impl, public Tablespace_file {
   }
 
  public:
-  Object_key *create_primary_key() const override;
-  bool has_new_primary_key() const override;
+  virtual Object_key *create_primary_key() const;
+  virtual bool has_new_primary_key() const;
 
  private:
   // Fields
