@@ -1,16 +1,15 @@
-/* Copyright (c) 2019, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,10 +24,9 @@
 #include "sql/dd/impl/bootstrap/bootstrapper.h"
 #include "sql/dd/impl/utils.h"
 
-#include <assert.h>
-#include <regex>
 #include <set>
 
+#include "my_dbug.h"
 #include "mysql/components/services/log_builtins.h"
 #include "mysql_version.h"  // MYSQL_VERSION_ID
 #include "mysqld_error.h"
@@ -36,7 +34,6 @@
 #include "sql/dd/impl/bootstrap/bootstrap_ctx.h"        // DD_bootstrap_ctx
 #include "sql/dd/impl/cache/shared_dictionary_cache.h"  // Shared_dictionary_cache
 #include "sql/dd/impl/system_registry.h"                // dd::System_tables
-#include "sql/dd/impl/tables/columns.h"                 // dd::tables::Columns
 #include "sql/dd/impl/tables/dd_properties.h"  // dd::tables::DD_properties
 #include "sql/dd/impl/tables/events.h"         // dd::tables::Events
 #include "sql/dd/impl/tables/foreign_key_column_usage.h"  // dd::tables::Fore...
@@ -76,7 +73,7 @@ bool create_temporary_schemas(THD *thd, Object_id *mysql_schema_id,
   const dd::Schema *schema = nullptr;
   std::stringstream ss;
 
-  assert(target_table_schema_name != nullptr);
+  DBUG_ASSERT(target_table_schema_name != nullptr);
   *target_table_schema_name = String_type("");
   ss << "dd_upgrade_targets_" << MYSQL_VERSION_ID;
   String_type tmp_schema_name_base{ss.str().c_str()};
@@ -139,11 +136,11 @@ bool create_temporary_schemas(THD *thd, Object_id *mysql_schema_id,
   if (dd::execute_query(
           thd, dd::String_type("CREATE SCHEMA ") + actual_table_schema_name +
                    dd::String_type(" DEFAULT COLLATE '") +
-                   dd::String_type(default_charset_info->m_coll_name) + "'") ||
+                   dd::String_type(default_charset_info->name) + "'") ||
       dd::execute_query(
           thd, dd::String_type("CREATE SCHEMA ") + *target_table_schema_name +
                    dd::String_type(" DEFAULT COLLATE '") +
-                   dd::String_type(default_charset_info->m_coll_name) + "'") ||
+                   dd::String_type(default_charset_info->name) + "'") ||
       dd::execute_query(thd,
                         dd::String_type("USE ") + *target_table_schema_name)) {
     return true;
@@ -158,22 +155,22 @@ bool create_temporary_schemas(THD *thd, Object_id *mysql_schema_id,
       schema == nullptr)
     return true;
 
-  assert(mysql_schema_id != nullptr);
+  DBUG_ASSERT(mysql_schema_id != nullptr);
   *mysql_schema_id = schema->id();
-  assert(*mysql_schema_id == 1);
+  DBUG_ASSERT(*mysql_schema_id == 1);
 
   if (thd->dd_client()->acquire(*target_table_schema_name, &schema) ||
       schema == nullptr)
     return true;
 
-  assert(target_table_schema_id != nullptr);
+  DBUG_ASSERT(target_table_schema_id != nullptr);
   *target_table_schema_id = schema->id();
 
   if (thd->dd_client()->acquire(actual_table_schema_name, &schema) ||
       schema == nullptr)
     return true;
 
-  assert(actual_table_schema_id != nullptr);
+  DBUG_ASSERT(actual_table_schema_id != nullptr);
   *actual_table_schema_id = schema->id();
 
   return false;
@@ -195,18 +192,17 @@ void establish_table_name_sets(std::set<String_type> *create_set,
       are either new tables in the target version, or they replace an
       existing table in the actual version.
   */
-  assert(create_set != nullptr && create_set->empty());
-  assert(remove_set != nullptr && remove_set->empty());
+  DBUG_ASSERT(create_set != nullptr && create_set->empty());
+  DBUG_ASSERT(remove_set != nullptr && remove_set->empty());
   for (System_tables::Const_iterator it = System_tables::instance()->begin();
        it != System_tables::instance()->end(); ++it) {
     if (is_non_inert_dd_or_ddse_table((*it)->property())) {
-      bool ignore_equivalent_utf8_declarations = false;
       /*
         In this context, all tables should have an Object_table. Minor
         downgrade is the only situation where an Object_table may not exist,
         but minor upgrade will never enter this code path.
       */
-      assert((*it)->entity() != nullptr);
+      DBUG_ASSERT((*it)->entity() != nullptr);
 
       String_type target_ddl_statement("");
       const Object_table_definition *target_table_def =
@@ -230,45 +226,12 @@ void establish_table_name_sets(std::set<String_type> *create_set,
         actual_ddl_statement = actual_table_def->get_ddl();
       }
 
-      // These changes can be ignored:
-      //   utf8     <=> utf8mb3
-      //   utf8_bin <=> utf8mb3_bin
-      // In the dictionary these are equivalent.
-      String_type target_ddl_copy = target_ddl_statement;
-      if (actual_table_def && target_table_def &&
-          actual_ddl_statement.length() != target_ddl_statement.length()) {
-        // See definition of innodb_ddl_log in innobase_ddse_dict_init()
-        std::regex regexp("COLLATE UTF8MB3_BIN");
-        target_ddl_copy =
-            std::regex_replace(target_ddl_copy, regexp, "COLLATE UTF8_BIN");
-
-        // See Object_table_impl::Object_table_impl()
-        regexp = "COLLATE=utf8mb3_bin";
-        target_ddl_copy =
-            std::regex_replace(target_ddl_copy, regexp, "COLLATE=utf8_bin");
-
-        regexp = "DEFAULT CHARSET=utf8mb3";
-        target_ddl_copy =
-            std::regex_replace(target_ddl_copy, regexp, "DEFAULT CHARSET=utf8");
-
-        if (target_ddl_copy == actual_ddl_statement)
-          ignore_equivalent_utf8_declarations = true;
-      }
       /*
         Remove and/or create as needed. If the definitions are non-null
         and equal, no change has been done, and hence upgrade of the table
         is irrelevant.
       */
-      if (ignore_equivalent_utf8_declarations) {
-        // Nothing, "utf8_bin" and "utf8mb3_bin" are equivalent in the data
-        // dictionary. If we take changes here into account and create new
-        // target tables, migrate the data, and then delete the old tables,
-        // then we will run into problems with innodb internal tables such as
-        // 'innodb_ddl_log' which are held open across transactions. A fix of
-        // this issue must close the innodb internal tables and associated
-        // data structures, and reopen them using the target tables that are
-        // created. See Bug#34608061.
-      } else if (target_table_def == nullptr && actual_table_def != nullptr)
+      if (target_table_def == nullptr && actual_table_def != nullptr)
         remove_set->insert((*it)->entity()->name());
       else if (target_table_def != nullptr && actual_table_def == nullptr)
         create_set->insert((*it)->entity()->name());
@@ -332,7 +295,7 @@ bool update_meta_data(THD *thd) {
       Remove ENCRYPTION clause for unencrypted non-InnoDB tablespaces.
       Because its only InnoDB that support encryption in 8.0.16.
     */
-    static_assert(dd::tables::Tablespaces::NUMBER_OF_FIELDS == 7,
+    static_assert(dd::tables::Tablespaces::NUMBER_OF_FIELDS == 6,
                   "SQL statements rely on a specific table definition");
     if (dd::execute_query(
             thd,
@@ -345,7 +308,7 @@ bool update_meta_data(THD *thd) {
     }
 
     // Remove ENCRYPTION clause for non-InnoDB tables.
-    static_assert(dd::tables::Tables::NUMBER_OF_FIELDS == 37,
+    static_assert(dd::tables::Tables::NUMBER_OF_FIELDS == 35,
                   "SQL statements rely on a specific table definition");
     if (dd::execute_query(
             thd,
@@ -377,9 +340,9 @@ bool update_meta_data(THD *thd) {
                   "SQL statements rely on a specific table definition");
     static_assert(dd::tables::Table_partitions::NUMBER_OF_FIELDS == 12,
                   "SQL statements rely on a specific table definition");
-    static_assert(dd::tables::Tables::NUMBER_OF_FIELDS == 37,
+    static_assert(dd::tables::Tables::NUMBER_OF_FIELDS == 35,
                   "SQL statements rely on a specific table definition");
-    static_assert(dd::tables::Indexes::NUMBER_OF_FIELDS == 17,
+    static_assert(dd::tables::Indexes::NUMBER_OF_FIELDS == 15,
                   "SQL statements rely on a specific table definition");
     if (dd::execute_query(
             thd,
@@ -412,11 +375,11 @@ bool update_meta_data(THD *thd) {
       This is done as we expect all innodb tablespaces to have proper
       'encryption' flag set.
     */
-    static_assert(dd::tables::Indexes::NUMBER_OF_FIELDS == 17,
+    static_assert(dd::tables::Indexes::NUMBER_OF_FIELDS == 15,
                   "SQL statements rely on a specific table definition");
-    static_assert(dd::tables::Tablespaces::NUMBER_OF_FIELDS == 7,
+    static_assert(dd::tables::Tablespaces::NUMBER_OF_FIELDS == 6,
                   "SQL statements rely on a specific table definition");
-    static_assert(dd::tables::Tables::NUMBER_OF_FIELDS == 37,
+    static_assert(dd::tables::Tables::NUMBER_OF_FIELDS == 35,
                   "SQL statements rely on a specific table definition");
     if (dd::execute_query(
             thd,
@@ -435,7 +398,7 @@ bool update_meta_data(THD *thd) {
       Update ENCRYPTION clause for unencrypted InnoDB tablespaces.
       Where the 'encryption' key value is empty string ''.
     */
-    static_assert(dd::tables::Tablespaces::NUMBER_OF_FIELDS == 7,
+    static_assert(dd::tables::Tablespaces::NUMBER_OF_FIELDS == 6,
                   "SQL statements rely on a specific table definition");
     if (dd::execute_query(
             thd,
@@ -451,7 +414,7 @@ bool update_meta_data(THD *thd) {
       Store ENCRYPTION clause for unencrypted InnoDB general tablespaces,
       when the 'encryption' key is not yet present.
     */
-    static_assert(dd::tables::Tablespaces::NUMBER_OF_FIELDS == 7,
+    static_assert(dd::tables::Tablespaces::NUMBER_OF_FIELDS == 6,
                   "SQL statements rely on a specific table definition");
     if (dd::execute_query(
             thd,
@@ -470,9 +433,9 @@ bool update_meta_data(THD *thd) {
       table as of 8.0.15, so we ignore to check for partitioned tables
       using general tablespace.
     */
-    static_assert(dd::tables::Tables::NUMBER_OF_FIELDS == 37,
+    static_assert(dd::tables::Tables::NUMBER_OF_FIELDS == 35,
                   "SQL statements rely on a specific table definition");
-    static_assert(dd::tables::Tablespaces::NUMBER_OF_FIELDS == 7,
+    static_assert(dd::tables::Tablespaces::NUMBER_OF_FIELDS == 6,
                   "SQL statements rely on a specific table definition");
     if (dd::execute_query(
             thd,
@@ -492,7 +455,7 @@ bool update_meta_data(THD *thd) {
       Store 'encrypt_type=N' for unencrypted InnoDB file-per-table tables,
       for tables which does not have a 'encrypt_type' key stored already.
     */
-    static_assert(dd::tables::Tables::NUMBER_OF_FIELDS == 37,
+    static_assert(dd::tables::Tables::NUMBER_OF_FIELDS == 35,
                   "SQL statements rely on a specific table definition");
     if (dd::execute_query(
             thd,
@@ -503,106 +466,6 @@ bool update_meta_data(THD *thd) {
       return dd::end_transaction(thd, true);
     }
   }
-  /* Upgrade from 8.0.20 or previous. */
-  if (bootstrap::DD_bootstrap_ctx::instance().is_dd_upgrade_from_before(
-          bootstrap::DD_VERSION_80021)) {
-    /*
-      Fix discard attribute for partitioned Tables.
-
-      Due to bug, prior to 8.0.21, discard attribute for partitioned tables
-      doesn't reflect reality - that is, it can be set to true even if no
-      partition is discarded or it can be set to false, even if some
-      partitions are discarded.
-
-      Moreover, due to the bug, the attribute is only set for a whole Table,
-      whereas only some partitions of the table may be discarded.
-
-      However, Tablespace state==discarded reflect reality properly.
-
-      Since 8.0.21, discard attribute for partitioned tables follow the rules:
-      a) dd::Table for partitioned Table cannot have discard attribute
-      b) only leaf dd::Partitions can have discard attribute
-
-      We can recreate such state looking at Tablespace state in three steps:
-      a) Remove discard attribute from dd::Tables which have any partitions
-      b) Remove discard attribute from all dd::Partitions
-      c) For each leaf_partition (taken from index_partitions) set discard
-      attribute depending on whether its Tablespace state is discarded or not
-    */
-    static_assert(dd::tables::Index_partitions::NUMBER_OF_FIELDS == 5,
-                  "SQL statements rely on a specific table definition");
-    static_assert(dd::tables::Table_partitions::NUMBER_OF_FIELDS == 12,
-                  "SQL statements rely on a specific table definition");
-    static_assert(dd::tables::Tables::NUMBER_OF_FIELDS == 37,
-                  "SQL statements rely on a specific table definition");
-    static_assert(dd::tables::Tablespaces::NUMBER_OF_FIELDS == 7,
-                  "SQL statements rely on a specific table definition");
-
-    /*
-      Remove discard attribute from all Tables
-      Use mysql.index_partitions to find all partitions
-    */
-    if (dd::execute_query(
-            thd,
-            "UPDATE mysql.tables tbl "
-            "JOIN mysql.table_partitions tp ON tp.table_id = tbl.id "
-            "JOIN mysql.index_partitions ip ON ip.partition_id = tp.id "
-            "SET "
-            "tbl.se_private_data=NULLIF(REMOVE_DD_PROPERTY_KEY(tbl.se_private_"
-            "data, 'discard'), '') "
-            "WHERE tbl.engine='InnoDB' AND "
-            "GET_DD_PROPERTY_KEY_VALUE(tbl.se_private_data, 'discard') IS NOT "
-            "NULL ")) {
-      return dd::end_transaction(thd, true);
-    }
-
-    /*
-      Remove discard attribute from all table_partitions.
-
-      Even though we didn't store discard attribute in mysql.table_partitions,
-      it's still possible there is such attribute there: in versions 8.0.0 to
-      8.0.14 it was possible to upgrade from 5.7, when MySQL had discarded
-      partitions. Such upgrade would add discard attribute to
-      mysql.table_partitions
-    */
-    if (dd::execute_query(thd,
-                          "UPDATE mysql.table_partitions tp "
-                          "SET "
-                          "tp.se_private_data=NULLIF(REMOVE_DD_PROPERTY_KEY(tp."
-                          "se_private_data, 'discard'), '') "
-                          "WHERE tp.engine='InnoDB' AND "
-                          "GET_DD_PROPERTY_KEY_VALUE(tp.se_private_data, "
-                          "'discard') IS NOT NULL ")) {
-      return dd::end_transaction(thd, true);
-    }
-
-    /* Set discard attribute for all leaf_partitions by looking at its
-     * Tablespace state. mysql.index_partitions is used to access Tablespaces */
-    if (dd::execute_query(
-            thd,
-            "UPDATE mysql.table_partitions tp "
-            "JOIN mysql.index_partitions ip ON ip.partition_id = tp.id "
-            "JOIN mysql.tablespaces ts ON ip.tablespace_id = ts.id "
-            "SET tp.se_private_data = "
-            "CONCAT('discard=1;',IFNULL(tp.se_private_data,'')) "
-            "WHERE tp.engine='InnoDB' AND "
-            "GET_DD_PROPERTY_KEY_VALUE(ts.se_private_data, "
-            "'state')='discarded' ")) {
-      return dd::end_transaction(thd, true);
-    }
-  }
-
-  /*
-    8.3.0 introduced a new "auto-update" property for histograms.
-    We add this property to any existing histograms and set it to false.
-  */
-  if (bootstrap::DD_bootstrap_ctx::instance().is_dd_upgrade_from_before(
-          bootstrap::DD_VERSION_80300)) {
-    if (dd::execute_query(thd,
-                          "UPDATE mysql.column_statistics SET histogram = "
-                          "json_set(histogram, '$.\"auto-update\"', false)"))
-      return dd::end_transaction(thd, true);
-  }
 
   /*
     Turn foreign key checks back on and commit explicitly.
@@ -666,9 +529,9 @@ bool migrate_meta_data(THD *thd, const std::set<String_type> &create_set,
     the previous one.
   */
   auto migrate_table = [&](const String_type &name, const String_type &stmt) {
-    assert(create_set.find(name) != create_set.end());
+    DBUG_ASSERT(create_set.find(name) != create_set.end());
     /* A table must be migrated only once. */
-    assert(migrated_set.find(name) == migrated_set.end());
+    DBUG_ASSERT(migrated_set.find(name) == migrated_set.end());
     migrated_set.insert(name);
     if (dd::execute_query(thd, stmt)) {
       return dd::end_transaction(thd, true);
@@ -683,76 +546,31 @@ bool migrate_meta_data(THD *thd, const std::set<String_type> &create_set,
 
   /********************* Migration of mysql.tables *********************/
   /* Upgrade from 80012 or earlier. */
-  static_assert(dd::tables::Tables::NUMBER_OF_FIELDS == 37,
+  static_assert(dd::tables::Tables::NUMBER_OF_FIELDS == 35,
                 "SQL statements rely on a specific table definition");
   if (is_dd_upgrade_from_before(bootstrap::DD_VERSION_80013)) {
     /* Column 'last_checked_for_upgrade' was added. */
-    if (migrate_table(
-            "tables",
-            "INSERT INTO tables SELECT *, 0, NULL, NULL FROM mysql.tables")) {
-      return true;
-    }
-  } else if (is_dd_upgrade_from_before(bootstrap::DD_VERSION_80021)) {
-    /*
-      Upgrade from 80020 and before.
-      Store NULL for new columns mysql.tables.engine_attribute and
-      mysql.tables.secondary_engine_attribute
-    */
-
-    if (migrate_table(
-            "tables",
-            "INSERT INTO tables SELECT *, NULL, NULL FROM mysql.tables")) {
-      return true;
-    }
-  }
-  /********************* Migration of mysql.columns *********************/
-  /* Upgrade from 80020 or earlier. */
-  static_assert(dd::tables::Columns::NUMBER_OF_FIELDS == 32,
-                "SQL statements rely on a specific table definition");
-  if (is_dd_upgrade_from_before(bootstrap::DD_VERSION_80021)) {
-    /*
-      Upgrade from 80020 and before.
-      Store NULL for new columns mysql.columns.engine_attribute and
-      mysql.columns.secondary_engine_attribute
-    */
-    if (migrate_table(
-            "columns",
-            "INSERT INTO columns SELECT *, NULL, NULL FROM mysql.columns")) {
+    if (migrate_table("tables",
+                      "INSERT INTO tables SELECT *, 0 FROM mysql.tables")) {
       return true;
     }
   }
 
   /********************* Migration of mysql.events *********************/
-  /*
-    Upgrade for versions before 80014:
-      SQL mode 'INVALID_DATES' was renamed to 'ALLOW_INVALID_DATES'
-    Upgrade for versions before 80200:
-      The status enum changes from
-        'ENABLED','DISABLED','SLAVESIDE_DISABLED'
-      to
-        'ENABLED','DISABLED','REPLICA_SIDE_DISABLED'
-  */
+  /* Upgrade from 80013 or earlier. */
   static_assert(dd::tables::Events::NUMBER_OF_FIELDS == 24,
                 "SQL statements rely on a specific table definition");
-  if (is_dd_upgrade_from_before(bootstrap::DD_VERSION_80200)) {
+  if (is_dd_upgrade_from_before(bootstrap::DD_VERSION_80014)) {
     /*
-      For versions before 80014
-         Migrate the SQL mode as an integer.
-      For versions before 80200
-         Replace SLAVESIDE_DISABLED with REPLICA_SIDE_DISABLED
-
-      Since the change for versions lower than 80014 is idempotent
-      we simplify the code and execute it as well for all versions
-      lower than 80200 but higher than 80014
+      SQL mode 'INVALID_DATES' was renamed to 'ALLOW_INVALID_DATES'.
+      Migrate the SQL mode as an integer.
     */
     if (migrate_table(
             "events",
             "INSERT INTO events SELECT  id, schema_id, name, definer, "
             "  time_zone, definition, definition_utf8, execute_at, "
             "  interval_value, interval_field, sql_mode+0, starts, ends, "
-            "  IF(status = 'SLAVESIDE_DISABLED', 'REPLICA_SIDE_DISABLED', "
-            "status), "
-            "  on_completion, created, last_altered, "
+            "  status, on_completion, created, last_altered, "
             "  last_executed, comment, originator, client_collation_id, "
             "  connection_collation_id, schema_collation_id, options "
             "  FROM mysql.events")) {
@@ -836,46 +654,6 @@ bool migrate_meta_data(THD *thd, const std::set<String_type> &create_set,
     }
   }
 
-  /********************* Migration of mysql.indexes *********************/
-  /*
-    DD version 80020 adds new columns 'engine_attribute' and
-    'secondary_engine_atribute'
-  */
-  static_assert(dd::tables::Indexes::NUMBER_OF_FIELDS == 17,
-                "SQL statements rely on a specific table definition");
-  if (is_dd_upgrade_from_before(bootstrap::DD_VERSION_80021)) {
-    /*
-      Upgrade from 80020 and before.
-      Store NULL for new columns mysql.indexes.engine_attribute and
-      mysql.indexes.secondary_engine_attribute
-    */
-    if (migrate_table(
-            "indexes",
-            "INSERT INTO indexes SELECT *, NULL, NULL FROM mysql.indexes")) {
-      return true;
-    }
-  }
-
-  /********************* Migration of mysql.tablespaces *********************/
-  /*
-    DD version 80020 adds new columns 'engine_attribute' and
-    'secondary_engine_atribute'
-  */
-  static_assert(dd::tables::Tablespaces::NUMBER_OF_FIELDS == 7,
-                "SQL statements rely on a specific table definition");
-  if (is_dd_upgrade_from_before(bootstrap::DD_VERSION_80021)) {
-    /*
-      Upgrade from 80021 and before.
-      Store NULL for new columns mysql.tablespaces.engine_attribute and
-      mysql.tablespaces.secondary_engine_attribute
-    */
-    if (migrate_table("tablespaces",
-                      "INSERT INTO tablespaces SELECT *, NULL FROM "
-                      "mysql.tablespaces")) {
-      return true;
-    }
-  }
-
   /*
     Default handling: Copy all meta data for the tables that have been
     modified (i.e., all tables which are both in the remove- and create set),
@@ -1183,12 +961,13 @@ bool upgrade_tables(THD *thd) {
       update_object_ids(thd, create_set, remove_set, mysql_schema_id,
                         target_table_schema_id, target_table_schema_name,
                         actual_table_schema_id) ||
-      update_versions(thd))
+      update_versions(thd, false))
     return true;
 
   LogErr(SYSTEM_LEVEL, ER_DD_UPGRADE_COMPLETED,
          bootstrap::DD_bootstrap_ctx::instance().get_actual_dd_version(),
          dd::DD_VERSION);
+  log_sink_buffer_check_timeout();
   sysd::notify("STATUS=Data Dictionary upgrade complete\n");
 
   /*
@@ -1196,7 +975,7 @@ bool upgrade_tables(THD *thd) {
     DD cache and re-initialize based on 'mysql.dd_properties', hence,
     we will lose track of the fact that we have done a DD upgrade as part
     of this restart. Thus, we record this fact in the bootstrap context
-    so we can check it e.g. when initializing the information schema,
+    so we can check it e.g. when initializeing the information schema,
     where we need to regenerate the meta data if the underlying tables
     have changed.
   */
@@ -1212,7 +991,7 @@ bool upgrade_tables(THD *thd) {
 
   /*
     Reset the encryption attribute in object table def since we will now
-    start over by creating the scaffolding, which expects an unencrypted
+    start over by creating the scaffolding, which expectes an unencrypted
     DD tablespace.
   */
   Object_table_definition_impl::set_dd_tablespace_encrypted(false);
