@@ -1,16 +1,15 @@
-/* Copyright (c) 2017, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -169,7 +168,7 @@ namespace info_schema {
     This bug updates LAST_EXECUTED to include time zones in
     I_S.EVENTS.
 
-  800201: Published in 8.0.20
+  800201: Current
   ------------------------------------
   Changes from version 80020:
 
@@ -177,95 +176,11 @@ namespace info_schema {
     This bug updates the character set of columns returned from JSON_TABLE
     expressions in INFORMATION_SCHEMA views.
 
-  80021: Published in 8.0.21
+  80021: Next IS version number after the previous is public.
   ------------------------------------
-  Changes from version 800201:
-
-     - WL#13562 CREATE/ALTER USER COMMENT 'JSON'
-
-     - WL#13341: Store options for secondary engines.
-
-     - Bug#30766181 and Bug#30216864 updates I_S.KEY_COLUMN_USAGE and
-       I_S.TABLE_CONSTRAINTS views.
-
-  80022: Published in 8.0.22
-  ------------------------------------
-  Changes from version 80021:
-
-  - WL#13369: Added new I_S view 'SCHEMATA_EXTENSIONS'.
-  - Bug #31427410: Added a WHERE clause to I_S.USER_ATTRIBUTES
-
-  80023: Published in 8.0.23
-  ------------------------------------
-  Changes from version 80022:
-
-  - WL#12819: Affects size of I_S.KEYWORDS.WORD column length.
-    Following bug was raised for the same.
-    Bug#31982157 THE KEYWORD LEN IN I_S GETS UPDATED WHEN
-    A KEYWORD > EXISTING MAX LEN IS ADDED
-
-  - WL#10905: INFORMATION_SCHEMA.COLUMNS table is modified to list "INVISIBLE"
-              value in EXTRA column for INVISIBLE columns.
-
-  80024..80028: Not published.
-  ----------------------------------------------------------------------------
-  There are no changes from version 80023. Hence server versions 80024..80028
-  uses I_S version 80023.
-
-  80029: Published in 8.0.29
-  ------------------------------------
-  Changes from version 80023:
-
-  - Bug#33781534: INFORMATION_SCHEMA.KEY_COLUMN_USAGE table is modified to
-                  list invisible columns with key constraints.
-
-  80030: Published in 8.0.30
-  ------------------------------------
-  Changes from version 80029:
-  - Bug #33787300 Rename utf8_xxx collations to utf8mb3_xxx
-
-  - WL#13784: Following INFORMATION_SCHEMA tables are modified to skip listing
-              generated invisible primary key and key column when system
-              variable show_gipk_in_create_table_and_information_schema = OFF,
-                INFORMATION_SCHEMA.COLUMNS
-                INFORMATION_SCHEMA.COLUMNS_EXTENSIONS
-                INFORMATION_SCHEMA.KEY_COLUMN_USAGE
-                INFORMATION_SCHEMA.STATISTICS
-                INFORMATION_SCHEMA.TABLE_CONSTRAINTS
-                INFORMATION_SCHEMA.TABLE_CONSTRAINTS_EXTENSIONS
-
-  80031..80033: Not published.
-  ----------------------------------------------------------------------------
-  There are no changes from version 80030. Hence server versions 80031..80033
-  uses I_S version 80030.
-
-  80034: Current
-  ----------------------------------------------------------------------------
-  Changes from version 80030:
-  - WL#15454: INFORMATION_SCHEMA.ROUTINES is modified so that ROUTINE_BODY
-              now depends on the value of EXTERNAL_LANGUAGE
-
-  80035: Next IS version number after the previous is public.
-  ------------------------------------
-  Changes from version 80035:
-
-  80200:
-  ----------------------------------------------------------------------------
-  Changes:
-  - WL#14190: Replace old terms in replication SQL commands on the SOURCE
-      Changed the status column of information_schema.events to show
-      correct terminology
-
-  80400:
-  ----------------------------------------------------------------------------
-  Changes:
-  - Bug#36112585: Data type is changed from VAR_STRING to STRING by UNION
-                  if all values are empty string.
-    Changed derived data type from CHAR(0) to VARCHAR(0) for some columns in
-    such view definitions.
 */
 
-static const uint IS_DD_VERSION = 80400;
+static const uint IS_DD_VERSION = 800201;
 static_assert((IS_DD_VERSION <= MYSQL_VERSION_ID) ||
                   ((IS_DD_VERSION == 800201) && (MYSQL_VERSION_ID >= 80020)),
               "This release can not use a version number from the future");
@@ -343,7 +258,7 @@ bool remove_I_S_view_metadata(THD *thd, const dd::String_type &view_name);
 
   @param schema_name Schema name.
   @param view_name   I_S view name.
-  @param definition  [out] The CREATE VIEW command to create system view.
+  @param definition  [out] The CREATE VIEW command to create sytem view.
                            A pointer to a preallocated string should be
                            supplied.
 
