@@ -1,4 +1,4 @@
-/* Copyright (c) 2016, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2016, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -24,10 +24,10 @@
 #define DD__EVENT_INCLUDED
 
 #include "my_inttypes.h"
-#include "my_time_t.h"                    // my_time_t
 #include "sql/dd/impl/raw/object_keys.h"  // IWYU pragma: keep
 #include "sql/dd/types/entity_object.h"   // dd::Entity_object
 
+typedef long my_time_t;
 struct MDL_key;
 
 namespace dd {
@@ -93,16 +93,12 @@ class Event : virtual public Entity_object {
     IF_SECOND_MICROSECOND
   };
 
-  enum enum_event_status {
-    ES_ENABLED = 1,
-    ES_DISABLED,
-    ES_REPLICA_SIDE_DISABLED
-  };
+  enum enum_event_status { ES_ENABLED = 1, ES_DISABLED, ES_SLAVESIDE_DISABLED };
 
   enum enum_on_completion { OC_DROP = 1, OC_PRESERVE };
 
  public:
-  ~Event() override = default;
+  virtual ~Event() {}
 
  public:
   /////////////////////////////////////////////////////////////////////////
@@ -265,21 +261,13 @@ class Event : virtual public Entity_object {
   virtual void set_schema_collation_id(Object_id schema_collation_id) = 0;
 
   /**
-    Allocate a new object graph and invoke the copy constructor for
+    Allocate a new object graph and invoke the copy contructor for
     each object. Only used in unit testing.
 
     @return pointer to dynamically allocated copy
   */
   virtual Event *clone() const = 0;
 
-  /**
-    Allocate a new object which can serve as a placeholder for the original
-    object in the Dictionary_client's dropped registry. Such object has the
-    same keys as the original but has no other info and as result occupies
-    less memory.
-  */
-  virtual Event *clone_dropped_object_placeholder() const = 0;
-
   static void create_mdl_key(const String_type &schema_name,
                              const String_type &name, MDL_key *key);
 };
