@@ -1,4 +1,4 @@
-/* Copyright (c) 2017, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -24,7 +24,6 @@
 
 #include <assert.h>
 #include <errno.h>
-#include <stdint.h>
 #include <string.h>
 #include <sys/stat.h>
 #ifdef HAVE_SYS_TIME_H
@@ -35,7 +34,6 @@
 #include <unistd.h>
 #endif
 #include <algorithm>
-#include <memory>
 #include <string>
 
 #include "lex_string.h"
@@ -46,16 +44,14 @@
 #include "my_dir.h"
 #include "my_inttypes.h"
 #include "my_io.h"
+#include "my_loglevel.h"
 #include "my_sys.h"
 #include "my_user.h"  // parse_user
-#include "mysql/components/services/bits/psi_bits.h"
 #include "mysql/components/services/log_builtins.h"
-#include "mysql/my_loglevel.h"
-#include "mysql/strings/m_ctype.h"
+#include "mysql/psi/psi_base.h"
 #include "mysql/udf_registration_types.h"
 #include "mysql_com.h"
 #include "mysqld_error.h"                    // ER_*
-#include "nulls.h"                           // NullS
 #include "sql/dd/cache/dictionary_client.h"  // dd::cache::Dictionary_client
 #include "sql/dd/dd.h"                       // dd::get_dictionary
 #include "sql/dd/dd_schema.h"                // Schema_MDL_locker
@@ -63,7 +59,6 @@
 #include "sql/dd/dd_trigger.h"               // dd::create_trigger
 #include "sql/dd/dd_view.h"                  // create_view
 #include "sql/dd/dictionary.h"
-#include "sql/dd/impl/upgrade/server.h"
 #include "sql/dd/impl/utils.h"  // execute_query
 #include "sql/dd/properties.h"
 #include "sql/dd/string_type.h"
@@ -103,9 +98,10 @@
 #include "sql/trigger_chain.h"
 #include "sql/trigger_def.h"
 #include "sql_string.h"
-#include "string_with_len.h"
 #include "thr_lock.h"
 
+#include "sql/dd/impl/upgrade/server.h"
+
 class Sroutine_hash_entry;
 
 bool Table_trigger_dispatcher::reorder_57_list(MEM_ROOT *mem_root,
@@ -119,7 +115,7 @@ bool Table_trigger_dispatcher::reorder_57_list(MEM_ROOT *mem_root,
   }
 
   // The individual triggers are mem_root-allocated. We can now empty the list.
-  triggers->clear();
+  triggers->empty();
 
   // And then, we iterate over the chains and re-add the triggers to the list.
   for (int i = 0; i < (int)TRG_EVENT_MAX; i++)
@@ -210,7 +206,7 @@ struct Trg_file_data {
   List<LEX_STRING> definitions;
 
   // List of 'sql mode' values.
-  List<uint64_t> sql_modes;
+  List<ulonglong> sql_modes;
 
   // List of 'definer' values.
   List<LEX_STRING> definers_list;
@@ -275,8 +271,8 @@ class Handle_old_incorrect_sql_modes_hook : public Unknown_key_hook {
 
  public:
   Handle_old_incorrect_sql_modes_hook(char *file_path) : m_path(file_path) {}
-  bool process_unknown_string(const char *&unknown_key, uchar *base,
-                              MEM_ROOT *mem_root, const char *end) override;
+  virtual bool process_unknown_string(const char *&unknown_key, uchar *base,
+                                      MEM_ROOT *mem_root, const char *end);
 };
 
 /**
@@ -353,11 +349,11 @@ bool Trigger_loader::load_triggers(THD *thd, MEM_ROOT *mem_root,
     return true;
 
   if (trg.definitions.is_empty()) {
-    assert(trg.sql_modes.is_empty());
-    assert(trg.definers_list.is_empty());
-    assert(trg.client_cs_names.is_empty());
-    assert(trg.connection_cl_names.is_empty());
-    assert(trg.db_cl_names.is_empty());
+    DBUG_ASSERT(trg.sql_modes.is_empty());
+    DBUG_ASSERT(trg.definers_list.is_empty());
+    DBUG_ASSERT(trg.client_cs_names.is_empty());
+    DBUG_ASSERT(trg.connection_cl_names.is_empty());
+    DBUG_ASSERT(trg.db_cl_names.is_empty());
     return false;
   }
 
@@ -385,11 +381,10 @@ bool Trigger_loader::load_triggers(THD *thd, MEM_ROOT *mem_root,
 
     lex_string_set(
         &default_connection_cl_name,
-        const_cast<char *>(thd->variables.collation_connection->m_coll_name));
+        const_cast<char *>(thd->variables.collation_connection->name));
 
-    lex_string_set(
-        &default_db_cl_name,
-        const_cast<char *>(thd->variables.collation_database->m_coll_name));
+    lex_string_set(&default_db_cl_name,
+                   const_cast<char *>(thd->variables.collation_database->name));
   }
 
   LEX_CSTRING db_name_str = {db_name, strlen(db_name)};
@@ -473,10 +468,10 @@ bool Trigger_loader::load_triggers(THD *thd, MEM_ROOT *mem_root,
     LEX_CSTRING definer_host_name{definer_host.str, definer_host.length};
 
     // Set timeval to use for Created field.
-    my_timeval timestamp_value;
+    timeval timestamp_value;
     if (created_timestamp) {
-      timestamp_value.m_tv_sec = *created_timestamp / 100;
-      timestamp_value.m_tv_usec = (*created_timestamp % 100) * 10000;
+      timestamp_value.tv_sec = static_cast<long>(*created_timestamp / 100);
+      timestamp_value.tv_usec = (*created_timestamp % 100) * 10000;
     } else {
       // Trigger created before 5.7.2, set created value.
       timestamp_value = thd->query_start_timeval_trunc(2);
@@ -505,7 +500,6 @@ bool Trigger_loader::load_triggers(THD *thd, MEM_ROOT *mem_root,
         body_utf8, sql_mode, definer_user_name, definer_host_name, client_cs,
         cn_cl, db_cl, enum_trigger_event_type::TRG_EVENT_MAX,
         enum_trigger_action_time_type::TRG_ACTION_MAX, 0, timestamp_value);
-    if (t == nullptr) return true;
 
     /*
       NOTE: new trigger object is not fully initialized here.
@@ -513,7 +507,7 @@ bool Trigger_loader::load_triggers(THD *thd, MEM_ROOT *mem_root,
       will be done in parse_triggers().
     */
     if (triggers->push_back(t, mem_root)) {
-      ::destroy_at(t);
+      destroy(t);
       return true;
     }
   }
@@ -592,8 +586,7 @@ class Upgrade_MDL_guard {
   bool acquire_lock_tablespace(Tablespace_hash_set *tablespace_names) {
     m_tablespace_lock = true;
     return lock_tablespace_names(m_thd, tablespace_names,
-                                 m_thd->variables.lock_wait_timeout,
-                                 m_thd->mem_root);
+                                 m_thd->variables.lock_wait_timeout);
   }
 
   Upgrade_MDL_guard(THD *thd)
@@ -682,7 +675,7 @@ class Table_upgrade_guard {
 
     free_table_share(m_table->s);
 
-    if (m_handler != nullptr) ::destroy_at(m_handler);
+    destroy(m_handler);
   }
 };
 
@@ -722,40 +715,40 @@ static const int REQUIRED_VIEW_PARAMETERS = 12;
 */
 static File_option view_parameters[] = {
     {{STRING_WITH_LEN("query")},
-     my_offsetof_upgrade(Table_ref, select_stmt),
+     my_offsetof_upgrade(TABLE_LIST, select_stmt),
      FILE_OPTIONS_ESTRING},
     {{STRING_WITH_LEN("updatable")},
-     my_offsetof_upgrade(Table_ref, updatable_view),
+     my_offsetof_upgrade(TABLE_LIST, updatable_view),
      FILE_OPTIONS_ULONGLONG},
     {{STRING_WITH_LEN("algorithm")},
-     my_offsetof_upgrade(Table_ref, algorithm),
+     my_offsetof_upgrade(TABLE_LIST, algorithm),
      FILE_OPTIONS_ULONGLONG},
     {{STRING_WITH_LEN("definer_user")},
-     my_offsetof_upgrade(Table_ref, definer.user),
+     my_offsetof_upgrade(TABLE_LIST, definer.user),
      FILE_OPTIONS_STRING},
     {{STRING_WITH_LEN("definer_host")},
-     my_offsetof_upgrade(Table_ref, definer.host),
+     my_offsetof_upgrade(TABLE_LIST, definer.host),
      FILE_OPTIONS_STRING},
     {{STRING_WITH_LEN("suid")},
-     my_offsetof_upgrade(Table_ref, view_suid),
+     my_offsetof_upgrade(TABLE_LIST, view_suid),
      FILE_OPTIONS_ULONGLONG},
     {{STRING_WITH_LEN("with_check_option")},
-     my_offsetof_upgrade(Table_ref, with_check),
+     my_offsetof_upgrade(TABLE_LIST, with_check),
      FILE_OPTIONS_ULONGLONG},
     {{STRING_WITH_LEN("timestamp")},
-     my_offsetof_upgrade(Table_ref, timestamp),
+     my_offsetof_upgrade(TABLE_LIST, timestamp),
      FILE_OPTIONS_TIMESTAMP},
     {{STRING_WITH_LEN("source")},
-     my_offsetof_upgrade(Table_ref, source),
+     my_offsetof_upgrade(TABLE_LIST, source),
      FILE_OPTIONS_ESTRING},
     {{STRING_WITH_LEN("client_cs_name")},
-     my_offsetof_upgrade(Table_ref, view_client_cs_name),
+     my_offsetof_upgrade(TABLE_LIST, view_client_cs_name),
      FILE_OPTIONS_STRING},
     {{STRING_WITH_LEN("connection_cl_name")},
-     my_offsetof_upgrade(Table_ref, view_connection_cl_name),
+     my_offsetof_upgrade(TABLE_LIST, view_connection_cl_name),
      FILE_OPTIONS_STRING},
     {{STRING_WITH_LEN("view_body_utf8")},
-     my_offsetof_upgrade(Table_ref, view_body_utf8),
+     my_offsetof_upgrade(TABLE_LIST, view_body_utf8),
      FILE_OPTIONS_ESTRING},
     {{NullS, 0}, 0, FILE_OPTIONS_STRING}};
 
@@ -764,27 +757,27 @@ static File_option view_parameters[] = {
   information.
 
   @param[in] thd       Thread handle.
-  @param[in] view_ref  Table_ref to store view data.
+  @param[in] view_ref  TABLE_LIST to store view data.
 
   @retval false  ON SUCCESS
   @retval true   ON FAILURE
 */
-static bool create_unlinked_view(THD *thd, Table_ref *view_ref) {
-  Query_block *backup_query_block = thd->lex->query_block;
-  Table_ref *saved_query_tables = thd->lex->query_tables;
+static bool create_unlinked_view(THD *thd, TABLE_LIST *view_ref) {
+  SELECT_LEX *backup_select = thd->lex->select_lex;
+  TABLE_LIST *saved_query_tables = thd->lex->query_tables;
   SQL_I_List<Sroutine_hash_entry> saved_sroutines_list;
   // For creation of view without column information.
-  Query_block select(thd->mem_root, nullptr, nullptr);
+  SELECT_LEX select(thd->mem_root, nullptr, nullptr);
 
   // Backup
-  thd->lex->query_block = &select;
+  thd->lex->select_lex = &select;
   thd->lex->query_tables = nullptr;
   thd->lex->sroutines_list.save_and_clear(&saved_sroutines_list);
 
   dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
   const dd::Schema *schema = nullptr;
   if (thd->dd_client()->acquire(view_ref->db, &schema)) return true;
-  assert(schema != nullptr);  // Should be impossible during upgrade.
+  DBUG_ASSERT(schema != nullptr);  // Should be impossible during upgrade.
 
   // Disable autocommit option in thd variable
   Disable_autocommit_guard autocommit_guard(thd);
@@ -800,7 +793,7 @@ static bool create_unlinked_view(THD *thd, Table_ref *view_ref) {
     result = trans_commit_stmt(thd) || trans_commit(thd);
 
   // Restore
-  thd->lex->query_block = backup_query_block;
+  thd->lex->select_lex = backup_select;
   thd->lex->sroutines_list.push_front(&saved_sroutines_list);
   thd->lex->query_tables = saved_query_tables;
 
@@ -810,16 +803,16 @@ static bool create_unlinked_view(THD *thd, Table_ref *view_ref) {
 /**
   Construct ALTER VIEW statement to fix the column list
   and dependency information but retains the previous
-  view definition entry in DD.
+  view defintion entry in DD.
 
   @param[in]  thd       Thread handle.
-  @param[in]  view_ref  Table_ref to store view data.
+  @param[in]  view_ref  TABLE_LIST to store view data.
   @param[out] str       String object to store view definition.
   @param[in]  db_name   database name.
   @param[in]  view_name view name.
   @param[in]  cs        Charset Information.
 */
-static void create_alter_view_stmt(const THD *thd, Table_ref *view_ref,
+static void create_alter_view_stmt(const THD *thd, TABLE_LIST *view_ref,
                                    String *str, const String_type &db_name,
                                    const String_type &view_name,
                                    const CHARSET_INFO *cs) {
@@ -845,7 +838,7 @@ static void create_alter_view_stmt(const THD *thd, Table_ref *view_ref,
   View will be marked invalid if ALTER VIEW statement fails on the view.
 
   @param[in] thd                     Thread handle.
-  @param[in] view_ref                Table_ref with view data.
+  @param[in] view_ref                TABLE_LIST with view data.
   @param[in] db_name                 database name.
   @param[in] view_name               view name.
 
@@ -853,7 +846,7 @@ static void create_alter_view_stmt(const THD *thd, Table_ref *view_ref,
   @retval true   ON FAILURE
 
 */
-static bool fix_view_cols_and_deps(THD *thd, Table_ref *view_ref,
+static bool fix_view_cols_and_deps(THD *thd, TABLE_LIST *view_ref,
                                    const String_type &db_name,
                                    const String_type &view_name) {
   bool error = false;
@@ -952,8 +945,8 @@ static bool migrate_view_to_dd(THD *thd, const FRM_context &frm_context,
                                const String_type &db_name,
                                const String_type &view_name, MEM_ROOT *mem_root,
                                bool is_fix_view_cols_and_deps) {
-  Table_ref table_list(db_name.c_str(), db_name.length(), view_name.c_str(),
-                       view_name.length(), view_name.c_str(), TL_READ);
+  TABLE_LIST table_list(db_name.c_str(), db_name.length(), view_name.c_str(),
+                        view_name.length(), view_name.c_str(), TL_READ);
 
   // Initialize timestamp
   table_list.timestamp.str = table_list.timestamp_buffer;
@@ -1015,14 +1008,14 @@ static bool migrate_view_to_dd(THD *thd, const FRM_context &frm_context,
   if (invalid_ctx) {
     cs = system_charset_info;
     size_t cs_length = strlen(cs->csname);
-    size_t length = strlen(cs->m_coll_name);
+    size_t length = strlen(cs->name);
 
     table_list.view_client_cs_name.str =
         strmake_root(mem_root, cs->csname, cs_length);
     table_list.view_client_cs_name.length = cs_length;
 
     table_list.view_connection_cl_name.str =
-        strmake_root(mem_root, cs->m_coll_name, length);
+        strmake_root(mem_root, cs->name, length);
     table_list.view_connection_cl_name.length = length;
 
     if (table_list.view_client_cs_name.str == nullptr ||
@@ -1033,18 +1026,6 @@ static bool migrate_view_to_dd(THD *thd, const FRM_context &frm_context,
     }
   }
 
-  // Validate body definition to avoid invalid UTF8 characters.
-  std::string invalid_sub_str;
-  if (is_invalid_string(LEX_CSTRING{table_list.view_body_utf8.str,
-                                    table_list.view_body_utf8.length},
-                        system_charset_info, invalid_sub_str)) {
-    // Provide contextual information
-    my_error(ER_DEFINITION_CONTAINS_INVALID_STRING, MYF(0), "view",
-             db_name.c_str(), view_name.c_str(), system_charset_info->csname,
-             invalid_sub_str.c_str());
-    return true;
-  }
-
   // View is already created, we are recreating it now.
   if (is_fix_view_cols_and_deps) {
     if (fix_view_cols_and_deps(thd, &table_list, db_name, view_name)) {
@@ -1183,7 +1164,7 @@ static bool add_triggers_to_table(THD *thd, TABLE *table,
       */
       if (t->get_event() < t_type ||
           (t->get_event() == t_type && t->get_action_time() < t_time)) {
-        assert(false);
+        DBUG_ASSERT(false);
         LogErr(ERROR_LEVEL, ER_TRG_WRONG_ORDER, t->get_db_name().str,
                t->get_trigger_name().str, schema_name.c_str(),
                table_name.c_str());
@@ -1249,7 +1230,7 @@ static bool add_triggers_to_table(THD *thd, TABLE *table,
       sp_head::destroy(sp);
 
     }  // End of while loop
-  }    // End of If condition to check Trigger existence
+  }    // End of If condition to check Trigger existance
   return false;
 }
 
@@ -1284,7 +1265,7 @@ static bool fix_generated_columns_for_upgrade(
           error = true;
           LogErr(ERROR_LEVEL,
                  ER_CANT_PROCESS_EXPRESSION_FOR_GENERATED_COLUMN_TO_DD,
-                 to_string(sql_field->gcol_info->expr_str).c_str(),
+                 to_string((*field_ptr)->gcol_info->expr_str).c_str(),
                  table->s->db.str, table->s->table_name.str,
                  (*field_ptr)->field_name);
           break;
@@ -1513,11 +1494,9 @@ static bool migrate_table_to_dd(THD *thd, const String_type &schema_name,
   }
 
   // Create table share for tables and views.
-  int r = create_table_share_for_upgrade(
-      thd, path, &share, &frm_context, schema_name.c_str(), table_name.c_str(),
-      is_fix_view_cols_and_deps);
-  if (r == -2) return false;  // No error, ignoring file
-  if (r != 0) {
+  if (create_table_share_for_upgrade(thd, path, &share, &frm_context,
+                                     schema_name.c_str(), table_name.c_str(),
+                                     is_fix_view_cols_and_deps)) {
     LogErr(ERROR_LEVEL, ER_CANT_CREATE_TABLE_SHARE_FROM_FRM,
            table_name.c_str());
     return true;
@@ -1686,15 +1665,14 @@ static bool migrate_table_to_dd(THD *thd, const String_type &schema_name,
   List_iterator<Create_field> it_create(alter_info.create_list);
 
   for (int field_no = 0; (sql_field = it_create++); field_no++) {
-    if (prepare_create_field(thd, schema_name.c_str(), table_name.c_str(),
-                             &create_info, &alter_info.create_list,
+    if (prepare_create_field(thd, &create_info, &alter_info.create_list,
                              &select_field_pos, table.file, sql_field,
                              field_no))
       return true;
   }
 
   // open_table_from_share and partition expression parsing needs a
-  // valid Query_block to parse generated columns
+  // valid SELECT_LEX to parse generated columns
   LEX *lex_saved = thd->lex;
   thd->lex = &lex;
   lex_start(thd);
@@ -1799,10 +1777,6 @@ static bool migrate_table_to_dd(THD *thd, const String_type &schema_name,
       key_info_buffer, key_count, Alter_info::ENABLE, fk_key_info_buffer,
       fk_number, &cc_spec_list_unused, table.file);
 
-  // Check for usage of prefix key index in PARTITION BY KEY() function.
-  dd::warn_on_deprecated_prefix_key_partition(
-      thd, schema_name.c_str(), table_name.c_str(), table_def.get(), true);
-
   if (!table_def || thd->dd_client()->store(table_def.get())) {
     LogErr(ERROR_LEVEL, ER_DD_ERROR_CREATING_ENTRY, schema_name.c_str(),
            table_name.c_str());
@@ -1844,11 +1818,9 @@ bool migrate_plugin_table_to_dd(THD *thd) {
 }
 
 /**
-  Migration of NDB tables is deferred until later, except for:
-  1. Legacy privilege tables stored in NDB, which must be migrated now so that
-     they can be moved to InnoDB later in the upgrade.
-  2. Tables that have associated triggers which must be migrated now to avoid
-     loss of the triggers.
+  Migration of NDB tables is deferred until later, except for legacy privilege
+  tables stored in NDB, which must be migrated now so that they can be moved to
+  InnoDB later in the upgrade.
 
   To check whether the table is a NDB table, look for the presence of a
   table_name.ndb file in the data directory. These files still exist at this
@@ -1873,8 +1845,6 @@ static bool is_skipped_ndb_table(const char *db_name, const char *table_name) {
     }
   }
 
-  if (Trigger_loader::trg_file_exists(db_name, table_name)) return false;
-
   return true;
 }
 
@@ -1926,11 +1896,7 @@ bool migrate_all_frm_to_dd(THD *thd, const char *dbname,
       file.erase(file.size() - 4, 4);
       // Construct the schema name from its canonical format.
       filename_to_tablename(dbname, schema_name, sizeof(schema_name));
-
-      bool has_invalid_name = false;
-      filename_to_tablename(file.c_str(), table_name, sizeof(table_name), false,
-                            &has_invalid_name);
-      if (has_invalid_name) error = true;
+      filename_to_tablename(file.c_str(), table_name, sizeof(table_name));
 
       /*
         Check that schema- and table names do not break the selected l_c_t_n
@@ -1956,13 +1922,13 @@ bool migrate_all_frm_to_dd(THD *thd, const char *dbname,
         hence, we do not need to check this issue at the SQL layer.
 
         See also the corresponding check in migrate_schema_to_dd(). We do
-        not repeat the assertS here, but check only if l_c_t_n = 1.
+        not repeat the DBUG_ASSERTS here, but check only if l_c_t_n = 1.
         Additionally, we do the check below only once, hence the test for
         is_fix_view_cols_and_deps = false.
       */
       if (!is_fix_view_cols_and_deps && lower_case_table_names == 1) {
         // Already checked schema name while migrating schema meta data.
-        assert(is_string_in_lowercase(schema_name, system_charset_info));
+        DBUG_ASSERT(is_string_in_lowercase(schema_name, system_charset_info));
         // Upper case table names break invariant when l_c_t_n = 1.
         if (!is_string_in_lowercase(table_name, system_charset_info)) {
           LogErr(ERROR_LEVEL, ER_TABLE_NAME_IN_UPPER_CASE_NOT_ALLOWED,
@@ -1987,6 +1953,8 @@ bool migrate_all_frm_to_dd(THD *thd, const char *dbname,
       // Skip NDB tables which are upgraded later by the ndbcluster plugin
       if (is_skipped_ndb_table(schema_name, table_name)) continue;
 
+      log_sink_buffer_check_timeout();
+
       // Create an entry in the new DD.
       bool result = false;
       result = migrate_table_to_dd(thd, schema_name, table_name,
@@ -2002,9 +1970,9 @@ bool migrate_all_frm_to_dd(THD *thd, const char *dbname,
         Upgrade process does not stop immediately if it encounters any error.
         All the tables in the data directory are processed and all error are
         reported to user at once. Server code has many checks for error in DA.
-        if thd->is_error() return true, attempt to upgrade all subsequent tables
+        if thd->is_error() return true, atempt to upgrade all subsequent tables
         will fail and error log will report error false positives.
-       */
+     */
       thd->clear_error();
       root.ClearForReuse();
     }
