@@ -1,4 +1,4 @@
-/* Copyright (c) 2014, 2022, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -26,7 +26,6 @@
 
 #include <string.h>
 #include <algorithm>
-#include <optional>
 #include <string>
 #include <type_traits>
 
@@ -41,13 +40,14 @@
 #include "my_dbug.h"
 #include "my_loglevel.h"
 #include "my_macros.h"
-#include "mysql/components/services/bits/psi_bits.h"
 #include "mysql/components/services/log_builtins.h"
 #include "mysql/components/services/log_shared.h"
 #include "mysql/plugin.h"
+#include "mysql/psi/psi_base.h"
 #include "mysql/udf_registration_types.h"
 #include "mysql_com.h"
 #include "mysqld_error.h"
+#include "nullable.h"
 #include "sql/dd/collection.h"
 #include "sql/dd/dd_table.h"       // dd::FIELD_NAME_SEPARATOR_CHAR
 #include "sql/dd/dd_tablespace.h"  // dd::get_tablespace_name
@@ -190,7 +190,7 @@ enum_field_types dd_get_old_field_type(dd::enum_column_types type) {
       return MYSQL_TYPE_JSON;
 
     default:
-      assert(!"Should not hit here"); /* purecov: deadcode */
+      DBUG_ASSERT(!"Should not hit here"); /* purecov: deadcode */
   }
 
   return MYSQL_TYPE_LONG;
@@ -216,7 +216,7 @@ static enum ha_key_alg dd_get_old_index_algorithm_type(
       return HA_KEY_ALG_FULLTEXT;
 
     default:
-      assert(!"Should not hit here"); /* purecov: deadcode */
+      DBUG_ASSERT(!"Should not hit here"); /* purecov: deadcode */
   }
 
   return HA_KEY_ALG_SE_SPECIFIC;
@@ -234,7 +234,7 @@ bool is_suitable_for_primary_key(KEY_PART_INFO *key_part, Field *table_field) {
 
   /*
     If the key column is of NOT NULL BLOB type, then it
-    will definitely have key prefix. And if key part prefix size
+    will definitly have key prefix. And if key part prefix size
     is equal to the BLOB column max size, then we can promote
     it to primary key.
    */
@@ -260,6 +260,7 @@ bool is_suitable_for_primary_key(KEY_PART_INFO *key_part, Field *table_field) {
 static bool prepare_share(THD *thd, TABLE_SHARE *share,
                           const dd::Table *table_def) {
   my_bitmap_map *bitmaps;
+  bool use_hash;
   handler *handler_file = nullptr;
 
   // Mark 'system' tables (tables with one row) to help the Optimizer.
@@ -268,6 +269,18 @@ static bool prepare_share(THD *thd, TABLE_SHARE *share,
 
   bool use_extended_sk = ha_check_storage_engine_flag(
       share->db_type(), HTON_SUPPORTS_EXTENDED_KEYS);
+  // Setup name_hash for quick look-up
+  use_hash = share->fields >= MAX_FIELDS_BEFORE_HASH;
+  if (use_hash) {
+    Field **field_ptr = share->field;
+    share->name_hash = new collation_unordered_map<std::string, Field **>(
+        system_charset_info, PSI_INSTRUMENT_ME);
+    share->name_hash->reserve(share->fields);
+
+    for (uint i = 0; i < share->fields; i++, field_ptr++) {
+      share->name_hash->emplace((*field_ptr)->field_name, field_ptr);
+    }
+  }
 
   share->m_histograms =
       new malloc_unordered_map<uint, const histograms::Histogram *>(
@@ -315,10 +328,10 @@ static bool prepare_share(THD *thd, TABLE_SHARE *share,
       keyinfo->name = share->keynames.type_names[key];
 
       /* Check that fulltext and spatial keys have correct algorithm set. */
-      assert(!(share->key_info[key].flags & HA_FULLTEXT) ||
-             share->key_info[key].algorithm == HA_KEY_ALG_FULLTEXT);
-      assert(!(share->key_info[key].flags & HA_SPATIAL) ||
-             share->key_info[key].algorithm == HA_KEY_ALG_RTREE);
+      DBUG_ASSERT(!(share->key_info[key].flags & HA_FULLTEXT) ||
+                  share->key_info[key].algorithm == HA_KEY_ALG_FULLTEXT);
+      DBUG_ASSERT(!(share->key_info[key].flags & HA_SPATIAL) ||
+                  share->key_info[key].algorithm == HA_KEY_ALG_RTREE);
 
       if (primary_key >= MAX_KEY && (keyinfo->flags & HA_NOSAME)) {
         /*
@@ -339,7 +352,7 @@ static bool prepare_share(THD *thd, TABLE_SHARE *share,
           Check that dd::Index::is_candidate_key() used by SEs works in
           the same way as above call to is_suitable_for_primary_key().
         */
-        assert((primary_key == key) == (*idx_it)->is_candidate_key());
+        DBUG_ASSERT((primary_key == key) == (*idx_it)->is_candidate_key());
       }
 
       dd::Index::Index_elements::const_iterator idx_el_it(
@@ -379,16 +392,16 @@ static bool prepare_share(THD *thd, TABLE_SHARE *share,
         setup_key_part_field(share, handler_file, primary_key, keyinfo, key, i,
                              &usable_parts, true);
 
-        field->set_flag(PART_KEY_FLAG);
+        field->flags |= PART_KEY_FLAG;
         if (key == primary_key) {
-          field->set_flag(PRI_KEY_FLAG);
+          field->flags |= PRI_KEY_FLAG;
           /*
              If this field is part of the primary key and all keys contains
              the primary key, then we can use any key to find this column
              */
           if (ha_option & HA_PRIMARY_KEY_IN_READ_INDEX) {
             if (field->key_length() == key_part->length &&
-                !field->is_flag_set(BLOB_FLAG))
+                !(field->flags & BLOB_FLAG))
               field->part_of_key = share->keys_in_use;
             if (field->part_of_sortkey.is_set(key))
               field->part_of_sortkey = share->keys_in_use;
@@ -427,8 +440,9 @@ static bool prepare_share(THD *thd, TABLE_SHARE *share,
           Check that dd::Index_element::is_prefix() used by SEs works in
           the same way as code which sets HA_PART_KEY_SEG flag.
         */
-        assert((*idx_el_it)->is_prefix() ==
-               static_cast<bool>(key_part->key_part_flag & HA_PART_KEY_SEG));
+        DBUG_ASSERT(
+            (*idx_el_it)->is_prefix() ==
+            static_cast<bool>(key_part->key_part_flag & HA_PART_KEY_SEG));
         ++idx_el_it;
       }
 
@@ -438,11 +452,11 @@ static bool prepare_share(THD *thd, TABLE_SHARE *share,
       */
       keyinfo->actual_flags = keyinfo->flags;
 
-      if (primary_key < MAX_KEY && key != primary_key &&
-          (ha_option & HA_PRIMARY_KEY_IN_READ_INDEX))
-        key_part += add_pk_parts_to_sk(keyinfo, key, share->key_info,
-                                       primary_key, share, handler_file,
-                                       &usable_parts, use_extended_sk);
+      if (use_extended_sk && primary_key < MAX_KEY && key &&
+          !(keyinfo->flags & HA_NOSAME))
+        key_part +=
+            add_pk_parts_to_sk(keyinfo, key, share->key_info, primary_key,
+                               share, handler_file, &usable_parts);
 
       /* Skip unused key parts if they exist */
       key_part += keyinfo->unused_key_parts;
@@ -495,7 +509,7 @@ static bool prepare_share(THD *thd, TABLE_SHARE *share,
                "Wrong field definition.");
       return true;
     } else
-      reg_field->set_flag(AUTO_INCREMENT_FLAG);
+      reg_field->flags |= AUTO_INCREMENT_FLAG;
   }
 
   if (share->blob_fields) {
@@ -507,7 +521,7 @@ static bool prepare_share(THD *thd, TABLE_SHARE *share,
               (uint)(share->blob_fields * sizeof(uint)))))
       return true;  // OOM error message already reported
     for (k = 0, ptr = share->field; *ptr; ptr++, k++) {
-      if ((*ptr)->is_flag_set(BLOB_FLAG) || (*ptr)->is_array()) (*save++) = k;
+      if ((*ptr)->flags & BLOB_FLAG || (*ptr)->is_array()) (*save++) = k;
     }
   }
 
@@ -552,7 +566,7 @@ static row_type dd_get_old_row_format(dd::Table::enum_row_format new_format) {
     case dd::Table::RF_PAGED:
       return ROW_TYPE_PAGED;
     default:
-      assert(0);
+      DBUG_ASSERT(0);
       break;
   }
   return ROW_TYPE_FIXED;
@@ -570,7 +584,7 @@ static bool fill_share_from_dd(THD *thd, TABLE_SHARE *share,
   } else {
     // If no secondary storage engine is set, the share cannot
     // represent a table in a secondary engine.
-    assert(!share->is_secondary_engine());
+    DBUG_ASSERT(!share->is_secondary_engine());
   }
 
   // Read table engine type
@@ -580,12 +594,12 @@ static bool fill_share_from_dd(THD *thd, TABLE_SHARE *share,
 
   plugin_ref tmp_plugin = ha_resolve_by_name_raw(thd, engine_name);
   if (tmp_plugin) {
-#ifndef NDEBUG
+#ifndef DBUG_OFF
     handlerton *hton = plugin_data<handlerton *>(tmp_plugin);
 #endif
 
-    assert(hton && ha_storage_engine_is_enabled(hton));
-    assert(!ha_check_storage_engine_flag(hton, HTON_NOT_USER_SELECTABLE));
+    DBUG_ASSERT(hton && ha_storage_engine_is_enabled(hton));
+    DBUG_ASSERT(!ha_check_storage_engine_flag(hton, HTON_NOT_USER_SELECTABLE));
 
     plugin_unlock(nullptr, share->db_plugin);
     share->db_plugin = my_plugin_lock(nullptr, &tmp_plugin);
@@ -595,7 +609,7 @@ static bool fill_share_from_dd(THD *thd, TABLE_SHARE *share,
   }
 
   // Set temporarily a good value for db_low_byte_first.
-  assert(ha_legacy_type(share->db_type()) != DB_TYPE_ISAM);
+  DBUG_ASSERT(ha_legacy_type(share->db_type()) != DB_TYPE_ISAM);
   share->db_low_byte_first = true;
 
   // Read other table options
@@ -714,12 +728,6 @@ static bool fill_share_from_dd(THD *thd, TABLE_SHARE *share,
     share->comment.length = comment.length();
   }
 
-  // Copy SE attributes into share's memroot
-  share->engine_attribute = LexStringDupRootUnlessEmpty(
-      &share->mem_root, tab_obj->engine_attribute());
-  share->secondary_engine_attribute = LexStringDupRootUnlessEmpty(
-      &share->mem_root, tab_obj->secondary_engine_attribute());
-
   // Read Connection strings
   if (table_options.exists("connection_string"))
     table_options.get("connection_string", &share->connect_string,
@@ -770,11 +778,12 @@ inline void get_auto_flags(const dd::Column &col_obj, uint &auto_flags) {
       // "CURRENT_TIMESTAP(<integer>)". Stored functions with names
       // starting with "CURRENT_TIMESTAMP" should be filtered out before
       // we get here.
-      assert(col_obj.default_option().size() == 17 ||
-             (col_obj.default_option().size() >= 20 &&
-              col_obj.default_option()[17] == '(' &&
-              col_obj.default_option()[col_obj.default_option().size() - 1] ==
-                  ')'));
+      DBUG_ASSERT(
+          col_obj.default_option().size() == 17 ||
+          (col_obj.default_option().size() >= 20 &&
+           col_obj.default_option()[17] == '(' &&
+           col_obj.default_option()[col_obj.default_option().size() - 1] ==
+               ')'));
 
       auto_flags |= Field::DEFAULT_NOW;
     } else {
@@ -794,9 +803,9 @@ inline void get_auto_flags(const dd::Column &col_obj, uint &auto_flags) {
     Columns can't have AUTO_INCREMENT and DEFAULT/ON UPDATE CURRENT_TIMESTAMP at
     the same time.
   */
-  assert(!((auto_flags & (Field::DEFAULT_NOW | Field::ON_UPDATE_NOW |
-                          Field::GENERATED_FROM_EXPRESSION)) != 0 &&
-           (auto_flags & Field::NEXT_NUMBER) != 0));
+  DBUG_ASSERT(!((auto_flags & (Field::DEFAULT_NOW | Field::ON_UPDATE_NOW |
+                               Field::GENERATED_FROM_EXPRESSION)) != 0 &&
+                (auto_flags & Field::NEXT_NUMBER) != 0));
 }
 
 static Field *make_field(const dd::Column &col_obj, const CHARSET_INFO *charset,
@@ -847,14 +856,14 @@ static Field *make_field(const dd::Column &col_obj, const CHARSET_INFO *charset,
   // Column name
   char *name = nullptr;
   dd::String_type s = col_obj.name();
-  assert(!s.empty());
+  DBUG_ASSERT(!s.empty());
   name = strmake_root(&share->mem_root, s.c_str(), s.length());
   name[s.length()] = '\0';
 
   uint decimals;
   // Decimals
   if (field_type == MYSQL_TYPE_DECIMAL || field_type == MYSQL_TYPE_NEWDECIMAL) {
-    assert(col_obj.is_numeric_scale_null() == false);
+    DBUG_ASSERT(col_obj.is_numeric_scale_null() == false);
     decimals = col_obj.numeric_scale();
   } else if (field_type == MYSQL_TYPE_FLOAT ||
              field_type == MYSQL_TYPE_DOUBLE) {
@@ -905,7 +914,7 @@ static bool fill_column_from_dd(THD *thd, TABLE_SHARE *share,
 
   // Column name
   dd::String_type s = col_obj->name();
-  assert(!s.empty());
+  DBUG_ASSERT(!s.empty());
   name = strmake_root(&share->mem_root, s.c_str(), s.length());
   name[s.length()] = '\0';
 
@@ -934,7 +943,7 @@ static bool fill_column_from_dd(THD *thd, TABLE_SHARE *share,
 
   // Decimals
   if (field_type == MYSQL_TYPE_DECIMAL || field_type == MYSQL_TYPE_NEWDECIMAL)
-    assert(col_obj->is_numeric_scale_null() == false);
+    DBUG_ASSERT(col_obj->is_numeric_scale_null() == false);
 
   // Read geometry sub type
   if (field_type == MYSQL_TYPE_GEOMETRY) {
@@ -995,7 +1004,7 @@ static bool fill_column_from_dd(THD *thd, TABLE_SHARE *share,
   //
   reg_field =
       make_field(*col_obj, charset, share, rec_pos, null_pos, null_bit_pos);
-  reg_field->set_field_index(field_nr);
+  reg_field->field_index = field_nr;
   reg_field->stored_in_db = true;
 
   // Handle generated columns
@@ -1044,7 +1053,7 @@ static bool fill_column_from_dd(THD *thd, TABLE_SHARE *share,
     share->found_next_number_field = &share->field[field_nr];
 
   // Set field flags
-  if (col_obj->has_no_default()) reg_field->set_flag(NO_DEFAULT_VALUE_FLAG);
+  if (col_obj->has_no_default()) reg_field->flags |= NO_DEFAULT_VALUE_FLAG;
 
   // Set default value or NULL. Reset required for e.g. CHAR.
   if (col_obj->is_default_value_null()) {
@@ -1082,16 +1091,10 @@ static bool fill_column_from_dd(THD *thd, TABLE_SHARE *share,
 
   // NOT SECONDARY column option.
   if (column_options->exists("not_secondary"))
-    reg_field->set_flag(NOT_SECONDARY_FLAG);
+    reg_field->flags |= NOT_SECONDARY_FLAG;
 
   reg_field->set_hidden(col_obj->hidden());
 
-  reg_field->m_engine_attribute = LexStringDupRootUnlessEmpty(
-      &share->mem_root, col_obj->engine_attribute());
-
-  reg_field->m_secondary_engine_attribute = LexStringDupRootUnlessEmpty(
-      &share->mem_root, col_obj->secondary_engine_attribute());
-
   // Field is prepared. Store it in 'share'
   share->field[field_nr] = reg_field;
 
@@ -1113,7 +1116,7 @@ static bool fill_columns_from_dd(THD *thd, TABLE_SHARE *share,
   share->gen_def_field_count = 0;
 
   // Iterate through all the columns.
-  uchar *null_flags [[maybe_unused]];
+  uchar *null_flags MY_ATTRIBUTE((unused));
   uchar *null_pos, *rec_pos;
   null_flags = null_pos = share->default_values;
   rec_pos = share->default_values + share->null_bytes;
@@ -1192,9 +1195,10 @@ static bool fill_columns_from_dd(THD *thd, TABLE_SHARE *share,
   }
 
   // Make sure the scan of the columns is consistent with other data.
-  assert(share->null_bytes == (null_pos - null_flags + (null_bit_pos + 7) / 8));
-  assert(share->last_null_bit_pos == null_bit_pos);
-  assert(share->fields == field_nr);
+  DBUG_ASSERT(share->null_bytes ==
+              (null_pos - null_flags + (null_bit_pos + 7) / 8));
+  DBUG_ASSERT(share->last_null_bit_pos == null_bit_pos);
+  DBUG_ASSERT(share->fields == field_nr);
 
   return (false);
 }
@@ -1214,7 +1218,7 @@ static void fill_index_element_from_dd(TABLE_SHARE *share,
   keypart->fieldnr = idx_elem_obj->column().ordinal_position();
 
   // field
-  assert(keypart->fieldnr > 0);
+  DBUG_ASSERT(keypart->fieldnr > 0);
   Field *field = keypart->field = share->field[keypart->fieldnr - 1];
 
   // offset
@@ -1315,7 +1319,7 @@ static bool fill_index_from_dd(THD *thd, TABLE_SHARE *share,
       keyinfo->flags = HA_NOSAME;
       break;
     default:
-      assert(0); /* purecov: deadcode */
+      DBUG_ASSERT(0); /* purecov: deadcode */
       keyinfo->flags = 0;
       break;
   }
@@ -1349,7 +1353,7 @@ static bool fill_index_from_dd(THD *thd, TABLE_SHARE *share,
   */
   uint32 stored_flags = 0;
   idx_options.get("flags", &stored_flags);
-  assert((stored_flags & ~(HA_PACK_KEY | HA_BINARY_PACK_KEY)) == 0);
+  DBUG_ASSERT((stored_flags & ~(HA_PACK_KEY | HA_BINARY_PACK_KEY)) == 0);
 
   //  Beginning in 8.0.12 HA_PACK_KEY and HA_BINARY_PACK_KEY are no longer set
   //  if the SE does not support it. If the index was created prior to 8.0.12
@@ -1375,7 +1379,7 @@ static bool fill_index_from_dd(THD *thd, TABLE_SHARE *share,
   if (idx_options.exists("block_size")) {
     idx_options.get("block_size", &keyinfo->block_size);
 
-    assert(keyinfo->block_size);
+    DBUG_ASSERT(keyinfo->block_size);
 
     keyinfo->flags |= HA_USES_BLOCK_SIZE;
   }
@@ -1384,7 +1388,7 @@ static bool fill_index_from_dd(THD *thd, TABLE_SHARE *share,
   if (idx_options.exists("parser_name")) {
     LEX_CSTRING parser_name;
     if (idx_options.get("parser_name", &parser_name, &share->mem_root))
-      assert(false);
+      DBUG_ASSERT(false);
 
     keyinfo->parser =
         my_plugin_lock_by_name(nullptr, parser_name, MYSQL_FTPARSER_PLUGIN);
@@ -1407,15 +1411,7 @@ static bool fill_index_from_dd(THD *thd, TABLE_SHARE *share,
 
     keyinfo->flags |= HA_USES_COMMENT;
   }
-  keyinfo->engine_attribute = LexStringDupRootUnlessEmpty(
-      &share->mem_root, idx_obj->engine_attribute());
-  if (keyinfo->engine_attribute.length > 0)
-    keyinfo->flags |= HA_INDEX_USES_ENGINE_ATTRIBUTE;
 
-  keyinfo->secondary_engine_attribute = LexStringDupRootUnlessEmpty(
-      &share->mem_root, idx_obj->secondary_engine_attribute());
-  if (keyinfo->secondary_engine_attribute.length > 0)
-    keyinfo->flags |= HA_INDEX_USES_SECONDARY_ENGINE_ATTRIBUTE;
   return (false);
 }
 
@@ -1468,7 +1464,7 @@ static bool fill_indexes_from_dd(THD *thd, TABLE_SHARE *share,
 
   // Count number of keys and total number of key parts in the table.
 
-  assert(share->keys == 0 && share->key_parts == 0);
+  DBUG_ASSERT(share->keys == 0 && share->key_parts == 0);
 
   for (const dd::Index *idx_obj : tab_obj->indexes()) {
     // Skip hidden indexes
@@ -1513,15 +1509,18 @@ static bool fill_indexes_from_dd(THD *thd, TABLE_SHARE *share,
       return true; /* purecov: inspected */
 
     //
-    // Alloc buffers to hold keys and key_parts
+    // Alloc buffer to hold keys and key_parts
     //
 
-    if (!(share->key_info = share->mem_root.ArrayAlloc<KEY>(share->keys)))
+    if (!(share->key_info = (KEY *)share->mem_root.Alloc(
+              share->keys * sizeof(KEY) +
+              total_key_parts * sizeof(KEY_PART_INFO))))
       return true; /* purecov: inspected */
 
-    if (!(key_part =
-              share->mem_root.ArrayAlloc<KEY_PART_INFO>(total_key_parts)))
-      return true; /* purecov: inspected */
+    memset(
+        share->key_info, 0,
+        (share->keys * sizeof(KEY) + total_key_parts * sizeof(KEY_PART_INFO)));
+    key_part = (KEY_PART_INFO *)(share->key_info + share->keys);
 
     //
     // Alloc buffer to hold keynames
@@ -1536,8 +1535,8 @@ static bool fill_indexes_from_dd(THD *thd, TABLE_SHARE *share,
     share->keynames.count = share->keys;
 
     // In first iteration get all the index_obj, so that we get all
-    // user_defined_key_parts for each key. This is required to properly
-    // allocate key_part memory for keys.
+    // user_defined_key_parts for each key. This is required to propertly
+    // allocation key_part memory for keys.
     const dd::Index *index_at_pos[MAX_INDEXES];
     uint key_nr = 0;
     for (const dd::Index *idx_obj : tab_obj->indexes()) {
@@ -1631,7 +1630,7 @@ static void get_partition_options(MEM_ROOT *mem_root,
   if (part_options.exists("nodegroup_id"))
     part_options.get("nodegroup_id", &nodegroup_id);
 
-  assert(nodegroup_id <= 0xFFFF);
+  DBUG_ASSERT(nodegroup_id <= 0xFFFF);
   part_elem->nodegroup_id = nodegroup_id;
 }
 
@@ -1650,7 +1649,7 @@ static bool get_part_column_values(MEM_ROOT *mem_root,
     entries++;
   }
   if (entries != ((max_column_id + 1) * (max_list_index + 1))) {
-    assert(0); /* purecov: deadcode */
+    DBUG_ASSERT(0); /* purecov: deadcode */
     return true;
   }
 
@@ -1693,11 +1692,11 @@ static bool get_part_column_values(MEM_ROOT *mem_root,
 
   for (list_index = 0; list_index <= max_list_index; list_index++) {
     p_val = &p_elem_values[list_index];
-#ifndef NDEBUG
+#ifndef DBUG_OFF
     for (uint i = 0; i < part_info->num_columns; i++) {
-      assert(p_val->col_val_array[i].null_value ||
-             p_val->col_val_array[i].max_value ||
-             p_val->col_val_array[i].column_value.value_str);
+      DBUG_ASSERT(p_val->col_val_array[i].null_value ||
+                  p_val->col_val_array[i].max_value ||
+                  p_val->col_val_array[i].column_value.value_str);
     }
 #endif
     if (part_elem->list_val_list.push_back(p_val, mem_root)) return true;
@@ -1738,10 +1737,10 @@ static bool setup_partition_from_dd(THD *thd, MEM_ROOT *mem_root,
       if (get_part_column_values(mem_root, part_info, part_elem, part_obj))
         return true;
     } else {
-      assert(part_obj->values().size() == 1);
+      DBUG_ASSERT(part_obj->values().size() == 1);
       const dd::Partition_value *part_value = *part_obj->values().begin();
-      assert(part_value->list_num() == 0);
-      assert(part_value->column_num() == 0);
+      DBUG_ASSERT(part_value->list_num() == 0);
+      DBUG_ASSERT(part_value->column_num() == 0);
       if (part_value->max_value()) {
         part_elem->max_value = true;
       } else {
@@ -1771,13 +1770,13 @@ static bool setup_partition_from_dd(THD *thd, MEM_ROOT *mem_root,
         max_index = std::max(max_index, part_value->list_num());
         entries++;
         if (part_value->value_utf8().empty()) {
-          assert(!part_elem->has_null_value);
+          DBUG_ASSERT(!part_elem->has_null_value);
           part_elem->has_null_value = true;
           null_entry = part_value->list_num();
         }
       }
       if (entries != (max_index + 1)) {
-        assert(0); /* purecov: deadcode */
+        DBUG_ASSERT(0); /* purecov: deadcode */
         return true;
       }
       /* If a list entry is NULL then it is only flagged on the part_elem. */
@@ -1791,9 +1790,9 @@ static bool setup_partition_from_dd(THD *thd, MEM_ROOT *mem_root,
       }
 
       for (const dd::Partition_value *part_value : part_obj->values()) {
-        assert(part_value->column_num() == 0);
+        DBUG_ASSERT(part_value->column_num() == 0);
         if (part_value->value_utf8().empty()) {
-          assert(part_value->list_num() == null_entry);
+          DBUG_ASSERT(part_value->list_num() == null_entry);
           continue;
         }
         list_index = part_value->list_num();
@@ -1805,7 +1804,7 @@ static bool setup_partition_from_dd(THD *thd, MEM_ROOT *mem_root,
         */
         if (part_elem->has_null_value && list_index > null_entry) list_index--;
         list_val = &list_val_array[list_index];
-        assert(!list_val->unsigned_flag && !list_val->value);
+        DBUG_ASSERT(!list_val->unsigned_flag && !list_val->value);
         if (part_value->value_utf8()[0] == '-') {
           list_val->unsigned_flag = false;
           if (dd::Properties::from_str(part_value->value_utf8(),
@@ -1824,9 +1823,9 @@ static bool setup_partition_from_dd(THD *thd, MEM_ROOT *mem_root,
       }
     }
   } else {
-#ifndef NDEBUG
-    assert(part_info->part_type == partition_type::HASH);
-    assert(part_obj->values().empty());
+#ifndef DBUG_OFF
+    DBUG_ASSERT(part_info->part_type == partition_type::HASH);
+    DBUG_ASSERT(part_obj->values().empty());
 #endif
   }
   return false;
@@ -1857,7 +1856,7 @@ static bool set_field_list(MEM_ROOT *mem_root, dd::String_type &str,
     if (dd::eat_str(field_name, it, end, dd::FIELD_NAME_SEPARATOR_CHAR))
       return true;
     size_t len = field_name.length();
-    assert(len);
+    DBUG_ASSERT(len);
     char *name = static_cast<char *>(mem_root->Alloc(len + 1));
     if (!name) return true; /* purecov: inspected */
     memcpy(name, field_name.c_str(), len);
@@ -1910,7 +1909,7 @@ static bool fill_partitioning_from_dd(THD *thd, TABLE_SHARE *share,
 
   handlerton *hton = plugin_data<handlerton *>(
       ha_resolve_by_name_raw(thd, lex_cstring_handle(tab_obj->engine())));
-  assert(hton && ha_storage_engine_is_enabled(hton));
+  DBUG_ASSERT(hton && ha_storage_engine_is_enabled(hton));
   part_info->default_engine_type = hton;
   if (!part_info->default_engine_type) return true;
 
@@ -1919,26 +1918,26 @@ static bool fill_partitioning_from_dd(THD *thd, TABLE_SHARE *share,
     case dd::Table::PT_RANGE_COLUMNS:
       part_info->column_list = true;
       part_info->list_of_part_fields = true;
-      [[fallthrough]];
+      // Fall through.
     case dd::Table::PT_RANGE:
       part_info->part_type = partition_type::RANGE;
       break;
     case dd::Table::PT_LIST_COLUMNS:
       part_info->column_list = true;
       part_info->list_of_part_fields = true;
-      [[fallthrough]];
+      // Fall through.
     case dd::Table::PT_LIST:
       part_info->part_type = partition_type::LIST;
       break;
     case dd::Table::PT_LINEAR_HASH:
       part_info->linear_hash_ind = true;
-      [[fallthrough]];
+      // Fall through.
     case dd::Table::PT_HASH:
       part_info->part_type = partition_type::HASH;
       break;
     case dd::Table::PT_LINEAR_KEY_51:
       part_info->linear_hash_ind = true;
-      [[fallthrough]];
+      // Fall through.
     case dd::Table::PT_KEY_51:
       part_info->key_algorithm = enum_key_algorithm::KEY_ALGORITHM_51;
       part_info->list_of_part_fields = true;
@@ -1946,7 +1945,7 @@ static bool fill_partitioning_from_dd(THD *thd, TABLE_SHARE *share,
       break;
     case dd::Table::PT_LINEAR_KEY_55:
       part_info->linear_hash_ind = true;
-      [[fallthrough]];
+      // Fall through.
     case dd::Table::PT_KEY_55:
       part_info->key_algorithm = enum_key_algorithm::KEY_ALGORITHM_55;
       part_info->list_of_part_fields = true;
@@ -1954,7 +1953,7 @@ static bool fill_partitioning_from_dd(THD *thd, TABLE_SHARE *share,
       break;
     case dd::Table::PT_AUTO_LINEAR:
       part_info->linear_hash_ind = true;
-      [[fallthrough]];
+      // Fall through.
     case dd::Table::PT_AUTO:
       part_info->key_algorithm = enum_key_algorithm::KEY_ALGORITHM_55;
       part_info->part_type = partition_type::HASH;
@@ -1964,7 +1963,7 @@ static bool fill_partitioning_from_dd(THD *thd, TABLE_SHARE *share,
       break;
     default:
       // Unknown partitioning type!
-      assert(0); /* purecov: deadcode */
+      DBUG_ASSERT(0); /* purecov: deadcode */
       return true;
   }
   switch (tab_obj->subpartition_type()) {
@@ -1973,13 +1972,13 @@ static bool fill_partitioning_from_dd(THD *thd, TABLE_SHARE *share,
       break;
     case dd::Table::ST_LINEAR_HASH:
       part_info->linear_hash_ind = true;
-      [[fallthrough]];
+      // Fall through.
     case dd::Table::ST_HASH:
       part_info->subpart_type = partition_type::HASH;
       break;
     case dd::Table::ST_LINEAR_KEY_51:
       part_info->linear_hash_ind = true;
-      [[fallthrough]];
+      // Fall through.
     case dd::Table::ST_KEY_51:
       part_info->key_algorithm = enum_key_algorithm::KEY_ALGORITHM_51;
       part_info->list_of_subpart_fields = true;
@@ -1987,7 +1986,7 @@ static bool fill_partitioning_from_dd(THD *thd, TABLE_SHARE *share,
       break;
     case dd::Table::ST_LINEAR_KEY_55:
       part_info->linear_hash_ind = true;
-      [[fallthrough]];
+      // Fall through.
     case dd::Table::ST_KEY_55:
       part_info->key_algorithm = enum_key_algorithm::KEY_ALGORITHM_55;
       part_info->list_of_subpart_fields = true;
@@ -1995,7 +1994,7 @@ static bool fill_partitioning_from_dd(THD *thd, TABLE_SHARE *share,
       break;
     default:
       // Unknown sub partitioning type!
-      assert(0); /* purecov: deadcode */
+      DBUG_ASSERT(0); /* purecov: deadcode */
       return true;
   }
 
@@ -2037,17 +2036,17 @@ static bool fill_partitioning_from_dd(THD *thd, TABLE_SHARE *share,
 
   /* Partitions are sorted first on level and then on number. */
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   uint number = 0;
 #endif
   for (const dd::Partition *part_obj : tab_obj->partitions()) {
-#ifndef NDEBUG
+#ifndef DBUG_OFF
     /* Must be in sorted order (sorted by level first and then on number). */
-    assert(part_obj->number() >= number);
+    DBUG_ASSERT(part_obj->number() >= number);
     number = part_obj->number();
 #endif
 
-    assert(part_obj->parent_partition_id() == dd::INVALID_OBJECT_ID);
+    DBUG_ASSERT(part_obj->parent_partition_id() == dd::INVALID_OBJECT_ID);
 
     curr_part_elem = new (&share->mem_root) partition_element;
     if (!curr_part_elem) {
@@ -2099,7 +2098,7 @@ static bool fill_partitioning_from_dd(THD *thd, TABLE_SHARE *share,
       break;
     case dd::Table::DP_NONE:
     default:
-      assert(0); /* purecov: deadcode */
+      DBUG_ASSERT(0); /* purecov: deadcode */
   }
   switch (tab_obj->default_subpartitioning()) {
     case dd::Table::DP_NO:
@@ -2115,10 +2114,10 @@ static bool fill_partitioning_from_dd(THD *thd, TABLE_SHARE *share,
       part_info->use_default_num_subpartitions = false;
       break;
     case dd::Table::DP_NONE:
-      assert(!part_info->is_sub_partitioned());
+      DBUG_ASSERT(!part_info->is_sub_partitioned());
       break;
     default:
-      assert(0); /* purecov: deadcode */
+      DBUG_ASSERT(0); /* purecov: deadcode */
   }
 
   char *buf;
@@ -2147,7 +2146,7 @@ static bool fill_partitioning_from_dd(THD *thd, TABLE_SHARE *share,
 
 static bool fill_foreign_keys_from_dd(TABLE_SHARE *share,
                                       const dd::Table *tab_obj) {
-  assert(share->foreign_keys == 0 && share->foreign_key_parents == 0);
+  DBUG_ASSERT(share->foreign_keys == 0 && share->foreign_key_parents == 0);
 
   share->foreign_keys = tab_obj->foreign_keys().size();
   share->foreign_key_parents = tab_obj->foreign_key_parents().size();
@@ -2242,7 +2241,7 @@ static bool fill_foreign_keys_from_dd(TABLE_SHARE *share,
 */
 static bool fill_check_constraints_from_dd(TABLE_SHARE *share,
                                            const dd::Table *tab_obj) {
-  assert(share->check_constraint_share_list == nullptr);
+  DBUG_ASSERT(share->check_constraint_share_list == nullptr);
 
   if (tab_obj->check_constraints().size() > 0) {
     share->check_constraint_share_list = new (&share->mem_root)
@@ -2312,9 +2311,9 @@ bool open_table_def(THD *thd, TABLE_SHARE *share, const dd::Table &table_def) {
 */
 class Open_table_error_handler : public Internal_error_handler {
  public:
-  bool handle_condition(THD *, uint sql_errno, const char *,
-                        Sql_condition::enum_severity_level *,
-                        const char *) override {
+  virtual bool handle_condition(THD *, uint sql_errno, const char *,
+                                Sql_condition::enum_severity_level *,
+                                const char *) {
     return (sql_errno == ER_UNKNOWN_COLLATION ||
             sql_errno == ER_PLUGIN_IS_NOT_LOADED);
   }
