@@ -1,16 +1,15 @@
-/* Copyright (c) 2009, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2009, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -54,7 +53,7 @@
                           name into a global list and broadcasts the event which
                           wakes the threads that wait for this event.
 
-    - wait for a signal:  Wait on a event identified by the signal name until
+    - wait for a signal:  Wait on a event indentified by the signal name until
                           the signal thread signals the event.
 
   By default, all sync points are inactive. They do nothing (except to
@@ -357,38 +356,40 @@
 #include <time.h>
 #include <algorithm>
 #include <atomic>
+#include <boost/algorithm/string/classification.hpp>
+#include <boost/algorithm/string/split.hpp>
+#include <boost/algorithm/string/trim.hpp>
+#include <boost/concept/usage.hpp>
+#include <boost/iterator/iterator_facade.hpp>
+#include <boost/type_index/type_index_facade.hpp>
 #include <memory>
 #include <vector>
 
+#include "boost/algorithm/string/detail/classification.hpp"
+#include "m_ctype.h"
 #include "my_dbug.h"
 #include "my_inttypes.h"
+#include "my_loglevel.h"
 #include "my_macros.h"
 #include "my_psi_config.h"
 #include "my_sys.h"
 #include "my_systime.h"
 #include "my_thread.h"
-#include "mysql/components/services/bits/mysql_cond_bits.h"
-#include "mysql/components/services/bits/mysql_mutex_bits.h"
-#include "mysql/components/services/bits/psi_bits.h"
-#include "mysql/components/services/bits/psi_cond_bits.h"
-#include "mysql/components/services/bits/psi_memory_bits.h"
-#include "mysql/components/services/bits/psi_mutex_bits.h"
 #include "mysql/components/services/log_builtins.h"
-#include "mysql/my_loglevel.h"
+#include "mysql/components/services/mysql_cond_bits.h"
+#include "mysql/components/services/mysql_mutex_bits.h"
+#include "mysql/components/services/psi_cond_bits.h"
+#include "mysql/components/services/psi_memory_bits.h"
+#include "mysql/components/services/psi_mutex_bits.h"
 #include "mysql/plugin.h"
 #include "mysql/psi/mysql_cond.h"
 #include "mysql/psi/mysql_mutex.h"
+#include "mysql/psi/psi_base.h"
 #include "mysql/service_mysql_alloc.h"
-#include "mysql/strings/int2str.h"
-#include "mysql/strings/m_ctype.h"
 #include "mysqld_error.h"
-#include "nulls.h"
 #include "sql/sql_error.h"
 #include "sql/thr_malloc.h"
 #include "sql_string.h"
-#include "string_with_len.h"
-#include "strxnmov.h"
-#include "template_utils.h"
 #include "thr_mutex.h"
 
 #if defined(ENABLED_DEBUG_SYNC)
@@ -408,7 +409,7 @@ using std::min;
 /*
   Action to perform at a synchronization point.
   NOTE: This structure is moved around in memory by realloc(), qsort(),
-        and memmove(). Do not add objects with nontrivial constructors
+        and memmove(). Do not add objects with non-trivial constuctors
         or destructors, which might prevent moving of this structure
         with these functions.
 */
@@ -512,7 +513,7 @@ static void debug_sync_C_callback(const char *sync_point_name,
 }
 
 static PSI_memory_key key_debug_THD_debug_sync_control;
-static PSI_memory_key key_debug_THD_debug_sync_action;
+static PSI_memory_key key_debug_sync_action;
 
 #ifdef HAVE_PSI_INTERFACE
 static PSI_mutex_key key_debug_sync_globals_ds_mutex;
@@ -529,9 +530,9 @@ static PSI_cond_info all_debug_sync_conds[] = {
 
 static PSI_memory_info all_debug_sync_memory[] = {
     {&key_debug_THD_debug_sync_control, "THD::debug_sync_control", 0, 0,
-     "Structure to control debug sync per thread."},
-    {&key_debug_THD_debug_sync_action, "THD::debug_sync_action", 0, 0,
-     "Debug sync actions to perform per thread."}};
+     PSI_DOCUMENT_ME},
+    {&key_debug_sync_action, "debug_sync_control::debug_sync_action", 0, 0,
+     PSI_DOCUMENT_ME}};
 
 static void init_debug_sync_psi_keys(void) {
   const char *category = "sql";
@@ -559,8 +560,8 @@ static void init_debug_sync_psi_keys(void) {
   spurious data.
 */
 static const char *debug_sync_thd_proc_info(THD *thd, const char *info) {
-  const char *old_proc_info = thd->proc_info();
-  thd->set_proc_info(info);
+  const char *old_proc_info = thd->proc_info;
+  thd->proc_info = info;
   return old_proc_info;
 }
 
@@ -660,7 +661,7 @@ static void debug_sync_emergency_disable(void) {
 
 void debug_sync_init_thread(THD *thd) {
   DBUG_TRACE;
-  assert(thd);
+  DBUG_ASSERT(thd);
 
   if (opt_debug_sync_timeout) {
     thd->debug_sync_control = (st_debug_sync_control *)my_malloc(
@@ -676,9 +677,9 @@ void debug_sync_init_thread(THD *thd) {
   }
 }
 
-void debug_sync_claim_memory_ownership(THD *thd, bool claim) {
+void debug_sync_claim_memory_ownership(THD *thd) {
   DBUG_TRACE;
-  assert(thd);
+  DBUG_ASSERT(thd);
 
   st_debug_sync_control *ds_control = thd->debug_sync_control;
 
@@ -687,14 +688,14 @@ void debug_sync_claim_memory_ownership(THD *thd, bool claim) {
       st_debug_sync_action *action = ds_control->ds_action;
       st_debug_sync_action *action_end = action + ds_control->ds_allocated;
       for (; action < action_end; action++) {
-        action->signal.mem_claim(claim);
-        action->wait_for.mem_claim(claim);
-        action->sync_point.mem_claim(claim);
+        action->signal.mem_claim();
+        action->wait_for.mem_claim();
+        action->sync_point.mem_claim();
       }
-      my_claim(ds_control->ds_action, claim);
+      my_claim(ds_control->ds_action);
     }
 
-    my_claim(ds_control, claim);
+    my_claim(ds_control);
   }
 }
 
@@ -706,7 +707,7 @@ void debug_sync_claim_memory_ownership(THD *thd, bool claim) {
 
 void debug_sync_end_thread(THD *thd) {
   DBUG_TRACE;
-  assert(thd);
+  DBUG_ASSERT(thd);
 
   if (thd->debug_sync_control) {
     st_debug_sync_control *ds_control = thd->debug_sync_control;
@@ -754,15 +755,15 @@ void debug_sync_end_thread(THD *thd) {
 
 static char *debug_sync_bmove_len(char *to, char *to_end, const char *from,
                                   size_t length) {
-  assert(to);
-  assert(to_end);
-  assert(!length || from);
+  DBUG_ASSERT(to);
+  DBUG_ASSERT(to_end);
+  DBUG_ASSERT(!length || from);
   length = std::min(length, size_t(to_end - to));
   memcpy(to, from, length);
   return (to + length);
 }
 
-#if !defined(NDEBUG)
+#if !defined(DBUG_OFF)
 
 /**
   Create a string that describes an action.
@@ -776,12 +777,12 @@ static void debug_sync_action_string(char *result, uint size,
                                      st_debug_sync_action *action) {
   char *wtxt = result;
   char *wend = wtxt + size - 1; /* Allow emergency '\0'. */
-  assert(result);
-  assert(action);
+  DBUG_ASSERT(result);
+  DBUG_ASSERT(action);
 
   /* If an execute count is present, signal or wait_for are needed too. */
-  assert(!action->execute || action->signal.length() ||
-         action->wait_for.length());
+  DBUG_ASSERT(!action->execute || action->signal.length() ||
+              action->wait_for.length());
 
   if (action->execute) {
     if (action->signal.length()) {
@@ -825,7 +826,7 @@ static void debug_sync_print_actions(THD *thd) {
   st_debug_sync_control *ds_control = thd->debug_sync_control;
   uint idx;
   DBUG_TRACE;
-  assert(thd);
+  DBUG_ASSERT(thd);
 
   if (!ds_control) return;
 
@@ -839,7 +840,7 @@ static void debug_sync_print_actions(THD *thd) {
   }
 }
 
-#endif /* !defined(NDEBUG) */
+#endif /* !defined(DBUG_OFF) */
 
 /**
   Find a debug sync action.
@@ -865,9 +866,9 @@ static st_debug_sync_action *debug_sync_find(st_debug_sync_action *actionarr,
   int high;
   int mid;
   int diff;
-  assert(actionarr);
-  assert(dsp_name);
-  assert(name_len);
+  DBUG_ASSERT(actionarr);
+  DBUG_ASSERT(dsp_name);
+  DBUG_ASSERT(name_len);
 
   low = 0;
   high = quantity;
@@ -907,8 +908,8 @@ static st_debug_sync_action *debug_sync_find(st_debug_sync_action *actionarr,
 static void debug_sync_reset(THD *thd) {
   st_debug_sync_control *ds_control = thd->debug_sync_control;
   DBUG_TRACE;
-  assert(thd);
-  assert(ds_control);
+  DBUG_ASSERT(thd);
+  DBUG_ASSERT(ds_control);
 
   /* Remove all actions of this thread. */
   ds_control->ds_active = 0;
@@ -928,7 +929,7 @@ static void debug_sync_reset(THD *thd) {
   @description
     Removing an action mainly means to decrement the ds_active counter.
     But if the action is between other active action in the array, then
-    the array needs to be shrunk. The active actions above the one to
+    the array needs to be shrinked. The active actions above the one to
     be removed have to be moved down by one slot.
 */
 
@@ -936,11 +937,10 @@ static void debug_sync_remove_action(st_debug_sync_control *ds_control,
                                      st_debug_sync_action *action) {
   uint dsp_idx = static_cast<uint>(action - ds_control->ds_action);
   DBUG_TRACE;
-  assert(ds_control);
-  assert(current_thd == nullptr ||
-         ds_control == current_thd->debug_sync_control);
-  assert(action);
-  assert(dsp_idx < ds_control->ds_active);
+  DBUG_ASSERT(ds_control);
+  DBUG_ASSERT(ds_control == current_thd->debug_sync_control);
+  DBUG_ASSERT(action);
+  DBUG_ASSERT(dsp_idx < ds_control->ds_active);
 
   /* Decrement the number of currently active actions. */
   ds_control->ds_active--;
@@ -994,25 +994,25 @@ static st_debug_sync_action *debug_sync_get_action(THD *thd,
   st_debug_sync_control *ds_control = thd->debug_sync_control;
   st_debug_sync_action *action;
   DBUG_TRACE;
-  assert(thd);
-  assert(dsp_name);
-  assert(name_len);
-  assert(ds_control);
+  DBUG_ASSERT(thd);
+  DBUG_ASSERT(dsp_name);
+  DBUG_ASSERT(name_len);
+  DBUG_ASSERT(ds_control);
   DBUG_PRINT("debug_sync", ("sync_point: '%.*s'", (int)name_len, dsp_name));
   DBUG_PRINT("debug_sync", ("active: %u  allocated: %u", ds_control->ds_active,
                             ds_control->ds_allocated));
 
   /* There cannot be more active actions than allocated. */
-  assert(ds_control->ds_active <= ds_control->ds_allocated);
+  DBUG_ASSERT(ds_control->ds_active <= ds_control->ds_allocated);
   /* If there are active actions, the action array must be present. */
-  assert(!ds_control->ds_active || ds_control->ds_action);
+  DBUG_ASSERT(!ds_control->ds_active || ds_control->ds_action);
 
   /* Try to reuse existing action if there is one for this sync point. */
   if (ds_control->ds_active &&
       (action = debug_sync_find(ds_control->ds_action, ds_control->ds_active,
                                 dsp_name, name_len))) {
     /* Reuse an already active sync point action. */
-    assert((uint)(action - ds_control->ds_action) < ds_control->ds_active);
+    DBUG_ASSERT((uint)(action - ds_control->ds_action) < ds_control->ds_active);
     DBUG_PRINT("debug_sync", ("reuse action idx: %ld",
                               (long)(action - ds_control->ds_action)));
   } else {
@@ -1023,7 +1023,7 @@ static st_debug_sync_action *debug_sync_get_action(THD *thd,
     if (ds_control->ds_active > ds_control->ds_allocated) {
       uint new_alloc = ds_control->ds_active + 3;
       void *new_action =
-          my_malloc(key_debug_THD_debug_sync_action,
+          my_malloc(key_debug_sync_action,
                     new_alloc * sizeof(st_debug_sync_action), MYF(MY_WME));
       if (!new_action) {
         /* Error is reported by my_malloc(). */
@@ -1057,8 +1057,8 @@ static st_debug_sync_action *debug_sync_get_action(THD *thd,
     }
     action->need_sort = true;
   }
-  assert(action >= ds_control->ds_action);
-  assert(action < ds_control->ds_action + ds_control->ds_active);
+  DBUG_ASSERT(action >= ds_control->ds_action);
+  DBUG_ASSERT(action < ds_control->ds_action + ds_control->ds_active);
   DBUG_PRINT("debug_sync", ("action: %p  array: %p  count: %u", action,
                             ds_control->ds_action, ds_control->ds_active));
 
@@ -1103,9 +1103,9 @@ static bool debug_sync_set_action(THD *thd, st_debug_sync_action *action) {
   st_debug_sync_control *ds_control = thd->debug_sync_control;
   bool is_dsp_now = false;
   DBUG_TRACE;
-  assert(thd);
-  assert(action);
-  assert(ds_control);
+  DBUG_ASSERT(thd);
+  DBUG_ASSERT(action);
+  DBUG_ASSERT(ds_control);
 
   action->activation_count = max(action->hit_limit, action->execute);
   if (!action->activation_count) {
@@ -1264,9 +1264,9 @@ static inline const char *get_token_end_ptr(const char *ptr) {
 
 static char *debug_sync_token(char **token_p, size_t *token_length_p,
                               char *ptr) {
-  assert(token_p);
-  assert(token_length_p);
-  assert(ptr);
+  DBUG_ASSERT(token_p);
+  DBUG_ASSERT(token_length_p);
+  DBUG_ASSERT(ptr);
 
   /* Skip leading space */
   ptr = const_cast<char *>(skip_whitespace(ptr));
@@ -1328,8 +1328,8 @@ static char *debug_sync_number(ulong *number_p, char *actstrptr) {
   char *ept;
   char *token;
   size_t token_length;
-  assert(number_p);
-  assert(actstrptr);
+  DBUG_ASSERT(number_p);
+  DBUG_ASSERT(actstrptr);
 
   /* Get token from string. */
   if (!(ptr = debug_sync_token(&token, &token_length, actstrptr))) goto end;
@@ -1380,8 +1380,8 @@ static bool debug_sync_eval_action(THD *thd, char *action_str) {
   char *token;
   size_t token_length = 0;
   DBUG_TRACE;
-  assert(thd);
-  assert(action_str);
+  DBUG_ASSERT(thd);
+  DBUG_ASSERT(action_str);
 
   /*
     Get debug sync point name. Or a special command.
@@ -1427,7 +1427,7 @@ static bool debug_sync_eval_action(THD *thd, char *action_str) {
     Check for pseudo actions first. Start with actions that work on
     an existing action.
   */
-  assert(action);
+  DBUG_ASSERT(action);
 
   /*
     Try TEST.
@@ -1752,21 +1752,21 @@ static inline void clear_signal_event(const std::string *signal_name) {
 */
 
 static void debug_sync_execute(THD *thd, st_debug_sync_action *action) {
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   const char *dsp_name = action->sync_point.c_ptr();
   const char *sig_emit = action->signal.c_ptr();
   const char *sig_wait = action->wait_for.c_ptr();
 #endif
   DBUG_TRACE;
-  assert(thd);
-  assert(action);
+  DBUG_ASSERT(thd);
+  DBUG_ASSERT(action);
   DBUG_PRINT("debug_sync",
              ("sync_point: '%s'  activation_count: %lu  hit_limit: %lu  "
               "execute: %lu  timeout: %lu  signal: '%s'  wait_for: '%s'",
               dsp_name, action->activation_count, action->hit_limit,
               action->execute, action->timeout, sig_emit, sig_wait));
 
-  assert(action->activation_count);
+  DBUG_ASSERT(action->activation_count);
   action->activation_count--;
 
   if (action->execute) {
@@ -1783,7 +1783,7 @@ static void debug_sync_execute(THD *thd, st_debug_sync_action *action) {
       st_debug_sync_control *ds_control = thd->debug_sync_control;
       strxnmov(ds_control->ds_proc_info, sizeof(ds_control->ds_proc_info) - 1,
                "debug sync point: ", action->sync_point.c_ptr(), NullS);
-      old_proc_info = thd->proc_info();
+      old_proc_info = thd->proc_info;
       debug_sync_thd_proc_info(thd, ds_control->ds_proc_info);
     }
 
@@ -1797,16 +1797,15 @@ static void debug_sync_execute(THD *thd, st_debug_sync_action *action) {
 
     if (action->signal.length()) {
       std::string signal = action->signal.ptr();
-      myu::Split(
-          signal.begin(), signal.end(), myu::IsComma,
-          [](std::string::const_iterator f, std::string::const_iterator l) {
-            auto tr = myu::FindTrimmedRange(f, l, myu::IsSpace);
-            if (tr.first != tr.second) {
-              std::string s{tr.first, tr.second};
-              add_signal_event(&s);
-            }
-          });
-
+      std::vector<std::string> signals;
+      boost::split(signals, signal, boost::is_any_of(","));
+      for (std::vector<std::string>::const_iterator it = signals.begin();
+           it != signals.end(); ++it) {
+        /* Copy the signal to the global set. */
+        std::string s = *it;
+        boost::trim(s);
+        if (!s.empty()) add_signal_event(&s);
+      }
       /* Wake threads waiting in a sync point. */
       mysql_cond_broadcast(&debug_sync_global.ds_cond);
       DBUG_PRINT("debug_sync_exec",
@@ -1925,10 +1924,10 @@ void debug_sync(THD *thd, const char *sync_point_name, size_t name_len) {
   st_debug_sync_control *ds_control = thd->debug_sync_control;
   st_debug_sync_action *action;
   DBUG_TRACE;
-  assert(thd);
-  assert(sync_point_name);
-  assert(name_len);
-  assert(ds_control);
+  DBUG_ASSERT(thd);
+  DBUG_ASSERT(sync_point_name);
+  DBUG_ASSERT(name_len);
+  DBUG_ASSERT(ds_control);
   DBUG_PRINT("debug_sync_point", ("hit: '%s'", sync_point_name));
 
   /* Statistics. */
@@ -1977,8 +1976,8 @@ bool debug_sync_set_action(THD *thd, const char *action_str, size_t len) {
   bool rc;
   char *value;
   DBUG_TRACE;
-  assert(thd);
-  assert(action_str);
+  DBUG_ASSERT(thd);
+  DBUG_ASSERT(action_str);
 
   value = strmake_root(thd->mem_root, action_str, len);
   rc = debug_sync_eval_action(thd, value);
@@ -1986,23 +1985,14 @@ bool debug_sync_set_action(THD *thd, const char *action_str, size_t len) {
 }
 
 void conditional_sync_point(std::string name) {
-  if (current_thd == nullptr) return;
-  const std::string debug_symbol = "syncpoint_" + name;
-  DBUG_EXECUTE_IF(debug_symbol.c_str(), {
-    DBUG_PRINT("info", ("reached sync point '%s' (enabled by debug symbol "
-                        "'%s'); signalling 'reached_%s' and waiting for "
-                        "'continue_%s'",
-                        name.c_str(), debug_symbol.c_str(), name.c_str(),
-                        name.c_str()));
+  DBUG_EXECUTE_IF(("syncpoint_" + name).c_str(), {
     std::string act =
         "now SIGNAL reached_" + name + " WAIT_FOR continue_" + name;
-    bool ret = debug_sync_set_action(current_thd, act.c_str(), act.length());
-    assert(!ret);
+    DBUG_ASSERT(!debug_sync_set_action(current_thd, act.c_str(), act.length()));
   });
 }
 
 void conditional_sync_point_for_timestamp(std::string name) {
-  if (current_thd == nullptr) return;
   conditional_sync_point(name + "_" +
                          std::to_string(current_thd->start_time.tv_sec));
 }
