@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2011, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2011, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,13 +21,16 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
-#include <cstdio>
-#include <cstdlib>
-#include <cstring>
+#include <limits.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+
+/* We only need the tokens here */
+#define YYSTYPE_IS_DECLARED
 
 #include "sql/lex.h"
 #include "sql/lex_symbol.h"
-#include "sql/sql_hints.yy.h"
 #include "sql/sql_yacc.h"
 #include "welcome_copyright_notice.h" /* ORACLE_WELCOME_COPYRIGHT_NOTICE */
 
@@ -51,13 +53,13 @@
 
   As of now (8.0.0), the mapping looks like this:
   - PART 1: [0 .. 255] tokens of single-character lexemes
-  - PART 2: [256 .. ...] tokens < YYUNDEF from sql_yacc.yy
-  - PART 3: [... .. 999] reserved for sql_yacc.yy new tokens < YYUNDEF
+  - PART 2: [256 .. ...] tokens < NOT_A_TOKEN_SYM from sql_yacc.yy
+  - PART 3: [... .. 999] reserved for sql_yacc.yy new tokens < NOT_A_TOKEN_SYM
   - PART 4: [1000 .. ...] tokens from sql_hints.yy
   - PART 5: [... .. 1099] reserved for sql_hints.yy new tokens
   - PART 6: [1100 .. ...] digest special fake tokens
   - PART 7: [... .. 1149] reserved for new digest special fake tokens
-  - PART 8: [1150 .. ...] tokens > YYUNDEF from sql_yacc.yy
+  - PART 8: [1150 .. ...] tokens > NOT_A_TOKEN_SYM from sql_yacc.yy
 
   Should gen_lex_token fail when tokens are exhausted
   (maybe you are reading this comment because of a fprintf(stderr) below),
@@ -142,7 +144,7 @@ struct gen_lex_token_string {
   - mode named tokens from bison (sql_yacc.yy).
   See also YYMAXUTOK.
 */
-const int MY_MAX_TOKEN = 1251;
+const int MY_MAX_TOKEN = 1200;
 
 gen_lex_token_string compiled_token_array[MY_MAX_TOKEN];
 
@@ -235,20 +237,10 @@ struct range {
   int max_seen;
 };
 
-/*
-  Newer Bison versions (e.g 3.7) will rename YYUNDEF to
-  MY_SQL_PARSER_UNDEF, older versions (e.g. 3.0) will not.
- */
-#if !defined(MY_SQL_PARSER_UNDEF)
-#define MY_SQL_PARSER_UNDEF YYUNDEF
-#endif
-
-static_assert(
-    MY_SQL_PARSER_UNDEF == 1150,
-    "MY_SQL_PARSER_UNDEF must be stable, because raw token numbers are used in "
-    "PFS digest calculations");
-range range_for_sql_yacc2{"sql/sql_yacc.yy (before MY_SQL_PARSER_UNDEF)",
-                          MY_SQL_PARSER_UNDEF, MY_MAX_TOKEN};
+static_assert(NOT_A_TOKEN_SYM == 1150,
+              "NOT_A_TOKEN_SYM should be equal to 1150");
+range range_for_sql_yacc2{"sql/sql_yacc.yy (before NOT_A_TOKEN_SYM)",
+                          NOT_A_TOKEN_SYM, MY_MAX_TOKEN};
 
 range range_for_digests{"digest specials", 1100, range_for_sql_yacc2.start - 1};
 
@@ -257,7 +249,7 @@ static_assert(MAX_EXECUTION_TIME_HINT == 1000,
 range range_for_sql_hints{"sql/sql_hints.yy", MAX_EXECUTION_TIME_HINT,
                           range_for_digests.start - 1};
 
-range range_for_sql_yacc1{"sql/sql_yacc.yy (after MY_SQL_PARSER_UNDEF)", 256,
+range range_for_sql_yacc1{"sql/sql_yacc.yy (after NOT_A_TOKEN_SYM)", 256,
                           range_for_sql_hints.start - 1};
 
 int tok_generic_value = 0;
@@ -327,7 +319,7 @@ static void compute_tokens() {
   */
   for (const SYMBOL &sym : symbols) {
     if ((sym.group & SG_MAIN_PARSER) != 0) {
-      if (sym.tok < MY_SQL_PARSER_UNDEF)
+      if (sym.tok < NOT_A_TOKEN_SYM)
         range_for_sql_yacc1.set_token(sym.tok, sym.name, __LINE__);
       else
         range_for_sql_yacc2.set_token(sym.tok, sym.name, __LINE__);
