@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2000, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2000, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -34,7 +34,6 @@
 #include <stdint.h>
 #include <stdlib.h>
 #include <sys/types.h>
-
 #include <algorithm>
 #include <cctype>
 #include <iterator>
@@ -56,13 +55,12 @@
 #include "mysql_time.h"
 #include "mysqld_error.h"
 #include "sql/item.h"
-#include "sql/item_cmpfunc.h"    // Item_func_any_value
-#include "sql/item_func.h"       // Item_func_udf_str
-#include "sql/item_geofunc.h"    // Item_func_st_area
-#include "sql/item_gtid_func.h"  // Item_wait_for_executed_gtid_set Item_master_gtid_set_wait Item_func_gtid_subset
-#include "sql/item_inetfunc.h"   // Item_func_inet_ntoa
-#include "sql/item_json_func.h"  // Item_func_json
-#include "sql/item_pfs_func.h"   // Item_pfs_func_thread_id
+#include "sql/item_cmpfunc.h"      // Item_func_any_value
+#include "sql/item_func.h"         // Item_func_udf_str
+#include "sql/item_geofunc.h"      // Item_func_st_area
+#include "sql/item_inetfunc.h"     // Item_func_inet_ntoa
+#include "sql/item_json_func.h"    // Item_func_json
+#include "sql/item_pfs_func.h"     // Item_pfs_func_thread_id
 #include "sql/item_regexp_func.h"  // Item_func_regexp_xxx
 #include "sql/item_strfunc.h"      // Item_func_aes_encrypt
 #include "sql/item_sum.h"          // Item_sum_udf_str
@@ -81,15 +79,13 @@
 #include "sql/sql_udf.h"
 #include "sql/system_variables.h"
 #include "sql_string.h"
-#include "tztime.h"  // convert_time_zone_displacement
+#include "tztime.h"  // adjust_time_zone
 
 /**
   @addtogroup GROUP_PARSER
   @{
 */
 
-namespace {
-
 /**
   @defgroup Instantiators Instantiator functions
 
@@ -126,9 +122,11 @@ namespace {
 
   @see Function_factory::create_func()
 */
-constexpr auto MAX_ARGLIST_SIZE =
+static const auto MAX_ARGLIST_SIZE =
     std::numeric_limits<decltype(PT_item_list().elements())>::max();
 
+namespace {
+
 /**
   Instantiates a function class with the list of arguments.
 
@@ -345,7 +343,7 @@ class Instantiator<Function_class, 0, 1> {
       case 1:
         return new (thd->mem_root) Function_class(POS(), (*args)[0]);
       default:
-        assert(false);
+        DBUG_ASSERT(false);
         return nullptr;
     }
   }
@@ -371,7 +369,7 @@ class Instantiator<Function_class, 1, 2> {
         return new (thd->mem_root)
             Function_class(POS(), (*args)[0], (*args)[1]);
       default:
-        assert(false);
+        DBUG_ASSERT(false);
         return nullptr;
     }
   }
@@ -400,7 +398,7 @@ class Instantiator<Function_class, 1, 3> {
         return new (thd->mem_root)
             Function_class(POS(), (*args)[0], (*args)[1], (*args)[2]);
       default:
-        assert(false);
+        DBUG_ASSERT(false);
         return nullptr;
     }
   }
@@ -429,7 +427,7 @@ class Instantiator_with_thd<Function_class, 1, 3> {
         return new (thd->mem_root)
             Function_class(thd, POS(), (*args)[0], (*args)[1], (*args)[2]);
       default:
-        assert(false);
+        DBUG_ASSERT(false);
         return nullptr;
     }
   }
@@ -455,7 +453,7 @@ class Instantiator_with_thd<Function_class, 1, 2> {
         return new (thd->mem_root)
             Function_class(thd, POS(), (*args)[0], (*args)[1]);
       default:
-        assert(false);
+        DBUG_ASSERT(false);
         return nullptr;
     }
   }
@@ -482,7 +480,7 @@ class Instantiator<Function_class, 2, 3> {
         return new (thd->mem_root)
             Function_class(POS(), (*args)[0], (*args)[1], (*args)[2]);
       default:
-        assert(false);
+        DBUG_ASSERT(false);
         return nullptr;
     }
   }
@@ -512,44 +510,7 @@ class Instantiator<Function_class, 2, 4> {
         return new (thd->mem_root) Function_class(POS(), (*args)[0], (*args)[1],
                                                   (*args)[2], (*args)[3]);
       default:
-        assert(false);
-        return nullptr;
-    }
-  }
-};
-
-/**
-  Instantiates a function class with between two and six arguments.
-
-  @tparam Function_class The class that implements the function. Does not need
-  to inherit Item_func.
-*/
-template <typename Function_class>
-class Instantiator<Function_class, 2, 6> {
- public:
-  static const uint Min_argcount = 2;
-  static const uint Max_argcount = 6;
-
-  Item *instantiate(THD *thd, PT_item_list *args) {
-    switch (args->elements()) {
-      case 2:
-        return new (thd->mem_root)
-            Function_class(POS(), (*args)[0], (*args)[1]);
-      case 3:
-        return new (thd->mem_root)
-            Function_class(POS(), (*args)[0], (*args)[1], (*args)[2]);
-      case 4:
-        return new (thd->mem_root) Function_class(POS(), (*args)[0], (*args)[1],
-                                                  (*args)[2], (*args)[3]);
-      case 5:
-        return new (thd->mem_root) Function_class(
-            POS(), (*args)[0], (*args)[1], (*args)[2], (*args)[3], (*args)[4]);
-      case 6:
-        return new (thd->mem_root)
-            Function_class(POS(), (*args)[0], (*args)[1], (*args)[2],
-                           (*args)[3], (*args)[4], (*args)[5]);
-      default:
-        assert(false);
+        DBUG_ASSERT(false);
         return nullptr;
     }
   }
@@ -579,7 +540,7 @@ class Instantiator<Function_class, 3, 5> {
         return new (thd->mem_root) Function_class(
             POS(), (*args)[0], (*args)[1], (*args)[2], (*args)[3], (*args)[4]);
       default:
-        assert(false);
+        DBUG_ASSERT(false);
         return nullptr;
     }
   }
@@ -626,7 +587,7 @@ class Geometry_instantiator {
         return new (thd->mem_root)
             Geometry_class(POS(), (*args)[0], (*args)[1], (*args)[2], Functype);
       default:
-        assert(false);
+        DBUG_ASSERT(false);
         return nullptr;
     }
   }
@@ -679,6 +640,8 @@ using Pointfromwkb_instantiator = G_i<I_wkb, wkb_ft::POINTFROMWKB>;
 using Polyfromwkb_instantiator = G_i<I_wkb, wkb_ft::POLYFROMWKB>;
 using Polygonfromwkb_instantiator = G_i<I_wkb, wkb_ft::POLYGONFROMWKB>;
 
+}  // namespace
+
 class Bin_instantiator {
  public:
   static const uint Min_argcount = 1;
@@ -786,7 +749,7 @@ class From_unixtime_instantiator {
             Item_func_date_format(POS(), ut, (*args)[1], false);
       }
       default:
-        assert(false);
+        DBUG_ASSERT(false);
         return nullptr;
     }
   }
@@ -808,7 +771,7 @@ class Round_instantiator {
         return new (thd->mem_root)
             Item_func_round(POS(), (*args)[0], (*args)[1], false);
       default:
-        assert(false);
+        DBUG_ASSERT(false);
         return nullptr;
     }
   }
@@ -830,7 +793,7 @@ class Locate_instantiator {
         return new (thd->mem_root)
             Item_func_locate(POS(), (*args)[1], (*args)[0], (*args)[2]);
       default:
-        assert(false);
+        DBUG_ASSERT(false);
         return nullptr;
     }
   }
@@ -850,7 +813,7 @@ class Srid_instantiator {
         return new (thd->mem_root)
             Item_func_st_srid_mutator(POS(), (*args)[0], (*args)[1]);
       default:
-        assert(false);
+        DBUG_ASSERT(false);
         return nullptr;
     }
   }
@@ -871,7 +834,7 @@ class Latitude_instantiator {
             Item_func_st_latitude_mutator(POS(), (*args)[0], (*args)[1]);
       default:
         /* purecov: begin deadcode */
-        assert(false);
+        DBUG_ASSERT(false);
         return nullptr;
         /* purecov: end */
     }
@@ -893,7 +856,7 @@ class Longitude_instantiator {
             Item_func_st_longitude_mutator(POS(), (*args)[0], (*args)[1]);
       default:
         /* purecov: begin deadcode */
-        assert(false);
+        DBUG_ASSERT(false);
         return nullptr;
         /* purecov: end */
     }
@@ -913,7 +876,7 @@ class X_instantiator {
         return new (thd->mem_root)
             Item_func_st_x_mutator(POS(), (*args)[0], (*args)[1]);
       default:
-        assert(false);
+        DBUG_ASSERT(false);
         return nullptr;
     }
   }
@@ -932,7 +895,7 @@ class Y_instantiator {
         return new (thd->mem_root)
             Item_func_st_y_mutator(POS(), (*args)[0], (*args)[1]);
       default:
-        assert(false);
+        DBUG_ASSERT(false);
         return nullptr;
     }
   }
@@ -953,7 +916,7 @@ class Yearweek_instantiator {
         return new (thd->mem_root)
             Item_func_yearweek(POS(), (*args)[0], (*args)[1]);
       default:
-        assert(false);
+        DBUG_ASSERT(false);
         return nullptr;
     }
   }
@@ -970,27 +933,6 @@ class Make_set_instantiator {
   }
 };
 
-/// Instantiates a call to JSON_LENGTH, which may take either one or
-/// two arguments. The two-argument variant is rewritten from
-/// JSON_LENGTH(doc, path) to JSON_LENGTH(JSON_EXTRACT(doc, path)).
-class Json_length_instantiator {
- public:
-  static constexpr int Min_argcount = 1;
-  static constexpr int Max_argcount = 2;
-
-  Item *instantiate(THD *thd, PT_item_list *args) {
-    if (args->elements() == 1) {
-      return new (thd->mem_root) Item_func_json_length(POS(), (*args)[0]);
-    } else {
-      assert(args->elements() == 2);
-      auto arg = new (thd->mem_root)
-          Item_func_json_extract(thd, POS(), (*args)[0], (*args)[1]);
-      if (arg == nullptr) return nullptr;
-      return new (thd->mem_root) Item_func_json_length(POS(), arg);
-    }
-  }
-};
-
 /// @} (end of group Instantiators)
 
 uint arglist_length(const PT_item_list *args) {
@@ -1009,6 +951,8 @@ bool check_argcount_bounds(THD *, LEX_STRING function_name,
   return false;
 }
 
+namespace {
+
 /**
   Factory for creating function objects. Performs validation check that the
   number of arguments is correct, then calls upon the instantiator function to
@@ -1041,7 +985,7 @@ class Function_factory : public Create_func {
   }
 
  private:
-  Function_factory() = default;
+  Function_factory() {}
   Instantiator_fn m_instantiator;
 };
 
@@ -1068,7 +1012,7 @@ class Odd_argcount_function_factory : public Create_func {
   }
 
  private:
-  Odd_argcount_function_factory() = default;
+  Odd_argcount_function_factory() {}
   Instantiator_fn m_instantiator;
 };
 
@@ -1095,7 +1039,7 @@ class Even_argcount_function_factory : public Create_func {
   }
 
  private:
-  Even_argcount_function_factory() = default;
+  Even_argcount_function_factory() {}
   Instantiator_fn m_instantiator;
 };
 
@@ -1130,7 +1074,7 @@ class Internal_function_factory : public Create_func {
   }
 
  private:
-  Internal_function_factory() = default;
+  Internal_function_factory() {}
   Instantiator_fn m_instantiator;
 };
 
@@ -1145,16 +1089,16 @@ Internal_function_factory<Instantiator_fn>
 */
 class Create_sp_func : public Create_qfunc {
  public:
-  Item *create(THD *thd, LEX_STRING db, LEX_STRING name, bool use_explicit_name,
-               PT_item_list *item_list) override;
+  virtual Item *create(THD *thd, LEX_STRING db, LEX_STRING name,
+                       bool use_explicit_name, PT_item_list *item_list);
 
   static Create_sp_func s_singleton;
 
  protected:
   /** Constructor. */
-  Create_sp_func() = default;
+  Create_sp_func() {}
   /** Destructor. */
-  ~Create_sp_func() override = default;
+  virtual ~Create_sp_func() {}
 };
 
 Item *Create_qfunc::create_func(THD *thd, LEX_STRING name,
@@ -1167,7 +1111,7 @@ Create_udf_func Create_udf_func::s_singleton;
 Item *Create_udf_func::create_func(THD *thd, LEX_STRING name,
                                    PT_item_list *item_list) {
   udf_func *udf = find_udf(name.str, name.length);
-  assert(udf);
+  DBUG_ASSERT(udf);
   return create(thd, udf, item_list);
 }
 
@@ -1175,10 +1119,11 @@ Item *Create_udf_func::create(THD *thd, udf_func *udf,
                               PT_item_list *item_list) {
   DBUG_TRACE;
 
-  assert((udf->type == UDFTYPE_FUNCTION) || (udf->type == UDFTYPE_AGGREGATE));
+  DBUG_ASSERT((udf->type == UDFTYPE_FUNCTION) ||
+              (udf->type == UDFTYPE_AGGREGATE));
 
   Item *func = nullptr;
-  POS pos{};
+  POS pos;
 
   switch (udf->returns) {
     case STRING_RESULT:
@@ -1230,7 +1175,7 @@ Item *Create_sp_func::create(THD *thd, LEX_STRING db, LEX_STRING name,
 
 /**
   Shorthand macro to reference the singleton instance when there is a
-  specialized instantiator.
+  specialized intantiatior.
 
   @param INSTANTIATOR The instantiator class.
 */
@@ -1372,8 +1317,8 @@ static const std::pair<const char *, Create_func *> func_array[] = {
     {"ABS", SQL_FN(Item_func_abs, 1)},
     {"ACOS", SQL_FN(Item_func_acos, 1)},
     {"ADDTIME", SQL_FN(Item_func_add_time, 2)},
-    {"AES_DECRYPT", SQL_FN_V(Item_func_aes_decrypt, 2, 6)},
-    {"AES_ENCRYPT", SQL_FN_V(Item_func_aes_encrypt, 2, 6)},
+    {"AES_DECRYPT", SQL_FN_V(Item_func_aes_decrypt, 2, 3)},
+    {"AES_ENCRYPT", SQL_FN_V(Item_func_aes_encrypt, 2, 3)},
     {"ANY_VALUE", SQL_FN(Item_func_any_value, 1)},
     {"ASIN", SQL_FN(Item_func_asin, 1)},
     {"ATAN", SQL_FN_V(Item_func_atan, 1, 2)},
@@ -1439,7 +1384,7 @@ static const std::pair<const char *, Create_func *> func_array[] = {
     {"JSON_CONTAINS", SQL_FN_V_LIST_THD(Item_func_json_contains, 2, 3)},
     {"JSON_CONTAINS_PATH",
      SQL_FN_V_THD(Item_func_json_contains_path, 3, MAX_ARGLIST_SIZE)},
-    {"JSON_LENGTH", SQL_FACTORY(Json_length_instantiator)},
+    {"JSON_LENGTH", SQL_FN_V_THD(Item_func_json_length, 1, 2)},
     {"JSON_DEPTH", SQL_FN(Item_func_json_depth, 1)},
     {"JSON_PRETTY", SQL_FN(Item_func_json_pretty, 1)},
     {"JSON_TYPE", SQL_FN(Item_func_json_type, 1)},
@@ -1480,7 +1425,7 @@ static const std::pair<const char *, Create_func *> func_array[] = {
     {"LCASE", SQL_FN(Item_func_lower, 1)},
     {"LEAST", SQL_FN_V_LIST(Item_func_min, 2, MAX_ARGLIST_SIZE)},
     {"LENGTH", SQL_FN(Item_func_length, 1)},
-#ifndef NDEBUG
+#ifndef DBUG_OFF
     {"LIKE_RANGE_MIN", SQL_FN(Item_func_like_range_min, 2)},
     {"LIKE_RANGE_MAX", SQL_FN(Item_func_like_range_max, 2)},
 #endif
@@ -1543,7 +1488,6 @@ static const std::pair<const char *, Create_func *> func_array[] = {
     {"SIN", SQL_FN(Item_func_sin, 1)},
     {"SLEEP", SQL_FN(Item_func_sleep, 1)},
     {"SOUNDEX", SQL_FN(Item_func_soundex, 1)},
-    {"SOURCE_POS_WAIT", SQL_FN_V(Item_source_pos_wait, 2, 4)},
     {"SPACE", SQL_FN(Item_func_space, 1)},
     {"STATEMENT_DIGEST", SQL_FN(Item_func_statement_digest, 1)},
     {"STATEMENT_DIGEST_TEXT", SQL_FN(Item_func_statement_digest_text, 1)},
@@ -1560,7 +1504,7 @@ static const std::pair<const char *, Create_func *> func_array[] = {
     {"ST_ASTEXT", SQL_FN_V(Item_func_as_wkt, 1, 2)},
     {"ST_ASWKB", SQL_FN_V(Item_func_as_wkb, 1, 2)},
     {"ST_ASWKT", SQL_FN_V(Item_func_as_wkt, 1, 2)},
-    {"ST_BUFFER", SQL_FN_V_LIST(Item_func_st_buffer, 2, 5)},
+    {"ST_BUFFER", SQL_FN_V_LIST(Item_func_buffer, 2, 5)},
     {"ST_BUFFER_STRATEGY", SQL_FN_V_LIST(Item_func_buffer_strategy, 1, 2)},
     {"ST_CENTROID", SQL_FN(Item_func_centroid, 1)},
     {"ST_CONTAINS", SQL_FN(Item_func_st_contains, 2)},
@@ -1575,7 +1519,6 @@ static const std::pair<const char *, Create_func *> func_array[] = {
     {"ST_ENVELOPE", SQL_FN(Item_func_envelope, 1)},
     {"ST_EQUALS", SQL_FN(Item_func_st_equals, 2)},
     {"ST_EXTERIORRING", SQL_FACTORY(Exteriorring_instantiator)},
-    {"ST_FRECHETDISTANCE", SQL_FN_V_LIST(Item_func_st_frechet_distance, 2, 3)},
     {"ST_GEOHASH", SQL_FN_V(Item_func_geohash, 2, 3)},
     {"ST_GEOMCOLLFROMTEXT", SQL_FACTORY(Geomcollfromtext_instantiator)},
     {"ST_GEOMCOLLFROMTXT", SQL_FACTORY(Geomcollfromtxt_instantiator)},
@@ -1591,8 +1534,6 @@ static const std::pair<const char *, Create_func *> func_array[] = {
     {"ST_GEOMFROMGEOJSON", SQL_FN_V(Item_func_geomfromgeojson, 1, 3)},
     {"ST_GEOMFROMTEXT", SQL_FACTORY(Geomfromtext_instantiator)},
     {"ST_GEOMFROMWKB", SQL_FACTORY(Geomfromwkb_instantiator)},
-    {"ST_HAUSDORFFDISTANCE",
-     SQL_FN_V_LIST(Item_func_st_hausdorff_distance, 2, 3)},
     {"ST_INTERIORRINGN", SQL_FACTORY(Sp_interiorringn_instantiator)},
     {"ST_INTERSECTS", SQL_FN(Item_func_st_intersects, 2)},
     {"ST_INTERSECTION", SQL_FN(Item_func_st_intersection, 2)},
@@ -1605,8 +1546,6 @@ static const std::pair<const char *, Create_func *> func_array[] = {
     {"ST_LENGTH", SQL_FN_V_LIST(Item_func_st_length, 1, 2)},
     {"ST_LINEFROMTEXT", SQL_FACTORY(Linefromtext_instantiator)},
     {"ST_LINEFROMWKB", SQL_FACTORY(Linefromwkb_instantiator)},
-    {"ST_LINEINTERPOLATEPOINT", SQL_FN(Item_func_lineinterpolatepoint, 2)},
-    {"ST_LINEINTERPOLATEPOINTS", SQL_FN(Item_func_lineinterpolatepoints, 2)},
     {"ST_LINESTRINGFROMTEXT", SQL_FACTORY(Linestringfromtext_instantiator)},
     {"ST_LINESTRINGFROMWKB", SQL_FACTORY(Linestringfromwkb_instantiator)},
     {"ST_LONGFROMGEOHASH", SQL_FN(Item_func_longfromgeohash, 1)},
@@ -1631,7 +1570,6 @@ static const std::pair<const char *, Create_func *> func_array[] = {
     {"ST_NUMINTERIORRINGS", SQL_FN(Item_func_numinteriorring, 1)},
     {"ST_NUMPOINTS", SQL_FN(Item_func_numpoints, 1)},
     {"ST_OVERLAPS", SQL_FN(Item_func_st_overlaps, 2)},
-    {"ST_POINTATDISTANCE", SQL_FN(Item_func_st_pointatdistance, 2)},
     {"ST_POINTFROMGEOHASH", SQL_FN(Item_func_pointfromgeohash, 2)},
     {"ST_POINTFROMTEXT", SQL_FACTORY(Pointfromtext_instantiator)},
     {"ST_POINTFROMWKB", SQL_FACTORY(Pointfromwkb_instantiator)},
@@ -1683,8 +1621,6 @@ static const std::pair<const char *, Create_func *> func_array[] = {
      SQL_FN_LIST_INTERNAL(Item_func_get_dd_index_sub_part_length, 5)},
     {"GET_DD_CREATE_OPTIONS",
      SQL_FN_INTERNAL(Item_func_get_dd_create_options, 3)},
-    {"GET_DD_SCHEMA_OPTIONS",
-     SQL_FN_INTERNAL(Item_func_get_dd_schema_options, 1)},
     {"GET_DD_TABLESPACE_PRIVATE_DATA",
      SQL_FN_INTERNAL(Item_func_get_dd_tablespace_private_data, 2)},
     {"GET_DD_INDEX_PRIVATE_DATA",
@@ -1699,13 +1635,12 @@ static const std::pair<const char *, Create_func *> func_array[] = {
     {"CAN_ACCESS_ROUTINE",
      SQL_FN_LIST_INTERNAL(Item_func_can_access_routine, 5)},
     {"CAN_ACCESS_EVENT", SQL_FN_INTERNAL(Item_func_can_access_event, 1)},
-    {"CAN_ACCESS_USER", SQL_FN_INTERNAL(Item_func_can_access_user, 2)},
     {"ICU_VERSION", SQL_FN(Item_func_icu_version, 0)},
     {"CAN_ACCESS_RESOURCE_GROUP",
      SQL_FN_INTERNAL(Item_func_can_access_resource_group, 1)},
     {"CONVERT_CPU_ID_MASK", SQL_FN_INTERNAL(Item_func_convert_cpu_id_mask, 1)},
     {"IS_VISIBLE_DD_OBJECT",
-     SQL_FN_INTERNAL_V(Item_func_is_visible_dd_object, 1, 3)},
+     SQL_FN_INTERNAL_V(Item_func_is_visible_dd_object, 1, 2)},
     {"INTERNAL_TABLE_ROWS",
      SQL_FN_LIST_INTERNAL_V(Item_func_internal_table_rows, 8, 9)},
     {"INTERNAL_AVG_ROW_LENGTH",
@@ -1773,7 +1708,7 @@ static const std::pair<const char *, Create_func *> func_array[] = {
     {"CONVERT_INTERVAL_TO_USER_INTERVAL",
      SQL_FN_INTERNAL(Item_func_convert_interval_to_user_interval, 2)},
     {"INTERNAL_GET_DD_COLUMN_EXTRA",
-     SQL_FN_LIST_INTERNAL(Item_func_internal_get_dd_column_extra, 8)},
+     SQL_FN_LIST_INTERNAL(Item_func_internal_get_dd_column_extra, 6)},
     {"INTERNAL_GET_USERNAME",
      SQL_FN_LIST_INTERNAL_V(Item_func_internal_get_username, 0, 1)},
     {"INTERNAL_GET_HOSTNAME",
@@ -1828,13 +1763,12 @@ Item *create_func_cast(THD *thd, const POS &pos, Item *a,
   type.charset = cs;
   type.length = nullptr;
   type.dec = nullptr;
-  return create_func_cast(thd, pos, a, type, false);
+  return create_func_cast(thd, pos, a, &type);
 }
 
 /**
   Validates a cast target type and extracts the specified length and precision
-  of the target type. Helper function for creating Items representing CAST
-  expressions, and Items performing CAST-like tasks, such as JSON_VALUE.
+  of the target type.
 
   @param thd        thread handler
   @param pos        the location of the expression
@@ -1885,13 +1819,6 @@ static bool validate_cast_type_and_extract_length(
     case ITEM_CAST_UNSIGNED_INT:
     case ITEM_CAST_DATE:
       return false;
-    case ITEM_CAST_YEAR:
-      if (as_array) {
-        my_error(ER_NOT_SUPPORTED_YET, MYF(0),
-                 "CAST-ing data to array of YEAR");
-        return true;
-      }
-      return false;
     case ITEM_CAST_TIME:
     case ITEM_CAST_DATETIME: {
       uint dec = c_dec ? strtoul(c_dec, nullptr, 10) : 0;
@@ -1968,7 +1895,7 @@ static bool validate_cast_type_and_extract_length(
         len = my_strtoll10(c_len, nullptr, &error);
         if ((error != 0) || (len > MAX_FIELD_BLOBLENGTH)) {
           my_error(ER_TOO_BIG_DISPLAYWIDTH, MYF(0), "cast as char",
-                   static_cast<unsigned long>(MAX_FIELD_BLOBLENGTH));
+                   MAX_FIELD_BLOBLENGTH);
           return true;
         }
       }
@@ -2016,77 +1943,26 @@ static bool validate_cast_type_and_extract_length(
         return true;
       }
       return false;
-    case ITEM_CAST_POINT:
-      if (as_array) {
-        my_error(ER_NOT_SUPPORTED_YET, MYF(0),
-                 "CAST-ing data to array of POINT");
-        return true;
-      }
-      return false;
-    case ITEM_CAST_LINESTRING:
-      if (as_array) {
-        my_error(ER_NOT_SUPPORTED_YET, MYF(0),
-                 "CAST-ing data to array of LINESTRING");
-        return true;
-      }
-      return false;
-    case ITEM_CAST_POLYGON:
-      if (as_array) {
-        my_error(ER_NOT_SUPPORTED_YET, MYF(0),
-                 "CAST-ing data to array of POLYGON");
-        return true;
-      }
-      return false;
-    case ITEM_CAST_MULTIPOINT:
-      if (as_array) {
-        my_error(ER_NOT_SUPPORTED_YET, MYF(0),
-                 "CAST-ing data to array of MULTIPOINT");
-        return true;
-      }
-      return false;
-    case ITEM_CAST_MULTILINESTRING:
-      if (as_array) {
-        my_error(ER_NOT_SUPPORTED_YET, MYF(0),
-                 "CAST-ing data to array of MULTILINESTRING>");
-        return true;
-      }
-      return false;
-    case ITEM_CAST_MULTIPOLYGON:
-      if (as_array) {
-        my_error(ER_NOT_SUPPORTED_YET, MYF(0),
-                 "CAST-ing data to array of MULTIPOLYGON");
-        return true;
-      }
-      return false;
-    case ITEM_CAST_GEOMETRYCOLLECTION:
-      if (as_array) {
-        my_error(ER_NOT_SUPPORTED_YET, MYF(0),
-                 "CAST-ing data to array of GEOMETRYCOLLECTION");
-        return true;
-      }
-      return false;
   }
   /* purecov: begin deadcode */
-  assert(false);
+  DBUG_ASSERT(false);
   return true;
   /* purecov: end */
 }
 
-/// This function does not store the reference to `type`.
 Item *create_func_cast(THD *thd, const POS &pos, Item *arg,
-                       const Cast_type &type, bool as_array) {
+                       const Cast_type *type, bool as_array) {
   int64_t length = 0;
   unsigned precision = 0;
-  if (validate_cast_type_and_extract_length(thd, pos, arg, type, as_array,
+  if (validate_cast_type_and_extract_length(thd, pos, arg, *type, as_array,
                                             &length, &precision))
     return nullptr;
 
-  if (as_array) {
+  if (as_array)
     return new (thd->mem_root) Item_func_array_cast(
-        pos, arg, type.target, length, precision, type.charset);
-  }
+        pos, arg, type->target, length, precision, type->charset);
 
-  switch (type.target) {
+  switch (type->target) {
     case ITEM_CAST_SIGNED_INT:
       return new (thd->mem_root) Item_typecast_signed(pos, arg);
     case ITEM_CAST_UNSIGNED_INT:
@@ -2097,13 +1973,11 @@ Item *create_func_cast(THD *thd, const POS &pos, Item *arg,
       return new (thd->mem_root) Item_typecast_time(pos, arg, precision);
     case ITEM_CAST_DATETIME:
       return new (thd->mem_root) Item_typecast_datetime(pos, arg, precision);
-    case ITEM_CAST_YEAR:
-      return new (thd->mem_root) Item_typecast_year(pos, arg);
     case ITEM_CAST_DECIMAL:
       return new (thd->mem_root)
           Item_typecast_decimal(pos, arg, length, precision);
     case ITEM_CAST_CHAR: {
-      const CHARSET_INFO *cs = type.charset;
+      const CHARSET_INFO *cs = type->charset;
       if (cs == nullptr) cs = thd->variables.collation_connection;
       return new (thd->mem_root) Item_typecast_char(pos, arg, length, cs);
     }
@@ -2115,51 +1989,14 @@ Item *create_func_cast(THD *thd, const POS &pos, Item *arg,
     case ITEM_CAST_DOUBLE:
       return new (thd->mem_root)
           Item_typecast_real(pos, arg, /*as_double=*/true);
-    case ITEM_CAST_POINT:
-      return new (thd->mem_root) Item_typecast_point(pos, arg);
-    case ITEM_CAST_LINESTRING:
-      return new (thd->mem_root) Item_typecast_linestring(pos, arg);
-    case ITEM_CAST_POLYGON:
-      return new (thd->mem_root) Item_typecast_polygon(pos, arg);
-    case ITEM_CAST_MULTIPOINT:
-      return new (thd->mem_root) Item_typecast_multipoint(pos, arg);
-    case ITEM_CAST_MULTILINESTRING:
-      return new (thd->mem_root) Item_typecast_multilinestring(pos, arg);
-    case ITEM_CAST_MULTIPOLYGON:
-      return new (thd->mem_root) Item_typecast_multipolygon(pos, arg);
-    case ITEM_CAST_GEOMETRYCOLLECTION:
-      return new (thd->mem_root) Item_typecast_geometrycollection(pos, arg);
   }
 
   /* purecov: begin deadcode */
-  assert(false);
+  DBUG_ASSERT(false);
   return nullptr;
   /* purecov: end */
 }
 
-Item *create_func_json_value(THD *thd, const POS &pos, Item *arg, Item *path,
-                             const Cast_type &cast_type,
-                             Json_on_response_type on_empty_type,
-                             Item *on_empty_default,
-                             Json_on_response_type on_error_type,
-                             Item *on_error_default) {
-  int64_t length = 0;
-  unsigned precision = 0;
-  if (validate_cast_type_and_extract_length(thd, pos, arg, cast_type, false,
-                                            &length, &precision))
-    return nullptr;
-
-  // Create dummy items for the default values, if they haven't been specified.
-  if (on_empty_default == nullptr)
-    on_empty_default = new (thd->mem_root) Item_null;
-  if (on_error_default == nullptr)
-    on_error_default = new (thd->mem_root) Item_null;
-
-  return new (thd->mem_root) Item_func_json_value(
-      pos, arg, path, cast_type, length, precision, on_empty_type,
-      on_empty_default, on_error_type, on_error_default);
-}
-
 /**
   Builder for datetime literals:
     TIME'00:00:00', DATE'2001-01-01', TIMESTAMP'2001-01-01 00:00:00'.
@@ -2190,10 +2027,8 @@ Item *create_temporal_literal(THD *thd, const char *str, size_t length,
       if (!propagate_datetime_overflow(
               thd, &status.warnings,
               str_to_datetime(cs, str, length, &ltime, flags, &status)) &&
-          ltime.time_type == MYSQL_TIMESTAMP_DATE && !status.warnings) {
-        check_deprecated_datetime_format(thd, cs, status);
+          ltime.time_type == MYSQL_TIMESTAMP_DATE && !status.warnings)
         item = new (thd->mem_root) Item_date_literal(&ltime);
-      }
       break;
     case MYSQL_TYPE_DATETIME:
       if (!propagate_datetime_overflow(
@@ -2202,23 +2037,19 @@ Item *create_temporal_literal(THD *thd, const char *str, size_t length,
           (ltime.time_type == MYSQL_TIMESTAMP_DATETIME ||
            ltime.time_type == MYSQL_TIMESTAMP_DATETIME_TZ) &&
           !status.warnings) {
-        check_deprecated_datetime_format(thd, cs, status);
-        if (convert_time_zone_displacement(thd->time_zone(), &ltime))
-          return nullptr;
+        adjust_time_zone_displacement(thd->time_zone(), &ltime);
         item = new (thd->mem_root) Item_datetime_literal(
             &ltime, status.fractional_digits, thd->time_zone());
       }
       break;
     case MYSQL_TYPE_TIME:
       if (!str_to_time(cs, str, length, &ltime, 0, &status) &&
-          ltime.time_type == MYSQL_TIMESTAMP_TIME && !status.warnings) {
-        check_deprecated_datetime_format(thd, cs, status);
+          ltime.time_type == MYSQL_TIMESTAMP_TIME && !status.warnings)
         item = new (thd->mem_root)
             Item_time_literal(&ltime, status.fractional_digits);
-      }
       break;
     default:
-      assert(0);
+      DBUG_ASSERT(0);
   }
 
   if (item) return item;
