@@ -1,7 +1,7 @@
 #ifndef ITEM_GEOFUNC_INCLUDED
 #define ITEM_GEOFUNC_INCLUDED
 
-/* Copyright (c) 2000, 2022, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -23,25 +23,21 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
-#include <assert.h>
+#include <stddef.h>
 #include <sys/types.h>
-
-#include <cstddef>
 #include <vector>
 
-#include "field_types.h"  // MYSQL_TYPE_BLOB
-
+#include "my_dbug.h"
 #include "my_inttypes.h"
 #include "my_sys.h"
-#include "mysql/components/services/bits/psi_bits.h"
+#include "mysql/psi/psi_base.h"
 #include "mysql_com.h"
 #include "mysqld_error.h"
 #include "prealloced_array.h"
 #include "sql/enum_query_type.h"
 #include "sql/field.h"
-#include "sql/gis/buffer_strategies.h"  // gis::buffer_strategies
+#include "sql/gis/geometries.h"
 #include "sql/gis/srid.h"
-#include "sql/parse_location.h"  // POS
 /* This file defines all spatial functions */
 #include "sql/inplace_vector.h"
 #include "sql/item.h"
@@ -49,7 +45,8 @@
 #include "sql/item_func.h"
 #include "sql/item_json_func.h"  // Item_json_func
 #include "sql/item_strfunc.h"    // Item_str_func
-#include "sql/spatial.h"         // gis_wkb_raw_free
+#include "sql/parse_tree_node_base.h"
+#include "sql/spatial.h"  // gis_wkb_raw_free
 #include "sql_string.h"
 
 class Json_array;
@@ -58,7 +55,6 @@ class Json_object;
 class Json_wrapper;
 class PT_item_list;
 class THD;
-struct Parse_context;
 struct TABLE;
 
 enum class enum_json_type;
@@ -69,7 +65,6 @@ class Spatial_reference_system;
 
 namespace gis {
 class Geometry;
-class Point;
 }  // namespace gis
 
 /**
@@ -171,8 +166,12 @@ class BG_geometry_collection {
   size_t num_isolated() const { return m_num_isolated; }
 
   Gis_geometry_collection *as_geometry_collection(String *geodata) const;
+  template <typename Coordsys>
+  void merge_components(bool *pnull_value);
 
  private:
+  template <typename Coordsys>
+  bool merge_one_run(Item_func_st_union *ifsu, bool *pnull_value);
   bool store_geometry(const Geometry *geo, bool break_multi_geom);
   Geometry *store(const Geometry *geo);
 };
@@ -250,12 +249,6 @@ class Item_func_geometry_from_text : public Item_geometry_func {
   bool itemize(Parse_context *pc, Item **res) override;
   const char *func_name() const override;
   String *val_str(String *) override;
-  bool resolve_type(THD *thd) override {
-    if (param_type_is_default(thd, 0, 1)) return true;
-    if (param_type_is_default(thd, 1, 2, MYSQL_TYPE_LONGLONG)) return true;
-    if (param_type_is_default(thd, 2, 3)) return true;
-    return Item_geometry_func::resolve_type(thd);
-  }
 };
 
 class Item_func_geometry_from_wkb : public Item_geometry_func {
@@ -297,12 +290,6 @@ class Item_func_geometry_from_wkb : public Item_geometry_func {
     @retval false The geometry type is not allowed
   */
   bool is_allowed_wkb_type(Geometry::wkbType type) const;
-  bool resolve_type(THD *thd) override {
-    if (param_type_is_default(thd, 0, 1)) return true;
-    if (param_type_is_default(thd, 1, 2, MYSQL_TYPE_LONGLONG)) return true;
-    if (param_type_is_default(thd, 2, 3)) return true;
-    return Item_geometry_func::resolve_type(thd);
-  }
 
  public:
   Item_func_geometry_from_wkb(const POS &pos, Item *a, Functype functype)
@@ -337,10 +324,9 @@ class Item_func_as_wkb : public Item_geometry_func {
   const char *func_name() const override { return "st_aswkb"; }
   String *val_str(String *) override;
   bool resolve_type(THD *thd) override {
-    if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_GEOMETRY)) return true;
-    if (param_type_is_default(thd, 1, 2)) return true;
     if (Item_geometry_func::resolve_type(thd)) return true;
-    set_data_type_blob(Field::MAX_LONG_BLOB_WIDTH);
+    set_data_type(MYSQL_TYPE_BLOB);
+    // @todo - what about max_length???
     return false;
   }
 };
@@ -351,11 +337,10 @@ class Item_func_geometry_type : public Item_str_ascii_func {
       : Item_str_ascii_func(pos, a) {}
   String *val_str_ascii(String *) override;
   const char *func_name() const override { return "st_geometrytype"; }
-  bool resolve_type(THD *thd) override {
-    if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_GEOMETRY)) return true;
+  bool resolve_type(THD *) override {
     // "MultiLinestring" is the longest
     set_data_type_string(15, default_charset());
-    set_nullable(true);
+    maybe_null = true;
     return false;
   }
 };
@@ -705,10 +690,6 @@ class Item_func_centroid : public Item_geometry_func {
   const char *func_name() const override { return "st_centroid"; }
   String *val_str(String *) override;
   Field::geometry_type get_geometry_type() const override;
-  bool resolve_type(THD *thd) override {
-    if (param_type_is_default(thd, 0, -1, MYSQL_TYPE_GEOMETRY)) return true;
-    return Item_geometry_func::resolve_type(thd);
-  }
 };
 
 class Item_func_convex_hull : public Item_geometry_func {
@@ -722,10 +703,6 @@ class Item_func_convex_hull : public Item_geometry_func {
   const char *func_name() const override { return "st_convexhull"; }
   String *val_str(String *) override;
   Field::geometry_type get_geometry_type() const override;
-  bool resolve_type(THD *thd) override {
-    if (param_type_is_default(thd, 0, -1, MYSQL_TYPE_GEOMETRY)) return true;
-    return Item_geometry_func::resolve_type(thd);
-  }
 };
 
 class Item_func_envelope : public Item_geometry_func {
@@ -734,10 +711,6 @@ class Item_func_envelope : public Item_geometry_func {
   const char *func_name() const override { return "st_envelope"; }
   String *val_str(String *) override;
   Field::geometry_type get_geometry_type() const override;
-  bool resolve_type(THD *thd) override {
-    if (param_type_is_default(thd, 0, -1, MYSQL_TYPE_GEOMETRY)) return true;
-    return Item_geometry_func::resolve_type(thd);
-  }
 };
 
 class Item_func_make_envelope : public Item_geometry_func {
@@ -747,10 +720,6 @@ class Item_func_make_envelope : public Item_geometry_func {
   const char *func_name() const override { return "st_makeenvelope"; }
   String *val_str(String *) override;
   Field::geometry_type get_geometry_type() const override;
-  bool resolve_type(THD *thd) override {
-    if (param_type_is_default(thd, 0, -1, MYSQL_TYPE_GEOMETRY)) return true;
-    return Item_geometry_func::resolve_type(thd);
-  }
 };
 
 class Item_func_validate : public Item_geometry_func {
@@ -759,10 +728,6 @@ class Item_func_validate : public Item_geometry_func {
  public:
   Item_func_validate(const POS &pos, Item *a) : Item_geometry_func(pos, a) {}
   const char *func_name() const override { return "st_validate"; }
-  bool resolve_type(THD *thd) override {
-    if (param_type_is_default(thd, 0, -1, MYSQL_TYPE_GEOMETRY)) return true;
-    return Item_geometry_func::resolve_type(thd);
-  }
   String *val_str(String *) override;
 };
 
@@ -772,11 +737,6 @@ class Item_func_st_simplify : public Item_geometry_func {
  public:
   Item_func_st_simplify(const POS &pos, Item *a, Item *b)
       : Item_geometry_func(pos, a, b) {}
-  bool resolve_type(THD *thd) override {
-    if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_GEOMETRY)) return true;
-    if (param_type_is_default(thd, 1, 2, MYSQL_TYPE_DOUBLE)) return true;
-    return Item_geometry_func::resolve_type(thd);
-  }
   String *val_str(String *) override;
 
   const char *func_name() const override { return "st_simplify"; }
@@ -789,10 +749,6 @@ class Item_func_point : public Item_geometry_func {
   const char *func_name() const override { return "point"; }
   String *val_str(String *) override;
   Field::geometry_type get_geometry_type() const override;
-  bool resolve_type(THD *thd) override {
-    if (param_type_is_default(thd, 0, -1, MYSQL_TYPE_DOUBLE)) return true;
-    return Item_geometry_func::resolve_type(thd);
-  }
 };
 
 /**
@@ -827,7 +783,6 @@ class Item_func_pointfromgeohash : public Item_geometry_func {
   const char *func_name() const override { return "st_pointfromgeohash"; }
   String *val_str(String *) override;
   bool fix_fields(THD *thd, Item **ref) override;
-  bool resolve_type(THD *thd) override;
   Field::geometry_type get_geometry_type() const override {
     return Field::GEOM_POINT;
   }
@@ -835,10 +790,6 @@ class Item_func_pointfromgeohash : public Item_geometry_func {
 
 class Item_func_spatial_decomp : public Item_geometry_func {
   enum Functype decomp_func;
-  bool resolve_type(THD *thd) override {
-    if (param_type_is_default(thd, 0, -1, MYSQL_TYPE_GEOMETRY)) return true;
-    return Item_geometry_func::resolve_type(thd);
-  }
 
  public:
   Item_func_spatial_decomp(const POS &pos, Item *a, Item_func::Functype ft)
@@ -854,7 +805,7 @@ class Item_func_spatial_decomp : public Item_geometry_func {
       case SP_EXTERIORRING:
         return "st_exteriorring";
       default:
-        assert(0);  // Should never happened
+        DBUG_ASSERT(0);  // Should never happened
         return "spatial_decomp_unknown";
     }
   }
@@ -863,11 +814,6 @@ class Item_func_spatial_decomp : public Item_geometry_func {
 
 class Item_func_spatial_decomp_n : public Item_geometry_func {
   enum Functype decomp_func_n;
-  bool resolve_type(THD *thd) override {
-    if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_GEOMETRY)) return true;
-    if (param_type_is_default(thd, 1, 2, MYSQL_TYPE_LONGLONG)) return true;
-    return Item_geometry_func::resolve_type(thd);
-  }
 
  public:
   Item_func_spatial_decomp_n(const POS &pos, Item *a, Item *b,
@@ -884,7 +830,7 @@ class Item_func_spatial_decomp_n : public Item_geometry_func {
       case SP_INTERIORRINGN:
         return "st_interiorringn";
       default:
-        assert(0);  // Should never happened
+        DBUG_ASSERT(0);  // Should never happened
         return "spatial_decomp_n_unknown";
     }
   }
@@ -906,7 +852,6 @@ class Item_func_spatial_collection : public Item_geometry_func {
   }
   String *val_str(String *) override;
   bool resolve_type(THD *thd) override {
-    if (param_type_is_default(thd, 0, -1, MYSQL_TYPE_GEOMETRY)) return true;
     if (Item_geometry_func::resolve_type(thd)) return true;
     for (unsigned int i = 0; i < arg_count; ++i) {
       if (args[i]->fixed && args[i]->data_type() != MYSQL_TYPE_GEOMETRY) {
@@ -959,25 +904,79 @@ class Item_func_spatial_mbr_rel : public Item_bool_func2 {
     Item_func::print(thd, str, query_type);
   }
   bool resolve_type(THD *) override {
-    set_nullable(true);
+    maybe_null = true;
     return false;
   }
   bool is_null() override {
     val_int();
     return null_value;
   }
+  bool cast_incompatible_args(uchar *) override { return false; }
+};
+
+class Item_func_spatial_rel : public Item_bool_func2 {
+ public:
+  Item_func_spatial_rel(const POS &pos, Item *a, Item *b)
+      : Item_bool_func2(pos, a, b) {}
+  longlong val_int() override;
+
+  // Use is not restricted to subclasses. Also used by setops,
+  // BG_geometry_collection::merge_one_run() and
+  // linear_areal_intersect_infinite().
+  static int bg_geo_relation_check(Geometry *g1, Geometry *g2,
+                                   Functype relchk_type, bool *);
+
+ private:
+  template <typename Geom_types>
+  friend class BG_wrap;
+
+  template <typename Geotypes>
+  static int within_check(Geometry *g1, Geometry *g2, bool *pnull_value);
+  template <typename Geotypes>
+  static int equals_check(Geometry *g1, Geometry *g2, bool *pnull_value);
+  template <typename Geotypes>
+  static int disjoint_check(Geometry *g1, Geometry *g2, bool *pnull_value);
+  template <typename Geotypes>
+  static int intersects_check(Geometry *g1, Geometry *g2, bool *pnull_value);
+  template <typename Geotypes>
+  static int overlaps_check(Geometry *g1, Geometry *g2, bool *pnull_value);
+  template <typename Geotypes>
+  static int touches_check(Geometry *g1, Geometry *g2, bool *pnull_value);
+  template <typename Geotypes>
+  static int crosses_check(Geometry *g1, Geometry *g2, bool *pnull_value);
+
+  template <typename Coordsys>
+  int multipoint_within_geometry_collection(
+      Gis_multi_point *mpts,
+      const typename BG_geometry_collection::Geometry_list *gv2,
+      const void *prtree);
+
+  template <typename Coordsys>
+  int geocol_relation_check(Geometry *g1, Geometry *g2);
+  int geocol_relcheck_intersect_disjoint(
+      const BG_geometry_collection::Geometry_list *gv1,
+      const BG_geometry_collection::Geometry_list *gv2);
+  template <typename Coordsys>
+  int geocol_relcheck_within(
+      const typename BG_geometry_collection::Geometry_list *gv1,
+      const typename BG_geometry_collection::Geometry_list *gv2,
+      Functype spatial_rel);
+  template <typename Coordsys>
+  int geocol_equals_check(
+      const typename BG_geometry_collection::Geometry_list *gv1,
+      const typename BG_geometry_collection::Geometry_list *gv2);
+  bool cast_incompatible_args(uchar *) override { return false; }
 };
 
 class Item_func_spatial_relation : public Item_bool_func2 {
  public:
   Item_func_spatial_relation(const POS &pos, Item *a, Item *b)
       : Item_bool_func2(pos, a, b) {}
-  bool resolve_type(THD *thd) override {
-    if (param_type_is_default(thd, 0, -1, MYSQL_TYPE_GEOMETRY)) return true;
+  bool resolve_type(THD *) override {
     // Spatial relation functions may return NULL if either parameter is NULL or
     // an empty geometry. Since we can't check for empty geometries at resolve
     // time, this item is always nullable.
-    set_nullable(true);
+    maybe_null = true;
     return false;
   }
   void print(const THD *thd, String *str,
@@ -1010,6 +1009,7 @@ class Item_func_spatial_relation : public Item_bool_func2 {
   virtual bool eval(const dd::Spatial_reference_system *srs,
                     const gis::Geometry *g1, const gis::Geometry *g2,
                     bool *result, bool *null) = 0;
+  bool cast_incompatible_args(uchar *) override { return false; }
 };
 
 class Item_func_st_contains final : public Item_func_spatial_relation {
@@ -1203,46 +1203,102 @@ class Item_func_st_within final : public Item_func_spatial_relation {
   Spatial operations
 */
 class Item_func_spatial_operation : public Item_geometry_func {
+ public:
+  String *val_str(String *) override;
+
  protected:
-  Item_func_spatial_operation(const POS &pos, Item *a, Item *b)
-      : Item_geometry_func(pos, a, b) {}
+  enum op_type {
+    op_union = 0x10000000,
+    op_intersection = 0x20000000,
+    op_symdifference = 0x30000000,
+    op_difference = 0x40000000,
+  };
+
+  Item_func_spatial_operation(const POS &pos, Item *a, Item *b, op_type sp_op)
+      : Item_geometry_func(pos, a, b), m_spatial_op(sp_op) {}
 
  private:
-  bool resolve_type(THD *thd) override {
-    if (param_type_is_default(thd, 0, -1, MYSQL_TYPE_GEOMETRY)) return true;
-    return Item_geometry_func::resolve_type(thd);
-  }
+  // It will call the protected member functions in this class,
+  // no data member accessed directly.
+  template <typename Geotypes>
+  friend class BG_setop_wrapper;
+
+  // Calls bg_geo_set_op.
+  friend class BG_geometry_collection;
+
+  template <typename Coordsys>
+  Geometry *bg_geo_set_op(Geometry *g1, Geometry *g2, String *result);
+
+  template <typename Coordsys>
+  Geometry *combine_sub_results(Geometry *g1, Geometry *g2,
+                                gis::srid_t default_srid, String *result);
+  Geometry *simplify_multilinestring(Gis_multi_line_string *mls,
+                                     String *result);
+
+  template <typename Coordsys>
+  Geometry *geometry_collection_set_operation(Geometry *g1, Geometry *g2,
+                                              String *result);
+
+  Geometry *empty_result(String *str, gis::srid_t srid);
+
+  bool assign_result(Geometry *geo, String *result);
+
+  template <typename Geotypes>
+  Geometry *intersection_operation(Geometry *g1, Geometry *g2, String *result);
+  template <typename Geotypes>
+  Geometry *union_operation(Geometry *g1, Geometry *g2, String *result);
+  template <typename Geotypes>
+  Geometry *difference_operation(Geometry *g1, Geometry *g2, String *result);
+  template <typename Geotypes>
+  Geometry *symdifference_operation(Geometry *g1, Geometry *g2, String *result);
+  template <typename Coordsys>
+  Geometry *geocol_symdifference(const BG_geometry_collection &bggc1,
+                                 const BG_geometry_collection &bggc2,
+                                 String *result);
+  template <typename Coordsys>
+  Geometry *geocol_difference(const BG_geometry_collection &bggc1,
+                              const BG_geometry_collection &bggc2,
+                              String *result);
+  template <typename Coordsys>
+  Geometry *geocol_intersection(const BG_geometry_collection &bggc1,
+                                const BG_geometry_collection &bggc2,
+                                String *result);
+  template <typename Coordsys>
+  Geometry *geocol_union(const BG_geometry_collection &bggc1,
+                         const BG_geometry_collection &bggc2, String *result);
+
+  op_type m_spatial_op;
+  String m_result_buffer;
+  String m_tmp_value1;
+  String m_tmp_value2;
+  BG_result_buf_mgr m_bg_resbuf_mgr;
 };
 
 class Item_func_st_difference final : public Item_func_spatial_operation {
  public:
   Item_func_st_difference(const POS &pos, Item *a, Item *b)
-      : Item_func_spatial_operation(pos, a, b) {}
-  String *val_str(String *) override;
+      : Item_func_spatial_operation(pos, a, b, op_difference) {}
   const char *func_name() const override { return "st_difference"; }
 };
 
 class Item_func_st_intersection final : public Item_func_spatial_operation {
  public:
   Item_func_st_intersection(const POS &pos, Item *a, Item *b)
-      : Item_func_spatial_operation(pos, a, b) {}
-  String *val_str(String *) override;
+      : Item_func_spatial_operation(pos, a, b, op_intersection) {}
   const char *func_name() const override { return "st_intersection"; }
 };
 
 class Item_func_st_symdifference final : public Item_func_spatial_operation {
  public:
   Item_func_st_symdifference(const POS &pos, Item *a, Item *b)
-      : Item_func_spatial_operation(pos, a, b) {}
-  String *val_str(String *) override;
+      : Item_func_spatial_operation(pos, a, b, op_symdifference) {}
   const char *func_name() const override { return "st_symdifference"; }
 };
 
 class Item_func_st_union final : public Item_func_spatial_operation {
  public:
   Item_func_st_union(const POS &pos, Item *a, Item *b)
-      : Item_func_spatial_operation(pos, a, b) {}
-  String *val_str(String *) override;
+      : Item_func_spatial_operation(pos, a, b, op_union) {}
   const char *func_name() const override { return "st_union"; }
 };
 
@@ -1305,16 +1361,11 @@ class Item_func_buffer : public Item_geometry_func {
   String tmp_value;  // Stores current buffer result.
   String m_tmp_geombuf;
   void set_strategies();
-  bool resolve_type(THD *thd) override {
-    if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_GEOMETRY)) return true;
-    if (param_type_is_default(thd, 1, 2, MYSQL_TYPE_DOUBLE)) return true;
-    if (param_type_is_default(thd, 2, -1)) return true;
-    return Item_geometry_func::resolve_type(thd);
-  }
 
  public:
   Item_func_buffer(const POS &pos, PT_item_list *ilist);
   const char *func_name() const override { return "st_buffer"; }
+  String *val_str(String *) override;
 };
 
 class Item_func_buffer_strategy : public Item_str_func {
@@ -1335,9 +1386,8 @@ class Item_func_isempty : public Item_bool_func {
   longlong val_int() override;
   optimize_type select_optimize(const THD *) override { return OPTIMIZE_NONE; }
   const char *func_name() const override { return "st_isempty"; }
-  bool resolve_type(THD *thd) override {
-    if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_GEOMETRY)) return true;
-    set_nullable(true);
+  bool resolve_type(THD *) override {
+    maybe_null = true;
     return false;
   }
 };
@@ -1347,10 +1397,6 @@ class Item_func_st_issimple : public Item_bool_func {
   Item_func_st_issimple(const POS &pos, Item *a) : Item_bool_func(pos, a) {}
   longlong val_int() override;
   const char *func_name() const override { return "st_issimple"; }
-  bool resolve_type(THD *thd) override {
-    if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_GEOMETRY)) return true;
-    return Item_bool_func::resolve_type(thd);
-  }
 };
 
 class Item_func_isclosed : public Item_bool_func {
@@ -1359,9 +1405,8 @@ class Item_func_isclosed : public Item_bool_func {
   longlong val_int() override;
   optimize_type select_optimize(const THD *) override { return OPTIMIZE_NONE; }
   const char *func_name() const override { return "st_isclosed"; }
-  bool resolve_type(THD *thd) override {
-    if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_GEOMETRY)) return true;
-    set_nullable(true);
+  bool resolve_type(THD *) override {
+    maybe_null = true;
     return false;
   }
 };
@@ -1372,10 +1417,6 @@ class Item_func_isvalid : public Item_bool_func {
   longlong val_int() override;
   optimize_type select_optimize(const THD *) override { return OPTIMIZE_NONE; }
   const char *func_name() const override { return "st_isvalid"; }
-  bool resolve_type(THD *thd) override {
-    if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_GEOMETRY)) return true;
-    return Item_bool_func::resolve_type(thd);
-  }
 };
 
 class Item_func_dimension : public Item_int_func {
@@ -1385,10 +1426,9 @@ class Item_func_dimension : public Item_int_func {
   Item_func_dimension(const POS &pos, Item *a) : Item_int_func(pos, a) {}
   longlong val_int() override;
   const char *func_name() const override { return "st_dimension"; }
-  bool resolve_type(THD *thd) override {
-    if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_GEOMETRY)) return true;
+  bool resolve_type(THD *) override {
     max_length = 10;
-    set_nullable(true);
+    maybe_null = true;
     return false;
   }
 };
@@ -1413,11 +1453,6 @@ class Item_func_coordinate_mutator : public Item_geometry_func {
   /// @return The coordinate number to access.
   virtual int coordinate_number(
       const dd::Spatial_reference_system *srs) const = 0;
-  bool resolve_type(THD *thd) override {
-    if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_GEOMETRY)) return true;
-    if (param_type_is_default(thd, 1, 2, MYSQL_TYPE_DOUBLE)) return true;
-    return Item_geometry_func::resolve_type(thd);
-  }
 
  private:
   /// Whether this item will accept only geographic geometries/SRSs.
@@ -1443,10 +1478,6 @@ class Item_func_coordinate_observer : public Item_real_func {
   /// @return The coordinate number to access.
   virtual int coordinate_number(
       const dd::Spatial_reference_system *srs) const = 0;
-  bool resolve_type(THD *thd) override {
-    if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_GEOMETRY)) return true;
-    return Item_real_func::resolve_type(thd);
-  }
 
  private:
   /// Whether this item will accept only geographic geometries/SRSs.
@@ -1562,15 +1593,10 @@ class Item_func_st_y_observer final : public Item_func_coordinate_observer {
 };
 
 class Item_func_swap_xy : public Item_geometry_func {
-  bool resolve_type(THD *thd) override {
-    if (param_type_is_default(thd, 0, -1, MYSQL_TYPE_GEOMETRY)) return true;
-    return Item_geometry_func::resolve_type(thd);
-  }
-
  public:
   Item_func_swap_xy(const POS &pos, Item *a) : Item_geometry_func(pos, a) {}
-  const char *func_name() const override { return "st_swapxy"; }
-  String *val_str(String *) override;
+  const char *func_name() const { return "st_swapxy"; }
+  String *val_str(String *);
 };
 
 class Item_func_numgeometries : public Item_int_func {
@@ -1580,10 +1606,9 @@ class Item_func_numgeometries : public Item_int_func {
   Item_func_numgeometries(const POS &pos, Item *a) : Item_int_func(pos, a) {}
   longlong val_int() override;
   const char *func_name() const override { return "st_numgeometries"; }
-  bool resolve_type(THD *thd) override {
-    if (param_type_is_default(thd, 0, -1, MYSQL_TYPE_GEOMETRY)) return true;
+  bool resolve_type(THD *) override {
     max_length = 10;
-    set_nullable(true);
+    maybe_null = true;
     return false;
   }
 };
@@ -1595,10 +1620,9 @@ class Item_func_numinteriorring : public Item_int_func {
   Item_func_numinteriorring(const POS &pos, Item *a) : Item_int_func(pos, a) {}
   longlong val_int() override;
   const char *func_name() const override { return "st_numinteriorrings"; }
-  bool resolve_type(THD *thd) override {
-    if (param_type_is_default(thd, 0, -1, MYSQL_TYPE_GEOMETRY)) return true;
+  bool resolve_type(THD *) override {
     max_length = 10;
-    set_nullable(true);
+    maybe_null = true;
     return false;
   }
 };
@@ -1610,10 +1634,9 @@ class Item_func_numpoints : public Item_int_func {
   Item_func_numpoints(const POS &pos, Item *a) : Item_int_func(pos, a) {}
   longlong val_int() override;
   const char *func_name() const override { return "st_numpoints"; }
-  bool resolve_type(THD *thd) override {
-    if (param_type_is_default(thd, 0, -1, MYSQL_TYPE_GEOMETRY)) return true;
+  bool resolve_type(THD *) override {
     max_length = 10;
-    set_nullable(true);
+    maybe_null = true;
     return false;
   }
 };
@@ -1623,33 +1646,13 @@ class Item_func_st_area : public Item_real_func {
   Item_func_st_area(const POS &pos, Item *a) : Item_real_func(pos, a) {}
   double val_real() override;
   const char *func_name() const override { return "st_area"; }
-  bool resolve_type(THD *thd) override {
-    if (param_type_is_default(thd, 0, -1, MYSQL_TYPE_GEOMETRY)) return true;
+  bool resolve_type(THD *) override {
     // ST_Area returns NULL if the geometry is empty.
-    set_nullable(true);
+    maybe_null = true;
     return false;
   }
 };
 
-class Item_func_st_buffer : public Item_geometry_func {
- public:
-  /// Parses strategy stored in String object, and sets values in strats.
-  bool parse_strategy(String *arg, gis::BufferStrategies &strats);
-
-  Item_func_st_buffer(const POS &pos, PT_item_list *ilist)
-      : Item_geometry_func(pos, ilist) {}
-
-  String *val_str(String *) override;
-  const char *func_name() const override { return "st_buffer"; }
-  bool resolve_type(THD *thd) override {
-    if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_GEOMETRY)) return true;
-    if (param_type_is_default(thd, 1, 2, MYSQL_TYPE_DOUBLE)) return true;
-    if (param_type_is_default(thd, 2, -1))
-      return true;  // Does nothing with the strategy args
-    return Item_geometry_func::resolve_type(thd);
-  }
-};
-
 class Item_func_st_length : public Item_real_func {
   String value;
 
@@ -1659,10 +1662,8 @@ class Item_func_st_length : public Item_real_func {
   double val_real() override;
   const char *func_name() const override { return "st_length"; }
   bool resolve_type(THD *thd) override {
-    if (param_type_is_default(thd, 0, -1, MYSQL_TYPE_GEOMETRY)) return true;
-    if (param_type_is_default(thd, 0, -1, MYSQL_TYPE_GEOMETRY)) return true;
     if (Item_real_func::resolve_type(thd)) return true;
-    set_nullable(true);
+    maybe_null = true;
     return false;
   }
 };
@@ -1675,11 +1676,6 @@ class Item_func_st_srid_mutator : public Item_geometry_func {
       : Item_geometry_func(pos, a, b) {}
   String *val_str(String *) override;
   const char *func_name() const override { return "st_srid"; }
-  bool resolve_type(THD *thd) override {
-    if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_GEOMETRY)) return true;
-    if (param_type_is_default(thd, 1, 2, MYSQL_TYPE_LONGLONG)) return true;
-    return Item_geometry_func::resolve_type(thd);
-  }
 };
 
 /// This class implements the one-parameter ST_SRID function which
@@ -1690,7 +1686,6 @@ class Item_func_st_srid_observer : public Item_int_func {
   longlong val_int() override;
   const char *func_name() const override { return "st_srid"; }
   bool resolve_type(THD *thd) override {
-    if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_GEOMETRY)) return true;
     bool error = Item_int_func::resolve_type(thd);
     max_length = 10;
     return error;
@@ -1714,12 +1709,12 @@ class Item_func_distance : public Item_real_func {
       Either operand can be an empty geometry collection, and it's meaningless
       for a distance between them.
     */
-    set_nullable(true);
+    maybe_null = true;
   }
 
-  bool resolve_type(THD *thd) override {
-    if (param_type_is_default(thd, 0, -1, MYSQL_TYPE_GEOMETRY)) return true;
-    set_nullable(true);
+  virtual bool resolve_type(THD *thd) override {
+    if (Item_real_func::resolve_type(thd)) return true;
+    maybe_null = true;
     return false;
   }
 
@@ -1727,95 +1722,12 @@ class Item_func_distance : public Item_real_func {
   const char *func_name() const override { return "st_distance"; }
 };
 
-class Item_func_st_frechet_distance : public Item_real_func {
- public:
-  Item_func_st_frechet_distance(const POS &pos, PT_item_list *ilist)
-      : Item_real_func(pos, ilist) {}
-  double val_real() override;
-  const char *func_name() const override { return "st_frechetdistance"; }
-  bool resolve_type(THD *thd) override {
-    param_type_is_default(thd, 0, -1, MYSQL_TYPE_GEOMETRY);
-    if (Item_real_func::resolve_type(thd)) return true;
-    set_nullable(true);
-    return false;
-  }
-};
-
-class Item_func_st_hausdorff_distance : public Item_real_func {
- public:
-  Item_func_st_hausdorff_distance(const POS &pos, PT_item_list *ilist)
-      : Item_real_func(pos, ilist) {}
-  double val_real() override;
-  const char *func_name() const override { return "st_hausdorffdistance"; }
-  bool resolve_type(THD *thd) override {
-    param_type_is_default(thd, 0, -1, MYSQL_TYPE_GEOMETRY);
-    if (Item_real_func::resolve_type(thd)) return true;
-    set_nullable(true);
-    return false;
-  }
-};
-
 class Item_func_st_distance_sphere : public Item_real_func {
  public:
   Item_func_st_distance_sphere(const POS &pos, PT_item_list *ilist)
       : Item_real_func(pos, ilist) {}
   double val_real() override;
   const char *func_name() const override { return "st_distance_sphere"; }
-  bool resolve_type(THD *thd) override {
-    if (param_type_is_default(thd, 0, 2, MYSQL_TYPE_GEOMETRY)) return true;
-    if (param_type_is_default(thd, 2, 3, MYSQL_TYPE_DOUBLE)) return true;
-    return Item_real_func::resolve_type(thd);
-  }
-};
-
-// The abstract superclass for interpolating point(s) along a line.
-class Item_func_lineinterpolate : public Item_geometry_func {
- public:
-  Item_func_lineinterpolate(const POS &pos, Item *a, Item *b)
-      : Item_geometry_func(pos, a, b) {}
-  String *val_str(String *str) override;
-
- protected:
-  virtual bool isFractionalDistance() const = 0;
-  virtual bool returnMultiplePoints() const = 0;
-  bool resolve_type(THD *thd) override {
-    if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_GEOMETRY)) return true;
-    if (param_type_is_default(thd, 1, 2, MYSQL_TYPE_DOUBLE)) return true;
-    return Item_geometry_func::resolve_type(thd);
-  }
-};
-
-class Item_func_lineinterpolatepoint final : public Item_func_lineinterpolate {
- public:
-  Item_func_lineinterpolatepoint(const POS &pos, Item *a, Item *b)
-      : Item_func_lineinterpolate(pos, a, b) {}
-
- protected:
-  const char *func_name() const override { return "st_lineinterpolatepoint"; }
-  bool isFractionalDistance() const override { return true; }
-  bool returnMultiplePoints() const override { return false; }
-};
-
-class Item_func_lineinterpolatepoints final : public Item_func_lineinterpolate {
- public:
-  Item_func_lineinterpolatepoints(const POS &pos, Item *a, Item *b)
-      : Item_func_lineinterpolate(pos, a, b) {}
-
- protected:
-  const char *func_name() const override { return "st_lineinterpolatepoints"; }
-  bool isFractionalDistance() const override { return true; }
-  bool returnMultiplePoints() const override { return true; }
-};
-
-class Item_func_st_pointatdistance final : public Item_func_lineinterpolate {
- public:
-  Item_func_st_pointatdistance(const POS &pos, Item *a, Item *b)
-      : Item_func_lineinterpolate(pos, a, b) {}
-
- protected:
-  const char *func_name() const override { return "st_pointatdistance"; }
-  bool isFractionalDistance() const override { return false; }
-  bool returnMultiplePoints() const override { return false; }
 };
 
 /// This class implements ST_Transform function that transforms a geometry from
@@ -1830,136 +1742,4 @@ class Item_func_st_transform final : public Item_geometry_func {
   const char *func_name() const override { return "st_transform"; }
 };
 
-// This is an abstract class that is inherited by geometry cast items.
-class Item_typecast_geometry : public Item_geometry_func {
- public:
-  Item_typecast_geometry(const POS &pos, Item *a)
-      : Item_geometry_func(pos, a) {}
-  String *val_str(String *str) override;
-  void print(const THD *thd, String *str,
-             enum_query_type query_type) const override = 0;
-  const char *func_name() const override = 0;
-  enum Functype functype() const override { return TYPECAST_FUNC; }
-  Field::geometry_type get_geometry_type() const override = 0;
-  bool resolve_type(THD *thd) override {
-    param_type_is_default(thd, 0, 1, MYSQL_TYPE_GEOMETRY);
-    return Item_geometry_func::resolve_type(thd);
-  }
-
-  /// Casts certain geometry types to certain target geometry types.
-  ///
-  /// @param[in] srs The srs of the geometry being cast.
-  /// @param[in] source_geometry The geometry being cast.
-  /// @param[out] target_geometry The result geometry of the cast.
-  ///
-  /// @retval false Success.
-  /// @retval true An error has occurred. The error has been reported with
-  /// my_error().
-
-  virtual bool cast(const dd::Spatial_reference_system *srs,
-                    std::unique_ptr<gis::Geometry> *source_geometry,
-                    std::unique_ptr<gis::Geometry> *target_geometry) const = 0;
-};
-
-// This class implements CAST from certain geometries to POINT type.
-class Item_typecast_point : public Item_typecast_geometry {
- public:
-  Item_typecast_point(const POS &pos, Item *a)
-      : Item_typecast_geometry(pos, a) {}
-  void print(const THD *thd, String *str,
-             enum_query_type query_type) const override;
-  const char *func_name() const override { return "cast_as_point"; }
-  Field::geometry_type get_geometry_type() const override;
-  bool cast(const dd::Spatial_reference_system *,
-            std::unique_ptr<gis::Geometry> *source_geometry,
-            std::unique_ptr<gis::Geometry> *target_geometry) const override;
-};
-
-// This class implements CAST from certain geometries to LINESTRING type.
-class Item_typecast_linestring : public Item_typecast_geometry {
- public:
-  Item_typecast_linestring(const POS &pos, Item *a)
-      : Item_typecast_geometry(pos, a) {}
-  void print(const THD *thd, String *str,
-             enum_query_type query_type) const override;
-  const char *func_name() const override { return "cast_as_linestring"; }
-  Field::geometry_type get_geometry_type() const override;
-  bool cast(const dd::Spatial_reference_system *,
-            std::unique_ptr<gis::Geometry> *source_geometry,
-            std::unique_ptr<gis::Geometry> *target_geometry) const override;
-};
-
-// This class implements CAST from certain geometries to POLYGON type.
-class Item_typecast_polygon : public Item_typecast_geometry {
- public:
-  Item_typecast_polygon(const POS &pos, Item *a)
-      : Item_typecast_geometry(pos, a) {}
-  void print(const THD *thd, String *str,
-             enum_query_type query_type) const override;
-  const char *func_name() const override { return "cast_as_polygon"; }
-  Field::geometry_type get_geometry_type() const override;
-  bool cast(const dd::Spatial_reference_system *srs,
-            std::unique_ptr<gis::Geometry> *source_geometry,
-            std::unique_ptr<gis::Geometry> *target_geometry) const override;
-};
-
-// This class implements CAST from certain geometries to MULTIPOINT type.
-class Item_typecast_multipoint : public Item_typecast_geometry {
- public:
-  Item_typecast_multipoint(const POS &pos, Item *a)
-      : Item_typecast_geometry(pos, a) {}
-  void print(const THD *thd, String *str,
-             enum_query_type query_type) const override;
-  const char *func_name() const override { return "cast_as_multipoint"; }
-  Field::geometry_type get_geometry_type() const override;
-  bool cast(const dd::Spatial_reference_system *,
-            std::unique_ptr<gis::Geometry> *source_geometry,
-            std::unique_ptr<gis::Geometry> *target_geometry) const override;
-};
-
-// This class implements CAST from certain geometries to MULTILINESTRING type.
-class Item_typecast_multilinestring : public Item_typecast_geometry {
- public:
-  Item_typecast_multilinestring(const POS &pos, Item *a)
-      : Item_typecast_geometry(pos, a) {}
-  void print(const THD *thd, String *str,
-             enum_query_type query_type) const override;
-  const char *func_name() const override { return "cast_as_multilinestring"; }
-  Field::geometry_type get_geometry_type() const override;
-  bool cast(const dd::Spatial_reference_system *,
-            std::unique_ptr<gis::Geometry> *source_geometry,
-            std::unique_ptr<gis::Geometry> *target_geometry) const override;
-};
-
-// This class implements CAST from certain geometries to MULTIPOLYGON type.
-class Item_typecast_multipolygon : public Item_typecast_geometry {
- public:
-  Item_typecast_multipolygon(const POS &pos, Item *a)
-      : Item_typecast_geometry(pos, a) {}
-  void print(const THD *thd, String *str,
-             enum_query_type query_type) const override;
-  const char *func_name() const override { return "cast_as_multipolygon"; }
-  Field::geometry_type get_geometry_type() const override;
-  bool cast(const dd::Spatial_reference_system *srs,
-            std::unique_ptr<gis::Geometry> *source_geometry,
-            std::unique_ptr<gis::Geometry> *target_geometry) const override;
-};
-
-// This class implements CAST from certain geometries to GEOMETRYCOLLECTION
-// type.
-class Item_typecast_geometrycollection : public Item_typecast_geometry {
- public:
-  Item_typecast_geometrycollection(const POS &pos, Item *a)
-      : Item_typecast_geometry(pos, a) {}
-  void print(const THD *thd, String *str,
-             enum_query_type query_type) const override;
-  const char *func_name() const override {
-    return "cast_as_geometrycollection";
-  }
-  Field::geometry_type get_geometry_type() const override;
-  bool cast(const dd::Spatial_reference_system *,
-            std::unique_ptr<gis::Geometry> *source_geometry,
-            std::unique_ptr<gis::Geometry> *target_geometry) const override;
-};
-
 #endif /*ITEM_GEOFUNC_INCLUDED*/
