@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -24,16 +24,14 @@
 #ifndef ITEM_STRFUNC_INCLUDED
 #define ITEM_STRFUNC_INCLUDED
 
-#include <assert.h>
+#include <stddef.h>
 #include <sys/types.h>
-
-#include <cstdint>  // uint32_t
+#include <algorithm>
 
 #include "lex_string.h"
-#include "libbinlogevents/include/uuid.h"  // Uuid
+#include "libbinlogevents/include/control_events.h"
 #include "m_ctype.h"
-
-#include "my_hostname.h"  // HOSTNAME_LENGTH
+#include "my_dbug.h"
 #include "my_inttypes.h"
 #include "my_table_map.h"
 #include "my_time.h"
@@ -43,19 +41,17 @@
 #include "sql/enum_query_type.h"
 #include "sql/field.h"
 #include "sql/item.h"
-#include "sql/item_cmpfunc.h"    // Item_bool_func
-#include "sql/item_func.h"       // Item_func
-#include "sql/parse_location.h"  // POS
+#include "sql/item_cmpfunc.h"  // Item_bool_func
+#include "sql/item_func.h"     // Item_func
+#include "sql/parse_tree_node_base.h"
 #include "sql/sql_const.h"
+#include "sql/sql_digest.h"  // DIGEST_HASH_TO_STRING[_LENGTH]
 #include "sql_string.h"
-#include "template_utils.h"  // pointer_cast
 
 class MY_LOCALE;
 class PT_item_list;
 class THD;
 class my_decimal;
-struct Parse_context;
-
 template <class T>
 class List;
 
@@ -76,43 +72,64 @@ class Item_str_func : public Item_func {
   typedef Item_func super;
 
  public:
-  Item_str_func() : Item_func() {}
+  Item_str_func() : Item_func() { set_data_type_string_init(); }
 
-  explicit Item_str_func(const POS &pos) : super(pos) {}
+  explicit Item_str_func(const POS &pos) : super(pos) {
+    set_data_type_string_init();
+  }
 
-  Item_str_func(Item *a) : Item_func(a) {}
+  Item_str_func(Item *a) : Item_func(a) { set_data_type_string_init(); }
 
-  Item_str_func(const POS &pos, Item *a) : Item_func(pos, a) {}
+  Item_str_func(const POS &pos, Item *a) : Item_func(pos, a) {
+    set_data_type_string_init();
+  }
 
-  Item_str_func(Item *a, Item *b) : Item_func(a, b) {}
+  Item_str_func(Item *a, Item *b) : Item_func(a, b) {
+    set_data_type_string_init();
+  }
 
-  Item_str_func(const POS &pos, Item *a, Item *b) : Item_func(pos, a, b) {}
-
-  Item_str_func(Item *a, Item *b, Item *c) : Item_func(a, b, c) {}
+  Item_str_func(const POS &pos, Item *a, Item *b) : Item_func(pos, a, b) {
+    set_data_type_string_init();
+  }
 
+  Item_str_func(Item *a, Item *b, Item *c) : Item_func(a, b, c) {
+    set_data_type_string_init();
+  }
   Item_str_func(const POS &pos, Item *a, Item *b, Item *c)
-      : Item_func(pos, a, b, c) {}
+      : Item_func(pos, a, b, c) {
+    set_data_type_string_init();
+  }
 
-  Item_str_func(Item *a, Item *b, Item *c, Item *d) : Item_func(a, b, c, d) {}
+  Item_str_func(Item *a, Item *b, Item *c, Item *d) : Item_func(a, b, c, d) {
+    set_data_type_string_init();
+  }
 
   Item_str_func(const POS &pos, Item *a, Item *b, Item *c, Item *d)
-      : Item_func(pos, a, b, c, d) {}
+      : Item_func(pos, a, b, c, d) {
+    set_data_type_string_init();
+  }
 
   Item_str_func(Item *a, Item *b, Item *c, Item *d, Item *e)
-      : Item_func(a, b, c, d, e) {}
+      : Item_func(a, b, c, d, e) {
+    set_data_type_string_init();
+  }
 
   Item_str_func(const POS &pos, Item *a, Item *b, Item *c, Item *d, Item *e)
-      : Item_func(pos, a, b, c, d, e) {}
-  Item_str_func(const POS &pos, Item *a, Item *b, Item *c, Item *d, Item *e,
-                Item *f)
-      : Item_func(pos, a, b, c, d, e, f) {}
-  explicit Item_str_func(mem_root_deque<Item *> *list) : Item_func(list) {}
+      : Item_func(pos, a, b, c, d, e) {
+    set_data_type_string_init();
+  }
+
+  Item_str_func(List<Item> &list) : Item_func(list) {
+    set_data_type_string_init();
+  }
 
   Item_str_func(const POS &pos, PT_item_list *opt_list)
-      : Item_func(pos, opt_list) {}
+      : Item_func(pos, opt_list) {
+    set_data_type_string_init();
+  }
 
-  longlong val_int() override { return val_int_from_string(); }
-  double val_real() override { return val_real_from_string(); }
+  longlong val_int() override;
+  double val_real() override;
   my_decimal *val_decimal(my_decimal *) override;
   bool get_date(MYSQL_TIME *ltime, my_time_flags_t fuzzydate) override {
     return get_date_from_string(ltime, fuzzydate);
@@ -121,12 +138,8 @@ class Item_str_func : public Item_func {
     return get_time_from_string(ltime);
   }
   enum Item_result result_type() const override { return STRING_RESULT; }
-  void left_right_max_length(THD *thd);
+  void left_right_max_length();
   bool fix_fields(THD *thd, Item **ref) override;
-  bool resolve_type(THD *thd) override {
-    if (param_type_is_default(thd, 0, -1)) return true;
-    return false;
-  }
   String *val_str_from_val_str_ascii(String *str, String *str2);
 
  protected:
@@ -174,7 +187,7 @@ class Item_str_ascii_func : public Item_str_func {
   String *val_str(String *str) override {
     return val_str_from_val_str_ascii(str, &ascii_buf);
   }
-  String *val_str_ascii(String *) override = 0;
+  virtual String *val_str_ascii(String *) override = 0;
 };
 
 class Item_func_md5 final : public Item_str_ascii_func {
@@ -281,14 +294,7 @@ class Item_func_aes_encrypt final : public Item_str_func {
       : Item_str_func(pos, a, b) {}
   Item_func_aes_encrypt(const POS &pos, Item *a, Item *b, Item *c)
       : Item_str_func(pos, a, b, c) {}
-  Item_func_aes_encrypt(const POS &pos, Item *a, Item *b, Item *c, Item *d)
-      : Item_str_func(pos, a, b, c, d) {}
-  Item_func_aes_encrypt(const POS &pos, Item *a, Item *b, Item *c, Item *d,
-                        Item *e)
-      : Item_str_func(pos, a, b, c, d, e) {}
-  Item_func_aes_encrypt(const POS &pos, Item *a, Item *b, Item *c, Item *d,
-                        Item *e, Item *f)
-      : Item_str_func(pos, a, b, c, d, e, f) {}
+
   bool itemize(Parse_context *pc, Item **res) override;
   String *val_str(String *) override;
   bool resolve_type(THD *) override;
@@ -303,14 +309,7 @@ class Item_func_aes_decrypt : public Item_str_func {
       : Item_str_func(pos, a, b) {}
   Item_func_aes_decrypt(const POS &pos, Item *a, Item *b, Item *c)
       : Item_str_func(pos, a, b, c) {}
-  Item_func_aes_decrypt(const POS &pos, Item *a, Item *b, Item *c, Item *d)
-      : Item_str_func(pos, a, b, c, d) {}
-  Item_func_aes_decrypt(const POS &pos, Item *a, Item *b, Item *c, Item *d,
-                        Item *e)
-      : Item_str_func(pos, a, b, c, d, e) {}
-  Item_func_aes_decrypt(const POS &pos, Item *a, Item *b, Item *c, Item *d,
-                        Item *e, Item *f)
-      : Item_str_func(pos, a, b, c, d, e, f) {}
+
   bool itemize(Parse_context *pc, Item **res) override;
   String *val_str(String *) override;
   bool resolve_type(THD *thd) override;
@@ -331,9 +330,6 @@ class Item_func_random_bytes : public Item_str_func {
   String *val_str(String *a) override;
 
   const char *func_name() const override { return "random_bytes"; }
-  table_map get_initial_pseudo_tables() const override {
-    return RAND_TABLE_BIT;
-  }
 };
 
 class Item_func_concat : public Item_str_func {
@@ -353,8 +349,7 @@ class Item_func_concat : public Item_str_func {
 class Item_func_concat_ws : public Item_str_func {
   String tmp_value{"", 0, collation.collation};  // Initialize to empty
  public:
-  explicit Item_func_concat_ws(mem_root_deque<Item *> *list)
-      : Item_str_func(list) {
+  Item_func_concat_ws(List<Item> &list) : Item_str_func(list) {
     null_on_null = false;
   }
   Item_func_concat_ws(const POS &pos, PT_item_list *opt_list)
@@ -611,8 +606,8 @@ class Item_func_database : public Item_func_sysconst {
 
   String *val_str(String *) override;
   bool resolve_type(THD *) override {
-    set_data_type_string(uint32{MAX_FIELD_NAME});
-    set_nullable(true);
+    set_data_type_string(uint32(MAX_FIELD_NAME));
+    maybe_null = true;
     return false;
   }
   const char *func_name() const override { return "database"; }
@@ -625,12 +620,10 @@ class Item_func_user : public Item_func_sysconst {
   typedef Item_func_sysconst super;
 
  protected:
-  /// True when function value is evaluated, set to false after each execution
-  bool m_evaluated = false;
-
-  /// Evaluate user name, must be called once per execution
-  bool evaluate(const char *user, const char *host);
-  type_conversion_status save_in_field_inner(Field *field, bool) override;
+  bool init(const char *user, const char *host);
+  type_conversion_status save_in_field_inner(Field *field, bool) override {
+    return save_str_value_in_field(field, &str_value);
+  }
 
  public:
   Item_func_user() { str_value.set("", 0, system_charset_info); }
@@ -638,35 +631,28 @@ class Item_func_user : public Item_func_sysconst {
     str_value.set("", 0, system_charset_info);
   }
 
-  table_map get_initial_pseudo_tables() const override {
-    return INNER_TABLE_BIT;
-  }
-
   bool itemize(Parse_context *pc, Item **res) override;
 
+  String *val_str(String *) override {
+    DBUG_ASSERT(fixed == 1);
+    return (null_value ? nullptr : &str_value);
+  }
+  bool fix_fields(THD *thd, Item **ref) override;
   bool check_function_as_value_generator(uchar *checker_args) override {
     Check_function_as_value_generator_parameters *func_arg =
         pointer_cast<Check_function_as_value_generator_parameters *>(
             checker_args);
     func_arg->banned_function_name = func_name();
-    return ((func_arg->source == VGS_GENERATED_COLUMN) ||
-            (func_arg->source == VGS_CHECK_CONSTRAINT));
+    return true;
   }
   bool resolve_type(THD *) override {
-    set_data_type_string(uint32{USERNAME_CHAR_LENGTH + HOSTNAME_LENGTH + 1U});
+    set_data_type_string(uint32(USERNAME_CHAR_LENGTH + HOSTNAME_LENGTH + 1U));
     return false;
   }
-  void cleanup() override {
-    m_evaluated = false;
-    str_value.mem_free();
-    super::cleanup();
-  }
   const char *func_name() const override { return "user"; }
   const Name_string fully_qualified_func_name() const override {
     return NAME_STRING("user()");
   }
-
-  String *val_str(String *) override;
 };
 
 class Item_func_current_user : public Item_func_user {
@@ -674,19 +660,16 @@ class Item_func_current_user : public Item_func_user {
 
   Name_resolution_context *context;
 
- protected:
-  type_conversion_status save_in_field_inner(Field *field, bool) override;
-
  public:
   explicit Item_func_current_user(const POS &pos) : super(pos) {}
 
   bool itemize(Parse_context *pc, Item **res) override;
+
+  bool fix_fields(THD *thd, Item **ref) override;
   const char *func_name() const override { return "current_user"; }
   const Name_string fully_qualified_func_name() const override {
     return NAME_STRING("current_user()");
   }
-
-  String *val_str(String *) override;
 };
 
 class Item_func_soundex : public Item_str_func {
@@ -724,14 +707,14 @@ class Item_func_make_set final : public Item_str_func {
   bool itemize(Parse_context *pc, Item **res) override;
   String *val_str(String *str) override;
   bool fix_fields(THD *thd, Item **ref) override {
-    assert(fixed == 0);
+    DBUG_ASSERT(fixed == 0);
     bool res = ((!item->fixed && item->fix_fields(thd, &item)) ||
                 item->check_cols(1) || Item_func::fix_fields(thd, ref));
-    set_nullable(is_nullable() || item->is_nullable());
+    maybe_null |= item->maybe_null;
     return res;
   }
   void split_sum_func(THD *thd, Ref_item_array ref_item_array,
-                      mem_root_deque<Item *> *fields) override;
+                      List<Item> &fields) override;
   bool resolve_type(THD *) override;
   void update_used_tables() override;
   const char *func_name() const override { return "make_set"; }
@@ -779,8 +762,7 @@ class Item_func_char final : public Item_str_func {
     collation.set(cs);
   }
   String *val_str(String *) override;
-  bool resolve_type(THD *thd) override {
-    if (param_type_is_default(thd, 0, -1, MYSQL_TYPE_LONGLONG)) return true;
+  bool resolve_type(THD *) override {
     set_data_type_string(arg_count * 4U);
     return false;
   }
@@ -865,7 +847,7 @@ class Item_func_is_uuid final : public Item_bool_func {
   const char *func_name() const override { return "is_uuid"; }
   bool resolve_type(THD *thd) override {
     bool res = super::resolve_type(thd);
-    set_nullable(true);
+    maybe_null = true;
     return res;
   }
 };
@@ -888,7 +870,10 @@ class Item_func_hex : public Item_str_ascii_func {
   Item_func_hex(const POS &pos, Item *a) : Item_str_ascii_func(pos, a) {}
   const char *func_name() const override { return "hex"; }
   String *val_str_ascii(String *) override;
-  bool resolve_type(THD *thd) override;
+  bool resolve_type(THD *) override {
+    set_data_type_string(args[0]->max_length * 2U, default_charset());
+    return false;
+  }
 };
 
 class Item_func_unhex final : public Item_str_func {
@@ -897,14 +882,17 @@ class Item_func_unhex final : public Item_str_func {
  public:
   Item_func_unhex(const POS &pos, Item *a) : Item_str_func(pos, a) {
     /* there can be bad hex strings */
-    set_nullable(true);
+    maybe_null = true;
   }
   const char *func_name() const override { return "unhex"; }
   String *val_str(String *) override;
-  bool resolve_type(THD *thd) override;
+  bool resolve_type(THD *) override {
+    set_data_type_string((1U + args[0]->max_length) / 2U, &my_charset_bin);
+    return false;
+  }
 };
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
 class Item_func_like_range : public Item_str_func {
  protected:
   String min_str;
@@ -914,13 +902,11 @@ class Item_func_like_range : public Item_str_func {
  public:
   Item_func_like_range(const POS &pos, Item *a, Item *b, bool is_min_arg)
       : Item_str_func(pos, a, b), is_min(is_min_arg) {
-    set_nullable(true);
+    maybe_null = true;
   }
   String *val_str(String *) override;
-  bool resolve_type(THD *thd) override {
-    if (param_type_is_default(thd, 0, 1)) return true;
-    if (param_type_is_default(thd, 1, 2, MYSQL_TYPE_LONGLONG)) return true;
-    set_data_type_string(uint32{MAX_BLOB_WIDTH}, args[0]->collation);
+  bool resolve_type(THD *) override {
+    set_data_type_string(uint32(MAX_BLOB_WIDTH), args[0]->collation);
     return false;
   }
 };
@@ -940,81 +926,23 @@ class Item_func_like_range_max final : public Item_func_like_range {
 };
 #endif
 
-/**
-  The following types of conversions are considered safe:
-
-  Conversion to and from "binary".
-  Conversion to Unicode.
-  Other kind of conversions are potentially lossy.
-*/
-class Item_charset_conversion : public Item_str_func {
- protected:
-  /// If true, conversion is needed so do it, else allow string copy.
-  bool m_charset_conversion{false};
-  /// The character set we are converting to
-  const CHARSET_INFO *m_cast_cs;
-  /// The character set we are converting from
-  const CHARSET_INFO *m_from_cs{nullptr};
-  String m_tmp_value;
-  /// Marks whether the underlying Item is constant and may be cached.
-  bool m_use_cached_value{false};
-  /// Length argument value, if any given.
-  longlong m_cast_length{-1};  // a priori not used
- public:
-  bool m_safe;
-
- protected:
-  /**
-    Helper for CAST and CONVERT type resolution: common logic to compute the
-    maximum numbers of characters of the type of the conversion.
-
-    @returns the maximum numbers of characters possible after the conversion
-  */
-  uint32 compute_max_char_length();
-
-  bool resolve_type(THD *thd) override;
+class Item_typecast_char final : public Item_str_func {
+  longlong cast_length;
+  const CHARSET_INFO *cast_cs, *from_cs;
+  bool charset_conversion;
+  String tmp_value;
 
  public:
-  Item_charset_conversion(THD *thd, Item *a, const CHARSET_INFO *cs_arg,
-                          bool cache_if_const)
-      : Item_str_func(a), m_cast_cs(cs_arg) {
-    if (cache_if_const && args[0]->may_evaluate_const(thd)) {
-      uint errors = 0;
-      String tmp, *str = args[0]->val_str(&tmp);
-      if (!str || str_value.copy(str->ptr(), str->length(), str->charset(),
-                                 m_cast_cs, &errors))
-        null_value = true;
-      m_use_cached_value = true;
-      str_value.mark_as_const();
-      m_safe = (errors == 0);
-    } else {
-      m_use_cached_value = false;
-      // Marks whether the conversion is safe
-      m_safe = (args[0]->collation.collation == &my_charset_bin ||
-                cs_arg == &my_charset_bin || (cs_arg->state & MY_CS_UNICODE));
-    }
-  }
-  Item_charset_conversion(const POS &pos, Item *a, const CHARSET_INFO *cs_arg)
-      : Item_str_func(pos, a), m_cast_cs(cs_arg) {}
-
-  String *val_str(String *) override;
-};
-
-class Item_typecast_char final : public Item_charset_conversion {
- public:
-  Item_typecast_char(THD *thd, Item *a, longlong length_arg,
-                     const CHARSET_INFO *cs_arg)
-      : Item_charset_conversion(thd, a, cs_arg, false) {
-    m_cast_length = length_arg;
-  }
+  Item_typecast_char(Item *a, longlong length_arg, const CHARSET_INFO *cs_arg)
+      : Item_str_func(a), cast_length(length_arg), cast_cs(cs_arg) {}
   Item_typecast_char(const POS &pos, Item *a, longlong length_arg,
                      const CHARSET_INFO *cs_arg)
-      : Item_charset_conversion(pos, a, cs_arg) {
-    m_cast_length = length_arg;
-  }
+      : Item_str_func(pos, a), cast_length(length_arg), cast_cs(cs_arg) {}
   enum Functype functype() const override { return TYPECAST_FUNC; }
   bool eq(const Item *item, bool binary_cmp) const override;
   const char *func_name() const override { return "cast_as_char"; }
+  String *val_str(String *a) override;
+  bool resolve_type(THD *) override;
   void print(const THD *thd, String *str,
              enum_query_type query_type) const override;
 };
@@ -1030,14 +958,10 @@ class Item_load_file final : public Item_str_func {
   bool itemize(Parse_context *pc, Item **res) override;
   String *val_str(String *) override;
   const char *func_name() const override { return "load_file"; }
-  table_map get_initial_pseudo_tables() const override {
-    return INNER_TABLE_BIT;
-  }
-  bool resolve_type(THD *thd) override {
-    if (param_type_is_default(thd, 0, 1)) return true;
+  bool resolve_type(THD *) override {
     collation.set(&my_charset_bin, DERIVATION_COERCIBLE);
     set_data_type_blob(MAX_BLOB_WIDTH);
-    set_nullable(true);
+    maybe_null = true;
     return false;
   }
   bool check_function_as_value_generator(uchar *checker_args) override {
@@ -1073,18 +997,52 @@ class Item_func_quote : public Item_str_func {
   bool resolve_type(THD *thd) override;
 };
 
-class Item_func_conv_charset final : public Item_charset_conversion {
+class Item_func_conv_charset final : public Item_str_func {
+  /// Marks weather the underlying Item is constant and may be cached.
+  bool use_cached_value;
+  String tmp_value;
+
  public:
+  /**
+    The following types of conversions are considered safe:
+
+    Conversion to and from "binary".
+    Conversion to Unicode.
+    Other kind of conversions are potentially lossy.
+  */
+  bool safe;
+  const CHARSET_INFO *conv_charset;  // keep it public
   Item_func_conv_charset(const POS &pos, Item *a, const CHARSET_INFO *cs)
-      : Item_charset_conversion(pos, a, cs) {
-    m_safe = false;
+      : Item_str_func(pos, a) {
+    conv_charset = cs;
+    use_cached_value = false;
+    safe = false;
   }
 
   Item_func_conv_charset(THD *thd, Item *a, const CHARSET_INFO *cs,
                          bool cache_if_const)
-      : Item_charset_conversion(thd, a, cs, cache_if_const) {
-    assert(args[0]->fixed);
+      : Item_str_func(a) {
+    DBUG_ASSERT(args[0]->fixed);
+
+    conv_charset = cs;
+    if (cache_if_const && args[0]->may_evaluate_const(thd)) {
+      uint errors = 0;
+      String tmp, *str = args[0]->val_str(&tmp);
+      if (!str || str_value.copy(str->ptr(), str->length(), str->charset(),
+                                 conv_charset, &errors))
+        null_value = true;
+      use_cached_value = true;
+      str_value.mark_as_const();
+      safe = (errors == 0);
+    } else {
+      use_cached_value = false;
+      // Marks weather the conversion is safe
+      safe = (args[0]->collation.collation == &my_charset_bin ||
+              cs == &my_charset_bin || (cs->state & MY_CS_UNICODE));
+    }
   }
+  String *val_str(String *) override;
+  bool resolve_type(THD *) override;
   const char *func_name() const override { return "convert"; }
   void print(const THD *thd, String *str,
              enum_query_type query_type) const override;
@@ -1121,10 +1079,10 @@ class Item_func_charset final : public Item_str_func {
   }
   String *val_str(String *) override;
   const char *func_name() const override { return "charset"; }
-  bool resolve_type(THD *thd) override {
+  bool resolve_type(THD *) override {
     set_data_type_string(64U, system_charset_info);
-    set_nullable(false);
-    return Item_str_func::resolve_type(thd);
+    maybe_null = false;
+    return false;
   }
 };
 
@@ -1135,10 +1093,9 @@ class Item_func_collation : public Item_str_func {
   }
   String *val_str(String *) override;
   const char *func_name() const override { return "collation"; }
-  bool resolve_type(THD *thd) override {
-    if (Item_str_func::resolve_type(thd)) return true;
+  bool resolve_type(THD *) override {
     set_data_type_string(64U, system_charset_info);
-    set_nullable(false);
+    maybe_null = false;
     return false;
   }
 };
@@ -1150,7 +1107,7 @@ class Item_func_weight_string final : public Item_str_func {
   uint flags;
   const uint num_codepoints;
   const uint result_length;
-  Item_field *m_field_ref{nullptr};
+  Field *field;
   const bool as_binary;
 
  public:
@@ -1161,6 +1118,7 @@ class Item_func_weight_string final : public Item_str_func {
         flags(flags_arg),
         num_codepoints(num_codepoints_arg),
         result_length(result_length_arg),
+        field(nullptr),
         as_binary(as_binary_arg) {}
 
   bool itemize(Parse_context *pc, Item **res) override;
@@ -1181,8 +1139,7 @@ class Item_func_crc32 final : public Item_int_func {
     unsigned_flag = true;
   }
   const char *func_name() const override { return "crc32"; }
-  bool resolve_type(THD *thd) override {
-    if (param_type_is_default(thd, 0, 1)) return true;
+  bool resolve_type(THD *) override {
     max_length = 10;
     return false;
   }
@@ -1196,8 +1153,7 @@ class Item_func_uncompressed_length final : public Item_int_func {
   Item_func_uncompressed_length(const POS &pos, Item *a)
       : Item_int_func(pos, a) {}
   const char *func_name() const override { return "uncompressed_length"; }
-  bool resolve_type(THD *thd) override {
-    if (param_type_is_default(thd, 0, 1)) return true;
+  bool resolve_type(THD *) override {
     max_length = 10;
     return false;
   }
@@ -1209,7 +1165,10 @@ class Item_func_compress final : public Item_str_func {
 
  public:
   Item_func_compress(const POS &pos, Item *a) : Item_str_func(pos, a) {}
-  bool resolve_type(THD *thd) override;
+  bool resolve_type(THD *) override {
+    set_data_type_string((args[0]->max_length * 120U) / 100U + 12U);
+    return false;
+  }
   const char *func_name() const override { return "compress"; }
   String *val_str(String *str) override;
 };
@@ -1219,10 +1178,9 @@ class Item_func_uncompress final : public Item_str_func {
 
  public:
   Item_func_uncompress(const POS &pos, Item *a) : Item_str_func(pos, a) {}
-  bool resolve_type(THD *thd) override {
-    if (Item_str_func::resolve_type(thd)) return true;
-    set_nullable(true);
-    set_data_type_string(uint32{MAX_BLOB_WIDTH});
+  bool resolve_type(THD *) override {
+    maybe_null = true;
+    set_data_type_string(uint32(MAX_BLOB_WIDTH));
     return false;
   }
   const char *func_name() const override { return "uncompress"; }
@@ -1253,6 +1211,17 @@ class Item_func_uuid final : public Item_str_func {
   }
 };
 
+class Item_func_gtid_subtract final : public Item_str_ascii_func {
+  String buf1, buf2;
+
+ public:
+  Item_func_gtid_subtract(const POS &pos, Item *a, Item *b)
+      : Item_str_ascii_func(pos, a, b) {}
+  bool resolve_type(THD *) override;
+  const char *func_name() const override { return "gtid_subtract"; }
+  String *val_str_ascii(String *) override;
+};
+
 class Item_func_current_role final : public Item_func_sysconst {
   typedef Item_func_sysconst super;
 
@@ -1333,8 +1302,8 @@ class Item_func_get_dd_column_privileges final : public Item_str_func {
       per privileges is 11 chars.
       So, setting max approximate to 200.
     */
-    set_data_type_string(14U * 11U, system_charset_info);
-    set_nullable(true);
+    set_data_type_string(14 * 11, system_charset_info);
+    maybe_null = true;
     null_on_null = false;
 
     return false;
@@ -1354,8 +1323,8 @@ class Item_func_get_dd_create_options final : public Item_str_func {
   bool resolve_type(THD *) override {
     // maximum string length of all options is expected
     // to be less than 256 characters.
-    set_data_type_string(256U, system_charset_info);
-    set_nullable(false);
+    set_data_type_string(256, system_charset_info);
+    maybe_null = false;
     null_on_null = false;
 
     return false;
@@ -1366,27 +1335,6 @@ class Item_func_get_dd_create_options final : public Item_str_func {
   String *val_str(String *) override;
 };
 
-class Item_func_get_dd_schema_options final : public Item_str_func {
- public:
-  Item_func_get_dd_schema_options(const POS &pos, Item *a)
-      : Item_str_func(pos, a) {}
-
-  enum Functype functype() const override { return DD_INTERNAL_FUNC; }
-  bool resolve_type(THD *) override {
-    // maximum string length of all options is expected
-    // to be less than 256 characters.
-    set_data_type_string(256, system_charset_info);
-    set_nullable(false);
-    null_on_null = false;
-
-    return false;
-  }
-
-  const char *func_name() const override { return "get_dd_schema_options"; }
-
-  String *val_str(String *) override;
-};
-
 class Item_func_internal_get_comment_or_error final : public Item_str_func {
  public:
   Item_func_internal_get_comment_or_error(const POS &pos, PT_item_list *list)
@@ -1399,8 +1347,8 @@ class Item_func_internal_get_comment_or_error final : public Item_str_func {
       which is same as size of column holding comments in dictionary,
       i.e., the mysql.tables.comment DD column.
     */
-    set_data_type_string(2048U, system_charset_info);
-    set_nullable(true);
+    set_data_type_string(2048, system_charset_info);
+    maybe_null = true;
     null_on_null = false;
 
     return false;
@@ -1422,8 +1370,8 @@ class Item_func_get_dd_tablespace_private_data final : public Item_str_func {
   bool resolve_type(THD *) override {
     /* maximum string length of the property value is expected
     to be less than 256 characters. */
-    set_data_type_string(256U);
-    set_nullable(false);
+    max_length = 256;
+    maybe_null = false;
     null_on_null = false;
 
     return false;
@@ -1445,8 +1393,8 @@ class Item_func_get_dd_index_private_data final : public Item_str_func {
   bool resolve_type(THD *) override {
     /* maximum string length of the property value is expected
     to be less than 256 characters. */
-    set_data_type_string(256U);
-    set_nullable(false);
+    max_length = 256;
+    maybe_null = false;
     null_on_null = false;
 
     return false;
@@ -1466,8 +1414,8 @@ class Item_func_get_partition_nodegroup final : public Item_str_func {
   bool resolve_type(THD *) override {
     // maximum string length of all options is expected
     // to be less than 256 characters.
-    set_data_type_string(256U, system_charset_info);
-    set_nullable(true);
+    set_data_type_string(256, system_charset_info);
+    maybe_null = true;
     null_on_null = false;
 
     return false;
@@ -1490,8 +1438,8 @@ class Item_func_internal_tablespace_type : public Item_str_func {
   bool resolve_type(THD *) override {
     // maximum string length of all options is expected
     // to be less than 256 characters.
-    set_data_type_string(256U, system_charset_info);
-    set_nullable(true);
+    set_data_type_string(256, system_charset_info);
+    maybe_null = true;
     null_on_null = false;
 
     return false;
@@ -1512,8 +1460,8 @@ class Item_func_internal_tablespace_logfile_group_name : public Item_str_func {
   bool resolve_type(THD *) override {
     // maximum string length of all options is expected
     // to be less than 256 characters.
-    set_data_type_string(256U, system_charset_info);
-    set_nullable(true);
+    set_data_type_string(256, system_charset_info);
+    maybe_null = true;
     null_on_null = false;
 
     return false;
@@ -1536,8 +1484,8 @@ class Item_func_internal_tablespace_status : public Item_str_func {
   bool resolve_type(THD *) override {
     // maximum string length of all options is expected
     // to be less than 256 characters.
-    set_data_type_string(256U, system_charset_info);
-    set_nullable(true);
+    set_data_type_string(256, system_charset_info);
+    maybe_null = true;
     null_on_null = false;
 
     return false;
@@ -1559,8 +1507,8 @@ class Item_func_internal_tablespace_row_format : public Item_str_func {
   bool resolve_type(THD *) override {
     // maximum string length of all options is expected
     // to be less than 256 characters.
-    set_data_type_string(256U, system_charset_info);
-    set_nullable(true);
+    set_data_type_string(256, system_charset_info);
+    maybe_null = true;
     null_on_null = false;
 
     return false;
@@ -1583,8 +1531,8 @@ class Item_func_internal_tablespace_extra : public Item_str_func {
   bool resolve_type(THD *) override {
     // maximum string length of all options is expected
     // to be less than 256 characters.
-    set_data_type_string(256U, system_charset_info);
-    set_nullable(true);
+    set_data_type_string(256, system_charset_info);
+    maybe_null = true;
     null_on_null = false;
 
     return false;
@@ -1601,8 +1549,9 @@ class Item_func_convert_cpu_id_mask final : public Item_str_func {
       : Item_str_func(pos, list) {}
 
   bool resolve_type(THD *) override {
-    set_nullable(false);
-    set_data_type_string(1024U, &my_charset_bin);
+    max_length = 1024;
+    maybe_null = false;
+    set_data_type_string(1024, &my_charset_bin);
     return false;
   }
 
@@ -1619,7 +1568,7 @@ class Item_func_get_dd_property_key_value final : public Item_str_func {
   enum Functype functype() const override { return DD_INTERNAL_FUNC; }
   bool resolve_type(THD *) override {
     set_data_type_string(MAX_BLOB_WIDTH, system_charset_info);
-    set_nullable(true);
+    maybe_null = true;
     null_on_null = false;
 
     return false;
@@ -1638,7 +1587,7 @@ class Item_func_remove_dd_property_key final : public Item_str_func {
   enum Functype functype() const override { return DD_INTERNAL_FUNC; }
   bool resolve_type(THD *) override {
     set_data_type_string(MAX_BLOB_WIDTH, system_charset_info);
-    set_nullable(true);
+    maybe_null = true;
     null_on_null = false;
 
     return false;
@@ -1658,8 +1607,8 @@ class Item_func_convert_interval_to_user_interval final : public Item_str_func {
   bool resolve_type(THD *) override {
     // maximum string length of all options is expected
     // to be less than 256 characters.
-    set_data_type_string(256U, system_charset_info);
-    set_nullable(true);
+    set_data_type_string(256, system_charset_info);
+    maybe_null = true;
     null_on_null = false;
 
     return false;
@@ -1680,7 +1629,7 @@ class Item_func_internal_get_username final : public Item_str_func {
   enum Functype functype() const override { return DD_INTERNAL_FUNC; }
   bool resolve_type(THD *) override {
     set_data_type_string(uint32(USERNAME_LENGTH + 1), system_charset_info);
-    set_nullable(true);
+    maybe_null = true;
     null_on_null = false;
 
     return false;
@@ -1699,7 +1648,7 @@ class Item_func_internal_get_hostname final : public Item_str_func {
   enum Functype functype() const override { return DD_INTERNAL_FUNC; }
   bool resolve_type(THD *) override {
     set_data_type_string(uint32(HOSTNAME_LENGTH + 1), system_charset_info);
-    set_nullable(true);
+    maybe_null = true;
     null_on_null = false;
 
     return false;
@@ -1718,7 +1667,7 @@ class Item_func_internal_get_enabled_role_json final : public Item_str_func {
   enum Functype functype() const override { return DD_INTERNAL_FUNC; }
   bool resolve_type(THD *) override {
     set_data_type_string(uint32(MAX_BLOB_WIDTH), system_charset_info);
-    set_nullable(true);
+    maybe_null = true;
     null_on_null = false;
 
     return false;
@@ -1739,7 +1688,7 @@ class Item_func_internal_get_mandatory_roles_json final : public Item_str_func {
   enum Functype functype() const override { return DD_INTERNAL_FUNC; }
   bool resolve_type(THD *) override {
     set_data_type_string(uint32(MAX_BLOB_WIDTH), system_charset_info);
-    set_nullable(true);
+    maybe_null = true;
     null_on_null = false;
 
     return false;
@@ -1761,8 +1710,8 @@ class Item_func_internal_get_dd_column_extra final : public Item_str_func {
   bool resolve_type(THD *) override {
     // maximum string length of all options is expected
     // to be less than 256 characters.
-    set_data_type_string(256U, system_charset_info);
-    set_nullable(false);
+    set_data_type_string(256, system_charset_info);
+    maybe_null = false;
     null_on_null = false;
 
     return false;
