@@ -1,19 +1,18 @@
 #ifndef LEX_INCLUDED
 #define LEX_INCLUDED
 
-/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,8 +25,8 @@
 
 /* This file includes all reserved words and functions */
 
+#include "m_string.h"
 #include "sql/lex_symbol.h"
-#include "string_with_len.h"
 
 /* We don't want to include sql_yacc.h into gen_lex_hash */
 #ifdef NO_YACC_SYMBOLS
@@ -95,9 +94,6 @@ static const SYMBOL symbols[] = {
     {SYM("ASCII", ASCII_SYM)},
     {SYM("ASENSITIVE", ASENSITIVE_SYM)},
     {SYM("AT", AT_SYM)},
-    {SYM("ATTRIBUTE", ATTRIBUTE_SYM)},
-    {SYM("AUTHENTICATION", AUTHENTICATION_SYM)},
-    {SYM("AUTO", AUTO_SYM)},
     {SYM("AUTO_INCREMENT", AUTO_INC)},
     {SYM("AUTOEXTEND_SIZE", AUTOEXTEND_SIZE_SYM)},
     {SYM("AVG", AVG_SYM)},
@@ -105,7 +101,6 @@ static const SYMBOL symbols[] = {
     {SYM("BACKUP", BACKUP_SYM)},
     {SYM("BEFORE", BEFORE_SYM)},
     {SYM("BEGIN", BEGIN_SYM)},
-    {SYM("BERNOULLI", BERNOULLI_SYM)},
     {SYM("BETWEEN", BETWEEN_SYM)},
     {SYM("BIGINT", BIGINT_SYM)},
     {SYM("BINARY", BINARY_SYM)},
@@ -118,7 +113,6 @@ static const SYMBOL symbols[] = {
     {SYM("BOTH", BOTH)},
     {SYM("BTREE", BTREE_SYM)},
     {SYM("BUCKETS", BUCKETS_SYM)},
-    {SYM("BULK", BULK_SYM)},
     {SYM("BY", BY)},
     {SYM("BYTE", BYTE_SYM)},
     {SYM("CACHE", CACHE_SYM)},
@@ -128,7 +122,6 @@ static const SYMBOL symbols[] = {
     {SYM("CASE", CASE_SYM)},
     {SYM("CATALOG_NAME", CATALOG_NAME_SYM)},
     {SYM("CHAIN", CHAIN_SYM)},
-    {SYM("CHALLENGE_RESPONSE", CHALLENGE_RESPONSE_SYM)},
     {SYM("CHANGE", CHANGE)},
     {SYM("CHANGED", CHANGED)},
     {SYM("CHANNEL", CHANNEL_SYM)},
@@ -234,7 +227,6 @@ static const SYMBOL symbols[] = {
     {SYM("ENDS", ENDS_SYM)},
     {SYM("ENFORCED", ENFORCED_SYM)},
     {SYM("ENGINE", ENGINE_SYM)},
-    {SYM("ENGINE_ATTRIBUTE", ENGINE_ATTRIBUTE_SYM)},
     {SYM("ENGINES", ENGINES_SYM)},
     {SYM("ENUM", ENUM_SYM)},
     {SYM("ERROR", ERROR_SYM)},
@@ -256,7 +248,6 @@ static const SYMBOL symbols[] = {
     {SYM("EXPLAIN", DESCRIBE)},
     {SYM("EXTENDED", EXTENDED_SYM)},
     {SYM("EXTENT_SIZE", EXTENT_SIZE_SYM)},
-    {SYM("FACTOR", FACTOR_SYM)},
     {SYM("FAILED_LOGIN_ATTEMPTS", FAILED_LOGIN_ATTEMPTS_SYM)},
     {SYM("FALSE", FALSE_SYM)},
     {SYM("FAST", FAST_SYM)},
@@ -266,7 +257,6 @@ static const SYMBOL symbols[] = {
     {SYM("FILE", FILE_SYM)},
     {SYM("FILE_BLOCK_SIZE", FILE_BLOCK_SIZE_SYM)},
     {SYM("FILTER", FILTER_SYM)},
-    {SYM("FINISH", FINISH_SYM)},
     {SYM("FIRST", FIRST_SYM)},
     {SYM("FIRST_VALUE", FIRST_VALUE_SYM)},
     {SYM("FIXED", FIXED_SYM)},
@@ -291,9 +281,8 @@ static const SYMBOL symbols[] = {
     {SYM("GEOMETRY", GEOMETRY_SYM)},
     {SYM("GEOMETRYCOLLECTION", GEOMETRYCOLLECTION_SYM)},
     {SYM("GET_FORMAT", GET_FORMAT)},
-    {SYM("GET_SOURCE_PUBLIC_KEY", GET_SOURCE_PUBLIC_KEY_SYM)},
+    {SYM("GET_MASTER_PUBLIC_KEY", GET_MASTER_PUBLIC_KEY_SYM)},
     {SYM("GET", GET_SYM)},
-    {SYM("GENERATE", GENERATE_SYM)},
     {SYM("GENERATED", GENERATED)},
     {SYM("GLOBAL", GLOBAL_SYM)},
     {SYM("GRANT", GRANT)},
@@ -301,8 +290,6 @@ static const SYMBOL symbols[] = {
     {SYM("GROUP", GROUP_SYM)},
     {SYM("GROUPING", GROUPING_SYM)},
     {SYM("GROUPS", GROUPS_SYM)},
-    {SYM("GTIDS", GTIDS_SYM)},
-    {SYM("GTID_ONLY", GTID_ONLY_SYM)},
     {SYM("HANDLER", HANDLER_SYM)},
     {SYM("HASH", HASH_SYM)},
     {SYM("HAVING", HAVING)},
@@ -325,10 +312,8 @@ static const SYMBOL symbols[] = {
     {SYM("INACTIVE", INACTIVE_SYM)},
     {SYM("INDEX", INDEX_SYM)},
     {SYM("INDEXES", INDEXES)},
-    {SYM("INFILE", INFILE_SYM)},
-    {SYM("INITIAL", INITIAL_SYM)},
+    {SYM("INFILE", INFILE)},
     {SYM("INITIAL_SIZE", INITIAL_SIZE_SYM)},
-    {SYM("INITIATE", INITIATE_SYM)},
     {SYM("INNER", INNER_SYM)},
     {SYM("INOUT", INOUT_SYM)},
     {SYM("INSENSITIVE", INSENSITIVE_SYM)},
@@ -342,7 +327,6 @@ static const SYMBOL symbols[] = {
     {SYM("INT4", INT_SYM)},
     {SYM("INT8", BIGINT_SYM)},
     {SYM("INTEGER", INT_SYM)},
-    {SYM("INTERSECT", INTERSECT_SYM)},
     {SYM("INTERVAL", INTERVAL_SYM)},
     {SYM("INTO", INTO)},
     {SYM("IO", IO_SYM)},
@@ -359,9 +343,7 @@ static const SYMBOL symbols[] = {
     {SYM("JOIN", JOIN_SYM)},
     {SYM("JSON", JSON_SYM)},
     {SYM("JSON_TABLE", JSON_TABLE_SYM)},
-    {SYM("JSON_VALUE", JSON_VALUE_SYM)},
     {SYM("KEY", KEY_SYM)},
-    {SYM("KEYRING", KEYRING_SYM)},
     {SYM("KEYS", KEYS)},
     {SYM("KEY_BLOCK_SIZE", KEY_BLOCK_SIZE)},
     {SYM("KILL", KILL_SYM)},
@@ -392,14 +374,39 @@ static const SYMBOL symbols[] = {
     {SYM("LOCKS", LOCKS_SYM)},
     {SYM("LOGFILE", LOGFILE_SYM)},
     {SYM("LOGS", LOGS_SYM)},
-    {SYM("LOG", LOG_SYM)},
     {SYM("LONG", LONG_SYM)},
     {SYM("LONGBLOB", LONGBLOB_SYM)},
     {SYM("LONGTEXT", LONGTEXT_SYM)},
     {SYM("LOOP", LOOP_SYM)},
     {SYM("LOW_PRIORITY", LOW_PRIORITY)},
-    {SYM("MANUAL", MANUAL_SYM)},
     {SYM("MASTER", MASTER_SYM)},
+    {SYM("MASTER_AUTO_POSITION", MASTER_AUTO_POSITION_SYM)},
+    {SYM("MASTER_BIND", MASTER_BIND_SYM)},
+    {SYM("MASTER_CONNECT_RETRY", MASTER_CONNECT_RETRY_SYM)},
+    {SYM("MASTER_COMPRESSION_ALGORITHMS", MASTER_COMPRESSION_ALGORITHM_SYM)},
+    {SYM("MASTER_DELAY", MASTER_DELAY_SYM)},
+    {SYM("MASTER_HEARTBEAT_PERIOD", MASTER_HEARTBEAT_PERIOD_SYM)},
+    {SYM("MASTER_HOST", MASTER_HOST_SYM)},
+    {SYM("MASTER_LOG_FILE", MASTER_LOG_FILE_SYM)},
+    {SYM("MASTER_LOG_POS", MASTER_LOG_POS_SYM)},
+    {SYM("MASTER_PASSWORD", MASTER_PASSWORD_SYM)},
+    {SYM("MASTER_PORT", MASTER_PORT_SYM)},
+    {SYM("MASTER_PUBLIC_KEY_PATH", MASTER_PUBLIC_KEY_PATH_SYM)},
+    {SYM("MASTER_RETRY_COUNT", MASTER_RETRY_COUNT_SYM)},
+    {SYM("MASTER_SERVER_ID", MASTER_SERVER_ID_SYM)},
+    {SYM("MASTER_SSL", MASTER_SSL_SYM)},
+    {SYM("MASTER_SSL_CA", MASTER_SSL_CA_SYM)},
+    {SYM("MASTER_SSL_CAPATH", MASTER_SSL_CAPATH_SYM)},
+    {SYM("MASTER_SSL_CERT", MASTER_SSL_CERT_SYM)},
+    {SYM("MASTER_SSL_CIPHER", MASTER_SSL_CIPHER_SYM)},
+    {SYM("MASTER_SSL_CRL", MASTER_SSL_CRL_SYM)},
+    {SYM("MASTER_SSL_CRLPATH", MASTER_SSL_CRLPATH_SYM)},
+    {SYM("MASTER_SSL_KEY", MASTER_SSL_KEY_SYM)},
+    {SYM("MASTER_SSL_VERIFY_SERVER_CERT", MASTER_SSL_VERIFY_SERVER_CERT_SYM)},
+    {SYM("MASTER_TLS_CIPHERSUITES", MASTER_TLS_CIPHERSUITES_SYM)},
+    {SYM("MASTER_TLS_VERSION", MASTER_TLS_VERSION_SYM)},
+    {SYM("MASTER_USER", MASTER_USER_SYM)},
+    {SYM("MASTER_ZSTD_COMPRESSION_LEVEL", MASTER_ZSTD_COMPRESSION_LEVEL_SYM)},
     {SYM("MATCH", MATCH)},
     {SYM("MAX_CONNECTIONS_PER_HOUR", MAX_CONNECTIONS_PER_HOUR)},
     {SYM("MAX_QUERIES_PER_HOUR", MAX_QUERIES_PER_HOUR)},
@@ -486,10 +493,8 @@ static const SYMBOL symbols[] = {
     {SYM("OWNER", OWNER_SYM)},
     {SYM("PACK_KEYS", PACK_KEYS_SYM)},
     {SYM("PATH", PATH_SYM)},
-    {SYM("PARSE_TREE", PARSE_TREE_SYM)},
     {SYM("PARSER", PARSER_SYM)},
     {SYM("PAGE", PAGE_SYM)},
-    {SYM("PARALLEL", PARALLEL_SYM)},
     {SYM("PARTIAL", PARTIAL)},
     {SYM("PARTITION", PARTITION_SYM)},
     {SYM("PARTITIONING", PARTITIONING_SYM)},
@@ -522,7 +527,6 @@ static const SYMBOL symbols[] = {
     {SYM("PROFILES", PROFILES_SYM)},
     {SYM("PROXY", PROXY_SYM)},
     {SYM("PURGE", PURGE)},
-    {SYM("QUALIFY", QUALIFY_SYM)},
     {SYM("QUARTER", QUARTER_SYM)},
     {SYM("QUERY", QUERY_SYM)},
     {SYM("QUICK", QUICK)},
@@ -542,7 +546,6 @@ static const SYMBOL symbols[] = {
     {SYM("REFERENCE", REFERENCE_SYM)},
     {SYM("REFERENCES", REFERENCES)},
     {SYM("REGEXP", REGEXP)},
-    {SYM("REGISTRATION", REGISTRATION_SYM)},
     {SYM("RELAY", RELAY)},
     {SYM("RELAYLOG", RELAYLOG_SYM)},
     {SYM("RELAY_LOG_FILE", RELAY_LOG_FILE_SYM)},
@@ -552,13 +555,9 @@ static const SYMBOL symbols[] = {
     {SYM("RELOAD", RELOAD)},
     {SYM("REMOVE", REMOVE_SYM)},
     {SYM("RENAME", RENAME)},
-    {SYM("ASSIGN_GTIDS_TO_ANONYMOUS_TRANSACTIONS",
-         ASSIGN_GTIDS_TO_ANONYMOUS_TRANSACTIONS_SYM)},
     {SYM("REORGANIZE", REORGANIZE_SYM)},
     {SYM("REPAIR", REPAIR)},
     {SYM("REPEATABLE", REPEATABLE_SYM)},
-    {SYM("REPLICA", REPLICA_SYM)},
-    {SYM("REPLICAS", REPLICAS_SYM)},
     {SYM("REPLICATION", REPLICATION)},
     {SYM("REPLICATE_DO_DB", REPLICATE_DO_DB)},
     {SYM("REPLICATE_IGNORE_DB", REPLICATE_IGNORE_DB)},
@@ -583,7 +582,6 @@ static const SYMBOL symbols[] = {
     {SYM("RETAIN", RETAIN_SYM)},
     {SYM("RETURNED_SQLSTATE", RETURNED_SQLSTATE_SYM)},
     {SYM("RETURN", RETURN_SYM)},
-    {SYM("RETURNING", RETURNING_SYM)},
     {SYM("RETURNS", RETURNS_SYM)},
     {SYM("REUSE", REUSE_SYM)},
     {SYM("REVERSE", REVERSE_SYM)},
@@ -601,7 +599,6 @@ static const SYMBOL symbols[] = {
     {SYM("ROWS", ROWS_SYM)},
     {SYM("ROW_FORMAT", ROW_FORMAT_SYM)},
     {SYM("RTREE", RTREE_SYM)},
-    {SYM("S3", S3_SYM)},
     {SYM("SAVEPOINT", SAVEPOINT_SYM)},
     {SYM("SCHEDULE", SCHEDULE_SYM)},
     {SYM("SCHEMA", DATABASE)},
@@ -611,7 +608,6 @@ static const SYMBOL symbols[] = {
     {SYM("SECOND_MICROSECOND", SECOND_MICROSECOND_SYM)},
     {SYM("SECONDARY", SECONDARY_SYM)},
     {SYM("SECONDARY_ENGINE", SECONDARY_ENGINE_SYM)},
-    {SYM("SECONDARY_ENGINE_ATTRIBUTE", SECONDARY_ENGINE_ATTRIBUTE_SYM)},
     {SYM("SECONDARY_LOAD", SECONDARY_LOAD_SYM)},
     {SYM("SECONDARY_UNLOAD", SECONDARY_UNLOAD_SYM)},
     {SYM("SECURITY", SECURITY_SYM)},
@@ -638,34 +634,6 @@ static const SYMBOL symbols[] = {
     {SYM("SONAME", SONAME_SYM)},
     {SYM("SOUNDS", SOUNDS_SYM)},
     {SYM("SOURCE", SOURCE_SYM)},
-    {SYM("SOURCE_AUTO_POSITION", SOURCE_AUTO_POSITION_SYM)},
-    {SYM("SOURCE_BIND", SOURCE_BIND_SYM)},
-    {SYM("SOURCE_COMPRESSION_ALGORITHMS", SOURCE_COMPRESSION_ALGORITHM_SYM)},
-    {SYM("SOURCE_CONNECT_RETRY", SOURCE_CONNECT_RETRY_SYM)},
-    {SYM("SOURCE_CONNECTION_AUTO_FAILOVER",
-         SOURCE_CONNECTION_AUTO_FAILOVER_SYM)},
-    {SYM("SOURCE_DELAY", SOURCE_DELAY_SYM)},
-    {SYM("SOURCE_HEARTBEAT_PERIOD", SOURCE_HEARTBEAT_PERIOD_SYM)},
-    {SYM("SOURCE_HOST", SOURCE_HOST_SYM)},
-    {SYM("SOURCE_LOG_FILE", SOURCE_LOG_FILE_SYM)},
-    {SYM("SOURCE_LOG_POS", SOURCE_LOG_POS_SYM)},
-    {SYM("SOURCE_PASSWORD", SOURCE_PASSWORD_SYM)},
-    {SYM("SOURCE_PORT", SOURCE_PORT_SYM)},
-    {SYM("SOURCE_PUBLIC_KEY_PATH", SOURCE_PUBLIC_KEY_PATH_SYM)},
-    {SYM("SOURCE_RETRY_COUNT", SOURCE_RETRY_COUNT_SYM)},
-    {SYM("SOURCE_SSL_CAPATH", SOURCE_SSL_CAPATH_SYM)},
-    {SYM("SOURCE_SSL_CA", SOURCE_SSL_CA_SYM)},
-    {SYM("SOURCE_SSL_CERT", SOURCE_SSL_CERT_SYM)},
-    {SYM("SOURCE_SSL_CIPHER", SOURCE_SSL_CIPHER_SYM)},
-    {SYM("SOURCE_SSL_CRL", SOURCE_SSL_CRL_SYM)},
-    {SYM("SOURCE_SSL_CRLPATH", SOURCE_SSL_CRLPATH_SYM)},
-    {SYM("SOURCE_SSL_KEY", SOURCE_SSL_KEY_SYM)},
-    {SYM("SOURCE_SSL", SOURCE_SSL_SYM)},
-    {SYM("SOURCE_SSL_VERIFY_SERVER_CERT", SOURCE_SSL_VERIFY_SERVER_CERT_SYM)},
-    {SYM("SOURCE_TLS_CIPHERSUITES", SOURCE_TLS_CIPHERSUITES_SYM)},
-    {SYM("SOURCE_TLS_VERSION", SOURCE_TLS_VERSION_SYM)},
-    {SYM("SOURCE_USER", SOURCE_USER_SYM)},
-    {SYM("SOURCE_ZSTD_COMPRESSION_LEVEL", SOURCE_ZSTD_COMPRESSION_LEVEL_SYM)},
     {SYM("SPATIAL", SPATIAL_SYM)},
     {SYM("SPECIFIC", SPECIFIC_SYM)},
     {SYM("SQL", SQL_SYM)},
@@ -717,7 +685,6 @@ static const SYMBOL symbols[] = {
     {SYM("TABLE", TABLE_SYM)},
     {SYM("TABLE_NAME", TABLE_NAME_SYM)},
     {SYM("TABLES", TABLES)},
-    {SYM("TABLESAMPLE", TABLESAMPLE_SYM)},
     {SYM("TABLESPACE", TABLESPACE_SYM)},
     {SYM("TABLE_CHECKSUM", TABLE_CHECKSUM_SYM)},
     {SYM("TEMPORARY", TEMPORARY)},
@@ -735,7 +702,6 @@ static const SYMBOL symbols[] = {
     {SYM("TINYBLOB", TINYBLOB_SYM)},
     {SYM("TINYINT", TINYINT_SYM)},
     {SYM("TINYTEXT", TINYTEXT_SYN)},
-    {SYM("TLS", TLS_SYM)},
     {SYM("TO", TO_SYM)},
     {SYM("TRAILING", TRAILING)},
     {SYM("TRANSACTION", TRANSACTION_SYM)},
@@ -757,11 +723,9 @@ static const SYMBOL symbols[] = {
     {SYM("UNKNOWN", UNKNOWN_SYM)},
     {SYM("UNLOCK", UNLOCK_SYM)},
     {SYM("UNINSTALL", UNINSTALL_SYM)},
-    {SYM("UNREGISTER", UNREGISTER_SYM)},
     {SYM("UNSIGNED", UNSIGNED_SYM)},
     {SYM("UNTIL", UNTIL_SYM)},
     {SYM("UPGRADE", UPGRADE_SYM)},
-    {SYM("URL", URL_SYM)},
     {SYM("USAGE", USAGE)},
     {SYM("USE", USE_SYM)},
     {SYM("USER", USER)},
@@ -804,7 +768,6 @@ static const SYMBOL symbols[] = {
     {SYM("YEAR", YEAR_SYM)},
     {SYM("YEAR_MONTH", YEAR_MONTH_SYM)},
     {SYM("ZEROFILL", ZEROFILL_SYM)},
-    {SYM("ZONE", ZONE_SYM)},
     {SYM("||", OR_OR_SYM)},
     /*
       Place keywords that accept optimizer hints below this comment.
@@ -842,7 +805,6 @@ static const SYMBOL symbols[] = {
     {SYM_FN("STDDEV", STD_SYM)},
     {SYM_FN("STDDEV_POP", STD_SYM)},
     {SYM_FN("STDDEV_SAMP", STDDEV_SAMP_SYM)},
-    {SYM_FN("ST_COLLECT", ST_COLLECT_SYM)},
     {SYM_FN("SUBDATE", SUBDATE_SYM)},
     {SYM_FN("SUBSTR", SUBSTRING)},
     {SYM_FN("SUBSTRING", SUBSTRING)},
@@ -895,9 +857,6 @@ static const SYMBOL symbols[] = {
     {SYM_H("GROUP_INDEX", GROUP_INDEX_HINT)},
     {SYM_H("NO_GROUP_INDEX", NO_GROUP_INDEX_HINT)},
     {SYM_H("ORDER_INDEX", ORDER_INDEX_HINT)},
-    {SYM_H("NO_ORDER_INDEX", NO_ORDER_INDEX_HINT)},
-    {SYM_H("DERIVED_CONDITION_PUSHDOWN", DERIVED_CONDITION_PUSHDOWN_HINT)},
-    {SYM_H("NO_DERIVED_CONDITION_PUSHDOWN",
-           NO_DERIVED_CONDITION_PUSHDOWN_HINT)}};
+    {SYM_H("NO_ORDER_INDEX", NO_ORDER_INDEX_HINT)}};
 
 #endif /* LEX_INCLUDED */
