@@ -1,16 +1,15 @@
-/* Copyright (c) 2005, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2005, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -49,19 +48,19 @@
 #include "my_dbug.h"
 #include "my_inttypes.h"
 #include "my_io.h"
+#include "my_loglevel.h"
 #include "my_psi_config.h"
 #include "my_thread_local.h"
-#include "mysql/components/services/bits/mysql_mutex_bits.h"
-#include "mysql/components/services/bits/mysql_rwlock_bits.h"
-#include "mysql/components/services/bits/psi_file_bits.h"
-#include "mysql/my_loglevel.h"
+#include "mysql/components/services/mysql_mutex_bits.h"
+#include "mysql/components/services/mysql_rwlock_bits.h"
+#include "mysql/components/services/psi_file_bits.h"
 #include "mysql/psi/mysql_mutex.h"
 #include "mysql_com.h"
 #include "sql/auth/sql_security_ctx.h"  // Security_context
 
 class THD;
 struct CHARSET_INFO;
-class Table_ref;
+struct TABLE_LIST;
 
 ////////////////////////////////////////////////////////////
 //
@@ -107,7 +106,7 @@ class Table_ref;
     Values: ON, OFF
     Log statements such as OPTIMIZE TABLE, ALTER TABLE to the slow query log.
 
-  --log-slow-replica-statements
+  --log-slow-slave-statements
     Values: ON, OFF
 
   log_throttle_queries_not_using_indexes
@@ -158,8 +157,8 @@ enum enum_log_table_type {
 */
 class Log_event_handler {
  public:
-  Log_event_handler() = default;
-  virtual ~Log_event_handler() = default;
+  Log_event_handler() {}
+  virtual ~Log_event_handler() {}
 
   /**
      Log a query to the slow log.
@@ -175,6 +174,8 @@ class Log_event_handler {
                                is a query or an administrator command
      @param sql_text           The query or administrator in textual form
      @param sql_text_len       The length of sql_text string
+     @param query_start_status Pointer to a snapshot of thd->status_var taken
+                               at the start of execution
 
      @return true if error, false otherwise.
   */
@@ -182,7 +183,8 @@ class Log_event_handler {
                         ulonglong query_start_arg, const char *user_host,
                         size_t user_host_len, ulonglong query_utime,
                         ulonglong lock_utime, bool is_command,
-                        const char *sql_text, size_t sql_text_len) = 0;
+                        const char *sql_text, size_t sql_text_len,
+                        struct System_status_var *query_start_status) = 0;
 
   /**
      Log command to the general log.
@@ -224,17 +226,19 @@ class Log_event_handler {
 class Log_to_csv_event_handler : public Log_event_handler {
  public:
   /** @see Log_event_handler::log_slow(). */
-  bool log_slow(THD *thd, ulonglong current_utime, ulonglong query_start_arg,
-                const char *user_host, size_t user_host_len,
-                ulonglong query_utime, ulonglong lock_utime, bool is_command,
-                const char *sql_text, size_t sql_text_len) override;
+  virtual bool log_slow(THD *thd, ulonglong current_utime,
+                        ulonglong query_start_arg, const char *user_host,
+                        size_t user_host_len, ulonglong query_utime,
+                        ulonglong lock_utime, bool is_command,
+                        const char *sql_text, size_t sql_text_len,
+                        struct System_status_var *query_start_status);
 
   /** @see Log_event_handler::log_general(). */
-  bool log_general(THD *thd, ulonglong event_utime, const char *user_host,
-                   size_t user_host_len, my_thread_id thread_id,
-                   const char *command_type, size_t command_type_len,
-                   const char *sql_text, size_t sql_text_len,
-                   const CHARSET_INFO *client_cs) override;
+  virtual bool log_general(THD *thd, ulonglong event_utime,
+                           const char *user_host, size_t user_host_len,
+                           my_thread_id thread_id, const char *command_type,
+                           size_t command_type_len, const char *sql_text,
+                           size_t sql_text_len, const CHARSET_INFO *client_cs);
 
  private:
   /**
@@ -329,16 +333,13 @@ class Query_logger {
      @param thd                 THD of the statement being logged.
      @param query               The query string being logged.
      @param query_length        The length of the query string.
-     @param aggregate           True if writing log throttle record
-     @param lock_usec           Lock time, in microseconds.
-                                Only used when aggregate is true.
-     @param exec_usec           Execution time, in microseconds.
-                                Only used when aggregate is true.
+     @param query_start_status  Pointer to a snapshot of thd->status_var taken
+                                at the start of execution
 
      @return true if error, false otherwise.
   */
   bool slow_log_write(THD *thd, const char *query, size_t query_length,
-                      bool aggregate, ulonglong lock_usec, ulonglong exec_usec);
+                      struct System_status_var *query_start_status);
 
   /**
      Write printf style message to general query log.
@@ -412,16 +413,16 @@ class Query_logger {
   bool set_log_file(enum_log_table_type log_type);
 
   /**
-     Check if given Table_ref has a query log table name and
+     Check if given TABLE_LIST has a query log table name and
      optionally check if the query log is currently enabled.
 
-     @param table_list       Table_ref representing the table to check
+     @param table_list       TABLE_LIST representing the table to check
      @param check_if_opened  Always return QUERY_LOG_NONE unless the
                              query log table is enabled.
 
      @retval QUERY_LOG_NONE, QUERY_LOG_SLOW or QUERY_LOG_GENERAL
   */
-  enum_log_table_type check_if_log_table(Table_ref *table_list,
+  enum_log_table_type check_if_log_table(TABLE_LIST *table_list,
                                          bool check_if_opened) const;
 };
 
@@ -468,8 +469,10 @@ bool log_slow_applicable(THD *thd);
   exists) to the slow query log.
 
   @param thd                 thread handle
+  @param query_start_status  Pointer to a snapshot of thd->status_var taken
+                             at the start of execution
 */
-void log_slow_do(THD *thd);
+void log_slow_do(THD *thd, struct System_status_var *query_start_status);
 
 /**
   Check whether we need to write the current statement to the slow query
@@ -482,8 +485,10 @@ void log_slow_do(THD *thd);
   statement.
 
   @param thd                 thread handle
+  @param query_start_status  Pointer to a snapshot of thd->status_var taken
+                             at the start of execution
 */
-void log_slow_statement(THD *thd);
+void log_slow_statement(THD *thd, struct System_status_var *query_start_status);
 
 /**
   @class Log_throttle
@@ -527,7 +532,7 @@ class Log_throttle {
 
     @param rate  Limit on records to be logged during the throttling window.
 
-    @retval true -  log rate limit is exceeded, so record should be suppressed.
+    @retval true -  log rate limit is exceeded, so record should be supressed.
     @retval false - log rate limit is not exceeded, record should be logged.
   */
   bool inc_log_count(ulong rate) { return (++count > rate); }
@@ -569,10 +574,6 @@ class Log_throttle {
   static const ulong LOG_THROTTLE_WINDOW_SIZE = 60000000;
 };
 
-typedef bool (*log_summary_t)(THD *thd, const char *query, size_t query_length,
-                              bool aggregate, ulonglong lock_usec,
-                              ulonglong exec_usec);
-
 /**
   @class Slow_log_throttle
   @brief Used for rate-limiting the slow query log.
@@ -607,9 +608,10 @@ class Slow_log_throttle : public Log_throttle {
   ulong *rate;
 
   /**
-    The routine we call to actually log a line (our summary).
+    The routine we call to actually log a line (i.e. our summary).
+    The signature miraculously coincides with slow_log_print().
   */
-  log_summary_t log_summary;
+  bool (*log_summary)(THD *, const char *, size_t, struct System_status_var *);
 
   /**
     Slow_log_throttle is shared between THDs.
@@ -636,7 +638,9 @@ class Slow_log_throttle : public Log_throttle {
     @param msg           use this template containing %lu as only non-literal
   */
   Slow_log_throttle(ulong *threshold, mysql_mutex_t *lock, ulong window_usecs,
-                    log_summary_t logger, const char *msg);
+                    bool (*logger)(THD *, const char *, size_t,
+                                   struct System_status_var *),
+                    const char *msg);
 
   /**
     Prepare and print a summary of suppressed lines to log.
@@ -648,8 +652,8 @@ class Slow_log_throttle : public Log_throttle {
     locking/unlocking.
 
     @param thd                 The THD that tries to log the statement.
-    @retval false              Logging was not suppressed, no summary needed.
-    @retval true               Logging was suppressed; a summary was printed.
+    @retval false              Logging was not supressed, no summary needed.
+    @retval true               Logging was supressed; a summary was printed.
   */
   bool flush(THD *thd);
 
@@ -657,8 +661,8 @@ class Slow_log_throttle : public Log_throttle {
     Top-level function.
     @param thd                 The THD that tries to log the statement.
     @param eligible            Is the statement of the type we might suppress?
-    @retval true               Logging should be suppressed.
-    @retval false              Logging should not be suppressed.
+    @retval true               Logging should be supressed.
+    @retval false              Logging should not be supressed.
   */
   bool log(THD *thd, bool eligible);
 };
@@ -1113,23 +1117,12 @@ log_line *log_line_init();
 
 /**
   Release a log_line allocated with log_line_init.
+
+  @retval nullptr  could not set up buffer (too small?)
+  @retval other    address of the newly initialized log_line
 */
 void log_line_exit(log_line *ll);
 
-/**
-  Get log-line's output buffer.
-  If the logger core provides this buffer, the log-service may use it
-  to assemble its output therein and implicitly return it to the core.
-  Participation is required for services that support populating
-  performance_schema.error_log, and optional for all others.
-
-  @param  ll  the log_line to examine
-
-  @retval  nullptr    success, an output buffer is available
-  @retval  otherwise  failure, no output buffer is available
-*/
-log_item *log_line_get_output_buffer(log_line *ll);
-
 /**
   Release log line item (key/value pair) with the index elem in log line ll.
   This frees whichever of key and value were dynamically allocated.
@@ -1391,19 +1384,6 @@ log_item_data *log_line_item_set(log_line *ll, log_item_type t);
 */
 const char *log_label_from_prio(int prio);
 
-/**
-  Derive the event's priority (SYSTEM_LEVEL, ERROR_LEVEL, ...)
-  from a textual label. If the label can not be identified,
-  default to ERROR_LEVEL as it is better to keep something
-  that needn't be kept than to discard something that shouldn't
-  be.
-
-  @param  label  The prio label as a \0 terminated C-string.
-
-  @retval  the priority (as an enum loglevel)
-*/
-enum loglevel log_prio_from_label(const char *label);
-
 /**
   Complete, filter, and write submitted log items.
 
@@ -1422,87 +1402,18 @@ enum loglevel log_prio_from_label(const char *label);
 */
 int log_line_submit(log_line *ll);
 
-/**
-  Whether to generate a UTC timestamp, or one following system-time.
-  These values are not arbitrary; they must correspond to the range
-  and meaning of opt_log_timestamps.
-*/
-enum enum_iso8601_tzmode {
-  iso8601_sysvar_logtimestamps = -1, /**< use value of opt_log_timestamps */
-  iso8601_utc = 0,                   /**< create UTC timestamp */
-  iso8601_system_time = 1            /**< use system time */
-};
-
 /**
   Make and return an ISO 8601 / RFC 3339 compliant timestamp.
-  Accepts the log_timestamps global variable in its third parameter.
+  Heeds log_timestamps.
 
-  @param buf       A buffer of at least iso8601_size bytes to store
-                   the timestamp in. The timestamp will be \0 terminated.
-  @param utime     Microseconds since the epoch
-  @param mode      if 0, use UTC; if 1, use local time
+  @param         buf         A buffer of at least 26 bytes to store
+                             the timestamp in (19 + tzinfo tail + \0)
+  @param         utime       Microseconds since the epoch
+  @param         mode        if 0, use UTC; if 1, use local time
 
-  @retval          length of timestamp (excluding \0)
+  @retval                    length of timestamp (excluding \0)
 */
-int make_iso8601_timestamp(char *buf, ulonglong utime,
-                           enum enum_iso8601_tzmode mode);
-
-/**
-  Parse a ISO8601 timestamp and return the number of microseconds
-  since the epoch. Heeds +/- timezone info if present.
-
-  @see make_iso8601_timestamp()
-
-  @param timestamp  an ASCII string containing an ISO8601 timestamp
-  @param len        Length in bytes of the aforementioned string
-
-  @return microseconds since the epoch
-*/
-ulonglong iso8601_timestamp_to_microseconds(const char *timestamp, size_t len);
-
-/**
-  Well-known values returned by log_error_stack().
-*/
-typedef enum enum_log_error_stack_error {
-  /// success
-  LOG_ERROR_STACK_SUCCESS = 0,
-
-  /// expected delimiter not found
-  LOG_ERROR_STACK_DELIMITER_MISSING = -1,
-
-  /// one or more services not found
-  LOG_ERROR_STACK_SERVICE_MISSING = -2,
-
-  /// couldn't create service cache entry
-  LOG_ERROR_STACK_CACHE_ENTRY_OOM = -3,
-
-  /// tried to multi-open singleton
-  LOG_ERROR_STACK_MULTITON_DENIED = -4,
-
-  /// couldn't create service instance entry
-  LOG_ERROR_STACK_SERVICE_INSTANCE_OOM = -5,
-
-  /// last element in pipeline should be a sink
-  LOG_ERROR_STACK_ENDS_IN_NON_SINK = -6,
-
-  /// service only available during start-up (may not be set by the user)
-  LOG_ERROR_STACK_SERVICE_UNAVAILABLE = -7,
-
-  /// check-only warning: no sink supporting pfs given
-  LOG_ERROR_STACK_NO_PFS_SUPPORT = -50,
-
-  /// check-only warning: no log-parser given
-  LOG_ERROR_STACK_NO_LOG_PARSER = -51,
-
-  /// check-only warning: more than one log-filter given
-  LOG_ERROR_MULTIPLE_FILTERS = -52,
-
-  /// service name may not start with a delimiter
-  LOG_ERROR_UNEXPECTED_DELIMITER_FOUND = -101,
-
-  /// delimiters ',' and ';' may not be mixed
-  LOG_ERROR_MIXED_DELIMITERS = -102
-} log_error_stack_error;
+int make_iso8601_timestamp(char *buf, ulonglong utime, int mode);
 
 /**
   Set up custom error logging stack.
@@ -1519,53 +1430,25 @@ typedef enum enum_log_error_stack_error {
                             the error occurred will be written to the
                             pointed-to size_t.
 
-  @retval  LOG_ERROR_STACK_SUCCESS               success
-
-  @retval LOG_ERROR_STACK_DELIMITER_MISSING      expected delimiter not found
-
-  @retval LOG_ERROR_STACK_SERVICE_MISSING        one or more services not found
-
-  @retval LOG_ERROR_STACK_CACHE_ENTRY_OOM        couldn't create service cache
-                                                 entry
-
-  @retval LOG_ERROR_STACK_MULTITON_DENIED        tried to multi-open singleton
-
-  @retval LOG_ERROR_STACK_SERVICE_INSTANCE_OOM   couldn't create service
-                                                 instance entry
-
-  @retval LOG_ERROR_STACK_ENDS_IN_NON_SINK       last element should be a sink
-
-  @retval LOG_ERROR_STACK_SERVICE_UNAVAILABLE    service only available during
-                                                 start-up (may not be set by the
-                                                 user)
-
-
-  @retval  LOG_ERROR_STACK_NO_PFS_SUPPORT        (check_only warning)
-                                                 no sink with performance_schema
-                                                 support selected
-
-  @retval  LOG_ERROR_STACK_NO_LOG_PARSER         (check_only warning)
-                                                 no sink providing a log-parser
-                                                 selected
-
-  @retval  LOG_ERROR_MULTIPLE_FILTERS            (check_only warning)
-                                                 more than one filter service
-                                                 selected
-
-  @retval  LOG_ERROR_UNEXPECTED_DELIMITER_FOUND  service starts with a delimiter
-
-  @retval  LOG_ERROR_MIXED_DELIMITERS            use ',' or ';', not both!
+  @retval              0    success
+  @retval             -1    expected delimiter not found
+  @retval             -2    one or more services not found
+  @retval             -3    failed to create service cache entry
+  @retval             -4    tried to open multiple instances of a singleton
+  @retval             -5    failed to create service instance entry
+  @retval             -6    last element in pipeline should be a sink
+  @retval             -101  service name may not start with a delimiter
+  @retval             -102  delimiters ',' and ';' may not be mixed
 */
-log_error_stack_error log_builtins_error_stack(const char *conf,
-                                               bool check_only, size_t *pos);
+int log_builtins_error_stack(const char *conf, bool check_only, size_t *pos);
 
 /**
   Call flush() on all log_services.
   flush() function must not try to log anything, as we hold an
   exclusive lock on the stack.
 
-  @returns 0 if no problems occurred, otherwise the negative count
-             of the components that failed to flush
+  @retval   0   no problems
+  @retval  -1   error
 */
 int log_builtins_error_stack_flush();
 
@@ -1582,7 +1465,8 @@ int log_builtins_error_stack_flush();
   @retval -2  couldn't initialize built-in default filter
   @retval -3  couldn't set up service hash
   @retval -4  couldn't initialize syseventlog lock
-  @retval -5  couldn't initialize buffered logging lock
+  @retval -5  couldn't set service pipeline
+  @retval -6  couldn't initialize buffered logging lock
 */
 int log_builtins_init();
 
