@@ -1,16 +1,15 @@
-/* Copyright (c) 2011, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2011, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,10 +23,13 @@
 #ifndef OPT_TRACE_INCLUDED
 #define OPT_TRACE_INCLUDED
 
+#include "my_config.h"
+
 #include <limits.h>
 #include <string.h>
 #include <sys/types.h>
 
+#include "m_ctype.h"
 #include "my_compiler.h"
 #include "my_inttypes.h"
 #include "my_sqlcommand.h"          // enum_sql_command
@@ -39,8 +41,7 @@ class THD;
 class set_var_base;
 class sp_head;
 class sp_printable;
-struct CHARSET_INFO;
-class Table_ref;
+struct TABLE_LIST;
 template <class T>
 class List;
 
@@ -460,12 +461,7 @@ class Opt_trace_struct {
   */
   Opt_trace_struct(Opt_trace_context *ctx_arg, bool requires_key_arg,
                    const char *key, Opt_trace_context::feature_value feature)
-      : started(false),
-        requires_key(false),
-        has_disabled_I_S(false),
-        empty(false),
-        stmt(nullptr),
-        saved_key(nullptr) {
+      : started(false) {
     // A first inlined test
     if (unlikely(ctx_arg->is_started())) {
       // Tracing enabled: must fully initialize the structure.
@@ -577,14 +573,16 @@ class Opt_trace_struct {
      @param  item   the Item
      @return a reference to the structure
   */
-  Opt_trace_struct &add(const char *key, const Item *item) {
+  Opt_trace_struct &add(const char *key, Item *item) {
     if (likely(!started)) return *this;
     return do_add(key, item);
   }
-  Opt_trace_struct &add(const Item *item) {
+  Opt_trace_struct &add(Item *item) {
     if (likely(!started)) return *this;
     return do_add(nullptr, item);
   }
+
+ public:
   Opt_trace_struct &add(const char *key, bool value) {
     if (likely(!started)) return *this;
     return do_add(key, value);
@@ -641,6 +639,15 @@ class Opt_trace_struct {
     if (likely(!started)) return *this;
     return do_add(nullptr, value);
   }
+  /// Adds a 64-bit integer to trace, in hexadecimal format
+  Opt_trace_struct &add_hex(const char *key, uint64 value) {
+    if (likely(!started)) return *this;
+    return do_add_hex(key, value);
+  }
+  Opt_trace_struct &add_hex(uint64 value) {
+    if (likely(!started)) return *this;
+    return do_add_hex(nullptr, value);
+  }
   /// Adds a JSON null object (==Python's "None")
   Opt_trace_struct &add_null(const char *key) {
     if (likely(!started)) return *this;
@@ -650,16 +657,19 @@ class Opt_trace_struct {
      Helper to put the database/table name in an object.
      @param  tab  TABLE* pointer
   */
-  Opt_trace_struct &add_utf8_table(const Table_ref *tab) {
+  Opt_trace_struct &add_utf8_table(const TABLE_LIST *tab) {
     if (likely(!started)) return *this;
     return do_add_utf8_table(tab);
   }
   /**
-     Helper to put the number of query_block in an object.
-     @param  select_number  number of query_block
+     Helper to put the number of select_lex in an object.
+     @param  select_number  number of select_lex
   */
   Opt_trace_struct &add_select_number(uint select_number) {
-    return add("select#", select_number);
+    return unlikely(select_number >= INT_MAX) ?
+                                              // Clearer than any huge number.
+               add_alnum("select#", "fake")
+                                              : add("select#", select_number);
   }
   /**
      Add a value to the structure.
@@ -746,13 +756,14 @@ class Opt_trace_struct {
   */
   Opt_trace_struct &do_add(const char *key, const char *value,
                            size_t val_length, bool escape);
-  Opt_trace_struct &do_add(const char *key, const Item *item);
+  Opt_trace_struct &do_add(const char *key, Item *item);
   Opt_trace_struct &do_add(const char *key, bool value);
   Opt_trace_struct &do_add(const char *key, longlong value);
   Opt_trace_struct &do_add(const char *key, ulonglong value);
   Opt_trace_struct &do_add(const char *key, double value);
+  Opt_trace_struct &do_add_hex(const char *key, uint64 value);
   Opt_trace_struct &do_add_null(const char *key);
-  Opt_trace_struct &do_add_utf8_table(const Table_ref *tab);
+  Opt_trace_struct &do_add_utf8_table(const TABLE_LIST *tab);
   Opt_trace_struct &do_add(const char *key, const Cost_estimate &value);
 
   Opt_trace_struct(const Opt_trace_struct &);             ///< not defined
@@ -784,7 +795,7 @@ class Opt_trace_struct {
   Opt_trace_stmt *stmt;  ///< Trace owning the structure
   /// Key if the structure is the value of a key/value pair, NULL otherwise
   const char *saved_key;
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   /**
      Fixed-length prefix of previous key in this structure, if this structure
      is an object. Serves to detect when adding two same consecutive keys to
@@ -946,7 +957,7 @@ class Opt_trace_start {
     @note Each sub-statement is responsible for ending the trace which it
     has started; this class achieves this by keeping some memory inside.
   */
-  Opt_trace_start(THD *thd_arg, Table_ref *tbl,
+  Opt_trace_start(THD *thd_arg, TABLE_LIST *tbl,
                   enum enum_sql_command sql_command,
                   List<set_var_base> *set_vars, const char *query,
                   size_t query_length, sp_printable *instr,
@@ -958,24 +969,24 @@ class Opt_trace_start {
   bool error;  ///< whether trace start() had an error
 };
 
-class Query_block;
+class SELECT_LEX;
 
 /**
    Prints SELECT query to optimizer trace. It is not the original query (as in
    @c Opt_trace_context::set_query()) but a printout of the parse tree
    (Item-s).
    @param  thd         the THD
-   @param  query_block  query's parse tree
+   @param  select_lex  query's parse tree
    @param  trace_object  Opt_trace_object to which the query will be added
 */
-void opt_trace_print_expanded_query(const THD *thd, Query_block *query_block,
+void opt_trace_print_expanded_query(const THD *thd, SELECT_LEX *select_lex,
                                     Opt_trace_object *trace_object);
 
 /**
    If the security context is not that of the connected user, inform the trace
    system that a privilege is missing. With one exception: see below.
 
-   @param  thd         the THD
+   @param thd
 
    This serves to eliminate the following issue.
    Any information readable by a SELECT may theoretically end up in
@@ -1023,8 +1034,8 @@ void opt_trace_disable_if_no_security_context_access(THD *thd);
    @param view              view to check
    @param underlying_tables underlying tables/views of 'view'
  */
-void opt_trace_disable_if_no_view_access(THD *thd, Table_ref *view,
-                                         Table_ref *underlying_tables);
+void opt_trace_disable_if_no_view_access(THD *thd, TABLE_LIST *view,
+                                         TABLE_LIST *underlying_tables);
 
 /**
   If tracing is on, checks additional privileges on a stored routine, to make
@@ -1039,7 +1050,7 @@ void opt_trace_disable_if_no_view_access(THD *thd, Table_ref *view,
 
   For triggers, see note in sp_head::execute_trigger().
 
-  @param thd The THD
+  @param thd
   @param sp  routine to check
  */
 void opt_trace_disable_if_no_stored_proc_func_access(THD *thd, sp_head *sp);
@@ -1049,7 +1060,7 @@ void opt_trace_disable_if_no_stored_proc_func_access(THD *thd, sp_head *sp);
    @retval 0 ok
    @retval 1 error
 */
-int fill_optimizer_trace_info(THD *thd, Table_ref *tables, Item *);
+int fill_optimizer_trace_info(THD *thd, TABLE_LIST *tables, Item *);
 
 //@}
 
@@ -1067,13 +1078,13 @@ int fill_optimizer_trace_info(THD *thd, Table_ref *tables, Item *);
    @param trace          optimizer trace
    @param object_level0  name of the outer Opt_trace_object C++ object
    @param object_level1  name of the inner Opt_trace_object C++ object
-   @param select_number  number of the being-transformed Query_block
+   @param select_number  number of the being-transformed SELECT_LEX
    @param from           description of the before-transformation state
    @param to             description of the after-transformation state
 */
 #define OPT_TRACE_TRANSFORM(trace, object_level0, object_level1, \
                             select_number, from, to)             \
-  const Opt_trace_object object_level0(trace);                   \
+  Opt_trace_object object_level0(trace);                         \
   Opt_trace_object object_level1(trace, "transformation");       \
   object_level1.add_select_number(select_number);                \
   object_level1.add_alnum("from", from).add_alnum("to", to);
