@@ -1,4 +1,4 @@
-/* Copyright (c) 2016, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -23,47 +23,26 @@
 #ifndef PARSE_TREE_COL_ATTRS_INCLUDED
 #define PARSE_TREE_COL_ATTRS_INCLUDED
 
-#include <assert.h>
-#include <sys/types.h>  // ulong, uint. TODO: replace with cstdint
-
-#include <optional>
 #include <type_traits>
-#include <vector>
 
-#include "field_types.h"
-#include "lex_string.h"
-#include "m_ctype.h"
-#include "my_alloc.h"
-#include "my_base.h"
-#include "my_compiler.h"
-
-#include "my_inttypes.h"
-#include "my_sys.h"
+#include "my_dbug.h"
+#include "mysql/mysql_lex_string.h"
 #include "mysql_com.h"
-#include "mysqld_error.h"
-#include "sql/derror.h"
-#include "sql/field.h"
+#include "nullable.h"
 #include "sql/gis/srid.h"
-#include "sql/item.h"
 #include "sql/item_timefunc.h"
-#include "sql/mem_root_array.h"
-#include "sql/parse_location.h"
-#include "sql/parse_tree_helpers.h"  // move_cf_appliers
 #include "sql/parse_tree_node_base.h"
-#include "sql/parser_yystype.h"
 #include "sql/sql_alter.h"
 #include "sql/sql_check_constraint.h"  // Sql_check_constraint_spec
 #include "sql/sql_class.h"
 #include "sql/sql_error.h"
 #include "sql/sql_lex.h"
-#include "sql/sql_list.h"
 #include "sql/sql_parse.h"
-#include "sql/system_variables.h"
 
-class String;
+using Mysql::Nullable;
 
 /**
-  Parse context for column type attribute specific parse tree nodes.
+  Parse context for column type attribyte specific parse tree nodes.
 
   For internal use in the contextualization code.
 
@@ -71,8 +50,8 @@ class String;
 */
 struct Column_parse_context : public Parse_context {
   const bool is_generated;  ///< Owner column is a generated one.
-  std::vector<CreateFieldApplier> cf_appliers;
-  Column_parse_context(THD *thd_arg, Query_block *select_arg, bool is_generated)
+
+  Column_parse_context(THD *thd_arg, SELECT_LEX *select_arg, bool is_generated)
       : Parse_context(thd_arg, select_arg), is_generated(is_generated) {}
 };
 
@@ -83,7 +62,7 @@ struct Column_parse_context : public Parse_context {
 */
 class PT_column_attr_base : public Parse_tree_node_tmpl<Column_parse_context> {
  protected:
-  PT_column_attr_base() = default;
+  PT_column_attr_base() {}
 
  public:
   typedef decltype(Alter_info::flags) alter_info_flags_t;
@@ -94,15 +73,14 @@ class PT_column_attr_base : public Parse_tree_node_tmpl<Column_parse_context> {
   virtual void apply_default_value(Item **) const {}
   virtual void apply_gen_default_value(Value_generator **) {}
   virtual void apply_on_update_value(Item **) const {}
-  virtual void apply_srid_modifier(std::optional<gis::srid_t> *) const {}
-  virtual bool apply_collation(Column_parse_context *,
-                               const CHARSET_INFO **to [[maybe_unused]],
-                               bool *has_explicit_collation
-                               [[maybe_unused]]) const {
+  virtual void apply_srid_modifier(Nullable<gis::srid_t> *) const {}
+  virtual bool apply_collation(
+      Column_parse_context *, const CHARSET_INFO **to MY_ATTRIBUTE((unused)),
+      bool *has_explicit_collation MY_ATTRIBUTE((unused))) const {
     return false;
   }
   virtual bool add_check_constraints(
-      Sql_check_constraint_spec_list *check_const_list [[maybe_unused]]) {
+      Sql_check_constraint_spec_list *check_const_list MY_ATTRIBUTE((unused))) {
     return false;
   }
 
@@ -132,7 +110,8 @@ class PT_column_attr_base : public Parse_tree_node_tmpl<Column_parse_context> {
     @returns false if success, true if error (e.g. if [NOT] ENFORCED follows
              something other than the CHECK clause.)
   */
-  virtual bool set_constraint_enforcement(bool enforced [[maybe_unused]]) {
+  virtual bool set_constraint_enforcement(
+      bool enforced MY_ATTRIBUTE((unused))) {
     return true;  // error
   }
 };
@@ -231,7 +210,7 @@ class PT_check_constraint_column_attr : public PT_column_attr_base {
 
   bool add_check_constraints(
       Sql_check_constraint_spec_list *check_const_list) override {
-    assert(check_const_list != nullptr);
+    DBUG_ASSERT(check_const_list != nullptr);
     return (check_const_list->push_back(&col_cc_spec));
   }
 
@@ -283,7 +262,7 @@ class PT_collate_column_attr : public PT_column_attr_base {
  public:
   explicit PT_collate_column_attr(const POS &pos, const CHARSET_INFO *collation)
       : m_pos(pos), m_collation(collation) {
-    assert(m_collation != nullptr);
+    DBUG_ASSERT(m_collation != nullptr);
   }
 
   bool apply_collation(Column_parse_context *pc, const CHARSET_INFO **to,
@@ -465,7 +444,7 @@ class PT_srid_column_attr : public PT_column_attr_base {
  public:
   explicit PT_srid_column_attr(gis::srid_t srid) : m_srid(srid) {}
 
-  void apply_srid_modifier(std::optional<gis::srid_t> *srid) const override {
+  void apply_srid_modifier(Nullable<gis::srid_t> *srid) const override {
     *srid = m_srid;
   }
 };
@@ -502,24 +481,6 @@ class PT_generated_default_val_column_attr : public PT_column_attr_base {
   Value_generator m_default_value_expression;
 };
 
-/**
-  Node for the @SQL{VISIBLE|INVISIBLE} column attribute
-
-  @ingroup ptn_column_attrs
-*/
-class PT_column_visibility_attr : public PT_column_attr_base {
- public:
-  explicit PT_column_visibility_attr(bool is_visible)
-      : m_is_visible(is_visible) {}
-  void apply_type_flags(unsigned long *type_flags) const override {
-    *type_flags &= ~FIELD_IS_INVISIBLE;
-    if (!m_is_visible) *type_flags |= FIELD_IS_INVISIBLE;
-  }
-
- private:
-  const bool m_is_visible;
-};
-
 // Type nodes:
 
 /**
@@ -541,7 +502,6 @@ class PT_type : public Parse_tree_node {
   virtual const CHARSET_INFO *get_charset() const { return nullptr; }
   virtual uint get_uint_geom_type() const { return 0; }
   virtual List<String> *get_interval_list() const { return nullptr; }
-  virtual bool is_serial_type() const { return false; }
 };
 
 /**
@@ -568,7 +528,7 @@ class PT_numeric_type : public PT_type {
         length(length),
         dec(dec),
         options(options) {
-    assert((options & ~(UNSIGNED_FLAG | ZEROFILL_FLAG)) == 0);
+    DBUG_ASSERT((options & ~(UNSIGNED_FLAG | ZEROFILL_FLAG)) == 0);
 
     if (type_arg != Numeric_type::DECIMAL && dec != nullptr) {
       push_warning(thd, Sql_condition::SL_WARNING,
@@ -587,7 +547,7 @@ class PT_numeric_type : public PT_type {
         length(length),
         dec(nullptr),
         options(options) {
-    assert((options & ~(UNSIGNED_FLAG | ZEROFILL_FLAG)) == 0);
+    DBUG_ASSERT((options & ~(UNSIGNED_FLAG | ZEROFILL_FLAG)) == 0);
 
     if (length != nullptr) {
       push_warning(thd, Sql_condition::SL_WARNING,
@@ -650,7 +610,7 @@ class PT_char_type : public PT_type {
         length(length),
         charset(charset),
         force_binary(force_binary) {
-    assert(charset == nullptr || !force_binary);
+    DBUG_ASSERT(charset == nullptr || !force_binary);
   }
   PT_char_type(Char_type char_type, const CHARSET_INFO *charset,
                bool force_binary = false)
@@ -690,7 +650,7 @@ class PT_blob_type : public PT_type {
         length(nullptr),
         charset(charset),
         force_binary(force_binary) {
-    assert(charset == nullptr || !force_binary);
+    DBUG_ASSERT(charset == nullptr || !force_binary);
   }
   explicit PT_blob_type(const char *length)
       : PT_type(MYSQL_TYPE_BLOB),
@@ -822,7 +782,7 @@ class PT_enum_type_tmpl : public PT_type {
         interval_list(interval_list),
         charset(charset),
         force_binary(force_binary) {
-    assert(charset == nullptr || !force_binary);
+    DBUG_ASSERT(charset == nullptr || !force_binary);
   }
 
   const CHARSET_INFO *get_charset() const override { return charset; }
@@ -853,7 +813,6 @@ class PT_serial_type : public PT_type {
   ulong get_type_flags() const override {
     return AUTO_INCREMENT_FLAG | NOT_NULL_FLAG | UNSIGNED_FLAG | UNIQUE_FLAG;
   }
-  bool is_serial_type() const override { return true; }
 };
 
 /**
@@ -892,7 +851,7 @@ class PT_field_def_base : public Parse_tree_node {
   Value_generator *gcol_info;
   /// Holds the expression to generate default values
   Value_generator *default_val_info;
-  std::optional<gis::srid_t> m_srid;
+  Nullable<gis::srid_t> m_srid;
   // List of column check constraint's specification.
   Sql_check_constraint_spec_list *check_const_spec_list{nullptr};
 
@@ -966,11 +925,7 @@ class PT_field_def : public PT_field_def_base {
 
   bool contextualize(Parse_context *pc_arg) override {
     Column_parse_context pc(pc_arg->thd, pc_arg->select, false);
-    if (super::contextualize(&pc) || contextualize_attrs(&pc, opt_attrs))
-      return true;
-
-    move_cf_appliers(pc_arg, &pc);
-    return false;
+    return super::contextualize(&pc) || contextualize_attrs(&pc, opt_attrs);
   }
 };
 
@@ -1001,12 +956,6 @@ class PT_generated_field_def : public PT_field_def_base {
         expr->itemize(&pc, &expr))
       return true;
 
-    // column of type serial cannot be generated
-    if (type_node->is_serial_type()) {
-      my_error(ER_WRONG_USAGE, MYF(0), "SERIAL", "generated column");
-      return true;
-    }
-
     gcol_info = new (pc.mem_root) Value_generator;
     if (gcol_info == nullptr) return true;  // OOM
     gcol_info->expr_item = expr;
@@ -1018,6 +967,4 @@ class PT_generated_field_def : public PT_field_def_base {
   }
 };
 
-void move_cf_appliers(Parse_context *tddlpc, Column_parse_context *cpc);
-
 #endif /* PARSE_TREE_COL_ATTRS_INCLUDED */
