@@ -1,4 +1,4 @@
-/* Copyright (c) 2013, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2013, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -22,7 +22,7 @@
 
 #include "sql/parse_tree_items.h"
 
-#include <sys/types.h>  // TODO: replace with cstdint
+#include <sys/types.h>
 
 #include "m_string.h"
 #include "my_dbug.h"
@@ -35,7 +35,6 @@
 #include "sql/auth/auth_acls.h"
 #include "sql/item_cmpfunc.h"  // Item_func_eq
 #include "sql/item_create.h"
-#include "sql/item_subselect.h"
 #include "sql/mysqld.h"  // using_udf_functions
 #include "sql/parse_tree_nodes.h"
 #include "sql/protocol.h"
@@ -75,7 +74,7 @@ static Item *change_truth_value_of_condition(Parse_context *pc, Item *expr,
     case Item::BOOL_NOT_FALSE:
       break;
     default:
-      assert(false);
+      DBUG_ASSERT(false);
   }
   // Ensure that all incomplete predicates are made complete:
   if (!expr->is_bool_func()) {
@@ -115,7 +114,7 @@ static Item *handle_sql2003_note184_exception(Parse_context *pc, Item *left,
     should be re-interpreted as an Item_in_subselect, which corresponds
     to a <table subquery> when used inside an <in predicate>.
 
-    Our reading of Note 184 is recursive, so that all:
+    Our reading of Note 184 is reccursive, so that all:
     - IN (( <subquery> ))
     - IN ((( <subquery> )))
     - IN '('^N <subquery> ')'^N
@@ -133,7 +132,7 @@ static Item *handle_sql2003_note184_exception(Parse_context *pc, Item *left,
 
     if (expr2->substype() == Item_subselect::SINGLEROW_SUBS) {
       Item_singlerow_subselect *expr3 = (Item_singlerow_subselect *)expr2;
-      Query_block *subselect;
+      SELECT_LEX *subselect;
 
       /*
         Implement the mandated change, by altering the semantic tree:
@@ -143,7 +142,7 @@ static Item *handle_sql2003_note184_exception(Parse_context *pc, Item *left,
         which is represented as
           Item_in_subselect(left, subselect)
       */
-      subselect = expr3->invalidate_and_restore_query_block();
+      subselect = expr3->invalidate_and_restore_select_lex();
       result = new (pc->mem_root) Item_in_subselect(left, subselect);
 
       if (!equal)
@@ -162,6 +161,18 @@ static Item *handle_sql2003_note184_exception(Parse_context *pc, Item *left,
   return result;
 }
 
+bool PTI_table_wild::itemize(Parse_context *pc, Item **item) {
+  if (super::itemize(pc, item)) return true;
+
+  schema = pc->thd->get_protocol()->has_client_capability(CLIENT_NO_SCHEMA)
+               ? nullptr
+               : schema;
+  *item = new (pc->mem_root) Item_field(POS(), schema, table, "*");
+  if (*item == nullptr || (*item)->itemize(pc, item)) return true;
+  pc->select->with_wild++;
+  return false;
+}
+
 bool PTI_comp_op::itemize(Parse_context *pc, Item **res) {
   if (super::itemize(pc, res) || left->itemize(pc, &left) ||
       right->itemize(pc, &right))
@@ -264,7 +275,7 @@ bool PTI_function_call_generic_ident_sys::itemize(Parse_context *pc,
       *res = Create_udf_func::s_singleton.create(thd, udf, opt_udf_expr_list);
     } else {
       builder = find_qualified_function_builder(thd);
-      assert(builder);
+      DBUG_ASSERT(builder);
       *res = builder->create_func(thd, ident, opt_udf_expr_list);
     }
   }
@@ -294,7 +305,7 @@ bool PTI_function_call_generic_2d::itemize(Parse_context *pc, Item **res) {
   if (sp_check_name(&func)) return true;
 
   Create_qfunc *builder = find_qualified_function_builder(pc->thd);
-  assert(builder);
+  DBUG_ASSERT(builder);
   *res = builder->create(pc->thd, db, func, true, opt_expr_list);
   return *res == nullptr || (*res)->itemize(pc, res);
 }
@@ -303,7 +314,7 @@ bool PTI_text_literal_nchar_string::itemize(Parse_context *pc, Item **res) {
   if (super::itemize(pc, res)) return true;
 
   uint repertoire = is_7bit ? MY_REPERTOIRE_ASCII : MY_REPERTOIRE_UNICODE30;
-  assert(my_charset_is_ascii_based(national_charset_info));
+  DBUG_ASSERT(my_charset_is_ascii_based(national_charset_info));
   init(literal.str, literal.length, national_charset_info, DERIVATION_COERCIBLE,
        repertoire);
   return false;
@@ -360,7 +371,7 @@ bool PTI_simple_ident_ident::itemize(Parse_context *pc, Item **res) {
   if (pctx && (spv = pctx->find_variable(ident.str, ident.length, false))) {
     sp_head *sp = lex->sphead;
 
-    assert(sp);
+    DBUG_ASSERT(sp);
 
     /* We're compiling a stored procedure and found a variable */
     if (!lex->parsing_options.allows_variable) {
@@ -393,7 +404,6 @@ bool PTI_simple_ident_q_3d::itemize(Parse_context *pc, Item **res) {
                                                                    : db;
   if (pc->select->no_table_names_allowed) {
     my_error(ER_TABLENAME_NOT_ALLOWED_HERE, MYF(0), table, thd->where);
-    return true;
   }
   if ((pc->select->parsing_place != CTX_HAVING) ||
       (pc->select->get_in_sum_expr() > 0)) {
@@ -411,9 +421,9 @@ bool PTI_simple_ident_q_2d::itemize(Parse_context *pc, Item **res) {
   sp_head *sp = lex->sphead;
 
   /*
-    References with OLD and NEW designators can be used in expressions in
-    triggers. Semantic checks must ensure they are not used in invalid
-    contexts, such as assignment targets.
+    FIXME This will work ok in simple_ident_nospvar case because
+    we can't meet simple_ident_nospvar in trigger now. But it
+    should be changed in future.
   */
   if (sp && sp->m_type == enum_sp_type::TRIGGER &&
       (!my_strcasecmp(system_charset_info, table, "NEW") ||
@@ -432,8 +442,8 @@ bool PTI_simple_ident_q_2d::itemize(Parse_context *pc, Item **res) {
       return true;
     }
 
-    assert(!new_row || (sp->m_trg_chistics.event == TRG_EVENT_INSERT ||
-                        sp->m_trg_chistics.event == TRG_EVENT_UPDATE));
+    DBUG_ASSERT(!new_row || (sp->m_trg_chistics.event == TRG_EVENT_INSERT ||
+                             sp->m_trg_chistics.event == TRG_EVENT_UPDATE));
     const bool read_only =
         !(new_row && sp->m_trg_chistics.action_time == TRG_ACTION_BEFORE);
     Item_trigger_field *trg_fld = new (pc->mem_root)
@@ -441,7 +451,7 @@ bool PTI_simple_ident_q_2d::itemize(Parse_context *pc, Item **res) {
                            SELECT_ACL, read_only);
     if (trg_fld == nullptr || trg_fld->itemize(pc, (Item **)&trg_fld))
       return true;
-    assert(trg_fld->type() == TRIGGER_FIELD_ITEM);
+    DBUG_ASSERT(trg_fld->type() == TRIGGER_FIELD_ITEM);
 
     /*
       Let us add this item to list of all Item_trigger_field objects
@@ -509,7 +519,7 @@ bool PTI_text_literal_concat::itemize(Parse_context *pc, Item **res) {
   Item *tmp_head;
   if (super::itemize(pc, res) || head->itemize(pc, &tmp_head)) return true;
 
-  assert(tmp_head->type() == STRING_ITEM);
+  DBUG_ASSERT(tmp_head->type() == STRING_ITEM);
   Item_string *head_str = down_cast<Item_string *>(tmp_head);
   head_str->append(literal.str, literal.length);
   if ((head_str->collation.repertoire & MY_REPERTOIRE_EXTENDED) == 0) {
@@ -542,7 +552,7 @@ bool PTI_variable_aux_set_var::itemize(Parse_context *pc, Item **res) {
   return lex->set_var_list.push_back(this);
 }
 
-bool PTI_user_variable::itemize(Parse_context *pc, Item **res) {
+bool PTI_variable_aux_ident_or_text::itemize(Parse_context *pc, Item **res) {
   if (super::itemize(pc, res)) return true;
 
   LEX *lex = pc->thd->lex;
@@ -554,7 +564,7 @@ bool PTI_user_variable::itemize(Parse_context *pc, Item **res) {
   return false;
 }
 
-bool PTI_get_system_variable::itemize(Parse_context *pc, Item **res) {
+bool PTI_variable_aux_3d::itemize(Parse_context *pc, Item **res) {
   if (super::itemize(pc, res)) return true;
 
   LEX *lex = pc->thd->lex;
@@ -563,9 +573,33 @@ bool PTI_get_system_variable::itemize(Parse_context *pc, Item **res) {
     return true;
   }
 
-  if (m_opt_prefix.str == nullptr &&
-      (is_identifier(m_name.str, "warning_count") ||
-       is_identifier(m_name.str, "error_count"))) {
+  /* disallow "SELECT @@global.global.variable" */
+  if (ident1.str && ident2.str && check_reserved_words(ident1.str)) {
+    error(pc, ident1_pos);
+    return true;
+  }
+
+  LEX_STRING *domain;
+  LEX_STRING *variable;
+  if (ident2.str && !is_key_cache_variable_suffix(ident2.str)) {
+    LEX_STRING component_var;
+    domain = &ident1;
+    variable = &ident2;
+    String tmp_name;
+    if (tmp_name.reserve(domain->length + 1 + variable->length + 1) ||
+        tmp_name.append(domain->str) || tmp_name.append('.') ||
+        tmp_name.append(variable->str))
+      return true;  // OOM
+    component_var.str = tmp_name.c_ptr();
+    component_var.length = tmp_name.length();
+    variable->str = nullptr;
+    variable->length = 0;
+    *res = get_system_var(pc, var_type, component_var, *variable);
+  } else
+    *res = get_system_var(pc, var_type, ident1, ident2);
+  if (*res == nullptr) return true;
+  if (is_identifier(ident1, "warning_count") ||
+      is_identifier(ident1, "error_count")) {
     /*
       "Diagnostics variable" used in a non-diagnostics statement.
       Save the information we need for the former, but clear the
@@ -574,8 +608,10 @@ bool PTI_get_system_variable::itemize(Parse_context *pc, Item **res) {
     */
     lex->keep_diagnostics = DA_KEEP_COUNTS;
   }
-  *res = get_system_variable(pc, m_scope, m_opt_prefix, m_name, true);
-  return *res == nullptr;
+  if (!down_cast<Item_func_get_system_var *>(*res)->is_written_to_binlog())
+    lex->set_stmt_unsafe(LEX::BINLOG_STMT_UNSAFE_SYSTEM_VARIABLE);
+
+  return false;
 }
 
 bool PTI_count_sym::itemize(Parse_context *pc, Item **res) {
@@ -630,34 +666,27 @@ bool PTI_odbc_date::itemize(Parse_context *pc, Item **res) {
   return false;
 }
 
-bool PTI_int_splocal::itemize(Parse_context *pc, Item **res) {
-  if (super::itemize(pc, res)) return true;  // OOM
+bool PTI_limit_option_ident::itemize(Parse_context *pc, Item **res) {
+  if (super::itemize(pc, res)) return true;
 
-  LEX *const lex = pc->thd->lex;
-  sp_head *const sp = lex->sphead;
+  LEX *lex = pc->thd->lex;
+  sp_head *sp = lex->sphead;
   const char *query_start_ptr =
       sp ? sp->m_parser_data.get_current_stmt_start_ptr() : nullptr;
 
-  Item_splocal *v =
-      create_item_for_sp_var(pc->thd, m_name, nullptr, query_start_ptr,
-                             m_location.raw.start, m_location.raw.end);
-  if (v == nullptr) {
-    return true;  // undefined variable or OOM
-  }
+  Item_splocal *v = create_item_for_sp_var(
+      pc->thd, ident, nullptr, query_start_ptr, ident_loc.start, ident_loc.end);
+  if (v == nullptr) return true;
+
+  lex->safe_to_cache_query = false;
+
   if (v->type() != Item::INT_ITEM) {
-    pc->thd->syntax_error_at(m_location, ER_SPVAR_NONINTEGER_TYPE, m_name.str);
+    my_error(ER_WRONG_SPVAR_TYPE_IN_LIMIT, MYF(0));
     return true;
   }
 
-  lex->safe_to_cache_query = false;
-  *res = v;
-  return false;
-}
-
-bool PTI_limit_option_ident::itemize(Parse_context *pc, Item **res) {
-  if (super::itemize(pc, res)) return true;
-  auto *v = down_cast<Item_splocal *>(*res);
   v->limit_clause_param = true;
+  *res = v;
   return false;
 }
 
@@ -674,8 +703,9 @@ bool PTI_limit_option_param_marker::itemize(Parse_context *pc, Item **res) {
     parameter with "this" pointer of Item_param object, so we can skip
     the check and the assignment.
   */
-  assert(tmp_param == param_marker);
+  DBUG_ASSERT(tmp_param == param_marker);
 
+  param_marker->limit_clause_param = true;
   *res = param_marker;
   return false;
 }
@@ -693,9 +723,9 @@ bool PTI_context::itemize(Parse_context *pc, Item **res) {
   }
 
   // Ensure we're resetting parsing place of the right select
-  assert(pc->select->parsing_place == m_parsing_place);
+  DBUG_ASSERT(pc->select->parsing_place == m_parsing_place);
   pc->select->parsing_place = CTX_NONE;
-  assert(expr != nullptr);
+  DBUG_ASSERT(expr != nullptr);
   expr->apply_is_true();
 
   *res = expr;
