@@ -1,4 +1,4 @@
-/* Copyright (c) 2013, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2013, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -23,14 +23,16 @@
 #ifndef PARSE_TREE_ITEMS_INCLUDED
 #define PARSE_TREE_ITEMS_INCLUDED
 
+#include <stddef.h>
+
 #include "field_types.h"  // enum_field_types
 #include "lex_string.h"
-#include "my_inttypes.h"  // TODO: replace with cstdint
-#include "mysql/strings/m_ctype.h"
-#include "sql/comp_creator.h"
+#include "m_ctype.h"
+#include "my_inttypes.h"
 #include "sql/field.h"
 #include "sql/item.h"
 #include "sql/item_func.h"
+#include "sql/item_subselect.h"
 #include "sql/item_sum.h"       // Item_sum_count
 #include "sql/item_timefunc.h"  // Item_func_now_local
 #include "sql/parse_location.h"
@@ -42,6 +44,20 @@ class PT_subquery;
 class PT_window;
 struct udf_func;
 
+class PTI_table_wild : public Parse_tree_item {
+  typedef Parse_tree_item super;
+
+  const char *schema;
+  const char *table;
+
+ public:
+  explicit PTI_table_wild(const POS &pos, const char *schema_arg,
+                          const char *table_arg)
+      : super(pos), schema(schema_arg), table(table_arg) {}
+
+  bool itemize(Parse_context *pc, Item **item) override;
+};
+
 class PTI_truth_transform : public Parse_tree_item {
   typedef Parse_tree_item super;
 
@@ -52,10 +68,7 @@ class PTI_truth_transform : public Parse_tree_item {
   PTI_truth_transform(const POS &pos, Item *expr_arg, Bool_test truth_test)
       : super(pos), expr(expr_arg), truth_test(truth_test) {}
 
-  bool do_itemize(Parse_context *pc, Item **res) override;
-
- protected:
-  void add_json_info(Json_object *obj) override;
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
 class PTI_comp_op : public Parse_tree_item {
@@ -74,13 +87,7 @@ class PTI_comp_op : public Parse_tree_item {
         boolfunc2creator(boolfunc2creator_arg),
         right(right_arg) {}
 
-  bool do_itemize(Parse_context *pc, Item **res) override;
-
- protected:
-  void add_json_info(Json_object *obj) override {
-    obj->add_alias("operator", create_dom_ptr<Json_string>(
-                                   (*boolfunc2creator)(false)->symbol(false)));
-  }
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
 class PTI_comp_op_all : public Parse_tree_item {
@@ -91,13 +98,6 @@ class PTI_comp_op_all : public Parse_tree_item {
   bool is_all;
   PT_subquery *subselect;
 
- protected:
-  void add_json_info(Json_object *obj) override {
-    obj->add_alias("is_all", create_dom_ptr<Json_boolean>(is_all));
-    obj->add_alias("operator", create_dom_ptr<Json_string>(
-                                   (*comp_op)(false)->symbol(false)));
-  }
-
  public:
   PTI_comp_op_all(const POS &pos, Item *left_arg,
                   chooser_compare_func_creator comp_op_arg, bool is_all_arg,
@@ -108,7 +108,7 @@ class PTI_comp_op_all : public Parse_tree_item {
         is_all(is_all_arg),
         subselect(subselect_arg) {}
 
-  bool do_itemize(Parse_context *pc, Item **res) override;
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
 class PTI_simple_ident_ident : public Parse_tree_item {
@@ -121,7 +121,7 @@ class PTI_simple_ident_ident : public Parse_tree_item {
   PTI_simple_ident_ident(const POS &pos, const LEX_CSTRING &ident_arg)
       : super(pos), ident(ident_arg), raw(pos.raw) {}
 
-  bool do_itemize(Parse_context *pc, Item **res) override;
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
 /**
@@ -140,7 +140,7 @@ class PTI_simple_ident_q_3d : public Parse_tree_item {
                         const char *table_arg, const char *field_arg)
       : super(pos), db(db_arg), table(table_arg), field(field_arg) {}
 
-  bool do_itemize(Parse_context *pc, Item **res) override;
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
 /**
@@ -154,7 +154,7 @@ class PTI_simple_ident_q_2d : public PTI_simple_ident_q_3d {
                         const char *field_arg)
       : super(pos, nullptr, table_arg, field_arg) {}
 
-  bool do_itemize(Parse_context *pc, Item **res) override;
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
 class PTI_simple_ident_nospvar_ident : public Parse_tree_item {
@@ -166,7 +166,7 @@ class PTI_simple_ident_nospvar_ident : public Parse_tree_item {
   PTI_simple_ident_nospvar_ident(const POS &pos, const LEX_STRING &ident_arg)
       : super(pos), ident(ident_arg) {}
 
-  bool do_itemize(Parse_context *pc, Item **res) override;
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
 class PTI_function_call_nonkeyword_now final : public Item_func_now_local {
@@ -176,7 +176,7 @@ class PTI_function_call_nonkeyword_now final : public Item_func_now_local {
   PTI_function_call_nonkeyword_now(const POS &pos, uint8 dec_arg)
       : super(pos, dec_arg) {}
 
-  bool do_itemize(Parse_context *pc, Item **res) override;
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
 class PTI_function_call_nonkeyword_sysdate : public Parse_tree_item {
@@ -188,7 +188,7 @@ class PTI_function_call_nonkeyword_sysdate : public Parse_tree_item {
   explicit PTI_function_call_nonkeyword_sysdate(const POS &pos, uint8 dec_arg)
       : super(pos), dec(dec_arg) {}
 
-  bool do_itemize(Parse_context *pc, Item **res) override;
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
 class PTI_udf_expr : public Parse_tree_item {
@@ -207,7 +207,7 @@ class PTI_udf_expr : public Parse_tree_item {
         select_alias(select_alias_arg),
         expr_loc(expr_loc_arg) {}
 
-  bool do_itemize(Parse_context *pc, Item **res) override;
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
 class PTI_function_call_generic_ident_sys : public Parse_tree_item {
@@ -226,13 +226,7 @@ class PTI_function_call_generic_ident_sys : public Parse_tree_item {
         ident(ident_arg),
         opt_udf_expr_list(opt_udf_expr_list_arg) {}
 
-  bool do_itemize(Parse_context *pc, Item **res) override;
-
- protected:
-  void add_json_info(Json_object *obj) override {
-    obj->add_alias("func_name",
-                   create_dom_ptr<Json_string>(ident.str, ident.length));
-  }
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
 /**
@@ -254,10 +248,7 @@ class PTI_function_call_generic_2d : public Parse_tree_item {
         func(func_arg),
         opt_expr_list(opt_expr_list_arg) {}
 
-  bool do_itemize(Parse_context *pc, Item **res) override;
-
- protected:
-  void add_json_info(Json_object *obj) override;
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
 class PTI_text_literal : public Item_string {
@@ -280,7 +271,7 @@ class PTI_text_literal_text_string : public PTI_text_literal {
                                const LEX_STRING &literal_arg)
       : super(pos, is_7bit_arg, literal_arg) {}
 
-  bool do_itemize(Parse_context *pc, Item **res) override;
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
 class PTI_text_literal_nchar_string : public PTI_text_literal {
@@ -291,7 +282,7 @@ class PTI_text_literal_nchar_string : public PTI_text_literal {
                                 const LEX_STRING &literal_arg)
       : super(pos, is_7bit_arg, literal_arg) {}
 
-  bool do_itemize(Parse_context *pc, Item **res) override;
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
 class PTI_text_literal_underscore_charset : public PTI_text_literal {
@@ -305,8 +296,8 @@ class PTI_text_literal_underscore_charset : public PTI_text_literal {
                                       const LEX_STRING &literal_arg)
       : super(pos, is_7bit_arg, literal_arg), cs(cs_arg) {}
 
-  bool do_itemize(Parse_context *pc, Item **res) override {
-    if (super::do_itemize(pc, res)) return true;
+  bool itemize(Parse_context *pc, Item **res) override {
+    if (super::itemize(pc, res)) return true;
 
     init(literal.str, literal.length, cs, DERIVATION_COERCIBLE,
          MY_REPERTOIRE_UNICODE30);
@@ -326,7 +317,7 @@ class PTI_text_literal_concat : public PTI_text_literal {
                           PTI_text_literal *head_arg, const LEX_STRING &tail)
       : super(pos, is_7bit_arg, tail), head(head_arg) {}
 
-  bool do_itemize(Parse_context *pc, Item **res) override;
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
 class PTI_temporal_literal : public Parse_tree_item {
@@ -345,7 +336,7 @@ class PTI_temporal_literal : public Parse_tree_item {
         field_type(field_type_arg),
         cs(cs_arg) {}
 
-  bool do_itemize(Parse_context *pc, Item **res) override;
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
 class PTI_literal_underscore_charset_hex_num : public Item_string {
@@ -359,8 +350,8 @@ class PTI_literal_underscore_charset_hex_num : public Item_string {
               Item_hex_string::make_hex_str(literal.str, literal.length),
               charset) {}
 
-  bool do_itemize(Parse_context *pc, Item **res) override {
-    if (super::do_itemize(pc, res)) return true;
+  bool itemize(Parse_context *pc, Item **res) override {
+    if (super::itemize(pc, res)) return true;
 
     set_repertoire_from_value();
     set_cs_specified(true);
@@ -379,8 +370,8 @@ class PTI_literal_underscore_charset_bin_num : public Item_string {
               Item_bin_string::make_bin_str(literal.str, literal.length),
               charset) {}
 
-  bool do_itemize(Parse_context *pc, Item **res) override {
-    if (super::do_itemize(pc, res)) return true;
+  bool itemize(Parse_context *pc, Item **res) override {
+    if (super::itemize(pc, res)) return true;
 
     set_cs_specified(true);
     return check_well_formed_result(&str_value, true, true) == nullptr;
@@ -392,18 +383,19 @@ class PTI_variable_aux_set_var final : public Item_func_set_user_var {
 
  public:
   PTI_variable_aux_set_var(const POS &pos, const LEX_STRING &var, Item *expr)
-      : super(pos, var, expr) {}
+      : super(pos, var, expr, false) {}
 
-  bool do_itemize(Parse_context *pc, Item **res) override;
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
-class PTI_user_variable final : public Item_func_get_user_var {
+class PTI_variable_aux_ident_or_text final : public Item_func_get_user_var {
   typedef Item_func_get_user_var super;
 
  public:
-  PTI_user_variable(const POS &pos, const LEX_STRING &var) : super(pos, var) {}
+  PTI_variable_aux_ident_or_text(const POS &pos, const LEX_STRING &var)
+      : super(pos, var) {}
 
-  bool do_itemize(Parse_context *pc, Item **res) override;
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
 /**
@@ -411,26 +403,25 @@ class PTI_user_variable final : public Item_func_get_user_var {
 
   Example: \@global.default.x
 */
-class PTI_get_system_variable : public Parse_tree_item {
+class PTI_variable_aux_3d : public Parse_tree_item {
   typedef Parse_tree_item super;
 
+  enum_var_type var_type;
+  LEX_STRING ident1;
+  POS ident1_pos;
+  LEX_STRING ident2;
+
  public:
-  PTI_get_system_variable(const POS &pos, enum_var_type scope,
-                          const POS &name_pos, const LEX_CSTRING &opt_prefix,
-                          const LEX_CSTRING &name)
-      : super{pos},
-        m_scope{scope},
-        m_name_pos{name_pos},
-        m_opt_prefix{opt_prefix},
-        m_name{name} {}
+  PTI_variable_aux_3d(const POS &pos, enum_var_type var_type_arg,
+                      const LEX_STRING &ident1_arg, const POS &ident1_pos_arg,
+                      const LEX_STRING &ident2_arg)
+      : super(pos),
+        var_type(var_type_arg),
+        ident1(ident1_arg),
+        ident1_pos(ident1_pos_arg),
+        ident2(ident2_arg) {}
 
-  bool do_itemize(Parse_context *pc, Item **res) override;
-
- private:
-  const enum_var_type m_scope;
-  const POS m_name_pos;
-  const LEX_CSTRING m_opt_prefix;
-  const LEX_CSTRING m_name;
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
 class PTI_count_sym : public Item_sum_count {
@@ -440,7 +431,7 @@ class PTI_count_sym : public Item_sum_count {
   PTI_count_sym(const POS &pos, PT_window *w)
       : super(pos, (Item *)nullptr, w) {}
 
-  bool do_itemize(Parse_context *pc, Item **res) override;
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
 class PTI_in_sum_expr : public Parse_tree_item {
@@ -452,7 +443,7 @@ class PTI_in_sum_expr : public Parse_tree_item {
   PTI_in_sum_expr(const POS &pos, Item *expr_arg)
       : super(pos), expr(expr_arg) {}
 
-  bool do_itemize(Parse_context *pc, Item **res) override;
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
 class PTI_singlerow_subselect : public Parse_tree_item {
@@ -464,7 +455,7 @@ class PTI_singlerow_subselect : public Parse_tree_item {
   PTI_singlerow_subselect(const POS &pos, PT_subquery *subselect_arg)
       : super(pos), subselect(subselect_arg) {}
 
-  bool do_itemize(Parse_context *pc, Item **res) override;
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
 class PTI_exists_subselect : public Parse_tree_item {
@@ -476,7 +467,7 @@ class PTI_exists_subselect : public Parse_tree_item {
   PTI_exists_subselect(const POS &pos, PT_subquery *subselect_arg)
       : super(pos), subselect(subselect_arg) {}
 
-  bool do_itemize(Parse_context *pc, Item **res) override;
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
 class PTI_odbc_date : public Parse_tree_item {
@@ -485,17 +476,11 @@ class PTI_odbc_date : public Parse_tree_item {
   LEX_STRING ident;
   Item *expr;
 
- protected:
-  void add_json_info(Json_object *obj) override {
-    obj->add_alias("date_type",
-                   create_dom_ptr<Json_string>(ident.str, ident.length));
-  }
-
  public:
   PTI_odbc_date(const POS &pos, const LEX_STRING &ident_arg, Item *expr_arg)
       : super(pos), ident(ident_arg), expr(expr_arg) {}
 
-  bool do_itemize(Parse_context *pc, Item **res) override;
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
 class PTI_handle_sql2003_note184_exception : public Parse_tree_item {
@@ -513,10 +498,7 @@ class PTI_handle_sql2003_note184_exception : public Parse_tree_item {
         is_negation(is_negation_arg),
         right(right_arg) {}
 
-  bool do_itemize(Parse_context *pc, Item **res) override;
-  void add_json_info(Json_object *obj) override {
-    obj->add_alias("negated", create_dom_ptr<Json_boolean>(is_negation));
-  }
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
 class PTI_expr_with_alias : public Parse_tree_item {
@@ -532,37 +514,21 @@ class PTI_expr_with_alias : public Parse_tree_item {
                       const LEX_CSTRING &alias_arg)
       : super(pos), expr(expr_arg), expr_loc(expr_loc_arg), alias(alias_arg) {}
 
-  bool do_itemize(Parse_context *pc, Item **res) override;
-
- protected:
-  void add_json_info(Json_object *obj) override;
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
-class PTI_int_splocal : public Parse_tree_item {
-  using super = Parse_tree_item;
+class PTI_limit_option_ident : public Parse_tree_item {
+  typedef Parse_tree_item super;
+
+  LEX_CSTRING ident;
+  Symbol_location ident_loc;
 
  public:
-  PTI_int_splocal(const POS &pos, const LEX_CSTRING &name)
-      : super(pos), m_location{pos}, m_name{name} {}
+  PTI_limit_option_ident(const POS &pos, const LEX_CSTRING &ident_arg,
+                         const Symbol_location &ident_loc_arg)
+      : super(pos), ident(ident_arg), ident_loc(ident_loc_arg) {}
 
-  bool do_itemize(Parse_context *pc, Item **res) override;
-
- private:
-  /// Location of the variable name.
-  const POS m_location;
-
-  /// Same data as in PTI_in_sum_expr#m_location but 0-terminated "for free".
-  const LEX_CSTRING m_name;
-};
-
-class PTI_limit_option_ident : public PTI_int_splocal {
-  using super = PTI_int_splocal;
-
- public:
-  PTI_limit_option_ident(const POS &pos, const LEX_CSTRING &name)
-      : super{pos, name} {}
-
-  bool do_itemize(Parse_context *pc, Item **res) override;
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
 class PTI_limit_option_param_marker : public Parse_tree_item {
@@ -575,7 +541,7 @@ class PTI_limit_option_param_marker : public Parse_tree_item {
                                          Item_param *param_marker_arg)
       : super(pos), param_marker(param_marker_arg) {}
 
-  bool do_itemize(Parse_context *pc, Item **res) override;
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
 class PTI_context : public Parse_tree_item {
@@ -588,7 +554,7 @@ class PTI_context : public Parse_tree_item {
       : super(pos), expr(expr_arg), m_parsing_place(place) {}
 
  public:
-  bool do_itemize(Parse_context *pc, Item **res) override;
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
 class PTI_where final : public PTI_context {
@@ -603,10 +569,4 @@ class PTI_having final : public PTI_context {
       : PTI_context(pos, expr_arg, CTX_HAVING) {}
 };
 
-class PTI_qualify final : public PTI_context {
- public:
-  PTI_qualify(const POS &pos, Item *expr_arg)
-      : PTI_context(pos, expr_arg, CTX_QUALIFY) {}
-};
-
 #endif /* PARSE_TREE_ITEMS_INCLUDED */
