@@ -1,16 +1,15 @@
-/* Copyright (c) 2006, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2006, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,24 +24,22 @@
 
 #include <limits.h>
 #include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
-
 #include <algorithm>
 #include <memory>
 #include <string>
 #include <utility>
 
 #include "lex_string.h"
+#include "m_ctype.h"
+#include "m_string.h"
 #include "map_helpers.h"
 #include "my_compiler.h"
 #include "my_dbug.h"
 #include "my_sqlcommand.h"
 #include "my_sys.h"
-#include "mysql/components/services/bits/psi_bits.h"
 #include "mysql/plugin.h"
-#include "mysql/strings/int2str.h"
-#include "mysql/strings/m_ctype.h"
+#include "mysql/psi/psi_base.h"
 #include "mysql/udf_registration_types.h"
 #include "mysqld_error.h"
 #include "sql/auth/auth_acls.h"
@@ -63,13 +60,13 @@
 #include "sql/sql_partition.h"
 #include "sql/sql_tablespace.h"  // validate_tablespace_name
 #include "sql/system_variables.h"
-#include "sql/table.h"                     // Table_ref
+#include "sql/table.h"                     // TABLE_LIST
 #include "sql/table_trigger_dispatcher.h"  // Table_trigger_dispatcher
 #include "sql/thr_malloc.h"
 #include "sql/trigger_chain.h"  // Trigger_chain
 #include "sql/trigger_def.h"
 #include "sql_string.h"
-#include "string_with_len.h"
+#include "varlen_sort.h"
 
 using std::string;
 
@@ -87,8 +84,8 @@ partition_info *partition_info::get_clone(THD *thd, bool reset /* = false */) {
   new (&(clone->read_partitions)) MY_BITMAP;
   new (&(clone->lock_partitions)) MY_BITMAP;
   clone->bitmaps_are_initialized = false;
-  clone->partitions.clear();
-  clone->temp_partitions.clear();
+  clone->partitions.empty();
+  clone->temp_partitions.empty();
 
   while ((part = (part_it++))) {
     List_iterator<partition_element> subpart_it(part->subpartitions);
@@ -122,7 +119,7 @@ partition_info *partition_info::get_clone(THD *thd, bool reset /* = false */) {
       }
     }
 
-    part_clone->subpartitions.clear();
+    part_clone->subpartitions.empty();
     while ((subpart = (subpart_it++))) {
       partition_element *subpart_clone =
           new (thd->mem_root) partition_element(*subpart);
@@ -170,11 +167,11 @@ bool partition_info::add_named_partition(const char *part_name, size_t length) {
   PART_NAME_DEF *part_def;
   Partition_share *part_share;
   DBUG_TRACE;
-  assert(table && table->s && table->s->ha_share);
+  DBUG_ASSERT(table && table->s && table->s->ha_share);
   part_share = static_cast<Partition_share *>((table->s->ha_share));
-  assert(part_share->partition_name_hash != nullptr);
+  DBUG_ASSERT(part_share->partition_name_hash != nullptr);
   auto part_name_hash = part_share->partition_name_hash.get();
-  assert(!part_name_hash->empty());
+  DBUG_ASSERT(!part_name_hash->empty());
 
   part_def = find_or_nullptr(*part_name_hash, string(part_name, length));
   if (!part_def) {
@@ -249,7 +246,7 @@ bool partition_info::set_read_partitions(List<String> *partition_names) {
 /**
   Set read/lock_partitions bitmap over non pruned partitions
 
-  @param table_list   Possible Table_ref which can contain
+  @param table_list   Possible TABLE_LIST which can contain
                       list of partition names to query
 
   @return Operation status
@@ -260,11 +257,11 @@ bool partition_info::set_read_partitions(List<String> *partition_names) {
   @note OK to call multiple times without the need for free_bitmaps.
 */
 
-bool partition_info::set_partition_bitmaps(Table_ref *table_list) {
+bool partition_info::set_partition_bitmaps(TABLE_LIST *table_list) {
   DBUG_TRACE;
 
-  assert(bitmaps_are_initialized);
-  assert(table);
+  DBUG_ASSERT(bitmaps_are_initialized);
+  DBUG_ASSERT(table);
   is_pruning_completed = false;
   if (!bitmaps_are_initialized) return true;
 
@@ -285,7 +282,7 @@ bool partition_info::set_partition_bitmaps(Table_ref *table_list) {
     DBUG_PRINT("info", ("Set all partitions"));
   }
   bitmap_copy(&lock_partitions, &read_partitions);
-  assert(bitmap_get_first_set(&lock_partitions) != MY_BIT_NONE);
+  DBUG_ASSERT(bitmap_get_first_set(&lock_partitions) != MY_BIT_NONE);
   return false;
 }
 
@@ -309,14 +306,15 @@ bool partition_info::set_partition_bitmaps(Table_ref *table_list) {
     @retval true   Failure
 */
 
-bool partition_info::can_prune_insert(
-    THD *thd, enum_duplicates duplic, COPY_INFO &update,
-    const mem_root_deque<Item *> &update_fields,
-    const mem_root_deque<Item *> &fields, bool empty_values,
-    enum_can_prune *can_prune_partitions, bool *prune_needs_default_values,
-    MY_BITMAP *used_partitions) {
+bool partition_info::can_prune_insert(THD *thd, enum_duplicates duplic,
+                                      COPY_INFO &update,
+                                      List<Item> &update_fields,
+                                      List<Item> &fields, bool empty_values,
+                                      enum_can_prune *can_prune_partitions,
+                                      bool *prune_needs_default_values,
+                                      MY_BITMAP *used_partitions) {
   *can_prune_partitions = PRUNE_NO;
-  assert(bitmaps_are_initialized);
+  DBUG_ASSERT(bitmaps_are_initialized);
   DBUG_TRACE;
 
   if (table->s->db_type()->partition_flags() & HA_USE_AUTO_PARTITION)
@@ -346,19 +344,19 @@ bool partition_info::can_prune_insert(
   if (table->vfield) {
     Field **fld;
     for (fld = table->vfield; *fld; fld++) {
-      if (bitmap_is_set(&full_part_field_set, (*fld)->field_index()))
+      if (bitmap_is_set(&full_part_field_set, (*fld)->field_index))
         return false;
     }
   }
 
   /*
-    Can't prune partitions over generated default expressions, as their values
+    Can't prune partitions over generated default expresssions, as their values
     are calculated much later.
   */
   if (table->gen_def_fields_ptr) {
     Field **fld;
     for (fld = table->gen_def_fields_ptr; *fld; fld++) {
-      if (bitmap_is_set(&full_part_field_set, (*fld)->field_index()))
+      if (bitmap_is_set(&full_part_field_set, (*fld)->field_index))
         return false;
     }
   }
@@ -370,7 +368,7 @@ bool partition_info::can_prune_insert(
       is not 0 (with NO_AUTO_VALUE_ON_ZERO sql_mode), then pruning is possible!
     */
     if (bitmap_is_set(&full_part_field_set,
-                      table->found_next_number_field->field_index()))
+                      table->found_next_number_field->field_index))
       return false;
   }
 
@@ -421,11 +419,11 @@ bool partition_info::can_prune_insert(
       2) only copy those fields from the default record.
   */
   *prune_needs_default_values = false;
-  if (empty_values) {
-    *prune_needs_default_values = true;  // like 'INSERT INTO t () VALUES ()'
-  } else if (!fields.empty()) {
+  if (fields.elements) {
     if (!is_full_part_expr_in_fields(fields))
       *prune_needs_default_values = true;
+  } else if (empty_values) {
+    *prune_needs_default_values = true;  // like 'INSERT INTO t () VALUES ()'
   } else {
     /*
       In case of INSERT INTO t VALUES (...) we must get values for
@@ -443,7 +441,7 @@ bool partition_info::can_prune_insert(
   /*
     If no partitioning field in set (e.g. defaults) check pruning only once.
   */
-  if (!fields.empty() && !is_fields_in_part_expr(fields))
+  if (fields.elements && !is_fields_in_part_expr(fields))
     *can_prune_partitions = PRUNE_DEFAULTS;
   else
     *can_prune_partitions = PRUNE_YES;
@@ -454,7 +452,6 @@ bool partition_info::can_prune_insert(
 /**
   Mark the partition, the record belongs to, as used.
 
-  @param thd              Thread handler
   @param fields           Fields to set
   @param values           Values to use
   @param info             COPY_INFO used for default values handling
@@ -468,25 +465,27 @@ bool partition_info::can_prune_insert(
   so caller must check thd->is_error().
 */
 
-bool partition_info::set_used_partition(THD *thd,
-                                        const mem_root_deque<Item *> &fields,
-                                        const mem_root_deque<Item *> &values,
+bool partition_info::set_used_partition(List<Item> &fields, List<Item> &values,
                                         COPY_INFO &info,
                                         bool copy_default_values,
                                         MY_BITMAP *used_partitions) {
+  THD *thd = table->in_use;
   uint32 part_id;
   longlong func_value;
 
-  assert(thd);
+  DBUG_ASSERT(thd);
 
   /* Only allow checking of constant values */
-  for (Item *item : values) {
+  List_iterator_fast<Item> v(values);
+  Item *item;
+
+  while ((item = v++)) {
     if (!item->const_item()) return true;
   }
 
   if (copy_default_values) restore_record(table, s->default_values);
 
-  if (!fields.empty() || values.empty()) {
+  if (fields.elements || !values.elements) {
     if (fill_record(thd, table, fields, values, &full_part_field_set, nullptr,
                     false))
       return true;
@@ -502,9 +501,9 @@ bool partition_info::set_used_partition(THD *thd,
     overhead. Not yet done, since mostly only one DEFAULT function per
     table, or at least very few such columns.
   */
-  if (info.function_defaults_apply_on_columns(&full_part_field_set)) {
-    if (info.set_function_defaults(table)) return true;
-  }
+  if (info.function_defaults_apply_on_columns(&full_part_field_set))
+    info.set_function_defaults(table);
+
   {
     /*
       This function is used in INSERT; 'values' are supplied by user,
@@ -802,7 +801,7 @@ bool partition_info::set_up_defaults_for_partitioning(
     no parameters
 
   RETURN VALUE
-    Erroneous field name  Error, there are two fields with same name
+    Erroneus field name  Error, there are two fields with same name
     NULL                 Ok, no field defined twice
 
   DESCRIPTION
@@ -850,7 +849,7 @@ partition_element *partition_info::get_part_elem(const char *partition_name,
   List_iterator<partition_element> part_it(partitions);
   uint i = 0;
   DBUG_TRACE;
-  assert(part_id);
+  DBUG_ASSERT(part_id);
   *part_id = NOT_A_PARTITION_ID;
   do {
     partition_element *part_elem = part_it++;
@@ -898,6 +897,7 @@ partition_element *partition_info::get_part_elem(const char *partition_name,
 const char *partition_info::find_duplicate_name() {
   collation_unordered_set<string> partition_names{system_charset_info,
                                                   PSI_INSTRUMENT_ME};
+  uint max_names;
   List_iterator<partition_element> parts_it(partitions);
   partition_element *p_elem;
 
@@ -909,6 +909,8 @@ const char *partition_info::find_duplicate_name() {
     And this only happens when in ALTER TABLE with full table copy.
   */
 
+  max_names = num_parts;
+  if (is_sub_partitioned()) max_names += num_parts * num_subparts;
   while ((p_elem = (parts_it++))) {
     const char *partition_name = p_elem->partition_name;
     if (!partition_names.insert(partition_name).second) return partition_name;
@@ -1123,7 +1125,7 @@ bool partition_info::check_range_constants(THD *thd) {
         List_iterator<part_elem_value> list_val_it(part_def->list_val_list);
         part_elem_value *range_val = list_val_it++;
         part_column_list_val *col_val = range_val->col_val_array;
-        assert(part_def->list_val_list.elements == 1);
+        DBUG_ASSERT(part_def->list_val_list.elements == 1);
 
         if (fix_column_value_functions(thd, range_val, i)) goto end;
         memcpy(loc_range_col_array, (const void *)col_val, size_entries);
@@ -1177,35 +1179,6 @@ range_not_increasing_error:
   goto end;
 }
 
-static int partition_info_compare_column_values(
-    const part_column_list_val *first, const part_column_list_val *second) {
-  for (Field **field = first->part_info->part_field_array; *field;
-       field++, first++, second++) {
-    /*
-      If both are maxvalue, they are equal (don't check the rest of the parts).
-      Otherwise, maxvalue > *.
-     */
-    if (first->max_value) {
-      return second->max_value ? 0 : 1;
-    } else if (second->max_value) {
-      return -1;
-    }
-
-    // NULLs sort before non-NULLs.
-    if (first->null_value != second->null_value) {
-      return first->null_value ? -1 : 1;
-    }
-
-    // For non-NULLs, compare the actual fields.
-    if (!first->null_value) {
-      int res = (*field)->cmp(first->column_value.field_image,
-                              second->column_value.field_image);
-      if (res != 0) return res;
-    }
-  }
-  return 0;
-}
-
 /*
   Compare two lists of column values in RANGE/LIST partitioning
   SYNOPSIS
@@ -1217,10 +1190,34 @@ static int partition_info_compare_column_values(
     false                    first >= second
 */
 
+static bool partition_info_compare_column_values(
+    const part_column_list_val *first, const part_column_list_val *second) {
+  for (Field **field = first->part_info->part_field_array; *field;
+       field++, first++, second++) {
+    /*
+      If both are maxvalue, they are equal (don't check the rest of the parts).
+      Otherwise, maxvalue > *.
+    */
+    if (first->max_value || second->max_value)
+      return first->max_value < second->max_value;
+
+    // NULLs sort before non-NULLs.
+    if (first->null_value != second->null_value) return first->null_value;
+
+    // For non-NULLs, compare the actual fields.
+    if (!first->null_value) {
+      int res = (*field)->cmp(first->column_value.field_image,
+                              second->column_value.field_image);
+      if (res != 0) return res < 0;
+    }
+  }
+  return false;
+}
+
 bool partition_info::compare_column_values(
     const part_column_list_val *first_arg,
     const part_column_list_val *second_arg) {
-  return partition_info_compare_column_values(first_arg, second_arg) < 0;
+  return partition_info_compare_column_values(first_arg, second_arg);
 }
 
 /*
@@ -1315,17 +1312,14 @@ bool partition_info::check_list_constants(THD *thd) {
       }
     } while (++part_id < num_parts);
 
-    qsort(list_col_array, num_list_values, size_entries,
-          [](const void *a, const void *b) {
-            return partition_info_compare_column_values(
-                static_cast<const part_column_list_val *>(a),
-                static_cast<const part_column_list_val *>(b));
-          });
+    varlen_sort(list_col_array,
+                list_col_array + num_list_values * num_column_values,
+                size_entries, partition_info_compare_column_values);
 
     for (uint i = 1; i < num_list_values; ++i) {
-      if (partition_info_compare_column_values(
+      if (!partition_info_compare_column_values(
               &list_col_array[num_column_values * (i - 1)],
-              &list_col_array[num_column_values * i]) >= 0) {
+              &list_col_array[num_column_values * i])) {
         my_error(ER_MULTIPLE_DEF_CONST_IN_LIST_PART_ERROR, MYF(0));
         goto end;
       }
@@ -1364,7 +1358,7 @@ bool partition_info::check_list_constants(THD *thd) {
       goto end;
     }
   }
-  assert(fixed);
+  DBUG_ASSERT(fixed);
   result = false;
 end:
   return result;
@@ -1427,14 +1421,14 @@ bool partition_info::check_partition_info(THD *thd, handlerton **eng_type,
 
     /* Check for partition expression. */
     if (!list_of_part_fields) {
-      assert(part_expr);
+      DBUG_ASSERT(part_expr);
       err = part_expr->walk(&Item::check_partition_func_processor,
                             enum_walk::POSTFIX, nullptr);
     }
 
     /* Check for sub partition expression. */
     if (!err && is_sub_partitioned() && !list_of_subpart_fields) {
-      assert(subpart_expr);
+      DBUG_ASSERT(subpart_expr);
       err = subpart_expr->walk(&Item::check_partition_func_processor,
                                enum_walk::POSTFIX, nullptr);
     }
@@ -1592,7 +1586,7 @@ bool partition_info::check_partition_info(THD *thd, handlerton **eng_type,
     goto end;
   }
 
-  assert(table_engine == default_engine_type);
+  DBUG_ASSERT(table_engine == default_engine_type);
 
   if (eng_type) *eng_type = table_engine;
 
@@ -1628,7 +1622,7 @@ end:
 void partition_info::print_no_partition_found(THD *thd, TABLE *table_arg) {
   char buf[100];
   const char *buf_ptr = buf;
-  Table_ref table_list;
+  TABLE_LIST table_list;
 
   table_list.db = table_arg->s->db.str;
   table_list.table_name = table_arg->s->table_name.str;
@@ -1740,6 +1734,7 @@ bool partition_info::set_up_charset_field_preps() {
   uchar **char_ptrs;
   unsigned i;
   size_t size;
+  uint tot_fields = 0;
   uint tot_part_fields = 0;
   uint tot_subpart_fields = 0;
   DBUG_TRACE;
@@ -1751,6 +1746,7 @@ bool partition_info::set_up_charset_field_preps() {
     while ((field = *(ptr++))) {
       if (field_is_partition_charset(field)) {
         tot_part_fields++;
+        tot_fields++;
       }
     }
     size = tot_part_fields * sizeof(char *);
@@ -1781,6 +1777,7 @@ bool partition_info::set_up_charset_field_preps() {
     while ((field = *(ptr++))) {
       if (field_is_partition_charset(field)) {
         tot_subpart_fields++;
+        tot_fields++;
       }
     }
     size = tot_subpart_fields * sizeof(char *);
@@ -1908,13 +1905,15 @@ void partition_info::report_part_expr_error(bool use_subpart_expr) {
     @retval false No field is within any partitioning expression.
 */
 
-bool partition_info::is_fields_in_part_expr(
-    const mem_root_deque<Item *> &fields) {
+bool partition_info::is_fields_in_part_expr(List<Item> &fields) {
+  List_iterator<Item> it(fields);
+  Item *item;
+  Item_field *field;
   DBUG_TRACE;
-  for (Item *item : fields) {
-    Item_field *field = item->field_for_view_update();
-    assert(field->field->table == table);
-    if (bitmap_is_set(&full_part_field_set, field->field->field_index()))
+  while ((item = it++)) {
+    field = item->field_for_view_update();
+    DBUG_ASSERT(field->field->table == table);
+    if (bitmap_is_set(&full_part_field_set, field->field->field_index))
       return true;
   }
   return false;
@@ -1924,10 +1923,9 @@ bool partition_info::is_fields_in_part_expr(
   Check if all partitioning fields are included.
 */
 
-bool partition_info::is_full_part_expr_in_fields(
-    const mem_root_deque<Item *> &fields) {
+bool partition_info::is_full_part_expr_in_fields(List<Item> &fields) {
   Field **part_field = full_part_field_array;
-  assert(*part_field);
+  DBUG_ASSERT(*part_field);
   DBUG_TRACE;
   /*
     It is very seldom many fields in full_part_field_array, so it is OK
@@ -1935,11 +1933,14 @@ bool partition_info::is_full_part_expr_in_fields(
     to compare with.
   */
   do {
+    List_iterator<Item> it(fields);
+    Item *item;
+    Item_field *field;
     bool found = false;
 
-    for (Item *item : fields) {
-      Item_field *field = item->field_for_view_update();
-      assert(field->field->table == table);
+    while ((item = it++)) {
+      field = item->field_for_view_update();
+      DBUG_ASSERT(field->field->table == table);
       if (*part_field == field->field) {
         found = true;
         break;
@@ -2063,8 +2064,8 @@ void Parser_partition_info::init_col_val(part_column_list_val *col_val,
 
 bool Parser_partition_info::add_column_list_value(THD *thd, Item *item) {
   part_column_list_val *col_val;
-  Name_resolution_context *context = &thd->lex->current_query_block()->context;
-  Table_ref *save_list = context->table_list;
+  Name_resolution_context *context = &thd->lex->current_select()->context;
+  TABLE_LIST *save_list = context->table_list;
   const char *save_where = thd->where;
   DBUG_TRACE;
 
@@ -2169,8 +2170,9 @@ bool Parser_partition_info::reorganize_into_single_field_col_val() {
   uint num_values = part_info->num_columns;
   uint i;
   DBUG_TRACE;
-  assert(part_info->part_type == partition_type::LIST);
-  assert(!part_info->num_columns || part_info->num_columns == val->added_items);
+  DBUG_ASSERT(part_info->part_type == partition_type::LIST);
+  DBUG_ASSERT(!part_info->num_columns ||
+              part_info->num_columns == val->added_items);
 
   if (!num_values) num_values = val->added_items;
   part_info->num_columns = 1;
@@ -2223,7 +2225,7 @@ bool partition_info::fix_partition_values(part_elem_value *val,
   }
   if (col_val->max_value) {
     /* The parser ensures we're not LIST partitioned here */
-    assert(part_type == partition_type::RANGE);
+    DBUG_ASSERT(part_type == partition_type::RANGE);
     if (defined_max_value) {
       my_error(ER_PARTITION_MAXVALUE_ERROR, MYF(0));
       return true;
@@ -2342,7 +2344,7 @@ bool partition_info::fix_column_value_functions(THD *thd, part_elem_value *val,
           goto end;
         }
         col_val->column_value.field_image = val_ptr;
-        memcpy(val_ptr, field->field_ptr(), len);
+        memcpy(val_ptr, field->ptr, len);
       }
     }
     col_val->fixed = 2;
@@ -2433,7 +2435,7 @@ bool partition_info::fix_parser_data(THD *thd) {
     part_elem = it++;
     List_iterator<part_elem_value> list_val_it(part_elem->list_val_list);
     num_elements = part_elem->list_val_list.elements;
-    assert(part_type == partition_type::RANGE ? num_elements == 1U : true);
+    DBUG_ASSERT(part_type == partition_type::RANGE ? num_elements == 1U : true);
     for (j = 0; j < num_elements; j++) {
       part_elem_value *val = list_val_it++;
       if (column_list) {
@@ -2503,7 +2505,7 @@ static bool strcmp_null(const char *a, const char *b) {
 bool partition_info::has_same_partitioning(partition_info *new_part_info) {
   DBUG_TRACE;
 
-  assert(part_field_array && part_field_array[0]);
+  DBUG_ASSERT(part_field_array && part_field_array[0]);
 
   /*
     Only consider pre 5.5.3 .frm's to have same partitioning as
@@ -2617,7 +2619,7 @@ bool partition_info::has_same_partitioning(partition_info *new_part_info) {
           }
           if (new_list_vals++) return false;
         } else {
-          assert(part_type == partition_type::RANGE);
+          DBUG_ASSERT(part_type == partition_type::RANGE);
           if (new_part_elem->range_value != part_elem->range_value)
             return false;
         }
@@ -2726,7 +2728,7 @@ bool partition_info::same_key_column_order(List<Create_field> *create_list) {
   return true;
 }
 
-void partition_info::print_debug(const char *str [[maybe_unused]],
+void partition_info::print_debug(const char *str MY_ATTRIBUTE((unused)),
                                  uint *value) {
   DBUG_TRACE;
   if (value)
@@ -2821,7 +2823,7 @@ bool validate_partition_tablespace_name_lengths(partition_info *part_info) {
 
 bool validate_partition_tablespace_names(partition_info *part_info,
                                          const handlerton *default_engine) {
-  assert(default_engine);
+  DBUG_ASSERT(default_engine);
 
   // Do nothing if the table is not partitioned.
   if (!part_info) return false;
