@@ -1,19 +1,18 @@
 #ifndef PROTOCOL_CLASSIC_INCLUDED
 #define PROTOCOL_CLASSIC_INCLUDED
 
-/* Copyright (c) 2002, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2002, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,14 +26,12 @@
 #include <stddef.h>
 #include <sys/types.h>
 
-#include "my_compiler.h"  // include before mysql_com.h because of STDCALL
-#include "mysql_com.h"    // TODO: fix mysql_com.h dependency on my_compiler.h
-
 #include "field_types.h"  // enum_field_types
+#include "m_ctype.h"
 #include "my_command.h"
 #include "my_inttypes.h"
 #include "my_io.h"
-#include "mysql/strings/m_ctype.h"
+#include "mysql_com.h"
 #include "mysql_time.h"
 #include "sql/protocol.h"  // Protocol
 #include "violite.h"
@@ -65,7 +62,7 @@ class Protocol_classic : public Protocol {
   String convert;
   uint field_pos;
   bool send_metadata;
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   enum enum_field_types *field_types;
   uint count;
 #endif
@@ -98,7 +95,7 @@ class Protocol_classic : public Protocol {
     init(thd);
   }
   void init(THD *thd_arg);
-  bool store_field(const Field *field) final;
+  bool store_field(const Field *field) final override;
   bool store_string(const char *from, size_t length,
                     const CHARSET_INFO *cs) final;
   int read_packet() override;
@@ -148,7 +145,7 @@ class Protocol_classic : public Protocol {
   // NET interaction functions
   /* Initialize NET */
   bool init_net(Vio *vio);
-  void claim_memory_ownership(bool claim);
+  void claim_memory_ownership();
   /* Deinitialize NET */
   void end_net();
   /* Write data to NET buffer */
@@ -197,12 +194,11 @@ class Protocol_classic : public Protocol {
   /* Return packet string */
   String *get_output_packet();
   /* return packet length */
-  ulong get_packet_length() { return input_packet_length; }
+  uint get_packet_length() { return input_packet_length; }
   /* Return raw packet buffer */
   uchar *get_raw_packet() { return input_raw_packet; }
   /* Set read timeout */
-  virtual void set_read_timeout(ulong read_timeout,
-                                bool on_full_packet = false);
+  virtual void set_read_timeout(ulong read_timeout);
   /* Set write timeout */
   virtual void set_write_timeout(ulong write_timeout);
 
@@ -219,7 +215,7 @@ class Protocol_classic : public Protocol {
 
 class Protocol_text : public Protocol_classic {
  public:
-  Protocol_text() = default;
+  Protocol_text() {}
   Protocol_text(THD *thd_arg) : Protocol_classic(thd_arg) {}
   bool store_null() override;
   bool store_tiny(longlong from, uint32 zerofill) override;
@@ -244,7 +240,7 @@ class Protocol_binary final : public Protocol_text {
   uint bit_fields;
 
  public:
-  Protocol_binary() = default;
+  Protocol_binary() {}
   Protocol_binary(THD *thd_arg) : Protocol_text(thd_arg) {}
   void start_row() override;
   bool store_null() override;
