@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -34,38 +34,34 @@
 #include "my_compiler.h"
 #include "my_dbug.h"
 #include "my_inttypes.h"
+#include "my_loglevel.h"
 #include "my_sys.h"
 #include "my_thread.h"
-#include "mysql/components/services/bits/psi_bits.h"
-#include "mysql/components/services/bits/psi_stage_bits.h"
 #include "mysql/components/services/log_builtins.h"
-#include "mysql/my_loglevel.h"
+#include "mysql/components/services/psi_stage_bits.h"
 #include "mysql/psi/mysql_cond.h"
 #include "mysql/psi/mysql_mutex.h"
+#include "mysql/psi/psi_base.h"
 #include "mysql/service_mysql_alloc.h"
 #include "mysql_com.h"
 #include "mysqld_error.h"
 #include "sql/auth/sql_security_ctx.h"
 #include "sql/binlog.h"
-#include "sql/changestreams/apply/replication_thread_status.h"
 #include "sql/current_thd.h"
 #include "sql/log.h"
 #include "sql/log_event.h"
-#include "sql/mysqld.h"              // opt_mts_replica_parallel_workers
+#include "sql/mysqld.h"              // opt_mts_slave_parallel_workers
 #include "sql/mysqld_thd_manager.h"  // Global_THD_manager
 #include "sql/protocol_classic.h"
-#include "sql/raii/sentry.h"
-#include "sql/rpl_async_conn_failover_configuration_propagation.h"
-#include "sql/rpl_channel_credentials.h"
 #include "sql/rpl_gtid.h"
 #include "sql/rpl_info_factory.h"
 #include "sql/rpl_info_handler.h"
 #include "sql/rpl_mi.h"
 #include "sql/rpl_msr.h" /* Multisource replication */
-#include "sql/rpl_mta_submode.h"
-#include "sql/rpl_replica.h"
+#include "sql/rpl_mts_submode.h"
 #include "sql/rpl_rli.h"
 #include "sql/rpl_rli_pdb.h"
+#include "sql/rpl_slave.h"
 #include "sql/rpl_trx_boundary_parser.h"
 #include "sql/sql_class.h"
 #include "sql/sql_lex.h"
@@ -90,6 +86,13 @@ int channel_stop(Master_info *mi, int threads_to_stop, long timeout);
 int initialize_channel_service_interface() {
   DBUG_TRACE;
 
+  // master info and relay log repositories must be TABLE
+  if (opt_mi_repository_id != INFO_REPOSITORY_TABLE ||
+      opt_rli_repository_id != INFO_REPOSITORY_TABLE) {
+    LogErr(ERROR_LEVEL, ER_RPL_CHANNELS_REQUIRE_TABLES_AS_INFO_REPOSITORIES);
+    return 1;
+  }
+
   // server id must be different from 0
   if (server_id == 0) {
     LogErr(ERROR_LEVEL, ER_RPL_CHANNELS_REQUIRE_NON_ZERO_SERVER_ID);
@@ -106,18 +109,14 @@ static void set_mi_settings(Master_info *mi,
 
   mi->rli->set_thd_tx_priority(channel_info->thd_tx_priority);
 
-  mi->rli->set_ignore_write_set_memory_limit(
-      channel_info->m_ignore_write_set_memory_limit);
-  mi->rli->set_allow_drop_write_set(channel_info->m_allow_drop_write_set);
-
   mi->rli->replicate_same_server_id =
       (channel_info->replicate_same_server_id == RPL_SERVICE_SERVER_DEFAULT)
           ? replicate_same_server_id
           : channel_info->replicate_same_server_id;
 
-  mi->rli->opt_replica_parallel_workers =
+  mi->rli->opt_slave_parallel_workers =
       (channel_info->channel_mts_parallel_workers == RPL_SERVICE_SERVER_DEFAULT)
-          ? opt_mts_replica_parallel_workers
+          ? opt_mts_slave_parallel_workers
           : channel_info->channel_mts_parallel_workers;
 
   if (channel_info->channel_mts_parallel_type == RPL_SERVICE_SERVER_DEFAULT) {
@@ -134,27 +133,11 @@ static void set_mi_settings(Master_info *mi,
   }
 
   mi->rli->checkpoint_group =
-      (channel_info->channel_mta_checkpoint_group == RPL_SERVICE_SERVER_DEFAULT)
-          ? opt_mta_checkpoint_group
-          : channel_info->channel_mta_checkpoint_group;
+      (channel_info->channel_mts_checkpoint_group == RPL_SERVICE_SERVER_DEFAULT)
+          ? opt_mts_checkpoint_group
+          : channel_info->channel_mts_checkpoint_group;
 
-  Format_description_log_event *fde = new Format_description_log_event();
-  /*
-    Group replication applier channel shall not use checksum on its relay log
-    files.
-  */
-  if (channel_map.is_group_replication_channel_name(mi->get_channel(), true)) {
-    fde->footer()->checksum_alg = mysql::binlog::event::BINLOG_CHECKSUM_ALG_OFF;
-    /*
-      When the receiver thread connects to the master, it gets its current
-      binlog checksum algorithm, but as GR applier channel has no receiver
-      thread (and also does not connect to a master), we will set the variable
-      here to BINLOG_CHECKSUM_ALG_OFF as events queued after certification have
-      no checksum information.
-    */
-    mi->checksum_alg_before_fd = mysql::binlog::event::BINLOG_CHECKSUM_ALG_OFF;
-  }
-  mi->set_mi_description_event(fde);
+  mi->set_mi_description_event(new Format_description_log_event());
 
   mysql_mutex_unlock(&mi->data_lock);
   mysql_mutex_unlock(mi->rli->relay_log.get_log_lock());
@@ -190,7 +173,7 @@ void initialize_channel_creation_info(Channel_creation_info *channel_info) {
   channel_info->auto_position = RPL_SERVICE_SERVER_DEFAULT;
   channel_info->channel_mts_parallel_type = RPL_SERVICE_SERVER_DEFAULT;
   channel_info->channel_mts_parallel_workers = RPL_SERVICE_SERVER_DEFAULT;
-  channel_info->channel_mta_checkpoint_group = RPL_SERVICE_SERVER_DEFAULT;
+  channel_info->channel_mts_checkpoint_group = RPL_SERVICE_SERVER_DEFAULT;
   channel_info->replicate_same_server_id = RPL_SERVICE_SERVER_DEFAULT;
   channel_info->thd_tx_priority = 0;
   channel_info->sql_delay = RPL_SERVICE_SERVER_DEFAULT;
@@ -201,8 +184,6 @@ void initialize_channel_creation_info(Channel_creation_info *channel_info) {
   channel_info->get_public_key = 0;
   channel_info->compression_algorithm = nullptr;
   channel_info->zstd_compression_level = 0;
-  channel_info->m_ignore_write_set_memory_limit = false;
-  channel_info->m_allow_drop_write_set = false;
 }
 
 void initialize_channel_ssl_info(Channel_ssl_info *channel_ssl_info) {
@@ -288,11 +269,11 @@ int channel_create(const char *channel, Channel_creation_info *channel_info) {
   if (!strcmp(channel_map.get_default_channel(), channel))
     return RPL_CHANNEL_SERVICE_DEFAULT_CHANNEL_CREATION_ERROR;
 
-  /* Service channels are not supposed to use sql_replica_skip_counter */
-  mysql_mutex_lock(&LOCK_sql_replica_skip_counter);
-  if (sql_replica_skip_counter > 0)
+  /* Service channels are not supposed to use sql_slave_skip_counter */
+  mysql_mutex_lock(&LOCK_sql_slave_skip_counter);
+  if (sql_slave_skip_counter > 0)
     error = RPL_CHANNEL_SERVICE_SLAVE_SKIP_COUNTER_ACTIVE;
-  mysql_mutex_unlock(&LOCK_sql_replica_skip_counter);
+  mysql_mutex_unlock(&LOCK_sql_slave_skip_counter);
   if (error) return error;
 
   channel_map.wrlock();
@@ -358,19 +339,6 @@ int channel_create(const char *channel, Channel_creation_info *channel_info) {
     lex_mi->zstd_compression_level = channel_info->zstd_compression_level;
   }
 
-  lex_mi->m_source_connection_auto_failover = LEX_MASTER_INFO::LEX_MI_UNCHANGED;
-  if (channel_info->m_source_connection_auto_failover) {
-    if (mi && !mi->is_source_connection_auto_failover()) {
-      lex_mi->m_source_connection_auto_failover =
-          LEX_MASTER_INFO::LEX_MI_ENABLE;
-    }
-  } else {
-    if (mi && mi->is_source_connection_auto_failover()) {
-      lex_mi->m_source_connection_auto_failover =
-          LEX_MASTER_INFO::LEX_MI_DISABLE;
-    }
-  }
-
   if (channel_info->ssl_info != nullptr) {
     set_mi_ssl_options(lex_mi, channel_info->ssl_info);
   }
@@ -390,9 +358,9 @@ int channel_create(const char *channel, Channel_creation_info *channel_info) {
   set_mi_settings(mi, channel_info);
 
   if (channel_map.is_group_replication_channel_name(mi->get_channel())) {
-    thd->variables.max_allowed_packet = replica_max_allowed_packet;
-    thd->get_protocol_classic()->set_max_packet_size(
-        replica_max_allowed_packet + MAX_LOG_EVENT_HEADER);
+    thd->variables.max_allowed_packet = slave_max_allowed_packet;
+    thd->get_protocol_classic()->set_max_packet_size(slave_max_allowed_packet +
+                                                     MAX_LOG_EVENT_HEADER);
   }
 
 err:
@@ -408,9 +376,7 @@ err:
 }
 
 int channel_start(const char *channel, Channel_connection_info *connection_info,
-                  int threads_to_start, int wait_for_connection,
-                  bool use_server_mta_configuration,
-                  bool channel_map_already_locked) {
+                  int threads_to_start, int wait_for_connection) {
   DBUG_TRACE;
   int error = 0;
   int thread_mask = 0;
@@ -418,20 +384,15 @@ int channel_start(const char *channel, Channel_connection_info *connection_info,
   ulong thread_start_id = 0;
   bool thd_created = false;
   THD *thd = current_thd;
-  String_set user, pass, auth;
 
-  /* Service channels are not supposed to use sql_replica_skip_counter */
-  mysql_mutex_lock(&LOCK_sql_replica_skip_counter);
-  if (sql_replica_skip_counter > 0)
+  /* Service channels are not supposed to use sql_slave_skip_counter */
+  mysql_mutex_lock(&LOCK_sql_slave_skip_counter);
+  if (sql_slave_skip_counter > 0)
     error = RPL_CHANNEL_SERVICE_SLAVE_SKIP_COUNTER_ACTIVE;
-  mysql_mutex_unlock(&LOCK_sql_replica_skip_counter);
+  mysql_mutex_unlock(&LOCK_sql_slave_skip_counter);
   if (error) return error;
 
-  if (channel_map_already_locked) {
-    channel_map.assert_some_wrlock();
-  } else {
-    channel_map.wrlock();
-  }
+  channel_map.wrlock();
 
   Master_info *mi = channel_map.get_mi(channel);
 
@@ -453,16 +414,6 @@ int channel_start(const char *channel, Channel_connection_info *connection_info,
   LEX_SLAVE_CONNECTION lex_connection;
   lex_connection.reset();
 
-  if (!Rpl_channel_credentials::get_instance().get_credentials(channel, user,
-                                                               pass, auth)) {
-    lex_connection.user =
-        (user.first ? const_cast<char *>(user.second.c_str()) : nullptr);
-    lex_connection.password =
-        (pass.first ? const_cast<char *>(pass.second.c_str()) : nullptr);
-    lex_connection.plugin_auth =
-        (auth.first ? const_cast<char *>(auth.second.c_str()) : nullptr);
-  }
-
   if (connection_info->until_condition != CHANNEL_NO_UNTIL_CONDITION) {
     switch (connection_info->until_condition) {
       case CHANNEL_UNTIL_APPLIER_AFTER_GTIDS:
@@ -477,11 +428,11 @@ int channel_start(const char *channel, Channel_connection_info *connection_info,
         lex_mi.until_after_gaps = true;
         break;
       case CHANNEL_UNTIL_VIEW_ID:
-        assert((thread_mask & SLAVE_SQL) && connection_info->view_id);
+        DBUG_ASSERT((thread_mask & SLAVE_SQL) && connection_info->view_id);
         lex_mi.view_id = connection_info->view_id;
         break;
       default:
-        assert(0);
+        DBUG_ASSERT(0);
     }
   }
 
@@ -493,8 +444,7 @@ int channel_start(const char *channel, Channel_connection_info *connection_info,
     thd = create_surrogate_thread();
   }
 
-  error = start_slave(thd, &lex_connection, &lex_mi, thread_mask, mi,
-                      use_server_mta_configuration);
+  error = start_slave(thd, &lex_connection, &lex_mi, thread_mask, mi, false);
 
   if (wait_for_connection && (thread_mask & SLAVE_IO) && !error) {
     mysql_mutex_lock(&mi->run_lock);
@@ -518,9 +468,7 @@ int channel_start(const char *channel, Channel_connection_info *connection_info,
   }
 
 err:
-  if (!channel_map_already_locked) {
-    channel_map.unlock();
-  }
+  channel_map.unlock();
 
   if (thd_created) {
     delete_surrogate_thread(thd);
@@ -554,10 +502,6 @@ int channel_stop(Master_info *mi, int threads_to_stop, long timeout) {
       (server_thd_mask & SLAVE_IO)) {
     thread_mask |= SLAVE_IO;
   }
-  if ((threads_to_stop & CHANNEL_RECEIVER_THREAD) &&
-      (server_thd_mask & SLAVE_MONITOR)) {
-    thread_mask |= SLAVE_MONITOR;
-  }
 
   if (thread_mask == 0) {
     goto end;
@@ -600,7 +544,7 @@ int channel_stop_all(int threads_to_stop, long timeout,
                      std::string *error_message) {
   Master_info *mi = nullptr;
 
-  /* Error related variables */
+  /* Error related varaiables */
   int error = 0;
   std::stringstream err_msg_ss;
   err_msg_ss << "Error stopping channel(s): ";
@@ -644,12 +588,12 @@ int channel_stop_all(int threads_to_stop, long timeout,
 
 class Kill_binlog_dump : public Do_THD_Impl {
  public:
-  Kill_binlog_dump() = default;
+  Kill_binlog_dump() {}
 
-  void operator()(THD *thd_to_kill) override {
+  virtual void operator()(THD *thd_to_kill) {
     if (thd_to_kill->get_command() == COM_BINLOG_DUMP ||
         thd_to_kill->get_command() == COM_BINLOG_DUMP_GTID) {
-      assert(thd_to_kill != current_thd);
+      DBUG_ASSERT(thd_to_kill != current_thd);
       MUTEX_LOCK(thd_data_lock, &thd_to_kill->LOCK_thd_data);
       thd_to_kill->duplicate_slave_id = true;
       thd_to_kill->awake(THD::KILL_CONNECTION);
@@ -716,25 +660,24 @@ bool channel_is_active(const char *channel,
     case CHANNEL_APPLIER_THREAD:
       return thread_mask & SLAVE_SQL;
     default:
-      assert(0);
+      DBUG_ASSERT(0);
   }
   return false;
 }
 
 int channel_get_thread_id(const char *channel,
                           enum_channel_thread_types thd_type,
-                          unsigned long **thread_id, bool need_lock) {
+                          unsigned long **thread_id) {
   DBUG_TRACE;
 
   int number_threads = -1;
 
-  if (need_lock) channel_map.rdlock();
-  channel_map.assert_some_lock();
+  channel_map.rdlock();
 
   Master_info *mi = channel_map.get_mi(channel);
 
   if (mi == nullptr) {
-    if (need_lock) channel_map.unlock();
+    channel_map.unlock();
     return RPL_CHANNEL_SERVICE_CHANNEL_DOES_NOT_EXISTS_ERROR;
   }
 
@@ -753,7 +696,7 @@ int channel_get_thread_id(const char *channel,
       if (mi->rli != nullptr) {
         mysql_mutex_lock(&mi->rli->run_lock);
 
-        if (mi->rli->replica_parallel_workers > 0) {
+        if (mi->rli->slave_parallel_workers > 0) {
           // Parallel applier.
           size_t num_workers = mi->rli->get_worker_count();
           number_threads = 1 + num_workers;
@@ -809,7 +752,7 @@ int channel_get_thread_id(const char *channel,
       break;
   }
 
-  if (need_lock) channel_map.unlock();
+  channel_map.unlock();
 
   return number_threads;
 }
@@ -828,17 +771,17 @@ long long channel_get_last_delivered_gno(const char *channel, int sidno) {
 
   rpl_gno last_gno = 0;
 
-  Checkable_rwlock *tsid_lock = mi->rli->get_tsid_lock();
-  tsid_lock->rdlock();
+  Checkable_rwlock *sid_lock = mi->rli->get_sid_lock();
+  sid_lock->rdlock();
   last_gno = mi->rli->get_gtid_set()->get_last_gno(sidno);
-  tsid_lock->unlock();
+  sid_lock->unlock();
 
-#if !defined(NDEBUG)
+#if !defined(DBUG_OFF)
   const Gtid_set *retrieved_gtid_set = mi->rli->get_gtid_set();
   char *retrieved_gtid_set_string = nullptr;
-  tsid_lock->wrlock();
+  sid_lock->wrlock();
   retrieved_gtid_set->to_string(&retrieved_gtid_set_string);
-  tsid_lock->unlock();
+  sid_lock->unlock();
   DBUG_PRINT("info", ("get_last_delivered_gno retrieved_set_string: %s",
                       retrieved_gtid_set_string));
   my_free(retrieved_gtid_set_string);
@@ -859,12 +802,12 @@ int channel_add_executed_gtids_to_received_gtids(const char *channel) {
     return RPL_CHANNEL_SERVICE_CHANNEL_DOES_NOT_EXISTS_ERROR;
   }
 
-  global_tsid_lock->wrlock();
+  global_sid_lock->wrlock();
 
   enum_return_status return_status =
       mi->rli->add_gtid_set(gtid_state->get_executed_gtids());
 
-  global_tsid_lock->unlock();
+  global_sid_lock->unlock();
   channel_map.unlock();
 
   return return_status != RETURN_STATUS_OK;
@@ -907,16 +850,16 @@ int channel_wait_until_apply_queue_applied(const char *channel,
   channel_map.unlock();
 
   /*
-    The retrieved_gtid_set (rli->get_gtid_set) has its own tsid_map/tsid_lock
-    and do not use global_tsid_map/global_tsid_lock. Instead of blocking both
+    The retrieved_gtid_set (rli->get_gtid_set) has its own sid_map/sid_lock
+    and do not use global_sid_map/global_sid_lock. Instead of blocking both
     sid locks on each wait iteration at rli->wait_for_gtid_set(Gtid_set), it
     would be better to use rli->wait_for_gtid_set(char *) that will create a
-    new Gtid_set based on global_tsid_map.
+    new Gtid_set based on global_sid_map.
   */
   char *retrieved_gtid_set_buf;
-  mi->rli->get_tsid_lock()->wrlock();
+  mi->rli->get_sid_lock()->wrlock();
   mi->rli->get_gtid_set()->to_string(&retrieved_gtid_set_buf);
-  mi->rli->get_tsid_lock()->unlock();
+  mi->rli->get_sid_lock()->unlock();
 
   int error = mi->rli->wait_for_gtid_set(current_thd, retrieved_gtid_set_buf,
                                          timeout, false);
@@ -972,8 +915,8 @@ int channel_is_applier_waiting(const char *channel) {
   }
 
   unsigned long *thread_ids = nullptr;
-  int number_appliers = channel_get_thread_id(channel, CHANNEL_APPLIER_THREAD,
-                                              &thread_ids, false);
+  int number_appliers =
+      channel_get_thread_id(channel, CHANNEL_APPLIER_THREAD, &thread_ids);
 
   if (number_appliers <= 0) {
     goto end;
@@ -1010,16 +953,19 @@ int channel_is_applier_thread_waiting(unsigned long thread_id, bool worker) {
   int result = -1;
 
   Find_thd_with_id find_thd_with_id(thread_id);
-  THD_ptr thd_ptr =
-      Global_THD_manager::get_instance()->find_thd(&find_thd_with_id);
-  if (thd_ptr) {
-    if (thd_ptr->get_current_stage_key() ==
-        (worker ? stage_replica_waiting_event_from_coordinator
-                : stage_replica_has_read_all_relay_log)
-            .m_key)
-      result = 1;
-    else
-      result = 0;
+  THD *thd = Global_THD_manager::get_instance()->find_thd(&find_thd_with_id);
+  if (thd) {
+    result = 0;
+
+    const char *proc_info = thd->get_proc_info();
+    if (proc_info) {
+      const char *stage_name = stage_slave_has_read_all_relay_log.m_name;
+      if (worker)
+        stage_name = stage_slave_waiting_event_from_coordinator.m_name;
+
+      if (!strcmp(proc_info, stage_name)) result = 1;
+    }
+    mysql_mutex_unlock(&thd->LOCK_thd_data);
   }
 
   return result;
@@ -1069,59 +1015,28 @@ int channel_get_retrieved_gtid_set(const char *channel, char **retrieved_set) {
   return error;
 }
 
-int channel_get_credentials(const char *channel, std::string &username,
-                            std::string &password) {
-  DBUG_TRACE;
-  String_set user_store, pass_store, auth_store;
-
-  if (!Rpl_channel_credentials::get_instance().get_credentials(
-          channel, user_store, pass_store, auth_store)) {
-    if (user_store.first) username = user_store.second;
-    if (pass_store.first) password = pass_store.second;
-    return 0;
-  }
+int channel_get_credentials(const char *channel, const char **user, char **pass,
+                            size_t *pass_size) {
+  DBUG_ENTER("channel_get_credentials(channel,user,password, pass_size)");
 
   channel_map.rdlock();
+
   Master_info *mi = channel_map.get_mi(channel);
 
   if (mi == nullptr) {
     channel_map.unlock();
-    return RPL_CHANNEL_SERVICE_CHANNEL_DOES_NOT_EXISTS_ERROR;
+    DBUG_RETURN(RPL_CHANNEL_SERVICE_CHANNEL_DOES_NOT_EXISTS_ERROR);
   }
 
   mi->inc_reference();
   channel_map.unlock();
 
-  char pass[MAX_PASSWORD_LENGTH + 1];
-  size_t pass_size;
-  mi->get_password(pass, &pass_size);
-  username.assign(mi->get_user());
-  password.assign(pass, pass_size);
+  *user = mi->get_user();
+  mi->get_password(*pass, pass_size);
 
   mi->dec_reference();
 
-  return 0;
-}
-
-int channel_get_network_namespace(const char *channel, std::string &net_ns) {
-  DBUG_TRACE;
-
-  channel_map.rdlock();
-  Master_info *mi = channel_map.get_mi(channel);
-
-  if (mi == nullptr) {
-    channel_map.unlock();
-    return RPL_CHANNEL_SERVICE_CHANNEL_DOES_NOT_EXISTS_ERROR;
-  }
-
-  mi->inc_reference();
-  channel_map.unlock();
-
-  net_ns.assign(mi->network_namespace_str());
-
-  mi->dec_reference();
-
-  return 0;
+  DBUG_RETURN(0);
 }
 
 bool channel_is_stopping(const char *channel,
@@ -1146,7 +1061,7 @@ bool channel_is_stopping(const char *channel,
       is_stopping = likely(mi->rli->atomic_is_stopping);
       break;
     default:
-      assert(0);
+      DBUG_ASSERT(0);
   }
 
   channel_map.unlock();
@@ -1167,29 +1082,6 @@ bool is_partial_transaction_on_channel_relay_log(const char *channel) {
   return ret;
 }
 
-bool channel_has_same_uuid_as_group_name(const char *group_name) {
-  DBUG_TRACE;
-  Master_info *mi = nullptr;
-  channel_map.rdlock();
-  raii::Sentry<> map_lock_sentry{[&]() -> void { channel_map.unlock(); }};
-
-  for (mi_map::iterator it = channel_map.begin(); it != channel_map.end();
-       it++) {
-    mi = it->second;
-    if (mi != nullptr &&
-        mi->rli->m_assign_gtids_to_anonymous_transactions_info.get_type() >
-            Assign_gtids_to_anonymous_transactions_info::enum_type::AGAT_OFF) {
-      if (!(strcmp((mi->rli->m_assign_gtids_to_anonymous_transactions_info
-                        .get_value())
-                       .data(),
-                   group_name))) {
-        return true;
-      }
-    }
-  }
-  return false;
-}
-
 bool is_any_slave_channel_running(int thread_mask) {
   DBUG_TRACE;
   Master_info *mi = nullptr;
@@ -1228,47 +1120,6 @@ bool is_any_slave_channel_running(int thread_mask) {
   return false;
 }
 
-bool is_any_slave_channel_running_with_failover_enabled(int thread_mask) {
-  DBUG_TRACE;
-  Master_info *mi = nullptr;
-  bool is_running;
-
-  channel_map.rdlock();
-
-  for (mi_map::iterator it = channel_map.begin(); it != channel_map.end();
-       it++) {
-    mi = it->second;
-
-    if (mi && Master_info::is_configured(mi) &&
-        mi->is_source_connection_auto_failover()) {
-      if ((thread_mask & SLAVE_IO) != 0) {
-        mysql_mutex_lock(&mi->run_lock);
-        is_running = mi->slave_running;
-        mysql_mutex_unlock(&mi->run_lock);
-        if (is_running) {
-          channel_map.unlock();
-          return true;
-        }
-      }
-
-      if ((thread_mask & SLAVE_SQL) != 0) {
-        mysql_mutex_lock(&mi->rli->run_lock);
-        is_running = mi->rli->slave_running;
-        mysql_mutex_unlock(&mi->rli->run_lock);
-        if (is_running) {
-          channel_map.unlock();
-          return true;
-        }
-      }
-    }
-  }
-
-  assert(!Source_IO_monitor::get_instance()->is_monitoring_process_running());
-
-  channel_map.unlock();
-  return false;
-}
-
 enum_slave_channel_status
 has_any_slave_channel_open_temp_table_or_is_its_applier_running() {
   DBUG_TRACE;
@@ -1319,119 +1170,3 @@ has_any_slave_channel_open_temp_table_or_is_its_applier_running() {
 
   return SLAVE_CHANNEL_NO_APPLIER_RUNNING_AND_NO_OPEN_TEMPORARY_TABLE;
 }
-
-int channel_delete_credentials(const char *channel_name) {
-  DBUG_TRACE;
-  return Rpl_channel_credentials::get_instance().delete_credentials(
-      channel_name);
-}
-
-bool start_failover_channels() {
-  DBUG_TRACE;
-  bool error = false;
-  channel_map.wrlock();
-
-  for (mi_map::iterator it = channel_map.begin();
-       !error && it != channel_map.end(); it++) {
-    Master_info *mi = it->second;
-    if (Master_info::is_configured(mi) &&
-        mi->is_source_connection_auto_failover()) {
-      Channel_connection_info info;
-      initialize_channel_connection_info(&info);
-
-      int thread_mask = 0;
-      thread_mask |= CHANNEL_APPLIER_THREAD;
-      thread_mask |= CHANNEL_RECEIVER_THREAD;
-
-      DBUG_EXECUTE_IF("force_error_on_start_failover_channels", {
-        channel_map.unlock();
-        return 1;
-      });
-      error = channel_start(mi->get_channel(), &info, thread_mask, false,
-                            true /* use_server_mta_configuration*/,
-                            true /* channel_map_already_locked */);
-    }
-  }
-
-  channel_map.unlock();
-  return error;
-}
-
-bool channel_change_source_connection_auto_failover(const char *channel,
-                                                    bool status) {
-  bool error = false;
-  channel_map.assert_some_wrlock();
-
-  Master_info *mi = channel_map.get_mi(channel);
-  if (!Master_info::is_configured(mi)) {
-    LogErr(ERROR_LEVEL, ER_GRP_RPL_FAILOVER_CONF_CHANNEL_DOES_NOT_EXIST,
-           channel);
-    return true;
-  }
-
-  mi->channel_wrlock();
-  lock_slave_threads(mi);
-
-  if (status && !mi->is_source_connection_auto_failover()) {
-    mi->set_source_connection_auto_failover();
-    error |= (flush_master_info(mi, true, true, false) != 0);
-  }
-
-  if (!status && mi->is_source_connection_auto_failover()) {
-    mi->unset_source_connection_auto_failover();
-    error |= (flush_master_info(mi, true, true, false) != 0);
-  }
-
-  unlock_slave_threads(mi);
-  mi->channel_unlock();
-
-  return error;
-}
-
-bool unset_source_connection_auto_failover_on_all_channels() {
-  channel_map.assert_some_wrlock();
-  bool error = false;
-
-  for (mi_map::iterator it = channel_map.begin();
-       !error && it != channel_map.end(); it++) {
-    Master_info *mi = it->second;
-    if (Master_info::is_configured(mi) &&
-        mi->is_source_connection_auto_failover()) {
-      error |= channel_change_source_connection_auto_failover(mi->get_channel(),
-                                                              false);
-    }
-  }
-
-  return error;
-}
-
-void reload_failover_channels_status() {
-  DBUG_TRACE;
-  channel_map.rdlock();
-  rpl_acf_configuration_handler->reload_failover_channels_status();
-  channel_map.unlock();
-}
-
-bool get_replication_failover_channels_configuration(
-    std::string &serialized_configuration) {
-  DBUG_TRACE;
-  return rpl_acf_configuration_handler->get_configuration(
-      serialized_configuration);
-}
-
-bool set_replication_failover_channels_configuration(
-    const std::vector<std::string>
-        &exchanged_replication_failover_channels_serialized_configuration) {
-  DBUG_TRACE;
-  channel_map.wrlock();
-  bool error = rpl_acf_configuration_handler->set_configuration(
-      exchanged_replication_failover_channels_serialized_configuration);
-  channel_map.unlock();
-  return error;
-}
-
-bool force_my_replication_failover_channels_configuration_on_all_members() {
-  DBUG_TRACE;
-  return rpl_acf_configuration_handler
-      ->force_my_replication_failover_channels_configuration_on_all_members();
-}
