@@ -1,4 +1,4 @@
-/* Copyright (c) 2012, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2012, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -26,14 +26,13 @@
 #include <algorithm>
 #include <atomic>
 
+#include "libbinlogevents/include/control_events.h"
 #include "m_string.h"
 #include "my_dbug.h"
 #include "my_inttypes.h"
 #include "my_macros.h"
 #include "my_thread.h"
-#include "mysql/binlog/event/control_events.h"
 #include "mysql/psi/mysql_mutex.h"
-#include "nulls.h"
 #include "sql/rpl_gtid.h"
 #include "typelib.h"
 
@@ -54,10 +53,47 @@ struct mysql_mutex_t;
 #include "client/mysqlbinlog.h"
 #endif
 
-#include "sql/log.h"
+// Todo: move other global gtid variable declarations here.
+Checkable_rwlock *gtid_mode_lock = nullptr;
+std::atomic<ulong> gtid_mode_counter;
 
-using mysql::gtid::Tsid;
-using mysql::utils::Return_status;
+ulong _gtid_mode;
+const char *gtid_mode_names[] = {"OFF", "OFF_PERMISSIVE", "ON_PERMISSIVE", "ON",
+                                 NullS};
+TYPELIB gtid_mode_typelib = {array_elements(gtid_mode_names) - 1, "",
+                             gtid_mode_names, nullptr};
+
+#ifdef MYSQL_SERVER
+enum_gtid_mode get_gtid_mode(enum_gtid_mode_lock have_lock) {
+  switch (have_lock) {
+    case GTID_MODE_LOCK_NONE:
+      global_sid_lock->rdlock();
+      break;
+    case GTID_MODE_LOCK_SID:
+      global_sid_lock->assert_some_lock();
+      break;
+    case GTID_MODE_LOCK_CHANNEL_MAP:
+      channel_map.assert_some_lock();
+      break;
+    case GTID_MODE_LOCK_GTID_MODE:
+      gtid_mode_lock->assert_some_lock();
+
+      /*
+        This lock is currently not used explicitly by any of the places
+        that calls get_gtid_mode.  Still it would be valid for a caller to
+        use it to protect reads of GTID_MODE, so we keep the code here in
+        case it is needed in the future.
+
+        case GTID_MODE_LOCK_LOG:
+          mysql_mutex_assert_owner(mysql_bin_log.get_log_lock());
+          break;
+      */
+  }
+  enum_gtid_mode ret = (enum_gtid_mode)_gtid_mode;
+  if (have_lock == GTID_MODE_LOCK_NONE) global_sid_lock->unlock();
+  return ret;
+}
+#endif
 
 ulong _gtid_consistency_mode;
 const char *gtid_consistency_mode_names[] = {"OFF", "ON", "WARN", NullS};
@@ -67,173 +103,83 @@ TYPELIB gtid_consistency_mode_typelib = {
 
 #ifdef MYSQL_SERVER
 enum_gtid_consistency_mode get_gtid_consistency_mode() {
-  global_tsid_lock->assert_some_lock();
+  global_sid_lock->assert_some_lock();
   return (enum_gtid_consistency_mode)_gtid_consistency_mode;
 }
 #endif
 
-std::size_t Gtid::skip_whitespace(const char *text, std::size_t pos) {
+enum_return_status Gtid::parse(Sid_map *sid_map, const char *text) {
   DBUG_TRACE;
-  while (my_isspace(&my_charset_utf8mb3_general_ci, text[pos])) {
-    ++pos;
-  }
-  return pos;
-}
+  rpl_sid sid;
+  const char *s = text;
 
-std::tuple<Return_status, rpl_sid, std::size_t> Gtid::parse_sid_str(
-    const char *text, std::size_t pos) {
-  DBUG_TRACE;
-  Return_status status = Return_status::ok;
-  rpl_sid sid{};
-  sid.clear();
-  pos = skip_whitespace(text, pos);
-  if (sid.parse(&text[pos], mysql::gtid::Uuid::TEXT_LENGTH) == 0) {
-    pos += mysql::gtid::Uuid::TEXT_LENGTH;
-    pos = skip_whitespace(text, pos);
-  } else {
-    DBUG_PRINT("info", ("not a uuid at char %d in '%s'", (int)pos, text));
-    status = Return_status::error;
-  }
-  return std::make_tuple(status, sid, pos);
-}
+  SKIP_WHITESPACE();
 
-std::pair<Gtid::Tag, std::size_t> Gtid::parse_tag_str(const char *text,
-                                                      std::size_t pos) {
-  DBUG_TRACE;
-  Gtid::Tag tag;
-  pos = skip_whitespace(text, pos);
-  pos += tag.from_cstring(text + pos);
-  pos = skip_whitespace(text, pos);
-  return std::make_pair(tag, pos);
-}
+  // parse sid
+  if (sid.parse(s, binary_log::Uuid::TEXT_LENGTH) == 0) {
+    rpl_sidno sidno_var = sid_map->add_sid(sid);
+    if (sidno_var <= 0) RETURN_REPORTED_ERROR;
+    s += binary_log::Uuid::TEXT_LENGTH;
 
-std::tuple<Return_status, rpl_gno, std::size_t> Gtid::parse_gno_str(
-    const char *text, std::size_t pos) {
-  DBUG_TRACE;
-  auto status = Return_status::ok;
-  const char *text_cpy = text + pos;
-  rpl_gno gno_var = parse_gno(&text_cpy);
-  pos = text_cpy - text;
-  if (gno_var > 0) {
-    pos = skip_whitespace(text, pos);
-    if (text[pos] != '\0') {
-      DBUG_PRINT("info", ("expected end of string, found garbage '%.80s' "
-                          "at char %d in '%s'",
-                          text_cpy, (int)pos, text));
-      status = Return_status::error;
-      gno_var = 0;
-    }
-  } else {
+    SKIP_WHITESPACE();
+
+    // parse colon
+    if (*s == ':') {
+      s++;
+
+      SKIP_WHITESPACE();
+
+      // parse gno
+      rpl_gno gno_var = parse_gno(&s);
+      if (gno_var > 0) {
+        SKIP_WHITESPACE();
+        if (*s == '\0') {
+          sidno = sidno_var;
+          gno = gno_var;
+          RETURN_OK;
+        } else
+          DBUG_PRINT("info", ("expected end of string, found garbage '%.80s' "
+                              "at char %d in '%s'",
+                              s, (int)(s - text), text));
+      } else
+        DBUG_PRINT("info", ("GNO was zero or invalid (%lld) at char %d in '%s'",
+                            gno_var, (int)(s - text), text));
+    } else
+      DBUG_PRINT("info",
+                 ("missing colon at char %d in '%s'", (int)(s - text), text));
+  } else
     DBUG_PRINT("info",
-               ("GNO was zero or invalid (%" PRId64 ") at char %d in '%s'",
-                gno_var, (int)pos, text));
-    status = Return_status::error;
-  }
-  return std::make_tuple(status, gno_var, pos);
-}
-
-void Gtid::report_parsing_error(const char *text) {
-  DBUG_TRACE;
+               ("not a uuid at char %d in '%s'", (int)(s - text), text));
   BINLOG_ERROR(("Malformed GTID specification: %.200s", text),
                (ER_MALFORMED_GTID_SPECIFICATION, MYF(0), text));
+  RETURN_REPORTED_ERROR;
 }
 
-std::pair<Return_status, std::size_t> Gtid::parse_gtid_separator(
-    const char *text, std::size_t pos) {
+int Gtid::to_string(const rpl_sid &sid, char *buf) const {
   DBUG_TRACE;
-  auto status = Return_status::ok;
-  pos = skip_whitespace(text, pos);
-  if (text[pos] != gtid_separator) {
-    DBUG_PRINT("info", ("missing colon at char %d in '%s'", (int)pos, text));
-    status = Return_status::error;
-  } else {
-    pos = skip_whitespace(text, pos + 1);
-  }
-  return std::make_pair(status, pos);
+  char *s = buf + sid.to_string(buf);
+  *s = ':';
+  s++;
+  s += format_gno(s, gno);
+  return (int)(s - buf);
 }
 
-std::pair<Return_status, mysql::gtid::Gtid> Gtid::parse_gtid_from_cstring(
-    const char *text) {
-  DBUG_TRACE;
-  auto invalid_gtid = std::make_pair(Return_status::error, mysql::gtid::Gtid());
-
-  auto [status, uuid, pos] = parse_sid_str(text, 0);
-  if (status != Return_status::ok) {
-    return invalid_gtid;
-  }
-  std::tie(status, pos) = parse_gtid_separator(text, pos);
-  if (status != Return_status::ok) {
-    return invalid_gtid;
-  }
-  mysql::gtid::Tag tag;
-  std::tie(tag, pos) = parse_tag_str(text, pos);
-  if (tag.is_defined()) {
-    std::tie(status, pos) = parse_gtid_separator(text, pos);
-    if (status != Return_status::ok) {
-      return invalid_gtid;
-    }
-  }
-  rpl_gno gno;
-  std::tie(status, gno, pos) = parse_gno_str(text, pos);
-  if (status != Return_status::ok) {
-    return invalid_gtid;
-  }
-  return std::make_pair(Return_status::ok,
-                        mysql::gtid::Gtid(mysql::gtid::Tsid(uuid, tag), gno));
-}
-
-Return_status Gtid::parse(Tsid_map *tsid_map, const char *text) {
-  DBUG_TRACE;
-  auto [status, parsed_gtid] = parse_gtid_from_cstring(text);
-  if (status == Return_status::error) {
-    report_parsing_error(text);
-    return status;
-  }
-  rpl_sidno sidno_var = tsid_map->add_tsid(parsed_gtid.get_tsid());
-  if (sidno_var <= 0) {
-    status = Return_status::error;
-  }
-  rpl_gno gno_var = parsed_gtid.get_gno();
-  if (status == Return_status::ok) {
-    sidno = sidno_var;
-    gno = gno_var;
-  }
-  return status;
-}
-
-int Gtid::to_string_gno(char *buf) const {
-  DBUG_TRACE;
-  int id = 0;
-  *buf = ':';
-  ++id;
-  id += format_gno(buf + id, gno);
-  return id;
-}
-
-int Gtid::to_string(const Tsid &tsid, char *buf) const {
-  DBUG_TRACE;
-  int id = 0;
-  id += tsid.to_string(buf);
-  id += to_string_gno(buf + id);
-  return id;
-}
-
-int Gtid::to_string(const Tsid_map *tsid_map, char *buf, bool need_lock) const {
+int Gtid::to_string(const Sid_map *sid_map, char *buf, bool need_lock) const {
   DBUG_TRACE;
   int ret;
-  if (tsid_map != nullptr) {
-    Checkable_rwlock *lock = tsid_map->get_tsid_lock();
+  if (sid_map != nullptr) {
+    Checkable_rwlock *lock = sid_map->get_sid_lock();
     if (lock) {
       if (need_lock)
         lock->rdlock();
       else
         lock->assert_some_lock();
     }
-    const auto tsid = tsid_map->sidno_to_tsid(sidno);
+    const rpl_sid &sid = sid_map->sidno_to_sid(sidno);
     if (lock && need_lock) lock->unlock();
-    ret = to_string(tsid, buf);
+    ret = to_string(sid, buf);
   } else {
-#ifdef NDEBUG
+#ifdef DBUG_OFF
     /*
       NULL is only allowed in debug mode, since the sidno does not
       make sense for users but is useful to include in debug
@@ -241,54 +187,79 @@ int Gtid::to_string(const Tsid_map *tsid_map, char *buf, bool need_lock) const {
       Since there is no ASSERT in non-debug mode, we use abort
       instead.
     */
-    my_abort();
+    abort();
 #endif
-    ret = sprintf(buf, "%d:%" PRId64, sidno, gno);
+    ret = sprintf(buf, "%d:%lld", sidno, gno);
   }
   return ret;
 }
 
 bool Gtid::is_valid(const char *text) {
   DBUG_TRACE;
-  Return_status status;
-  std::tie(status, std::ignore) = parse_gtid_from_cstring(text);
-  return status == Return_status::ok;
+  const char *s = text;
+  SKIP_WHITESPACE();
+  if (!rpl_sid::is_valid(s, binary_log::Uuid::TEXT_LENGTH)) {
+    DBUG_PRINT("info",
+               ("not a uuid at char %d in '%s'", (int)(s - text), text));
+    return false;
+  }
+  s += binary_log::Uuid::TEXT_LENGTH;
+  SKIP_WHITESPACE();
+  if (*s != ':') {
+    DBUG_PRINT("info",
+               ("missing colon at char %d in '%s'", (int)(s - text), text));
+    return false;
+  }
+  s++;
+  SKIP_WHITESPACE();
+  if (parse_gno(&s) <= 0) {
+    DBUG_PRINT("info", ("GNO was zero or invalid at char %d in '%s'",
+                        (int)(s - text), text));
+    return false;
+  }
+  SKIP_WHITESPACE();
+  if (*s != 0) {
+    DBUG_PRINT("info", ("expected end of string, found garbage '%.80s' "
+                        "at char %d in '%s'",
+                        s, (int)(s - text), text));
+    return false;
+  }
+  return true;
 }
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
 void check_return_status(enum_return_status status, const char *action,
                          const char *status_name, int allow_unreported) {
   if (status != RETURN_STATUS_OK) {
-    assert(allow_unreported || status == RETURN_STATUS_REPORTED_ERROR);
+    DBUG_ASSERT(allow_unreported || status == RETURN_STATUS_REPORTED_ERROR);
     if (status == RETURN_STATUS_REPORTED_ERROR) {
-#if defined(MYSQL_SERVER) && !defined(NDEBUG)
+#if defined(MYSQL_SERVER) && !defined(DBUG_OFF)
       THD *thd = current_thd;
       /*
         We create a new system THD with 'SYSTEM_THREAD_COMPRESS_GTID_TABLE'
         when initializing gtid state by fetching gtids during server startup,
         so we can check on it before diagnostic area is active and skip the
         assert in this case. We assert that diagnostic area logged the error
-        outside server startup since the assert is really useful.
+        outside server startup since the assert is realy useful.
      */
-      assert(thd == nullptr ||
-             thd->get_stmt_da()->status() == Diagnostics_area::DA_ERROR ||
-             (thd->get_stmt_da()->status() == Diagnostics_area::DA_EMPTY &&
-              thd->system_thread == SYSTEM_THREAD_COMPRESS_GTID_TABLE));
+      DBUG_ASSERT(thd == nullptr ||
+                  thd->get_stmt_da()->status() == Diagnostics_area::DA_ERROR ||
+                  (thd->get_stmt_da()->status() == Diagnostics_area::DA_EMPTY &&
+                   thd->system_thread == SYSTEM_THREAD_COMPRESS_GTID_TABLE));
 #endif
     }
     DBUG_PRINT("info", ("%s error %d (%s)", action, status, status_name));
   }
 }
-#endif  // ! NDEBUG
+#endif  // ! DBUG_OFF
 
 #ifdef MYSQL_SERVER
-
-rpl_sidno get_sidno_from_global_tsid_map(const Tsid &tsid) {
+rpl_sidno get_sidno_from_global_sid_map(rpl_sid sid) {
   DBUG_TRACE;
 
-  global_tsid_lock->rdlock();
-  rpl_sidno sidno = global_tsid_map->add_tsid(tsid);
-  global_tsid_lock->unlock();
+  global_sid_lock->rdlock();
+  rpl_sidno sidno = global_sid_map->add_sid(sid);
+  global_sid_lock->unlock();
 
   return sidno;
 }
@@ -296,9 +267,9 @@ rpl_sidno get_sidno_from_global_tsid_map(const Tsid &tsid) {
 rpl_gno get_last_executed_gno(rpl_sidno sidno) {
   DBUG_TRACE;
 
-  global_tsid_lock->rdlock();
+  global_sid_lock->rdlock();
   rpl_gno gno = gtid_state->get_last_executed_gno(sidno);
-  global_tsid_lock->unlock();
+  global_sid_lock->unlock();
 
   return gno;
 }
@@ -337,22 +308,22 @@ void Trx_monitoring_info::clear() {
   last_transient_error_timestamp = 0;
   transaction_retries = 0;
   is_retrying = false;
-  compression_type = mysql::binlog::event::compression::type::NONE;
+  compression_type = binary_log::transaction::compression::type::NONE;
   compressed_bytes = 0;
   uncompressed_bytes = 0;
 }
 
-void Trx_monitoring_info::copy_to_ps_table(Tsid_map *tsid_map, char *gtid_arg,
+void Trx_monitoring_info::copy_to_ps_table(Sid_map *sid_map, char *gtid_arg,
                                            uint *gtid_length_arg,
                                            ulonglong *original_commit_ts_arg,
                                            ulonglong *immediate_commit_ts_arg,
-                                           ulonglong *start_time_arg) const {
-  assert(tsid_map);
-  assert(gtid_arg);
-  assert(gtid_length_arg);
-  assert(original_commit_ts_arg);
-  assert(immediate_commit_ts_arg);
-  assert(start_time_arg);
+                                           ulonglong *start_time_arg) {
+  DBUG_ASSERT(sid_map);
+  DBUG_ASSERT(gtid_arg);
+  DBUG_ASSERT(gtid_length_arg);
+  DBUG_ASSERT(original_commit_ts_arg);
+  DBUG_ASSERT(immediate_commit_ts_arg);
+  DBUG_ASSERT(start_time_arg);
 
   if (is_info_set) {
     // The trx_monitoring_info is populated
@@ -362,14 +333,14 @@ void Trx_monitoring_info::copy_to_ps_table(Tsid_map *tsid_map, char *gtid_arg,
       *gtid_length_arg = 9;
     } else {
       // The GTID is set
-      Checkable_rwlock *tsid_lock = tsid_map->get_tsid_lock();
-      tsid_lock->rdlock();
-      *gtid_length_arg = gtid.to_string(tsid_map, gtid_arg);
-      tsid_lock->unlock();
+      Checkable_rwlock *sid_lock = sid_map->get_sid_lock();
+      sid_lock->rdlock();
+      *gtid_length_arg = gtid.to_string(sid_map, gtid_arg);
+      sid_lock->unlock();
     }
     *original_commit_ts_arg = original_commit_timestamp;
     *immediate_commit_ts_arg = immediate_commit_timestamp;
-    *start_time_arg = start_time;
+    *start_time_arg = start_time / 10;
   } else {
     // This monitoring info is not populated, so let's zero the input
     memcpy(gtid_arg, "", 1);
@@ -380,36 +351,36 @@ void Trx_monitoring_info::copy_to_ps_table(Tsid_map *tsid_map, char *gtid_arg,
   }
 }
 
-void Trx_monitoring_info::copy_to_ps_table(Tsid_map *tsid_map, char *gtid_arg,
+void Trx_monitoring_info::copy_to_ps_table(Sid_map *sid_map, char *gtid_arg,
                                            uint *gtid_length_arg,
                                            ulonglong *original_commit_ts_arg,
                                            ulonglong *immediate_commit_ts_arg,
                                            ulonglong *start_time_arg,
-                                           ulonglong *end_time_arg) const {
-  assert(end_time_arg);
+                                           ulonglong *end_time_arg) {
+  DBUG_ASSERT(end_time_arg);
 
-  *end_time_arg = is_info_set ? end_time : 0;
-  copy_to_ps_table(tsid_map, gtid_arg, gtid_length_arg, original_commit_ts_arg,
+  *end_time_arg = is_info_set ? end_time / 10 : 0;
+  copy_to_ps_table(sid_map, gtid_arg, gtid_length_arg, original_commit_ts_arg,
                    immediate_commit_ts_arg, start_time_arg);
 }
 
 void Trx_monitoring_info::copy_to_ps_table(
-    Tsid_map *tsid_map, char *gtid_arg, uint *gtid_length_arg,
+    Sid_map *sid_map, char *gtid_arg, uint *gtid_length_arg,
     ulonglong *original_commit_ts_arg, ulonglong *immediate_commit_ts_arg,
     ulonglong *start_time_arg, uint *last_transient_errno_arg,
     char *last_transient_errmsg_arg, uint *last_transient_errmsg_length_arg,
-    ulonglong *last_transient_timestamp_arg, ulong *retries_count_arg) const {
-  assert(last_transient_errno_arg);
-  assert(last_transient_errmsg_arg);
-  assert(last_transient_errmsg_length_arg);
-  assert(last_transient_timestamp_arg);
-  assert(retries_count_arg);
+    ulonglong *last_transient_timestamp_arg, ulong *retries_count_arg) {
+  DBUG_ASSERT(last_transient_errno_arg);
+  DBUG_ASSERT(last_transient_errmsg_arg);
+  DBUG_ASSERT(last_transient_errmsg_length_arg);
+  DBUG_ASSERT(last_transient_timestamp_arg);
+  DBUG_ASSERT(retries_count_arg);
 
   if (is_info_set) {
     *last_transient_errno_arg = last_transient_error_number;
     strcpy(last_transient_errmsg_arg, last_transient_error_message);
     *last_transient_errmsg_length_arg = strlen(last_transient_error_message);
-    *last_transient_timestamp_arg = last_transient_error_timestamp;
+    *last_transient_timestamp_arg = last_transient_error_timestamp / 10;
     *retries_count_arg = transaction_retries;
   } else {
     *last_transient_errno_arg = 0;
@@ -418,21 +389,21 @@ void Trx_monitoring_info::copy_to_ps_table(
     *last_transient_timestamp_arg = 0;
     *retries_count_arg = 0;
   }
-  copy_to_ps_table(tsid_map, gtid_arg, gtid_length_arg, original_commit_ts_arg,
+  copy_to_ps_table(sid_map, gtid_arg, gtid_length_arg, original_commit_ts_arg,
                    immediate_commit_ts_arg, start_time_arg);
 }
 
 void Trx_monitoring_info::copy_to_ps_table(
-    Tsid_map *tsid_map, char *gtid_arg, uint *gtid_length_arg,
+    Sid_map *sid_map, char *gtid_arg, uint *gtid_length_arg,
     ulonglong *original_commit_ts_arg, ulonglong *immediate_commit_ts_arg,
     ulonglong *start_time_arg, ulonglong *end_time_arg,
     uint *last_transient_errno_arg, char *last_transient_errmsg_arg,
     uint *last_transient_errmsg_length_arg,
-    ulonglong *last_transient_timestamp_arg, ulong *retries_count_arg) const {
-  assert(end_time_arg);
+    ulonglong *last_transient_timestamp_arg, ulong *retries_count_arg) {
+  DBUG_ASSERT(end_time_arg);
 
-  *end_time_arg = is_info_set ? end_time : 0;
-  copy_to_ps_table(tsid_map, gtid_arg, gtid_length_arg, original_commit_ts_arg,
+  *end_time_arg = is_info_set ? end_time / 10 : 0;
+  copy_to_ps_table(sid_map, gtid_arg, gtid_length_arg, original_commit_ts_arg,
                    immediate_commit_ts_arg, start_time_arg,
                    last_transient_errno_arg, last_transient_errmsg_arg,
                    last_transient_errmsg_length_arg,
@@ -465,8 +436,8 @@ void Gtid_monitoring_info::atomic_lock() {
       */
       my_thread_yield();
     }
-#ifndef NDEBUG
-    assert(!is_locked);
+#ifndef DBUG_OFF
+    DBUG_ASSERT(!is_locked);
     is_locked = true;
 #endif
   } else {
@@ -477,8 +448,8 @@ void Gtid_monitoring_info::atomic_lock() {
 
 void Gtid_monitoring_info::atomic_unlock() {
   if (atomic_mutex == nullptr) {
-#ifndef NDEBUG
-    assert(is_locked);
+#ifndef DBUG_OFF
+    DBUG_ASSERT(is_locked);
     is_locked = false;
 #endif
     atomic_locked = false;
@@ -505,7 +476,7 @@ void Gtid_monitoring_info::clear_last_processed_trx() {
   atomic_unlock();
 }
 
-void Gtid_monitoring_info::update(mysql::binlog::event::compression::type t,
+void Gtid_monitoring_info::update(binary_log::transaction::compression::type t,
                                   size_t payload_size,
                                   size_t uncompressed_size) {
   processing_trx->compression_type = t;
@@ -537,7 +508,7 @@ void Gtid_monitoring_info::start(Gtid gtid_arg, ulonglong original_ts_arg,
     processing_trx->last_transient_error_timestamp = 0;
     processing_trx->transaction_retries = 0;
     processing_trx->compression_type =
-        mysql::binlog::event::compression::type::NONE;
+        binary_log::transaction::compression::type::NONE;
     processing_trx->compressed_bytes = 0;
     processing_trx->uncompressed_bytes = 0;
     atomic_unlock();
@@ -596,7 +567,7 @@ const Gtid *Gtid_monitoring_info::get_processing_trx_gtid() {
   /*
     This function is only called by relay log recovery/queuing.
   */
-  assert(atomic_mutex != nullptr);
+  DBUG_ASSERT(atomic_mutex != nullptr);
   mysql_mutex_assert_owner(atomic_mutex);
   return &processing_trx->gtid;
 }
