@@ -1,4 +1,4 @@
-/* Copyright (c) 2013, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -24,24 +24,25 @@
 
 #include "my_config.h"
 
-#include "mysql/my_loglevel.h"
+#include "my_loglevel.h"
 #include "sql/derror.h"
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 #include <list>
 
+#include "libbinlogevents/include/control_events.h"
+#include "m_ctype.h"
+#include "m_string.h"
 #include "my_base.h"
 #include "my_command.h"
 #include "my_dbug.h"
 #include "my_sys.h"
 #include "my_thread.h"
-#include "mysql/binlog/event/control_events.h"
 #include "mysql/components/services/log_builtins.h"
 #include "mysql/psi/mysql_cond.h"
 #include "mysql/psi/mysql_mutex.h"
 #include "mysql/psi/mysql_thread.h"
-#include "mysql/strings/m_ctype.h"
 #include "mysql/thread_type.h"
 #include "sql/auth/sql_security_ctx.h"
 #include "sql/current_thd.h"
@@ -60,7 +61,6 @@
 #include "sql/sql_parse.h"  // mysql_reset_thd_for_next_command
 #include "sql/system_variables.h"
 #include "sql_string.h"
-#include "string_with_len.h"
 
 using std::list;
 using std::string;
@@ -153,7 +153,7 @@ bool Gtid_table_access_context::init(THD **thd, TABLE **table, bool is_write) {
       the main transaction thanks to rejection to update
       'mysql.gtid_executed' by XA main transaction.
     */
-    assert(
+    DBUG_ASSERT(
         (*thd)->get_transaction()->xid_state()->has_state(XID_STATE::XA_IDLE) ||
         (*thd)->get_transaction()->xid_state()->has_state(
             XID_STATE::XA_PREPARED));
@@ -177,7 +177,7 @@ bool Gtid_table_access_context::deinit(THD *thd, TABLE *table, bool error,
 
   /*
     This fails on errors committing the info, or when
-    replica_preserve_commit_order is enabled and a previous transaction
+    slave_preserve_commit_order is enabled and a previous transaction
     has failed.  In both cases, the error is reported already.
   */
   err = this->close_table(thd, table, &m_backup, 0 != error, need_commit);
@@ -185,7 +185,7 @@ bool Gtid_table_access_context::deinit(THD *thd, TABLE *table, bool error,
   /*
     If Gtid is inserted through Attachable_trx_rw its has been done
     in the above close_table() through ha_commit_trans().
-    It does not have any side effect on the global transaction state
+    It does not have any side effect to the global transaction state
     as the only vulnerable part there relates to gtid (and is blocked
     from recursive invocation).
   */
@@ -193,7 +193,7 @@ bool Gtid_table_access_context::deinit(THD *thd, TABLE *table, bool error,
     thd->end_attachable_transaction();
 
   thd->is_operating_gtid_table_implicitly = false;
-  /* Re-enable binlog */
+  /* Reenable binlog */
   if (m_is_write)
     thd->variables.option_bits = m_tmp_disable_binlog__save_options;
   if (this->m_skip_readonly_set) {
@@ -206,14 +206,12 @@ bool Gtid_table_access_context::deinit(THD *thd, TABLE *table, bool error,
 }
 
 int Gtid_table_persistor::fill_fields(Field **fields, const char *sid,
-                                      const char *tag, rpl_gno gno_start,
-                                      rpl_gno gno_end) {
+                                      rpl_gno gno_start, rpl_gno gno_end) {
   DBUG_TRACE;
 
   /* Store SID */
   fields[0]->set_notnull();
-  if (fields[0]->store(sid, mysql::gtid::Uuid::TEXT_LENGTH, &my_charset_bin) !=
-      0U) {
+  if (fields[0]->store(sid, binary_log::Uuid::TEXT_LENGTH, &my_charset_bin)) {
     my_error(ER_RPL_INFO_DATA_TOO_LONG, MYF(0), fields[0]->field_name);
     goto err;
   }
@@ -232,21 +230,13 @@ int Gtid_table_persistor::fill_fields(Field **fields, const char *sid,
     goto err;
   }
 
-  // Store tag
-  fields[3]->set_notnull();
-  if (fields[3]->store(tag, strlen(tag), &my_charset_bin)) {
-    my_error(ER_RPL_INFO_DATA_TOO_LONG, MYF(0), fields[3]->field_name);
-    goto err;
-  }
-
   return 0;
 err:
   return -1;
 }
 
 int Gtid_table_persistor::write_row(TABLE *table, const char *sid,
-                                    const char *tag, rpl_gno gno_start,
-                                    rpl_gno gno_end) {
+                                    rpl_gno gno_start, rpl_gno gno_end) {
   DBUG_TRACE;
   int error = 0;
   Field **fields = nullptr;
@@ -254,7 +244,7 @@ int Gtid_table_persistor::write_row(TABLE *table, const char *sid,
   fields = table->field;
   empty_record(table);
 
-  if (fill_fields(fields, sid, tag, gno_start, gno_end)) return -1;
+  if (fill_fields(fields, sid, gno_start, gno_end)) return -1;
 
   /* Inserts a new row into the gtid_executed table. */
   error = table->file->ha_write_row(table->record[0]);
@@ -278,8 +268,7 @@ int Gtid_table_persistor::write_row(TABLE *table, const char *sid,
 }
 
 int Gtid_table_persistor::update_row(TABLE *table, const char *sid,
-                                     const char *tag, rpl_gno gno_start,
-                                     rpl_gno new_gno_end) {
+                                     rpl_gno gno_start, rpl_gno new_gno_end) {
   DBUG_TRACE;
   int error = 0;
   Field **fields = nullptr;
@@ -290,8 +279,7 @@ int Gtid_table_persistor::update_row(TABLE *table, const char *sid,
 
   /* Store SID */
   fields[0]->set_notnull();
-  if (fields[0]->store(sid, mysql::gtid::Uuid::TEXT_LENGTH, &my_charset_bin) !=
-      0U) {
+  if (fields[0]->store(sid, binary_log::Uuid::TEXT_LENGTH, &my_charset_bin)) {
     my_error(ER_RPL_INFO_DATA_TOO_LONG, MYF(0), fields[0]->field_name);
     return -1;
   }
@@ -303,13 +291,6 @@ int Gtid_table_persistor::update_row(TABLE *table, const char *sid,
     return -1;
   }
 
-  // Store tag, to be able to use the key
-  fields[3]->set_notnull();
-  if (fields[3]->store(tag, strlen(tag), &my_charset_bin)) {
-    my_error(ER_RPL_INFO_DATA_TOO_LONG, MYF(0), fields[3]->field_name);
-    goto end;
-  }
-
   key_copy(user_key, table->record[0], table->key_info,
            table->key_info->key_length);
 
@@ -337,7 +318,6 @@ int Gtid_table_persistor::update_row(TABLE *table, const char *sid,
 
   /* Update a row in the gtid_executed table. */
   error = table->file->ha_update_row(table->record[1], table->record[0]);
-
   if (DBUG_EVALUATE_IF("simulate_error_on_compress_gtid_table", (error = -1),
                        error)) {
     table->file->print_error(error, MYF(0));
@@ -361,39 +341,37 @@ int Gtid_table_persistor::save(THD *thd, const Gtid *gtid) {
   int error = 0;
   TABLE *table = nullptr;
   Gtid_table_access_context table_access_ctx;
+  char buf[binary_log::Uuid::TEXT_LENGTH + 1];
 
   /* Get source id */
-  global_tsid_lock->rdlock();
-  const auto &tsid = global_tsid_map->sidno_to_tsid(gtid->sidno);
-  global_tsid_lock->unlock();
-  std::string sid_str = tsid.get_uuid().to_string();
-  std::string tag_str = tsid.get_tag().to_string();
+  global_sid_lock->rdlock();
+  rpl_sid sid = global_sid_map->sidno_to_sid(gtid->sidno);
+  global_sid_lock->unlock();
+  sid.to_string(buf);
 
   if (table_access_ctx.init(&thd, &table, true)) {
     error = 1;
     goto end;
   }
 
-  /* Write directly to gtid_executed table only to satisfy debug test. */
-  DBUG_EXECUTE_IF("disable_se_persists_gtid", {
-    error = write_row(table, sid_str.c_str(), tag_str.c_str(), gtid->gno,
-                      gtid->gno);
-  });
-
-  thd->request_persist_gtid_by_se();
-
-  DBUG_EXECUTE_IF("simulate_err_on_write_gtid_into_table", {
-    error = -1;
-    table->file->print_error(error, MYF(0));
-    thd->reset_gtid_persisted_by_se();
-  });
+  /* Save the gtid info into table. */
+  error = write_row(table, buf, gtid->gno, gtid->gno);
 
 end:
-  if (table_access_ctx.deinit(thd, table, 0 != error, false)) {
-    thd->reset_gtid_persisted_by_se();
-    error = -1;
+  if (table_access_ctx.deinit(thd, table, 0 != error, false)) error = -1;
+
+  /* Do not protect m_atomic_count for improving transactions' concurrency */
+  if (error == 0 && gtid_executed_compression_period != 0) {
+    uint32 count = (uint32)m_atomic_count++;
+    if (count == gtid_executed_compression_period ||
+        DBUG_EVALUATE_IF("compress_gtid_table", 1, 0)) {
+      mysql_mutex_lock(&LOCK_compress_gtid_table);
+      should_compress = true;
+      mysql_cond_signal(&COND_compress_gtid_table);
+      mysql_mutex_unlock(&LOCK_compress_gtid_table);
+    }
   }
-  /* Compression is triggered by GTID background thread as required. */
+
   return error;
 }
 
@@ -405,11 +383,15 @@ int Gtid_table_persistor::save(const Gtid_set *gtid_set, bool compress) {
   Gtid_table_access_context table_access_ctx;
   THD *thd = current_thd;
 
-  DBUG_EXECUTE_IF("crash_before_gtid_table_persistor_save",
-                  { DBUG_SUICIDE(); });
-
   if (table_access_ctx.init(&thd, &table, true)) {
-    error = ret = 1;
+    error = 1;
+    /*
+      Gtid table is not ready to be used, so failed to
+      open it. Ignore the error.
+    */
+    thd->clear_error();
+    if (!thd->get_stmt_da()->is_set())
+      thd->get_stmt_da()->set_ok_status(0, 0, nullptr);
     goto end;
   }
 
@@ -441,15 +423,13 @@ int Gtid_table_persistor::save(TABLE *table, const Gtid_set *gtid_set) {
   /* Get GTID intervals from gtid_set. */
   gtid_set->get_gtid_intervals(&gtid_intervals);
   for (iter = gtid_intervals.begin(); iter != gtid_intervals.end(); iter++) {
-    /* Get transaction source id. */
-    const auto &tsid = gtid_set->get_tsid_map()->sidno_to_tsid(iter->sidno);
-    auto sid_str = tsid.get_uuid().to_string();
-    auto tag_str = tsid.get_tag().to_string();
+    /* Get source id. */
+    char buf[binary_log::Uuid::TEXT_LENGTH + 1];
+    rpl_sid sid = gtid_set->get_sid_map()->sidno_to_sid(iter->sidno);
+    sid.to_string(buf);
 
     /* Save the gtid interval into table. */
-    if ((error = write_row(table, sid_str.c_str(), tag_str.c_str(),
-                           iter->gno_start, iter->gno_end)))
-      break;
+    if ((error = write_row(table, buf, iter->gno_start, iter->gno_end))) break;
   }
 
   gtid_intervals.clear();
@@ -466,7 +446,7 @@ int Gtid_table_persistor::save(TABLE *table, const Gtid_set *gtid_set) {
     @retval 0    OK.
     @retval -1   Error.
 */
-#ifndef NDEBUG
+#ifndef DBUG_OFF
 static int dbug_test_on_compress(THD *thd) {
   DBUG_TRACE;
   /*
@@ -476,8 +456,8 @@ static int dbug_test_on_compress(THD *thd) {
   DBUG_EXECUTE_IF("fetch_compression_thread_stage_info", sleep(5););
   DBUG_EXECUTE_IF("fetch_compression_thread_stage_info", {
     const char act[] = "now signal fetch_thread_stage";
-    assert(opt_debug_sync_timeout > 0);
-    assert(!debug_sync_set_action(thd, STRING_WITH_LEN(act)));
+    DBUG_ASSERT(opt_debug_sync_timeout > 0);
+    DBUG_ASSERT(!debug_sync_set_action(thd, STRING_WITH_LEN(act)));
   };);
   /* Sleep a little, so that we can always fetch the correct stage info. */
   DBUG_EXECUTE_IF("fetch_compression_thread_stage_info", sleep(1););
@@ -488,8 +468,8 @@ static int dbug_test_on_compress(THD *thd) {
   */
   DBUG_EXECUTE_IF("simulate_crash_on_compress_gtid_table", {
     const char act[] = "now wait_for notified_thread_complete";
-    assert(opt_debug_sync_timeout > 0);
-    assert(!debug_sync_set_action(thd, STRING_WITH_LEN(act)));
+    DBUG_ASSERT(opt_debug_sync_timeout > 0);
+    DBUG_ASSERT(!debug_sync_set_action(thd, STRING_WITH_LEN(act)));
   };);
   DBUG_EXECUTE_IF("simulate_crash_on_compress_gtid_table", DBUG_SUICIDE(););
 
@@ -509,8 +489,8 @@ int Gtid_table_persistor::compress(THD *thd) {
 
   DBUG_EXECUTE_IF("compress_gtid_table", {
     const char act[] = "now signal complete_compression";
-    assert(opt_debug_sync_timeout > 0);
-    assert(!debug_sync_set_action(thd, STRING_WITH_LEN(act)));
+    DBUG_ASSERT(opt_debug_sync_timeout > 0);
+    DBUG_ASSERT(!debug_sync_set_action(thd, STRING_WITH_LEN(act)));
   };);
 
   return error;
@@ -537,7 +517,7 @@ int Gtid_table_persistor::compress_in_single_transaction(THD *thd,
 
   if ((error = compress_first_consecutive_range(table, is_complete))) goto end;
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   error = dbug_test_on_compress(thd);
 #endif
 
@@ -555,16 +535,12 @@ int Gtid_table_persistor::compress_first_consecutive_range(TABLE *table,
   int err = 0;
   /* Record the source id of the first consecutive gtid. */
   string sid;
-  /* Record the tag of the first consecutive gtid. */
-  string tag;
   /* Record the first GNO of the first consecutive gtid. */
   rpl_gno gno_start = 0;
   /* Record the last GNO of the last consecutive gtid. */
   rpl_gno gno_end = 0;
   /* Record the gtid interval of the current gtid. */
   string cur_sid;
-  /* Record the tag of the current gtid. */
-  string cur_tag;
   rpl_gno cur_gno_start = 0;
   rpl_gno cur_gno_end = 0;
   /*
@@ -581,12 +557,12 @@ int Gtid_table_persistor::compress_first_consecutive_range(TABLE *table,
   err = table->file->ha_index_first(table->record[0]);
   /* Compress the first consecutive range of gtids. */
   while (!err) {
-    get_gtid_interval(table, cur_sid, cur_tag, cur_gno_start, cur_gno_end);
+    get_gtid_interval(table, cur_sid, cur_gno_start, cur_gno_end);
     /*
       Check if gtid intervals of previous gtid and current gtid
       are consecutive.
     */
-    if (sid == cur_sid && tag == cur_tag && gno_end + 1 == cur_gno_start) {
+    if (sid == cur_sid && gno_end + 1 == cur_gno_start) {
       find_first_consecutive_gtids = true;
       gno_end = cur_gno_end;
       /* Delete the consecutive gtid. We do not delete the first
@@ -600,7 +576,6 @@ int Gtid_table_persistor::compress_first_consecutive_range(TABLE *table,
 
       /* Record the gtid interval of the first consecutive gtid. */
       sid = cur_sid;
-      tag = cur_tag;
       gno_start = cur_gno_start;
       gno_end = cur_gno_end;
     }
@@ -613,13 +588,12 @@ int Gtid_table_persistor::compress_first_consecutive_range(TABLE *table,
 
   if (err != HA_ERR_END_OF_FILE && err != 0)
     ret = -1;
-  else if (find_first_consecutive_gtids) {
+  else if (find_first_consecutive_gtids)
     /*
       Update the gno_end of the first consecutive gtid with the gno_end of
       the last consecutive gtid for the first consecutive range of gtids.
     */
-    ret = update_row(table, sid.c_str(), tag.c_str(), gno_start, gno_end);
-  }
+    ret = update_row(table, sid.c_str(), gno_start, gno_end);
 
   return ret;
 }
@@ -635,7 +609,7 @@ int Gtid_table_persistor::reset(THD *thd) {
     error = 1;
     goto end;
   }
-  /* Resetting the counter as gtid_executed table will also
+  /* Reseting the counter as gtid_executed table will also
      be emptied.
   */
   m_atomic_count = 0;
@@ -656,22 +630,18 @@ string Gtid_table_persistor::encode_gtid_text(TABLE *table) {
   /* Fetch gtid interval from the table */
   table->field[0]->val_str(&str);
   string gtid_text(str.c_ptr_safe());
-  table->field[3]->val_str(&str);
-  if (str.is_empty() == false) {
-    gtid_text.append(Gtid_set::default_string_format.tag_sid_separator);
-    gtid_text.append(str.c_ptr_safe());
-  }
-  gtid_text.append(Gtid_set::default_string_format.tsid_gno_separator);
+  gtid_text.append(Gtid_set::default_string_format.sid_gno_separator);
   table->field[1]->val_str(&str);
   gtid_text.append(str.c_ptr_safe());
   gtid_text.append(Gtid_set::default_string_format.gno_start_end_separator);
   table->field[2]->val_str(&str);
   gtid_text.append(str.c_ptr_safe());
+
   return gtid_text;
 }
 
 void Gtid_table_persistor::get_gtid_interval(TABLE *table, string &sid,
-                                             string &tag, rpl_gno &gno_start,
+                                             rpl_gno &gno_start,
                                              rpl_gno &gno_end) {
   DBUG_TRACE;
   char buff[MAX_FIELD_WIDTH];
@@ -682,11 +652,6 @@ void Gtid_table_persistor::get_gtid_interval(TABLE *table, string &sid,
   sid = string(str.c_ptr_safe());
   gno_start = table->field[1]->val_int();
   gno_end = table->field[2]->val_int();
-  table->field[3]->val_str(&str);
-  tag.clear();
-  if (str.is_empty() == false) {
-    tag = string(str.c_ptr_safe());
-  }
 }
 
 int Gtid_table_persistor::fetch_gtids(Gtid_set *gtid_set) {
@@ -712,18 +677,18 @@ int Gtid_table_persistor::fetch_gtids(Gtid_set *gtid_set) {
 
     /**
       @todo:
-      - take only global_tsid_lock->rdlock(), and take
-        gtid_state->tsid_lock for each iteration.
+      - take only global_sid_lock->rdlock(), and take
+        gtid_state->sid_lock for each iteration.
       - Add wrapper around Gtid_set::add_gno_interval and call that
         instead.
     */
-    global_tsid_lock->wrlock();
+    global_sid_lock->wrlock();
     if (gtid_set->add_gtid_text(encode_gtid_text(table).c_str()) !=
         RETURN_STATUS_OK) {
-      global_tsid_lock->unlock();
+      global_sid_lock->unlock();
       break;
     }
-    global_tsid_lock->unlock();
+    global_sid_lock->unlock();
   }
 
   table->file->ha_rnd_end();
@@ -814,8 +779,8 @@ static void *compress_gtid_table(void *p_thd) {
         thd->clear_error();
         DBUG_EXECUTE_IF("simulate_error_on_compress_gtid_table", {
           const char act[] = "now signal compression_failed";
-          assert(opt_debug_sync_timeout > 0);
-          assert(!debug_sync_set_action(thd, STRING_WITH_LEN(act)));
+          DBUG_ASSERT(opt_debug_sync_timeout > 0);
+          DBUG_ASSERT(!debug_sync_set_action(thd, STRING_WITH_LEN(act)));
         };);
       }
     }
@@ -883,9 +848,9 @@ void terminate_compress_gtid_table_thread() {
   mysql_cond_signal(&COND_compress_gtid_table);
   mysql_mutex_unlock(&LOCK_compress_gtid_table);
 
-  if (compress_thread_id.thread != null_thread_initializer) {
+  if (compress_thread_id.thread != 0) {
     error = my_thread_join(&compress_thread_id, nullptr);
-    compress_thread_id.thread = null_thread_initializer;
+    compress_thread_id.thread = 0;
   }
 
   if (error != 0)
