@@ -1,4 +1,4 @@
-/* Copyright (c) 2008, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2008, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -34,14 +34,13 @@
 #include "my_compiler.h"
 #include "my_dbug.h"
 #include "my_io.h"
-#include "mysql/components/services/bits/psi_bits.h"
+#include "my_loglevel.h"
 #include "mysql/components/services/log_builtins.h"
 #include "mysql/components/services/log_shared.h"
-#include "mysql/my_loglevel.h"
 #include "mysql/plugin.h"
 #include "mysql/psi/mysql_mutex.h"
+#include "mysql/psi/psi_base.h"
 #include "mysql/service_mysql_alloc.h"
-#include "mysql/strings/m_ctype.h"
 #include "mysqld_error.h"
 #include "prealloced_array.h"
 #include "sql/current_thd.h"
@@ -52,7 +51,6 @@
 #include "sql/log.h"
 #include "sql/mysqld.h"  // server_uuid
 #include "sql/psi_memory_key.h"
-#include "sql/raii/sentry.h"  // raii::Sentry<T>
 #include "sql/replication.h"  // Trans_param
 #include "sql/rpl_gtid.h"
 #include "sql/rpl_mi.h"     // Master_info
@@ -75,9 +73,6 @@ Server_state_delegate *server_state_delegate;
 Binlog_transmit_delegate *binlog_transmit_delegate;
 Binlog_relay_IO_delegate *binlog_relay_io_delegate;
 
-bool opt_replication_optimize_for_static_plugin_config{false};
-std::atomic<bool> opt_replication_sender_observe_commit_only{false};
-
 Observer_info::Observer_info(void *ob, st_plugin_int *p)
     : observer(ob), plugin_int(p) {
   plugin = plugin_int_to_ref(plugin_int);
@@ -89,202 +84,15 @@ Delegate::Delegate(
 #endif
 ) {
   inited = false;
-  m_configured_lock_type.store(opt_replication_optimize_for_static_plugin_config
-                                   ? DELEGATE_SPIN_LOCK
-                                   : DELEGATE_OS_LOCK);
-  m_acquired_locks.store(0);
 #ifdef HAVE_PSI_RWLOCK_INTERFACE
   if (mysql_rwlock_init(key, &lock)) return;
 #else
   if (mysql_rwlock_init(0, &lock)) return;
 #endif
+  init_sql_alloc(key_memory_delegate, &memroot, 1024, 0);
   inited = true;
 }
 
-Delegate::~Delegate() {
-  inited = false;
-  mysql_rwlock_destroy(&lock);
-}
-
-int Delegate::add_observer(void *observer, st_plugin_int *plugin) {
-  int ret = false;
-  if (!inited) return true;
-  write_lock();
-  Observer_info_iterator iter(observer_info_list);
-  Observer_info *info = iter++;
-  while (info && info->observer != observer) info = iter++;
-  if (!info) {
-    info = new Observer_info(observer, plugin);
-    if (!info || observer_info_list.push_back(info, &memroot))
-      ret = true;
-    else if (this->use_spin_lock_type())
-      acquire_plugin_ref_count(info);
-  } else
-    ret = true;
-  unlock();
-  return ret;
-}
-
-int Delegate::remove_observer(void *observer) {
-  int ret = false;
-  if (!inited) return true;
-  write_lock();
-  Observer_info_iterator iter(observer_info_list);
-  Observer_info *info = iter++;
-  while (info && info->observer != observer) info = iter++;
-  if (info) {
-    iter.remove();
-    delete info;
-  } else
-    ret = true;
-  unlock();
-  return ret;
-}
-
-Delegate::Observer_info_iterator Delegate::observer_info_iter() {
-  return Observer_info_iterator(observer_info_list);
-}
-
-bool Delegate::is_empty() {
-  DBUG_PRINT("debug", ("is_empty: %d", observer_info_list.is_empty()));
-  return observer_info_list.is_empty();
-}
-
-int Delegate::read_lock() {
-  if (!inited) return true;
-  this->lock_it(DELEGATE_LOCK_MODE_SHARED);
-  return 0;
-}
-
-int Delegate::write_lock() {
-  if (!inited) return true;
-  this->lock_it(DELEGATE_LOCK_MODE_EXCLUSIVE);
-  return 0;
-}
-
-int Delegate::unlock() {
-  if (!inited) return true;
-
-  int result = 0;
-
-  if (m_acquired_locks.load() > 0) {
-    m_acquired_locks -= DELEGATE_SPIN_LOCK;
-    if (m_spin_lock.is_exclusive_acquisition())
-      m_spin_lock.release_exclusive();
-    else {
-      assert(m_spin_lock.is_shared_acquisition());
-      m_spin_lock.release_shared();
-    }
-  } else {
-    assert(m_acquired_locks.load() < 0);
-    m_acquired_locks -= DELEGATE_OS_LOCK;
-    result = mysql_rwlock_unlock(&lock);
-  }
-
-  return result;
-}
-
-bool Delegate::is_inited() { return inited; }
-
-void Delegate::update_lock_type() {
-  if (!inited) return;
-
-  int opt_value = opt_replication_optimize_for_static_plugin_config
-                      ? DELEGATE_SPIN_LOCK
-                      : DELEGATE_OS_LOCK;
-  m_configured_lock_type.store(opt_value);
-}
-
-void Delegate::update_plugin_ref_count() {
-  if (!inited) return;
-
-  int opt_value = opt_replication_optimize_for_static_plugin_config
-                      ? DELEGATE_SPIN_LOCK
-                      : DELEGATE_OS_LOCK;
-  int intern_value = m_configured_lock_type.load();
-
-  if (intern_value == DELEGATE_SPIN_LOCK && opt_value == DELEGATE_OS_LOCK) {
-    for (auto ref : m_acquired_references) {
-      for (size_t count = ref.second; count != 0; --count)
-        plugin_unlock(nullptr, ref.first);
-    }
-    m_acquired_references.clear();
-  } else if (intern_value == DELEGATE_OS_LOCK &&
-             opt_value == DELEGATE_SPIN_LOCK) {
-    Observer_info_iterator iter = observer_info_iter();
-    for (Observer_info *info = iter++; info; info = iter++) {
-      acquire_plugin_ref_count(info);
-    }
-  }
-}
-
-bool Delegate::use_rw_lock_type() {
-  return m_acquired_locks.load() < 0 ||  // If there are acquisitions using
-                                         // the read-write lock
-         (m_configured_lock_type.load() ==
-              DELEGATE_OS_LOCK &&  // or the lock type has been set to use the
-                                   // read-write lock
-          m_acquired_locks.load() == 0);  // and there are no outstanding
-                                          // acquisitions using shared
-                                          // spin-lock, use the read-write
-                                          // lock
-}
-
-bool Delegate::use_spin_lock_type() {
-  return m_acquired_locks.load() > 0 ||  // If there are acquisitions using
-                                         // the shared spin-lock
-         (m_configured_lock_type.load() ==
-              DELEGATE_SPIN_LOCK &&  // or the lock type has been set to use the
-                                     // shared spin-lock
-          m_acquired_locks.load() == 0);  // and there are no outstanding
-                                          // acquisitions using read-write
-                                          // lock, use the shared
-                                          // sping-lock
-}
-
-void Delegate::acquire_plugin_ref_count(Observer_info *info) {
-  plugin_ref internal_ref = plugin_lock(nullptr, &info->plugin);
-  ++(m_acquired_references[internal_ref]);
-}
-
-void Delegate::lock_it(enum_delegate_lock_mode mode) {
-  do {
-    if (this->use_spin_lock_type()) {
-      if (mode == DELEGATE_LOCK_MODE_SHARED)
-        m_spin_lock.acquire_shared();
-      else
-        m_spin_lock.acquire_exclusive();
-
-      if (m_configured_lock_type.load() !=
-          DELEGATE_SPIN_LOCK) {  // Lock type changed in the meanwhile, lets
-                                 // revert the acquisition and try again
-        if (mode == DELEGATE_LOCK_MODE_SHARED)
-          m_spin_lock.release_shared();
-        else
-          m_spin_lock.release_exclusive();
-      } else {
-        m_acquired_locks += DELEGATE_SPIN_LOCK;
-        break;
-      }
-    }
-    if (this->use_rw_lock_type()) {
-      if (mode == DELEGATE_LOCK_MODE_SHARED)
-        mysql_rwlock_rdlock(&lock);
-      else
-        mysql_rwlock_wrlock(&lock);
-
-      if (m_configured_lock_type.load() !=
-          DELEGATE_OS_LOCK)  // Lock type changed in the meanwhile, lets revert
-                             // the acquisition and try again
-        mysql_rwlock_unlock(&lock);
-      else {
-        m_acquired_locks += DELEGATE_OS_LOCK;
-        break;
-      }
-    }
-  } while (true);
-}
-
 /*
   structure to save transaction log filename and position
 */
@@ -389,15 +197,6 @@ int delegates_init() {
   return 0;
 }
 
-void delegates_shutdown() {
-  if (opt_replication_optimize_for_static_plugin_config) {
-    opt_replication_optimize_for_static_plugin_config = false;
-    delegates_acquire_locks();
-    delegates_update_lock_type();
-    delegates_release_locks();
-  }
-}
-
 void delegates_destroy() {
   if (transaction_delegate) transaction_delegate->~Trans_delegate();
   if (binlog_storage_delegate)
@@ -409,43 +208,6 @@ void delegates_destroy() {
     binlog_relay_io_delegate->~Binlog_relay_IO_delegate();
 }
 
-static void delegates_update_plugin_ref_count() {
-  if (transaction_delegate) transaction_delegate->update_plugin_ref_count();
-  if (binlog_storage_delegate)
-    binlog_storage_delegate->update_plugin_ref_count();
-  if (server_state_delegate) server_state_delegate->update_plugin_ref_count();
-  if (binlog_transmit_delegate)
-    binlog_transmit_delegate->update_plugin_ref_count();
-  if (binlog_relay_io_delegate)
-    binlog_relay_io_delegate->update_plugin_ref_count();
-}
-
-void delegates_acquire_locks() {
-  if (transaction_delegate) transaction_delegate->write_lock();
-  if (binlog_storage_delegate) binlog_storage_delegate->write_lock();
-  if (server_state_delegate) server_state_delegate->write_lock();
-  if (binlog_transmit_delegate) binlog_transmit_delegate->write_lock();
-  if (binlog_relay_io_delegate) binlog_relay_io_delegate->write_lock();
-}
-
-void delegates_release_locks() {
-  if (transaction_delegate) transaction_delegate->unlock();
-  if (binlog_storage_delegate) binlog_storage_delegate->unlock();
-  if (server_state_delegate) server_state_delegate->unlock();
-  if (binlog_transmit_delegate) binlog_transmit_delegate->unlock();
-  if (binlog_relay_io_delegate) binlog_relay_io_delegate->unlock();
-}
-
-void delegates_update_lock_type() {
-  delegates_update_plugin_ref_count();
-
-  if (transaction_delegate) transaction_delegate->update_lock_type();
-  if (binlog_storage_delegate) binlog_storage_delegate->update_lock_type();
-  if (server_state_delegate) server_state_delegate->update_lock_type();
-  if (binlog_transmit_delegate) binlog_transmit_delegate->update_lock_type();
-  if (binlog_relay_io_delegate) binlog_relay_io_delegate->update_lock_type();
-}
-
 /*
   This macro is used by almost all the Delegate methods to iterate
   over all the observers running given callback function of the
@@ -454,98 +216,83 @@ void delegates_update_lock_type() {
   Add observer plugins to the thd->lex list, after each statement, all
   plugins add to thd->lex will be automatically unlocked.
  */
-#define FOREACH_OBSERVER(r, f, args)                                    \
-  r = 0;                                                                \
-  Prealloced_array<plugin_ref, 8> plugins(PSI_NOT_INSTRUMENTED);        \
-  read_lock();                                                          \
-  Observer_info_iterator iter = observer_info_iter();                   \
-  Observer_info *info = iter++;                                         \
-  bool replication_optimize_for_static_plugin_config =                  \
-      this->use_spin_lock_type();                                       \
-  for (; info; info = iter++) {                                         \
-    plugin_ref plugin = (replication_optimize_for_static_plugin_config  \
-                             ? info->plugin                             \
-                             : my_plugin_lock(nullptr, &info->plugin)); \
-    if (!plugin) {                                                      \
-      /* plugin is not initialized or deleted, this is not an error */  \
-      continue;                                                         \
-    }                                                                   \
-    if (!replication_optimize_for_static_plugin_config)                 \
-      plugins.push_back(plugin);                                        \
-    if (((Observer *)info->observer)->f &&                              \
-        ((Observer *)info->observer)->f args) {                         \
-      r = 1;                                                            \
-      LogEvent()                                                        \
-          .prio(ERROR_LEVEL)                                            \
-          .errcode(ER_RPL_PLUGIN_FUNCTION_FAILED)                       \
-          .subsys(LOG_SUBSYSTEM_TAG)                                    \
-          .function(#f)                                                 \
-          .message("Run function '" #f "' in plugin '%s' failed",       \
-                   info->plugin_int->name.str);                         \
-      break;                                                            \
-    }                                                                   \
-  }                                                                     \
-  unlock();                                                             \
-  /*                                                                    \
-     Unlock plugins should be done after we released the Delegate lock  \
-     to avoid possible deadlock when this is the last user of the       \
-     plugin, and when we unlock the plugin, it will try to              \
-     deinitialize the plugin, which will try to lock the Delegate in    \
-     order to remove the observers.                                     \
-  */                                                                    \
-  if (!plugins.empty())                                                 \
-    plugin_unlock_list(nullptr, &plugins[0], plugins.size());
+#define FOREACH_OBSERVER(r, f, args)                                   \
+  Prealloced_array<plugin_ref, 8> plugins(PSI_NOT_INSTRUMENTED);       \
+  read_lock();                                                         \
+  Observer_info_iterator iter = observer_info_iter();                  \
+  Observer_info *info = iter++;                                        \
+  for (; info; info = iter++) {                                        \
+    plugin_ref plugin = my_plugin_lock(0, &info->plugin);              \
+    if (!plugin) {                                                     \
+      /* plugin is not intialized or deleted, this is not an error */  \
+      r = 0;                                                           \
+      break;                                                           \
+    }                                                                  \
+    plugins.push_back(plugin);                                         \
+    if (((Observer *)info->observer)->f &&                             \
+        ((Observer *)info->observer)->f args) {                        \
+      r = 1;                                                           \
+      LogEvent()                                                       \
+          .prio(ERROR_LEVEL)                                           \
+          .errcode(ER_RPL_PLUGIN_FUNCTION_FAILED)                      \
+          .subsys(LOG_SUBSYSTEM_TAG)                                   \
+          .function(#f)                                                \
+          .message("Run function '" #f "' in plugin '%s' failed",      \
+                   info->plugin_int->name.str);                        \
+      break;                                                           \
+    }                                                                  \
+  }                                                                    \
+  unlock();                                                            \
+  /*                                                                   \
+     Unlock plugins should be done after we released the Delegate lock \
+     to avoid possible deadlock when this is the last user of the      \
+     plugin, and when we unlock the plugin, it will try to             \
+     deinitialize the plugin, which will try to lock the Delegate in   \
+     order to remove the observers.                                    \
+  */                                                                   \
+  if (!plugins.empty()) plugin_unlock_list(0, &plugins[0], plugins.size());
 
-#define FOREACH_OBSERVER_ERROR_OUT(r, f, args, out)                     \
-  r = 0;                                                                \
-  Prealloced_array<plugin_ref, 8> plugins(PSI_NOT_INSTRUMENTED);        \
-  read_lock();                                                          \
-  Observer_info_iterator iter = observer_info_iter();                   \
-  Observer_info *info = iter++;                                         \
-                                                                        \
-  bool replication_optimize_for_static_plugin_config =                  \
-      this->use_spin_lock_type();                                       \
-  int error_out = 0;                                                    \
-  for (; info; info = iter++) {                                         \
-    plugin_ref plugin = (replication_optimize_for_static_plugin_config  \
-                             ? info->plugin                             \
-                             : my_plugin_lock(nullptr, &info->plugin)); \
-    if (!plugin) {                                                      \
-      /* plugin is not initialized or deleted, this is not an error */  \
-      continue;                                                         \
-    }                                                                   \
-    if (!replication_optimize_for_static_plugin_config)                 \
-      plugins.push_back(plugin);                                        \
-                                                                        \
-    if (nullptr == ((Observer *)info->observer)->f) {                   \
-      continue;                                                         \
-    }                                                                   \
-    bool hook_error = false;                                            \
-    hook_error = ((Observer *)info->observer)->f(args, error_out);      \
-                                                                        \
-    out += error_out;                                                   \
-    if (hook_error) {                                                   \
-      r = 1;                                                            \
-      LogEvent()                                                        \
-          .prio(ERROR_LEVEL)                                            \
-          .errcode(ER_RPL_PLUGIN_FUNCTION_FAILED)                       \
-          .subsys(LOG_SUBSYSTEM_TAG)                                    \
-          .function(#f)                                                 \
-          .message("Run function '" #f "' in plugin '%s' failed",       \
-                   info->plugin_int->name.str);                         \
-      break;                                                            \
-    }                                                                   \
-  }                                                                     \
-  unlock();                                                             \
-  /*                                                                    \
-     Unlock plugins should be done after we released the Delegate lock  \
-     to avoid possible deadlock when this is the last user of the       \
-     plugin, and when we unlock the plugin, it will try to              \
-     deinitialize the plugin, which will try to lock the Delegate in    \
-     order to remove the observers.                                     \
-  */                                                                    \
-  if (!plugins.empty())                                                 \
-    plugin_unlock_list(nullptr, &plugins[0], plugins.size());
+#define FOREACH_OBSERVER_ERROR_OUT(r, f, args, out)                    \
+  Prealloced_array<plugin_ref, 8> plugins(PSI_NOT_INSTRUMENTED);       \
+  read_lock();                                                         \
+  Observer_info_iterator iter = observer_info_iter();                  \
+  Observer_info *info = iter++;                                        \
+                                                                       \
+  int error_out = 0;                                                   \
+  for (; info; info = iter++) {                                        \
+    plugin_ref plugin = my_plugin_lock(0, &info->plugin);              \
+    if (!plugin) {                                                     \
+      /* plugin is not intialized or deleted, this is not an error */  \
+      r = 0;                                                           \
+      break;                                                           \
+    }                                                                  \
+    plugins.push_back(plugin);                                         \
+                                                                       \
+    bool hook_error = false;                                           \
+    hook_error = ((Observer *)info->observer)->f(args, error_out);     \
+                                                                       \
+    out += error_out;                                                  \
+    if (hook_error) {                                                  \
+      r = 1;                                                           \
+      LogEvent()                                                       \
+          .prio(ERROR_LEVEL)                                           \
+          .errcode(ER_RPL_PLUGIN_FUNCTION_FAILED)                      \
+          .subsys(LOG_SUBSYSTEM_TAG)                                   \
+          .function(#f)                                                \
+          .message("Run function '" #f "' in plugin '%s' failed",      \
+                   info->plugin_int->name.str);                        \
+      break;                                                           \
+    }                                                                  \
+  }                                                                    \
+  unlock();                                                            \
+  /*                                                                   \
+     Unlock plugins should be done after we released the Delegate lock \
+     to avoid possible deadlock when this is the last user of the      \
+     plugin, and when we unlock the plugin, it will try to             \
+     deinitialize the plugin, which will try to lock the Delegate in   \
+     order to remove the observers.                                    \
+  */                                                                   \
+  if (!plugins.empty()) plugin_unlock_list(0, &plugins[0], plugins.size());
 
 static bool se_before_commit(THD *, plugin_ref plugin, void *arg) {
   handlerton *hton = plugin_data<handlerton *>(plugin);
@@ -566,7 +313,6 @@ int Trans_delegate::before_commit(THD *thd, bool all,
   param.thread_id = thd->thread_id();
   param.gtid_info.type = thd->variables.gtid_next.type;
   param.gtid_info.sidno = thd->variables.gtid_next.gtid.sidno;
-  param.gtid_info.automatic_tag = thd->variables.gtid_next.automatic_tag;
   param.gtid_info.gno = thd->variables.gtid_next.gtid.gno;
   param.trx_cache_log = trx_cache_log;
   param.stmt_cache_log = stmt_cache_log;
@@ -578,48 +324,12 @@ int Trans_delegate::before_commit(THD *thd, bool all,
       thd->variables.group_replication_consistency;
   param.original_server_version = &(thd->variables.original_server_version);
   param.immediate_server_version = &(thd->variables.immediate_server_version);
-  param.is_create_table_as_query_block =
-      (thd->lex->sql_command == SQLCOM_CREATE_TABLE &&
-       !thd->lex->query_block->field_list_is_empty());
-  param.thd = thd;
 
   bool is_real_trans =
       (all || !thd->get_transaction()->is_active(Transaction_ctx::SESSION));
   if (is_real_trans) param.flags |= TRANS_IS_REAL_TRANS;
 
   int ret = 0;
-
-  /* After this debug point we mark the transaction as committing in THD. */
-  DBUG_EXECUTE_IF("trans_delegate_before_commit_before_before_call_observers", {
-    const char act[] =
-        "now signal "
-        "signal.trans_delegate_before_commit_before_before_call_observers_"
-        "reached "
-        "wait_for "
-        "signal.trans_delegate_before_commit_before_before_call_observers_"
-        "waiting";
-    assert(!debug_sync_set_action(thd, STRING_WITH_LEN(act)));
-  });
-
-  thd->rpl_thd_ctx.set_tx_rpl_delegate_stage_status(
-      Rpl_thd_context::TX_RPL_STAGE_BEFORE_COMMIT);
-
-  /**
-    If thread is killed or commits are blocked do not commit the transaction.
-    Post this thread cannot be killed.
-  */
-  if (thd->is_killed() || m_rollback_transaction_not_reached_before_commit) {
-    /**
-      Disconnect the client connection if not already done.
-      Do not KILL connection if the transaction is going to be rolledback.
-    */
-    if (!thd->is_killed()) {
-      mysql_mutex_lock(&thd->LOCK_thd_data);
-      thd->awake(THD::KILL_CONNECTION);
-      mysql_mutex_unlock(&thd->LOCK_thd_data);
-    }
-    return 1;
-  }
   FOREACH_OBSERVER(ret, before_commit, (&param));
   plugin_foreach(thd, se_before_commit, MYSQL_STORAGE_ENGINE_PLUGIN, &param);
   return ret;
@@ -744,6 +454,10 @@ static void prepare_transaction_context(THD *thd,
   // Extracting the global mutable value of binlog checksum
   ctx_info.binlog_checksum_options = binlog_checksum_options;
 
+  // Extracting the session value of transaction_write_set_extraction
+  ctx_info.transaction_write_set_extraction =
+      thd->variables.transaction_write_set_extraction;
+
   // Extracting transaction isolation level
   ctx_info.tx_isolation = thd->tx_isolation;
 }
@@ -788,8 +502,6 @@ int Trans_delegate::before_rollback(THD *thd, bool all) {
   if (is_real_trans) param.flags |= TRANS_IS_REAL_TRANS;
 
   int ret = 0;
-  thd->rpl_thd_ctx.set_tx_rpl_delegate_stage_status(
-      Rpl_thd_context::TX_RPL_STAGE_BEFORE_ROLLBACK);
   FOREACH_OBSERVER(ret, before_rollback, (&param));
   plugin_foreach(thd, se_before_rollback, MYSQL_STORAGE_ENGINE_PLUGIN, &param);
   return ret;
@@ -813,10 +525,6 @@ int Trans_delegate::after_commit(THD *thd, bool all) {
   param.gtid_info.sidno = gtid.sidno;
   param.gtid_info.gno = gtid.gno;
 
-  mysql::gtid::Tsid tsid;
-  thd->rpl_thd_ctx.last_used_gtid_tracker_ctx().get_last_used_tsid(tsid);
-  param.gtid_info.tsid = mysql::gtid::Tsid_plain(tsid);
-
   bool is_real_trans =
       (all || !thd->get_transaction()->is_active(Transaction_ctx::SESSION));
   if (is_real_trans) param.flags |= TRANS_IS_REAL_TRANS;
@@ -856,11 +564,6 @@ int Trans_delegate::after_rollback(THD *thd, bool all) {
 
 int Trans_delegate::trans_begin(THD *thd, int &out) {
   DBUG_TRACE;
-  if (m_rollback_transaction_on_begin) {
-    out = ER_OPERATION_NOT_ALLOWED_WHILE_PRIMARY_CHANGE_IS_RUNNING;
-    return 0;
-  }
-
   Trans_param param;
   TRANS_PARAM_ZERO(param);
   param.server_uuid = server_uuid;
@@ -870,39 +573,12 @@ int Trans_delegate::trans_begin(THD *thd, int &out) {
   param.hold_timeout = thd->variables.net_wait_timeout;
   param.server_id = thd->server_id;
   param.rpl_channel_type = thd->rpl_thd_ctx.get_rpl_channel_type();
-  param.thd = thd;
 
   int ret = 0;
-  thd->rpl_thd_ctx.set_tx_rpl_delegate_stage_status(
-      Rpl_thd_context::TX_RPL_STAGE_BEGIN);
   FOREACH_OBSERVER_ERROR_OUT(ret, begin, &param, out);
   return ret;
 }
 
-int Trans_delegate::set_transactions_at_begin_must_fail() {
-  DBUG_TRACE;
-  m_rollback_transaction_on_begin = true;
-  return false;
-}
-
-int Trans_delegate::set_no_restrictions_at_transaction_begin() {
-  DBUG_TRACE;
-  m_rollback_transaction_on_begin = false;
-  return false;
-}
-
-int Trans_delegate::set_transactions_not_reached_before_commit_must_fail() {
-  DBUG_TRACE;
-  m_rollback_transaction_not_reached_before_commit = true;
-  return false;
-}
-
-int Trans_delegate::set_no_restrictions_at_transactions_before_commit() {
-  DBUG_TRACE;
-  m_rollback_transaction_not_reached_before_commit = false;
-  return false;
-}
-
 int Binlog_storage_delegate::after_flush(THD *thd, const char *log_file,
                                          my_off_t log_pos) {
   DBUG_TRACE;
@@ -1028,7 +704,7 @@ int Binlog_storage_delegate::after_sync(THD *thd, const char *log_file,
   Binlog_storage_param param;
   param.server_id = thd->server_id;
 
-  assert(log_pos != 0);
+  DBUG_ASSERT(log_pos != 0);
   int ret = 0;
   FOREACH_OBSERVER(ret, after_sync, (&param, log_file, log_pos));
 
@@ -1064,12 +740,12 @@ int Binlog_transmit_delegate::transmit_stop(THD *thd, ushort flags) {
 
 int Binlog_transmit_delegate::reserve_header(THD *thd, ushort flags,
                                              String *packet) {
-  /*
-    NOTE2ME: Maximum extra header size for each observer, I hope 32
-    bytes should be enough for each Observer to reserve their
-    extra header. If later found this is not enough, we can increase this /HEZX
-   */
-  constexpr int RESERVE_HEADER_SIZE = 32;
+/* NOTE2ME: Maximum extra header size for each observer, I hope 32
+   bytes should be enough for each Observer to reserve their extra
+   header. If later found this is not enough, we can increase this
+   /HEZX
+*/
+#define RESERVE_HEADER_SIZE 32
   unsigned char header[RESERVE_HEADER_SIZE];
   ulong hlen;
   Binlog_transmit_param param;
@@ -1082,29 +758,21 @@ int Binlog_transmit_delegate::reserve_header(THD *thd, ushort flags,
   read_lock();
   Observer_info_iterator iter = observer_info_iter();
   Observer_info *info = iter++;
-  bool replication_optimize_for_static_plugin_config =
-      this->use_spin_lock_type();
   for (; info; info = iter++) {
-    plugin_ref plugin = (replication_optimize_for_static_plugin_config
-                             ? info->plugin
-                             : my_plugin_lock(thd, &info->plugin));
+    plugin_ref plugin = my_plugin_lock(thd, &info->plugin);
     if (!plugin) {
       ret = 1;
       break;
     }
     hlen = 0;
-    {  // `unlock_guard` scope
-      raii::Sentry<> unlock_guard{[&]() -> void {
-        if (!replication_optimize_for_static_plugin_config)
-          plugin_unlock(thd, plugin);
-      }};
-      if (((Observer *)info->observer)->reserve_header &&
-          ((Observer *)info->observer)
-              ->reserve_header(&param, header, RESERVE_HEADER_SIZE, &hlen)) {
-        ret = 1;
-        break;
-      }
-    }  // `unlock_guard` scope
+    if (((Observer *)info->observer)->reserve_header &&
+        ((Observer *)info->observer)
+            ->reserve_header(&param, header, RESERVE_HEADER_SIZE, &hlen)) {
+      ret = 1;
+      plugin_unlock(thd, plugin);
+      break;
+    }
+    plugin_unlock(thd, plugin);
     if (hlen == 0) continue;
     if (hlen > RESERVE_HEADER_SIZE || packet->append((char *)header, hlen)) {
       ret = 1;
@@ -1172,8 +840,6 @@ void Binlog_relay_IO_delegate::init_param(Binlog_relay_IO_param *param,
   param->port = mi->port;
   param->master_log_name = const_cast<char *>(mi->get_master_log_name());
   param->master_log_pos = mi->get_master_log_pos();
-  param->source_connection_auto_failover =
-      mi->is_source_connection_auto_failover();
 }
 
 int Binlog_relay_IO_delegate::thread_start(THD *thd, Master_info *mi) {
@@ -1353,40 +1019,7 @@ int unregister_binlog_relay_io_observer(Binlog_relay_IO_observer *observer,
   return binlog_relay_io_delegate->remove_observer(observer);
 }
 
-static bool is_show_status(enum_sql_command sql_command) {
-  switch (sql_command) {
-    case SQLCOM_SHOW_VARIABLES:
-    case SQLCOM_SHOW_STATUS:
-    case SQLCOM_SHOW_ENGINE_LOGS:
-    case SQLCOM_SHOW_ENGINE_STATUS:
-    case SQLCOM_SHOW_ENGINE_MUTEX:
-    case SQLCOM_SHOW_PROCESSLIST:
-    case SQLCOM_SHOW_MASTER_STAT:
-    case SQLCOM_SHOW_SLAVE_STAT:
-    case SQLCOM_SHOW_CHARSETS:
-    case SQLCOM_SHOW_COLLATIONS:
-    case SQLCOM_SHOW_BINLOGS:
-    case SQLCOM_SHOW_OPEN_TABLES:
-    case SQLCOM_SHOW_SLAVE_HOSTS:
-    case SQLCOM_SHOW_BINLOG_EVENTS:
-    case SQLCOM_SHOW_WARNS:
-    case SQLCOM_SHOW_ERRORS:
-    case SQLCOM_SHOW_STORAGE_ENGINES:
-    case SQLCOM_SHOW_PRIVILEGES:
-    case SQLCOM_SHOW_STATUS_PROC:
-    case SQLCOM_SHOW_STATUS_FUNC:
-    case SQLCOM_SHOW_PLUGINS:
-    case SQLCOM_SHOW_EVENTS:
-    case SQLCOM_SHOW_PROFILE:
-    case SQLCOM_SHOW_PROFILES:
-    case SQLCOM_SHOW_RELAYLOG_EVENTS:
-      return true;
-    default:
-      return false;
-  }
-}
-
-int launch_hook_trans_begin(THD *thd, Table_ref *all_tables) {
+int launch_hook_trans_begin(THD *thd, TABLE_LIST *all_tables) {
   DBUG_TRACE;
   LEX *lex = thd->lex;
   enum_sql_command sql_command = lex->sql_command;
@@ -1400,10 +1033,12 @@ int launch_hook_trans_begin(THD *thd, Table_ref *all_tables) {
     return 0;
   }
 
-  bool is_show = is_show_status(sql_command);
+  bool is_show = ((sql_command_flags[sql_command] & CF_STATUS_COMMAND) &&
+                  (sql_command != SQLCOM_BINLOG_BASE64_EVENT)) ||
+                 (sql_command == SQLCOM_SHOW_RELAYLOG_EVENTS);
   bool is_set = (sql_command == SQLCOM_SET_OPTION);
-  bool is_query_block =
-      (sql_command == SQLCOM_SELECT || sql_command == SQLCOM_DO);
+  bool is_select = (sql_command == SQLCOM_SELECT);
+  bool is_do = (sql_command == SQLCOM_DO);
   bool is_empty = (sql_command == SQLCOM_EMPTY_QUERY);
   bool is_use = (sql_command == SQLCOM_CHANGE_DB);
   bool is_stop_gr = (sql_command == SQLCOM_STOP_GROUP_REPLICATION);
@@ -1411,19 +1046,21 @@ int launch_hook_trans_begin(THD *thd, Table_ref *all_tables) {
   bool is_reset_persist =
       (sql_command == SQLCOM_RESET && lex->option_type == OPT_PERSIST);
 
-  if ((is_set || is_show || is_empty || is_use || is_stop_gr || is_shutdown ||
-       is_reset_persist) &&
+  if ((is_set || is_do || is_show || is_empty || is_use || is_stop_gr ||
+       is_shutdown || is_reset_persist) &&
       !lex->uses_stored_routines()) {
     return 0;
   }
 
-  if (is_query_block) {
+  if (is_select) {
     bool is_udf = false;
 
-    // If SELECT or DO is a UDF
-    Query_block *query_block_elem = lex->unit->first_query_block();
-    while (query_block_elem != nullptr) {
-      for (Item *item : query_block_elem->visible_fields()) {
+    // if select is an udf function
+    SELECT_LEX *select_lex_elem = lex->unit->first_select();
+    while (select_lex_elem != nullptr) {
+      Item *item;
+      List_iterator_fast<Item> it(select_lex_elem->fields_list);
+      while ((item = it++)) {
         if (item->type() == Item::FUNC_ITEM) {
           Item_func *func_item = down_cast<Item_func *>(item);
           Item_func::Functype functype = func_item->functype();
@@ -1431,11 +1068,11 @@ int launch_hook_trans_begin(THD *thd, Table_ref *all_tables) {
             is_udf = true;
         }
       }
-      query_block_elem = query_block_elem->next_query_block();
+      select_lex_elem = select_lex_elem->next_select();
     }
 
     if (!is_udf && all_tables == nullptr) {
-      // SELECT or DO that don't use tables and isn't a UDF
+      // SELECT that don't use tables and isn't a UDF
       hold_command = false;
     }
 
@@ -1446,9 +1083,9 @@ int launch_hook_trans_begin(THD *thd, Table_ref *all_tables) {
       bool is_sys_db = false;
       bool stop_db_check = false;
 
-      for (Table_ref *table = all_tables; table && !stop_db_check;
+      for (TABLE_LIST *table = all_tables; table && !stop_db_check;
            table = table->next_global) {
-        assert(table->db && table->table_name);
+        DBUG_ASSERT(table->db && table->table_name);
 
         if (is_perfschema_db(table->db, table->db_length))
           is_perf_schema_table = true;
@@ -1474,7 +1111,8 @@ int launch_hook_trans_begin(THD *thd, Table_ref *all_tables) {
   }
 
   if (hold_command) {
-    DBUG_EXECUTE_IF("launch_hook_trans_begin_assert_if_hold", { assert(0); };);
+    DBUG_EXECUTE_IF("launch_hook_trans_begin_assert_if_hold",
+                    { DBUG_ASSERT(0); };);
 
     PSI_stage_info old_stage;
     thd->enter_stage(&stage_hook_begin_trans, &old_stage, __func__, __FILE__,
