@@ -1,16 +1,15 @@
-/* Copyright (c) 2018, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,23 +25,21 @@
 #include <string.h>
 #include <algorithm>
 #include <sstream>
+#include "libbinlogevents/include/event_reader.h"
 #include "mutex_lock.h"
 #include "my_byteorder.h"
-#include "mysql/binlog/event/event_reader.h"
-#include "scope_guard.h"
 #include "sql/basic_istream.h"
 #include "sql/basic_ostream.h"
 #include "sql/sql_class.h"
 
 #ifdef MYSQL_SERVER
-#include "keyring_operations_helper.h"
+#include "libbinlogevents/include/byteorder.h"
 #include "my_aes.h"
 #include "my_rnd.h"
-#include "mysql/binlog/event/byteorder.h"
 #include "mysql/components/services/log_builtins.h"
+#include "mysql/service_mysql_keyring.h"
 #include "sql/binlog.h"
-#include "sql/rpl_replica.h"
-#include "sql/server_component/mysql_server_keyring_lockable_imp.h"
+#include "sql/rpl_slave.h"
 
 Rpl_encryption rpl_encryption;
 
@@ -99,14 +96,14 @@ void Rpl_encryption::report_keyring_error(Keyring_status error,
       break;
     case Keyring_status::SUCCESS:
     default:
-      assert(false);
+      DBUG_ASSERT(false);
   }
 }
 
 bool Rpl_encryption::initialize() {
   DBUG_TRACE;
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   m_initialized = true;
   DBUG_PRINT("debug", ("m_enabled= %s", m_enabled ? "true" : "false"));
   DBUG_PRINT("debug", ("m_rotate_at_startup= %s",
@@ -122,7 +119,7 @@ bool Rpl_encryption::initialize() {
   if (m_enabled) {
     if (recover_master_key()) return true;
     if (m_rotate_at_startup && rotate_master_key()) return true;
-    assert(m_master_key_seqno > 0);
+    DBUG_ASSERT(m_master_key_seqno > 0);
   }
 
   return false;
@@ -130,7 +127,7 @@ bool Rpl_encryption::initialize() {
 
 bool Rpl_encryption::remove_remaining_seqnos_from_keyring() {
   DBUG_TRACE;
-  assert(m_enabled);
+  DBUG_ASSERT(m_enabled);
 
   auto master_key_seqno = get_master_key_seqno_from_keyring();
   /* keyring error */
@@ -192,7 +189,7 @@ bool Rpl_encryption::remove_remaining_seqnos_from_keyring() {
 
 bool Rpl_encryption::recover_master_key() {
   DBUG_TRACE;
-  assert(m_master_key_recovered == false);
+  DBUG_ASSERT(m_master_key_recovered == false);
   std::pair<Rpl_encryption::Keyring_status, unsigned int> new_master_key_seqno;
   std::pair<Rpl_encryption::Keyring_status, unsigned int> old_master_key_seqno;
 
@@ -361,10 +358,10 @@ err2:
 
 const Rpl_encryption::Rpl_encryption_key Rpl_encryption::get_master_key() {
   DBUG_TRACE;
-  assert(m_initialized);
+  DBUG_ASSERT(m_initialized);
   /* A master key shall already exists when this function is called */
-  assert(!m_master_key.m_id.empty());
-  assert(!m_master_key.m_value.empty());
+  DBUG_ASSERT(!m_master_key.m_id.empty());
+  DBUG_ASSERT(!m_master_key.m_value.empty());
   return m_master_key;
 }
 
@@ -404,7 +401,7 @@ std::pair<Rpl_encryption::Keyring_status, Key_string> Rpl_encryption::get_key(
 
 bool Rpl_encryption::enable(THD *thd) {
   DBUG_TRACE;
-  assert(m_initialized);
+  DBUG_ASSERT(m_initialized);
 
   MUTEX_LOCK(lock, &LOCK_rotate_binlog_master_key);
   m_enabled = true;
@@ -419,7 +416,7 @@ bool Rpl_encryption::enable(THD *thd) {
 
   if (!res) {
     DBUG_PRINT("debug", ("m_master_key_seqno= %u", m_master_key_seqno));
-    assert(m_master_key_seqno > 0);
+    DBUG_ASSERT(m_master_key_seqno > 0);
     if (!m_skip_logs_rotation) rotate_logs(thd);
   }
 
@@ -436,7 +433,7 @@ bool Rpl_encryption::enable(THD *thd) {
 
 bool Rpl_encryption::enable_for_xtrabackup() {
   DBUG_ENTER("Rpl_encryption::enable_for_xtrabackup");
-  assert(m_initialized);
+  DBUG_ASSERT(m_initialized);
 
   m_enabled = true;
   m_skip_logs_rotation = true;
@@ -464,7 +461,7 @@ bool Rpl_encryption::enable_for_xtrabackup() {
 
 void Rpl_encryption::disable(THD *thd) {
   DBUG_TRACE;
-  assert(m_initialized);
+  DBUG_ASSERT(m_initialized);
 
   MUTEX_LOCK(lock, &LOCK_rotate_binlog_master_key);
   m_enabled = false;
@@ -498,20 +495,15 @@ Rpl_encryption::fetch_key_from_keyring(const std::string &key_id,
   DBUG_TRACE;
   size_t key_len = 0;
   char *retrieved_key_type = nullptr;
-  unsigned char *key = nullptr;
+  void *key = nullptr;
   Keyring_status error = Keyring_status::SUCCESS;
 
-  auto fetch_key = [&]() -> int {
-    return keyring_operations_helper::read_secret(
-        srv_keyring_reader, key_id.c_str(), nullptr, &key, &key_len,
-        &retrieved_key_type, PSI_INSTRUMENT_ME);
-  };
-
   /* Error fetching the key */
   if (DBUG_EVALUATE_IF("failed_to_fetch_master_key_seqno_from_keyring", true,
                        false) ||
-      fetch_key() == -1) {
-    assert(key == nullptr);
+      my_key_fetch(key_id.c_str(), &retrieved_key_type, nullptr, &key,
+                   &key_len)) {
+    DBUG_ASSERT(key == nullptr);
     error = Keyring_status::KEYRING_ERROR_FETCHING;
   } else {
     /* Key was not found in keyring */
@@ -588,7 +580,7 @@ bool Rpl_encryption::purge_unused_keys() {
       for (uint32_t seqno = 1; seqno < m_master_key_seqno; seqno++) {
         std::string key_id = Rpl_encryption_header::seqno_to_key_id(seqno);
         auto key = get_key(key_id, Rpl_encryption_header::get_key_type());
-        assert(key.first == Keyring_status::KEY_NOT_FOUND);
+        DBUG_ASSERT(key.first == Keyring_status::KEY_NOT_FOUND);
       });
 
   return false;
@@ -612,9 +604,9 @@ bool Rpl_encryption::rotate_master_key(Key_rotation_step step,
       }
       DBUG_EXECUTE_IF("crash_after_set_old_master_key_seqno_on_keyring",
                       DBUG_SUICIDE(););
-      [[fallthrough]];
+      /* FALLTHROUGH */
     case Key_rotation_step::DETERMINE_NEXT_SEQNO: {
-      assert(new_master_key_seqno == 0);
+      DBUG_ASSERT(new_master_key_seqno == 0);
       Keyring_status candidate_key_fetch_status;
       new_master_key_seqno = m_master_key_seqno;
       do {
@@ -638,7 +630,7 @@ bool Rpl_encryption::rotate_master_key(Key_rotation_step step,
       DBUG_EXECUTE_IF("crash_after_set_new_master_key_seqno_on_keyring",
                       DBUG_SUICIDE(););
     }
-      [[fallthrough]];
+      /* FALLTHROUGH */
     case Key_rotation_step::GENERATE_NEW_MASTER_KEY:
       /*
         Request the keyring to generate a new master key by key id
@@ -648,7 +640,7 @@ bool Rpl_encryption::rotate_master_key(Key_rotation_step step,
       if (generate_master_key_on_keyring(new_master_key_seqno)) goto err1;
       DBUG_EXECUTE_IF("crash_after_generate_new_master_key_on_keyring",
                       DBUG_SUICIDE(););
-      [[fallthrough]];
+      /* FALLTHROUGH */
     case Key_rotation_step::REMOVE_MASTER_KEY_INDEX:
       /*
         We did not store a master key seqno into keyring if
@@ -659,14 +651,14 @@ bool Rpl_encryption::rotate_master_key(Key_rotation_step step,
         DBUG_EXECUTE_IF("crash_after_remove_master_key_seqno_from_keyring",
                         DBUG_SUICIDE(););
       }
-      [[fallthrough]];
+      /* FALLTHROUGH */
     case Key_rotation_step::STORE_MASTER_KEY_INDEX:
       if (set_master_key_seqno_on_keyring(new_master_key_seqno)) goto err1;
       DBUG_EXECUTE_IF("crash_after_set_master_key_seqno_on_keyring",
                       DBUG_SUICIDE(););
       /* The master key is now usable */
       m_master_key_seqno = new_master_key_seqno;
-      [[fallthrough]];
+      /* FALLTHROUGH */
     case Key_rotation_step::ROTATE_LOGS: {
       /* We do not rotate and re-encrypt logs during recovery. */
       if (m_master_key_recovered && current_thd) {
@@ -686,7 +678,7 @@ bool Rpl_encryption::rotate_master_key(Key_rotation_step step,
         if (reencrypt_relay_logs()) return true;
       }
     }
-      [[fallthrough]];
+      /* FALLTHROUGH */
     case Key_rotation_step::PURGE_UNUSED_ENCRYPTION_KEYS: {
       if (m_master_key_recovered && current_thd) {
         /* We do not purge unused encryption keys during recovery. */
@@ -699,7 +691,7 @@ bool Rpl_encryption::rotate_master_key(Key_rotation_step step,
       DBUG_EXECUTE_IF("crash_after_remove_old_master_key_seqno_from_keyring",
                       DBUG_SUICIDE(););
     }
-      [[fallthrough]];
+      /* FALLTHROUGH */
     case Key_rotation_step::REMOVE_KEY_ROTATION_TAG:
       if (remove_new_master_key_seqno_from_keyring()) goto warn2;
       DBUG_EXECUTE_IF("crash_after_remove_new_master_key_seqno_from_keyring",
@@ -709,7 +701,7 @@ bool Rpl_encryption::rotate_master_key(Key_rotation_step step,
   return false;
 
 warn1:
-  assert(m_master_key_recovered && current_thd);
+  DBUG_ASSERT(m_master_key_recovered && current_thd);
   if (current_thd->is_error()) current_thd->clear_error();
   /*
     We just report the warning for the command
@@ -783,9 +775,9 @@ bool Rpl_encryption::set_seqno_on_keyring(std::string key_id, uint32_t seqno) {
   unsigned char key[SEQNO_KEY_LENGTH]{0};
   int4store(key, seqno);
   DBUG_PRINT("debug", ("key_id= '%s'. seqno= %u", key_id.c_str(), seqno));
-#ifdef NDEBUG
-  if (srv_keyring_writer->store(key_id.c_str(), nullptr, key, SEQNO_KEY_LENGTH,
-                                SEQNO_KEY_TYPE) != 0) {
+#ifdef DBUG_OFF
+  if (my_key_store(key_id.c_str(), SEQNO_KEY_TYPE, nullptr, key,
+                   SEQNO_KEY_LENGTH)) {
 #else
   if ((DBUG_EVALUATE_IF("rpl_encryption_first_time_enable_1", true, false) &&
        key_id.compare(get_new_master_key_seqno_key_id()) == 0) ||
@@ -803,8 +795,8 @@ bool Rpl_encryption::set_seqno_on_keyring(std::string key_id, uint32_t seqno) {
       (DBUG_EVALUATE_IF("fail_to_set_last_purged_master_key_seqno_on_keyring",
                         true, false) &&
        key_id.compare(get_last_purged_master_key_seqno_key_id()) == 0) ||
-      srv_keyring_writer->store(key_id.c_str(), nullptr, key, SEQNO_KEY_LENGTH,
-                                SEQNO_KEY_TYPE) != 0) {
+      my_key_store(key_id.c_str(), SEQNO_KEY_TYPE, nullptr, key,
+                   SEQNO_KEY_LENGTH)) {
 #endif
     report_keyring_error(Keyring_status::KEYRING_ERROR_STORING);
     return true;
@@ -814,8 +806,8 @@ bool Rpl_encryption::set_seqno_on_keyring(std::string key_id, uint32_t seqno) {
 
 bool Rpl_encryption::remove_key_from_keyring(std::string key_id) {
   DBUG_TRACE;
-#ifdef NDEBUG
-  if (srv_keyring_writer->remove(key_id.c_str(), nullptr) != 0) {
+#ifdef DBUG_OFF
+  if (my_key_remove(key_id.c_str(), nullptr)) {
 #else
   if (DBUG_EVALUATE_IF("rpl_encryption_first_time_enable_4", true, false) ||
       (DBUG_EVALUATE_IF("fail_to_remove_master_key_from_keyring", true,
@@ -833,7 +825,7 @@ bool Rpl_encryption::remove_key_from_keyring(std::string key_id) {
       (DBUG_EVALUATE_IF("fail_to_remove_unused_key_from_keyring",
                         !current_thd->is_error(), false) &&
        key_id.compare(get_master_key_seqno_key_id()) != 0) ||
-      srv_keyring_writer->remove(key_id.c_str(), nullptr) != 0) {
+      my_key_remove(key_id.c_str(), nullptr)) {
 #endif
     report_keyring_error(Keyring_status::KEYRING_ERROR_REMOVING);
     return true;
@@ -958,9 +950,8 @@ bool Rpl_encryption::generate_master_key_on_keyring(uint32 seqno) {
   /* Generate the new key */
   if (DBUG_EVALUATE_IF("rpl_encryption_first_time_enable_2", true, false) ||
       DBUG_EVALUATE_IF("fail_to_generate_key_on_keyring", true, false) ||
-      srv_keyring_generator->generate(
-          key_id.c_str(), nullptr, Rpl_encryption_header_v1::KEY_TYPE,
-          Rpl_encryption_header_v1::KEY_LENGTH) != 0) {
+      my_key_generate(key_id.c_str(), Rpl_encryption_header_v1::KEY_TYPE,
+                      nullptr, Rpl_encryption_header_v1::KEY_LENGTH) != 0) {
     Rpl_encryption::report_keyring_error(
         Keyring_status::KEYRING_ERROR_GENERATING);
     return true;
@@ -1070,19 +1061,19 @@ bool Rpl_encryption_header_v1::serialize(Basic_ostream *ostream) {
   memcpy(header, ENCRYPTION_MAGIC, ENCRYPTION_MAGIC_SIZE);
   header[VERSION_OFFSET] = m_version;
 
-  assert(m_key_id.length() < 255);
+  DBUG_ASSERT(m_key_id.length() < 255);
   ptr = header + OPTIONAL_FIELD_OFFSET;
   *ptr++ = KEY_ID;
   *ptr++ = m_key_id.length();
   memcpy(ptr, m_key_id.data(), m_key_id.length());
   ptr += m_key_id.length();
 
-  assert(m_encrypted_password.length() == PASSWORD_FIELD_SIZE);
+  DBUG_ASSERT(m_encrypted_password.length() == PASSWORD_FIELD_SIZE);
   *ptr++ = ENCRYPTED_FILE_PASSWORD;
   memcpy(ptr, m_encrypted_password.data(), m_encrypted_password.length());
   ptr += PASSWORD_FIELD_SIZE;
 
-  assert(m_iv.length() == IV_FIELD_SIZE);
+  DBUG_ASSERT(m_iv.length() == IV_FIELD_SIZE);
   *ptr++ = IV_FOR_FILE_PASSWORD;
   memcpy(ptr, m_iv.data(), m_iv.length());
 
@@ -1111,7 +1102,7 @@ bool Rpl_encryption_header_v1::deserialize(Basic_istream *istream) {
   m_iv.clear();
 
   const char *header_buffer = reinterpret_cast<char *>(header);
-  mysql::binlog::event::Event_reader reader(header_buffer, HEADER_SIZE);
+  binary_log::Event_reader reader(header_buffer, HEADER_SIZE);
   reader.go_to(OPTIONAL_FIELD_OFFSET);
   uint8_t field_type = 0;
 
@@ -1235,7 +1226,7 @@ bool Rpl_encryption_header_v1::encrypt_file_password(Key_string password_str) {
       rpl_encryption.get_master_key();
 
   /* Get the master key id */
-  assert(master_key.m_id.length() > 0);
+  DBUG_ASSERT(master_key.m_id.length() > 0);
   m_key_id = master_key.m_id;
 
   /* Generate iv, it is a random string. */
@@ -1285,8 +1276,8 @@ std::string Rpl_encryption_header_v1::key_id_prefix() {
   return ostr.str();
 }
 
-std::string Rpl_encryption_header_v1::seqno_to_key_id(uint32_t seqno
-                                                      [[maybe_unused]]) {
+std::string Rpl_encryption_header_v1::seqno_to_key_id(
+    uint32_t seqno MY_ATTRIBUTE((unused))) {
   std::ostringstream ostr;
 #ifdef MYSQL_SERVER
   ostr << key_id_prefix() << "_" << seqno;
@@ -1294,8 +1285,8 @@ std::string Rpl_encryption_header_v1::seqno_to_key_id(uint32_t seqno
   return ostr.str();
 }
 
-std::string Rpl_encryption_header_v1::key_id_with_suffix(const char *suffix
-                                                         [[maybe_unused]]) {
+std::string Rpl_encryption_header_v1::key_id_with_suffix(
+    const char *suffix MY_ATTRIBUTE((unused))) {
   std::ostringstream ostr;
 #ifdef MYSQL_SERVER
   ostr << key_id_prefix() << "_" << suffix;
