@@ -1,4 +1,4 @@
-/* Copyright (c) 2006, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2006, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -25,16 +25,15 @@
 #include <string.h>
 #include <iterator>
 #include <new>
-#include <regex>
 #include <utility>
 
 #include "lex_string.h"
+#include "libbinlogevents/export/binary_log_funcs.h"
 #include "my_byteorder.h"
 #include "my_dbug.h"
+#include "my_loglevel.h"
 #include "my_sys.h"
-#include "mysql/binlog/event/export/binary_log_funcs.h"
 #include "mysql/components/services/log_builtins.h"
-#include "mysql/my_loglevel.h"
 #include "mysql/service_mysql_alloc.h"
 #include "sql/thr_malloc.h"
 
@@ -44,16 +43,14 @@ struct TYPELIB;
 
 #include <algorithm>
 
+#include "libbinlogevents/include/binlog_event.h"  // checksum_crv32
+#include "m_ctype.h"
 #include "m_string.h"
 #include "my_base.h"
 #include "my_bitmap.h"
-#include "mysql/binlog/event/binlog_event.h"  // checksum_crv32
 #include "mysql/components/services/log_builtins.h"
 #include "mysql/psi/psi_memory.h"
-#include "mysql/strings/m_ctype.h"
 #include "mysqld_error.h"
-#include "sql-common/my_decimal.h"
-#include "sql/changestreams/misc/replicated_columns_view_factory.h"  // get_columns_view
 #include "sql/create_field.h"
 #include "sql/dd/dd.h"          // get_dictionary
 #include "sql/dd/dictionary.h"  // is_dd_table_access_allowed
@@ -61,14 +58,14 @@ struct TYPELIB;
 #include "sql/field.h"          // Field
 #include "sql/log.h"
 #include "sql/log_event.h"  // Log_event
-#include "sql/mysqld.h"     // replica_type_conversions_options
+#include "sql/my_decimal.h"
+#include "sql/mysqld.h"  // slave_type_conversions_options
 #include "sql/psi_memory_key.h"
-#include "sql/rpl_replica.h"
-#include "sql/rpl_rli.h"    // Relay_log_info
+#include "sql/rpl_rli.h"  // Relay_log_info
+#include "sql/rpl_slave.h"
 #include "sql/sql_class.h"  // THD
 #include "sql/sql_const.h"
-#include "sql/sql_gipk.h"  // table_has_generated_invisible_primary_key
-#include "sql/sql_lex.h"   // LEX
+#include "sql/sql_lex.h"  // LEX
 #include "sql/sql_list.h"
 #include "sql/sql_plugin_ref.h"
 #include "sql/sql_tmp_table.h"  // create_tmp_table_from_fields
@@ -77,7 +74,7 @@ struct TYPELIB;
 #include "template_utils.h"  // delete_container_pointers
 #include "typelib.h"
 
-using mysql::binlog::event::checksum_crc32;
+using binary_log::checksum_crc32;
 using std::max;
 using std::min;
 using std::unique_ptr;
@@ -146,10 +143,10 @@ static bool is_conversion_ok(int order) {
   DBUG_TRACE;
   bool allow_non_lossy, allow_lossy;
 
-  allow_non_lossy = replica_type_conversions_options &
-                    (1ULL << REPLICA_TYPE_CONVERSIONS_ALL_NON_LOSSY);
-  allow_lossy = replica_type_conversions_options &
-                (1ULL << REPLICA_TYPE_CONVERSIONS_ALL_LOSSY);
+  allow_non_lossy = slave_type_conversions_options &
+                    (1ULL << SLAVE_TYPE_CONVERSIONS_ALL_NON_LOSSY);
+  allow_lossy = slave_type_conversions_options &
+                (1ULL << SLAVE_TYPE_CONVERSIONS_ALL_LOSSY);
 
   DBUG_PRINT("enter", ("order: %d, flags:%s%s", order,
                        allow_non_lossy ? " ALL_NON_LOSSY" : "",
@@ -235,7 +232,7 @@ static bool can_convert_field_to(Field *field, enum_field_types source_type,
                                  Relay_log_info *rli, uint16 mflags,
                                  int *order_var) {
   DBUG_TRACE;
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   char field_type_buf[MAX_FIELD_WIDTH];
   String field_type(field_type_buf, sizeof(field_type_buf), &my_charset_latin1);
   field->sql_type(field_type);
@@ -273,7 +270,7 @@ static bool can_convert_field_to(Field *field, enum_field_types source_type,
     else
       return false;
   } else if (is_array) {
-    // Can't convert between typed array of different types
+    // Can't covert between typed array of different types
     return false;
   } else if (metadata == 0 &&
              (timestamp_cross_check(field->real_type(), source_type) ||
@@ -303,7 +300,7 @@ static bool can_convert_field_to(Field *field, enum_field_types source_type,
     */
     *order_var = -1;
     return true;
-  } else if (!replica_type_conversions_options)
+  } else if (!slave_type_conversions_options)
     return false;
 
   /*
@@ -336,7 +333,7 @@ static bool can_convert_field_to(Field *field, enum_field_types source_type,
             *order_var = 1;  // Always require lossy conversions
           else
             *order_var = compare_lengths(field, source_type, metadata);
-          assert(*order_var != 0);
+          DBUG_ASSERT(*order_var != 0);
           return is_conversion_ok(*order_var);
         }
 
@@ -349,21 +346,19 @@ static bool can_convert_field_to(Field *field, enum_field_types source_type,
       The length comparison check will do the correct job of comparing
       the field lengths (in bytes) of two integer types.
     */
-    case MYSQL_TYPE_BOOL:
     case MYSQL_TYPE_TINY:
     case MYSQL_TYPE_SHORT:
     case MYSQL_TYPE_INT24:
     case MYSQL_TYPE_LONG:
     case MYSQL_TYPE_LONGLONG:
       switch (field->real_type()) {
-        case MYSQL_TYPE_BOOL:
         case MYSQL_TYPE_TINY:
         case MYSQL_TYPE_SHORT:
         case MYSQL_TYPE_INT24:
         case MYSQL_TYPE_LONG:
         case MYSQL_TYPE_LONGLONG:
           *order_var = compare_lengths(field, source_type, metadata);
-          assert(*order_var != 0);
+          DBUG_ASSERT(*order_var != 0);
           return is_conversion_ok(*order_var);
 
         default:
@@ -422,14 +417,13 @@ static bool can_convert_field_to(Field *field, enum_field_types source_type,
     case MYSQL_TYPE_DATETIME:
     case MYSQL_TYPE_YEAR:
     case MYSQL_TYPE_NEWDATE:
+    case MYSQL_TYPE_NULL:
     case MYSQL_TYPE_ENUM:
     case MYSQL_TYPE_SET:
     case MYSQL_TYPE_TIMESTAMP2:
     case MYSQL_TYPE_DATETIME2:
     case MYSQL_TYPE_TIME2:
     case MYSQL_TYPE_TYPED_ARRAY:
-    case MYSQL_TYPE_NULL:
-    case MYSQL_TYPE_INVALID:
       return false;
   }
   return false;  // To keep GCC happy
@@ -442,7 +436,7 @@ static bool can_convert_field_to(Field *field, enum_field_types source_type,
   This function first finds out whether the table belongs to the data
   dictionary. When not, it will compare the master table with an existing
   table on the slave and see if they are compatible with respect to the
-  current settings of @c REPLICA_TYPE_CONVERSIONS.
+  current settings of @c SLAVE_TYPE_CONVERSIONS.
 
   If the tables are compatible and conversions are required, @c
   *tmp_table_var will be set to a virtual temporary table with field
@@ -464,14 +458,14 @@ static bool can_convert_field_to(Field *field, enum_field_types source_type,
                 master table is not compatible with slave table.
 */
 bool table_def::compatible_with(THD *thd, Relay_log_info *rli, TABLE *table,
-                                TABLE **conv_table_var) {
+                                TABLE **conv_table_var) const {
   /*
     Prohibit replication into dictionary internal tables. We know this is
     not DDL (which will be replicated as statements, and rejected by the
     corresponding check for SQL statements), thus 'false' in the call below.
     Also sserting that this is not a DD system thread.
   */
-  assert(!thd->is_dd_system_thread());
+  DBUG_ASSERT(!thd->is_dd_system_thread());
   const dd::Dictionary *dictionary = dd::get_dictionary();
   if (dictionary && !dictionary->is_dd_table_access_allowed(
                         false, false, table->s->db.str, table->s->db.length,
@@ -487,54 +481,16 @@ bool table_def::compatible_with(THD *thd, Relay_log_info *rli, TABLE *table,
     return false;
   }
 
-  if (compute_source_table_gipk_info(*thd, table)) {
-    rli->report(ERROR_LEVEL, ER_REPLICATION_INCOMPATIBLE_TABLE_WITH_GIPK,
-                ER_THD(thd, ER_REPLICATION_INCOMPATIBLE_TABLE_WITH_GIPK),
-                static_cast<int>(size()),
-                decimal_numeric_version_to_string(
-                    thd->variables.immediate_server_version)
-                    .c_str(),
-                table->s->db.str, table->s->table_name.str, table->s->fields);
-    thd->is_slave_error = true;
-    return false;
-  }
-
-  bool replica_has_gipk = table_has_generated_invisible_primary_key(table);
-  std::unique_ptr<cs::util::ReplicatedColumnsView> fields =
-      cs::util::ReplicatedColumnsViewFactory::
-          get_columns_view_with_inbound_filters(rli->info_thd, table, this);
-
   /*
-    We only count columns which exist both in the table and in the event. The
-    computation depends on the following possible differences in table columns
-    and event columns:
-     1. If the table has generated columns, there are extra columns at the right
-        end of the table, which are excluded in the event from 8.0.18 and up.
-     2. It is possible for the replica-side table to have an extra GIPK, when
-        the source has no GIPK.
-     3. It is possible for the source-side table to have an extra GIPK, when the
-        replica has no GIPK.
-
-     4. For the remaining columns, not counting generated columns or differences
-        in GIPK columns, it is allowed that either the source or the replica has
-        less columns at the right of the table.
-
-    To account for cases 1 and 2, we use fields->filtered_size(), which
-    retrieves the number of columns on the replica, not counting generated
-    columns or extra GIPK columns. To account for case 3, we use
-    filtered_size(replica_has_gipk), which retrieves the number of columns in
-    the event, not counting any extra GIPK column in the event. To account for
-    case 4, we compute the minimum of these two numbers.
+    We only check the initial columns for the tables.
   */
-
-  uint const cols_to_check =
-      min<ulong>(fields->filtered_size(), filtered_size(replica_has_gipk));
+  Replicated_columns_view fields{table, Replicated_columns_view::INBOUND, thd};
+  uint const cols_to_check = min<ulong>(fields.filtered_size(), size());
   TABLE *tmp_table = nullptr;
 
-  for (auto it = fields->begin(); it.filtered_pos() < cols_to_check; ++it) {
+  for (auto it = fields.begin(); it.filtered_pos() < cols_to_check; ++it) {
     Field *const field = *it;
-    size_t col = it.translated_pos();
-    size_t absolute_col_pos = it.absolute_pos();
+    size_t col = it.filtered_pos();
     int order;
     if (can_convert_field_to(field, type(col), field_metadata(col),
                              is_array(col), rli, m_flags, &order)) {
@@ -542,7 +498,7 @@ bool table_def::compatible_with(THD *thd, Relay_log_info *rli, TABLE *table,
                            " field '%s' can be converted - order: %d",
                            static_cast<long unsigned int>(col),
                            field->field_name, order));
-      assert(order >= -1 && order <= 1);
+      DBUG_ASSERT(order >= -1 && order <= 1);
 
       /*
         If order is not 0, a conversion is required, so we need to set
@@ -553,24 +509,22 @@ bool table_def::compatible_with(THD *thd, Relay_log_info *rli, TABLE *table,
           This will create the full table with all fields. This is
           necessary to ge the correct field lengths for the record.
         */
-        tmp_table = create_conversion_table(thd, rli, table, replica_has_gipk);
+        tmp_table = create_conversion_table(thd, rli, table);
         if (tmp_table == nullptr) return false;
         /*
           Clear all fields up to, but not including, this column.
         */
-        for (unsigned int i = 0; i < absolute_col_pos; ++i)
-          tmp_table->field[i] = nullptr;
+        for (unsigned int i = 0; i < col; ++i) tmp_table->field[i] = nullptr;
       }
 
-      if (order == 0 && tmp_table != nullptr)
-        tmp_table->field[absolute_col_pos] = nullptr;
+      if (order == 0 && tmp_table != nullptr) tmp_table->field[col] = nullptr;
     } else {
       DBUG_PRINT("debug",
                  ("Checking column %lu -"
                   " field '%s' can not be converted",
                   static_cast<long unsigned int>(col), field->field_name));
-      assert(col < size() && col < table->s->fields);
-      assert(table->s->db.str && table->s->table_name.str);
+      DBUG_ASSERT(col < size() && col < table->s->fields);
+      DBUG_ASSERT(table->s->db.str && table->s->table_name.str);
       const char *db_name = table->s->db.str;
       const char *tbl_name = table->s->table_name.str;
       char source_buf[MAX_FIELD_WIDTH];
@@ -582,7 +536,7 @@ bool table_def::compatible_with(THD *thd, Relay_log_info *rli, TABLE *table,
       show_sql_type(type(col), is_array(col), field_metadata(col),
                     &source_type);
       field->sql_type(target_type);
-      if (!ignored_error_code(ER_SERVER_REPLICA_CONVERSION_FAILED)) {
+      if (!ignored_error_code(ER_SERVER_SLAVE_CONVERSION_FAILED)) {
         report_level = ERROR_LEVEL;
         thd->is_slave_error = true;
       } else if (log_error_verbosity >= 2)
@@ -602,15 +556,15 @@ bool table_def::compatible_with(THD *thd, Relay_log_info *rli, TABLE *table,
         field->sql_type(target_type);
 
       if (report_level != INFORMATION_LEVEL)
-        rli->report(report_level, ER_SERVER_REPLICA_CONVERSION_FAILED,
-                    ER_THD(thd, ER_SERVER_REPLICA_CONVERSION_FAILED),
-                    static_cast<int>(col), db_name, tbl_name,
-                    source_type.c_ptr_safe(), target_type.c_ptr_safe());
+        rli->report(report_level, ER_SERVER_SLAVE_CONVERSION_FAILED,
+                    ER_THD(thd, ER_SERVER_SLAVE_CONVERSION_FAILED), col,
+                    db_name, tbl_name, source_type.c_ptr_safe(),
+                    target_type.c_ptr_safe());
       return false;
     }
   }
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   if (tmp_table) {
     for (unsigned int col = 0; col < tmp_table->s->fields; ++col)
       if (tmp_table->field[col]) {
@@ -644,76 +598,52 @@ bool table_def::compatible_with(THD *thd, Relay_log_info *rli, TABLE *table,
  */
 
 TABLE *table_def::create_conversion_table(THD *thd, Relay_log_info *rli,
-                                          TABLE *target_table,
-                                          bool replica_has_gipk) const {
+                                          TABLE *target_table) const {
   DBUG_TRACE;
 
   List<Create_field> field_list;
   TABLE *conv_table = nullptr;
+  /*
+    At slave, columns may differ. So we should create
+    min(columns@master, columns@slave) columns in the
+    conversion table.
+  */
+  uint const cols_to_create = min<ulong>(target_table->s->fields, size());
 
   // Default value : treat all values signed
   bool unsigned_flag = false;
 
-  // Check if replica_type_conversions contains ALL_UNSIGNED
-  unsigned_flag = replica_type_conversions_options &
-                  (1ULL << REPLICA_TYPE_CONVERSIONS_ALL_UNSIGNED);
+  // Check if slave_type_conversions contains ALL_UNSIGNED
+  unsigned_flag = slave_type_conversions_options &
+                  (1ULL << SLAVE_TYPE_CONVERSIONS_ALL_UNSIGNED);
 
-  // Check if replica_type_conversions contains ALL_SIGNED
+  // Check if slave_type_conversions contains ALL_SIGNED
   unsigned_flag =
-      unsigned_flag && !(replica_type_conversions_options &
-                         (1ULL << REPLICA_TYPE_CONVERSIONS_ALL_SIGNED));
-
-  std::unique_ptr<cs::util::ReplicatedColumnsView> fields = cs::util::
-      ReplicatedColumnsViewFactory::get_columns_view_with_inbound_filters(
-          rli->info_thd, target_table, this);
-
-  /*
-    At the replica, columns may differ. So we should create
-    min(columns@source, columns@replica) columns in the
-    conversion table.
-  */
-  uint const cols_to_create =
-      min<ulong>(fields->filtered_size(), filtered_size(replica_has_gipk));
-
-  bool source_has_gipk = is_gipk_present_on_source_table();
-
-  // When the GIPK is only on the replica, add a GIPK to the conv table
-  if (replica_has_gipk && !source_has_gipk) {
-    Create_field *field_def = new (thd->mem_root) Create_field();
-    if (field_list.push_back(field_def)) return nullptr;
-    field_def->init_for_tmp_table(MYSQL_TYPE_LONGLONG, 20, 0,
-                                  true,           // maybe_null
-                                  unsigned_flag,  // unsigned_flag
-                                  0);
-    field_def->charset = default_charset_info;
-    field_def->interval = nullptr;
-  }
-
-  for (auto it = fields->begin(); it.filtered_pos() < cols_to_create; ++it) {
-    size_t col_i = it.translated_pos();
+      unsigned_flag && !(slave_type_conversions_options &
+                         (1ULL << SLAVE_TYPE_CONVERSIONS_ALL_SIGNED));
 
+  for (uint col = 0; col < cols_to_create; ++col) {
     Create_field *field_def = new (thd->mem_root) Create_field();
     if (field_list.push_back(field_def)) return nullptr;
 
     uint decimals = 0;
     TYPELIB *interval = nullptr;
     uint pack_length_override = 0;  // 0 => NA. Only assigned below when needed.
-    enum_field_types field_type = type(col_i);
+    enum_field_types field_type = type(col);
     uint32 max_length =
-        max_display_length_for_field(field_type, field_metadata(col_i));
+        max_display_length_for_field(field_type, field_metadata(col));
 
     switch (field_type) {
       uint precision;
       case MYSQL_TYPE_ENUM:
       case MYSQL_TYPE_SET:
-
-        interval = static_cast<Field_enum *>(*it)->typelib;
+        interval = static_cast<Field_enum *>(target_table->field[col])->typelib;
         /*
           Number of elements in interval on master and slave might differ.
           Use pack length from binary log instead of one calculated from
           number of interval elements on slave.
         */
-        pack_length_override = field_metadata(col_i) & 0x00ff;
+        pack_length_override = field_metadata(col) & 0x00ff;
         break;
 
       case MYSQL_TYPE_NEWDECIMAL:
@@ -722,15 +652,15 @@ TABLE *table_def::create_conversion_table(THD *thd, Relay_log_info *rli,
           length that should be supplied to make_field, so we correct
           the length here.
          */
-        precision = field_metadata(col_i) >> 8;
-        decimals = field_metadata(col_i) & 0x00ff;
+        precision = field_metadata(col) >> 8;
+        decimals = field_metadata(col) & 0x00ff;
         max_length = my_decimal_precision_to_length(precision, decimals, false);
         break;
 
       case MYSQL_TYPE_DECIMAL:
         LogErr(ERROR_LEVEL, ER_RPL_INCOMPATIBLE_DECIMAL_IN_RBR,
                target_table->s->db.str, target_table->s->table_name.str,
-               it->field_name);
+               target_table->field[col]->field_name);
         goto err;
 
       case MYSQL_TYPE_BLOB:
@@ -743,7 +673,7 @@ TABLE *table_def::create_conversion_table(THD *thd, Relay_log_info *rli,
           derived from the exact type, i.e. for ENUM and SET (see
           above).
         */
-        field_type = blob_type_from_pack_length(field_metadata(col_i) & 0x00ff);
+        field_type = blob_type_from_pack_length(field_metadata(col) & 0x00ff);
         break;
 
       default:
@@ -754,13 +684,13 @@ TABLE *table_def::create_conversion_table(THD *thd, Relay_log_info *rli,
         "debug",
         ("sql_type: %d, target_field: '%s', max_length: %d, decimals: %d,"
          " maybe_null: %d, unsigned_flag: %d",
-         binlog_type(col_i), it->field_name, max_length, decimals, true,
-         unsigned_flag));
+         binlog_type(col), target_table->field[col]->field_name, max_length,
+         decimals, true, unsigned_flag));
     field_def->init_for_tmp_table(field_type, max_length, decimals,
                                   true,           // maybe_null
                                   unsigned_flag,  // unsigned_flag
                                   pack_length_override);
-    field_def->charset = it->charset();
+    field_def->charset = target_table->field[col]->charset();
     field_def->interval = interval;
   }
 
@@ -770,68 +700,20 @@ TABLE *table_def::create_conversion_table(THD *thd, Relay_log_info *rli,
 err:
   if (conv_table == nullptr) {
     enum loglevel report_level = INFORMATION_LEVEL;
-    if (!ignored_error_code(ER_REPLICA_CANT_CREATE_CONVERSION)) {
+    if (!ignored_error_code(ER_SLAVE_CANT_CREATE_CONVERSION)) {
       report_level = ERROR_LEVEL;
       thd->is_slave_error = true;
     } else if (log_error_verbosity >= 2)
       report_level = WARNING_LEVEL;
 
     if (report_level != INFORMATION_LEVEL)
-      rli->report(report_level, ER_REPLICA_CANT_CREATE_CONVERSION,
-                  ER_THD(thd, ER_REPLICA_CANT_CREATE_CONVERSION),
+      rli->report(report_level, ER_SLAVE_CANT_CREATE_CONVERSION,
+                  ER_THD(thd, ER_SLAVE_CANT_CREATE_CONVERSION),
                   target_table->s->db.str, target_table->s->table_name.str);
   }
   return conv_table;
 }
 
-bool table_def::is_gipk_present_on_source_table() const {
-  assert(m_is_gipk_set);
-  return m_is_gipk_on_table;
-}
-
-bool table_def::compute_source_table_gipk_info(THD &thd, TABLE *table) {
-  m_is_gipk_set = true;
-
-  // Check if source signals the use of GIPKs on tables
-  if (is_immediate_server_gipk_ready(thd)) {
-    m_is_gipk_on_table =
-        (m_flags & Table_map_log_event::TM_GENERATED_INVISIBLE_PK_F);
-    return false;
-  }
-
-  // If the replica has no GIPK assume the source doesn't also
-  if (!table_has_generated_invisible_primary_key(table)) {
-    m_is_gipk_on_table = false;
-    return false;
-  }
-
-  // column difference = number of columns in source - replica
-  longlong column_difference =
-      static_cast<longlong>(size()) - static_cast<longlong>(table->s->fields);
-
-  // if there is no difference assume the source has a GIPK
-  if (0 == column_difference) {
-    m_is_gipk_on_table = true;
-    return false;
-  }
-  /*
-    Here it is known the replica has a GIPK.
-    - If the replica has exactly 1 column more than the source,
-      then assume the source does not contain a GIPK
-    - If the replica has less columns than the source
-      or the replica has 2 or more columns extra in relation to the source,
-      we consider that an error
-  */
-  if (-1 == column_difference) {
-    m_is_gipk_on_table = false;
-    return false;
-  }
-
-  m_is_gipk_on_table = false;
-  m_is_gipk_set = false;
-  return true;
-}
-
 #endif /* MYSQL_SERVER */
 
 /**
@@ -937,9 +819,7 @@ table_def::table_def(unsigned char *types, ulong size, uchar *field_metadata,
       m_flags(flags),
       m_memory(nullptr),
       m_json_column_count(-1),
-      m_is_array(nullptr),
-      m_is_gipk_set(false),
-      m_is_gipk_on_table(false) {
+      m_is_array(nullptr) {
   m_memory = (uchar *)my_multi_malloc(
       key_memory_table_def_memory, MYF(MY_WME), &m_type, size,
       &m_field_metadata, size * sizeof(uint), &m_is_array, size * sizeof(bool),
@@ -971,7 +851,7 @@ table_def::table_def(unsigned char *types, ulong size, uchar *field_metadata,
       m_field_metadata[i] = pack.second.first;
       m_is_array[i] = pack.second.second;
       index += pack.first;
-      assert(index <= metadata_size);
+      DBUG_ASSERT(index <= metadata_size);
     }
   }
   if (m_size && null_bitmap) memcpy(m_null_bits, null_bitmap, (m_size + 7) / 8);
@@ -979,7 +859,7 @@ table_def::table_def(unsigned char *types, ulong size, uchar *field_metadata,
 
 table_def::~table_def() {
   my_free(m_memory);
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   m_type = nullptr;
   m_size = 0;
 #endif
@@ -1119,7 +999,7 @@ HASH_ROW_ENTRY *Hash_slave_rows::get(TABLE *table, MY_BITMAP *cols) {
 
 bool Hash_slave_rows::next(HASH_ROW_ENTRY **entry) {
   DBUG_TRACE;
-  assert(*entry);
+  DBUG_ASSERT(*entry);
 
   if (*entry == nullptr) return true;
 
@@ -1160,7 +1040,7 @@ bool Hash_slave_rows::next(HASH_ROW_ENTRY **entry) {
 
 bool Hash_slave_rows::del(HASH_ROW_ENTRY *entry) {
   DBUG_TRACE;
-  assert(entry);
+  DBUG_ASSERT(entry);
 
   erase_specific_element(&m_hash, entry->preamble->hash_value, entry);
   return false;
@@ -1208,14 +1088,14 @@ uint Hash_slave_rows::make_hash_key(TABLE *table, MY_BITMAP *cols) {
     DBUG_PRINT("debug", ("make_hash_entry: hash after null_flags: %u", crc));
   }
 
-  for (Field **ptr = table->field;
-       *ptr && ((*ptr)->field_index() < cols->n_bits); ptr++) {
+  for (Field **ptr = table->field; *ptr && ((*ptr)->field_index < cols->n_bits);
+       ptr++) {
     Field *f = (*ptr);
 
     /*
       Field is set in the read_set and is isn't NULL.
      */
-    if (bitmap_is_set(cols, f->field_index()) &&
+    if (bitmap_is_set(cols, f->field_index) &&
         !f->is_virtual_gcol() &&  // Avoid virtual generated columns on hashes
         !f->is_null()) {
       /*
@@ -1236,10 +1116,10 @@ uint Hash_slave_rows::make_hash_key(TABLE *table, MY_BITMAP *cols) {
           break;
         }
         default:
-          crc = checksum_crc32(crc, f->field_ptr(), f->data_length());
+          crc = checksum_crc32(crc, f->ptr, f->data_length());
           break;
       }
-#ifndef NDEBUG
+#ifndef DBUG_OFF
       String tmp;
       f->val_str(&tmp);
       DBUG_PRINT("debug", ("make_hash_entry: hash after field %s=%s: %u",
@@ -1286,7 +1166,7 @@ bool Deferred_log_events::is_empty() { return m_array.empty(); }
 bool Deferred_log_events::execute(Relay_log_info *rli) {
   bool res = false;
 
-  assert(rli->deferred_events_collecting);
+  DBUG_ASSERT(rli->deferred_events_collecting);
 
   rli->deferred_events_collecting = false;
   for (Log_event **it = m_array.begin(); !res && it != m_array.end(); ++it) {
@@ -1345,16 +1225,7 @@ THD_instance_guard::~THD_instance_guard() {
 
 THD_instance_guard::operator THD *() { return this->m_target; }
 
-/**
-  This method shall evaluate if a command being executed goes against any of
-  the restrictions of server variable session.require_row_format.
-
-  @param thd The thread associated to the command
-  @return true if it violates any restrictions
-          false otherwise
- */
-bool is_require_row_format_violation(const THD *thd) {
-  DBUG_TRACE;
+bool evaluate_command_row_only_restrictions(THD *thd) {
   LEX *const lex = thd->lex;
 
   switch (lex->sql_command) {
@@ -1382,74 +1253,4 @@ bool is_require_row_format_violation(const THD *thd) {
   return false;
 }
 
-// TODO: once the old syntax is removed, remove this as well.
-static const std::unordered_map<std::string, std::string> deprecated_field_map{
-    {"Replica_IO_State", "Slave_IO_State"},
-    {"Source_Host", "Master_Host"},
-    {"Source_User", "Master_User"},
-    {"Source_Port", "Master_Port"},
-    {"Source_Log_File", "Master_Log_File"},
-    {"Read_Source_Log_Pos", "Read_Master_Log_Pos"},
-    {"Relay_Source_Log_File", "Relay_Master_Log_File"},
-    {"Replica_IO_Running", "Slave_IO_Running"},
-    {"Replica_SQL_Running", "Slave_SQL_Running"},
-    {"Exec_Source_Log_Pos", "Exec_Master_Log_Pos"},
-    {"Source_SSL_Allowed", "Master_SSL_Allowed"},
-    {"Source_SSL_CA_File", "Master_SSL_CA_File"},
-    {"Source_SSL_CA_Path", "Master_SSL_CA_Path"},
-    {"Source_SSL_Cert", "Master_SSL_Cert"},
-    {"Source_SSL_Cipher", "Master_SSL_Cipher"},
-    {"Source_SSL_Key", "Master_SSL_Key"},
-    {"Seconds_Behind_Source", "Seconds_Behind_Master"},
-    {"Source_SSL_Verify_Server_Cert", "Master_SSL_Verify_Server_Cert"},
-    {"Source_Server_Id", "Master_Server_Id"},
-    {"Source_UUID", "Master_UUID"},
-    {"Source_Info_File", "Master_Info_File"},
-    {"Replica_SQL_Running_State", "Slave_SQL_Running_State"},
-    {"Source_Retry_Count", "Master_Retry_Count"},
-    {"Source_Bind", "Master_Bind"},
-    {"Source_SSL_Crl", "Master_SSL_Crl"},
-    {"Source_SSL_Crlpath", "Master_SSL_Crlpath"},
-    {"Source_TLS_Version", "Master_TLS_Version"},
-    {"Source_public_key_path", "Master_public_key_path"},
-    {"Get_Source_public_key", "Get_master_public_key"},
-    {"Server_Id", "Server_id"},
-    {"Source_Id", "Master_id"},
-    {"Replica_UUID", "Slave_UUID"}};
-
-void rename_fields_use_old_replica_source_terms(
-    THD *thd, mem_root_deque<Item *> &field_list) {
-  for (auto &item : field_list) {
-    std::string name{item->full_name()};
-    auto itr = deprecated_field_map.find(name);
-    if (itr != deprecated_field_map.end()) name = itr->second;
-    item->rename(thd->mem_strdup(name.c_str()));
-  }
-}
-
-bool is_immediate_server_gipk_ready(THD &thd) {
-  return thd.variables.immediate_server_version != UNDEFINED_SERVER_VERSION &&
-         thd.variables.immediate_server_version >= 80030;
-}
-
-bool does_source_table_contain_gipk(Relay_log_info const *rli, TABLE *table) {
-  table_def *tabledef = nullptr;
-  TABLE *conv_table = nullptr;
-  rli->get_table_data(table, &tabledef, &conv_table);
-  assert(tabledef != nullptr);
-  return tabledef->is_gipk_present_on_source_table();
-}
-
-std::string decimal_numeric_version_to_string(uint32 version) {
-  if (version == UNDEFINED_SERVER_VERSION ||
-      version == UNKNOWN_SERVER_VERSION) {
-    return "unknown";
-  } else {
-    std::stringstream version_str;
-    version_str << (version / 10000) << "." << ((version / 100) % 100) << "."
-                << (version % 100);
-    return version_str.str();
-  }
-}
-
-#endif  // MYSQL_SERVER
+#endif  // MYSQL_SERVER
\ No newline at end of file
