@@ -1,16 +1,15 @@
-/* Copyright (c) 2019, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,6 +23,21 @@
 #include "mysql_connection_attributes_iterator_imp.h"
 #include <sql/sql_class.h>
 
+/**
+  Initialize an iterator.
+
+  Also position at the first attribute.
+
+  @param      opaque_thd      The session to operate on. Can be NULL to use the
+                              current THD.
+  @param[out] iterator        Iterator pointer.
+
+  @return
+    @retval false Succeeded.
+    @retval true  Failed.
+
+  @sa mysql_connection_attributes_iterator::init
+*/
 DEFINE_BOOL_METHOD(mysql_connection_attributes_iterator_imp::init,
                    (MYSQL_THD opaque_thd,
                     my_h_connection_attributes_iterator *iterator)) {
@@ -43,6 +57,17 @@ DEFINE_BOOL_METHOD(mysql_connection_attributes_iterator_imp::init,
   }
 }
 
+/**
+  Deinitialize an iterator.
+
+  @param iterator Iterator pointer.
+
+  @return
+    @retval false Succeeded.
+    @retval true  Failed.
+
+  @sa mysql_connection_attributes_iterator::deinit
+*/
 DEFINE_BOOL_METHOD(mysql_connection_attributes_iterator_imp::deinit,
                    (my_h_connection_attributes_iterator iterator)) {
   if (iterator != nullptr) {
@@ -67,7 +92,7 @@ std::tuple<const char *, size_t, bool> parse_length_encoded_string(
   // check if we're in the connection attributes string boundaries
   if (*str < connection_attributes || *str > connection_attributes +
                                                  connection_attributes_length -
-                                                 sizeof(char))
+                                                 sizeof(size_t))
     return std::make_tuple(nullptr, 0, false);
 
   // The first item of the string is the length of the attribute
@@ -92,6 +117,27 @@ std::tuple<const char *, size_t, bool> parse_length_encoded_string(
   return std::make_tuple(result, length, true);
 }
 
+/**
+  Fetch the current name/value pair from the iterator and move it forward.
+  Note: the attribute's name and value pointers are valid until the THD object
+  is alive.
+
+  @param      opaque_thd      The session to operate on. Can be NULL to use the
+                              current THD.
+  @param      iterator        Iterator pointer.
+  @param[out] name            The attribute name.
+  @param[out] name_length     The attribute name's length.
+  @param[out] value           The attribute value.
+  @param[out] value_length    The attribute value's length.
+  @param[out] client_charset  The character set, used for encoding the
+                              connection attributes pair
+
+  @return
+    @retval false Succeeded.
+    @retval true  Failed.
+
+  @sa mysql_connection_attributes_iterator::get
+*/
 DEFINE_BOOL_METHOD(mysql_connection_attributes_iterator_imp::get,
                    (MYSQL_THD opaque_thd,
                     my_h_connection_attributes_iterator *iterator,
@@ -117,7 +163,7 @@ DEFINE_BOOL_METHOD(mysql_connection_attributes_iterator_imp::get,
 
       if (!isParsingSuccessful) return true;
 
-      *client_charset = thd->variables.character_set_client->m_coll_name;
+      *client_charset = thd->variables.character_set_client->name;
 
       return false;  // Success
     } else
