@@ -1,4 +1,4 @@
-/* Copyright (c) 2014, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -44,7 +44,6 @@
 #include "mysql_com.h"
 #include "mysqld_error.h"
 #include "sql/current_thd.h"
-#include "sql/protocol.h"
 #include "sql/psi_memory_key.h"
 #include "sql/query_options.h"
 #include "sql/rpl_context.h"
@@ -62,10 +61,6 @@
 
 static void store_lenenc_string(String &to, const char *from, size_t length);
 
-static bool is_component_registered_var_name(const char *name) {
-  return strchr(name, '.') != nullptr;
-}
-
 /**
   Session_sysvars_tracker
   -----------------------
@@ -119,7 +114,7 @@ class Session_sysvars_tracker : public State_tracker {
    public:
     vars_list(const CHARSET_INFO *char_set) { init(char_set); }
 
-    void claim_memory_ownership(bool claim) { my_claim(variables_list, claim); }
+    void claim_memory_ownership() { my_claim(variables_list); }
 
     ~vars_list() {
       if (variables_list) my_free(variables_list);
@@ -163,7 +158,7 @@ class Session_sysvars_tracker : public State_tracker {
   }
 
   /** Destructor */
-  ~Session_sysvars_tracker() override {
+  ~Session_sysvars_tracker() {
     if (orig_list) delete orig_list;
     if (tool_list) delete tool_list;
   }
@@ -182,17 +177,17 @@ class Session_sysvars_tracker : public State_tracker {
   }
 
   void reset();
-  bool enable(THD *thd) override;
-  bool check(THD *thd, set_var *var) override;
-  bool update(THD *thd) override;
-  bool store(THD *thd, String &buf) override;
-  void mark_as_changed(THD *thd, LEX_CSTRING tracked_item_name) override;
+  bool enable(THD *thd);
+  bool check(THD *thd, set_var *var);
+  bool update(THD *thd);
+  bool store(THD *thd, String &buf);
+  void mark_as_changed(THD *thd, LEX_CSTRING *tracked_item_name);
   /* callback */
   static const uchar *sysvars_get_key(const uchar *entry, size_t *length);
 
-  void claim_memory_ownership(bool claim) override {
-    if (orig_list != nullptr) orig_list->claim_memory_ownership(claim);
-    if (tool_list != nullptr) tool_list->claim_memory_ownership(claim);
+  virtual void claim_memory_ownership() {
+    if (orig_list != nullptr) orig_list->claim_memory_ownership();
+    if (tool_list != nullptr) tool_list->claim_memory_ownership();
   }
 };
 
@@ -212,11 +207,11 @@ class Current_schema_tracker : public State_tracker {
   /** Constructor */
   Current_schema_tracker() { schema_track_inited = false; }
 
-  bool enable(THD *thd) override { return update(thd); }
-  bool check(THD *, set_var *) override { return false; }
-  bool update(THD *thd) override;
-  bool store(THD *thd, String &buf) override;
-  void mark_as_changed(THD *thd, LEX_CSTRING tracked_item_name) override;
+  bool enable(THD *thd) { return update(thd); }
+  bool check(THD *, set_var *) { return false; }
+  bool update(THD *thd);
+  bool store(THD *thd, String &buf);
+  void mark_as_changed(THD *thd, LEX_CSTRING *tracked_item_name);
 };
 
 /* To be used in expanding the buffer. */
@@ -238,8 +233,8 @@ static const unsigned int EXTRA_ALLOC = 1024;
 */
 class Session_gtids_ctx_encoder {
  public:
-  Session_gtids_ctx_encoder() = default;
-  virtual ~Session_gtids_ctx_encoder() = default;
+  Session_gtids_ctx_encoder() {}
+  virtual ~Session_gtids_ctx_encoder() {}
 
   /*
    This function SHALL encode the collected GTIDs into the buffer.
@@ -268,12 +263,12 @@ class Session_gtids_ctx_encoder {
 
 class Session_gtids_ctx_encoder_string : public Session_gtids_ctx_encoder {
  public:
-  Session_gtids_ctx_encoder_string() = default;
-  ~Session_gtids_ctx_encoder_string() override = default;
+  Session_gtids_ctx_encoder_string() {}
+  ~Session_gtids_ctx_encoder_string() {}
 
-  ulonglong encoding_specification() override { return 0; }
+  ulonglong encoding_specification() { return 0; }
 
-  bool encode(THD *thd, String &buf) override {
+  bool encode(THD *thd, String &buf) {
     const Gtid_set *state = thd->rpl_thd_ctx.session_gtids_ctx().state();
 
     if (!state->is_empty()) {
@@ -351,27 +346,25 @@ class Session_gtids_tracker
       : Session_consistency_gtids_ctx::Ctx_change_listener(),
         m_encoder(nullptr) {}
 
-  ~Session_gtids_tracker() override {
+  ~Session_gtids_tracker() {
     /*
      Unregister the listener if the tracker is being freed. This is needed
      since this may happen after a change user command.
      */
-    if (current_thd)
+    if (m_enabled && current_thd)
       current_thd->rpl_thd_ctx.session_gtids_ctx()
           .unregister_ctx_change_listener(this);
     if (m_encoder) delete m_encoder;
   }
 
-  bool enable(THD *thd) override { return update(thd); }
-  bool check(THD *, set_var *) override { return false; }
-  bool update(THD *thd) override;
-  bool store(THD *thd, String &buf) override;
-  void mark_as_changed(THD *thd, LEX_CSTRING tracked_item_name) override;
+  bool enable(THD *thd) { return update(thd); }
+  bool check(THD *, set_var *) { return false; }
+  bool update(THD *thd);
+  bool store(THD *thd, String &buf);
+  void mark_as_changed(THD *thd, LEX_CSTRING *tracked_item_name);
 
   // implementation of the Session_gtids_ctx::Ctx_change_listener
-  void notify_session_gtids_ctx_change() override {
-    mark_as_changed(nullptr, {});
-  }
+  void notify_session_gtids_ctx_change() { mark_as_changed(nullptr, nullptr); }
 };
 
 void Session_sysvars_tracker::vars_list::reset() {
@@ -387,7 +380,7 @@ void Session_sysvars_tracker::vars_list::reset() {
   the members from the other.
 
   @@param  from    Source vars_list object.
-  @@param  thd     THD handle to retrieve the charset in use.
+  @@param  thd     THD handle to retrive the charset in use.
 
   @@return    true if the m_registered_sysvars hash has any records.
               Else the value of track_all.
@@ -468,6 +461,7 @@ bool Session_sysvars_tracker::vars_list::parse_var_list(
     THD *thd, LEX_STRING var_list, bool throw_error,
     const CHARSET_INFO *char_set, bool session_created) {
   const char *separator = ",";
+  char *token, *lasts = nullptr; /* strtok_r */
 
   if (!var_list.str) {
     variables_list = nullptr;
@@ -487,98 +481,52 @@ bool Session_sysvars_tracker::vars_list::parse_var_list(
     }
   }
 
-  bool needs_system_variable_hash_lock = false;
-  bool needs_plugin_lock = false;
-  if (thd == nullptr || session_created) {
-    for (char *lasts, *token = my_strtok_r(variables_list, separator, &lasts);
-         token != nullptr; token = my_strtok_r(nullptr, separator, &lasts)) {
-      LEX_STRING var{token, strlen(token)};
-      /* Remove leading/trailing whitespace. */
-      trim_whitespace(char_set, &var);
-      if (var.length == 0) {
-        continue;
-      }
-      switch (System_variable_tracker::make_tracker(to_string_view(var))
-                  .lifetime()) {
-        case System_variable_tracker::PLUGIN:
-          needs_plugin_lock = true;
-          needs_system_variable_hash_lock = true;
-          break;
-        case System_variable_tracker::COMPONENT:
-          needs_system_variable_hash_lock = true;
-          break;
-        default:
-          break;
-      }
-      if (needs_plugin_lock && needs_system_variable_hash_lock) {
-        break;
-      }
-    }
-  }
-  strncpy(variables_list, var_list.str, var_list.length);
-
-  struct RAII {
-    RAII(bool needs_system_variable_hash_lock, bool needs_plugin_lock)
-        : m_needs_system_variable_hash_lock{needs_system_variable_hash_lock},
-          m_needs_plugin_lock{needs_plugin_lock} {
-      extern mysql_mutex_t LOCK_plugin;
-      extern mysql_rwlock_t LOCK_system_variables_hash;
-      if (m_needs_system_variable_hash_lock) {
-        mysql_mutex_assert_not_owner(&LOCK_plugin);
-        mysql_rwlock_rdlock(&LOCK_system_variables_hash);
-      }
-      if (m_needs_plugin_lock) {
-        mysql_mutex_lock(&LOCK_plugin);
-      }
-    }
-    ~RAII() {
-      if (m_needs_plugin_lock) {
-        extern mysql_mutex_t LOCK_plugin;
-        mysql_mutex_unlock(&LOCK_plugin);
-      }
-      if (m_needs_system_variable_hash_lock) {
-        extern mysql_rwlock_t LOCK_system_variables_hash;
-        mysql_rwlock_unlock(&LOCK_system_variables_hash);
-      }
-    }
-    const bool m_needs_system_variable_hash_lock;
-    const bool m_needs_plugin_lock;
-  } raii{needs_system_variable_hash_lock, needs_plugin_lock};
+  token = my_strtok_r(variables_list, separator, &lasts);
 
   track_all = false;
+  /*
+    If Lock to the plugin mutex is not acquired here itself, it results
+    in having to acquire it multiple times in find_sys_var_ex for each
+    token value. Hence the mutex is handled here to avoid a performance
+    overhead.
+  */
+  if (!thd || session_created) lock_plugin_mutex();
+  while (token) {
+    LEX_STRING var;
+    var.str = token;
+    var.length = strlen(token);
 
-  for (char *lasts, *token = my_strtok_r(variables_list, separator, &lasts);
-       token != nullptr; token = my_strtok_r(nullptr, separator, &lasts)) {
-    LEX_STRING var{token, strlen(token)};
     /* Remove leading/trailing whitespace. */
     trim_whitespace(char_set, &var);
-    if (var.length == 0) {
-      continue;
-    }
 
     if (!thd || session_created) {
-      if (!System_variable_tracker::make_tracker(to_string_view(var))
-               .access_system_variable(
-                   thd, {}, Suppress_not_found_error::YES,
-                   Force_sensitive_system_variable_access::NO,
-                   Is_already_locked::YES)) {
+      if (find_sys_var_ex(thd, var.str, var.length, throw_error, true)) {
         if (insert(nullptr, to_lex_cstring(var)) == true) {
-          return true;  // Error inserting into the hash.
+          /* Error inserting into the hash. */
+          unlock_plugin_mutex();
+          return true; /* Error */
         }
-      } else if (throw_error) {
-        assert(thd);
+      }
+
+      else if (throw_error) {
+        DBUG_ASSERT(thd);
         push_warning_printf(
             thd, Sql_condition::SL_WARNING, ER_WRONG_VALUE_FOR_VAR,
             "%s is not a valid system variable and will be ignored.", token);
       } else {
+        unlock_plugin_mutex();
         return true;
       }
     } else {
       if (insert(nullptr, to_lex_cstring(var)) == true) {
-        return true;  // Error inserting into the hash.
+        /* Error inserting into the hash. */
+        return true; /* Error */
       }
     }
+
+    token = my_strtok_r(nullptr, separator, &lasts);
   }
+  if (!thd || session_created) unlock_plugin_mutex();
 
   return false;
 }
@@ -676,7 +624,13 @@ bool Session_sysvars_tracker::update(THD *thd) {
 */
 
 bool Session_sysvars_tracker::store(THD *thd, String &buf) {
+  char val_buf[1024];
+  const char *value;
   SHOW_VAR *show;
+  sys_var *var;
+  const CHARSET_INFO *charset;
+  size_t val_length, length;
+  uchar *to;
 
   if (!(show = (SHOW_VAR *)thd->alloc(sizeof(SHOW_VAR)))) return true;
 
@@ -698,35 +652,20 @@ bool Session_sysvars_tracker::store(THD *thd, String &buf) {
             });
 
   for (sysvar_node_st *node : vars) {
-    if (!node->m_changed) {
-      continue;
-    }
-    if (is_component_registered_var_name(node->m_sysvar_name.str)) {
-      /*
-        The current system variable is a component-registered one.
-        Currently, all component variables are "global" (i.e. not "session").
-        Thus, simply skip them:
-      */
-      continue;
-    }
-    auto f = [thd, &buf, show, node](const System_variable_tracker &,
-                                     sys_var *var) {
+    if (node->m_changed &&
+        (var = find_sys_var_ex(thd, node->m_sysvar_name.str,
+                               node->m_sysvar_name.length, true, false))) {
       show->name = var->name.str;
       show->value = (char *)var;
 
-      char val_buf[1024];
-      size_t val_length;
-      const CHARSET_INFO *charset;
-      const char *value =
-          get_one_variable(thd, show, OPT_SESSION, show->type, nullptr,
-                           &charset, val_buf, &val_length);
+      value = get_one_variable(thd, show, OPT_SESSION, show->type, nullptr,
+                               &charset, val_buf, &val_length);
 
-      size_t length = net_length_size(node->m_sysvar_name.length) +
-                      node->m_sysvar_name.length + net_length_size(val_length) +
-                      val_length;
-      /* allocate 1 bytes more for session state type. */
-      uchar *to =
-          (uchar *)buf.prep_append(net_length_size(length) + 1, EXTRA_ALLOC);
+      length = net_length_size(node->m_sysvar_name.length) +
+               node->m_sysvar_name.length + net_length_size(val_length) +
+               val_length;
+
+      to = (uchar *)buf.prep_append(net_length_size(length) + 1, EXTRA_ALLOC);
 
       /* Session state type (SESSION_TRACK_SYSTEM_VARIABLES) */
       to = net_store_length(to, (ulonglong)SESSION_TRACK_SYSTEM_VARIABLES);
@@ -758,11 +697,6 @@ bool Session_sysvars_tracker::store(THD *thd, String &buf) {
 
       /* System variable's value (length-encoded string). */
       store_lenenc_string(buf, value, val_length);
-    };
-    if (System_variable_tracker::make_tracker(
-            to_string_view(node->m_sysvar_name))
-            .access_system_variable(thd, f, Suppress_not_found_error::YES)) {
-      continue;
     }
   }
 
@@ -779,14 +713,17 @@ bool Session_sysvars_tracker::store(THD *thd, String &buf) {
 */
 
 void Session_sysvars_tracker::mark_as_changed(THD *thd,
-                                              LEX_CSTRING tracked_item_name) {
-  assert(tracked_item_name.str);
+                                              LEX_CSTRING *tracked_item_name) {
+  DBUG_ASSERT(tracked_item_name->str);
   sysvar_node_st *node = nullptr;
+  LEX_CSTRING tmp;
+  tmp.str = tracked_item_name->str;
+  tmp.length = tracked_item_name->length;
   /*
     Check if the specified system variable is being tracked, if so
     mark it as changed and also set the class's m_changed flag.
   */
-  if ((node = orig_list->search(node, tracked_item_name))) {
+  if ((node = orig_list->search(node, tmp))) {
     node->m_changed = true;
     m_changed = true;
     /* do not cache the statement when there is change in session state */
@@ -865,6 +802,7 @@ bool Current_schema_tracker::store(THD *thd, String &buf) {
 
   /* Session state type (SESSION_TRACK_SCHEMA) */
   to = net_store_length(to, (ulonglong)SESSION_TRACK_SCHEMA);
+
   /* Length of the overall entity. */
   to = net_store_length(to, length);
 
@@ -886,9 +824,8 @@ bool Current_schema_tracker::store(THD *thd, String &buf) {
   @param tracked_item_name Always null (unused).
 */
 
-void Current_schema_tracker::mark_as_changed(THD *thd,
-                                             LEX_CSTRING tracked_item_name
-                                             [[maybe_unused]]) {
+void Current_schema_tracker::mark_as_changed(
+    THD *thd, LEX_CSTRING *tracked_item_name MY_ATTRIBUTE((unused))) {
   m_changed = true;
   thd->lex->safe_to_cache_query = false;
 }
@@ -936,7 +873,7 @@ bool Transaction_state_tracker::update(THD *thd) {
     }
     if (thd->variables.session_track_transaction_info == TX_TRACK_CHISTICS)
       tx_changed |= TX_CHG_CHISTICS;
-    mark_as_changed(thd, {});
+    mark_as_changed(thd, nullptr);
   } else
     m_enabled = false;
 
@@ -1039,7 +976,7 @@ bool Transaction_state_tracker::store(THD *thd, String &buf) {
            legal and equivalent syntax in MySQL, or START TRANSACTION
            sans options) will re-use any one-shots set up so far
            (with SET before the first transaction started, and with
-           all subsequent STARTs), except for WITH CONSISTENT SNAPSHOT,
+           all subsequent STARTs), except for WITH CONSISTANT SNAPSHOT,
            which will never be chained and only applies when explicitly
            given.
 
@@ -1053,7 +990,7 @@ bool Transaction_state_tracker::store(THD *thd, String &buf) {
         statement even for a transaction that isn't the first in an
         ongoing chain. Consider
 
-          SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED;
+          SET TRANSACTION ISOLATION LEVEL READ UNCOMMITED;
           START TRANSACTION READ ONLY, WITH CONSISTENT SNAPSHOT;
           # work
           COMMIT AND CHAIN;
@@ -1061,7 +998,7 @@ bool Transaction_state_tracker::store(THD *thd, String &buf) {
         If we switch away at this point, the replay in the new session
         needs to be
 
-          SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED;
+          SET TRANSACTION ISOLATION LEVEL READ UNCOMMITED;
           START TRANSACTION READ ONLY;
 
         When a transaction ends (COMMIT/ROLLBACK sans CHAIN), all
@@ -1197,13 +1134,13 @@ bool Transaction_state_tracker::store(THD *thd, String &buf) {
       */
       length += net_length_size(length);  // ... plus that of its length
 
-      /* allocate extra 1 bytes for session state type. */
       uchar *to =
           (uchar *)buf.prep_append(net_length_size(length) + 1, EXTRA_ALLOC);
 
       /* Session state type (SESSION_TRACK_TRANSACTION_CHARACTERISTICS) */
       to = net_store_length(
           (uchar *)to, (ulonglong)SESSION_TRACK_TRANSACTION_CHARACTERISTICS);
+
       /* Length of the overall entity. */
       to = net_store_length((uchar *)to, length);
 
@@ -1221,7 +1158,7 @@ bool Transaction_state_tracker::store(THD *thd, String &buf) {
   Mark the tracker as changed.
 */
 
-void Transaction_state_tracker::mark_as_changed(THD *, LEX_CSTRING) {
+void Transaction_state_tracker::mark_as_changed(THD *, LEX_CSTRING *) {
   m_changed = true;
 }
 
@@ -1354,7 +1291,7 @@ void Transaction_state_tracker::set_read_flags(THD *thd,
   if (tx_read_flags != flags) {
     tx_read_flags = flags;
     tx_changed |= TX_CHG_CHISTICS;
-    mark_as_changed(thd, {});
+    mark_as_changed(thd, nullptr);
   }
 }
 
@@ -1371,7 +1308,7 @@ void Transaction_state_tracker::set_isol_level(THD *thd,
   if (tx_isol_level != level) {
     tx_isol_level = level;
     tx_changed |= TX_CHG_CHISTICS;
-    mark_as_changed(thd, {});
+    mark_as_changed(thd, nullptr);
   }
 }
 
@@ -1422,14 +1359,15 @@ bool Session_state_change_tracker::update(THD *thd) { return enable(thd); }
 bool Session_state_change_tracker::store(THD *, String &buf) {
   /* since its a boolean tracker length is always 1 */
   const ulonglong length = 1;
-  /*
-    format of the payload is as follows:
-    [ tracker type][length] [1 byte flag]
-  */
+
   uchar *to = (uchar *)buf.prep_append(3, EXTRA_ALLOC);
 
+  /* format of the payload is as follows:
+     [ tracker type] [length] [1 byte flag] */
+
   /* Session state type (SESSION_TRACK_STATE_CHANGE) */
   to = net_store_length(to, (ulonglong)SESSION_TRACK_STATE_CHANGE);
+
   /* Length of the overall entity it is always 1 byte */
   to = net_store_length(to, length);
 
@@ -1447,12 +1385,11 @@ bool Session_state_change_tracker::store(THD *, String &buf) {
 */
 
 void Session_state_change_tracker::mark_as_changed(
-    THD *thd, LEX_CSTRING tracked_item_name) {
+    THD *thd, LEX_CSTRING *tracked_item_name) {
   /* do not send the boolean flag for the tracker itself
      in the OK packet */
-  if (tracked_item_name.str &&
-      (strncmp(tracked_item_name.str,
-               STRING_WITH_LEN("session_track_state_change")) == 0))
+  if (tracked_item_name &&
+      (strncmp(tracked_item_name->str, "session_track_state_change", 26) == 0))
     m_changed = false;
   else {
     m_changed = true;
@@ -1478,20 +1415,6 @@ bool Session_state_change_tracker::is_state_changed() { return m_changed; }
 
 ///////////////////////////////////////////////////////////////////////////////
 
-class Session_transaction_state : public State_tracker {
- public:
-  /** Constructor */
-  Session_transaction_state() {}
-
-  bool enable(THD *) override { return false; }
-  bool check(THD *, set_var *) override { return false; }
-  bool update(THD *) override { return false; }
-  bool store(THD *, String &) override { return false; }
-  void mark_as_changed(THD *, LEX_CSTRING) override {}
-};
-
-///////////////////////////////////////////////////////////////////////////////
-
 /**
   @brief Initialize session tracker objects.
 
@@ -1508,13 +1431,11 @@ void Session_tracker::init(const CHARSET_INFO *char_set) {
   m_trackers[SESSION_GTIDS_TRACKER] = new (std::nothrow) Session_gtids_tracker;
   m_trackers[TRANSACTION_INFO_TRACKER] =
       new (std::nothrow) Transaction_state_tracker;
-  m_trackers[TRACK_TRANSACTION_STATE] =
-      new (std::nothrow) Session_transaction_state;
 }
 
-void Session_tracker::claim_memory_ownership(bool claim) {
+void Session_tracker::claim_memory_ownership() {
   for (int i = 0; i <= SESSION_TRACKER_END; i++)
-    m_trackers[i]->claim_memory_ownership(claim);
+    m_trackers[i]->claim_memory_ownership();
 }
 
 /**
@@ -1654,11 +1575,9 @@ bool Session_gtids_tracker::update(THD *thd) {
     We are updating this using the previous value. No change needed.
     Bailing out.
   */
-  if (m_enabled ==
-      (thd->variables.session_track_gtids != SESSION_TRACK_GTIDS_OFF))
-    return false;
+  if (m_enabled == (thd->variables.session_track_gtids != OFF)) return false;
 
-  m_enabled = thd->variables.session_track_gtids != SESSION_TRACK_GTIDS_OFF &&
+  m_enabled = thd->variables.session_track_gtids != OFF &&
               /* No need to track GTIDs for system threads. */
               thd->system_thread == NON_SYSTEM_THREAD;
   if (m_enabled) {
@@ -1690,7 +1609,7 @@ bool Session_gtids_tracker::update(THD *thd) {
          state-change (see reset()).
 
 
-  @param thd                The thd handle.
+  @param thd           The thd handle.
   @param [in,out] buf       Buffer to store the information to.
 
   @return
@@ -1711,9 +1630,9 @@ bool Session_gtids_tracker::store(THD *thd, String &buf) {
   @param tracked_item_name          Always null.
 */
 
-void Session_gtids_tracker::mark_as_changed(THD *thd [[maybe_unused]],
-                                            LEX_CSTRING tracked_item_name
-                                            [[maybe_unused]]) {
+void Session_gtids_tracker::mark_as_changed(THD *thd MY_ATTRIBUTE((unused)),
+                                            LEX_CSTRING *tracked_item_name
+                                                MY_ATTRIBUTE((unused))) {
   m_changed = true;
 }
 
@@ -1737,5 +1656,3 @@ void Session_gtids_tracker::reset() {
   }
   m_changed = false;
 }
-
-///////////////////////////////////////////////////////////////////////////////
