@@ -1,7 +1,7 @@
 #ifndef SESSION_TRACKER_INCLUDED
 #define SESSION_TRACKER_INCLUDED
 
-/* Copyright (c) 2014, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -39,20 +39,11 @@ enum enum_session_tracker {
   SESSION_SYSVARS_TRACKER, /* Session system variables */
   CURRENT_SCHEMA_TRACKER,  /* Current schema */
   SESSION_STATE_CHANGE_TRACKER,
-  SESSION_GTIDS_TRACKER,    /* Tracks GTIDs */
-  TRANSACTION_INFO_TRACKER, /* Transaction state */
-  /*
-    There should be a one-to-one mapping between this enum members and the
-    members from enum enum_session_state_type defined in mysql_com.h.
-    TRANSACTION_INFO_TRACKER maps to 2 types of tracker which are:
-    SESSION_TRACK_TRANSACTION_CHARACTERISTICS, SESSION_TRACK_TRANSACTION_STATE.
-    Thus introduced a dummy tracker type on server side to keep trackers in sync
-    between client and server.
-  */
-  TRACK_TRANSACTION_STATE,
+  SESSION_GTIDS_TRACKER,   /* Tracks GTIDs */
+  TRANSACTION_INFO_TRACKER /* Transaction state */
 };
 
-#define SESSION_TRACKER_END TRACK_TRANSACTION_STATE
+#define SESSION_TRACKER_END TRANSACTION_INFO_TRACKER
 
 #define TX_TRACKER_GET(a)                                            \
   Transaction_state_tracker *a =                                     \
@@ -92,7 +83,7 @@ class State_tracker {
   State_tracker() : m_enabled(false), m_changed(false) {}
 
   /** Destructor */
-  virtual ~State_tracker() = default;
+  virtual ~State_tracker() {}
 
   /** Getters */
   bool is_enabled() const { return m_enabled; }
@@ -112,9 +103,9 @@ class State_tracker {
   virtual bool store(THD *thd, String &buf) = 0;
 
   /** Mark the entity as changed. */
-  virtual void mark_as_changed(THD *thd, LEX_CSTRING name) = 0;
+  virtual void mark_as_changed(THD *thd, LEX_CSTRING *name) = 0;
 
-  virtual void claim_memory_ownership(bool claim [[maybe_unused]]) {}
+  virtual void claim_memory_ownership() {}
 };
 
 /**
@@ -135,10 +126,10 @@ class Session_tracker {
   Session_tracker &operator=(Session_tracker const &) = delete;
 
   /** Constructor */
-  Session_tracker() = default;
+  Session_tracker() {}
 
   /** Destructor */
-  ~Session_tracker() = default;
+  ~Session_tracker() {}
   /**
     Initialize Session_tracker objects and enable them based on the
     tracker_xxx variables' value that the session inherit from global
@@ -166,7 +157,7 @@ class Session_tracker {
     for (int i = 0; i <= SESSION_TRACKER_END; i++) delete m_trackers[i];
   }
 
-  void claim_memory_ownership(bool claim);
+  void claim_memory_ownership();
 };
 
 /*
@@ -188,11 +179,11 @@ class Session_state_change_tracker : public State_tracker {
 
  public:
   Session_state_change_tracker();
-  bool enable(THD *thd) override;
-  bool check(THD *, set_var *) override { return false; }
-  bool update(THD *thd) override;
-  bool store(THD *, String &buf) override;
-  void mark_as_changed(THD *thd, LEX_CSTRING tracked_item_name) override;
+  bool enable(THD *thd);
+  bool check(THD *, set_var *) { return false; }
+  bool update(THD *thd);
+  bool store(THD *, String &buf);
+  void mark_as_changed(THD *thd, LEX_CSTRING *tracked_item_name);
   bool is_state_changed();
 };
 
@@ -218,8 +209,7 @@ enum enum_tx_state {
   TX_RESULT_SET = 128,     ///< result-set was sent
   TX_WITH_SNAPSHOT = 256,  ///< WITH CONSISTENT SNAPSHOT was used
   TX_LOCKED_TABLES = 512,  ///< LOCK TABLES is active
-  TX_STMT_DML = 1024,      ///< a DML statement (known before data is accessed)
-  TX_STMT_DDL = 2048       ///< a DDL statement
+  TX_STMT_DML = 1024       ///< a DML statement (known before data is accessed)
 };
 
 /**
@@ -255,11 +245,11 @@ class Transaction_state_tracker : public State_tracker {
  public:
   /** Constructor */
   Transaction_state_tracker();
-  bool enable(THD *thd) override { return update(thd); }
-  bool check(THD *, set_var *) override { return false; }
-  bool update(THD *thd) override;
-  bool store(THD *thd, String &buf) override;
-  void mark_as_changed(THD *thd, LEX_CSTRING tracked_item_name) override;
+  bool enable(THD *thd) { return update(thd); }
+  bool check(THD *, set_var *) { return false; }
+  bool update(THD *thd);
+  bool store(THD *thd, String &buf);
+  void mark_as_changed(THD *thd, LEX_CSTRING *tracked_item_name);
 
   /** Change transaction characteristics */
   void set_read_flags(THD *thd, enum enum_tx_read_flags flags);
@@ -300,12 +290,12 @@ class Transaction_state_tracker : public State_tracker {
 
   inline void update_change_flags(THD *thd) {
     tx_changed &= ~TX_CHG_STATE;
-    // Flag state changes other than "is DDL/DML"
-    tx_changed |= ((tx_curr_state & ~(TX_STMT_DML | TX_STMT_DDL)) !=
-                   (tx_reported_state & ~(TX_STMT_DML | TX_STMT_DDL)))
-                      ? TX_CHG_STATE
-                      : 0;
-    if (tx_changed != TX_CHG_NONE) mark_as_changed(thd, {});
+    // Flag state changes other than "is DML"
+    tx_changed |=
+        ((tx_curr_state & ~TX_STMT_DML) != (tx_reported_state & ~TX_STMT_DML))
+            ? TX_CHG_STATE
+            : 0;
+    if (tx_changed != TX_CHG_NONE) mark_as_changed(thd, nullptr);
   }
 };
 
