@@ -1,4 +1,4 @@
-/* Copyright (c) 2002, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2002, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -36,7 +36,7 @@
 #include "my_psi_config.h"
 #include "my_sqlcommand.h"
 #include "my_sys.h"
-#include "mysql/components/services/bits/psi_statement_bits.h"
+#include "mysql/components/services/psi_statement_bits.h"
 #include "mysqld_error.h"
 #include "sql/auth/sql_security_ctx.h"
 #include "sql/create_field.h"
@@ -405,13 +405,7 @@ class sp_head {
       b) because in CONTAINS SQL case they don't provide enough
       information anyway.
      */
-    MODIFIES_DATA = 4096,
-    /**
-      Set when a stored program contains sub-statement(s) that creates or drops
-      temporary table(s). When set, used to mark invoking statement as unsafe to
-      be binlogged in STATEMENT format, when in MIXED mode.
-    */
-    HAS_TEMP_TABLE_DDL = 8192
+    MODIFIES_DATA = 4096
   };
 
  public:
@@ -472,8 +466,8 @@ class sp_head {
   ulong m_recursion_level;
 
   /**
-    A list of different recursion level instances for the same procedure.
-    For every recursion level we have an sp_head instance. This instances
+    A list of diferent recursion level instances for the same procedure.
+    For every recursion level we have a sp_head instance. This instances
     connected in the list. The list ordered by increasing recursion level
     (m_recursion_level).
   */
@@ -651,7 +645,7 @@ class sp_head {
     @return Error status.
   */
 
-  bool execute_procedure(THD *thd, mem_root_deque<Item *> *args);
+  bool execute_procedure(THD *thd, List<Item> *args);
 
   /**
     Add instruction to SP.
@@ -671,14 +665,6 @@ class sp_head {
   */
   bool modifies_data() const { return m_flags & MODIFIES_DATA; }
 
-  /**
-    @returns true if stored program has sub-statement(s) to CREATE/DROP
-    temporary table(s).
-      @retval true   if HAS_TEMP_TABLE_DDL is set in m_flags.
-      @retval false  Otherwise.
-  */
-  bool has_temp_table_ddl() const { return m_flags & HAS_TEMP_TABLE_DDL; }
-
   uint instructions() { return static_cast<uint>(m_instructions.size()); }
 
   sp_instr *last_instruction() { return m_instructions.back(); }
@@ -759,7 +745,7 @@ class sp_head {
     Get SP-instruction at given index.
 
     NOTE: it is important to have *unsigned* int here, sometimes we get (-1)
-    passed here, so it gets converted to MAX_INT, and the result of the
+    passed here, so it get's converted to MAX_INT, and the result of the
     function call is NULL.
   */
   sp_instr *get_instr(uint i) {
@@ -785,9 +771,9 @@ class sp_head {
     routine, NULL if none.
   */
   void add_used_tables_to_table_list(THD *thd,
-                                     Table_ref ***query_tables_last_ptr,
+                                     TABLE_LIST ***query_tables_last_ptr,
                                      enum_sql_command sql_command,
-                                     Table_ref *belong_to_view);
+                                     TABLE_LIST *belong_to_view);
 
   /**
     Check if this stored routine contains statements disallowed
@@ -813,7 +799,7 @@ class sp_head {
              HAS_COMMIT_OR_ROLLBACK | HAS_SQLCOM_RESET | HAS_SQLCOM_FLUSH));
   }
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   /**
     Return the routine instructions as a result set.
     @return Error status.
@@ -838,12 +824,6 @@ class sp_head {
     DBUG_PRINT("info", ("sp_head(0x%p=%s)->unsafe_flags: 0x%x", this, name(),
                         unsafe_flags));
     prelocking_ctx->set_stmt_unsafe_flags(unsafe_flags);
-
-    /*
-      If temporary table is created or dropped in the stored program then
-      statement is unsafe to be logged in STATEMENT format, when in MIXED mode.
-    */
-    if (has_temp_table_ddl()) prelocking_ctx->set_stmt_unsafe_with_mixed_mode();
   }
 
   /**
@@ -983,7 +963,7 @@ class sp_head {
 
     @return Error status.
   */
-  bool merge_table_list(THD *thd, Table_ref *table, LEX *lex_for_tmp_check);
+  bool merge_table_list(THD *thd, TABLE_LIST *table, LEX *lex_for_tmp_check);
 
   friend sp_head *sp_start_parsing(THD *thd, enum_sp_type sp_type,
                                    sp_name *sp_name);
