@@ -1,16 +1,15 @@
-/* Copyright (c) 2012, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2012, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,27 +23,25 @@
 #ifndef _SP_INSTR_H_
 #define _SP_INSTR_H_
 
-#include <assert.h>
 #include <limits.h>
 #include <string.h>
 #include <sys/types.h>
 
 #include "field_types.h"
 #include "lex_string.h"
+#include "m_string.h"
 #include "my_alloc.h"
 #include "my_compiler.h"
-
+#include "my_dbug.h"
 #include "my_inttypes.h"
 #include "my_psi_config.h"
 #include "my_sys.h"
-#include "mysql/components/services/bits/psi_statement_bits.h"
+#include "mysql/components/services/psi_statement_bits.h"
 #include "sql/sql_class.h"  // Query_arena
-#include "sql/sql_const.h"
 #include "sql/sql_error.h"
 #include "sql/sql_lex.h"
 #include "sql/sql_list.h"
 #include "sql_string.h"
-#include "string_with_len.h"
 
 class Item;
 class Item_case_expr;
@@ -54,7 +51,7 @@ class sp_handler;
 class sp_head;
 class sp_pcontext;
 class sp_variable;
-class Table_ref;
+struct TABLE_LIST;
 
 ///////////////////////////////////////////////////////////////////////////
 // This file contains SP-instruction classes.
@@ -68,7 +65,7 @@ class sp_printable {
  public:
   virtual void print(const THD *thd, String *str) = 0;
 
-  virtual ~sp_printable() = default;
+  virtual ~sp_printable() {}
 };
 
 /**
@@ -93,7 +90,7 @@ class sp_branch_instr {
   */
   virtual void backpatch(uint dest) = 0;
 
-  virtual ~sp_branch_instr() = default;
+  virtual ~sp_branch_instr() {}
 };
 
 ///////////////////////////////////////////////////////////////////////////
@@ -110,7 +107,7 @@ class sp_instr : public sp_printable {
         m_ip(ip),
         m_parsing_ctx(ctx) {}
 
-  ~sp_instr() override { m_arena.free_items(); }
+  virtual ~sp_instr() { m_arena.free_items(); }
 
   /**
     Execute this instruction
@@ -160,7 +157,8 @@ class sp_instr : public sp_printable {
     index to the next instruction. Jump instruction will add their
     destination to the leads list.
   */
-  virtual uint opt_mark(sp_head *, List<sp_instr> *leads [[maybe_unused]]) {
+  virtual uint opt_mark(sp_head *,
+                        List<sp_instr> *leads MY_ATTRIBUTE((unused))) {
     m_marked = true;
     return get_ip() + 1;
   }
@@ -171,7 +169,8 @@ class sp_instr : public sp_printable {
     used to prevent the mark sweep from looping for ever. Return the
     end destination.
   */
-  virtual uint opt_shortcut_jump(sp_head *, sp_instr *start [[maybe_unused]]) {
+  virtual uint opt_shortcut_jump(sp_head *,
+                                 sp_instr *start MY_ATTRIBUTE((unused))) {
     return get_ip();
   }
 
@@ -181,7 +180,8 @@ class sp_instr : public sp_printable {
     must also take care of their destination pointers. Forward jumps get
     pushed to the backpatch list 'ibp'.
   */
-  virtual void opt_move(uint dst, List<sp_branch_instr> *ibp [[maybe_unused]]) {
+  virtual void opt_move(uint dst,
+                        List<sp_branch_instr> *ibp MY_ATTRIBUTE((unused))) {
     m_ip = dst;
   }
 
@@ -233,7 +233,7 @@ class sp_lex_instr : public sp_instr {
     set_lex(lex, is_lex_owner);
   }
 
-  ~sp_lex_instr() override {
+  virtual ~sp_lex_instr() {
     free_lex();
     /*
       If the instruction is reparsed, m_lex_mem_root was used to allocate
@@ -264,7 +264,7 @@ class sp_lex_instr : public sp_instr {
   */
   bool validate_lex_and_execute_core(THD *thd, uint *nextp, bool open_tables);
 
-  SQL_I_List<Item_trigger_field> *get_instr_trig_field_list() override {
+  virtual SQL_I_List<Item_trigger_field> *get_instr_trig_field_list() {
     return &m_trig_field_list;
   }
 
@@ -289,8 +289,6 @@ class sp_lex_instr : public sp_instr {
   */
   bool reset_lex_and_exec_core(THD *thd, uint *nextp, bool open_tables);
 
-  bool execute_expression(THD *thd, uint *nextp);
-
   /**
     (Re-)parse the query corresponding to this instruction and return a new
     LEX-object.
@@ -324,7 +322,7 @@ class sp_lex_instr : public sp_instr {
   // sp_instr implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  bool execute(THD *thd, uint *nextp) override {
+  virtual bool execute(THD *thd, uint *nextp) {
     /*
       SP instructions with expressions should clear DA before execution.
       Note that sp_instr_stmt will override execute(), but it clears DA
@@ -373,12 +371,6 @@ class sp_lex_instr : public sp_instr {
   */
   virtual void get_query(String *sql_query) const;
 
-  /**
-    Some expressions may be re-parsed as SELECT statements, but need to be
-    adjusted to another SQL command. This function facilitates that change.
-  */
-  virtual void adjust_sql_command(LEX *) {}
-
   /**
     @return the expression query string. This string can not be passed directly
     to the parser as it is most likely not a valid SQL-statement.
@@ -401,7 +393,7 @@ class sp_lex_instr : public sp_instr {
 
     @return Error flag.
   */
-  virtual bool on_after_expr_parsing(THD *thd [[maybe_unused]]) {
+  virtual bool on_after_expr_parsing(THD *thd MY_ATTRIBUTE((unused))) {
     return false;
   }
 
@@ -422,7 +414,7 @@ class sp_lex_instr : public sp_instr {
     mem-root is freed when a reparse is triggered or the stored
     routine is dropped.
   */
-  MEM_ROOT m_lex_mem_root{PSI_NOT_INSTRUMENTED, MEM_ROOT_BLOCK_SIZE};
+  MEM_ROOT m_lex_mem_root;
 
   /**
     Indicates whether this sp_lex_instr instance is responsible for
@@ -447,13 +439,13 @@ class sp_lex_instr : public sp_instr {
     List of additional tables this statement needs to lock when it
     enters/leaves prelocked mode on its own.
   */
-  Table_ref *m_prelocking_tables;
+  TABLE_LIST *m_prelocking_tables;
 
   /**
     The value m_lex->query_tables_own_last should be set to this when the
     statement enters/leaves prelocked mode on its own.
   */
-  Table_ref **m_lex_query_tables_own_last;
+  TABLE_LIST **m_lex_query_tables_own_last;
 
   /**
     List of all the Item_trigger_field's of instruction.
@@ -481,29 +473,29 @@ class sp_instr_stmt : public sp_lex_instr {
   // sp_instr implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  bool execute(THD *thd, uint *nextp) override;
+  virtual bool execute(THD *thd, uint *nextp);
 
   /////////////////////////////////////////////////////////////////////////
   // sp_printable implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  void print(const THD *thd, String *str) override;
+  virtual void print(const THD *thd, String *str);
 
   /////////////////////////////////////////////////////////////////////////
   // sp_lex_instr implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  bool exec_core(THD *thd, uint *nextp) override;
+  virtual bool exec_core(THD *thd, uint *nextp);
 
-  bool is_invalid() const override { return !m_valid; }
+  virtual bool is_invalid() const { return !m_valid; }
 
-  void invalidate() override { m_valid = false; }
+  virtual void invalidate() { m_valid = false; }
 
-  void get_query(String *sql_query) const override {
+  virtual void get_query(String *sql_query) const {
     sql_query->append(m_query.str, m_query.length);
   }
 
-  bool on_after_expr_parsing(THD *) override {
+  virtual bool on_after_expr_parsing(THD *) {
     m_valid = true;
     return false;
   }
@@ -517,7 +509,7 @@ class sp_instr_stmt : public sp_lex_instr {
 
 #ifdef HAVE_PSI_INTERFACE
  public:
-  PSI_statement_info *get_psi_info() override { return &psi_info; }
+  virtual PSI_statement_info *get_psi_info() { return &psi_info; }
 
   static PSI_statement_info psi_info;
 #endif
@@ -541,31 +533,27 @@ class sp_instr_set : public sp_lex_instr {
   // sp_printable implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  void print(const THD *thd, String *str) override;
+  virtual void print(const THD *thd, String *str);
 
   /////////////////////////////////////////////////////////////////////////
   // sp_lex_instr implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  bool exec_core(THD *thd, uint *nextp) override;
+  virtual bool exec_core(THD *thd, uint *nextp);
 
-  bool is_invalid() const override { return m_value_item == nullptr; }
+  virtual bool is_invalid() const { return m_value_item == nullptr; }
 
-  void invalidate() override { m_value_item = nullptr; }
+  virtual void invalidate() { m_value_item = nullptr; }
 
-  bool on_after_expr_parsing(THD *thd) override {
-    m_value_item = thd->lex->query_block->single_visible_field();
-    assert(m_value_item != nullptr);
+  virtual bool on_after_expr_parsing(THD *thd) {
+    DBUG_ASSERT(thd->lex->select_lex->item_list.elements == 1);
+
+    m_value_item = thd->lex->select_lex->item_list.head();
 
     return false;
   }
 
-  LEX_CSTRING get_expr_query() const override { return m_value_query; }
-
-  void adjust_sql_command(LEX *lex) override {
-    assert(lex->sql_command == SQLCOM_SELECT);
-    lex->sql_command = SQLCOM_SET_OPTION;
-  }
+  virtual LEX_CSTRING get_expr_query() const { return m_value_query; }
 
  private:
   /// Frame offset.
@@ -580,7 +568,7 @@ class sp_instr_set : public sp_lex_instr {
 #ifdef HAVE_PSI_INTERFACE
  public:
   static PSI_statement_info psi_info;
-  PSI_statement_info *get_psi_info() override { return &psi_info; }
+  virtual PSI_statement_info *get_psi_info() { return &psi_info; }
 #endif
 };
 
@@ -605,23 +593,23 @@ class sp_instr_set_trigger_field : public sp_lex_instr {
   // sp_printable implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  void print(const THD *thd, String *str) override;
+  virtual void print(const THD *thd, String *str);
 
   /////////////////////////////////////////////////////////////////////////
   // sp_lex_instr implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  bool exec_core(THD *thd, uint *nextp) override;
+  virtual bool exec_core(THD *thd, uint *nextp);
 
-  bool is_invalid() const override { return m_value_item == nullptr; }
+  virtual bool is_invalid() const { return m_value_item == nullptr; }
 
-  void invalidate() override { m_value_item = nullptr; }
+  virtual void invalidate() { m_value_item = nullptr; }
 
-  bool on_after_expr_parsing(THD *thd) override;
+  virtual bool on_after_expr_parsing(THD *thd);
 
-  void cleanup_before_parsing(THD *thd) override;
+  virtual void cleanup_before_parsing(THD *thd);
 
-  LEX_CSTRING get_expr_query() const override { return m_value_query; }
+  virtual LEX_CSTRING get_expr_query() const { return m_value_query; }
 
  private:
   /// Trigger field name ("field_name" of the "NEW.field_name").
@@ -638,7 +626,7 @@ class sp_instr_set_trigger_field : public sp_lex_instr {
 
 #ifdef HAVE_PSI_INTERFACE
  public:
-  PSI_statement_info *get_psi_info() override { return &psi_info; }
+  virtual PSI_statement_info *get_psi_info() { return &psi_info; }
 
   static PSI_statement_info psi_info;
 #endif
@@ -662,13 +650,13 @@ class sp_instr_freturn : public sp_lex_instr {
   // sp_printable implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  void print(const THD *thd, String *str) override;
+  virtual void print(const THD *thd, String *str);
 
   /////////////////////////////////////////////////////////////////////////
   // sp_instr implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  uint opt_mark(sp_head *, List<sp_instr> *) override {
+  virtual uint opt_mark(sp_head *, List<sp_instr> *) {
     m_marked = true;
     return UINT_MAX;
   }
@@ -677,22 +665,24 @@ class sp_instr_freturn : public sp_lex_instr {
   // sp_lex_instr implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  bool exec_core(THD *thd, uint *nextp) override;
+  virtual bool exec_core(THD *thd, uint *nextp);
 
-  bool is_invalid() const override { return m_expr_item == nullptr; }
+  virtual bool is_invalid() const { return m_expr_item == nullptr; }
 
-  void invalidate() override {
+  virtual void invalidate() {
     // it's already deleted.
     m_expr_item = nullptr;
   }
 
-  bool on_after_expr_parsing(THD *thd) override {
-    m_expr_item = thd->lex->query_block->single_visible_field();
-    assert(m_expr_item != nullptr);
+  virtual bool on_after_expr_parsing(THD *thd) {
+    DBUG_ASSERT(thd->lex->select_lex->item_list.elements == 1);
+
+    m_expr_item = thd->lex->select_lex->item_list.head();
+
     return false;
   }
 
-  LEX_CSTRING get_expr_query() const override { return m_expr_query; }
+  virtual LEX_CSTRING get_expr_query() const { return m_expr_query; }
 
  private:
   /// RETURN-expression item.
@@ -706,7 +696,7 @@ class sp_instr_freturn : public sp_lex_instr {
 
 #ifdef HAVE_PSI_INTERFACE
  public:
-  PSI_statement_info *get_psi_info() override { return &psi_info; }
+  virtual PSI_statement_info *get_psi_info() { return &psi_info; }
 
   static PSI_statement_info psi_info;
 #endif
@@ -736,34 +726,34 @@ class sp_instr_jump : public sp_instr, public sp_branch_instr {
   // sp_printable implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  void print(const THD *thd, String *str) override;
+  virtual void print(const THD *thd, String *str);
 
   /////////////////////////////////////////////////////////////////////////
   // sp_instr implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  bool execute(THD *, uint *nextp) override {
+  virtual bool execute(THD *, uint *nextp) {
     *nextp = m_dest;
     return false;
   }
 
-  uint opt_mark(sp_head *sp, List<sp_instr> *leads) override;
+  virtual uint opt_mark(sp_head *sp, List<sp_instr> *leads);
 
-  uint opt_shortcut_jump(sp_head *sp, sp_instr *start) override;
+  virtual uint opt_shortcut_jump(sp_head *sp, sp_instr *start);
 
-  void opt_move(uint dst, List<sp_branch_instr> *ibp) override;
+  virtual void opt_move(uint dst, List<sp_branch_instr> *ibp);
 
   /////////////////////////////////////////////////////////////////////////
   // sp_branch_instr implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  void set_destination(uint old_dest, uint new_dest) override {
+  virtual void set_destination(uint old_dest, uint new_dest) {
     if (m_dest == old_dest) m_dest = new_dest;
   }
 
-  void backpatch(uint dest) override {
+  virtual void backpatch(uint dest) {
     /* Calling backpatch twice is a logic flaw in jump resolution. */
-    assert(m_dest == 0);
+    DBUG_ASSERT(m_dest == 0);
     m_dest = dest;
   }
 
@@ -776,7 +766,7 @@ class sp_instr_jump : public sp_instr, public sp_branch_instr {
 
 #ifdef HAVE_PSI_INTERFACE
  public:
-  PSI_statement_info *get_psi_info() override { return &psi_info; }
+  virtual PSI_statement_info *get_psi_info() { return &psi_info; }
 
   static PSI_statement_info psi_info;
 #endif
@@ -817,45 +807,40 @@ class sp_lex_branch_instr : public sp_lex_instr, public sp_branch_instr {
   // sp_instr implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  uint opt_mark(sp_head *sp, List<sp_instr> *leads) override;
+  virtual uint opt_mark(sp_head *sp, List<sp_instr> *leads);
 
-  void opt_move(uint dst, List<sp_branch_instr> *ibp) override;
+  virtual void opt_move(uint dst, List<sp_branch_instr> *ibp);
 
-  uint get_cont_dest() const override { return m_cont_dest; }
+  virtual uint get_cont_dest() const { return m_cont_dest; }
 
   /////////////////////////////////////////////////////////////////////////
   // sp_lex_instr implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  bool is_invalid() const override { return m_expr_item == nullptr; }
+  virtual bool is_invalid() const { return m_expr_item == nullptr; }
 
-  void invalidate() override {
+  virtual void invalidate() {
     m_expr_item = nullptr; /* it's already deleted. */
   }
 
-  LEX_CSTRING get_expr_query() const override { return m_expr_query; }
+  virtual LEX_CSTRING get_expr_query() const { return m_expr_query; }
 
   /////////////////////////////////////////////////////////////////////////
   // sp_branch_instr implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  void set_destination(uint old_dest, uint new_dest) override {
+  virtual void set_destination(uint old_dest, uint new_dest) {
     if (m_dest == old_dest) m_dest = new_dest;
 
     if (m_cont_dest == old_dest) m_cont_dest = new_dest;
   }
 
-  void backpatch(uint dest) override {
+  virtual void backpatch(uint dest) {
     /* Calling backpatch twice is a logic flaw in jump resolution. */
-    assert(m_dest == 0);
+    DBUG_ASSERT(m_dest == 0);
     m_dest = dest;
   }
 
-  void adjust_sql_command(LEX *lex) override {
-    assert(lex->sql_command == SQLCOM_SELECT);
-    lex->sql_command = SQLCOM_END;
-  }
-
  protected:
   /// Where we will go.
   uint m_dest;
@@ -896,23 +881,25 @@ class sp_instr_jump_if_not : public sp_lex_branch_instr {
   // sp_printable implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  void print(const THD *thd, String *str) override;
+  virtual void print(const THD *thd, String *str);
 
   /////////////////////////////////////////////////////////////////////////
   // sp_lex_instr implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  bool exec_core(THD *thd, uint *nextp) override;
+  virtual bool exec_core(THD *thd, uint *nextp);
+
+  virtual bool on_after_expr_parsing(THD *thd) {
+    DBUG_ASSERT(thd->lex->select_lex->item_list.elements == 1);
+
+    m_expr_item = thd->lex->select_lex->item_list.head();
 
-  bool on_after_expr_parsing(THD *thd) override {
-    m_expr_item = thd->lex->query_block->single_visible_field();
-    assert(m_expr_item != nullptr);
     return false;
   }
 
 #ifdef HAVE_PSI_INTERFACE
  public:
-  PSI_statement_info *get_psi_info() override { return &psi_info; }
+  virtual PSI_statement_info *get_psi_info() { return &psi_info; }
 
   static PSI_statement_info psi_info;
 #endif
@@ -938,22 +925,22 @@ class sp_instr_set_case_expr : public sp_lex_branch_instr {
   // sp_printable implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  void print(const THD *thd, String *str) override;
+  virtual void print(const THD *thd, String *str);
 
   /////////////////////////////////////////////////////////////////////////
   // sp_instr implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  uint opt_mark(sp_head *sp, List<sp_instr> *leads) override;
+  virtual uint opt_mark(sp_head *sp, List<sp_instr> *leads);
 
-  void opt_move(uint dst, List<sp_branch_instr> *ibp) override;
+  virtual void opt_move(uint dst, List<sp_branch_instr> *ibp);
 
   /////////////////////////////////////////////////////////////////////////
   // sp_branch_instr implementation.
   /////////////////////////////////////////////////////////////////////////
 
   /*
-    NOTE: set_destination() and backpatch() are overridden here just because the
+    NOTE: set_destination() and backpatch() are overriden here just because the
     m_dest attribute is not used by this class, so there is no need to do
     anything about it.
 
@@ -965,21 +952,23 @@ class sp_instr_set_case_expr : public sp_lex_branch_instr {
     hierarchy.
   */
 
-  void set_destination(uint old_dest, uint new_dest) override {
+  virtual void set_destination(uint old_dest, uint new_dest) {
     if (m_cont_dest == old_dest) m_cont_dest = new_dest;
   }
 
-  void backpatch(uint) override {}
+  virtual void backpatch(uint) {}
 
   /////////////////////////////////////////////////////////////////////////
   // sp_lex_instr implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  bool exec_core(THD *thd, uint *nextp) override;
+  virtual bool exec_core(THD *thd, uint *nextp);
+
+  virtual bool on_after_expr_parsing(THD *thd) {
+    DBUG_ASSERT(thd->lex->select_lex->item_list.elements == 1);
+
+    m_expr_item = thd->lex->select_lex->item_list.head();
 
-  bool on_after_expr_parsing(THD *thd) override {
-    m_expr_item = thd->lex->query_block->single_visible_field();
-    assert(m_expr_item != nullptr);
     return false;
   }
 
@@ -989,7 +978,7 @@ class sp_instr_set_case_expr : public sp_lex_branch_instr {
 
 #ifdef HAVE_PSI_INTERFACE
  public:
-  PSI_statement_info *get_psi_info() override { return &psi_info; }
+  virtual PSI_statement_info *get_psi_info() { return &psi_info; }
 
   static PSI_statement_info psi_info;
 #endif
@@ -1016,15 +1005,15 @@ class sp_instr_jump_case_when : public sp_lex_branch_instr {
   // sp_printable implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  void print(const THD *thd, String *str) override;
+  virtual void print(const THD *thd, String *str);
 
   /////////////////////////////////////////////////////////////////////////
   // sp_lex_instr implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  bool exec_core(THD *thd, uint *nextp) override;
+  virtual bool exec_core(THD *thd, uint *nextp);
 
-  void invalidate() override {
+  virtual void invalidate() {
     // Items should be already deleted in lex-keeper.
     m_case_expr_item = nullptr;
     m_eq_item = nullptr;
@@ -1051,7 +1040,7 @@ class sp_instr_jump_case_when : public sp_lex_branch_instr {
 
     @return Error flag.
   */
-  bool on_after_expr_parsing(THD *thd) override;
+  virtual bool on_after_expr_parsing(THD *thd);
 
  private:
   /// Identifier (index) of the CASE-expression in the runtime context.
@@ -1069,7 +1058,7 @@ class sp_instr_jump_case_when : public sp_lex_branch_instr {
 
 #ifdef HAVE_PSI_INTERFACE
  public:
-  PSI_statement_info *get_psi_info() override { return &psi_info; }
+  virtual PSI_statement_info *get_psi_info() { return &psi_info; }
 
   static PSI_statement_info psi_info;
 #endif
@@ -1083,7 +1072,7 @@ class sp_instr_hpush_jump : public sp_instr_jump {
  public:
   sp_instr_hpush_jump(uint ip, sp_pcontext *ctx, sp_handler *handler);
 
-  ~sp_instr_hpush_jump() override;
+  virtual ~sp_instr_hpush_jump();
 
   void add_condition(sp_condition_value *condition_value);
 
@@ -1093,25 +1082,25 @@ class sp_instr_hpush_jump : public sp_instr_jump {
   // sp_printable implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  void print(const THD *thd, String *str) override;
+  virtual void print(const THD *thd, String *str);
 
   /////////////////////////////////////////////////////////////////////////
   // sp_instr implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  bool execute(THD *thd, uint *nextp) override;
+  virtual bool execute(THD *thd, uint *nextp);
 
-  uint opt_mark(sp_head *sp, List<sp_instr> *leads) override;
+  virtual uint opt_mark(sp_head *sp, List<sp_instr> *leads);
 
   /** Override sp_instr_jump's shortcut; we stop here. */
-  uint opt_shortcut_jump(sp_head *, sp_instr *) override { return get_ip(); }
+  virtual uint opt_shortcut_jump(sp_head *, sp_instr *) { return get_ip(); }
 
   /////////////////////////////////////////////////////////////////////////
   // sp_branch_instr implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  void backpatch(uint dest) override {
-    assert(!m_dest || !m_opt_hpop);
+  virtual void backpatch(uint dest) {
+    DBUG_ASSERT(!m_dest || !m_opt_hpop);
     if (!m_dest)
       m_dest = dest;
     else
@@ -1131,7 +1120,7 @@ class sp_instr_hpush_jump : public sp_instr_jump {
 
 #ifdef HAVE_PSI_INTERFACE
  public:
-  PSI_statement_info *get_psi_info() override { return &psi_info; }
+  virtual PSI_statement_info *get_psi_info() { return &psi_info; }
 
   static PSI_statement_info psi_info;
 #endif
@@ -1147,7 +1136,7 @@ class sp_instr_hpop : public sp_instr {
   // sp_printable implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  void print(const THD *, String *str) override {
+  virtual void print(const THD *, String *str) {
     str->append(STRING_WITH_LEN("hpop"));
   }
 
@@ -1155,11 +1144,11 @@ class sp_instr_hpop : public sp_instr {
   // sp_instr implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  bool execute(THD *thd, uint *nextp) override;
+  virtual bool execute(THD *thd, uint *nextp);
 
 #ifdef HAVE_PSI_INTERFACE
  public:
-  PSI_statement_info *get_psi_info() override { return &psi_info; }
+  virtual PSI_statement_info *get_psi_info() { return &psi_info; }
 
   static PSI_statement_info psi_info;
 #endif
@@ -1175,18 +1164,18 @@ class sp_instr_hreturn : public sp_instr_jump {
   // sp_printable implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  void print(const THD *thd, String *str) override;
+  virtual void print(const THD *thd, String *str);
 
   /////////////////////////////////////////////////////////////////////////
   // sp_instr implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  bool execute(THD *thd, uint *nextp) override;
+  virtual bool execute(THD *thd, uint *nextp);
 
   /** Override sp_instr_jump's shortcut; we stop here. */
-  uint opt_shortcut_jump(sp_head *, sp_instr *) override { return get_ip(); }
+  virtual uint opt_shortcut_jump(sp_head *, sp_instr *) { return get_ip(); }
 
-  uint opt_mark(sp_head *sp, List<sp_instr> *leads) override;
+  virtual uint opt_mark(sp_head *sp, List<sp_instr> *leads);
 
  private:
   // This attribute is needed for SHOW PROCEDURE CODE only (i.e. it's needed in
@@ -1195,7 +1184,7 @@ class sp_instr_hreturn : public sp_instr_jump {
 
 #ifdef HAVE_PSI_INTERFACE
  public:
-  PSI_statement_info *get_psi_info() override { return &psi_info; }
+  virtual PSI_statement_info *get_psi_info() { return &psi_info; }
 
   static PSI_statement_info psi_info;
 #endif
@@ -1241,29 +1230,29 @@ class sp_instr_cpush : public sp_lex_instr {
   // sp_printable implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  void print(const THD *thd, String *str) override;
+  virtual void print(const THD *thd, String *str);
 
   /////////////////////////////////////////////////////////////////////////
   // sp_instr implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  bool execute(THD *thd, uint *nextp) override;
+  virtual bool execute(THD *thd, uint *nextp);
 
   /////////////////////////////////////////////////////////////////////////
   // sp_lex_instr implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  bool exec_core(THD *thd, uint *nextp) override;
+  virtual bool exec_core(THD *thd, uint *nextp);
 
-  bool is_invalid() const override { return !m_valid; }
+  virtual bool is_invalid() const { return !m_valid; }
 
-  void invalidate() override { m_valid = false; }
+  virtual void invalidate() { m_valid = false; }
 
-  void get_query(String *sql_query) const override {
+  virtual void get_query(String *sql_query) const {
     sql_query->append(m_cursor_query.str, m_cursor_query.length);
   }
 
-  bool on_after_expr_parsing(THD *) override {
+  virtual bool on_after_expr_parsing(THD *) {
     m_valid = true;
     return false;
   }
@@ -1281,7 +1270,7 @@ class sp_instr_cpush : public sp_lex_instr {
 
 #ifdef HAVE_PSI_INTERFACE
  public:
-  PSI_statement_info *get_psi_info() override { return &psi_info; }
+  virtual PSI_statement_info *get_psi_info() { return &psi_info; }
 
   static PSI_statement_info psi_info;
 #endif
@@ -1302,20 +1291,20 @@ class sp_instr_cpop : public sp_instr {
   // sp_printable implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  void print(const THD *thd, String *str) override;
+  virtual void print(const THD *thd, String *str);
 
   /////////////////////////////////////////////////////////////////////////
   // sp_instr implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  bool execute(THD *thd, uint *nextp) override;
+  virtual bool execute(THD *thd, uint *nextp);
 
  private:
   uint m_count;
 
 #ifdef HAVE_PSI_INTERFACE
  public:
-  PSI_statement_info *get_psi_info() override { return &psi_info; }
+  virtual PSI_statement_info *get_psi_info() { return &psi_info; }
 
   static PSI_statement_info psi_info;
 #endif
@@ -1336,13 +1325,13 @@ class sp_instr_copen : public sp_instr {
   // sp_printable implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  void print(const THD *thd, String *str) override;
+  virtual void print(const THD *thd, String *str);
 
   /////////////////////////////////////////////////////////////////////////
   // sp_instr implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  bool execute(THD *thd, uint *nextp) override;
+  virtual bool execute(THD *thd, uint *nextp);
 
  private:
   /// Used to identify the cursor in the sp_rcontext.
@@ -1350,7 +1339,7 @@ class sp_instr_copen : public sp_instr {
 
 #ifdef HAVE_PSI_INTERFACE
  public:
-  PSI_statement_info *get_psi_info() override { return &psi_info; }
+  virtual PSI_statement_info *get_psi_info() { return &psi_info; }
 
   static PSI_statement_info psi_info;
 #endif
@@ -1372,13 +1361,13 @@ class sp_instr_cclose : public sp_instr {
   // sp_printable implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  void print(const THD *thd, String *str) override;
+  virtual void print(const THD *thd, String *str);
 
   /////////////////////////////////////////////////////////////////////////
   // sp_instr implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  bool execute(THD *thd, uint *nextp) override;
+  virtual bool execute(THD *thd, uint *nextp);
 
  private:
   /// Used to identify the cursor in the sp_rcontext.
@@ -1386,7 +1375,7 @@ class sp_instr_cclose : public sp_instr {
 
 #ifdef HAVE_PSI_INTERFACE
  public:
-  PSI_statement_info *get_psi_info() override { return &psi_info; }
+  virtual PSI_statement_info *get_psi_info() { return &psi_info; }
 
   static PSI_statement_info psi_info;
 #endif
@@ -1408,13 +1397,13 @@ class sp_instr_cfetch : public sp_instr {
   // sp_printable implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  void print(const THD *thd, String *str) override;
+  virtual void print(const THD *thd, String *str);
 
   /////////////////////////////////////////////////////////////////////////
   // sp_instr implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  bool execute(THD *thd, uint *nextp) override;
+  virtual bool execute(THD *thd, uint *nextp);
 
   void add_to_varlist(sp_variable *var) { m_varlist.push_back(var); }
 
@@ -1427,7 +1416,7 @@ class sp_instr_cfetch : public sp_instr {
 
 #ifdef HAVE_PSI_INTERFACE
  public:
-  PSI_statement_info *get_psi_info() override { return &psi_info; }
+  virtual PSI_statement_info *get_psi_info() { return &psi_info; }
 
   static PSI_statement_info psi_info;
 #endif
@@ -1450,19 +1439,19 @@ class sp_instr_error : public sp_instr {
   // sp_printable implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  void print(const THD *thd, String *str) override;
+  virtual void print(const THD *thd, String *str);
 
   /////////////////////////////////////////////////////////////////////////
   // sp_instr implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  bool execute(THD *, uint *nextp) override {
+  virtual bool execute(THD *, uint *nextp) {
     my_error(m_errcode, MYF(0));
     *nextp = get_ip() + 1;
     return true;
   }
 
-  uint opt_mark(sp_head *, List<sp_instr> *) override {
+  virtual uint opt_mark(sp_head *, List<sp_instr> *) {
     m_marked = true;
     return UINT_MAX;
   }
@@ -1473,7 +1462,7 @@ class sp_instr_error : public sp_instr {
 
 #ifdef HAVE_PSI_INTERFACE
  public:
-  PSI_statement_info *get_psi_info() override { return &psi_info; }
+  virtual PSI_statement_info *get_psi_info() { return &psi_info; }
 
   static PSI_statement_info psi_info;
 #endif
