@@ -1,4 +1,4 @@
-/* Copyright (c) 2010, 2022, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -26,15 +26,13 @@
 #include <assert.h>
 #include <stddef.h>
 #include <sys/types.h>
-#include <functional>  // std::function
-#include <optional>
 
 #include "lex_string.h"
-
-#include "field_types.h"
+#include "my_dbug.h"
 #include "my_io.h"
 #include "my_sqlcommand.h"
-#include "mysql/components/services/bits/psi_bits.h"
+#include "mysql/psi/psi_base.h"
+#include "nullable.h"
 #include "sql/dd/types/column.h"
 #include "sql/gis/srid.h"
 #include "sql/mdl.h"                   // MDL_request
@@ -45,7 +43,6 @@
 #include "sql/sql_list.h"              // List
 #include "sql/thr_malloc.h"
 
-class Alter_info;
 class Create_field;
 class FOREIGN_KEY;
 class Value_generator;
@@ -55,6 +52,10 @@ class String;
 class THD;
 struct TABLE_LIST;
 
+enum enum_field_types : int;
+
+using Mysql::Nullable;
+
 /**
   Class representing DROP COLUMN, DROP KEY, DROP FOREIGN KEY, DROP CHECK
   CONSTRAINT and DROP CONSTRAINT clauses in ALTER TABLE statement.
@@ -68,13 +69,13 @@ class Alter_drop {
 
   Alter_drop(drop_type par_type, const char *par_name)
       : name(par_name), type(par_type) {
-    assert(par_name != nullptr);
+    DBUG_ASSERT(par_name != nullptr);
   }
 };
 
 /**
-  Class representing SET DEFAULT, DROP DEFAULT, RENAME COLUMN, SET VISIBLE and
-  SET INVISIBLE clause in ALTER TABLE statement.
+  Class representing SET DEFAULT, DROP DEFAULT and RENAME
+  COLUMN clause in ALTER TABLE statement.
 */
 
 class Alter_column {
@@ -85,19 +86,13 @@ class Alter_column {
   /// The default value supplied.
   Item *def;
 
-  /// The expression to be used to generate the default value.
+  /// The expression to be used to generated the default value.
   Value_generator *m_default_val_expr;
 
-  /// The new column name.
+  /// The new colum name.
   const char *m_new_name;
 
-  enum class Type {
-    SET_DEFAULT,
-    DROP_DEFAULT,
-    RENAME_COLUMN,
-    SET_COLUMN_VISIBLE,
-    SET_COLUMN_INVISIBLE
-  };
+  enum class Type { SET_DEFAULT, DROP_DEFAULT, RENAME_COLUMN };
 
  public:
   /// Type of change requested in ALTER TABLE.
@@ -131,13 +126,6 @@ class Alter_column {
         m_new_name(new_name),
         m_type(Type::RENAME_COLUMN) {}
 
-  /// Constructor used while altering column visibility.
-  Alter_column(const char *par_name, bool par_is_visible)
-      : name(par_name), def(nullptr), m_new_name(nullptr) {
-    m_type = (par_is_visible ? Type::SET_COLUMN_VISIBLE
-                             : Type::SET_COLUMN_INVISIBLE);
-  }
-
  private:
   Type m_type;
 };
@@ -189,12 +177,10 @@ class Alter_constraint_enforcement {
   Alter_constraint_enforcement(Type par_type, const char *par_name,
                                bool par_is_enforced)
       : name(par_name), type(par_type), is_enforced(par_is_enforced) {
-    assert(par_name != nullptr);
+    DBUG_ASSERT(par_name != nullptr);
   }
 };
 
-using CreateFieldApplier = std::function<bool(Create_field *, Alter_info *)>;
-
 /**
   Data describing the table being created by CREATE TABLE or
   altered by ALTER TABLE.
@@ -286,7 +272,7 @@ class Alter_info {
     /// Set for DROP FOREIGN KEY
     DROP_FOREIGN_KEY = 1ULL << 22,
 
-    /// Set for EXCHANGE PARTITION
+    /// Set for EXCHANGE PARITION
     ALTER_EXCHANGE_PARTITION = 1ULL << 23,
 
     /// Set by Sql_cmd_alter_table_truncate_partition::execute()
@@ -333,15 +319,6 @@ class Alter_info {
 
     /// Set for ALTER CONSTRAINT symbol NOT ENFORCED.
     SUSPEND_ANY_CONSTRAINT = 1ULL << 38,
-
-    /// Set if ANY engine attribute is used (also in CREATE) Note that
-    /// this is NOT to be set for SECONDARY_ENGINE_ATTRIBUTE as this flag
-    /// controls if execution should check if SE supports engine
-    /// attributes.
-    ANY_ENGINE_ATTRIBUTE = 1ULL << 39,
-
-    /// Set for column visibility attribute alter.
-    ALTER_COLUMN_VISIBILITY = 1ULL << 40
   };
 
   enum enum_enable_or_disable { LEAVE_AS_IS, ENABLE, DISABLE };
@@ -369,10 +346,10 @@ class Alter_info {
      Describes the level of concurrency during ALTER TABLE.
   */
   enum enum_alter_table_lock {
-    // Maximum supported level of concurrency for the given operation.
+    // Maximum supported level of concurency for the given operation.
     ALTER_TABLE_LOCK_DEFAULT,
 
-    // Allow concurrent reads & writes. If not supported, give error.
+    // Allow concurrent reads & writes. If not supported, give erorr.
     ALTER_TABLE_LOCK_NONE,
 
     // Allow concurrent reads only. If not supported, give error.
@@ -421,8 +398,6 @@ class Alter_info {
 
   // List of columns, used by both CREATE and ALTER TABLE.
   List<Create_field> create_list;
-  std::vector<CreateFieldApplier> cf_appliers;
-
   // Type of ALTER TABLE operation.
   ulonglong flags;
   // Enable or disable keys.
@@ -494,7 +469,7 @@ class Alter_info {
                  const CHARSET_INFO *cs, bool has_explicit_collation,
                  uint uint_geom_type, Value_generator *gcol_info,
                  Value_generator *default_val_expr, const char *opt_after,
-                 std::optional<gis::srid_t> srid,
+                 Nullable<gis::srid_t> srid,
                  Sql_check_constraint_spec_list *check_cons_list,
                  dd::Column::enum_hidden_type hidden, bool is_array = false);
 
@@ -535,7 +510,7 @@ class Alter_table_ctx {
      @return path to the original table.
   */
   const char *get_path() const {
-    assert(!tmp_table);
+    DBUG_ASSERT(!tmp_table);
     return path;
   }
 
@@ -584,7 +559,7 @@ class Alter_table_ctx {
   char new_path[FN_REFLEN + 1];
   char tmp_path[FN_REFLEN + 1];
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   /** Indicates that we are altering temporary table. Used only in asserts. */
   bool tmp_table;
 #endif
@@ -603,10 +578,12 @@ class Sql_cmd_common_alter_table : public Sql_cmd_ddl_table {
 
   ~Sql_cmd_common_alter_table() override = 0;  // force abstract class
 
-  enum_sql_command sql_command_code() const final { return SQLCOM_ALTER_TABLE; }
+  enum_sql_command sql_command_code() const override final {
+    return SQLCOM_ALTER_TABLE;
+  }
 };
 
-inline Sql_cmd_common_alter_table::~Sql_cmd_common_alter_table() = default;
+inline Sql_cmd_common_alter_table::~Sql_cmd_common_alter_table() {}
 
 /**
   Represents the generic ALTER TABLE statement.
