@@ -1,4 +1,4 @@
-/* Copyright (c) 2009, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2009, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -28,8 +28,7 @@
 #ifndef SQL_CMD_INCLUDED
 #define SQL_CMD_INCLUDED
 
-#include <assert.h>
-
+#include "my_dbug.h"
 #include "my_sqlcommand.h"
 #include "sql/select_lex_visitor.h"
 
@@ -39,24 +38,6 @@ struct handlerton;
 struct MYSQL_LEX_STRING;
 struct MYSQL_LEX_CSTRING;
 
-/**
-  What type of Sql_cmd we're dealing with (DML, DDL, ...).
-
-  "Other" may be used for commands that are neither DML or DDL, such as
-  shutdown.
-
-  Theoretically, a command can run through code paths of both DDL and DML
-  (e.g. CREATE TABLE ... AS SELECT ...), but at this point, a command
-  must identify as only one thing.
-*/
-enum enum_sql_cmd_type {
-  SQL_CMD_UNDETERMINED = 0,
-  SQL_CMD_DDL = 1,
-  SQL_CMD_DML = 2,
-  SQL_CMD_DCL = 4,
-  SQL_CMD_OTHER = 8
-};
-
 /**
   Representation of an SQL command.
 
@@ -90,36 +71,20 @@ class Sql_cmd {
   */
   virtual enum_sql_command sql_command_code() const = 0;
 
-  /**
-    @return true if object represents a preparable statement, ie. a query
-    that is prepared with a PREPARE statement and executed with an EXECUTE
-    statement. False is returned for regular statements (non-preparable
-    statements) that are executed directly. Also false if statement is part
-    of a stored procedure.
-  */
-  bool needs_explicit_preparation() const {
-    return m_owner != nullptr && !m_part_of_sp;
-  }
-  /**
-    @return true if statement is regular, ie not prepared statement and not
-    part of stored procedure.
-  */
-  bool is_regular() const { return m_owner == nullptr && !m_part_of_sp; }
-
   /// @return true if this statement is prepared
   bool is_prepared() const { return m_prepared; }
 
   /**
     Prepare this SQL statement.
 
-    param thd the current thread
+    @param thd the current thread
 
     @returns false if success, true if error
   */
-  virtual bool prepare(THD *) {
+  virtual bool prepare(THD *thd MY_ATTRIBUTE((unused))) {
     // Default behavior for a statement is to have no preparation code.
     /* purecov: begin inspected */
-    assert(!is_prepared());
+    DBUG_ASSERT(!is_prepared());
     set_prepared();
     return false;
     /* purecov: end */
@@ -135,44 +100,47 @@ class Sql_cmd {
   /**
     Command-specific reinitialization before execution of prepared statement
 
-    param thd  Current THD.
+    @see reinit_stmt_before_use()
+
+    @note Currently this function is overloaded for INSERT/REPLACE stmts only.
+
+    @param thd  Current THD.
   */
-  virtual void cleanup(THD *) { m_secondary_engine = nullptr; }
+  virtual void cleanup(THD *thd MY_ATTRIBUTE((unused))) {
+    m_secondary_engine = nullptr;
+  }
 
   /// Set the owning prepared statement
-  void set_owner(Prepared_statement *stmt) {
-    assert(!m_part_of_sp);
-    m_owner = stmt;
-  }
+  void set_owner(Prepared_statement *stmt) { m_owner = stmt; }
 
   /// Get the owning prepared statement
-  Prepared_statement *owner() const { return m_owner; }
+  Prepared_statement *get_owner() { return m_owner; }
 
-  /**
-    Mark statement as part of procedure. Such statements can be executed
-    multiple times, the first execute() call will also prepare it.
-  */
-  void set_as_part_of_sp() {
-    assert(!m_part_of_sp && m_owner == nullptr);
-    m_part_of_sp = true;
-  }
-  /// @returns true if statement is part of a stored procedure
-  bool is_part_of_sp() const { return m_part_of_sp; }
-
-  /// @return SQL command type (DML, DDL, ... -- "undetermined" by default)
-  virtual enum enum_sql_cmd_type sql_cmd_type() const {
-    return SQL_CMD_UNDETERMINED;
-  }
+  /// @return true if SQL command is a DML statement
+  virtual bool is_dml() const { return false; }
 
   /// @return true if implemented as single table plan, DML statement only
   virtual bool is_single_table_plan() const {
     /* purecov: begin inspected */
-    assert(sql_cmd_type() == SQL_CMD_DML);
+    DBUG_ASSERT(is_dml());
     return false;
     /* purecov: end */
   }
 
-  virtual bool accept(THD *, Select_lex_visitor *) { return false; }
+  /**
+    Temporary function used to "unprepare" a prepared statement after
+    preparation, so that a subsequent execute statement will reprepare it.
+    This is done because UNIT::cleanup() will un-resolve all resolved QBs.
+  */
+  virtual void unprepare(THD *thd MY_ATTRIBUTE((unused))) {
+    DBUG_ASSERT(is_prepared());
+    m_prepared = false;
+  }
+
+  virtual bool accept(THD *thd MY_ATTRIBUTE((unused)),
+                      Select_lex_visitor *visitor MY_ATTRIBUTE((unused))) {
+    return false;
+  }
 
   /**
     Is this statement of a type and on a form that makes it eligible
@@ -182,26 +150,20 @@ class Sql_cmd {
     the statement is not eligible for execution in a secondary storage
     engine
   */
-  virtual const MYSQL_LEX_CSTRING *eligible_secondary_storage_engine(
-      THD *) const {
+  virtual const MYSQL_LEX_CSTRING *eligible_secondary_storage_engine() const {
     return nullptr;
   }
 
-  /** @return true if the operation is BULK LOAD. */
-  virtual bool is_bulk_load() const { return false; }
-
   /**
     Disable use of secondary storage engines in this statement. After
     a call to this function, the statement will not try to use a
     secondary storage engine until it is reprepared.
   */
   void disable_secondary_storage_engine() {
-    assert(m_secondary_engine == nullptr);
+    DBUG_ASSERT(m_secondary_engine == nullptr);
     m_secondary_engine_enabled = false;
   }
 
-  void enable_secondary_storage_engine() { m_secondary_engine_enabled = true; }
-
   /**
     Has use of secondary storage engines been disabled for this statement?
   */
@@ -210,22 +172,17 @@ class Sql_cmd {
   }
 
   /**
-  Mark the current statement as using a secondary storage engine.
-  This function must be called before the statement starts opening
-  tables in a secondary engine.
+    Mark the current statement as using a secondary storage engine.
+    This function must be called before the statement starts opening
+    tables in a secondary engine.
   */
   void use_secondary_storage_engine(const handlerton *hton) {
-    assert(m_secondary_engine_enabled);
+    DBUG_ASSERT(m_secondary_engine_enabled);
     m_secondary_engine = hton;
   }
 
   /**
     Is this statement using a secondary storage engine?
-    @note that this is reliable during optimization and afterwards; during
-    preparation, if this is an explicit preparation (SQL PREPARE, C API
-    PREPARE, and automatic repreparation), it may be false as RAPID tables have
-    not yet been opened. Therefore, during preparation, it is safer to test
-    THD::secondary_engine_optimization().
   */
   bool using_secondary_storage_engine() const {
     return m_secondary_engine != nullptr;
@@ -247,7 +204,7 @@ class Sql_cmd {
   }
 
  protected:
-  Sql_cmd() : m_owner(nullptr), m_part_of_sp(false), m_prepared(false) {}
+  Sql_cmd() : m_owner(nullptr), m_prepared(false), prepare_only(true) {}
 
   virtual ~Sql_cmd() {
     /*
@@ -256,16 +213,25 @@ class Sql_cmd {
       simply destroyed instead.
       Do not rely on the destructor for any cleanup.
     */
-    assert(false);
+    DBUG_ASSERT(false);
   }
 
+  /**
+    @return true if object represents a preparable statement, ie. a query
+    that is prepared with a PREPARE statement and executed with an EXECUTE
+    statement. False is returned for regular statements (non-preparable
+    statements) that are executed directly.
+    @todo replace with "m_owner != nullptr" when prepare-once is implemented
+  */
+  bool needs_explicit_preparation() const { return prepare_only; }
+
   /// Set this statement as prepared
   void set_prepared() { m_prepared = true; }
 
  private:
-  Prepared_statement *m_owner;  /// Owning prepared statement, NULL if non-prep.
-  bool m_part_of_sp;            /// True when statement is part of stored proc.
-  bool m_prepared;              /// True when statement has been prepared
+  Prepared_statement
+      *m_owner;     /// Owning prepared statement, nullptr if non-prep.
+  bool m_prepared;  /// True when statement has been prepared
 
   /**
     Tells if a secondary storage engine can be used for this
@@ -286,6 +252,10 @@ class Sql_cmd {
     This property is reset at the start of each execution.
   */
   const handlerton *m_secondary_engine{nullptr};
+
+ protected:
+  bool prepare_only;  /// @see needs_explicit_preparation
+                      /// @todo remove when prepare-once is implemented
 };
 
 #endif  // SQL_CMD_INCLUDED
