@@ -1,19 +1,18 @@
 #ifndef SQL_EXECUTOR_INCLUDED
 #define SQL_EXECUTOR_INCLUDED
 
-/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,35 +28,40 @@
   Classes for query execution.
 */
 
+#include <string.h>
 #include <sys/types.h>
-#include <string>
+#include <memory>
 #include <vector>
 
 #include "my_alloc.h"
+#include "my_base.h"
+#include "my_compiler.h"
 #include "my_inttypes.h"
 #include "my_table_map.h"
+#include "sql/row_iterator.h"
 #include "sql/sql_lex.h"
 #include "sql/sql_opt_exec_shared.h"  // QEP_shared_owner
 #include "sql/table.h"
 #include "sql/temp_table_param.h"  // Temp_table_param
 
+class CacheInvalidatorIterator;
 class Cached_item;
 class Field;
 class Field_longlong;
 class Filesort;
+class FollowTailIterator;
 class Item;
 class Item_sum;
 class JOIN;
 class JOIN_TAB;
-class KEY;
+class MultiRangeRowIterator;
 class Opt_trace_object;
 class QEP_TAB;
-class RowIterator;
+class QUICK_SELECT_I;
 class THD;
-template <class T>
-class mem_root_deque;
-
-struct AccessPath;
+class Window;
+enum class Window_retrieve_cached_row_reason;
+struct CACHE_FIELD;
 struct POSITION;
 template <class T>
 class List;
@@ -134,7 +138,7 @@ class SJ_TMP_TABLE {
 */
 class Semijoin_mat_exec {
  public:
-  Semijoin_mat_exec(Table_ref *sj_nest, bool is_scan, uint table_count,
+  Semijoin_mat_exec(TABLE_LIST *sj_nest, bool is_scan, uint table_count,
                     uint mat_table_index, uint inner_table_index)
       : sj_nest(sj_nest),
         is_scan(is_scan),
@@ -143,8 +147,8 @@ class Semijoin_mat_exec {
         inner_table_index(inner_table_index),
         table_param(),
         table(nullptr) {}
-  ~Semijoin_mat_exec() = default;
-  Table_ref *const sj_nest;      ///< Semi-join nest for this materialization
+  ~Semijoin_mat_exec() {}
+  TABLE_LIST *const sj_nest;     ///< Semi-join nest for this materialization
   const bool is_scan;            ///< true if executing a scan, false if lookup
   const uint table_count;        ///< Number of tables in the sj-nest
   const uint mat_table_index;    ///< Index in join_tab for materialized table
@@ -155,10 +159,11 @@ class Semijoin_mat_exec {
 
 void setup_tmptable_write_func(QEP_TAB *tab, Opt_trace_object *trace);
 
-[[nodiscard]] bool copy_fields(Temp_table_param *param, const THD *thd,
-                               bool reverse_copy = false);
+MY_ATTRIBUTE((warn_unused_result))
+bool copy_fields(Temp_table_param *param, const THD *thd,
+                 bool reverse_copy = false);
 
-enum Copy_func_type : int {
+enum Copy_func_type {
   /**
     In non-windowing step, copies functions
   */
@@ -179,7 +184,7 @@ enum Copy_func_type : int {
     that is we need to read all rows of a partition before we can compute the
     wf's value for the the first row in the partition.
   */
-  CFT_WF_NEEDS_PARTITION_CARDINALITY,
+  CFT_WF_NEEDS_CARD,
   /**
     In windowing step, copies framing window functions that read only one row
     per frame.
@@ -201,15 +206,22 @@ enum Copy_func_type : int {
   */
   CFT_WF,
   /**
-    Copies Item_field only (typically because other functions might depend
-    on those fields).
-  */
-  CFT_FIELDS,
+    Copies all items that are expressions containing aggregates, but are not
+    themselves aggregates. Such expressions are typically split into their
+    constituent parts during setup_fields(), such that the parts that are
+    _not_ aggregates are replaced by Item_refs that point into a slice.
+    See AggregateIterator::Read() for more details.
+   */
+  CFT_DEPENDING_ON_AGGREGATE
 };
 
 bool copy_funcs(Temp_table_param *, const THD *thd,
                 Copy_func_type type = CFT_ALL);
 
+// Combines copy_fields() and copy_funcs().
+bool copy_fields_and_funcs(Temp_table_param *param, const THD *thd,
+                           Copy_func_type type = CFT_ALL);
+
 /**
   Copy the lookup key into the table ref's key buffer.
 
@@ -218,38 +230,42 @@ bool copy_funcs(Temp_table_param *, const THD *thd,
   @param ref   information about the index lookup key
 
   @retval false ref key copied successfully
-  @retval true  error detected during copying of key
+  @retval true  error dectected during copying of key
 */
-bool construct_lookup(THD *thd, TABLE *table, Index_lookup *ref);
+bool construct_lookup_ref(THD *thd, TABLE *table, TABLE_REF *ref);
 
 /** Help function when we get some an error from the table handler. */
 int report_handler_error(TABLE *table, int error);
 
+int safe_index_read(QEP_TAB *tab);
+
 int join_read_const_table(JOIN_TAB *tab, POSITION *pos);
 
 int do_sj_dups_weedout(THD *thd, SJ_TMP_TABLE *sjtbl);
 int update_item_cache_if_changed(List<Cached_item> &list);
 
-// Create list for using with temporary table
-bool change_to_use_tmp_fields(mem_root_deque<Item *> *fields, THD *thd,
+// Create list for using with tempory table
+bool change_to_use_tmp_fields(List<Item> &all_fields,
+                              size_t num_select_elements, THD *thd,
                               Ref_item_array ref_item_array,
-                              mem_root_deque<Item *> *res_fields,
-                              size_t added_non_hidden_fields,
-                              bool windowing = false);
-// Create list for using with temporary table
-bool change_to_use_tmp_fields_except_sums(mem_root_deque<Item *> *fields,
-                                          THD *thd, Query_block *select,
-                                          Ref_item_array ref_item_array,
-                                          mem_root_deque<Item *> *res_fields,
-                                          size_t added_non_hidden_fields);
+                              List<Item> *res_selected_fields,
+                              List<Item> *res_all_fields);
+// Create list for using with tempory table
+bool change_refs_to_tmp_fields(List<Item> &all_fields,
+                               size_t num_select_elements, THD *thd,
+                               Ref_item_array ref_item_array,
+                               List<Item> *res_selected_fields,
+                               List<Item> *res_all_fields);
 bool prepare_sum_aggregators(Item_sum **func_ptr, bool need_distinct);
 bool setup_sum_funcs(THD *thd, Item_sum **func_ptr);
 bool make_group_fields(JOIN *main_join, JOIN *curr_join);
-bool check_unique_fields(TABLE *table);
-ulonglong calc_row_hash(TABLE *table);
-ulonglong calc_field_hash(const Field *field, ulonglong *hash);
-int read_const(TABLE *table, Index_lookup *ref);
-bool table_rec_cmp(TABLE *table);
+bool setup_copy_fields(List<Item> &all_fields, size_t num_select_elements,
+                       THD *thd, Temp_table_param *param,
+                       Ref_item_array ref_item_array,
+                       List<Item> *res_selected_fields,
+                       List<Item> *res_all_fields);
+bool check_unique_constraint(TABLE *table);
+ulonglong unique_hash(const Field *field, ulonglong *hash);
 
 class QEP_TAB : public QEP_shared_owner {
  public:
@@ -261,12 +277,19 @@ class QEP_TAB : public QEP_shared_owner {
         firstmatch_return(NO_PLAN_IDX),
         loosescan_key_len(0),
         match_tab(NO_PLAN_IDX),
+        first_unmatched(NO_PLAN_IDX),
         rematerialize(false),
+        used_null_fields(false),
+        used_uneven_bit_fields(false),
+        copy_current_rowid(nullptr),
         not_used_in_distinct(false),
+        cache_idx_cond(nullptr),
         having(nullptr),
         tmp_table_param(nullptr),
         filesort(nullptr),
         ref_item_slice(REF_SLICE_SAVED_BASE),
+        m_condition_optim(nullptr),
+        m_quick_optim(nullptr),
         m_keyread_optim(false),
         m_reversed_access(false),
         lateral_derived_tables_depend_on_me(0) {}
@@ -279,6 +302,8 @@ class QEP_TAB : public QEP_shared_owner {
   // Getters and setters
 
   Item *condition_optim() const { return m_condition_optim; }
+  QUICK_SELECT_I *quick_optim() const { return m_quick_optim; }
+  void set_quick_optim() { m_quick_optim = quick(); }
   void set_condition_optim() { m_condition_optim = condition(); }
   bool keyread_optim() const { return m_keyread_optim; }
   void set_keyread_optim() {
@@ -292,6 +317,16 @@ class QEP_TAB : public QEP_shared_owner {
     if (t) t->reginfo.qep_tab = this;
   }
 
+  bool temporary_table_deduplicates() const {
+    return m_temporary_table_deduplicates;
+  }
+  void set_temporary_table_deduplicates(bool arg) {
+    m_temporary_table_deduplicates = arg;
+  }
+
+  bool using_table_scan() const { return m_using_table_scan; }
+  void set_using_table_scan(bool arg) { m_using_table_scan = arg; }
+
   /// @returns semijoin strategy for this table.
   uint get_sj_strategy() const;
 
@@ -309,7 +344,7 @@ class QEP_TAB : public QEP_shared_owner {
 
   /**
     A helper function that allocates appropriate join cache object and
-    sets next_query_block function of previous tab.
+    sets next_select function of previous tab.
   */
   void init_join_cache(JOIN_TAB *join_tab);
 
@@ -330,11 +365,15 @@ class QEP_TAB : public QEP_shared_owner {
   bool use_order() const;  ///< Use ordering provided by chosen index?
 
   /**
-    Construct an access path for reading from this table in the query,
-    using the access method that has been determined previously
-    (e.g., table scan, ref access, optional sort afterwards, etc.).
-   */
-  AccessPath *access_path();
+     Used to begin a new execution of a subquery. Necessary if this subquery
+     has done a filesort which which has cleared condition/quick.
+  */
+  void restore_quick_optim_and_condition() {
+    if (m_condition_optim) set_condition(m_condition_optim);
+    if (m_quick_optim) set_quick(m_quick_optim);
+  }
+
+  void pick_table_access_method();
   void push_index_cond(const JOIN_TAB *join_tab, uint keyno,
                        Opt_trace_object *trace_obj);
 
@@ -345,7 +384,7 @@ class QEP_TAB : public QEP_shared_owner {
 
  public:
   /// Pointer to table reference
-  Table_ref *table_ref;
+  TABLE_LIST *table_ref;
 
   /* Variables for semi-join duplicate elimination */
   SJ_TMP_TABLE *flush_weedout_table;
@@ -372,6 +411,8 @@ class QEP_TAB : public QEP_shared_owner {
   */
   plan_idx match_tab;
 
+  plan_idx first_unmatched; /**< used for optimization purposes only   */
+
   /// Dependent table functions have to be materialized on each new scan
   bool rematerialize;
 
@@ -383,6 +424,80 @@ class QEP_TAB : public QEP_shared_owner {
   };
   Setup_func materialize_table = NO_SETUP;
   bool using_dynamic_range = false;
+  unique_ptr_destroy_only<RowIterator> iterator;
+
+  // join-cache-related members
+  bool used_null_fields;
+  bool used_uneven_bit_fields;
+
+  // Whether the row ID is needed for this table, and where the row ID can be
+  // found.
+  //
+  // If rowid_status != NO_ROWID_NEEDED, it indicates that this table is part of
+  // weedout. In order for weedout to eliminate duplicate rows, it needs a
+  // unique ID for each row it reads. In general, any operator that needs the
+  // row ID should ask the storage engine directly for the ID of the last row
+  // read by calling handler::position(). However, it is not that simple...
+  //
+  // As mentioned, position() will ask the storage engine to provide the row ID
+  // of the last row read. But some iterators (i.e. HashJoinIterator) buffer
+  // rows, so that the last row returned by i.e. HashJoinIterator is not
+  // necessarily the same as the last row returned by the storage engine.
+  // This means that any iterator that buffers rows without using a temporary
+  // table must store and restore the row ID itself. If a temporary table is
+  // used, the temporary table engine will provide the row ID.
+  //
+  // When creating the iterator tree, any iterator that needs to interact with
+  // row IDs must adhere to the following rules:
+  //
+  //   1. Any iterator that buffers rows without using a temporary table must
+  //      store and restore the row ID if rowid_status != NO_ROWID_NEEDED.
+  //      In addition, they must mark that they do so by changing the value of
+  //      rowid_status to ROWID_PROVIDED_BY_ITERATOR_READ_CALL in their
+  //      constructor.
+  //   2. Any iterator that needs the row ID (currently only WeedoutIterator)
+  //      must check rowid_status to see if they should call position() or trust
+  //      that a row ID is provided by another iterator. Note that when filesort
+  //      sorts by row ID, it handles everything regarding row ID itself.
+  //      It manages this because sorting by row ID always goes through a
+  //      temporary table, which in turn will provide the row ID to filesort.
+  //   3. As the value of rowid_status may change while building the iterator
+  //      tree, all iterators interacting with row IDs must cache the
+  //      value they see in their constructor.
+  //
+  //  Consider the following example:
+  //
+  //        Weedout (t1,t3)
+  //              |
+  //         Nested loop
+  //        /          |
+  //    Hash join      t3
+  //    /      |
+  //   t1      t2
+  //
+  // During query planning, rowid_status will be set to
+  // NEED_TO_CALL_POSITION_FOR_ROWID on t1 and t3 due to the planned weedout.
+  // When the iterator tree is constructed, the hash join constructor will be
+  // called first. It caches the value of rowid_status for t1 per rule 3 above,
+  // and changes the value to ROWID_PROVIDED_BY_ITERATOR_READ_CALL per rule 1.
+  // This notifies any iterator above itself that they should not call
+  // position(). When the nested loop constructor is called, nothing happens, as
+  // the iterator does not interact with row IDs in any way. When the weedout
+  // constructor is called, it caches the value of rowid_status for t1 and t3
+  // per rule 3. During execution, the weedout will call position() on t3,
+  // since rowid_status was NEED_TO_CALL_POSITION_FOR_ROWID when the iterator
+  // was constructed. It will not call position() on t1, as rowid_status was set
+  // to ROWID_PROVIDED_BY_ITERATOR_READ_CALL by the hash join iterator.
+  //
+  // Note that if you have a NULL-complemented row, there is no guarantee that
+  // position() will provide a valid row ID, or not even a valid row ID pointer.
+  // So all operations must check for NULL-complemented rows before trying to
+  // use/copy a row ID.
+  rowid_statuses rowid_status{NO_ROWID_NEEDED};
+
+  // Helper structure for copying the row ID. Only used by BNL and BKA in the
+  // non-iterator executor.
+  CACHE_FIELD *copy_current_rowid;
 
   /** true <=> remove duplicates on this table. */
   bool needs_duplicate_removal = false;
@@ -394,6 +509,9 @@ class QEP_TAB : public QEP_shared_owner {
   // in older versions of MySQL.
   bool not_used_in_distinct;
 
+  /// Index condition for BKA access join
+  Item *cache_idx_cond;
+
   /** HAVING condition for checking prior saving a record into tmp table*/
   Item *having;
 
@@ -426,7 +544,7 @@ class QEP_TAB : public QEP_shared_owner {
 
   /**
     If we pushed a global ORDER BY down onto this first table, that ORDER BY
-    list will be preserved here.
+    list will be preseved here.
    */
   ORDER *filesort_pushed_order = nullptr;
 
@@ -436,9 +554,26 @@ class QEP_TAB : public QEP_shared_owner {
   */
   uint ref_item_slice;
 
-  /// Condition as it was set by the optimizer, used for EXPLAIN.
-  /// m_condition may be overwritten at a later stage.
-  Item *m_condition_optim = nullptr;
+  /// @see m_quick_optim
+  Item *m_condition_optim;
+
+  /**
+     m_quick is the quick "to be used at this stage of execution".
+     It can happen that filesort uses the quick (produced by the optimizer) to
+     produce a sorted result, then the read of this result has to be done
+     without "quick", so we must reset m_quick to NULL, but we want to delay
+     freeing of m_quick or it would close the filesort's result and the table
+     prematurely.
+     In that case, we move m_quick to m_quick_optim (=> delay deletion), reset
+     m_quick to NULL (read of filesort's result will be without quick); if
+     this is a subquery which is later executed a second time,
+     QEP_TAB::reset() will restore the quick from m_quick_optim into m_quick.
+     quick_optim stands for "the quick decided by the optimizer".
+     EXPLAIN reads this member and m_condition_optim; so if you change them
+     after exposing the plan (setting plan_state), do it with the
+     LOCK_query_plan mutex.
+  */
+  QUICK_SELECT_I *m_quick_optim;
 
   /**
      True if only index is going to be read for this table. This is the
@@ -459,20 +594,73 @@ class QEP_TAB : public QEP_shared_owner {
      LDT, for efficiency (less useless calls to QEP_TAB::refresh_lateral())
      and clarity in EXPLAIN.
   */
-  qep_tab_map lateral_derived_tables_depend_on_me;
+  table_map lateral_derived_tables_depend_on_me;
 
-  Mem_root_array<const AccessPath *> *invalidators = nullptr;
+  Mem_root_array<const CacheInvalidatorIterator *> *invalidators = nullptr;
+
+  /**
+    If this table is a temporary table used for whole-JOIN materialization
+    (e.g. before sorting): true iff the table deduplicates, typically by way
+    of an unique index.
+
+    Otherwise, unused.
+   */
+  bool m_temporary_table_deduplicates = false;
+
+  /**
+    True if iterator is a TableScanIterator. Used so that we can know whether
+    to stream directly across derived tables and into sorts (we cannot if there
+    is a ref access).
+   */
+  bool m_using_table_scan = false;
+
+  /**
+    If this table is a recursive reference(to a CTE), contains a pointer to the
+    iterator here. This is so that MaterializeIterator can get a list of all
+    such iterators, to coordinate rematerialization and other signals.
+   */
+  FollowTailIterator *recursive_iterator = nullptr;
+
+  /**
+    If this table is a multi-range row iterator (the inner part of BKA),
+    contains a pointer to the iterator here. This is solely for use during
+    construction of the iterator tree, so that when we set up the BKAIterator,
+    we have easy access to the MRR iterator.
+   */
+  MultiRangeRowIterator *mrr_iterator = nullptr;
 
   QEP_TAB(const QEP_TAB &);             // not defined
   QEP_TAB &operator=(const QEP_TAB &);  // not defined
 };
 
-bool set_record_buffer(TABLE *table, double expected_rows_to_fetch);
+/**
+   @returns a pointer to the QEP_TAB whose index is qtab->member. For
+   example, QEP_AT(x,first_inner) is the first_inner table of x.
+*/
+#define QEP_AT(qtab, member) (qtab->join()->qep_tab[qtab->member])
+
+/**
+   Use this class when you need a QEP_TAB not connected to any JOIN_TAB.
+*/
+class QEP_TAB_standalone {
+ public:
+  QEP_TAB_standalone() { m_qt.set_qs(&m_qs); }
+  ~QEP_TAB_standalone() { m_qt.cleanup(); }
+  /// @returns access to the QEP_TAB
+  QEP_TAB &as_QEP_TAB() { return m_qt; }
+
+ private:
+  QEP_shared m_qs;
+  QEP_TAB m_qt;
+};
+
+void copy_sum_funcs(Item_sum **func_ptr, Item_sum **end_ptr);
+bool set_record_buffer(const QEP_TAB *tab);
+bool init_sum_functions(Item_sum **func_ptr, Item_sum **end_ptr);
 void init_tmptable_sum_functions(Item_sum **func_ptr);
+bool update_sum_func(Item_sum **func_ptr);
 void update_tmptable_sum_func(Item_sum **func_ptr, TABLE *tmp_table);
 bool has_rollup_result(Item *item);
-bool is_rollup_group_wrapper(const Item *item);
-Item *unwrap_rollup_group(Item *item);
 
 /*
   If a condition cannot be applied right away, for instance because it is a
@@ -485,34 +673,6 @@ struct PendingCondition {
   int table_index_to_attach_to;  // -1 means “on the last possible outer join”.
 };
 
-/**
-  Cache invalidator iterators we need to apply, but cannot yet due to outer
-  joins. As soon as “table_index_to_invalidate” is visible in our current join
-  nest (which means there could no longer be NULL-complemented rows we could
-  forget), we can and must output this invalidator and remove it from the array.
- */
-struct PendingInvalidator {
-  /**
-    The table whose every (post-join) row invalidates one or more derived
-    lateral tables.
-   */
-  QEP_TAB *qep_tab;
-  plan_idx table_index_to_invalidate;
-};
-
-enum CallingContext {
-  TOP_LEVEL,
-  DIRECTLY_UNDER_SEMIJOIN,
-  DIRECTLY_UNDER_OUTER_JOIN,
-  DIRECTLY_UNDER_WEEDOUT
-};
-
-/**
-  Create an AND conjunction of all given items. If there are no items, returns
-  nullptr. If there's only one item, returns that item.
- */
-Item *CreateConjunction(List<Item> *items);
-
 unique_ptr_destroy_only<RowIterator> PossiblyAttachFilterIterator(
     unique_ptr_destroy_only<RowIterator> iterator,
     const std::vector<Item *> &conditions, THD *thd);
@@ -520,85 +680,22 @@ unique_ptr_destroy_only<RowIterator> PossiblyAttachFilterIterator(
 void SplitConditions(Item *condition, QEP_TAB *current_table,
                      std::vector<Item *> *predicates_below_join,
                      std::vector<PendingCondition> *predicates_above_join,
-                     std::vector<PendingCondition> *join_conditions,
-                     plan_idx semi_join_table_idx, qep_tab_map left_tables);
+                     std::vector<PendingCondition> *join_conditions);
 
-/**
-  For a MATERIALIZE access path, move any non-basic iterators (e.g. sorts and
-  filters) from table_path to above the path, for easier EXPLAIN and generally
-  simpler structure. Note the assert in CreateIteratorFromAccessPath() that we
-  succeeded. (ALTERNATIVE counts as a basic iterator in this regard.)
+bool process_buffered_windowing_record(THD *thd, Temp_table_param *param,
+                                       const bool new_partition_or_eof,
+                                       bool *output_row_ready);
+bool buffer_windowing_record(THD *thd, Temp_table_param *param,
+                             bool *new_partition);
+bool bring_back_frame_row(THD *thd, Window *w, Temp_table_param *out_param,
+                          int64 rowno, Window_retrieve_cached_row_reason reason,
+                          int fno = 0);
 
-  We do this by finding the second-bottommost access path, and inserting our
-  materialize node as its child. The bottommost one becomes the actual table
-  access path.
-
-  If a ZERO_ROWS access path is materialized, we simply replace the MATERIALIZE
-  path with the ZERO_ROWS path, since there is nothing to materialize.
-  @param thd The current thread.
-  @param path the MATERIALIZE path.
-  @param query_block The query block in which 'path' belongs.
-  @returns The new root of the set of AccessPaths formed by 'path' and its
-  descendants.
- */
-AccessPath *MoveCompositeIteratorsFromTablePath(THD *thd, AccessPath *path,
-                                                const Query_block &query_block);
-
-AccessPath *GetAccessPathForDerivedTable(
-    THD *thd, Table_ref *table_ref, TABLE *table, bool rematerialize,
-    Mem_root_array<const AccessPath *> *invalidators, bool need_rowid,
-    AccessPath *table_path);
-
-void ConvertItemsToCopy(const mem_root_deque<Item *> &items, Field **fields,
-                        Temp_table_param *param);
-std::string RefToString(const Index_lookup &ref, const KEY &key,
+unique_ptr_destroy_only<RowIterator> GetIteratorForDerivedTable(
+    THD *thd, QEP_TAB *qep_tab);
+void ConvertItemsToCopy(List<Item> *items, Field **fields,
+                        Temp_table_param *param, JOIN *join);
+std::string RefToString(const TABLE_REF &ref, const KEY *key,
                         bool include_nulls);
 
-bool MaterializeIsDoingDeduplication(TABLE *table);
-
-/**
-  Split AND conditions into their constituent parts, recursively.
-  Conditions that are not AND conditions are appended unchanged onto
-  condition_parts. E.g. if you have ((a AND b) AND c), condition_parts
-  will contain [a, b, c], plus whatever it contained before the call.
- */
-bool ExtractConditions(Item *condition,
-                       Mem_root_array<Item *> *condition_parts);
-
-AccessPath *create_table_access_path(THD *thd, TABLE *table,
-                                     AccessPath *range_scan,
-                                     Table_ref *table_ref, POSITION *position,
-                                     bool count_examined_rows);
-
-/**
-  Creates an iterator for the given table, then calls Init() on the resulting
-  iterator. Unlike create_table_iterator(), this can create iterators for sort
-  buffer results (which are set in the TABLE object during query execution).
-  Returns nullptr on failure.
- */
-unique_ptr_destroy_only<RowIterator> init_table_iterator(
-    THD *thd, TABLE *table, AccessPath *range_scan, Table_ref *table_ref,
-    POSITION *position, bool ignore_not_found_rows, bool count_examined_rows);
-
-/**
-  A short form for when there's no range scan, recursive CTEs or cost
-  information; just a unique_result or a simple table scan. Normally, you should
-  prefer just instantiating an iterator yourself -- this is for legacy use only.
- */
-inline unique_ptr_destroy_only<RowIterator> init_table_iterator(
-    THD *thd, TABLE *table, bool ignore_not_found_rows,
-    bool count_examined_rows) {
-  return init_table_iterator(thd, table, nullptr, nullptr, nullptr,
-                             ignore_not_found_rows, count_examined_rows);
-}
-
-AccessPath *ConnectJoins(plan_idx upper_first_idx, plan_idx first_idx,
-                         plan_idx last_idx, QEP_TAB *qep_tabs, THD *thd,
-                         CallingContext calling_context,
-                         std::vector<PendingCondition> *pending_conditions,
-                         std::vector<PendingInvalidator> *pending_invalidators,
-                         std::vector<PendingCondition> *pending_join_conditions,
-                         qep_tab_map *unhandled_duplicates,
-                         table_map *conditions_depend_on_outer_tables);
-
 #endif /* SQL_EXECUTOR_INCLUDED */
