@@ -1,16 +1,15 @@
-/* Copyright (c) 2014, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,15 +28,15 @@
 #ifndef SQL_OPT_EXEC_SHARED_INCLUDED
 #define SQL_OPT_EXEC_SHARED_INCLUDED
 
-#include <assert.h>
 #include "my_base.h"
-
+#include "my_dbug.h"
 #include "sql/item.h"
 
 class JOIN;
 class Item_func_match;
 class store_key;
 struct POSITION;
+class QUICK_SELECT_I;
 
 /**
    This represents the index of a JOIN_TAB/QEP_TAB in an array. "plan_idx":
@@ -51,20 +50,12 @@ struct POSITION;
    before-first-table" (firstmatch_return==PRE_FIRST_PLAN_IDX) from "No
    FirstMatch" (firstmatch_return==NO_PLAN_IDX).
 */
-using plan_idx = int;
+typedef int8 plan_idx;
 #define NO_PLAN_IDX (-2)  ///< undefined index
 #define PRE_FIRST_PLAN_IDX \
   (-1)  ///< right before the first (first's index is 0)
 
-/**
-  Structure used for index-based lookups.
-
-  @todo Remove any references to the old name: it used to be called TABLE_REF,
-        hence many variables still have the ref prefix. In MySQL "index lookup"
-        and "ref" access are used interchangeable but in EXPLAIN we output the
-        former.
-*/
-struct Index_lookup {
+struct TABLE_REF {
   bool key_err;
   uint key_parts;    ///< num of ...
   uint key_length;   ///< length of key_buff
@@ -127,7 +118,7 @@ struct Index_lookup {
    */
   ulonglong *keypart_hash = nullptr;
 
-  Index_lookup()
+  TABLE_REF()
       : key_err(true),
         key_parts(0),
         key_length(0),
@@ -148,10 +139,9 @@ struct Index_lookup {
     a match.
   */
   bool impossible_null_ref() const {
-    if (null_rejecting == 0) return false;
-    for (uint i = 0; i < key_parts; i++) {
-      if ((null_rejecting & 1 << i) && items[i]->is_null()) {
-        return true;
+    if (null_rejecting != 0) {
+      for (uint i = 0; i < key_parts; i++) {
+        if ((null_rejecting & 1 << i) && items[i]->is_null()) return true;
       }
     }
     return false;
@@ -166,7 +156,7 @@ struct Index_lookup {
   */
 
   bool has_guarded_conds() const {
-    assert(key_parts == 0 || cond_guards != nullptr);
+    DBUG_ASSERT(key_parts == 0 || cond_guards != nullptr);
 
     for (uint i = 0; i < key_parts; i++) {
       if (cond_guards[i]) return true;
@@ -252,6 +242,7 @@ class QEP_shared {
         m_condition(nullptr),
         m_keys(),
         m_records(0),
+        m_quick(nullptr),
         prefix_tables_map(0),
         added_tables_map(0),
         m_ft_func(nullptr),
@@ -266,11 +257,11 @@ class QEP_shared {
   JOIN *join() const { return m_join; }
   void set_join(JOIN *j) { m_join = j; }
   plan_idx idx() const {
-    assert(m_idx >= 0);  // Index must be valid
+    DBUG_ASSERT(m_idx >= 0);  // Index must be valid
     return m_idx;
   }
   void set_idx(plan_idx i) {
-    assert(m_idx == NO_PLAN_IDX);  // Index should not change in lifetime
+    DBUG_ASSERT(m_idx == NO_PLAN_IDX);  // Index should not change in lifetime
     m_idx = i;
   }
   TABLE *table() const { return m_table; }
@@ -289,7 +280,7 @@ class QEP_shared {
   void set_first_upper(plan_idx i) { m_first_upper = i; }
   plan_idx last_inner() { return m_last_inner; }
   plan_idx first_upper() { return m_first_upper; }
-  Index_lookup &ref() { return m_ref; }
+  TABLE_REF &ref() { return m_ref; }
   uint index() const { return m_index; }
   void set_index(uint i) { m_index = i; }
   enum join_type type() const { return m_type; }
@@ -305,8 +296,8 @@ class QEP_shared {
   Key_map &keys() { return m_keys; }
   ha_rows records() const { return m_records; }
   void set_records(ha_rows r) { m_records = r; }
-  AccessPath *range_scan() const { return m_range_scan; }
-  void set_range_scan(AccessPath *q) { m_range_scan = q; }
+  QUICK_SELECT_I *quick() const { return m_quick; }
+  void set_quick(QUICK_SELECT_I *q) { m_quick = q; }
   table_map prefix_tables() const { return prefix_tables_map; }
   table_map added_tables() const { return added_tables_map; }
   Item_func_match *ft_func() const { return m_ft_func; }
@@ -403,7 +394,7 @@ class QEP_shared {
      Used when we read constant tables, in misc optimization (like
      remove_const()), and in execution.
   */
-  Index_lookup m_ref;
+  TABLE_REF m_ref;
 
   /// ID of index used for index scan or semijoin LooseScan
   uint m_index;
@@ -443,10 +434,9 @@ class QEP_shared {
 
   /**
      Non-NULL if quick-select used.
-     Filled in optimization, converted to a RowIterator before execution
-     (used to find rows), and in EXPLAIN.
+     Filled in optimization, used in execution to find rows, and in EXPLAIN.
   */
-  AccessPath *m_range_scan = nullptr;
+  QUICK_SELECT_I *m_quick;
 
   /*
     Maps below are shared because of dynamic range: in execution, it needs to
@@ -482,7 +472,7 @@ class QEP_shared_owner {
   /// Instructs to share the QEP_shared with another owner
   void share_qs(QEP_shared_owner *other) { other->set_qs(m_qs); }
   void set_qs(QEP_shared *q) {
-    assert(!m_qs);
+    DBUG_ASSERT(!m_qs);
     m_qs = q;
   }
 
@@ -496,10 +486,7 @@ class QEP_shared_owner {
   // (before optimization).
   plan_idx idx() const { return m_qs->idx(); }
   void set_idx(plan_idx i) { return m_qs->set_idx(i); }
-  qep_tab_map idx_map() const {
-    assert(m_qs->idx() < static_cast<plan_idx>(CHAR_BIT * sizeof(qep_tab_map)));
-    return qep_tab_map{1} << m_qs->idx();
-  }
+  qep_tab_map idx_map() const { return qep_tab_map{1} << m_qs->idx(); }
 
   TABLE *table() const { return m_qs->table(); }
   POSITION *position() const { return m_qs->position(); }
@@ -518,7 +505,7 @@ class QEP_shared_owner {
   void set_first_sj_inner(plan_idx i) { return m_qs->set_first_sj_inner(i); }
   void set_last_sj_inner(plan_idx i) { return m_qs->set_last_sj_inner(i); }
   void set_first_upper(plan_idx i) { return m_qs->set_first_upper(i); }
-  Index_lookup &ref() const { return m_qs->ref(); }
+  TABLE_REF &ref() const { return m_qs->ref(); }
   uint index() const { return m_qs->index(); }
   void set_index(uint i) { return m_qs->set_index(i); }
   enum join_type type() const { return m_qs->type(); }
@@ -534,8 +521,8 @@ class QEP_shared_owner {
   Key_map &keys() const { return m_qs->keys(); }
   ha_rows records() const { return m_qs->records(); }
   void set_records(ha_rows r) { return m_qs->set_records(r); }
-  AccessPath *range_scan() const { return m_qs->range_scan(); }
-  void set_range_scan(AccessPath *q) { return m_qs->set_range_scan(q); }
+  QUICK_SELECT_I *quick() const { return m_qs->quick(); }
+  void set_quick(QUICK_SELECT_I *q) { return m_qs->set_quick(q); }
   table_map prefix_tables() const { return m_qs->prefix_tables(); }
   table_map added_tables() const { return m_qs->added_tables(); }
   Item_func_match *ft_func() const { return m_qs->ft_func(); }
@@ -584,7 +571,7 @@ enum {
   /**
      The slice which is used during evaluation of expressions; Item_ref::ref
      points there. This is the only slice that is not allocated on the heap;
-     it always points to query_block->base_ref_items.
+     it always points to select_lex->base_ref_items.
 
      If we have a simple query (no temporary tables or GROUP BY needed),
      this slice always contains the base slice, i.e., the actual Items used
@@ -629,6 +616,51 @@ enum {
      table
   */
   REF_SLICE_TMP2,
+  /**
+     Stores the unfinished aggregated row when doing GROUP BY on an
+     ordered table.
+
+     For certain queries with GROUP BY (e.g., when using an index),
+     rows arrive already sorted in the right order for grouping.
+     In that case, we do not need nor use a temporary table, but can
+     just group values as we go. However, we do not necessarily know when
+     a group ends -- a group implicitly ends when we see that the
+     group index values have changed, and by that time, it's too late to
+     output them in the aggregated row (the Fields already point to the
+     new row, so the data is lost).
+
+     Thus, we need to store the values for the current group somewhere.
+     We use a set of Items, which together represent a one-row
+     pseudo-tmp-table holding the current group. These items are created
+     by setup_copy_fields().
+
+     When we have finished reading a row from the last pre-grouping table, we
+     process it either with end_send_group() or end_send():
+
+       * end_send_group(): Compare the new row with the current group.
+         If it belongs to the current group, we update the aggregation functions
+         and move on. If not, we output the aggregated row and overwrite the
+         contents of this slice with the new group.
+
+       * end_send(): Used when we know there's exactly one row for each group
+         (e.g., during a loose index scan). In this case, we can skip the
+         comparison and just output the group directly; however, we still need
+         the temporary table to avoid evaluating Items more than once (see the
+         next paragraph).
+
+     Both functions build the group by copying values of items from the previous
+     stages into a pseudo-table, e.g.
+
+       SELECT a, RAND() AS r FROM t GROUP BY a HAVING r=1;
+
+     copies "a" from "t" and stores it into the pseudo-table (this slice),
+     evaluates rand() and stores it, then finally evaluates "r=1" based on the
+     stored value (so that "r" in the SELECT list and "r" in "r=1" match).
+
+     Groups from this slice are always directly sent to the query's result,
+     and never buffered to any further temporary table.
+  */
+  REF_SLICE_ORDERED_GROUP_BY,
   /**
      The slice with pointers to columns of table(s), ie., the actual Items.
      Only used for queries involving temporary tables or the likes; for simple
