@@ -1,16 +1,15 @@
-/* Copyright (c) 2017, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,15 +27,13 @@
 #include <unordered_map>
 #include <utility>
 
+#include "m_ctype.h"
+#include "m_string.h"
 #include "map_helpers.h"
 #include "my_dbug.h"
 #include "my_list.h"
 #include "mysql/psi/mysql_mutex.h"
 #include "mysql/status_var.h"
-#include "mysql/strings/dtoa.h"
-#include "mysql/strings/int2str.h"
-#include "mysql/strings/m_ctype.h"
-#include "nulls.h"
 #include "sql/current_thd.h"
 #include "sql/item.h"
 #include "sql/mysqld.h"
@@ -45,11 +42,10 @@
 #include "sql/sql_class.h"  // THD
 #include "sql/sql_const.h"
 #include "sql/sql_plugin.h"
-#include "sql/strfunc.h"  // find_type
-#include "sql/sys_vars_shared.h"
+#include "sql/strfunc.h"          // find_type
+#include "sql/sys_vars_shared.h"  // intern_find_sys_var
 #include "sql/system_variables.h"
 #include "sql_string.h"
-#include "strxmov.h"
 #include "template_utils.h"
 #include "typelib.h"
 
@@ -129,7 +125,7 @@ bool plugin_var_memalloc_session_update(THD *thd, SYS_VAR *var, char **dest,
   DBUG_TRACE;
 
   if (value) {
-    const size_t length = strlen(value) + 1;
+    size_t length = strlen(value) + 1;
     LIST *element;
     if (!(element = (LIST *)my_malloc(key_memory_THD_variables,
                                       sizeof(LIST) + length, MYF(MY_WME))))
@@ -179,50 +175,11 @@ SHOW_TYPE pluginvar_show_type(SYS_VAR *plugin_var) {
     case PLUGIN_VAR_DOUBLE:
       return SHOW_DOUBLE;
     default:
-      assert(0);
+      DBUG_ASSERT(0);
       return SHOW_UNDEF;
   }
 }
 
-const void *pluginvar_default_value(SYS_VAR *plugin_var) {
-  switch (plugin_var->flags & (PLUGIN_VAR_TYPEMASK | PLUGIN_VAR_THDLOCAL)) {
-    case PLUGIN_VAR_INT:
-      return &((sysvar_uint_t *)plugin_var)->def_val;
-    case PLUGIN_VAR_LONG:
-      return &((sysvar_ulong_t *)plugin_var)->def_val;
-    case PLUGIN_VAR_LONGLONG:
-      return &((sysvar_ulonglong_t *)plugin_var)->def_val;
-    case PLUGIN_VAR_ENUM:
-      return &((sysvar_enum_t *)plugin_var)->def_val;
-    case PLUGIN_VAR_SET:
-      return &((sysvar_set_t *)plugin_var)->def_val;
-    case PLUGIN_VAR_BOOL:
-      return &((sysvar_bool_t *)plugin_var)->def_val;
-    case PLUGIN_VAR_STR:
-      return &((sysvar_str_t *)plugin_var)->def_val;
-    case PLUGIN_VAR_DOUBLE:
-      return &((sysvar_double_t *)plugin_var)->def_val;
-    case PLUGIN_VAR_INT | PLUGIN_VAR_THDLOCAL:
-      return &((thdvar_uint_t *)plugin_var)->def_val;
-    case PLUGIN_VAR_LONG | PLUGIN_VAR_THDLOCAL:
-      return &((thdvar_ulong_t *)plugin_var)->def_val;
-    case PLUGIN_VAR_LONGLONG | PLUGIN_VAR_THDLOCAL:
-      return &((thdvar_ulonglong_t *)plugin_var)->def_val;
-    case PLUGIN_VAR_ENUM | PLUGIN_VAR_THDLOCAL:
-      return &((thdvar_enum_t *)plugin_var)->def_val;
-    case PLUGIN_VAR_SET | PLUGIN_VAR_THDLOCAL:
-      return &((thdvar_set_t *)plugin_var)->def_val;
-    case PLUGIN_VAR_BOOL | PLUGIN_VAR_THDLOCAL:
-      return &((thdvar_bool_t *)plugin_var)->def_val;
-    case PLUGIN_VAR_STR | PLUGIN_VAR_THDLOCAL:
-      return &((thdvar_str_t *)plugin_var)->def_val;
-    case PLUGIN_VAR_DOUBLE | PLUGIN_VAR_THDLOCAL:
-      return &((thdvar_double_t *)plugin_var)->def_val;
-    default:
-      return nullptr;
-  }
-}
-
 /*
   returns a pointer to the memory which holds the thd-local variable or
   a pointer to the global variable if thd==null.
@@ -230,8 +187,8 @@ const void *pluginvar_default_value(SYS_VAR *plugin_var) {
   has not yet been allocated in the current thread.
 */
 uchar *intern_sys_var_ptr(THD *thd, int offset, bool global_lock) {
-  assert(offset >= 0);
-  assert((uint)offset <= global_system_variables.dynamic_variables_head);
+  DBUG_ASSERT(offset >= 0);
+  DBUG_ASSERT((uint)offset <= global_system_variables.dynamic_variables_head);
 
   if (!thd)
     return (uchar *)global_system_variables.dynamic_variables_ptr + offset;
@@ -320,7 +277,7 @@ bool sys_var_pluginvar::check_update_type(Item_result type) {
 }
 
 uchar *sys_var_pluginvar::real_value_ptr(THD *thd, enum_var_type type) {
-  assert(thd || (type == OPT_GLOBAL) || (type == OPT_PERSIST));
+  DBUG_ASSERT(thd || (type == OPT_GLOBAL) || (type == OPT_PERSIST));
   if (plugin_var->flags & PLUGIN_VAR_THDLOCAL) {
     /* scope of OPT_PERSIST is always GLOBAL */
     if (type == OPT_GLOBAL || type == OPT_PERSIST) thd = nullptr;
@@ -347,7 +304,7 @@ TYPELIB *sys_var_pluginvar::plugin_var_typelib(void) {
 }
 
 uchar *sys_var_pluginvar::do_value_ptr(THD *running_thd, THD *target_thd,
-                                       enum_var_type type, std::string_view) {
+                                       enum_var_type type, LEX_STRING *) {
   uchar *result;
 
   result = real_value_ptr(target_thd, type);
@@ -363,7 +320,7 @@ uchar *sys_var_pluginvar::do_value_ptr(THD *running_thd, THD *target_thd,
 
 bool sys_var_pluginvar::do_check(THD *thd, set_var *var) {
   st_item_value_holder value;
-  assert(plugin_var->check);
+  DBUG_ASSERT(plugin_var->check);
 
   value.value_type = item_value_type;
   value.val_str = item_val_str;
@@ -377,9 +334,9 @@ bool sys_var_pluginvar::do_check(THD *thd, set_var *var) {
 
 bool sys_var_pluginvar::session_update(THD *thd, set_var *var) {
   bool rc = false;
-  assert(!is_readonly());
-  assert(plugin_var->flags & PLUGIN_VAR_THDLOCAL);
-  assert(thd == current_thd);
+  DBUG_ASSERT(!is_readonly());
+  DBUG_ASSERT(plugin_var->flags & PLUGIN_VAR_THDLOCAL);
+  DBUG_ASSERT(thd == current_thd);
 
   mysql_mutex_lock(&LOCK_global_system_variables);
   void *tgt = real_value_ptr(thd, var->type);
@@ -400,23 +357,74 @@ bool sys_var_pluginvar::session_update(THD *thd, set_var *var) {
 
 bool sys_var_pluginvar::global_update(THD *thd, set_var *var) {
   bool rc = false;
-  assert(!is_readonly());
+  DBUG_ASSERT(!is_readonly());
   mysql_mutex_assert_owner(&LOCK_global_system_variables);
 
   void *tgt = real_value_ptr(thd, var->type);
   const void *src = &var->save_result;
 
-  if (!var->value) src = pluginvar_default_value(plugin_var);
+  if (!var->value) {
+    switch (plugin_var->flags & (PLUGIN_VAR_TYPEMASK | PLUGIN_VAR_THDLOCAL)) {
+      case PLUGIN_VAR_INT:
+        src = &((sysvar_uint_t *)plugin_var)->def_val;
+        break;
+      case PLUGIN_VAR_LONG:
+        src = &((sysvar_ulong_t *)plugin_var)->def_val;
+        break;
+      case PLUGIN_VAR_LONGLONG:
+        src = &((sysvar_ulonglong_t *)plugin_var)->def_val;
+        break;
+      case PLUGIN_VAR_ENUM:
+        src = &((sysvar_enum_t *)plugin_var)->def_val;
+        break;
+      case PLUGIN_VAR_SET:
+        src = &((sysvar_set_t *)plugin_var)->def_val;
+        break;
+      case PLUGIN_VAR_BOOL:
+        src = &((sysvar_bool_t *)plugin_var)->def_val;
+        break;
+      case PLUGIN_VAR_STR:
+        src = &((sysvar_str_t *)plugin_var)->def_val;
+        break;
+      case PLUGIN_VAR_DOUBLE:
+        src = &((sysvar_double_t *)plugin_var)->def_val;
+        break;
+      case PLUGIN_VAR_INT | PLUGIN_VAR_THDLOCAL:
+        src = &((thdvar_uint_t *)plugin_var)->def_val;
+        break;
+      case PLUGIN_VAR_LONG | PLUGIN_VAR_THDLOCAL:
+        src = &((thdvar_ulong_t *)plugin_var)->def_val;
+        break;
+      case PLUGIN_VAR_LONGLONG | PLUGIN_VAR_THDLOCAL:
+        src = &((thdvar_ulonglong_t *)plugin_var)->def_val;
+        break;
+      case PLUGIN_VAR_ENUM | PLUGIN_VAR_THDLOCAL:
+        src = &((thdvar_enum_t *)plugin_var)->def_val;
+        break;
+      case PLUGIN_VAR_SET | PLUGIN_VAR_THDLOCAL:
+        src = &((thdvar_set_t *)plugin_var)->def_val;
+        break;
+      case PLUGIN_VAR_BOOL | PLUGIN_VAR_THDLOCAL:
+        src = &((thdvar_bool_t *)plugin_var)->def_val;
+        break;
+      case PLUGIN_VAR_STR | PLUGIN_VAR_THDLOCAL:
+        src = &((thdvar_str_t *)plugin_var)->def_val;
+        break;
+      case PLUGIN_VAR_DOUBLE | PLUGIN_VAR_THDLOCAL:
+        src = &((thdvar_double_t *)plugin_var)->def_val;
+        break;
+      default:
+        DBUG_ASSERT(0);
+    }
+  }
 
   if ((plugin_var->flags & PLUGIN_VAR_TYPEMASK) == PLUGIN_VAR_STR &&
-      plugin_var->flags & PLUGIN_VAR_MEMALLOC) {
+      plugin_var->flags & PLUGIN_VAR_MEMALLOC)
     rc = plugin_var_memalloc_global_update(thd, plugin_var,
                                            static_cast<char **>(tgt),
                                            *static_cast<char *const *>(src));
-  } else {
+  else
     plugin_var->update(thd, plugin_var, tgt, src);
-    return (thd->is_error() ? 1 : 0);
-  }
 
   return rc;
 }
@@ -489,10 +497,11 @@ ulonglong sys_var_pluginvar::get_max_value() {
   @retval true not valid
   @retval false valid
 */
-bool sys_var_pluginvar::on_check_pluginvar(sys_var *self [[maybe_unused]],
+bool sys_var_pluginvar::on_check_pluginvar(sys_var *self MY_ATTRIBUTE((unused)),
                                            THD *, set_var *var) {
   /* This handler is installed only if NO_DEFAULT is specified */
-  assert(((sys_var_pluginvar *)self)->plugin_var->flags & PLUGIN_VAR_NODEFAULT);
+  DBUG_ASSERT(((sys_var_pluginvar *)self)->plugin_var->flags &
+              PLUGIN_VAR_NODEFAULT);
 
   return (!var->value);
 }
@@ -574,7 +583,7 @@ void sys_var_pluginvar::saved_value_to_string(THD *, set_var *var,
         my_fcvt(var->save_result.double_value, 6, def_val, nullptr);
         return;
       default:
-        assert(0);
+        DBUG_ASSERT(0);
     }
   }
 }
@@ -596,10 +605,10 @@ int check_func_bool(THD *, SYS_VAR *, void *save, st_mysql_value *value) {
       goto err;
   } else {
     if (value->val_int(value, &tmp) < 0) goto err;
-    if (tmp > 1 || tmp < 0) goto err;
+    if (tmp > 1) goto err;
     result = (int)tmp;
   }
-  *(bool *)save = (result != 0);
+  *(bool *)save = result ? true : false;
   return 0;
 err:
   return 1;
@@ -806,8 +815,10 @@ st_bookmark *find_bookmark(const char *plugin, const char *name, int flags) {
   varname[0] = flags & PLUGIN_VAR_TYPEMASK;
 
   const auto it = get_bookmark_hash()->find(std::string(varname, length - 1));
-  if (it == get_bookmark_hash()->end()) return nullptr;
-  return it->second;
+  if (it == get_bookmark_hash()->end())
+    return nullptr;
+  else
+    return it->second;
 }
 
 void plugin_opt_set_limits(struct my_option *options, const SYS_VAR *opt) {
@@ -920,7 +931,7 @@ void plugin_opt_set_limits(struct my_option *options, const SYS_VAR *opt) {
           (intptr)pointer_cast<const thdvar_str_t *>(opt)->def_val;
       break;
     default:
-      assert(0);
+      DBUG_ASSERT(0);
   }
   options->arg_type = REQUIRED_ARG;
   if (opt->flags & PLUGIN_VAR_NOCMDARG) options->arg_type = NO_ARG;
