@@ -1,16 +1,15 @@
-/* Copyright (c) 2007, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2007, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -43,6 +42,7 @@
 #include <algorithm>
 
 #include "decimal.h"
+#include "m_ctype.h"
 #include "m_string.h"
 #include "my_base.h"
 #include "my_compiler.h"
@@ -50,11 +50,9 @@
 #include "my_sqlcommand.h"
 #include "my_sys.h"
 #include "my_systime.h"
-#include "mysql/strings/m_ctype.h"
-#include "nulls.h"
-#include "sql-common/my_decimal.h"
 #include "sql/field.h"
 #include "sql/item.h"
+#include "sql/my_decimal.h"
 #include "sql/protocol.h"
 #include "sql/psi_memory_key.h"
 #include "sql/query_options.h"
@@ -74,21 +72,21 @@ using std::min;
 #define TIME_I_S_DECIMAL_SIZE \
   (TIME_FLOAT_DIGITS * 100) + (TIME_FLOAT_DIGITS - 3)
 
-[[maybe_unused]] static const size_t MAX_QUERY_LENGTH = 300;
+static const size_t MAX_QUERY_LENGTH = 300;
 #define MAX_QUERY_HISTORY 101U
 
 /**
   Connects Information_Schema and Profiling.
 */
-int fill_query_profile_statistics_info(THD *thd [[maybe_unused]],
-                                       Table_ref *tables [[maybe_unused]],
-                                       Item *) {
+int fill_query_profile_statistics_info(
+    THD *thd MY_ATTRIBUTE((unused)), TABLE_LIST *tables MY_ATTRIBUTE((unused)),
+    Item *) {
 #if defined(ENABLED_PROFILING)
   const char *old = thd->lex->sql_command == SQLCOM_SHOW_PROFILE
                         ? "SHOW PROFILE"
                         : "INFORMATION_SCHEMA.PROFILING";
 
-  assert(thd->lex->sql_command != SQLCOM_SHOW_PROFILES);
+  DBUG_ASSERT(thd->lex->sql_command != SQLCOM_SHOW_PROFILES);
 
   push_deprecated_warn(thd, old, "Performance Schema");
   return (thd->profiling->fill_statistics_info(thd, tables));
@@ -148,7 +146,7 @@ int make_profile_table_for_show(THD *thd, ST_SCHEMA_TABLE *schema_table) {
   };
 
   ST_FIELD_INFO *field_info;
-  Name_resolution_context *context = &thd->lex->query_block->context;
+  Name_resolution_context *context = &thd->lex->select_lex->context;
   int i;
 
   for (i = 0; schema_table->fields_info[i].field_name != nullptr; i++) {
@@ -225,7 +223,7 @@ void PROF_MEASUREMENT::set_label(const char *status_arg,
 
   allocated_status_memory = (char *)my_malloc(
       key_memory_PROFILE, sizes[0] + sizes[1] + sizes[2], MYF(0));
-  assert(allocated_status_memory != nullptr);
+  DBUG_ASSERT(allocated_status_memory != nullptr);
 
   cursor = allocated_status_memory;
 
@@ -300,7 +298,7 @@ void QUERY_PROFILE::set_query_source(const char *query_source_arg,
   /* Truncate to avoid DoS attacks. */
   size_t length = min(MAX_QUERY_LENGTH, query_length_arg);
 
-  assert(m_query_source.str == nullptr); /* we don't leak memory */
+  DBUG_ASSERT(m_query_source.str == nullptr); /* we don't leak memory */
   if (query_source_arg != nullptr) {
     m_query_source.str =
         my_strndup(key_memory_PROFILE, query_source_arg, length, MYF(0));
@@ -313,7 +311,7 @@ void QUERY_PROFILE::new_status(const char *status_arg, const char *function_arg,
   PROF_MEASUREMENT *prof;
   DBUG_TRACE;
 
-  assert(status_arg != nullptr);
+  DBUG_ASSERT(status_arg != nullptr);
 
   if ((function_arg != nullptr) && (file_arg != nullptr))
     prof = new PROF_MEASUREMENT(this, status_arg, function_arg,
@@ -382,7 +380,7 @@ void PROFILING::start_new_query(const char *initial_state) {
 
   if (!enabled) return;
 
-  assert(current == nullptr);
+  DBUG_ASSERT(current == nullptr);
   current = new QUERY_PROFILE(this, initial_state);
 }
 
@@ -432,19 +430,19 @@ void PROFILING::finish_current_query() {
 bool PROFILING::show_profiles() {
   DBUG_TRACE;
   QUERY_PROFILE *prof;
-  mem_root_deque<Item *> field_list(current_thd->mem_root);
+  List<Item> field_list;
 
   field_list.push_back(new Item_return_int("Query_ID", 10, MYSQL_TYPE_LONG));
   field_list.push_back(new Item_return_int("Duration", TIME_FLOAT_DIGITS - 1,
                                            MYSQL_TYPE_DOUBLE));
   field_list.push_back(new Item_empty_string("Query", 40));
 
-  if (thd->send_result_metadata(field_list,
+  if (thd->send_result_metadata(&field_list,
                                 Protocol::SEND_NUM_ROWS | Protocol::SEND_EOF))
     return true;
 
-  Query_block *sel = thd->lex->query_block;
-  Query_expression *unit = thd->lex->unit;
+  SELECT_LEX *sel = thd->lex->select_lex;
+  SELECT_LEX_UNIT *unit = thd->lex->unit;
   ha_rows idx = 0;
   Protocol *protocol = thd->get_protocol();
 
@@ -499,7 +497,7 @@ void PROFILING::set_query_source(const char *query_source_arg,
   There are two ways to get to this function:  Selecting from the information
   schema, and a SHOW command.
 */
-int PROFILING::fill_statistics_info(THD *thd_arg, Table_ref *tables) {
+int PROFILING::fill_statistics_info(THD *thd_arg, TABLE_LIST *tables) {
   DBUG_TRACE;
   TABLE *table = tables->table;
   ulonglong row_number = 0;
