@@ -1,7 +1,7 @@
 #ifndef SQL_SELECT_INCLUDED
 #define SQL_SELECT_INCLUDED
 
-/* Copyright (c) 2000, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -27,19 +27,15 @@
   @file sql/sql_select.h
 */
 
-#include <assert.h>
+#include <limits.h>
+#include <stddef.h>
 #include <sys/types.h>
 
-#include <climits>
-#include <string_view>
-
 #include "my_base.h"
-
+#include "my_dbug.h"
 #include "my_inttypes.h"
 #include "my_sqlcommand.h"
 #include "my_table_map.h"
-#include "sql/field.h"  // Copy_field
-#include "sql/handler.h"
 #include "sql/item_cmpfunc.h"  // Item_cond_and
 #include "sql/opt_costmodel.h"
 #include "sql/sql_bitmap.h"
@@ -47,14 +43,14 @@
 #include "sql/sql_const.h"
 #include "sql/sql_opt_exec_shared.h"  // join_type
 
-class Func_ptr;
+class Field;
 class Item;
 class Item_func;
 class JOIN_TAB;
 class KEY;
 class QEP_TAB;
 class Query_result;
-class Query_block;
+class SELECT_LEX;
 class Select_lex_visitor;
 class SJ_TMP_TABLE;
 class Temp_table_param;
@@ -66,11 +62,9 @@ struct MYSQL_LEX_CSTRING;
 struct ORDER;
 struct SJ_TMP_TABLE_TAB;
 struct TABLE;
-class Table_ref;
-class Window;
+struct TABLE_LIST;
 
 typedef ulonglong nested_join_map;
-typedef Mem_root_array<Func_ptr> Func_ptr_array;
 
 class Sql_cmd_select : public Sql_cmd_dml {
  public:
@@ -84,13 +78,11 @@ class Sql_cmd_select : public Sql_cmd_dml {
 
   bool accept(THD *thd, Select_lex_visitor *visitor) override;
 
-  const MYSQL_LEX_CSTRING *eligible_secondary_storage_engine(
-      THD *thd) const override;
+  const MYSQL_LEX_CSTRING *eligible_secondary_storage_engine() const override;
 
  protected:
-  bool may_use_cursor() const override { return true; }
   bool precheck(THD *thd) override;
-  bool check_privileges(THD *thd) override;
+
   bool prepare_inner(THD *thd) override;
 };
 
@@ -192,7 +184,7 @@ class Key_use {
         fanout(0.0),
         read_cost(0.0) {}
 
-  Key_use(Table_ref *table_ref_arg, Item *val_arg, table_map used_tables_arg,
+  Key_use(TABLE_LIST *table_ref_arg, Item *val_arg, table_map used_tables_arg,
           uint key_arg, uint keypart_arg, uint optimize_arg,
           key_part_map keypart_map_arg, ha_rows ref_table_rows_arg,
           bool null_rejecting_arg, bool *cond_guard_arg, uint sj_pred_no_arg)
@@ -211,7 +203,8 @@ class Key_use {
         fanout(0.0),
         read_cost(0.0) {}
 
-  Table_ref *table_ref;  ///< table owning the index
+  TABLE_LIST *table_ref;  ///< table owning the index
+
   /**
     Value used for lookup into @c key. It may be an Item_field, a
     constant or any other expression. If @c val contains a field from
@@ -219,14 +212,11 @@ class Key_use {
     the field(s) in @c val should be before @c table in the join plan.
   */
   Item *val;
+
   /**
     All tables used in @c val, that is all tables that provide bindings
     for the expression @c val. These tables must be in the plan before
     executing the equi-join described by a Key_use.
-    For all expressions except for the MATCH function, this is the same
-    as val->used_tables().
-    For the MATCH function, val is the actual MATCH function, and used_tables
-    is the set of tables used in the AGAINST argument.
   */
   table_map used_tables;
   uint key;                  ///< number of index
@@ -236,7 +226,7 @@ class Key_use {
   ha_rows ref_table_rows;    ///< Estimate of how many rows for a key value
   /**
     If true, the comparison this value was created from will not be
-    satisfied if val has NULL 'value' (unless KEY_OPTIMIZE_REF_OR_NULL is set).
+    satisfied if val has NULL 'value'.
     Not used if the index is fulltext (such index cannot be used for
     equalities).
   */
@@ -303,8 +293,7 @@ class Key_use {
 };
 
 /// @returns join type according to quick select type used
-///   (which must be a form of range scan, or asserts will happen)
-join_type calc_join_type(AccessPath *path);
+join_type calc_join_type(int quick_type);
 
 class JOIN;
 
@@ -575,17 +564,6 @@ struct POSITION {
     }
     prefix_rowcount *= filter_effect;
   }
-
-  void set_suffix_lateral_deps(table_map deps) { m_suffix_lateral_deps = deps; }
-
-  table_map get_suffix_lateral_deps() const { return m_suffix_lateral_deps; }
-
- private:
-  /**
-     The lateral dependendencies of 'table' and all subsequent JOIN_TABs
-     in the join plan.
-   */
-  table_map m_suffix_lateral_deps;
 };
 
 /**
@@ -604,15 +582,15 @@ class JOIN_TAB : public QEP_shared_owner {
 
   void set_table(TABLE *t);
 
-  /// Sets the pointer to the join condition of Table_ref
-  void init_join_cond_ref(Table_ref *tl);
+  /// Sets the pointer to the join condition of TABLE_LIST
+  void init_join_cond_ref(TABLE_LIST *tl);
 
   /// @returns join condition
   Item *join_cond() const { return *m_join_cond_ref; }
 
   /**
      Sets join condition
-     @note this also changes Table_ref::m_join_cond.
+     @note this also changes TABLE_LIST::m_join_cond.
   */
   void set_join_cond(Item *cond) { *m_join_cond_ref = cond; }
 
@@ -630,7 +608,7 @@ class JOIN_TAB : public QEP_shared_owner {
   Key_use *keyuse() const { return m_keyuse; }
   void set_keyuse(Key_use *k) { m_keyuse = k; }
 
-  Table_ref *table_ref; /**< points to table reference               */
+  TABLE_LIST *table_ref; /**< points to table reference               */
 
  private:
   Key_use *m_keyuse; /**< pointer to first used key               */
@@ -641,10 +619,10 @@ class JOIN_TAB : public QEP_shared_owner {
     - if this is a table with position==NULL (e.g. internal sort/group
       temporary table), pointer is NULL
 
-    - otherwise, pointer is the address of some Table_ref::m_join_cond.
-      Thus, the pointee is the same as Table_ref::m_join_cond (changing
-    one changes the other; thus, optimizations made on the second are reflected
-      in Query_block::print_table_array() which uses the first one).
+    - otherwise, pointer is the address of some TABLE_LIST::m_join_cond.
+      Thus, the pointee is the same as TABLE_LIST::m_join_cond (changing one
+      changes the other; thus, optimizations made on the second are reflected
+      in SELECT_LEX::print_table_array() which uses the first one).
   */
   Item **m_join_cond_ref;
 
@@ -728,7 +706,7 @@ class JOIN_TAB : public QEP_shared_owner {
     It is the closest semijoin or antijoin nest.
     This variable holds the result of table pullout.
   */
-  Table_ref *emb_sj_nest;
+  TABLE_LIST *emb_sj_nest;
 
   /* NestedOuterJoins: Bitmap of nested joins this table is part of */
   nested_join_map embedding_map;
@@ -775,13 +753,13 @@ inline JOIN_TAB::JOIN_TAB()
       reversed_access(false) {}
 
 /* Extern functions in sql_select.cc */
-void count_field_types(const Query_block *query_block, Temp_table_param *param,
-                       const mem_root_deque<Item *> &fields,
-                       bool reset_with_sum_func, bool save_sum_fields);
+void count_field_types(SELECT_LEX *select_lex, Temp_table_param *param,
+                       List<Item> &fields, bool reset_with_sum_func,
+                       bool save_sum_fields);
 uint find_shortest_key(TABLE *table, const Key_map *usable_keys);
 
 /* functions from opt_sum.cc */
-bool is_simple_predicate(Item_func *func_item, Item **args, bool *inv_order);
+bool simple_pred(Item_func *func_item, Item **args, bool *inv_order);
 
 enum aggregate_evaluated {
   AGGR_COMPLETE,  // All aggregates were evaluated
@@ -790,9 +768,9 @@ enum aggregate_evaluated {
   AGGR_EMPTY      // Source tables empty, aggregates are NULL or 0 (for COUNT)
 };
 
-bool optimize_aggregated_query(THD *thd, Query_block *select,
-                               const mem_root_deque<Item *> &all_fields,
-                               Item *conds, aggregate_evaluated *decision);
+bool optimize_aggregated_query(THD *thd, SELECT_LEX *select,
+                               List<Item> &all_fields, Item *conds,
+                               aggregate_evaluated *decision);
 
 /* from sql_delete.cc, used by opt_range.cc */
 extern "C" int refpos_order_cmp(const void *arg, const void *a, const void *b);
@@ -800,30 +778,15 @@ extern "C" int refpos_order_cmp(const void *arg, const void *a, const void *b);
 /// The name of store_key instances that represent constant items.
 constexpr const char *STORE_KEY_CONST_NAME = "const";
 
-/// Check privileges for all columns referenced from join expression
-bool check_privileges_for_join(THD *thd, mem_root_deque<Table_ref *> *tables);
-
-/// Check privileges for all columns referenced from an expression list
-bool check_privileges_for_list(THD *thd, const mem_root_deque<Item *> &items,
-                               ulong privileges);
-
 /** class to copying an field/item to a key struct */
 
 class store_key {
  public:
   bool null_key{false}; /* true <=> the value of the key has a null part */
   enum store_key_result { STORE_KEY_OK, STORE_KEY_FATAL, STORE_KEY_CONV };
-  store_key(THD *thd, Field *to_field_arg, uchar *ptr, uchar *null_ptr_arg,
-            uint length, Item *item_arg);
+  store_key(THD *thd, Field *field_arg, uchar *ptr, uchar *null, uint length);
   virtual ~store_key() = default;
-  virtual const char *name() const {
-    // Compatible with legacy behavior.
-    if (item->type() == Item::FIELD_ITEM) {
-      return item->full_name();
-    } else {
-      return "func";
-    }
-  }
+  virtual const char *name() const = 0;
 
   /**
     @brief sets ignore truncation warnings mode and calls the real copy method
@@ -835,25 +798,55 @@ class store_key {
 
  protected:
   Field *to_field;  // Store data here
+
+  virtual enum store_key_result copy_inner() = 0;
+};
+
+class store_key_field final : public store_key {
+  Copy_field m_copy_field;
+  const char *m_field_name;
+
+ public:
+  store_key_field(THD *thd, Field *to_field_arg, uchar *ptr,
+                  uchar *null_ptr_arg, uint length, Field *from_field,
+                  const char *name_arg);
+
+  const char *name() const override { return m_field_name; }
+
+  // Change the source field to be another field. Used only by
+  // CreateBKAIterator, when rewriting multi-equalities used in ref access.
+  void replace_from_field(Field *from_field);
+
+ protected:
+  enum store_key_result copy_inner() override;
+};
+class store_key_item : public store_key {
+ protected:
   Item *item;
 
-  virtual enum store_key_result copy_inner();
+ public:
+  store_key_item(THD *thd, Field *to_field_arg, uchar *ptr, uchar *null_ptr_arg,
+                 uint length, Item *item_arg);
+  const char *name() const override { return "func"; }
+
+ protected:
+  enum store_key_result copy_inner() override;
 };
 
 /*
   Class used for unique constraint implementation by subselect_hash_sj_engine.
-  It uses store_key implementation to do actual copying, but after
+  It uses store_key_item implementation to do actual copying, but after
   that, copy_inner calculates hash of each key part for unique constraint.
 */
 
-class store_key_hash_item final : public store_key {
+class store_key_hash_item final : public store_key_item {
   ulonglong *hash;
 
  public:
   store_key_hash_item(THD *thd, Field *to_field_arg, uchar *ptr,
                       uchar *null_ptr_arg, uint length, Item *item_arg,
                       ulonglong *hash_arg)
-      : store_key(thd, to_field_arg, ptr, null_ptr_arg, length, item_arg),
+      : store_key_item(thd, to_field_arg, ptr, null_ptr_arg, length, item_arg),
         hash(hash_arg) {}
   const char *name() const override { return "func"; }
 
@@ -861,19 +854,25 @@ class store_key_hash_item final : public store_key {
   enum store_key_result copy_inner() override;
 };
 
+bool error_if_full_join(JOIN *join);
+bool handle_query(THD *thd, LEX *lex, Query_result *result,
+                  ulonglong added_options, ulonglong removed_options);
+
 // Statement timeout function(s)
 bool set_statement_timer(THD *thd);
 void reset_statement_timer(THD *thd);
 
-void free_underlaid_joins(Query_block *select);
+void free_underlaid_joins(THD *thd, SELECT_LEX *select);
 
-void calc_used_field_length(TABLE *table, bool needs_rowid,
-                            uint *p_used_fieldlength);
+void calc_used_field_length(TABLE *table, bool needs_rowid, uint *p_used_fields,
+                            uint *p_used_fieldlength, uint *p_used_blobs,
+                            bool *p_used_null_fields,
+                            bool *p_used_uneven_bit_fields);
 
 ORDER *simple_remove_const(ORDER *order, Item *where);
-bool check_field_is_const(Item *cond, const Item *order_item,
-                          const Field *order_field = nullptr,
-                          Item **const_item = nullptr);
+bool const_expression_in_where(Item *cond, Item *comp_item,
+                               const Field *comp_field = nullptr,
+                               Item **const_item = nullptr);
 bool test_if_subpart(ORDER *a, ORDER *b);
 void calc_group_buffer(JOIN *join, ORDER *group);
 bool make_join_readinfo(JOIN *join, uint no_jbuf_after);
@@ -893,18 +892,18 @@ bool and_conditions(Item **e1, Item *e2);
   must not be a null pointer.
 
   @param cond  the first argument to the new AND condition
-  @param item  the second argument to the new AND condition
+  @param item  the second argument to the new AND condtion
 
   @return the new AND item
 */
 static inline Item *and_items(Item *cond, Item *item) {
-  assert(item != nullptr);
+  DBUG_ASSERT(item != nullptr);
   return (cond ? (new Item_cond_and(cond, item)) : item);
 }
 
 /// A variant of the above, guaranteed to return Item_bool_func.
 static inline Item_bool_func *and_items(Item *cond, Item_bool_func *item) {
-  assert(item != nullptr);
+  DBUG_ASSERT(item != nullptr);
   return (cond ? (new Item_cond_and(cond, item)) : item);
 }
 
@@ -912,9 +911,8 @@ uint actual_key_parts(const KEY *key_info);
 
 class ORDER_with_src;
 
-uint get_index_for_order(ORDER_with_src *order, TABLE *table, ha_rows limit,
-                         AccessPath *range_scan, bool *need_sort,
-                         bool *reverse);
+uint get_index_for_order(ORDER_with_src *order, QEP_TAB *tab, ha_rows limit,
+                         bool *need_sort, bool *reverse);
 int test_if_order_by_key(ORDER_with_src *order, TABLE *table, uint idx,
                          uint *used_key_parts, bool *skip_quick);
 bool test_if_cheaper_ordering(const JOIN_TAB *tab, ORDER_with_src *order,
@@ -925,8 +923,7 @@ bool test_if_cheaper_ordering(const JOIN_TAB *tab, ORDER_with_src *order,
                               uint *saved_best_key_parts = nullptr);
 /**
   Calculate properties of ref key: key length, number of used key parts,
-  dependency map, possibility of null. After calling this function
-  thd::is_error() needs to be checked, as it can set an error.
+  dependency map, possibility of null.
 
   @param keyuse               Array of keys to consider
   @param tab                  join_tab to calculate ref parameters for
@@ -945,25 +942,6 @@ void calc_length_and_keyparts(Key_use *keyuse, JOIN_TAB *tab, const uint key,
                               uint *length_out, uint *keyparts_out,
                               table_map *dep_map, bool *maybe_null);
 
-/**
-  Initialize the given TABLE_REF; setting basic fields and allocating memory
-  for arrays. Call init_ref_part() for each keypart (index field) that is to
-  take part in the ref lookup.
- */
-bool init_ref(THD *thd, unsigned keyparts, unsigned length, unsigned keyno,
-              Index_lookup *ref);
-
-/**
-  Initialize a given keypart in the table ref. In particular, sets up the
-  right function pointer to copy the value from “val” into the ref at
-  execution time (or copies the value right now, if it is constant).
- */
-bool init_ref_part(THD *thd, unsigned part_no, Item *val, bool *cond_guard,
-                   bool null_rejecting, table_map const_tables,
-                   table_map used_tables, bool nullable,
-                   const KEY_PART_INFO *key_part_info, uchar *key_buff,
-                   Index_lookup *ref);
-
 /**
   Set up the support structures (NULL bits, row offsets, etc.) for a semijoin
   duplicate weedout table. The object is allocated on the given THD's MEM_ROOT.
@@ -978,122 +956,4 @@ SJ_TMP_TABLE *create_sj_tmp_table(THD *thd, JOIN *join,
                                   SJ_TMP_TABLE_TAB *first_tab,
                                   SJ_TMP_TABLE_TAB *last_tab);
 
-/**
-  Returns key flags depending on
-  OPTIMIZER_SWITCH_USE_INDEX_EXTENSIONS flag.
-
-  @param  key_info  pointer to KEY structure
-
-  @return key flags.
- */
-uint actual_key_flags(const KEY *key_info);
-
-/**
-  Check if equality can be used to remove sub-clause of GROUP BY/ORDER BY
-
-  @param func   comparison operator (= or <=>)
-  @param v      variable comparison operand (validated to be equal to
-                                             ordering expression)
-  @param c      other comparison operand (likely to be a constant)
-
-  @returns whether equality determines uniqueness
-
-    Checks if an equality predicate can be used to remove a GROUP BY/ORDER BY
-    sub-clause when it is known to be true for exactly one distinct value
-    (e.g. "expr" == "const").
-    Arguments must be of the same type because e.g. "string_field" = "int_const"
-     may match more than one distinct value from the column.
- */
-bool equality_determines_uniqueness(const Item_func_comparison *func,
-                                    const Item *v, const Item *c);
-
-/**
-  Check whether equality between two items is exact, ie., there are no implicit
-  casts involved. This is especially important for GROUP BY/ORDER BY, as it
-  means they can be treated interchangeably. The primary difference between this
-  and equality_determines_uniqueness() is that item2 does not need to be
-  a constant (which makes it stricter in other aspects).
- */
-bool equality_has_no_implicit_casts(const Item_func_comparison *func,
-                                    const Item *item1, const Item *item2);
-
-bool CreateFramebufferTable(
-    THD *thd, const Temp_table_param &tmp_table_param,
-    const Query_block &query_block, const mem_root_deque<Item *> &source_fields,
-    const mem_root_deque<Item *> &window_output_fields,
-    Func_ptr_array *mapping_from_source_to_window_output, Window *window);
-
-/**
-  Validates a query that uses the secondary engine
-
-  No validations are done if query has not been prepared against the secondary
-  engine.
-
-  @param lex Parse tree descriptor.
-
-  @return True if error, false otherwise.
-*/
-bool validate_use_secondary_engine(const LEX *lex);
-
-/**
-  Perform query optimizations that are specific to a secondary storage
-  engine.
-
-  @param thd      the current session
-  @return true on error, false on success
-*/
-bool optimize_secondary_engine(THD *thd);
-
-/**
-  Calculates the cost of executing a statement, including all its
-  subqueries and stores it in thd->m_current_query_cost.
-
-  @param lex the statement
-*/
-void accumulate_statement_cost(const LEX *lex);
-
-/**
-  Returns secondary_engine handler for the statement.
-  If none exist, nullptr is returned.
-
-  @param lex the statement
-*/
-const handlerton *get_secondary_engine_handlerton(const LEX *lex);
-
-/**
-  Checks if any of the tables referenced belong to an external engine.
-  If an external table is found, return true, false otherwise.
-
-  @param lex the statement
-*/
-bool has_external_table(const LEX *lex);
-
-/**
-  Sets the reason of failure for the statement to the external engine.
-
-  @param lex the statement
-  @param reason the reason of failure
-*/
-void set_external_engine_fail_reason(const LEX *lex, const char *reason);
-
-/**
-  Notify plugins about an executed SELECT statement.
-
-  @param thd the current session
-  @param cmd command to be notified about
-*/
-void notify_plugins_after_select(THD *thd, const Sql_cmd *cmd);
-
-std::string_view get_secondary_engine_fail_reason(const LEX *lex);
-
-void set_fail_reason_and_raise_error(const LEX *lex, std::string_view reason);
-
-const MYSQL_LEX_CSTRING *get_eligible_secondary_engine_from(const LEX *lex);
-
-std::string_view find_secondary_engine_fail_reason(const LEX *lex);
-
-void find_and_set_offload_fail_reason(const LEX *lex);
-
-bool reads_not_secondary_columns(const LEX *lex);
-
 #endif /* SQL_SELECT_INCLUDED */
