@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -60,28 +60,27 @@
 #include "my_macros.h"
 #include "my_psi_config.h"
 #include "my_sys.h"
-#include "mysql/components/services/bits/mysql_rwlock_bits.h"
-#include "mysql/components/services/bits/psi_bits.h"
-#include "mysql/components/services/bits/psi_memory_bits.h"
-#include "mysql/components/services/bits/psi_rwlock_bits.h"
 #include "mysql/components/services/log_builtins.h"
+#include "mysql/components/services/mysql_rwlock_bits.h"
+#include "mysql/components/services/psi_memory_bits.h"
+#include "mysql/components/services/psi_rwlock_bits.h"
 #include "mysql/psi/mysql_memory.h"
 #include "mysql/psi/mysql_mutex.h"
 #include "mysql/psi/mysql_rwlock.h"
+#include "mysql/psi/psi_base.h"
 #include "mysqld_error.h"
 #include "sql/auth/auth_acls.h"
 #include "sql/auth/auth_common.h"
 #include "sql/field.h"
 #include "sql/handler.h"
-#include "sql/iterators/row_iterator.h"
-#include "sql/psi_memory_key.h"   // key_memory_servers
+#include "sql/psi_memory_key.h"  // key_memory_servers
+#include "sql/records.h"         // init_read_record
+#include "sql/row_iterator.h"
 #include "sql/sql_backup_lock.h"  // acquire_shared_backup_lock
 #include "sql/sql_base.h"         // close_mysql_tables
 #include "sql/sql_class.h"
 #include "sql/sql_const.h"
 #include "sql/sql_error.h"
-#include "sql/sql_executor.h"            // init_read_record
-#include "sql/sql_system_table_check.h"  // System_table_intact
 #include "sql/system_variables.h"
 #include "sql/table.h"
 #include "sql/thd_raii.h"
@@ -113,36 +112,6 @@ enum enum_servers_table_field {
   SERVERS_FIELD_OWNER
 };
 
-// mysql.servers table definition.
-static const int MYSQL_SERVERS_TABLE_FIELD_COUNT = 9;
-static const TABLE_FIELD_TYPE
-    mysql_servers_table_fields[MYSQL_SERVERS_TABLE_FIELD_COUNT] = {
-        {{STRING_WITH_LEN("Server_name")},
-         {STRING_WITH_LEN("char(64)")},
-         {nullptr, 0}},
-        {{STRING_WITH_LEN("Host")},
-         {STRING_WITH_LEN("char(255)")},
-         {STRING_WITH_LEN("ascii")}},
-        {{STRING_WITH_LEN("Db")}, {STRING_WITH_LEN("char(64)")}, {nullptr, 0}},
-        {{STRING_WITH_LEN("Username")},
-         {STRING_WITH_LEN("char(64)")},
-         {nullptr, 0}},
-        {{STRING_WITH_LEN("Password")},
-         {STRING_WITH_LEN("char(64)")},
-         {nullptr, 0}},
-        {{STRING_WITH_LEN("Port")}, {STRING_WITH_LEN("int")}, {nullptr, 0}},
-        {{STRING_WITH_LEN("Socket")},
-         {STRING_WITH_LEN("char(64)")},
-         {nullptr, 0}},
-        {{STRING_WITH_LEN("Wrapper")},
-         {STRING_WITH_LEN("char(64)")},
-         {nullptr, 0}},
-        {{STRING_WITH_LEN("Owner")},
-         {STRING_WITH_LEN("char(64)")},
-         {nullptr, 0}}};
-static const TABLE_FIELD_DEF mysql_servers_table_def = {
-    MYSQL_SERVERS_TABLE_FIELD_COUNT, mysql_servers_table_fields};
-
 static bool get_server_from_table_to_cache(TABLE *table);
 
 #ifdef HAVE_PSI_INTERFACE
@@ -154,7 +123,7 @@ static PSI_rwlock_info all_servers_cache_rwlocks[] = {
 
 static PSI_memory_info all_servers_cache_memory[] = {
     {&key_memory_servers, "servers_cache", PSI_FLAG_ONLY_GLOBAL_STAT, 0,
-     "Cache infrastructure and mem root for servers cache."}};
+     PSI_DOCUMENT_ME}};
 
 static void init_servers_cache_psi_keys(void) {
   const char *category = "sql";
@@ -175,8 +144,8 @@ static void init_servers_cache_psi_keys(void) {
 
   SYNOPSIS
     servers_init()
-      thd thread for reading the servers table and initializing necessary
-          structures.
+      dont_read_server_table  true if we want to skip loading data from
+                            server table and disable privilege checking.
 
   NOTES
     This function is mostly responsible for preparatory steps, main work
@@ -187,7 +156,8 @@ static void init_servers_cache_psi_keys(void) {
     1	Could not initialize servers
 */
 
-bool servers_init(THD *thd) {
+bool servers_init(bool dont_read_servers_table) {
+  THD *thd;
   bool return_val = false;
   DBUG_TRACE;
 
@@ -204,24 +174,26 @@ bool servers_init(THD *thd) {
       system_charset_info, key_memory_servers);
 
   /* Initialize the mem root for data */
-  init_sql_alloc(key_memory_servers, &mem, ACL_ALLOC_BLOCK_SIZE);
+  init_sql_alloc(key_memory_servers, &mem, ACL_ALLOC_BLOCK_SIZE, 0);
 
-  if (thd == nullptr) {
-    /* To be able to run this from boot, we allocate a temporary THD. */
-    thd = new (std::nothrow) THD;
-    if (thd == nullptr) return true;
-    thd->thread_stack = (char *)&thd;
-    thd->store_globals();
-    /*
-      It is safe to call servers_reload() since servers_* arrays and hashes
-      which will be freed there are global static objects and thus are
-      initialized by zeros at startup.
-    */
-    return_val = servers_reload(thd);
-    delete thd;
-    return return_val;
-  }
-  return servers_reload(thd);
+  if (dont_read_servers_table) goto end;
+
+  /*
+    To be able to run this from boot, we allocate a temporary THD
+  */
+  if (!(thd = new THD)) return true;
+  thd->thread_stack = (char *)&thd;
+  thd->store_globals();
+  /*
+    It is safe to call servers_reload() since servers_* arrays and hashes which
+    will be freed there are global static objects and thus are initialized
+    by zeros at startup.
+  */
+  return_val = servers_reload(thd);
+  delete thd;
+
+end:
+  return return_val;
 }
 
 /*
@@ -246,12 +218,12 @@ static bool servers_load(THD *thd, TABLE *table) {
   if (servers_cache != nullptr) {
     servers_cache->clear();
   }
-  mem.Clear();
-  init_sql_alloc(key_memory_servers, &mem, ACL_ALLOC_BLOCK_SIZE);
+  free_root(&mem, MYF(0));
+  init_sql_alloc(key_memory_servers, &mem, ACL_ALLOC_BLOCK_SIZE, 0);
 
   unique_ptr_destroy_only<RowIterator> iterator =
-      init_table_iterator(thd, table, /*ignore_not_found_rows=*/false,
-                          /*count_examined_rows=*/false);
+      init_table_iterator(thd, table, nullptr, false,
+                          /*ignore_not_found_rows=*/false);
   if (iterator == nullptr) return true;
 
   while (!(iterator->Read())) {
@@ -263,7 +235,7 @@ static bool servers_load(THD *thd, TABLE *table) {
 
 /*
   Forget current servers cache and read new servers
-  from the connection table.
+  from the conneciton table.
 
   SYNOPSIS
     servers_reload()
@@ -287,7 +259,7 @@ bool servers_reload(THD *thd) {
   DBUG_PRINT("info", ("locking servers_cache"));
   mysql_rwlock_wrlock(&THR_LOCK_servers);
 
-  Table_ref tables("mysql", "servers", TL_READ);
+  TABLE_LIST tables("mysql", "servers", TL_READ);
   if (open_trans_system_tables_for_read(thd, &tables)) {
     /*
       Execution might have been interrupted; only print the error message
@@ -390,10 +362,10 @@ static bool get_server_from_table_to_cache(TABLE *table) {
 static bool close_cached_connection_tables(THD *thd,
                                            const char *connection_string,
                                            size_t connection_length) {
-  Table_ref tmp, *tables = nullptr;
+  TABLE_LIST tmp, *tables = nullptr;
   bool result = false;
   DBUG_TRACE;
-  assert(thd);
+  DBUG_ASSERT(thd);
 
   mysql_mutex_lock(&LOCK_open);
 
@@ -449,7 +421,7 @@ static bool close_cached_connection_tables(THD *thd,
     tmp.table_name = share->table_name.str;
     tmp.next_local = tables;
 
-    tables = new (thd->mem_root) Table_ref(tmp);
+    tables = new (thd->mem_root) TABLE_LIST(tmp);
   }
   mysql_mutex_unlock(&LOCK_open);
 
@@ -644,34 +616,10 @@ bool Sql_cmd_common_server::check_and_open_table(THD *thd) {
       acquire_shared_backup_lock(thd, thd->variables.lock_wait_timeout))
     return true;
 
-  Table_ref tables("mysql", "servers", TL_WRITE);
+  TABLE_LIST tables("mysql", "servers", TL_WRITE);
 
   table = open_ltable(thd, &tables, TL_WRITE, MYSQL_LOCK_IGNORE_TIMEOUT);
-  if (table == nullptr) return true;
-
-  /*
-    System table mysql.servers is supported by only InnoDB engine. Changing
-    table's engine is not allowed. But to support logical upgrade creating
-    system table is allowed in MyISAM engine. CREATE, ALTER and DROP SERVER
-    operations are not allowed in this case.
-  */
-  if ((table->file->ht->is_supported_system_table != nullptr) &&
-      !table->file->ht->is_supported_system_table(tables.db, tables.table_name,
-                                                  true)) {
-    my_error(ER_UNSUPPORTED_ENGINE, MYF(0),
-             ha_resolve_storage_engine_name(table->file->ht), tables.db,
-             tables.table_name);
-    return true;
-  }
-
-  /*
-    CREATE, ALTER and DROP SERVER operations are *not* allowed if table
-    structure is changed.
-  */
-  System_table_intact table_intact(thd);
-  if (table_intact.check(thd, table, &mysql_servers_table_def)) return true;
-
-  return false;
+  return (table == nullptr);
 }
 
 bool Sql_cmd_create_server::execute(THD *thd) {
@@ -705,7 +653,7 @@ bool Sql_cmd_create_server::execute(THD *thd) {
 
     /* read index until record is that specified in server_name */
     error = table->file->ha_index_read_idx_map(
-        table->record[0], 0, table->field[SERVERS_FIELD_NAME]->field_ptr(),
+        table->record[0], 0, table->field[SERVERS_FIELD_NAME]->ptr,
         HA_WHOLE_KEY, HA_READ_KEY_EXACT);
 
     if (!error) {
@@ -764,7 +712,7 @@ bool Sql_cmd_alter_server::execute(THD *thd) {
 
   int error;
   {
-    Disable_binlog_guard binlog_guard(thd);
+    Disable_binlog_guard binlog_guard(table->in_use);
     table->use_all_columns();
 
     /* set the field that's the PK to the value we're looking for */
@@ -773,7 +721,7 @@ bool Sql_cmd_alter_server::execute(THD *thd) {
         m_server_options->m_server_name.length, system_charset_info);
 
     error = table->file->ha_index_read_idx_map(
-        table->record[0], 0, table->field[SERVERS_FIELD_NAME]->field_ptr(),
+        table->record[0], 0, table->field[SERVERS_FIELD_NAME]->ptr,
         ~(longlong)0, HA_READ_KEY_EXACT);
     if (error) {
       if (error != HA_ERR_KEY_NOT_FOUND && error != HA_ERR_END_OF_FILE)
@@ -827,7 +775,7 @@ bool Sql_cmd_drop_server::execute(THD *thd) {
   int error;
   mysql_rwlock_wrlock(&THR_LOCK_servers);
   {
-    Disable_binlog_guard binlog_guard(thd);
+    Disable_binlog_guard binlog_guard(table->in_use);
     table->use_all_columns();
 
     /* set the field that's the PK to the value we're looking for */
@@ -835,7 +783,7 @@ bool Sql_cmd_drop_server::execute(THD *thd) {
         m_server_name.str, m_server_name.length, system_charset_info);
 
     error = table->file->ha_index_read_idx_map(
-        table->record[0], 0, table->field[SERVERS_FIELD_NAME]->field_ptr(),
+        table->record[0], 0, table->field[SERVERS_FIELD_NAME]->ptr,
         HA_WHOLE_KEY, HA_READ_KEY_EXACT);
     if (error) {
       if (error != HA_ERR_KEY_NOT_FOUND && error != HA_ERR_END_OF_FILE)
@@ -881,12 +829,12 @@ void servers_free(bool end) {
   DBUG_TRACE;
   if (servers_cache == nullptr) return;
   if (!end) {
-    mem.ClearForReuse();
+    free_root(&mem, MYF(MY_MARK_BLOCKS_FREE));
     servers_cache->clear();
     return;
   }
   mysql_rwlock_destroy(&THR_LOCK_servers);
-  mem.Clear();
+  free_root(&mem, MYF(0));
   delete servers_cache;
   servers_cache = nullptr;
 }
@@ -938,38 +886,29 @@ static FOREIGN_SERVER *clone_server(MEM_ROOT *mem, const FOREIGN_SERVER *server,
 FOREIGN_SERVER *get_server_by_name(MEM_ROOT *mem, const char *server_name,
                                    FOREIGN_SERVER *buff) {
   size_t server_name_length;
-  FOREIGN_SERVER *server = nullptr;
+  FOREIGN_SERVER *server;
   DBUG_TRACE;
-
-  DBUG_EXECUTE_IF("bug33962357_simulate_null_server",
-                  { server_name = nullptr; });
-  if (!server_name || !strlen(server_name)) {
-    DBUG_PRINT("info", ("server_name not defined!"));
-    return nullptr;
-  }
   DBUG_PRINT("info", ("server_name %s", server_name));
 
   server_name_length = strlen(server_name);
-  const std::string str_server(server_name, server_name_length);
+
+  if (!server_name || !strlen(server_name)) {
+    DBUG_PRINT("info", ("server_name not defined!"));
+    return (FOREIGN_SERVER *)nullptr;
+  }
 
   DBUG_PRINT("info", ("locking servers_cache"));
   mysql_rwlock_rdlock(&THR_LOCK_servers);
-
-  collation_unordered_map<std::string, FOREIGN_SERVER *> *cache = servers_cache;
-  DBUG_EXECUTE_IF("bug33962357_simulate_null_cache", { cache = nullptr; });
-
-  if (!cache) {
-    DBUG_PRINT("error", ("server_cache not initialized!"));
-  } else {
-    const auto it = cache->find(str_server);
-    if (it == cache->end()) {
-      DBUG_PRINT("info", ("server_name %s length %u not found!", server_name,
-                          (unsigned)server_name_length));
-    }
-    /* otherwise, make copy of server */
-    else
-      server = clone_server(mem, it->second, buff);
+  const auto it =
+      servers_cache->find(std::string(server_name, server_name_length));
+  if (it == servers_cache->end()) {
+    DBUG_PRINT("info", ("server_name %s length %u not found!", server_name,
+                        (unsigned)server_name_length));
+    server = (FOREIGN_SERVER *)nullptr;
   }
+  /* otherwise, make copy of server */
+  else
+    server = clone_server(mem, it->second, buff);
 
   DBUG_PRINT("info", ("unlocking servers_cache"));
   mysql_rwlock_unlock(&THR_LOCK_servers);
