@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -33,28 +33,26 @@
 
 #include "sql/sql_time.h"
 
-#include <assert.h>
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
 
 #include "decimal.h"
+#include "m_ctype.h"
+#include "m_string.h"
 #include "my_compiler.h"
-
+#include "my_dbug.h"
 #include "my_macros.h"
-#include "mysql/strings/m_ctype.h"
 #include "mysql_com.h"
 #include "mysqld_error.h"
-#include "nulls.h"
-#include "sql-common/my_decimal.h"
 #include "sql/current_thd.h"
 #include "sql/derror.h"
 #include "sql/field.h"
+#include "sql/my_decimal.h"
 #include "sql/sql_class.h"  // THD, MODE_STRICT_ALL_TABLES, MODE_STRICT_TRANS_TABLES
 #include "sql/sql_const.h"
 #include "sql/system_variables.h"
 #include "sql/tztime.h"  // struct Time_zone
-#include "string_with_len.h"
 
 /**
   Name description of interval names used in statements.
@@ -128,7 +126,7 @@ bool str_to_time(const CHARSET_INFO *cs, const char *str, size_t length,
                  MYSQL_TIME *l_time, my_time_flags_t flags,
                  MYSQL_TIME_STATUS *status) {
   char cnv[MAX_TIME_FULL_WIDTH + 3];  // +3 for nanoseconds (for rounding)
-  if (!my_charset_is_ascii_based(cs)) {
+  if ((cs->state & MY_CS_NONASCII) != 0) {
     length = to_ascii(cs, str, length, cnv, sizeof(cnv));
     str = cnv;
   }
@@ -149,7 +147,7 @@ bool str_to_datetime(const CHARSET_INFO *cs, const char *str, size_t length,
                      MYSQL_TIME *l_time, my_time_flags_t flags,
                      MYSQL_TIME_STATUS *status) {
   char cnv[MAX_DATETIME_FULL_WIDTH + 3];  // +3 for nanoseconds (for rounding)
-  if (!my_charset_is_ascii_based(cs)) {
+  if ((cs->state & MY_CS_NONASCII) != 0) {
     length = to_ascii(cs, str, length, cnv, sizeof(cnv));
     str = cnv;
   }
@@ -184,12 +182,11 @@ bool str_to_datetime_with_warn(String *str, MYSQL_TIME *l_time,
                                      ErrConvString(str), l_time->time_type,
                                      NullS))
       return true;
-    if (ret_val) status.squelch_deprecation();
   }
-  check_deprecated_datetime_format(current_thd, str->charset(), status);
 
-  if (ret_val) return true;
-  return convert_time_zone_displacement(thd->time_zone(), l_time);
+  adjust_time_zone_displacement(thd->time_zone(), l_time);
+
+  return ret_val;
 }
 
 /**
@@ -211,8 +208,7 @@ static bool lldiv_t_to_datetime(lldiv_t lld, MYSQL_TIME *ltime,
     if (!*warnings) /* Neither sets warnings in case of ZERO DATE */
       *warnings |= MYSQL_TIME_WARN_TRUNCATED;
     return true;
-  }
-  if (ltime->time_type == MYSQL_TIMESTAMP_DATE) {
+  } else if (ltime->time_type == MYSQL_TIMESTAMP_DATE) {
     /*
       Generate a warning in case of DATE with fractional part:
         20011231.1234 -> '2001-12-31'
@@ -228,25 +224,6 @@ static bool lldiv_t_to_datetime(lldiv_t lld, MYSQL_TIME *ltime,
   return false;
 }
 
-/**
-  Convert decimal value to datetime.
-
-  @param       decimal The value to convert from.
-  @param[out]  ltime   The variable to convert to.
-  @param       flags   Conversion flags.
-
-  @returns false on success, true if not convertible to datetime.
-*/
-bool decimal_to_datetime(const my_decimal *decimal, MYSQL_TIME *ltime,
-                         my_time_flags_t flags) {
-  lldiv_t lld;
-  if (my_decimal2lldiv_t(0, decimal, &lld)) {
-    return true;
-  }
-  int warnings = 0;
-  return lldiv_t_to_datetime(lld, ltime, flags, &warnings);
-}
-
 /**
   Convert decimal value to datetime value with a warning.
   @param       decimal The value to convert from.
@@ -279,24 +256,6 @@ bool my_decimal_to_datetime_with_warn(const my_decimal *decimal,
   return rc;
 }
 
-/**
-  Convert double value to datetime.
-
-  @param       nr      The value to convert from.
-  @param[out]  ltime   The variable to convert to.
-  @param       flags   Conversion flags.
-
-  @returns false on success, true if not convertible to datetime.
-*/
-bool double_to_datetime(double nr, MYSQL_TIME *ltime, my_time_flags_t flags) {
-  lldiv_t lld;
-  if (double2lldiv_t(nr, &lld)) {
-    return true;
-  }
-  int warnings = 0;
-  return lldiv_t_to_datetime(lld, ltime, flags, &warnings);
-}
-
 /**
   Convert double value to datetime value with a warning.
   @param       nr      The value to convert from.
@@ -332,7 +291,7 @@ bool my_double_to_datetime_with_warn(double nr, MYSQL_TIME *ltime,
   Convert longlong value to datetime value with a warning.
   @param       nr      The value to convert from.
   @param[out]  ltime   The variable to convert to.
-  @param       flags   Conversion flags
+  @param       flags
 
   @return False on success, true on error.
 */
@@ -372,24 +331,6 @@ static bool lldiv_t_to_time(lldiv_t lld, MYSQL_TIME *ltime, int *warnings) {
                                           current_thd->is_fsp_truncate_mode());
 }
 
-/**
-  Convert decimal number to TIME
-
-  @param      decimal  The number to convert from.
-  @param[out] ltime          The variable to convert to.
-
-  @returns false on success, true if not convertible to time.
-*/
-bool decimal_to_time(const my_decimal *decimal, MYSQL_TIME *ltime) {
-  lldiv_t lld;
-  int warnings = 0;
-
-  if (my_decimal2lldiv_t(0, decimal, &lld)) {
-    return true;
-  }
-  return lldiv_t_to_time(lld, ltime, &warnings);
-}
-
 /**
   Convert decimal number to TIME
   @param      decimal  The number to convert from.
@@ -419,23 +360,6 @@ bool my_decimal_to_time_with_warn(const my_decimal *decimal,
   return rc;
 }
 
-/**
-  Convert double number to TIME
-
-  @param      nr      The number to convert from.
-  @param[out] ltime   The variable to convert to.
-
-  @returns false on success, true if not convertible to time.
-*/
-bool double_to_time(double nr, MYSQL_TIME *ltime) {
-  lldiv_t lld;
-  if (double2lldiv_t(nr, &lld) != E_DEC_OK) {
-    return true;
-  }
-  int warnings = 0;
-  return lldiv_t_to_time(lld, ltime, &warnings);
-}
-
 /**
   Convert double number to TIME
 
@@ -486,37 +410,66 @@ bool my_longlong_to_time_with_warn(longlong nr, MYSQL_TIME *ltime) {
 }
 
 /**
-  Converts a datetime in MYSQL_TIME representation to corresponding `struct
-  timeval` value.
+  Convert a datetime from broken-down MYSQL_TIME representation
+  to corresponding TIMESTAMP value.
 
-  `ltime` must be previously checked for `TIME_NO_ZERO_IN_DATE`.
+  @param  thd             - current thread
+  @param  t               - datetime in broken-down representation,
+  @param  in_dst_time_gap - pointer to bool which is set to true if t represents
+                            value which doesn't exists (falls into the spring
+                            time-gap) or to false otherwise.
+
+  @retval  Number seconds in UTC since start of Unix Epoch corresponding to t.
+  @retval  0 - t contains datetime value which is out of TIMESTAMP range.
+*/
+my_time_t TIME_to_timestamp(THD *thd, const MYSQL_TIME *t,
+                            bool *in_dst_time_gap) {
+  my_time_t timestamp;
+
+  *in_dst_time_gap = false;
+
+  timestamp = thd->time_zone()->TIME_to_gmt_sec(t, in_dst_time_gap);
+  if (timestamp) {
+    return timestamp;
+  }
+
+  /* If we are here we have range error. */
+  return (0);
+}
+
+/**
+  Convert a datetime MYSQL_TIME representation
+  to corresponding "struct timeval" value.
+
+  ltime must previously be checked for TIME_NO_ZERO_IN_DATE.
   Things like '0000-01-01', '2000-00-01', '2000-01-00' are not allowed
   and asserted.
 
   Things like '0000-00-00 10:30:30' or '0000-00-00 00:00:00.123456'
   (i.e. empty date with non-empty time) return error.
 
-  Zero datetime '0000-00-00 00:00:00.000000' is allowed and is mapped to
-  {tv_sec=0, tv_usec=0}.
+  Zero datetime '0000-00-00 00:00:00.000000'
+  is allowed and is mapper to {tv_sec=0, tv_usec=0}.
 
-  @note In case of error, tm value is not initialized.
+  Note: In case of error, tm value is not initialized.
 
-  @note `warnings` is not initialized to zero, so new warnings are added to the
-  old ones. The caller must make sure to initialize `warnings`.
+  Note: "warnings" is not initialized to zero,
+  so new warnings are added to the old ones.
+  Caller must make sure to initialize "warnings".
 
-  @param[in]  ltime    Datetime value
-  @param[in]  tz       Time zone to convert to.
-  @param[out] tm       Timeval value
-  @param[out] warnings Pointer to warnings.
+  @param[in]  thd       current thd
+  @param[in]  ltime     datetime value
+  @param[out] tm        timeval value
+  @param[out] warnings  pointer to warnings vector
 
   @return False on success, true on error.
 */
-bool datetime_with_no_zero_in_date_to_timeval(const MYSQL_TIME *ltime,
-                                              const Time_zone &tz,
-                                              my_timeval *tm, int *warnings) {
+bool datetime_with_no_zero_in_date_to_timeval(THD *thd, const MYSQL_TIME *ltime,
+                                              struct timeval *tm,
+                                              int *warnings) {
   if (!ltime->month) /* Zero date */
   {
-    assert(!ltime->year && !ltime->day);
+    DBUG_ASSERT(!ltime->year && !ltime->day);
     if (non_zero_time(*ltime)) {
       /*
         Return error for zero date with non-zero time, e.g.:
@@ -525,20 +478,19 @@ bool datetime_with_no_zero_in_date_to_timeval(const MYSQL_TIME *ltime,
       *warnings |= MYSQL_TIME_WARN_TRUNCATED;
       return true;
     }
-    tm->m_tv_sec = tm->m_tv_usec = 0;  // '0000-00-00 00:00:00.000000'
+    tm->tv_sec = tm->tv_usec = 0;  // '0000-00-00 00:00:00.000000'
     return false;
   }
 
-  bool is_in_dst_time_gap = false;
-  if (!(tm->m_tv_sec = tz.TIME_to_gmt_sec(ltime, &is_in_dst_time_gap))) {
+  bool in_dst_time_gap;
+  if (!(tm->tv_sec = TIME_to_timestamp(thd, ltime, &in_dst_time_gap))) {
     /*
       Date was outside of the supported timestamp range.
       For example: '3001-01-01 00:00:00' or '1000-01-01 00:00:00'
     */
     *warnings |= MYSQL_TIME_WARN_OUT_OF_RANGE;
     return true;
-  }
-  if (is_in_dst_time_gap) {
+  } else if (in_dst_time_gap) {
     /*
       Set MYSQL_TIME_WARN_INVALID_TIMESTAMP warning to indicate
       that date was fine but pointed to winter/summer time switch gap.
@@ -547,7 +499,7 @@ bool datetime_with_no_zero_in_date_to_timeval(const MYSQL_TIME *ltime,
     */
     *warnings |= MYSQL_TIME_WARN_INVALID_TIMESTAMP;
   }
-  tm->m_tv_usec = ltime->second_part;
+  tm->tv_usec = ltime->second_part;
   return false;
 }
 
@@ -570,18 +522,18 @@ bool datetime_with_no_zero_in_date_to_timeval(const MYSQL_TIME *ltime,
   so new warnings are added to the old ones.
   Caller must make sure to initialize "warnings".
 
+  @param[in]  thd       current thd
   @param[in]  ltime     datetime value
-  @param[in]  tz       The time zone.
   @param[out] tm        timeval value
   @param[out] warnings  pointer to warnings vector
 
   @return False on success, true on error.
 */
-bool datetime_to_timeval(const MYSQL_TIME *ltime, const Time_zone &tz,
-                         my_timeval *tm, int *warnings) {
+bool datetime_to_timeval(THD *thd, const MYSQL_TIME *ltime, struct timeval *tm,
+                         int *warnings) {
   return check_date(*ltime, non_zero_date(*ltime), TIME_NO_ZERO_IN_DATE,
                     warnings) ||
-         datetime_with_no_zero_in_date_to_timeval(ltime, tz, tm, warnings);
+         datetime_with_no_zero_in_date_to_timeval(thd, ltime, tm, warnings);
 }
 
 /**
@@ -607,9 +559,8 @@ bool str_to_time_with_warn(String *str, MYSQL_TIME *l_time) {
                                      NullS))
       return true;
   }
-  check_deprecated_datetime_format(current_thd, str->charset(), status);
-  if (!ret_val)
-    if (convert_time_zone_displacement(thd->time_zone(), l_time)) return true;
+
+  if (!ret_val) adjust_time_zone_displacement(thd->time_zone(), l_time);
 
   return ret_val;
 }
@@ -618,8 +569,8 @@ bool str_to_time_with_warn(String *str, MYSQL_TIME *l_time) {
   Convert time to datetime.
 
   The time value is added to the current datetime value.
-  @param thd            Thread context
-  @param [in] ltime     Time value to convert from.
+  @param thd
+  @param [in] ltime    Time value to convert from.
   @param [out] ltime2   Datetime value to convert to.
 */
 void time_to_datetime(THD *thd, const MYSQL_TIME *ltime, MYSQL_TIME *ltime2) {
@@ -634,8 +585,10 @@ void time_to_datetime(THD *thd, const MYSQL_TIME *ltime, MYSQL_TIME *ltime2) {
    Return format string according format name.
    If name is unknown, result is NULL
 
-   @returns format string according format name.
-   @retval NULL if name is unknown.
+   @param format
+   @param type
+
+   @return False on success, true on error.
 */
 const char *get_date_time_format_str(const Known_date_time_format *format,
                                      enum_mysql_timestamp_type type) {
@@ -647,7 +600,7 @@ const char *get_date_time_format_str(const Known_date_time_format *format,
     case MYSQL_TIMESTAMP_TIME:
       return format->time_format;
     default:
-      assert(0);  // Impossible
+      DBUG_ASSERT(0);  // Impossible
       return nullptr;
   }
 }
@@ -657,7 +610,7 @@ const char *get_date_time_format_str(const Known_date_time_format *format,
    @page DEFAULT_TIME_FUNCS Functions to create default time/date/datetime
    strings
    @note
-    For the moment the Date_time_format argument is ignored because
+    For the moment the Date_time_format argument is ignored becasue
     MySQL doesn't support comparing of date/time/datetime strings that
     are not in arbutary order as dates are compared as strings in some
     context)
@@ -675,10 +628,9 @@ const char *get_date_time_format_str(const Known_date_time_format *format,
   @param[out] str      String to convert to
   @param      dec      Number of fractional digits.
 */
-void make_time(const Date_time_format *format [[maybe_unused]],
+void make_time(const Date_time_format *format MY_ATTRIBUTE((unused)),
                const MYSQL_TIME *l_time, String *str, uint dec) {
-  const uint length =
-      static_cast<uint>(my_time_to_str(*l_time, str->ptr(), dec));
+  uint length = static_cast<uint>(my_time_to_str(*l_time, str->ptr(), dec));
   str->length(length);
   str->set_charset(&my_charset_numeric);
 }
@@ -689,9 +641,9 @@ void make_time(const Date_time_format *format [[maybe_unused]],
   @param      l_time   DATE value
   @param[out] str      String to convert to
 */
-void make_date(const Date_time_format *format [[maybe_unused]],
+void make_date(const Date_time_format *format MY_ATTRIBUTE((unused)),
                const MYSQL_TIME *l_time, String *str) {
-  const uint length = static_cast<uint>(my_date_to_str(*l_time, str->ptr()));
+  uint length = static_cast<uint>(my_date_to_str(*l_time, str->ptr()));
   str->length(length);
   str->set_charset(&my_charset_numeric);
 }
@@ -703,10 +655,9 @@ void make_date(const Date_time_format *format [[maybe_unused]],
   @param[out] str      String to convert to
   @param      dec      Number of fractional digits.
 */
-void make_datetime(const Date_time_format *format [[maybe_unused]],
+void make_datetime(const Date_time_format *format MY_ATTRIBUTE((unused)),
                    const MYSQL_TIME *l_time, String *str, uint dec) {
-  const uint length =
-      static_cast<uint>(my_datetime_to_str(*l_time, str->ptr(), dec));
+  uint length = static_cast<uint>(my_datetime_to_str(*l_time, str->ptr(), dec));
   str->length(length);
   str->set_charset(&my_charset_numeric);
 }
@@ -826,7 +777,7 @@ my_decimal *my_decimal_from_datetime_packed(my_decimal *dec,
       TIME_from_longlong_datetime_packed(&ltime, packed_value);
       return date2my_decimal(&ltime, dec);
     default:
-      assert(0);
+      DBUG_ASSERT(0);
       ulonglong2decimal(0, dec);
       return dec;
   }
