@@ -1,4 +1,4 @@
-/* Copyright (c) 2014, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -31,7 +31,7 @@
 #include "my_sys.h"
 #include "my_thread_local.h"
 #include "my_timer.h"  // my_timer_t
-#include "mysql/components/services/bits/mysql_mutex_bits.h"
+#include "mysql/components/services/mysql_mutex_bits.h"
 #include "mysql/psi/mysql_mutex.h"
 #include "mysql/service_mysql_alloc.h"
 #include "sql/mysqld.h"              // key_thd_timer_mutex
@@ -99,20 +99,20 @@ static THD_timer_info *thd_timer_create(void) {
 
 static bool timer_notify(THD_timer_info *thd_timer) {
   Find_thd_with_id find_thd_with_id(thd_timer->thread_id);
-  THD_ptr thd_ptr =
-      Global_THD_manager::get_instance()->find_thd(&find_thd_with_id);
+  THD *thd = Global_THD_manager::get_instance()->find_thd(&find_thd_with_id);
 
-  assert(!thd_timer->destroy || !thd_timer->thread_id);
+  DBUG_ASSERT(!thd_timer->destroy || !thd_timer->thread_id);
   /*
     Statement might have finished while the timer notification
     was being delivered. If this is the case, the timer object
     was detached (orphaned) and has no associated session (thd).
   */
-  if (thd_ptr) {
+  if (thd) {
     /* process only if thread is not already undergoing any kill connection. */
-    if (thd_ptr->killed != THD::KILL_CONNECTION) {
-      thd_ptr->awake(THD::KILL_TIMEOUT);
+    if (thd->killed != THD::KILL_CONNECTION) {
+      thd->awake(THD::KILL_TIMEOUT);
     }
+    mysql_mutex_unlock(&thd->LOCK_thd_data);
   }
 
   /* Mark the object as unreachable. */
@@ -161,7 +161,7 @@ THD_timer_info *thd_timer_set(THD *thd, THD_timer_info *thd_timer,
   if (thd_timer == nullptr && (thd_timer = thd_timer_create()) == nullptr)
     return nullptr;
 
-  assert(!thd_timer->destroy && !thd_timer->thread_id);
+  DBUG_ASSERT(!thd_timer->destroy && !thd_timer->thread_id);
 
   /* Mark the notification as pending. */
   thd_timer->thread_id = thd->thread_id();
@@ -181,25 +181,24 @@ THD_timer_info *thd_timer_set(THD *thd, THD_timer_info *thd_timer,
   Reap a (possibly) pending timer object.
 
   @param  thd_timer   Thread timer object.
-  @param  pending     State of the timer object. true if timer is not expired.
-  false otherwise.
+  @param  pending
 
   @return true if the timer object is unreachable.
 */
 
 static bool reap_timer(THD_timer_info *thd_timer, bool pending) {
   /* Cannot be tagged for destruction. */
-  assert(!thd_timer->destroy);
+  DBUG_ASSERT(!thd_timer->destroy);
 
   /* If not pending, timer hasn't fired. */
-  assert(pending || thd_timer->thread_id);
+  DBUG_ASSERT(pending || thd_timer->thread_id);
 
   /*
     The timer object can be reused if the timer was stopped before
     expiring. Otherwise, the timer notification function might be
     executing asynchronously in the context of a separate thread.
   */
-  const bool unreachable = pending ? thd_timer->thread_id == 0 : true;
+  bool unreachable = pending ? thd_timer->thread_id == 0 : true;
 
   thd_timer->thread_id = 0;
 
@@ -217,8 +216,7 @@ static bool reap_timer(THD_timer_info *thd_timer, bool pending) {
 
 THD_timer_info *thd_timer_reset(THD_timer_info *thd_timer) {
   bool unreachable;
-  int status;
-  int state{0};
+  int status, state;
   DBUG_TRACE;
 
   status = my_timer_cancel(&thd_timer->timer, &state);
