@@ -1,17 +1,16 @@
 /*
-  Copyright (c) 2019, 2024, Oracle and/or its affiliates.
+  Copyright (c) 2019, 2020, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -84,26 +83,6 @@
                   << std::flush;
       }
     }
-
-  The list of generated columns is kept separately in the `TABLE` class, in
-  the `TABLE::vfield` member. Although we could achieve accurate filtering
-  using the above described methods, as a performance optimization, this
-  class allows applying the filter and iteration directly and exclusively
-  on the generated columns. For that we can use the `VFIELDS_ONLY` option:
-
-    void print_virtual_generated_columns(TABLE *table) {
-      Table_columns_view<> fields{
-          table, [](TABLE const *table, size_t column_index) -> bool {
-            return table->field[column_index]->is_virtual_gcol();
-          },
-          Table_columns_view<>::VFIELDS_ONLY
-      };
-
-      for (auto field : fields) {
-        std::cout << field->field_index << ". " << field << std::endl
-                  << std::flush;
-      }
-    }
  */
 template <typename ExclusionFilter = std::function<bool(TABLE const *, size_t)>>
 class Table_columns_view {
@@ -113,27 +92,12 @@ class Table_columns_view {
    */
   using filter_fn_type = ExclusionFilter;
 
-  /**
-    Default set of options.
-   */
-  static constexpr unsigned long DEFAULTS = 0;
-  /**
-    Request the view excluding filter to operate `TABLE::vfields` instead
-    of the full set.
-  */
-  static constexpr unsigned long VFIELDS_ONLY = 1;
-
   /**
     Empty constructor, only available when the predicate type is a lambda
     function.
-
-    @param options optional parameter for filtering and iterating
-                   options. Options should be combined with `|`. Available
-                   options are `VFIELDS_ONLY`.
    */
   template <typename U = ExclusionFilter>
   Table_columns_view(
-      unsigned long options = 0,
       typename std::enable_if<std::is_same<
           U, std::function<bool(TABLE const *, size_t)>>::value>::type * =
           nullptr);
@@ -142,38 +106,26 @@ class Table_columns_view {
     predicate type is a lambda function.
 
     @param table reference to the target TABLE object.
-    @param options optional parameter for filtering and iterating
-                   options. Options should be combined with `|`. Available
-                   options are `VFIELDS_ONLY`.
    */
   template <typename U = ExclusionFilter>
   Table_columns_view(
-      TABLE const *table, unsigned long options = 0,
+      TABLE const *table,
       typename std::enable_if<std::is_same<
-          U, std::function<bool(TABLE const *, size_t)>>::value>::type * =
-          nullptr);
+          U, std::function<bool(TABLE const *, size_t)>>::value>::type * = 0);
   /**
     Constructor which takes a predicate used to filter this container iteration.
 
     @param filtering_predicate the predicate to filter this container iteration.
-    @param options optional parameter for filtering and iterating
-                   options. Options should be combined with `|`. Available
-                   options are `VFIELDS_ONLY`.
    */
-  Table_columns_view(ExclusionFilter filtering_predicate,
-                     unsigned long options = 0);
+  Table_columns_view(ExclusionFilter filtering_predicate);
   /**
     Constructor which takes the TABLE object whose field set will be iterated
     and a predicate used to filter this container iteration.
 
     @param table reference to the target TABLE object.
     @param filtering_predicate the predicate to filter this container iteration.
-    @param options optional parameter for filtering and iterating
-                   options. Options should be combined with `|`. Available
-                   options are `VFIELDS_ONLY`.
    */
-  Table_columns_view(TABLE const *table, ExclusionFilter filtering_predicate,
-                     unsigned long options = 0);
+  Table_columns_view(TABLE const *table, ExclusionFilter filtering_predicate);
   /**
     Destructor for the class.
    */
@@ -186,7 +138,7 @@ class Table_columns_view {
 
     @return a reference to this object.
    */
-  virtual Table_columns_view &set_table(const TABLE *rhs);
+  Table_columns_view &set_table(const TABLE *rhs);
   /**
     Setter which initializes the internal filtering predicate of type
     `ExclusionFilter`.
@@ -195,7 +147,7 @@ class Table_columns_view {
 
     @return a reference to this object.
    */
-  virtual Table_columns_view &set_filter(ExclusionFilter rhs);
+  Table_columns_view &set_filter(ExclusionFilter rhs);
 
   // --> Deleted constructors and methods to remove default move/copy semantics
   Table_columns_view(const Table_columns_view &rhs) = delete;
@@ -229,25 +181,6 @@ class Table_columns_view {
       @param col initial local table field set position.
      */
     explicit iterator(Table_columns_view &parent, long pos, long col);
-    /**
-      Constructor for the iterator. It takes the parent Table_columns_view
-      object and the initial positions for the replicated table and for the
-      local table. It also includes a translation factor so we can get the
-      iterated position in relation to a different set of columns.
-
-      @note When this iterator is used in the context of a replica that is
-      applying an event, the translation offset represents the number of extra
-      columns that the event has to the left of other columns, which the table
-      does not have (if any). So, for example, when the event has a GIPK column
-      to the left, and the replica does not have that, then the offset is 1.
-
-      @param parent reference to the target Table_columns_view object.
-      @param pos initial replicated table field set position.
-      @param col initial local table field set position.
-      @param translation_offset the translation offset for translated_pos()
-     */
-    explicit iterator(Table_columns_view &parent, long pos, long col,
-                      long translation_offset);
     /**
       Copy constructor.
 
@@ -290,26 +223,14 @@ class Table_columns_view {
               field set.
      */
     size_t absolute_pos();
-    /**
-      Returns the position this iterator relative to the set of table columns
-      which are not excluded by the associated filters
-
-      @return the position this object is pointing to considering the non
-              filtered columns
-     */
-    size_t filtered_pos();
     /**
       Returns the position this iterator object is pointing to, within the
-      replicated table field set plus the translation_offset
-
-      @note When this iterator is used in the context of a replica that is
-      applying an event, use translated_pos to get the position within the
-      event."
+      replicated table field set.
 
       @return the position this object is pointing to, within the replicated
-              table field set adjusted to another frame of reference.
+              table field set.
      */
-    size_t translated_pos();
+    size_t filtered_pos();
 
     friend struct TABLE;
     friend class Table_columns_view;
@@ -327,16 +248,6 @@ class Table_columns_view {
       iterator is pointing to.
     */
     long m_filtered_pos;
-    /**
-      Translation unit used on top of the iterator filtered position, so
-      we can adjust the position to another frame of reference.
-
-      When this iterator is used in the context of a replica that is applying an
-      event, use translated_pos to get the position within the event. This
-      number should be set to N when the event has N extra columns to the left,
-      which do not exist in the replica table.
-    */
-    long m_translation_offset;
   };
 
   /**
@@ -357,13 +268,13 @@ class Table_columns_view {
 
     @return an iterator pointing to the beginning of the field set.
   */
-  virtual iterator begin();
+  iterator begin();
   /**
     Creates an iterator object, pointing at the end of the table field set.
 
     @return an iterator pointing to the end of the field set.
   */
-  virtual iterator end();
+  iterator end();
   /**
     Returns whether or not the field at `index` is to be excluded from the field
     set iteration process.
@@ -401,26 +312,16 @@ class Table_columns_view {
   */
   Table_columns_view &translate_bitmap(MY_BITMAP &source,
                                        MY_BITMAP &destination);
-
   /**
-    For the absolute position on the table that equals the given position given
-    as a parameter, return the translated position.
+    Translates a position in the received replicated table into a position in
+    the local table.
 
-    @param source the position in the local table
+    @param source the position in the received replicated table
 
-    @return the translated position within the local table.
+    @return the relative position within the local table.
   */
   size_t translate_position(size_t source);
 
-  /**
-    Returns the iterator for the (absolute) position in the table.
-
-    @param absolute_pos the position in the local table
-
-    @return the iterator for the position, if found
-  */
-  iterator find_by_absolute_pos(size_t absolute_pos);
-
  protected:
   /**
     Initializes the internal included and excluded fields bitmaps. After each
@@ -452,10 +353,6 @@ class Table_columns_view {
     are to be excluded from the replicated row.
    */
   MY_BITMAP m_excluded_fields_bitmap;
-  /**
-    Set of options to apply to view behaviour
-   */
-  unsigned long m_options{0};
 
   /**
     Default filtering predicate.
@@ -466,36 +363,31 @@ class Table_columns_view {
 template <typename F>
 template <typename U>
 Table_columns_view<F>::Table_columns_view(
-    unsigned long options,
     typename std::enable_if<std::is_same<
         U, std::function<bool(TABLE const *, size_t)>>::value>::type *)
-    : m_filtering_predicate{Table_columns_view::default_filter},
-      m_options{options} {
+    : m_filtering_predicate{Table_columns_view::default_filter} {
   this->set_filter(Table_columns_view::default_filter);
 }
 
 template <typename F>
 template <typename U>
 Table_columns_view<F>::Table_columns_view(
-    TABLE const *table, unsigned long options,
+    TABLE const *table,
     typename std::enable_if<std::is_same<
         U, std::function<bool(TABLE const *, size_t)>>::value>::type *)
-    : m_filtering_predicate{Table_columns_view::default_filter},
-      m_options{options} {
+    : m_filtering_predicate{Table_columns_view::default_filter} {
   this->set_filter(Table_columns_view::default_filter)  //
       .set_table(table);
 }
 
 template <typename F>
-Table_columns_view<F>::Table_columns_view(F filtering_predicate,
-                                          unsigned long options)
-    : Table_columns_view{nullptr, filtering_predicate, options} {}
+Table_columns_view<F>::Table_columns_view(F filtering_predicate)
+    : Table_columns_view{nullptr, filtering_predicate} {}
 
 template <typename F>
 Table_columns_view<F>::Table_columns_view(TABLE const *target,
-                                          F filtering_predicate,
-                                          unsigned long options)
-    : m_filtering_predicate{filtering_predicate}, m_options{options} {
+                                          F filtering_predicate)
+    : m_filtering_predicate{filtering_predicate} {
   this->set_filter(filtering_predicate)  //
       .set_table(target);
 }
@@ -569,12 +461,13 @@ Table_columns_view<F> &Table_columns_view<F>::translate_bitmap(
 
   bitmap_init(&destination, nullptr, this->m_table->s->fields);
 
-  for (auto it = begin(); it != end(); ++it) {
-    size_t source_pos = it.translated_pos();
-    size_t abs_pos = it.absolute_pos();
-    if (source_pos >= source.n_bits) break;
-    if (bitmap_is_set(&source, static_cast<uint>(source_pos))) {
-      bitmap_set_bit(&destination, static_cast<uint>(abs_pos));
+  for (size_t d = 0, s = 0; d != destination.n_bits && s != source.n_bits;
+       ++d) {
+    if (!this->is_excluded(d)) {
+      if (bitmap_is_set(&source, static_cast<uint>(s))) {
+        bitmap_set_bit(&destination, static_cast<uint>(d));
+      }
+      ++s;
     }
   }
 
@@ -582,20 +475,15 @@ Table_columns_view<F> &Table_columns_view<F>::translate_bitmap(
 }
 
 template <typename F>
-typename Table_columns_view<F>::iterator
-Table_columns_view<F>::find_by_absolute_pos(size_t absolute_pos) {
-  return std::find_if(begin(), end(), [absolute_pos](auto it) {
-    return it->field_index() == absolute_pos;
-  });
-}
-
-template <typename F>
-size_t Table_columns_view<F>::translate_position(size_t orig_pos) {
-  for (auto it = begin(); it != end(); ++it) {
-    size_t abs_pos = it.absolute_pos();
-    if (abs_pos == orig_pos) return it.translated_pos();
+size_t Table_columns_view<F>::translate_position(size_t source) {
+  if (this->m_table == nullptr) return source;
+  size_t d = 0;
+  for (size_t s = 0; s != source; ++d) {
+    if (!this->is_excluded(d)) {
+      ++s;
+    }
   }
-  return orig_pos;
+  return d;
 }
 
 template <typename F>
@@ -610,26 +498,15 @@ Table_columns_view<F> &Table_columns_view<F>::init_fields_bitmaps() {
               this->m_table->s->fields);
 
   this->m_filtered_size = 0;
-  if ((this->m_options & VFIELDS_ONLY) == VFIELDS_ONLY) {
-    bitmap_set_all(&this->m_excluded_fields_bitmap);
-    for (auto fld = this->m_table->vfield; *fld != nullptr; ++fld) {
-      auto idx = (*fld)->field_index();
-      if (!this->m_filtering_predicate(this->m_table, idx)) {
-        bitmap_set_bit(&this->m_included_fields_bitmap, idx);
-        bitmap_clear_bit(&this->m_excluded_fields_bitmap, idx);
-        ++this->m_filtered_size;
-      }
-    }
-  } else {
-    for (size_t idx = 0; idx != this->m_table->s->fields; ++idx) {
-      if (this->m_filtering_predicate(this->m_table, idx)) {
-        bitmap_set_bit(&this->m_excluded_fields_bitmap, idx);
-      } else {
-        bitmap_set_bit(&this->m_included_fields_bitmap, idx);
-        ++this->m_filtered_size;
-      }
+  for (size_t idx = 0; idx != this->m_table->s->fields; ++idx) {
+    if (this->m_filtering_predicate(this->m_table, idx)) {
+      bitmap_set_bit(&this->m_excluded_fields_bitmap, idx);
+    } else {
+      bitmap_set_bit(&this->m_included_fields_bitmap, idx);
+      ++this->m_filtered_size;
     }
   }
+
   return (*this);
 }
 
@@ -643,17 +520,7 @@ Table_columns_view<F>::iterator::iterator(Table_columns_view &parent,
                                           long absolute_pos, long filtered_pos)
     : m_parent{&parent},
       m_absolute_pos{absolute_pos},
-      m_filtered_pos{filtered_pos},
-      m_translation_offset{0} {}
-
-template <typename F>
-Table_columns_view<F>::iterator::iterator(Table_columns_view &parent,
-                                          long absolute_pos, long filtered_pos,
-                                          long translation_offset)
-    : m_parent{&parent},
-      m_absolute_pos{absolute_pos},
-      m_filtered_pos{filtered_pos},
-      m_translation_offset{translation_offset} {}
+      m_filtered_pos{filtered_pos} {}
 
 template <typename F>
 Table_columns_view<F>::iterator::iterator(const iterator &rhs) {
@@ -661,19 +528,17 @@ Table_columns_view<F>::iterator::iterator(const iterator &rhs) {
 }
 
 template <typename F>
-typename Table_columns_view<F>::iterator &
-Table_columns_view<F>::iterator::operator=(
-    const Table_columns_view<F>::iterator &rhs) {
+typename Table_columns_view<F>::iterator &Table_columns_view<F>::iterator::
+operator=(const Table_columns_view<F>::iterator &rhs) {
   this->m_parent = rhs.m_parent;
   this->m_absolute_pos = rhs.m_absolute_pos;
   this->m_filtered_pos = rhs.m_filtered_pos;
-  this->m_translation_offset = rhs.m_translation_offset;
   return (*this);
 }
 
 template <typename F>
-typename Table_columns_view<F>::iterator &
-Table_columns_view<F>::iterator::operator++() {
+typename Table_columns_view<F>::iterator &Table_columns_view<F>::iterator::
+operator++() {
   if (this->m_parent->m_table != nullptr &&
       this->m_absolute_pos !=
           static_cast<long>(this->m_parent->absolute_size())) {
@@ -689,7 +554,7 @@ Table_columns_view<F>::iterator::operator++() {
 
 template <typename F>
 typename Table_columns_view<F>::iterator::reference
-Table_columns_view<F>::iterator::operator*() const {
+    Table_columns_view<F>::iterator::operator*() const {
   if (this->m_parent->m_table != nullptr &&
       this->m_absolute_pos !=
           static_cast<long>(this->m_parent->absolute_size())) {
@@ -699,8 +564,8 @@ Table_columns_view<F>::iterator::operator*() const {
 }
 
 template <typename F>
-typename Table_columns_view<F>::iterator
-Table_columns_view<F>::iterator::operator++(int) {
+typename Table_columns_view<F>::iterator Table_columns_view<F>::iterator::
+operator++(int) {
   typename Table_columns_view<F>::iterator to_return = (*this);
   ++(*this);
   return to_return;
@@ -708,7 +573,7 @@ Table_columns_view<F>::iterator::operator++(int) {
 
 template <typename F>
 typename Table_columns_view<F>::iterator::pointer
-Table_columns_view<F>::iterator::operator->() const {
+    Table_columns_view<F>::iterator::operator->() const {
   return this->operator*();
 }
 
@@ -726,8 +591,8 @@ bool Table_columns_view<F>::iterator::operator!=(
 }
 
 template <typename F>
-typename Table_columns_view<F>::iterator &
-Table_columns_view<F>::iterator::operator--() {
+typename Table_columns_view<F>::iterator &Table_columns_view<F>::iterator::
+operator--() {
   if (this->m_parent->m_table != nullptr && this->m_absolute_pos != 0) {
     do {
       --this->m_absolute_pos;
@@ -739,8 +604,8 @@ Table_columns_view<F>::iterator::operator--() {
 }
 
 template <typename F>
-typename Table_columns_view<F>::iterator
-Table_columns_view<F>::iterator::operator--(int) {
+typename Table_columns_view<F>::iterator Table_columns_view<F>::iterator::
+operator--(int) {
   typename Table_columns_view<F>::iterator to_return = (*this);
   --(*this);
   return to_return;
@@ -756,9 +621,4 @@ size_t Table_columns_view<F>::iterator::filtered_pos() {
   return static_cast<size_t>(this->m_filtered_pos);
 }
 
-template <typename F>
-size_t Table_columns_view<F>::iterator::translated_pos() {
-  return static_cast<size_t>(this->m_filtered_pos + this->m_translation_offset);
-}
-
 #endif  // _table_column_iterator_h
