@@ -1,4 +1,4 @@
-/* Copyright (c) 2017, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -28,19 +28,20 @@
 #include <utility>
 
 #include "field_types.h"
+#include "m_ctype.h"
+#include "m_string.h"
 #include "my_sys.h"
-#include "mysql/components/services/bits/psi_bits.h"
-#include "mysql/strings/m_ctype.h"
+#include "mysql/psi/psi_base.h"
 #include "mysql_com.h"
 #include "mysqld_error.h"
 #include "prealloced_array.h"
-#include "sql-common/json_dom.h"
-#include "sql-common/json_path.h"
 #include "sql/error_handler.h"
 #include "sql/field.h"
 #include "sql/handler.h"
 #include "sql/item.h"
 #include "sql/item_json_func.h"
+#include "sql/json_dom.h"
+#include "sql/json_path.h"
 #include "sql/psi_memory_key.h"
 #include "sql/sql_class.h"  // THD
 #include "sql/sql_exception_handler.h"
@@ -51,15 +52,14 @@
 #include "sql/table.h"
 #include "sql/thd_raii.h"
 #include "sql_string.h"
-#include "string_with_len.h"
 
 /******************************************************************************
   Implementation of Table_function
 ******************************************************************************/
 
-bool Table_function::create_result_table(THD *thd, ulonglong options,
+bool Table_function::create_result_table(ulonglong options,
                                          const char *table_alias) {
-  assert(table == nullptr);
+  DBUG_ASSERT(table == nullptr);
 
   table = create_tmp_table_from_fields(thd, *get_field_list(), false, options,
                                        table_alias);
@@ -71,21 +71,19 @@ bool Table_function::write_row() {
 
   if ((error = table->file->ha_write_row(table->record[0]))) {
     if (!table->file->is_ignorable_error(error) &&
-        create_ondisk_from_heap(
-            current_thd, table, error, /*insert_last_record=*/true,
-            /*ignore_last_dup=*/true, /*is_duplicate=*/nullptr))
+        create_ondisk_from_heap(thd, table, error, true, nullptr))
       return true;  // Not a table_is_full error
   }
   return false;
 }
 
 void Table_function::empty_table() {
-  assert(table->is_created());
+  DBUG_ASSERT(table->is_created());
   (void)table->empty_result_table();
 }
 
 bool Table_function::init_args() {
-  assert(!inited);
+  if (inited) return false;
   if (do_init_args()) return true;
   table->pos_in_table_list->dep_tables |= used_tables();
   inited = true;
@@ -95,11 +93,11 @@ bool Table_function::init_args() {
 /******************************************************************************
   Implementation of JSON_TABLE function
 ******************************************************************************/
-Table_function_json::Table_function_json(const char *alias, Item *a,
-                                         List<Json_table_column> *cols)
-    : Table_function(),
+Table_function_json::Table_function_json(THD *thd_arg, const char *alias,
+                                         Item *a, List<Json_table_column> *cols)
+    : Table_function(thd_arg),
       m_columns(cols),
-      m_all_columns(current_thd->mem_root),
+      m_all_columns(thd->mem_root),
       m_table_alias(alias),
       is_source_parsed(false),
       source(a) {}
@@ -115,6 +113,26 @@ List<Create_field> *Table_function_json::get_field_list() {
   return reinterpret_cast<List<Create_field> *>(&m_vt_list);
 }
 
+/**
+  Initialize columns and lists for json table
+
+  @details This function does several things:
+  1) sets up list of fields (vt_list) for result table creation
+  2) fills array of all columns (m_all_columns) for execution
+  3) for each column that has default ON EMPTY or ON ERROR clauses, checks
+    the value to be proper json and initializes column appropriately
+  4) for each column that involves path, the path is checked to be correct.
+  The function goes recursively, starting from the top NESTED PATH clause
+  and going in the depth-first way, traverses the tree of columns.
+
+  @param nest_idx  index of parent's element in the nesting data array
+  @param parent    Parent of the NESTED PATH clause being initialized
+
+  @returns
+    false  ok
+    true   an error occurred
+*/
+
 bool Table_function_json::init_json_table_col_lists(uint *nest_idx,
                                                     Json_table_column *parent) {
   List_iterator<Json_table_column> li(*parent->m_nested_columns);
@@ -126,7 +144,7 @@ bool Table_function_json::init_json_table_col_lists(uint *nest_idx,
     This need to be set up once per statement, as it doesn't change between
     EXECUTE calls.
   */
-  const Prepared_stmt_arena_holder ps_arena_holder(current_thd);
+  Prepared_stmt_arena_holder ps_arena_holder(thd);
 
   while ((col = li++)) {
     String buffer;
@@ -142,7 +160,7 @@ bool Table_function_json::init_json_table_col_lists(uint *nest_idx,
       if ((col->sql_type == MYSQL_TYPE_ENUM ||
            col->sql_type == MYSQL_TYPE_SET) &&
           !col->interval)
-        col->interval = create_typelib(current_thd->mem_root, col);
+        col->interval = create_typelib(thd->mem_root, col);
     }
     m_all_columns.push_back(col);
 
@@ -153,30 +171,30 @@ bool Table_function_json::init_json_table_col_lists(uint *nest_idx,
       }
       case enum_jt_column::JTC_PATH: {
         const String *path = col->m_path_string->val_str(&buffer);
-        assert(path != nullptr);
+        DBUG_ASSERT(path != nullptr);
         if (parse_path(*path, false, &col->m_path_json)) return true;
-        if (col->m_on_empty == Json_on_response_type::DEFAULT) {
+        if (col->m_on_empty == enum_jtc_on::JTO_DEFAULT) {
           const String *default_string =
               col->m_default_empty_string->val_str(&buffer);
-          assert(default_string != nullptr);
+          DBUG_ASSERT(default_string != nullptr);
           Json_dom_ptr dom;  //@< we'll receive a DOM here
-          JsonParseDefaultErrorHandler parse_handler("JSON_TABLE", 0);
-          if (parse_json(*default_string, &dom, true, parse_handler,
-                         JsonDepthErrorHandler) ||
+          bool parse_error;
+          if (parse_json(*default_string, 0, "JSON_TABLE", &dom, true,
+                         &parse_error) ||
               (col->sql_type != MYSQL_TYPE_JSON && !dom->is_scalar())) {
             my_error(ER_INVALID_DEFAULT, MYF(0), col->field_name);
             return true;
           }
           col->m_default_empty_json = Json_wrapper(std::move(dom));
         }
-        if (col->m_on_error == Json_on_response_type::DEFAULT) {
+        if (col->m_on_error == enum_jtc_on::JTO_DEFAULT) {
           const String *default_string =
               col->m_default_error_string->val_str(&buffer);
-          assert(default_string != nullptr);
+          DBUG_ASSERT(default_string != nullptr);
           Json_dom_ptr dom;  //@< we'll receive a DOM here
-          JsonParseDefaultErrorHandler parse_handler("JSON_TABLE", 0);
-          if (parse_json(*default_string, &dom, true, parse_handler,
-                         JsonDepthErrorHandler) ||
+          bool parse_error;
+          if (parse_json(*default_string, 0, "JSON_TABLE", &dom, true,
+                         &parse_error) ||
               (col->sql_type != MYSQL_TYPE_JSON && !dom->is_scalar())) {
             my_error(ER_INVALID_DEFAULT, MYF(0), col->field_name);
             return true;
@@ -187,7 +205,7 @@ bool Table_function_json::init_json_table_col_lists(uint *nest_idx,
       }
       case enum_jt_column::JTC_EXISTS: {
         const String *path = col->m_path_string->val_str(&buffer);
-        assert(path != nullptr);
+        DBUG_ASSERT(path != nullptr);
         if (parse_path(*path, false, &col->m_path_json)) return true;
         break;
       }
@@ -201,7 +219,7 @@ bool Table_function_json::init_json_table_col_lists(uint *nest_idx,
         col->m_child_jds_elt = &m_jds[*nest_idx];
 
         const String *path = col->m_path_string->val_str(&buffer);
-        assert(path != nullptr);
+        DBUG_ASSERT(path != nullptr);
         if (nested) {
           nested->m_next_nested = col;
           col->m_prev_nested = nested;
@@ -214,7 +232,7 @@ bool Table_function_json::init_json_table_col_lists(uint *nest_idx,
         break;
       }
       default:
-        assert(0);
+        DBUG_ASSERT(0);
     }
   }
   return false;
@@ -229,27 +247,16 @@ bool Table_function_json::init_json_table_col_lists(uint *nest_idx,
 */
 
 bool Table_function_json::do_init_args() {
-  assert(!is_source_parsed);
+  DBUG_ASSERT(!is_source_parsed);
 
   Item *dummy = source;
-  if (source->fix_fields(current_thd, &dummy)) return true;
+  if (source->fix_fields(thd, &dummy)) return true;
 
-  /*
-    For the default type of '?', two choices make sense: VARCHAR and JSON. The
-    latter would lead to a call to Item_param::val_json() which isn't
-    implemented. So we use the former.
-  */
-  if (source->propagate_type(current_thd)) return true;
-  assert(source->data_type() != MYSQL_TYPE_VAR_STRING);
+  DBUG_ASSERT(source->data_type() != MYSQL_TYPE_VAR_STRING);
   if (source->has_aggregation() || source->has_subquery() || source != dummy) {
     my_error(ER_WRONG_ARGUMENTS, MYF(0), "JSON_TABLE");
     return true;
   }
-
-  if (source->check_cols(1)) {
-    return true;
-  }
-
   try {
     /*
       Check whether given JSON source is a const and it's valid, see also
@@ -272,15 +279,15 @@ bool Table_function_json::do_init_args() {
   // Validate that all the DEFAULT values are convertible to the target type.
   for (const Json_table_column *col : m_all_columns) {
     if (col->m_jtc_type != enum_jt_column::JTC_PATH) continue;
-    assert(col->m_field_idx >= 0);
-    if (col->m_on_empty == Json_on_response_type::DEFAULT) {
-      if (save_json_to_field(current_thd, get_field(col->m_field_idx),
+    DBUG_ASSERT(col->m_field_idx >= 0);
+    if (col->m_on_empty == enum_jtc_on::JTO_DEFAULT) {
+      if (save_json_to_field(thd, get_field(col->m_field_idx),
                              &col->m_default_empty_json, false)) {
         return true;
       }
     }
-    if (col->m_on_error == Json_on_response_type::DEFAULT) {
-      if (save_json_to_field(current_thd, get_field(col->m_field_idx),
+    if (col->m_on_error == enum_jtc_on::JTO_DEFAULT) {
+      if (save_json_to_field(thd, get_field(col->m_field_idx),
                              &col->m_default_error_json, false)) {
         return true;
       }
@@ -294,14 +301,23 @@ bool Table_function_json::init() {
   if (m_vt_list.elements == 0) {
     uint nest_idx = 0;
     if (init_json_table_col_lists(&nest_idx, &top)) return true;
+    List_iterator<Json_table_column> li(m_vt_list);
 
-    // Check for duplicate field names.
-    for (Create_field &outer : m_vt_list) {
-      Name_string outer_name(to_lex_cstring(outer.field_name));
-      for (Create_field &inner : m_vt_list) {
-        if (&outer == &inner) break;
-        if (outer_name.eq(inner.field_name)) {
-          my_error(ER_DUP_FIELDNAME, MYF(0), inner.field_name);
+    /*
+      Check for duplicate names.
+      Two iterators over vt_list are used. First is used to get a field,
+      second - to compare the field with fields in the rest of the list.
+      For each iteration of the first list, we skip fields prior to the
+      first iterator's field.
+    */
+    Json_table_column *first;
+    while ((first = li++)) {
+      Json_table_column *col;
+      List_iterator<Json_table_column> li2(m_vt_list);
+      // Compare 'first' with all columns prior to it
+      while ((col = li2++) && col != first) {
+        if (!strncmp(first->field_name, col->field_name, NAME_CHAR_LEN)) {
+          my_error(ER_DUP_FIELDNAME, MYF(0), first->field_name);
           return true;
         }
       }
@@ -310,6 +326,14 @@ bool Table_function_json::init() {
   return false;
 }
 
+/**
+  A helper function which sets all columns under given NESTED PATH column
+  to nullptr. Used to evaluate sibling NESTED PATHS.
+
+  @param       root  root NESTED PATH column
+  @param [out] last  last column which belongs to the given NESTED PATH
+*/
+
 void Table_function_json::set_subtree_to_null(Json_table_column *root,
                                               Json_table_column **last) {
   List_iterator<Json_table_column> li(*root->m_nested_columns);
@@ -327,6 +351,32 @@ void Table_function_json::set_subtree_to_null(Json_table_column *root,
   }
 }
 
+/**
+  Fill a json table column
+
+  @details Fills a column with data, according to specification in
+  JSON_TABLE. This function handles all kinds of columns:
+  Ordinality)  just saves the counter into the column's field
+  Path)        extracts value, saves it to the column's field and handles
+               ON ERROR/ON EMPTY clauses
+  Exists)      checks the path existence and saves either 1 or 0 into result
+               field
+  Nested path) matches the path expression against data source. If there're
+               matches, this function sets NESTED PATH's iterator over those
+               matches and resets ordinality counter.
+
+  @param[in]   table_function the JSON table function
+  @param[out]  skip  true <=> it's a NESTED PATH node and its path
+                     expression didn't return any matches or a
+                     previous sibling NESTED PATH clause still producing
+                     records, thus all columns of this NESTED PATH node
+                     should be skipped
+
+  @returns
+    false column is filled
+    true  an error occurred, execution should be stopped
+*/
+
 bool Json_table_column::fill_column(Table_function_json *table_function,
                                     jt_skip_reason *skip) {
   *skip = JTS_NONE;
@@ -334,8 +384,8 @@ bool Json_table_column::fill_column(Table_function_json *table_function,
   Field *const fld = m_jtc_type == enum_jt_column::JTC_NESTED_PATH
                          ? nullptr
                          : table_function->get_field(m_field_idx);
-  assert(m_jtc_type == enum_jt_column::JTC_NESTED_PATH ||
-         (fld != nullptr && fld->field_index() == m_field_idx));
+  DBUG_ASSERT(m_jtc_type == enum_jt_column::JTC_NESTED_PATH ||
+              (fld != nullptr && fld->field_index == m_field_idx));
 
   switch (m_jtc_type) {
     case enum_jt_column::JTC_ORDINALITY: {
@@ -344,7 +394,7 @@ bool Json_table_column::fill_column(Table_function_json *table_function,
       break;
     }
     case enum_jt_column::JTC_PATH: {
-      THD *thd = current_thd;
+      THD *thd = fld->table->in_use;
       // Vector of matches
       Json_wrapper_vector data_v(key_memory_JSON);
       m_jds_elt->jdata.seek(m_path_json, m_path_json.leg_count(), &data_v, true,
@@ -358,7 +408,7 @@ bool Json_table_column::fill_column(Table_function_json *table_function,
             Json_array *a = new (std::nothrow) Json_array();
             if (!a) return true;
             for (Json_wrapper &w : data_v) {
-              if (a->append_alias(w.clone_dom())) {
+              if (a->append_alias(w.clone_dom(thd))) {
                 delete a; /* purecov: inspected */
                 return true;
               }
@@ -367,7 +417,7 @@ bool Json_table_column::fill_column(Table_function_json *table_function,
           } else {
             is_error = true;
             // Thrown an error when save_json_to_field() isn't called
-            if (m_on_error == Json_on_response_type::ERROR)
+            if (m_on_error == enum_jtc_on::JTO_ERROR)
               my_error(ER_WRONG_JSON_TABLE_VALUE, MYF(0), field_name);
           }
         } else
@@ -380,25 +430,22 @@ bool Json_table_column::fill_column(Table_function_json *table_function,
           // Strict_error_handler and prevent the ON ERROR clause from being
           // respected.
           Ignore_warnings_error_handler ignore_warnings;
-          const bool no_error = m_on_error != Json_on_response_type::ERROR;
+          const bool no_error = m_on_error != enum_jtc_on::JTO_ERROR;
           if (no_error) thd->push_internal_handler(&ignore_warnings);
-          if (buf.type() == enum_json_type::J_NULL)  // see JSON_VALUE
-            fld->set_null();
-          else
-            is_error = save_json_to_field(thd, fld, &buf, no_error);
+          is_error = save_json_to_field(thd, fld, &buf, no_error);
           if (no_error) thd->pop_internal_handler();
         }
         if (is_error) {
           switch (m_on_error) {
-            case Json_on_response_type::ERROR: {
+            case enum_jtc_on::JTO_ERROR: {
               return true;
               break;
             }
-            case Json_on_response_type::DEFAULT: {
+            case enum_jtc_on::JTO_DEFAULT: {
               save_json_to_field(thd, fld, &m_default_error_json, true);
               break;
             }
-            case Json_on_response_type::NULL_VALUE:
+            case enum_jtc_on::JTO_NULL:
             default: {
               fld->set_null();
               break;
@@ -407,15 +454,15 @@ bool Json_table_column::fill_column(Table_function_json *table_function,
         }
       } else {
         switch (m_on_empty) {
-          case Json_on_response_type::ERROR: {
+          case enum_jtc_on::JTO_ERROR: {
             my_error(ER_MISSING_JSON_TABLE_VALUE, MYF(0), field_name);
             return true;
           }
-          case Json_on_response_type::DEFAULT: {
+          case enum_jtc_on::JTO_DEFAULT: {
             save_json_to_field(thd, fld, &m_default_empty_json, true);
             break;
           }
-          case Json_on_response_type::NULL_VALUE:
+          case enum_jtc_on::JTO_NULL:
           default: {
             fld->set_null();
             break;
@@ -433,7 +480,6 @@ bool Json_table_column::fill_column(Table_function_json *table_function,
         fld->store(1, true);
       else
         fld->store(0, true);
-      if (current_thd->is_error()) return true;
       fld->set_notnull();
       break;
     }
@@ -453,7 +499,7 @@ bool Json_table_column::fill_column(Table_function_json *table_function,
       }
       // Run only one sibling nested path at a time
       for (Json_table_column *tc = m_prev_nested; tc; tc = tc->m_prev_nested) {
-        assert(tc->m_jtc_type == enum_jt_column::JTC_NESTED_PATH);
+        DBUG_ASSERT(tc->m_jtc_type == enum_jt_column::JTC_NESTED_PATH);
         if (tc->m_child_jds_elt->producing_records) {
           *skip = JTS_SIBLING;
           return false;
@@ -474,19 +520,19 @@ bool Json_table_column::fill_column(Table_function_json *table_function,
       break;
     }
     default: {
-      assert(0);
+      DBUG_ASSERT(0);
       break;
     }
   }
   return false;
 }
 
-Json_table_column::~Json_table_column() {
+void Json_table_column::cleanup() {
   // Reset paths and wrappers to free allocated memory.
-  m_path_json = Json_path(key_memory_JSON);
-  if (m_on_empty == Json_on_response_type::DEFAULT)
+  m_path_json = Json_path();
+  if (m_on_empty == enum_jtc_on::JTO_DEFAULT)
     m_default_empty_json = Json_wrapper();
-  if (m_on_error == Json_on_response_type::DEFAULT)
+  if (m_on_error == enum_jtc_on::JTO_DEFAULT)
     m_default_error_json = Json_wrapper();
 }
 
@@ -546,11 +592,10 @@ Json_table_column::~Json_table_column() {
 bool Table_function_json::fill_json_table() {
   // 'Stack' of nested NESTED PATH clauses
   Prealloced_array<uint, MAX_NESTED_PATH> nested(PSI_NOT_INSTRUMENTED);
-
   // The column being processed
   uint col_idx = 0;
   jt_skip_reason skip_subtree;
-  const enum_check_fields check_save = current_thd->check_for_truncated_fields;
+  const enum_check_fields check_save = thd->check_for_truncated_fields;
 
   do {
     skip_subtree = JTS_NONE;
@@ -595,13 +640,13 @@ bool Table_function_json::fill_json_table() {
     }
   } while (nested.size() != 0 || skip_subtree != JTS_EOD);
 
-  current_thd->check_for_truncated_fields = check_save;
+  thd->check_for_truncated_fields = check_save;
   return false;
 }
 
 bool Table_function_json::fill_result_table() {
   String buf;
-  assert(!table->materialized);
+  DBUG_ASSERT(!table->materialized);
   // reset table
   empty_table();
 
@@ -617,15 +662,11 @@ bool Table_function_json::fill_result_table() {
       3. Non-const, e.g. a table field: source will be parsed here EVERY TIME
          fill_result_table() is called
     */
-    if (!source->const_item() || !is_source_parsed) {
-      if (get_json_wrapper(args, 0, &buf, func_name(), &m_jds[0].jdata)) {
-        return true;
-      }
-    }
-    if (args[0]->null_value) {
+    if (((!source->const_item() || !is_source_parsed) &&
+         get_json_wrapper(args, 0, &buf, func_name(), &m_jds[0].jdata)) ||
+        args[0]->null_value)
       // No need to set null_value as it's not used by table functions
       return false;
-    }
     is_source_parsed = true;
     return fill_json_table();
   } catch (...) {
@@ -637,50 +678,23 @@ bool Table_function_json::fill_result_table() {
   return false;
 }
 
-void print_on_empty_or_error(const THD *thd, String *str,
-                             enum_query_type query_type, bool on_empty,
-                             Json_on_response_type response_type,
-                             const Item *default_string) {
-  switch (response_type) {
-    case Json_on_response_type::ERROR:
-      str->append(STRING_WITH_LEN(" error"));
-      break;
-    case Json_on_response_type::NULL_VALUE:
-      str->append(STRING_WITH_LEN(" null"));
-      break;
-    case Json_on_response_type::DEFAULT:
-      str->append(STRING_WITH_LEN(" default "));
+static bool print_on_empty_error(const THD *thd, String *str,
+                                 enum_query_type query_type, enum_jtc_on jto,
+                                 const Item *default_string) {
+  switch (jto) {
+    case enum_jtc_on::JTO_ERROR:
+      return str->append(STRING_WITH_LEN(" error on"));
+    case enum_jtc_on::JTO_NULL:
+      return str->append(STRING_WITH_LEN(" null on"));
+    case enum_jtc_on::JTO_DEFAULT:
+      if (str->append(STRING_WITH_LEN(" default "))) return true;
       default_string->print(thd, str, query_type);
+      return str->append(STRING_WITH_LEN(" on"));
+    case enum_jtc_on::JTO_IMPLICIT:
       break;
-    case Json_on_response_type::IMPLICIT:
-      // Nothing to print when the clause was implicit.
-      return;
   };
-
-  if (on_empty)
-    str->append(STRING_WITH_LEN(" on empty"));
-  else
-    str->append(STRING_WITH_LEN(" on error"));
-}
-
-/// Prints the type of a column in a JSON_TABLE expression.
-static bool print_json_table_column_type(const Field *field, String *str) {
-  StringBuffer<STRING_BUFFER_USUAL_SIZE> type;
-  field->sql_type(type);
-  if (str->append(type)) return true;
-  if (field->has_charset()) {
-    // Append the character set.
-    if (str->append(STRING_WITH_LEN(" character set ")) ||
-        str->append(field->charset()->csname))
-      return true;
-    // Append the collation, if it is not the primary collation of the
-    // character set.
-    if ((field->charset()->state & MY_CS_PRIMARY) == 0 &&
-        (str->append(STRING_WITH_LEN(" collate ")) ||
-         str->append(field->charset()->m_coll_name)))
-      return true;
-  }
-  return false;
+  DBUG_ASSERT(false);
+  return true;
 }
 
 /**
@@ -714,13 +728,21 @@ static bool print_nested_path(const THD *thd, const TABLE *table,
       case enum_jt_column::JTC_PATH: {
         append_identifier(thd, str, jtc.field_name, strlen(jtc.field_name));
         if (str->append(' ')) return true;
-        if (table == nullptr) {
-          if (str->append(STRING_WITH_LEN("<column type not resolved yet>"))) {
+        const Field *field = table->field[jtc.m_field_idx];
+        StringBuffer<STRING_BUFFER_USUAL_SIZE> type;
+        field->sql_type(type);
+        if (str->append(type)) return true;
+        if (field->has_charset()) {
+          // Append the character set.
+          if (str->append(STRING_WITH_LEN(" character set ")) ||
+              str->append(field->charset()->csname))
+            return true;
+          // Append the collation, if it is not the primary collation of the
+          // character set.
+          if ((field->charset()->state & MY_CS_PRIMARY) == 0 &&
+              (str->append(STRING_WITH_LEN(" collate ")) ||
+               str->append(field->charset()->name)))
             return true;
-          }
-        } else if (print_json_table_column_type(table->field[jtc.m_field_idx],
-                                                str)) {
-          return true;
         }
         if (jtc.m_jtc_type == enum_jt_column::JTC_EXISTS) {
           if (str->append(STRING_WITH_LEN(" exists"))) return true;
@@ -728,12 +750,18 @@ static bool print_nested_path(const THD *thd, const TABLE *table,
         if (str->append(STRING_WITH_LEN(" path "))) return true;
         jtc.m_path_string->print(thd, str, query_type);
         if (jtc.m_jtc_type == enum_jt_column::JTC_EXISTS) break;
-        // ON EMPTY
-        print_on_empty_or_error(thd, str, query_type, /*on_empty=*/true,
-                                jtc.m_on_empty, jtc.m_default_empty_string);
-        // ON ERROR
-        print_on_empty_or_error(thd, str, query_type, /*on_empty=*/false,
-                                jtc.m_on_error, jtc.m_default_error_string);
+        if (jtc.m_on_empty != enum_jtc_on::JTO_IMPLICIT) {
+          if (print_on_empty_error(thd, str, query_type, jtc.m_on_empty,
+                                   jtc.m_default_empty_string) ||
+              str->append(STRING_WITH_LEN(" empty")))
+            return true;
+        }
+        if (jtc.m_on_error != enum_jtc_on::JTO_IMPLICIT) {
+          if (print_on_empty_error(thd, str, query_type, jtc.m_on_error,
+                                   jtc.m_default_error_string) ||
+              str->append(STRING_WITH_LEN(" error")))
+            return true;
+        }
         break;
       }
       case enum_jt_column::JTC_NESTED_PATH: {
@@ -747,11 +775,10 @@ static bool print_nested_path(const THD *thd, const TABLE *table,
   return str->append(')');
 }
 
-bool Table_function_json::print(const THD *thd, String *str,
-                                enum_query_type query_type) const {
+bool Table_function_json::print(String *str, enum_query_type query_type) const {
   if (str->append(STRING_WITH_LEN("json_table("))) return true;
   source->print(thd, str, query_type);
-  return (str->append(STRING_WITH_LEN(", ")) ||
+  return (thd->is_error() || str->append(STRING_WITH_LEN(", ")) ||
           print_nested_path(thd, table, m_columns->head(), query_type, str) ||
           str->append(')'));
 }
@@ -763,9 +790,13 @@ void Table_function_json::do_cleanup() {
   is_source_parsed = false;
   for (uint i = 0; i < MAX_NESTED_PATH; i++) m_jds[i].cleanup();
   for (uint i = 0; i < m_all_columns.size(); i++) m_all_columns[i]->cleanup();
+  m_all_columns.clear();
+  m_vt_list.empty();
 }
 
 void JT_data_source::cleanup() {
+  jdata = Json_wrapper();
   v.clear();
+  v.shrink_to_fit();
   producing_records = false;
 }
