@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2013, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -23,19 +23,17 @@
 
 #include "sql/table_trigger_dispatcher.h"
 
-#include <assert.h>
 #include <sys/types.h>
-
-#include <memory>
 #include <string>
 #include <utility>
 
+#include "m_ctype.h"
+#include "m_string.h"
 #include "my_alloc.h"
+#include "my_dbug.h"
 #include "my_sqlcommand.h"
-#include "mysql/strings/m_ctype.h"
 #include "sql/auth/auth_acls.h"
-#include "sql/auth/auth_common.h"        // check_global_access
-#include "sql/auth/sql_authorization.h"  // check_valid_definer
+#include "sql/auth/auth_common.h"  // check_global_access
 #include "sql/auth/sql_security_ctx.h"
 #include "sql/dd/cache/dictionary_client.h"
 #include "sql/dd/dd_trigger.h"  // dd::create_trigger
@@ -56,7 +54,6 @@
 #include "sql/thr_malloc.h"
 #include "sql/trigger.h"
 #include "sql/trigger_chain.h"
-#include "string_with_len.h"
 #include "thr_lock.h"
 
 namespace dd {
@@ -100,24 +97,41 @@ Table_trigger_dispatcher::~Table_trigger_dispatcher() {
 
   if (m_record1_field) {
     for (Field **fld_ptr = m_record1_field; *fld_ptr; fld_ptr++)
-      ::destroy_at(*fld_ptr);
+      destroy(*fld_ptr);
   }
 
   // Destroy trigger chains.
 
-  for (int i = 0; i < (int)TRG_EVENT_MAX; ++i) {
-    for (int j = 0; j < (int)TRG_ACTION_MAX; ++j) {
-      if (m_trigger_map[i][j] != nullptr) ::destroy_at(m_trigger_map[i][j]);
-    }
-  }
+  for (int i = 0; i < (int)TRG_EVENT_MAX; ++i)
+    for (int j = 0; j < (int)TRG_ACTION_MAX; ++j) destroy(m_trigger_map[i][j]);
 }
+
+/**
+  Create trigger for table.
+
+  @param      thd   thread context
+  @param[out] binlog_create_trigger_stmt
+                    well-formed CREATE TRIGGER statement for putting into binlog
+                    (after successful execution)
+
+  @note
+    - Assumes that trigger name is fully qualified.
+    - NULL-string means the following LEX_STRING instance:
+    { str = 0; length = 0 }.
+    - In other words, definer_user and definer_host should contain
+    simultaneously NULL-strings (non-SUID/old trigger) or valid strings
+    (SUID/new trigger).
+
+  @return Operation status.
+    @retval false Success
+    @retval true  Failure
+*/
+
 bool Table_trigger_dispatcher::create_trigger(
-    THD *thd, String *binlog_create_trigger_stmt, bool if_not_exists,
-    bool &already_exists) {
-  assert(m_subject_table);
-  assert(!already_exists);
+    THD *thd, String *binlog_create_trigger_stmt) {
+  DBUG_ASSERT(m_subject_table);
   LEX *lex = thd->lex;
-  const dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
+  dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
 
   // If this table has broken triggers, CREATE TRIGGER is not allowed.
   if (check_for_broken_triggers()) return true;
@@ -134,14 +148,20 @@ bool Table_trigger_dispatcher::create_trigger(
   if (thd->dd_client()->acquire(lex->spname->m_db.str, &sch_obj)) return true;
 
   // The table is already open, so the schema must exist.
-  assert(sch_obj != nullptr);
+  DBUG_ASSERT(sch_obj != nullptr);
 
   dd::String_type table_name;
   if (thd->dd_client()->get_table_name_by_trigger_name(
           *sch_obj, lex->spname->m_name.str, &table_name))
     return true;
 
+  if (table_name != "") {
+    my_error(ER_TRG_ALREADY_EXISTS, MYF(0));
+    return true;
+  }
+
   // Make sure DEFINER clause is specified.
+
   if (!lex->definer) {
     /*
       DEFINER-clause is missing.
@@ -171,40 +191,33 @@ bool Table_trigger_dispatcher::create_trigger(
     }
   }
 
-  if (table_name != "") {
-    // Trigger with the same name already exists in this schema.
-    if (if_not_exists) {
-      /*
-        IF NOT EXISTS clause is only supported for triggers associated with the
-        same table.
-      */
-      if (my_strcasecmp(table_alias_charset, m_subject_table->s->table_name.str,
-                        table_name.c_str())) {
-        my_error(ER_IF_NOT_EXISTS_UNSUPPORTED_TRG_EXISTS_ON_DIFFERENT_TABLE,
-                 MYF(0), m_subject_table->s->db.str, lex->spname->m_name.str);
-        return true;
-      }
-
-      // Trigger with the same name already exists on the same table.
-      already_exists = true;
-
-      push_warning_printf(
-          thd, Sql_condition::SL_NOTE, ER_WARN_TRG_ALREADY_EXISTS,
-          ER_THD(thd, ER_WARN_TRG_ALREADY_EXISTS), lex->spname->m_name.str,
-          m_subject_table->s->db.str, table_name.c_str());
-
-      /* SUID trigger is only supported (DEFINER is specified by the user). */
-      return (Trigger::construct_create_trigger_stmt_with_definer(
-          thd, binlog_create_trigger_stmt, lex->definer->user,
-          lex->definer->host));
+  /*
+    If the specified definer differs from the current user, we should check
+    that the current user has SUPER privilege (in order to create trigger
+    under another user one must have SUPER privilege).
+  */
+  Security_context *sctx = thd->security_context();
+  if (lex->definer &&
+      (strcmp(lex->definer->user.str, sctx->priv_user().str) ||
+       my_strcasecmp(system_charset_info, lex->definer->host.str,
+                     sctx->priv_host().str))) {
+    if (!sctx->check_access(SUPER_ACL) &&
+        !sctx->has_global_grant(STRING_WITH_LEN("SET_USER_ID")).first) {
+      my_error(ER_SPECIFIC_ACCESS_DENIED_ERROR, MYF(0), "SUPER or SET_USER_ID");
+      return true;
     }
-    my_error(ER_TRG_ALREADY_EXISTS, MYF(0));
-    return true;
+    if (sctx->can_operate_with({lex->definer}, consts::system_user, true))
+      return true;
   }
 
-  if (check_valid_definer(thd, lex->definer)) return true;
+  if (lex->definer &&
+      !is_acl_user(thd, lex->definer->host.str, lex->definer->user.str)) {
+    push_warning_printf(thd, Sql_condition::SL_NOTE, ER_NO_SUCH_USER,
+                        ER_THD(thd, ER_NO_SUCH_USER), lex->definer->user.str,
+                        lex->definer->host.str);
 
-  if (thd->get_stmt_da()->is_error()) return true;
+    if (thd->get_stmt_da()->is_error()) return true;
+  }
 
   /*
     Check if all references to fields in OLD/NEW-rows in this trigger are valid.
@@ -244,7 +257,7 @@ bool Table_trigger_dispatcher::create_trigger(
       &m_subject_table->mem_root, t->get_event(), t->get_action_time());
 
   if (!tc) {
-    ::destroy_at(t);
+    destroy(t);
     return true;
   }
 
@@ -253,7 +266,7 @@ bool Table_trigger_dispatcher::create_trigger(
   if (tc->add_trigger(&m_subject_table->mem_root, t,
                       lex->sphead->m_trg_chistics.ordering_clause,
                       lex->sphead->m_trg_chistics.anchor_trigger_name)) {
-    ::destroy_at(t);
+    destroy(t);
     return true;
   }
 
@@ -274,7 +287,7 @@ bool Table_trigger_dispatcher::create_trigger(
 bool Table_trigger_dispatcher::prepare_record1_accessors() {
   Field **fld, **old_fld;
 
-  assert(m_subject_table);
+  DBUG_ASSERT(m_subject_table);
 
   m_record1_field = (Field **)m_subject_table->mem_root.Alloc(
       (m_subject_table->s->fields + 1) * sizeof(Field *));
@@ -308,7 +321,7 @@ bool Table_trigger_dispatcher::prepare_record1_accessors() {
 */
 
 bool Table_trigger_dispatcher::check_n_load(THD *thd, const dd::Table &table) {
-  assert(m_subject_table);
+  DBUG_ASSERT(m_subject_table);
 
   // Load triggers from Data Dictionary.
 
@@ -371,8 +384,8 @@ bool Table_trigger_dispatcher::check_n_load(THD *thd, const dd::Table &table) {
 Trigger_chain *Table_trigger_dispatcher::create_trigger_chain(
     MEM_ROOT *mem_root, enum_trigger_event_type event,
     enum_trigger_action_time_type action_time) {
-  assert(event != TRG_EVENT_MAX);
-  assert(action_time != TRG_ACTION_MAX);
+  DBUG_ASSERT(event != TRG_EVENT_MAX);
+  DBUG_ASSERT(action_time != TRG_ACTION_MAX);
 
   Trigger_chain *tc = get_triggers(event, action_time);
 
@@ -437,7 +450,7 @@ void Table_trigger_dispatcher::parse_triggers(THD *thd, List<Trigger> *triggers,
 
     if (!t) break;
 
-    const bool fatal_parse_error = t->parse(thd, is_upgrade);
+    bool fatal_parse_error = t->parse(thd, is_upgrade);
 
     /*
       There are two kinds of parse errors here:
@@ -466,7 +479,7 @@ void Table_trigger_dispatcher::parse_triggers(THD *thd, List<Trigger> *triggers,
     */
 
     if (fatal_parse_error || t->has_parse_error()) {
-      assert(!t->get_sp());  // SP must be NULL.
+      DBUG_ASSERT(!t->get_sp());  // SP must be NULL.
 
       if (t->has_parse_error())
         set_parse_error_message(t->get_parse_error_message());
@@ -482,14 +495,14 @@ void Table_trigger_dispatcher::parse_triggers(THD *thd, List<Trigger> *triggers,
       }
 
       if (fatal_parse_error) {
-        ::destroy_at(t);
+        destroy(t);
         it.remove();
       }
 
       continue;
     }
 
-    assert(!t->has_parse_error());
+    DBUG_ASSERT(!t->has_parse_error());
 
     sp_head *sp = t->get_sp();
 
@@ -522,7 +535,7 @@ bool Table_trigger_dispatcher::process_triggers(
 
   if (!tc) return false;
 
-  assert(m_subject_table);
+  DBUG_ASSERT(m_subject_table);
 
   if (old_row_is_record1) {
     m_old_field = m_record1_field;
@@ -535,10 +548,10 @@ bool Table_trigger_dispatcher::process_triggers(
     This trigger must have been processed by the pre-locking
     algorithm.
   */
-  assert(m_subject_table->pos_in_table_list->trg_event_map &
-         static_cast<uint>(1 << static_cast<int>(event)));
+  DBUG_ASSERT(m_subject_table->pos_in_table_list->trg_event_map &
+              static_cast<uint>(1 << static_cast<int>(event)));
 
-  const bool rc = tc->execute_triggers(thd);
+  bool rc = tc->execute_triggers(thd);
 
   m_new_field = nullptr;
   m_old_field = nullptr;
@@ -561,9 +574,9 @@ bool Table_trigger_dispatcher::process_triggers(
 */
 
 bool Table_trigger_dispatcher::add_tables_and_routines_for_triggers(
-    THD *thd, Query_tables_list *prelocking_ctx, Table_ref *table_list) {
-  assert(static_cast<int>(table_list->lock_descriptor().type) >=
-         static_cast<int>(TL_WRITE_ALLOW_WRITE));
+    THD *thd, Query_tables_list *prelocking_ctx, TABLE_LIST *table_list) {
+  DBUG_ASSERT(static_cast<int>(table_list->lock_descriptor().type) >=
+              static_cast<int>(TL_WRITE_ALLOW_WRITE));
 
   for (int i = 0; i < (int)TRG_EVENT_MAX; ++i) {
     if (table_list->trg_event_map & static_cast<uint8>(1 << i)) {
@@ -585,7 +598,7 @@ bool Table_trigger_dispatcher::add_tables_and_routines_for_triggers(
 */
 
 void Table_trigger_dispatcher::enable_fields_temporary_nullability(THD *thd) {
-  assert(m_subject_table);
+  DBUG_ASSERT(m_subject_table);
 
   for (Field **next_field = m_subject_table->field; *next_field; ++next_field) {
     (*next_field)->set_tmp_nullable();
@@ -613,7 +626,7 @@ void Table_trigger_dispatcher::enable_fields_temporary_nullability(THD *thd) {
 */
 
 void Table_trigger_dispatcher::disable_fields_temporary_nullability() {
-  assert(m_subject_table);
+  DBUG_ASSERT(m_subject_table);
 
   for (Field **next_field = m_subject_table->field; *next_field; ++next_field)
     (*next_field)->reset_tmp_nullable();
@@ -661,7 +674,7 @@ void Table_trigger_dispatcher::print_upgrade_warnings(THD *thd) {
 bool Table_trigger_dispatcher::mark_fields(enum_trigger_event_type event) {
   if (check_for_broken_triggers()) return true;
 
-  assert(m_subject_table);
+  DBUG_ASSERT(m_subject_table);
 
   for (int i = 0; i < (int)TRG_ACTION_MAX; ++i) {
     Trigger_chain *tc = get_triggers(event, i);
