@@ -1,4 +1,4 @@
-/* Copyright (c) 2001, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2001, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -50,8 +50,8 @@
 #include "my_dbug.h"
 #include "my_io.h"
 #include "my_tree.h"  // element_count
-#include "mysql/components/services/bits/psi_bits.h"
 #include "mysql/psi/mysql_file.h"
+#include "mysql/psi/psi_base.h"
 #include "mysql/service_mysql_alloc.h"
 #include "priority_queue.h"
 #include "sql/field.h"    // Field
@@ -64,7 +64,7 @@
 #include "sql/sql_base.h"  // TEMP_PREFIX
 #include "sql/sql_class.h"
 #include "sql/sql_const.h"
-#include "sql/sql_executor.h"  // check_unique_fields
+#include "sql/sql_executor.h"  // check_unique_constraint
 #include "sql/sql_sort.h"
 #include "sql/sql_tmp_table.h"  // create_duplicate_weedout_tmp_table
 #include "sql/table.h"
@@ -137,7 +137,7 @@ static uint uniq_read_to_buffer(IO_CACHE *fromfile, Merge_chunk *merge_chunk,
 /**
   Merge buffers to one buffer.
 
-  @param thd            thread context
+  @param thd
   @param param          Sort parameter
   @param from_file      File with source data (Merge_chunks point to this file)
   @param to_file        File to write the sorted result data.
@@ -147,13 +147,15 @@ static uint uniq_read_to_buffer(IO_CACHE *fromfile, Merge_chunk *merge_chunk,
   @param chunk_array    Array of chunks to merge.
   @param flag           0 - write full record, 1 - write addon/ref
 
-  @retval 0      OK
-  @retval other  error
+  @returns
+    0      OK
+  @returns
+    other  error
 */
 static int merge_buffers(THD *thd, Uniq_param *param, IO_CACHE *from_file,
                          IO_CACHE *to_file, Sort_buffer sort_buffer,
                          Merge_chunk *last_chunk, Merge_chunk_array chunk_array,
-                         int flag [[maybe_unused]]) {
+                         int flag MY_ATTRIBUTE((unused))) {
   int error = 0;
   uint rec_length;
   ha_rows maxcount;
@@ -176,9 +178,9 @@ static int merge_buffers(THD *thd, Uniq_param *param, IO_CACHE *from_file,
   org_max_rows = max_rows = param->max_rows;
 
   /* The following will fire if there is not enough space in sort_buffer */
-  assert(maxcount != 0);
+  DBUG_ASSERT(maxcount != 0);
 
-  assert(param->unique_buff != nullptr);
+  DBUG_ASSERT(param->unique_buff != nullptr);
 
   cmp = param->compare;
   first_cmp_arg = &param->cmp_context;
@@ -191,8 +193,8 @@ static int merge_buffers(THD *thd, Uniq_param *param, IO_CACHE *from_file,
   Priority_queue<Merge_chunk *,
                  std::vector<Merge_chunk *, Malloc_allocator<Merge_chunk *>>,
                  decltype(greater)>
-      queue(greater,
-            Malloc_allocator<Merge_chunk *>(key_memory_Filesort_info_merge));
+  queue(greater,
+        Malloc_allocator<Merge_chunk *>(key_memory_Filesort_info_merge));
 
   if (queue.reserve(chunk_array.size())) return 1;
 
@@ -201,7 +203,7 @@ static int merge_buffers(THD *thd, Uniq_param *param, IO_CACHE *from_file,
     merge_chunk->set_buffer(
         strpos, strpos + (sort_buffer.size() / (chunk_array.size())));
     merge_chunk->set_max_keys(maxcount);
-    const uint bytes_read = uniq_read_to_buffer(from_file, merge_chunk, param);
+    uint bytes_read = uniq_read_to_buffer(from_file, merge_chunk, param);
 
     if (static_cast<int>(bytes_read) == -1) return -1; /* purecov: inspected */
 
@@ -412,10 +414,10 @@ static inline double log2_n_fact(ulong n) {
 
   SYNOPSIS
     get_merge_buffers_cost()
-      total_buf_elems   Number of elements to merge in all
-      n_buffers         Number of different buffers the elements are sprea dover
-      elem_size         Size of element stored in buffer
-      cost_model        Cost model to use for constants
+      buff_elems  Array of #s of elements in buffers
+      elem_size   Size of element stored in buffer
+      first       Pointer to first merged element size
+      last        Pointer to last merged element size
 
   RETURN
     Cost of merge_buffers operation in disk seeks.
@@ -436,9 +438,16 @@ static inline double log2_n_fact(ulong n) {
       key_compare_cost(total_buf_elems * log2(n_buffers));
 */
 
-static double get_merge_buffers_cost(size_t total_buf_elems, size_t n_buffers,
-                                     uint elem_size,
+static double get_merge_buffers_cost(Unique::Imerge_cost_buf_type buff_elems,
+                                     uint elem_size, uint first, uint last,
                                      const Cost_model_table *cost_model) {
+  uint total_buf_elems = 0;
+  for (uint pbuf = first; pbuf <= last; pbuf++)
+    total_buf_elems += buff_elems[pbuf];
+  buff_elems[last] = total_buf_elems;
+
+  const size_t n_buffers = last - first + 1;
+
   const double io_ops =
       static_cast<double>(total_buf_elems * elem_size) / IO_SIZE;
   const double io_cost = cost_model->io_block_read_cost(io_ops);
@@ -457,15 +466,16 @@ static double get_merge_buffers_cost(size_t total_buf_elems, size_t n_buffers,
 
   SYNOPSIS
     get_merge_many_buffs_cost()
-      num_full_buffers       # of full buffers
-      elems_per_buffer       # of elements in each full buffers
-      additional_num_elems   # of elements in last buffer
-      elem_size              size of each buffer element
+      buffer        buffer space for temporary data, at least
+                    Unique::get_cost_calc_buff_size bytes
+      maxbuffer     # of full buffers
+      max_n_elems   # of elements in first maxbuffer buffers
+      last_n_elems  # of elements in last buffer
+      elem_size     size of buffer element
 
   NOTES
-    num_full_buffers+1 buffers are merged, where first "num_full_buffers"
-    buffers contain "elems_per_buffer" elements each and last buffer contains
-    "additional_num_elems" elements.
+    maxbuffer+1 buffers are merged, where first maxbuffer buffers contain
+    max_n_elems elements each and last buffer contains last_n_elems elements.
 
     The current implementation does a dumb simulation of merge_many_buffs
     function actions.
@@ -474,52 +484,43 @@ static double get_merge_buffers_cost(size_t total_buf_elems, size_t n_buffers,
     Cost of merge in disk seeks.
 */
 
-static double get_merge_many_buffs_cost(uint num_full_buffers,
-                                        uint elems_per_buffer,
-                                        uint additional_num_elems,
-                                        int elem_size,
+static double get_merge_many_buffs_cost(Unique::Imerge_cost_buf_type buffer,
+                                        uint maxbuffer, uint max_n_elems,
+                                        uint last_n_elems, int elem_size,
                                         const Cost_model_table *cost_model) {
-  if (num_full_buffers == 0 && additional_num_elems == 0) {
-    return 0.0;
-  }
+  uint i;
   double total_cost = 0.0;
+  Unique::Imerge_cost_buf_type buff_elems =
+      buffer; /* #s of elements in each of merged sequences */
 
-  // For uniformity, make sure additional_num_elems > 0, always.
-  if (additional_num_elems == 0) {
-    additional_num_elems = elems_per_buffer;
-    --num_full_buffers;
-  }
+  /*
+    Set initial state: first maxbuffer sequences contain max_n_elems elements
+    each, last sequence contains last_n_elems elements.
+  */
+  for (i = 0; i < maxbuffer; i++) buff_elems[i] = max_n_elems;
+  buff_elems[maxbuffer] = last_n_elems;
 
   /*
     Do it exactly as merge_many_buff function does, calling
     get_merge_buffers_cost to get cost of merge_buffers.
   */
-  while ((num_full_buffers + 1) > MERGEBUFF2) {
-    uint new_num_full_buffers = 0;
-    unsigned i;
-    for (i = 0; i < (num_full_buffers + 1) - MERGEBUFF * 3U / 2U;
-         i += MERGEBUFF) {
-      ++new_num_full_buffers;
+  if (maxbuffer >= MERGEBUFF2) {
+    while (maxbuffer >= MERGEBUFF2) {
+      uint lastbuff = 0;
+      for (i = 0; i <= maxbuffer - MERGEBUFF * 3U / 2U; i += MERGEBUFF) {
+        total_cost += get_merge_buffers_cost(buff_elems, elem_size, i,
+                                             i + MERGEBUFF - 1, cost_model);
+        lastbuff++;
+      }
+      total_cost += get_merge_buffers_cost(buff_elems, elem_size, i, maxbuffer,
+                                           cost_model);
+      maxbuffer = lastbuff;
     }
-    total_cost += new_num_full_buffers *
-                  get_merge_buffers_cost(elems_per_buffer * MERGEBUFF,
-                                         MERGEBUFF, elem_size, cost_model);
-
-    const uint leftover_full_buffers = num_full_buffers - i;
-    total_cost += get_merge_buffers_cost(
-        elems_per_buffer * leftover_full_buffers + additional_num_elems,
-        leftover_full_buffers + 1, elem_size, cost_model);
-    additional_num_elems += elems_per_buffer * leftover_full_buffers;
-
-    elems_per_buffer *= MERGEBUFF;
-    num_full_buffers = new_num_full_buffers;
   }
 
-  // Merge the final buffers (<= MERGEBUFF2) in one big merge.
-  total_cost += get_merge_buffers_cost(
-      elems_per_buffer * num_full_buffers + additional_num_elems,
-      num_full_buffers + 1, elem_size, cost_model);
-
+  /* Simulate final merge_buff call. */
+  total_cost +=
+      get_merge_buffers_cost(buff_elems, elem_size, 0, maxbuffer, cost_model);
   return total_cost;
 }
 
@@ -529,6 +530,8 @@ static double get_merge_many_buffs_cost(uint num_full_buffers,
 
   SYNOPSIS
     Unique::get_use_cost()
+      buffer    space for temporary data, use Unique::get_cost_calc_buff_size
+                to get # bytes needed.
       nkeys     #of elements in Unique
       key_size  size of each elements in bytes
       max_in_memory_size amount of memory Unique will be allowed to use
@@ -568,8 +571,8 @@ static double get_merge_many_buffs_cost(uint num_full_buffers,
       these will be random seeks.
 */
 
-double Unique::get_use_cost(uint nkeys, uint key_size,
-                            ulonglong max_in_memory_size,
+double Unique::get_use_cost(Imerge_cost_buf_type buffer, uint nkeys,
+                            uint key_size, ulonglong max_in_memory_size,
                             const Cost_model_table *cost_model) {
   ulong max_elements_in_tree;
   ulong last_tree_elems;
@@ -604,8 +607,8 @@ double Unique::get_use_cost(uint nkeys, uint key_size,
             ceil(((double)key_size) * last_tree_elems / IO_SIZE);
 
   /* Cost of merge */
-  const double merge_cost =
-      get_merge_many_buffs_cost(n_full_trees, max_elements_in_tree,
+  double merge_cost =
+      get_merge_many_buffs_cost(buffer, n_full_trees, max_elements_in_tree,
                                 last_tree_elems, key_size, cost_model);
   if (merge_cost < 0.0) return merge_cost;
 
@@ -732,7 +735,7 @@ static bool merge_walk(uchar *merge_buffer, size_t merge_buffer_size,
       merge_buffer_size < (ulong)(key_length * (end - begin + 1)))
     return true;
 
-  const Merge_chunk_compare_context compare_context = {compare, compare_arg};
+  Merge_chunk_compare_context compare_context = {compare, compare_arg};
   Priority_queue<Merge_chunk *,
                  std::vector<Merge_chunk *, Malloc_allocator<Merge_chunk *>>,
                  Merge_chunk_less>
@@ -743,10 +746,10 @@ static bool merge_walk(uchar *merge_buffer, size_t merge_buffer_size,
   /* we need space for one key when a piece of merge buffer is re-read */
   merge_buffer_size -= key_length;
   uchar *save_key_buff = merge_buffer + merge_buffer_size;
-  const uint max_key_count_per_piece =
+  uint max_key_count_per_piece =
       (uint)(merge_buffer_size / (end - begin) / key_length);
   /* if piece_size is aligned reuse_freed_buffer will always hit */
-  const size_t piece_size = max_key_count_per_piece * key_length;
+  size_t piece_size = max_key_count_per_piece * key_length;
   uint bytes_read; /* to hold return value of uniq_read_to_buffer */
   Merge_chunk *top;
   int res = 1;
@@ -767,7 +770,7 @@ static bool merge_walk(uchar *merge_buffer, size_t merge_buffer_size,
     top->set_max_keys(max_key_count_per_piece);
     bytes_read = uniq_read_to_buffer(file, top, &uniq_param);
     if (bytes_read == (uint)(-1)) goto end;
-    assert(bytes_read);
+    DBUG_ASSERT(bytes_read);
     queue.push(top);
   }
   top = queue.top();
@@ -889,12 +892,11 @@ bool Unique::walk(tree_walk_action action, void *walk_action_arg) {
 */
 
 bool Unique::get(TABLE *table) {
-  THD *thd = current_thd;
   table->unique_result.found_records = elements + tree.elements_in_tree;
 
   if (my_b_tell(&file) == 0) {
     /* Whole tree is in memory;  Don't use disk if you don't need to */
-    assert(table->unique_result.sorted_result == nullptr);
+    DBUG_ASSERT(table->unique_result.sorted_result == nullptr);
     table->unique_result.sorted_result.reset(
         (uchar *)my_malloc(key_memory_Filesort_info_record_pointers,
                            size * tree.elements_in_tree, MYF(0)));
@@ -914,7 +916,7 @@ bool Unique::get(TABLE *table) {
   bool error = true;
 
   /* Open cached file if it isn't open */
-  assert(table->unique_result.io_cache == nullptr);
+  DBUG_ASSERT(table->unique_result.io_cache == nullptr);
   outfile = table->unique_result.io_cache = (IO_CACHE *)my_malloc(
       key_memory_TABLE_sort_io_cache, sizeof(IO_CACHE), MYF(MY_ZEROFILL));
 
@@ -944,14 +946,15 @@ bool Unique::get(TABLE *table) {
   uniq_param.cmp_context.key_compare_arg = tree.custom_arg;
 
   /* Merge the buffers to one file, removing duplicates */
-  if (merge_many_buff(thd, &uniq_param, Sort_buffer(sort_memory, num_bytes),
+  if (merge_many_buff(table->in_use, &uniq_param,
+                      Sort_buffer(sort_memory, num_bytes),
                       Merge_chunk_array(file_ptrs.begin(), file_ptrs.size()),
                       &num_chunks, &file))
     goto err;
   if (flush_io_cache(&file) ||
       reinit_io_cache(&file, READ_CACHE, 0L, false, false))
     goto err;
-  if (merge_buffers(thd, &uniq_param, &file, outfile,
+  if (merge_buffers(table->in_use, &uniq_param, &file, outfile,
                     Sort_buffer(sort_memory, num_bytes), file_ptr,
                     Merge_chunk_array(file_ptr, num_chunks), 0))
     goto err;
@@ -968,17 +971,15 @@ err:
 }
 
 bool Unique_on_insert::unique_add(void *ptr) {
-  THD *thd = current_thd;
   Field *key = *m_table->visible_field_ptr();
   if (key->store((const char *)ptr, m_size, &my_charset_bin) != TYPE_OK)
     return true; /* purecov: inspected */
-  if (!check_unique_fields(m_table)) return true;
-  const uint res = m_table->file->ha_write_row(m_table->record[0]);
+  if (!check_unique_constraint(m_table)) return true;
+  uint res = m_table->file->ha_write_row(m_table->record[0]);
   if (res) {
     bool dup = false;
     if (res == HA_ERR_FOUND_DUPP_KEY) return true;
-    if (create_ondisk_from_heap(thd, m_table, res, /*insert_last_record=*/true,
-                                /*ignore_last_dup=*/false, &dup) ||
+    if (create_ondisk_from_heap(m_table->in_use, m_table, res, false, &dup) ||
         dup)
       return true;
   }
@@ -1002,6 +1003,5 @@ bool Unique_on_insert::init() {
 
 void Unique_on_insert::cleanup() {
   reset(false);
-  close_tmp_table(m_table);
-  free_tmp_table(m_table);
+  free_tmp_table(m_table->in_use, m_table);
 }
