@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2004, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2004, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -31,7 +30,6 @@
 #include <mysql/plugin.h>
 
 #include "lex_string.h"
-#include "m_string.h"
 #include "my_byteorder.h"
 #include "my_compiler.h"
 #include "my_dbug.h"
@@ -41,7 +39,6 @@
 #include "mysql/plugin.h"
 #include "mysql/psi/mysql_file.h"
 #include "mysql/psi/mysql_memory.h"
-#include "nulls.h"
 #include "sql/derror.h"
 #include "sql/field.h"
 #include "sql/sql_class.h"
@@ -82,7 +79,7 @@
 
   At some point a recovery method for such a drastic case needs to be divised.
 
-  Locks are row level, and you will get a consistent read.
+  Locks are row level, and you will get a consistant read.
 
   For performance as far as table scans go it is quite fast. I don't have
   good numbers but locally it has out performed both Innodb and MyISAM. For
@@ -191,8 +188,8 @@ static PSI_memory_info all_archive_memory[] = {
 #endif /* HAVE_PSI_MEMORY_INTERFACE */
 
 static void init_archive_psi_keys(void) {
-  const char *category [[maybe_unused]] = "archive";
-  int count [[maybe_unused]];
+  const char *category MY_ATTRIBUTE((unused)) = "archive";
+  int count MY_ATTRIBUTE((unused));
 
 #ifdef HAVE_PSI_MUTEX_INTERFACE
   count = static_cast<int>(array_elements(all_archive_mutexes));
@@ -360,7 +357,7 @@ unsigned int ha_archive::pack_row_v1(uchar *record) {
     Field_blob *field = down_cast<Field_blob *>(table->field[*blob]);
     const uint32 length = field->get_length();
     if (length) {
-      memcpy(pos, field->get_blob_data(), length);
+      memcpy(pos, field->get_ptr(), length);
       pos += length;
     }
   }
@@ -461,7 +458,7 @@ Archive_share *ha_archive::get_share(const char *table_name, int *rc) {
 err:
   unlock_shared_ha_data();
 
-  assert(tmp_share || *rc);
+  DBUG_ASSERT(tmp_share || *rc);
 
   return tmp_share;
 }
@@ -471,7 +468,7 @@ int Archive_share::init_archive_writer() {
   /*
     It is expensive to open and close the data files and since you can't have
     a gzip file that can be both read and written we keep a writer open
-    that is shared among all open tables.
+    that is shared amoung all open tables.
   */
   if (!(azopen(&archive_write, data_file_name, O_RDWR))) {
     DBUG_PRINT("ha_archive", ("Could not open archive write file"));
@@ -501,7 +498,7 @@ int ha_archive::init_archive_reader() {
   /*
     It is expensive to open and close the data files and since you can't have
     a gzip file that can be both read and written we keep a writer open
-    that is shared among all open tables, but have one reader open for
+    that is shared amoung all open tables, but have one reader open for
     each handler instance.
   */
   if (!archive_reader_open) {
@@ -539,7 +536,7 @@ int ha_archive::open(const char *name, int, uint open_options,
       break;
     case HA_ERR_CRASHED_ON_USAGE:
       if (open_options & HA_OPEN_FOR_REPAIR) break;
-      [[fallthrough]];
+      /* fall through */
     default:
       return rc;
   }
@@ -619,7 +616,7 @@ int ha_archive::create(const char *name, TABLE *table_arg,
     for (; key_part != key_part_end; key_part++) {
       Field *field = key_part->field;
 
-      if (!field->is_flag_set(AUTO_INCREMENT_FLAG)) {
+      if (!(field->flags & AUTO_INCREMENT_FLAG)) {
         error = -1;
         DBUG_PRINT("ha_archive", ("Index error in creating archive table"));
         goto error;
@@ -762,7 +759,8 @@ unsigned int ha_archive::pack_row(uchar *record, azio_stream *writer) {
   ptr = record_buffer->buffer + table->s->null_bytes + ARCHIVE_ROW_HEADER_SIZE;
 
   for (Field **field = table->field; *field; field++) {
-    if (!((*field)->is_null())) ptr = (*field)->pack(ptr);
+    if (!((*field)->is_null()))
+      ptr = (*field)->pack(ptr, record + (*field)->offset(record));
   }
 
   int4store(record_buffer->buffer,
@@ -803,7 +801,7 @@ int ha_archive::write_row(uchar *buf) {
   if (table->next_number_field && record == table->record[0]) {
     KEY *mkey = &table->s->key_info[0];  // We only support one key right now
     update_auto_increment();
-    temp_auto = (table->next_number_field->is_unsigned() ||
+    temp_auto = (((Field_num *)table->next_number_field)->unsigned_flag ||
                          table->next_number_field->val_int() > 0
                      ? table->next_number_field->val_int()
                      : 0);
@@ -957,7 +955,7 @@ int ha_archive::get_row(azio_stream *file_to_read, uchar *buf) {
 bool ha_archive::fix_rec_buff(unsigned int length) {
   DBUG_TRACE;
   DBUG_PRINT("ha_archive", ("Fixing %u for %u", length, record_buffer->length));
-  assert(record_buffer->buffer);
+  DBUG_ASSERT(record_buffer->buffer);
 
   if (length > record_buffer->length) {
     uchar *newptr;
@@ -969,7 +967,7 @@ bool ha_archive::fix_rec_buff(unsigned int length) {
     record_buffer->length = length;
   }
 
-  assert(length <= record_buffer->length);
+  DBUG_ASSERT(length <= record_buffer->length);
 
   return false;
 }
@@ -998,7 +996,7 @@ int ha_archive::unpack_row(azio_stream *file_to_read, uchar *record) {
   if (fix_rec_buff(row_len)) {
     return HA_ERR_OUT_OF_MEM;
   }
-  assert(row_len <= record_buffer->length);
+  DBUG_ASSERT(row_len <= record_buffer->length);
 
   read = azread(file_to_read, record_buffer->buffer, row_len, &error);
 
@@ -1023,8 +1021,7 @@ int ha_archive::unpack_row(azio_stream *file_to_read, uchar *record) {
   ptr += table->s->null_bytes;
   for (Field **field = table->field; *field; field++) {
     if (!((*field)->is_null_in_record(record))) {
-      ptr =
-          (*field)->unpack(record + (*field)->offset(table->record[0]), ptr, 0);
+      ptr = (*field)->unpack(record + (*field)->offset(table->record[0]), ptr);
     }
   }
   return 0;
@@ -1069,7 +1066,7 @@ int ha_archive::get_row_version2(azio_stream *file_to_read, uchar *buf) {
   for (ptr = table->s->blob_field, end = ptr + table->s->blob_fields;
        ptr != end; ptr++) {
     if (bitmap_is_set(read_set,
-                      (((Field_blob *)table->field[*ptr])->field_index())))
+                      (((Field_blob *)table->field[*ptr])->field_index)))
       total_blob_length += ((Field_blob *)table->field[*ptr])->get_length();
   }
 
@@ -1083,7 +1080,7 @@ int ha_archive::get_row_version2(azio_stream *file_to_read, uchar *buf) {
     size_t size = ((Field_blob *)table->field[*ptr])->get_length();
     if (size) {
       if (bitmap_is_set(read_set,
-                        ((Field_blob *)table->field[*ptr])->field_index())) {
+                        ((Field_blob *)table->field[*ptr])->field_index)) {
         read = azread(file_to_read, const_cast<char *>(last), size, &error);
 
         if (error) return HA_ERR_CRASHED_ON_USAGE;
@@ -1233,7 +1230,7 @@ int ha_archive::optimize(THD *, HA_CHECK_OPT *check_opt) {
 
   table->copy_blobs = true;
 
-  /* read rows up to the remembered rows */
+  /* read rows upto the remembered rows */
   for (ha_rows cur_count = count; cur_count; cur_count--) {
     if ((rc = get_row(&archive, table->record[0]))) break;
     real_write_row(table->record[0], &writer);
@@ -1353,7 +1350,7 @@ int ha_archive::info(uint flag) {
   mysql_mutex_lock(&share->mutex);
   if (share->dirty) {
     DBUG_PRINT("ha_archive", ("archive flushing out rows for scan"));
-    assert(share->archive_write_open);
+    DBUG_ASSERT(share->archive_write_open);
     azflush(&(share->archive_write), Z_SYNC_FLUSH);
     share->dirty = false;
   }
@@ -1413,7 +1410,7 @@ int ha_archive::info(uint flag) {
     @return != 0 Error
 */
 
-int ha_archive::extra(enum ha_extra_function operation [[maybe_unused]]) {
+int ha_archive::extra(enum ha_extra_function operation MY_ATTRIBUTE((unused))) {
   int ret = 0;
   DBUG_TRACE;
   /* On windows we need to close all files before rename/delete. */
