@@ -1,4 +1,4 @@
-/* Copyright (c) 2004, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2004, 2020, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -61,10 +61,8 @@ TODO:
 #include "my_psi_config.h"
 #include "mysql/plugin.h"
 #include "mysql/psi/mysql_memory.h"
-#include "sql/derror.h"
 #include "sql/field.h"
 #include "sql/sql_class.h"
-#include "sql/sql_lex.h"
 #include "sql/system_variables.h"
 #include "sql/table.h"
 #include "template_utils.h"
@@ -104,11 +102,6 @@ static unique_ptr<collation_unordered_multimap<string, TINA_SHARE *>>
 static handler *tina_create_handler(handlerton *hton, TABLE_SHARE *table,
                                     bool partitioned, MEM_ROOT *mem_root);
 
-/* Interface to mysqld, to check system tables supported by SE */
-static bool tina_is_supported_system_table(const char *db,
-                                           const char *table_name,
-                                           bool is_sql_layer_system_table);
-
 /*****************************************************************************
  ** TINA tables
  *****************************************************************************/
@@ -183,7 +176,6 @@ static int tina_init_func(void *p) {
       (HTON_CAN_RECREATE | HTON_SUPPORT_LOG_TABLES | HTON_NO_PARTITION);
   tina_hton->file_extensions = ha_tina_exts;
   tina_hton->rm_tmp_tables = default_rm_tmp_tables;
-  tina_hton->is_supported_system_table = tina_is_supported_system_table;
   return 0;
 }
 
@@ -473,39 +465,6 @@ static handler *tina_create_handler(handlerton *hton, TABLE_SHARE *table, bool,
   return new (mem_root) ha_tina(hton, table);
 }
 
-/**
-  @brief Check if the given db.tablename is a system table for this SE.
-
-  @param db                         database name.
-  @param table_name                 table name to check.
-  @param is_sql_layer_system_table  if the supplied db.table_name is a SQL
-                                    layer system table.
-
-  @retval true   Given db.table_name is supported system table.
-  @retval false  Given db.table_name is not a supported system table.
-*/
-static bool tina_is_supported_system_table(const char *db,
-                                           const char *table_name,
-                                           bool is_sql_layer_system_table) {
-  /*
-   Currently CSV does not support any other SE specific system tables. If
-   in future it does, please see ha_example.cc for reference implementation
-  */
-  if (!is_sql_layer_system_table) return false;
-
-  // Creating system tables in this SE is allowed to support logical upgrade.
-  THD *thd = current_thd;
-  if (thd->lex->sql_command == SQLCOM_CREATE_TABLE) {
-    push_warning_printf(thd, Sql_condition::SL_WARNING, ER_UNSUPPORTED_ENGINE,
-                        ER_THD(thd, ER_UNSUPPORTED_ENGINE), "CSV", db,
-                        table_name);
-    return true;
-  }
-
-  // Any other usage is not allowed.
-  return false;
-}
-
 ha_tina::ha_tina(handlerton *hton, TABLE_SHARE *table_arg)
     : handler(hton, table_arg),
       /*
@@ -643,6 +602,9 @@ int ha_tina::find_current_row(uchar *buf) {
   bool read_all;
   DBUG_TRACE;
 
+  // Clear BLOB data from the previous row.
+  blobroot.ClearForReuse();
+
   /*
     We do not read further then local_saved_data_file_length in order
     not to conflict with undergoing concurrent insert.
@@ -652,9 +614,6 @@ int ha_tina::find_current_row(uchar *buf) {
       0)
     return HA_ERR_END_OF_FILE;
 
-  // Clear BLOB data from the previous row.
-  blobroot.ClearForReuse();
-
   /* We must read all columns in case a table is opened for update */
   read_all = !bitmap_is_clear_all(table->write_set);
   /* Avoid asserts in ::store() for columns that are not going to be updated */
@@ -769,7 +728,7 @@ int ha_tina::find_current_row(uchar *buf) {
       }
     }
 
-    if (read_all || bitmap_is_set(table->read_set, (*field)->field_index())) {
+    if (read_all || bitmap_is_set(table->read_set, (*field)->field_index)) {
       bool is_enum = ((*field)->real_type() == MYSQL_TYPE_ENUM);
       /*
         Here CHECK_FIELD_WARN checks that all values in the csv file are valid
@@ -779,13 +738,13 @@ int ha_tina::find_current_row(uchar *buf) {
         Thus, for enums we silence the warning, as it doesn't really mean
         an invalid value.
       */
-      if ((*field)->store(buffer.ptr(), buffer.length(), (*field)->charset(),
+      if ((*field)->store(buffer.ptr(), buffer.length(), buffer.charset(),
                           is_enum ? CHECK_FIELD_IGNORE : CHECK_FIELD_WARN)) {
         if (!is_enum) goto err;
       }
-      if ((*field)->is_flag_set(BLOB_FLAG)) {
+      if ((*field)->flags & BLOB_FLAG) {
         Field_blob *blob_field = down_cast<Field_blob *>(*field);
-        size_t length = blob_field->get_length();
+        size_t length = blob_field->get_length(blob_field->ptr);
         // BLOB data is not stored inside buffer. It only contains a
         // pointer to it. Copy the BLOB data into a separate memory
         // area so that it is not overwritten by subsequent calls to
@@ -793,7 +752,7 @@ int ha_tina::find_current_row(uchar *buf) {
         if (length > 0) {
           unsigned char *new_blob = new (&blobroot) unsigned char[length];
           if (new_blob == nullptr) return HA_ERR_OUT_OF_MEM;
-          memcpy(new_blob, blob_field->get_blob_data(), length);
+          memcpy(new_blob, blob_field->get_ptr(), length);
           blob_field->set_ptr(length, new_blob);
         }
       }
@@ -916,7 +875,7 @@ int ha_tina::open(const char *name, int, uint open_options, const dd::Table *) {
 }
 
 /*
-  Close a database file. We remove ourselves from the shared structure.
+  Close a database file. We remove ourselves from the shared strucutre.
   If it is empty we destroy it.
 */
 int ha_tina::close(void) {
@@ -1016,7 +975,7 @@ int ha_tina::update_row(const uchar *, uchar *new_data) {
   rc = 0;
 
   /* UPDATE should never happen on the log tables */
-  assert(!share->is_log_table);
+  DBUG_ASSERT(!share->is_log_table);
 
 err:
   DBUG_PRINT("info", ("rc = %d", rc));
@@ -1040,13 +999,13 @@ int ha_tina::delete_row(const uchar *) {
 
   stats.records--;
   /* Update shared info */
-  assert(share->rows_recorded);
+  DBUG_ASSERT(share->rows_recorded);
   mysql_mutex_lock(&share->mutex);
   share->rows_recorded--;
   mysql_mutex_unlock(&share->mutex);
 
   /* DELETE should never happen on the log table */
-  assert(!share->is_log_table);
+  DBUG_ASSERT(!share->is_log_table);
 
   return 0;
 }
@@ -1253,9 +1212,9 @@ int ha_tina::rnd_end() {
 
     /*
       The sort is needed when there were updates/deletes with random orders.
-      It sorts so that we move the first blocks to the beginning.
+      It sorts so that we move the firts blocks to the beginning.
 
-      We assume that intervals do not intersect. So, it is enough to compare
+      We assume that intervals do not intersect. So, it is enought to compare
       any two points. Here we take start of intervals for comparison.
     */
     std::sort(chain, chain_ptr, [](const tina_set &a, const tina_set &b) {
