@@ -1,18 +1,17 @@
 /*****************************************************************************
 
-Copyright (c) 2018, 2024, Oracle and/or its affiliates.
+Copyright (c) 2018, 2020, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is designed to work with certain software (including
-but not limited to OpenSSL) that is licensed under separate terms,
-as designated in a particular file or component or in included license
-documentation.  The authors of MySQL hereby grant you an additional
-permission to link the program and your derivative works with the
-separately licensed software that they have either included with
-the program or referenced in the documentation.
+This program is also distributed with certain software (including but not
+limited to OpenSSL) that is licensed under separate terms, as designated in a
+particular file or component or in included license documentation. The authors
+of MySQL hereby grant you an additional permission to link the program and
+your derivative works with the separately licensed software that they have
+included with MySQL.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -32,7 +31,6 @@ this program; if not, write to the Free Software Foundation, Inc.,
 #include "clone0repl.h"
 #include "clone0api.h"
 #include "clone0clone.h"
-#include "mysql/gtid/gtid.h"
 #include "sql/field.h"
 #include "sql/mysqld.h"
 #include "sql/rpl_gtid_persist.h"
@@ -51,23 +49,12 @@ void Clone_persist_gtid::add(const Gtid_desc &gtid_desc) {
   if (!is_active() || gtid_table_persistor == nullptr) {
     return;
   }
-  ut_ad(trx_sys_serialisation_mutex_own());
-
-  /* If too many GTIDs are accumulated, wait for all to get flushed. Ignore
-  timeout and loop to avoid possible hang. The insert should already be
-  slowed down by the wait here. */
-  if (check_max_gtid_threshold() && is_thread_active()) {
-    trx_sys_serialisation_mutex_exit();
-    wait_flush(false, false, nullptr);
-    trx_sys_serialisation_mutex_enter();
-  }
-
-  ut_ad(trx_sys_serialisation_mutex_own());
+  ut_ad(trx_sys_mutex_own());
   /* Get active GTID list */
   auto &current_gtids = get_active_list();
 
   /* Add input GTID to the set */
-  current_gtids.push_back(gtid_desc);
+  current_gtids.push_back(gtid_desc.m_info);
   /* Atomic increment. */
   int current_value = ++m_num_gtid_mem;
 
@@ -75,34 +62,14 @@ void Clone_persist_gtid::add(const Gtid_desc &gtid_desc) {
   if (current_value == s_gtid_threshold) {
     os_event_set(m_event);
   }
-
-  DBUG_EXECUTE_IF("dont_compress_gtid_table", {
-    /* For predictable outcome of mtr test we flush the GTID immediately. */
-    trx_sys_serialisation_mutex_exit();
-    wait_flush(false, false, nullptr);
-    trx_sys_serialisation_mutex_enter();
-  });
 }
 
-trx_undo_t::Gtid_storage Clone_persist_gtid::persists_gtid(const trx_t *trx) {
+bool Clone_persist_gtid::persists_gtid(const trx_t *trx) {
   auto thd = trx->mysql_thd;
-
   if (thd == nullptr) {
     thd = thd_get_current_thd();
   }
-
-  if (thd == nullptr || !thd->se_persists_gtid()) {
-    /* No need to persist GTID. */
-    return trx_undo_t::Gtid_storage::NONE;
-  }
-
-  if (thd->is_extrenal_xa()) {
-    /* Need to persist both XA prepare and commit GTID. */
-    return trx_undo_t::Gtid_storage::PREPARE_AND_COMMIT;
-  }
-
-  /* Need to persist only commit GTID. */
-  return trx_undo_t::Gtid_storage::COMMIT;
+  return (thd != nullptr && thd->se_persists_gtid());
 }
 
 void Clone_persist_gtid::set_persist_gtid(trx_t *trx, bool set) {
@@ -132,6 +99,14 @@ void Clone_persist_gtid::set_persist_gtid(trx_t *trx, bool set) {
     return;
   }
 
+  /* If GTID table is updated directly, don't add GTID for the transaction.
+  Try flushing in memory ones. */
+  if (thd->is_operating_gtid_table_implicitly) {
+    /* Trigger flush, don't wait; we could be in the middle of operation. */
+    os_event_set(m_event);
+    return;
+  }
+
   /* This is an optimization to skip GTID allocation, if transaction
   is guaranteed to not have GTID. */
   if (!thd->se_persists_gtid()) {
@@ -159,15 +134,13 @@ void Clone_persist_gtid::set_persist_gtid(trx_t *trx, bool set) {
   thd->set_gtid_persisted_by_se();
 }
 
-bool Clone_persist_gtid::trx_check_set(trx_t *trx, bool prepare, bool rollback,
-                                       bool &set_explicit) {
+bool Clone_persist_gtid::trx_check_set(trx_t *trx, bool prepare,
+                                       bool rollback) {
   auto thd = trx->mysql_thd;
   bool alloc_check = false;
 
   bool gtid_exists = has_gtid(trx, thd, alloc_check);
 
-  set_explicit = false;
-
   if (prepare) {
     /* Check for XA prepare. */
     gtid_exists = check_gtid_prepare(thd, trx, gtid_exists, alloc_check);
@@ -177,7 +150,7 @@ bool Clone_persist_gtid::trx_check_set(trx_t *trx, bool prepare, bool rollback,
     alloc_check = gtid_exists;
   } else {
     /* Check for Commit. */
-    gtid_exists = check_gtid_commit(thd, gtid_exists, set_explicit);
+    gtid_exists = check_gtid_commit(thd, gtid_exists);
     alloc_check = gtid_exists;
   }
   /* Set transaction to persist GTID. This is one single point of decision
@@ -202,12 +175,10 @@ bool Clone_persist_gtid::check_gtid_prepare(THD *thd, trx_t *trx,
   auto thd_trx = thd->get_transaction();
   auto xid_state = thd_trx->xid_state();
   /* In permissive mode GTID could be assigned during XA commit/rollback. */
-  if (xid_state->has_state(XID_STATE::XA_IDLE)) {
-    auto gtid_mode = global_gtid_mode.get();
-    if (gtid_mode == Gtid_mode::ON_PERMISSIVE ||
-        gtid_mode == Gtid_mode::OFF_PERMISSIVE) {
-      alloc = true;
-    }
+  if (xid_state->has_state(XID_STATE::XA_IDLE) &&
+      (get_gtid_mode(GTID_MODE_LOCK_NONE) == GTID_MODE_ON_PERMISSIVE ||
+       get_gtid_mode(GTID_MODE_LOCK_NONE) == GTID_MODE_OFF_PERMISSIVE)) {
+    alloc = true;
   }
   /* Skip GTID if not set */
   if (!found_gtid) {
@@ -215,8 +186,8 @@ bool Clone_persist_gtid::check_gtid_prepare(THD *thd, trx_t *trx,
   }
   /* Skip GTID if External XA transaction is not in IDLE state. */
   if (!xid_state->has_state(XID_STATE::XA_IDLE)) {
-    ut_d(ut_error);
-    ut_o(return (false));
+    ut_ad(false);
+    return (false);
   }
   /* Skip if SE is not set to persist GTID.*/
   if (!thd->se_persists_gtid()) {
@@ -226,15 +197,10 @@ bool Clone_persist_gtid::check_gtid_prepare(THD *thd, trx_t *trx,
   return (true);
 }
 
-bool Clone_persist_gtid::check_gtid_commit(THD *thd, bool found_gtid,
-                                           bool &set_explicit) {
-  set_explicit = (thd == nullptr) ? false : thd->se_persists_gtid_explicit();
-
+bool Clone_persist_gtid::check_gtid_commit(THD *thd, bool found_gtid) {
   if (!found_gtid) {
-    ut_ad(!set_explicit || thd->is_attachable_transaction_active());
     return (false);
   }
-
   /* Persist if SE is set to persist GTID.*/
   return (thd->se_persists_gtid());
 }
@@ -244,11 +210,8 @@ bool Clone_persist_gtid::check_gtid_rollback(THD *thd, trx_t *trx,
   if (!found_gtid) {
     return (false);
   }
-
-  ut_ad(trx_can_be_handled_by_current_thread_or_is_hp_victim(trx));
-
   /* Skip GTID if XA transaction not in prepared state. */
-  if (trx->state.load(std::memory_order_relaxed) != TRX_STATE_PREPARED) {
+  if (trx->state != TRX_STATE_PREPARED) {
     return (false);
   }
 
@@ -279,13 +242,8 @@ bool Clone_persist_gtid::check_gtid_rollback(THD *thd, trx_t *trx,
 
 bool Clone_persist_gtid::has_gtid(trx_t *trx, THD *&thd, bool &passed_check) {
   passed_check = false;
-
-  /* Note, that the assertion does nothing useful if thd == nullptr. */
-  ut_ad(trx_can_be_handled_by_current_thread_or_is_hp_victim(trx));
-
   /* Transaction is not associated with mysql foreground session. */
-  if (trx->state.load(std::memory_order_relaxed) == TRX_STATE_PREPARED &&
-      thd == nullptr) {
+  if (trx->state == TRX_STATE_PREPARED && thd == nullptr) {
     /* For XA transaction, the current transaction THD could be NULL. Also
     check the default THD of current thread. */
     thd = thd_get_current_thd();
@@ -299,29 +257,12 @@ bool Clone_persist_gtid::has_gtid(trx_t *trx, THD *&thd, bool &passed_check) {
     return (false);
   }
 
-  /* Attachable transactions can be started and committed while
-  the main transaction is in progress. We don't want to consider
-  GTID persistence for such transactions. */
-  if (thd->is_attachable_transaction_active()) {
+  /* Transaction is updating GTID table implicitly. */
+  if (thd->is_operating_gtid_table_implicitly ||
+      thd->is_operating_substatement_implicitly ||
+      thd->is_attachable_transaction_active()) {
     return (false);
   }
-
-  /* Explicit request is made while slave threads wants
-  to persist GTID for non innodb table. */
-  bool explicit_request = thd->se_persists_gtid_explicit();
-
-  if (!explicit_request) {
-    /* Transaction is updating GTID table implicitly. */
-    if (thd->is_operating_gtid_table_implicitly ||
-        thd->is_operating_substatement_implicitly) {
-      /* On slave, explicit request can be set after making some
-      modification and thus allocating undo. Allow space for GTID in
-      undo log for sub-statements always. */
-      passed_check = thd->is_operating_substatement_implicitly;
-      return (false);
-    }
-  }
-
   /* Transaction passed checks other than GTID. */
   passed_check = true;
 
@@ -343,33 +284,23 @@ void Clone_persist_gtid::get_gtid_info(trx_t *trx, Gtid_desc &gtid_desc) {
   auto thd = trx->mysql_thd;
 
   if (!has_gtid(trx, thd, thd_check)) {
-    ut_d(ut_error);
-    ut_o(return );
+    ut_ad(false);
+    return;
   }
 
-  uint32_t encoded_version = GTID_VERSION;
-  DBUG_EXECUTE_IF("gtid_persistor_use_gtid_version_one", encoded_version = 1;);
-  gtid_desc.m_version = encoded_version;
-  const auto &trx_gtid = thd->owned_gtid;
-  const auto &trx_tsid = thd->owned_tsid;
+  gtid_desc.m_version = GTID_VERSION;
+  auto &trx_gtid = thd->owned_gtid;
+  auto &trx_sid = thd->owned_sid;
 
   ut_ad(trx_gtid.sidno > 0);
   ut_ad(trx_gtid.gno > 0);
-  ut_ad(trx_gtid.gno < GNO_END);
 
   /* Build GTID string. */
   gtid_desc.m_info.fill(0);
   auto char_buf = reinterpret_cast<char *>(&gtid_desc.m_info[0]);
-  mysql::gtid::Gtid tsid_gtid(trx_tsid, trx_gtid.gno);
-  std::size_t len = 0;
-  if (unlikely(encoded_version ==
-               1)) {  // encode version 1, for debug purposes only
-    len = trx_gtid.to_string(trx_tsid, char_buf);
-  } else {  // encode version 2
-    len = tsid_gtid.encode_gtid_tagged(
-        reinterpret_cast<unsigned char *>(char_buf));
-  }
+  auto len = trx_gtid.to_string(trx_sid, char_buf);
   ut_a((size_t)len <= GTID_INFO_SIZE);
+
   gtid_desc.m_is_set = true;
 }
 
@@ -382,26 +313,19 @@ int Clone_persist_gtid::write_other_gtids() {
 }
 
 bool Clone_persist_gtid::check_compress() {
-  /* Check for explicit flush request. */
-  if (m_explicit_request.load()) {
+  /* Check local threshold on number of flush. */
+  if (m_compression_counter >= s_compression_threshold) {
     return (true);
   }
-
-  /* Wait for explicit request when debug compress request is set.
-  This is to make the debug test outcome predictable. */
-  DBUG_EXECUTE_IF("compress_gtid_table", { return false; });
-
   /* Check replication global threshold on number of GTIDs. */
   if (!opt_bin_log && gtid_executed_compression_period != 0 &&
       m_compression_gtid_counter > gtid_executed_compression_period) {
     return (true);
   }
-
-  /* Check local threshold on number of flush. */
-  if (m_compression_counter >= s_compression_threshold) {
+  /* Check for explicit flush request. */
+  if (m_explicit_request.load()) {
     return (true);
   }
-
   return (false);
 }
 
@@ -418,9 +342,9 @@ bool Clone_persist_gtid::debug_skip_write(bool compression) {
 
 int Clone_persist_gtid::write_to_table(uint64_t flush_list_number,
                                        Gtid_set &table_gtid_set,
-                                       Tsid_map &tsid_map) {
+                                       Sid_map &sid_map) {
   int err = 0;
-  Gtid_set write_gtid_set(&tsid_map, nullptr);
+  Gtid_set write_gtid_set(&sid_map, nullptr);
 
   /* Allocate some intervals from stack */
   static const int PREALLOCATED_INTERVAL_COUNT = 64;
@@ -429,21 +353,9 @@ int Clone_persist_gtid::write_to_table(uint64_t flush_list_number,
 
   auto &flush_list = get_list(flush_list_number);
   /* Extract GTIDs from flush list. */
-  for (auto &gtid_desc : flush_list) {
-    auto status = RETURN_STATUS_UNREPORTED_ERROR;
-    if (gtid_desc.m_version == 1) {
-      auto gtid_str = reinterpret_cast<const char *>(&(gtid_desc.m_info[0]));
-      status = write_gtid_set.add_gtid_text(gtid_str);
-    } else {  // version 2
-      auto gtid_str =
-          reinterpret_cast<const unsigned char *>(&(gtid_desc.m_info[0]));
-      mysql::gtid::Gtid saved_gtid;
-      auto gtid_bytes_read =
-          saved_gtid.decode_gtid_tagged(gtid_str, GTID_INFO_SIZE);
-      if (gtid_bytes_read != 0) {
-        status = write_gtid_set.add_gtid(saved_gtid);
-      }
-    }
+  for (auto &gtid_info : flush_list) {
+    auto gtid_str = reinterpret_cast<const char *>(&gtid_info[0]);
+    auto status = write_gtid_set.add_gtid_text(gtid_str);
     if (status != RETURN_STATUS_OK) {
       err = ER_INTERNAL_ERROR;
       return (err);
@@ -500,10 +412,8 @@ void Clone_persist_gtid::update_gtid_trx_no(trx_id_t new_gtid_trx_no) {
 
 void Clone_persist_gtid::flush_gtids(THD *thd) {
   int err = 0;
-  Tsid_map tsid_map(nullptr);
-  Gtid_set table_gtid_set(&tsid_map, nullptr);
-
-  DBUG_EXECUTE_IF("gtid_persist_flush_disable", return;);
+  Sid_map sid_map(nullptr);
+  Gtid_set table_gtid_set(&sid_map, nullptr);
 
   /* During recovery, fetch existing GTIDs from gtid_executed table. */
   bool is_recovery = !m_thread_active.load();
@@ -513,7 +423,7 @@ void Clone_persist_gtid::flush_gtids(THD *thd) {
 
   bool explicit_request = m_explicit_request.load();
 
-  trx_sys_serialisation_mutex_enter();
+  trx_sys_mutex_enter();
   /* Get oldest transaction number that is yet to be committed. Any transaction
   with lower transaction number is committed and is added to GTID list. */
   auto oldest_trx_no = trx_sys_oldest_trx_no();
@@ -524,8 +434,8 @@ void Clone_persist_gtid::flush_gtids(THD *thd) {
     /* Switch active list and get the previous list to write to disk table. */
     auto flush_list_number = switch_active_list();
     /* Exit trx mutex during write to table. */
-    trx_sys_serialisation_mutex_exit();
-    err = write_to_table(flush_list_number, table_gtid_set, tsid_map);
+    trx_sys_mutex_exit();
+    err = write_to_table(flush_list_number, table_gtid_set, sid_map);
     m_flush_in_progress.store(false);
     /* Compress always after recovery, if GTIDs are added. */
     if (!m_thread_active.load()) {
@@ -533,7 +443,7 @@ void Clone_persist_gtid::flush_gtids(THD *thd) {
       ib::info(ER_IB_CLONE_GTID_PERSIST) << "GTID compression after recovery. ";
     }
   } else {
-    trx_sys_serialisation_mutex_exit();
+    trx_sys_mutex_exit();
   }
 
   if (is_recovery) {
@@ -551,7 +461,7 @@ void Clone_persist_gtid::flush_gtids(THD *thd) {
     my_free(gtid_buffer);
   }
 
-  /* Update trx number up to which GTID is written to table. */
+  /* Update trx number upto which GTID is written to table. */
   update_gtid_trx_no(oldest_trx_no);
 
   /* Request Compression once the counter reaches threshold. */
@@ -576,40 +486,15 @@ void Clone_persist_gtid::flush_gtids(THD *thd) {
   }
 }
 
-bool Clone_persist_gtid::check_max_gtid_threshold() {
-  ut_ad(trx_sys_serialisation_mutex_own());
-  /* Allow only one GTID to flush at a time. */
-  DBUG_EXECUTE_IF("dont_compress_gtid_table",
-                  { return m_num_gtid_mem.load() > 0; });
-  return m_num_gtid_mem.load() >= s_max_gtid_threshold;
-}
-
 void Clone_persist_gtid::periodic_write() {
-  auto thd = create_internal_thd();
+  auto thd = create_thd(false, true, true, PSI_NOT_INSTRUMENTED);
+
+  m_thread_id = thd_get_thread_id(thd);
 
   /* Allow GTID to be persisted on read only server. */
   thd->set_skip_readonly_check();
 
-  /* Write all accumulated GTIDs while starting server. These GTIDs
-  are found in undo log during recovery. We must make sure all these
-  GTIDs are flushed and on disk before server is open for new operation
-  and new GTIDs are generated.
-
-  Why is it needed ?
-
-  1. mysql.gtid_executed table must be up to date at this point as global
-     variable gtid_executed is updated from it when binary log is disabled.
-
-  2. In older versions we used to have only one GTID storage in undo log
-     and PREAPARE GTID was stored in same place as COMMIT GTID. We used to
-     wait for PREPARE GTID to flush before writing commit GTID. Now this
-     limitation is removed and we no longer wait for PREPARE GTID to get
-     flushed before COMMIT as we store the PREPARE GTID in separate location.
-     However, while upgrading from previous version, there could be XA
-     transaction in PREPARED state with GTID stored in place of commit GTID.
-     Those GTIDs are also flushed here so that they are not overwritten later
-     at COMMIT.
-*/
+  /* Write all accumulated GTIDs while starting. */
   flush_gtids(thd);
 
   /* Let the caller wait till first set of GTIDs are persisted to table
@@ -618,7 +503,7 @@ void Clone_persist_gtid::periodic_write() {
 
   for (;;) {
     /* Exit if last phase of shutdown */
-    auto is_shutdown = (srv_shutdown_state.load() >= SRV_SHUTDOWN_CLEANUP);
+    auto is_shutdown = (srv_shutdown_state.load() != SRV_SHUTDOWN_NONE);
 
     if (is_shutdown || m_close_thread.load()) {
       /* Stop accepting any more GTID */
@@ -627,7 +512,7 @@ void Clone_persist_gtid::periodic_write() {
     }
 
     if (!flush_immediate()) {
-      os_event_wait_time(m_event, s_time_threshold);
+      os_event_wait_time(m_event, s_time_threshold_ms * 1000);
     }
     os_event_reset(m_event);
     /* Write accumulated GTIDs to disk table */
@@ -644,7 +529,7 @@ void Clone_persist_gtid::periodic_write() {
   }
 
   m_active.store(false);
-  destroy_internal_thd(thd);
+  destroy_thd(thd);
   m_thread_active.store(false);
 }
 
@@ -660,7 +545,7 @@ bool Clone_persist_gtid::wait_thread(bool start, bool wait_flush,
         result = false;
         return (0);
       }
-      /* If it is flushed up to the point requested. */
+      /* If it is flushed upto the point requested. */
       if (check_flushed(flush_number)) {
         /* Check if compression is done if requested. */
         if (!compress || !m_explicit_request.load()) {
@@ -687,10 +572,8 @@ bool Clone_persist_gtid::wait_thread(bool start, bool wait_flush,
     }
 
     ++count;
-    /* Force early exit from wait loop. We wait for about 1 sec
-    for early timeout: 10 x 100ms + 5 iteration for ramp up
-    from 1ms to 100ms. */
-    if (early_timeout && count > 15) {
+    /* Force early exit from wait loop. */
+    if (early_timeout && count > 1000) {
       return (ER_QUERY_TIMEOUT);
     }
     return (0);
@@ -698,8 +581,8 @@ bool Clone_persist_gtid::wait_thread(bool start, bool wait_flush,
 
   bool is_timeout = false;
 
-  /* Sleep starts with 1ms and backs off to 100 ms. */
-  Clone_Msec sleep_time(100);
+  /* Sleep for 1 millisecond */
+  Clone_Msec sleep_time(1);
   /* Generate alert message every 5 seconds. */
   Clone_Sec alert_interval(5);
   /* Wait for 5 minutes. */
@@ -711,7 +594,7 @@ bool Clone_persist_gtid::wait_thread(bool start, bool wait_flush,
 }
 
 /** Persist GTID to on disk table from time to time.
-@param[in,out]  persist_gtid    GTID persister */
+@param[in,out]	persist_gtid	GTID persister */
 static void clone_gtid_thread(Clone_persist_gtid *persist_gtid) {
   persist_gtid->periodic_write();
 }
@@ -723,13 +606,13 @@ bool Clone_persist_gtid::start() {
   }
 
   srv_threads.m_gtid_persister =
-      os_thread_create(clone_gtid_thread_key, 0, clone_gtid_thread, this);
+      os_thread_create(clone_gtid_thread_key, clone_gtid_thread, this);
   srv_threads.m_gtid_persister.start();
 
   if (!wait_thread(true, false, 0, false, false, nullptr)) {
     ib::error(ER_IB_CLONE_TIMEOUT) << "Wait for GTID thread to start timed out";
-    ut_d(ut_error);
-    ut_o(return (false));
+    ut_ad(false);
+    return (false);
   }
   m_active.store(true);
   return (true);
@@ -740,12 +623,22 @@ void Clone_persist_gtid::stop() {
   if (m_thread_active.load() &&
       !wait_thread(false, false, 0, false, false, nullptr)) {
     ib::error(ER_IB_CLONE_TIMEOUT) << "Wait for GTID thread to stop timed out";
-    ut_d(ut_error);
+    ut_ad(false);
   }
 }
 
-void Clone_persist_gtid::wait_flush(bool compress_gtid, bool early_timeout,
-                                    Clone_Alert_Func cbk) {
+void Clone_persist_gtid::flush_if_implicit_gtid(THD *thd) {
+  /* Avoid recursive wait, when we are writing GTID. */
+  if (thd == nullptr || m_thread_id == thd_get_thread_id(thd)) {
+    return;
+  }
+  if (thd->is_operating_gtid_table_implicitly) {
+    wait_flush(false, false, false, nullptr);
+  }
+}
+
+void Clone_persist_gtid::wait_flush(bool wait, bool compress_gtid,
+                                    bool early_timeout, Clone_Alert_Func cbk) {
   /* During recovery, avoid wait if called before persister is active. */
   if (!is_thread_active()) {
     return;
@@ -753,7 +646,11 @@ void Clone_persist_gtid::wait_flush(bool compress_gtid, bool early_timeout,
   auto request_number = request_immediate_flush(compress_gtid);
   os_event_set(m_event);
 
-  /* For RESET BINARY LOGS AND GTIDS we must wait for the flush. */
+  if (!wait) {
+    return;
+  }
+
+  /* For RESET MASTER we must wait for the flush. */
   auto thd = thd_get_current_thd();
   if (thd != nullptr && thd->is_log_reset()) {
     early_timeout = false;
@@ -767,6 +664,6 @@ void Clone_persist_gtid::wait_flush(bool compress_gtid, bool early_timeout,
   /* No error for early timeout. */
   if (!success && !early_timeout) {
     ib::error(ER_IB_CLONE_TIMEOUT) << "Wait for GTID thread to flush timed out";
-    ut_d(ut_error);
+    ut_ad(false);
   }
 }
