@@ -1,18 +1,17 @@
 /*****************************************************************************
 
-Copyright (c) 1996, 2024, Oracle and/or its affiliates.
+Copyright (c) 1996, 2020, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is designed to work with certain software (including
-but not limited to OpenSSL) that is licensed under separate terms,
-as designated in a particular file or component or in included license
-documentation.  The authors of MySQL hereby grant you an additional
-permission to link the program and your derivative works with the
-separately licensed software that they have either included with
-the program or referenced in the documentation.
+This program is also distributed with certain software (including but not
+limited to OpenSSL) that is licensed under separate terms, as designated in a
+particular file or component or in included license documentation. The authors
+of MySQL hereby grant you an additional permission to link the program and
+your derivative works with the separately licensed software that they have
+included with MySQL.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -41,8 +40,6 @@ this program; if not, write to the Free Software Foundation, Inc.,
 #include "dict0load.h"
 #include "ha_innodb.h"
 #include "ha_innopart.h"
-#include "log0buf.h"
-#include "log0chkp.h"
 #include "row0sel.h"
 #include "srv0start.h"
 
@@ -60,8 +57,8 @@ static std::vector<std::string> tables_with_fts;
 
 /** Fill foreign key information from InnoDB table to
 server table
-@param[in]      ib_table        InnoDB table object
-@param[in,out]  dd_table        DD table object
+@param[in]	ib_table	InnoDB table object
+@param[in,out]	dd_table	DD table object
 @return false on success, otherwise true */
 static bool dd_upgrade_table_fk(dict_table_t *ib_table, dd::Table *dd_table) {
   for (dict_foreign_set::iterator it = ib_table->foreign_set.begin();
@@ -170,12 +167,13 @@ static bool dd_upgrade_table_fk(dict_table_t *ib_table, dd::Table *dd_table) {
 /** Get Server Tablespace object for a InnoDB table. The tablespace is
 acquired with MDL and for modification, so the caller can update the
 dd::Tablespace object returned.
-@param[in,out]  dd_client       dictionary client to retrieve tablespace
+@param[in]	thd		server thread object
+@param[in,out]	dd_client	dictionary client to retrieve tablespace
                                 object
-@param[in]      ib_table        InnoDB table
+@param[in]	ib_table	InnoDB table
 @return dd::Tablespace object or nullptr */
 static dd::Tablespace *dd_upgrade_get_tablespace(
-    dd::cache::Dictionary_client *dd_client, dict_table_t *ib_table) {
+    THD *thd, dd::cache::Dictionary_client *dd_client, dict_table_t *ib_table) {
   std::string tablespace_name;
 
   dd::Tablespace *ts_obj = nullptr;
@@ -199,21 +197,21 @@ static dd::Tablespace *dd_upgrade_get_tablespace(
 
   /* MDL on tablespace name */
   if (dd_tablespace_get_mdl(tablespace_name.c_str())) {
-    ut_error;
+    ut_a(false);
   }
 
   /* For file per table tablespaces and general tablespaces, we will get
   the tablespace object and then get space_id. */
   if (dd_client->acquire_for_modification(tablespace_name.c_str(), &ts_obj)) {
-    ut_error;
+    ut_a(false);
   }
 
   return (ts_obj);
 }
 
 /** Get field from Server table object
-@param[in]      srv_table       server table object
-@param[in]      name            Field name
+@param[in]	srv_table	server table object
+@param[in]	name		Field name
 @return field object if found or null on failure. */
 static Field *dd_upgrade_get_field(const TABLE *srv_table, const char *name) {
   for (uint i = 0; i < srv_table->s->fields; i++) {
@@ -226,18 +224,18 @@ static Field *dd_upgrade_get_field(const TABLE *srv_table, const char *name) {
 }
 
 /** Return true if table has primary key given by user else false
-@param[in]      dd_table        Server table object
-@retval         true            Table has PK given by user
-@retval         false           Primary Key is hidden and generated */
+@param[in]	dd_table	Server table object
+@retval		true		Table has PK given by user
+@retval		false		Primary Key is hidden and generated */
 static bool dd_has_explicit_pk(const dd::Table *dd_table) {
   return (!dd_table->indexes().front()->is_hidden());
 }
 
 /** Match InnoDB column object and Server column object
-@param[in]      field   Server field object
-@param[in]      col     InnoDB column object
-@retval         false   column definition matches
-@retval         true    column definition mismatch */
+@param[in]	field	Server field object
+@param[in]	col	InnoDB column object
+@retval		false	column definition matches
+@retval		true	column definition mismatch */
 static bool dd_upgrade_match_single_col(const Field *field,
                                         const dict_col_t *col) {
   ulint unsigned_type;
@@ -247,16 +245,7 @@ static bool dd_upgrade_match_single_col(const Field *field,
 
   DBUG_EXECUTE_IF("dd_upgrade_strict_mode", ut_ad(col->mtype == col_type););
 
-  /* The columns of datatype MYSQL_TYPE_GEOMETRY were represented in InnoDB
-  as DATA_BLOB until 5.7 where they were changed to DATA_GEOMETRY type.
-  However, the following check fails while upgrading a 5.7 database with
-  GEOMETRY columns originally created with 5.6.
-  It is safe to ignore this datatype mismatch here so that upgrade can proceed
-  without affecting anything else. The correct datatype will be reflected in the
-  metadata once it is upgraded. */
-  if (col_type == DATA_GEOMETRY && col->mtype == DATA_BLOB) {
-    ib::warn(ER_IB_WRN_OLD_GEOMETRY_TYPE, field->field_name);
-  } else if (col->mtype != col_type) {
+  if (col->mtype != col_type) {
     ib::error(ER_IB_MSG_239)
         << "Column datatype mismatch for col: " << field->field_name;
     failure = true;
@@ -289,9 +278,9 @@ static bool dd_upgrade_match_single_col(const Field *field,
   ulint long_true_varchar = 0;
 
   if (field->type() == MYSQL_TYPE_VARCHAR) {
-    col_len -= field->get_length_bytes();
+    col_len -= ((Field_varstring *)field)->length_bytes;
 
-    if (field->get_length_bytes() == 2) {
+    if (((Field_varstring *)field)->length_bytes == 2) {
       long_true_varchar = DATA_LONG_TRUE_VARCHAR;
     }
   }
@@ -341,13 +330,13 @@ static bool dd_upgrade_match_single_col(const Field *field,
   return (failure);
 }
 
-/* Match definition of all columns in InnoDB table and DD table
-@param[in]      srv_table       Server table object
-@param[in]      dd_table        New DD table object
-@param[in]      ib_table        InnoDB table object
-@param[in]      skip_fts_col    Skip FTS_DOC_ID column match
-@retval         true            failure
-@retval         false           success, all columns matched */
+/* Match defintion of all columns in InnoDB table and DD table
+@param[in]	srv_table	Server table object
+@param[in]	dd_table	New DD table object
+@param[in]	ib_table	InnoDB table object
+@param[in]	skip_fts_col	Skip FTS_DOC_ID column match
+@retval		true		failure
+@retval		false		success, all columns matched */
 static bool dd_upgrade_match_cols(const TABLE *srv_table,
                                   const dd::Table *dd_table,
                                   const dict_table_t *ib_table,
@@ -442,9 +431,9 @@ static bool dd_upgrade_match_cols(const TABLE *srv_table,
 }
 
 /** Find key number from a server table object
-@param[in]      srv_table       server table object
-@param[in]      name            index name
-@retval UINT32_MAX if index not found, else key number */
+@param[in]	srv_table	server table object
+@param[in]	name		index name
+@retval	UINT32_MAX if index not found, else key number */
 static uint32_t dd_upgrade_find_index(TABLE *srv_table, const char *name) {
   for (uint32_t i = 0; i < srv_table->s->keys; i++) {
     KEY *key = srv_table->key_info + i;
@@ -456,10 +445,10 @@ static uint32_t dd_upgrade_find_index(TABLE *srv_table, const char *name) {
 }
 
 /** Match InnoDB index definition from Server object
-@param[in]      srv_table       Server table object
-@param[in]      index           InnoDB index
-@retval         false           Index definition matches
-@retval         true            Index definition mismatch */
+@param[in]	srv_table	Server table object
+@param[in]	index		InnoDB index
+@retval		false		Index definition matches
+@retval		true		Index definition mismatch */
 static bool dd_upgrade_match_index(TABLE *srv_table, dict_index_t *index) {
   uint32_t key_no = dd_upgrade_find_index(srv_table, index->name);
 
@@ -518,7 +507,7 @@ static bool dd_upgrade_match_index(TABLE *srv_table, dict_index_t *index) {
   for (ulint i = 0; i < key->user_defined_key_parts; i++) {
     KEY_PART_INFO *key_part = key->key_part + i;
 
-    Field *field = srv_table->field[key_part->field->field_index()];
+    Field *field = srv_table->field[key_part->field->field_index];
     if (field == nullptr) ut_error;
 
     const char *field_name = key_part->field->field_name;
@@ -544,7 +533,8 @@ static bool dd_upgrade_match_index(TABLE *srv_table, dict_index_t *index) {
         (key_part->length < field->pack_length() &&
          field->type() != MYSQL_TYPE_VARCHAR) ||
         (field->type() == MYSQL_TYPE_VARCHAR &&
-         key_part->length < field->pack_length() - field->get_length_bytes())) {
+         key_part->length <
+             field->pack_length() - ((Field_varstring *)field)->length_bytes)) {
       switch (col_type) {
         default:
           prefix_len = key_part->length;
@@ -589,11 +579,11 @@ static bool dd_upgrade_match_index(TABLE *srv_table, dict_index_t *index) {
 }
 
 /* Check if the table has auto inc field
-@param[in]      srv_tabl                server table object
-@param[in,out]  auto_inc_index_name     Index name on which auto inc exists
-@param[in,out]  auto_inc_col_name       Column name of the auto inc field
-@retval         true                    if auto inc field exists
-@retval         false                   if auto inc field doesn't exist */
+@param[in]	srv_tabl		server table object
+@param[in,out]	auto_inc_index_name	Index name on which auto inc exists
+@param[in,out]	auto_inc_col_name	Column name of the auto inc field
+@retval		true			if auto inc field exists
+@retval		false			if auot inc field doesn't exist */
 static bool dd_upgrade_check_for_autoinc(TABLE *srv_table,
                                          const char *&auto_inc_index_name,
                                          const char *&auto_inc_col_name) {
@@ -621,9 +611,9 @@ static bool dd_upgrade_check_for_autoinc(TABLE *srv_table,
 }
 
 /** Set auto-inc field value in dd::Table object
-@param[in]      srv_table       server table object
-@param[in,out]  dd_table        dd table object to be filled
-@param[in,out]  auto_inc_value  auto_inc value */
+@param[in]	srv_table	server table object
+@param[in,out]	dd_table	dd table object to be filled
+@param[in,out]	auto_inc_value	auto_inc value */
 static void dd_upgrade_set_auto_inc(const TABLE *srv_table, dd::Table *dd_table,
                                     uint64_t auto_inc_value) {
   ulonglong col_max_value;
@@ -643,14 +633,14 @@ static void dd_upgrade_set_auto_inc(const TABLE *srv_table, dd::Table *dd_table,
 
 /* Set DD Index se_private_data and also read auto_inc if it the index
 matches with auto_inc index name
-@param[in,out]  dd_index                dd::Index object
-@param[in]      index                   InnoDB index object
-@param[in]      dd_space_id             Server space id for index
+@param[in,out]	dd_index		dd::Index object
+@param[in]	index			InnoDB index object
+@param[in]	dd_space_id		Server space id for index
                                         (not InnoDB space id)
-@param[in]      has_auto_inc            true if table has auto inc field
-@param[in]      auto_inc_index_name     Index name on which auto_inc exists
-@param[in]      auto_inc_col_name       col name on which auto_inc exists
-@param[in,out]  read_auto_inc           auto inc value read */
+@param[in]	has_auto_inc		true if table has auto inc field
+@param[in]	auto_inc_index_name	Index name on which auto_inc exists
+@param[in]	auto_inc_col_name	col name on which auto_inc exists
+@param[in,out]	read_auto_inc		auto inc value read */
 template <typename Index>
 static void dd_upgrade_process_index(Index dd_index, dict_index_t *index,
                                      dd::Object_id dd_space_id,
@@ -674,43 +664,17 @@ static void dd_upgrade_process_index(Index dd_index, dict_index_t *index,
       dberr_t err =
           row_search_max_autoinc(index, auto_inc_col_name, read_auto_inc);
       if (err != DB_SUCCESS) {
-        ut_d(ut_error);
+        ut_ad(0);
       }
     }
   }
 }
 
-/** Ensures that the ib_table->dd_space_id is properly initialized.
-@param[in]  thd       The THD to identify as during lookup
-@param[in]  ib_table  The instance to be initialized
-@return true iff it succeeded */
-static bool dd_upgrade_ensure_has_dd_space_id(THD *thd,
-                                              dict_table_t *ib_table) {
-  if (ib_table->dd_space_id != dd::INVALID_OBJECT_ID) {
-    /* Already initialized, nothing to do. */
-    return true;
-  }
-  if (ib_table->space == SYSTEM_TABLE_SPACE) {
-    ib_table->dd_space_id = dict_sys_t::s_dd_sys_space_id;
-    /* Tables in system tablespace cannot be discarded. */
-    ut_ad(!dict_table_is_discarded(ib_table));
-    return true;
-  }
-  dd::cache::Dictionary_client *dd_client = dd::get_dd_client(thd);
-  dd::cache::Dictionary_client::Auto_releaser releaser(dd_client);
-  dd::Tablespace *dd_space = dd_upgrade_get_tablespace(dd_client, ib_table);
-  if (dd_space == nullptr) {
-    return false;
-  }
-  ib_table->dd_space_id = dd_space->id();
-  return true;
-}
-
 /** Migrate partitions to new dictionary
-@param[in]      thd             Server thread object
-@param[in]      norm_name       partition table name
-@param[in,out]  dd_table        Server new DD table object to be filled
-@param[in]      srv_table       Server table object
+@param[in]	thd		Server thread object
+@param[in]	norm_name	partition table name
+@param[in,out]	dd_table	Server new DD table object to be filled
+@param[in]	srv_table	Server table object
 @return false on success, true on error */
 static bool dd_upgrade_partitions(THD *thd, const char *norm_name,
                                   dd::Table *dd_table, TABLE *srv_table) {
@@ -733,7 +697,7 @@ static bool dd_upgrade_partitions(THD *thd, const char *norm_name,
     dict_name::build_table("", norm_name, part_str, false, false, table_name);
 
     dict_table_t *part_table = dict_table_open_on_name(
-        table_name.c_str(), false, true, DICT_ERR_IGNORE_NONE);
+        table_name.c_str(), FALSE, TRUE, DICT_ERR_IGNORE_NONE);
 
     if (part_table == nullptr) {
       ib::error(ER_IB_MSG_DICT_PARTITION_NOT_FOUND, table_name.c_str());
@@ -774,13 +738,39 @@ static bool dd_upgrade_partitions(THD *thd, const char *norm_name,
           dd_table_key_strings[DD_TABLE_DATA_DIRECTORY], true);
     }
 
-    /* We don't support upgrade from 5.7 with discarded Tablespaces.
-     Upgrade should stop in a dd_upgrade_tablespace function. */
-    ut_ad(!dict_table_is_discarded(part_table));
+    /* Set Discarded attribute in DD table se_private_data */
+    if (dict_table_is_discarded(part_table)) {
+      part_obj->se_private_data().set(dd_table_key_strings[DD_TABLE_DISCARD],
+                                      true);
+    }
 
-    if (!dd_upgrade_ensure_has_dd_space_id(thd, part_table)) {
-      ut_d(ut_error);
-      ut_o(return true);
+    dd::Object_id dd_space_id;
+
+    if (part_table->space == SYSTEM_TABLE_SPACE) {
+      dd_space_id = dict_sys_t::s_dd_sys_space_id;
+      /* Tables in system tablespace cannot be discarded. */
+      ut_ad(!dict_table_is_discarded(part_table));
+    } else {
+      dd::cache::Dictionary_client *dd_client = dd::get_dd_client(thd);
+      dd::cache::Dictionary_client::Auto_releaser releaser(dd_client);
+      dd::Tablespace *dd_space =
+          dd_upgrade_get_tablespace(thd, dd_client, part_table);
+      ut_ad(dd_space != nullptr);
+
+      if (dd_space == nullptr) {
+        dict_table_close(part_table, false, false);
+        return (true);
+      }
+
+      dd_space_id = dd_space->id();
+      /* If table is discarded, set discarded attribute in tablespace
+      object */
+      if (dict_table_is_discarded(part_table)) {
+        dd_tablespace_set_state(dd_space, DD_SPACE_STATE_DISCARDED);
+        if (dd_client->update(dd_space)) {
+          ut_ad(0);
+        }
+      }
     }
 
     dd_set_table_options(part_obj, part_table);
@@ -792,12 +782,13 @@ static bool dd_upgrade_partitions(THD *thd, const char *norm_name,
                           << "Partition Index " << part_index->name()
                           << " from server for table: " << part_table->name;);
 
-      for (auto index : part_table->indexes) {
+      for (dict_index_t *index = UT_LIST_GET_FIRST(part_table->indexes);
+           index != nullptr; index = UT_LIST_GET_NEXT(indexes, index)) {
         if (strcmp(part_index->name().c_str(), index->name()) == 0) {
           uint64_t read_auto_inc = 0;
-          dd_upgrade_process_index(part_index, index, part_table->dd_space_id,
-                                   has_auto_inc, auto_inc_index_name,
-                                   auto_inc_col_name, &read_auto_inc);
+          dd_upgrade_process_index(part_index, index, dd_space_id, has_auto_inc,
+                                   auto_inc_index_name, auto_inc_col_name,
+                                   &read_auto_inc);
           ++processed_indexes_num;
           if (has_auto_inc) {
             max_auto_inc = std::max(max_auto_inc, read_auto_inc);
@@ -827,8 +818,8 @@ static bool dd_upgrade_partitions(THD *thd, const char *norm_name,
 }
 
 /* Set the ROW_FORMAT in dd_table based on InnoDB dictionary table
-@param[in]      ib_table        InnoDB table
-@param[in,out]  dd_table        Server table object */
+@param[in]	ib_table	InnoDB table
+@param[in,out]	dd_table	Server table object */
 static void dd_upgrade_set_row_type(dict_table_t *ib_table,
                                     dd::Table *dd_table) {
   if (ib_table) {
@@ -848,7 +839,7 @@ static void dd_upgrade_set_row_type(dict_table_t *ib_table,
         dd_table->set_row_format(dd::Table::RF_DYNAMIC);
         break;
       default:
-        ut_d(ut_error);
+        ut_ad(0);
     }
   }
 }
@@ -856,8 +847,8 @@ static void dd_upgrade_set_row_type(dict_table_t *ib_table,
 /* Check Innodb table definition and add FTS_DOC_ID column and index to DD table
 if needed. This is required when all FTS index are dropped but Innodb still
 retains the FTS_DOC_ID column and FTS_DOC_ID_INDEX.
-@param[in,out]  dd_table        Server table object
-@param[in]      ib_table        Innodb table
+@param[in,out]	dd_table	Server table object
+@param[in]	ib_table	Innodb table
 @return true if fix FTS_DOC_ID column, false otherwise. */
 bool dd_upgrade_fix_fts_column(dd::Table *dd_table, dict_table_t *ib_table) {
   if (DICT_TF2_FLAG_IS_SET(ib_table, DICT_TF2_FTS_HAS_DOC_ID) &&
@@ -878,11 +869,11 @@ bool dd_upgrade_fix_fts_column(dd::Table *dd_table, dict_table_t *ib_table) {
 Dictionary. Since FTS tables contain table_id in their physical file name
 and during upgrade we reserve DICT_MAX_DD_TABLES for dictionary tables.
 So we rename FTS tablespace files
-@param[in]      thd             Server thread object
-@param[in]      db_name         database name
-@param[in]      table_name      table name
-@param[in,out]  dd_table        new dictionary table object to be filled
-@param[in]      srv_table       server table object
+@param[in]	thd		Server thread object
+@param[in]	db_name		database name
+@param[in]	table_name	table name
+@param[in,out]	dd_table	new dictionary table object to be filled
+@param[in]	srv_table	server table object
 @return false on success, true on failure. */
 bool dd_upgrade_table(THD *thd, const char *db_name, const char *table_name,
                       dd::Table *dd_table, TABLE *srv_table) {
@@ -899,8 +890,8 @@ bool dd_upgrade_table(THD *thd, const char *db_name, const char *table_name,
 
   if (truncated || !normalize_table_name(norm_name, buf)) {
     /* purecov: begin inspected */
-    ut_d(ut_error);
-    ut_o(return (true));
+    ut_ad(false);
+    return (true);
     /* purecov: end */
   }
 
@@ -911,7 +902,7 @@ bool dd_upgrade_table(THD *thd, const char *db_name, const char *table_name,
   }
 
   ib_table =
-      dict_table_open_on_name(norm_name, false, true, DICT_ERR_IGNORE_NONE);
+      dict_table_open_on_name(norm_name, FALSE, TRUE, DICT_ERR_IGNORE_NONE);
 
   if (ib_table == nullptr) {
     ib::error(ER_IB_MSG_258)
@@ -919,10 +910,6 @@ bool dd_upgrade_table(THD *thd, const char *db_name, const char *table_name,
     return (true);
   }
 
-  /* We don't support upgrade from 5.7 with discarded Tablespaces.
-   Upgrade should stop in a dd_upgrade_tablespace function. */
-  ut_ad(!dict_table_is_discarded(ib_table));
-
   /* If all FTS index are dropped but Innodb still retains the
   FTS_DOC_ID column then add FTS_DOC_ID column and index to DD table */
   bool added_fts_col = dd_upgrade_fix_fts_column(dd_table, ib_table);
@@ -941,9 +928,31 @@ bool dd_upgrade_table(THD *thd, const char *db_name, const char *table_name,
                                      ib_table->id);
   }
 
-  if (!dd_upgrade_ensure_has_dd_space_id(thd, ib_table)) {
-    dict_table_close(ib_table, false, false);
-    return true;
+  dd::Object_id dd_space_id;
+  if (ib_table->space == SYSTEM_TABLE_SPACE) {
+    dd_space_id = dict_sys_t::s_dd_sys_space_id;
+    /* Tables in system tablespace cannot be discarded. */
+    ut_ad(!dict_table_is_discarded(ib_table));
+  } else {
+    dd::cache::Dictionary_client *dd_client = dd::get_dd_client(thd);
+    dd::cache::Dictionary_client::Auto_releaser releaser(dd_client);
+    dd::Tablespace *dd_space =
+        dd_upgrade_get_tablespace(thd, dd_client, ib_table);
+
+    if (dd_space == nullptr) {
+      dict_table_close(ib_table, false, false);
+      return (true);
+    }
+
+    dd_space_id = dd_space->id();
+    /* If table is discarded, set discarded attribute in tablespace
+    object */
+    if (dict_table_is_discarded(ib_table)) {
+      dd_tablespace_set_state(dd_space, DD_SPACE_STATE_DISCARDED);
+      if (dd_client->update(dd_space)) {
+        ut_ad(0);
+      }
+    }
   }
 
   dd_table->set_se_private_id(ib_table->id);
@@ -955,6 +964,12 @@ bool dd_upgrade_table(THD *thd, const char *db_name, const char *table_name,
         dd_table_key_strings[DD_TABLE_DATA_DIRECTORY], true);
   }
 
+  /* Set Discarded attribute in DD table se_private_data */
+  if (dict_table_is_discarded(ib_table)) {
+    dd_table->se_private_data().set(dd_table_key_strings[DD_TABLE_DISCARD],
+                                    true);
+  }
+
   /* Set row_type */
   dd_upgrade_set_row_type(ib_table, dd_table);
 
@@ -991,15 +1006,16 @@ bool dd_upgrade_table(THD *thd, const char *db_name, const char *table_name,
                         << "Index " << dd_index->name()
                         << " from server for table: " << ib_table->name;);
 
-    for (auto index : ib_table->indexes) {
+    for (dict_index_t *index = UT_LIST_GET_FIRST(ib_table->indexes);
+         index != nullptr; index = UT_LIST_GET_NEXT(indexes, index)) {
       if (strcmp(dd_index->name().c_str(), index->name()) == 0) {
         if (!dd_index->is_hidden()) {
           failure = dd_upgrade_match_index(srv_table, index);
         }
 
-        dd_upgrade_process_index(dd_index, index, ib_table->dd_space_id,
-                                 has_auto_inc, auto_inc_index_name,
-                                 auto_inc_col_name, &auto_inc);
+        dd_upgrade_process_index(dd_index, index, dd_space_id, has_auto_inc,
+                                 auto_inc_index_name, auto_inc_col_name,
+                                 &auto_inc);
         ++processed_indexes_num;
         break;
       }
@@ -1023,16 +1039,16 @@ bool dd_upgrade_table(THD *thd, const char *db_name, const char *table_name,
     ib_table->autoinc = auto_inc == 0 ? 0 : auto_inc + 1;
   }
 
-  if (dict_table_has_fts_index(ib_table) || added_fts_col) {
+  if (dict_table_has_fts_index(ib_table)) {
     dberr_t err = fts_upgrade_aux_tables(ib_table);
 
     if (err != DB_SUCCESS) {
       dict_table_close(ib_table, false, false);
       return (true);
     } else {
-      dict_sys_mutex_enter();
+      mutex_enter(&dict_sys->mutex);
       dict_table_prevent_eviction(ib_table);
-      dict_sys_mutex_exit();
+      mutex_exit(&dict_sys->mutex);
 
       tables_with_fts.push_back(ib_table->name.m_name);
     }
@@ -1059,9 +1075,9 @@ typedef struct {
 
 /** Register InnoDB tablespace to mysql
 dictionary table mysql.tablespaces
-@param[in]      dd_client       dictionary client
-@param[in]      dd_space        server tablespace object
-@param[in]      upgrade_space   upgrade tablespace object
+@param[in]	dd_client	dictionary client
+@param[in]	dd_space	server tablespace object
+@param[in]	upgrade_space	upgrade tablespace object
 @return 0 on success, non-zero on error */
 static uint32_t dd_upgrade_register_tablespace(
     dd::cache::Dictionary_client *dd_client, dd::Tablespace *dd_space,
@@ -1109,9 +1125,9 @@ static uint32_t dd_upgrade_register_tablespace(
 /** Migrate tablespace entries from InnoDB SYS_TABLESPACES to new data
 dictionary. FTS Tablespaces are not registered as they are handled differently.
 FTS tablespaces have table_id in their name and we increment table_id of each
-table by DICT_MAX_DD_TABLES
+table by DICT_MAX_DD_TABLES.
 @param[in,out]  thd             THD
-@return MySQL error code */
+@return MySQL error code*/
 int dd_upgrade_tablespace(THD *thd) {
   DBUG_TRACE;
   btr_pcur_t pcur;
@@ -1119,15 +1135,10 @@ int dd_upgrade_tablespace(THD *thd) {
   mem_heap_t *heap;
   mtr_t mtr;
 
-  if (has_discarded_tablespaces) {
-    ib::error(ER_IB_CANNOT_UPGRADE_WITH_DISCARDED_TABLESPACES);
-    return HA_ERR_TABLESPACE_MISSING;
-  }
-
-  heap = mem_heap_create(100, UT_LOCATION_HERE);
+  heap = mem_heap_create(1000);
   dd::cache::Dictionary_client *dd_client = dd::get_dd_client(thd);
   dd::cache::Dictionary_client::Auto_releaser releaser(dd_client);
-  dict_sys_mutex_enter();
+  mutex_enter(&dict_sys->mutex);
   mtr_start(&mtr);
 
   /* Pattern for matching the FTS auxiliary tablespace name which starts with
@@ -1146,7 +1157,7 @@ int dd_upgrade_tablespace(THD *thd) {
     err_msg = dict_process_sys_tablespaces(heap, rec, &space, &name, &flags);
 
     mtr_commit(&mtr);
-    dict_sys_mutex_exit();
+    mutex_exit(&dict_sys->mutex);
     std::string tablespace_name(name);
 
     if (!err_msg && !regex_search(tablespace_name, fts_regex)) {
@@ -1180,12 +1191,12 @@ int dd_upgrade_tablespace(THD *thd) {
         upgrade_space.name = name;
       }
 
-      dict_sys_mutex_enter();
+      mutex_enter(&dict_sys->mutex);
       char *filename = dict_get_first_path(space);
-      dict_sys_mutex_exit();
+      mutex_exit(&dict_sys->mutex);
 
       std::string orig_name(filename);
-      ut::free(filename);
+      ut_free(filename);
       filename = nullptr;
 
       /* To migrate statistics from 57 satistics tables, we rename the
@@ -1202,9 +1213,9 @@ int dd_upgrade_tablespace(THD *thd) {
 
         /* Validate whether the tablespace file exists before making
         the entry in dd::tablespaces*/
-        dict_sys_mutex_enter();
+        mutex_enter(&dict_sys->mutex);
         fil_space_t *fil_space = fil_space_get(space);
-        dict_sys_mutex_exit();
+        mutex_exit(&dict_sys->mutex);
 
         /* If the file is not already opened, check for its existence
         by opening it in read-only mode. */
@@ -1213,7 +1224,7 @@ int dd_upgrade_tablespace(THD *thd) {
           df.set_filepath(orig_name.c_str());
           if (df.open_read_only(false) != DB_SUCCESS) {
             mem_heap_free(heap);
-            pcur.close();
+            btr_pcur_close(&pcur);
             return HA_ERR_TABLESPACE_MISSING;
           }
           df.close();
@@ -1238,12 +1249,12 @@ int dd_upgrade_tablespace(THD *thd) {
     mem_heap_empty(heap);
 
     /* Get the next record */
-    dict_sys_mutex_enter();
+    mutex_enter(&dict_sys->mutex);
     mtr_start(&mtr);
   }
 
   mtr_commit(&mtr);
-  dict_sys_mutex_exit();
+  mutex_exit(&dict_sys->mutex);
 
   /* These are file_per_table tablespaces(created using 5.5 or
   earlier). These are not found in SYS_TABLESPACES but discovered
@@ -1282,16 +1293,16 @@ int dd_upgrade_tablespace(THD *thd) {
   return 0;
 }
 
-/** Add server and space version number to tablespace while upgrading.
-@param[in]      space_id                space id of tablespace
-@param[in]      server_version_only     leave space version unchanged
+/** Add server version number to tablespace while upgrading.
+@param[in]      space_id              space id of tablespace
+@param[in]      server_version_only   leave space version unchanged
 @return false on success, true on failure. */
-bool upgrade_space_version(const uint32_t space_id, bool server_version_only) {
+bool upgrade_space_version(const uint32 space_id, bool server_version_only) {
   buf_block_t *block;
   page_t *page;
   mtr_t mtr;
 
-  fil_space_t *space = fil_space_acquire_silent(space_id);
+  fil_space_t *space = fil_space_acquire(space_id);
 
   if (space == nullptr) {
     return (true);
@@ -1306,8 +1317,7 @@ bool upgrade_space_version(const uint32_t space_id, bool server_version_only) {
     mtr.set_log_mode(MTR_LOG_NO_REDO);
   }
 
-  block = buf_page_get(page_id_t(space_id, 0), page_size, RW_SX_LATCH,
-                       UT_LOCATION_HERE, &mtr);
+  block = buf_page_get(page_id_t(space_id, 0), page_size, RW_SX_LATCH, &mtr);
 
   page = buf_block_get_frame(block);
 
@@ -1327,18 +1337,23 @@ bool upgrade_space_version(const uint32_t space_id, bool server_version_only) {
 @param[in]      tablespace              dd::Tablespace
 @return false on success, true on failure. */
 bool upgrade_space_version(dd::Tablespace *tablespace) {
-  uint32_t space_id;
+  uint32 space_id;
 
   if (tablespace->se_private_data().get("id", &space_id)) {
     /* error, attribute not found */
-    ut_d(ut_error);
-    ut_o(return (true));
+    ut_ad(0);
+    return (true);
   }
   /* Upgrade both server and space version */
   return (upgrade_space_version(space_id, false));
 }
 
-int dd_upgrade_logs(THD *) {
+/** Upgrade innodb undo logs after upgrade. Also increment the table_id
+offset by DICT_MAX_DD_TABLES. This offset increment is because the
+first 256 table_ids are reserved for dictionary.
+@param[in,out]  thd             THD
+@return MySQL error code*/
+int dd_upgrade_logs(THD *thd) {
   int error = 0; /* return zero for success */
   DBUG_TRACE;
 
@@ -1367,7 +1382,7 @@ the end of successful upgrade */
 static void dd_upgrade_drop_sys_tables() {
   ut_ad(srv_is_upgrade_mode);
 
-  dict_sys_mutex_enter();
+  mutex_enter(&dict_sys->mutex);
 
   bool found;
   const page_size_t page_size(
@@ -1402,7 +1417,7 @@ static void dd_upgrade_drop_sys_tables() {
   dict_sys->sys_fields = nullptr;
   dict_sys->sys_virtual = nullptr;
 
-  dict_sys_mutex_exit();
+  mutex_exit(&dict_sys->mutex);
 }
 
 /** Stat backup tables(innodb_*_stats_backup57) are created by server before
@@ -1424,7 +1439,7 @@ static void dd_upgrade_drop_57_backup_spaces() {
       continue;
     }
 
-    auto err = fil_delete_tablespace(space_id, BUF_REMOVE_NONE);
+    auto err = fil_delete_tablespace(space_id, BUF_REMOVE_FLUSH_NO_WRITE);
 
     if (err != DB_SUCCESS) {
       ib::warn(ER_IB_MSG_57_STAT_SPACE_DELETE_FAIL, space_name);
@@ -1434,17 +1449,17 @@ static void dd_upgrade_drop_57_backup_spaces() {
 
 /** Rename back the FTS AUX tablespace names from 8.0 format to 5.7
 format on upgrade failure, else mark FTS aux tables evictable
-@param[in]      failed_upgrade          true on upgrade failure, else
+@param[in]	failed_upgrade		true on upgrade failure, else
                                         false */
 static void dd_upgrade_fts_rename_cleanup(bool failed_upgrade) {
   for (std::string &name : tables_with_fts) {
-    dict_table_t *ib_table = dict_table_open_on_name(name.c_str(), false, true,
+    dict_table_t *ib_table = dict_table_open_on_name(name.c_str(), FALSE, TRUE,
                                                      DICT_ERR_IGNORE_NONE);
     ut_ad(ib_table != nullptr);
     if (ib_table != nullptr) {
       fts_upgrade_rename(ib_table, failed_upgrade);
 
-      dict_sys_mutex_enter();
+      mutex_enter(&dict_sys->mutex);
 
       /* Do not mark the table ready for eviction if there is
       a foreign key relationship on this table */
@@ -1452,12 +1467,18 @@ static void dd_upgrade_fts_rename_cleanup(bool failed_upgrade) {
         dict_table_allow_eviction(ib_table);
       }
       dict_table_close(ib_table, true, false);
-      dict_sys_mutex_exit();
+      mutex_exit(&dict_sys->mutex);
     }
   }
 }
 
-int dd_upgrade_finish(THD *, bool failed_upgrade) {
+/** If upgrade is successful, this API is used to flush innodb
+dirty pages to disk. In case of server crash, this function
+sets storage engine for rollback any changes.
+@param[in,out]  thd             THD
+@param[in]	failed_upgrade	true when upgrade failed
+@return MySQL error code*/
+int dd_upgrade_finish(THD *thd, bool failed_upgrade) {
   DBUG_TRACE;
 
   dd_upgrade_fts_rename_cleanup(failed_upgrade);
