@@ -1,19 +1,18 @@
 /*****************************************************************************
 
-Copyright (c) 1996, 2024, Oracle and/or its affiliates.
+Copyright (c) 1996, 2019, Oracle and/or its affiliates. All Rights Reserved.
 Copyright (c) 2012, Facebook Inc.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is designed to work with certain software (including
-but not limited to OpenSSL) that is licensed under separate terms,
-as designated in a particular file or component or in included license
-documentation.  The authors of MySQL hereby grant you an additional
-permission to link the program and your derivative works with the
-separately licensed software that they have either included with
-the program or referenced in the documentation.
+This program is also distributed with certain software (including but not
+limited to OpenSSL) that is licensed under separate terms, as designated in a
+particular file or component or in included license documentation. The authors
+of MySQL hereby grant you an additional permission to link the program and
+your derivative works with the separately licensed software that they have
+included with MySQL.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -40,9 +39,7 @@ external tools. */
 
 #include "dict0dict.h"
 #ifndef UNIV_HOTBACKUP
-#ifndef UNIV_LIBRARY
 #include "lock0lock.h"
-#endif /* !UNIV_LIBRARY */
 #endif /* !UNIV_HOTBACKUP */
 
 /** Append 'name' to 'col_names'.  @see dict_table_t::col_names
@@ -93,7 +90,7 @@ void dict_mem_table_free(dict_table_t *table) /*!< in: table */
 {
   ut_ad(table);
   ut_ad(table->magic_n == DICT_TABLE_MAGIC_N);
-  ut_d(table->cached = false);
+  ut_d(table->cached = FALSE);
 
 #ifndef UNIV_HOTBACKUP
 #ifndef UNIV_LIBRARY
@@ -118,7 +115,7 @@ void dict_mem_table_free(dict_table_t *table) /*!< in: table */
 #endif /* !UNIV_LIBRARY */
 #endif /* !UNIV_HOTBACKUP */
 
-  ut::free(table->name.m_name);
+  ut_free(table->name.m_name);
   table->name.m_name = nullptr;
 
 #ifndef UNIV_HOTBACKUP
@@ -128,19 +125,19 @@ void dict_mem_table_free(dict_table_t *table) /*!< in: table */
   for (ulint i = 0; i < table->n_v_def; i++) {
     dict_v_col_t *vcol = dict_table_get_nth_v_col(table, i);
 
-    ut::delete_(vcol->v_indexes);
+    UT_DELETE(vcol->v_indexes);
   }
 #endif /* !UNIV_LIBRARY */
 #endif /* !UNIV_HOTBACKUP */
 
   if (table->s_cols != nullptr) {
-    ut::delete_(table->s_cols);
+    UT_DELETE(table->s_cols);
   }
 
 #ifndef UNIV_HOTBACKUP
   if (table->temp_prebuilt != nullptr) {
     ut_ad(table->is_intrinsic());
-    ut::delete_(table->temp_prebuilt);
+    UT_DELETE(table->temp_prebuilt);
   }
 #endif /* !UNIV_HOTBACKUP */
 
@@ -179,10 +176,19 @@ static bool dict_mem_table_is_system(const std::string name) {
   }
 }
 
-dict_table_t *dict_mem_table_create(const char *name, space_id_t space,
-                                    ulint n_cols, ulint n_v_cols,
-                                    ulint n_m_v_cols, uint32_t flags,
-                                    uint32_t flags2, uint32_t n_drop_cols) {
+/** Creates a table memory object.
+ @return own: table object */
+dict_table_t *dict_mem_table_create(
+    const char *name, /*!< in: table name */
+    space_id_t space, /*!< in: space where the clustered index of
+                      the table is placed */
+    ulint n_cols,     /*!< in: total number of columns including
+                      virtual and non-virtual columns */
+    ulint n_v_cols,   /*!< in: number of virtual columns */
+    ulint n_m_v_cols, /*!< in: number of multi-value virtual columns */
+    uint32_t flags,   /*!< in: table flags */
+    uint32_t flags2)  /*!< in: table flags2 */
+{
   dict_table_t *table;
   mem_heap_t *heap;
 
@@ -192,17 +198,17 @@ dict_table_t *dict_mem_table_create(const char *name, space_id_t space,
   ut_a(!(flags2 & DICT_TF2_UNUSED_BIT_MASK));
 #endif /* !UNIV_HOTBACKUP */
 
-  heap = mem_heap_create(DICT_HEAP_SIZE, UT_LOCATION_HERE);
+  heap = mem_heap_create(DICT_HEAP_SIZE);
 
   table = static_cast<dict_table_t *>(mem_heap_zalloc(heap, sizeof(*table)));
 
 #ifndef UNIV_HOTBACKUP
 #ifndef UNIV_LIBRARY
-  UT_LIST_INIT(table->locks);
+  lock_table_lock_list_init(&table->locks);
 #endif /* !UNIV_LIBRARY */
 #endif /* !UNIV_HOTBACKUP */
 
-  UT_LIST_INIT(table->indexes);
+  UT_LIST_INIT(table->indexes, &dict_index_t::indexes);
 
   table->heap = heap;
 
@@ -221,7 +227,7 @@ dict_table_t *dict_mem_table_create(const char *name, space_id_t space,
   table->n_instant_cols = table->n_cols;
 
   table->cols = static_cast<dict_col_t *>(
-      mem_heap_alloc(heap, (table->n_cols + n_drop_cols) * sizeof(dict_col_t)));
+      mem_heap_alloc(heap, table->n_cols * sizeof(dict_col_t)));
   table->v_cols = static_cast<dict_v_col_t *>(
       mem_heap_alloc(heap, n_v_cols * sizeof(*table->v_cols)));
 
@@ -258,7 +264,7 @@ dict_table_t *dict_mem_table_create(const char *name, space_id_t space,
   }
 
   if (DICT_TF_HAS_SHARED_SPACE(table->flags)) {
-    dict_get_and_save_space_name(table);
+    dict_get_and_save_space_name(table, true);
   }
 
   new (&table->foreign_set) dict_foreign_set();
@@ -289,11 +295,10 @@ dict_index_t *dict_mem_index_create(
 
   ut_ad(table_name && index_name);
 
-  heap = mem_heap_create(DICT_HEAP_SIZE, UT_LOCATION_HERE);
+  heap = mem_heap_create(DICT_HEAP_SIZE);
 
   index = static_cast<dict_index_t *>(mem_heap_zalloc(heap, sizeof(*index)));
 
-  new (&index->fields_array)(decltype(index->fields_array))();
   dict_mem_fill_index_struct(index, heap, table_name, index_name, space, type,
                              n_fields);
 
@@ -307,8 +312,7 @@ dict_index_t *dict_mem_index_create(
         mem_heap_alloc(heap, sizeof(*index->rtr_track)));
     mutex_create(LATCH_ID_RTR_ACTIVE_MUTEX,
                  &index->rtr_track->rtr_active_mutex);
-    index->rtr_track->rtr_active =
-        ut::new_withkey<rtr_info_active>(UT_NEW_THIS_FILE_PSI_KEY);
+    index->rtr_track->rtr_active = UT_NEW_NOKEY(rtr_info_active());
   }
 #endif /* !UNIV_LIBRARY */
 #endif /* !UNIV_HOTBACKUP */
@@ -317,10 +321,14 @@ dict_index_t *dict_mem_index_create(
 }
 
 /** Adds a column definition to a table. */
-void dict_mem_table_add_col(dict_table_t *table, mem_heap_t *heap,
-                            const char *name, ulint mtype, ulint prtype,
-                            ulint len, bool is_visible, uint32_t phy_pos,
-                            uint8_t v_added, uint8_t v_dropped) {
+void dict_mem_table_add_col(
+    dict_table_t *table, /*!< in: table */
+    mem_heap_t *heap,    /*!< in: temporary memory heap, or NULL */
+    const char *name,    /*!< in: column name, or NULL */
+    ulint mtype,         /*!< in: main datatype */
+    ulint prtype,        /*!< in: precise type */
+    ulint len)           /*!< in: precision */
+{
   dict_col_t *col;
   ulint i;
 
@@ -335,7 +343,7 @@ void dict_mem_table_add_col(dict_table_t *table, mem_heap_t *heap,
   table->n_t_def++;
 
   if (name) {
-    if (table->n_def == (table->n_cols + table->get_n_instant_drop_cols())) {
+    if (table->n_def == table->n_cols) {
       heap = table->heap;
     }
     if (i && !table->col_names) {
@@ -350,14 +358,18 @@ void dict_mem_table_add_col(dict_table_t *table, mem_heap_t *heap,
 
   col = table->get_col(i);
 
-  dict_mem_fill_column_struct(col, i, mtype, prtype, len, is_visible, phy_pos,
-                              v_added, v_dropped);
+  dict_mem_fill_column_struct(col, i, mtype, prtype, len);
 }
 
-void dict_mem_fill_column_struct(dict_col_t *column, ulint col_pos, ulint mtype,
-                                 ulint prtype, ulint col_len, bool is_visible,
-                                 uint32_t phy_pos, uint8_t v_added,
-                                 uint8_t v_dropped) {
+/** This function populates a dict_col_t memory structure with
+ supplied information. */
+void dict_mem_fill_column_struct(dict_col_t *column, /*!< out: column struct to
+                                                     be filled */
+                                 ulint col_pos,      /*!< in: column position */
+                                 ulint mtype,        /*!< in: main data type */
+                                 ulint prtype,       /*!< in: precise type */
+                                 ulint col_len)      /*!< in: column length */
+{
   column->ind = (unsigned int)col_pos;
   column->ord_part = 0;
   column->max_prefix = 0;
@@ -365,10 +377,6 @@ void dict_mem_fill_column_struct(dict_col_t *column, ulint col_pos, ulint mtype,
   column->prtype = (unsigned int)prtype;
   column->len = (unsigned int)col_len;
   column->instant_default = nullptr;
-  column->is_visible = is_visible;
-  column->set_phy_pos(phy_pos);
-  column->set_version_added(v_added);
-  column->set_version_dropped(v_dropped);
 #ifndef UNIV_HOTBACKUP
 #ifndef UNIV_LIBRARY
   ulint mbminlen;
