@@ -1,18 +1,17 @@
 /*****************************************************************************
 
-Copyright (c) 2013, 2024, Oracle and/or its affiliates.
+Copyright (c) 2013, 2019, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is designed to work with certain software (including
-but not limited to OpenSSL) that is licensed under separate terms,
-as designated in a particular file or component or in included license
-documentation.  The authors of MySQL hereby grant you an additional
-permission to link the program and your derivative works with the
-separately licensed software that they have either included with
-the program or referenced in the documentation.
+This program is also distributed with certain software (including but not
+limited to OpenSSL) that is licensed under separate terms, as designated in a
+particular file or component or in included license documentation. The authors
+of MySQL hereby grant you an additional permission to link the program and
+your derivative works with the separately licensed software that they have
+included with MySQL.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -37,20 +36,18 @@ this program; if not, write to the Free Software Foundation, Inc.,
 #include "fts0tokenize.h"
 #include "mysql/plugin.h"
 
-struct CHARSET_INFO;
-
 /** FTS default parser init
  @return 0 */
-static int fts_default_parser_init(MYSQL_FTPARSER_PARAM *param [
-    [maybe_unused]]) /*!< in: plugin parser param */
+static int fts_default_parser_init(
+    MYSQL_FTPARSER_PARAM *param) /*!< in: plugin parser param */
 {
   return (0);
 }
 
 /** FTS default parser deinit
  @return 0 */
-static int fts_default_parser_deinit(MYSQL_FTPARSER_PARAM *param [
-    [maybe_unused]]) /*!< in: plugin parser param */
+static int fts_default_parser_deinit(
+    MYSQL_FTPARSER_PARAM *param) /*!< in: plugin parser param */
 {
   return (0);
 }
@@ -115,7 +112,7 @@ static int fts_query_add_word_for_parser(
       if (cur_node->type != FTS_AST_PARSER_PHRASE_LIST) {
         break;
       }
-      [[fallthrough]];
+      // Fall through.
 
     case FT_TOKEN_WORD:
       term_node = fts_ast_create_node_term_for_parser(state, word, word_len);
@@ -239,7 +236,7 @@ static int fts_parse_query_internal(
 
 /** fts parse query by plugin parser.
  @return 0 if parse successfully, or return non-zero. */
-int fts_parse_by_parser(bool mode,        /*!< in: parse boolean mode */
+int fts_parse_by_parser(ibool mode,       /*!< in: parse boolean mode */
                         uchar *query_str, /*!< in: query string */
                         ulint query_len,  /*!< in: query string length */
                         st_mysql_ftparser *parser, /*!< in: fts plugin parser */
