@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 2016, 2023, Oracle and/or its affiliates.
+Copyright (c) 2016, 2019, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
@@ -33,7 +33,6 @@ this program; if not, write to the Free Software Foundation, Inc.,
 
 #include <stdlib.h>
 #include <sys/types.h>
-#include <type_traits>
 
 #include "lock0iter.h"
 #include "lock0lock.h"
@@ -144,6 +143,14 @@ this program; if not, write to the Free Software Foundation, Inc.,
 static const char *g_engine = "INNODB";
 static const size_t g_engine_length = 6;
 
+inline trx_t *get_next_trx(const trx_t *trx, bool read_write) {
+  if (read_write) {
+    return (UT_LIST_GET_NEXT(trx_list, trx));
+  } else {
+    return (UT_LIST_GET_NEXT(mysql_trx_list, trx));
+  }
+}
+
 /** Pass of a given scan. */
 enum scan_pass {
   INIT_SCANNING,
@@ -177,7 +184,7 @@ class Innodb_trx_scan_state {
         m_end_trx_id_range(SCAN_RANGE),
         m_next_trx_id_range(TRX_ID_MAX) {}
 
-  ~Innodb_trx_scan_state() = default;
+  ~Innodb_trx_scan_state() {}
 
   scan_pass get_pass() { return m_scan_pass; }
 
@@ -220,7 +227,7 @@ class Innodb_trx_scan_state {
 
   /** Check if a transaction belongs to the current range.
   As a side effect, compute the next range.
-  @param[in] trx_id     Transaction id to evaluate
+  @param[in] trx_id	Transaction id to evaluate
   @return True if transaction is within range.
   */
   bool trx_id_in_range(trx_id_t trx_id) {
@@ -252,39 +259,40 @@ class Innodb_trx_scan_state {
 class Innodb_data_lock_iterator : public PSI_engine_data_lock_iterator {
  public:
   Innodb_data_lock_iterator();
-  ~Innodb_data_lock_iterator() override;
+  ~Innodb_data_lock_iterator();
 
-  bool scan(PSI_server_data_lock_container *container,
-            bool with_lock_data) override;
+  virtual bool scan(PSI_server_data_lock_container *container,
+                    bool with_lock_data);
 
-  bool fetch(PSI_server_data_lock_container *container,
-             const char *engine_lock_id, size_t engine_lock_id_length,
-             bool with_lock_data) override;
+  virtual bool fetch(PSI_server_data_lock_container *container,
+                     const char *engine_lock_id, size_t engine_lock_id_length,
+                     bool with_lock_data);
 
  private:
   /** Scan a trx list.
-  @param[in] container          The container to fill
-  @param[in] with_lock_data     True if column LOCK_DATA
+  @param[in] container		The container to fill
+  @param[in] with_lock_data	True if column LOCK_DATA
   needs to be populated.
-  @param[in] trx_list           The trx list to scan
+  @param[in] read_write		True if the trx list is the RW list
+  @param[in] trx_list		The trx list to scan
   @returns The number of records found
   */
-  template <typename Trx_list>
   size_t scan_trx_list(PSI_server_data_lock_container *container,
-                       bool with_lock_data, Trx_list *trx_list);
+                       bool with_lock_data, bool read_write,
+                       trx_ut_list_t *trx_list);
 
   /** Scan a given trx.
   Either scan all the locks for a transaction,
   or scan only records matching a given lock.
-  @param[in] container          The container to fill
-  @param[in] with_lock_data     True if column LOCK_DATA
+  @param[in] container		The container to fill
+  @param[in] with_lock_data	True if column LOCK_DATA
   needs to be populated.
-  @param[in] trx                        The trx to scan
-  @param[in] with_filter                True if looking for a specific record
+  @param[in] trx			The trx to scan
+  @param[in] with_filter		True if looking for a specific record
   only.
   @param[in] filter_lock_immutable_id     Immutable id of lock_t we are looking
   for
-  @param[in] filter_heap_id     Heap id to look for, when filtering
+  @param[in] filter_heap_id	Heap id to look for, when filtering
   @returns The number of records found
   */
   size_t scan_trx(PSI_server_data_lock_container *container,
@@ -300,37 +308,37 @@ class Innodb_data_lock_iterator : public PSI_engine_data_lock_iterator {
 class Innodb_data_lock_wait_iterator
     : public PSI_engine_data_lock_wait_iterator {
  public:
-  Innodb_data_lock_wait_iterator() = default;
-  ~Innodb_data_lock_wait_iterator() override = default;
+  Innodb_data_lock_wait_iterator();
+  ~Innodb_data_lock_wait_iterator();
 
-  bool scan(PSI_server_data_lock_wait_container *container) override;
+  virtual bool scan(PSI_server_data_lock_wait_container *container);
 
-  bool fetch(PSI_server_data_lock_wait_container *container,
-             const char *requesting_engine_lock_id,
-             size_t requesting_engine_lock_id_length,
-             const char *blocking_engine_lock_id,
-             size_t blocking_engine_lock_id_length) override;
+  virtual bool fetch(PSI_server_data_lock_wait_container *container,
+                     const char *requesting_engine_lock_id,
+                     size_t requesting_engine_lock_id_length,
+                     const char *blocking_engine_lock_id,
+                     size_t blocking_engine_lock_id_length);
 
  private:
   /** Scan a given transaction list.
-  @param[in] container          The container to fill
-  @param[in] trx_list           The trx list to scan
+  @param[in] container		The container to fill
+  @param[in] read_write		True if the transaction list is the RW list.
+  @param[in] trx_list		The trx list to scan
   @returns the number of records found.
   */
-  template <typename Trx_list>
   size_t scan_trx_list(PSI_server_data_lock_wait_container *container,
-                       Trx_list *trx_list);
+                       bool read_write, trx_ut_list_t *trx_list);
 
   /** Scan a given transaction.
   Either scan all the waits for a transaction,
   or scan only records matching a given wait.
-  @param[in] container          The container to fill
-  @param[in] trx                        The trx to scan
-  @param[in] with_filter                True if looking for a given wait only.
-  @param[in] filter_requesting_lock_immutable_id                Immutable id of
-  lock_t for the requesting lock, when filtering
-  @param[in] filter_blocking_lock_immutable_id          Immutable idof
-  lock_t for the blocking lock, when filtering
+  @param[in] container		          The container to fill
+  @param[in] trx		          The trx to scan
+  @param[in] with_filter		  True if looking for a given wait only.
+  @param[in] filter_requesting_lock_immutable_id  Immutable id of lock_t for
+  the requesting lock, when filtering
+  @param[in] filter_blocking_lock_immutable_id	  Immutable id of lock_t
+  for the blocking lock, when filtering
   @returns the number of records found.
   */
   size_t scan_trx(PSI_server_data_lock_wait_container *container,
@@ -352,7 +360,7 @@ that are writing data and have an id, are also discarded.
 @returns True if the trx should be discarded
 */
 bool discard_trx(const trx_t *trx, bool read_write) {
-  if (!trx_was_started(trx)) {
+  if (!trx_is_started(trx)) {
     return true;
   }
 
@@ -362,41 +370,26 @@ bool discard_trx(const trx_t *trx, bool read_write) {
 
   return false;
 }
-template <typename Trx_list>
-static constexpr bool is_read_write() {
-  constexpr bool is_read_write =
-      std::is_same<Trx_list, decltype(trx_sys->rw_trx_list)>::value;
-  constexpr bool is_mysql_trx_list =
-      std::is_same<Trx_list, decltype(trx_sys->mysql_trx_list)>::value;
-  static_assert(is_read_write || is_mysql_trx_list,
-                "only rw_trx_list and mysql_trx_list are supported");
-  static_assert(!is_read_write || !is_mysql_trx_list,
-                "can not distinguish rw_trx_list from mysql_trx_list by type");
-  return is_read_write;
-}
 
 /** Find a transaction in a TRX LIST.
 @param[in] filter_trx_immutable_id  The transaction immutable id
-@param[in] trx_list                 The transaction list
+@param[in] read_write	            True for the RW TRX LIST
+@param[in] trx_list	            The transaction list
 @returns The transaction when found, or NULL
 */
-template <typename Trx_list>
 static const trx_t *fetch_trx_in_trx_list(uint64_t filter_trx_immutable_id,
-                                          Trx_list *trx_list) {
-  constexpr bool read_write = is_read_write<Trx_list>();
+                                          bool read_write,
+                                          trx_ut_list_t *trx_list) {
+  const trx_t *trx;
 
-  /* It is not obvious if and why we need lock_sys exclusive access, but we do
-  own exclusive latch here, so treat this assert more as a documentation */
-  ut_ad(locksys::owns_exclusive_global_latch());
+  ut_ad(lock_mutex_own());
   ut_ad(trx_sys_mutex_own());
 
-  for (auto trx : *trx_list) {
+  for (trx = UT_LIST_GET_FIRST(*trx_list); trx != nullptr;
+       trx = get_next_trx(trx, read_write)) {
     if (discard_trx(trx, read_write)) {
       continue;
     }
-    /* Note: for read-only transactions the check above might be based on stale
-    values and does not bring any guarantee after it is finished, because state
-    of read-only transaction might be modified outside the trx_sys->mutex. */
 
     if (filter_trx_immutable_id == trx_immutable_id(trx)) {
       return trx;
@@ -406,6 +399,10 @@ static const trx_t *fetch_trx_in_trx_list(uint64_t filter_trx_immutable_id,
   return nullptr;
 }
 
+Innodb_data_lock_inspector::Innodb_data_lock_inspector() {}
+
+Innodb_data_lock_inspector::~Innodb_data_lock_inspector() {}
+
 PSI_engine_data_lock_iterator *
 Innodb_data_lock_inspector::create_data_lock_iterator() {
   return new Innodb_data_lock_iterator();
@@ -427,22 +424,21 @@ void Innodb_data_lock_inspector::destroy_data_lock_wait_iterator(
 }
 
 /** Allocate identifier in performance schema container.
-@param[in]  container        The container to fill
-@param[in]  kind             The identifier kind
-@param[in]  id_str           The identifier string
-@param[out] cached_id        The cached identifier string
-@param[out] cached_id_length The cached identifier string length
+@param[in]	container	The container to fill
+@param[in]	id_str		The identifier string
+@param[out]	id_length	The identifier string length
+@returns string allocated in the performance schema container.
 */
-void alloc_identifier(PSI_server_data_lock_container *container,
-                      PSI_identifier kind, const std::string &id_str,
-                      const char **cached_id, size_t *cached_id_length) {
-  if (id_str.length() > 0) {
-    container->cache_identifier(kind, id_str.c_str(), id_str.length(),
-                                cached_id, cached_id_length);
-  } else {
-    *cached_id = nullptr;
-    *cached_id_length = 0;
+const char *alloc_identifier(PSI_server_data_lock_container *container,
+                             std::string &id_str, size_t *id_length) {
+  *id_length = id_str.length();
+  const char *id_name = nullptr;
+
+  if (*id_length > 0) {
+    id_name = container->cache_data(id_str.c_str(), *id_length);
   }
+
+  return (id_name);
 }
 
 /** Parse a table path string.
@@ -450,21 +446,23 @@ Isolate the table schema, name, partition and sub partition
 from a table path string.
 Convert these strings and store them in the performance schema container.
 @note String returned are not zero terminated.
-@param[in] container                    The container to fill
-@param[in] table_path                   The table path string
-@param[out] table_schema                The table schema
-@param[out] table_schema_length         The table schema length
-@param[out] table_name                  The table name
-@param[out] table_name_length           The table name length
-@param[out] partition_name              Partition name
-@param[out] partition_name_length       Partition name length
-@param[out] subpartition_name           Sub partition name
-@param[out] subpartition_name_length    Sub partition name length
+@param[in] container			The container to fill
+@param[in] table_path			The table path string
+@param[in] table_path_length		The table path string length
+@param[out] table_schema		The table schema
+@param[out] table_schema_length		The table schema length
+@param[out] table_name			The table name
+@param[out] table_name_length		The table name length
+@param[out] partition_name		Partition name
+@param[out] partition_name_length	Partition name length
+@param[out] subpartition_name		Sub partition name
+@param[out] subpartition_name_length	Sub partition name length
 */
 void parse_table_path(PSI_server_data_lock_container *container,
-                      const char *table_path, const char **table_schema,
-                      size_t *table_schema_length, const char **table_name,
-                      size_t *table_name_length, const char **partition_name,
+                      const char *table_path, size_t table_path_length,
+                      const char **table_schema, size_t *table_schema_length,
+                      const char **table_name, size_t *table_name_length,
+                      const char **partition_name,
                       size_t *partition_name_length,
                       const char **subpartition_name,
                       size_t *subpartition_name_length) {
@@ -485,20 +483,17 @@ void parse_table_path(PSI_server_data_lock_container *container,
     dict_name::get_partition(partition, true, part, sub_part);
   }
 
-  alloc_identifier(container, PSI_IDENTIFIER_SCHEMA, schema, table_schema,
-                   table_schema_length);
-  alloc_identifier(container, PSI_IDENTIFIER_TABLE, table, table_name,
-                   table_name_length);
-  alloc_identifier(container, PSI_IDENTIFIER_PARTITION, part, partition_name,
-                   partition_name_length);
-  alloc_identifier(container, PSI_IDENTIFIER_SUBPARTITION, sub_part,
-                   subpartition_name, subpartition_name_length);
+  *table_schema = alloc_identifier(container, schema, table_schema_length);
+  *table_name = alloc_identifier(container, table, table_name_length);
+  *partition_name = alloc_identifier(container, part, partition_name_length);
+  *subpartition_name =
+      alloc_identifier(container, sub_part, subpartition_name_length);
 }
 
 /** Print a table lock id.
-@param[in]      lock            The lock to print
-@param[in,out]  lock_id         Printing buffer
-@param[in]      lock_id_size    Printing buffer length
+@param[in]	lock		The lock to print
+@param[in,out]	lock_id		Printing buffer
+@param[in]	lock_id_size	Printing buffer length
 */
 void print_table_lock_id(const lock_t *lock, char *lock_id,
                          size_t lock_id_size) {
@@ -510,10 +505,10 @@ void print_table_lock_id(const lock_t *lock, char *lock_id,
 }
 
 /** Print a record lock id.
-@param[in]      lock            The lock to print
-@param[in]      heap_no         Lock heap number
-@param[in,out]  lock_id         Printing buffer
-@param[in]      lock_id_size    Printing buffer length
+@param[in]	lock		The lock to print
+@param[in]	heap_no		Lock heap number
+@param[in,out]	lock_id		Printing buffer
+@param[in]	lock_id_size	Printing buffer length
 */
 void print_record_lock_id(const lock_t *lock, ulint heap_no, char *lock_id,
                           size_t lock_id_size) {
@@ -525,10 +520,10 @@ void print_record_lock_id(const lock_t *lock, ulint heap_no, char *lock_id,
 }
 
 /** Print a lock id.
-@param[in]      lock            The lock to print
-@param[in]      heap_no         Lock heap number
-@param[in,out]  lock_id         Printing buffer
-@param[in]      lock_id_size    Printing buffer length
+@param[in]	lock		The lock to print
+@param[in]	heap_no		Lock heap number
+@param[in,out]	lock_id		Printing buffer
+@param[in]	lock_id_size	Printing buffer length
 */
 void print_lock_id(const lock_t *lock, ulint heap_no, char *lock_id,
                    size_t lock_id_size) {
@@ -546,14 +541,14 @@ void print_lock_id(const lock_t *lock, ulint heap_no, char *lock_id,
 
 /** Scan a lock id string and extract information necessary to find a row
 by primary key.
-@param[in]      lock_id                 The lock id string to parse
-@param[out]     trx_immutable_id        The immutable id of lock->trx
-@param[out]     lock_immutable_id       The immutable id of lock
-@param[out]     heap_id                 The heap number found, for record locks
+@param[in] 	lock_id		        The lock id string to parse
+@param[out]	trx_immutable_id	The immutable id of lock->trx
+@param[out]	lock_immutable_id       The immutable id of lock
+@param[out]	heap_id		        The heap number found, for record locks
 @returns The type of lock found.
-@retval LOCK_TABLE      Table lock
-@retval LOCK_REC        Record lock
-@retval 0               Format error
+@retval LOCK_TABLE	Table lock
+@retval LOCK_REC	Record lock
+@retval 0		Format error
 */
 int scan_lock_id(const char *lock_id, uint64_t *trx_immutable_id,
                  uint64_t *lock_immutable_id, ulint *heap_id) {
@@ -570,9 +565,9 @@ int scan_lock_id(const char *lock_id, uint64_t *trx_immutable_id,
   return lock_type;
 }
 
-Innodb_data_lock_iterator::Innodb_data_lock_iterator() = default;
+Innodb_data_lock_iterator::Innodb_data_lock_iterator() {}
 
-Innodb_data_lock_iterator::~Innodb_data_lock_iterator() = default;
+Innodb_data_lock_iterator::~Innodb_data_lock_iterator() {}
 
 bool Innodb_data_lock_iterator::scan(PSI_server_data_lock_container *container,
                                      bool with_lock_data) {
@@ -588,34 +583,38 @@ bool Innodb_data_lock_iterator::scan(PSI_server_data_lock_container *container,
     return true;
   }
 
-  /* We want locks reported in a single scan to be a consistent snapshot. */
-  locksys::Global_exclusive_latch_guard guard{UT_LOCATION_HERE};
+  lock_mutex_enter();
 
   trx_sys_mutex_enter();
 
   size_t found = 0;
 
   while ((m_scan_state.get_pass() == SCANNING_RW_TRX_LIST) && (found == 0)) {
-    found = scan_trx_list(container, with_lock_data, &trx_sys->rw_trx_list);
+    found =
+        scan_trx_list(container, with_lock_data, true, &trx_sys->rw_trx_list);
     m_scan_state.prepare_next_scan();
   }
 
   while ((m_scan_state.get_pass() == SCANNING_MYSQL_TRX_LIST) && (found == 0)) {
-    found = scan_trx_list(container, with_lock_data, &trx_sys->mysql_trx_list);
+    found = scan_trx_list(container, with_lock_data, false,
+                          &trx_sys->mysql_trx_list);
     m_scan_state.prepare_next_scan();
   }
 
   trx_sys_mutex_exit();
 
+  lock_mutex_exit();
+
   return false;
 }
 
 bool Innodb_data_lock_iterator::fetch(PSI_server_data_lock_container *container,
-                                      const char *engine_lock_id, size_t,
+                                      const char *engine_lock_id,
+                                      size_t engine_lock_id_length,
                                       bool with_lock_data) {
   int record_type;
   uint64_t trx_immutable_id;
-  ulint heap_id{0};
+  ulint heap_id;
   uint64_t lock_immutable_id;
   const trx_t *trx;
 
@@ -630,15 +629,15 @@ bool Innodb_data_lock_iterator::fetch(PSI_server_data_lock_container *container,
     return true;
   }
 
-  /* scan_trx() requires exclusive global latch to iterate over locks of trx */
-  locksys::Global_exclusive_latch_guard guard{UT_LOCATION_HERE};
+  lock_mutex_enter();
 
   trx_sys_mutex_enter();
 
-  trx = fetch_trx_in_trx_list(trx_immutable_id, &trx_sys->rw_trx_list);
+  trx = fetch_trx_in_trx_list(trx_immutable_id, true, &trx_sys->rw_trx_list);
 
   if (trx == nullptr) {
-    trx = fetch_trx_in_trx_list(trx_immutable_id, &trx_sys->mysql_trx_list);
+    trx = fetch_trx_in_trx_list(trx_immutable_id, false,
+                                &trx_sys->mysql_trx_list);
   }
 
   if (trx != nullptr) {
@@ -647,34 +646,34 @@ bool Innodb_data_lock_iterator::fetch(PSI_server_data_lock_container *container,
 
   trx_sys_mutex_exit();
 
+  lock_mutex_exit();
+
   return true;
 }
+
 /** Scan a trx list.
-@param[in] container            The container to fill
-@param[in] with_lock_data       True if column LOCK_DATA
+@param[in] container		The container to fill
+@param[in] with_lock_data	True if column LOCK_DATA
 needs to be populated.
-@param[in] trx_list             The trx list to scan
+@param[in] read_write		True if the trx list is the RW list
+@param[in] trx_list		The trx list to scan
 @returns The number of records found
 */
-template <typename Trx_list>
 size_t Innodb_data_lock_iterator::scan_trx_list(
     PSI_server_data_lock_container *container, bool with_lock_data,
-    Trx_list *trx_list) {
+    bool read_write, trx_ut_list_t *trx_list) {
+  const trx_t *trx;
   trx_id_t trx_id;
   size_t found = 0;
-  constexpr bool read_write = is_read_write<Trx_list>();
-  /* We are about to scan over various locks of multiple transactions not
-  limited to any particular shard thus we need an exclusive latch on lock_sys */
-  ut_ad(locksys::owns_exclusive_global_latch());
+
+  ut_ad(lock_mutex_own());
   ut_ad(trx_sys_mutex_own());
 
-  for (auto trx : *trx_list) {
+  for (trx = UT_LIST_GET_FIRST(*trx_list); trx != nullptr;
+       trx = get_next_trx(trx, read_write)) {
     if (discard_trx(trx, read_write)) {
       continue;
     }
-    /* Note: for read-only transactions the check above might be based on stale
-    values and does not bring any guarantee after it is finished, because state
-    of read-only transaction might be modified outside the trx_sys->mutex. */
 
     trx_id = trx_get_id_for_print(trx);
 
@@ -691,14 +690,14 @@ size_t Innodb_data_lock_iterator::scan_trx_list(
 /** Scan a given trx.
 Either scan all the locks for a transaction,
 or scan only records matching a given lock.
-@param[in] container                  The container to fill
-@param[in] with_lock_data             True if column LOCK_DATA
+@param[in] container		      The container to fill
+@param[in] with_lock_data	      True if column LOCK_DATA
 needs to be populated.
-@param[in] trx                        The trx to scan
-@param[in] with_filter                True if looking for a specific
+@param[in] trx			      The trx to scan
+@param[in] with_filter		      True if looking for a specific
 record only.
 @param[in] filter_lock_immutable_id   Immutable id of lock_t we are looking for
-@param[in] filter_heap_id             Heap id to look for, when filtering
+@param[in] filter_heap_id	      Heap id to look for, when filtering
 @returns The number of records found
 */
 size_t Innodb_data_lock_iterator::scan_trx(
@@ -733,7 +732,7 @@ size_t Innodb_data_lock_iterator::scan_trx(
   ulint heap_no;
   int record_type;
   lock_t *wait_lock;
-  ut_ad(locksys::owns_exclusive_global_latch());
+
   wait_lock = trx->lock.wait_lock;
 
   trx_id = trx_get_id_for_print(trx);
@@ -758,10 +757,10 @@ size_t Innodb_data_lock_iterator::scan_trx(
     }
 
     table_path = lock_get_table_name(lock).m_name;
-    parse_table_path(container, table_path, &table_schema, &table_schema_length,
-                     &table_name, &table_name_length, &partition_name,
-                     &partition_name_length, &subpartition_name,
-                     &subpartition_name_length);
+    parse_table_path(container, table_path, strlen(table_path), &table_schema,
+                     &table_schema_length, &table_name, &table_name_length,
+                     &partition_name, &partition_name_length,
+                     &subpartition_name, &subpartition_name_length);
 
     if (!container->accept_object(table_schema, table_schema_length, table_name,
                                   table_name_length, partition_name,
@@ -837,6 +836,12 @@ size_t Innodb_data_lock_iterator::scan_trx(
   return found;
 }
 
+Innodb_data_lock_wait_iterator::Innodb_data_lock_wait_iterator()
+
+{}
+
+Innodb_data_lock_wait_iterator::~Innodb_data_lock_wait_iterator() {}
+
 bool Innodb_data_lock_wait_iterator::scan(
     PSI_server_data_lock_wait_container *container) {
   if (m_scan_state.get_pass() == INIT_SCANNING) {
@@ -851,32 +856,35 @@ bool Innodb_data_lock_wait_iterator::scan(
     return true;
   }
 
-  /* We want locks reported in a single scan to be a consistent snapshot. */
-  locksys::Global_exclusive_latch_guard guard{UT_LOCATION_HERE};
+  lock_mutex_enter();
 
   trx_sys_mutex_enter();
 
   size_t found = 0;
 
   while ((m_scan_state.get_pass() == SCANNING_RW_TRX_LIST) && (found == 0)) {
-    found = scan_trx_list(container, &trx_sys->rw_trx_list);
+    found = scan_trx_list(container, true, &trx_sys->rw_trx_list);
     m_scan_state.prepare_next_scan();
   }
 
   while ((m_scan_state.get_pass() == SCANNING_MYSQL_TRX_LIST) && (found == 0)) {
-    found = scan_trx_list(container, &trx_sys->mysql_trx_list);
+    found = scan_trx_list(container, false, &trx_sys->mysql_trx_list);
     m_scan_state.prepare_next_scan();
   }
 
   trx_sys_mutex_exit();
 
+  lock_mutex_exit();
+
   return false;
 }
 
 bool Innodb_data_lock_wait_iterator::fetch(
     PSI_server_data_lock_wait_container *container,
-    const char *requesting_engine_lock_id, size_t,
-    const char *blocking_engine_lock_id, size_t) {
+    const char *requesting_engine_lock_id,
+    size_t requesting_engine_lock_id_length,
+    const char *blocking_engine_lock_id,
+    size_t blocking_engine_lock_id_length) {
   int requesting_record_type;
   uint64_t requesting_trx_immutable_id;
   ulint requesting_heap_id;
@@ -907,16 +915,15 @@ bool Innodb_data_lock_wait_iterator::fetch(
     return true;
   }
 
-  /* scan_trx() requires exclusive global latch to iterate over locks of trx */
-  locksys::Global_exclusive_latch_guard guard{UT_LOCATION_HERE};
+  lock_mutex_enter();
 
   trx_sys_mutex_enter();
 
-  trx =
-      fetch_trx_in_trx_list(requesting_trx_immutable_id, &trx_sys->rw_trx_list);
+  trx = fetch_trx_in_trx_list(requesting_trx_immutable_id, true,
+                              &trx_sys->rw_trx_list);
 
   if (trx == nullptr) {
-    trx = fetch_trx_in_trx_list(requesting_trx_immutable_id,
+    trx = fetch_trx_in_trx_list(requesting_trx_immutable_id, false,
                                 &trx_sys->mysql_trx_list);
   }
 
@@ -927,33 +934,32 @@ bool Innodb_data_lock_wait_iterator::fetch(
 
   trx_sys_mutex_exit();
 
+  lock_mutex_exit();
+
   return true;
 }
 
 /** Scan a given transaction list.
-@param[in] container            The container to fill
-@param[in] trx_list             The trx list to scan
+@param[in] container		The container to fill
+@param[in] read_write		True if the transaction list is the RW list.
+@param[in] trx_list		The trx list to scan
 @returns the number of records found.
 */
-template <typename Trx_list>
 size_t Innodb_data_lock_wait_iterator::scan_trx_list(
-    PSI_server_data_lock_wait_container *container, Trx_list *trx_list) {
+    PSI_server_data_lock_wait_container *container, bool read_write,
+    trx_ut_list_t *trx_list) {
+  const trx_t *trx;
   trx_id_t trx_id;
   size_t found = 0;
-  constexpr bool read_write = is_read_write<Trx_list>();
 
-  /* We are about to scan over various locks of multiple transactions not
-  limited to any particular shard thus we need an exclusive latch on lock_sys */
-  ut_ad(locksys::owns_exclusive_global_latch());
+  ut_ad(lock_mutex_own());
   ut_ad(trx_sys_mutex_own());
 
-  for (auto trx : *trx_list) {
+  for (trx = UT_LIST_GET_FIRST(*trx_list); trx != nullptr;
+       trx = get_next_trx(trx, read_write)) {
     if (discard_trx(trx, read_write)) {
       continue;
     }
-    /* Note: for read-only transactions the check above might be based on stale
-    values and does not bring any guarantee after it is finished, because state
-    of read-only transaction might be modified outside the trx_sys->mutex. */
 
     trx_id = trx_get_id_for_print(trx);
 
@@ -970,12 +976,12 @@ size_t Innodb_data_lock_wait_iterator::scan_trx_list(
 /** Scan a given transaction.
 Either scan all the waits for a transaction,
 or scan only records matching a given wait.
-@param[in] container            The container to fill
-@param[in] trx                  The trx to scan
-@param[in] with_filter          True if looking for a given wait only.
-@param[in] filter_requesting_lock_immutable_id          Immutable id of
+@param[in] container		The container to fill
+@param[in] trx			The trx to scan
+@param[in] with_filter		True if looking for a given wait only.
+@param[in] filter_requesting_lock_immutable_id		Immutable id of
 lock_t for the requesting lock, when filtering
-@param[in] filter_blocking_lock_immutable_id            Immutable idof
+@param[in] filter_blocking_lock_immutable_id		Immutable idof
 lock_t for the blocking lock, when filtering
 @returns the number of records found.
 */
@@ -1001,8 +1007,7 @@ size_t Innodb_data_lock_wait_iterator::scan_trx(
   const void *blocking_identity;
   char blocking_engine_lock_id[TRX_I_S_LOCK_ID_MAX_LEN + 1];
   size_t blocking_engine_lock_id_length;
-  ut_ad(locksys::owns_exclusive_global_latch());
-  const lock_t *wait_lock = trx->lock.wait_lock;
+  lock_t *wait_lock = trx->lock.wait_lock;
   const lock_t *curr_lock;
   int requesting_record_type;
   size_t found = 0;
@@ -1043,7 +1048,7 @@ size_t Innodb_data_lock_wait_iterator::scan_trx(
 
   requesting_identity = wait_lock;
   lock_queue_iterator_reset(&iter, wait_lock, ULINT_UNDEFINED);
-  locksys::Trx_locks_cache wait_lock_cache{};
+
   for (curr_lock = lock_queue_iterator_get_prev(&iter); curr_lock != nullptr;
        curr_lock = lock_queue_iterator_get_prev(&iter)) {
     if (with_filter &&
@@ -1051,7 +1056,7 @@ size_t Innodb_data_lock_wait_iterator::scan_trx(
       continue;
     }
 
-    if (locksys::has_to_wait(wait_lock, curr_lock, wait_lock_cache)) {
+    if (lock_has_to_wait(wait_lock, curr_lock)) {
       blocking_trx_id = lock_get_trx_id(curr_lock);
       if (!container->accept_blocking_transaction_id(blocking_trx_id)) {
         continue;
