@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1996, 2023, Oracle and/or its affiliates.
+Copyright (c) 1996, 2019, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
@@ -46,7 +46,7 @@ this program; if not, write to the Free Software Foundation, Inc.,
 #include <deque>
 
 /** A stack of table names related through foreign key constraints */
-typedef std::deque<const char *, ut::allocator<const char *>> dict_names_t;
+typedef std::deque<const char *, ut_allocator<const char *>> dict_names_t;
 
 /** enum that defines all system table IDs. @see SYSTEM_TABLE_NAME[] */
 enum dict_system_id_t {
@@ -82,41 +82,36 @@ char *dict_get_first_table_name_in_db(
     const char *name); /*!< in: database name which ends to '/' */
 
 /** Get the first filepath from SYS_DATAFILES for a given space_id.
-@param[in]      space_id        Tablespace ID
-@return First filepath (caller must invoke ut::free() on it)
+@param[in]	space_id	Tablespace ID
+@return First filepath (caller must invoke ut_free() on it)
 @retval NULL if no SYS_DATAFILES entry was found. */
 char *dict_get_first_path(ulint space_id);
 
-/** Make sure the data_dir_path is saved in dict_table_t if DATA DIRECTORY
-was used. Try to read it from the fil_system first, then from SYS_DATAFILES.
-@param[in]      table           Table object
-@param[in]      dict_mutex_own  true if dict_sys->mutex is owned already */
+/** Make sure the data_file_name is saved in dict_table_t if needed.
+Try to read it from the fil_system first, then from SYS_DATAFILES.
+@param[in]	table		Table object
+@param[in]	dict_mutex_own	true if dict_sys->mutex is owned already */
 void dict_get_and_save_data_dir_path(dict_table_t *table, bool dict_mutex_own);
 
-/** Make sure the tablespace name is saved in dict_table_t if the table
-uses a general tablespace.
-Try to read it from the fil_system_t first, then from SYS_TABLESPACES.
-@param[in]  table           Table object */
-void dict_get_and_save_space_name(dict_table_t *table);
+/** Make sure the tablespace name is saved in dict_table_t if needed.
+Try to read it from the file dictionary first, then from SYS_TABLESPACES.
+@param[in]	table		Table object
+@param[in]	dict_mutex_own	true if dict_sys->mutex is owned already */
+void dict_get_and_save_space_name(dict_table_t *table, bool dict_mutex_own);
 
 /** Loads a table definition and also all its index definitions, and also
 the cluster definition if the table is a member in a cluster. Also loads
 all foreign key constraints where the foreign key is in the table or where
 a foreign key references columns in this table.
-@param[in]      name            Table name in the dbname/tablename format
-@param[in]      cached          true=add to cache, false=do not
-@param[in]      ignore_err      Error to be ignored when loading
+@param[in]	name		Table name in the dbname/tablename format
+@param[in]	cached		true=add to cache, false=do not
+@param[in]	ignore_err	Error to be ignored when loading
                                 table and its index definition
-@param[in]      prev_table      previous table name. The current table load
-                                is happening because of the load of the
-                                previous table name.  This parameter is used
-                                to check for cyclic calls.
 @return table, NULL if does not exist; if the table is stored in an
 .ibd file, but the file does not exist, then we set the ibd_file_missing
 flag in the table object we return. */
 dict_table_t *dict_load_table(const char *name, bool cached,
-                              dict_err_ignore_t ignore_err,
-                              const std::string *prev_table = nullptr);
+                              dict_err_ignore_t ignore_err);
 
 /** This function is called when the database is booted.
  Loads system table index definitions except for the clustered index which
@@ -131,7 +126,7 @@ void dict_load_sys_table(dict_table_t *table); /*!< in: system table */
  cache, then it is added to the output parameter (fk_tables).
 
  @return DB_SUCCESS or error code */
-[[nodiscard]] dberr_t dict_load_foreigns(
+dberr_t dict_load_foreigns(
     const char *table_name,       /*!< in: table name */
     const char **col_names,       /*!< in: column names, or NULL
                                   to use table->col_names */
@@ -141,10 +136,11 @@ void dict_load_sys_table(dict_table_t *table); /*!< in: system table */
     bool check_charsets,          /*!< in: whether to check
                                   charset compatibility */
     dict_err_ignore_t ignore_err, /*!< in: error to be ignored */
-    dict_names_t &fk_tables);     /*!< out: stack of table names
-                                 which must be loaded
-                                 subsequently to load all the
-                                 foreign key constraints. */
+    dict_names_t &fk_tables)      /*!< out: stack of table names
+                                  which must be loaded
+                                  subsequently to load all the
+                                  foreign key constraints. */
+    MY_ATTRIBUTE((warn_unused_result));
 
 /** This function opens a system table, and return the first record.
  @return first record of the system table */
@@ -168,19 +164,18 @@ const char *dict_process_sys_tablespaces(
     space_id_t *space, /*!< out: space id */
     const char **name, /*!< out: tablespace name */
     uint32_t *flags);  /*!< out: tablespace flags */
-
 /** Opens a tablespace for dict_load_table_one()
-@param[in,out]  table           A table that refers to the tablespace to open
-@param[in]      ignore_err      Whether to ignore an error. */
-void dict_load_tablespace(dict_table_t *table, dict_err_ignore_t ignore_err);
+@param[in,out]	table		A table that refers to the tablespace to open
+@param[in,out]	heap		A memory heap
+@param[in]	ignore_err	Whether to ignore an error. */
+void dict_load_tablespace(dict_table_t *table, mem_heap_t *heap,
+                          dict_err_ignore_t ignore_err);
 
 /** Using the table->heap, copy the null-terminated filepath into
-table->data_dir_path. The data directory path is derived from the
+table->data_dir_path. The data directory patch is derived form the
 filepath by stripping the the table->name.m_name component suffix.
-If the filepath is not of the correct form (".../db/table.ibd"),
-then table->data_dir_path will remain nullptr.
-@param[in,out]  table           table instance
-@param[in]      filepath        filepath of tablespace */
+@param[in,out]	table		table obj
+@param[in]	filepath	filepath of tablespace */
 void dict_save_data_dir_path(dict_table_t *table, char *filepath);
 
 /** Load all tablespaces during upgrade */
@@ -200,10 +195,4 @@ tablespaces to dictionary table mysql.tablespaces */
 using missing_sys_tblsp_t = std::set<fil_space_t *, space_compare>;
 extern missing_sys_tblsp_t missing_spaces;
 
-/** This bool denotes if we found a Table or Partition with discarded Tablespace
-during load of SYS_TABLES (in dict_check_sys_tables).
-
-We use it to stop upgrade from 5.7 to 8.0 if there are discarded Tablespaces. */
-extern bool has_discarded_tablespaces;
-
 #endif
