@@ -1,18 +1,17 @@
 /*****************************************************************************
 
-Copyright (c) 2013, 2024, Oracle and/or its affiliates.
+Copyright (c) 2013, 2019, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is designed to work with certain software (including
-but not limited to OpenSSL) that is licensed under separate terms,
-as designated in a particular file or component or in included license
-documentation.  The authors of MySQL hereby grant you an additional
-permission to link the program and your derivative works with the
-separately licensed software that they have either included with
-the program or referenced in the documentation.
+This program is also distributed with certain software (including but not
+limited to OpenSSL) that is licensed under separate terms, as designated in a
+particular file or component or in included license documentation. The authors
+of MySQL hereby grant you an additional permission to link the program and
+your derivative works with the separately licensed software that they have
+included with MySQL.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -56,9 +55,9 @@ struct OSTrackMutex {
   ~OSTrackMutex() UNIV_NOTHROW { ut_ad(!m_destroy_at_exit || !m_locked); }
 
   /** Initialise the mutex.
-  @param[in]    id              Mutex ID
-  @param[in]    filename        File where mutex was created
-  @param[in]    line            Line in filename */
+  @param[in]	id              Mutex ID
+  @param[in]	filename	File where mutex was created
+  @param[in]	line		Line in filename */
   void init(latch_id_t id, const char *filename, uint32_t line) UNIV_NOTHROW {
     ut_ad(m_freed);
     ut_ad(!m_locked);
@@ -92,14 +91,12 @@ struct OSTrackMutex {
   }
 
   /** Acquire the mutex.
-  @param[in]    max_spins       max number of spins
-  @param[in]    max_delay       max delay per spin
-  @param[in]    filename        from where called
-  @param[in]    line            within filename */
-  void enter(uint32_t max_spins [[maybe_unused]],
-             uint32_t max_delay [[maybe_unused]],
-             const char *filename [[maybe_unused]],
-             uint32_t line [[maybe_unused]]) UNIV_NOTHROW {
+  @param[in]	max_spins	max number of spins
+  @param[in]	max_delay	max delay per spin
+  @param[in]	filename	from where called
+  @param[in]	line		within filename */
+  void enter(uint32_t max_spins, uint32_t max_delay, const char *filename,
+             uint32_t line) UNIV_NOTHROW {
     ut_ad(innodb_calling_exit || !m_freed);
 
     m_mutex.enter();
@@ -108,9 +105,6 @@ struct OSTrackMutex {
     ut_d(m_locked = true);
   }
 
-  void lock() { enter(); }
-  void unlock() { exit(); }
-
   /** @return true if locking succeeded */
   bool try_lock() UNIV_NOTHROW {
     ut_ad(innodb_calling_exit || !m_freed);
@@ -167,60 +161,35 @@ template <template <typename> class Policy = NoPolicy>
 struct TTASFutexMutex {
   typedef Policy<TTASFutexMutex> MutexPolicy;
 
-  /** The type of second argument to syscall(SYS_futex, uint32_t *uaddr,...)*/
-  using futex_word_t = uint32_t;
-  /** Mutex states. */
-  enum class mutex_state_t : futex_word_t {
-    /** Mutex is free */
-    UNLOCKED = 0,
-
-    /** Mutex is acquired by some thread. */
-    LOCKED = 1,
-
-    /** Mutex is contended and there are threads waiting on the lock. */
-    LOCKED_WITH_WAITERS = 2
-  };
-  using lock_word_t = mutex_state_t;
-
-  TTASFutexMutex() UNIV_NOTHROW : m_lock_word(mutex_state_t::UNLOCKED) {
-    /* The futex API operates on uint32_t futex words, aligned to 4 byte
-    boundaries. OTOH we want the luxury of accessing it via std::atomic<>.
-    Thus we need to verify that std::atomic doesn't add any extra fluff,
-    and is properly aligned */
-    using m_lock_word_t = decltype(m_lock_word);
-    static_assert(m_lock_word_t::is_always_lock_free);
-    static_assert(sizeof(m_lock_word_t) == sizeof(futex_word_t));
-#ifdef UNIV_DEBUG
-    const auto addr = reinterpret_cast<std::uintptr_t>(&m_lock_word);
-    ut_a(addr % alignof(m_lock_word_t) == 0);
-    ut_a(addr % 4 == 0);
-#endif
+  TTASFutexMutex() UNIV_NOTHROW : m_lock_word(MUTEX_STATE_UNLOCKED) {
+    /* Check that lock_word is aligned. */
+    ut_ad(!((ulint)&m_lock_word % sizeof(ulint)));
   }
 
-  ~TTASFutexMutex() { ut_a(m_lock_word == mutex_state_t::UNLOCKED); }
+  ~TTASFutexMutex() { ut_a(m_lock_word == MUTEX_STATE_UNLOCKED); }
 
   /** Called when the mutex is "created". Note: Not from the constructor
   but when the mutex is initialised.
-  @param[in]    id              Mutex ID
-  @param[in]    filename        File where mutex was created
-  @param[in]    line            Line in filename */
+  @param[in]	id		Mutex ID
+  @param[in]	filename	File where mutex was created
+  @param[in]	line		Line in filename */
   void init(latch_id_t id, const char *filename, uint32_t line) UNIV_NOTHROW {
-    ut_a(m_lock_word == mutex_state_t::UNLOCKED);
+    ut_a(m_lock_word == MUTEX_STATE_UNLOCKED);
     m_policy.init(*this, id, filename, line);
   }
 
   /** Destroy the mutex. */
   void destroy() UNIV_NOTHROW {
     /* The destructor can be called at shutdown. */
-    ut_a(m_lock_word == mutex_state_t::UNLOCKED);
+    ut_a(m_lock_word == MUTEX_STATE_UNLOCKED);
     m_policy.destroy();
   }
 
   /** Acquire the mutex.
-  @param[in]    max_spins       max number of spins
-  @param[in]    max_delay       max delay per spin
-  @param[in]    filename        from where called
-  @param[in]    line            within filename */
+  @param[in]	max_spins	max number of spins
+  @param[in]	max_delay	max delay per spin
+  @param[in]	filename	from where called
+  @param[in]	line		within filename */
   void enter(uint32_t max_spins, uint32_t max_delay, const char *filename,
              uint32_t line) UNIV_NOTHROW {
     uint32_t n_spins;
@@ -233,10 +202,18 @@ struct TTASFutexMutex {
     by then. In this case the thread can assume it
     was granted the mutex. */
 
-    const uint32_t n_waits = (lock == mutex_state_t::LOCKED_WITH_WAITERS ||
-                              (lock == mutex_state_t::LOCKED && !set_waiters()))
-                                 ? wait()
-                                 : 0;
+    uint32_t n_waits;
+
+    if (lock != MUTEX_STATE_UNLOCKED) {
+      if (lock != MUTEX_STATE_LOCKED || !set_waiters()) {
+        n_waits = wait();
+      } else {
+        n_waits = 0;
+      }
+
+    } else {
+      n_waits = 0;
+    }
 
     m_policy.add(n_spins, n_waits);
   }
@@ -249,10 +226,10 @@ struct TTASFutexMutex {
 
     std::atomic_thread_fence(std::memory_order_acquire);
 
-    if (state() == mutex_state_t::LOCKED_WITH_WAITERS) {
-      m_lock_word = mutex_state_t::UNLOCKED;
+    if (state() == MUTEX_STATE_WAITERS) {
+      m_lock_word = MUTEX_STATE_UNLOCKED;
 
-    } else if (unlock() == mutex_state_t::LOCKED) {
+    } else if (unlock() == MUTEX_STATE_LOCKED) {
       /* No threads waiting, no need to signal a wakeup. */
       return;
     }
@@ -263,21 +240,16 @@ struct TTASFutexMutex {
   /** Try and lock the mutex.
   @return the old state of the mutex */
   lock_word_t trylock() UNIV_NOTHROW {
-    lock_word_t unlocked = mutex_state_t::UNLOCKED;
-    m_lock_word.compare_exchange_strong(unlocked, mutex_state_t::LOCKED);
-    return unlocked;
+    return (CAS(&m_lock_word, MUTEX_STATE_UNLOCKED, MUTEX_STATE_LOCKED));
   }
 
   /** Try and lock the mutex.
   @return true if successful */
-  bool try_lock() UNIV_NOTHROW {
-    lock_word_t unlocked = mutex_state_t::UNLOCKED;
-    return m_lock_word.compare_exchange_strong(unlocked, mutex_state_t::LOCKED);
-  }
+  bool try_lock() UNIV_NOTHROW { return (trylock() == MUTEX_STATE_UNLOCKED); }
 
   /** @return true if mutex is unlocked */
   bool is_locked() const UNIV_NOTHROW {
-    return (state() != mutex_state_t::UNLOCKED);
+    return (state() != MUTEX_STATE_UNLOCKED);
   }
 
 #ifdef UNIV_DEBUG
@@ -298,16 +270,22 @@ struct TTASFutexMutex {
   lock_word_t state() const UNIV_NOTHROW { return (m_lock_word); }
 
   /** Release the mutex.
-  @return the old state of the mutex */
+  @return the new state of the mutex */
   lock_word_t unlock() UNIV_NOTHROW {
-    return m_lock_word.exchange(mutex_state_t::UNLOCKED);
+    return (TAS(&m_lock_word, MUTEX_STATE_UNLOCKED));
   }
 
   /** Note that there are threads waiting and need to be woken up.
-  @return true if state was mutex_state_t::UNLOCKED (ie. granted) */
+  @return true if state was MUTEX_STATE_UNLOCKED (ie. granted) */
   bool set_waiters() UNIV_NOTHROW {
-    return m_lock_word.exchange(mutex_state_t::LOCKED_WITH_WAITERS) ==
-           mutex_state_t::UNLOCKED;
+    return (TAS(&m_lock_word, MUTEX_STATE_WAITERS) == MUTEX_STATE_UNLOCKED);
+  }
+
+  /** Set the waiters flag, only if the mutex is locked
+  @return true if succesful. */
+  bool try_set_waiters() UNIV_NOTHROW {
+    return (CAS(&m_lock_word, MUTEX_STATE_LOCKED, MUTEX_STATE_WAITERS) !=
+            MUTEX_STATE_UNLOCKED);
   }
 
   /** Wait if the lock is contended.
@@ -321,8 +299,8 @@ struct TTASFutexMutex {
     do {
       ++n_waits;
 
-      syscall(SYS_futex, &m_lock_word, FUTEX_WAIT_PRIVATE,
-              mutex_state_t::LOCKED_WITH_WAITERS, 0, 0, 0);
+      syscall(SYS_futex, &m_lock_word, FUTEX_WAIT_PRIVATE, MUTEX_STATE_WAITERS,
+              0, 0, 0);
 
       // Since we are retrying the operation the return
       // value doesn't matter.
@@ -338,9 +316,9 @@ struct TTASFutexMutex {
   }
 
   /** Poll waiting for mutex to be unlocked.
-  @param[in]    max_spins       max spins
-  @param[in]    max_delay       max delay per spin
-  @param[out]   n_spins         retries before acquire
+  @param[in]	max_spins	max spins
+  @param[in]	max_delay	max delay per spin
+  @param[out]	n_spins		retries before acquire
   @return value of lock word before locking. */
   lock_word_t ttas(uint32_t max_spins, uint32_t max_delay,
                    uint32_t &n_spins) UNIV_NOTHROW {
@@ -350,13 +328,13 @@ struct TTASFutexMutex {
       if (!is_locked()) {
         lock_word_t lock = trylock();
 
-        if (lock == mutex_state_t::UNLOCKED) {
+        if (lock == MUTEX_STATE_UNLOCKED) {
           /* Lock successful */
           return (lock);
         }
       }
 
-      ut_delay(ut::random_from_interval_fast(0, max_delay));
+      ut_delay(ut_rnd_interval(0, max_delay));
     }
 
     return (trylock());
@@ -366,47 +344,182 @@ struct TTASFutexMutex {
   /** Policy data */
   MutexPolicy m_policy;
 
-  alignas(4) std::atomic<lock_word_t> m_lock_word;
+  /** lock_word is the target of the atomic test-and-set instruction
+  when atomic operations are enabled. */
+  alignas(ulint) lock_word_t m_lock_word;
 };
 
 #endif /* HAVE_IB_LINUX_FUTEX */
 
 template <template <typename> class Policy = NoPolicy>
-struct TTASEventMutex {
-  typedef Policy<TTASEventMutex> MutexPolicy;
+struct TTASMutex {
+  typedef Policy<TTASMutex> MutexPolicy;
 
-  TTASEventMutex() UNIV_NOTHROW {
-    /* Check that m_owner is aligned. */
-    using m_owner_t = decltype(m_owner);
-    ut_ad(reinterpret_cast<std::uintptr_t>(&m_owner) % alignof(m_owner_t) == 0);
-    static_assert(m_owner_t::is_always_lock_free);
+  TTASMutex() UNIV_NOTHROW : m_lock_word(MUTEX_STATE_UNLOCKED) {
+    /* Check that lock_word is aligned. */
+    ut_ad(!((ulint)&m_lock_word % sizeof(ulint)));
   }
 
-  ~TTASEventMutex() UNIV_NOTHROW { ut_ad(!is_locked()); }
-
-  /** If the lock is locked, returns the current owner of the lock, otherwise
-  returns the default std::thread::id{} */
-  std::thread::id peek_owner() const UNIV_NOTHROW { return m_owner.load(); }
+  ~TTASMutex() { ut_ad(m_lock_word == MUTEX_STATE_UNLOCKED); }
 
   /** Called when the mutex is "created". Note: Not from the constructor
   but when the mutex is initialised.
-  @param[in]    id              Mutex ID
-  @param[in]    filename        File where mutex was created
-  @param[in]    line            Line in filename */
+  @param[in]	id		Mutex ID
+  @param[in]	filename	File where mutex was created
+  @param[in]	line		Line in filename */
+  void init(latch_id_t id, const char *filename, uint32_t line) UNIV_NOTHROW {
+    ut_ad(m_lock_word == MUTEX_STATE_UNLOCKED);
+    m_policy.init(*this, id, filename, line);
+  }
+
+  /** Destroy the mutex. */
+  void destroy() UNIV_NOTHROW {
+    /* The destructor can be called at shutdown. */
+    ut_ad(m_lock_word == MUTEX_STATE_UNLOCKED);
+    m_policy.destroy();
+  }
+
+  /**
+  Try and acquire the lock using TestAndSet.
+  @return	true if lock succeeded */
+  bool tas_lock() UNIV_NOTHROW {
+    return (TAS(&m_lock_word, MUTEX_STATE_LOCKED) == MUTEX_STATE_UNLOCKED);
+  }
+
+  /** In theory __sync_lock_release should be used to release the lock.
+  Unfortunately, it does not work properly alone. The workaround is
+  that more conservative __sync_lock_test_and_set is used instead. */
+  void tas_unlock() UNIV_NOTHROW {
+#ifdef UNIV_DEBUG
+    ut_ad(state() == MUTEX_STATE_LOCKED);
+
+    lock_word_t lock =
+#endif /* UNIV_DEBUG */
+
+        TAS(&m_lock_word, MUTEX_STATE_UNLOCKED);
+
+    ut_ad(lock == MUTEX_STATE_LOCKED);
+  }
+
+  /** Try and lock the mutex.
+  @return true on success */
+  bool try_lock() UNIV_NOTHROW { return (tas_lock()); }
+
+  /** Release the mutex. */
+  void exit() UNIV_NOTHROW { tas_unlock(); }
+
+  /** Acquire the mutex.
+  @param max_spins	max number of spins
+  @param max_delay	max delay per spin
+  @param filename		from where called
+  @param line		within filename */
+  void enter(uint32_t max_spins, uint32_t max_delay, const char *filename,
+             uint32_t line) UNIV_NOTHROW {
+    if (!try_lock()) {
+      uint32_t n_spins = ttas(max_spins, max_delay);
+
+      /* No OS waits for spin mutexes */
+      m_policy.add(n_spins, 0);
+    }
+  }
+
+  /** @return the lock state. */
+  lock_word_t state() const UNIV_NOTHROW { return (m_lock_word); }
+
+  /** @return true if locked by some thread */
+  bool is_locked() const UNIV_NOTHROW {
+    return (m_lock_word != MUTEX_STATE_UNLOCKED);
+  }
+
+#ifdef UNIV_DEBUG
+  /** @return true if the calling thread owns the mutex. */
+  bool is_owned() const UNIV_NOTHROW {
+    return (is_locked() && m_policy.is_owned());
+  }
+#endif /* UNIV_DEBUG */
+
+  /** @return non-const version of the policy */
+  MutexPolicy &policy() UNIV_NOTHROW { return (m_policy); }
+
+  /** @return const version of the policy */
+  const MutexPolicy &policy() const UNIV_NOTHROW { return (m_policy); }
+
+ private:
+  /** Spin and try to acquire the lock.
+  @param[in]	max_spins	max spins
+  @param[in]	max_delay	max delay per spin
+  @return number spins before acquire */
+  uint32_t ttas(uint32_t max_spins, uint32_t max_delay) UNIV_NOTHROW {
+    uint32_t i = 0;
+    const uint32_t step = max_spins;
+
+    std::atomic_thread_fence(std::memory_order_acquire);
+
+    do {
+      while (is_locked()) {
+        ut_delay(ut_rnd_interval(0, max_delay));
+
+        ++i;
+
+        if (i >= max_spins) {
+          max_spins += step;
+
+          os_thread_yield();
+
+          break;
+        }
+      }
+
+    } while (!try_lock());
+
+    return (i);
+  }
+
+ private:
+  // Disable copying
+  TTASMutex(const TTASMutex &);
+  TTASMutex &operator=(const TTASMutex &);
+
+  /** Policy data */
+  MutexPolicy m_policy;
+
+  /** lock_word is the target of the atomic test-and-set instruction
+  when atomic operations are enabled. */
+  lock_word_t m_lock_word;
+};
+
+template <template <typename> class Policy = NoPolicy>
+struct TTASEventMutex {
+  typedef Policy<TTASEventMutex> MutexPolicy;
+
+  TTASEventMutex() UNIV_NOTHROW : m_lock_word(MUTEX_STATE_UNLOCKED),
+                                  m_waiters(),
+                                  m_event() {
+    /* Check that lock_word is aligned. */
+    ut_ad(!((ulint)&m_lock_word % sizeof(ulint)));
+  }
+
+  ~TTASEventMutex() UNIV_NOTHROW { ut_ad(!m_lock_word.load()); }
+
+  /** Called when the mutex is "created". Note: Not from the constructor
+  but when the mutex is initialised.
+  @param[in]	id		Mutex ID
+  @param[in]	filename	File where mutex was created
+  @param[in]	line		Line in filename */
   void init(latch_id_t id, const char *filename, uint32_t line) UNIV_NOTHROW {
     ut_a(m_event == nullptr);
-    ut_a(!is_locked());
+    ut_a(!m_lock_word.load(std::memory_order_relaxed));
 
-    m_event = os_event_create();
+    m_event = os_event_create(sync_latch_get_name(id));
 
     m_policy.init(*this, id, filename, line);
   }
 
-  /** This is the real destructor. This mutex can be created in BSS and
-  its destructor will be called on exit(). We can't call
+  /** This is the real desctructor. This mutex can be created in BSS and
+  its desctructor will be called on exit(). We can't call
   os_event_destroy() at that stage. */
   void destroy() UNIV_NOTHROW {
-    ut_ad(!is_locked());
+    ut_ad(!m_lock_word.load(std::memory_order_relaxed));
 
     /* We have to free the event before InnoDB shuts down. */
     os_event_destroy(m_event);
@@ -418,25 +531,25 @@ struct TTASEventMutex {
   /** Try and lock the mutex. Note: POSIX returns 0 on success.
   @return true on success */
   bool try_lock() UNIV_NOTHROW {
-    auto expected = std::thread::id{};
-    return m_owner.compare_exchange_strong(expected,
-                                           std::this_thread::get_id());
+    bool expected = false;
+    return (m_lock_word.compare_exchange_strong(expected, true));
   }
 
   /** Release the mutex. */
   void exit() UNIV_NOTHROW {
-    m_owner.store(std::thread::id{});
+    m_lock_word.store(false);
+    std::atomic_thread_fence(std::memory_order_acquire);
 
-    if (m_waiters.load()) {
+    if (m_waiters.load(std::memory_order_acquire)) {
       signal();
     }
   }
 
   /** Acquire the mutex.
-  @param[in]    max_spins       max number of spins
-  @param[in]    max_delay       max delay per spin
-  @param[in]    filename        from where called
-  @param[in]    line            within filename */
+  @param[in]	max_spins	max number of spins
+  @param[in]	max_delay	max delay per spin
+  @param[in]	filename	from where called
+  @param[in]	line		within filename */
   void enter(uint32_t max_spins, uint32_t max_delay, const char *filename,
              uint32_t line) UNIV_NOTHROW {
     if (!try_lock()) {
@@ -444,19 +557,26 @@ struct TTASEventMutex {
     }
   }
 
+  /** @return true if locked. */
+  bool state() const UNIV_NOTHROW {
+    return (m_lock_word.load(std::memory_order_relaxed));
+  }
+
   /** The event that the mutex will wait in sync0arr.cc
   @return even instance */
   os_event_t event() UNIV_NOTHROW { return (m_event); }
 
   /** @return true if locked by some thread */
   bool is_locked() const UNIV_NOTHROW {
-    return peek_owner() != std::thread::id{};
+    return (m_lock_word.load(std::memory_order_relaxed));
   }
 
+#ifdef UNIV_DEBUG
   /** @return true if the calling thread owns the mutex. */
   bool is_owned() const UNIV_NOTHROW {
-    return peek_owner() == std::this_thread::get_id();
+    return (is_locked() && m_policy.is_owned());
   }
+#endif /* UNIV_DEBUG */
 
   /** @return non-const version of the policy */
   MutexPolicy &policy() UNIV_NOTHROW { return (m_policy); }
@@ -466,16 +586,16 @@ struct TTASEventMutex {
 
  private:
   /** Wait in the sync array.
-  @param[in]    filename        from where it was called
-  @param[in]    line            line number in file
-  @param[in]    spin            retry this many times again
+  @param[in]	filename	from where it was called
+  @param[in]	line		line number in file
+  @param[in]	spin		retry this many times again
   @return true if the mutex acquisition was successful. */
   bool wait(const char *filename, uint32_t line, uint32_t spin) UNIV_NOTHROW;
 
   /** Spin and wait for the mutex to become free.
-  @param[in]    max_spins       max spins
-  @param[in]    max_delay       max delay per spin
-  @param[in,out]        n_spins         spin start index
+  @param[in]	max_spins	max spins
+  @param[in]	max_delay	max delay per spin
+  @param[in,out]	n_spins		spin start index
   @return true if unlocked */
   bool is_free(uint32_t max_spins, uint32_t max_delay,
                uint32_t &n_spins) const UNIV_NOTHROW {
@@ -492,7 +612,7 @@ struct TTASEventMutex {
         return (true);
       }
 
-      ut_delay(ut::random_from_interval_fast(0, max_delay));
+      ut_delay(ut_rnd_interval(0, max_delay));
 
       ++n_spins;
 
@@ -502,10 +622,10 @@ struct TTASEventMutex {
   }
 
   /** Spin while trying to acquire the mutex
-  @param[in]    max_spins       max number of spins
-  @param[in]    max_delay       max delay per spin
-  @param[in]    filename        from where called
-  @param[in]    line            within filename */
+  @param[in]	max_spins	max number of spins
+  @param[in]	max_delay	max delay per spin
+  @param[in]	filename	from where called
+  @param[in]	line		within filename */
   void spin_and_try_lock(uint32_t max_spins, uint32_t max_delay,
                          const char *filename, uint32_t line) UNIV_NOTHROW {
     uint32_t n_spins = 0;
@@ -528,7 +648,7 @@ struct TTASEventMutex {
 
       ++n_waits;
 
-      std::this_thread::yield();
+      os_thread_yield();
 
       /* The 4 below is a heuristic that has existed for a
       very long time now. It is unclear if changing this
@@ -552,11 +672,20 @@ struct TTASEventMutex {
     m_policy.add(n_spins, n_waits);
   }
 
+  /** @return the value of the m_waiters flag */
+  lock_word_t waiters() UNIV_NOTHROW {
+    return (m_waiters.load(std::memory_order_relaxed));
+  }
+
   /** Note that there are threads waiting on the mutex */
-  void set_waiters() UNIV_NOTHROW { m_waiters.store(true); }
+  void set_waiters() UNIV_NOTHROW {
+    m_waiters.store(true, std::memory_order_release);
+  }
 
   /** Note that there are no threads waiting on the mutex */
-  void clear_waiters() UNIV_NOTHROW { m_waiters.store(false); }
+  void clear_waiters() UNIV_NOTHROW {
+    m_waiters.store(false, std::memory_order_release);
+  }
 
   /** Wakeup any waiting thread(s). */
   void signal() UNIV_NOTHROW;
@@ -568,19 +697,19 @@ struct TTASEventMutex {
   TTASEventMutex &operator=(TTASEventMutex &&) = delete;
   TTASEventMutex &operator=(const TTASEventMutex &) = delete;
 
-  /** Set to owner's thread's id when locked, and reset to the default
-  std::thread::id{} when unlocked. */
-  std::atomic<std::thread::id> m_owner{std::thread::id{}};
-
-  /** Used by sync0arr.cc for the wait queue */
-  os_event_t m_event{};
-
-  /** Policy data */
-  MutexPolicy m_policy;
+  /** lock_word is the target of the atomic test-and-set instruction
+  when atomic operations are enabled. */
+  std::atomic_bool m_lock_word;
 
   /** true if there are (or may be) threads waiting
   in the global wait array for this mutex to be released. */
-  std::atomic_bool m_waiters{false};
+  std::atomic_bool m_waiters;
+
+  /** Used by sync0arr.cc for the wait queue */
+  os_event_t m_event;
+
+  /** Policy data */
+  MutexPolicy m_policy;
 };
 
 /** Mutex interface for all policy mutexes. This class handles the interfacing
@@ -596,7 +725,7 @@ struct PolicyMutex {
 #endif /* UNIV_PFS_MUTEX */
   }
 
-  ~PolicyMutex() = default;
+  ~PolicyMutex() {}
 
   /** @return non-const version of the policy */
   Policy &policy() UNIV_NOTHROW { return (m_impl.policy()); }
@@ -616,10 +745,10 @@ struct PolicyMutex {
   }
 
   /** Acquire the mutex.
-  @param n_spins        max number of spins
-  @param n_delay        max delay per spin
-  @param name   filename where locked
-  @param line   line number where locked */
+  @param n_spins	max number of spins
+  @param n_delay	max delay per spin
+  @param name	filename where locked
+  @param line	line number where locked */
   void enter(uint32_t n_spins, uint32_t n_delay, const char *name,
              uint32_t line) UNIV_NOTHROW {
 #ifdef UNIV_PFS_MUTEX
@@ -643,8 +772,8 @@ struct PolicyMutex {
   }
 
   /** Try and lock the mutex, return 0 on SUCCESS and 1 otherwise.
-  @param name   filename where locked
-  @param line   line number where locked */
+  @param name	filename where locked
+  @param line	line number where locked */
   int trylock(const char *name, uint32_t line) UNIV_NOTHROW {
 #ifdef UNIV_PFS_MUTEX
     /* Note: locker is really an alias for state. That's why
@@ -658,7 +787,7 @@ struct PolicyMutex {
 
     /* There is a subtlety here, we check the mutex ordering
     after locking here. This is only done to avoid add and
-    then remove if the trylock was unsuccessful. */
+    then remove if the trylock was unsuccesful. */
 
     int ret = m_impl.try_lock() ? 0 : 1;
 
@@ -683,9 +812,9 @@ struct PolicyMutex {
   /**
   Initialise the mutex.
 
-  @param[in]    id              Mutex ID
-  @param[in]    filename        file where created
-  @param[in]    line            line number in file where created */
+  @param[in]	id              Mutex ID
+  @param[in]	filename	file where created
+  @param[in]	line		line number in file where created */
   void init(latch_id_t id, const char *filename, uint32_t line) UNIV_NOTHROW {
 #ifdef UNIV_PFS_MUTEX
     pfs_add(sync_latch_get_pfs_key(id));
@@ -730,10 +859,8 @@ struct PolicyMutex {
                                    const char *name,
                                    uint32_t line) UNIV_NOTHROW {
     if (m_ptr != nullptr) {
-      if (m_ptr->m_enabled) {
-        return (PSI_MUTEX_CALL(start_mutex_wait)(state, m_ptr, PSI_MUTEX_LOCK,
-                                                 name, (uint)line));
-      }
+      return (PSI_MUTEX_CALL(start_mutex_wait)(state, m_ptr, PSI_MUTEX_LOCK,
+                                               name, (uint)line));
     }
 
     return (nullptr);
@@ -747,10 +874,8 @@ struct PolicyMutex {
                                       const char *name,
                                       uint32_t line) UNIV_NOTHROW {
     if (m_ptr != nullptr) {
-      if (m_ptr->m_enabled) {
-        return (PSI_MUTEX_CALL(start_mutex_wait)(
-            state, m_ptr, PSI_MUTEX_TRYLOCK, name, (uint)line));
-      }
+      return (PSI_MUTEX_CALL(start_mutex_wait)(state, m_ptr, PSI_MUTEX_TRYLOCK,
+                                               name, (uint)line));
     }
 
     return (nullptr);
@@ -768,9 +893,7 @@ struct PolicyMutex {
   /** Performance schema monitoring - register mutex release */
   void pfs_exit() {
     if (m_ptr != nullptr) {
-      if (m_ptr->m_enabled) {
-        PSI_MUTEX_CALL(unlock_mutex)(m_ptr);
-      }
+      PSI_MUTEX_CALL(unlock_mutex)(m_ptr);
     }
   }
 
