@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 2019, 2023, Oracle and/or its affiliates.
+Copyright (c) 2020, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
@@ -99,31 +99,14 @@ class Histogram_sampler {
   void set_error_state(dberr_t err) { m_err = err; }
 
   /** @return true if in error state. */
-  [[nodiscard]] bool is_error_set() const { return (m_err != DB_SUCCESS); }
-
-  /** Each parallel reader thread's init function.
-  @param[in]  reader_thread_ctx  context information related to the thread
-  @return DB_SUCCESS or error code. */
-  [[nodiscard]] dberr_t start_callback(
-      Parallel_reader::Thread_ctx *reader_thread_ctx);
-
-  /** Each parallel reader thread's end function.
-  @param[in]  reader_thread_ctx  context information related to the thread
-  @return DB_SUCCESS or error code. */
-  [[nodiscard]] dberr_t finish_callback(
-      Parallel_reader::Thread_ctx *reader_thread_ctx);
+  bool is_error_set() const MY_ATTRIBUTE((warn_unused_result)) {
+    return (m_err != DB_SUCCESS);
+  }
 
   /** Convert the row in InnoDB format to MySQL format and store in the buffer
-  for server to use.
-  @param[in]  ctx       Parallel read context.
-  @param[in]  rec       record that needs to be converted
-  @param[in]  offsets   offsets belonging to the record
-  @param[in]  index     index of the record
-  @param[in]  prebuilt  Row meta-data cache.
-  @return DB_SUCCESS or error code. */
-  dberr_t sample_rec(const Parallel_reader::Ctx *ctx, const rec_t *rec,
-                     ulint *offsets, const dict_index_t *index,
-                     row_prebuilt_t *prebuilt);
+  for server to use. */
+  dberr_t sample_rec(ulint thread_id, const rec_t *rec, ulint *offsets,
+                     const dict_index_t *index, row_prebuilt_t *prebuilt);
 
   /** For each record in a non-leaf block at level 1 (if leaf level is 0)
   check if the child page needs to be sampled and if so sample all the rows in
@@ -131,8 +114,9 @@ class Histogram_sampler {
   @param[in]  ctx       Parallel read context.
   @param[in]  prebuilt  Row meta-data cache.
   @return error code */
-  [[nodiscard]] dberr_t process_non_leaf_rec(const Parallel_reader::Ctx *ctx,
-                                             row_prebuilt_t *prebuilt);
+  dberr_t process_non_leaf_rec(const Parallel_reader::Ctx *ctx,
+                               row_prebuilt_t *prebuilt)
+      MY_ATTRIBUTE((warn_unused_result));
 
   /** Process the record in the leaf page. This would happen only when the root
   page is the leaf page and in such a case we process the page regardless of
@@ -140,8 +124,9 @@ class Histogram_sampler {
   @param[in]  ctx       Parallel read context.
   @param[in]  prebuilt  Row meta-data cache.
   @return error code */
-  [[nodiscard]] dberr_t process_leaf_rec(const Parallel_reader::Ctx *ctx,
-                                         row_prebuilt_t *prebuilt);
+  dberr_t process_leaf_rec(const Parallel_reader::Ctx *ctx,
+                           row_prebuilt_t *prebuilt)
+      MY_ATTRIBUTE((warn_unused_result));
 
  private:
   /** Buffer to store the sampled row which is in the MySQL format. */
@@ -159,7 +144,7 @@ class Histogram_sampler {
   /** The parallel reader. */
   Parallel_reader m_parallel_reader;
 
-  /** Random generator engine used to provide us random uniformly distributed
+  /** Random generator engine used to provide us random uniformly distrubuted
   values required to decide if the row in question needs to be sampled or
   not. */
   std::mt19937 m_random_generator;
@@ -176,6 +161,19 @@ class Histogram_sampler {
   /** Sampling seed to be used for sampling */
   int m_sampling_seed{};
 
+  /** BLOB heap per thread.
+
+  There are data members in row_prebuilt_t that cannot be accessed in
+  multi-threaded mode e.g., blob_heap.
+
+  row_prebuilt_t is designed for single threaded access and to share
+  it among threads is not recommended unless "you know what you are doing".
+  This is very fragile code as it stands.
+
+  To solve the blob heap issue in prebuilt we use per thread m_blob_heaps.
+  Pass the blob heap to the InnoDB to MySQL row format conversion function. */
+  std::vector<mem_heap_t *, ut_allocator<mem_heap_t *>> m_blob_heaps{};
+
   /** Number of rows sampled */
   std::atomic_size_t m_n_sampled;
 };
