@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 2017, 2022, Oracle and/or its affiliates.
+Copyright (c) 2017, 2019, Oracle and/or its affiliates. All Rights Reserved.
 
 Portions of this file contain modifications contributed and copyrighted by
 Google, Inc. Those modifications are gratefully acknowledged and are described
@@ -47,7 +47,6 @@ this program; if not, write to the Free Software Foundation, Inc.,
 #include "dict0mem.h"
 #include "dict0stats.h"
 #include "ha_innodb.h"
-#include "log0chkp.h"
 #include "log0ddl.h"
 #include "mysql/plugin.h"
 #include "pars0pars.h"
@@ -122,7 +121,8 @@ static uint32_t crash_before_alter_encrypt_space_log_counter = 1;
 /** Crash injection counter used after writing ALTER ENCRYPT TABLESPACE log */
 static uint32_t crash_after_alter_encrypt_space_log_counter = 1;
 
-void ddl_log_crash_reset(THD *, SYS_VAR *, void *, const void *save) {
+void ddl_log_crash_reset(THD *thd, SYS_VAR *var, void *var_ptr,
+                         const void *save) {
   const bool reset = *static_cast<const bool *>(save);
 
   innodb_ddl_log_crash_reset_debug = reset;
@@ -167,7 +167,7 @@ void DDL_Record::set_old_file_path(const char *name) {
   ulint len = strlen(name);
 
   if (m_heap == nullptr) {
-    m_heap = mem_heap_create(FN_REFLEN + 1, UT_LOCATION_HERE);
+    m_heap = mem_heap_create(FN_REFLEN + 1);
   }
 
   m_old_file_path = mem_heap_strdupl(m_heap, name, len);
@@ -175,7 +175,7 @@ void DDL_Record::set_old_file_path(const char *name) {
 
 void DDL_Record::set_old_file_path(const byte *data, ulint len) {
   if (m_heap == nullptr) {
-    m_heap = mem_heap_create(FN_REFLEN + 1, UT_LOCATION_HERE);
+    m_heap = mem_heap_create(FN_REFLEN + 1);
   }
 
   m_old_file_path = static_cast<char *>(mem_heap_dup(m_heap, data, len + 1));
@@ -186,7 +186,7 @@ void DDL_Record::set_new_file_path(const char *name) {
   ulint len = strlen(name);
 
   if (m_heap == nullptr) {
-    m_heap = mem_heap_create(FN_REFLEN + 1, UT_LOCATION_HERE);
+    m_heap = mem_heap_create(FN_REFLEN + 1);
   }
 
   m_new_file_path = mem_heap_strdupl(m_heap, name, len);
@@ -194,7 +194,7 @@ void DDL_Record::set_new_file_path(const char *name) {
 
 void DDL_Record::set_new_file_path(const byte *data, ulint len) {
   if (m_heap == nullptr) {
-    m_heap = mem_heap_create(FN_REFLEN + 1, UT_LOCATION_HERE);
+    m_heap = mem_heap_create(FN_REFLEN + 1);
   }
 
   m_new_file_path = static_cast<char *>(mem_heap_dup(m_heap, data, len + 1));
@@ -231,11 +231,8 @@ std::ostream &DDL_Record::print(std::ostream &out) const {
     case Log_Type::ALTER_ENCRYPT_TABLESPACE_LOG:
       out << "ALTER ENCRYPT TABLESPACE";
       break;
-    case Log_Type::ALTER_UNENCRYPT_TABLESPACE_LOG:
-      out << "ALTER UNENCRYPT TABLESPACE";
-      break;
     default:
-      ut_d(ut_error);
+      ut_ad(0);
   }
 
   out << ",";
@@ -306,8 +303,8 @@ std::ostream &DDL_Record::print(std::ostream &out) const {
 }
 
 /** Display a DDL record
-@param[in,out]  o       output stream
-@param[in]      record  DDL record to display
+@param[in,out]	o	output stream
+@param[in]	record	DDL record to display
 @return the output stream */
 std::ostream &operator<<(std::ostream &o, const DDL_Record &record) {
   return (record.print(o));
@@ -318,7 +315,7 @@ DDL_Log_Table::DDL_Log_Table() : DDL_Log_Table(nullptr) {}
 DDL_Log_Table::DDL_Log_Table(trx_t *trx)
     : m_table(dict_sys->ddl_log), m_tuple(nullptr), m_trx(trx), m_thr(nullptr) {
   ut_ad(m_trx == nullptr || m_trx->ddl_operation);
-  m_heap = mem_heap_create(100, UT_LOCATION_HERE);
+  m_heap = mem_heap_create(1000);
   if (m_trx != nullptr) {
     start_query_thread();
   }
@@ -463,7 +460,7 @@ dberr_t DDL_Log_Table::insert(const DDL_Record &record) {
   dict_index_t *index = m_table->first_index();
   dtuple_t *entry;
   uint32_t flags = BTR_NO_LOCKING_FLAG;
-  mem_heap_t *offsets_heap = mem_heap_create(100, UT_LOCATION_HERE);
+  mem_heap_t *offsets_heap = mem_heap_create(1000);
   static std::atomic<uint64_t> count(0);
 
   if (count++ % 64 == 0) {
@@ -473,19 +470,8 @@ dberr_t DDL_Log_Table::insert(const DDL_Record &record) {
   create_tuple(record);
   entry = row_build_index_entry(m_tuple, nullptr, index, m_heap);
 
-#ifdef UNIV_DEBUG
-  bool insert = true;
-  DBUG_EXECUTE_IF("ddl_log_return_error_from_insert", insert = false;);
-
-  if (insert) {
-#endif
-    error = row_ins_clust_index_entry_low(flags, BTR_MODIFY_LEAF, index,
-                                          index->n_uniq, entry, m_thr, false);
-#ifdef UNIV_DEBUG
-  } else {
-    error = DB_ERROR;
-  }
-#endif
+  error = row_ins_clust_index_entry_low(flags, BTR_MODIFY_LEAF, index,
+                                        index->n_uniq, entry, m_thr, false);
 
   if (error == DB_FAIL) {
     error = row_ins_clust_index_entry_low(flags, BTR_MODIFY_TREE, index,
@@ -493,12 +479,6 @@ dberr_t DDL_Log_Table::insert(const DDL_Record &record) {
     ut_ad(error == DB_SUCCESS);
   }
 
-  if (error != DB_SUCCESS) {
-    ib::error(ER_IB_ERR_DDL_LOG_INSERT_FAILURE);
-    mem_heap_free(offsets_heap);
-    return error;
-  }
-
   index = index->next();
 
   entry = row_build_index_entry(m_tuple, nullptr, index, m_heap);
@@ -530,8 +510,7 @@ void DDL_Log_Table::convert_to_ddl_record(bool is_clustered, rec_t *rec,
         continue;
       }
 
-      const dict_index_t *clust_index = m_table->first_index();
-      data = rec_get_nth_field(clust_index, rec, offsets, i, &len);
+      data = rec_get_nth_field(rec, offsets, i, &len);
 
       if (len != UNIV_SQL_NULL) {
         set_field(data, i, len, record);
@@ -548,7 +527,7 @@ ulint DDL_Log_Table::parse_id(const dict_index_t *index, rec_t *rec,
   ulint len;
   ulint index_offset = index->get_col_pos(s_id_col_no);
 
-  const byte *data = rec_get_nth_field(index, rec, offsets, index_offset, &len);
+  const byte *data = rec_get_nth_field(rec, offsets, index_offset, &len);
   ut_ad(len == s_id_col_len);
 
   return (mach_read_from_8(data));
@@ -595,7 +574,7 @@ void DDL_Log_Table::set_field(const byte *data, ulint index_offset, ulint len,
     case s_old_file_path_col_no:
     case s_new_file_path_col_no:
     default:
-      ut_d(ut_error);
+      ut_ad(0);
   }
 }
 
@@ -616,11 +595,11 @@ ulint DDL_Log_Table::fetch_value(const byte *data, ulint offset) {
     case s_new_file_path_col_no:
     case s_old_file_path_col_no:
     default:
-      ut_d(ut_error);
-      ut_o(break);
+      ut_ad(0);
+      break;
   }
 
-  ut_o(return (value));
+  return (value);
 }
 
 dberr_t DDL_Log_Table::search_all(DDL_Records &records) {
@@ -635,28 +614,28 @@ dberr_t DDL_Log_Table::search_all(DDL_Records &records) {
   mtr_start(&mtr);
 
   /** Scan the index in decreasing order. */
-  pcur.open_at_side(false, index, BTR_SEARCH_LEAF, true, 0, &mtr);
+  btr_pcur_open_at_index_side(false, index, BTR_SEARCH_LEAF, &pcur, true, 0,
+                              &mtr);
 
-  for (; move == true; move = pcur.move_to_prev(&mtr)) {
-    rec = pcur.get_rec();
+  for (; move == true; move = btr_pcur_move_to_prev(&pcur, &mtr)) {
+    rec = btr_pcur_get_rec(&pcur);
 
     if (page_rec_is_infimum(rec) || page_rec_is_supremum(rec)) {
       continue;
     }
 
-    offsets = rec_get_offsets(rec, index, nullptr, ULINT_UNDEFINED,
-                              UT_LOCATION_HERE, &m_heap);
+    offsets = rec_get_offsets(rec, index, nullptr, ULINT_UNDEFINED, &m_heap);
 
     if (rec_get_deleted_flag(rec, dict_table_is_comp(m_table))) {
       continue;
     }
 
-    DDL_Record *record = ut::new_withkey<DDL_Record>(UT_NEW_THIS_FILE_PSI_KEY);
+    DDL_Record *record = UT_NEW_NOKEY(DDL_Record());
     convert_to_ddl_record(index->is_clustered(), rec, offsets, *record);
     records.push_back(record);
   }
 
-  pcur.close();
+  btr_pcur_close(&pcur);
   mtr_commit(&mtr);
 
   return (error);
@@ -677,7 +656,7 @@ dberr_t DDL_Log_Table::search(ulint thread_id, DDL_Records &records) {
   }
 
   for (auto record : records_of_thread_id) {
-    ut::delete_(record);
+    UT_DELETE(record);
   }
 
   return (error);
@@ -695,18 +674,17 @@ dberr_t DDL_Log_Table::search_by_id(ulint id, dict_index_t *index,
   mtr_start(&mtr);
 
   create_tuple(id, index);
-  pcur.open_no_init(index, m_tuple, PAGE_CUR_GE, BTR_SEARCH_LEAF, 0, &mtr,
-                    UT_LOCATION_HERE);
+  btr_pcur_open_with_no_init(index, m_tuple, PAGE_CUR_GE, BTR_SEARCH_LEAF,
+                             &pcur, 0, &mtr);
 
-  for (; move == true; move = pcur.move_to_next(&mtr)) {
-    rec = pcur.get_rec();
+  for (; move == true; move = btr_pcur_move_to_next(&pcur, &mtr)) {
+    rec = btr_pcur_get_rec(&pcur);
 
     if (page_rec_is_infimum(rec) || page_rec_is_supremum(rec)) {
       continue;
     }
 
-    offsets = rec_get_offsets(rec, index, nullptr, ULINT_UNDEFINED,
-                              UT_LOCATION_HERE, &m_heap);
+    offsets = rec_get_offsets(rec, index, nullptr, ULINT_UNDEFINED, &m_heap);
 
     if (cmp_dtuple_rec(m_tuple, rec, index, offsets) != 0) {
       break;
@@ -716,7 +694,7 @@ dberr_t DDL_Log_Table::search_by_id(ulint id, dict_index_t *index,
       continue;
     }
 
-    DDL_Record *record = ut::new_withkey<DDL_Record>(UT_NEW_THIS_FILE_PSI_KEY);
+    DDL_Record *record = UT_NEW_NOKEY(DDL_Record());
     convert_to_ddl_record(index->is_clustered(), rec, offsets, *record);
     records.push_back(record);
   }
@@ -749,23 +727,23 @@ dberr_t DDL_Log_Table::remove(ulint id) {
 
   mtr_start(&mtr);
 
-  pcur.open(clust_index, 0, m_tuple, PAGE_CUR_LE,
-            BTR_MODIFY_TREE | BTR_LATCH_FOR_DELETE, &mtr, UT_LOCATION_HERE);
+  btr_pcur_open(clust_index, m_tuple, PAGE_CUR_LE,
+                BTR_MODIFY_TREE | BTR_LATCH_FOR_DELETE, &pcur, &mtr);
 
-  btr_cur = pcur.get_btr_cur();
+  btr_cur = btr_pcur_get_btr_cur(&pcur);
 
-  if (page_rec_is_infimum(pcur.get_rec()) ||
-      pcur.get_low_match() < clust_index->n_uniq) {
-    pcur.close();
+  if (page_rec_is_infimum(btr_pcur_get_rec(&pcur)) ||
+      btr_pcur_get_low_match(&pcur) < clust_index->n_uniq) {
+    btr_pcur_close(&pcur);
     mtr_commit(&mtr);
     return (DB_SUCCESS);
   }
 
-  offsets = rec_get_offsets(pcur.get_rec(), clust_index, nullptr,
-                            ULINT_UNDEFINED, UT_LOCATION_HERE, &m_heap);
+  offsets = rec_get_offsets(btr_pcur_get_rec(&pcur), clust_index, nullptr,
+                            ULINT_UNDEFINED, &m_heap);
 
-  row = row_build(ROW_COPY_DATA, clust_index, pcur.get_rec(), offsets, nullptr,
-                  nullptr, nullptr, nullptr, m_heap);
+  row = row_build(ROW_COPY_DATA, clust_index, btr_pcur_get_rec(&pcur), offsets,
+                  nullptr, nullptr, nullptr, nullptr, m_heap);
 
   rec = btr_cur_get_rec(btr_cur);
 
@@ -775,7 +753,7 @@ dberr_t DDL_Log_Table::remove(ulint id) {
                                          &mtr);
   }
 
-  pcur.close();
+  btr_pcur_close(&pcur);
   mtr_commit(&mtr);
 
   if (err != DB_SUCCESS) {
@@ -788,22 +766,22 @@ dberr_t DDL_Log_Table::remove(ulint id) {
   entry = row_build_index_entry(row, nullptr, index, m_heap);
   search_result = row_search_index_entry(
       index, entry, BTR_MODIFY_LEAF | BTR_DELETE_MARK, &pcur, &mtr);
-  btr_cur = pcur.get_btr_cur();
+  btr_cur = btr_pcur_get_btr_cur(&pcur);
 
   if (search_result == ROW_NOT_FOUND) {
-    pcur.close();
+    btr_pcur_close(&pcur);
     mtr_commit(&mtr);
-    ut_d(ut_error);
-    ut_o(return (DB_CORRUPTION));
+    ut_ad(0);
+    return (DB_CORRUPTION);
   }
 
   rec = btr_cur_get_rec(btr_cur);
 
   if (!rec_get_deleted_flag(rec, dict_table_is_comp(m_table))) {
-    err = btr_cur_del_mark_set_sec_rec(flags, btr_cur, true, m_thr, &mtr);
+    err = btr_cur_del_mark_set_sec_rec(flags, btr_cur, TRUE, m_thr, &mtr);
   }
 
-  pcur.close();
+  btr_pcur_close(&pcur);
   mtr_commit(&mtr);
 
   return (err);
@@ -882,9 +860,7 @@ dberr_t Log_DDL::write_free_tree_log(trx_t *trx, const dict_index_t *index,
   if (is_drop_table) {
     /* Drop index case, if committed, will be redo only */
     err = insert_free_tree_log(trx, index, id, thread_id);
-    if (err != DB_SUCCESS) {
-      return err;
-    }
+    ut_ad(err == DB_SUCCESS);
 
     DBUG_INJECT_CRASH("ddl_log_crash_after_free_tree_log",
                       crash_after_free_tree_log_counter++);
@@ -892,9 +868,7 @@ dberr_t Log_DDL::write_free_tree_log(trx_t *trx, const dict_index_t *index,
     /* This is the case of building index during create table
     scenario. The index will be dropped if ddl is rolled back */
     err = insert_free_tree_log(nullptr, index, id, thread_id);
-    if (err != DB_SUCCESS) {
-      return err;
-    }
+    ut_ad(err == DB_SUCCESS);
 
     DBUG_INJECT_CRASH("ddl_log_crash_after_free_tree_log",
                       crash_after_free_tree_log_counter++);
@@ -924,10 +898,10 @@ dberr_t Log_DDL::insert_free_tree_log(trx_t *trx, const dict_index_t *index,
   bool has_dd_trx = (trx != nullptr);
   if (!has_dd_trx) {
     trx = trx_allocate_for_background();
-    trx_start_internal(trx, UT_LOCATION_HERE);
+    trx_start_internal(trx);
     trx->ddl_operation = true;
   } else {
-    trx_start_if_not_started(trx, true, UT_LOCATION_HERE);
+    trx_start_if_not_started(trx, true);
   }
 
   ut_ad(trx->ddl_operation);
@@ -943,6 +917,7 @@ dberr_t Log_DDL::insert_free_tree_log(trx_t *trx, const dict_index_t *index,
   {
     DDL_Log_Table ddl_log(trx);
     error = ddl_log.insert(record);
+    ut_ad(error == DB_SUCCESS);
   }
 
   if (!has_dd_trx) {
@@ -950,7 +925,7 @@ dberr_t Log_DDL::insert_free_tree_log(trx_t *trx, const dict_index_t *index,
     trx_free_for_background(trx);
   }
 
-  if (error == DB_SUCCESS && srv_print_ddl_logs) {
+  if (srv_print_ddl_logs) {
     ib::info(ER_IB_MSG_647) << "DDL log insert : " << record;
   }
 
@@ -980,18 +955,14 @@ dberr_t Log_DDL::write_delete_space_log(trx_t *trx, const dict_table_t *table,
   if (is_drop) {
     err = insert_delete_space_log(trx, id, thread_id, space_id, file_path,
                                   dict_locked);
-    if (err != DB_SUCCESS) {
-      return err;
-    }
+    ut_ad(err == DB_SUCCESS);
 
     DBUG_INJECT_CRASH("ddl_log_crash_after_delete_space_log",
                       crash_after_delete_space_log_counter++);
   } else {
     err = insert_delete_space_log(nullptr, id, thread_id, space_id, file_path,
                                   dict_locked);
-    if (err != DB_SUCCESS) {
-      return err;
-    }
+    ut_ad(err == DB_SUCCESS);
 
     DBUG_INJECT_CRASH("ddl_log_crash_after_delete_space_log",
                       crash_after_delete_space_log_counter++);
@@ -1021,16 +992,16 @@ dberr_t Log_DDL::insert_delete_space_log(trx_t *trx, uint64_t id,
 
   if (!has_dd_trx) {
     trx = trx_allocate_for_background();
-    trx_start_internal(trx, UT_LOCATION_HERE);
+    trx_start_internal(trx);
     trx->ddl_operation = true;
   } else {
-    trx_start_if_not_started(trx, true, UT_LOCATION_HERE);
+    trx_start_if_not_started(trx, true);
   }
 
   ut_ad(trx->ddl_operation);
 
   if (dict_locked) {
-    dict_sys_mutex_exit();
+    mutex_exit(&dict_sys->mutex);
   }
 
   DDL_Record record;
@@ -1043,10 +1014,11 @@ dberr_t Log_DDL::insert_delete_space_log(trx_t *trx, uint64_t id,
   {
     DDL_Log_Table ddl_log(trx);
     error = ddl_log.insert(record);
+    ut_ad(error == DB_SUCCESS);
   }
 
   if (dict_locked) {
-    dict_sys_mutex_enter();
+    mutex_enter(&dict_sys->mutex);
   }
 
   if (!has_dd_trx) {
@@ -1054,7 +1026,7 @@ dberr_t Log_DDL::insert_delete_space_log(trx_t *trx, uint64_t id,
     trx_free_for_background(trx);
   }
 
-  if (error == DB_SUCCESS && srv_print_ddl_logs) {
+  if (srv_print_ddl_logs) {
     ib::info(ER_IB_MSG_648) << "DDL log insert : " << record;
   }
 
@@ -1090,16 +1062,11 @@ dberr_t Log_DDL::write_rename_space_log(space_id_t space_id,
 
   dberr_t err = insert_rename_space_log(id, thread_id, space_id, old_file_path,
                                         new_file_path);
-  if (err != DB_SUCCESS) {
-    return err;
-  }
+  ut_ad(err == DB_SUCCESS);
 
   DBUG_INJECT_CRASH("ddl_log_crash_after_rename_space_log",
                     crash_after_rename_space_log_counter++);
 
-  DBUG_EXECUTE_IF("ddl_log_crash_after_rename_space_log_insert",
-                  DBUG_SUICIDE(););
-
   DBUG_EXECUTE_IF("DDL_Log_remove_inject_error_4",
                   srv_inject_too_many_concurrent_trxs = true;);
 
@@ -1121,11 +1088,11 @@ dberr_t Log_DDL::insert_rename_space_log(uint64_t id, ulint thread_id,
                                          const char *new_file_path) {
   dberr_t error;
   trx_t *trx = trx_allocate_for_background();
-  trx_start_internal(trx, UT_LOCATION_HERE);
+  trx_start_internal(trx);
   trx->ddl_operation = true;
 
-  ut_ad(dict_sys_mutex_own());
-  dict_sys_mutex_exit();
+  ut_ad(mutex_own(&dict_sys->mutex));
+  mutex_exit(&dict_sys->mutex);
 
   DDL_Record record;
   record.set_id(id);
@@ -1138,34 +1105,22 @@ dberr_t Log_DDL::insert_rename_space_log(uint64_t id, ulint thread_id,
   {
     DDL_Log_Table ddl_log(trx);
     error = ddl_log.insert(record);
+    ut_ad(error == DB_SUCCESS);
   }
 
-  dict_sys_mutex_enter();
+  mutex_enter(&dict_sys->mutex);
 
   trx_commit_for_mysql(trx);
   trx_free_for_background(trx);
 
-  if (error == DB_SUCCESS && srv_print_ddl_logs) {
+  if (srv_print_ddl_logs) {
     ib::info(ER_IB_MSG_649) << "DDL log insert : " << record;
   }
 
   return (error);
 }
 
-DDL_Record *Log_DDL::find_alter_encrypt_record(space_id_t space_id) {
-  if (!ts_encrypt_ddl_records.empty()) {
-    for (const auto it : ts_encrypt_ddl_records) {
-      if (it->get_space_id() == space_id) {
-        return it;
-      }
-    }
-  }
-  return nullptr;
-}
-
-dberr_t Log_DDL::write_alter_encrypt_space_log(space_id_t space_id,
-                                               Encryption::Progress type,
-                                               DDL_Record *existing_rec) {
+dberr_t Log_DDL::write_alter_encrypt_space_log(space_id_t space_id) {
   /* Missing current_thd, it happens during crash recovery */
   if (!current_thd) {
     return (DB_SUCCESS);
@@ -1185,73 +1140,47 @@ dberr_t Log_DDL::write_alter_encrypt_space_log(space_id_t space_id,
   DBUG_INJECT_CRASH("ddl_log_crash_before_alter_encrypt_space_log",
                     crash_before_alter_encrypt_space_log_counter++);
 
-  dberr_t err = insert_alter_encrypt_space_log(id, thread_id, space_id, type,
-                                               existing_rec);
-  if (err != DB_SUCCESS) {
-    return err;
-  }
+  dberr_t err = insert_alter_encrypt_space_log(id, thread_id, space_id);
+  ut_ad(err == DB_SUCCESS);
 
   DBUG_INJECT_CRASH("ddl_log_crash_after_alter_encrypt_space_log",
                     crash_after_alter_encrypt_space_log_counter++);
 
-  DBUG_EXECUTE_IF("DDL_Log_remove_inject_startup_error_1",
-                  srv_inject_too_many_concurrent_trxs = true;);
-
-  /* This record to be removed with main transaction commit */
-  err = delete_by_id(trx, id, false);
-  ut_ad(err == DB_SUCCESS || err == DB_TOO_MANY_CONCURRENT_TRXS);
-
-  DBUG_EXECUTE_IF("DDL_Log_remove_inject_startup_error_1",
-                  srv_inject_too_many_concurrent_trxs = false;);
-
   return (err);
 }
 
 dberr_t Log_DDL::insert_alter_encrypt_space_log(uint64_t id, ulint thread_id,
-                                                space_id_t space_id,
-                                                Encryption::Progress type,
-                                                DDL_Record *existing_rec) {
-  dberr_t err = DB_SUCCESS;
+                                                space_id_t space_id) {
+  dberr_t error;
   trx_t *trx = trx_allocate_for_background();
-  trx_start_internal(trx, UT_LOCATION_HERE);
+  trx_start_internal(trx);
   trx->ddl_operation = true;
 
+  ut_ad(mutex_own(&dict_sys->mutex));
+  mutex_exit(&dict_sys->mutex);
+
   DDL_Record record;
   record.set_id(id);
   record.set_thread_id(thread_id);
-
-  if (type == Encryption::Progress::ENCRYPTION) {
-    record.set_type(Log_Type::ALTER_ENCRYPT_TABLESPACE_LOG);
-  } else {
-    ut_ad(type == Encryption::Progress::DECRYPTION);
-    record.set_type(Log_Type::ALTER_UNENCRYPT_TABLESPACE_LOG);
-  }
+  record.set_type(Log_Type::ALTER_ENCRYPT_TABLESPACE_LOG);
   record.set_space_id(space_id);
 
   {
     DDL_Log_Table ddl_log(trx);
-    if (existing_rec != nullptr) {
-      err = ddl_log.remove(existing_rec->get_id());
-      ut_ad(err == DB_SUCCESS || err == DB_TOO_MANY_CONCURRENT_TRXS);
-
-      if (err == DB_TOO_MANY_CONCURRENT_TRXS) {
-        ib::error(ER_IB_MSG_DDL_LOG_DELETE_BY_ID_TMCT);
-      }
-    }
-
-    if (err == DB_SUCCESS) {
-      err = ddl_log.insert(record);
-    }
+    error = ddl_log.insert(record);
+    ut_ad(error == DB_SUCCESS);
   }
 
+  mutex_enter(&dict_sys->mutex);
+
   trx_commit_for_mysql(trx);
   trx_free_for_background(trx);
 
-  if (err == DB_SUCCESS && srv_print_ddl_logs) {
+  if (srv_print_ddl_logs) {
     ib::info(ER_IB_MSG_1284) << "DDL log insert : " << record;
   }
 
-  return (err);
+  return (error);
 }
 
 dberr_t Log_DDL::write_drop_log(trx_t *trx, const table_id_t table_id) {
@@ -1269,9 +1198,7 @@ dberr_t Log_DDL::write_drop_log(trx_t *trx, const table_id_t table_id) {
 
   dberr_t err;
   err = insert_drop_log(trx, id, thread_id, table_id);
-  if (err != DB_SUCCESS) {
-    return err;
-  }
+  ut_ad(err == DB_SUCCESS);
 
   DBUG_INJECT_CRASH("ddl_log_crash_after_drop_log",
                     crash_after_drop_log_counter++);
@@ -1282,11 +1209,11 @@ dberr_t Log_DDL::write_drop_log(trx_t *trx, const table_id_t table_id) {
 dberr_t Log_DDL::insert_drop_log(trx_t *trx, uint64_t id, ulint thread_id,
                                  const table_id_t table_id) {
   ut_ad(trx->ddl_operation);
-  ut_ad(dict_sys_mutex_own());
+  ut_ad(mutex_own(&dict_sys->mutex));
 
-  trx_start_if_not_started(trx, true, UT_LOCATION_HERE);
+  trx_start_if_not_started(trx, true);
 
-  dict_sys_mutex_exit();
+  mutex_exit(&dict_sys->mutex);
 
   dberr_t error;
   DDL_Record record;
@@ -1298,11 +1225,12 @@ dberr_t Log_DDL::insert_drop_log(trx_t *trx, uint64_t id, ulint thread_id,
   {
     DDL_Log_Table ddl_log(trx);
     error = ddl_log.insert(record);
+    ut_ad(error == DB_SUCCESS);
   }
 
-  dict_sys_mutex_enter();
+  mutex_enter(&dict_sys->mutex);
 
-  if (error == DB_SUCCESS && srv_print_ddl_logs) {
+  if (srv_print_ddl_logs) {
     ib::info(ER_IB_MSG_650) << "DDL log insert : " << record;
   }
 
@@ -1325,9 +1253,7 @@ dberr_t Log_DDL::write_rename_table_log(dict_table_t *table,
 
   dberr_t err =
       insert_rename_table_log(id, thread_id, table->id, old_name, new_name);
-  if (err != DB_SUCCESS) {
-    return err;
-  }
+  ut_ad(err == DB_SUCCESS);
 
   DBUG_EXECUTE_IF("DDL_Log_remove_inject_error_5",
                   srv_inject_too_many_concurrent_trxs = true;);
@@ -1347,11 +1273,11 @@ dberr_t Log_DDL::insert_rename_table_log(uint64_t id, ulint thread_id,
                                          const char *new_name) {
   dberr_t error;
   trx_t *trx = trx_allocate_for_background();
-  trx_start_internal(trx, UT_LOCATION_HERE);
+  trx_start_internal(trx);
   trx->ddl_operation = true;
 
-  ut_ad(dict_sys_mutex_own());
-  dict_sys_mutex_exit();
+  ut_ad(mutex_own(&dict_sys->mutex));
+  mutex_exit(&dict_sys->mutex);
 
   DDL_Record record;
   record.set_id(id);
@@ -1364,14 +1290,15 @@ dberr_t Log_DDL::insert_rename_table_log(uint64_t id, ulint thread_id,
   {
     DDL_Log_Table ddl_log(trx);
     error = ddl_log.insert(record);
+    ut_ad(error == DB_SUCCESS);
   }
 
-  dict_sys_mutex_enter();
+  mutex_enter(&dict_sys->mutex);
 
   trx_commit_for_mysql(trx);
   trx_free_for_background(trx);
 
-  if (error == DB_SUCCESS && srv_print_ddl_logs) {
+  if (srv_print_ddl_logs) {
     ib::info(ER_IB_MSG_651) << "DDL log insert : " << record;
   }
 
@@ -1392,9 +1319,7 @@ dberr_t Log_DDL::write_remove_cache_log(trx_t *trx, dict_table_t *table) {
 
   dberr_t err =
       insert_remove_cache_log(id, thread_id, table->id, table->name.m_name);
-  if (err != DB_SUCCESS) {
-    return err;
-  }
+  ut_ad(err == DB_SUCCESS);
 
   DBUG_EXECUTE_IF("DDL_Log_remove_inject_error_3",
                   srv_inject_too_many_concurrent_trxs = true;);
@@ -1413,7 +1338,7 @@ dberr_t Log_DDL::insert_remove_cache_log(uint64_t id, ulint thread_id,
                                          const char *table_name) {
   dberr_t error;
   trx_t *trx = trx_allocate_for_background();
-  trx_start_internal(trx, UT_LOCATION_HERE);
+  trx_start_internal(trx);
   trx->ddl_operation = true;
 
   DDL_Record record;
@@ -1426,12 +1351,13 @@ dberr_t Log_DDL::insert_remove_cache_log(uint64_t id, ulint thread_id,
   {
     DDL_Log_Table ddl_log(trx);
     error = ddl_log.insert(record);
+    ut_ad(error == DB_SUCCESS);
   }
 
   trx_commit_for_mysql(trx);
   trx_free_for_background(trx);
 
-  if (error == DB_SUCCESS && srv_print_ddl_logs) {
+  if (srv_print_ddl_logs) {
     ib::info(ER_IB_MSG_652) << "DDL log insert : " << record;
   }
 
@@ -1441,12 +1367,12 @@ dberr_t Log_DDL::insert_remove_cache_log(uint64_t id, ulint thread_id,
 dberr_t Log_DDL::delete_by_id(trx_t *trx, uint64_t id, bool dict_locked) {
   dberr_t err;
 
-  trx_start_if_not_started(trx, true, UT_LOCATION_HERE);
+  trx_start_if_not_started(trx, true);
 
   ut_ad(trx->ddl_operation);
 
   if (dict_locked) {
-    dict_sys_mutex_exit();
+    mutex_exit(&dict_sys->mutex);
   }
 
   {
@@ -1460,7 +1386,7 @@ dberr_t Log_DDL::delete_by_id(trx_t *trx, uint64_t id, bool dict_locked) {
   }
 
   if (dict_locked) {
-    dict_sys_mutex_enter();
+    mutex_enter(&dict_sys->mutex);
   }
 
   if (srv_print_ddl_logs && err == DB_SUCCESS) {
@@ -1480,22 +1406,22 @@ dberr_t Log_DDL::replay_all() {
   ut_ad(err == DB_SUCCESS);
 
   for (auto record : records) {
-    err = log_ddl->replay(*record);
-    if (err != DB_SUCCESS) {
-      break;
+    log_ddl->replay(*record);
+    /* If this is alter tablespace encrypt entry, don't delete it yet.
+    This is to handle crash during resume operation. This entry will be deleted
+    once resume operation is finished. */
+    if (record->get_type() == Log_Type::ALTER_ENCRYPT_TABLESPACE_LOG) {
+      ts_encrypt_ddl_records.push_back(record);
+      record->set_deletable(false);
     }
   }
 
-  if (err != DB_SUCCESS) {
-    return err;
-  }
-
   err = delete_by_ids(records);
   ut_ad(err == DB_SUCCESS || err == DB_TOO_MANY_CONCURRENT_TRXS);
 
   for (auto record : records) {
     if (record->get_deletable()) {
-      ut::delete_(record);
+      UT_DELETE(record);
     }
   }
 
@@ -1510,15 +1436,7 @@ dberr_t Log_DDL::replay_by_thread_id(ulint thread_id) {
   ut_ad(err == DB_SUCCESS);
 
   for (auto record : records) {
-    if (record->get_type() == Log_Type::ALTER_ENCRYPT_TABLESPACE_LOG ||
-        record->get_type() == Log_Type::ALTER_UNENCRYPT_TABLESPACE_LOG) {
-      DDL_Record *rec = find_alter_encrypt_record(record->get_space_id());
-      if (rec != nullptr) {
-        ut_ad(record->get_id() != rec->get_id());
-      }
-    } else {
-      log_ddl->replay(*record);
-    }
+    log_ddl->replay(*record);
   }
 
   err = delete_by_ids(records);
@@ -1526,14 +1444,14 @@ dberr_t Log_DDL::replay_by_thread_id(ulint thread_id) {
 
   for (auto record : records) {
     if (record->get_deletable()) {
-      ut::delete_(record);
+      UT_DELETE(record);
     }
   }
 
   return (err);
 }
 
-constexpr uint32_t DELETE_IDS_RETRIES_MAX = 10;
+#define DELETE_IDS_RETRIES_MAX 10
 
 dberr_t Log_DDL::delete_by_ids(DDL_Records &records) {
   dberr_t err = DB_SUCCESS;
@@ -1546,7 +1464,7 @@ dberr_t Log_DDL::delete_by_ids(DDL_Records &records) {
   for (t = DELETE_IDS_RETRIES_MAX; t > 0; t--) {
     trx_t *trx;
     trx = trx_allocate_for_background();
-    trx_start_if_not_started(trx, true, UT_LOCATION_HERE);
+    trx_start_if_not_started(trx, true);
     trx->ddl_operation = true;
 
     {
@@ -1600,7 +1518,7 @@ dberr_t Log_DDL::replay(DDL_Record &record) {
       break;
 
     case Log_Type::RENAME_TABLE_LOG:
-      replay_rename_table_log(record.get_old_file_path(),
+      replay_rename_table_log(record.get_table_id(), record.get_old_file_path(),
                               record.get_new_file_path());
       break;
 
@@ -1610,8 +1528,7 @@ dberr_t Log_DDL::replay(DDL_Record &record) {
       break;
 
     case Log_Type::ALTER_ENCRYPT_TABLESPACE_LOG:
-    case Log_Type::ALTER_UNENCRYPT_TABLESPACE_LOG:
-      err = replay_alter_encrypt_space_log(record);
+      replay_alter_encrypt_space_log(record.get_space_id());
       break;
 
     default:
@@ -1641,7 +1558,7 @@ void Log_DDL::replay_free_tree_log(space_id_t space_id, page_no_t page_no,
   }
 
   /* This is required by dropping hash index afterwards. */
-  dict_sys_mutex_enter();
+  mutex_enter(&dict_sys->mutex);
 
   mtr_t mtr;
   mtr_start(&mtr);
@@ -1650,7 +1567,7 @@ void Log_DDL::replay_free_tree_log(space_id_t space_id, page_no_t page_no,
 
   mtr_commit(&mtr);
 
-  dict_sys_mutex_exit();
+  mutex_exit(&dict_sys->mutex);
 
   DBUG_INJECT_CRASH("ddl_log_crash_after_replay", crash_after_replay_counter++);
 }
@@ -1660,9 +1577,6 @@ void Log_DDL::replay_delete_space_log(space_id_t space_id,
   THD *thd = current_thd;
 
   if (fsp_is_undo_tablespace(space_id)) {
-    /* Serialize this delete with all undo tablespace DDLs. */
-    mutex_enter(&undo::ddl_mutex);
-
     /* If this is called during DROP UNDO TABLESPACE, then the undo_space
     is already gone. But if this is called at startup after a crash, that
     memory object might exist. If the crash occurred just before the file
@@ -1686,9 +1600,9 @@ void Log_DDL::replay_delete_space_log(space_id_t space_id,
     /* For general tablespace, MDL on SDI tables is already
     acquired at innobase_drop_tablespace() and for file_per_table
     tablespace, MDL is acquired at row_drop_table_for_mysql() */
-    dict_sys_mutex_enter();
+    mutex_enter(&dict_sys->mutex);
     dict_sdi_remove_from_cache(space_id, nullptr, true);
-    dict_sys_mutex_exit();
+    mutex_exit(&dict_sys->mutex);
   }
 
   /* A master key rotation blocks all DDLs using backup_lock, so it is assured
@@ -1711,8 +1625,6 @@ void Log_DDL::replay_delete_space_log(space_id_t space_id,
     undo::spaces->x_lock();
     undo::unuse_space_id(space_id);
     undo::spaces->x_unlock();
-
-    mutex_exit(&undo::ddl_mutex);
   }
 
   DBUG_INJECT_CRASH("ddl_log_crash_after_replay", crash_after_replay_counter++);
@@ -1746,58 +1658,9 @@ void Log_DDL::replay_rename_space_log(space_id_t space_id,
   DBUG_INJECT_CRASH("ddl_log_crash_after_replay", crash_after_replay_counter++);
 }
 
-static dberr_t replace_and_insert(DDL_Record *record) {
-  dberr_t err = DB_SUCCESS;
-  trx_t *trx = trx_allocate_for_background();
-  trx_start_internal(trx, UT_LOCATION_HERE);
-  trx->ddl_operation = true;
-
-  /* update the thread_id for the record */
-  record->set_thread_id(ULINT_MAX);
-
-  {
-    DDL_Log_Table ddl_log(trx);
-    /* Remove old record and insert the new record */
-    err = ddl_log.remove(record->get_id());
-    ut_ad(err == DB_SUCCESS || err == DB_TOO_MANY_CONCURRENT_TRXS);
-
-    if (err == DB_SUCCESS) {
-      /* Insert new record entry */
-      err = ddl_log.insert(*record);
-    }
-  }
-
-  trx_commit_for_mysql(trx);
-  trx_free_for_background(trx);
-
-  return err;
-}
-
-dberr_t Log_DDL::replay_alter_encrypt_space_log(DDL_Record &record) {
-  dberr_t error = DB_SUCCESS;
-  /* Normal operation, we shouldn't come here during post_ddl */
-  ut_ad(is_in_recovery());
-
-  error = replace_and_insert(&record);
-  if (error != DB_SUCCESS) {
-    return error;
-  }
-  ut_ad(record.get_thread_id() == ULINT_MAX);
-
-  /* We could have resume encrypiton execution one by one for each tablespace
-  from here by calling SQL API to run the query. But then it would be blocking
-  server bootstrap. We need to resume ths encryption in BG thread so we need
-  to just make a note of this space and operation here and don't do any real
-  operation. */
-  ts_encrypt_ddl_records.push_back(&record);
-
-  /* Make sure not to delete this record till resume operation finishes.
-  This is to make sure that if there is a crash before that, we can resume
-  encryption in the next restart. */
-  record.set_deletable(false);
-
+void Log_DDL::replay_alter_encrypt_space_log(space_id_t space_id) {
+  /* NOOP */
   DBUG_INJECT_CRASH("ddl_log_crash_after_replay", crash_after_replay_counter++);
-  return error;
 }
 
 void Log_DDL::replay_drop_log(const table_id_t table_id) {
@@ -1809,7 +1672,7 @@ void Log_DDL::replay_drop_log(const table_id_t table_id) {
   DBUG_INJECT_CRASH("ddl_log_crash_after_replay", crash_after_replay_counter++);
 }
 
-void Log_DDL::replay_rename_table_log(const char *old_name,
+void Log_DDL::replay_rename_table_log(table_id_t table_id, const char *old_name,
                                       const char *new_name) {
   if (is_in_recovery()) {
     if (srv_print_ddl_logs) {
@@ -1823,9 +1686,9 @@ void Log_DDL::replay_rename_table_log(const char *old_name,
   trx_t *trx;
   trx = trx_allocate_for_background();
   trx->mysql_thd = current_thd;
-  trx_start_if_not_started(trx, true, UT_LOCATION_HERE);
+  trx_start_if_not_started(trx, true);
 
-  row_mysql_lock_data_dictionary(trx, UT_LOCATION_HERE);
+  row_mysql_lock_data_dictionary(trx);
   trx_set_dict_operation(trx, TRX_DICT_OP_TABLE);
 
   /* Convert partition table name DDL log, if needed. Required if
@@ -1893,11 +1756,11 @@ void Log_DDL::replay_remove_cache_log(table_id_t table_id,
   if (table != nullptr) {
     ut_ad(strcmp(table->name.m_name, table_name) == 0);
 
-    dict_sys_mutex_enter();
+    mutex_enter(&dict_sys->mutex);
     dd_table_close(table, nullptr, nullptr, true);
     btr_drop_ahi_for_table(table);
     dict_table_remove_from_cache(table);
-    dict_sys_mutex_exit();
+    mutex_exit(&dict_sys->mutex);
   }
 }
 
@@ -1957,3 +1820,17 @@ dberr_t Log_DDL::recover() {
 
   return (err);
 }
+
+dberr_t Log_DDL::post_ts_encryption(DDL_Records &records) {
+  for (auto record : records) {
+    record->set_deletable(true);
+  }
+
+  dberr_t err = Log_DDL::delete_by_ids(records);
+
+  for (auto record : records) {
+    UT_DELETE(record);
+  }
+
+  return (err);
+}
