@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 2005, 2022, Oracle and/or its affiliates.
+Copyright (c) 2005, 2019, Oracle and/or its affiliates. All Rights Reserved.
 Copyright (c) 2012, Facebook Inc.
 
 This program is free software; you can redistribute it and/or modify it under
@@ -44,7 +44,6 @@ external tools. */
 #include "page/zipdecompress.h"
 #include "page0page.h"
 #include "rem0rec.h"
-#include "rem0wrec.h"
 
 /* Enable some extra debugging output.  This code can be enabled
 independently of any UNIV_ debugging conditions. */
@@ -88,9 +87,9 @@ static void *page_zip_zalloc(void *opaque, /*!< in/out: memory heap */
 }
 
 /** Deallocate memory for zlib. */
-static void page_zip_free(void *opaque [[maybe_unused]], /*!< in: memory heap */
-                          void *address
-                          [[maybe_unused]]) /*!< in: object to free */
+static void page_zip_free(
+    void *opaque MY_ATTRIBUTE((unused)),  /*!< in: memory heap */
+    void *address MY_ATTRIBUTE((unused))) /*!< in: object to free */
 {}
 
 } /* extern "C" */
@@ -110,10 +109,10 @@ static void page_zip_fields_free(
   }
 }
 
-/** Configure the zlib allocator to use the given memory heap.
-@param[in,out] stream zlib stream
-@param[in] heap Memory heap to use */
-void page_zip_set_alloc(void *stream, mem_heap_t *heap) {
+/** Configure the zlib allocator to use the given memory heap. */
+void page_zip_set_alloc(void *stream,     /*!< in/out: zlib stream */
+                        mem_heap_t *heap) /*!< in: memory heap to use */
+{
   z_stream *strm = static_cast<z_stream *>(stream);
 
   strm->zalloc = page_zip_zalloc;
@@ -123,7 +122,7 @@ void page_zip_set_alloc(void *stream, mem_heap_t *heap) {
 
 /** Populate the sparse page directory from the dense directory.
  @return true on success, false on failure */
-[[nodiscard]] static bool page_zip_dir_decode(
+static MY_ATTRIBUTE((warn_unused_result)) ibool page_zip_dir_decode(
     const page_zip_des_t *page_zip, /*!< in: dense page directory on
                                    compressed page */
     page_t *page,                   /*!< in: compact page with valid header;
@@ -143,7 +142,7 @@ void page_zip_set_alloc(void *stream, mem_heap_t *heap) {
   if (UNIV_UNLIKELY(n_recs > n_dense)) {
     page_zip_fail(
         ("page_zip_dir_decode 1: %lu > %lu\n", (ulong)n_recs, (ulong)n_dense));
-    return false;
+    return (FALSE);
   }
 
   /* Traverse the list of stored records in the sorting order,
@@ -173,7 +172,7 @@ void page_zip_set_alloc(void *stream, mem_heap_t *heap) {
                       PAGE_ZIP_START + REC_N_NEW_EXTRA_BYTES)) {
       page_zip_fail(("page_zip_dir_decode 2: %u %u %lx\n", (unsigned)i,
                      (unsigned)n_recs, (ulong)offs));
-      return false;
+      return (FALSE);
     }
 
     recs[i] = page + (offs & PAGE_ZIP_DIR_SLOT_MASK);
@@ -187,7 +186,7 @@ void page_zip_set_alloc(void *stream, mem_heap_t *heap) {
     if (UNIV_UNLIKELY(slot != last_slot)) {
       page_zip_fail(("page_zip_dir_decode 3: %p != %p\n", (const void *)slot,
                      (const void *)last_slot));
-      return false;
+      return (FALSE);
     }
   }
 
@@ -198,23 +197,23 @@ void page_zip_set_alloc(void *stream, mem_heap_t *heap) {
     if (UNIV_UNLIKELY(offs & ~PAGE_ZIP_DIR_SLOT_MASK)) {
       page_zip_fail(("page_zip_dir_decode 4: %u %u %lx\n", (unsigned)i,
                      (unsigned)n_dense, (ulong)offs));
-      return false;
+      return (FALSE);
     }
 
     recs[i] = page + offs;
   }
 
   std::sort(recs, recs + n_dense);
-  return true;
+  return (TRUE);
 }
 
 /** Read the index information for the compressed page.
-@param[in]      buf             index information
-@param[in]      end             end of buf
-@param[in]      trx_id_col      NULL for non-leaf pages; for leaf pages,
+@param[in]	buf		index information
+@param[in]	end		end of buf
+@param[in]	trx_id_col	NULL for non-leaf pages; for leaf pages,
                                 pointer to where to store the position of the
                                 trx_id column
-@param[in]      is_spatial      is spatial index or not
+@param[in]	is_spatial	is spatial index or not
 @return own: dummy index describing the page, or NULL on error */
 static dict_index_t *page_zip_fields_decode(const byte *buf, const byte *end,
                                             ulint *trx_id_col,
@@ -252,7 +251,7 @@ static dict_index_t *page_zip_fields_decode(const byte *buf, const byte *end,
   index->table = table;
   index->n_uniq = n;
   /* avoid ut_ad(index->cached) in dict_index_get_n_unique_in_tree */
-  index->cached = true;
+  index->cached = TRUE;
 
   /* Initialize the fields. */
   for (b = buf, i = 0; i < n; i++) {
@@ -281,7 +280,7 @@ static dict_index_t *page_zip_fields_decode(const byte *buf, const byte *end,
     }
 
     dict_mem_table_add_col(table, nullptr, nullptr, mtype,
-                           val & 1 ? DATA_NOT_NULL : 0, len, true);
+                           val & 1 ? DATA_NOT_NULL : 0, len);
 
     /* The is_ascending flag does not matter during decompression,
     because we do not compare for "less than" or "greater than" */
@@ -321,28 +320,23 @@ static dict_index_t *page_zip_fields_decode(const byte *buf, const byte *end,
     index->type |= DICT_SPATIAL;
   }
 
-  index->set_instant_nullable(index->n_nullable);
-  if (index->is_clustered()) {
-    index->instant_cols = index->table->has_instant_cols();
-    index->row_versions = index->table->has_row_versions();
-  }
+  index->n_instant_nullable = index->n_nullable;
+  index->instant_cols =
+      (index->is_clustered() && index->table->has_instant_cols());
 
   return (index);
 }
 
 /** Apply the modification log to a record containing externally stored
-columns.  Do not copy the fields that are stored separately.
-@param[in]  index index
-@param[in,out]  rec  record
-@param[in]  offsets  rec_get_offsets(rec)
-@param[in]  trx_id_col  position of of DB_TRX_ID
-@param[in]  data  modification log
-@param[in]  end  end of modification log
-@return pointer to modification log, or NULL on failure */
-static const byte *page_zip_apply_log_ext(const dict_index_t *index, rec_t *rec,
-                                          const ulint *offsets,
-                                          ulint trx_id_col, const byte *data,
-                                          const byte *end) {
+ columns.  Do not copy the fields that are stored separately.
+ @return pointer to modification log, or NULL on failure */
+static const byte *page_zip_apply_log_ext(
+    rec_t *rec,           /*!< in/out: record */
+    const ulint *offsets, /*!< in: rec_get_offsets(rec) */
+    ulint trx_id_col,     /*!< in: position of of DB_TRX_ID */
+    const byte *data,     /*!< in: modification log */
+    const byte *end)      /*!< in: end of modification log */
+{
   ulint i;
   ulint len;
   byte *next_out = rec;
@@ -356,10 +350,10 @@ static const byte *page_zip_apply_log_ext(const dict_index_t *index, rec_t *rec,
 
     if (UNIV_UNLIKELY(i == trx_id_col)) {
       /* Skip trx_id and roll_ptr */
-      dst = rec_get_nth_field(index, rec, offsets, i, &len);
+      dst = rec_get_nth_field(rec, offsets, i, &len);
       if (UNIV_UNLIKELY(dst - next_out >= end - data) ||
           UNIV_UNLIKELY(len < (DATA_TRX_ID_LEN + DATA_ROLL_PTR_LEN)) ||
-          rec_offs_nth_extern(index, offsets, i)) {
+          rec_offs_nth_extern(offsets, i)) {
         page_zip_fail(
             ("page_zip_apply_log_ext:"
              " trx_id len %lu,"
@@ -372,8 +366,8 @@ static const byte *page_zip_apply_log_ext(const dict_index_t *index, rec_t *rec,
       memcpy(next_out, data, dst - next_out);
       data += dst - next_out;
       next_out = dst + (DATA_TRX_ID_LEN + DATA_ROLL_PTR_LEN);
-    } else if (rec_offs_nth_extern(index, offsets, i)) {
-      dst = rec_get_nth_field(index, rec, offsets, i, &len);
+    } else if (rec_offs_nth_extern(offsets, i)) {
+      dst = rec_get_nth_field(rec, offsets, i, &len);
       ut_ad(len >= BTR_EXTERN_FIELD_REF_SIZE);
 
       len += dst - next_out - BTR_EXTERN_FIELD_REF_SIZE;
@@ -493,8 +487,7 @@ static const byte *page_zip_apply_log(
       /* Clear the data bytes of the record. */
       mem_heap_t *heap = nullptr;
       ulint *offs;
-      offs = rec_get_offsets(rec, index, offsets, ULINT_UNDEFINED,
-                             UT_LOCATION_HERE, &heap);
+      offs = rec_get_offsets(rec, index, offsets, ULINT_UNDEFINED, &heap);
       memset(rec, 0, rec_offs_data_size(offs));
 
       if (UNIV_LIKELY_NULL(heap)) {
@@ -503,7 +496,9 @@ static const byte *page_zip_apply_log(
       continue;
     }
 
-    static_assert(REC_STATUS_NODE_PTR == true, "REC_STATUS_NODE_PTR != true");
+#if REC_STATUS_NODE_PTR != TRUE
+#error "REC_STATUS_NODE_PTR != TRUE"
+#endif
     rec_get_offsets_reverse(data, index, hs & REC_STATUS_NODE_PTR, offsets);
     rec_offs_make_valid(rec, index, offsets);
 
@@ -528,7 +523,7 @@ static const byte *page_zip_apply_log(
         return (nullptr);
       }
 
-      data = page_zip_apply_log_ext(index, rec, offsets, trx_id_col, data, end);
+      data = page_zip_apply_log_ext(rec, offsets, trx_id_col, data, end);
 
       if (UNIV_UNLIKELY(!data)) {
         return (nullptr);
@@ -562,7 +557,7 @@ static const byte *page_zip_apply_log(
       data += len;
     } else {
       /* Skip DB_TRX_ID and DB_ROLL_PTR. */
-      ulint l = rec_get_nth_field_offs(index, offsets, trx_id_col, &len);
+      ulint l = rec_get_nth_field_offs(offsets, trx_id_col, &len);
       byte *b;
 
       if (UNIV_UNLIKELY(data + l >= end) ||
@@ -597,14 +592,14 @@ static const byte *page_zip_apply_log(
 /** Set the heap_no in a record, and skip the fixed-size record header
  that is not included in the d_stream.
  @return true on success, false if d_stream does not end at rec */
-static bool page_zip_decompress_heap_no(
+static ibool page_zip_decompress_heap_no(
     z_stream *d_stream, /*!< in/out: compressed page stream */
     rec_t *rec,         /*!< in/out: record */
     ulint &heap_status) /*!< in/out: heap_no and status bits */
 {
   if (d_stream->next_out != rec - REC_N_NEW_EXTRA_BYTES) {
     /* n_dense has grown since the page was last compressed. */
-    return false;
+    return (FALSE);
   }
 
   /* Skip the REC_N_NEW_EXTRA_BYTES. */
@@ -618,12 +613,12 @@ static bool page_zip_decompress_heap_no(
   that this record is not instant can pass in rec_init_offsets() */
   rec[-REC_N_NEW_EXTRA_BYTES] = 0;
 
-  return true;
+  return (TRUE);
 }
 
 /** Decompress the records of a node pointer page.
  @return true on success, false on failure */
-static bool page_zip_decompress_node_ptrs(
+static ibool page_zip_decompress_node_ptrs(
     page_zip_des_t *page_zip, /*!< in/out: compressed page */
     z_stream *d_stream,       /*!< in/out: compressed page stream */
     rec_t **recs,             /*!< in: dense page directory
@@ -659,7 +654,7 @@ static bool page_zip_decompress_node_ptrs(
         if (!d_stream->avail_out) {
           break;
         }
-        [[fallthrough]];
+        /* fall through */
       default:
         page_zip_fail(
             ("page_zip_decompress_node_ptrs:"
@@ -669,12 +664,11 @@ static bool page_zip_decompress_node_ptrs(
     }
 
     if (!page_zip_decompress_heap_no(d_stream, rec, heap_status)) {
-      ut_d(ut_error);
+      ut_ad(0);
     }
 
     /* Read the offsets. The status bits are needed here. */
-    offsets = rec_get_offsets(rec, index, offsets, ULINT_UNDEFINED,
-                              UT_LOCATION_HERE, &heap);
+    offsets = rec_get_offsets(rec, index, offsets, ULINT_UNDEFINED, &heap);
 
     /* Non-leaf nodes should not have any externally
     stored columns. */
@@ -692,7 +686,7 @@ static bool page_zip_decompress_node_ptrs(
         if (!d_stream->avail_out) {
           break;
         }
-        [[fallthrough]];
+        /* fall through */
       default:
         page_zip_fail(
             ("page_zip_decompress_node_ptrs:"
@@ -731,7 +725,7 @@ static bool page_zip_decompress_node_ptrs(
          d_stream->msg));
   zlib_error:
     inflateEnd(d_stream);
-    return false;
+    return (FALSE);
   }
 
   /* Note that d_stream->avail_out > 0 may hold here
@@ -763,7 +757,7 @@ zlib_done:
                                      heap_status, index, offsets);
 
     if (UNIV_UNLIKELY(!mod_log_ptr)) {
-      return false;
+      return (FALSE);
     }
     page_zip->m_end = mod_log_ptr - page_zip->data;
     page_zip->m_nonempty = mod_log_ptr != d_stream->next_in;
@@ -778,7 +772,7 @@ zlib_done:
          (ulong)page_zip_get_trailer_len(page_zip, index->is_clustered()),
          (ulong)page_zip->m_end, (ulong)page_zip_get_size(page_zip),
          (ulong)index->is_clustered()));
-    return false;
+    return (FALSE);
   }
 
   /* Restore the uncompressed columns in heap_no order. */
@@ -787,8 +781,7 @@ zlib_done:
   for (slot = 0; slot < n_dense; slot++) {
     rec_t *rec = recs[slot];
 
-    offsets = rec_get_offsets(rec, index, offsets, ULINT_UNDEFINED,
-                              UT_LOCATION_HERE, &heap);
+    offsets = rec_get_offsets(rec, index, offsets, ULINT_UNDEFINED, &heap);
     /* Non-leaf nodes should not have any externally
     stored columns. */
     ut_ad(!rec_offs_any_extern(offsets));
@@ -798,12 +791,12 @@ zlib_done:
            REC_NODE_PTR_SIZE);
   }
 
-  return true;
+  return (TRUE);
 }
 
 /** Decompress the records of a leaf node of a secondary index.
  @return true on success, false on failure */
-static bool page_zip_decompress_sec(
+static ibool page_zip_decompress_sec(
     page_zip_des_t *page_zip, /*!< in/out: compressed page */
     z_stream *d_stream,       /*!< in/out: compressed page stream */
     rec_t **recs,             /*!< in: dense page directory
@@ -838,7 +831,7 @@ static bool page_zip_decompress_sec(
           if (!d_stream->avail_out) {
             break;
           }
-          [[fallthrough]];
+          /* fall through */
         default:
           page_zip_fail(
               ("page_zip_decompress_sec:"
@@ -849,7 +842,7 @@ static bool page_zip_decompress_sec(
     }
 
     if (!page_zip_decompress_heap_no(d_stream, rec, heap_status)) {
-      ut_d(ut_error);
+      ut_ad(0);
     }
   }
 
@@ -875,7 +868,7 @@ static bool page_zip_decompress_sec(
          d_stream->msg));
   zlib_error:
     inflateEnd(d_stream);
-    return false;
+    return (FALSE);
   }
 
   /* Note that d_stream->avail_out > 0 may hold here
@@ -907,30 +900,30 @@ zlib_done:
                                      heap_status, index, offsets);
 
     if (UNIV_UNLIKELY(!mod_log_ptr)) {
-      return false;
+      return (FALSE);
     }
     page_zip->m_end = mod_log_ptr - page_zip->data;
     page_zip->m_nonempty = mod_log_ptr != d_stream->next_in;
   }
 
-  if (UNIV_UNLIKELY(page_zip_get_trailer_len(page_zip, false) +
+  if (UNIV_UNLIKELY(page_zip_get_trailer_len(page_zip, FALSE) +
                         page_zip->m_end >=
                     page_zip_get_size(page_zip))) {
     page_zip_fail(("page_zip_decompress_sec: %lu + %lu >= %lu\n",
-                   (ulong)page_zip_get_trailer_len(page_zip, false),
+                   (ulong)page_zip_get_trailer_len(page_zip, FALSE),
                    (ulong)page_zip->m_end, (ulong)page_zip_get_size(page_zip)));
-    return false;
+    return (FALSE);
   }
 
   /* There are no uncompressed columns on leaf pages of
   secondary indexes. */
 
-  return true;
+  return (TRUE);
 }
 
 /** Initialize the REC_N_NEW_EXTRA_BYTES of each record.
  @return true on success, false on failure */
-static bool page_zip_set_extra_bytes(
+static ibool page_zip_set_extra_bytes(
     const page_zip_des_t *page_zip, /*!< in: compressed page */
     page_t *page,                   /*!< in/out: uncompressed page */
     ulint info_bits)                /*!< in: REC_INFO_MIN_REC_FLAG or 0 */
@@ -962,7 +955,7 @@ static bool page_zip_set_extra_bytes(
           ("page_zip_set_extra_bytes 1:"
            " %u %u %lx\n",
            (unsigned)i, (unsigned)n, (ulong)offs));
-      return false;
+      return (FALSE);
     }
 
     rec_set_next_offs_new(rec, offs);
@@ -982,12 +975,12 @@ static bool page_zip_set_extra_bytes(
 
   if (i >= n) {
     if (UNIV_LIKELY(i == n)) {
-      return true;
+      return (TRUE);
     }
 
     page_zip_fail(
         ("page_zip_set_extra_bytes 2: %u != %u\n", (unsigned)i, (unsigned)n));
-    return false;
+    return (FALSE);
   }
 
   offs = page_zip_dir_get(page_zip, i);
@@ -996,7 +989,7 @@ static bool page_zip_set_extra_bytes(
   for (;;) {
     if (UNIV_UNLIKELY(!offs) || UNIV_UNLIKELY(offs & ~PAGE_ZIP_DIR_SLOT_MASK)) {
       page_zip_fail(("page_zip_set_extra_bytes 3: %lx\n", (ulong)offs));
-      return false;
+      return (FALSE);
     }
 
     rec = page + offs;
@@ -1014,21 +1007,18 @@ static bool page_zip_set_extra_bytes(
   rec[-REC_N_NEW_EXTRA_BYTES] = 0; /* info_bits and n_owned */
   rec_set_next_offs_new(rec, 0);
 
-  return true;
+  return (TRUE);
 }
 
 /** Decompress a record of a leaf node of a clustered index that contains
-externally stored columns.
-@param[in,out]  d_stream  compressed page stream
-@param[in]  index  index
-@param[in,out]  rec  record
-@param[in]  offsets  rec_get_offsets(rec)
-@param[in]  trx_id_col  position of of DB_TRX_ID
-@return true on success */
-static bool page_zip_decompress_clust_ext(z_stream *d_stream,
-                                          const dict_index_t *index, rec_t *rec,
-                                          const ulint *offsets,
-                                          ulint trx_id_col) {
+ externally stored columns.
+ @return true on success */
+static ibool page_zip_decompress_clust_ext(
+    z_stream *d_stream,   /*!< in/out: compressed page stream */
+    rec_t *rec,           /*!< in/out: record */
+    const ulint *offsets, /*!< in: rec_get_offsets(rec) */
+    ulint trx_id_col)     /*!< in: position of of DB_TRX_ID */
+{
   ulint i;
 
   for (i = 0; i < rec_offs_n_fields(offsets); i++) {
@@ -1037,21 +1027,21 @@ static bool page_zip_decompress_clust_ext(z_stream *d_stream,
 
     if (UNIV_UNLIKELY(i == trx_id_col)) {
       /* Skip trx_id and roll_ptr */
-      dst = rec_get_nth_field(index, rec, offsets, i, &len);
+      dst = rec_get_nth_field(rec, offsets, i, &len);
       if (UNIV_UNLIKELY(len < DATA_TRX_ID_LEN + DATA_ROLL_PTR_LEN)) {
         page_zip_fail(
             ("page_zip_decompress_clust_ext:"
              " len[%lu] = %lu\n",
              (ulong)i, (ulong)len));
-        return false;
+        return (FALSE);
       }
 
-      if (rec_offs_nth_extern(index, offsets, i)) {
+      if (rec_offs_nth_extern(offsets, i)) {
         page_zip_fail(
             ("page_zip_decompress_clust_ext:"
              " DB_TRX_ID at %lu is ext\n",
              (ulong)i));
-        return false;
+        return (FALSE);
       }
 
       d_stream->avail_out = static_cast<uInt>(dst - d_stream->next_out);
@@ -1063,13 +1053,13 @@ static bool page_zip_decompress_clust_ext(z_stream *d_stream,
           if (!d_stream->avail_out) {
             break;
           }
-          [[fallthrough]];
+          /* fall through */
         default:
           page_zip_fail(
               ("page_zip_decompress_clust_ext:"
                " 1 inflate(Z_SYNC_FLUSH)=%s\n",
                d_stream->msg));
-          return false;
+          return (FALSE);
       }
 
       ut_ad(d_stream->next_out == dst);
@@ -1080,8 +1070,8 @@ static bool page_zip_decompress_clust_ext(z_stream *d_stream,
       memset(dst, 0, DATA_TRX_ID_LEN + DATA_ROLL_PTR_LEN);
 
       d_stream->next_out += DATA_TRX_ID_LEN + DATA_ROLL_PTR_LEN;
-    } else if (rec_offs_nth_extern(index, offsets, i)) {
-      dst = rec_get_nth_field(index, rec, offsets, i, &len);
+    } else if (rec_offs_nth_extern(offsets, i)) {
+      dst = rec_get_nth_field(rec, offsets, i, &len);
       ut_ad(len >= BTR_EXTERN_FIELD_REF_SIZE);
       dst += len - BTR_EXTERN_FIELD_REF_SIZE;
 
@@ -1093,13 +1083,13 @@ static bool page_zip_decompress_clust_ext(z_stream *d_stream,
           if (!d_stream->avail_out) {
             break;
           }
-          [[fallthrough]];
+          /* fall through */
         default:
           page_zip_fail(
               ("page_zip_decompress_clust_ext:"
                " 2 inflate(Z_SYNC_FLUSH)=%s\n",
                d_stream->msg));
-          return false;
+          return (FALSE);
       }
 
       ut_ad(d_stream->next_out == dst);
@@ -1120,12 +1110,12 @@ static bool page_zip_decompress_clust_ext(z_stream *d_stream,
     }
   }
 
-  return true;
+  return (TRUE);
 }
 
 /** Compress the records of a leaf node of a clustered index.
  @return true on success, false on failure */
-static bool page_zip_decompress_clust(
+static ibool page_zip_decompress_clust(
     page_zip_des_t *page_zip, /*!< in/out: compressed page */
     z_stream *d_stream,       /*!< in/out: compressed page stream */
     rec_t **recs,             /*!< in: dense page directory
@@ -1167,7 +1157,7 @@ static bool page_zip_decompress_clust(
         if (UNIV_LIKELY(!d_stream->avail_out)) {
           break;
         }
-        [[fallthrough]];
+        /* fall through */
       default:
         page_zip_fail(
             ("page_zip_decompress_clust:"
@@ -1177,12 +1167,11 @@ static bool page_zip_decompress_clust(
     }
 
     if (!page_zip_decompress_heap_no(d_stream, rec, heap_status)) {
-      ut_d(ut_error);
+      ut_ad(0);
     }
 
     /* Read the offsets. The status bits are needed here. */
-    offsets = rec_get_offsets(rec, index, offsets, ULINT_UNDEFINED,
-                              UT_LOCATION_HERE, &heap);
+    offsets = rec_get_offsets(rec, index, offsets, ULINT_UNDEFINED, &heap);
 
     /* This is a leaf page in a clustered index. */
 
@@ -1191,14 +1180,14 @@ static bool page_zip_decompress_clust(
     BTR_EXTERN_FIELD_REF separately. */
 
     if (rec_offs_any_extern(offsets)) {
-      if (UNIV_UNLIKELY(!page_zip_decompress_clust_ext(d_stream, index, rec,
-                                                       offsets, trx_id_col))) {
+      if (UNIV_UNLIKELY(!page_zip_decompress_clust_ext(d_stream, rec, offsets,
+                                                       trx_id_col))) {
         goto zlib_error;
       }
     } else {
       /* Skip trx_id and roll_ptr */
       ulint len;
-      byte *dst = rec_get_nth_field(index, rec, offsets, trx_id_col, &len);
+      byte *dst = rec_get_nth_field(rec, offsets, trx_id_col, &len);
       if (UNIV_UNLIKELY(len < DATA_TRX_ID_LEN + DATA_ROLL_PTR_LEN)) {
         page_zip_fail(
             ("page_zip_decompress_clust:"
@@ -1216,7 +1205,7 @@ static bool page_zip_decompress_clust(
           if (!d_stream->avail_out) {
             break;
           }
-          [[fallthrough]];
+          /* fall through */
         default:
           page_zip_fail(
               ("page_zip_decompress_clust:"
@@ -1246,7 +1235,7 @@ static bool page_zip_decompress_clust(
         if (!d_stream->avail_out) {
           break;
         }
-        [[fallthrough]];
+        /* fall through */
       default:
         page_zip_fail(
             ("page_zip_decompress_clust:"
@@ -1277,7 +1266,7 @@ static bool page_zip_decompress_clust(
          d_stream->msg));
   zlib_error:
     inflateEnd(d_stream);
-    return false;
+    return (FALSE);
   }
 
   /* Note that d_stream->avail_out > 0 may hold here
@@ -1309,19 +1298,19 @@ zlib_done:
                            n_dense, trx_id_col, heap_status, index, offsets);
 
     if (UNIV_UNLIKELY(!mod_log_ptr)) {
-      return false;
+      return (FALSE);
     }
     page_zip->m_end = mod_log_ptr - page_zip->data;
     page_zip->m_nonempty = mod_log_ptr != d_stream->next_in;
   }
 
-  if (UNIV_UNLIKELY(page_zip_get_trailer_len(page_zip, true) +
+  if (UNIV_UNLIKELY(page_zip_get_trailer_len(page_zip, TRUE) +
                         page_zip->m_end >=
                     page_zip_get_size(page_zip))) {
     page_zip_fail(("page_zip_decompress_clust: %lu + %lu >= %lu\n",
-                   (ulong)page_zip_get_trailer_len(page_zip, true),
+                   (ulong)page_zip_get_trailer_len(page_zip, TRUE),
                    (ulong)page_zip->m_end, (ulong)page_zip_get_size(page_zip)));
-    return false;
+    return (FALSE);
   }
 
   storage = page_zip_dir_start_low(page_zip, n_dense);
@@ -1335,11 +1324,10 @@ zlib_done:
     ulint len;
     byte *dst;
     rec_t *rec = recs[slot];
-    auto exists = !page_zip_dir_find_free(page_zip, page_offset(rec));
-    offsets = rec_get_offsets(rec, index, offsets, ULINT_UNDEFINED,
-                              UT_LOCATION_HERE, &heap);
+    ibool exists = !page_zip_dir_find_free(page_zip, page_offset(rec));
+    offsets = rec_get_offsets(rec, index, offsets, ULINT_UNDEFINED, &heap);
 
-    dst = rec_get_nth_field(index, rec, offsets, trx_id_col, &len);
+    dst = rec_get_nth_field(rec, offsets, trx_id_col, &len);
     ut_ad(len >= DATA_TRX_ID_LEN + DATA_ROLL_PTR_LEN);
     storage -= DATA_TRX_ID_LEN + DATA_ROLL_PTR_LEN;
     memcpy(dst, storage, DATA_TRX_ID_LEN + DATA_ROLL_PTR_LEN);
@@ -1353,17 +1341,17 @@ zlib_done:
     }
 
     for (i = 0; i < rec_offs_n_fields(offsets); i++) {
-      if (!rec_offs_nth_extern(index, offsets, i)) {
+      if (!rec_offs_nth_extern(offsets, i)) {
         continue;
       }
-      dst = rec_get_nth_field(index, rec, offsets, i, &len);
+      dst = rec_get_nth_field(rec, offsets, i, &len);
 
       if (UNIV_UNLIKELY(len < BTR_EXTERN_FIELD_REF_SIZE)) {
         page_zip_fail(
             ("page_zip_decompress_clust:"
              " %lu < 20\n",
              (ulong)len));
-        return false;
+        return (FALSE);
       }
 
       dst += len - BTR_EXTERN_FIELD_REF_SIZE;
@@ -1380,7 +1368,7 @@ zlib_done:
                " %p < %p + %lu\n",
                (const void *)externs, (const void *)page_zip->data,
                (ulong)page_zip->m_end));
-          return false;
+          return (FALSE);
         }
 
         memcpy(dst, externs, BTR_EXTERN_FIELD_REF_SIZE);
@@ -1394,21 +1382,21 @@ zlib_done:
     }
   }
 
-  return true;
+  return (TRUE);
 }
 
 /** Decompress a page.  This function should tolerate errors on the compressed
- page.  Instead of letting assertions fail, it will return false if an
+ page.  Instead of letting assertions fail, it will return FALSE if an
  inconsistency is detected.
  @return true on success, false on failure */
-bool page_zip_decompress_low(
+ibool page_zip_decompress_low(
     page_zip_des_t *page_zip, /*!< in: data, ssize;
                              out: m_start, m_end, m_nonempty, n_blobs */
     page_t *page,             /*!< out: uncompressed page, may be trashed */
-    bool all)                 /*!< in: true=decompress the whole page;
-                               false=verify but do not copy some
-                               page header fields that should not change
-                               after page creation */
+    ibool all)                /*!< in: TRUE=decompress the whole page;
+                              FALSE=verify but do not copy some
+                              page header fields that should not change
+                              after page creation */
 {
   z_stream d_stream;
   dict_index_t *index = nullptr;
@@ -1428,11 +1416,10 @@ bool page_zip_decompress_low(
                     page_zip_get_size(page_zip))) {
     page_zip_fail(("page_zip_decompress 1: %lu %lu\n", (ulong)n_dense,
                    (ulong)page_zip_get_size(page_zip)));
-    return false;
+    return (FALSE);
   }
 
-  heap = mem_heap_create(n_dense * (3 * sizeof *recs) + UNIV_PAGE_SIZE,
-                         UT_LOCATION_HERE);
+  heap = mem_heap_create(n_dense * (3 * sizeof *recs) + UNIV_PAGE_SIZE);
 
   recs = static_cast<rec_t **>(mem_heap_alloc(heap, n_dense * sizeof *recs));
 
@@ -1470,7 +1457,7 @@ bool page_zip_decompress_low(
   if (UNIV_UNLIKELY(!page_zip_dir_decode(page_zip, page, recs, n_dense))) {
   zlib_error:
     mem_heap_free(heap);
-    return false;
+    return (FALSE);
   }
 
   /* Copy the infimum and supremum records. */
@@ -1568,7 +1555,7 @@ bool page_zip_decompress_low(
     err_exit:
       page_zip_fields_free(index);
       mem_heap_free(heap);
-      return false;
+      return (FALSE);
     }
   } else {
     /* This is a leaf page in a clustered index. */
@@ -1589,5 +1576,5 @@ bool page_zip_decompress_low(
   page_zip_fields_free(index);
   mem_heap_free(heap);
 
-  return true;
+  return (TRUE);
 }
