@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1996, 2023, Oracle and/or its affiliates.
+Copyright (c) 1996, 2019, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
@@ -37,7 +37,7 @@ this program; if not, write to the Free Software Foundation, Inc.,
 #include "eval0proc.h"
 #include "ha_prototypes.h"
 #include "lock0lock.h"
-#include "log0chkp.h"
+#include "log0log.h"
 #include "pars0types.h"
 #include "que0que.h"
 #include "row0ins.h"
@@ -144,15 +144,15 @@ que_fork_t *que_fork_create(
 
   fork->graph = (graph != nullptr) ? graph : fork;
 
-  UT_LIST_INIT(fork->thrs);
+  UT_LIST_INIT(fork->thrs, &que_thr_t::thrs);
 
   return (fork);
 }
 
 /** Creates a query graph thread node.
-@param[in]      parent          parent node, i.e., a fork node
-@param[in]      heap            memory heap where created
-@param[in]      prebuilt        row prebuilt structure
+@param[in]	parent		parent node, i.e., a fork node
+@param[in]	heap		memory heap where created
+@param[in]	prebuilt	row prebuilt structure
 @return own: query thread node */
 que_thr_t *que_thr_create(que_fork_t *parent, mem_heap_t *heap,
                           row_prebuilt_t *prebuilt) {
@@ -190,11 +190,13 @@ que_thr_t *que_thr_create(que_fork_t *parent, mem_heap_t *heap,
 que_thr_t *que_thr_end_lock_wait(trx_t *trx) /*!< in: transaction with que_state
                                              in QUE_THR_LOCK_WAIT */
 {
-  ut_ad(locksys::owns_lock_shard(trx->lock.wait_lock));
+  que_thr_t *thr;
+  ibool was_active;
 
+  ut_ad(lock_mutex_own());
   ut_ad(trx_mutex_own(trx));
 
-  que_thr_t *const thr = trx->lock.wait_thr;
+  thr = trx->lock.wait_thr;
 
   ut_ad(thr != nullptr);
 
@@ -202,7 +204,7 @@ que_thr_t *que_thr_end_lock_wait(trx_t *trx) /*!< in: transaction with que_state
   /* In MySQL this is the only possible state here */
   ut_a(thr->state == QUE_THR_LOCK_WAIT);
 
-  bool const was_active = thr->is_active;
+  was_active = thr->is_active;
 
   que_thr_move_to_run_state(thr);
 
@@ -213,11 +215,12 @@ que_thr_t *que_thr_end_lock_wait(trx_t *trx) /*!< in: transaction with que_state
   /* In MySQL we let the OS thread (not just the query thread) to wait
   for the lock to be released: */
 
-  return !was_active ? thr : nullptr;
+  return ((!was_active && thr != nullptr) ? thr : nullptr);
 }
 
 /** Inits a query thread for a command. */
-static inline void que_thr_init_command(que_thr_t *thr) /*!< in: query thread */
+UNIV_INLINE
+void que_thr_init_command(que_thr_t *thr) /*!< in: query thread */
 {
   thr->run_node = thr;
   thr->prev_node = thr->common.parent;
@@ -275,6 +278,7 @@ que_thr_t *que_fork_scheduler_round_robin(
  caller */
 que_thr_t *que_fork_start_command(que_fork_t *fork) /*!< in: a query fork */
 {
+  que_thr_t *thr;
   que_thr_t *suspended_thr = nullptr;
   que_thr_t *completed_thr = nullptr;
 
@@ -296,7 +300,8 @@ que_thr_t *que_fork_start_command(que_fork_t *fork) /*!< in: a query fork */
 
   /* We make a single pass over the thr list within which we note which
   threads are ready to run. */
-  for (auto thr : fork->thrs) {
+  for (thr = UT_LIST_GET_FIRST(fork->thrs); thr != nullptr;
+       thr = UT_LIST_GET_NEXT(thrs, thr)) {
     switch (thr->state) {
       case QUE_THR_COMMAND_WAIT:
 
@@ -330,7 +335,7 @@ que_thr_t *que_fork_start_command(que_fork_t *fork) /*!< in: a query fork */
         ut_error;
     }
   }
-  que_thr_t *thr;
+
   if (suspended_thr) {
     thr = suspended_thr;
     que_thr_move_to_run_state(thr);
@@ -375,7 +380,7 @@ void que_graph_free_recursive(que_node_t *node) /*!< in: query graph node */
   }
 
   DBUG_PRINT("que_graph_free_recursive",
-             ("node: %p, type: " ULINTPF, node, que_node_get_type(node)));
+             ("node: %p, type: %lu", node, que_node_get_type(node)));
 
   switch (que_node_get_type(node)) {
     case QUE_NODE_FORK:
@@ -432,20 +437,16 @@ void que_graph_free_recursive(que_node_t *node) /*!< in: query graph node */
     case QUE_NODE_PURGE:
       purge = static_cast<purge_node_t *>(node);
 
-      purge->deinit();
+      mem_heap_free(purge->heap);
 
       break;
 
     case QUE_NODE_UPDATE:
       upd = static_cast<upd_node_t *>(node);
 
-      if (upd->update) {
-        upd->update->free_per_stmt_heap();
-      }
-
       if (upd->in_mysql_interface) {
-        btr_pcur_t::free_for_mysql(upd->pcur);
-        upd->in_mysql_interface = false;
+        btr_pcur_free_for_mysql(upd->pcur);
+        upd->in_mysql_interface = FALSE;
       }
 
       que_graph_free_recursive(upd->cascade_node);
@@ -512,7 +513,7 @@ void que_graph_free(que_t *graph) /*!< in: query graph; we assume that the
                                   afterwards! */
 {
   ut_ad(graph);
-  ut_ad(!dict_sys_mutex_own());
+  ut_ad(!mutex_own(&dict_sys->mutex));
 
   if (graph->sym_tab) {
     /* The following call frees dynamic memory allocated
@@ -584,16 +585,22 @@ static void que_thr_move_to_run_state(
 
     trx->lock.n_active_thrs++;
 
-    thr->is_active = true;
+    thr->is_active = TRUE;
   }
 
   thr->state = QUE_THR_RUNNING;
 }
 
-bool que_thr_stop(que_thr_t *thr) {
-  que_t *graph = thr->graph;
+/** Stops a query thread if graph or trx is in a state requiring it. The
+ conditions are tested in the order (1) graph, (2) trx.
+ @return true if stopped */
+ibool que_thr_stop(que_thr_t *thr) /*!< in: query thread */
+{
+  que_t *graph;
   trx_t *trx = thr_get_trx(thr);
 
+  graph = thr->graph;
+
   ut_ad(trx_mutex_own(trx));
 
   if (graph->state == QUE_FORK_COMMAND_WAIT) {
@@ -613,10 +620,10 @@ bool que_thr_stop(que_thr_t *thr) {
   } else {
     ut_ad(graph->state == QUE_FORK_ACTIVE);
 
-    return false;
+    return (FALSE);
   }
 
-  return true;
+  return (TRUE);
 }
 
 /** Decrements the query thread reference counts in the query graph and the
@@ -676,7 +683,7 @@ static void que_thr_dec_refer_count(
 
   --fork->n_active_thrs;
 
-  thr->is_active = false;
+  thr->is_active = FALSE;
 }
 
 /** A patch for MySQL used to 'stop' a dummy query thread used in MySQL. The
@@ -706,11 +713,11 @@ void que_thr_stop_for_mysql(que_thr_t *thr) /*!< in: query thread */
     }
   }
 
-  ut_ad(thr->is_active == true);
+  ut_ad(thr->is_active == TRUE);
   ut_ad(trx->lock.n_active_thrs == 1);
   ut_ad(thr->graph->n_active_thrs == 1);
 
-  thr->is_active = false;
+  thr->is_active = FALSE;
   thr->graph->n_active_thrs--;
 
   trx->lock.n_active_thrs--;
@@ -732,26 +739,26 @@ void que_thr_move_to_run_state_for_mysql(
 
     trx->lock.n_active_thrs++;
 
-    thr->is_active = true;
+    thr->is_active = TRUE;
   }
 
   thr->state = QUE_THR_RUNNING;
 }
 
 /** A patch for MySQL used to 'stop' a dummy query thread used in MySQL
- select, when there is no error or lock wait.
-@param[in] thr Query thread
-@param[in] trx Transaction */
-void que_thr_stop_for_mysql_no_error(que_thr_t *thr, trx_t *trx) {
+ select, when there is no error or lock wait. */
+void que_thr_stop_for_mysql_no_error(que_thr_t *thr, /*!< in: query thread */
+                                     trx_t *trx)     /*!< in: transaction */
+{
   ut_ad(thr->state == QUE_THR_RUNNING);
-  ut_ad(thr->is_active == true);
+  ut_ad(thr->is_active == TRUE);
   ut_ad(trx->lock.n_active_thrs == 1);
   ut_ad(thr->graph->n_active_thrs == 1);
   ut_a(thr->magic_n == QUE_THR_MAGIC_N);
 
   thr->state = QUE_THR_COMPLETED;
 
-  thr->is_active = false;
+  thr->is_active = FALSE;
   thr->graph->n_active_thrs--;
 
   trx->lock.n_active_thrs--;
@@ -786,7 +793,7 @@ que_node_t *que_node_get_containing_loop_node(que_node_t *node) /*!< in: node */
 #ifdef UNIV_DEBUG
 /** Gets information of an SQL query graph node.
 @return type description */
-[[nodiscard]] static const char *que_node_type_string(
+static MY_ATTRIBUTE((warn_unused_result)) const char *que_node_type_string(
     const que_node_t *node) /*!< in: query graph node */
 {
   switch (que_node_get_type(node)) {
@@ -829,8 +836,8 @@ que_node_t *que_node_get_containing_loop_node(que_node_t *node) /*!< in: node */
     case QUE_NODE_EXIT:
       return ("EXIT");
     default:
-      ut_d(ut_error);
-      ut_o(return ("UNKNOWN NODE TYPE"));
+      ut_ad(0);
+      return ("UNKNOWN NODE TYPE");
   }
 }
 #endif /* UNIV_DEBUG */
@@ -838,7 +845,8 @@ que_node_t *que_node_get_containing_loop_node(que_node_t *node) /*!< in: node */
 /** Performs an execution step on a query thread.
  @return query thread to run next: it may differ from the input
  parameter if, e.g., a subprocedure call is made */
-static inline que_thr_t *que_thr_step(que_thr_t *thr) /*!< in: query thread */
+UNIV_INLINE
+que_thr_t *que_thr_step(que_thr_t *thr) /*!< in: query thread */
 {
   que_node_t *node;
   que_thr_t *old_thr;
@@ -1036,7 +1044,16 @@ loop:
   }
 }
 
-dberr_t que_eval_sql(pars_info_t *info, const char *sql, trx_t *trx) {
+/** Evaluate the given SQL.
+ @return error code or DB_SUCCESS */
+dberr_t que_eval_sql(pars_info_t *info, /*!< in: info struct, or NULL */
+                     const char *sql,   /*!< in: SQL string */
+                     ibool reserve_dict_mutex,
+                     /*!< in: if TRUE, acquire/release
+                     dict_sys->mutex around call to pars_sql. */
+                     trx_t *trx) /*!< in: trx */
+{
+  que_thr_t *thr;
   que_t *graph;
 
   DBUG_TRACE;
@@ -1055,8 +1072,7 @@ dberr_t que_eval_sql(pars_info_t *info, const char *sql, trx_t *trx) {
 
   graph->fork_type = QUE_FORK_MYSQL_INTERFACE;
 
-  auto thr = que_fork_start_command(graph);
-  ut_a(thr);
+  ut_a(thr = que_fork_start_command(graph));
 
   que_run_threads(thr);
 
