@@ -1,18 +1,17 @@
 /*****************************************************************************
 
-Copyright (c) 2018, 2024, Oracle and/or its affiliates.
+Copyright (c) 2019, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is designed to work with certain software (including
-but not limited to OpenSSL) that is licensed under separate terms,
-as designated in a particular file or component or in included license
-documentation.  The authors of MySQL hereby grant you an additional
-permission to link the program and your derivative works with the
-separately licensed software that they have either included with
-the program or referenced in the documentation.
+This program is also distributed with certain software (including but not
+limited to OpenSSL) that is licensed under separate terms, as designated in a
+particular file or component or in included license documentation. The authors
+of MySQL hereby grant you an additional permission to link the program and
+your derivative works with the separately licensed software that they have
+included with MySQL.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -64,7 +63,7 @@ Tablespace_pool *tbsp_pool = nullptr;
 /* Directory to store session temporary tablespaces, provided by user */
 char *srv_temp_dir = nullptr;
 
-/** Session Temporary tablespace */
+/** Sesssion Temporary tablespace */
 Tablespace::Tablespace()
     : m_space_id(++m_last_used_space_id), m_inited(), m_thread_id() {
   ut_ad(m_space_id <= dict_sys_t::s_max_temp_space_id);
@@ -84,10 +83,9 @@ Tablespace::~Tablespace() {
                                           &file_pre_exists);
 
   if (file_pre_exists && !success) {
-    ib::error(ER_IB_FAILED_TO_DELETE_TABLESPACE_FILE)
-        << "Failed to delete file " << path();
+    ib::error() << "Failed to delete file " << path();
     os_file_get_last_error(true);
-    ut_d(ut_error);
+    ut_ad(0);
   }
 }
 
@@ -116,12 +114,14 @@ dberr_t Tablespace::create() {
 
   mtr_set_log_mode(&mtr, MTR_LOG_NO_REDO);
 
-  bool ret = fsp_header_init(m_space_id, FIL_IBT_FILE_INITIAL_SIZE, &mtr);
+  bool ret =
+      fsp_header_init(m_space_id, FIL_IBT_FILE_INITIAL_SIZE, &mtr, false);
   mtr_commit(&mtr);
 
   if (!ret) {
     return (DB_ERROR);
   }
+  buf_LRU_flush_or_remove_pages(m_space_id, BUF_REMOVE_FLUSH_WRITE, nullptr);
   return (DB_SUCCESS);
 }
 
@@ -136,21 +136,19 @@ bool Tablespace::close() const {
 
 bool Tablespace::truncate() {
   if (!m_inited) {
-    return false;
+    return (false);
   }
 
-  if (!fil_truncate_tablespace(m_space_id, FIL_IBT_FILE_INITIAL_SIZE)) {
-    return false;
+  bool success = fil_truncate_tablespace(m_space_id, FIL_IBT_FILE_INITIAL_SIZE);
+  if (!success) {
+    return (success);
   }
-
   mtr_t mtr;
-
   mtr_start(&mtr);
   mtr_set_log_mode(&mtr, MTR_LOG_NO_REDO);
-  fsp_header_init(m_space_id, FIL_IBT_FILE_INITIAL_SIZE, &mtr);
+  fsp_header_init(m_space_id, FIL_IBT_FILE_INITIAL_SIZE, &mtr, false);
   mtr_commit(&mtr);
-
-  return true;
+  return (true);
 }
 
 uint32_t Tablespace::file_id() const {
@@ -186,14 +184,14 @@ Tablespace_pool::Tablespace_pool(size_t init_size)
 Tablespace_pool::~Tablespace_pool() {
   mutex_destroy(&m_mutex);
   for (Tablespace *ts : *m_active) {
-    ut::delete_(ts);
+    UT_DELETE(ts);
   }
 
   for (Tablespace *ts : *m_free) {
-    ut::delete_(ts);
+    UT_DELETE(ts);
   }
-  ut::delete_(m_active);
-  ut::delete_(m_free);
+  UT_DELETE(m_active);
+  UT_DELETE(m_free);
 }
 
 Tablespace *Tablespace_pool::get(my_thread_id id, enum tbsp_purpose purpose) {
@@ -208,8 +206,7 @@ Tablespace *Tablespace_pool::get(my_thread_id id, enum tbsp_purpose purpose) {
       /* Failure to expand the pool means that there is no disk space
       available to create .IBT files */
       release();
-      ib::error(ER_IB_UNABLE_TO_EXPAND_TEMPORARY_TABLESPACE_POOL)
-          << "Unable to expand the temporary tablespace pool";
+      ib::error() << "Unable to expand the temporary tablespace pool";
       return (nullptr);
     }
   }
@@ -238,7 +235,7 @@ void Tablespace_pool::free_ts(Tablespace *ts) {
   if (it != m_active->end()) {
     m_active->erase(it);
   } else {
-    ut_d(ut_error);
+    ut_ad(0);
   }
 
   m_free->push_back(ts);
@@ -253,12 +250,12 @@ dberr_t Tablespace_pool::initialize(bool create_new_db) {
 
   ut_ad(m_active == nullptr && m_free == nullptr);
 
-  m_active = ut::new_withkey<Pool>(UT_NEW_THIS_FILE_PSI_KEY);
+  m_active = UT_NEW_NOKEY(Pool());
   if (m_active == nullptr) {
     return (DB_OUT_OF_MEMORY);
   }
 
-  m_free = ut::new_withkey<Pool>(UT_NEW_THIS_FILE_PSI_KEY);
+  m_free = UT_NEW_NOKEY(Pool());
   if (m_free == nullptr) {
     return (DB_OUT_OF_MEMORY);
   }
@@ -276,10 +273,10 @@ dberr_t Tablespace_pool::initialize(bool create_new_db) {
 dberr_t Tablespace_pool::expand(size_t size) {
   ut_ad(!m_pool_initialized || mutex_own(&m_mutex));
   for (size_t i = 0; i < size; i++) {
-    Tablespace *ts = ut::new_withkey<Tablespace>(UT_NEW_THIS_FILE_PSI_KEY);
+    Tablespace *ts = UT_NEW_NOKEY(Tablespace());
 
     if (ts == nullptr) {
-      return DB_OUT_OF_MEMORY;
+      return (DB_OUT_OF_MEMORY);
     }
 
     dberr_t err = ts->create();
@@ -287,11 +284,11 @@ dberr_t Tablespace_pool::expand(size_t size) {
     if (err == DB_SUCCESS) {
       m_free->push_back(ts);
     } else {
-      ut::delete_(ts);
-      return err;
+      UT_DELETE(ts);
+      return (err);
     }
   }
-  return DB_SUCCESS;
+  return (DB_SUCCESS);
 }
 
 void Tablespace_pool::delete_old_pool(bool create_new_db) {
@@ -299,8 +296,7 @@ void Tablespace_pool::delete_old_pool(bool create_new_db) {
     return;
   }
 
-  ib::info(ER_IB_MSG_SCANNING_TEMP_TABLESPACE_DIR)
-      << "Scanning temp tablespace dir:'" << temp_tbsp_dir << "'";
+  ib::info() << "Scanning temp tablespace dir:'" << temp_tbsp_dir << "'";
 
   os_file_type_t type;
   bool exists = false;
@@ -344,8 +340,7 @@ static dberr_t create_temp_dir() {
     temp_tbsp_dir.append(DIR_NAME);
     bool ret = os_file_create_directory(temp_tbsp_dir.c_str(), false);
     if (!ret) {
-      ib::error(ER_IB_TMP_TABLESPACE_CANNOT_CREATE_DIRECTORY)
-          << "Cannot create directory: " << temp_tbsp_dir.c_str();
+      ib::error() << "Cannot create directory: " << temp_tbsp_dir.c_str();
       return (DB_CANNOT_OPEN_FILE);
     }
     temp_tbsp_dir += OS_PATH_SEPARATOR;
@@ -367,8 +362,7 @@ dberr_t open_or_create(bool create_new_db) {
     return (err);
   }
 
-  tbsp_pool =
-      ut::new_withkey<Tablespace_pool>(UT_NEW_THIS_FILE_PSI_KEY, INIT_SIZE);
+  tbsp_pool = UT_NEW_NOKEY(Tablespace_pool(INIT_SIZE));
   if (tbsp_pool == nullptr) {
     return (DB_OUT_OF_MEMORY);
   }
@@ -383,7 +377,7 @@ void free_tmp(Tablespace *ts) {
   tbsp_pool->free_ts(ts);
 }
 
-void delete_pool_manager() { ut::delete_(tbsp_pool); }
+void delete_pool_manager() { UT_DELETE(tbsp_pool); }
 
 void close_files() {
   auto close = [&](const ibt::Tablespace *ts) { ts->close(); };
