@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1994, 2023, Oracle and/or its affiliates.
+Copyright (c) 1994, 2019, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
@@ -32,7 +32,6 @@ this program; if not, write to the Free Software Foundation, Inc.,
 
 #include <stdlib.h>
 
-#include "my_dbug.h"
 #include "univ.i"
 
 #ifndef UNIV_HOTBACKUP
@@ -48,18 +47,18 @@ void ut_set_assert_callback(std::function<void()> &callback) {
   assert_callback = callback;
 }
 
-/** Report a failed assertion.
-@param[in] expr The failed assertion
-@param[in] file Source file containing the assertion
-@param[in] line Line number of the assertion */
-[[noreturn]] void ut_dbg_assertion_failed(const char *expr, const char *file,
-                                          uint64_t line) {
+/** Report a failed assertion. */
+[[noreturn]] void ut_dbg_assertion_failed(
+    const char *expr, /*!< in: the failed assertion (optional) */
+    const char *file, /*!< in: source file containing the assertion */
+    ulint line)       /*!< in: line number of the assertion */
+{
 #if !defined(UNIV_HOTBACKUP) && !defined(UNIV_NO_ERR_MSGS) && \
     !defined(XTRABACKUP)
   ib::error(ER_IB_MSG_1273)
       << "Assertion failure: " << innobase_basename(file) << ":" << line
       << ((expr != nullptr) ? ":" : "") << ((expr != nullptr) ? expr : "")
-      << " thread " << to_string(std::this_thread::get_id());
+      << " thread " << os_thread_handle();
 
   flush_error_log_messages();
 
@@ -71,19 +70,19 @@ void ut_set_assert_callback(std::function<void()> &callback) {
   }
 
   fprintf(stderr,
-          "InnoDB: Assertion failure: %s:" UINT64PF
+          "InnoDB: Assertion failure: %s:" ULINTPF
           "%s%s\n"
-          "InnoDB: thread %s",
+          "InnoDB: thread " UINT64PF,
           filename, line, expr != nullptr ? ":" : "",
-          expr != nullptr ? expr : "",
-          to_string(std::this_thread::get_id()).c_str());
+          expr != nullptr ? expr : "", os_thread_handle());
 #endif /* !UNIV_HOTBACKUP */
 
   fputs(
       "InnoDB: We intentionally generate a memory trap.\n"
       "InnoDB: Submit a detailed bug report"
       " to https://jira.percona.com/projects/PXB.\n"
-      "InnoDB: If you get repeated assertion failures or crashes, even\n"
+      "InnoDB: If you get repeated assertion failures"
+      " or crashes, even\n"
       "InnoDB: immediately after the mysqld startup, there may be\n"
       "InnoDB: corruption in the InnoDB tablespace. Please refer to\n"
       "InnoDB: " REFMAN
@@ -97,5 +96,5 @@ void ut_set_assert_callback(std::function<void()> &callback) {
   if (assert_callback) {
     assert_callback();
   }
-  my_abort();
+  abort();
 }
