@@ -1,5 +1,5 @@
 /******************************************************
-Copyright (c) 2014, 2021 Percona LLC and/or its affiliates.
+Copyright (c) 2014-2020 Percona LLC and/or its affiliates.
 
 xbcloud utility. Manage backups on cloud storage services.
 
@@ -18,6 +18,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 
 *******************************************************/
 
+#include <my_alloc.h>
 #include <my_default.h>
 #include <my_dir.h>
 #include <my_getopt.h>
@@ -31,6 +32,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 #include <set>
 #include <string>
 #include <unordered_map>
+#include "template_utils.h"
 
 #include <curl/curl.h>
 
@@ -40,29 +42,20 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 
 #include "crc_glue.h"
 
-#include "xbcloud/azure.h"
 #include "xbcloud/s3.h"
 #include "xbcloud/swift.h"
 #include "xbcloud/util.h"
-#include "xbcrypt_common.h"
 
 using namespace xbcloud;
 
 #define XBCLOUD_VERSION XTRABACKUP_VERSION
-#define XBCLOUD_REVISION XTRABACKUP_REVISION
 
 /*****************************************************************************/
 
 const char *config_file = "my"; /* Default config file */
 
-const char *azure_development_access_key =
-    "Eby8vdM02xNOcqFlqUwJPLlmEtlCDXJ1OUzFT50uSRZ6IFsuFq2UVErCz4I6tq/K1SZFPTOtr/"
-    "KBHBeksoGMGw==";
-const char *azure_development_storage_account = "devstoreaccount1";
-const char *azure_development_container = "testcontainer";
-
-enum { SWIFT, S3, GOOGLE, AZURE };
-const char *storage_names[] = {"SWIFT", "S3", "GOOGLE", "AZURE", NullS};
+enum { SWIFT, S3, GOOGLE };
+const char *storage_names[] = {"SWIFT", "S3", "GOOGLE", NullS};
 
 const char *s3_bucket_lookup_names[] = {"AUTO", "DNS", "PATH", NullS};
 
@@ -93,7 +86,6 @@ static char *opt_s3_endpoint = nullptr;
 static char *opt_s3_access_key = nullptr;
 static char *opt_s3_secret_key = nullptr;
 static char *opt_s3_session_token = nullptr;
-static char *opt_s3_storage_class = nullptr;
 static char *opt_s3_bucket = nullptr;
 static ulong opt_s3_bucket_lookup;
 static ulong opt_s3_api_version = 0;
@@ -103,22 +95,11 @@ static char *opt_google_endpoint = nullptr;
 static char *opt_google_access_key = nullptr;
 static char *opt_google_secret_key = nullptr;
 static char *opt_google_session_token = nullptr;
-static char *opt_google_storage_class = nullptr;
 static char *opt_google_bucket = nullptr;
 
-static char *opt_azure_account = nullptr;
-static char *opt_azure_container = nullptr;
-static char *opt_azure_access_key = nullptr;
-static char *opt_azure_endpoint = nullptr;
-static char *opt_azure_storage_class = nullptr;
-static bool opt_azure_development_storage = 0;
-
 static std::string backup_name;
 static char *opt_cacert = nullptr;
 static ulong opt_parallel = 1;
-static ulong opt_max_retries = 10;
-static u_int32_t opt_max_backoff = 300000;
-
 static bool opt_insecure = false;
 static bool opt_md5 = false;
 static enum { MODE_GET, MODE_PUT, MODE_DELETE } opt_mode;
@@ -138,8 +119,6 @@ TYPELIB s3_bucket_lookup_typelib = {array_elements(s3_bucket_lookup_names) - 1,
 TYPELIB s3_api_version_typelib = {array_elements(s3_api_version_names) - 1, "",
                                   s3_api_version_names, nullptr};
 
-Http_client http_client;
-
 enum {
   OPT_STORAGE = 256,
 
@@ -166,36 +145,23 @@ enum {
   OPT_S3_ACCESS_KEY,
   OPT_S3_SECRET_KEY,
   OPT_S3_SESSION_TOKEN,
-  OPT_S3_STORAGE_CLASS,
   OPT_S3_BUCKET,
   OPT_S3_BUCKET_LOOKUP,
   OPT_S3_API_VERSION,
 
-  OPT_AZURE_ACCOUNT,
-  OPT_AZURE_CONTAINER,
-  OPT_AZURE_ACCESS_KEY,
-  OPT_AZURE_STORAGE_CLASS,
-  OPT_AZURE_ENDPOINT,
-  OPT_AZURE_DEVELOPMENT_STORAGE,
-
   OPT_GOOGLE_REGION,
   OPT_GOOGLE_ENDPOINT,
   OPT_GOOGLE_ACCESS_KEY,
   OPT_GOOGLE_SECRET_KEY,
   OPT_GOOGLE_SESSION_TOKEN,
-  OPT_GOOGLE_STORAGE_CLASS,
   OPT_GOOGLE_BUCKET,
 
   OPT_PARALLEL,
-  OPT_MAX_RETRIES,
-  OPT_MAX_BACKOFF,
   OPT_CACERT,
   OPT_HEADER,
   OPT_INSECURE,
   OPT_MD5,
-  OPT_VERBOSE,
-  OPT_CURL_RETRIABLE_ERRORS,
-  OPT_HTTP_RETRIABLE_ERRORS
+  OPT_VERBOSE
 };
 
 static struct my_option my_long_options[] = {
@@ -207,12 +173,8 @@ static struct my_option my_long_options[] = {
     {"help", '?', "Display this help and exit.", 0, 0, 0, GET_NO_ARG, NO_ARG, 0,
      0, 0, 0, 0, 0},
 
-    {"version", 'V', "Display version and exit.", 0, 0, 0, GET_NO_ARG, NO_ARG,
-     0, 0, 0, 0, 0, 0},
-
-    {"storage", OPT_STORAGE, "Specify storage type S3/SWIFT/GOOGLE/AZURE.",
-     &opt_storage, &opt_storage, &storage_typelib, GET_ENUM, REQUIRED_ARG, 0, 0,
-     0, 0, 0, 0},
+    {"storage", OPT_STORAGE, "Specify storage type S3/SWIFT.", &opt_storage,
+     &opt_storage, &storage_typelib, GET_ENUM, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
 
     {"swift-auth-version", OPT_SWIFT_AUTH_VERSION,
      "Swift authentication verison to use.", &opt_swift_auth_version,
@@ -291,19 +253,6 @@ static struct my_option my_long_options[] = {
      &opt_parallel, &opt_parallel, 0, GET_ULONG, REQUIRED_ARG, 1, 1, ULONG_MAX,
      0, 0, 0},
 
-    {"max-retries", OPT_MAX_RETRIES,
-     "Number of retries of chunk uploads/downloads after a failure (Default "
-     "10).",
-     &opt_max_retries, &opt_max_retries, 0, GET_ULONG, REQUIRED_ARG, 10, 1,
-     ULONG_MAX, 0, 0, 0},
-
-    {"max-backoff", OPT_MAX_BACKOFF,
-     "Maximum backoff delay in milliseconds in between chunk uploads/downloads "
-     "retries "
-     "(Default 300000).",
-     &opt_max_backoff, &opt_max_backoff, 0, GET_UINT32, REQUIRED_ARG, 300000, 1,
-     UINT_MAX32, 0, 0, 0},
-
     {"s3-region", OPT_S3_REGION, "S3 region.", &opt_s3_region, &opt_s3_region,
      0, GET_STR_ALLOC, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
 
@@ -320,14 +269,6 @@ static struct my_option my_long_options[] = {
      &opt_s3_session_token, &opt_s3_session_token, 0, GET_STR_ALLOC,
      REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
 
-    {"s3-storage-class", OPT_S3_STORAGE_CLASS,
-     "S3 storage class. STANDARD|STANDARD_IA|GLACIER|...     "
-     "... is meant for passing "
-     "custom storage class names provided by other S3 implementations such as "
-     "MinIO CephRadosGW, etc.",
-     &opt_s3_storage_class, &opt_s3_storage_class, 0, GET_STR_ALLOC,
-     REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
-
     {"s3-bucket", OPT_S3_BUCKET, "S3 bucket.", &opt_s3_bucket, &opt_s3_bucket,
      0, GET_STR_ALLOC, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
 
@@ -339,36 +280,6 @@ static struct my_option my_long_options[] = {
      &opt_s3_api_version, &opt_s3_api_version, &s3_api_version_typelib,
      GET_ENUM, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
 
-    {"azure-storage-account", OPT_AZURE_ACCOUNT, "AZURE storage account. ",
-     &opt_azure_account, &opt_azure_account, 0, GET_STR_ALLOC, REQUIRED_ARG, 0,
-     0, 0, 0, 0, 0},
-
-    {"azure-container-name", OPT_AZURE_CONTAINER, "AZURE container name. ",
-     &opt_azure_container, &opt_azure_container, 0, GET_STR_ALLOC, REQUIRED_ARG,
-     0, 0, 0, 0, 0, 0},
-
-    {"azure-access-key", OPT_AZURE_ACCESS_KEY, "AZURE access key.",
-     &opt_azure_access_key, &opt_azure_access_key, 0, GET_STR_ALLOC,
-     REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
-
-    {"azure-development-storage", OPT_AZURE_DEVELOPMENT_STORAGE,
-     "To run against azurite emulator use --azure-development-storage. It can "
-     "work with the default credentials provided by azurite. For example, it "
-     "uses http://127.0.0.1:10000 as the default endpoint, which can be "
-     "overwritten by --azure-endpoint. Users can also provide "
-     "--azure-access-key,  --azure-storage-account, --azure-container-name",
-     &opt_azure_development_storage, &opt_azure_development_storage, 0,
-     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
-
-    {"azure-endpoint", OPT_AZURE_ENDPOINT, "Azure cloud storage endpoint.",
-     &opt_azure_endpoint, &opt_azure_endpoint, 0, GET_STR_ALLOC, REQUIRED_ARG,
-     0, 0, 0, 0, 0, 0},
-
-    {"azure-tier-class", OPT_AZURE_STORAGE_CLASS,
-     "Azure cloud tier class. Hot|Cool|Archive", &opt_azure_storage_class,
-     &opt_azure_storage_class, 0, GET_STR_ALLOC, REQUIRED_ARG, 0, 0, 0, 0, 0,
-     0},
-
     {"google-region", OPT_GOOGLE_REGION, "Google cloud storage region.",
      &opt_google_region, &opt_google_region, 0, GET_STR_ALLOC, REQUIRED_ARG, 0,
      0, 0, 0, 0, 0},
@@ -378,24 +289,19 @@ static struct my_option my_long_options[] = {
      0, 0, 0, 0, 0, 0},
 
     {"google-access-key", OPT_GOOGLE_ACCESS_KEY,
-     "Google cloud storage access key.", &opt_google_access_key,
+     "Goolge cloud storage access key.", &opt_google_access_key,
      &opt_google_access_key, 0, GET_STR_ALLOC, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
 
     {"google-secret-key", OPT_GOOGLE_SECRET_KEY,
-     "Google cloud storage secret key.", &opt_google_secret_key,
+     "Goolge cloud storage secret key.", &opt_google_secret_key,
      &opt_google_secret_key, 0, GET_STR_ALLOC, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
 
     {"google-session-token", OPT_GOOGLE_SESSION_TOKEN,
-     "Google cloud storage session token.", &opt_google_session_token,
+     "Goolge cloud storage session token.", &opt_google_session_token,
      &opt_google_session_token, 0, GET_STR_ALLOC, REQUIRED_ARG, 0, 0, 0, 0, 0,
      0},
 
-    {"google-storage-class", OPT_GOOGLE_STORAGE_CLASS,
-     "Google cloud storage class. STANDARD|NEARLINE|COLDLINE|ARCHIVE",
-     &opt_google_storage_class, &opt_google_storage_class, 0, GET_STR_ALLOC,
-     REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
-
-    {"google-bucket", OPT_GOOGLE_BUCKET, "Google cloud storage bucket.",
+    {"google-bucket", OPT_GOOGLE_BUCKET, "Goolge cloud storage bucket.",
      &opt_google_bucket, &opt_google_bucket, 0, GET_STR_ALLOC, REQUIRED_ARG, 0,
      0, 0, 0, 0, 0},
 
@@ -414,26 +320,16 @@ static struct my_option my_long_options[] = {
     {"verbose", OPT_VERBOSE, "Turn ON cURL tracing.", &opt_verbose,
      &opt_verbose, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
 
-    {"curl-retriable-errors", OPT_CURL_RETRIABLE_ERRORS,
-     "Add a new curl error code as retriable. For multiple codes, use a comma "
-     "separated list of codes.",
-     0, 0, 0, GET_STR_ALLOC, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
-
-    {"http-retriable-errors", OPT_HTTP_RETRIABLE_ERRORS,
-     "Add a new http error code as retriable. For multiple codes, use a comma "
-     "separated list of codes.",
-     0, 0, 0, GET_STR_ALLOC, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
-
     {0, 0, 0, 0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0}};
 
 static void print_version() {
-  printf("%s  Ver %s for %s (%s) (revision id: %s)\n", my_progname, XBCLOUD_VERSION, SYSTEM_TYPE,
-         MACHINE_TYPE, XBCLOUD_REVISION);
+  printf("%s  Ver %s for %s (%s)\n", my_progname, XBCLOUD_VERSION, SYSTEM_TYPE,
+         MACHINE_TYPE);
 }
 
 static void usage() {
   print_version();
-  puts("Copyright (C) 2015, 2021 Percona LLC and/or its affiliates.");
+  puts("Copyright (C) 2015-2019 Percona LLC and/or its affiliates.");
   puts(
       "This software comes with ABSOLUTELY NO WARRANTY. "
       "This is free software,\nand you are welcome to modify and "
@@ -459,17 +355,13 @@ static void usage() {
   my_print_help(my_long_options);
 }
 
-static my_bool get_one_option(int optid,
-                              const struct my_option *opt
-                              __attribute__((unused)),
-                              char *argument) {
+static bool get_one_option(int optid,
+                           const struct my_option *opt __attribute__((unused)),
+                           char *argument) {
   switch (optid) {
     case '?':
       usage();
       exit(0);
-    case 'V':
-      print_version();
-      exit(0);
     case OPT_SWIFT_PASSWORD:
     case OPT_SWIFT_KEY:
     case OPT_SWIFT_TENANT:
@@ -480,8 +372,6 @@ static my_bool get_one_option(int optid,
     case OPT_GOOGLE_ACCESS_KEY:
     case OPT_GOOGLE_SECRET_KEY:
     case OPT_GOOGLE_SESSION_TOKEN:
-    case OPT_AZURE_ACCESS_KEY:
-    case OPT_AZURE_ACCOUNT:
       if (argument != nullptr) {
         while (*argument) *argument++ = 0;  // Destroy argument
       }
@@ -491,31 +381,6 @@ static my_bool get_one_option(int optid,
         extra_http_headers.insert(parse_http_header(argument));
       }
       break;
-    case OPT_CURL_RETRIABLE_ERRORS:
-      if (argument != nullptr) {
-        std::istringstream iss(argument);
-        for (std::string val; std::getline(iss, val, ',');) {
-          char *ptr;
-          long int code = strtol(val.c_str(), &ptr, 10);
-          if (!*ptr) {
-            http_client.set_curl_retriable_errors(
-                std::move(static_cast<CURLcode>(code)));
-          }
-        }
-      }
-      break;
-    case OPT_HTTP_RETRIABLE_ERRORS:
-      if (argument != nullptr) {
-        std::istringstream iss(argument);
-        for (std::string val; std::getline(iss, val, ',');) {
-          char *ptr;
-          long int code = strtol(val.c_str(), &ptr, 10);
-          if (!*ptr) {
-            http_client.set_http_retriable_errors(std::move(code));
-          }
-        }
-      }
-      break;
   }
 
   return (false);
@@ -550,7 +415,6 @@ static void get_env_args() {
   get_env_value(opt_s3_access_key, "S3_ACCESS_KEY_ID");
   get_env_value(opt_s3_secret_key, "S3_SECRET_ACCESS_KEY");
   get_env_value(opt_s3_session_token, "S3_SESSION_TOKEN");
-  get_env_value(opt_s3_storage_class, "S3_STORAGE_CLASS");
   get_env_value(opt_s3_region, "S3_DEFAULT_REGION");
   get_env_value(opt_cacert, "S3_CA_BUNDLE");
   get_env_value(opt_s3_endpoint, "S3_ENDPOINT");
@@ -558,7 +422,6 @@ static void get_env_args() {
   get_env_value(opt_s3_access_key, "AWS_ACCESS_KEY_ID");
   get_env_value(opt_s3_secret_key, "AWS_SECRET_ACCESS_KEY");
   get_env_value(opt_s3_session_token, "AWS_SESSION_TOKEN");
-  get_env_value(opt_s3_storage_class, "AWS_STORAGE_CLASS");
   get_env_value(opt_s3_region, "AWS_DEFAULT_REGION");
   get_env_value(opt_cacert, "AWS_CA_BUNDLE");
   get_env_value(opt_s3_endpoint, "AWS_ENDPOINT");
@@ -568,12 +431,6 @@ static void get_env_args() {
   get_env_value(opt_s3_region, "DEFAULT_REGION");
   get_env_value(opt_s3_endpoint, "ENDPOINT");
 
-  get_env_value(opt_azure_account, "AZURE_STORAGE_ACCOUNT");
-  get_env_value(opt_azure_container, "AZURE_CONTAINER_NAME");
-  get_env_value(opt_azure_access_key, "AZURE_ACCESS_KEY");
-  get_env_value(opt_azure_storage_class, "AZURE_STORAGE_CLASS");
-  get_env_value(opt_azure_endpoint, "AZURE_ENDPOINT");
-
   get_env_value(opt_google_access_key, "ACCESS_KEY_ID");
   get_env_value(opt_google_secret_key, "SECRET_ACCESS_KEY");
   get_env_value(opt_google_session_token, "SESSION_TOKEN");
@@ -582,10 +439,10 @@ static void get_env_args() {
 }
 
 static char **defaults_argv = nullptr;
+static MEM_ROOT argv_alloc{PSI_NOT_INSTRUMENTED, 512};
 
 static bool parse_args(int argc, char **argv) {
-  if (load_defaults("my", load_default_groups, &argc,
-                    &argv /*, &argv_alloc*/)) {
+  if (load_defaults("my", load_default_groups, &argc, &argv, &argv_alloc)) {
     return true;
   }
 
@@ -647,11 +504,8 @@ static bool parse_args(int argc, char **argv) {
     backup_name_uri_formatted = true;
     opt_storage = SWIFT;
     backup_uri = backup_uri.substr(8);
-  } else if (starts_with(backup_uri, "azure://")) {
-    backup_name_uri_formatted = true;
-    opt_storage = AZURE;
-    backup_uri = backup_uri.substr(8);
   }
+
   if (backup_name_uri_formatted) {
     auto slash_pos = backup_uri.find('/');
     if (slash_pos != std::string::npos) {
@@ -670,18 +524,10 @@ static bool parse_args(int argc, char **argv) {
         my_free(opt_swift_container);
         opt_swift_container =
             my_strdup(PSI_NOT_INSTRUMENTED, bucket_name.c_str(), MYF(MY_WME));
-      } else if (opt_storage == AZURE) {
-        my_free(opt_azure_container);
-        opt_azure_container =
-            my_strdup(PSI_NOT_INSTRUMENTED, bucket_name.c_str(), MYF(MY_WME));
       }
     }
   }
 
-  if (opt_azure_development_storage) {
-    opt_storage = AZURE;
-  }
-
   /* validate arguments */
   if (opt_storage == SWIFT) {
     if (opt_swift_user == nullptr) {
@@ -705,11 +551,6 @@ static bool parse_args(int argc, char **argv) {
       msg_ts("S3 secret key is not specified\n");
       return true;
     }
-  } else if (opt_storage == AZURE) {
-    if (opt_azure_account == nullptr && !opt_azure_development_storage) {
-      msg_ts("Azure account name is not specified\n");
-      return true;
-    }
   }
 
   while (argc > 0) {
@@ -775,10 +616,11 @@ bool xbcloud_put(Object_store *store, const std::string &container,
   }
   auto thread = h.run();
 
-  do {
+  while (!has_errors) {
     res = xb_stream_read_chunk(stream, &chunk);
     if (res != XB_STREAM_READ_CHUNK) {
       my_free(chunk.raw_data);
+      my_free(chunk.sparse_map);
       break;
     }
     if (chunk.type == XB_CHUNK_TYPE_UNKNOWN &&
@@ -849,52 +691,27 @@ bool xbcloud_put(Object_store *store, const std::string &container,
     }
 
     memset(&chunk, 0, sizeof(chunk));
-  } while (!has_errors);
+  }
 
   h.stop();
   thread.join();
 
   xb_stream_read_done(stream);
 
+  bool ret = (res != XB_STREAM_READ_ERROR) && (!has_errors);
 
-  // check if backup directory exists and it has some files
-  if (res == XB_STREAM_READ_ERROR || has_errors ||
-      !store->list_objects_in_directory(container, backup_name, object_list) ||
-      object_list.size() == 0) {
-    msg_ts("%s: Upload failed.\n", my_progname);
-    return false;
-  }
-
-  /* check if the xtrabackup_info.gz.00000000000 or
-  xtrabackup_info.00000000000000000000 is uploaded to cloud storage to
-  determine successful xbcloud "put" operation. */
-  bool file_found = false;
-  if (!object_list.empty()) {
-    std::string last_file_prefix = backup_name + "/xtrabackup_info";
-    auto last_file_size = last_file_prefix.size();
-    for (auto cur_file : object_list) {
-      if (cur_file.size() >= last_file_size &&
-	  cur_file.substr(0, last_file_size).compare(last_file_prefix) == 0) {
-	  file_found = true;
-        break;
-      }
-    }
-  }
-  if (!file_found) {
-    msg_ts("%s: Upload failed: backup is incomplete.\n", my_progname);
-    return false;
-  }
-
-  if (upload_md5) {
+  if (ret && upload_md5) {
     msg_ts("%s: Uploading md5\n", my_progname);
-    if (!store->upload_object(container, backup_name + ".md5", buf_md5)) {
-      msg_ts("%s: Upload failed: Error uploading md5.\n", my_progname);
-      return false;
-    }
+    ret = store->upload_object(container, backup_name + ".md5", buf_md5);
   }
 
-  msg_ts("%s: Upload completed.\n", my_progname);
-  return true;
+  if (!ret) {
+    msg_ts("%s: Upload failed.\n", my_progname);
+  } else {
+    msg_ts("%s: Upload completed.\n", my_progname);
+  }
+
+  return ret;
 }
 
 bool chunk_name_to_file_name(const std::string &chunk_name,
@@ -1104,9 +921,6 @@ bool xbcloud_download(Object_store *store, const std::string &container,
 
 struct main_exit_hook {
   ~main_exit_hook() {
-    if (defaults_argv != nullptr) {
-      free_defaults(defaults_argv);
-    }
     http_cleanup();
     my_end(0);
   }
@@ -1118,7 +932,6 @@ int main(int argc, char **argv) {
 #ifndef NO_SIGPIPE
   signal(SIGPIPE, SIG_IGN);
 #endif
-  xb_libgcrypt_init();
 
   http_init();
   crc_init();
@@ -1131,6 +944,7 @@ int main(int argc, char **argv) {
   }
 
   std::unique_ptr<Object_store> object_store = nullptr;
+  Http_client http_client;
   if (opt_verbose) {
     http_client.set_verbose(true);
   }
@@ -1241,25 +1055,20 @@ int main(int argc, char **argv) {
     msg_ts("Object store URL: %s\n", auth_info.url.c_str());
 
     object_store = std::unique_ptr<Object_store>(
-        new Swift_object_store(&http_client, auth_info.url, auth_info.token,
-                               opt_max_retries, opt_max_backoff));
+        new Swift_object_store(&http_client, auth_info.url, auth_info.token));
 
     container_name = opt_swift_container;
 
   } else if (opt_storage == S3) {
-    std::string region =
-        opt_s3_region != nullptr ? opt_s3_region : default_s3_region;
+    std::string region = opt_s3_region != nullptr ? opt_s3_region : "us-east-1";
     std::string access_key =
         opt_s3_access_key != nullptr ? opt_s3_access_key : "";
     std::string secret_key =
         opt_s3_secret_key != nullptr ? opt_s3_secret_key : "";
     std::string session_token =
         opt_s3_session_token != nullptr ? opt_s3_session_token : "";
-    std::string storage_class =
-        opt_s3_storage_class != nullptr ? opt_s3_storage_class : "";
     object_store = std::unique_ptr<Object_store>(new S3_object_store(
         &http_client, region, access_key, secret_key, session_token,
-        storage_class, opt_max_retries, opt_max_backoff,
         opt_s3_endpoint != nullptr ? opt_s3_endpoint : "",
         static_cast<s3_bucket_lookup_t>(opt_s3_bucket_lookup),
         static_cast<s3_api_version_t>(opt_s3_api_version)));
@@ -1287,15 +1096,11 @@ int main(int argc, char **argv) {
         opt_google_secret_key != nullptr ? opt_google_secret_key : "";
     std::string session_token =
         opt_google_session_token != nullptr ? opt_google_session_token : "";
-    std::string storage_class =
-        opt_google_storage_class != nullptr ? opt_google_storage_class : "";
-
     object_store = std::unique_ptr<Object_store>(new S3_object_store(
         &http_client, region, access_key, secret_key, session_token,
-        storage_class, opt_max_retries, opt_max_backoff,
         opt_google_endpoint != nullptr ? opt_google_endpoint
                                        : "https://storage.googleapis.com/",
-        LOOKUP_DNS, S3_V4));
+        LOOKUP_DNS, S3_V2));
 
     if (opt_google_bucket == nullptr) {
       msg_ts("%s: Google bucket is not specified.\n", my_progname);
@@ -1311,49 +1116,6 @@ int main(int argc, char **argv) {
              ->probe_api_version_and_lookup(container_name)) {
       return EXIT_FAILURE;
     }
-  } else if (opt_storage == AZURE) {
-    std::string storage_account =
-        opt_azure_account != nullptr ? opt_azure_account : "";
-    if (storage_account.empty() && opt_azure_development_storage) {
-      storage_account.assign(azure_development_storage_account);
-    }
-    if (storage_account.empty()) {
-      msg_ts("%s: Azure storage account is not specified.\n", my_progname);
-      return EXIT_FAILURE;
-    }
-
-    std::string access_key =
-        opt_azure_access_key != nullptr ? opt_azure_access_key : "";
-    if (access_key.empty() && opt_azure_development_storage) {
-      access_key.assign(azure_development_access_key);
-    }
-    if (access_key.empty()) {
-      msg_ts("%s: Azure access key is not specified.\n", my_progname);
-      return EXIT_FAILURE;
-    }
-
-    container_name = opt_azure_container != nullptr ? opt_azure_container : "";
-    if (container_name.empty() && opt_azure_development_storage) {
-      container_name.assign(azure_development_container);
-    }
-    if (container_name.empty()) {
-      msg_ts("%s: Azure container is not specified.\n", my_progname);
-      return EXIT_FAILURE;
-    }
-
-    std::string storage_class =
-        opt_azure_storage_class != nullptr ? opt_azure_storage_class : "";
-
-    std::string azure_endpoint =
-        opt_azure_endpoint != nullptr ? opt_azure_endpoint : "";
-
-    object_store = std::unique_ptr<Object_store>(new Azure_object_store(
-        &http_client, storage_account, access_key,
-        opt_azure_development_storage, storage_class, opt_max_retries,
-        opt_max_backoff, azure_endpoint));
-
-    reinterpret_cast<Azure_object_store *>(object_store.get())
-        ->set_extra_http_headers(extra_http_headers);
   }
 
   if (opt_mode == MODE_PUT) {
