@@ -1,6 +1,5 @@
 /******************************************************
-
-Copyright (c) 2011-2023 Percona LLC and/or its affiliates.
+Copyright (c) 2011-2020 Percona LLC and/or its affiliates.
 
 The xbstream utility: serialize/deserialize files in the XBSTREAM format.
 
@@ -26,28 +25,20 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 #include <my_thread.h>
 #include <mysql/service_mysql_alloc.h>
 #include <mysql_version.h>
+#include <pthread.h>
 #include <typelib.h>
-#include <list>
-#include <mutex>
-#include <thread>
 #include <unordered_map>
-#include <vector>
 #include "common.h"
 #include "crc_glue.h"
 #include "datasink.h"
 #include "ds_decompress.h"
 #include "ds_decompress_lz4.h"
-#include "ds_decompress_zstd.h"
 #include "ds_decrypt.h"
-#include "file_utils.h"
-#include "msg.h"
-#include "nulls.h"
 #include "template_utils.h"
 #include "xbcrypt_common.h"
 #include "xtrabackup_version.h"
 
 #define XBSTREAM_VERSION XTRABACKUP_VERSION
-#define XBSTREAM_REVISION XTRABACKUP_REVISION
 #define XBSTREAM_BUFFER_SIZE (10 * 1024 * 1024UL)
 
 typedef enum { RUN_MODE_NONE, RUN_MODE_CREATE, RUN_MODE_EXTRACT } run_mode_t;
@@ -64,43 +55,25 @@ datasink_t datasink_archive;
 datasink_t datasink_xbstream;
 datasink_t datasink_compress;
 datasink_t datasink_compress_lz4;
-datasink_t datasink_compress_zstd;
 datasink_t datasink_tmpfile;
 datasink_t datasink_encrypt;
-datasink_t datasink_fifo;
 
 static run_mode_t opt_mode;
 static char *opt_directory = NULL;
 static bool opt_verbose = 0;
 static int opt_parallel = 1;
-static int opt_fifo_streams = 1;
-static char *opt_fifo_dir = nullptr;
-static uint opt_fifo_timeout = 60;
 static ulong opt_encrypt_algo;
 static char *opt_encrypt_key_file = NULL;
 static char *opt_encrypt_key = NULL;
 static int opt_encrypt_threads = 1;
 static bool opt_decompress = 0;
 static uint opt_decompress_threads = 1;
-static bool opt_absolute_names = 0;
 
-static const int compression_prefix_len = 4;
-static const int compression_and_encryption_prefix_len = 12;
-
-enum {
-  OPT_DECOMPRESS = 256,
-  OPT_DECOMPRESS_THREADS,
-  OPT_ENCRYPT_THREADS,
-  OPT_PARALLEL,
-  OPT_FIFO_DIR,
-  OPT_FIFO_TIMEOUT
-};
+enum { OPT_DECOMPRESS = 256, OPT_DECOMPRESS_THREADS, OPT_ENCRYPT_THREADS };
 
 static struct my_option my_long_options[] = {
     {"help", '?', "Display this help and exit.", 0, 0, 0, GET_NO_ARG, NO_ARG, 0,
      0, 0, 0, 0, 0},
-    {"version", 'V', "Display version and exit.", 0, 0, 0, GET_NO_ARG, NO_ARG,
-     0, 0, 0, 0, 0, 0},
     {"create", 'c', "Stream the specified files to the standard output.", 0, 0,
      0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0},
     {"decompress", OPT_DECOMPRESS, "Decompress individual backup files.",
@@ -121,21 +94,9 @@ static struct my_option my_long_options[] = {
      0, 0},
     {"verbose", 'v', "Print verbose output.", &opt_verbose, &opt_verbose, 0,
      GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
-    {"parallel", OPT_PARALLEL,
-     "Number of worker threads for reading / writing.", &opt_parallel,
-     &opt_parallel, 0, GET_INT, REQUIRED_ARG, 1, 1, INT_MAX, 0, 0, 0},
-    {"fifo-streams", 'f',
-     "Number of FIFO files to use for parallel datafiles stream. Setting this "
-     "parameter to 1 disables FIFO and stream is sent to STDOUT",
-     &opt_fifo_streams, &opt_fifo_streams, 0, GET_INT, REQUIRED_ARG, 1, 1,
-     INT_MAX, 0, 0, 0},
-    {"fifo-dir", OPT_FIFO_DIR, "Directory to read Named Pipe.", &opt_fifo_dir,
-     &opt_fifo_dir, 0, GET_STR_ALLOC, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
-    {"fifo-timeout", OPT_FIFO_TIMEOUT,
-     "How many seconds to wait for other end to open the stream. "
-     "Default 60 seconds",
-     &opt_fifo_timeout, &opt_fifo_timeout, 0, GET_INT, REQUIRED_ARG, 60, 1,
-     INT_MAX, 0, 0, 0},
+    {"parallel", 'p', "Number of worker threads for reading / writing.",
+     &opt_parallel, &opt_parallel, 0, GET_INT, REQUIRED_ARG, 1, 1, INT_MAX, 0,
+     0, 0},
     {"decrypt", 'd', "Decrypt files ending with .xbcrypt.", &opt_encrypt_algo,
      &opt_encrypt_algo, &xbstream_encrypt_algo_typelib, GET_ENUM, REQUIRED_ARG,
      0, 0, 0, 0, 0, 0},
@@ -149,10 +110,6 @@ static struct my_option my_long_options[] = {
      "The default value is 1.",
      &opt_encrypt_threads, &opt_encrypt_threads, 0, GET_INT, REQUIRED_ARG, 1, 1,
      INT_MAX, 0, 0, 0},
-    {"absolute-names", 'P',
-     "Don't strip leading slashes from file names when creating archives.",
-     &opt_absolute_names, &opt_absolute_names, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0,
-     0, 0},
 
     {0, 0, 0, 0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0}};
 
@@ -161,23 +118,19 @@ typedef struct {
   uint pathlen;
   my_off_t offset;
   ds_file_t *file;
-  std::mutex *mutex;
+  pthread_mutex_t mutex;
 } file_entry_t;
 
 typedef struct {
-  int thread_id;
-  std::unordered_map<std::string, file_entry_t *> *filehash;
-  std::list<xb_rstream_t *> *streams;
+  std::unordered_map<std::string, file_entry_t *> filehash;
+  xb_rstream_t *stream;
   ds_ctxt_t *ds_ctxt;
   ds_ctxt_t *ds_decompress_quicklz_ctxt;
   ds_ctxt_t *ds_decompress_lz4_ctxt;
-  ds_ctxt_t *ds_decompress_zstd_ctxt;
   ds_ctxt_t *ds_decrypt_quicklz_ctxt;
   ds_ctxt_t *ds_decrypt_lz4_ctxt;
-  ds_ctxt_t *ds_decrypt_zstd_ctxt;
   ds_ctxt_t *ds_decrypt_uncompressed_ctxt;
-  std::mutex *mutex;
-  std::atomic<bool> *has_errors;
+  pthread_mutex_t *mutex;
 } extract_ctxt_t;
 
 static int get_options(int *argc, char ***argv);
@@ -205,21 +158,6 @@ int main(int argc, char **argv) {
     goto err;
   }
 
-  if (opt_mode == RUN_MODE_EXTRACT && opt_fifo_streams > 1) {
-    if (opt_fifo_dir == nullptr) {
-      msg("%s: --fifo-streams requires --fifo-dir parameter.\n", my_progname);
-      goto err;
-    }
-    /* adjust n_threads to at least fifo streams */
-    if (opt_parallel < opt_fifo_streams) {
-      msg_ts(
-          "%s: Adjusting number of threads from %d to %d to match number of "
-          "fifo streams.\n",
-          my_progname, opt_parallel, opt_fifo_streams);
-      opt_parallel = opt_fifo_streams;
-    }
-  }
-
   if (opt_encrypt_algo || opt_encrypt_key) {
     xb_libgcrypt_init();
   }
@@ -256,9 +194,8 @@ static int get_options(int *argc, char ***argv) {
 }
 
 static void print_version(void) {
-  printf("%s  Ver %s%s for %s (%s) (revision id: %s)\n", my_progname,
-         XBSTREAM_VERSION, get_suffix_str().c_str(), SYSTEM_TYPE, MACHINE_TYPE,
-         XBSTREAM_REVISION);
+  printf("%s  Ver %s for %s (%s)\n", my_progname, XBSTREAM_VERSION, SYSTEM_TYPE,
+         MACHINE_TYPE);
 }
 
 static void usage(void) {
@@ -313,9 +250,6 @@ static bool get_one_option(int optid,
     case 'k':
       hide_option(argument, &opt_encrypt_key);
       break;
-    case 'V':
-      print_version();
-      exit(0);
     case '?':
       usage();
       exit(0);
@@ -327,7 +261,7 @@ static bool get_one_option(int optid,
 static int stream_one_file(File file, xb_wstream_file_t *xbfile) {
   uchar *buf;
   size_t bytes;
-  [[maybe_unused]] size_t offset;
+  size_t offset;
 
   posix_fadvise(file, 0, 0, POSIX_FADV_SEQUENTIAL);
   offset = my_tell(file, MYF(MY_WME));
@@ -372,8 +306,6 @@ static int mode_create(int argc, char **argv) {
 
   for (i = 0; i < argc; i++) {
     char *filepath = argv[i];
-    const char *filepath_dst;
-    int filepath_prefix_len;
     File src_file;
     xb_wstream_file_t *file;
 
@@ -390,10 +322,7 @@ static int mode_create(int argc, char **argv) {
       goto err;
     }
 
-    filepath_dst = opt_absolute_names
-                       ? filepath
-                       : safer_name_suffix(filepath, &filepath_prefix_len);
-    file = xb_stream_write_open(stream, filepath_dst, &mystat, NULL, NULL);
+    file = xb_stream_write_open(stream, filepath, &mystat, NULL, NULL);
     if (file == NULL) {
       goto err;
     }
@@ -431,12 +360,12 @@ static file_entry_t *file_entry_new(extract_ctxt_t *ctxt, const char *path,
                                     uint pathlen) {
   file_entry_t *entry;
   ds_file_t *file;
+
   entry = (file_entry_t *)my_malloc(PSI_NOT_INSTRUMENTED, sizeof(file_entry_t),
                                     MYF(MY_WME | MY_ZEROFILL));
   if (entry == NULL) {
     return NULL;
   }
-  entry->mutex = new std::mutex();
 
   entry->path = my_strndup(PSI_NOT_INSTRUMENTED, path, pathlen, MYF(MY_WME));
   if (entry->path == NULL) {
@@ -448,8 +377,6 @@ static file_entry_t *file_entry_new(extract_ctxt_t *ctxt, const char *path,
     file = ds_open(ctxt->ds_decrypt_quicklz_ctxt, path, NULL);
   } else if (ctxt->ds_decrypt_lz4_ctxt && ends_with(path, ".lz4.xbcrypt")) {
     file = ds_open(ctxt->ds_decrypt_lz4_ctxt, path, NULL);
-  } else if (ctxt->ds_decrypt_zstd_ctxt && ends_with(path, ".zst.xbcrypt")) {
-    file = ds_open(ctxt->ds_decrypt_zstd_ctxt, path, NULL);
   } else if (ctxt->ds_decrypt_uncompressed_ctxt &&
              ends_with(path, ".xbcrypt")) {
     file = ds_open(ctxt->ds_decrypt_uncompressed_ctxt, path, NULL);
@@ -457,8 +384,6 @@ static file_entry_t *file_entry_new(extract_ctxt_t *ctxt, const char *path,
     file = ds_open(ctxt->ds_decompress_quicklz_ctxt, path, NULL);
   } else if (ctxt->ds_decompress_lz4_ctxt && ends_with(path, ".lz4")) {
     file = ds_open(ctxt->ds_decompress_lz4_ctxt, path, NULL);
-  } else if (ctxt->ds_decompress_zstd_ctxt && ends_with(path, ".zst")) {
-    file = ds_open(ctxt->ds_decompress_zstd_ctxt, path, NULL);
   } else {
     file = ds_open(ctxt->ds_ctxt, path, NULL);
   }
@@ -473,6 +398,8 @@ static file_entry_t *file_entry_new(extract_ctxt_t *ctxt, const char *path,
 
   entry->file = file;
 
+  pthread_mutex_init(&entry->mutex, NULL);
+
   return entry;
 
 err:
@@ -485,81 +412,55 @@ err:
 }
 
 static void file_entry_free(file_entry_t *entry) {
+  pthread_mutex_destroy(&entry->mutex);
   ds_close(entry->file);
   my_free(entry->path);
-  delete entry->mutex;
   my_free(entry);
 }
 
-static void extract_worker_thread_func(extract_ctxt_t &ctxt) {
-  xb_rstream_t *stream = NULL;
+static void *extract_worker_thread_func(void *arg) {
   xb_rstream_chunk_t chunk;
   file_entry_t *entry;
-  xb_rstream_result_t res = XB_STREAM_READ_CHUNK;
+  xb_rstream_result_t res;
 
-  ctxt.mutex->lock();
-  stream = ctxt.streams->front();
-  ctxt.streams->pop_front();
-  ctxt.streams->push_back(stream);
-  ctxt.mutex->unlock();
+  extract_ctxt_t *ctxt = (extract_ctxt_t *)arg;
 
   my_thread_init();
 
   memset(&chunk, 0, sizeof(chunk));
 
   while (1) {
-    /* Abort in case of error in any thread */
-    if (ctxt.has_errors->load()) {
-      break;
-    }
+    pthread_mutex_lock(ctxt->mutex);
+    res = xb_stream_read_chunk(ctxt->stream, &chunk);
 
-    stream->mutex->lock();
-    res = xb_stream_read_chunk(stream, &chunk);
     if (res != XB_STREAM_READ_CHUNK) {
-      stream->mutex->unlock();
+      pthread_mutex_unlock(ctxt->mutex);
       break;
     }
 
     /* If unknown type and ignorable flag is set, skip this chunk */
     if (chunk.type == XB_CHUNK_TYPE_UNKNOWN &&
         !(chunk.flags & XB_STREAM_FLAG_IGNORABLE)) {
-      stream->mutex->unlock();
+      pthread_mutex_unlock(ctxt->mutex);
       continue;
     }
 
-    if (!opt_absolute_names) {
-      int filepath_prefix_len;
-      safer_name_suffix(chunk.path, &filepath_prefix_len);
-      if (filepath_prefix_len != 0) {
-        msg("%s: absolute path not allowed: %.*s.\n", my_progname,
-            chunk.pathlen, chunk.path);
-        res = XB_STREAM_READ_ERROR;
-        stream->mutex->unlock();
-        break;
-      }
-    }
-
-    ctxt.mutex->lock();
-    stream->mutex->unlock();
     /* See if we already have this file open */
-    std::unordered_map<std::string, file_entry_t *>::const_iterator entry_it =
-        ctxt.filehash->find(chunk.path);
+    entry = ctxt->filehash[chunk.path];
 
-    if (entry_it == ctxt.filehash->end()) {
-      entry = file_entry_new(&ctxt, chunk.path, chunk.pathlen);
+    if (entry == NULL) {
+      entry = file_entry_new(ctxt, chunk.path, chunk.pathlen);
       if (entry == NULL) {
         res = XB_STREAM_READ_ERROR;
-        ctxt.mutex->unlock();
+        pthread_mutex_unlock(ctxt->mutex);
         break;
       }
-      ctxt.filehash->insert({chunk.path, entry});
-    } else {
-      entry = entry_it->second;
+      ctxt->filehash[chunk.path] = entry;
     }
 
-    entry->mutex->lock();
+    pthread_mutex_lock(&entry->mutex);
 
-    ctxt.mutex->unlock();
+    pthread_mutex_unlock(ctxt->mutex);
 
     if (chunk.type == XB_CHUNK_TYPE_PAYLOAD ||
         chunk.type == XB_CHUNK_TYPE_SPARSE) {
@@ -567,39 +468,16 @@ static void extract_worker_thread_func(extract_ctxt_t &ctxt) {
     }
 
     if (res != XB_STREAM_READ_CHUNK) {
-      entry->mutex->unlock();
+      pthread_mutex_unlock(&entry->mutex);
       break;
     }
 
     if (chunk.type == XB_CHUNK_TYPE_EOF) {
-      ctxt.mutex->lock();
-      entry->mutex->unlock();
-      ctxt.filehash->erase(entry->path);
+      pthread_mutex_lock(ctxt->mutex);
+      pthread_mutex_unlock(&entry->mutex);
+      ctxt->filehash.erase(entry->path);
       file_entry_free(entry);
-      ctxt.mutex->unlock();
-      /*
-       * no need for mutex here. At this point, we are guarantee that all other
-       * threads have completed its work with this file
-       */
-      if (opt_decompress && ctxt.ds_ctxt->fs_support_punch_hole &&
-          (is_compressed_suffix(chunk.path) ||
-           is_encrypted_and_compressed_suffix(chunk.path))) {
-        char path[FN_REFLEN] = {0};
-        memcpy(path, chunk.path, strlen(chunk.path));
-        unsigned short int qpress_offset = is_qpress_file(path) ? 1 : 0;
-        if (is_compressed_suffix(path))
-          path[strlen(path) - compression_prefix_len + qpress_offset] = 0;
-        if (is_encrypted_and_compressed_suffix(path))
-          path[strlen(path) - compression_and_encryption_prefix_len +
-               qpress_offset] = 0;
-
-        char error[512];
-        if (!restore_sparseness(path, XBSTREAM_BUFFER_SIZE, error,
-                                opt_verbose)) {
-          msg("%s: restore_sparseness failed for file %s: %s\n", my_progname,
-              chunk.path, error);
-        }
-      }
+      pthread_mutex_unlock(ctxt->mutex);
 
       continue;
     }
@@ -608,7 +486,7 @@ static void extract_worker_thread_func(extract_ctxt_t &ctxt) {
       msg("%s: out-of-order chunk: real offset = 0x%llx, "
           "expected offset = 0x%llx\n",
           my_progname, chunk.offset, entry->offset);
-      entry->mutex->unlock();
+      pthread_mutex_unlock(&entry->mutex);
       res = XB_STREAM_READ_ERROR;
       break;
     }
@@ -616,7 +494,7 @@ static void extract_worker_thread_func(extract_ctxt_t &ctxt) {
     if (chunk.type == XB_CHUNK_TYPE_PAYLOAD) {
       if (ds_write(entry->file, chunk.data, chunk.length)) {
         msg("%s: my_write() failed.\n", my_progname);
-        entry->mutex->unlock();
+        pthread_mutex_unlock(&entry->mutex);
         res = XB_STREAM_READ_ERROR;
         break;
       }
@@ -624,10 +502,9 @@ static void extract_worker_thread_func(extract_ctxt_t &ctxt) {
       entry->offset += chunk.length;
     } else if (chunk.type == XB_CHUNK_TYPE_SPARSE) {
       if (ds_write_sparse(entry->file, chunk.data, chunk.length,
-                          chunk.sparse_map_size, chunk.sparse_map,
-                          ctxt.ds_ctxt->fs_support_punch_hole)) {
+                          chunk.sparse_map_size, chunk.sparse_map)) {
         msg("%s: my_write() failed.\n", my_progname);
-        entry->mutex->unlock();
+        pthread_mutex_unlock(&entry->mutex);
         res = XB_STREAM_READ_ERROR;
         break;
       }
@@ -637,7 +514,7 @@ static void extract_worker_thread_func(extract_ctxt_t &ctxt) {
       entry->offset += chunk.length;
     }
 
-    entry->mutex->unlock();
+    pthread_mutex_unlock(&entry->mutex);
   }
 
   my_free(chunk.raw_data);
@@ -645,30 +522,30 @@ static void extract_worker_thread_func(extract_ctxt_t &ctxt) {
 
   my_thread_end();
 
-  if (res == XB_STREAM_READ_ERROR) ctxt.has_errors->store(res);
+  return (void *)(res);
 }
 
 static int mode_extract(int n_threads, int argc __attribute__((unused)),
                         char **argv __attribute__((unused))) {
+  xb_rstream_t *stream = NULL;
   ds_ctxt_t *ds_ctxt = NULL;
   ds_ctxt_t *ds_decrypt_lz4_ctxt = NULL;
-  ds_ctxt_t *ds_decrypt_zstd_ctxt = NULL;
   ds_ctxt_t *ds_decrypt_quicklz_ctxt = NULL;
   ds_ctxt_t *ds_decrypt_uncompressed_ctxt = NULL;
   ds_ctxt_t *ds_decompress_quicklz_ctxt = NULL;
   ds_ctxt_t *ds_decompress_lz4_ctxt = NULL;
-  ds_ctxt_t *ds_decompress_zstd_ctxt = NULL;
-  extract_ctxt_t *data_threads = NULL;
-  std::list<xb_rstream_t *> *streams = new std::list<xb_rstream_t *>();
-  xb_rstream_t *stream = NULL;
-  std::atomic<bool> has_errors{false};
-  std::vector<std::thread> threads;
-  std::unordered_map<std::string, file_entry_t *> *filehash =
-      new std::unordered_map<std::string, file_entry_t *>();
+  extract_ctxt_t ctxt;
   int i;
-  std::mutex mutex;
+  pthread_t *tids = NULL;
+  void **retvals = NULL;
+  pthread_mutex_t mutex;
   int ret = 0;
 
+  if (pthread_mutex_init(&mutex, NULL)) {
+    msg("%s: failed to initialize mutex.\n", my_progname);
+    return 1;
+  }
+
   /* If --directory is specified, it is already set as CWD by now. */
   ds_ctxt = ds_create(".", DS_TYPE_LOCAL);
   if (ds_ctxt == NULL) {
@@ -692,13 +569,6 @@ static int mode_extract(int n_threads, int argc __attribute__((unused)),
       goto exit;
     }
     ds_set_pipe(ds_decompress_lz4_ctxt, ds_ctxt);
-
-    ds_decompress_zstd_ctxt = ds_create(".", DS_TYPE_DECOMPRESS_ZSTD);
-    if (ds_decompress_zstd_ctxt == NULL) {
-      ret = 1;
-      goto exit;
-    }
-    ds_set_pipe(ds_decompress_zstd_ctxt, ds_ctxt);
   }
 
   if (opt_encrypt_algo) {
@@ -720,74 +590,45 @@ static int mode_extract(int n_threads, int argc __attribute__((unused)),
       ds_decrypt_lz4_ctxt = ds_create(".", DS_TYPE_DECRYPT);
       ds_set_pipe(ds_decrypt_lz4_ctxt, ds_decompress_lz4_ctxt);
     }
-    if (ds_decompress_zstd_ctxt) {
-      ds_decrypt_zstd_ctxt = ds_create(".", DS_TYPE_DECRYPT);
-      ds_set_pipe(ds_decrypt_zstd_ctxt, ds_decompress_zstd_ctxt);
-    }
   }
-  if (opt_fifo_streams > 1) {
-    for (int idx = 0; idx < opt_fifo_streams; idx++) {
-      char filename[FN_REFLEN];
-      snprintf(filename, sizeof(filename), "%s%s%lu", opt_fifo_dir, "/thread_",
-               (ulong)idx);
-      stream = xb_stream_read_new_fifo(filename, opt_fifo_timeout);
-      if (stream == nullptr) {
-        msg_ts(
-            "%s: xb_stream_read_new_fifo() failed for thread %d. Possibly "
-            "sender "
-            "did "
-            "not start.\n",
-            my_progname, idx);
-        ret = 1;
-        goto exit;
-      }
-      streams->push_back(stream);
-    }
-  } else {
-    stream = xb_stream_read_new_stdin();
-    if (stream == NULL) {
-      msg("%s: xb_stream_read_new_stdin() failed.\n", my_progname);
+
+  stream = xb_stream_read_new();
+  if (stream == NULL) {
+    msg("%s: xb_stream_read_new() failed.\n", my_progname);
+    pthread_mutex_destroy(&mutex);
+    ret = 1;
+    goto exit;
+  }
+
+  ctxt.stream = stream;
+  ctxt.ds_ctxt = ds_ctxt;
+  ctxt.ds_decompress_quicklz_ctxt = ds_decompress_quicklz_ctxt;
+  ctxt.ds_decompress_lz4_ctxt = ds_decompress_lz4_ctxt;
+  ctxt.ds_decrypt_uncompressed_ctxt = ds_decrypt_uncompressed_ctxt;
+  ctxt.ds_decrypt_quicklz_ctxt = ds_decrypt_quicklz_ctxt;
+  ctxt.ds_decrypt_lz4_ctxt = ds_decrypt_lz4_ctxt;
+  ctxt.mutex = &mutex;
+
+  tids = new pthread_t[n_threads];
+  retvals = new void *[n_threads];
+
+  for (i = 0; i < n_threads; i++)
+    pthread_create(tids + i, NULL, extract_worker_thread_func, &ctxt);
+
+  for (i = 0; i < n_threads; i++) pthread_join(tids[i], retvals + i);
+
+  for (i = 0; i < n_threads; i++) {
+    if ((ulong)retvals[i] == XB_STREAM_READ_ERROR) {
       ret = 1;
       goto exit;
     }
-    streams->push_back(stream);
   }
 
-  data_threads = (extract_ctxt_t *)my_malloc(
-      PSI_NOT_INSTRUMENTED, sizeof(extract_ctxt_t) * (n_threads + 1),
-      MYF(MY_FAE));
-  for (int i = 0; i < n_threads; i++) {
-    data_threads[i].thread_id = i;
-    data_threads[i].filehash = filehash;
-    data_threads[i].ds_ctxt = ds_ctxt;
-    data_threads[i].ds_decompress_quicklz_ctxt = ds_decompress_quicklz_ctxt;
-    data_threads[i].ds_decompress_lz4_ctxt = ds_decompress_lz4_ctxt;
-    data_threads[i].ds_decompress_zstd_ctxt = ds_decompress_zstd_ctxt;
-    data_threads[i].ds_decrypt_uncompressed_ctxt = ds_decrypt_uncompressed_ctxt;
-    data_threads[i].ds_decrypt_quicklz_ctxt = ds_decrypt_quicklz_ctxt;
-    data_threads[i].ds_decrypt_lz4_ctxt = ds_decrypt_lz4_ctxt;
-    data_threads[i].ds_decrypt_zstd_ctxt = ds_decrypt_zstd_ctxt;
-    data_threads[i].mutex = &mutex;
-    data_threads[i].has_errors = &has_errors;
-    data_threads[i].streams = streams;
-
-    threads.push_back(
-        std::thread(extract_worker_thread_func, std::ref(data_threads[i])));
-  }
-
-  for (i = 0; i < n_threads; i++) threads.at(i).join();
-
-  if (has_errors.load()) ret = 1;
-
 exit:
+  pthread_mutex_destroy(&mutex);
 
-  for (uint i = 0; i < (uint)n_threads; i++) {
-    char filename[FN_REFLEN];
-    snprintf(filename, sizeof(filename), "%s%s%lu", opt_fifo_dir, "/thread_",
-             (ulong)i);
-    unlink(filename);
-  }
-  delete filehash;
+  delete[] tids;
+  delete[] retvals;
 
   if (ds_ctxt != NULL) {
     ds_destroy(ds_ctxt);
@@ -798,29 +639,17 @@ exit:
   if (ds_decrypt_lz4_ctxt != NULL) {
     ds_destroy(ds_decrypt_lz4_ctxt);
   }
-  if (ds_decrypt_zstd_ctxt != NULL) {
-    ds_destroy(ds_decrypt_zstd_ctxt);
-  }
   if (ds_decrypt_quicklz_ctxt != NULL) {
     ds_destroy(ds_decrypt_quicklz_ctxt);
   }
   if (ds_decompress_lz4_ctxt != NULL) {
     ds_destroy(ds_decompress_lz4_ctxt);
   }
-  if (ds_decompress_zstd_ctxt != NULL) {
-    ds_destroy(ds_decompress_zstd_ctxt);
-  }
   if (ds_decompress_quicklz_ctxt != NULL) {
     ds_destroy(ds_decompress_quicklz_ctxt);
   }
+  xb_stream_read_done(stream);
 
-  if (data_threads != nullptr) {
-    my_free(data_threads);
-  }
-
-  std::for_each(streams->begin(), streams->end(),
-                [](xb_rstream_t *s) { xb_stream_read_done(s); });
-  delete streams;
   if (ret) {
     msg("exit code: %d\n", ret);
   }
