@@ -1,10 +1,5 @@
 set -eu
 
-function innobackupex()
-{
-    run_cmd $IB_BIN $IB_ARGS "$@"
-}
-
 function xtrabackup()
 {
     run_cmd $XB_BIN $XB_ARGS "$@"
@@ -43,16 +38,10 @@ function call_mysql_install_db()
 
         CLIENT_VERSION_STRING=`${MYSQL} --version`
 
-        if [[ ${CLIENT_VERSION_STRING} =~ Ver.*5\.7\.[0-9] ]] ; then
-            INSTALL_CMD="${MYSQLD} \
-            --defaults-file=${MYSQLD_VARDIR}/my.cnf \
-            --basedir=${MYSQL_BASEDIR} \
-            --initialize-insecure"
-        else
-            INSTALL_CMD="${MYSQL_INSTALL_DB} \
-            --defaults-file=${MYSQLD_VARDIR}/my.cnf \
-            --basedir=${MYSQL_BASEDIR}"
-        fi
+        INSTALL_CMD="${MYSQLD} \
+        --defaults-file=${MYSQLD_VARDIR}/my.cnf \
+        --basedir=${MYSQL_BASEDIR} \
+        --initialize-insecure"
 
         if ! ${INSTALL_CMD} ${MYSQLD_EXTRA_ARGS}
         then
@@ -63,8 +52,6 @@ function call_mysql_install_db()
             exit -1
         fi
 
-        [ -d ${MYSQLD_DATADIR}/test ] || mkdir ${MYSQLD_DATADIR}/test
-
         cd - >/dev/null 2>&1
 }
 
@@ -122,31 +109,31 @@ function run_cmd_expect_failure()
 
 function load_sakila()
 {
-vlog "Loading sakila"
-${MYSQL} ${MYSQL_ARGS} -e "create database sakila"
-vlog "Loading sakila scheme"
-${MYSQL} ${MYSQL_ARGS} sakila < inc/sakila-db/sakila-schema.sql
-vlog "Loading sakila data"
-${MYSQL} ${MYSQL_ARGS} sakila < inc/sakila-db/sakila-data.sql
+    vlog "Loading sakila"
+    ${MYSQL} ${MYSQL_ARGS} -e "create database sakila"
+    vlog "Loading sakila scheme"
+    ${MYSQL} ${MYSQL_ARGS} sakila < inc/sakila-db/sakila-schema.sql
+    vlog "Loading sakila data"
+    ${MYSQL} ${MYSQL_ARGS} sakila < inc/sakila-db/sakila-data.sql
 }
 
 function load_dbase_schema()
 {
-vlog "Loading $1 database schema"
-${MYSQL} ${MYSQL_ARGS} -e "create database $1"
-${MYSQL} ${MYSQL_ARGS} $1 < inc/$1-db/$1-schema.sql
+    vlog "Loading $1 database schema"
+    ${MYSQL} ${MYSQL_ARGS} -e "create database $1"
+    ${MYSQL} ${MYSQL_ARGS} $1 < inc/$1-db/$1-schema.sql
 }
 
 function load_dbase_data()
 {
-vlog "Loading $1 database data"
-${MYSQL} ${MYSQL_ARGS} $1 < inc/$1-db/$1-data.sql
+    vlog "Loading $1 database data"
+    ${MYSQL} ${MYSQL_ARGS} $1 < inc/$1-db/$1-data.sql
 }
 
 function drop_dbase()
 {
-  vlog "Dropping database $1"
-  run_cmd ${MYSQL} ${MYSQL_ARGS} -e "DROP DATABASE $1"
+    vlog "Dropping database $1"
+    run_cmd ${MYSQL} ${MYSQL_ARGS} -e "DROP DATABASE $1"
 }
 
 ########################################################################
@@ -257,8 +244,6 @@ function switch_server()
 	MYSQLD_ARGS="$MYSQLD_ARGS --user=root"
     fi
 
-    IB_ARGS="--defaults-file=$MYSQLD_VARDIR/my.cnf \
---no-version-check ${IB_EXTRA_OPTS:-}"
     XB_ARGS="--defaults-file=$MYSQLD_VARDIR/my.cnf \
 --no-version-check ${XB_EXTRA_OPTS:-}"
 
@@ -313,6 +298,7 @@ basedir=${MYSQL_BASEDIR}
 datadir=${MYSQLD_DATADIR}
 tmpdir=${MYSQLD_TMPDIR}
 log-error=${MYSQLD_ERRFILE}
+log-error-verbosity=3
 log-bin=mysql-bin
 relay-log=mysql-relay-bin
 pid-file=${MYSQLD_PIDFILE}
@@ -331,6 +317,8 @@ user=root
 ${XB_EXTRA_MY_CNF_OPTS:-}
 EOF
 
+        local new_instance
+
         # Create datadir and call mysql_install_db if it doesn't exist
         if [ ! -d "$MYSQLD_DATADIR" ]
         then
@@ -339,9 +327,12 @@ EOF
 	    # Reserve 900 series for SST nodes
             if [[ $id -lt 900 ]];then
                 call_mysql_install_db
-            else 
+            else
                 vlog "Skiping mysql_install_db of node $id for SST"
             fi
+            new_instance=yes
+        else
+            new_instance=no
         fi
 
         # Start the server
@@ -371,8 +362,12 @@ EOF
         fi
     done
 
-     vlog "Server with id=$id has been started on port $MYSQLD_PORT, \
+    vlog "Server with id=$id has been started on port $MYSQLD_PORT, \
 socket $MYSQLD_SOCKET"
+
+    if [ $new_instance = yes ] ; then
+        ${MYSQL} ${MYSQL_ARGS} -e "CREATE DATABASE IF NOT EXISTS test"
+    fi
 }
 
 ########################################################################
@@ -627,6 +622,34 @@ function checksum_table()
     $MYSQL $MYSQL_ARGS -Ns -e "CHECKSUM TABLE $2 EXTENDED" $1 | awk {'print $2'}
 }
 
+########################################################################
+# Prints checksum for a given table.
+# Takes database, table name and the list of columns
+########################################################################
+function checksum_table_columns()
+{
+    local db_name=$1
+    local table_name=$2
+    local old_ifs="$IFS"
+    local col_list
+
+    shift ; shift ; IFS="," ; col_list="$*" ; IFS="$old_ifs"
+
+    $MYSQL $MYSQL_ARGS -Ns ${db_name} <<EOF | sed -n 2p
+set @crc := '', @cnt := 0;
+
+select min(least(
+      length(@crc := sha1(concat(
+         @crc,
+         sha1(concat_ws('#', ${col_list}))))),
+      @cnt := @cnt + 1
+   )) as discard
+from ${table_name} use index(PRIMARY);
+
+select concat_ws(":", @cnt, @crc);
+EOF
+}
+
 ##########################################################################
 # Dumps a given database using mysqldump                                 #
 ##########################################################################
@@ -760,26 +783,6 @@ $MYSQL_VERSION_MINOR $MYSQL_VERSION_PATCH`
     [[ $server_str > $version_str ]]
 }
 
-#########################################################################
-# Return 0 if server is debug version
-########################################################################
-function is_debug_server()
-{
-
-  [[ $MYSQL_VERSION =~ .*debug ]]
-
-}
-
-#########################################################################
-# Requires server debug version
-########################################################################
-function require_debug_server()
-{
-    if ! is_debug_server; then
-        skip_test "Requires debug server version"
-    fi
-}
-
 #########################################################################
 # Require a server version higher than the first argument
 ########################################################################
@@ -788,26 +791,6 @@ function require_server_version_higher_than()
     is_server_version_higher_than $1 || \
         skip_test "Requires server version higher than $1"
 }
-#########################################################################
-# Return 0 if debug pxb version
-########################################################################
-function is_debug_pxb_version()
-{
-    if $XB_BIN --help 2>&1 | grep -q debug.=name; then
-      return 0;
-    fi
-    return 1;
-}
-
-#########################################################################
-# Requires debug pxb version
-########################################################################
-function require_debug_pxb_version()
-{
-    if ! is_debug_pxb_version; then
-        skip_test "Requires debug build"
-    fi
-}
 
 ########################################################################
 # Return 0 if the server version is lower than the first argument
@@ -879,6 +862,16 @@ function require_qpress()
     fi
 }
 
+########################################################################
+# Skip the test if qpress binary is not available
+########################################################################
+function require_lz4()
+{
+    if ! which lz4 > /dev/null 2>&1 ; then
+        skip_test "Requires lz4 to be installed"
+    fi
+}
+
 function require_tokudb()
 {
     if ! [ -a $(dirname ${MYSQLD})/../lib/mysql/plugin/ha_tokudb.so ]; then
@@ -905,25 +898,6 @@ function require_tokudb()
     fi
 }
 
-########################################################################
-# Return 0 if the xtrabackup has AddressSanitizer support
-########################################################################
-function is_asan()
-{
-    ldd $XB_BIN | grep -q libasan
-    return $?
-}
-
-#########################################################################
-# Skip test if xtrabackup has AddressSanitizer support
-########################################################################
-function skip_if_asan()
-{
-    if is_asan; then
-        skip_test "Incompatible with AddressSanitizer"
-    fi
-}
-
 ##############################################################################
 # Start a server with TokuDB plugins loaded and enabled.
 # Server id is 1, any arguments are passsed to the mysqld.
@@ -944,6 +918,39 @@ tokudb_lock_waits=ha_tokudb.so:\
 tokudb_background_job_status=ha_tokudb.so"
 }
 
+function require_rocksdb()
+{
+   if ! test -a $(dirname ${MYSQLD})/../lib/mysql/plugin/ha_rocksdb.so ; then
+        skip_test "Requires RocksDB"
+   fi
+}
+
+function init_rocksdb()
+{
+    mysql <<EOF
+INSTALL PLUGIN ROCKSDB SONAME 'ha_rocksdb.so';
+INSTALL PLUGIN ROCKSDB_CFSTATS SONAME 'ha_rocksdb.so';
+INSTALL PLUGIN ROCKSDB_DBSTATS SONAME 'ha_rocksdb.so';
+INSTALL PLUGIN ROCKSDB_PERF_CONTEXT SONAME 'ha_rocksdb.so';
+INSTALL PLUGIN ROCKSDB_PERF_CONTEXT_GLOBAL SONAME 'ha_rocksdb.so';
+INSTALL PLUGIN ROCKSDB_CF_OPTIONS SONAME 'ha_rocksdb.so';
+INSTALL PLUGIN ROCKSDB_GLOBAL_INFO SONAME 'ha_rocksdb.so';
+INSTALL PLUGIN ROCKSDB_COMPACTION_STATS SONAME 'ha_rocksdb.so';
+INSTALL PLUGIN ROCKSDB_DDL SONAME 'ha_rocksdb.so';
+INSTALL PLUGIN ROCKSDB_INDEX_FILE_MAP SONAME 'ha_rocksdb.so';
+INSTALL PLUGIN ROCKSDB_LOCKS SONAME 'ha_rocksdb.so';
+INSTALL PLUGIN ROCKSDB_TRX SONAME 'ha_rocksdb.so';
+INSTALL PLUGIN ROCKSDB_DEADLOCK SONAME 'ha_rocksdb.so';
+EOF
+}
+
+function require_debug_sync()
+{
+    if ! $XB_BIN --help 2>&1 | grep -q debug-sync; then
+        skip_test "Requires --debug-sync support"
+    fi
+}
+
 ##############################################################################
 # Execute a multi-row INSERT into a specified table.
 #
@@ -1122,32 +1129,19 @@ function innodb_checkpoint_lsn()
         grep "Last checkpoint at" | awk '{ print $4 }'
 }
 
-#general method to restore
-########################################################################
-function backup_and_restore() {
-  local db=$1;
-  record_db_state $db
-  rm -rf $topdir/backup
-  mkdir -p $topdir/backup
-
-  ## backup and prepare based on if keyring is used
-  if [ -z ${plugin_dir+x} ]
-  then
-    run_cmd xtrabackup --backup --target-dir=$topdir/backup
-    run_cmd $XB_BIN $XB_ARGS --prepare --target-dir=$topdir/backup
-  else
-    run_cmd xtrabackup --backup --target-dir=$topdir/backup --xtrabackup-plugin-dir=${plugin_dir}
-    run_cmd $XB_BIN $XB_ARGS --xtrabackup-plugin-dir=${plugin_dir} --prepare \
-      --target-dir=$topdir/backup ${keyring_args}
-  fi
-
-  # Restore
-  stop_server
-  rm -rf $mysql_datadir
-  run_cmd xtrabackup --copy-back --target-dir=$topdir/backup
-  start_server
-  # Verify backup
-  verify_db_state $db
+# Grep mysql.general_log for statements executed by xtrabackup
+#######################################################################
+function grep_general_log()
+{
+    ${MYSQL} ${MYSQL_ARGS} -Ne \
+        "SELECT argument FROM mysql.general_log WHERE command_type = 'Query' \
+            AND argument IN ('FLUSH NO_WRITE_TO_BINLOG TABLES', \
+                             'FLUSH TABLES WITH READ LOCK', \
+                             'FLUSH NO_WRITE_TO_BINLOG ENGINE LOGS', \
+                             'UNLOCK TABLES', \
+                             'LOCK TABLES FOR BACKUP', \
+                             'LOCK INSTANCE FOR BACKUP', \
+                             'UNLOCK INSTANCE')"
 }
 
 # To avoid unbound variable error when no server have been started
