@@ -5,13 +5,7 @@
 . inc/common.sh
 
 plugin_load=keyring_vault.so
-if test -d ${XTRABACKUP_BASEDIR}/keyring_vault
-then
-  plugin_dir=${XTRABACKUP_BASEDIR}/keyring_vault
-else
-  plugin_dir=${XTRABACKUP_BASEDIR}/lib/plugin
-fi
-
+plugin_dir=$PWD/../../../../plugin_output_directory
 keyring_vault_config=${TEST_VAR_ROOT}/keyring_vault_config
 keyring_args="--keyring-vault-config=${keyring_vault_config}"
 
@@ -33,6 +27,7 @@ function keyring_vault_ping()
 {
 	curl -H "X-Vault-Token: ${VAULT_TOKEN}" \
 		--cacert "${VAULT_CA}" -k \
+		--connect-timeout 3 \
 		"${VAULT_URL}/v1/sys/mounts" || \
 	return 1
 }
