@@ -49,7 +49,7 @@ Usage: $0 [-f] [-g] [-h] [-s suite] [-t test_name] [-d mysql_basedir] [-c build_
 -j N        Run tests in N parallel processes.
 -T seconds  Test timeout (default is $TEST_TIMEOUT seconds).
 -x options  Extra options to pass to xtrabackup
--i options  Extra options to pass to innobackupex
+-X options  Extra options to pass to mysqld
 -r path     Use specified path as root directory for test workers.
 EOF
 }
@@ -92,6 +92,7 @@ function kill_worker()
 {
     local worker=$1
     local pid=${worker_pids[$worker]}
+    local cpids=`pgrep -P $pid`
 
     worker_pids[$worker]=""
 
@@ -103,6 +104,7 @@ function kill_worker()
 
     # First send SIGTERM to let worker exit gracefully
     kill -SIGTERM $pid >/dev/null 2>&1 || true
+    for cpid in $cpids ; do kill -SIGTERM $cpid >/dev/null 2>&1 ; done
 
     sleep 1
 
@@ -114,6 +116,7 @@ function kill_worker()
 
     # Now kill with SIGKILL
     kill -SIGKILL $pid >/dev/null 2>&1 || true
+    for cpid in $cpids ; do kill -SIGKILL $cpid >/dev/null 2>&1 ; done
     wait $pid >/dev/null 2>&1 || true
     release_port_locks $pid
 }
@@ -262,16 +265,24 @@ function set_vars()
 {
     if gnutar --version > /dev/null 2>&1
     then
-	TAR=gnutar
+        TAR=gnutar
     elif gtar --version > /dev/null 2>&1
     then
-	TAR=gtar
+        TAR=gtar
     else
-	TAR=tar
+        TAR=tar
+    fi
+
+    if gnused --version > /dev/null 2>&1
+    then
+        SED=gnused
+    elif gsed --version > /dev/null 2>&1
+    then
+        SED=gsed
+    else
+        SED=sed
     fi
 
-    find_program MYSQL_INSTALL_DB mysql_install_db $MYSQL_BASEDIR/bin \
-	$MYSQL_BASEDIR/scripts
     find_program MYSQLD mysqld $MYSQL_BASEDIR/bin/ $MYSQL_BASEDIR/libexec
     find_program MYSQL mysql $MYSQL_BASEDIR/bin
     find_program MYSQLADMIN mysqladmin $MYSQL_BASEDIR/bin
@@ -281,7 +292,7 @@ function set_vars()
     # appropriately
     if test -d $PWD/../src
     then
-        PATH="$PWD/..:$PWD/../src:$PWD/../src/xbcloud:$PATH"
+	PATH="$PWD/../../../../runtime_output_directory:$PWD/..:$PWD/../src/xbcloud:$PATH"
         XTRABACKUP_BASEDIR="$PWD/../src"
     fi
 
@@ -299,7 +310,7 @@ function set_vars()
     fi
     DYLD_LIBRARY_PATH="$LD_LIBRARY_PATH"
 
-    export TAR MYSQL_BASEDIR MYSQL MYSQLD MYSQLADMIN \
+    export TAR SED MYSQL_BASEDIR MYSQL MYSQLD MYSQLADMIN \
 MYSQL_INSTALL_DB PATH LD_LIBRARY_PATH DYLD_LIBRARY_PATH MYSQLDUMP \
 XTRABACKUP_BASEDIR
 
@@ -327,7 +338,6 @@ function get_version_info()
     MYSQLD_EXTRA_ARGS=
 
     XB_BIN=""
-    IB_ARGS="--user=root --ibbackup=$XB_BIN ${IB_EXTRA_OPTS:-}"
     XB_ARGS="--no-defaults"
     XB_BIN="xtrabackup"
 
@@ -406,13 +416,7 @@ function get_version_info()
                 INNODB_VERSION="$MYSQL_VERSION"
             fi
             ;;
-        5.2 | 5.3 )
-            ;;
-        5.5 )
-            ;;
-        5.6 | 10.0 | 10.1)
-            ;;
-        5.7 )
+        8.0 )
             ;;
         *)
             vlog "Unknown MySQL/InnoDB version: $MYSQL_VERSION/$INNODB_VERSION"
@@ -428,20 +432,11 @@ function get_version_info()
     fi
     XB_BIN="$XB_PATH"
 
-    # Set the correct binary for innobackupex
-    IB_BIN="`which innobackupex`"
-    if [ -z "$IB_BIN" ]
-    then
-	vlog "Cannot find 'innobackupex' in PATH"
-	return 1
-    fi
-
     stop_server >>$OUTFILE 2>&1
 
     export MYSQL_VERSION MYSQL_VERSION_COMMENT MYSQL_FLAVOR \
 	INNODB_VERSION XTRADB_VERSION INNODB_FLAVOR \
-	XB_BIN IB_BIN IB_ARGS XB_ARGS MYSQLD_EXTRA_ARGS \
-        WSREP_READY LIBGALERA_PATH
+	XB_BIN XB_ARGS MYSQLD_EXTRA_ARGS WSREP_READY LIBGALERA_PATH
 }
 
 ###########################################################################
@@ -757,7 +752,7 @@ SUBUNIT_OUT=test_results.subunit
 NWORKERS=
 DEBUG_WORKER=""
 
-while getopts "fgh?:t:s:d:c:j:T:x:i:r:" options; do
+while getopts "fgh?:t:s:d:c:j:T:x:X:i:r:" options; do
         case $options in
             f ) force="yes";;
             t )
@@ -799,8 +794,8 @@ recognized for compatibility";;
                 XB_EXTRA_MY_CNF_OPTS="$OPTARG"
                 ;;
 
-            i )
-                IB_EXTRA_OPTS="$OPTARG"
+            X )
+                MYSQLD_EXTRA_MY_CNF_OPTS="$OPTARG"
                 ;;
 
             r )
