@@ -45,8 +45,7 @@ start_server
 mkdir -p $topdir/backup
 
 # Backup
-xtrabackup --datadir=$mysql_datadir --lock-ddl=false --backup \
-    --target-dir=$topdir/backup \
+xtrabackup --datadir=$mysql_datadir --backup --target-dir=$topdir/backup \
     --debug-sync="data_copy_thread_func" &
 
 job_pid=$!
@@ -110,8 +109,9 @@ xtrabackup --datadir=$mysql_datadir --prepare --target-dir=$topdir/backup
 stop_server
 
 # Restore
-rm -rf $mysql_datadir
-xtrabackup --move-back --target-dir=$topdir/backup
+rm -rf $mysql_datadir/ibdata1 $mysql_datadir/ib_logfile* \
+    $mysql_datadir/test/*.ibd
+cp -r $topdir/backup/* $mysql_datadir
 
 start_server
 
