@@ -8,14 +8,13 @@
 
 . inc/common.sh
 . inc/ib_part.sh
-require_server_version_higher_than 8.0.29
 
-start_server --innodb_file_per_table --innodb_directories=$TEST_VAR_ROOT/remote 
+start_server --innodb_file_per_table
 
 require_partitioning
 
 # Create InnoDB partitioned table
-ib_part_init $TEST_VAR_ROOT/remote InnoDB
+ib_part_init $topdir InnoDB
 
 # Saving the checksum of original table
 checksum_a=`checksum_table test test`
@@ -28,10 +27,21 @@ vlog "Backup taken"
 
 mysql -e 'SELECT * FROM INFORMATION_SCHEMA.TABLES\G'
 
+# also test xtrabackup --stats work with --tables-file
+xtrabackup --stats --tables='test.test$' --datadir=$topdir/backup
+
+COUNT=`xtrabackup --stats --tables='test.test$' --datadir=$topdir/backup \
+       | grep table: | grep -v "mysql/" | awk '{print $2}' | sort -u | wc -l`
+
+if [ $COUNT != 5 ] ; then
+	vlog "xtrabackup --stats does not work"
+	exit -1
+fi
+
 stop_server
 
 # Restore partial backup
-ib_part_restore $TEST_VAR_ROOT/remote $mysql_datadir
+ib_part_restore $topdir $mysql_datadir
 
 start_server
 
