@@ -9,12 +9,12 @@
 . inc/common.sh
 . inc/ib_part.sh
 
-start_server --innodb_file_per_table --innodb_directories=$TEST_VAR_ROOT/remote 
+start_server --innodb_file_per_table
 
 require_partitioning
 
 # Create InnoDB partitioned table
-ib_part_init $TEST_VAR_ROOT/remote InnoDB
+ib_part_init $topdir InnoDB
 
 # Saving the checksum of original table
 checksum_a=`checksum_table test test`
@@ -31,7 +31,7 @@ vlog "Backup taken"
 stop_server
 
 # Restore partial backup
-ib_part_restore $TEST_VAR_ROOT/remote $mysql_datadir
+ib_part_restore $topdir $mysql_datadir
 
 start_server
 
