@@ -8,14 +8,13 @@
 
 . inc/common.sh
 . inc/ib_part.sh
-require_server_version_higher_than 8.0.29
 
-start_server --innodb_file_per_table --innodb_directories=$TEST_VAR_ROOT/remote
+start_server --innodb_file_per_table
 
 require_partitioning
 
 # Create InnoDB partitioned table
-ib_part_init  $TEST_VAR_ROOT/remote InnoDB
+ib_part_init $topdir InnoDB
 
 # Saving the checksum of original table
 checksum_a=`checksum_table test test`
@@ -28,22 +27,9 @@ EOF
 ib_part_add_mandatory_tables $mysql_datadir $topdir/tables
 xtrabackup --backup --tables-file=$topdir/tables --target-dir=$topdir/backup
 xtrabackup --prepare --target-dir=$topdir/backup
-
 vlog "Backup taken"
 
-stop_server
-
-# Restore partial backup
-ib_part_restore $TEST_VAR_ROOT/remote $mysql_datadir
-
-start_server
-
-ib_part_assert_checksum $checksum_a
-
-mysql -e "set global innodb_fast_shutdown=0"
-shutdown_server
-
-COUNT=`xtrabackup --stats --tables-file=$TEST_VAR_ROOT/remote/tables --datadir=$mysql_datadir \
+COUNT=`xtrabackup --stats --tables-file=$topdir/tables --datadir=$topdir/backup \
        | grep table: | grep -v mysql/ | grep -v SYS_ \
        | awk '{print $2}' | sort -u | wc -l`
 echo "COUNT = $COUNT"
@@ -51,3 +37,12 @@ if [ $COUNT != 5 ] ; then
 	vlog "xtrabackup --stats does not work"
 	exit -1
 fi
+
+stop_server
+
+# Restore partial backup
+ib_part_restore $topdir $mysql_datadir
+
+start_server
+
+ib_part_assert_checksum $checksum_a
