@@ -53,47 +53,37 @@ function alter_table() {
 		-e "ALTER TABLE sakila.payment ADD COLUMN col1 INT"
 }
 
-if ! has_backup_locks ;
-then
+xtrabackup --backup --lock-ddl \
+	--lock-ddl-timeout=2 --target-dir=$topdir/backup1
 
-	run_cmd_expect_failure $XB_BIN $XB_ARGS \
-		--backup --lock-ddl --target-dir=$topdir/backup1
+start_transaction &
+tr_job_id=$!
 
-else
+while ! mysql -e 'SHOW PROCESSLIST' | grep -q 'User sleep' ; do
+	sleep 1
+done
 
-	xtrabackup --backup --lock-ddl \
-		--lock-ddl-timeout=2 --target-dir=$topdir/backup1
+alter_table &
+ddl_job_id=$!
 
-	start_transaction &
-	tr_job_id=$!
+while ! mysql -e 'SHOW PROCESSLIST' | grep -q 'Waiting for table metadata lock' ; do
+	sleep 1
+done
 
-	while ! mysql -e 'SHOW PROCESSLIST' | grep -q 'User sleep' ; do
-		sleep 1
-	done
+# SELECT blocks ALTER TABLE, ALTER TABLE blocks LOCK TABLES FOR BACKUP
+run_cmd_expect_failure \
+	$XB_BIN $XB_ARGS --backup --lock-ddl --lock-ddl-timeout=2 \
+			 --target-dir=$topdir/backup2
 
-	alter_table &
-	ddl_job_id=$!
+mysql -Ne "SELECT CONCAT('KILL ', id, ';') FROM \
+INFORMATION_SCHEMA.PROCESSLIST WHERE info LIKE 'SELECT SLEEP%' \
+OR info LIKE 'ALTER TABLE%'" | mysql
 
-	while ! mysql -e 'SHOW PROCESSLIST' | grep -q 'Waiting for table metadata lock' ; do
-		sleep 1
-	done
+wait $tr_job_id
+wait $ddl_job_id
 
-	# SELECT blocks ALTER TABLE, ALTER TABLE blocks LOCK TABLES FOR BACKUP
-	run_cmd_expect_failure \
-		$XB_BIN $XB_ARGS --backup --lock-ddl --lock-ddl-timeout=2 \
-				 --target-dir=$topdir/backup2
-
-	mysql -Ne "SELECT CONCAT('KILL ', id, ';') FROM \
-	INFORMATION_SCHEMA.PROCESSLIST WHERE info LIKE 'SELECT SLEEP%' \
-	OR info LIKE 'ALTER TABLE%'" | mysql
-
-	wait $tr_job_id
-	wait $ddl_job_id
-
-	xtrabackup --backup --lock-ddl \
-		--lock-ddl-timeout=2 --target-dir=$topdir/backup3
-
-fi
+xtrabackup --backup --lock-ddl \
+	--lock-ddl-timeout=2 --target-dir=$topdir/backup3
 
 mysql -e "CREATE TABLE rcount (val INT)" test
 mysql -e "INSERT INTO rcount (val) VALUES (0)" test
