@@ -6,6 +6,8 @@ start_server
 
 has_backup_locks || skip_test "Requires backup locks support"
 
+mysql -e "CREATE TABLE t (a INT) ENGINE=MyISAM" test
+
 mysql -e "SET GLOBAL general_log = 'ON'"
 mysql -e "SET GLOBAL log_output = 'TABLE'"
 
@@ -29,5 +31,5 @@ run_cmd diff -u $topdir/log - <<EOF
 argument
 SET SESSION lock_wait_timeout=31536000
 SET SESSION lock_wait_timeout=3
-SET SESSION lock_wait_timeout=2
+SET SESSION lock_wait_timeout=31536000
 EOF
