@@ -10,10 +10,6 @@
 
 start_server
 
-#test server version check
-xtrabackup --backup --no-server-version-check --target-dir=$topdir/backup1
-rm -r $topdir/backup1
-
 # Override the directory where percona-version-check is created to make the test
 # stable
 
@@ -23,8 +19,11 @@ vc_file=$PTDEBUG_VERSION_CHECK_HOME/percona-version-check
 
 [ ! -f $vc_file ] || die "$vc_file exists!"
 
+# MySQL using "caching_sha2_password" authentication plugin by default, but
+# most distributions come with libmysqlclient 5.6.x which doesn't have it
+# available
 mysql -e "ALTER USER 'root'@'localhost' \
-IDENTIFIED WITH 'caching_sha2_password' BY ''"
+IDENTIFIED WITH mysql_native_password BY ''"
 
 # VersionCheck is enabled by default, but the test suite adds --no-version-check
 # to IB_ARGS so we don't execute it for all tests.
