@@ -32,7 +32,8 @@ echo ${XBCLOUD_CREDENTIALS} | sed 's/ *--/\'$'\n/g' >> $topdir/xbcloud.cnf
 load_dbase_schema sakila
 load_dbase_data sakila
 
-mysql -e "CREATE database emptydatabas"
+mysql -e "ALTER TABLE payment COMPRESSION='lz4'" sakila
+mysql -e "CREATE database emptydatabase"
 
 now=$(date +%s)
 pwdpart=($(pwd | md5sum))
@@ -122,49 +123,9 @@ fi
 #PXB-2198 xbcloud doesn't return the error on delete if the backup doesn't exist in s3 bucket
 xbcloud --defaults-file=$topdir/xbcloud.cnf delete somedummyjunkbackup 2>$topdir/pxb-2198.log
 
-if ! grep -q "error: backup" $topdir/pxb-2198.log ; then
+if ! grep -q failed $topdir/pxb-2198.log ; then
     die 'xbcloud did not exit with error on delete'
 fi
-
-#PXB-2202 Xbcloud does not display an error when xtrabackup fails to create a backup
-xtrabackup --backup --stream=xbstream --extra-lsndir=$full_backup_dir \
-	   --target-dir=/some/unknown/dir | \
-    run_cmd xbcloud --defaults-file=$topdir/xbcloud.cnf put \
-	    --parallel=4 \
-	    somedummyjunkbackup 2>$topdir/pxb-2202.log
-
-
-if ! grep -q "Upload failed" $topdir/pxb-2202.log ; then
-    die 'xbcloud did not exit with error on upload'
-fi
-
-#PXB-2279 Xbcloud: Upload failed: backup is incomplete
-vlog "take full backup to test compression"
-run_cmd xbcloud --defaults-file=$topdir/xbcloud.cnf delete \
-	${full_backup_name} --parallel=4
-rm -r $full_backup_dir
-xtrabackup --backup --stream=xbstream --compress --extra-lsndir=$full_backup_dir \
-	   --target-dir=$full_backup_dir | \
-    run_cmd xbcloud --defaults-file=$topdir/xbcloud.cnf put \
-	    --parallel=4 \
-	    ${full_backup_name} 2> $topdir/pxb-2279.log
-if  grep -q "Upload failed" $topdir/pxb-2279.log ; then
-    die 'xbcloud exit with error on upload'
-fi
-vlog "take full backup to test encryption"
-run_cmd xbcloud --defaults-file=$topdir/xbcloud.cnf delete \
-	${full_backup_name} --parallel=4
-rm -r $full_backup_dir
-xtrabackup --backup --stream=xbstream  --encrypt=AES256 \
-          --encrypt-key="percona_xtrabackup_is_awesome___" --extra-lsndir=$full_backup_dir \
-	   --target-dir=$full_backup_dir | \
-    run_cmd xbcloud --defaults-file=$topdir/xbcloud.cnf put \
-	    --parallel=4 \
-	    ${full_backup_name} 2> $topdir/pxb-2279.log
-if  grep -q "Upload failed" $topdir/pxb-2279.log ; then
-    die 'xbcloud exit with error on upload'
-fi
-
 # cleanup
 run_cmd xbcloud --defaults-file=$topdir/xbcloud.cnf delete \
 	${full_backup_name} --parallel=4
