@@ -51,6 +51,7 @@ append_arg_to_args () {
             --build_tarball=*) TARBALL="$val" ;;
             --branch=*) BRANCH="$val" ;;
             --install_deps=*) INSTALL="$val" ;;
+            --branch=*) BRANCH="$val" ;;
             --repo=*) REPO="$val" ;;
             --rpm_release=*) RPM_RELEASE="$val" ;;
             --deb_release=*) DEB_RELEASE="$val" ;;
@@ -88,17 +89,6 @@ add_percona_yum_repo(){
     return
 }
 
-add_raven_yum_repo(){
-    if [ -n "$1" ]; then
-        version="$1"
-    else
-        version="8"
-    fi
-    yum -y install https://pkgs.dyn.su/el${version}/base/x86_64/raven-release-1.0-3.el${version}.noarch.rpm
-    yum -y update --enablerepo=raven
-    return
-}
-
 get_sources(){
     cd $WORKDIR
     if [ $SOURCE = 0 ]
@@ -106,7 +96,7 @@ get_sources(){
         echo "Sources will not be downloaded"
         return 0
     fi
-    git clone --depth 1 --branch $BRANCH "$REPO"
+    git clone "$REPO"
     retval=$?
     if [ $retval != 0 ]
     then
@@ -124,27 +114,23 @@ get_sources(){
 
     REVISION=$(git rev-parse --short HEAD)
     git reset --hard
-    git submodule update --init
     #
-    sed -i 's:13:14:' XB_VERSION 
     source XB_VERSION
-    cat XB_VERSION > ../percona-xtrabackup-2.4.properties
-    echo "REVISION=${REVISION}" >> ../percona-xtrabackup-2.4.properties
+    cat XB_VERSION > ../percona-xtrabackup-8.0.properties
+    echo "REVISION=${REVISION}" >> ../percona-xtrabackup-8.0.properties
     BRANCH_NAME="${BRANCH}"
-    echo "BRANCH_NAME=$(echo ${BRANCH_NAME} | awk -F '/' '{print $(NF)}')" >> ../percona-xtrabackup-2.4.properties
-    echo "PRODUCT=Percona-XtraBackup-2.4" >> ../percona-xtrabackup-2.4.properties
-    echo "PRODUCT_FULL=Percona-XtraBackup-${XB_VERSION_MAJOR}.${XB_VERSION_MINOR}.${XB_VERSION_PATCH}${XB_VERSION_EXTRA}" >> ../percona-xtrabackup-2.4.properties
-    echo "PRODUCT_UL_DIR=Percona-XtraBackup-2.4" >> ../percona-xtrabackup-2.4.properties
-    PRODUCT="Percona-XtraBackup-2.4"
-    PRODUCT_FULL="Percona-XtraBackup-${XB_VERSION_MAJOR}.${XB_VERSION_MINOR}.${XB_VERSION_PATCH}${XB_VERSION_EXTRA}"
+    echo "BRANCH_NAME=$(echo ${BRANCH_NAME} | awk -F '/' '{print $(NF)}')" >> ../percona-xtrabackup-8.0.properties
+    echo "PRODUCT=Percona-XtraBackup-8.0" >> ../percona-xtrabackup-8.0.properties
+    echo "PRODUCT_FULL=Percona-XtraBackup-${XB_VERSION_MAJOR}.${XB_VERSION_MINOR}-${XB_VERSION_PATCH}${XB_VERSION_EXTRA}" >> ../percona-xtrabackup-8.0.properties
+    echo "PRODUCT_UL_DIR=Percona-XtraBackup-8.0" >> ../percona-xtrabackup-8.0.properties
     if [ -z "${DESTINATION}" ]; then
         export DESTINATION=experimental
     fi 
-    echo "DESTINATION=${DESTINATION}" >> percona-xtrabackup-2.4.properties
+    echo "DESTINATION=${DESTINATION}" >> percona-xtrabackup-8.0.properties
 
     enable_venv
 
-    $CMAKE_BIN . -DDOWNLOAD_BOOST=1 -DWITH_BOOST=${WORKDIR}/boost -DWITH_MAN_PAGES=1
+    $CMAKE_BIN . -DDOWNLOAD_BOOST=1 -DWITH_BOOST=${WORKDIR}/boost -DFORCE_INSOURCE_BUILD=1 -DWITH_MAN_PAGES=1
     make dist
     #
     EXPORTED_TAR=$(basename $(find . -type f -name percona-xtrabackup*.tar.gz | sort | tail -n 1))
@@ -155,7 +141,6 @@ get_sources(){
     rm -f ${EXPORTED_TAR}
 
     cd ${PXBDIR}
-    #rsync -av ../extra/libkmip/* extra/libkmip/
     sed -i "s:@@XB_VERSION_MAJOR@@:${XB_VERSION_MAJOR}:g" storage/innobase/xtrabackup/utils/percona-xtrabackup.spec
     sed -i "s:@@XB_VERSION_MINOR@@:${XB_VERSION_MINOR}:g" storage/innobase/xtrabackup/utils/percona-xtrabackup.spec
     sed -i "s:@@XB_VERSION_PATCH@@:${XB_VERSION_PATCH}:g" storage/innobase/xtrabackup/utils/percona-xtrabackup.spec
@@ -175,7 +160,6 @@ get_sources(){
     # create a PXB tar
     cd ${WORKDIR}/percona-xtrabackup
     tar --owner=0 --group=0 --exclude=.bzr --exclude=.git -czf ${PXBDIR}.tar.gz ${PXBDIR}
-    echo "UPLOAD=UPLOAD/experimental/BUILDS/${PRODUCT}/${PRODUCT_FULL}/${BRANCH}/${REVISION}/${BUILD_ID}" >> ../percona-xtrabackup-2.4.properties
     rm -rf ${PXBDIR}
 
     mkdir $WORKDIR/source_tarball
@@ -189,26 +173,24 @@ get_sources(){
 
 get_system(){
     if [ -f /etc/redhat-release ]; then
-        GLIBC_VER_TMP="$(rpm glibc -qa --qf %{VERSION})"
         export RHEL=$(rpm --eval %rhel)
         export ARCH=$(echo $(uname -m) | sed -e 's:i686:i386:g')
         export OS_NAME="el$RHEL"
         export OS="rpm"
     else
-        GLIBC_VER_TMP="$(dpkg-query -W -f='${Version}' libc6 | awk -F'-' '{print $1}')"
         export ARCH=$(uname -m)
         export OS_NAME="$(lsb_release -sc)"
         export OS="deb"
     fi
-    export GLIBC_VER=".glibc${GLIBC_VER_TMP}"
     return
 }
 
 enable_venv(){
     if [ "$OS" == "rpm" ]; then
-        if [ "${RHEL}" -eq 7 ]; then
+        if [ "${RHEL}" -eq 8 ]; then
+            export PATH="/usr/local/bin:$PATH"
+        elif [ "${RHEL}" -eq 7 ]; then
             source /opt/rh/devtoolset-7/enable
-            source /opt/rh/rh-python36/enable
             export CMAKE_BIN="cmake3"
         elif [ "${RHEL}" -eq 6 ]; then
             source /opt/rh/devtoolset-7/enable
@@ -232,45 +214,18 @@ install_deps() {
     CURPLACE=$(pwd)
     if [ "$OS" == "rpm" ]
     then
-        yum -y install git wget yum-utils curl
-        yum install -y https://repo.percona.com/yum/percona-release-latest.noarch.rpm
-        if [ $RHEL = 9 ]; then
-            yum-config-manager --enable ol9_distro_builder
-            yum-config-manager --enable ol9_codeready_builder
-            yum -y install https://dl.fedoraproject.org/pub/epel/epel-release-latest-9.noarch.rpm
-        else
-            add_percona_yum_repo
-        fi
-        percona-release enable tools testing
-        if [ ${RHEL} = 8 -o ${RHEL} = 9 ]; then
-            PKGLIST+=" binutils-devel python3-pip python3-setuptools"
+        add_percona_yum_repo
+        yum -y install git wget 
+        if [[ "${RHEL}" -eq 8 ]]; then         
+            PKGLIST+=" binutils-devel python3-pip python3-setuptools python3-wheel"
             PKGLIST+=" libcurl-devel cmake libaio-devel zlib-devel libev-devel bison make gcc"
             PKGLIST+=" rpm-build libgcrypt-devel ncurses-devel readline-devel openssl-devel gcc-c++"
-            PKGLIST+=" vim-common rpmlint patchelf python3-wheel libudev-devel"
-            if [ $RHEL = 9 ]; then
-                PKGLIST+=" rsync python3-sphinx dnf-utils"
-            else
-                yum-config-manager --enable powertools
-#                wget https://jenkins.percona.com/downloads/rpm/procps-ng-devel-3.3.15-6.el8.x86_64.rpm
-#                yum -y install ./procps-ng-devel-3.3.15-6.el8.x86_64.rpm
-#                rm procps-ng-devel-3.3.15-6.el8.x86_64.rpm
-                PKGLIST+=" libarchive"
-            fi
+            PKGLIST+=" vim-common rpmlint"
             until yum -y install ${PKGLIST}; do
                 echo "waiting"
                 sleep 1
             done
-            if [ $RHEL = 8 ]; then
-                DEVTOOLSET10_PKGLIST+=" gcc-toolset-10-gcc-c++ gcc-toolset-10-binutils"
-                DEVTOOLSET10_PKGLIST+=" gcc-toolset-10-valgrind gcc-toolset-10-valgrind-devel gcc-toolset-10-libatomic-devel"
-                DEVTOOLSET10_PKGLIST+=" gcc-toolset-10-libasan-devel gcc-toolset-10-libubsan-devel gcc-toolset-10-annobin"
-                yum -y install centos-release-stream
-                until yum -y install ${DEVTOOLSET10_PKGLIST}; do
-                    echo "waiting"
-                    sleep 1
-                done
-                yum -y remove centos-release-stream
-            fi
+            /usr/bin/pip3 install sphinx
         else
             until yum -y install epel-release centos-release-scl; do
                 yum clean all
@@ -282,16 +237,15 @@ install_deps() {
                 sleep 1
                 echo "waiting"
             done
-            PKGLIST+=" devtoolset-7-gcc-c++ devtoolset-7-binutils libopcodes"
+            PKGLIST+=" devtoolset-7-gcc-c++ devtoolset-7-binutils"
             PKGLIST+=" devtoolset-7-libasan-devel devtoolset-7-libubsan-devel"
             PKGLIST+=" devtoolset-7-valgrind devtoolset-7-valgrind-devel"
             PKGLIST+=" wget libcurl-devel cmake cmake3 make gcc gcc-c++ libev-devel openssl-devel rpm-build"
             PKGLIST+=" libaio-devel perl-DBD-MySQL vim-common ncurses-devel readline-devel readline"
-            PKGLIST+=" zlib-devel libgcrypt-devel bison patchelf"
-            PKGLIST+=" socat numactli libudev-devel libicu-devel"
-          #  PKGLIST+=" procps-ng-devel"
+            PKGLIST+=" zlib-devel libgcrypt-devel bison"
+            PKGLIST+=" socat numactl"
             if [[ "${RHEL}" -eq 7 ]]; then
-                PKGLIST+=" numactl-libs libopcodes perl-Digest-MD5  python3-pip python3-setuptools python3-wheel rh-python36-python-sphinx"
+                PKGLIST+=" numactl-libs perl-Digest-MD5  python3-pip python3-setuptools python3-wheel"
             elif [[ "${RHEL}" -eq 6 ]]; then
                 PKGLIST+=" rh-python36-python-pip rh-python36-docutils rh-python36-python-setuptools"
             fi
@@ -299,10 +253,9 @@ install_deps() {
                 echo "waiting"
                 sleep 1
             done
+            # Workaround for Centos > 7 Sphinx
             if [[ "${RHEL}" -eq 7 ]]; then
-                yum -y --enablerepo=centos-sclo-rh-testing install devtoolset-10-gcc-c++ devtoolset-10-binutils devtoolset-10-valgrind devtoolset-10-valgrind-devel devtoolset-10-libatomic-devel
-                yum -y --enablerepo=centos-sclo-rh-testing install devtoolset-10-libasan-devel devtoolset-10-libubsan-devel
-                yum -y update nss
+                /usr/bin/pip3 install sphinx
             elif [[ "${RHEL}" -eq 6 ]]; then
                 source /opt/rh/rh-python36/enable
                 pip install sphinx
@@ -310,36 +263,14 @@ install_deps() {
         fi
     else
         apt-get update
-        DEBIAN_FRONTEND=noninteractive apt-get -y install lsb-release gnupg git wget curl
-        export OS_NAME="$(lsb_release -sc)"
-        wget https://repo.percona.com/apt/percona-release_latest.$(lsb_release -sc)_all.deb && dpkg -i percona-release_latest.$(lsb_release -sc)_all.deb
-        percona-release enable tools testing
-        apt-get update
+        DEBIAN_FRONTEND=noninteractive apt-get -y install lsb-release gnupg git wget
 
         PKGLIST+=" bison cmake devscripts debconf debhelper automake bison ca-certificates libcurl4-openssl-dev"
-        PKGLIST+=" cmake debhelper libaio-dev libncurses-dev libtool libz-dev libsasl2-dev vim-common"
-        PKGLIST+=" libgcrypt-dev libev-dev lsb-release libudev-dev"
-        PKGLIST+=" build-essential rsync libdbd-mysql-perl libnuma1 socat libssl-dev patchelf libicu-dev"
-       # PKGLIST+=" libprocps-dev"
+        PKGLIST+=" cmake debhelper libaio-dev libncurses-dev libtool libz-dev"
+        PKGLIST+=" libgcrypt-dev libev-dev lsb-release"
+        PKGLIST+=" build-essential rsync libdbd-mysql-perl libnuma1 socat libssl-dev"
 
-        if [ "${OS_NAME}" == "bionic" ]; then
-            PKGLIST+=" gcc-8 g++-8"
-	fi
-        DEBIAN_FRONTEND=noninteractive apt-get -y install libreadline6 || true
-        if [ $DEBIAN_VERSION = bullseye -o $DEBIAN_VERSION = bookworm ]; then
-            apt install -y gcc-9 g++-9 cmake
-            ln -s -f /usr/bin/g++-9 /usr/bin/g++
-            ln -s -f /usr/bin/gcc-9 /usr/bin/gcc
-            ln -s -f /usr/bin/gcc-ar-9 /usr/bin/gcc-ar
-            ln -s -f /usr/bin/gcc-nm-9 /usr/bin/gcc-nm
-            ln -s -f /usr/bin/gcc-ranlib-9 /usr/bin/gcc-ranlib
-            ln -s -f /usr/bin/x86_64-linux-gnu-g++-9 /usr/bin/x86_64-linux-gnu-g++
-            ln -s -f /usr/bin/x86_64-linux-gnu-gcc-9 /usr/bin/x86_64-linux-gnu-gcc
-            ln -s -f /usr/bin/x86_64-linux-gnu-gcc-ar-9 /usr/bin/x86_64-linux-gnu-gcc-ar
-            ln -s -f /usr/bin/x86_64-linux-gnu-gcc-nm-9 /usr/bin/x86_64-linux-gnu-gcc-nm
-            ln -s -f /usr/bin/x86_64-linux-gnu-gcc-ranlib-9 /usr/bin/x86_64-linux-gnu-gcc-ranlib
-	fi
-        if [ "${OS_NAME}" == "focal" -o "${OS_NAME}" == "bullseye" -o "${OS_NAME}" == "jammy" -o "${OS_NAME}" == "bookworm" ]; then
+        if [ "${OS_NAME}" == "focal" ]; then
             PKGLIST+=" python3-sphinx python3-docutils"
         else
             PKGLIST+=" python-sphinx python-docutils"
@@ -414,8 +345,7 @@ build_srpm(){
 
     TARFILE=$(basename $(find . -name 'percona-xtrabackup-*.tar.gz' | sort | tail -n1))
     NAME=$(echo ${TARFILE}| awk -F '-' '{print $1"-"$2}')
-    #VERSION=$(echo ${TARFILE}| awk -F '-' '{print $3}')
-    VERSION=$(echo ${TARFILE}| awk -F '-' '{print $3}'|awk -F '.' '{print $1"."$2"."$3}')
+    VERSION=$(echo ${TARFILE}| awk -F '-' '{print $3}')
     #
     SHORTVER=$(echo ${VERSION} | awk -F '.' '{print $1"."$2}')
     TMPREL=$(echo ${TARFILE}| awk -F '-' '{print $4}')
@@ -474,8 +404,8 @@ build_rpm(){
     mkdir -vp rpmbuild/{SOURCES,SPECS,BUILD,SRPMS,RPMS}
     cp $SRC_RPM rpmbuild/SRPMS/
     #
-    echo "RHEL=${RHEL}" >> ${CURDIR}/percona-xtrabackup-2.4.properties
-    echo "ARCH=${ARCH}" >> ${CURDIR}/percona-xtrabackup-2.4.properties
+    echo "RHEL=${RHEL}" >> ${CURDIR}/percona-xtrabackup-8.0.properties
+    echo "ARCH=${ARCH}" >> ${CURDIR}/percona-xtrabackup-8.0.properties
     #
     SRCRPM=$(basename $(find . -name '*.src.rpm' | sort | tail -n1))
 
@@ -509,13 +439,12 @@ build_source_deb(){
 
     TARFILE=$(basename $(find . -name 'percona-xtrabackup-*.tar.gz' | sort | tail -n1))
     NAME=$(echo ${TARFILE}| awk -F '-' '{print $1"-"$2}')
-    #VERSION=$(echo ${TARFILE%.tar.gz} | awk -F '-' '{print $3"-"$4}')
-    VERSION=$(echo ${TARFILE}| awk -F '-' '{print $3}'|awk -F '.' '{print $1"."$2"."$3}')
+    VERSION=$(echo ${TARFILE%.tar.gz} | awk -F '-' '{print $3}')
     SHORTVER=$(echo ${VERSION} | awk -F '.' '{print $1"."$2}')
 
-    echo "DEB_RELEASE=${DEB_RELEASE}" >> ${CURDIR}/percona-xtrabackup-2.4.properties
+    echo "DEB_RELEASE=${DEB_RELEASE}" >> ${CURDIR}/percona-xtrabackup-8.0.properties
 
-    NEWTAR=${NAME}-24_${VERSION}.orig.tar.gz
+    NEWTAR=${NAME}-80_${VERSION}.orig.tar.gz
     mv ${TARFILE} ${NEWTAR}
 
     tar xzf ${NEWTAR}
@@ -562,9 +491,9 @@ build_deb(){
     DIRNAME=$(echo $DSC | sed -e 's:_:-:g' | awk -F'-' '{print $1"-"$2"-"$3"-"$4}')
     VERSION=$(echo $DSC | sed -e 's:_:-:g' | awk -F'-' '{print $4}')
     #
-    echo "DEB_RELEASE=${DEB_RELEASE}" >> ${CURDIR}/percona-xtrabackup-2.4.properties
-    echo "DEBIAN_VERSION=${OS_NAME}" >> ${CURDIR}/percona-xtrabackup-2.4.properties
-    echo "ARCH=${ARCH}" >> ${CURDIR}/percona-xtrabackup-2.4.properties
+    echo "DEB_RELEASE=${DEB_RELEASE}" >> ${CURDIR}/percona-xtrabackup-8.0.properties
+    echo "DEBIAN_VERSION=${OS_NAME}" >> ${CURDIR}/percona-xtrabackup-8.0.properties
+    echo "ARCH=${ARCH}" >> ${CURDIR}/percona-xtrabackup-8.0.properties
 
     dpkg-source -x $DSC
     cd $DIRNAME
@@ -589,18 +518,26 @@ build_tarball(){
     TARFILE=$(basename $(find . -name 'percona-xtrabackup-*.tar.gz' | sort | tail -n1))
     enable_venv
     #
-    NAME=$(echo ${TARFILE%.tar.gz}| awk -F '-' '{print $1"-"$2}')
-    VERSION=$(echo ${TARFILE%.tar.gz}| awk -F '-' '{print $3"-"$4}')
+    NAME=$(echo ${TARFILE}| awk -F '-' '{print $1"-"$2}')
+    VERSION=$(echo ${TARFILE}| awk -F '-' '{print $3}')
     #
     SHORTVER=$(echo ${VERSION} | awk -F '.' '{print $1"."$2}')
-    TMPREL=$(echo ${TARFILE%.tar.gz}| awk -F '-' '{print $5}')
+    TMPREL=$(echo ${TARFILE}| awk -F '-' '{print $4}')
     RELEASE=${TMPREL%.tar.gz}
+    if [ "$OS" == "rpm" ]; then
+        GCRYPT_VER_TMP=$(yum list installed|grep -i libgcrypt | grep -v dev|head -n1|awk '{print $2}'|awk -F "-" '{print $1}'|sed 's/\.//g'|sed 's/[a-z]$//')
+        export GCRYPT_VER=".libgcrypt${GCRYPT_VER_TMP}"
+    elif [ "$OS" == "deb" ]; then
+        GCRYPT_VER_TMP=$(dpkg -l|grep -i libgcrypt | grep -v dev | awk '{print $2}' | awk -F':' '{print $1}')
+        export GCRYPT_VER=".${GCRYPT_VER_TMP}"
+    fi
 
     rm -fr TARGET && mkdir TARGET
     rm -fr ${TARFILE%.tar.gz}
     tar xzf ${TARFILE}
     cd ${TARFILE%.tar.gz}
     #
+    sed -i 's|.tar.gz|.${OS_NAME}${GCRYPT_VER}.tar.gz|' ./storage/innobase/xtrabackup/utils/build-binary.sh
     bash -x ./storage/innobase/xtrabackup/utils/build-binary.sh ${WORKDIR}/TARGET
 
     mkdir -p ${WORKDIR}/tarball
@@ -610,7 +547,7 @@ build_tarball(){
 }
 
 CURDIR=$(pwd)
-VERSION_FILE=$CURDIR/percona-xtrabackup-2.4.properties
+VERSION_FILE=$CURDIR/percona-xtrabackup-8.0.properties
 args=
 WORKDIR=
 SRPM=0
@@ -623,11 +560,11 @@ OS_NAME=
 ARCH=
 OS=
 REVISION=0
-BRANCH="2.4"
+BRANCH="8.0"
 INSTALL=0
 RPM_RELEASE=1
 DEB_RELEASE=1
-REPO="https://github.com/percona/percona-xtrabackup.git"
+REPO="git://github.com/percona/percona-xtrabackup.git"
 CMAKE_BIN="cmake"
 parse_arguments PICK-ARGS-FROM-ARGV "$@"
 
