@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2000, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2000, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -28,11 +28,11 @@
 #include <limits.h>
 #include <stdarg.h>
 #include <algorithm>
-#include <bit>
 #include <new>
 
 #include "lex_string.h"
-#include "m_string.h"
+#include "m_ctype.h"
+#include "my_bit.h"
 #include "my_compiler.h"
 #include "my_dbug.h"
 #include "my_io.h"
@@ -40,9 +40,6 @@
 #include "myisam.h"
 #include "myisampack.h"
 #include "mysql/plugin.h"
-#include "mysql/strings/int2str.h"
-#include "mysql/strings/m_ctype.h"
-#include "nulls.h"
 #include "sql/current_thd.h"
 #include "sql/derror.h"
 #include "sql/field.h"
@@ -57,7 +54,6 @@
 #include "sql/system_variables.h"
 #include "storage/myisam/myisamdef.h"
 #include "storage/myisam/rt_index.h"
-#include "strxmov.h"
 
 #include "mysql/components/services/log_builtins.h"
 
@@ -68,7 +64,7 @@ ulonglong myisam_recover_options;
 static ulong opt_myisam_block_size;
 
 /* Interface to mysqld, to check system tables supported by SE */
-static bool myisam_is_supported_system_table(const char *db,
+static bool myisam_is_supported_system_table(const char *,
                                              const char *table_name,
                                              bool is_sql_layer_system_table);
 
@@ -110,6 +106,13 @@ static MYSQL_SYSVAR_SET(
     "DEFAULT, BACKUP, FORCE, QUICK, or OFF",
     nullptr, nullptr, 0, &myisam_recover_typelib);
 
+static MYSQL_THDVAR_ULONG(
+    repair_threads, PLUGIN_VAR_RQCMDARG,
+    "If larger than 1, when repairing a MyISAM table all indexes will be "
+    "created in parallel, with one thread per index. The value of 1 "
+    "disables parallel repair",
+    nullptr, nullptr, 1, 1, ULONG_MAX, 1);
+
 static MYSQL_THDVAR_ULONGLONG(
     sort_buffer_size, PLUGIN_VAR_RQCMDARG,
     "The buffer that is allocated when sorting the index when doing "
@@ -138,7 +141,7 @@ static MYSQL_THDVAR_ENUM(
     nullptr, nullptr, MI_STATS_METHOD_NULLS_NOT_EQUAL,
     &myisam_stats_method_typelib);
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
 /**
   Causes the thread to wait in a spin lock for a query kill signal.
   This function is used by the test frame work to identify race conditions.
@@ -202,7 +205,12 @@ static void mi_check_print_msg(MI_CHECK *param, const char *msg_type,
     TODO: switch from protocol to push_warning here. The main reason we didn't
     it yet is parallel repair. Due to following trace:
     mi_check_print_msg/push_warning/sql_alloc/my_pthread_getspecific_ptr.
+
+    Also we likely need to lock mutex here (in both cases with protocol and
+    push_warning).
   */
+  if (param->need_print_msg_lock) mysql_mutex_lock(&param->print_msg_mutex);
+
   protocol->start_row();
   protocol->store_string(name, length, system_charset_info);
   protocol->store(param->op_name, system_charset_info);
@@ -211,6 +219,8 @@ static void mi_check_print_msg(MI_CHECK *param, const char *msg_type,
   if (protocol->end_row())
     LogErr(ERROR_LEVEL, ER_MY_NET_WRITE_FAILED_FALLING_BACK_ON_STDERR, msgbuf);
 
+  if (param->need_print_msg_lock) mysql_mutex_unlock(&param->print_msg_mutex);
+
   return;
 }
 
@@ -262,10 +272,11 @@ int table2myisam(TABLE *table_arg, MI_KEYDEF **keydef_out,
   for (i = 0; i < share->keys; i++, pos++) {
     keydef[i].flag =
         ((uint16)pos->flags & (HA_NOSAME | HA_FULLTEXT | HA_SPATIAL));
-    assert(pos->algorithm != HA_KEY_ALG_SE_SPECIFIC);
-    assert(!(pos->flags & HA_SPATIAL) || pos->algorithm == HA_KEY_ALG_RTREE);
-    assert(!(pos->flags & HA_FULLTEXT) ||
-           pos->algorithm == HA_KEY_ALG_FULLTEXT);
+    DBUG_ASSERT(pos->algorithm != HA_KEY_ALG_SE_SPECIFIC);
+    DBUG_ASSERT(!(pos->flags & HA_SPATIAL) ||
+                pos->algorithm == HA_KEY_ALG_RTREE);
+    DBUG_ASSERT(!(pos->flags & HA_FULLTEXT) ||
+                pos->algorithm == HA_KEY_ALG_FULLTEXT);
     keydef[i].key_alg = pos->algorithm;
     keydef[i].block_length = pos->block_size;
     keydef[i].seg = keyseg;
@@ -283,7 +294,7 @@ int table2myisam(TABLE *table_arg, MI_KEYDEF **keydef_out,
              (type == HA_KEYTYPE_BINARY && !field->zero_pack()))) {
           /* No blobs here */
           if (j == 0) keydef[i].flag |= HA_PACK_KEY;
-          if (!field->is_flag_set(ZEROFILL_FLAG) &&
+          if (!(field->flags & ZEROFILL_FLAG) &&
               (field->type() == MYSQL_TYPE_STRING ||
                field->type() == MYSQL_TYPE_VAR_STRING ||
                ((int)(pos->key_part[j].length - field->decimals())) >= 4))
@@ -355,7 +366,7 @@ int table2myisam(TABLE *table_arg, MI_KEYDEF **keydef_out,
     }
     if (!found) break;
 
-    if (found->is_flag_set(BLOB_FLAG))
+    if (found->flags & BLOB_FLAG)
       recinfo_pos->type = (int)FIELD_BLOB;
     else if (found->type() == MYSQL_TYPE_VARCHAR)
       recinfo_pos->type = FIELD_VARCHAR;
@@ -365,7 +376,7 @@ int table2myisam(TABLE *table_arg, MI_KEYDEF **keydef_out,
       recinfo_pos->type = (int)FIELD_SKIP_ZERO;
     else
       recinfo_pos->type =
-          (int)((length <= 3 || (found->is_flag_set(ZEROFILL_FLAG)))
+          (int)((length <= 3 || (found->flags & ZEROFILL_FLAG))
                     ? FIELD_NORMAL
                     : found->type() == MYSQL_TYPE_STRING ||
                               found->type() == MYSQL_TYPE_VAR_STRING
@@ -428,7 +439,7 @@ int table2myisam(TABLE *table_arg, MI_KEYDEF **keydef_out,
     - compare FULLTEXT keys;
     - compare SPATIAL keys;
     - compare FIELD_SKIP_ZERO which is converted to FIELD_NORMAL correctly
-      (should be correctly detected in table2myisam).
+      (should be corretly detected in table2myisam).
 */
 
 int check_definition(MI_KEYDEF *t1_keyinfo, MI_COLUMNDEF *t1_recinfo,
@@ -648,52 +659,66 @@ static const char *ha_myisam_exts[] = {".MYI", ".MYD", NullS};
 /**
   @brief Check if the given db.tablename is a system table for this SE.
 
-  @param db                         database name.
   @param table_name                 table name to check.
   @param is_sql_layer_system_table  if the supplied db.table_name is a SQL
                                     layer system table.
 
-  @note As for 8.0, MySQL doesn't support MyISAM as an engine for the system
-        tables. Altering engine of system table to MyISAM is not allowed.
-        Creating system tables in MyISAM is allowed to upgrade from older
-        versions through mysqldump.
+  @note As for 5.7, mysql doesn't support MyISAM as an engine for the following
+        system tables: columns_priv, db, procs_priv, proxies_priv, tables_priv,
+        user.
 
-  @note In case there is a need to define MYISAM specific system database, then
-        please see reference implementation in ha_example.cc.
+  @note In case there is a need to define MYISAM specific system
+        database, then please see reference implementation in
+        ha_example.cc.
 
   @retval true   Given db.table_name is supported system table.
   @retval false  Given db.table_name is not a supported system table.
 */
 
-static bool myisam_is_supported_system_table(const char *db,
+static bool myisam_is_supported_system_table(const char *,
                                              const char *table_name,
                                              bool is_sql_layer_system_table) {
-  /*
-    Currently MYISAM does not support any other SE specific system tables. If
-    in future it does, please see ha_example.cc for reference implementation
-  */
-  if (!is_sql_layer_system_table) return false;
-
-  /*
-    During upgrade, server side execution of SQL statement is allowed to use
-    this SE for system tables.
-  */
   THD *thd = current_thd;
-  if (thd->is_server_upgrade_thread()) return true;
 
-  /*
-    For client side SQL, creating system tables in this SE is allowed to support
-    logical upgrade.
-  */
-  if (thd->lex->sql_command == SQLCOM_CREATE_TABLE) {
-    push_warning_printf(thd, Sql_condition::SL_WARNING, ER_UNSUPPORTED_ENGINE,
-                        ER_THD(thd, ER_UNSUPPORTED_ENGINE), "MyISAM", db,
-                        table_name);
-    return true;
+  if (thd->lex->sql_command == SQLCOM_CREATE_TABLE ||
+      thd->lex->sql_command == SQLCOM_ALTER_TABLE) {
+    /*
+      We allow creation of ACL tables in MyISAM to allow upgrade from
+      older versions through mysqldump and downgrade.
+    */
+    // Does MYISAM support "ALL" SQL layer system tables ?
+    if (is_sql_layer_system_table) return true;
+
+    /*
+      Currently MYISAM does not support any other SE specific
+      system tables. If in future it does, please see ha_example.cc
+      for reference implementation
+    */
+
+    return false;
+  } else {
+    static const char *unsupported_system_tables[] = {
+        "columns_priv", "db",   "procs_priv",         "proxies_priv",
+        "tables_priv",  "user", (const char *)nullptr};
+
+    if (is_sql_layer_system_table) {
+      for (unsigned i = 0; unsupported_system_tables[i] != nullptr; ++i) {
+        if (!strcmp(table_name, unsupported_system_tables[i]))
+          // Doesn't support MYISAM for this table name
+          return false;
+      }
+      // Support MYISAM for other system tables not listed explicitly
+      return true;
+    }
+
+    /*
+      Currently MYISAM does not support any other SE specific
+      system tables. If in future it does, please see ha_example.cc
+      for reference implementation
+    */
+
+    return false;
   }
-
-  // Any other usage is not allowed.
-  return false;
 }
 
 /* Name is here without an extension */
@@ -834,7 +859,7 @@ int ha_myisam::check(THD *thd, HA_CHECK_OPT *check_opt) {
   int error;
   MI_CHECK param;
   MYISAM_SHARE *share = file->s;
-  const char *old_proc_info = thd->proc_info();
+  const char *old_proc_info = thd->proc_info;
 
   thd_proc_info(thd, "Checking table");
   myisamchk_init(&param);
@@ -1004,7 +1029,7 @@ int ha_myisam::repair(THD *thd, MI_CHECK &param, bool do_optimize) {
   uint local_testflag = param.testflag;
   bool optimize_done = !do_optimize, statistics_done = false;
   bool has_old_locks = thd->locked_tables_mode || file->lock_type != F_UNLCK;
-  const char *old_proc_info = thd->proc_info();
+  const char *old_proc_info = thd->proc_info;
   char fixed_name[FN_REFLEN];
   MYISAM_SHARE *share = file->s;
   ha_rows rows = file->state->records;
@@ -1049,13 +1074,28 @@ int ha_myisam::repair(THD *thd, MI_CHECK &param, bool do_optimize) {
       local_testflag |= T_STATISTICS;
       param.testflag |= T_STATISTICS;  // We get this for free
       statistics_done = true;
-      thd_proc_info(thd, "Repair by sorting");
-      /*
-        The new file is created with the right stats, so we can skip
-        copying file stats from old to new.
-      */
-      error = mi_repair_by_sort(&param, file, fixed_name,
-                                param.testflag & T_QUICK, true);
+      if (THDVAR(thd, repair_threads) > 1) {
+        char buf[40];
+        /* TODO: respect myisam_repair_threads variable */
+        snprintf(buf, 40, "Repair with %d threads", my_count_bits(key_map));
+        thd_proc_info(thd, buf);
+        /*
+          The new file is created with the right stats, so we can skip
+          copying file stats from old to new.
+        */
+        error = mi_repair_parallel(&param, file, fixed_name,
+                                   param.testflag & T_QUICK, true);
+        thd_proc_info(thd, "Repair done");  // to reset proc_info, as
+                                            // it was pointing to local buffer
+      } else {
+        thd_proc_info(thd, "Repair by sorting");
+        /*
+          The new file is created with the right stats, so we can skip
+          copying file stats from old to new.
+        */
+        error = mi_repair_by_sort(&param, file, fixed_name,
+                                  param.testflag & T_QUICK, true);
+      }
     } else {
       thd_proc_info(thd, "Repair with keycache");
       param.testflag &= ~T_REP_BY_SORT;
@@ -1137,9 +1177,10 @@ int ha_myisam::repair(THD *thd, MI_CHECK &param, bool do_optimize) {
 
 int ha_myisam::assign_to_keycache(THD *thd, HA_CHECK_OPT *check_opt) {
   KEY_CACHE *new_key_cache = check_opt->key_cache;
+  const char *errmsg = nullptr;
   int error = HA_ADMIN_OK;
   ulonglong map;
-  Table_ref *table_list = table->pos_in_table_list;
+  TABLE_LIST *table_list = table->pos_in_table_list;
   DBUG_TRACE;
 
   table->keys_in_use_for_query.clear_all();
@@ -1150,10 +1191,11 @@ int ha_myisam::assign_to_keycache(THD *thd, HA_CHECK_OPT *check_opt) {
     /* use all keys if there's no list specified by the user through hints */
     map = table->keys_in_use_for_query.to_ulonglong();
 
-  char errmsg[STRING_BUFFER_USUAL_SIZE];
   if ((error = mi_assign_to_key_cache(file, map, new_key_cache))) {
-    snprintf(errmsg, sizeof(errmsg),
-             "Failed to flush to index file (errno: %d)", error);
+    char buf[STRING_BUFFER_USUAL_SIZE];
+    snprintf(buf, sizeof(buf), "Failed to flush to index file (errno: %d)",
+             error);
+    errmsg = buf;
     error = HA_ADMIN_CORRUPT;
   }
 
@@ -1179,7 +1221,7 @@ int ha_myisam::preload_keys(THD *thd, HA_CHECK_OPT *) {
   int error;
   const char *errmsg;
   ulonglong map;
-  Table_ref *table_list = table->pos_in_table_list;
+  TABLE_LIST *table_list = table->pos_in_table_list;
   bool ignore_leaves = table_list->ignore_leaves;
   char buf[MYSQL_ERRMSG_SIZE];
 
@@ -1316,7 +1358,7 @@ int ha_myisam::enable_indexes(uint mode) {
   } else if (mode == HA_KEY_SWITCH_NONUNIQ_SAVE) {
     THD *thd = current_thd;
     MI_CHECK param;
-    const char *save_proc_info = thd->proc_info();
+    const char *save_proc_info = thd->proc_info;
     thd_proc_info(thd, "Creating index");
     myisamchk_init(&param);
     param.op_name = "recreating_index";
@@ -1519,12 +1561,8 @@ int ha_myisam::rnd_end() {
 int ha_myisam::index_read_map(uchar *buf, const uchar *key,
                               key_part_map keypart_map,
                               enum ha_rkey_function find_flag) {
-  assert(inited == INDEX);
+  DBUG_ASSERT(inited == INDEX);
   ha_statistic_increment(&System_status_var::ha_read_key_count);
-  if (file->s->keyinfo[active_index].flag & HA_FULLTEXT) {
-    set_my_errno(HA_ERR_KEY_NOT_FOUND);
-    return HA_ERR_KEY_NOT_FOUND;
-  }
   int error = mi_rkey(file, buf, active_index, key, keypart_map, find_flag);
   return error;
 }
@@ -1532,13 +1570,9 @@ int ha_myisam::index_read_map(uchar *buf, const uchar *key,
 int ha_myisam::index_read_idx_map(uchar *buf, uint index, const uchar *key,
                                   key_part_map keypart_map,
                                   enum ha_rkey_function find_flag) {
-  assert(pushed_idx_cond == nullptr);
-  assert(pushed_idx_cond_keyno == MAX_KEY);
+  DBUG_ASSERT(pushed_idx_cond == nullptr);
+  DBUG_ASSERT(pushed_idx_cond_keyno == MAX_KEY);
   ha_statistic_increment(&System_status_var::ha_read_key_count);
-  if (file->s->keyinfo[active_index].flag & HA_FULLTEXT) {
-    set_my_errno(HA_ERR_KEY_NOT_FOUND);
-    return HA_ERR_KEY_NOT_FOUND;
-  }
   int error = mi_rkey(file, buf, index, key, keypart_map, find_flag);
   return error;
 }
@@ -1546,49 +1580,46 @@ int ha_myisam::index_read_idx_map(uchar *buf, uint index, const uchar *key,
 int ha_myisam::index_read_last_map(uchar *buf, const uchar *key,
                                    key_part_map keypart_map) {
   DBUG_TRACE;
-  assert(inited == INDEX);
+  DBUG_ASSERT(inited == INDEX);
   ha_statistic_increment(&System_status_var::ha_read_key_count);
-  if (file->s->keyinfo[active_index].flag & HA_FULLTEXT) {
-    set_my_errno(HA_ERR_KEY_NOT_FOUND);
-    return HA_ERR_KEY_NOT_FOUND;
-  }
   int error =
       mi_rkey(file, buf, active_index, key, keypart_map, HA_READ_PREFIX_LAST);
   return error;
 }
 
 int ha_myisam::index_next(uchar *buf) {
-  assert(inited == INDEX);
+  DBUG_ASSERT(inited == INDEX);
   ha_statistic_increment(&System_status_var::ha_read_next_count);
   int error = mi_rnext(file, buf, active_index);
   return error;
 }
 
 int ha_myisam::index_prev(uchar *buf) {
-  assert(inited == INDEX);
+  DBUG_ASSERT(inited == INDEX);
   ha_statistic_increment(&System_status_var::ha_read_prev_count);
   int error = mi_rprev(file, buf, active_index);
   return error;
 }
 
 int ha_myisam::index_first(uchar *buf) {
-  assert(inited == INDEX);
+  DBUG_ASSERT(inited == INDEX);
   ha_statistic_increment(&System_status_var::ha_read_first_count);
   int error = mi_rfirst(file, buf, active_index);
   return error;
 }
 
 int ha_myisam::index_last(uchar *buf) {
-  assert(inited == INDEX);
+  DBUG_ASSERT(inited == INDEX);
   ha_statistic_increment(&System_status_var::ha_read_last_count);
   int error = mi_rlast(file, buf, active_index);
   return error;
 }
 
-int ha_myisam::index_next_same(uchar *buf, const uchar *key [[maybe_unused]],
-                               uint length [[maybe_unused]]) {
+int ha_myisam::index_next_same(uchar *buf,
+                               const uchar *key MY_ATTRIBUTE((unused)),
+                               uint length MY_ATTRIBUTE((unused))) {
   int error;
-  assert(inited == INDEX);
+  DBUG_ASSERT(inited == INDEX);
   ha_statistic_increment(&System_status_var::ha_read_next_count);
   do {
     error = mi_rnext_same(file, buf);
@@ -1653,7 +1684,7 @@ int ha_myisam::info(uint flag) {
 
     /*
       Update share.
-      lock_shared_ha_data is slightly abused here, since there is no other
+      lock_shared_ha_data is slighly abused here, since there is no other
       way of locking the TABLE_SHARE.
     */
     lock_shared_ha_data();
@@ -1697,8 +1728,8 @@ int ha_myisam::extra(enum ha_extra_function operation) {
 
 int ha_myisam::reset(void) {
   /* Reset MyISAM specific part for index condition pushdown */
-  assert(pushed_idx_cond == nullptr);
-  assert(pushed_idx_cond_keyno == MAX_KEY);
+  DBUG_ASSERT(pushed_idx_cond == nullptr);
+  DBUG_ASSERT(pushed_idx_cond_keyno == MAX_KEY);
   mi_set_index_cond_func(file, nullptr, nullptr);
   ds_mrr.reset();
   return mi_reset(file);
@@ -1980,7 +2011,7 @@ static int myisam_init(void *p) {
   else
     myisam_recover_options = HA_RECOVER_OFF;
 
-  myisam_block_size = std::bit_floor(opt_myisam_block_size);
+  myisam_block_size = (uint)1 << my_bit_log2(opt_myisam_block_size);
 
   myisam_hton = (handlerton *)p;
   myisam_hton->state = SHOW_OPTION_YES;
@@ -2074,15 +2105,12 @@ Item *ha_myisam::idx_cond_push(uint keyno_arg, Item *idx_cond_arg) {
   return nullptr;
 }
 
-static SYS_VAR *myisam_sysvars[] = {MYSQL_SYSVAR(block_size),
-                                    MYSQL_SYSVAR(data_pointer_size),
-                                    MYSQL_SYSVAR(max_sort_file_size),
-                                    MYSQL_SYSVAR(recover_options),
-                                    MYSQL_SYSVAR(sort_buffer_size),
-                                    MYSQL_SYSVAR(use_mmap),
-                                    MYSQL_SYSVAR(mmap_size),
-                                    MYSQL_SYSVAR(stats_method),
-                                    nullptr};
+static SYS_VAR *myisam_sysvars[] = {
+    MYSQL_SYSVAR(block_size),         MYSQL_SYSVAR(data_pointer_size),
+    MYSQL_SYSVAR(max_sort_file_size), MYSQL_SYSVAR(recover_options),
+    MYSQL_SYSVAR(repair_threads),     MYSQL_SYSVAR(sort_buffer_size),
+    MYSQL_SYSVAR(use_mmap),           MYSQL_SYSVAR(mmap_size),
+    MYSQL_SYSVAR(stats_method),       nullptr};
 
 struct st_mysql_storage_engine myisam_storage_engine = {
     MYSQL_HANDLERTON_INTERFACE_VERSION};
