@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2000, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -28,8 +28,8 @@
 #include <time.h>
 
 #include <algorithm>
-#include <bit>
 
+#include "my_bit.h"
 #include "my_byteorder.h"
 #include "my_dbug.h"
 #include "my_inttypes.h"
@@ -45,7 +45,7 @@
 #include <fcntl.h>
 #include <process.h>
 #endif
-#include "mysql/strings/m_ctype.h"
+#include "m_ctype.h"
 
 /*
   Old options is used when recreating database, from myisamchk
@@ -309,7 +309,7 @@ int mi_create(const char *name, uint keys, MI_KEYDEF *keydefs, uint columns,
           keydef->flag &= ~HA_PACK_KEY;
           keydef->flag |= HA_BINARY_PACK_KEY | HA_VAR_LENGTH_KEY;
         } else {
-          keydef->seg[0].flag |= HA_PACK_KEY; /* for easier intern test */
+          keydef->seg[0].flag |= HA_PACK_KEY; /* for easyer intern test */
           keydef->flag |= HA_VAR_LENGTH_KEY;
           options |= HA_OPTION_PACK_KEYS; /* Using packed keys */
         }
@@ -353,7 +353,7 @@ int mi_create(const char *name, uint keys, MI_KEYDEF *keydefs, uint columns,
             break;
         }
         if (keyseg->flag & HA_SPACE_PACK) {
-          assert(!(keyseg->flag & HA_VAR_LENGTH_PART));
+          DBUG_ASSERT(!(keyseg->flag & HA_VAR_LENGTH_PART));
           keydef->flag |= HA_SPACE_PACK_USED | HA_VAR_LENGTH_KEY;
           options |= HA_OPTION_PACK_KEYS; /* Using packed keys */
           length++;                       /* At least one length byte */
@@ -364,8 +364,8 @@ int mi_create(const char *name, uint keys, MI_KEYDEF *keydefs, uint columns,
           }
         }
         if (keyseg->flag & (HA_VAR_LENGTH_PART | HA_BLOB_PART)) {
-          assert(!test_all_bits(keyseg->flag,
-                                (HA_VAR_LENGTH_PART | HA_BLOB_PART)));
+          DBUG_ASSERT(!test_all_bits(keyseg->flag,
+                                     (HA_VAR_LENGTH_PART | HA_BLOB_PART)));
           keydef->flag |= HA_VAR_LENGTH_KEY;
           length++;                       /* At least one length byte */
           options |= HA_OPTION_PACK_KEYS; /* Using packed keys */
@@ -399,8 +399,9 @@ int mi_create(const char *name, uint keys, MI_KEYDEF *keydefs, uint columns,
       share.state.rec_per_key_part[key_segs - 1] = 1L;
     length += key_length;
     /* Get block length for key, if defined by user */
-    block_length = keydef->block_length ? std::bit_ceil(keydef->block_length)
-                                        : myisam_block_size;
+    block_length =
+        (keydef->block_length ? my_round_up_to_next_power(keydef->block_length)
+                              : myisam_block_size);
     block_length = std::max(block_length, MI_MIN_KEY_BLOCK_LENGTH);
     block_length = std::min(block_length, MI_MAX_KEY_BLOCK_LENGTH);
 
@@ -498,8 +499,8 @@ int mi_create(const char *name, uint keys, MI_KEYDEF *keydefs, uint columns,
   mi_int2store(share.state.header.unique_key_parts, unique_key_parts);
 
   mi_set_all_keys_active(share.state.key_map, keys);
-  aligned_key_start = std::bit_ceil(max_key_block_length ? max_key_block_length
-                                                         : myisam_block_size);
+  aligned_key_start = my_round_up_to_next_power(
+      max_key_block_length ? max_key_block_length : myisam_block_size);
 
   share.base.keystart = share.state.state.key_file_length =
       MY_ALIGN(info_length, aligned_key_start);
@@ -539,7 +540,7 @@ int mi_create(const char *name, uint keys, MI_KEYDEF *keydefs, uint columns,
     int have_iext = iext && !strcmp(iext, MI_NAME_IEXT);
     if (options & HA_OPTION_TMP_TABLE) {
       char *path;
-      /* chop off the table name, temporary tables use generated name */
+      /* chop off the table name, tempory tables use generated name */
       if ((path = strrchr(const_cast<char *>(ci->index_file_name), FN_LIBCHAR)))
         *path = '\0';
       fn_format(filename, name, ci->index_file_name, MI_NAME_IEXT,
@@ -555,7 +556,7 @@ int mi_create(const char *name, uint keys, MI_KEYDEF *keydefs, uint columns,
     linkname_ptr = linkname;
     /*
       Don't create the table if the link or file exists to ensure that one
-      doesn't accidentally destroy another table.
+      doesn't accidently destroy another table.
     */
     create_flag = 0;
   } else {
@@ -602,7 +603,7 @@ int mi_create(const char *name, uint keys, MI_KEYDEF *keydefs, uint columns,
 
         if (options & HA_OPTION_TMP_TABLE) {
           char *path;
-          /* chop off the table name, temporary tables use generated name */
+          /* chop off the table name, tempory tables use generated name */
           if ((path =
                    strrchr(const_cast<char *>(ci->data_file_name), FN_LIBCHAR)))
             *path = '\0';
@@ -636,7 +637,7 @@ int mi_create(const char *name, uint keys, MI_KEYDEF *keydefs, uint columns,
   if (mi_state_info_write(file, &share.state, 2) ||
       mi_base_info_write(file, &share.base))
     goto err;
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   if ((uint)mysql_file_tell(file, MYF(0)) != base_pos + MI_BASE_INFO_SIZE) {
     uint pos = (uint)mysql_file_tell(file, MYF(0));
     DBUG_PRINT("warning", ("base_length: %d  != used_length: %d",
@@ -719,7 +720,7 @@ int mi_create(const char *name, uint keys, MI_KEYDEF *keydefs, uint columns,
   for (i = 0; i < share.base.fields; i++)
     if (mi_recinfo_write(file, &recinfo[i])) goto err;
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   if ((uint)mysql_file_tell(file, MYF(0)) != info_length) {
     uint pos = (uint)mysql_file_tell(file, MYF(0));
     DBUG_PRINT("warning",
@@ -749,7 +750,7 @@ err_no_lock:
   switch (errpos) {
     case 3:
       (void)mysql_file_close(dfile, MYF(0));
-      [[fallthrough]];
+      /* fall through */
     case 2:
       if (!(flags & HA_DONT_TOUCH_DATA))
         mysql_file_delete_with_symlink(
@@ -757,7 +758,7 @@ err_no_lock:
             fn_format(filename, name, "", MI_NAME_DEXT,
                       MY_UNPACK_FILENAME | MY_APPEND_EXT),
             MYF(0));
-      [[fallthrough]];
+      /* fall through */
     case 1:
       (void)mysql_file_close(file, MYF(0));
       if (!(flags & HA_DONT_TOUCH_DATA))
@@ -773,7 +774,7 @@ err_no_lock:
 }
 
 uint mi_get_pointer_length(ulonglong file_length, uint def) {
-  assert(def >= 2 && def <= 7);
+  DBUG_ASSERT(def >= 2 && def <= 7);
   if (file_length) /* If not default */
   {
     if (file_length >= 1ULL << 48)
