@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2010, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -23,8 +30,6 @@ import java.sql.SQLException;
 import java.sql.Timestamp;
 import java.util.Date;
 
-import org.junit.Ignore;
-
 import testsuite.clusterj.model.IdBase;
 import testsuite.clusterj.model.TimestampAsUtilDateTypes;
 
@@ -49,19 +54,8 @@ create index idx_timestamp_not_null_btree on timestamptypes(timestamp_not_null_b
 create unique index idx_timestamp_not_null_both on timestamptypes(timestamp_not_null_both);
 
  */
-@Ignore("writeJDBCreadNDB got failure to match column data for row 1 column 3 Expected: Tue Jan 01 01:01:02 CET 1980 actual: Mon Dec 31 23:01:02 CET 1979")
 public class TimestampAsUtilDateTypesTest extends AbstractClusterJModelTest {
 
-    @Override
-    public void localSetUp() {
-        super.localSetUp();
-        getConnection();
-        resetLocalSystemDefaultTimeZone(connection);
-        connection = null;
-        getConnection();
-        setAutoCommit(connection, false);
-    }
-
     static int NUMBER_OF_INSTANCES = 10;
 
     @Override
@@ -99,7 +93,7 @@ public class TimestampAsUtilDateTypesTest extends AbstractClusterJModelTest {
     public void testWriteNDBReadJDBC() {
         writeNDBreadJDBC();
         failOnError();
-   }
+    }
 
     public void testWriteJDBCReadJDBC() {
         writeJDBCreadJDBC();
@@ -109,9 +103,9 @@ public class TimestampAsUtilDateTypesTest extends AbstractClusterJModelTest {
     public void testWriteNDBReadNDB() {
         writeNDBreadNDB();
         failOnError();
-   }
+    }
 
-   static ColumnDescriptor not_null_hash = new ColumnDescriptor
+    static ColumnDescriptor not_null_hash = new ColumnDescriptor
             ("timestamp_not_null_hash", new InstanceHandler() {
         public void setFieldValue(IdBase instance, Object value) {
             ((TimestampAsUtilDateTypes)instance).setTimestamp_not_null_hash((Date)value);
