@@ -1,17 +1,16 @@
 /*
- *  Copyright (c) 2010, 2024, Oracle and/or its affiliates.
+ *  Copyright (c) 2010, 2017, Oracle and/or its affiliates. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2.0,
  *  as published by the Free Software Foundation.
  *
- *  This program is designed to work with certain software (including
+ *  This program is also distributed with certain software (including
  *  but not limited to OpenSSL) that is licensed under separate terms,
  *  as designated in a particular file or component or in included license
  *  documentation.  The authors of MySQL hereby grant you an additional
  *  permission to link the program and your derivative works with the
- *  separately licensed software that they have either included with
- *  the program or referenced in the documentation.
+ *  separately licensed software that they have included with MySQL.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -258,6 +257,20 @@ class DbImpl implements com.mysql.clusterj.core.store.Db {
         TableConst table = ndbDictionary.getTable(tableName);
         handleError(table, ndbDictionary);
         Key_part_ptrArray key_part_ptrArray = null;
+        if (keyPartsSize == 1) {
+            // extract the ByteBuffer and length from the keyPart
+            KeyPart keyPart = keyParts.get(0);
+            ByteBuffer buffer = keyPart.buffer;
+            int length = keyPart.length;
+            ndbTransaction = ndb.startTransaction(table, buffer, length);
+            if (ndbTransaction == null) {
+                logger.warn(local.message("ERR_Transaction_Start_Failed",
+                        tableName, buffer.position(), buffer.limit(), buffer.capacity(), length));
+            }
+            bufferManager.returnPartitionKeyPartBuffer(length, buffer);
+            handleError (ndbTransaction, ndb);
+            return ndbTransaction;
+        }
         key_part_ptrArray = Key_part_ptrArray.create(keyPartsSize + 1);
         try {
             // the key part pointer array has one entry for each key part
