@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -47,12 +47,10 @@ class BackupReq {
 
   friend bool printBACKUP_REQ(FILE *, const Uint32 *, Uint32, Uint16);
 public:
-  static constexpr Uint32 SignalLength = 4;
-  static constexpr Uint32 WAITCOMPLETED = 0x3;
-  static constexpr Uint32 USE_UNDO_LOG = 0x4;
-  static constexpr Uint32 MT_BACKUP = 0x8;
-  static constexpr Uint32 ENCRYPTED_BACKUP = 0x10;
-  static constexpr Uint32 NOWAIT_REPLY = 0x20;
+  STATIC_CONST( SignalLength = 4 );
+  STATIC_CONST( WAITCOMPLETED = 0x3 );
+  STATIC_CONST( USE_UNDO_LOG = 0x4 );
+  STATIC_CONST( MT_BACKUP = 0x8);
 
 private:
   Uint32 senderData;
@@ -77,7 +75,7 @@ class BackupData {
 
   friend bool printBACKUP_DATA(FILE *, const Uint32 *, Uint32, Uint16);
 public:
-  static constexpr Uint32 SignalLength = 25;
+  STATIC_CONST( SignalLength = 25 );
 
   enum KeyValues {
     /**
@@ -89,7 +87,7 @@ public:
     MaxWrite   = 4, // Maximum write as multiple of blocksize
     
     // Max throughput
-    // Parallel files
+    // Parallell files
     
     NoOfTables = 1000,
     TableName  = 1001  // char*
@@ -136,7 +134,7 @@ class BackupRef {
 
   friend bool printBACKUP_REF(FILE *, const Uint32 *, Uint32, Uint16);
 public:
-  static constexpr Uint32 SignalLength = 3;
+  STATIC_CONST( SignalLength = 3 );
 
 private:
   enum ErrorCodes {
@@ -147,11 +145,6 @@ private:
     SequenceFailure = 1304,
     BackupDefinitionNotImplemented = 1305,
     CannotBackupDiskless = 1306,
-    EncryptionNotSupported = 1307,
-    EncryptionPasswordMissing = 1308,
-    BadEncryptionPassword = 1309,
-    EncryptionPasswordTooLong = 1310,
-    EncryptionPasswordZeroLength = 1311,
     BackupDuringUpgradeUnsupported = 1329
   };
   Uint32 senderData;
@@ -177,7 +170,7 @@ class BackupConf {
 
   friend bool printBACKUP_CONF(FILE *, const Uint32 *, Uint32, Uint16);
 public:
-  static constexpr Uint32 SignalLength = 2;
+  STATIC_CONST( SignalLength = 2);
   
 private:
   Uint32 senderData;
@@ -200,7 +193,7 @@ class BackupAbortRep {
 
   friend bool printBACKUP_ABORT_REP(FILE *, const Uint32 *, Uint32, Uint16);
 public:
-  static constexpr Uint32 SignalLength = 3;
+  STATIC_CONST( SignalLength = 3 );
 
 private:
   Uint32 senderData;
@@ -224,7 +217,7 @@ class BackupCompleteRep {
 
   friend bool printBACKUP_COMPLETE_REP(FILE *, const Uint32 *, Uint32, Uint16);
 public:
-  static constexpr Uint32 SignalLength = 12;
+  STATIC_CONST( SignalLength = 12 );
 private:
   Uint32 senderData;
   Uint32 backupId;
@@ -259,7 +252,7 @@ class AbortBackupOrd {
 
   friend bool printABORT_BACKUP_ORD(FILE *, const Uint32 *, Uint32, Uint16);
 public:
-  static constexpr Uint32 SignalLength = 4;
+  STATIC_CONST( SignalLength = 4 );
   
   enum RequestType {
     ClientAbort = 1321,
