@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -31,14 +31,14 @@
 
 
 /**
- * This signal is used for transferring the sysfile
+ * This signal is used for transfering the sysfile 
  * between Dih on different nodes.
  *
  * The master will distributes the file to the other nodes
  *
  * Since the Sysfile can be larger than on StartMeConf signal,
  *   there might be more than on of these signals sent before
- *   the entire sysfile is transferred
+ *   the entire sysfile is transfered
  */
 class CopyGCIReq {
   /**
@@ -66,8 +66,8 @@ private:
   /**
    * No of free words to carry data
    */
-  static constexpr Uint32 SignalLength = 3;
-  static constexpr Uint32 DATA_SIZE = 22;
+  STATIC_CONST( SignalLength = 3);
+  STATIC_CONST( DATA_SIZE = 22 );
   
   Uint32 data[DATA_SIZE];
 };
