@@ -1,16 +1,15 @@
-/* Copyright (c) 2008, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2008, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,38 +27,40 @@
 
 #define JAM_FILE_ID 40
 
+
 // Only local async signal, not global.
 
-class EnableComReq {
+class EnableComReq  {
   friend class Qmgr;
   friend class Trpman;
   friend class TrpmanProxy;
 
- public:
-  static constexpr Uint32 SignalLength = 3;
+public:
+  STATIC_CONST( SignalLength = 3);
+private:
 
- private:
   Uint32 m_senderRef;
   Uint32 m_senderData;
   Uint32 m_enableNodeId;
-  NodeBitmask m_nodeIds;  // Not part of signal, but first section
+  NodeBitmask m_nodeIds; // Not part of signal, but first section
 };
 
-class EnableComConf {
+class EnableComConf  {
   friend class Qmgr;
   friend class Trpman;
   friend class TrpmanProxy;
   friend class Cmvmi;
 
- public:
-  static constexpr Uint32 SignalLength = 3;
+public:
+  STATIC_CONST( SignalLength = 3);
+private:
 
- private:
   Uint32 m_senderRef;
   Uint32 m_senderData;
   Uint32 m_enableNodeId;
 };
 
+
 #undef JAM_FILE_ID
 
 #endif
