@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -29,15 +29,15 @@
 
 #define JAM_FILE_ID 199
 
-class ReadNodesReq {
+
+class ReadNodesReq
+{
   friend class Qmgr;
   friend class Ndbcntr;
-
- public:
-  static constexpr Uint32 OldSignalLength = 1;
-  static constexpr Uint32 SignalLength = 2;
-
- private:
+public:
+  STATIC_CONST( OldSignalLength = 1);
+  STATIC_CONST( SignalLength = 2);
+private:
   Uint32 myRef;
   Uint32 myVersion;
 };
@@ -53,12 +53,12 @@ class ReadNodesConf {
    * Sender(s)
    */
   friend class Qmgr;
-
+  
   /**
    * Sender(s) / Reciver(s)
    */
   friend class Ndbcntr;
-
+  
   /**
    * Reciver(s)
    */
@@ -73,17 +73,16 @@ class ReadNodesConf {
   friend class Dbinfo;
   friend class Dbspj;
 
-  friend bool printREAD_NODES_CONF(FILE *, const Uint32 *, Uint32, Uint16);
-
- public:
-  static constexpr Uint32 SignalLength = 3;
-
- private:
+  friend bool printREAD_NODES_CONF(FILE*, const Uint32 *, Uint32, Uint16);
+public:
+  STATIC_CONST( SignalLength = 3);
+private:
+  
   Uint32 noOfNodes;
   Uint32 ndynamicId;
 
   /**
-   *
+   * 
    * NOTE Not valid when send from Qmgr
    */
   Uint32 masterNodeId;
@@ -97,15 +96,15 @@ class ReadNodesConf {
   NdbNodeBitmask definedNodes;
 
   /**
-   * This array describes whether the nodes are currently active
+   * This array describes wheather the nodes are currently active
    *
    * NOTE Not valid when send from Qmgr
    */
   NdbNodeBitmask inactiveNodes;
 
-  NdbNodeBitmask clusterNodes;   // From Qmgr
-  NdbNodeBitmask startingNodes;  // From Cntr
-  NdbNodeBitmask startedNodes;   // From Cntr
+  NdbNodeBitmask clusterNodes;  // From Qmgr
+  NdbNodeBitmask startingNodes; // From Cntr
+  NdbNodeBitmask startedNodes;  // From Cntr
 };
 
 class ReadNodesConf_v1 {
@@ -133,12 +132,11 @@ class ReadNodesConf_v1 {
   friend class Dbinfo;
   friend class Dbspj;
 
-  friend bool printREAD_NODES_CONF(FILE *, const Uint32 *, Uint32, Uint16);
+  friend bool printREAD_NODES_CONF(FILE*, const Uint32 *, Uint32, Uint16);
+public:
+  STATIC_CONST( SignalLength = 3 + 5 * NdbNodeBitmask48::Size);
+private:
 
- public:
-  static constexpr Uint32 SignalLength = 3 + 5 * NdbNodeBitmask48::Size;
-
- private:
   Uint32 noOfNodes;
   Uint32 ndynamicId;
 
@@ -157,17 +155,18 @@ class ReadNodesConf_v1 {
   };
 
   /**
-   * This array describes whether the nodes are currently active
+   * This array describes wheather the nodes are currently active
    *
    * NOTE Not valid when send from Qmgr
    */
   Uint32 inactiveNodes[NdbNodeBitmask48::Size];
 
-  Uint32 clusterNodes[NdbNodeBitmask48::Size];   // From Qmgr
-  Uint32 startingNodes[NdbNodeBitmask48::Size];  // From Cntr
-  Uint32 startedNodes[NdbNodeBitmask48::Size];   // From Cntr
+  Uint32 clusterNodes[NdbNodeBitmask48::Size];  // From Qmgr
+  Uint32 startingNodes[NdbNodeBitmask48::Size]; // From Cntr
+  Uint32 startedNodes[NdbNodeBitmask48::Size];  // From Cntr
 };
 
+
 #undef JAM_FILE_ID
 
 #endif
