@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2003, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,13 +25,17 @@
 #ifndef SETVARREQ_H
 #define SETVARREQ_H
 
-#include "ConfigParamId.hpp"
 #include "SignalData.hpp"
+#include "ConfigParamId.hpp"
 
 #define JAM_FILE_ID 183
 
+
 class SetVarReq {
- public:
+
+public:
+
+
   static UintR size();
 
   void mgmtSrvrBlockRef(UintR mgmtSrvrBlockRef);
@@ -44,13 +47,20 @@ class SetVarReq {
   void value(UintR value);
   UintR value(void) const;
 
- private:
+
+private:
+
   UintR _mgmtSrvrBlockRef;
   UintR _variable;
   UintR _value;
 };
 
-inline UintR SetVarReq::size(void) { return 3; }
+
+
+inline UintR SetVarReq::size(void) {
+  return 3;
+}
+
 
 inline void SetVarReq::mgmtSrvrBlockRef(UintR mgmtSrvrBlockRef) {
   _mgmtSrvrBlockRef = mgmtSrvrBlockRef;
@@ -60,18 +70,29 @@ inline UintR SetVarReq::mgmtSrvrBlockRef(void) const {
   return _mgmtSrvrBlockRef;
 }
 
+
 inline void SetVarReq::variable(ConfigParamId variable) {
   _variable = variable;
 }
 
+
 inline ConfigParamId SetVarReq::variable(void) const {
   return static_cast<ConfigParamId>(_variable);
 }
 
-inline void SetVarReq::value(UintR value) { _value = value; }
 
-inline UintR SetVarReq::value(void) const { return _value; }
+inline void SetVarReq::value(UintR value) {
+  _value = value;
+}
+
+inline UintR SetVarReq::value(void) const {
+  return _value;
+}
+
+
+
 
 #undef JAM_FILE_ID
 
-#endif  // SETVARREQ_H
+#endif // SETVARREQ_H
+
