@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -27,6 +27,7 @@
 
 #define JAM_FILE_ID 204
 
+
 /**
  * This signal is sent by starting DIH to master DIH
  *
@@ -38,14 +39,14 @@ class StartPermReq {
    * Sender(s) / Reciver(s)
    */
   friend class Dbdih;
-
- public:
-  static constexpr Uint32 SignalLength = 3;
-
- private:
+  
+public:
+  STATIC_CONST( SignalLength = 3 );
+private:
+  
   Uint32 blockRef;
   Uint32 nodeId;
-  Uint32 startType;
+  Uint32 startType;  
 };
 
 class StartPermConf {
@@ -53,13 +54,13 @@ class StartPermConf {
    * Sender(s) / Reciver(s)
    */
   friend class Dbdih;
-
- public:
-  static constexpr Uint32 SignalLength = 3;
-
- private:
+  
+public:
+  STATIC_CONST( SignalLength = 3 );
+private:
+  
   Uint32 startingNodeId;
-  Uint32 systemFailureNo;
+  Uint32 systemFailureNo;  
   Uint32 microGCP;
 };
 
@@ -68,22 +69,24 @@ class StartPermRef {
    * Sender(s) / Reciver(s)
    */
   friend class Dbdih;
-
- public:
-  static constexpr Uint32 SignalLength = 2;
-
- private:
+  
+public:
+  STATIC_CONST( SignalLength = 2 );
+private:
+  
   Uint32 startingNodeId;
-  Uint32 errorCode;
+  Uint32 errorCode;  
 
-  enum ErrorCode {
+  enum ErrorCode
+  {
     ZNODE_ALREADY_STARTING_ERROR = 305,
     ZNODE_START_DISALLOWED_ERROR = 309,
     InitialStartRequired = 320
   };
 };
 
-class StartPermRep {
+class StartPermRep
+{
   /**
    * Sender(s)
    */
@@ -93,11 +96,15 @@ class StartPermRep {
    */
   friend class Ndbcntr;
 
- public:
-  static constexpr Uint32 SignalLength = 2;
-  enum { PermissionToStart = 0, CompletedStart = 1 };
+public:
+  STATIC_CONST( SignalLength = 2 );
+  enum
+  {
+    PermissionToStart = 0,
+    CompletedStart = 1
+  };
 
- private:
+private:
   Uint32 startNodeId;
   Uint32 reason;
 };
