@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -29,15 +29,15 @@
 
 #define JAM_FILE_ID 169
 
+
 class TuxBoundInfo {
   friend class Dblqh;
   friend class Dbtux;
-
- public:
+public:
   // must match API (0-4 and no changes expected)
   enum BoundType {
-    BoundLE = 0,  // bit 1 for less/greater
-    BoundLT = 1,  // bit 0 for strict
+    BoundLE = 0,        // bit 1 for less/greater
+    BoundLT = 1,        // bit 0 for strict
     BoundGE = 2,
     BoundGT = 3,
     BoundEQ = 4,
@@ -55,9 +55,8 @@ class TuxBoundInfo {
     InvalidCharFormat = 744,
     TooMuchAttrInfo = 823
   };
-  static constexpr Uint32 SignalLength = 3;
-
- private:
+  STATIC_CONST( SignalLength = 3 );
+private:
   /*
    * Error code set by TUX.  Zero means no error.
    */
@@ -70,10 +69,11 @@ class TuxBoundInfo {
    * Number of words of bound info included after fixed signal data.
    */
   Uint32 boundAiLength;
-
+  
   Uint32 data[1];
 };
 
+
 #undef JAM_FILE_ID
 
 #endif
