@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2003, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,8 +25,8 @@
 #ifndef UTIL_PREPARE_REQ_HPP
 #define UTIL_PREPARE_REQ_HPP
 
-#include <SimpleProperties.hpp>
 #include "SignalData.hpp"
+#include <SimpleProperties.hpp>
 
 #define JAM_FILE_ID 18
 
@@ -52,39 +51,41 @@ class UtilPrepareReq {
   /**
    * For printing
    */
-  friend bool printUTIL_PREPARE_REQ(FILE *output, const Uint32 *theData,
-                                    Uint32 len, Uint16 receiverBlockNo);
+  friend bool printUTIL_PREPARE_REQ(FILE * output, 
+                                    const Uint32 * theData, 
+                                    Uint32 len, 
+                                    Uint16 receiverBlockNo);
 
- public:
+public:
   enum OperationTypeValue {
-    Read = 0,
-    Update = 1,
-    Insert = 2,
-    Delete = 3,
-    Write = 4,
-    Probe = 5  // check existence...
+    Read               = 0,
+    Update             = 1,
+    Insert             = 2,
+    Delete             = 3,
+    Write	       = 4,
+    Probe              = 5  // check existance...
   };
 
   enum KeyValue {
-    NoOfOperations = 1,  ///< No of operations in transaction
-    OperationType = 2,   ///
-    TableName = 3,       ///< String
-    AttributeName = 4,   ///< String
-    TableId = 5,
-    AttributeId = 6,
-    ScanTakeOverInd = 7,
-    ReorgInd = 8
+    NoOfOperations     = 1,  ///< No of operations in transaction
+    OperationType      = 2,  /// 
+    TableName          = 3,  ///< String
+    AttributeName      = 4,  ///< String
+    TableId	       = 5,
+    AttributeId	       = 6,
+    ScanTakeOverInd    = 7,
+    ReorgInd           = 8
   };
 
   // Signal constants
-  static constexpr Uint32 SignalLength = 3;
-  static constexpr Uint32 PROPERTIES_SECTION = 0;
-  static constexpr Uint32 NoOfSections = 1;
+  STATIC_CONST( SignalLength = 3 );
+  STATIC_CONST( PROPERTIES_SECTION = 0 );
+  STATIC_CONST( NoOfSections = 1 );
 
   GET_SET_SENDERREF
   GET_SET_SENDERDATA
- private:
-  Uint32 senderData;  // MUST be no 1!
+private:  
+  Uint32 senderData; // MUST be no 1!
   Uint32 senderRef;
   Uint32 schemaTransId;
 };
@@ -93,7 +94,7 @@ class UtilPrepareReq {
  * @class UtilPrepareConf
  *
  * Data format:
- * - UTIL_PREPARE_CONF <UtilPrepareId>
+ * - UTIL_PREPARE_CONF <UtilPrepareId> 
  */
 
 class UtilPrepareConf {
@@ -106,24 +107,27 @@ class UtilPrepareConf {
   /**
    * For printing
    */
-  friend bool printUTIL_PREPARE_CONF(FILE *output, const Uint32 *theData,
-                                     Uint32 len, Uint16 receiverBlockNo);
+  friend bool printUTIL_PREPARE_CONF(FILE * output, 
+				     const Uint32 * theData, 
+				     Uint32 len, 
+				     Uint16 receiverBlockNo);
 
- public:
-  static constexpr Uint32 SignalLength = 2;
+public:
+  STATIC_CONST( SignalLength = 2 );
 
   GET_SET_SENDERDATA
   GET_SET_PREPAREID
- private:
-  Uint32 senderData;  // MUST be no 1!
-  Uint32 prepareId;
+private:
+  Uint32 senderData; // MUST be no 1!
+  Uint32 prepareId; 
 };
 
+
 /**
  * @class UtilPrepareRef
  *
  * Data format:
- * - UTIL_PREPARE_REF
+ * - UTIL_PREPARE_REF 
  */
 
 class UtilPrepareRef {
@@ -137,10 +141,12 @@ class UtilPrepareRef {
   /**
    * For printing
    */
-  friend bool printUTIL_PREPARE_REF(FILE *output, const Uint32 *theData,
-                                    Uint32 len, Uint16 receiverBlockNo);
+  friend bool printUTIL_PREPARE_REF(FILE * output, 
+				    const Uint32 * theData, 
+				    Uint32 len, 
+				    Uint16 receiverBlockNo);
 
- public:
+public:
   enum ErrorCode {
     PREPARE_REF_NO_ERROR = 0,
     PREPARE_SEIZE_ERROR = 1,
@@ -150,16 +156,18 @@ class UtilPrepareRef {
     MISSING_PROPERTIES_SECTION = 5
   };
 
-  static constexpr Uint32 SignalLength = 3;
+  STATIC_CONST( SignalLength = 3 );
 
   GET_SET_SENDERDATA
   GET_SET_ERRORCODE
- private:
-  Uint32 senderData;  // MUST be no 1!
+private:
+  Uint32 senderData; // MUST be no 1!
   Uint32 errorCode;
-  Uint32 dictErrCode;  // If errorCode == DICT_TAB_INFO_ERROR
+  Uint32 dictErrCode; // If errorCode == DICT_TAB_INFO_ERROR
 };
 
+
+
 #undef JAM_FILE_ID
 
 #endif
