@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2003, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,13 +25,11 @@
 #ifndef SYSLOGHANDLER_H
 #define SYSLOGHANDLER_H
 
-#include <time.h>
-
 #include "LogHandler.hpp"
 
 /**
- * Logs messages to syslog. The default identity is 'NDB'.
- * See 'man 3 syslog'.
+ * Logs messages to syslog. The default identity is 'NDB'. 
+ * See 'man 3 syslog'. 
  *
  * It logs the following severity levels.
  * <pre>
@@ -58,51 +55,52 @@
  * @see LogHandler
  * @version #@ $Id: SysLogHandler.hpp,v 1.2 2003/09/01 10:15:53 innpeno Exp $
  */
-class SysLogHandler : public LogHandler {
- public:
+class SysLogHandler : public LogHandler
+{
+public:
   /**
    * Default constructor.
    */
   SysLogHandler();
-
+  
   /**
    * Create a new syslog handler with the specified identity.
    *
    * @param pIdentity a syslog identity.
    * @param facility syslog facility, defaults to LOG_USER
    */
-  SysLogHandler(const char *pIdentity, int facility);
+  SysLogHandler(const char* pIdentity, int facility);
 
   /**
    * Destructor.
    */
-  ~SysLogHandler() override;
+  virtual ~SysLogHandler();
+  
+  virtual bool open();
+  virtual bool close();
 
-  bool open() override;
-  bool close() override;
+  virtual bool is_open();
 
-  bool is_open() override;
-
-  bool setParam(const BaseString &param, const BaseString &value) override;
+  virtual bool setParam(const BaseString &param, const BaseString &value);
   bool setFacility(const BaseString &facility);
 
- protected:
-  void writeHeader(const char *pCategory, Logger::LoggerLevel level,
-                   time_t now) override;
-  void writeMessage(const char *pMsg) override;
-  void writeFooter() override;
+protected:	
+  virtual void writeHeader(const char* pCategory, Logger::LoggerLevel level,
+                           time_t now);
+  virtual void writeMessage(const char* pMsg);
+  virtual void writeFooter();
 
- private:
+private:
   /** Prohibit*/
-  SysLogHandler(const SysLogHandler &);
-  SysLogHandler operator=(const SysLogHandler &);
-  bool operator==(const SysLogHandler &);
+  SysLogHandler(const SysLogHandler&);
+  SysLogHandler operator = (const SysLogHandler&);
+  bool operator == (const SysLogHandler&);
 
   int m_severity;
-  const char *m_pCategory;
+  const char* m_pCategory;
 
   /** Syslog identity for all log entries. */
-  const char *m_pIdentity;
+  const char* m_pIdentity;
   int m_facility;
   bool m_open;
 };
