@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2003, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -36,15 +35,15 @@ class NdbRecAttr;
 
 /**
  * @class NdbEventOperation
- * @brief Class of operations for getting change events from database.
+ * @brief Class of operations for getting change events from database.  
  *
  * Brief description on how to work with events:
  *
- * - An event, represented by an NdbDictionary::Event, is created in the
+ * - An event, represented by an NdbDictionary::Event, i created in the 
  *   Database through
- *   NdbDictionary::Dictionary::createEvent() (note that this can be done
+ *   NdbDictionary::Dictionary::createEvent() (note that this can be done 
  *   by any application or thread and not necessarily by the "listener")
- * - To listen to events, an NdbEventOperation object is instantiated by
+ * - To listen to events, an NdbEventOperation object is instantiated by 
  *   Ndb::createEventOperation()
  * - execute() starts the event flow. Use Ndb::pollEvents() to wait
  *   for an event to occur.  Use Ndb::nextEvent() to iterate
@@ -65,7 +64,7 @@ class NdbRecAttr;
  *
  * - When several NdbEventOperation's are tied to the same event in the same
  * process they will share the circular buffer. The BufferLength will then
- * be the same for all and decided by the first NdbEventOperation
+ * be the same for all and decided by the first NdbEventOperation 
  * instantiation. Just make sure to instantiate the "largest" one first.
  * - Today all events INSERT/DELETE/UPDATE and all changed attributes are
  * sent to the API, even if only specific attributes have been specified.
@@ -90,18 +89,18 @@ class NdbRecAttr;
  * - ndb_select_all -d sys 'NDB$EVENTS_0'
  * shows contents in the system table containing created events.
  *
- * @note this is an interface to viewing events that is subject to change
+ * @note this is an inteface to viewing events that is subject to change
  */
 class NdbEventOperation {
- public:
+public:
   /**
    * State of the NdbEventOperation object
    */
   enum State {
-    EO_CREATED,    ///< Created but execute() not called
-    EO_EXECUTING,  ///< execute() called
-    EO_DROPPED,    ///< Waiting to be deleted, Object unusable.
-    EO_ERROR       ///< An error has occurred. Object unusable.
+    EO_CREATED,   ///< Created but execute() not called
+    EO_EXECUTING, ///< execute() called
+    EO_DROPPED,   ///< Waiting to be deleted, Object unusable.
+    EO_ERROR      ///< An error has occurred. Object unusable.
   };
   /**
    * Retrieve current state of the NdbEventOperation object
@@ -114,7 +113,7 @@ class NdbEventOperation {
 
   /**
    * Activates the NdbEventOperation to start receiving events. The
-   * changed attribute values may be retrieved after Ndb::nextEvent()
+   * changed attribute values may be retrieved after Ndb::nextEvent() 
    * has returned not NULL. The getValue() methods must be called
    * prior to execute().
    *
@@ -125,7 +124,7 @@ class NdbEventOperation {
   /**
    * Defines a retrieval operation of an attribute value.
    * The NDB API allocate memory for the NdbRecAttr object that
-   * will hold the returned attribute value.
+   * will hold the returned attribute value. 
    *
    * @note Note that it is the applications responsibility
    *       to allocate enough memory for aValue (if non-NULL).
@@ -138,51 +137,39 @@ class NdbEventOperation {
    *
    * @note There are two versions, getValue() and
    *       getPreValue() for retrieving the current and
-   *       previous value respectively.
+   *       previous value repectively.
    *
-   * @note This method does not fetch the attribute value from
-   *       the database!  The NdbRecAttr object returned by this method
-   *       is <em>not</em> readable/printable before the
+   * @note This method does not fetch the attribute value from 
+   *       the database!  The NdbRecAttr object returned by this method 
+   *       is <em>not</em> readable/printable before the 
    *       execute() has been made and
    *       Ndb::nextEvent() has returned not NULL.
-   *       If a specific attribute has not changed the corresponding
-   *       NdbRecAttr will be in state UNDEFINED.  This is checked by
+   *       If a specific attribute has not changed the corresponding 
+   *       NdbRecAttr will be in state UNDEFINED.  This is checked by 
    *       NdbRecAttr::isNULL() which then returns -1.
    *
-   * @param anAttrName  Attribute name
-   * @param aValue      If this is non-NULL, then the attribute value
+   * @param anAttrName  Attribute name 
+   * @param aValue      If this is non-NULL, then the attribute value 
    *                    will be returned in this parameter.<br>
-   *                    If NULL, then the attribute value will only
+   *                    If NULL, then the attribute value will only 
    *                    be stored in the returned NdbRecAttr object.
-   * @return            An NdbRecAttr object to hold the value of
-   *                    the attribute, or a NULL pointer
+   * @return            An NdbRecAttr object to hold the value of 
+   *                    the attribute, or a NULL pointer 
    *                    (indicating error).
    */
-  NdbRecAttr *getValue(const char *anAttrName, char *aValue = nullptr);
+  NdbRecAttr *getValue(const char *anAttrName, char *aValue = 0);
   /**
    * See getValue().
    */
-  NdbRecAttr *getPreValue(const char *anAttrName, char *aValue = nullptr);
+  NdbRecAttr *getPreValue(const char *anAttrName, char *aValue = 0);
 
   /**
    * These methods replace getValue/getPreValue for blobs.  Each
    * method creates a blob handle NdbBlob.  The handle supports only
    * read operations.  See NdbBlob.
    */
-  NdbBlob *getBlobHandle(const char *anAttrName);
-  NdbBlob *getPreBlobHandle(const char *anAttrName);
-
-  /**
-    Activate data node filtering of updates that have
-    the no-logging flag set in anyvalue.
-   */
-  void setFilterAnyvalueMySQLNoLogging();
-
-  /**
-    Activate data node filtering of updates applied by a replica, ie where the
-    serverid portion of anyvalue is set.
-   */
-  void setFilterAnyvalueMySQLNoReplicaUpdates();
+  NdbBlob* getBlobHandle(const char *anAttrName);
+  NdbBlob* getPreBlobHandle(const char *anAttrName);
 
   int isOverrun() const;
 
@@ -283,14 +270,14 @@ class NdbEventOperation {
    * Get the latest error
    *
    * @return   Error object.
-   */
-  const struct NdbError &getNdbError() const;
+   */			     
+  const struct NdbError & getNdbError() const;
 
   /**
    * Set allow empty updates
    *
    * To support monitoring of pseudo columns we need to
-   * explicitly allow for receiving events with no updates
+   * explicitely allow for receiving events with no updates
    * to user defined columns.
    * Normally update events with no changes to monitored columns
    * are filtered out by NdbApi. By calling setAllowEmptyUpdate(true),
@@ -314,33 +301,22 @@ class NdbEventOperation {
    * Check whether the consumed event data marks an error epoch
    * and get the error.
    */
-  bool isErrorEpoch(NdbDictionary::Event::TableEvent *error_type = nullptr);
+  bool isErrorEpoch(NdbDictionary::Event::TableEvent *error_type = 0);
+
 
 #ifndef DOXYGEN_SHOULD_SKIP_INTERNAL
   /** these are subject to change at any time */
-  const NdbDictionary::Table *getTable() const;
+  const NdbDictionary::Table* getTable() const;
   const NdbDictionary::Event *getEvent() const;
   const NdbRecAttr *getFirstPkAttr() const;
   const NdbRecAttr *getFirstPkPreAttr() const;
   const NdbRecAttr *getFirstDataAttr() const;
   const NdbRecAttr *getFirstDataPreAttr() const;
 
-  //  bool validateTable(NdbDictionary::Table &table) const;
+//  bool validateTable(NdbDictionary::Table &table) const;
 
-  void setCustomData(void *data);
-  void *getCustomData() const;
-
-  /**
-    @brief Callback for filtering any_value in all incoming row
-    changes. The filtered value will be aggregated for the epoch,
-    i.e. OR'ed together with other filtered values. The final result
-    is returned when iterating events in an epoch using
-    Ndb::getNextEventOpInEpoch4().
-    NOTE! Since the function is invoked for every row change,
-    care should be taken to avoid costly calculations in the callback.
-  */
-  using AnyValueFilterFn = Uint32 (*)(Uint32);
-  void setAnyValueFilter(AnyValueFilterFn);
+  void setCustomData(void * data);
+  void * getCustomData() const;
 
   void clearError();
   int hasError() const;
@@ -356,19 +332,19 @@ class NdbEventOperation {
   void print();
 #endif
 
- private:
+private:
 #ifndef DOXYGEN_SHOULD_SKIP_INTERNAL
   friend class NdbEventOperationImpl;
   friend class NdbEventBuffer;
 #endif
-  NdbEventOperation(Ndb *ndb, const NdbDictionary::Event *event);
+  NdbEventOperation(Ndb *theNdb, const char* eventName);
   ~NdbEventOperation();
   class NdbEventOperationImpl &m_impl;
-  NdbEventOperation(NdbEventOperationImpl &impl);
+  NdbEventOperation(NdbEventOperationImpl& impl);
 
-  NdbEventOperation(const NdbEventOperation &) = delete;
-  NdbEventOperation &operator=(const NdbEventOperation &) = delete;
+  NdbEventOperation(const NdbEventOperation&); // Not impl.
+  NdbEventOperation&operator=(const NdbEventOperation&);
 };
 
-typedef void (*NdbEventCallback)(NdbEventOperation *, Ndb *, void *);
+typedef void (* NdbEventCallback)(NdbEventOperation*, Ndb*, void*);
 #endif
