@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2003, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -34,7 +33,7 @@ class PollGuard;
 
 /**
  * @class NdbScanOperation
- * @brief Class of scan operations for use in transactions.
+ * @brief Class of scan operations for use in transactions.  
  */
 class NdbScanOperation : public NdbOperation {
 #ifndef DOXYGEN_SHOULD_SKIP_INTERNAL
@@ -48,7 +47,7 @@ class NdbScanOperation : public NdbOperation {
   friend class NdbQueryOperationImpl;
 #endif
 
- public:
+public:
   /**
    * Scan flags.  OR-ed together and passed as argument to
    * readTuples, scanIndex, and scanTable. Note that SF_MultiRange
@@ -67,7 +66,7 @@ class NdbScanOperation : public NdbOperation {
       Note that :
       1) Ordered indexes are distributed - there is one for each fragment of a
          table.
-      2) Range scans are often parallel - across all index fragments.
+      2) Range scans are often parallel - across all index fragments.  
          Occasionally they can be pruned to one index fragment.
       3) Each index fragment range scan will return results in either ascending
          or descending order.  Ascending is the default, but descending is
@@ -84,7 +83,7 @@ class NdbScanOperation : public NdbOperation {
            index fragments must be scanned in parallel.  (Non SF_OrderBy* flag
            scans can be executed with lower than full-parallelism)
         ii) Results from every index fragment must be available before returning
-            any row, to ensure a correct merge sort.  This serialises the
+            any row, to ensure a correct merge sort.  This serialises the 
             'scrolling' of the scan, potentially resulting in lower row
             throughput.
         iii) Non SF_OrderBy* flag scans can return rows to the Api before all
@@ -93,7 +92,7 @@ class NdbScanOperation : public NdbOperation {
     */
     SF_OrderBy = (1 << 24),
     /**
-     * Same as order by, except that it will automatically
+     * Same as order by, except that it will automatically 
      *   add all key columns into the read-mask
      */
     SF_OrderByFull = (16 << 24),
@@ -124,43 +123,47 @@ class NdbScanOperation : public NdbOperation {
     SF_KeyInfo = 1
   };
 
+
   /*
    * ScanOptions
-   *  These are options passed to the NdbRecord based scanTable and
+   *  These are options passed to the NdbRecord based scanTable and 
    *  scanIndex methods of the NdbTransaction class.
    *  Each option type is marked as present by setting the corresponding
-   *  bit in the optionsPresent field.  Only the option types marked
+   *  bit in the optionsPresent field.  Only the option types marked 
    *  in the optionsPresent field need have sensible data.
    *  All data is copied out of the ScanOptions structure (and any
    *  subtended structures) at operation definition time.
-   *  If no options are required, then NULL may be passed as the
+   *  If no options are required, then NULL may be passed as the 
    *  ScanOptions pointer.
    *
    *  Most methods take a supplementary sizeOfOptions parameter.  This
    *  is optional, and is intended to allow the interface implementation
-   *  to remain backwards compatible with older un-recompiled clients
-   *  that may pass an older (smaller) version of the ScanOptions
+   *  to remain backwards compatible with older un-recompiled clients 
+   *  that may pass an older (smaller) version of the ScanOptions 
    *  structure.  This effect is achieved by passing
    *  sizeof(NdbScanOperation::ScanOptions) into this parameter.
    */
-  struct ScanOptions {
+  struct ScanOptions
+  {
     /*
       Size of the ScanOptions structure.
     */
-    static inline Uint32 size() { return sizeof(ScanOptions); }
+    static inline Uint32 size()
+    {
+        return sizeof(ScanOptions);
+    }
 
     /* Which options are present - see below for possibilities */
     Uint64 optionsPresent;
 
-    enum Type {
-      SO_SCANFLAGS = 0x01,
-      SO_PARALLEL = 0x02,
-      SO_BATCH = 0x04,
-      SO_GETVALUE = 0x08,
-      SO_PARTITION_ID = 0x10,
-      SO_INTERPRETED = 0x20,
-      SO_CUSTOMDATA = 0x40,
-      SO_PART_INFO = 0x80
+    enum Type { SO_SCANFLAGS    = 0x01,
+                SO_PARALLEL     = 0x02,
+                SO_BATCH        = 0x04,
+                SO_GETVALUE     = 0x08,
+                SO_PARTITION_ID = 0x10,
+                SO_INTERPRETED  = 0x20,
+                SO_CUSTOMDATA   = 0x40,
+                SO_PART_INFO    = 0x80
     };
 
     /* Flags controlling scan behaviour
@@ -173,17 +176,17 @@ class NdbScanOperation : public NdbOperation {
      */
     Uint32 parallel;
 
-    /* Desired scan batchsize in rows
+    /* Desired scan batchsize in rows 
      * for NDBD -> API transfers
      * Default == 0 == Automatically chosen size
      */
     Uint32 batch;
-
+    
     /* Extra values to be read for each row meeting
      * scan criteria
      */
     NdbOperation::GetValueSpec *extraGetValues;
-    Uint32 numExtraGetValues;
+    Uint32                     numExtraGetValues;
 
     /* Specific partition to limit this scan to
      * Alternatively, an Ndb::PartitionSpec can be supplied.
@@ -196,24 +199,28 @@ class NdbScanOperation : public NdbOperation {
     const NdbInterpretedCode *interpretedCode;
 
     /* CustomData ptr to associate with the scan operation */
-    void *customData;
+    void * customData;
 
     /* Partition information for bounding this scan */
-    const Ndb::PartitionSpec *partitionInfo;
+    const Ndb::PartitionSpec* partitionInfo;
     Uint32 sizeOfPartInfo;
   };
 
+
   /**
    * readTuples
    * Method used in old scan Api to specify scan operation details
-   *
+   * 
    * @param lock_mode Lock mode
    * @param scan_flags see @ref ScanFlag
    * @param parallel Number of fragments to scan in parallel (0=max)
    * @param batch Number of rows to fetch in each batch
-   */
-  virtual int readTuples(LockMode lock_mode = LM_Read, Uint32 scan_flags = 0,
-                         Uint32 parallel = 0, Uint32 batch = 0);
+   */ 
+  virtual
+  int readTuples(LockMode lock_mode = LM_Read, 
+                 Uint32 scan_flags = 0, 
+		 Uint32 parallel = 0,
+		 Uint32 batch = 0);
 
 #ifndef DOXYGEN_SHOULD_SKIP_DEPRECATED
   /**
@@ -222,37 +229,36 @@ class NdbScanOperation : public NdbOperation {
    * @param batch No of rows to fetch from each fragment at a time
    * @param parallel No of fragments to scan in parallell
    * @note specifying 0 for batch and parallel means max performance
-   */
+   */ 
 #ifdef ndb_readtuples_impossible_overload
-  int readTuples(LockMode lock_mode = LM_Read, Uint32 batch = 0,
-                 Uint32 parallel = 0, bool keyinfo = false,
-                 bool multi_range = false);
+  int readTuples(LockMode lock_mode = LM_Read, 
+		 Uint32 batch = 0, Uint32 parallel = 0, 
+                 bool keyinfo = false, bool multi_range = false);
 #endif
-
-  inline int readTuples(int parallell) {
+  
+  inline int readTuples(int parallell){
     return readTuples(LM_Read, 0, parallell);
   }
-
-  inline int readTuplesExclusive(int parallell = 0) {
+  
+  inline int readTuplesExclusive(int parallell = 0){
     return readTuples(LM_Exclusive, 0, parallell);
   }
 
   /* First version of ScanOptions, defined here for backwards
    * compatibility reasons
    */
-  struct ScanOptions_v1 {
+  struct ScanOptions_v1
+  {
     /* Which options are present - see below for possibilities */
     Uint64 optionsPresent;
 
-    enum Type {
-      SO_SCANFLAGS = 0x01,
-      SO_PARALLEL = 0x02,
-      SO_BATCH = 0x04,
-      SO_GETVALUE = 0x08,
-      SO_PARTITION_ID = 0x10,
-      SO_INTERPRETED = 0x20,
-      SO_CUSTOMDATA = 0x40
-    };
+    enum Type { SO_SCANFLAGS    = 0x01,
+                SO_PARALLEL     = 0x02,
+                SO_BATCH        = 0x04,
+                SO_GETVALUE     = 0x08,
+                SO_PARTITION_ID = 0x10,
+                SO_INTERPRETED  = 0x20,
+                SO_CUSTOMDATA   = 0x40 };
 
     /* Flags controlling scan behaviour
      * See NdbScanOperation::ScanFlag for details
@@ -264,17 +270,17 @@ class NdbScanOperation : public NdbOperation {
      */
     Uint32 parallel;
 
-    /* Desired scan batchsize in rows
+    /* Desired scan batchsize in rows 
      * for NDBD -> API transfers
      * Default == 0 == Automatically chosen size
      */
     Uint32 batch;
-
+    
     /* Extra values to be read for each row meeting
      * scan criteria
      */
     NdbOperation::GetValueSpec *extraGetValues;
-    Uint32 numExtraGetValues;
+    Uint32                     numExtraGetValues;
 
     /* Specific partition to limit this scan to
      * Only applicable for tables defined with UserDefined partitioning
@@ -287,15 +293,15 @@ class NdbScanOperation : public NdbOperation {
     const NdbInterpretedCode *interpretedCode;
 
     /* CustomData ptr to associate with the scan operation */
-    void *customData;
+    void * customData;
   };
 #endif
-
+  
 #ifndef DOXYGEN_SHOULD_SKIP_INTERNAL
-  NdbBlob *getBlobHandle(const char *anAttrName) override;
-  NdbBlob *getBlobHandle(Uint32 anAttrId) override;
+  virtual NdbBlob* getBlobHandle(const char* anAttrName);
+  virtual NdbBlob* getBlobHandle(Uint32 anAttrId);
 
-  /**
+  /** 
    * setInterpretedCode
    *
    * This method is used to set an interpreted program to be executed
@@ -303,7 +309,7 @@ class NdbScanOperation : public NdbOperation {
    * rows out of the returned set.  This method is only supported for
    * old Api scans.  For NdbRecord scans, pass the interpreted program
    * via the ScanOptions structure.
-   *
+   * 
    * @param code The interpreted program to be executed for each
    * candidate result row in this scan.
    * @return 0 if successful, -1 otherwise
@@ -312,13 +318,14 @@ class NdbScanOperation : public NdbOperation {
 
 #endif
 
+
   /**
-   * Get the next tuple in a scan transaction.
-   *
+   * Get the next tuple in a scan transaction. 
+   * 
    * After each call to nextResult
-   * the buffers and NdbRecAttr objects defined in
-   * NdbOperation::getValue are updated with values
-   * from the scanned tuple.
+   * the buffers and NdbRecAttr objects defined in 
+   * NdbOperation::getValue are updated with values 
+   * from the scanned tuple. 
    *
    * @param fetchAllowed  If set to false, then fetching is disabled
    * @param forceSend If true send will occur immediately (see @ref secAdapt)
@@ -339,9 +346,9 @@ class NdbScanOperation : public NdbOperation {
    * in order to contact the NDB Kernel for more records, after taking over
    * locks as appropriate.
    *
-   * fetchAllowed = false is useful when you want to update or
+   * fetchAllowed = false is useful when you want to update or 
    * delete all the records fetched in one transaction(This will save a
-   *  lot of round trip time and make updates or deletes of scanned
+   *  lot of round trip time and make updates or deletes of scanned 
    * records a lot faster).
    *
    * While nextResult(false) returns 0, take over the record to
@@ -351,20 +358,20 @@ class NdbScanOperation : public NdbOperation {
    * deletes will be made and then the locks will be released.
    *
    * After that, call nextResult(true) which will fetch new records and
-   * cache them in the NdbApi.
-   *
-   * @note  If you don't take over the records to another transaction the
+   * cache them in the NdbApi. 
+   * 
+   * @note  If you don't take over the records to another transaction the 
    *        locks on those records will be released the next time NDB Kernel
    *        is contacted for more records.
    *
    * @note  Please contact for examples of efficient scan
    *        updates and deletes.
-   *
+   * 
    * @note  See ndb/examples/ndbapi_scan_example for usage.
    *
-   * @return
+   * @return 
    * -  -1: if unsuccessful,<br>
-   * -   0: if another tuple was received, and<br>
+   * -   0: if another tuple was received, and<br> 
    * -   1: if there are no more tuples to scan.
    * -   2: if there are no more cached records in NdbApi
    */
@@ -372,9 +379,9 @@ class NdbScanOperation : public NdbOperation {
 
   /*
    * NdbRecord version of nextResult.
-   *
-   * When 0 is returned, this method updates out_row_ptr to point
-   * to the next result row.  The location pointed to is valid
+   * 
+   * When 0 is returned, this method updates out_row_ptr to point 
+   * to the next result row.  The location pointed to is valid 
    * (only) until the next call to nextResult() with
    * fetchAllowed == true.
    * The NdbRecord object defining the row format was specified in the
@@ -383,7 +390,9 @@ class NdbScanOperation : public NdbOperation {
    * all must be supplied to avoid invoking the two-parameter, non
    * NdbRecord variant of nextResult.
    */
-  int nextResult(const char **out_row_ptr, bool fetchAllowed, bool forceSend);
+  int nextResult(const char ** out_row_ptr,
+                 bool fetchAllowed, 
+                 bool forceSend);
 
   /*
    * Alternate NdbRecord version of nextResult.
@@ -400,7 +409,9 @@ class NdbScanOperation : public NdbOperation {
    * -   1: if there are no more tuples to scan
    * -   2: if there are no more cached records in NdbApi
    */
-  int nextResultCopyOut(char *buffer, bool fetchAllowed, bool forceSend);
+  int nextResultCopyOut(char * buffer,
+                 bool fetchAllowed,
+                 bool forceSend);
 
   /**
    * Close scan
@@ -412,7 +423,7 @@ class NdbScanOperation : public NdbOperation {
    *
    * @return an NdbOperation or NULL.
    */
-  NdbOperation *lockCurrentTuple();
+  NdbOperation* lockCurrentTuple();
   /**
    * Lock current tuple
    *
@@ -420,13 +431,13 @@ class NdbScanOperation : public NdbOperation {
    *
    * @return an NdbOperation or NULL.
    */
-  NdbOperation *lockCurrentTuple(NdbTransaction *lockTrans);
+  NdbOperation*	lockCurrentTuple(NdbTransaction* lockTrans);
   /**
    * Update current tuple
    *
    * @return an NdbOperation or NULL.
    */
-  NdbOperation *updateCurrentTuple();
+  NdbOperation* updateCurrentTuple();
   /**
    * Update current tuple
    *
@@ -434,7 +445,7 @@ class NdbScanOperation : public NdbOperation {
    *
    * @return an NdbOperation or NULL.
    */
-  NdbOperation *updateCurrentTuple(NdbTransaction *updateTrans);
+  NdbOperation*	updateCurrentTuple(NdbTransaction* updateTrans);
 
   /**
    * Delete current tuple
@@ -448,8 +459,8 @@ class NdbScanOperation : public NdbOperation {
    *
    * @return 0 on success or -1 on failure
    */
-  int deleteCurrentTuple(NdbTransaction *takeOverTransaction);
-
+  int deleteCurrentTuple(NdbTransaction* takeOverTransaction);
+  
   /*
    * NdbRecord versions of scan lock take-over operations.
    *
@@ -460,45 +471,48 @@ class NdbScanOperation : public NdbOperation {
 
   /*
    * Take over the lock without changing the row.
-   * Optionally also read from the row (call with default value NULL for
+   * Optionally also read from the row (call with default value NULL for 
    * result_row to not read any attributes.).
    * The NdbRecord * is required even when not reading any attributes.
    * Supported OperationOptions : OO_ABORTOPTION, OO_GETVALUE, OO_ANYVALUE
    */
-  const NdbOperation *lockCurrentTuple(
-      NdbTransaction *takeOverTrans, const NdbRecord *result_rec,
-      char *result_row = nullptr, const unsigned char *result_mask = nullptr,
-      const NdbOperation::OperationOptions *opts = nullptr,
-      Uint32 sizeOfOptions = 0);
+  const NdbOperation *lockCurrentTuple(NdbTransaction *takeOverTrans,
+                                       const NdbRecord *result_rec,
+                                       char *result_row= 0,
+                                       const unsigned char *result_mask= 0,
+                                       const NdbOperation::OperationOptions *opts = 0,
+                                       Uint32 sizeOfOptions = 0);
 
   /*
    * Update the current tuple, NdbRecord version.
    * Values to update with are contained in the passed-in row.
-   * Supported OperationOptions : OO_ABORTOPTION, OO_SETVALUE,
+   * Supported OperationOptions : OO_ABORTOPTION, OO_SETVALUE, 
    *                              OO_INTERPRETED, OO_ANYVALUE
    */
-  const NdbOperation *updateCurrentTuple(
-      NdbTransaction *takeOverTrans, const NdbRecord *attr_rec,
-      const char *attr_row, const unsigned char *mask = nullptr,
-      const NdbOperation::OperationOptions *opts = nullptr,
-      Uint32 sizeOfOptions = 0);
+  const NdbOperation *updateCurrentTuple(NdbTransaction *takeOverTrans,
+                                         const NdbRecord *attr_rec,
+                                         const char *attr_row,
+                                         const unsigned char *mask= 0,
+                                         const NdbOperation::OperationOptions *opts = 0,
+                                         Uint32 sizeOfOptions = 0);
 
   /* Delete the current tuple. NdbRecord version.
    * The tuple can be read before being deleted.  Specify the columns to read
-   * and the result storage as usual with result_rec, result_row and
-   * result_mask. Supported OperationOptions : OO_ABORTOPTION, OO_GETVALUE,
-   * OO_ANYVALUE
+   * and the result storage as usual with result_rec, result_row and result_mask.
+   * Supported OperationOptions : OO_ABORTOPTION, OO_GETVALUE, OO_ANYVALUE
    */
-  const NdbOperation *deleteCurrentTuple(
-      NdbTransaction *takeOverTrans, const NdbRecord *result_rec,
-      char *result_row = nullptr, const unsigned char *result_mask = nullptr,
-      const NdbOperation::OperationOptions *opts = nullptr,
-      Uint32 sizeOfOptions = 0);
+  const NdbOperation *deleteCurrentTuple(NdbTransaction *takeOverTrans,
+                                         const NdbRecord *result_rec,
+                                         char *result_row = 0,
+                                         const unsigned char *result_mask = 0,
+                                         const NdbOperation::OperationOptions *opts = 0,
+                                         Uint32 sizeOfOptions = 0);
 
   /**
    * Get NdbTransaction object for this scan operation
    */
-  NdbTransaction *getNdbTransaction() const override;
+  NdbTransaction* getNdbTransaction() const;
+
 
   /**
    * Is scan operation pruned to a single table partition?
@@ -507,45 +521,48 @@ class NdbScanOperation : public NdbOperation {
    */
   bool getPruned() const;
 
- protected:
-  NdbScanOperation(Ndb *aNdb,
+protected:
+  NdbScanOperation(Ndb* aNdb,
                    NdbOperation::Type aType = NdbOperation::TableScan);
-  ~NdbScanOperation() override;
+  virtual ~NdbScanOperation();
 
-  NdbRecAttr *getValue_impl(const NdbColumnImpl *,
-                            char *aValue = nullptr) override;
-  NdbRecAttr *getValue_NdbRecord_scan(const NdbColumnImpl *, char *aValue);
-  NdbRecAttr *getValue_NdbRecAttr_scan(const NdbColumnImpl *, char *aValue);
+  virtual NdbRecAttr* getValue_impl(const NdbColumnImpl*, char* aValue = 0);
+  NdbRecAttr* getValue_NdbRecord_scan(const NdbColumnImpl*, char* aValue);
+  NdbRecAttr* getValue_NdbRecAttr_scan(const NdbColumnImpl*, char* aValue);
 
   int handleScanGetValuesOldApi();
   int addInterpretedCode();
-  int handleScanOptionsVersion(const ScanOptions *&optionsPtr,
-                               Uint32 sizeOfOptions, ScanOptions &currOptions);
+  int handleScanOptionsVersion(const ScanOptions*& optionsPtr, 
+                               Uint32 sizeOfOptions,
+                               ScanOptions& currOptions);
   int handleScanOptions(const ScanOptions *options);
-  int validatePartInfoPtr(const Ndb::PartitionSpec *&partInfo,
-                          Uint32 sizeOfPartInfo, Ndb::PartitionSpec &partValue);
-  int getPartValueFromInfo(const Ndb::PartitionSpec *partInfo,
-                           const NdbTableImpl *table, Uint32 *partValue);
-  int generatePackedReadAIs(const NdbRecord *reseult_record, bool &haveBlob,
-                            const Uint32 *readMask);
-  int scanImpl(const NdbScanOperation::ScanOptions *options,
-               const Uint32 *readMask);
+  int validatePartInfoPtr(const Ndb::PartitionSpec*& partInfo,
+                          Uint32 sizeOfPartInfo,
+                          Ndb::PartitionSpec& partValue);
+  int getPartValueFromInfo(const Ndb::PartitionSpec* partInfo,
+                           const NdbTableImpl* table,
+                           Uint32* partValue);
+  int generatePackedReadAIs(const NdbRecord *reseult_record, bool& haveBlob,
+                            const Uint32 * readMask);
+  int scanImpl(const NdbScanOperation::ScanOptions *options, 
+               const Uint32 * readMask);
   int scanTableImpl(const NdbRecord *result_record,
                     NdbOperation::LockMode lock_mode,
                     const unsigned char *result_mask,
                     const NdbScanOperation::ScanOptions *options,
                     Uint32 sizeOfOptions);
 
-  int nextResultNdbRecord(const char *&out_row, bool fetchAllowed,
-                          bool forceSend);
-  void release() override;
-
-  int close_impl(bool forceSend, PollGuard *poll_guard);
+  int nextResultNdbRecord(const char * & out_row,
+                          bool fetchAllowed, bool forceSend);
+  virtual void release();
+  
+  int close_impl(bool forceSend,
+                 PollGuard *poll_guard);
 
   /* Helper for NdbScanFilter to allocate an InterpretedCode
    * object owned by the Scan operation
    */
-  NdbInterpretedCode *allocInterpretedCodeOldApi();
+  NdbInterpretedCode* allocInterpretedCodeOldApi();
   void freeInterpretedCodeOldApi();
 
   int doSendSetAISectionSizes();
@@ -554,15 +571,15 @@ class NdbScanOperation : public NdbOperation {
   int executeCursor(int ProcessorId);
 
   // Overloaded private methods from NdbOperation
-  int init(const NdbTableImpl *tab, NdbTransaction *);
-  int prepareSend(Uint32 TC_ConnectPtr, Uint64 TransactionId,
-                  NdbOperation::AbortOption) override;
+  int init(const NdbTableImpl* tab, NdbTransaction*);
+  int prepareSend(Uint32  TC_ConnectPtr, Uint64  TransactionId,
+                  NdbOperation::AbortOption);
   int doSend(int ProcessorId);
-  void setReadLockMode(LockMode lockMode) override;
-
-  void setErrorCode(int aErrorCode) const override;
-  void setErrorCodeAbort(int aErrorCode) const override;
+  virtual void setReadLockMode(LockMode lockMode);
 
+  virtual void setErrorCode(int aErrorCode) const;
+  virtual void setErrorCodeAbort(int aErrorCode) const;
+  
   /* This is the transaction which defined this scan
    *   The transaction(connection) used for the scan is
    *   pointed to by NdbOperation::theNdbCon
@@ -584,15 +601,15 @@ class NdbScanOperation : public NdbOperation {
   void finaliseScan();
   int finaliseScanOldApi();
   int prepareSendScan(Uint32 TC_ConnectPtr, Uint64 TransactionId,
-                      const Uint32 *readMask);
-
+                      const Uint32 * readMask);
+  
   int fix_receivers(Uint32 parallel);
   void reset_receivers(Uint32 parallel, Uint32 ordered);
-  Uint32 *m_array;  // containing all arrays below
+  Uint32* m_array; // containing all arrays below
   Uint32 m_allocated_receivers;
-  NdbReceiver **m_receivers;  // All receivers
+  NdbReceiver** m_receivers;      // All receivers
 
-  Uint32 *m_prepared_receivers;  // These are to be sent
+  Uint32* m_prepared_receivers;   // These are to be sent
 
   /*
     Owned by API/user thread.
@@ -617,8 +634,8 @@ class NdbScanOperation : public NdbOperation {
    */
   Uint32 m_current_api_receiver;
   Uint32 m_api_receivers_count;
-  NdbReceiver **m_api_receivers;  // These are currently used by api
-
+  NdbReceiver** m_api_receivers;  // These are currently used by api
+  
   /*
     Shared by receiver thread and API thread.
     These are receivers that the receiver thread has obtained all attribute
@@ -626,29 +643,32 @@ class NdbScanOperation : public NdbOperation {
     API thread will move them (under mutex protection) to m_api_receivers on
     first access with nextResult().
    */
-  Uint32 m_conf_receivers_count;   // NOTE needs mutex to access
-  NdbReceiver **m_conf_receivers;  // receive thread puts them here
-
+  Uint32 m_conf_receivers_count;  // NOTE needs mutex to access
+  NdbReceiver** m_conf_receivers; // receive thread puts them here
+  
   /*
    Owned by receiver thread
    These are the receivers that the receiver thread is currently receiving
    attribute data for (of the current batch).
    Once all is received, they will be moved to m_conf_receivers.
   */
-  Uint32 m_sent_receivers_count;   // NOTE needs mutex to access
-  NdbReceiver **m_sent_receivers;  // receive thread puts them here
-
+  Uint32 m_sent_receivers_count;  // NOTE needs mutex to access
+  NdbReceiver** m_sent_receivers; // receive thread puts them here
+  
   int send_next_scan(Uint32 cnt, bool close);
-  void receiver_delivered(NdbReceiver *);
-  void receiver_completed(NdbReceiver *);
+  void receiver_delivered(NdbReceiver*);
+  void receiver_completed(NdbReceiver*);
   void execCLOSE_SCAN_REP();
 
-  int getKeyFromKEYINFO20(Uint32 *data, Uint32 &size);
-  NdbOperation *takeOverScanOp(OperationType opType, NdbTransaction *);
-  NdbOperation *takeOverScanOpNdbRecord(
-      OperationType opType, NdbTransaction *pTrans, const NdbRecord *record,
-      char *row, const unsigned char *mask,
-      const NdbOperation::OperationOptions *opts, Uint32 sizeOfOptions);
+  int getKeyFromKEYINFO20(Uint32* data, Uint32 & size);
+  NdbOperation*	takeOverScanOp(OperationType opType, NdbTransaction*);
+  NdbOperation* takeOverScanOpNdbRecord(OperationType opType,
+                                        NdbTransaction* pTrans,
+                                        const NdbRecord *record,
+                                        char *row,
+                                        const unsigned char *mask,
+                                        const NdbOperation::OperationOptions *opts,
+                                        Uint32 sizeOfOptions);
   bool m_ordered;
   bool m_descending;
   Uint32 m_read_range_no;
@@ -659,25 +679,27 @@ class NdbScanOperation : public NdbOperation {
     implicitly when using LM_Exclusive).
     Then comes range_no, if requested with SF_ReadRangeNo, included first in
     the list of sort columns to get sorting of multiple range scans right.
-    Then the 'real' columns that are participating in the scan.
+    Then the 'real' columns that are participating in the scan.    
   */
   NdbRecAttr *m_curr_row;
-  bool m_multi_range;  // Mark if operation is part of multi-range scan
-  bool m_executed;     // Marker if operation should be released at close
+  bool m_multi_range; // Mark if operation is part of multi-range scan
+  bool m_executed; // Marker if operation should be released at close
 
-  /* Buffer given to NdbReceivers for batch of rows received
+  /* Buffer given to NdbReceivers for batch of rows received 
      during NdbRecord scans, or NULL. Buffer is chunked up
      to construct several NdbReceiverBuffer, but is allocated
      as a single chunk from the NdbScanOperation
   */
   Uint32 *m_scan_buffer;
-
+  
   /* Initialise scan operation with user provided information */
-  virtual int processTableScanDefs(LockMode lock_mode, Uint32 scan_flags,
-                                   Uint32 parallel, Uint32 batch);
+  virtual int processTableScanDefs(LockMode lock_mode, 
+                                   Uint32 scan_flags, 
+                                   Uint32 parallel,
+                                   Uint32 batch);
 
   /* This flag indicates whether a scan operation is using the old API */
-  bool m_scanUsingOldApi;
+  bool  m_scanUsingOldApi;
 
   /* Whether readTuples has been called - only valid for old Api scans */
   bool m_readTuplesCalled;
@@ -691,27 +713,26 @@ class NdbScanOperation : public NdbOperation {
   /* NdbInterpretedCode object owned by ScanOperation to support
    * old NdbScanFilter Api
    */
-  NdbInterpretedCode *m_interpretedCodeOldApi;
+  NdbInterpretedCode* m_interpretedCodeOldApi;
 
   enum ScanPruningState {
-    SPS_UNKNOWN,         // Initial state
-    SPS_FIXED,           // Explicit partitionId passed in ScanOptions
-    SPS_ONE_PARTITION,   // Scan pruned to one partition by previous range
-    SPS_MULTI_PARTITION  // Scan cannot be pruned due to previous ranges
+    SPS_UNKNOWN,           // Initial state
+    SPS_FIXED,             // Explicit partitionId passed in ScanOptions
+    SPS_ONE_PARTITION,     // Scan pruned to one partition by previous range
+    SPS_MULTI_PARTITION    // Scan cannot be pruned due to previous ranges
   };
-
+  
   ScanPruningState m_pruneState;
   Uint32 m_pruningKey;  // Can be distr key hash or actual partition id.
 
   /**
-   * This flag indicates whether a scan operation was
-   * successfully finalised
+   * This flag indicates whether a scan operation was 
+   * succesfully finalised
    */
-  bool m_scanFinalisedOk;
-
- private:
-  NdbScanOperation(const NdbScanOperation &);  // Not impl.
-  NdbScanOperation &operator=(const NdbScanOperation &);
+  bool  m_scanFinalisedOk;
+private:
+  NdbScanOperation(const NdbScanOperation&); // Not impl.
+  NdbScanOperation&operator=(const NdbScanOperation&);
 
   /**
    * Const variants overloaded...calling NdbOperation::getBlobHandle()
@@ -721,42 +742,61 @@ class NdbScanOperation : public NdbOperation {
    * I'm not sure...but these doesn't seem to be an users of this...
    * so I make them private...
    */
-  NdbBlob *getBlobHandle(const char *anAttrName) const override;
-  NdbBlob *getBlobHandle(Uint32 anAttrId) const override;
+  virtual NdbBlob* getBlobHandle(const char* anAttrName) const;
+  virtual NdbBlob* getBlobHandle(Uint32 anAttrId) const;
 };
 
-inline NdbOperation *NdbScanOperation::lockCurrentTuple() {
+inline
+NdbOperation* 
+NdbScanOperation::lockCurrentTuple(){
   return lockCurrentTuple(m_transConnection);
 }
 
-inline NdbOperation *NdbScanOperation::lockCurrentTuple(
-    NdbTransaction *takeOverTrans) {
-  return takeOverScanOp(NdbOperation::ReadRequest, takeOverTrans);
+inline
+NdbOperation* 
+NdbScanOperation::lockCurrentTuple(NdbTransaction* takeOverTrans){
+  return takeOverScanOp(NdbOperation::ReadRequest, 
+			takeOverTrans);
 }
 
-inline NdbOperation *NdbScanOperation::updateCurrentTuple() {
+inline
+NdbOperation* 
+NdbScanOperation::updateCurrentTuple(){
   return updateCurrentTuple(m_transConnection);
 }
 
-inline NdbOperation *NdbScanOperation::updateCurrentTuple(
-    NdbTransaction *takeOverTrans) {
-  return takeOverScanOp(NdbOperation::UpdateRequest, takeOverTrans);
+inline
+NdbOperation* 
+NdbScanOperation::updateCurrentTuple(NdbTransaction* takeOverTrans){
+  return takeOverScanOp(NdbOperation::UpdateRequest, 
+			takeOverTrans);
 }
 
-inline int NdbScanOperation::deleteCurrentTuple() {
+inline
+int
+NdbScanOperation::deleteCurrentTuple(){
   return deleteCurrentTuple(m_transConnection);
 }
 
-inline int NdbScanOperation::deleteCurrentTuple(NdbTransaction *takeOverTrans) {
-  void *res = takeOverScanOp(NdbOperation::DeleteRequest, takeOverTrans);
-  if (res == nullptr) return -1;
+inline
+int
+NdbScanOperation::deleteCurrentTuple(NdbTransaction * takeOverTrans){
+  void * res = takeOverScanOp(NdbOperation::DeleteRequest, 
+			      takeOverTrans);
+  if(res == 0)
+    return -1;
   return 0;
 }
 
-inline const NdbOperation *NdbScanOperation::updateCurrentTuple(
-    NdbTransaction *takeOverTrans, const NdbRecord *attr_rec,
-    const char *attr_row, const unsigned char *mask,
-    const NdbOperation::OperationOptions *opts, Uint32 sizeOfOptions) {
+inline
+const NdbOperation *
+NdbScanOperation::updateCurrentTuple(NdbTransaction *takeOverTrans,
+                                     const NdbRecord *attr_rec,
+                                     const char *attr_row,
+                                     const unsigned char *mask,
+                                     const NdbOperation::OperationOptions *opts,
+                                     Uint32 sizeOfOptions)
+{
   /*
     We share the code implementing lockCurrentTuple() and updateCurrentTuple().
     For lock the row may be updated, for update it is const.
@@ -768,16 +808,24 @@ inline const NdbOperation *NdbScanOperation::updateCurrentTuple(
                                  opts, sizeOfOptions);
 }
 
-inline const NdbOperation *NdbScanOperation::deleteCurrentTuple(
-    NdbTransaction *takeOverTrans, const NdbRecord *result_rec,
-    char *result_row, const unsigned char *result_mask,
-    const NdbOperation::OperationOptions *opts, Uint32 sizeOfOptions) {
+inline
+const NdbOperation *
+NdbScanOperation::deleteCurrentTuple(NdbTransaction *takeOverTrans,
+                                     const NdbRecord *result_rec,
+                                     char *result_row,
+                                     const unsigned char *result_mask,
+                                     const NdbOperation::OperationOptions *opts,
+                                     Uint32 sizeOfOptions)
+{
   return takeOverScanOpNdbRecord(NdbOperation::DeleteRequest, takeOverTrans,
-                                 result_rec, result_row, result_mask, opts,
-                                 sizeOfOptions);
+                                 result_rec, result_row, result_mask,
+                                 opts, sizeOfOptions);
 }
 
-inline NdbTransaction *NdbScanOperation::getNdbTransaction() const {
+inline
+NdbTransaction*
+NdbScanOperation::getNdbTransaction() const
+{
   /* return the user visible transaction object ptr, not the
    * scan's 'internal' / buddy transaction object
    */
