@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2011, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2011, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,11 +26,9 @@
 #define NDB_PACK_HPP
 
 #include <ndb_global.h>
-#include <NdbSqlUtil.hpp>
-#include <kernel/AttributeHeader.hpp>
-#include "my_config.h"
 #include "my_sys.h"
-#include "portlib/ndb_compiler.h"
+#include <kernel/AttributeHeader.hpp>
+#include <NdbSqlUtil.hpp>
 
 class NdbOut;
 
@@ -59,7 +56,7 @@ class NdbOut;
  */
 
 class NdbPack {
- public:
+public:
   class Endian;
   class Type;
   class Spec;
@@ -74,31 +71,31 @@ class NdbPack {
   /*
    * Get SQL type.
    */
-  static const NdbSqlUtil::Type &getSqlType(Uint32 typeId);
+  static const NdbSqlUtil::Type& getSqlType(Uint32 typeId);
 
   /*
    * Error codes for core dumps.
    */
   class Error {
-   public:
+  public:
     enum {
-      TypeNotSet = -101,           // type id was not set
-      TypeOutOfRange = -102,       // type id is out of range
-      TypeNotSupported = -103,     // blob (and for now bit) types
-      TypeSizeZero = -104,         // max size was set to zero
-      TypeFixSizeInvalid = -105,   // fixed size specified wrong
-      TypeNullableNotBool = -106,  // nullable must be 0 or 1
-      CharsetNotSpecified = -107,  // char type with no charset number
-      CharsetNotFound = -108,      // cannot install in all_charsets[]
-      CharsetNotAllowed = -109,    // non-char type with charset
-      SpecBufOverflow = -201,      // more spec items than allocated
-      DataCntOverflow = -301,      // more data items than in spec
-      DataBufOverflow = -302,      // more data bytes than allocated
-      DataValueOverflow = -303,    // var length exceeds max size
-      DataNotNullable = -304,      // NULL value to not-nullable type
-      InvalidAttrInfo = -305,      // invalid plain old attr info
-      BoundEmptySide = -401,       // side not 0 for empty bound
-      BoundNonemptySide = -402,    // side not -1,+1 for non-empty bound
+      TypeNotSet = -101,          // type id was not set
+      TypeOutOfRange = -102,      // type id is out of range
+      TypeNotSupported = -103,    // blob (and for now bit) types
+      TypeSizeZero = -104,        // max size was set to zero
+      TypeFixSizeInvalid = -105,  // fixed size specified wrong
+      TypeNullableNotBool = -106, // nullable must be 0 or 1
+      CharsetNotSpecified = -107, // char type with no charset number
+      CharsetNotFound = -108,     // cannot install in all_charsets[]
+      CharsetNotAllowed = -109,   // non-char type with charset
+      SpecBufOverflow = -201,     // more spec items than allocated
+      DataCntOverflow = -301,     // more data items than in spec
+      DataBufOverflow = -302,     // more data bytes than allocated
+      DataValueOverflow = -303,   // var length exceeds max size
+      DataNotNullable = -304,     // NULL value to not-nullable type
+      InvalidAttrInfo = -305,     // invalid plain old attr info
+      BoundEmptySide = -401,      // side not 0 for empty bound
+      BoundNonemptySide = -402,   // side not -1,+1 for non-empty bound
       InternalError = -901,
       ValidationError = -902,
       NoError = 0
@@ -107,7 +104,7 @@ class NdbPack {
     int get_error_code() const;
     int get_error_line() const;
 
-   private:
+  private:
     friend class Endian;
     friend class Type;
     friend class Spec;
@@ -117,7 +114,7 @@ class NdbPack {
     friend class BoundC;
     friend class Bound;
     void set_error(int code, int line) const;
-    void set_error(const Error &e2) const;
+    void set_error(const Error& e2) const;
     mutable int m_error_code;
     mutable int m_error_line;
   };
@@ -126,21 +123,21 @@ class NdbPack {
    * Endian definitions.
    */
   class Endian {
-   public:
+  public:
     enum Value {
-      Native = 0,  // replaced by actual value
+      Native = 0, // replaced by actual value
       Little = 1,
       Big = 2
     };
     static Value get_endian();
-    static void convert(void *ptr, Uint32 len);
+    static void convert(void* ptr, Uint32 len);
   };
 
   /*
    * Data type.
    */
   class Type : public Error {
-   public:
+  public:
     Type();
     Type(int typeId, Uint32 byteSize, bool nullable, Uint32 csNumber);
     /*
@@ -157,12 +154,12 @@ class NdbPack {
     Uint32 get_cs_number() const;
     Uint32 get_array_type() const;
     // print
-    friend NdbOut &operator<<(NdbOut &, const Type &);
-    void print(NdbOut &out) const;
-    const char *print(char *buf, Uint32 bufsz) const;
+    friend NdbOut& operator<<(NdbOut&, const Type&);
+    void print(NdbOut& out) const;
+    const char* print(char* buf, Uint32 bufsz) const;
     int validate() const;
 
-   private:
+  private:
     friend class Spec;
     friend class Iter;
     friend class DataC;
@@ -171,7 +168,7 @@ class NdbPack {
     // verify and complete when added to specification
     int complete();
     Uint16 m_typeId;
-    Uint16 m_byteSize;  // fixed or max size in bytes
+    Uint16 m_byteSize;    // fixed or max size in bytes
     Uint16 m_nullable;
     Uint16 m_csNumber;
     Uint16 m_arrayType;   // 0,1,2 length bytes
@@ -180,25 +177,25 @@ class NdbPack {
 
   /*
    * Data specification i.e. array of types.  Usually constructed on the
-   * heap, so keep fairly small.  Used for both keys and bounds.
+   * heap, so keep fairly small.  Used for boths keys and bounds.
    */
   class Spec : public Error {
-   public:
+  public:
     Spec();
     ~Spec() {}
     // set initial buffer (calls reset)
-    void set_buf(Type *buf, Uint32 bufMaxCnt);
+    void set_buf(Type* buf, Uint32 bufMaxCnt);
     // use if buffer is relocated
-    void set_buf(Type *buf);
+    void set_buf(Type* buf);
     // reset but keep buffer
     void reset();
     // add type to specification once or number of times
     int add(Type type);
     int add(Type type, Uint32 cnt);
     // copy from
-    void copy(const Spec &s2);
+    void copy(const Spec& s2);
     // getters (bounds set allNullable)
-    const Type &get_type(Uint32 i) const;
+    const Type& get_type(Uint32 i) const;
     Uint32 get_cnt() const;
     Uint32 get_nullable_cnt(bool allNullable) const;
     Uint32 get_nullmask_len(bool allNullable) const;
@@ -207,26 +204,26 @@ class NdbPack {
     // minimum var bytes (if used by Data instance)
     Uint32 get_min_var_bytes(bool allNullable) const;
     // print
-    friend NdbOut &operator<<(NdbOut &, const Spec &);
-    void print(NdbOut &out) const;
-    const char *print(char *buf, Uint32 bufsz) const;
+    friend NdbOut& operator<<(NdbOut&, const Spec&);
+    void print(NdbOut& out) const;
+    const char* print(char* buf, Uint32 bufsz) const;
     int validate() const;
 
-   private:
+  private:
     friend class Iter;
     friend class DataC;
     friend class Data;
     friend class BoundC;
     friend class DataArray;
     // undefined
-    Spec(const Spec &);
-    Spec &operator=(const Spec &);
-    Type *m_buf;
+    Spec(const Spec&);
+    Spec& operator=(const Spec&);
+    Type* m_buf;
     Uint16 m_bufMaxCnt;
     Uint16 m_cnt;
     Uint16 m_nullableCnt;
     Uint16 m_varsizeCnt;
-    Uint32 m_maxByteSize;  // excludes null mask
+    Uint32 m_maxByteSize; // excludes null mask
   };
 
   /*
@@ -235,36 +232,36 @@ class NdbPack {
    * iterates on items added.
    */
   class Iter : public Error {
-   public:
+  public:
     // the data instance is only used to set metadata
-    Iter(const DataC &data);
+    Iter(const DataC& data);
     ~Iter() {}
     void reset();
 
-   private:
+  private:
     friend class DataC;
     friend class Data;
     friend class BoundC;
     friend class DataArray;
     // undefined
-    Iter(const Iter &);
-    Iter &operator=(const Iter &);
+    Iter(const Iter&);
+    Iter& operator=(const Iter&);
     // describe next non-null or null item and advance iterator
-    int desc(const Uint8 *item);
+    int desc(const Uint8* item);
     int desc_null();
     // compare current items (DataC buffers are passed)
-    int cmp(const Iter &r2, const Uint8 *buf1, const Uint8 *buf2) const;
+    int cmp(const Iter& r2, const Uint8* buf1, const Uint8* buf2) const;
 
-    const Spec &m_spec;
+    const Spec& m_spec;
     const bool m_allNullable;
     // iterator
-    Uint32 m_itemPos;  // position of current item in DataC buffer
-    Uint32 m_cnt;      // number of items described so far
+    Uint32 m_itemPos;     // position of current item in DataC buffer
+    Uint32 m_cnt;         // number of items described so far
     Uint32 m_nullCnt;
     // current item
-    Uint32 m_lenBytes;  // 0-2
-    Uint32 m_bareLen;   // excludes length bytes
-    Uint32 m_itemLen;   // full length, value zero means null
+    Uint32 m_lenBytes;    // 0-2
+    Uint32 m_bareLen;     // excludes length bytes
+    Uint32 m_itemLen;     // full length, value zero means null
   };
 
   /*
@@ -273,37 +270,37 @@ class NdbPack {
    * one data item at a time.  Values are native endian.
    */
   class DataC : public Error {
-   public:
-    DataC(const Spec &spec, bool allNullable);
+  public:
+    DataC(const Spec& spec, bool allNullable);
     // set buffer to previously constructed one with given item count
-    void set_buf(const void *buf, Uint32 bufMaxLen, Uint32 cnt);
+    void set_buf(const void* buf, Uint32 bufMaxLen, Uint32 cnt);
     // interpret next data item
-    int desc(Iter &r) const;
+    int desc(Iter& r) const;
     // compare cnt attrs and also return number of initial equal attrs
-    int cmp(const DataC &d2, Uint32 cnt, Uint32 &num_eq) const;
+    int cmp(const DataC& d2, Uint32 cnt, Uint32& num_eq) const;
     // getters
-    const Spec &get_spec() const;
-    const void *get_data_buf() const;
+    const Spec& get_spec() const;
+    const void* get_data_buf() const;
     Uint32 get_cnt() const;
     bool is_empty() const;
     bool is_full() const;
     // print
-    friend NdbOut &operator<<(NdbOut &, const DataC &);
-    void print(NdbOut &out) const;
-    const char *print(char *buf, Uint32 bufsz, bool convert_flag = false) const;
+    friend NdbOut& operator<<(NdbOut&, const DataC&);
+    void print(NdbOut& out) const;
+    const char* print(char* buf, Uint32 bufsz, bool convert_flag = false) const;
     int validate() const { return 0; }
 
-   private:
+  private:
     friend class Iter;
     friend class Data;
     friend class BoundC;
     friend class DataArray;
     // undefined
-    DataC(const Data &);
-    DataC &operator=(const DataC &);
-    const Spec &m_spec;
+    DataC(const Data&);
+    DataC& operator=(const DataC&);
+    const Spec& m_spec;
     const bool m_allNullable;
-    const Uint8 *m_buf;
+    const Uint8* m_buf;
     Uint32 m_bufMaxLen;
     // can be updated as part of Data instance
     Uint32 m_cnt;
@@ -319,25 +316,25 @@ class NdbPack {
    * first be completed with desc_all() and convert().
    */
   class Data : public DataC {
-   public:
-    Data(const Spec &spec, bool allNullable, Uint32 varBytes);
+  public:
+    Data(const Spec& spec, bool allNullable, Uint32 varBytes);
     // set buffer (calls reset)
-    void set_buf(void *buf, Uint32 bufMaxLen);
+    void set_buf(void* buf, Uint32 bufMaxLen);
     // reset but keep buffer (header is zeroed)
     void reset();
     // add non-null data items and return length in bytes
-    int add(const void *data, Uint32 *len_out);
-    int add(const void *data, Uint32 cnt, Uint32 *len_out);
+    int add(const void* data, Uint32* len_out);
+    int add(const void* data, Uint32 cnt, Uint32* len_out);
     // add null data items and return length 0 bytes
-    int add_null(Uint32 *len_out);
-    int add_null(Uint32 cnt, Uint32 *len_out);
+    int add_null(Uint32* len_out);
+    int add_null(Uint32 cnt, Uint32* len_out);
     // add from "plain old attr info"
-    int add_poai(const Uint32 *poai, Uint32 *len_out);
-    int add_poai(const Uint32 *poai, Uint32 cnt, Uint32 *len_out);
+    int add_poai(const Uint32* poai, Uint32* len_out);
+    int add_poai(const Uint32* poai, Uint32 cnt, Uint32* len_out);
     // call this before first use
     int finalize();
     // copy from
-    int copy(const DataC &d2);
+    int copy(const DataC& d2);
     // convert endian
     int convert(Endian::Value to_endian);
     // create complete instance from buffer contents
@@ -346,28 +343,27 @@ class NdbPack {
     Uint32 get_max_len() const;
     Uint32 get_max_len4() const;
     Uint32 get_var_bytes() const;
-    void *get_full_buf();
-    const void *get_full_buf() const;
+    const void* get_full_buf() const;
     Uint32 get_full_len() const;
     Uint32 get_data_len() const;
     Uint32 get_null_cnt() const;
     Endian::Value get_endian() const;
     // print
-    friend NdbOut &operator<<(NdbOut &, const Data &);
-    void print(NdbOut &out) const;
-    const char *print(char *buf, Uint32 bufsz) const;
+    friend NdbOut& operator<<(NdbOut&, const Data&);
+    void print(NdbOut& out) const;
+    const char* print(char* buf, Uint32 bufsz) const;
     int validate() const;
 
-   private:
+  private:
     friend class Iter;
     friend class Bound;
     // undefined
-    Data(const Data &);
-    Data &operator=(const Data &);
+    Data(const Data&);
+    Data& operator=(const Data&);
     int finalize_impl();
-    int convert_impl();
+    int convert_impl(Endian::Value to_endian);
     const Uint32 m_varBytes;
-    Uint8 *m_buf;
+    Uint8* m_buf;
     Uint32 m_bufMaxLen;
     Endian::Value m_endian;
     // iterator on items added
@@ -379,31 +375,31 @@ class NdbPack {
    * from a previously constructed Bound or DataC buffer.
    */
   class BoundC : public Error {
-   public:
-    BoundC(DataC &data);
+  public:
+    BoundC(DataC& data);
     ~BoundC() {}
     // call this before first use
     int finalize(int side);
     // compare bound to key (may return 0 if bound is longer)
-    int cmp(const DataC &d2, Uint32 cnt, Uint32 &num_eq) const;
+    int cmp(const DataC& d2, Uint32 cnt, Uint32& num_eq) const;
     // compare bounds (may return 0 if cnt is less than min length)
-    int cmp(const BoundC &b2, Uint32 cnt, Uint32 &num_eq) const;
+    int cmp(const BoundC& b2, Uint32 cnt, Uint32& num_eq) const;
     // getters
-    DataC &get_data() const;
+    DataC& get_data() const;
     int get_side() const;
     // print
-    friend NdbOut &operator<<(NdbOut &, const BoundC &);
-    void print(NdbOut &out) const;
-    const char *print(char *buf, Uint32 bufsz) const;
+    friend NdbOut& operator<<(NdbOut&, const BoundC&);
+    void print(NdbOut& out) const;
+    const char* print(char* buf, Uint32 bufsz) const;
     int validate() const;
 
-   private:
+  private:
     friend class Bound;
     friend class DataArray;
     // undefined
-    BoundC(const BoundC &);
-    BoundC &operator=(const BoundC &);
-    DataC &m_data;
+    BoundC(const BoundC&);
+    BoundC& operator=(const BoundC&);
+    DataC& m_data;
     int m_side;
   };
 
@@ -414,25 +410,25 @@ class NdbPack {
    * passed to Bound ctor by reference.
    */
   class Bound : public BoundC {
-   public:
-    Bound(Data &data);
+  public:
+    Bound(Data& data);
     ~Bound() {}
     void reset();
     // call this before first use
     int finalize(int side);
     // getters
-    Data &get_data() const;
+    Data& get_data() const;
     // print
-    friend NdbOut &operator<<(NdbOut &, const Bound &);
-    void print(NdbOut &out) const;
-    const char *print(char *buf, Uint32 bufsz) const;
+    friend NdbOut& operator<<(NdbOut&, const Bound&);
+    void print(NdbOut& out) const;
+    const char* print(char* buf, Uint32 bufsz) const;
     int validate() const;
 
-   private:
+  private:
     // undefined
-    Bound(const Bound &);
-    Bound &operator=(const Bound &);
-    Data &m_data;
+    Bound(const Bound&);
+    Bound& operator=(const Bound&);
+    Data& m_data;
   };
 
   /**
@@ -451,54 +447,58 @@ class NdbPack {
    * Bound supplied for a scan and we can build it for
    * searches to update the index.
    */
-  class DataEntry {
-   public:
+  class DataEntry
+  {
+  public:
     DataEntry() {}
     ~DataEntry() {}
-
-   private:
+  private:
     friend class DataArray;
-    const Uint8 *m_data_ptr;
+    Uint8* m_data_ptr;
     Uint32 m_data_len;
   };
 
-  class DataArray {
-   public:
+  class DataArray
+  {
+  public:
     DataArray() {}
     ~DataArray() {}
-    void init_poai(const Uint32 *buf, const Uint32 cnt);
-    void init_bound(const BoundC &, const Uint32 cnt);
-    int cmp(const Spec *spec, const DataArray *d2, const Uint32 cnt) const;
+    void init_poai(const Uint32* buf, const Uint32 cnt);
+    void init_bound(const BoundC&, const Uint32 cnt);
+    int cmp(const Spec* spec,
+            const DataArray* d2,
+            const Uint32 cnt) const;
     Uint32 cnt() const;
     Uint32 get_null_cnt() const;
     Uint32 get_data_len() const;
-
-   private:
+  private:
     friend class BoundArray;
     Uint32 m_cnt;
     Uint32 m_null_cnt;
     DataEntry m_entries[MAX_ATTRIBUTES_IN_INDEX];
   };
 
-  class BoundArray {
-   public:
-    BoundArray();
-    BoundArray(const Spec *, const DataArray *, const int side);
-    ~BoundArray() {}
-    int cmp(const DataArray *d2, const Uint32 cnt, bool ok_to_ret_eq) const;
-    Uint32 cnt() const;
-
-   private:
-    const Spec *m_spec;
-    const DataArray *m_data_array;
-    const int m_side;
+  class BoundArray
+  {
+    public:
+      BoundArray();
+      BoundArray(const Spec*,
+                 const DataArray*,
+                 const int side);
+      ~BoundArray() {}
+      int cmp(const DataArray* d2, const Uint32 cnt, bool ok_to_ret_eq) const;
+      Uint32 cnt() const;
+    private:
+      const Spec* m_spec;
+      const DataArray* m_data_array;
+      const int m_side;
   };
 
   /*
    * Helper for print() methods.
    */
   struct Print {
-   private:
+  private:
     friend class Endian;
     friend class Type;
     friend class Spec;
@@ -507,9 +507,10 @@ class NdbPack {
     friend class Data;
     friend class BoundC;
     friend class Bound;
-    Print(char *buf, Uint32 bufsz);
-    void print(const char *frm, ...) ATTRIBUTE_FORMAT(printf, 2, 3);
-    char *m_buf;
+    Print(char* buf, Uint32 bufsz);
+    void print(const char* frm, ...)
+      ATTRIBUTE_FORMAT(printf, 2, 3);
+    char* m_buf;
     Uint32 m_bufsz;
     Uint32 m_sz;
   };
@@ -517,20 +518,26 @@ class NdbPack {
 
 // NdbPack
 
-inline const NdbSqlUtil::Type &NdbPack::getSqlType(Uint32 typeId) {
+inline const NdbSqlUtil::Type&
+NdbPack::getSqlType(Uint32 typeId)
+{
   return NdbSqlUtil::m_typeList[typeId];
 }
 
 // NdbPack::Error
 
-inline NdbPack::Error::Error() {
+inline
+NdbPack::Error::Error()
+{
   m_error_code = 0;
   m_error_line = 0;
 }
 
 // NdbPack::Endian
 
-inline NdbPack::Endian::Value NdbPack::Endian::get_endian() {
+inline NdbPack::Endian::Value
+NdbPack::Endian::get_endian()
+{
 #ifndef WORDS_BIGENDIAN
   return Little;
 #else
@@ -540,7 +547,9 @@ inline NdbPack::Endian::Value NdbPack::Endian::get_endian() {
 
 // NdbPack::Type
 
-inline NdbPack::Type::Type() {
+inline
+NdbPack::Type::Type()
+{
   m_typeId = NDB_TYPE_UNDEFINED;
   m_byteSize = 0;
   m_nullable = true;
@@ -549,87 +558,138 @@ inline NdbPack::Type::Type() {
   m_nullbitPos = 0;
 }
 
-inline NdbPack::Type::Type(int typeId, Uint32 byteSize, bool nullable,
-                           Uint32 csNumber) {
+inline
+NdbPack::Type::Type(int typeId, Uint32 byteSize, bool nullable, Uint32 csNumber)
+{
   set(typeId, byteSize, nullable, csNumber);
 }
 
-inline void NdbPack::Type::set(Uint32 typeId, Uint32 byteSize, bool nullable,
-                               Uint32 csNumber) {
+inline void
+NdbPack::Type::set(Uint32 typeId, Uint32 byteSize, bool nullable, Uint32 csNumber)
+{
   m_typeId = typeId;
   m_byteSize = byteSize;
   m_nullable = nullable;
   m_csNumber = csNumber;
 }
 
-inline Uint32 NdbPack::Type::get_type_id() const { return m_typeId; }
+inline Uint32
+NdbPack::Type::get_type_id() const
+{
+  return m_typeId;
+}
 
-inline Uint32 NdbPack::Type::get_byte_size() const { return m_byteSize; }
+inline Uint32
+NdbPack::Type::get_byte_size() const
+{
+  return m_byteSize;
+}
 
-inline bool NdbPack::Type::get_nullable() const { return (bool)m_nullable; }
+inline bool
+NdbPack::Type::get_nullable() const
+{
+  return (bool)m_nullable;
+}
 
-inline Uint32 NdbPack::Type::get_cs_number() const { return m_csNumber; }
+inline Uint32
+NdbPack::Type::get_cs_number() const
+{
+  return m_csNumber;
+}
 
-inline Uint32 NdbPack::Type::get_array_type() const { return m_arrayType; }
+inline Uint32
+NdbPack::Type::get_array_type() const
+{
+  return m_arrayType;
+}
 
 // NdbPack::Spec
 
-inline NdbPack::Spec::Spec() {
+inline
+NdbPack::Spec::Spec()
+{
   reset();
-  m_buf = nullptr;
+  m_buf = 0;
   m_bufMaxCnt = 0;
 }
 
-inline void NdbPack::Spec::set_buf(Type *buf, Uint32 bufMaxCnt) {
+inline void
+NdbPack::Spec::set_buf(Type* buf, Uint32 bufMaxCnt)
+{
   reset();
   m_buf = buf;
   m_bufMaxCnt = bufMaxCnt;
 }
 
-inline void NdbPack::Spec::set_buf(Type *buf) { m_buf = buf; }
+inline void
+NdbPack::Spec::set_buf(Type* buf)
+{
+  m_buf = buf;
+}
 
-inline void NdbPack::Spec::reset() {
+inline void
+NdbPack::Spec::reset()
+{
   m_cnt = 0;
   m_nullableCnt = 0;
   m_varsizeCnt = 0;
   m_maxByteSize = 0;
 }
 
-inline const NdbPack::Type &NdbPack::Spec::get_type(Uint32 i) const {
+inline const NdbPack::Type&
+NdbPack::Spec::get_type(Uint32 i) const
+{
   assert(i < m_cnt);
   return m_buf[i];
 }
 
-inline Uint32 NdbPack::Spec::get_cnt() const { return m_cnt; }
+inline Uint32
+NdbPack::Spec::get_cnt() const
+{
+  return m_cnt;
+}
 
-inline Uint32 NdbPack::Spec::get_nullable_cnt(bool allNullable) const {
+inline Uint32
+NdbPack::Spec::get_nullable_cnt(bool allNullable) const
+{
   if (!allNullable)
     return m_nullableCnt;
   else
     return m_cnt;
 }
 
-inline Uint32 NdbPack::Spec::get_nullmask_len(bool allNullable) const {
+inline Uint32
+NdbPack::Spec::get_nullmask_len(bool allNullable) const
+{
   return (get_nullable_cnt(allNullable) + 7) / 8;
 }
 
-inline Uint32 NdbPack::Spec::get_max_data_len(bool allNullable) const {
+inline Uint32
+NdbPack::Spec::get_max_data_len(bool allNullable) const
+{
   return get_nullmask_len(allNullable) + m_maxByteSize;
 }
 
-inline Uint32 NdbPack::Spec::get_min_var_bytes(bool allNullable) const {
+inline Uint32
+NdbPack::Spec::get_min_var_bytes(bool allNullable) const
+{
   const Uint32 len = get_max_data_len(allNullable);
   return (len < 256 ? 1 : 2);
 }
 
 // NdbPack::Iter
 
-inline NdbPack::Iter::Iter(const DataC &data)
-    : m_spec(data.m_spec), m_allNullable(data.m_allNullable) {
+inline
+NdbPack::Iter::Iter(const DataC& data) :
+  m_spec(data.m_spec),
+  m_allNullable(data.m_allNullable)
+{
   reset();
 }
 
-inline void NdbPack::Iter::reset() {
+inline void
+NdbPack::Iter::reset()
+{
   m_itemPos = m_spec.get_nullmask_len(m_allNullable);
   m_cnt = 0;
   m_nullCnt = 0;
@@ -640,48 +700,80 @@ inline void NdbPack::Iter::reset() {
 
 // NdbPack::DataC
 
-inline NdbPack::DataC::DataC(const Spec &spec, bool allNullable)
-    : m_spec(spec), m_allNullable(allNullable) {
-  m_buf = nullptr;
+inline
+NdbPack::DataC::DataC(const Spec& spec, bool allNullable) :
+  m_spec(spec),
+  m_allNullable(allNullable)
+{
+  m_buf = 0;
   m_bufMaxLen = 0;
   m_cnt = 0;
 }
 
-inline void NdbPack::DataC::set_buf(const void *buf, Uint32 bufMaxLen,
-                                    Uint32 cnt) {
-  m_buf = static_cast<const Uint8 *>(buf);
+inline void
+NdbPack::DataC::set_buf(const void* buf, Uint32 bufMaxLen, Uint32 cnt)
+{
+  m_buf = static_cast<const Uint8*>(buf);
   m_bufMaxLen = bufMaxLen;
   m_cnt = cnt;
 }
 
-inline const NdbPack::Spec &NdbPack::DataC::get_spec() const { return m_spec; }
+inline const NdbPack::Spec&
+NdbPack::DataC::get_spec() const
+{
+  return m_spec;
+}
 
-inline const void *NdbPack::DataC::get_data_buf() const { return &m_buf[0]; }
+inline const void*
+NdbPack::DataC::get_data_buf() const
+{
+  return &m_buf[0];
+}
 
-inline Uint32 NdbPack::DataC::get_cnt() const { return m_cnt; }
+inline Uint32
+NdbPack::DataC::get_cnt() const
+{
+  return m_cnt;
+}
 
-inline bool NdbPack::DataC::is_empty() const { return m_cnt == 0; }
+inline bool
+NdbPack::DataC::is_empty() const
+{
+  return m_cnt == 0;
+}
 
-inline bool NdbPack::DataC::is_full() const { return m_cnt == m_spec.m_cnt; }
+inline bool
+NdbPack::DataC::is_full() const
+{
+  return m_cnt == m_spec.m_cnt;
+}
 
 // NdbPack::Data
 
-inline NdbPack::Data::Data(const Spec &spec, bool allNullable, Uint32 varBytes)
-    : DataC(spec, allNullable), m_varBytes(varBytes), m_iter(*this) {
-  m_buf = nullptr;
+inline
+NdbPack::Data::Data(const Spec& spec, bool allNullable, Uint32 varBytes) :
+  DataC(spec, allNullable),
+  m_varBytes(varBytes),
+  m_iter(*this)
+{
+  m_buf = 0;
   m_bufMaxLen = 0;
   m_endian = Endian::get_endian();
 }
 
-inline void NdbPack::Data::set_buf(void *buf, Uint32 bufMaxLen) {
-  m_buf = static_cast<Uint8 *>(buf);
+inline void
+NdbPack::Data::set_buf(void* buf, Uint32 bufMaxLen)
+{
+  m_buf = static_cast<Uint8*>(buf);
   m_bufMaxLen = bufMaxLen;
   reset();
   assert(bufMaxLen >= m_varBytes);
   DataC::set_buf(&m_buf[m_varBytes], m_bufMaxLen - m_varBytes, 0);
 }
 
-inline void NdbPack::Data::reset() {
+inline void
+NdbPack::Data::reset()
+{
   m_cnt = 0;  // in DataC
   const Uint32 bytes = m_varBytes + m_spec.get_nullmask_len(m_allNullable);
   memset(m_buf, 0, bytes);
@@ -689,26 +781,39 @@ inline void NdbPack::Data::reset() {
   m_iter.reset();
 }
 
-inline int NdbPack::Data::finalize() {
-  if (likely(m_varBytes == 0 || finalize_impl() == 0)) return 0;
+inline int
+NdbPack::Data::finalize()
+{
+  if (likely(m_varBytes == 0 ||
+             finalize_impl() == 0))
+    return 0;
   return -1;
 }
 
-inline int NdbPack::Data::convert(Endian::Value to_endian) {
-  if (to_endian == Endian::Native) to_endian = Endian::get_endian();
-  if (m_endian == to_endian) return 0;
-  if (convert_impl() == 0) {
+inline int
+NdbPack::Data::convert(Endian::Value to_endian)
+{
+  if (to_endian == Endian::Native)
+    to_endian = Endian::get_endian();
+  if (m_endian == to_endian)
+    return 0;
+  if (convert_impl(to_endian) == 0)
+  {
     m_endian = to_endian;
     return 0;
   }
   return -1;
 }
 
-inline Uint32 NdbPack::Data::get_max_len() const {
+inline Uint32
+NdbPack::Data::get_max_len() const
+{
   return m_varBytes + m_spec.get_max_data_len(m_allNullable);
 }
 
-inline Uint32 NdbPack::Data::get_max_len4() const {
+inline Uint32
+NdbPack::Data::get_max_len4() const
+{
   Uint32 len4 = get_max_len();
   len4 += 3;
   len4 /= 4;
@@ -716,91 +821,164 @@ inline Uint32 NdbPack::Data::get_max_len4() const {
   return len4;
 }
 
-inline Uint32 NdbPack::Data::get_var_bytes() const { return m_varBytes; }
+inline Uint32
+NdbPack::Data::get_var_bytes() const
+{
+  return m_varBytes;
+}
 
-inline const void *NdbPack::Data::get_full_buf() const { return m_buf; }
+inline const void*
+NdbPack::Data::get_full_buf() const
+{
+  return &m_buf[0];
+}
 
-inline void *NdbPack::Data::get_full_buf() { return m_buf; }
-
-inline Uint32 NdbPack::Data::get_full_len() const {
+inline Uint32
+NdbPack::Data::get_full_len() const
+{
   return m_varBytes + m_iter.m_itemPos + m_iter.m_itemLen;
 }
 
-inline Uint32 NdbPack::Data::get_data_len() const {
+inline Uint32
+NdbPack::Data::get_data_len() const
+{
   return m_iter.m_itemPos + m_iter.m_itemLen;
 }
 
-inline Uint32 NdbPack::Data::get_null_cnt() const { return m_iter.m_nullCnt; }
+inline Uint32
+NdbPack::Data::get_null_cnt() const
+{
+  return m_iter.m_nullCnt;
+}
 
-inline NdbPack::Endian::Value NdbPack::Data::get_endian() const {
+inline NdbPack::Endian::Value
+NdbPack::Data::get_endian() const
+{
   return m_endian;
 }
 
 // NdbPack::BoundC
 
-inline NdbPack::BoundC::BoundC(DataC &data) : m_data(data) { m_side = 0; }
+inline
+NdbPack::BoundC::BoundC(DataC& data) :
+  m_data(data)
+{
+  m_side = 0;
+}
 
-inline int NdbPack::BoundC::cmp(const DataC &d2, Uint32 cnt,
-                                Uint32 &num_eq) const {
-  const BoundC &b1 = *this;
-  const DataC &d1 = b1.m_data;
+inline int
+NdbPack::BoundC::cmp(const DataC& d2, Uint32 cnt, Uint32& num_eq) const
+{
+  const BoundC& b1 = *this;
+  const DataC& d1 = b1.m_data;
   int res = d1.cmp(d2, cnt, num_eq);
-  if (res == 0 && d1.m_cnt <= d2.m_cnt) res = b1.m_side;
+  if (res == 0 && d1.m_cnt <= d2.m_cnt)
+    res = b1.m_side;
   return res;
 }
 
-inline NdbPack::DataC &NdbPack::BoundC::get_data() const { return m_data; }
+inline NdbPack::DataC&
+NdbPack::BoundC::get_data() const
+{
+  return m_data;
+}
 
-inline int NdbPack::BoundC::get_side() const { return m_side; }
+inline int
+NdbPack::BoundC::get_side() const
+{
+  return m_side;
+}
 
 // NdbPack::Bound
 
-inline NdbPack::Bound::Bound(Data &data) : BoundC(data), m_data(data) {}
+inline
+NdbPack::Bound::Bound(Data& data) :
+  BoundC(data),
+  m_data(data)
+{
+}
 
-inline void NdbPack::Bound::reset() {
+inline void
+NdbPack::Bound::reset()
+{
   m_data.reset();
   m_side = 0;
 }
 
-inline int NdbPack::Bound::finalize(int side) {
-  if (unlikely(m_data.finalize() == -1)) {
+inline int
+NdbPack::Bound::finalize(int side)
+{
+  if (unlikely(m_data.finalize() == -1))
+  {
     set_error(m_data);
     return -1;
   }
-  if (unlikely(BoundC::finalize(side) == -1)) return -1;
+  if (unlikely(BoundC::finalize(side) == -1))
+    return -1;
   return 0;
 }
 
-inline NdbPack::Data &NdbPack::Bound::get_data() const { return m_data; }
+inline NdbPack::Data&
+NdbPack::Bound::get_data() const
+{
+  return m_data;
+}
 
-inline Uint32 NdbPack::DataArray::cnt() const { return m_cnt; }
+inline Uint32
+NdbPack::DataArray::cnt() const
+{
+  return m_cnt;
+}
 
-inline Uint32 NdbPack::DataArray::get_null_cnt() const { return m_null_cnt; }
+inline Uint32
+NdbPack::DataArray::get_null_cnt() const
+{
+  return m_null_cnt;
+}
 
-inline Uint32 NdbPack::DataArray::get_data_len() const {
+inline Uint32
+NdbPack::DataArray::get_data_len() const
+{
   Uint32 cnt = m_cnt;
   Uint32 len = 0;
-  for (Uint32 i = 0; i < cnt; i++) {
+  for (Uint32 i = 0; i < cnt; i++)
+  {
     len += m_entries[i].m_data_len;
   }
   return len;
 }
 
-inline NdbPack::BoundArray::BoundArray()
-    : m_spec(nullptr), m_data_array(nullptr), m_side(0) {}
+inline NdbPack::BoundArray::BoundArray() :
+  m_spec(NULL),
+  m_data_array(NULL),
+  m_side(0)
+{
+}
 
-inline NdbPack::BoundArray::BoundArray(const Spec *spec,
-                                       const DataArray *data_array,
-                                       const int side)
-    : m_spec(spec), m_data_array(data_array), m_side(side) {}
+inline NdbPack::BoundArray::BoundArray(
+                   const Spec* spec,
+                   const DataArray* data_array,
+                   const int side) :
+  m_spec(spec),
+  m_data_array(data_array),
+  m_side(side)
+{
+}
 
-inline int NdbPack::BoundArray::cmp(const DataArray *d2, const Uint32 cnt,
-                                    const bool ok_to_ret_eq) const {
+inline int
+NdbPack::BoundArray::cmp(const DataArray* d2,
+                         const Uint32 cnt,
+                         const bool ok_to_ret_eq) const
+{
   int res = m_data_array->cmp(m_spec, d2, cnt);
   if (res == 0 && !ok_to_ret_eq && m_data_array->m_cnt <= d2->cnt())
     res = m_side;
   return res;
 }
 
-inline Uint32 NdbPack::BoundArray::cnt() const { return m_data_array->cnt(); }
-#endif  // NDB_PACK_HPP
+inline Uint32
+NdbPack::BoundArray::cnt() const
+{
+  return m_data_array->cnt();
+}
+#endif // NDB_PACK_HPP
