@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2008, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2008, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,19 +26,20 @@
 #define NDB_TAP_HPP
 
 #include "unittest/mytap/tap.h"
-#include "util/require.h"
 
-#define OK(b) require(b)
+#ifdef VM_TRACE
+#define OK(b) assert(b);
+#else
+#define OK(b) if (!(b)) abort();
+#endif
 
-#define TAPTEST(name)                     \
-  int name##_test();                      \
-  int main(int argc, const char **argv) { \
-    (void)argc;                           \
-    (void)argv; /* unused args */         \
-    plan(1);                              \
-    ok(name##_test(), #name);             \
-    return exit_status();                 \
-  }                                       \
-  int name##_test()
+#define TAPTEST(name)                           \
+int name##_test();                              \
+int main(int argc, const char** argv){          \
+  plan(1);                                      \
+  ok(name##_test(), #name);                     \
+  return exit_status();                         \
+}                                               \
+int name##_test()
 
 #endif
