@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2022, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -25,7 +25,6 @@
 #ifndef PROPERTIES_HPP
 #define PROPERTIES_HPP
 
-#include <string_view>
 #include <ndb_global.h>
 #include <BaseString.hpp>
 #include <UtilBuffer.hpp>
@@ -50,7 +49,6 @@ struct Property {
   Property(const char* name, Uint32 val);
   Property(const char* name, Uint64 val);
   Property(const char* name, const char * value);
-  Property(const char* name, std::string_view value);
   Property(const char* name, const class Properties * value);
   // We have no copy or move constructors so delete also assignment operator.
   Property& operator=(const Property&) = delete;
@@ -67,8 +65,7 @@ private:
  */
 class Properties {
 public:
-  static constexpr char delimiter = ':';
-  static constexpr char truncated_prefix_mark[] = "...:";
+  static const char delimiter;
   static const char version[];
 
   Properties(bool case_insensitive= false);
@@ -96,10 +93,8 @@ public:
   bool put(const char * name, Uint32 value, bool replace = false);
   bool put64(const char * name, Uint64 value, bool replace = false);
   bool put(const char * name, const char * value, bool replace = false);
-  bool put(const char * name, std::string_view value, bool replace = false);
   bool put(const char * name, const Properties * value, bool replace = false);
   bool append(const char * name, const char * value);
-  bool append(const char * name, std::string_view value);
 
   /**
    * Same as put above,
@@ -109,7 +104,6 @@ public:
   bool put(const char *, Uint32 no, Uint32, bool replace = false);
   bool put64(const char *, Uint32 no, Uint64, bool replace = false);
   bool put(const char *, Uint32 no, const char *, bool replace = false);
-  bool put(const char *, Uint32 no, std::string_view, bool replace = false);
   bool put(const char *, Uint32 no, const Properties *, bool replace = false);
 
 
@@ -171,16 +165,15 @@ public:
   Uint32 getPropertiesErrno() const { return propErrno; }
   Uint32 getOSErrno() const { return osErrno; }
 
- private:
-  //  Get methods that fail may set error code without changing property
-  mutable Uint32 propErrno;
-  mutable Uint32 osErrno;
+private:
+  Uint32 propErrno;
+  Uint32 osErrno;
 
   friend class PropertiesImpl;
-  class PropertiesImpl *impl;
-  class Properties *parent;
+  class PropertiesImpl * impl;
+  class Properties * parent;
 
-  void setErrno(Uint32 pErr, Uint32 osErr = 0) const;
+  void setErrno(Uint32 pErr, Uint32 osErr = 0) const ;
 };
 
 /**
