@@ -1,6 +1,6 @@
 /*
- Copyright (c) 2013, 2023, Oracle and/or its affiliates.
-
+ Copyright (c) 2013, 2016, Oracle and/or its affiliates. All rights reserved.
+ 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2.0,
  as published by the Free Software Foundation.
@@ -34,11 +34,13 @@
 
 #include <NdbApi.hpp>
 
-#include "NativeMethodCall.h"
-#include "NdbWrapperErrors.h"
-#include "Record.h"
 #include "adapter_global.h"
 #include "js_wrapper_macros.h"
+#include "Record.h"
+#include "NativeMethodCall.h"
+#include "NdbWrapperErrors.h"
+
+using namespace v8;
 
 void get_status(Local<String>, const AccessorInfo &);
 void get_classification(Local<String>, const AccessorInfo &);
@@ -46,8 +48,9 @@ void get_code(Local<String>, const AccessorInfo &);
 void get_mysql_code(Local<String>, const AccessorInfo &);
 void get_message(Local<String>, const AccessorInfo &);
 
+
 class NdbErrorEnvelopeClass : public Envelope {
- public:
+public:
   NdbErrorEnvelopeClass() : Envelope("NdbError") {
     addAccessor("status", get_status);
     addAccessor("classification", get_classification);
@@ -59,37 +62,40 @@ class NdbErrorEnvelopeClass : public Envelope {
 
 NdbErrorEnvelopeClass NdbErrorEnvelope;
 
+// TODO: Verify that all callers have a HandleScope
 Local<Value> NdbError_Wrapper(const NdbError &err) {
-  return NdbErrorEnvelope.wrap(&err);
+  return NdbErrorEnvelope.wrap(& err);
 }
 
-#define V8STRING(MSG) scope.Escape(NewUtf8String(info.GetIsolate(), MSG))
+#define V8STRING(MSG) scope.Escape(String::NewFromUtf8(info.GetIsolate(), MSG))
 
 #define V8INTEGER(V) scope.Escape(Integer::New(info.GetIsolate(), V))
 
-#define MAP_CODE(CODE)                          \
-  case NdbError::CODE:                          \
+#define MAP_CODE(CODE) \
+  case NdbError::CODE: \
     info.GetReturnValue().Set(V8STRING(#CODE)); \
     return;
 
+
 void get_status(Local<String> property, const AccessorInfo &info) {
   EscapableHandleScope scope(info.GetIsolate());
   const NdbError *err = unwrapPointer<const NdbError *>(info.Holder());
-
-  switch (err->status) {
+  
+  switch(err->status) {
     MAP_CODE(Success);
     MAP_CODE(TemporaryError);
     MAP_CODE(PermanentError);
-    MAP_CODE(UnknownResult);
+    MAP_CODE(UnknownResult);  
   }
   info.GetReturnValue().Set(V8STRING("-unknown-"));
 }
 
+
 void get_classification(Local<String> property, const AccessorInfo &info) {
   EscapableHandleScope scope(info.GetIsolate());
   const NdbError *err = unwrapPointer<const NdbError *>(info.Holder());
 
-  switch (err->classification) {
+  switch(err->classification) {
     MAP_CODE(NoError);
     MAP_CODE(ApplicationError);
     MAP_CODE(NoDataFound);
@@ -122,7 +128,7 @@ void get_code(Local<String> property, const AccessorInfo &info) {
 void get_mysql_code(Local<String> property, const AccessorInfo &info) {
   EscapableHandleScope scope(info.GetIsolate());
   const NdbError *err = unwrapPointer<const NdbError *>(info.Holder());
-
+  
   info.GetReturnValue().Set(V8INTEGER(err->mysql_code));
 }
 
@@ -132,3 +138,4 @@ void get_message(Local<String> property, const AccessorInfo &info) {
 
   info.GetReturnValue().Set(V8STRING(err->message));
 }
+
