@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2000, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2000, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -30,30 +30,27 @@
   NDB Cluster
 */
 
-#include <array>
-
-#include "my_inttypes.h"
 #include "sql/partitioning/partition_handler.h"
+#include "sql/sql_base.h"
+#include "sql/table.h"
 #include "storage/ndb/include/kernel/ndb_limits.h"
 #include "storage/ndb/include/ndbapi/NdbApi.hpp"
 #include "storage/ndb/include/ndbapi/ndbapi_limits.h"
 #include "storage/ndb/plugin/ha_ndbcluster_cond.h"
-#include "storage/ndb/plugin/ndb_bitmap.h"
-#include "storage/ndb/plugin/ndb_blobs_buffer.h"
 #include "storage/ndb/plugin/ndb_conflict.h"
-#include "storage/ndb/plugin/ndb_ndbapi_util.h"
-#include "storage/ndb/plugin/ndb_share.h"
 #include "storage/ndb/plugin/ndb_table_map.h"
-#include "storage/ndb/plugin/ndb_thd_ndb.h"
 
-class Ndb;
-class NdbOperation;
-class NdbTransaction;
-class NdbRecAttr;
+#define NDB_HIDDEN_PRIMARY_KEY_LENGTH 8
+
+class Ndb;             // Forward declaration
+class NdbOperation;    // Forward declaration
+class NdbTransaction;  // Forward declaration
+class NdbRecAttr;      // Forward declaration
 class NdbScanOperation;
 class NdbIndexScanOperation;
 class NdbBlob;
 class NdbIndexStat;
+class NdbEventOperation;
 class ha_ndbcluster_cond;
 class NdbQuery;
 class NdbQueryOperation;
@@ -71,55 +68,56 @@ enum NDB_INDEX_TYPE {
 };
 
 struct NDB_INDEX_DATA {
-  NDB_INDEX_TYPE type{UNDEFINED_INDEX};
-  const NdbDictionary::Index *index{nullptr};
-  const NdbDictionary::Index *unique_index{nullptr};
+  NDB_INDEX_TYPE type;
+  enum { UNDEFINED = 0, ACTIVE = 1, TO_BE_DROPPED = 2 } status;
+  const NdbDictionary::Index *index;
+  const NdbDictionary::Index *unique_index;
+  unsigned char *unique_index_attrid_map;
+  bool null_in_unique_index;
+  /*
+    In mysqld, keys and rows are stored differently (using KEY_PART_INFO for
+    keys and Field for rows).
+    So we need to use different NdbRecord for an index for passing values
+    from a key and from a row.
+  */
+  NdbRecord *ndb_record_key;
+  NdbRecord *ndb_unique_record_key;
+  NdbRecord *ndb_unique_record_row;
+};
 
- private:
-  // Map from MySQL key to NDB column order, this is necessary when the order of
-  // the keys used by MySQL does not match the column order in NDB. The map
-  // is only created if necessary, otherwise the default sequential column order
-  // is used. The below table have both its primary key and unique key
-  // specified in a different order than the table:
-  //   CREATE TABLE t1 (
-  //     a int, b int, c int, d int, e int,
-  //     PRIMARY KEY(d,b,c),
-  //     UNIQUE_KEY(e,d,c)
-  //   ) engine = ndb;
-  class Attrid_map {
-    std::vector<unsigned char> m_ids;
-    // Verify that vector's type is large enough to store "index of NDB column"
-    // (currently 32 columns supported by NDB and 16 by MySQL)
-    static_assert(std::numeric_limits<decltype(m_ids)::value_type>::max() >
-                  NDB_MAX_NO_OF_ATTRIBUTES_IN_KEY);
+#include "storage/ndb/plugin/ndb_ndbapi_util.h"
+#include "storage/ndb/plugin/ndb_share.h"
 
-   public:
-    Attrid_map(const KEY *key_info, const NdbDictionary::Table *table);
-    Attrid_map(const KEY *key_info, const NdbDictionary::Index *index);
+struct Ndb_local_table_statistics {
+  int no_uncommitted_rows_count;
+  ulong last_count;
+  ha_rows records;
+};
 
-    void fill_column_map(uint column_map[]) const;
-  };
-  const Attrid_map *attrid_map{nullptr};
+#include "storage/ndb/plugin/ndb_thd_ndb.h"
 
- public:
-  // Create Attrid_map for primary key, if required
-  void create_attrid_map(const KEY *key_info,
-                         const NdbDictionary::Table *table);
-  // Create Attrid_map for unique key, if required
-  void create_attrid_map(const KEY *key_info,
-                         const NdbDictionary::Index *index);
-  // Delete the Attrid_map
-  void delete_attrid_map();
-  // Fill column_map for given KEY
-  void fill_column_map(const KEY *key_info, uint column_map[]) const;
-
-  bool null_in_unique_index{false};
-  // The keys and rows passed from MySQL Server are in different formats
-  // depending on whether it's a key (using KEY_PART_INFO) or row (using Field),
-  // thus different NdbRecord's need to be setup for each format.
-  NdbRecord *ndb_record_key{nullptr};
-  NdbRecord *ndb_unique_record_key{nullptr};
-  NdbRecord *ndb_unique_record_row{nullptr};
+struct st_ndb_status {
+  st_ndb_status() { memset(this, 0, sizeof(struct st_ndb_status)); }
+  long cluster_node_id;
+  const char *connected_host;
+  long connected_port;
+  long number_of_data_nodes;
+  long number_of_ready_data_nodes;
+  long connect_count;
+  long execute_count;
+  long trans_hint_count;
+  long scan_count;
+  long pruned_scan_count;
+  long schema_locks_count;
+  long sorted_scan_count;
+  long pushed_queries_defined;
+  long pushed_queries_dropped;
+  long pushed_queries_executed;
+  long pushed_reads;
+  long long last_commit_epoch_server;
+  long long last_commit_epoch_session;
+  long long api_client_stats[Ndb::NumClientStatistics];
+  const char *system_name;
 };
 
 int ndbcluster_commit(handlerton *, THD *thd, bool all);
@@ -134,7 +132,7 @@ class ha_ndbcluster : public handler, public Partition_handler {
 
   std::string explain_extra() const override;
 
-  int open(const char *path, int mode, uint test_if_locked,
+  int open(const char *name, int mode, uint test_if_locked,
            const dd::Table *table_def) override;
 
   int close(void) override;
@@ -227,8 +225,6 @@ class ha_ndbcluster : public handler, public Partition_handler {
   void update_create_info(HA_CREATE_INFO *create_info) override;
 
  private:
-  int get_old_table_comment_items(THD *thd, bool *comment_items_shown,
-                                  char *comment_str, unsigned comment_len);
   void update_comment_info(THD *thd, HA_CREATE_INFO *create_info,
                            const NdbDictionary::Table *tab);
 
@@ -247,8 +243,8 @@ class ha_ndbcluster : public handler, public Partition_handler {
   int rename_table(const char *from, const char *to,
                    const dd::Table *from_table_def,
                    dd::Table *to_table_def) override;
-  int delete_table(const char *path, const dd::Table *table_def) override;
-  bool upgrade_table(THD *thd, const char *db_name, const char *table_name,
+  int delete_table(const char *name, const dd::Table *table_def) override;
+  bool upgrade_table(THD *thd, const char *, const char *table_name,
                      dd::Table *dd_table) override;
 
   row_type get_real_row_type(const HA_CREATE_INFO *create_info) const override {
@@ -259,7 +255,7 @@ class ha_ndbcluster : public handler, public Partition_handler {
     // All other values uses DYNAMIC
     return ROW_TYPE_DYNAMIC;
   }
-  int create(const char *path, TABLE *table, HA_CREATE_INFO *info,
+  int create(const char *name, TABLE *form, HA_CREATE_INFO *info,
              dd::Table *table_def) override;
   int truncate(dd::Table *table_def) override;
   bool is_ignorable_error(int error) override {
@@ -283,11 +279,6 @@ class ha_ndbcluster : public handler, public Partition_handler {
   }
 
   double scan_time() override;
-
-  double read_time(uint index, uint ranges, ha_rows rows) override;
-  double page_read_cost(uint index, double rows) override;
-  double worst_seek_times(double reads) override;
-
   ha_rows records_in_range(uint inx, key_range *min_key,
                            key_range *max_key) override;
   void start_bulk_insert(ha_rows rows) override;
@@ -304,6 +295,14 @@ class ha_ndbcluster : public handler, public Partition_handler {
                      int is_bulk_update);
 
  public:
+  static void set_dbname(const char *pathname, char *dbname);
+  static void set_tabname(const char *pathname, char *tabname);
+  /*
+    static member function as it needs to access private
+    NdbTransaction methods
+  */
+  static void release_completed_operations(NdbTransaction *);
+
   /*
     Condition pushdown
   */
@@ -314,6 +313,8 @@ class ha_ndbcluster : public handler, public Partition_handler {
       cond_push()
       cond   Condition to be pushed. The condition tree must not be
       modified by the by the caller.
+      other_tbls_ok  Are other tables allowed to be referred
+      from the condition terms pushed down.
     RETURN
       The 'remainder' condition that caller must use to filter out records.
       NULL means the handler will not return rows that do not match the
@@ -332,7 +333,7 @@ class ha_ndbcluster : public handler, public Partition_handler {
     =, !=, >, >=, <, <=, like, "not like", "is null", and "is not null".
     Negated conditions are supported by NOT which generate NAND/NOR groups.
   */
-  const Item *cond_push(const Item *cond) override;
+  const Item *cond_push(const Item *cond, bool other_tbls_ok) override;
 
  public:
   /**
@@ -355,15 +356,8 @@ class ha_ndbcluster : public handler, public Partition_handler {
                                     const KEY *key_info,
                                     const key_range *start_key,
                                     const key_range *end_key);
-  /**
-   * NDB support join- and condition pushdown, so we return
-   * the NDB-handlerton to signal that
-   * handlerton::push_to_engine() need to be called.
-   */
-  const handlerton *hton_supporting_engine_pushdown() override { return ht; }
 
-  friend int ndbcluster_push_to_engine(THD *thd, AccessPath *, JOIN *);
-  friend void accept_pushed_conditions(const TABLE *table, AccessPath *filter);
+  int engine_push(AQP::Table_access *table) override;
 
  private:
   bool maybe_pushable_join(const char *&reason) const;
@@ -388,11 +382,10 @@ class ha_ndbcluster : public handler, public Partition_handler {
       TABLE *altered_table, Alter_inplace_info *ha_alter_info) override;
 
  private:
-  static bool inplace_parse_comment(
-      NdbDictionary::Table *new_tab, const NdbDictionary::Table *old_tab,
-      HA_CREATE_INFO *create_info, THD *thd, Ndb *ndb,
-      const char **unsupported_reason, bool &max_rows_changed,
-      bool *partition_balance_in_comment = nullptr);
+  bool parse_comment_changes(NdbDictionary::Table *new_tab,
+                             const NdbDictionary::Table *old_tab,
+                             HA_CREATE_INFO *create_info, THD *thd,
+                             bool &max_rows_changed) const;
 
  public:
   bool prepare_inplace_alter_table(TABLE *altered_table,
@@ -416,20 +409,23 @@ class ha_ndbcluster : public handler, public Partition_handler {
   bool open_table_set_key_fields();
   void release_key_fields();
   void release_ndb_share();
-  NDB_SHARE *open_share_before_schema_sync(THD *thd, const char *dbname,
-                                           const char *tabname) const;
-  void prepare_inplace__drop_index(uint index_num);
+  NDB_SHARE *open_table_before_schema_sync(THD *, const char *);
+  void prepare_inplace__drop_index(uint key_num);
+  int inplace__final_drop_index(TABLE *table_arg);
 
   enum_alter_inplace_result supported_inplace_field_change(Alter_inplace_info *,
                                                            Field *, Field *,
                                                            bool, bool) const;
+  bool table_storage_changed(HA_CREATE_INFO *) const;
+  bool column_has_index(TABLE *, uint, uint, uint) const;
   enum_alter_inplace_result supported_inplace_ndb_column_change(
       uint, TABLE *, Alter_inplace_info *, bool, bool) const;
   enum_alter_inplace_result supported_inplace_column_change(
-      NdbDictionary::Dictionary *dict, TABLE *, uint, Field *,
-      Alter_inplace_info *) const;
+      THD *, TABLE *, uint, Field *, Alter_inplace_info *) const;
   enum_alter_inplace_result check_inplace_alter_supported(
       TABLE *altered_table, Alter_inplace_info *ha_alter_info);
+  void check_implicit_column_format_change(
+      TABLE *altered_table, Alter_inplace_info *ha_alter_info) const;
 
   bool abort_inplace_alter_table(TABLE *altered_table,
                                  Alter_inplace_info *ha_alter_info);
@@ -447,53 +443,49 @@ class ha_ndbcluster : public handler, public Partition_handler {
 
   int prepare_inplace__add_index(THD *thd, KEY *key_info,
                                  uint num_of_keys) const;
-  int create_index_in_NDB(THD *thd, const char *name, const KEY *key_info,
+  int create_index_in_NDB(THD *thd, const char *name, KEY *key_info,
                           const NdbDictionary::Table *ndbtab,
                           bool unique) const;
-  int create_index(THD *thd, const char *name, const KEY *key_info,
+  int create_index(THD *thd, const char *name, KEY *key_info,
                    NDB_INDEX_TYPE idx_type,
                    const NdbDictionary::Table *ndbtab) const;
   // Index list management
   int create_indexes(THD *thd, TABLE *tab,
                      const NdbDictionary::Table *ndbtab) const;
-  int open_indexes(NdbDictionary::Dictionary *dict);
-  void release_indexes(NdbDictionary::Dictionary *dict, bool invalidate);
-  int inplace__drop_index(NdbDictionary::Dictionary *dict, uint index_num);
-  int open_index(NdbDictionary::Dictionary *dict, const KEY *key_info,
-                 const char *key_name, uint index_no);
+  int open_indexes(Ndb *ndb, TABLE *tab);
+  void release_indexes(NdbDictionary::Dictionary *dict, int invalidate);
+  void inplace__renumber_indexes(uint dropped_index_num);
+  int inplace__drop_indexes(Ndb *ndb, TABLE *tab);
+  int add_index_handle(NdbDictionary::Dictionary *dict, KEY *key_info,
+                       const char *key_name, uint index_no);
   int add_table_ndb_record(NdbDictionary::Dictionary *dict);
   int add_hidden_pk_ndb_record(NdbDictionary::Dictionary *dict);
-  int open_index_ndb_record(NdbDictionary::Dictionary *dict,
-                            const KEY *key_info, uint index_no);
-  static int create_fks(THD *thd, Ndb *ndb, const char *dbname,
-                        const char *tabname);
-  static int copy_fk_for_offline_alter(THD *thd, Ndb *ndb, const char *dbname,
-                                       const char *tabname);
-  static int inplace__drop_fks(THD *thd, Ndb *ndb, const char *dbname,
-                               const char *tabname);
-  static int recreate_fk_for_truncate(THD *thd, Ndb *ndb, const char *dbname,
-                                      const char *tabname,
+  int add_index_ndb_record(NdbDictionary::Dictionary *dict, KEY *key_info,
+                           uint index_no);
+  int create_fks(THD *thd, Ndb *ndb);
+  int copy_fk_for_offline_alter(THD *thd, Ndb *, const char *tabname);
+  int inplace__drop_fks(THD *, Ndb *, NdbDictionary::Dictionary *,
+                        const NdbDictionary::Table *);
+  static int recreate_fk_for_truncate(THD *, Ndb *, const char *,
                                       std::vector<NdbDictionary::ForeignKey> *);
-  bool has_fk_dependency(NdbDictionary::Dictionary *dict,
-                         const NdbDictionary::Column *) const;
-#ifndef NDEBUG
-  bool check_default_values() const;
-#endif
-  int get_metadata(Ndb *ndb, const char *dbname, const char *tabname,
-                   const dd::Table *table_def);
-  void release_metadata(NdbDictionary::Dictionary *dict,
-                        bool invalidate_objects);
+  bool has_fk_dependency(THD *, const NdbDictionary::Column *) const;
+  int check_default_values(const NdbDictionary::Table *ndbtab);
+  int get_metadata(THD *thd, const dd::Table *table_def);
+  void release_metadata(THD *thd);
   NDB_INDEX_TYPE get_index_type(uint idx_no) const;
-  NDB_INDEX_TYPE get_declared_index_type(uint index_num) const;
+  NDB_INDEX_TYPE get_index_type_from_table(uint index_no) const;
+  NDB_INDEX_TYPE get_index_type_from_key(uint index_no, KEY *key_info,
+                                         bool primary) const;
   bool has_null_in_unique_index(uint idx_no) const;
+  bool check_index_fields_not_null(KEY *key_info) const;
 
   bool check_if_pushable(int type,  // NdbQueryOperationDef::Type,
                          uint idx = MAX_KEY) const;
   bool check_is_pushed() const;
-  int create_pushed_join(const NdbQueryParamValue *keyFieldParams = nullptr,
+  int create_pushed_join(const NdbQueryParamValue *keyFieldParams = NULL,
                          uint paramCnt = 0);
 
-  int ndb_pk_update_row(const uchar *old_data, uchar *new_data);
+  int ndb_pk_update_row(THD *thd, const uchar *old_data, uchar *new_data);
   int pk_read(const uchar *key, uchar *buf, uint32 *part_id);
   int ordered_index_scan(const key_range *start_key, const key_range *end_key,
                          bool sorted, bool descending, uchar *buf,
@@ -511,11 +503,9 @@ class ha_ndbcluster : public handler, public Partition_handler {
 
   int ndb_optimize_table(THD *thd, uint delay) const;
 
-  bool peek_index_rows_check_index_fields_in_write_set(
-      const KEY *key_info) const;
-  bool peek_index_rows_check_ops(NdbTransaction *trans,
-                                 const NdbOperation *first,
-                                 const NdbOperation *last);
+  bool check_all_operations_for_error(NdbTransaction *trans,
+                                      const NdbOperation *first,
+                                      const NdbOperation *last, uint errcode);
 
   enum NDB_WRITE_OP { NDB_INSERT = 0, NDB_UPDATE = 1, NDB_PK_UPDATE = 2 };
 
@@ -523,13 +513,15 @@ class ha_ndbcluster : public handler, public Partition_handler {
   int scan_handle_lock_tuple(NdbScanOperation *scanOp, NdbTransaction *trans);
   int fetch_next(NdbScanOperation *op);
   int fetch_next_pushed();
-  int set_auto_inc(Ndb *ndb, Field *field);
-  int set_auto_inc_val(Ndb *ndb, Uint64 value) const;
+  int set_auto_inc(THD *thd, Field *field);
+  int set_auto_inc_val(THD *thd, Uint64 value);
   int next_result(uchar *buf);
   int close_scan();
   int unpack_record(uchar *dst_row, const uchar *src_row);
   int unpack_record_and_set_generated_fields(uchar *dst_row,
                                              const uchar *src_row);
+  void set_dbname(const char *pathname);
+  void set_tabname(const char *pathname);
 
   const NdbDictionary::Column *get_hidden_key_column() {
     return m_table->getColumn(m_table_map->get_hidden_key_column());
@@ -538,13 +530,14 @@ class ha_ndbcluster : public handler, public Partition_handler {
     return m_table->getColumn(m_table_map->get_partition_id_column());
   }
 
-  static int get_ndb_blobs_value_hook(NdbBlob *ndb_blob, void *arg);
+  uchar *get_buffer(Thd_ndb *thd_ndb, uint size);
+  uchar *copy_row_to_buffer(Thd_ndb *thd_ndb, const uchar *record);
 
   int get_blob_values(const NdbOperation *ndb_op, uchar *dst_record,
                       const MY_BITMAP *bitmap);
   int set_blob_values(const NdbOperation *ndb_op, ptrdiff_t row_offset,
-                      const MY_BITMAP *bitmap, uint *set_count,
-                      bool batch) const;
+                      const MY_BITMAP *bitmap, uint *set_count, bool batch);
+  friend int g_get_ndb_blobs_value(NdbBlob *ndb_blob, void *arg);
   void release_blobs_buffer();
   Uint32 setup_get_hidden_fields(NdbOperation::GetValueSpec gets[2]);
   void get_hidden_fields_keyop(NdbOperation::OperationOptions *options,
@@ -553,9 +546,13 @@ class ha_ndbcluster : public handler, public Partition_handler {
                               NdbOperation::GetValueSpec gets[2]);
   void get_read_set(bool use_cursor, uint idx);
 
+  void eventSetAnyValue(THD *thd,
+                        NdbOperation::OperationOptions *options) const;
+  bool check_index_fields_in_write_set(uint keyno);
+
   int log_exclusive_read(const NdbRecord *key_rec, const uchar *key, uchar *buf,
-                         Uint32 *ppartition_id) const;
-  int scan_log_exclusive_read(NdbScanOperation *, NdbTransaction *) const;
+                         Uint32 *ppartition_id);
+  int scan_log_exclusive_read(NdbScanOperation *, NdbTransaction *);
   const NdbOperation *pk_unique_index_read_key(uint idx, const uchar *key,
                                                uchar *buf,
                                                NdbOperation::LockMode lm,
@@ -574,7 +571,9 @@ class ha_ndbcluster : public handler, public Partition_handler {
 
   int check_ndb_connection(THD *thd) const;
 
-  void set_rec_per_key(THD *thd);
+  void set_rec_per_key();
+  void no_uncommitted_rows_execute_failure();
+  void no_uncommitted_rows_update(int);
 
   /* Ordered index statistics v4 */
   int ndb_index_stat_query(uint inx, const key_range *min_key,
@@ -599,12 +598,15 @@ class ha_ndbcluster : public handler, public Partition_handler {
 
   NdbTransaction *start_transaction_row(const NdbRecord *ndb_record,
                                         const uchar *record, int &error);
-  NdbTransaction *start_transaction_key(uint index_num, const uchar *key_data,
+  NdbTransaction *start_transaction_key(uint index, const uchar *key_data,
                                         int &error);
 
+  friend int check_completed_operations_pre_commit(Thd_ndb *, NdbTransaction *,
+                                                   const NdbOperation *,
+                                                   uint *ignore_count);
   friend int ndbcluster_commit(handlerton *, THD *thd, bool all);
-
   int start_statement(THD *thd, Thd_ndb *thd_ndb, uint table_count);
+  int init_handler_for_statement(THD *thd);
   /*
     Implementing Partition_handler API.
   */
@@ -620,12 +622,10 @@ class ha_ndbcluster : public handler, public Partition_handler {
   void set_part_info(partition_info *part_info, bool early) override;
   /* End of Partition_handler API */
 
+  Ndb_table_map *m_table_map;
   Thd_ndb *m_thd_ndb;
   NdbScanOperation *m_active_cursor;
-  // NDB table definition
-  const NdbDictionary::Table *m_table{nullptr};
-  // Mapping from MySQL table to NDB table
-  Ndb_table_map *m_table_map{nullptr};
+  const NdbDictionary::Table *m_table;
   /*
     Normal NdbRecord for accessing rows, with all fields including hidden
     fields (hidden primary key, user-defined partitioning function value).
@@ -636,25 +636,25 @@ class ha_ndbcluster : public handler, public Partition_handler {
 
   /* Bitmap used for NdbRecord operation column mask. */
   MY_BITMAP m_bitmap;
-  Ndb_bitmap_buf<NDB_MAX_ATTRIBUTES_IN_TABLE> m_bitmap_buf;
-
-  // Pointer to bitmap for the primary key columns (the actual bitmap is in
-  // m_key_fields array that have one bitmap for each index of the table)
+  my_bitmap_map
+      m_bitmap_buf[(NDB_MAX_ATTRIBUTES_IN_TABLE + 8 * sizeof(my_bitmap_map) -
+                    1) /
+                   (8 * sizeof(my_bitmap_map))];  // Buffer for m_bitmap
+  /* Bitmap with bit set for all primary key columns. */
   MY_BITMAP *m_pk_bitmap_p;
-  // Since all NDB table have primary key, the bitmap buffer is preallocated
-  Ndb_bitmap_buf<NDB_MAX_ATTRIBUTES_IN_TABLE> m_pk_bitmap_buf;
-
-  // Pointer to table stats for transaction
-  Thd_ndb::Trans_tables::Stats *m_trans_table_stats{nullptr};
-
+  my_bitmap_map
+      m_pk_bitmap_buf[(NDB_MAX_ATTRIBUTES_IN_TABLE + 8 * sizeof(my_bitmap_map) -
+                       1) /
+                      (8 * sizeof(my_bitmap_map))];  // Buffer for m_pk_bitmap
+  struct Ndb_local_table_statistics *m_table_info;
+  struct Ndb_local_table_statistics m_table_info_instance;
+  char m_dbname[FN_HEADLEN];
+  // char m_schemaname[FN_HEADLEN];
+  char m_tabname[FN_HEADLEN];
   THR_LOCK_DATA m_lock;
   bool m_lock_tuple;
-  NDB_SHARE *m_share{nullptr};
-
-  std::array<NDB_INDEX_DATA, MAX_INDEXES> m_index;
-  // Cached metadata variable, indicating if the open table have any unique
-  // indexes. Used as a quick optimization to avoid looping the list of indexes.
-  bool m_has_unique_index{false};
+  NDB_SHARE *m_share;
+  NDB_INDEX_DATA m_index[MAX_KEY];
 
   /*
     Pointer to row returned from scan nextResult().
@@ -678,6 +678,7 @@ class ha_ndbcluster : public handler, public Partition_handler {
   bool m_sorted;
   bool m_use_write;
   bool m_ignore_dup_key;
+  bool m_has_unique_index;
   bool m_ignore_no_key;
   bool m_read_before_write_removal_possible;
   bool m_read_before_write_removal_used;
@@ -685,35 +686,24 @@ class ha_ndbcluster : public handler, public Partition_handler {
   ha_rows m_rows_deleted;
   ha_rows m_rows_to_insert;  // TODO: merge it with
                              // handler::estimation_rows_to_insert?
+  ha_rows m_rows_inserted;
   bool m_delete_cannot_batch;
   bool m_update_cannot_batch;
-  // Approximate number of bytes that need to be sent to NDB when updating a row
-  // of this table, used for determining when batch should be flushed.
   uint m_bytes_per_write;
   bool m_skip_auto_increment;
+  bool m_blobs_pending;
+  bool m_slow_path;
   bool m_is_bulk_delete;
 
-  class Copying_alter {
-    Uint64 m_saved_commit_count;
-
-   public:
-    // Save the commit count for source table during copying ALTER,
-    // returns 0 on success, handler error otherwise
-    int save_commit_count(Thd_ndb *thd_ndb, const NdbDictionary::Table *ndbtab);
-    // Check commit count for source table during copying ALTER,
-    // returns 0 on success, handler error otherwise
-    int check_saved_commit_count(Thd_ndb *thd_ndb,
-                                 const NdbDictionary::Table *ndbtab) const;
-  } copying_alter;
-
   /* State for setActiveHook() callback for reading blob data. */
   uint m_blob_counter;
   uint m_blob_expected_count_per_row;
   uchar *m_blob_destination_record;
   Uint64 m_blobs_row_total_size; /* Bytes needed for all blobs in current row */
 
-  Ndb_blobs_buffer m_blobs_buffer;
-
+  // memory for blobs in one tuple
+  uchar *m_blobs_buffer;
+  Uint64 m_blobs_buffer_size;
   uint m_dupkey;
   // set from thread variables at external lock
   ha_rows m_autoincrement_prefetch;
@@ -733,10 +723,20 @@ class ha_ndbcluster : public handler, public Partition_handler {
   bool m_disable_multi_read;
   uchar *m_multi_range_result_ptr;
   NdbIndexScanOperation *m_multi_cursor;
+  Ndb *get_ndb(THD *thd) const;
 
-  int update_stats(THD *thd, bool do_read_stat);
+  int update_stats(THD *thd, bool do_read_stat, uint part_id = ~(uint)0);
+  int add_handler_to_open_tables(THD *, Thd_ndb *, ha_ndbcluster *handler);
 };
 
+// Global handler synchronization
+extern mysql_mutex_t ndbcluster_mutex;
+extern mysql_cond_t ndbcluster_cond;
+
+extern int ndb_setup_complete;
+
+static const int NDB_INVALID_SCHEMA_OBJECT = 241;
+
 int ndb_to_mysql_error(const NdbError *ndberr);
 
 #endif
