@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2017, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -32,8 +31,6 @@
 #include "my_inttypes.h"
 #include "storage/ndb/include/ndbapi/NdbDictionary.hpp"
 
-class Ndb_event_data;
-class NdbEventOperation;
 struct NDB_SHARE;
 struct st_conflict_fn_def;
 struct st_conflict_fn_arg;
@@ -50,22 +47,19 @@ class Ndb_binlog_client {
   /**
    * @brief log_warning, push the message as warning for user threads and
    *                     write the message to log file for other threads
+   * @param code
+   * @param fmt
    */
   void log_warning(uint code, const char *fmt, ...) const
       MY_ATTRIBUTE((format(printf, 3, 4)));
 
-  /**
-   * @brief Log the NDB error using log_warning()
-   */
-  void log_ndb_error(const NdbError &ndberr) const;
-
   /**
    * @brief event_name_for_table, generate name for the event for this table
    *
+   * @param event_name     the returned event name
    * @param db             database of table
-   * @param table_name     name of table
+   * @param tbl            name of table
    * @param full           create name for event with all columns
-   * @return the returned event name
    */
   static std::string event_name_for_table(const char *db,
                                           const char *table_name, bool full);
@@ -106,12 +100,12 @@ class Ndb_binlog_client {
 
   /**
    * @brief table_should_have_event_op, decide if a NdbEventOperation
-   * should be created for the current table. Only table which need to
-   * be binlogged would create such a event operation. The exception
+   * should be crated for the current table. Only table which need to
+   * be binlogged would create such a event operation. The exeception
    * is the ndb_schema table who subscribes to events for schema distribution.
    * @return  true if table should have a NdbEventOperation
    */
-  bool table_should_have_event_op(const NDB_SHARE *share) const;
+  bool table_should_have_event_op(const NDB_SHARE *share);
 
   /**
    * @brief event_exists_for_table, check if event already exists for this
@@ -126,17 +120,10 @@ class Ndb_binlog_client {
 
   int create_event(Ndb *ndb, const NdbDictionary::Table *ndbtab,
                    const NDB_SHARE *share);
-
- private:
-  NdbEventOperation *create_event_op_in_NDB(
-      Ndb *ndb, const NdbDictionary::Table *ndbtab,
-      const std::string &event_name, const Ndb_event_data *event_data,
-      bool skip_setup_datanode_anyvalue_filter);
-
- public:
-  int create_event_op(NDB_SHARE *share, const dd::Table *table_def,
-                      const NdbDictionary::Table *ndbtab,
-                      bool replace_op = false);
+  bool create_event_data(NDB_SHARE *share, const dd::Table *table_def,
+                         class Ndb_event_data **event_data) const;
+  int create_event_op(NDB_SHARE *share, const NdbDictionary::Table *ndbtab,
+                      const Ndb_event_data *event_data);
 
   /**
    * @brief drop_events_for_table, drop all binlog events for the table
@@ -149,10 +136,10 @@ class Ndb_binlog_client {
    * @param thd            thread context
    * @param ndb            Ndb pointer
    * @param dbname         database of table
-   * @param table_name     name of table
+   * @param tabname        name of table
    */
   static void drop_events_for_table(THD *thd, Ndb *ndb, const char *dbname,
-                                    const char *table_name);
+                                    const char *tabname);
 };
 
 #endif
