@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2017, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2017, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,11 +25,9 @@
 #include "storage/ndb/plugin/ndb_dd_client.h"
 
 #include <assert.h>
-
 #include <iostream>
 
 #include "my_dbug.h"
-#include "my_psi_config.h"         // HAVE_PSI_SP_INTERFACE
 #include "sql/auth/auth_common.h"  // check_readonly()
 #include "sql/dd/cache/dictionary_client.h"
 #include "sql/dd/dd.h"
@@ -52,19 +49,14 @@
 #include "storage/ndb/plugin/ndb_dd_upgrade_table.h"
 #include "storage/ndb/plugin/ndb_fk_util.h"
 #include "storage/ndb/plugin/ndb_log.h"
-#include "storage/ndb/plugin/ndb_pfs_init.h"
 #include "storage/ndb/plugin/ndb_schema_dist_table.h"
+#include "storage/ndb/plugin/ndb_tdc.h"
 #include "storage/ndb/plugin/ndb_thd.h"
 
-constexpr size_t NDB_DD_CLIENT_MEMROOT_BLOCK_SIZE = 1024;
-
 Ndb_dd_client::Ndb_dd_client(THD *thd)
     : m_thd(thd),
       m_client(thd->dd_client()),
-      m_save_mdl_locks(thd->mdl_context.mdl_savepoint()),
-      m_dd_mem_root(key_memory_ndb_dd_client_mem_root,
-                    NDB_DD_CLIENT_MEMROOT_BLOCK_SIZE),
-      m_prev_mem_root(thd->mem_root) {
+      m_save_mdl_locks(thd->mdl_context.mdl_savepoint()) {
   DBUG_TRACE;
   disable_autocommit();
 
@@ -73,18 +65,13 @@ Ndb_dd_client::Ndb_dd_client(THD *thd)
   // Dictionary_client in the ndb_dd_client header file
   m_auto_releaser =
       (void *)new dd::cache::Dictionary_client::Auto_releaser(m_client);
-
-  // Use dedicated MEM_ROOT while acessing DD
-  thd->mem_root = &m_dd_mem_root;
 }
 
 Ndb_dd_client::~Ndb_dd_client() {
   DBUG_TRACE;
   // Automatically restore the option_bits in THD if they have
   // been modified
-  if (m_save_option_bits != 0) {
-    m_thd->variables.option_bits = m_save_option_bits;
-  }
+  if (m_save_option_bits) m_thd->variables.option_bits = m_save_option_bits;
 
   if (m_auto_rollback) {
     // Automatically rollback unless commit has been called
@@ -94,9 +81,6 @@ Ndb_dd_client::~Ndb_dd_client() {
   // Release MDL locks
   mdl_locks_release();
 
-  // Restore previous MEM_ROOT
-  m_thd->mem_root = m_prev_mem_root;
-
   // Free the dictionary client auto releaser
   dd::cache::Dictionary_client::Auto_releaser *ar =
       (dd::cache::Dictionary_client::Auto_releaser *)m_auto_releaser;
@@ -116,10 +100,15 @@ bool Ndb_dd_client::mdl_lock_table(const char *schema_name,
   mdl_requests.push_front(&schema_request);
   mdl_requests.push_front(&mdl_request);
 
-  if (!mdl_locks_acquire(mdl_requests, m_thd->variables.lock_wait_timeout)) {
+  if (m_thd->mdl_context.acquire_locks(&mdl_requests,
+                                       m_thd->variables.lock_wait_timeout)) {
     return false;
   }
 
+  // Remember tickets of the acquired mdl locks
+  m_acquired_mdl_tickets.push_back(schema_request.ticket);
+  m_acquired_mdl_tickets.push_back(mdl_request.ticket);
+
   return true;
 }
 
@@ -151,10 +140,15 @@ bool Ndb_dd_client::mdl_lock_schema_exclusive(const char *schema_name,
     lock_wait_timeout = m_thd->variables.lock_wait_timeout;
   }
 
-  if (!mdl_locks_acquire(mdl_requests, lock_wait_timeout)) {
+  if (m_thd->mdl_context.acquire_locks(&mdl_requests, lock_wait_timeout)) {
     return false;
   }
 
+  // Remember tickets of the acquired mdl locks
+  m_acquired_mdl_tickets.push_back(schema_request.ticket);
+  m_acquired_mdl_tickets.push_back(backup_lock_request.ticket);
+  m_acquired_mdl_tickets.push_back(grl_request.ticket);
+
   return true;
 }
 
@@ -166,7 +160,8 @@ bool Ndb_dd_client::mdl_lock_schema(const char *schema_name) {
                    MDL_INTENTION_EXCLUSIVE, MDL_EXPLICIT);
   mdl_requests.push_front(&schema_request);
 
-  if (!mdl_locks_acquire(mdl_requests, m_thd->variables.lock_wait_timeout)) {
+  if (m_thd->mdl_context.acquire_locks(&mdl_requests,
+                                       m_thd->variables.lock_wait_timeout)) {
     return false;
   }
 
@@ -176,6 +171,9 @@ bool Ndb_dd_client::mdl_lock_schema(const char *schema_name) {
   */
   if (check_readonly(m_thd, true)) return false;
 
+  // Remember ticket(s) of the acquired mdl lock
+  m_acquired_mdl_tickets.push_back(schema_request.ticket);
+
   return true;
 }
 
@@ -207,7 +205,7 @@ bool Ndb_dd_client::mdl_lock_logfile_group_exclusive(
     lock_wait_timeout = m_thd->variables.lock_wait_timeout;
   }
 
-  if (!mdl_locks_acquire(mdl_requests, lock_wait_timeout)) {
+  if (m_thd->mdl_context.acquire_locks(&mdl_requests, lock_wait_timeout)) {
     return false;
   }
 
@@ -217,6 +215,11 @@ bool Ndb_dd_client::mdl_lock_logfile_group_exclusive(
   */
   if (check_readonly(m_thd, true)) return false;
 
+  // Remember tickets of the acquired mdl locks
+  m_acquired_mdl_tickets.push_back(logfile_group_request.ticket);
+  m_acquired_mdl_tickets.push_back(backup_lock_request.ticket);
+  m_acquired_mdl_tickets.push_back(grl_request.ticket);
+
   return true;
 }
 
@@ -232,10 +235,14 @@ bool Ndb_dd_client::mdl_lock_logfile_group(const char *logfile_group_name,
 
   mdl_requests.push_front(&logfile_group_request);
 
-  if (!mdl_locks_acquire(mdl_requests, m_thd->variables.lock_wait_timeout)) {
+  if (m_thd->mdl_context.acquire_locks(&mdl_requests,
+                                       m_thd->variables.lock_wait_timeout)) {
     return false;
   }
 
+  // Remember tickets of the acquired mdl locks
+  m_acquired_mdl_tickets.push_back(logfile_group_request.ticket);
+
   return true;
 }
 
@@ -267,7 +274,7 @@ bool Ndb_dd_client::mdl_lock_tablespace_exclusive(const char *tablespace_name,
     lock_wait_timeout = m_thd->variables.lock_wait_timeout;
   }
 
-  if (!mdl_locks_acquire(mdl_requests, lock_wait_timeout)) {
+  if (m_thd->mdl_context.acquire_locks(&mdl_requests, lock_wait_timeout)) {
     return false;
   }
 
@@ -277,6 +284,11 @@ bool Ndb_dd_client::mdl_lock_tablespace_exclusive(const char *tablespace_name,
   */
   if (check_readonly(m_thd, true)) return false;
 
+  // Remember tickets of the acquired mdl locks
+  m_acquired_mdl_tickets.push_back(tablespace_request.ticket);
+  m_acquired_mdl_tickets.push_back(backup_lock_request.ticket);
+  m_acquired_mdl_tickets.push_back(grl_request.ticket);
+
   return true;
 }
 
@@ -292,10 +304,14 @@ bool Ndb_dd_client::mdl_lock_tablespace(const char *tablespace_name,
 
   mdl_requests.push_front(&tablespace_request);
 
-  if (!mdl_locks_acquire(mdl_requests, m_thd->variables.lock_wait_timeout)) {
+  if (m_thd->mdl_context.acquire_locks(&mdl_requests,
+                                       m_thd->variables.lock_wait_timeout)) {
     return false;
   }
 
+  // Remember tickets of the acquired mdl locks
+  m_acquired_mdl_tickets.push_back(tablespace_request.ticket);
+
   return true;
 }
 
@@ -330,7 +346,7 @@ bool Ndb_dd_client::mdl_locks_acquire_exclusive(const char *schema_name,
     lock_wait_timeout = m_thd->variables.lock_wait_timeout;
   }
 
-  if (!mdl_locks_acquire(mdl_requests, lock_wait_timeout)) {
+  if (m_thd->mdl_context.acquire_locks(&mdl_requests, lock_wait_timeout)) {
     return false;
   }
 
@@ -340,20 +356,12 @@ bool Ndb_dd_client::mdl_locks_acquire_exclusive(const char *schema_name,
   */
   if (check_readonly(m_thd, true)) return false;
 
-  return true;
-}
-
-bool Ndb_dd_client::mdl_locks_acquire(MDL_request_list mdl_requests,
-                                      ulong lock_wait_timeout) {
-  if (m_thd->mdl_context.acquire_locks(&mdl_requests, lock_wait_timeout)) {
-    return false;
-  }
   // Remember tickets of the acquired mdl locks
-  MDL_request_list::Iterator it(mdl_requests);
-  MDL_request *request;
-  while ((request = it++)) {
-    m_acquired_mdl_tickets.push_back(request->ticket);
-  }
+  m_acquired_mdl_tickets.push_back(schema_request.ticket);
+  m_acquired_mdl_tickets.push_back(mdl_request.ticket);
+  m_acquired_mdl_tickets.push_back(backup_lock_request.ticket);
+  m_acquired_mdl_tickets.push_back(grl_request.ticket);
+
   return true;
 }
 
@@ -425,7 +433,7 @@ bool Ndb_dd_client::rename_table(
   }
   if (new_schema == nullptr) {
     // Database does not exist, unexpected
-    assert(false);
+    DBUG_ASSERT(false);
     return false;
   }
 
@@ -456,14 +464,14 @@ bool Ndb_dd_client::rename_table(
   to_table_def->set_schema_id(new_schema->id());
   to_table_def->set_name(new_table_name);
 
-  const Ndb_dd_handle dd_handle{new_table_id, new_table_version};
-  ndb_dd_table_set_spi_and_version(to_table_def, dd_handle);
+  ndb_dd_table_set_object_id_and_version(to_table_def, new_table_id,
+                                         new_table_version);
 
   // Rename foreign keys
   if (dd::rename_foreign_keys(m_thd, old_schema_name, old_table_name,
                               ndbcluster_hton, new_schema_name, to_table_def)) {
     // Failed to rename foreign keys or commit/rollback, unexpected
-    assert(false);
+    DBUG_ASSERT(false);
     return false;
   }
 
@@ -482,7 +490,7 @@ bool Ndb_dd_client::rename_table(
   // Save table in DD
   if (m_client->update(to_table_def)) {
     // Failed to save, unexpected
-    assert(false);
+    DBUG_ASSERT(false);
     return false;
   }
 
@@ -496,39 +504,6 @@ bool Ndb_dd_client::rename_table(
   return true;
 }
 
-bool Ndb_dd_client::remove_table(
-    const dd::Table *table, const char *schema_name, const char *table_name,
-    Ndb_referenced_tables_invalidator *invalidator) {
-  DBUG_TRACE;
-  DBUG_PRINT("enter", ("table_name: '%s'", table->name().c_str()));
-
-  if (table == nullptr) {
-    // Table does not exist
-    return true;
-  }
-
-  if (invalidator != nullptr &&
-      !invalidator->fetch_referenced_tables_to_invalidate(
-          schema_name, table_name, table, true)) {
-    return false;
-  }
-
-#ifdef HAVE_PSI_SP_INTERFACE
-  // Remove statistics, table is not using trigger(s) anymore
-  remove_all_triggers_from_perfschema(schema_name, *table);
-#endif
-
-  DBUG_PRINT("info", ("removing existing table"));
-  if (m_client->drop(table)) {
-    // Failed to remove table
-    // Catch in debug, unexpected error
-    assert(false);
-    return false;
-  }
-
-  return true;
-}
-
 bool Ndb_dd_client::remove_table(const char *schema_name,
                                  const char *table_name,
                                  Ndb_referenced_tables_invalidator *invalidator)
@@ -543,54 +518,30 @@ bool Ndb_dd_client::remove_table(const char *schema_name,
     return false;
   }
 
-  return remove_table(existing, schema_name, table_name, invalidator);
-}
-
-bool Ndb_dd_client::remove_table(
-    dd::Object_id spi, Ndb_referenced_tables_invalidator *invalidator) {
-  DBUG_TRACE;
-  DBUG_PRINT("enter", ("se_private_id: '%lld'", spi));
-
-  dd::Table *table;
-  dd::Schema *schema;
-  const char *engine = "ndbcluster";
-
-  // Find old table using the NDB tables id, skip missing SPI cache
-  if (m_client->acquire_uncached_table_by_se_private_id(engine, spi, &table,
-                                                        true)) {
-    ndb_log_warning("Error retrieving table definition ndbcluster-%lld", spi);
-    return false;
-  }
-
-  if (table == nullptr) {
-    // Table does not exist or was inadvertently cached. Nothing to do.
-    ndb_log_verbose(50, "Table definition with %s-%llu does not exist", engine,
-                    spi);
+  if (existing == nullptr) {
+    // Table does not exist
     return true;
   }
 
-#ifndef NDEBUG
-  // Double check that old table is in NDB
-  if (table->engine() != engine) {
-    assert(false);
+  if (invalidator != nullptr &&
+      !invalidator->fetch_referenced_tables_to_invalidate(
+          schema_name, table_name, existing, true)) {
     return false;
   }
+
+#ifdef HAVE_PSI_SP_INTERFACE
+  // Remove statistics, table is not using trigger(s) anymore
+  remove_all_triggers_from_perfschema(schema_name, *existing);
 #endif
 
-  if (m_client->acquire_uncached(table->schema_id(), &schema)) {
+  DBUG_PRINT("info", ("removing existing table"));
+  if (m_client->drop(existing)) {
+    // Failed to remove existing
+    DBUG_ASSERT(false);  // Catch in debug, unexpected error
     return false;
   }
 
-  const char *schema_name = schema->name().c_str();
-  const char *table_name = table->name().c_str();
-
-  // Take exclusive locks on old table
-  if (!mdl_locks_acquire_exclusive(schema_name, table_name)) {
-    // Failed to MDL lock old table
-    return false;
-  }
-
-  return remove_table(table, schema_name, table_name, invalidator);
+  return true;
 }
 
 bool Ndb_dd_client::deserialize_table(const dd::sdi_t &sdi,
@@ -635,7 +586,43 @@ bool Ndb_dd_client::store_table(dd::Table *install_table, int ndb_table_id) {
     // will set a new error
     m_thd->clear_error();
 
-    if (!remove_table(ndb_table_id)) {
+    // Find old table using the NDB tables id
+    dd::Table *old_table_def;
+    if (m_client->acquire_uncached_table_by_se_private_id(
+            "ndbcluster", ndb_table_id, &old_table_def)) {
+      // There was no old table
+      return false;
+    }
+
+    // Double check that old table is in NDB
+    if (old_table_def->engine() != "ndbcluster") {
+      DBUG_ASSERT(false);
+      return false;
+    }
+
+    // Lookup schema name of old table
+    dd::Schema *old_schema;
+    if (m_client->acquire_uncached(old_table_def->schema_id(), &old_schema)) {
+      return false;
+    }
+
+    if (old_schema == nullptr) {
+      DBUG_ASSERT(false);  // Database does not exist
+      return false;
+    }
+
+    const char *old_schema_name = old_schema->name().c_str();
+    const char *old_table_name = old_table_def->name().c_str();
+    DBUG_PRINT("info", ("Found old table '%s.%s', will try to remove it",
+                        old_schema_name, old_table_name));
+
+    // Take exclusive locks on old table
+    if (!mdl_locks_acquire_exclusive(old_schema_name, old_table_name)) {
+      // Failed to MDL lock old table
+      return false;
+    }
+
+    if (!remove_table(old_schema_name, old_table_name)) {
       // Failed to remove old table from DD
       return false;
     }
@@ -649,7 +636,7 @@ bool Ndb_dd_client::store_table(dd::Table *install_table, int ndb_table_id) {
     }
 
     // Removed old table and stored the new, return OK
-    assert(!m_thd->is_error());
+    DBUG_ASSERT(!m_thd->is_error());
     return true;
   }
 
@@ -667,7 +654,7 @@ bool Ndb_dd_client::install_table(
     return false;
   }
   if (schema == nullptr) {
-    assert(false);  // Database does not exist
+    DBUG_ASSERT(false);  // Database does not exist
     return false;
   }
 
@@ -678,20 +665,20 @@ bool Ndb_dd_client::install_table(
 
   // Verify that table_name in the unpacked table definition
   // matches the table name to install
-  assert(ndb_dd_fs_name_case(install_table->name()) == table_name);
+  DBUG_ASSERT(ndb_dd_fs_name_case(install_table->name()) == table_name);
 
-  // Verify that table definition unpacked from NDB
+  // Verify that table defintion unpacked from NDB
   // does not have any se_private fields set, those will be set
   // from the NDB table metadata
-  assert(install_table->se_private_id() == dd::INVALID_OBJECT_ID);
-  assert(install_table->se_private_data().raw_string() == "");
+  DBUG_ASSERT(install_table->se_private_id() == dd::INVALID_OBJECT_ID);
+  DBUG_ASSERT(install_table->se_private_data().raw_string() == "");
 
   // Assign the id of the schema to the table_object
   install_table->set_schema_id(schema->id());
 
   // Assign NDB id and version of the table
-  const Ndb_dd_handle dd_handle{ndb_table_id, ndb_table_version};
-  ndb_dd_table_set_spi_and_version(install_table.get(), dd_handle);
+  ndb_dd_table_set_object_id_and_version(install_table.get(), ndb_table_id,
+                                         ndb_table_version);
 
   // Check if the DD table object has the correct number of partitions.
   // Correct the number of partitions in the DD table object in case of
@@ -711,92 +698,63 @@ bool Ndb_dd_client::install_table(
     ndb_dd_table_set_tablespace_id(install_table.get(), tablespace_id);
   }
 
-  // Get the old table definition and compare the ndb_table_id with the
-  // already installed id (on se_private_id). First, attempt to get the
-  // definition using the table's full name. If the ids are different,
-  // attempt to retrieve the previous definition using the to-be-installed
-  // ndb_table_id. Drop all required definitions to install the new
-
-  const dd::Table *old_table_def = nullptr;
-  if (m_client->acquire(schema_name, table_name, &old_table_def)) {
+  const dd::Table *existing = nullptr;
+  if (m_client->acquire(schema_name, table_name, &existing)) {
     return false;
   }
 
   if (invalidator != nullptr &&
       !invalidator->fetch_referenced_tables_to_invalidate(
-          schema_name, table_name, old_table_def)) {
+          schema_name, table_name, existing)) {
     return false;
   }
 
-  if (old_table_def != nullptr) {
-    Ndb_dd_handle old_handle = ndb_dd_table_get_spi_and_version(old_table_def);
-    if (!old_handle.valid()) {
+  if (existing != nullptr) {
+    // Get id and version of existing table
+    int object_id, object_version;
+    if (!ndb_dd_table_get_object_id_and_version(existing, object_id,
+                                                object_version)) {
       DBUG_PRINT("error", ("Could not extract object_id and object_version "
                            "from table definition"));
-      assert(false);
+      DBUG_ASSERT(false);
       return false;
     }
 
-    // Table already exists so do force overwrite
-    if (!force_overwrite) {
-      assert(false);
-      return false;
-    }
-
-    // Check that se_private_id of the existing table in DD
+    // Check that id and version of the existing table in DD
     // matches NDB, otherwise it's a programming error
     // not to request "force_overwrite"
-    if (old_handle == dd_handle) {
+    if (ndb_table_id == object_id && ndb_table_version == object_version) {
       // Table is already installed, with same id and version
-      // return success
+      // return sucess
       return true;
     }
 
-    // Drop the old table definition of schema_name.table_name
-    // It needs to be done to free up "schema-id"-"table_name" DD entries
-    DBUG_PRINT("info", ("dropping existing table"));
-    if (m_client->drop(old_table_def)) {
-      // Failed to drop existing
-      assert(false);  // Catch in debug, unexpected error
+    // Table already exists
+    if (!force_overwrite) {
+      // Don't overwrite existing table
+      DBUG_ASSERT(false);
       return false;
     }
 
-    // If the table to be installed has a different se_private_id than
-    // the existing, then the currently installed se_private_id table must
-    // also be removed if exists.
-    if (dd_handle.spi != old_handle.spi) {
-      ndb_log_info(
-          "Cached se_private_id different than ndb_table_id. "
-          "Removing ndbcluster-%llu table definition if exists",
-          dd_handle.spi);
-
-      if (!remove_table(dd_handle.spi)) {
-        ndb_log_warning(
-            "Failed to remove table definition from DD for spi=%llu. ",
-            dd_handle.spi);
-      }
-    }
-  } else {
-    // Remove any stale DD table that may be occupying this
-    // ndbcluster-<id> place
-    if (!remove_table(dd_handle.spi)) {
-      ndb_log_info("Failed to remove ndbcluster-%llu from DD", dd_handle.spi);
+    // Continue and remove the old table before
+    // installing the new
+    DBUG_PRINT("info", ("dropping existing table"));
+    if (m_client->drop(existing)) {
+      // Failed to drop existing
+      DBUG_ASSERT(false);  // Catch in debug, unexpected error
+      return false;
     }
   }
 
   if (!store_table(install_table.get(), ndb_table_id)) {
-    const dd::sdi_t new_table_sdi = ndb_dd_sdi_serialize(
-        m_thd, *install_table.get(), dd::String_type(schema_name));
-
     ndb_log_error("Failed to store table: '%s.%s'", schema_name, table_name);
-    ndb_log_error("sdi for new table: %s",
-                  ndb_dd_sdi_prettify(new_table_sdi).c_str());
-
-    if (old_table_def) {
-      const dd::sdi_t old_table_sdi = ndb_dd_sdi_serialize(
-          m_thd, *old_table_def, dd::String_type(schema_name));
-      ndb_log_error("sdi for existing table: %s",
-                    ndb_dd_sdi_prettify(old_table_sdi).c_str());
+    ndb_log_error_dump("sdi for new table: %s",
+                       ndb_dd_sdi_prettify(sdi).c_str());
+    if (existing) {
+      const dd::sdi_t existing_sdi =
+          ndb_dd_sdi_serialize(m_thd, *existing, dd::String_type(schema_name));
+      ndb_log_error_dump("sdi for existing table: %s",
+                         ndb_dd_sdi_prettify(existing_sdi).c_str());
     }
     DBUG_ABORT();
     return false;
@@ -862,7 +820,7 @@ bool Ndb_dd_client::set_tablespace_id_in_table(const char *schema_name,
     return false;
   }
   if (table_def == nullptr) {
-    assert(false);
+    DBUG_ASSERT(false);
     return false;
   }
 
@@ -888,8 +846,7 @@ bool Ndb_dd_client::set_object_id_and_version_in_table(const char *schema_name,
   }
 
   /* Update id and version */
-  Ndb_dd_handle dd_handle{object_id, object_version};
-  ndb_dd_table_set_spi_and_version(table_def, dd_handle);
+  ndb_dd_table_set_object_id_and_version(table_def, object_id, object_version);
 
   /* Update it to DD */
   if (m_client->update(table_def)) {
@@ -1082,7 +1039,7 @@ bool Ndb_dd_client::is_local_table(const char *schema_name,
   }
   if (table == nullptr) {
     // The table doesn't exist
-    assert(false);
+    DBUG_ASSERT(false);
     return false;
   }
   local_table = table->engine() != "ndbcluster";
@@ -1126,7 +1083,7 @@ bool Ndb_dd_client::update_schema_version(const char *schema_name,
   DBUG_PRINT("enter", ("Schema : %s, counter : %u, node_id : %u", schema_name,
                        counter, node_id));
 
-  assert(m_thd->mdl_context.owns_equal_or_stronger_lock(
+  DBUG_ASSERT(m_thd->mdl_context.owns_equal_or_stronger_lock(
       MDL_key::SCHEMA, schema_name, "", MDL_EXCLUSIVE));
 
   dd::Schema *schema;
@@ -1154,11 +1111,11 @@ bool Ndb_dd_client::lookup_tablespace_id(const char *tablespace_name,
   DBUG_TRACE;
   DBUG_PRINT("enter", ("tablespace_name: %s", tablespace_name));
 
-  assert(m_thd->mdl_context.owns_equal_or_stronger_lock(
+  DBUG_ASSERT(m_thd->mdl_context.owns_equal_or_stronger_lock(
       MDL_key::TABLESPACE, "", tablespace_name, MDL_INTENTION_EXCLUSIVE));
 
   // Acquire tablespace.
-  const dd::Tablespace *ts_obj = nullptr;
+  const dd::Tablespace *ts_obj = NULL;
   if (m_client->acquire(tablespace_name, &ts_obj)) {
     // acquire() always fails with an error being reported.
     return false;
@@ -1274,7 +1231,7 @@ bool Ndb_dd_client::install_tablespace(
   tablespace->set_engine("ndbcluster");
 
   // Add data files
-  for (const auto &data_file_name : data_file_names) {
+  for (const auto data_file_name : data_file_names) {
     ndb_dd_disk_data_add_file(tablespace.get(), data_file_name.c_str());
   }
 
@@ -1428,7 +1385,7 @@ bool Ndb_dd_client::install_logfile_group(
   logfile_group->set_engine("ndbcluster");
 
   // Add undofiles
-  for (const auto &undo_file_name : undo_file_names) {
+  for (const auto undo_file_name : undo_file_names) {
     ndb_dd_disk_data_add_file(logfile_group.get(), undo_file_name.c_str());
   }
 
@@ -1510,7 +1467,7 @@ bool Ndb_dd_client::get_schema_uuid(dd::String_type *value) const {
   const dd::Table *table = nullptr;
   if (m_client->acquire(Ndb_schema_dist_table::DB_NAME.c_str(),
                         Ndb_schema_dist_table::TABLE_NAME.c_str(), &table)) {
-    assert(false);
+    DBUG_ASSERT(false);
     return false;
   }
 
@@ -1536,13 +1493,13 @@ bool Ndb_dd_client::update_schema_uuid(const char *value) const {
   if (m_client->acquire_for_modification(
           Ndb_schema_dist_table::DB_NAME.c_str(),
           Ndb_schema_dist_table::TABLE_NAME.c_str(), &table)) {
-    assert(false);
+    DBUG_ASSERT(false);
     return false;
   }
 
   if (table == nullptr) {
     // Table does not exist in DD
-    assert(false);
+    DBUG_ASSERT(false);
     return false;
   }
 
@@ -1550,7 +1507,7 @@ bool Ndb_dd_client::update_schema_uuid(const char *value) const {
   ndb_dd_table_set_schema_uuid(table, value);
 
   if (m_client->update(table)) {
-    assert(false);
+    DBUG_ASSERT(false);
     return false;
   }
 
@@ -1650,20 +1607,23 @@ bool Ndb_referenced_tables_invalidator::fetch_referenced_tables_to_invalidate(
 }
 
 /**
-  @brief Invalidate all the referenced tables from the MySQL DD cache.
+  Invalidate all the tables in the referenced_tables set by closing
+  any cached instances in the table definition cache and invalidating
+  the same from the local DD.
 
   @return true        On success.
   @return false       Invalidation failed.
 */
 bool Ndb_referenced_tables_invalidator::invalidate() const {
   DBUG_TRACE;
-  for (auto const &parent_it : m_referenced_tables) {
-    // Invalidate the table from DD
+  for (auto parent_it : m_referenced_tables) {
+    // Invalidate Table and Table Definition Caches too.
     const char *schema_name = parent_it.first.c_str();
     const char *table_name = parent_it.second.c_str();
     DBUG_PRINT("info",
                ("Invalidating parent table '%s.%s'", schema_name, table_name));
-    if (m_thd->dd_client()->invalidate(schema_name, table_name) != 0) {
+    if (ndb_tdc_close_cached_table(m_thd, schema_name, table_name) ||
+        m_thd->dd_client()->invalidate(schema_name, table_name) != 0) {
       DBUG_PRINT("error", ("Unable to invalidate table '%s.%s'", schema_name,
                            table_name));
       return false;
@@ -1671,179 +1631,3 @@ bool Ndb_referenced_tables_invalidator::invalidate() const {
   }
   return true;
 }
-
-#ifndef NDEBUG
-/*
-   Tables in DD are stored in the mysql.tables which is defined like this:
-
-   CREATE TABLE mysql.tables (
-     object_id BIGINT PRIMARY KEY,  // DD object's id
-     schema_name VARCHAR(255),
-     table_name VARCHAR(255),
-     ..
-     engine VARCHAR(255),
-     se_private_id INT,
-     se_private_data JSON,
-     UNIQUE(engine, se_private_id), // Unique index preventing each SE to
-                                    // store more than one table with same
-                                    // se_private_id
-   )
-   For each table in NDB, the ndbcluster plugin stores
-   the NDB table's id and version in the DD using:
-     - se_private_id -> the NDB table id
-     - se_private_data["object_version"] -> the NDB table version
-   For example, the hypothetical NDB table 'db1.t1' with NDB table id 37 and
-   version 9 would be returned like this using SQL:
-    SELECT .. FROM mysql.tables WHERE
-      schema_name = db1 AND table_name = t1;
-        object_id: 479
-        schema_name: db1
-        table_name: t1
-        ...
-        engine = ndbcluster
-        se_private_id = 37
-        se_private_data = {
-           object_version: 9
-        }
-*/
-bool Ndb_dd_client::dump_NDB_tables() {
-  fprintf(stderr, "Ndb_dd_client::dump_NDB_tables\n");
-  fprintf(stderr,
-          "| object_id | db_name | table_name | ... | engine | se_private_id | "
-          "se_private_data |\n");
-  // Fetch list of schemas in DD
-  std::vector<std::string> schema_names;
-  if (!fetch_schema_names(&schema_names)) {
-    return true;
-  }
-  // Iterate over each schema in DD and show all NDB tables
-  for (const std::string &schema_name : schema_names) {
-    const dd::Schema *schema = nullptr;
-    if (!mdl_lock_schema(schema_name.c_str()) ||
-        m_client->acquire(schema_name.c_str(), &schema)) {
-      return true;
-    }
-    assert(schema);
-    std::vector<const dd::Table *> ndb_tables;
-    if (m_client->fetch_schema_components(schema, &ndb_tables)) {
-      return true;
-    }
-    for (const dd::Table *table_def : ndb_tables) {
-      if (table_def->engine() != "ndbcluster") continue;
-      const Ndb_dd_handle ndb_id = ndb_dd_table_get_spi_and_version(table_def);
-      assert(table_def->se_private_id() == ndb_id.spi);
-      fprintf(stderr,
-              "| %llu | %s | %s | ... | %s | %llu | { object_version: %d} |\n",
-              table_def->id(), schema_name.c_str(), table_def->name().c_str(),
-              "ndbcluster", ndb_id.spi, ndb_id.version);
-    }
-  }
-  return false;
-}
-
-bool Ndb_dd_client::dbug_shuffle_spi_for_NDB_tables() {
-  fprintf(stderr, "Ndb_dd_client::dbug_shuffle_spi_for_NDB_tables\n");
-  // Fetch list of schemas in DD
-  std::vector<std::string> schema_names;
-  if (!fetch_schema_names(&schema_names)) {
-    return true;
-  }
-
-  dd::Object_id old_id = 0;
-  // Iterate over each schema in DD
-  for (const std::string &schema_name : schema_names) {
-    const dd::Schema *schema = nullptr;
-    if (!mdl_lock_schema(schema_name.c_str()) ||
-        m_client->acquire(schema_name.c_str(), &schema)) {
-      return true;
-    }
-    assert(schema);
-    std::vector<const dd::Table *> ndb_tables;
-    if (m_client->fetch_schema_components(schema, &ndb_tables)) {
-      return true;
-    }
-    for (const dd::Table *table_def : ndb_tables) {
-      if (table_def->engine() != "ndbcluster") {
-        // Skip non NDB tables
-        continue;
-      }
-      if (!mdl_locks_acquire_exclusive(schema_name.c_str(),
-                                       table_def->name().c_str())) {
-        assert(false);
-        return true;
-      }
-      const Ndb_dd_handle ndb_id = ndb_dd_table_get_spi_and_version(table_def);
-      const dd::Object_id new_id = old_id;
-      old_id = ndb_id.spi;
-
-      fprintf(stderr, " %s: %llu -> %llu\n", table_def->name().c_str(), old_id,
-              new_id);
-
-      if (!set_object_id_and_version_in_table(schema_name.c_str(),
-                                              table_def->name().c_str(), new_id,
-                                              ndb_id.version)) {
-        // Failure to update can happen when there is already another table
-        // using the decreased se_private_id, skip these tables for now and
-        // thus cause only some tables to later need to be reinstalled from NDB
-        // NOTE! To update _all_ tables it would be necessary to update tables
-        // in sorted order, starting from table with lowest se_private_id
-        const int error = m_thd->get_stmt_da()->mysql_errno();
-        assert(error == 1062);
-        fprintf(stderr, "Failed to update %s, error: %u\n",
-                table_def->name().c_str(), error);
-        continue;
-      }
-    }
-  }
-  // Save updated table definitions to DD
-  commit();
-  return false;
-}
-
-bool Ndb_dd_client::change_version_for_table(const char *schema_name,
-                                             const char *table_name,
-                                             uint new_version) {
-  DBUG_TRACE;
-
-  if (!mdl_lock_schema(schema_name)) {
-    assert(false);
-    return false;
-  }
-
-  // Read new schema from DD
-  const dd::Schema *schema = nullptr;
-  if (m_client->acquire(schema_name, &schema)) {
-    return false;
-  }
-  if (schema == nullptr) {
-    // Database does not exist
-    return false;
-  }
-
-  if (!mdl_locks_acquire_exclusive(schema_name, table_name)) {
-    assert(false);
-    return false;
-  }
-
-  // Read table from DD
-  const dd::Table *table_def = nullptr;
-  if (m_client->acquire(schema_name, table_name, &table_def)) return false;
-
-  if (table_def == nullptr) {
-    return false;
-  }
-
-  const Ndb_dd_handle ndb_id = ndb_dd_table_get_spi_and_version(table_def);
-  if (!set_object_id_and_version_in_table(schema_name, table_name, ndb_id.spi,
-                                          new_version)) {
-    return false;
-  }
-  // Save updated table definitions to DD
-  commit();
-
-  std::cout << "Ndb_dd_client: Changed version of " << schema_name << "."
-            << table_name << " to " << new_version << std::endl;
-
-  return true;
-}
-#endif
