@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2018, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -35,12 +34,11 @@
 #include "my_dbug.h"
 #include "mysql/psi/mysql_cond.h"   // mysql_cond_t
 #include "mysql/psi/mysql_mutex.h"  // mysql_mutex_t
-#include "nulls.h"                  // NullS
 #include "sql/sql_class.h"          // THD
 #include "sql/table.h"              // is_infoschema_db() / is_perfschema_db()
 #include "storage/ndb/include/ndbapi/NdbError.hpp"    // NdbError
 #include "storage/ndb/plugin/ha_ndbcluster_binlog.h"  // ndb_binlog_is_read_only
-#include "storage/ndb/plugin/ha_ndbcluster_connection.h"  // ndbcluster_is_ready
+#include "storage/ndb/plugin/ha_ndbcluster_connection.h"  // ndbcluster_is_connected
 #include "storage/ndb/plugin/ndb_dd_client.h"             // Ndb_dd_client
 #include "storage/ndb/plugin/ndb_ndbapi_util.h"           // ndb_get_*_names
 #include "storage/ndb/plugin/ndb_sleep.h"                 // ndb_milli_sleep
@@ -48,7 +46,7 @@
 #include "storage/ndb/plugin/ndb_thd_ndb.h"               // Thd_ndb
 
 Ndb_metadata_change_monitor::Ndb_metadata_change_monitor()
-    : Ndb_component("Metadata", "ndb_metadata"), m_mark_sync_complete{false} {}
+    : Ndb_component("Metadata"), m_mark_sync_complete{false} {}
 
 Ndb_metadata_change_monitor::~Ndb_metadata_change_monitor() {}
 
@@ -73,10 +71,6 @@ void Ndb_metadata_change_monitor::set_check_interval(
 }
 
 void Ndb_metadata_change_monitor::signal_metadata_sync_enabled() {
-  // Clear all excluded objects to enable the detection of all possible
-  // mismatches. This enables the user to easily retry the sync of objects
-  // that had previously failed due to permanent errors
-  ndbcluster_binlog_clear_sync_excluded_objects();
   mysql_mutex_lock(&m_wait_mutex);
   mysql_cond_signal(&m_wait_cond);
   mysql_mutex_unlock(&m_wait_mutex);
@@ -129,7 +123,7 @@ bool Ndb_metadata_change_monitor::detect_logfile_group_changes(
     return false;
   }
 
-  for (const auto &logfile_group_name : lfg_in_NDB) {
+  for (const auto logfile_group_name : lfg_in_NDB) {
     if (lfg_in_DD.find(logfile_group_name) == lfg_in_DD.end()) {
       // Exists in NDB but not in DD
       std::vector<std::string> undofile_names;
@@ -160,7 +154,7 @@ bool Ndb_metadata_change_monitor::detect_logfile_group_changes(
     }
   }
 
-  for (const auto &logfile_group_name : lfg_in_DD) {
+  for (const auto logfile_group_name : lfg_in_DD) {
     // Exists in DD but not in NDB
     if (ndbcluster_binlog_check_logfile_group_async(logfile_group_name)) {
       increment_metadata_detected_count();
@@ -193,7 +187,7 @@ bool Ndb_metadata_change_monitor::detect_tablespace_changes(
     return false;
   }
 
-  for (const auto &tablespace_name : tablespaces_in_NDB) {
+  for (const auto tablespace_name : tablespaces_in_NDB) {
     if (tablespaces_in_DD.find(tablespace_name) == tablespaces_in_DD.end()) {
       // Exists in NDB but not in DD
       std::vector<std::string> datafile_names;
@@ -223,7 +217,7 @@ bool Ndb_metadata_change_monitor::detect_tablespace_changes(
     }
   }
 
-  for (const auto &tablespace_name : tablespaces_in_DD) {
+  for (const auto tablespace_name : tablespaces_in_DD) {
     // Exists in DD but not in NDB
     if (ndbcluster_binlog_check_tablespace_async(tablespace_name)) {
       increment_metadata_detected_count();
@@ -263,9 +257,9 @@ bool Ndb_metadata_change_monitor::detect_schema_changes(
 
 bool Ndb_metadata_change_monitor::detect_table_changes_in_schema(
     THD *thd, const Thd_ndb *thd_ndb, const std::string &schema_name) const {
+  // Fetch list of tables in NDB
   NdbDictionary::Dictionary *dict = thd_ndb->ndb->getDictionary();
   std::unordered_set<std::string> ndb_tables_in_NDB;
-  // Fetch list of tables in NDB
   if (!ndb_get_table_names_in_schema(dict, schema_name, &ndb_tables_in_NDB)) {
     log_NDB_error(dict->getNdbError());
     log_info("Failed to get list of tables in schema '%s' from NDB",
@@ -292,17 +286,10 @@ bool Ndb_metadata_change_monitor::detect_table_changes_in_schema(
     return false;
   }
 
-  // Remove NDB utility table mysql.ndb_apply_status, it's managed
-  // by the binlog thread but not marked as hidden in DD. It's masked in similar
-  // way from the list of tables in NDB.
-  if (schema_name == "mysql") {
-    ndb_tables_in_DD.erase("ndb_apply_status");
-  }
-
   // Special case when all NDB tables belonging to a schema still exist in DD
   // but not in NDB
   if (ndb_tables_in_NDB.empty() && !ndb_tables_in_DD.empty()) {
-    for (const auto &ndb_table_name : ndb_tables_in_DD) {
+    for (const auto ndb_table_name : ndb_tables_in_DD) {
       // Exists in DD but not in NDB
       if (ndbcluster_binlog_check_table_async(schema_name, ndb_table_name)) {
         increment_metadata_detected_count();
@@ -318,7 +305,7 @@ bool Ndb_metadata_change_monitor::detect_table_changes_in_schema(
   // not in DD (as either NDB or shadow tables)
   if (!ndb_tables_in_NDB.empty() && ndb_tables_in_DD.empty() &&
       local_tables_in_DD.empty()) {
-    for (const auto &ndb_table_name : ndb_tables_in_NDB) {
+    for (const auto ndb_table_name : ndb_tables_in_NDB) {
       // Exists in NDB but not in DD
       if (ndbcluster_binlog_check_table_async(schema_name, ndb_table_name)) {
         increment_metadata_detected_count();
@@ -330,7 +317,7 @@ bool Ndb_metadata_change_monitor::detect_table_changes_in_schema(
     return true;
   }
 
-  for (const auto &ndb_table_name : ndb_tables_in_NDB) {
+  for (const auto ndb_table_name : ndb_tables_in_NDB) {
     if (ndb_tables_in_DD.find(ndb_table_name) == ndb_tables_in_DD.end() &&
         local_tables_in_DD.find(ndb_table_name) == local_tables_in_DD.end()) {
       // Exists in NDB but not in DD
@@ -346,7 +333,7 @@ bool Ndb_metadata_change_monitor::detect_table_changes_in_schema(
     }
   }
 
-  for (const auto &ndb_table_name : ndb_tables_in_DD) {
+  for (const auto ndb_table_name : ndb_tables_in_DD) {
     // Exists in DD but not in NDB
     if (ndbcluster_binlog_check_table_async(schema_name, ndb_table_name)) {
       increment_metadata_detected_count();
@@ -494,23 +481,15 @@ void Ndb_metadata_change_monitor::do_run() {
   Thread_handle_guard thd_guard;
   THD *thd = thd_guard.get_thd();
   if (thd == nullptr) {
-    assert(false);
+    DBUG_ASSERT(false);
     log_error("Failed to allocate THD");
     return;
   }
 
-  Thd_ndb_guard thd_ndb_guard(thd);
-  const Thd_ndb *thd_ndb = thd_ndb_guard.get_thd_ndb();
-  if (thd_ndb == nullptr) {
-    assert(false);
-    log_error("Failed to allocate Thd_ndb");
-    return;
-  }
-
   for (;;) {
     // Outer loop to ensure that if the connection to NDB is lost, a fresh
     // connection is established before the thread continues its processing
-    while (!ndb_connection_is_ready(thd_ndb->connection, 1)) {
+    while (!ndbcluster_is_connected(1)) {
       // No connection to NDB yet. Retry until connection is established while
       // checking if stop has been requested at 1 second intervals
       if (is_stop_requested()) {
@@ -518,9 +497,15 @@ void Ndb_metadata_change_monitor::do_run() {
       }
     }
 
-    for (;;) {
-      Ndb_thd_memory_guard metadata_change_loop_guard(thd);
+    Thd_ndb_guard thd_ndb_guard(thd);
+    const Thd_ndb *thd_ndb = thd_ndb_guard.get_thd_ndb();
+    if (thd_ndb == nullptr) {
+      DBUG_ASSERT(false);
+      log_error("Failed to allocate Thd_ndb");
+      return;
+    }
 
+    for (;;) {
       // Inner loop where each iteration represents one "lap" of the thread
       Run_controller controller(g_metadata_detected_count);
       while (!controller.check_enabled() && !controller.sync_enabled()) {
@@ -590,7 +575,7 @@ void Ndb_metadata_change_monitor::do_run() {
       }
 
       // Check if NDB connection is still valid
-      if (!ndb_connection_is_ready(thd_ndb->connection, 1)) {
+      if (!ndbcluster_is_connected(1)) {
         // Break out of inner loop
         log_info(
             "Connection to NDB was lost. Attempting to establish a new "
@@ -600,7 +585,7 @@ void Ndb_metadata_change_monitor::do_run() {
 
       log_verbose(10, "Metadata check started");
 
-      ndbcluster_binlog_validate_sync_excluded_objects(thd);
+      ndbcluster_binlog_validate_sync_blacklist(thd);
 
       if (!detect_logfile_group_changes(thd, thd_ndb)) {
         log_info("Failed to detect logfile group metadata changes");
@@ -626,54 +611,36 @@ void Ndb_metadata_change_monitor::do_run() {
       log_verbose(10, "Table metadata check completed");
       log_verbose(10, "Metadata check completed");
 
-      if (controller.get_metadata_sync()) {
-        if (controller.all_changes_detected()) {
-          /*
-            All changes at this point in time have been detected. Since the
-            ndb_metadata_sync option has been set, we don't expect more changes.
-            Stall the thread and prevent it from checking for further mismatches
-            until the current queue has been synchronized by the binlog thread
-          */
-          mysql_mutex_lock(&m_sync_done_mutex);
-          mysql_cond_wait(&m_sync_done_cond, &m_sync_done_mutex);
-          mysql_mutex_unlock(&m_sync_done_mutex);
-          if (!m_mark_sync_complete) {
-            /*
-              This is the first instance of the binlog thread having synced
-              all changes submitted to it. However, the change monitor thread
-              has been stalled for a while so we opt for at least one more
-              detection and sync cycle to ensure that all changes are synced.
-              This is particularly relevant to synchronization of schema objects
-              since they have to be installed in DD for their tables to be
-              detected. This synchronization is dependent on the load on the
-              binlog thread so an additional detection and sync run after we
-              know for a fact that such schemas have been installed could be
-              useful.
-
-              The below flag denotes that the we've already detected an instance
-              of all objects having been synchronized and that ndb_metadata_sync
-              can be flipped if the the same condition is detected in the
-              following run.
-            */
-            m_mark_sync_complete = true;
-          } else {
-            // Clear retry objects
-            ndbcluster_binlog_clear_sync_retry_objects();
-            // Set ndb_metadata_sync to false to denote that all changes have
-            // been detected and synchronized
-            opt_ndb_metadata_sync = false;
-            // Reset the flag to its default value
-            m_mark_sync_complete = false;
-            log_info("Metadata synchronization complete");
-          }
+      if (controller.get_metadata_sync() && controller.all_changes_detected()) {
+        // All changes at this point in time have been detected. Since the
+        // ndb_metadata_sync option has been set, we don't expect more changes.
+        // Stall the thread and prevent it from checking for further mismatches
+        // until the current queue has been synchronized by the binlog thread
+        mysql_mutex_lock(&m_sync_done_mutex);
+        mysql_cond_wait(&m_sync_done_cond, &m_sync_done_mutex);
+        mysql_mutex_unlock(&m_sync_done_mutex);
+        if (!m_mark_sync_complete) {
+          // This is the first instance of the binlog thread having synchronized
+          // all changes submitted to it. However, the change monitor thread
+          // has been stalled for a while so we opt for at least one more
+          // detection and sync cycle to ensure that all changes are synced.
+          // This is particularly relevant to synchronization of schema objects
+          // since they have to be installed in DD for their tables to be
+          // detected. This synchronization is dependent on the load on the
+          // binlog thread so an additional detection and sync run after we know
+          // for a fact that such schemata have been installed could be useful.
+          //
+          // The below flag denotes that the we've already detected an instance
+          // of all objects having been synchronized and that ndb_metadata_sync
+          // can be flipped the next time we detect the same condition
+          m_mark_sync_complete = true;
         } else {
-          /*
-            Changes detected in this run. The flag is checked to see if the
-            previous run had marked it as complete. It is only after consecutive
-            runs with no new changes detected that the ndb_metadata_sync is
-            flipped.
-          */
-          if (m_mark_sync_complete) m_mark_sync_complete = false;
+          log_info("Metadata synchronization complete");
+          // Set ndb_metadata_sync to false to denote that all changes have been
+          // detected and synchronized
+          opt_ndb_metadata_sync = false;
+          // Reset the flag to its default value
+          m_mark_sync_complete = false;
         }
       }
     }
