@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2019, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -47,11 +46,11 @@
     Execution will be retried only if the THD has not been killed. To disable
     this check, pass nullptr to thd param instead of passing a valid pointer.
 
-  @param ndb                The Ndb object.
-  @param thd                THD object pointer.
+  @param ndb*               The Ndb object.
+  @param thd*               THD object pointer.
   @param retry_sleep        The amount of time(in ms) to sleep before retrying
                             in case of a temporary error.
-  @param[out] last_ndb_err  The last NdbError in case of failure
+  @param last_ndb_err[out]  The last NdbError in case of failure
   @param ndb_func           The std::function instance of the function that
                             needs to be executed by this wrapper. The function
                             should take a NdbTransaction* parameter followed by
@@ -66,12 +65,12 @@
 template <typename... FunctionArgTypes, typename... FunctionArgs>
 bool ndb_execute_and_retry(
     Ndb *ndb, const THD *thd, unsigned int retry_sleep, NdbError &last_ndb_err,
-    const std::function<const NdbError *(NdbTransaction *, FunctionArgTypes...)>
-        &ndb_func,
+    std::function<const NdbError *(NdbTransaction *, FunctionArgTypes...)>
+        ndb_func,
     FunctionArgs... args) {
   int retries = 100;
   const NdbError *ndbError;
-  assert(ndb != nullptr);
+  DBUG_ASSERT(ndb != nullptr);
 
   do {
     /* Start transaction */
@@ -124,8 +123,8 @@ bool ndb_execute_and_retry(
 template <typename... FunctionArgTypes, typename... FunctionArgs>
 bool ndb_trans_retry(
     Ndb *ndb, const THD *thd, NdbError &last_ndb_err,
-    const std::function<const NdbError *(NdbTransaction *, FunctionArgTypes...)>
-        &ndb_func,
+    std::function<const NdbError *(NdbTransaction *, FunctionArgTypes...)>
+        ndb_func,
     FunctionArgs... args) {
   return ndb_execute_and_retry<FunctionArgTypes...>(
       ndb, thd, 30, last_ndb_err, ndb_func,
