@@ -1,17 +1,16 @@
 /*
-  Copyright (c) 2019, 2024, Oracle and/or its affiliates.
+  Copyright (c) 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,22 +28,16 @@
 #include <mutex>
 #include <unordered_set>
 
-#include "m_string.h"
-#include "mysql/components/my_service.h"
-#include "mysql/components/services/dynamic_privilege.h"
 #include "sql/auth/acl_change_notification.h"
 #include "sql/mem_root_array.h"
 #include "sql/sql_lex.h"
-#include "sql/statement/ed_connection.h"
+#include "sql/sql_prepare.h"
 #include "storage/ndb/plugin/ndb_local_connection.h"
 #include "storage/ndb/plugin/ndb_log.h"
-#include "storage/ndb/plugin/ndb_mysql_services.h"
 #include "storage/ndb/plugin/ndb_retry.h"
-#include "storage/ndb/plugin/ndb_rpl_filter.h"
 #include "storage/ndb/plugin/ndb_sql_metadata_table.h"
 #include "storage/ndb/plugin/ndb_thd.h"
 #include "storage/ndb/plugin/ndb_thd_ndb.h"
-#include "string_with_len.h"
 
 using ChangeNotice = const Acl_change_notification;
 
@@ -54,14 +47,14 @@ using ChangeNotice = const Acl_change_notification;
 namespace {
 
 /* Static file-scope data */
-static Ndb_sql_metadata_api metadata_table;
-static std::unordered_set<std::string> local_granted_users;
-static std::mutex local_granted_users_mutex;
+Ndb_sql_metadata_api metadata_table;
+std::unordered_set<std::string> local_granted_users;
+std::mutex local_granted_users_mutex;
 
 /* Utility functions */
 
 bool op_grants_or_revokes_ndb_storage(ChangeNotice *notice) {
-  for (const ChangeNotice::Priv priv : notice->get_dynamic_privilege_list())
+  for (const std::string &priv : notice->get_dynamic_privilege_list())
     if (!native_strncasecmp(priv.c_str(), "NDB_STORED_USER", priv.length()))
       return true;
   return false;
@@ -114,7 +107,6 @@ class ThreadContext : public Ndb_local_connection {
 
   bool get_local_user(const std::string &) const;
   int get_grants_for_user(std::string);
-  bool show_create_user(std::string, std::string &, bool use_hex = true);
   void get_create_user(std::string, int);
   void create_user(std::string &, std::string &);
 
@@ -290,44 +282,20 @@ void ThreadContext::deserialize_users(std::string &str) {
 
 /* returns false on success */
 bool ThreadContext::exec_sql(const std::string &statement) {
-  // Disable rpl_filter as otherwise the non-updating query fail in the applier
-  Ndb_rpl_filter_disable disable_filter(m_thd);
-
-  assert(m_closed);
+  DBUG_ASSERT(m_closed);
+  uint ignore_mysql_errors[1] = {0};  // Don't ignore any errors
+  MYSQL_LEX_STRING sql_text = {const_cast<char *>(statement.c_str()),
+                               statement.length()};
   /* execute_query_iso() returns false on success */
-  if (execute_query_iso(statement, nullptr)) {
-    // Query failed
-    return true;
-  }
-
-  if (get_results() == nullptr) {
-    // Query reported sucess but no result set, this indicates failure
-    ndb_log_error("No result set for query '%s'", statement.c_str());
-    assert(false);
-  }
-
-  m_closed = false;
+  m_closed = execute_query_iso(sql_text, ignore_mysql_errors, nullptr);
   return m_closed;
 }
 
-/* Run SHOW CREATE USER, and place the result SQL in result.
-   Return true on success.
-*/
-bool ThreadContext::show_create_user(std::string user, std::string &result,
-                                     bool use_hex) {
+void ThreadContext::get_create_user(std::string user, int ngrants) {
   std::string statement("SHOW CREATE USER " + user);
-  int exec_return_val;
-
-  {
-    bool saved_val = m_thd->variables.print_identified_with_as_hex;
-    m_thd->variables.print_identified_with_as_hex = use_hex;
-    exec_return_val = exec_sql(statement);
-    m_thd->variables.print_identified_with_as_hex = saved_val;
-  }
-
-  if (exec_return_val) {
+  if (exec_sql(statement)) {
     ndb_log_error("Failed SHOW CREATE USER for %s", user.c_str());
-    return false;
+    return;
   }
 
   List<Ed_row> results = *get_results();
@@ -335,25 +303,16 @@ bool ThreadContext::show_create_user(std::string user, std::string &result,
   if (results.elements != 1) {
     ndb_log_error("%s returned %d rows", statement.c_str(), results.elements);
     close();
-    return false;
+    return;
   }
 
   Ed_row *result_row = results[0];
   const MYSQL_LEX_STRING *result_sql = result_row->get_column(0);
-  result.assign(result_sql->str, result_sql->length);
+  unsigned int note = ngrants;
+  char *row = Row(TYPE_USER, user, 0, &note,
+                  std::string(result_sql->str, result_sql->length));
+  m_current_rows.push_back(row);
   close();
-  return true;
-}
-
-/* Run SHOW CREATE USER, create a row, and push it to m_current_rows
- */
-void ThreadContext::get_create_user(std::string user, int ngrants) {
-  std::string show_create;
-  if (show_create_user(user, show_create)) {
-    unsigned int note = ngrants;
-    char *row = Row(TYPE_USER, user, 0, &note, show_create);
-    m_current_rows.push_back(row);
-  }
   return;
 }
 
@@ -498,7 +457,7 @@ inline bool blacklisted(std::string user) {
    This query selects only those users that have been directly granted
    NDB_STORED_USER, not those granted it transitively via a role. This
    entails that the direct grant is required -- a limitation which must
-   be documented. If there were a table in information_schema analogous to
+   be documented. If there were a table in information_schema analagous to
    mysql.role_edges, we could solve this problem by issuing a JOIN query
    of user_privileges and role_edges. For now, though, living with the
    documented limitation is preferable to relying on the mysql table.
@@ -561,7 +520,7 @@ const NdbError *ThreadContext::write_snapshot(NdbTransaction *tx) {
 
     if (tx->execute(NoCommit)) return &tx->getNdbError();
 
-    assert(m_read_keys.size() == m_grant_count.size());
+    DBUG_ASSERT(m_read_keys.size() == m_grant_count.size());
     for (size_t i = 0; i < m_read_keys.size(); i++) {
       unsigned int n_stored_grants;
       const NdbError &op_error = read_ops[i]->getNdbError();
@@ -648,70 +607,35 @@ void ThreadContext::drop_user(std::string user, bool is_revoke) {
 int ThreadContext::drop_users(ChangeNotice *notice,
                               const Mem_root_array<std::string> &list) {
   for (std::string user : list) {
-    assert(local_granted_users.count(user));
+    DBUG_ASSERT(local_granted_users.count(user));
     drop_user(user, (notice->get_operation() != SQLCOM_DROP_USER));
   }
   return list.size();
 }
 
 /* Stored in the snapshot is a CREATE USER statement. This statement has come
-   from SHOW CREATE USER, so its exact format is known.
-
-   If the user already exists locally and the local SHOW CREATE USER exactly
-   matches the snapshot, then return without doing anything, so that the
-   last_mod timestamp on the user's password does not get unnecessarily reset.
-
-   Otherwise, try to apply the statement as-is. This can fail for several
-   reasons (some of which are tested in the apply_stored_grants test case),
-   but if might succeed.
-
-   If running the CREATE USER statement failed, it must be rewritten as
-   several statements. The final result is:
-      CREATE USER IF NOT EXISTS user@host  IDENTIFIED BY RANDOM PASSWORD;
+   from SHOW CREATE USER, so its exact format is known. For idempotence, it
+   must be rewritten as several statements. The final result is:
+      CREATE USER IF NOT EXISTS user@host;
       REVOKE ALL ON *.* FROM user@host;
       ALTER USER user@host ...;   [CLEAR RESOURCE LIMITS]
       ALTER USER user@host ...;   [SET VALUES FROM SHOW CREATE USER]
-      ALTER USER user@host DEFAULT ROLE ... ;
-
-  The DEFAULT ROLE statement is pushed to the end of a list and run later,
-  after some other statement in the snapshot possibly creates the named role.
 */
 void ThreadContext::create_user(std::string &name, std::string &statement) {
-  const std::string create_user_if_ne("CREATE USER IF NOT EXISTS ");
-  const std::string random_pass(" IDENTIFIED BY RANDOM PASSWORD");
+  const std::string create_user("CREATE USER IF NOT EXISTS ");
   const std::string alter_user("ALTER USER ");
   const std::string revoke_all("REVOKE ALL ON *.* FROM ");
   const std::string set_resource_defaults(
       " WITH MAX_QUERIES_PER_HOUR 0 MAX_UPDATES_PER_HOUR 0 "
       " MAX_CONNECTIONS_PER_HOUR 0 MAX_USER_CONNECTIONS 0");
 
-  const bool exists_in_cache = get_local_user(name);
-
-  if (exists_in_cache) {
-    /* Compare the snapshot user to the local one. Before comparing, we must
-       detect whether the password hash stored in the snapshot used hex encoding
-       or a plain string.  In statement.find(), 36 characters skips past
-       "CREATE USER `a`@`%` IDENTIFIED WITH".
-    */
-    std::string show_create;
-    bool is_hex = (statement.find(" AS 0x", 36) != std::string::npos);
-    if (show_create_user(name, show_create, is_hex) && show_create == statement)
-      return;  // Current SHOW CREATE USER already matches snapshot
-  } else {
-    local_granted_users.insert(name);
+  /* Run statement CREATE USER IF NOT EXISTS */
+  if (!get_local_user(name)) {
+    ndb_log_info("From stored snapshot, adding NDB stored user: %s",
+                 name.c_str());
+    run_acl_statement(create_user + name);
   }
 
-  ndb_log_info("From stored snapshot, adding NDB stored user: %s",
-               name.c_str());
-
-  /* Try to run the CREATE USER statement stored in the snapshot.
-     If this succeeds, we are done; but there are several reasons it might fail.
-  */
-  if (try_create_user(statement) == false) return;
-
-  /* Create the user with a random password (unless the user already exists) */
-  run_acl_statement(create_user_if_ne + name + random_pass);
-
   /* Revoke any privileges the user may have had prior to this snapshot. */
   run_acl_statement(revoke_all + name);
 
@@ -729,8 +653,8 @@ void ThreadContext::create_user(std::string &name, std::string &statement) {
   }
 
   /* Locate the part between DEFAULT ROLE and REQUIRE */
-  size_t require_pos = statement.find(" REQUIRE ", default_role_pos + 14);
-  assert(require_pos != std::string::npos);
+  size_t require_pos = statement.find("REQUIRE ", default_role_pos + 14);
+  DBUG_ASSERT(require_pos != std::string::npos);
   size_t role_clause_len = require_pos - default_role_pos;
 
   /*  Set default role. The role has not yet been granted, so this statement
@@ -748,9 +672,9 @@ void ThreadContext::create_user(std::string &name, std::string &statement) {
  */
 void ThreadContext::apply_current_snapshot() {
   for (const char *row : m_current_rows) {
-    unsigned int note = 0;
-    size_t str_length = 0;
-    const char *str_start = nullptr;
+    unsigned int note;
+    size_t str_length;
+    const char *str_start;
     bool is_null;
 
     int type = Buffer::getType(row);
@@ -782,7 +706,7 @@ void ThreadContext::apply_current_snapshot() {
         break;
       default:
         /* These records should have come from a bounded index scan */
-        assert(false);
+        DBUG_ASSERT(false);
         break;
     }  // switch()
   }    // for()
@@ -825,10 +749,10 @@ void ThreadContext::write_status_message_to_server_log() {
    Return the number of elements in m_statement_users.
 */
 int ThreadContext::get_user_lists_for_statement(ChangeNotice *notice) {
-  assert(m_statement_users.size() == 0);
-  assert(m_intersection.size() == 0);
+  DBUG_ASSERT(m_statement_users.size() == 0);
+  DBUG_ASSERT(m_intersection.size() == 0);
 
-  for (const ChangeNotice::User notice_user : notice->get_user_list()) {
+  for (const ChangeNotice::User &notice_user : notice->get_user_list()) {
     std::string user;
     format_user(user, notice_user);
     m_statement_users.push_back(user);
@@ -963,34 +887,10 @@ Ndb_stored_grants::Strategy ThreadContext::handle_change(ChangeNotice *notice) {
 // Public interface
 //
 
-bool Ndb_stored_grants::init() {
-  const Ndb_mysql_services services;
-
-  // Register the NDB_STORED_USER dynamic privilege
-  // NOTE! This privilege is never unregistered
-  my_service<SERVICE_TYPE(dynamic_privilege_register)> service(
-      "dynamic_privilege_register.mysql_server", services);
-  if (!service.is_valid() ||
-      service->register_privilege(STRING_WITH_LEN("NDB_STORED_USER"))) {
-    return false;
-  }
-  return true;
-}
-
-/* setup() is run as part of binlog setup.
+/* initialize() is run as part of binlog setup.
  */
-bool Ndb_stored_grants::setup(THD *thd, Thd_ndb *thd_ndb) {
-  ThreadContext context(thd);
-
-  if (metadata_table.isRestarting()) {
-    ndb_log_info("Ndb_stored_grants::setup() -- after deferred shutdown");
-    metadata_table.clear(thd_ndb->ndb->getDictionary());
-  } else if (metadata_table.isInitialized()) {
-    ndb_log_info("Ndb_stored_grants::setup() -- no op");
-    return true;
-  } else {
-    ndb_log_info("Ndb_stored_grants::setup() -- normal setup");
-  }
+bool Ndb_stored_grants::initialize(THD *thd, Thd_ndb *thd_ndb) {
+  if (metadata_table.isInitialized()) return true;
 
   /* Create or upgrade the ndb_sql_metadata table.
      If this fails, create_or_upgrade() will log an error message,
@@ -1002,40 +902,21 @@ bool Ndb_stored_grants::setup(THD *thd, Thd_ndb *thd_ndb) {
 
   metadata_table.setup(thd_ndb->ndb->getDictionary(),
                        sql_metadata_table.get_table());
-  const NdbError &err = metadata_table.initializeSnapshotLock(thd_ndb->ndb);
-  if (err.status != NdbError::Success) {
-    ndb_log_error("ndb_stored_grants initalizeSnapshotLock failure: %d %s",
-                  err.code, err.message);
-    return false;
-  }
-
   return true;
 }
 
-void Ndb_stored_grants::shutdown(THD *thd, Thd_ndb *thd_ndb, bool restarting) {
-  if (!(thd && thd_ndb)) {
-    ndb_log_info("Ndb_stored_grants::shutdown() -- no op");
-    return;
-  }
-
-  ThreadContext context(thd);
-  if (restarting) {
-    ndb_log_info("Ndb_stored_grants::shutdown() -- deferred");
-    metadata_table.setRestarting();
-  } else {
-    ndb_log_info("Ndb_stored_grants::shutdown() -- normal shutdown");
-    metadata_table.clear(thd_ndb->ndb->getDictionary());
-  }
+void Ndb_stored_grants::shutdown(Thd_ndb *thd_ndb) {
+  metadata_table.clear(thd_ndb->ndb->getDictionary());
 }
 
 bool Ndb_stored_grants::apply_stored_grants(THD *thd) {
-  ThreadContext context(thd);
-
   if (!metadata_table.isInitialized()) {
-    ndb_log_error("stored grants: not initialized.");
+    ndb_log_error("stored grants: initialization has failed.");
     return false;
   }
 
+  ThreadContext context(thd);
+
   if (!context.read_snapshot()) return false;
 
   (void)context.build_cache_of_ndb_users();
@@ -1050,15 +931,8 @@ bool Ndb_stored_grants::apply_stored_grants(THD *thd) {
 Ndb_stored_grants::Strategy Ndb_stored_grants::handle_local_acl_change(
     THD *thd, const Acl_change_notification *notice, std::string *user_list,
     bool *schema_dist_use_db, bool *must_refresh) {
-  ThreadContext context(thd);
-
-  if (notice == nullptr) {
-    ndb_log_error("stored grants: no Acl_change_notification");
-    return Strategy::ERROR;
-  }
-
   if (!metadata_table.isInitialized()) {
-    ndb_log_error("stored grants: not intialized.");
+    ndb_log_error("stored grants: initialization has failed.");
     return Strategy::ERROR;
   }
 
@@ -1068,6 +942,7 @@ Ndb_stored_grants::Strategy Ndb_stored_grants::handle_local_acl_change(
   const enum_sql_command operation = notice->get_operation();
   if (operation == SQLCOM_CREATE_USER) return Strategy::NONE;
 
+  ThreadContext context(thd);
   Strategy strategy = context.handle_change(notice);
 
   /* Set flags for caller.
@@ -1090,13 +965,13 @@ void Ndb_stored_grants::maintain_cache(THD *thd) {
 
 bool Ndb_stored_grants::update_users_from_snapshot(THD *thd,
                                                    std::string users) {
-  ThreadContext context(thd);
-
   if (!metadata_table.isInitialized()) {
-    ndb_log_error("stored grants: not intialized.");
+    ndb_log_error("stored grants: initialization has failed.");
     return false;
   }
 
+  ThreadContext context(thd);
+
   context.deserialize_users(users);
   if (!context.read_snapshot()) return false;
 
@@ -1105,17 +980,3 @@ bool Ndb_stored_grants::update_users_from_snapshot(THD *thd,
   context.handle_dropped_users();
   return true;  // success
 }
-
-NdbTransaction *Ndb_stored_grants::acquire_snapshot_lock(THD *thd) {
-  NdbTransaction *transaction;
-  Ndb *ndb = get_thd_ndb(thd)->ndb;
-  const NdbError &err = metadata_table.acquireSnapshotLock(ndb, transaction);
-  if (err.status != NdbError::Success)
-    ndb_log_error("acquire_snapshot_lock: Error %d '%s'", err.code,
-                  err.message);
-  return transaction;
-}
-
-void Ndb_stored_grants::release_snapshot_lock(NdbTransaction *transaction) {
-  metadata_table.releaseSnapshotLock(transaction);
-}
