@@ -1,4 +1,4 @@
-/* Copyright (c) 2007, 2023, Oracle and/or its affiliates.
+/* Copyright (C) 2007 MySQL AB
    Use is subject to license terms
 
    This program is free software; you can redistribute it and/or modify
@@ -21,17 +21,13 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
-#include <signaldata/DictTabInfo.hpp>
 #include <signaldata/GetTabInfo.hpp>
+#include <signaldata/DictTabInfo.hpp>
 
-bool printGET_TABINFO_REQ(FILE *output, const Uint32 *theData, Uint32 len,
-                          Uint16 /*rbn*/) {
-  if (len < GetTabInfoReq::SignalLength) {
-    assert(false);
-    return false;
-  }
-
-  const GetTabInfoReq *sig = (const GetTabInfoReq *)theData;
+bool
+printGET_TABINFO_REQ(FILE* output, const Uint32* theData, Uint32 len, Uint16 rbn)
+{
+  const GetTabInfoReq* sig = (const GetTabInfoReq*)theData;
   fprintf(output, " senderRef: 0x%x", sig->senderRef);
   fprintf(output, " senderData: %u", sig->senderData);
   fprintf(output, "\n");
@@ -39,25 +35,26 @@ bool printGET_TABINFO_REQ(FILE *output, const Uint32 *theData, Uint32 len,
   bool requestById = !(sig->requestType & GetTabInfoReq::RequestByName);
   bool requestByName = (sig->requestType & GetTabInfoReq::RequestByName);
   bool longSignalConf = (sig->requestType & GetTabInfoReq::LongSignalConf);
-  if (requestById) fprintf(output, " RequestById");
-  if (requestByName) fprintf(output, " RequestByName");
-  if (longSignalConf) fprintf(output, " LongSignalConf");
+  if (requestById)
+    fprintf(output, " RequestById");
+  if (requestByName)
+    fprintf(output, " RequestByName");
+  if (longSignalConf)
+    fprintf(output, " LongSignalConf");
   fprintf(output, "\n");
-  if (requestById) fprintf(output, " tableId: %u", sig->tableId);
-  if (requestByName) fprintf(output, " tableNameLen: %u", sig->tableNameLen);
+  if (requestById)
+    fprintf(output, " tableId: %u", sig->tableId);
+  if (requestByName)
+    fprintf(output, " tableNameLen: %u", sig->tableNameLen);
   fprintf(output, " schemaTransId: 0x%x", sig->schemaTransId);
   fprintf(output, "\n");
   return true;
 }
 
-bool printGET_TABINFO_CONF(FILE *output, const Uint32 *theData, Uint32 len,
-                           Uint16 /*rbn*/) {
-  if (len < GetTabInfoConf::SignalLength) {
-    assert(false);
-    return false;
-  }
-
-  const GetTabInfoConf *sig = (const GetTabInfoConf *)theData;
+bool
+printGET_TABINFO_CONF(FILE* output, const Uint32* theData, Uint32 len, Uint16 rbn)
+{
+  const GetTabInfoConf* sig = (const GetTabInfoConf*)theData;
   fprintf(output, " senderRef: 0x%x", sig->senderRef);
   fprintf(output, " senderData: %u", sig->senderData);
   fprintf(output, "\n");
@@ -65,34 +62,30 @@ bool printGET_TABINFO_CONF(FILE *output, const Uint32 *theData, Uint32 len,
   fprintf(output, " tableType: %u", sig->tableType);
   fprintf(output, "\n");
   switch (sig->tableType) {
-    case DictTabInfo::Tablespace:
-      fprintf(output, " freeExtents: %u", sig->freeExtents);
-      break;
-    case DictTabInfo::LogfileGroup:
-      fprintf(output, " freeWordsHi: %u", sig->freeWordsHi);
-      fprintf(output, " freeWordsLo: %u", sig->freeWordsLo);
-      break;
-    case DictTabInfo::Datafile:
-    case DictTabInfo::Undofile:
-      fprintf(output, " freeExtents: %u", sig->freeExtents);
-      break;
-    default:
-      fprintf(output, " gci: %u", sig->gci);
-      fprintf(output, " totalLen: %u", sig->totalLen);
-      break;
+  case DictTabInfo::Tablespace:
+    fprintf(output, " freeExtents: %u", sig->freeExtents);
+    break;
+  case DictTabInfo::LogfileGroup:
+    fprintf(output, " freeWordsHi: %u", sig->freeWordsHi);
+    fprintf(output, " freeWordsLo: %u", sig->freeWordsLo);
+    break;
+  case DictTabInfo::Datafile:
+  case DictTabInfo::Undofile:
+    fprintf(output, " freeExtents: %u", sig->freeExtents);
+    break;
+  default:
+    fprintf(output, " gci: %u", sig->gci);
+    fprintf(output, " totalLen: %u", sig->totalLen);
+    break;
   }
   fprintf(output, "\n");
   return true;
 }
 
-bool printGET_TABINFO_REF(FILE *output, const Uint32 *theData, Uint32 len,
-                          Uint16 /*rbn*/) {
-  if (len < GetTabInfoRef::SignalLength) {
-    assert(false);
-    return false;
-  }
-
-  const GetTabInfoRef *sig = (const GetTabInfoRef *)theData;
+bool
+printGET_TABINFO_REF(FILE* output, const Uint32* theData, Uint32 len, Uint16 rbn)
+{
+  const GetTabInfoRef* sig = (const GetTabInfoRef*)theData;
   fprintf(output, " senderRef: 0x%x", sig->senderRef);
   fprintf(output, " senderData: %u", sig->senderData);
   fprintf(output, "\n");
@@ -100,12 +93,17 @@ bool printGET_TABINFO_REF(FILE *output, const Uint32 *theData, Uint32 len,
   bool requestById = !(sig->requestType & GetTabInfoReq::RequestByName);
   bool requestByName = (sig->requestType & GetTabInfoReq::RequestByName);
   bool longSignalConf = (sig->requestType & GetTabInfoReq::LongSignalConf);
-  if (requestById) fprintf(output, " RequestById");
-  if (requestByName) fprintf(output, " RequestByName");
-  if (longSignalConf) fprintf(output, " LongSignalConf");
+  if (requestById)
+    fprintf(output, " RequestById");
+  if (requestByName)
+    fprintf(output, " RequestByName");
+  if (longSignalConf)
+    fprintf(output, " LongSignalConf");
   fprintf(output, "\n");
-  if (requestById) fprintf(output, " tableId: %u", sig->tableId);
-  if (requestByName) fprintf(output, " tableNameLen: %u", sig->tableNameLen);
+  if (requestById)
+    fprintf(output, " tableId: %u", sig->tableId);
+  if (requestByName)
+    fprintf(output, " tableNameLen: %u", sig->tableNameLen);
   fprintf(output, " schemaTransId: 0x%x", sig->schemaTransId);
   fprintf(output, " errorCode: %u", sig->errorCode);
   fprintf(output, " errorLine: %u", sig->errorLine);
