@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2004, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2004, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -22,42 +22,51 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
+
+
 #include <BlockNumbers.h>
-#include <signaldata/ScanFrag.hpp>
 #include <signaldata/ScanTab.hpp>
+#include <signaldata/ScanFrag.hpp>
 
-bool printSCAN_FRAGREQ(FILE *output, const Uint32 *theData, Uint32 len,
-                       Uint16 /*receiverBlockNo*/) {
-  if (len < ScanFragReq::SignalLength) {
-    assert(false);
-    return false;
-  }
-
-  const ScanFragReq *const sig = (const ScanFragReq *)theData;
+bool
+printSCAN_FRAGREQ(FILE * output, const Uint32 * theData, 
+		  Uint32 len, Uint16 receiverBlockNo) {
+  const ScanFragReq * const sig = (ScanFragReq *)theData;
   fprintf(output, " senderData: 0x%x\n", sig->senderData);
   fprintf(output, " resultRef: 0x%x\n", sig->resultRef);
   fprintf(output, " savePointId: %u\n", sig->savePointId);
 
   fprintf(output, " flags: ");
-  if (ScanFragReq::getLockMode(sig->requestInfo)) fprintf(output, "X");
-  if (ScanFragReq::getPrioAFlag(sig->requestInfo)) fprintf(output, "a");
-  if (ScanFragReq::getHoldLockFlag(sig->requestInfo)) fprintf(output, "h");
-  if (ScanFragReq::getKeyinfoFlag(sig->requestInfo)) fprintf(output, "k");
-  if (ScanFragReq::getReadCommittedFlag(sig->requestInfo)) fprintf(output, "d");
-  if (ScanFragReq::getRangeScanFlag(sig->requestInfo)) fprintf(output, "r");
+  if (ScanFragReq::getLockMode(sig->requestInfo))
+    fprintf(output, "X");
+  if (ScanFragReq::getPrioAFlag(sig->requestInfo))
+    fprintf(output, "a");
+  if (ScanFragReq::getHoldLockFlag(sig->requestInfo))
+    fprintf(output, "h");
+  if (ScanFragReq::getKeyinfoFlag(sig->requestInfo))
+    fprintf(output, "k");
+  if (ScanFragReq::getReadCommittedFlag(sig->requestInfo))
+    fprintf(output, "d");
+  if (ScanFragReq::getRangeScanFlag(sig->requestInfo))
+    fprintf(output, "r");
   if (ScanFragReq::getDescendingFlag(sig->requestInfo))
     fprintf(output, "(desc)");
-  if (ScanFragReq::getTupScanFlag(sig->requestInfo)) fprintf(output, "t");
-  if (ScanFragReq::getFirstMatchFlag(sig->requestInfo)) fprintf(output, "f");
-  if (ScanFragReq::getQueryThreadFlag(sig->requestInfo)) fprintf(output, "q");
-  if (ScanFragReq::getNoDiskFlag(sig->requestInfo)) fprintf(output, "(nodisk)");
-  fprintf(output, " attrLen: %u", ScanFragReq::getAttrLen(sig->requestInfo));
-  fprintf(output, " reorg: %u", ScanFragReq::getReorgFlag(sig->requestInfo));
+  if (ScanFragReq::getTupScanFlag(sig->requestInfo))
+    fprintf(output, "t");
+  if (ScanFragReq::getFirstMatchFlag(sig->requestInfo))
+    fprintf(output, "f");
+  if (ScanFragReq::getNoDiskFlag(sig->requestInfo))
+    fprintf(output, "(nodisk)");
+  fprintf(output, " attrLen: %u",
+          ScanFragReq::getAttrLen(sig->requestInfo));
+  fprintf(output, " reorg: %u",
+          ScanFragReq::getReorgFlag(sig->requestInfo));
   fprintf(output, " corr: %u",
           ScanFragReq::getCorrFactorFlag(sig->requestInfo));
   fprintf(output, " mfrag: %u",
           ScanFragReq::getMultiFragFlag(sig->requestInfo));
-  fprintf(output, " stat: %u", ScanFragReq::getStatScanFlag(sig->requestInfo));
+  fprintf(output, " stat: %u",
+          ScanFragReq::getStatScanFlag(sig->requestInfo));
   fprintf(output, " ni: %u",
           ScanFragReq::getNotInterpretedFlag(sig->requestInfo));
   fprintf(output, "\n");
@@ -72,7 +81,8 @@ bool printSCAN_FRAGREQ(FILE *output, const Uint32 *theData, Uint32 len,
   fprintf(output, " batch_size_rows: %u\n", sig->batch_size_rows);
   fprintf(output, " batch_size_bytes: %u\n", sig->batch_size_bytes);
 
-  if (ScanFragReq::getCorrFactorFlag(sig->requestInfo)) {
+  if (ScanFragReq::getCorrFactorFlag(sig->requestInfo))
+  {
     fprintf(output, " corrFactorLo: 0x%x\n", sig->variableData[0]);
     fprintf(output, " corrFactorHi: 0x%x\n", sig->variableData[1]);
   }
@@ -80,10 +90,12 @@ bool printSCAN_FRAGREQ(FILE *output, const Uint32 *theData, Uint32 len,
   return true;
 }
 
-bool printSCAN_FRAGCONF(FILE *output, const Uint32 *theData, Uint32 len,
-                        Uint16 /*receiverBlockNo*/) {
-  const ScanFragConf *const sig =
-      reinterpret_cast<const ScanFragConf *>(theData);
+bool
+printSCAN_FRAGCONF(FILE * output, const Uint32 * theData,
+                   Uint32 len, Uint16 receiverBlockNo)
+{
+  const ScanFragConf * const sig =
+    reinterpret_cast<const ScanFragConf*>(theData);
   fprintf(output, " senderData: 0x%x\n", sig->senderData);
   fprintf(output, " completedOps: %u\n", sig->completedOps);
   fprintf(output, " fragmentCompleted: 0x%x\n", sig->fragmentCompleted);
@@ -93,8 +105,7 @@ bool printSCAN_FRAGCONF(FILE *output, const Uint32 *theData, Uint32 len,
   if (len >= ScanFragConf::SignalLength_ext)
     fprintf(output, " activeMask: 0x%x\n", sig->activeMask);
   else
-    fprintf(output, " activeMask: 0(not an ext-signal)\n");
-  if (len >= ScanFragConf::SignalLength_query)
-    fprintf(output, " senderRef = %x\n", sig->senderRef);
+    fprintf(output, " activeMask: 0(not an ext-signal)");
+
   return true;
 }
