@@ -1,5 +1,5 @@
 /* 
-   Copyright (c) 2003, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -22,11 +22,7 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
-#include "util/require.h"
 #include <ndb_global.h>
-
-#include <cstring>
-
 #include <IPCConfig.hpp>
 
 #include <TransporterRegistry.hpp>
@@ -35,16 +31,17 @@
 #include <mgmapi_configuration.hpp>
 #include <NdbSpin.h>
 
-#include <EventLogger.hpp>
 
 /* Return true if node with "nodeId" is a MGM node */
-static bool is_mgmd(Uint32 nodeId, const ndb_mgm_configuration * config)
+static bool is_mgmd(Uint32 nodeId,
+                    const struct ndb_mgm_configuration & config)
 {
   ndb_mgm_configuration_iterator iter(config, CFG_SECTION_NODE);
-  require(iter.find(CFG_NODE_ID, nodeId) == 0);
-
+  if (iter.find(CFG_NODE_ID, nodeId))
+    abort();
   Uint32 type;
-  require(iter.get(CFG_TYPE_OF_SECTION, &type) == 0);
+  if(iter.get(CFG_TYPE_OF_SECTION, &type))
+    abort();
 
   return (type == NODE_TYPE_MGM);
 }
@@ -52,7 +49,7 @@ static bool is_mgmd(Uint32 nodeId, const ndb_mgm_configuration * config)
 
 bool
 IPCConfig::configureTransporters(Uint32 nodeId,
-                                 const ndb_mgm_configuration* config,
+                                 const struct ndb_mgm_configuration & config,
                                  class TransporterRegistry & tr,
                                  bool transporter_to_self)
 {
@@ -82,7 +79,7 @@ IPCConfig::configureTransporters(Uint32 nodeId,
       if(iter.get(CFG_NODE_HOST, &hostname)) continue;
       if( strlen(hostname) == 0 ) continue;
       if(iter.get(CFG_MGM_PORT, &port)) continue;
-      connect_string.appfmt("%s%s %u",separator,hostname,port);
+      connect_string.appfmt("%s%s:%u",separator,hostname,port);
       separator= ",";
     }
     NdbMgmHandle h= ndb_mgm_create_handle();
@@ -102,8 +99,8 @@ IPCConfig::configureTransporters(Uint32 nodeId,
     {
       // Transporter exist in TransporterRegistry but not
       // in configuration
-      g_eventLogger->info(
-          "The connection to node %d could not be removed at this time", i);
+      ndbout_c("The connection to node %d could not "
+               "be removed at this time", i);
       result= false; // Need restart
     }
   }
@@ -113,9 +110,9 @@ IPCConfig::configureTransporters(Uint32 nodeId,
   ndb_mgm_configuration_iterator iter(config, CFG_SECTION_CONNECTION);
   for(iter.first(); iter.valid(); iter.next()){
     
-    std::memset(&conf, 0, sizeof(conf));
+    bzero(&conf, sizeof(conf));
     Uint32 nodeId1, nodeId2, remoteNodeId;
-    const char * remoteHostName= nullptr, * localHostName= nullptr;
+    const char * remoteHostName= 0, * localHostName= 0;
     if(iter.get(CFG_CONNECTION_NODE_1, &nodeId1)) continue;
     if(iter.get(CFG_CONNECTION_NODE_2, &nodeId2)) continue;
 
@@ -128,7 +125,7 @@ IPCConfig::configureTransporters(Uint32 nodeId,
     }
 
     {
-      const char * host1= nullptr, * host2= nullptr;
+      const char * host1= 0, * host2= 0;
       iter.get(CFG_CONNECTION_HOSTNAME_1, &host1);
       iter.get(CFG_CONNECTION_HOSTNAME_2, &host2);
       localHostName  = (nodeId == nodeId1 ? host1 : host2);
@@ -210,12 +207,17 @@ IPCConfig::configureTransporters(Uint32 nodeId,
         DBUG_PRINT("error", ("Failed to configure SHM Transporter "
                              "from %d to %d",
 	           conf.localNodeId, conf.remoteNodeId));
-        g_eventLogger->info("Failed to configure SHM Transporter to node %d",
-                            conf.remoteNodeId);
+	ndbout_c("Failed to configure SHM Transporter to node %d",
+                conf.remoteNodeId);
         result = false;
       }
+#ifdef NDB_WIN32
+      ndbout_c("Shared memory transporters not supported on Windows");
+      result = false;
+#else
       DBUG_PRINT("info", ("Configured SHM Transporter using shmkey %d, "
 			  "buf size = %d", conf.shm.shmKey, conf.shm.shmSize));
+#endif
       break;
 
     case CONNECTION_TYPE_TCP:
@@ -240,8 +242,8 @@ IPCConfig::configureTransporters(Uint32 nodeId,
       conf.type = tt_TCP_TRANSPORTER;
       
       if(!tr.configureTransporter(&conf)){
-        g_eventLogger->info("Failed to configure TCP Transporter to node %d",
-                            conf.remoteNodeId);
+	ndbout_c("Failed to configure TCP Transporter to node %d",
+                 conf.remoteNodeId);
         result= false;
       }
       DBUG_PRINT("info", ("Configured TCP Transporter: sendBufferSize = %d, "
@@ -250,8 +252,8 @@ IPCConfig::configureTransporters(Uint32 nodeId,
       loopback_conf = conf; // reuse it...
       break;
     default:
-      g_eventLogger->info("Unknown transporter type from: %u to: %u", nodeId,
-                          remoteNodeId);
+      ndbout << "Unknown transporter type from: " << nodeId << 
+	" to: " << remoteNodeId << endl;
       break;
     } // switch
   } // for
@@ -275,7 +277,7 @@ IPCConfig::configureTransporters(Uint32 nodeId,
     loopback_conf.tcp.tcpOverloadLimit = 768*1024;
     if (!tr.configureTransporter(&loopback_conf))
     {
-      g_eventLogger->info("Failed to configure Loopback Transporter");
+      ndbout_c("Failed to configure Loopback Transporter");
       result= false;
     }
   }
