@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -25,62 +25,80 @@
 #ifndef SHM_Transporter_H
 #define SHM_Transporter_H
 
-#include "SHM_Buffer.hpp"
 #include "Transporter.hpp"
+#include "SHM_Buffer.hpp"
 
 #ifdef _WIN32
 typedef Uint32 key_t;
 #endif
 
-/**
+/** 
  * class SHMTransporter
  * @brief - main class for the SHM transporter.
  */
 
 class SHM_Transporter : public Transporter {
   friend class TransporterRegistry;
+public:
+  SHM_Transporter(TransporterRegistry &,
+                  TrpId transporterIndex,
+		  const char *lHostName,
+		  const char *rHostName, 
+		  int r_port,
+		  bool isMgmConnection,
+		  NodeId lNodeId,
+		  NodeId rNodeId,
+		  NodeId serverNodeId,
+		  bool checksum, 
+		  bool signalId,
+		  key_t shmKey,
+		  Uint32 shmSize,
+		  bool preSendChecksum,
+                  Uint32 spintime,
+                  Uint32 send_buffer_size);
 
- public:
-  SHM_Transporter(TransporterRegistry &, TrpId transporterIndex,
-                  const char *lHostName, const char *rHostName, int r_port,
-                  bool isMgmConnection, NodeId lNodeId, NodeId rNodeId,
-                  NodeId serverNodeId, bool checksum, bool signalId,
-                  key_t shmKey, Uint32 shmSize, bool preSendChecksum,
-                  Uint32 spintime, Uint32 send_buffer_size);
-
-  SHM_Transporter(TransporterRegistry &, const SHM_Transporter *);
-
+  SHM_Transporter(TransporterRegistry &,
+                  const SHM_Transporter*);
+ 
   /**
    * SHM destructor
    */
-  ~SHM_Transporter() override;
+  virtual ~SHM_Transporter();
 
   /**
    * Clear any data buffered in the transporter.
    * Should only be called in a disconnected state.
    */
-  void resetBuffers() override;
+  virtual void resetBuffers();
 
-  bool configure_derived(const TransporterConfiguration *conf) override;
+  virtual bool configure_derived(const TransporterConfiguration* conf);
 
   /**
    * Do initialization
    */
-  bool initTransporter() override;
-
-  void getReceivePtr(Uint32 **ptr, Uint32 **eod, Uint32 **end) {
-    reader->getReadPtr(*ptr, *eod, *end);
+  bool initTransporter();
+  
+  void getReceivePtr(Uint32 ** ptr,
+                     Uint32 ** eod,
+                     Uint32 ** end)
+  {
+    reader->getReadPtr(* ptr, * eod, * end);
   }
-
-  void updateReceivePtr(TransporterReceiveHandle &, Uint32 *ptr);
-
- protected:
+  
+  void updateReceivePtr(TransporterReceiveHandle&, Uint32 * ptr);
+  
+protected:
   /**
-   * Release resources used by SHM after disconnect
+   * disconnect a segmnet
    * -# deletes the shm buffer associated with a segment
    * -# marks the segment for removal
    */
-  void releaseAfterDisconnect() override;
+  void disconnectImpl();
+
+  /**
+   * Disconnect socket that was used for wakeup services.
+   */
+  void disconnect_socket();
 
   /**
    * Blocking
@@ -89,11 +107,12 @@ class SHM_Transporter : public Transporter {
    * -# Attach to it
    * -# Wait for someone to attach (max wait = timeout), then rerun again
    *    until connection established.
+   * @param timeOutMillis - the time to sleep before (ms) trying again.
    * @returns - True if the server managed to hook up with the client,
    *            i.e., both agrees that the other one has setup the segment.
    *            Otherwise false.
    */
-  bool connect_server_impl(NdbSocket &&) override;
+  virtual bool connect_server_impl(NDB_SOCKET_TYPE sockfd);
 
   /**
    * Blocking
@@ -102,19 +121,20 @@ class SHM_Transporter : public Transporter {
    * -# Check if the segment is setup
    * -# Check if the server set it up
    * -# If all clear, return.
+   * @param timeOutMillis - the time to sleep before (ms) trying again.
    * @returns - True if the client managed to hook up with the server,
    *            i.e., both agrees that the other one has setup the segment.
    *            Otherwise false.
    */
-  bool connect_client_impl(NdbSocket &&) override;
+  virtual bool connect_client_impl(NDB_SOCKET_TYPE sockfd);
 
-  bool connect_common();
+  bool connect_common(NDB_SOCKET_TYPE sockfd);
 
   bool ndb_shm_create();
   bool ndb_shm_get();
   bool ndb_shm_attach();
   void ndb_shm_destroy();
-  void set_socket(NdbSocket &&);
+  void set_socket(NDB_SOCKET_TYPE);
 
   /**
    * Check if there are two processes attached to the segment (a connection)
@@ -122,15 +142,16 @@ class SHM_Transporter : public Transporter {
    */
   bool checkConnected();
 
+  
   /**
-   * Initialises the SHM_Reader and SHM_Writer on the segment
+   * Initialises the SHM_Reader and SHM_Writer on the segment 
    */
   bool setupBuffers();
 
   /**
    * doSend (i.e signal receiver)
    */
-  bool doSend(bool need_wakeup = true) override;
+  bool doSend(bool need_wakeup = true);
   void doReceive();
   void wakeup();
 
@@ -155,13 +176,13 @@ class SHM_Transporter : public Transporter {
   int m_remote_pid;
   Uint32 m_signal_threshold;
 
- private:
+private:
   bool _shmSegCreated;
   bool _attached;
-
+  
   key_t shmKey;
-  volatile Uint32 *serverStatusFlag;
-  volatile Uint32 *clientStatusFlag;
+  volatile Uint32 * serverStatusFlag;
+  volatile Uint32 * clientStatusFlag;
 
   bool m_server_locked;
   bool m_client_locked;
@@ -176,15 +197,15 @@ class SHM_Transporter : public Transporter {
   NdbMutex *clientMutex;
 
   bool setupBuffersDone;
-
+  
 #ifdef _WIN32
   HANDLE hFileMapping;
 #else
-  int shmId{0};
+  int shmId;
 #endif
-
+  
   int shmSize;
-  char *shmBuf;
+  char * shmBuf;
 
   SHM_Reader *reader;
   SHM_Writer *writer;
@@ -195,14 +216,17 @@ class SHM_Transporter : public Transporter {
   /**
    * @return - True if the reader has data to read on its segment.
    */
-  bool hasDataToRead() const { return reader->empty() == false; }
+  bool hasDataToRead() const {
+    return reader->empty() == false;
+  }
 
   void make_error_info(char info[], int sz);
 
-  bool send_limit_reached(int bufsize) override {
+  bool send_limit_reached(int bufsize)
+  {
     return ((Uint32)bufsize >= m_signal_threshold);
   }
-  bool send_is_possible(int timeout_millisec) const override;
+  bool send_is_possible(int timeout_millisec) const;
   void detach_shm(bool rep_error);
 };
 
