@@ -1,14 +1,21 @@
-/* Copyright (c) 2003, 2015, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2003, 2017, Oracle and/or its affiliates. All rights reserved.
 
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -173,7 +180,11 @@ NdbOut::println(const char * fmt, ...)
 }
 
 static
-void 
+void
+vndbout_c(const char * fmt, va_list ap) ATTRIBUTE_FORMAT(printf, 1, 0);
+
+static
+void
 vndbout_c(const char * fmt, va_list ap)
 {
   if (fmt == NULL)
@@ -208,7 +219,7 @@ ndbout_c(const char * fmt, ...){
   va_end(ap);
 }
 
-extern "C" int ndbout_printer(const char * fmt, ...)
+int ndbout_printer(const char * fmt, ...)
 {
   va_list ap;
 
@@ -270,3 +281,23 @@ NdbOut_Init()
   new (&ndberrs_fileoutputstream) FileOutputStream(stderr);
   new (&ndberr) NdbOut(ndberrs_fileoutputstream);
 }
+
+void
+NdbOut_ReInit(OutputStream* stdout_ostream,
+              OutputStream* stderr_ostream)
+{
+  /**
+   * Re-initialise ndbout and ndberr globals with different OutputStreams
+   * Not thread safe, should be done at process start
+   */
+
+  /**
+   * Following probably can be removed as destructors(same file) are empty,
+   * but are present to handle any future changes
+   */
+    ndbout.~NdbOut();
+    //ndberr.~NdbErr();
+
+   new (&ndbout) NdbOut(*stdout_ostream);
+   new (&ndberr) NdbOut(*stderr_ostream);
+}
