@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2012, 2021, Oracle and/or its affiliates.
+   Copyright (c) 2012, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -19,7 +19,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
 #ifndef DBSPJ_H
@@ -31,7 +31,7 @@
 #include <AttributeHeader.hpp>
 #include <IntrusiveList.hpp>
 #include <ArenaPool.hpp>
-#include <DataBuffer2.hpp>
+#include <DataBuffer.hpp>
 #include <Bitmask.hpp>
 #include <signaldata/DbspjErr.hpp>
 #include "../dbtup/tuppage.hpp"
@@ -50,6 +50,8 @@ public:
   Dbspj(Block_context& ctx, Uint32 instanceNumber = 0);
   virtual ~Dbspj();
 
+  struct Request;
+  struct TreeNode;
 private:
   BLOCK_DEFINES(Dbspj);
 
@@ -71,8 +73,6 @@ private:
 
   void execDIH_SCAN_TAB_REF(Signal*);
   void execDIH_SCAN_TAB_CONF(Signal*);
-  void execDIH_SCAN_GET_NODES_REF(Signal*);
-  void execDIH_SCAN_GET_NODES_CONF(Signal*);
 
   void execSIGNAL_DROPPED_REP(Signal*);
 
@@ -105,15 +105,13 @@ protected:
   //virtual bool getParam(const char* name, Uint32* count);
 
 public:
-  struct Request;
-  struct TreeNode;
   struct ScanFragHandle;
-  typedef DataBuffer2<14, LocalArenaPoolImpl> Correlation_list;
-  typedef LocalDataBuffer2<14, LocalArenaPoolImpl> Local_correlation_list;
-  typedef DataBuffer2<14, LocalArenaPoolImpl> Dependency_map;
-  typedef LocalDataBuffer2<14, LocalArenaPoolImpl> Local_dependency_map;
-  typedef DataBuffer2<14, LocalArenaPoolImpl> PatternStore;
-  typedef LocalDataBuffer2<14, LocalArenaPoolImpl> Local_pattern_store;
+  typedef DataBuffer<14, LocalArenaPool<DataBufferSegment<14> > > Correlation_list;
+  typedef LocalDataBuffer<14, LocalArenaPool<DataBufferSegment<14> > > Local_correlation_list;
+  typedef DataBuffer<14, LocalArenaPool<DataBufferSegment<14> > > Dependency_map;
+  typedef LocalDataBuffer<14, LocalArenaPool<DataBufferSegment<14> > > Local_dependency_map;
+  typedef DataBuffer<14, LocalArenaPool<DataBufferSegment<14> > > PatternStore;
+  typedef LocalDataBuffer<14, LocalArenaPool<DataBufferSegment<14> > > Local_pattern_store;
   typedef Bitmask<(NDB_SPJ_MAX_TREE_NODES+31)/32> TreeNodeBitMask;
 
   /* *********** TABLE RECORD ********************************************* */
@@ -125,11 +123,11 @@ public:
   struct TableRecord {
     TableRecord() 
     : m_currentSchemaVersion(0), m_flags(0)
-    {};
+    {}
 
     TableRecord(Uint32 schemaVersion)
     : m_currentSchemaVersion(schemaVersion), m_flags(TR_PREPARED)
-    {};
+    {}
 
     Uint32 m_currentSchemaVersion;
     Uint16 m_flags;
@@ -138,6 +136,8 @@ public:
       TR_ENABLED      = 1 << 0,
       TR_DROPPING     = 1 << 1,
       TR_PREPARED     = 1 << 2
+      ,TR_READ_BACKUP = (1 << 5)
+      ,TR_FULLY_REPLICATED = (1 << 6)
     };
     Uint8 get_enabled()     const { return (m_flags & TR_ENABLED)      != 0; }
     Uint8 get_dropping()    const { return (m_flags & TR_DROPPING)     != 0; }
@@ -201,9 +201,18 @@ public:
    */
   struct RowPtr
   {
-    Uint32 m_type;
-    Uint32 m_src_node_ptrI;
-    Uint32 m_src_correlation;
+    enum RowType
+    {
+      RT_SECTION = 1,
+      RT_LINEAR = 2,
+      RT_END = 0
+    };
+
+    RowType m_type;
+    Uint32  m_src_node_ptrI;
+    Uint32  m_src_correlation;
+
+    TreeNodeBitMask *m_matched;  //If 'T_BUFFER_MATCH' is specified, else NULL
 
     struct Header
     {
@@ -228,13 +237,6 @@ public:
       struct Section m_section;
       struct Linear m_linear;
     } m_row_data;
-
-    enum RowType
-    {
-      RT_SECTION = 1,
-      RT_LINEAR = 2,
-      RT_END = 0
-    };
   };
 
   struct RowBuffer;  // forward decl.
@@ -480,6 +482,11 @@ public:
     Uint32 m_data[GLOBAL_PAGE_SIZE_WORDS - 7];
     STATIC_CONST( SIZE = GLOBAL_PAGE_SIZE_WORDS - 7 );
   };
+  typedef ArrayPool<RowPage> RowPage_pool;
+  typedef SLList<RowPage_pool> RowPage_list;
+  typedef LocalSLList<RowPage_pool> Local_RowPage_list;
+  typedef DLFifoList<RowPage_pool> RowPage_fifo;
+  typedef LocalDLFifoList<RowPage_pool> Local_RowPage_fifo;
 
   typedef Tup_varsize_page Var_page;
 
@@ -488,11 +495,11 @@ public:
     enum Buffer_type m_type;
 
     RowBuffer() : m_type(BUFFER_VOID) {}
-    DLFifoList<RowPage>::Head m_page_list;
+    RowPage_fifo::Head m_page_list;
 
     void init(enum Buffer_type type)
     {
-      new (&m_page_list) DLFifoList<RowPage>::Head();
+      new (&m_page_list) RowPage_fifo::Head();
       m_type = type;
       reset();
     }
@@ -555,10 +562,11 @@ public:
     void (Dbspj::*m_start)(Signal*, Ptr<Request>, Ptr<TreeNode>);
 
     /**
-     * This function is used when getting a TRANSID_AI
+     * This function is called when a waited for signal arrives.
+     * Sets Request::m_completed_tree_nodes if this completed the
+     * wait for this treeNode
      */
-    void (Dbspj::*m_execTRANSID_AI)(Signal*,Ptr<Request>,Ptr<TreeNode>,
-				    const RowPtr&);
+    void (Dbspj::*m_countSignal)(const Signal*, Ptr<Request>, Ptr<TreeNode>, Uint32 cnt);
 
     /**
      * This function is used when getting a LQHKEYREF
@@ -625,12 +633,26 @@ public:
      * This function is called on node-failure
      */
     Uint32 (Dbspj::*m_execNODE_FAILREP)(Signal*, Ptr<Request>, Ptr<TreeNode>,
-                                        NdbNodeBitmask);
+                                        const NdbNodeBitmask);
     /**
      * This function is called when request/node(s) is/are removed
      *  should only do local cleanup(s)
      */
     void (Dbspj::*m_cleanup)(Ptr<Request>, Ptr<TreeNode>);
+
+    /**
+     * This function is called to check the node validity within
+     * the request during debug execution
+     */
+    bool (Dbspj::*m_checkNode)(const Ptr<Request>, const Ptr<TreeNode>);
+
+    /**
+     * This function is called to dump request info to the node
+     * log for debugging purposes.  It should be used for
+     * treenode type-specific data, the generic treenode info
+     * is handled by dumpNodeCommon()
+     */
+    void (Dbspj::*m_dumpNode)(const Ptr<Request>, const Ptr<TreeNode>);
   };  //struct OpInfo
 
   struct LookupData
@@ -646,14 +668,6 @@ public:
     Uint32 m_lqhKeyReq[LqhKeyReq::FixedSignalLength + 4];
   };
 
-  struct ScanFragData
-  {
-    Uint32 m_rows_received;  // #execTRANSID_AI
-    Uint32 m_rows_expecting; // ScanFragConf
-    Uint32 m_scanFragReq[ScanFragReq::SignalLength + 2];
-    Uint32 m_scanFragHandlePtrI;
-  };
-
   struct ScanFragHandle
   {
     enum SFH_State
@@ -665,29 +679,21 @@ public:
       SFH_WAIT_CLOSE   = 4
     };
 
-    void init(Uint32 fid) {
+    void init(Uint32 fid, bool readBackup)
+    {
       m_ref = 0;
       m_fragId = fid;
       m_state = SFH_NOT_STARTED;
       m_rangePtrI = RNIL;
-      reset_ranges();
+      m_readBackup = readBackup;
     }
 
     Uint32 m_magic;
     Uint32 m_treeNodePtrI;
     Uint16 m_fragId;
-    Uint16 m_state;
+    Uint8 m_state;
+    Uint8 m_readBackup;
     Uint32 m_ref;
-
-    void reset_ranges() {
-      // m_rangePtrI is explicitly managed...in code
-      m_range_builder.m_range_cnt = m_range_builder.m_range_size = 0;
-    }
-    struct RangeBuilder
-    {
-      Uint32 m_range_size;
-      Uint16 m_range_cnt; // too set bounds info correctly
-    } m_range_builder;
     Uint32 m_rangePtrI;
     union {
       Uint32 nextList;
@@ -695,9 +701,9 @@ public:
     };
   };
 
-  typedef RecordPool<ScanFragHandle, ArenaPool> ScanFragHandle_pool;
-  typedef SLFifoListImpl<ScanFragHandle_pool, ScanFragHandle> ScanFragHandle_list;
-  typedef LocalSLFifoListImpl<ScanFragHandle_pool, ScanFragHandle> Local_ScanFragHandle_list;
+  typedef RecordPool<ArenaPool<ScanFragHandle> > ScanFragHandle_pool;
+  typedef SLFifoList<ScanFragHandle_pool> ScanFragHandle_list;
+  typedef LocalSLFifoList<ScanFragHandle_pool> Local_ScanFragHandle_list;
 
   /**
    * This class computes mean and standard deviation incrementally for a series
@@ -717,14 +723,20 @@ public:
     }
 
     // Add another sample.
-    void update(double sample);
+    void sample(double observation);
 
-    double getMean() const { return m_mean; }
+    double getMean() const {
+      return m_mean;
+    }
 
     double getStdDev() const { 
       return m_noOfSamples < 2 ? 0.0 : sqrt(m_sumSquare/(m_noOfSamples - 1));
     }
 
+    bool isValid() const {
+      return (m_noOfSamples > 0);
+    }
+
   private:
     // Mean of all samples
     double m_mean;
@@ -733,7 +745,7 @@ public:
     Uint32 m_noOfSamples;
   }; // IncrementalStatistics
 
-  struct ScanIndexData
+  struct ScanFragData
   {
     Uint16 m_frags_complete;
     Uint16 m_frags_outstanding;
@@ -751,11 +763,6 @@ public:
     Uint32 m_parallelism;
     // True if we are still receiving the first batch for this operation.
     bool   m_firstBatch;
-    /**
-     * True if this is the first instantiation of this operation. A child
-     * operation will be instantiated once for each batch of its parent.
-     */
-    bool m_firstExecution;
     /**
      * Mean and standard deviation for the optimal parallelism for earlier
      * executions of this operation.
@@ -807,25 +814,21 @@ public:
     }
   };
 
-  struct TreeNode_cursor_ptr
-  {
-    Uint32 nextList;
-  };
-
   /**
    * A node in a Query
    *   (This is an instantiated version of QueryNode in
    *    include/kernel/signal/QueryTree.hpp)
    */
-  struct TreeNode : TreeNode_cursor_ptr
+  struct TreeNode
   {
     STATIC_CONST ( MAGIC = ~RT_SPJ_TREENODE );
 
     TreeNode()
     : m_magic(MAGIC), m_state(TN_END),
       m_parentPtrI(RNIL), m_requestPtrI(RNIL),
-      m_ancestors(),
-      m_resumeEvents(0), m_resumePtrI(RNIL)
+      m_ancestors(), m_coverage(), m_predecessors(), m_dependencies(),
+      m_resumeEvents(0), m_resumePtrI(RNIL),
+      m_scanAncestorPtrI(RNIL)
     {
     }
 
@@ -833,9 +836,10 @@ public:
     : m_magic(MAGIC),
       m_info(0), m_bits(T_LEAF), m_state(TN_BUILDING),
       m_parentPtrI(RNIL), m_requestPtrI(request),
-      m_ancestors(),
+      m_ancestors(), m_coverage(), m_predecessors(), m_dependencies(),
       m_resumeEvents(0), m_resumePtrI(RNIL),
-      nextList(RNIL), prevList(RNIL)
+      m_scanAncestorPtrI(RNIL),
+      nextList(RNIL), prevList(RNIL), nextCursor(RNIL)
     {
 //    m_send.m_ref = 0;
       m_send.m_correlation = 0;
@@ -843,6 +847,10 @@ public:
       m_send.m_attrInfoPtrI = RNIL;
     }
 
+    // TreeNode represent either a 'lookup' or 'scan' operation
+    bool isLookup() const { return (m_info == &g_LookupOpInfo); }
+    bool isScan()   const { return (m_info != &g_LookupOpInfo); }
+
     const Uint32 m_magic;
     const struct OpInfo* m_info;
 
@@ -920,22 +928,23 @@ public:
       T_UNIQUE_INDEX_LOOKUP = 0x40,
 
       /*
-       * Should this node buffers its rows
+       * Should this node buffers its rows or 'm_matched' bitmask?
+       *  We could request the buffer to store either the 'ROW',
+       *  as received by TRANSID_AI, and/or a 'MATCH' bitMask.
+       *  If 'ROW' is not specified, the correlationId of the 
+       *  row is stored nevertheless. 
        */
-      T_ROW_BUFFER = 0x80,
+      T_BUFFER_ROW   = 0x80,
+      T_BUFFER_MATCH = 0x100,
+      T_BUFFER_ANY   = (T_BUFFER_ROW | T_BUFFER_MATCH),
 
       /**
-       * Should rows have dictionary (i.e random access capability)
+       * Should row/match buffers have dictionary (i.e random access capability)
        *  This is typically used when having nodes depending on multiple parents
-       *  so that when row gets availble from "last" parent, a key can be
+       *  so that when row gets available from "last" parent, a key can be
        *  constructed using correlation value from parents
        */
-      T_ROW_BUFFER_MAP = 0x100,
-
-      /**
-       * Does any child need to know when all its ancestors are complete
-       */
-      T_REPORT_BATCH_COMPLETE  = 0x200,
+      T_BUFFER_MAP = 0x200,
 
       /**
        * Do *I need* to know when all ancestors has completed this batch
@@ -953,20 +962,53 @@ public:
       T_PRUNE_PATTERN = 0x1000,
 
       /**
-       * Should index scan be parallel
+       * Should fragment scan be parallel
        */
       T_SCAN_PARALLEL = 0x2000,
 
       /**
-       * Possible requesting resultset for this index scan to be repeated
+       * Possible requesting resultset for this fragment scan to be repeated
        */
       T_SCAN_REPEATABLE = 0x4000,
 
+      // 0x8000, Deprecated, available for reuse
+
       /**
-       * Exec of a previous REQ must complete before we can proceed.
-       * A ResumeEvent will later resume exec. of this operation
+       * Does this node need the m_prepare() method to be called.
+       *  (Also implies RT_NEED_PREPARE is set)
        */
-      T_EXEC_SEQUENTIAL = 0x8000,
+      T_NEED_PREPARE = 0x10000,
+
+      /**
+       * Does this node need the m_complete() method to be called.
+       *  (Also implies RT_NEED_COMPLETE is set)
+       *
+       */
+      T_NEED_COMPLETE = 0x20000,
+
+      /**
+       * Allow inner-join optimizations for this treeNode.
+       * (No outer-join semantics required)
+       */
+      T_INNER_JOIN = 0x40000,
+
+      /**
+       * A TRANSID_AI signal is returned for each row found by the datanodes.
+       */
+      T_EXPECT_TRANSID_AI = 0x80000,
+
+      /**
+       * Results from this TreeNode need to be produced in sorted
+       * order, as defined by the index being used.
+       * (Also require that T_SCAN_PARALLEL is set)
+       */
+      T_SORTED_ORDER = 0x100000,
+
+      /**
+       * Allow FirstMatch elimination when multiple rows matching the
+       * same key or range
+       */
+      T_FIRST_MATCH = 0x200000,
 
       // End marker...
       T_END = 0
@@ -979,8 +1021,12 @@ public:
      */
     enum TreeNodeResumeEvents
     {
-      TN_RESUME_REF   = 0x01,
-      TN_RESUME_CONF  = 0x02
+      TN_ENQUEUE_OP   = 0x01,   // Enqueue and wait for RESUME_REF / _CONF
+      TN_RESUME_REF   = 0x02,
+      TN_RESUME_CONF  = 0x04,
+
+      TN_EXEC_WAIT    = 0x08,
+      TN_RESUME_NODE  = 0x10
     };
 
     bool isLeaf() const { return (m_bits & T_LEAF) != 0;}
@@ -998,11 +1044,60 @@ public:
     Uint32 m_batch_size;
     Uint32 m_parentPtrI;
     const Uint32 m_requestPtrI;
+
+    /**
+     * The TreeNode organize its descendant nodes in two different lists:
+     *
+     * m_child_nodes: (The dependent nodes)
+     *   Are the list of descendant nodes as organized by the request sent
+     *   from the SPJ API. All child-TreeNodes will have their 'm_parentPtrI'
+     *   referring 'this' TreeNode.
+     *
+     * m_next_nodes: (The execution order)
+     *   The list of TreeNodes having operations to be started after
+     *   this TreeNode, either when a single operation completes, or
+     *   after completion of entire 'batch' from this TreeNode.
+     *   All 'm_child_nodes' will either be directly included in
+     *   the 'next' list, or be included in the 'next' list of some
+     *   'next' TreeNodes.
+     *   Usage is to set up a more sequential execution plan than what
+     *   is available through the 'child' list.
+     */
+    Dependency_map::Head m_child_nodes;
+    Dependency_map::Head m_next_nodes;
+
+    /**
+     * We provide some TreeNodeBitMap's. Usefull to check how
+     * a specific node relates to other TreeNodes:
+     *
+     * - 'ancestors' are the set of TreeNodes reachable through
+     *    this node's (grand-)parentPtr(s)
+     * - 'coverage' is the set of (grand-)children reachable through
+     *    a 'm_child_nodes' dive. Also include this TreeNode itself.
+     * - 'predecessors' are the TreeNodes which will be executed prior
+     *    to this TreeNode. This include all 'ancestors', as well as
+     *    other TreeNodes which the SPJ 'query planner' may decide
+     *    to execute prior to this TreeNode.
+     * - 'dependencies' are the sub set of 'predecessors' where 
+     *    there are an inner-join relation specified between the
+     *    TreeNodes.
+     *
+     * 'ancestors' and 'coverage' relates to the topology of the query
+     * as represented by parentPtrI and m_child_nodes. 'predecessors' and
+     * 'dependencies' relates to the order of execution, as represented by
+     * 'm_next_nodes'.
+     */
     TreeNodeBitMask m_ancestors;
-    Dependency_map::Head m_dependent_nodes;
+    TreeNodeBitMask m_coverage;
+    TreeNodeBitMask m_predecessors;
+    TreeNodeBitMask m_dependencies;
+
     PatternStore::Head m_keyPattern;
     PatternStore::Head m_attrParamPattern;
 
+    // Memory Arena with lifetime limited to current result batch / node
+    ArenaHead m_batchArena;
+
     /**
      * Rows buffered by this node
      */
@@ -1024,11 +1119,16 @@ public:
     Uint32 m_resumeEvents;
     Uint32 m_resumePtrI;
 
+    /**
+     * The Scan-TreeNode being the head of the inner-joined-branch
+     * this node is a member of.
+     */
+    Uint32 m_scanAncestorPtrI;
+
     union
     {
       LookupData m_lookup_data;
-      ScanFragData m_scanfrag_data;
-      ScanIndexData m_scanindex_data;
+      ScanFragData m_scanFrag_data;
     };
 
     struct {
@@ -1047,17 +1147,18 @@ public:
       Uint32 nextPool;
     };
     Uint32 prevList;
+    Uint32 nextCursor;
   };  //struct TreeNode
 
   static const Ptr<TreeNode> NullTreeNodePtr;
 
-  typedef RecordPool<TreeNode, ArenaPool> TreeNode_pool;
-  typedef DLFifoListImpl<TreeNode_pool, TreeNode> TreeNode_list;
-  typedef LocalDLFifoListImpl<TreeNode_pool, TreeNode> Local_TreeNode_list;
+  typedef RecordPool<ArenaPool<TreeNode> > TreeNode_pool;
+  typedef DLFifoList<TreeNode_pool> TreeNode_list;
+  typedef LocalDLFifoList<TreeNode_pool> Local_TreeNode_list;
 
-  typedef SLListImpl<TreeNode_pool, TreeNode, TreeNode_cursor_ptr>
+  typedef SLList<TreeNode_pool, IA_Cursor>
   TreeNodeCursor_list;
-  typedef LocalSLListImpl<TreeNode_pool, TreeNode, TreeNode_cursor_ptr>
+  typedef LocalSLList<TreeNode_pool, IA_Cursor>
   Local_TreeNodeCursor_list;
 
   /**
@@ -1068,7 +1169,7 @@ public:
     enum RequestBits
     {
       RT_SCAN                = 0x1  // unbounded result set, scan interface
-      ,RT_ROW_BUFFERS        = 0x2  // Do any of the node use row-buffering
+      ,RT_BUFFERS            = 0x2  // Do any of nodes use row/match-buffering
       ,RT_MULTI_SCAN         = 0x4  // Is there several scans in request
 //    ,RT_VAR_ALLOC          = 0x8  // DEPRECATED
       ,RT_NEED_PREPARE       = 0x10 // Does any node need m_prepare hook
@@ -1088,7 +1189,7 @@ public:
 
       RS_ABORTED    = 0x2008, // Aborted and waiting for SCAN_NEXTREQ
       RS_END = 0
-    };  //struct Request
+    };
 
     Request() {}
     Request(const ArenaHead & arena) : m_arena(arena) {}
@@ -1101,14 +1202,15 @@ public:
     Uint32 m_senderData;
     Uint32 m_rootResultData;
     Uint32 m_rootFragId;
+    Uint32 m_rootFragCnt;
     Uint32 m_transId[2];
     TreeNode_list::Head m_nodes;
     TreeNodeCursor_list::Head m_cursor_nodes;
     Uint32 m_cnt_active;       // No of "running" nodes
     TreeNodeBitMask
-           m_active_nodes;     // Nodes which will return more data in NEXTREQ
+      m_active_tree_nodes;     // Nodes which will return more data in NEXTREQ
     TreeNodeBitMask
-           m_completed_nodes;  // Nodes wo/ any 'outstanding' signals
+      m_completed_tree_nodes;  // Nodes wo/ any 'outstanding' signals
     Uint32 m_rows;             // Rows accumulated in current batch
     Uint32 m_outstanding;      // Outstanding signals, when 0, batch is done
     Uint16 m_lookup_node_data[MAX_NDB_NODES];
@@ -1123,7 +1225,8 @@ public:
     NDB_TICKS m_save_time;
 #endif
 
-    bool isScan() const { return (m_bits & RT_SCAN) != 0;}
+    // Entire query may be either a 'scan' or 'lookup' type
+    bool isScan()   const { return (m_bits & RT_SCAN) != 0;}
     bool isLookup() const { return (m_bits & RT_SCAN) == 0;}
 
     bool equal(const Request & key) const {
@@ -1142,7 +1245,7 @@ public:
       Uint32 nextPool;
     };
     Uint32 prevHash;
-  };
+  }; //struct Request
 
 private:
   /**
@@ -1260,17 +1363,17 @@ private:
 
   } c_Counters;
 
-  typedef RecordPool<Request, ArenaPool> Request_pool;
-  typedef DLListImpl<Request_pool, Request> Request_list;
-  typedef LocalDLListImpl<Request_pool, Request> Local_Request_list;
-  typedef DLHashTableImpl<Request_pool, Request> Request_hash;
-  typedef DLHashTableImpl<Request_pool, Request>::Iterator Request_iterator;
+  typedef RecordPool<ArenaPool<Request> > Request_pool;
+  typedef DLList<Request_pool> Request_list;
+  typedef LocalDLList<Request_pool> Local_Request_list;
+  typedef DLHashTable<Request_pool> Request_hash;
+  typedef DLHashTable<Request_pool>::Iterator Request_iterator;
 
   ArenaAllocator m_arenaAllocator;
   Request_pool m_request_pool;
   Request_hash m_scan_request_hash;
   Request_hash m_lookup_request_hash;
-  ArenaPool m_dependency_map_pool;
+  ArenaPool<DataBufferSegment<14> > m_dependency_map_pool;
   TreeNode_pool m_treenode_pool;
   ScanFragHandle_pool m_scanfraghandle_pool;
 
@@ -1289,6 +1392,8 @@ private:
   void store_scan(Ptr<Request>);
   void handle_early_scanfrag_ref(Signal*, const ScanFragReq *, Uint32 err);
 
+  bool checkRequest(const Ptr<Request>);
+
   struct BuildKeyReq
   {
     Uint32 hashInfo[4]; // Used for hashing
@@ -1303,10 +1408,30 @@ private:
   const OpInfo* getOpInfo(Uint32 op);
   Uint32 build(Build_context&,Ptr<Request>,SectionReader&,SectionReader&);
   Uint32 initRowBuffers(Ptr<Request>);
-  void buildExecPlan(Ptr<Request>, Ptr<TreeNode> node, Ptr<TreeNode> next);
-  void checkPrepareComplete(Signal*, Ptr<Request>, Uint32 cnt);
-  void start(Signal*, Ptr<Request>);
-  void checkBatchComplete(Signal*, Ptr<Request>, Uint32 cnt);
+
+  void setupAncestors(Ptr<Request>  requestPtr,
+                      Ptr<TreeNode> treeNodePtr,
+                      Uint32 scanAncestorPtrI);
+  
+  Uint32 buildExecPlan(Ptr<Request> requestPtr);
+  Uint32 planParallelExec(Ptr<Request> requestPtr,
+			  Ptr<TreeNode> treeNodePtr);
+
+  Uint32 planSequentialExec(Ptr<Request>  requestPtr,
+                            const Ptr<TreeNode> branchPtr,
+                            Ptr<TreeNode> prevExecPtr,
+                            const Ptr<TreeNode> outerBranchPtr);
+
+  Uint32 appendTreeNode(Ptr<Request>  requestPtr,
+                        Ptr<TreeNode> treeNodePtr,
+                        Ptr<TreeNode> prevExecPtr,
+                        const Ptr<TreeNode> outerBranchPtr);
+
+  void dumpExecPlan(Ptr<Request>, Ptr<TreeNode> node);
+
+  void prepare(Signal*, Ptr<Request>);
+  void checkPrepareComplete(Signal*, Ptr<Request>);
+  void checkBatchComplete(Signal*, Ptr<Request>);
   void batchComplete(Signal*, Ptr<Request>);
   void prepareNextBatch(Signal*, Ptr<Request>);
   void sendConf(Signal*, Ptr<Request>, bool is_complete);
@@ -1329,8 +1454,8 @@ private:
   /**
    * Row buffering
    */
-  Uint32 storeRow(RowCollection& collection, RowPtr &row);
-  void releaseRow(RowCollection& collection, RowRef ref);
+  Uint32 storeRow(Ptr<TreeNode> treeNodePtr, const RowPtr &row);
+  void releaseRow(Ptr<TreeNode> treeNodePtr, RowRef ref);
   Uint32* stackAlloc(RowBuffer& dst, RowRef&, Uint32 len);
   Uint32* varAlloc(RowBuffer& dst, RowRef&, Uint32 len);
   Uint32* rowAlloc(RowBuffer& dst, RowRef&, Uint32 len);
@@ -1338,10 +1463,15 @@ private:
   void add_to_list(SLFifoRowList & list, RowRef);
   Uint32 add_to_map(RowMap& map, Uint32, RowRef);
 
-  void setupRowPtr(const RowCollection& collection,
+  void setupRowPtr(Ptr<TreeNode> treeNodePtr,
                    RowPtr& dst, RowRef, const Uint32 * src);
   Uint32 * get_row_ptr(RowRef pos);
 
+  void getBufferedRow(const Ptr<TreeNode>, Uint32 rowId,
+                      RowPtr *row);
+
+  void resumeBufferedNode(Signal*, Ptr<Request>, Ptr<TreeNode>);
+
   /**
    * SLFifoRowListIterator
    */
@@ -1376,7 +1506,7 @@ private:
   Uint32 appendParamHeadToPattern(Local_pattern_store&,const RowPtr::Linear&,
                                   Uint32);
 
-  Uint32 appendTreeToSection(Uint32 & ptrI, SectionReader &, Uint32);
+  Uint32 appendReaderToSection(Uint32 & ptrI, SectionReader&, Uint32);
   Uint32 appendColToSection(Uint32 & ptrI, const RowPtr::Linear&, Uint32 col, bool& hasNull);
   Uint32 appendColToSection(Uint32 & ptrI, const RowPtr::Section&, Uint32 col, bool& hasNull);
   Uint32 appendPkColToSection(Uint32 & ptrI, const RowPtr::Section&,Uint32 col);
@@ -1395,8 +1525,9 @@ private:
       return expandS(ptrI, p, r, hasNull);
     case RowPtr::RT_LINEAR:
       return expandL(ptrI, p, r, hasNull);
+    default:
+      return DbspjErr::InternalError;
     }
-    return DbspjErr::InternalError;
   }
   Uint32 expandS(Uint32 & ptrI, Local_pattern_store&, const RowPtr&, bool& hasNull);
   Uint32 expandL(Uint32 & ptrI, Local_pattern_store&, const RowPtr&, bool& hasNull);
@@ -1417,6 +1548,11 @@ private:
   void common_execTRANSID_AI(Signal*, Ptr<Request>, Ptr<TreeNode>,
 			     const RowPtr&);
 
+  void dumpScanFragHandle(const Ptr<ScanFragHandle> fragPtr) const;
+  void dumpNodeCommon(const Ptr<TreeNode>) const;
+  void dumpRequest(const char* reason,
+                   const Ptr<Request> requestPtr);
+
   /**
    * Lookup
    */
@@ -1426,8 +1562,7 @@ private:
   void lookup_start(Signal*, Ptr<Request>, Ptr<TreeNode>);
   void lookup_resume(Signal*, Ptr<Request>, Ptr<TreeNode>);
   void lookup_send(Signal*, Ptr<Request>, Ptr<TreeNode>);
-  void lookup_execTRANSID_AI(Signal*, Ptr<Request>, Ptr<TreeNode>,
-			     const RowPtr&);
+  void lookup_countSignal(const Signal*, Ptr<Request>, Ptr<TreeNode>, Uint32 cnt);
   void lookup_execLQHKEYREF(Signal*, Ptr<Request>, Ptr<TreeNode>);
   void lookup_execLQHKEYCONF(Signal*, Ptr<Request>, Ptr<TreeNode>);
   void lookup_stop_branch(Signal*, Ptr<Request>, Ptr<TreeNode>, Uint32 err);
@@ -1448,6 +1583,10 @@ private:
 
   Uint32 computeHash(Signal*, BuildKeyReq&, Uint32 table, Uint32 keyInfoPtrI);
   Uint32 computePartitionHash(Signal*, BuildKeyReq&, Uint32 table, Uint32 keyInfoPtrI);
+  bool lookup_checkNode(const Ptr<Request> requestPtr, 
+                        const Ptr<TreeNode> treeNodePtr);
+  void lookup_dumpNode(const Ptr<Request> requestPtr,
+                       const Ptr<TreeNode> treeNodePtr);
 
   /**
    * ScanFrag
@@ -1455,58 +1594,52 @@ private:
   static const OpInfo g_ScanFragOpInfo;
   Uint32 scanFrag_build(Build_context&, Ptr<Request>,
                         const QueryNode*, const QueryNodeParameters*);
+  Uint32 parseScanFrag(Build_context&, Ptr<Request>, Ptr<TreeNode>,
+                       DABuffer tree, Uint32 treeBits,
+                       DABuffer param, Uint32 paramBits);
   void scanFrag_start(Signal*, Ptr<Request>,Ptr<TreeNode>);
+  void scanFrag_prepare(Signal*, Ptr<Request>, Ptr<TreeNode>);
+  void scanFrag_countSignal(const Signal*, Ptr<Request>, Ptr<TreeNode>, Uint32 cnt);
+  void scanFrag_execSCAN_FRAGREF(Signal*, Ptr<Request>, Ptr<TreeNode>,
+                                 Ptr<ScanFragHandle>);
+  void scanFrag_execSCAN_FRAGCONF(Signal*, Ptr<Request>, Ptr<TreeNode>,
+                                  Ptr<ScanFragHandle>);
+  void scanFrag_parent_row(Signal*,Ptr<Request>,Ptr<TreeNode>, const RowPtr&);
+  void scanFrag_fixupBound(Ptr<ScanFragHandle> fragPtr, Uint32 ptrI, Uint32);
   void scanFrag_send(Signal*, Ptr<Request>, Ptr<TreeNode>);
-  void scanFrag_execTRANSID_AI(Signal*, Ptr<Request>, Ptr<TreeNode>,
-			       const RowPtr &);
-  void scanFrag_execSCAN_FRAGREF(Signal*, Ptr<Request>, Ptr<TreeNode>, Ptr<ScanFragHandle>);
-  void scanFrag_execSCAN_FRAGCONF(Signal*, Ptr<Request>, Ptr<TreeNode>, Ptr<ScanFragHandle>);
+  Uint32 scanFrag_send(Signal* signal,
+                       Ptr<Request> requestPtr,
+                       Ptr<TreeNode> treeNodePtr,
+                       Uint32 noOfFrags,
+                       Uint32 bs_bytes,
+                       Uint32 bs_rows,
+                       Uint32& batchRange);
+  void scanFrag_batchComplete(Signal* signal);
+  Uint32 scanFrag_findFrag(Local_ScanFragHandle_list &, Ptr<ScanFragHandle>&,
+                           Uint32 fragId);
+  void scanFrag_parent_batch_complete(Signal*, Ptr<Request>, Ptr<TreeNode>);
+  void scanFrag_parent_batch_repeat(Signal*, Ptr<Request>, Ptr<TreeNode>);
   void scanFrag_execSCAN_NEXTREQ(Signal*, Ptr<Request>,Ptr<TreeNode>);
+  void scanFrag_complete(Signal*, Ptr<Request>, Ptr<TreeNode>);
   void scanFrag_abort(Signal*, Ptr<Request>, Ptr<TreeNode>);
+  Uint32 scanFrag_execNODE_FAILREP(Signal*signal, Ptr<Request>, Ptr<TreeNode>,
+                                   NdbNodeBitmask);
+  void scanFrag_parent_batch_cleanup(Ptr<Request>, Ptr<TreeNode>);
   void scanFrag_cleanup(Ptr<Request>, Ptr<TreeNode>);
 
-  /**
-   * ScanIndex
-   */
-  static const OpInfo g_ScanIndexOpInfo;
-  Uint32 scanIndex_build(Build_context&, Ptr<Request>,
-                         const QueryNode*, const QueryNodeParameters*);
-  Uint32 parseScanIndex(Build_context&, Ptr<Request>, Ptr<TreeNode>,
-                        DABuffer tree, Uint32 treeBits,
-                        DABuffer param, Uint32 paramBits);
-  void scanIndex_prepare(Signal*, Ptr<Request>, Ptr<TreeNode>);
-  void scanIndex_execTRANSID_AI(Signal*, Ptr<Request>, Ptr<TreeNode>,
-                                const RowPtr &);
-  void scanIndex_execSCAN_FRAGREF(Signal*, Ptr<Request>, Ptr<TreeNode>, Ptr<ScanFragHandle>);
-  void scanIndex_execSCAN_FRAGCONF(Signal*, Ptr<Request>, Ptr<TreeNode>, Ptr<ScanFragHandle>);
-  void scanIndex_parent_row(Signal*,Ptr<Request>,Ptr<TreeNode>, const RowPtr&);
-  void scanIndex_fixupBound(Ptr<ScanFragHandle> fragPtr, Uint32 ptrI, Uint32);
-  Uint32 scanIndex_send(Signal* signal,
-                        Ptr<Request> requestPtr,
-                        Ptr<TreeNode> treeNodePtr,
-                        Uint32 noOfFrags,
-                        Uint32 bs_bytes,
-                        Uint32 bs_rows,
-                        Uint32& batchRange);
-  void scanIndex_batchComplete(Signal* signal);
-  Uint32 scanIndex_findFrag(Local_ScanFragHandle_list &, Ptr<ScanFragHandle>&,
-                            Uint32 fragId);
-  void scanIndex_parent_batch_complete(Signal*, Ptr<Request>, Ptr<TreeNode>);
-  void scanIndex_parent_batch_repeat(Signal*, Ptr<Request>, Ptr<TreeNode>);
-  void scanIndex_execSCAN_NEXTREQ(Signal*, Ptr<Request>,Ptr<TreeNode>);
-  void scanIndex_complete(Signal*, Ptr<Request>, Ptr<TreeNode>);
-  void scanIndex_abort(Signal*, Ptr<Request>, Ptr<TreeNode>);
-  Uint32 scanIndex_execNODE_FAILREP(Signal*signal, Ptr<Request>, Ptr<TreeNode>,
-                                  NdbNodeBitmask);
-  void scanIndex_parent_batch_cleanup(Ptr<Request>, Ptr<TreeNode>);
-  void scanIndex_cleanup(Ptr<Request>, Ptr<TreeNode>);
+  void scanFrag_release_rangekeys(Ptr<Request>, Ptr<TreeNode>);
 
-  void scanIndex_release_rangekeys(Ptr<Request>, Ptr<TreeNode>);
+  Uint32 scanFrag_sendDihGetNodesReq(Signal* signal,
+                                     Ptr<Request> requestPtr,
+                                     Ptr<TreeNode> treeNodePtr);
 
-  Uint32 scanindex_sendDihGetNodesReq(Signal* signal,
-                                      Ptr<Request> requestPtr,
-                                      Ptr<TreeNode> treeNodePtr);
+  bool scanFrag_checkNode(const Ptr<Request> requestPtr,
+                          const Ptr<TreeNode> treeNodePtr);
 
+  void scanFrag_dumpNode(const Ptr<Request> requestPtr,
+                         const Ptr<TreeNode> treeNodePtr);
+
+  Uint32 check_own_location_domain(const Uint32 *nodes, Uint32 node_count);
   /**
    * Page manager
    */
@@ -1514,8 +1647,8 @@ private:
   void releasePage(Ptr<RowPage>);
   void releasePages(Uint32 first, Ptr<RowPage> last);
   void releaseGlobal(Signal*);
-  SLList<RowPage>::Head m_free_page_list;
-  ArrayPool<RowPage> m_page_pool;
+  RowPage_list::Head m_free_page_list;
+  RowPage_pool m_page_pool;
 
   /* Random fault injection */
 
@@ -1524,6 +1657,8 @@ private:
                        const Uint32* src, Uint32 len);
 #endif
 
+  Uint32 m_location_domain_id[MAX_NDB_NODES];
+  Uint32 m_load_balancer_location;
   /**
    * Scratch buffers...
    */
@@ -1531,7 +1666,6 @@ private:
   Uint32 m_buffer1[16*1024]; // 64k
 };
 
-
 #undef JAM_FILE_ID
 
 #endif
