@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2003, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,9 +25,9 @@
 #ifndef LONG_SIGNAL_HPP
 #define LONG_SIGNAL_HPP
 
+#include "pc.hpp"
 #include <ArrayPool.hpp>
 #include "DataBuffer.hpp"
-#include "pc.hpp"
 
 #define JAM_FILE_ID 288
 
@@ -57,8 +56,9 @@
  * Section handling
  */
 struct SectionSegment {
-  static constexpr Uint32 DataLength = NDB_SECTION_SEGMENT_SZ;
 
+  STATIC_CONST( DataLength = NDB_SECTION_SEGMENT_SZ );
+  
   union {
     Uint32 m_sz;
     Uint32 chunkSize;
@@ -69,7 +69,7 @@ struct SectionSegment {
   };
   union {
     Uint32 m_lastSegment;
-    Uint32 lastChunk;  //
+    Uint32 lastChunk;  // 
   };
   union {
     Uint32 m_nextSegment;
@@ -83,13 +83,14 @@ typedef CachedArrayPool<SectionSegment> SectionSegment_basepool;
  * Pool for SectionSegments
  */
 
-class SectionSegmentPool : public SectionSegment_basepool {
- private:
+class SectionSegmentPool : public SectionSegment_basepool
+{
+private:
   // Print an informative error message.
-  static void handleOutOfSegments(SectionSegment_basepool &pool);
-
- public:
-  SectionSegmentPool() : SectionSegment_basepool(&handleOutOfSegments) {}
+  static void handleOutOfSegments(SectionSegment_basepool& pool);
+public:
+  SectionSegmentPool() : 
+    SectionSegment_basepool(&handleOutOfSegments){}
 };
 
 /**
@@ -102,58 +103,59 @@ extern SectionSegmentPool g_sectionSegmentPool;
  * Section Segment pool - hides details of
  * cache / mt etc.
  */
-class SegmentUtils {
- public:
+class SegmentUtils
+{
+public:
   virtual ~SegmentUtils() {}
 
   /* 'Provider interface' */
   /* Low level ops needed to build tools */
-  virtual SectionSegment *getSegmentPtr(Uint32 iVal) = 0;
-  void getSegmentPtr(Ptr<SectionSegment> &ptr, Uint32 iVal);
-  virtual bool seizeSegment(Ptr<SectionSegment> &p) = 0;
+  virtual SectionSegment* getSegmentPtr(Uint32 iVal) = 0;
+  void getSegmentPtr(Ptr<SectionSegment>& ptr, Uint32 iVal);
+  virtual bool seizeSegment(Ptr<SectionSegment>& p) = 0;
   virtual void releaseSegment(Uint32 iVal) = 0;
 
   /* Release a linked list of segments with valid size) */
   virtual void releaseSegmentList(Uint32 iVal) = 0;
 };
 
-inline void SegmentUtils::getSegmentPtr(Ptr<SectionSegment> &ptr, Uint32 iVal) {
+inline void SegmentUtils::getSegmentPtr(Ptr<SectionSegment>& ptr, Uint32 iVal)
+{
   ptr.i = iVal;
   ptr.p = getSegmentPtr(iVal);
 }
 
 /* Higher level utils */
 /* Currently defined in SegmentList.cpp, should move to somewhere else */
-bool sectionAppend(SegmentUtils &su, Uint32 &firstSegmentIVal,
-                   const Uint32 *src, Uint32 len);
-bool sectionConsume(SegmentUtils &su, Uint32 &firstSegmentIVal, Uint32 *dst,
-                    Uint32 len);
-bool sectionVerify(SegmentUtils &su, Uint32 firstIVal);
+bool sectionAppend(SegmentUtils& su, Uint32& firstSegmentIVal, const Uint32* src, Uint32 len);
+bool sectionConsume(SegmentUtils& su, Uint32& firstSegmentIVal, Uint32* dst, Uint32 len);
+bool sectionVerify(SegmentUtils& su, Uint32 firstIVal);
 
 /**
  * Function prototypes
  */
-void print(SegmentedSectionPtr ptr, FILE *out);
-void copy(Uint32 *dst, SegmentedSectionPtr src);
-void copy(Uint32 *dst, Uint32 srcFirstIVal);
+void print(SegmentedSectionPtr ptr, FILE* out);
+void copy(Uint32 * dst, SegmentedSectionPtr src);
+void copy(Uint32 * dst, Uint32 srcFirstIVal);
 
 extern class SectionSegmentPool g_sectionSegmentPool;
 
 /* Defined in SimulatedBlock.cpp */
-void getSection(SegmentedSectionPtr &ptr, Uint32 id);
+void getSection(SegmentedSectionPtr & ptr, Uint32 id);
 void getSections(Uint32 secCount, SegmentedSectionPtr ptr[3]);
 Uint32 getSectionSz(Uint32 id);
-Uint32 *getLastWordPtr(Uint32 id);
+Uint32* getLastWordPtr(Uint32 id);
 
 /* Internal verification */
-bool verifySection(Uint32 firstIVal,
-                   SectionSegmentPool &thePool = g_sectionSegmentPool);
+bool verifySection(Uint32 firstIVal, 
+                   SectionSegmentPool& thePool= g_sectionSegmentPool);
 
-template <Uint32 sz, typename Pool, Uint32 Type_id>
-void append(DataBuffer<sz, Pool, Type_id> &dst, SegmentedSectionPtr ptr,
-            SectionSegmentPool &pool) {
+template<Uint32 sz, typename Pool, Uint32 Type_id>
+void
+append(DataBuffer<sz, Pool, Type_id>& dst, SegmentedSectionPtr ptr, SectionSegmentPool& pool)
+{
   Uint32 len = ptr.sz;
-  while (len > SectionSegment::DataLength) {
+  while(len > SectionSegment::DataLength){
     dst.append(ptr.p->theData, SectionSegment::DataLength);
     ptr.p = pool.getPtr(ptr.p->m_nextSegment);
     len -= SectionSegment::DataLength;
@@ -167,24 +169,27 @@ void setResOwner(Uint32 id);
 Uint32 getResOwner();
 
 /* Util for custom-owner within a scope */
-class ResOwnerGuard {
- private:
+class ResOwnerGuard
+{
+private:
   Uint32 oldOwner;
-
- public:
-  ResOwnerGuard(Uint32 id) {
+public:
+  ResOwnerGuard(Uint32 id)
+  {
     oldOwner = getResOwner();
     setResOwner(id);
   }
-  ~ResOwnerGuard() { setResOwner(oldOwner); }
+  ~ResOwnerGuard()
+  {
+    setResOwner(oldOwner);
+  }
 };
 
 #define DEBUG_RES_OWNER_GUARD(x) ResOwnerGuard _ROG_TMP(x)
 
 #else
 
-#define DEBUG_RES_OWNER_GUARD(x) \
-  {}
+#define DEBUG_RES_OWNER_GUARD(x) { }
 
 #endif
 
