@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2004, 2023, Oracle and/or its affiliates.
+   Copyright (C) 2004-2006 MySQL AB
     Use is subject to license terms.
 
    This program is free software; you can redistribute it and/or modify
@@ -27,26 +27,26 @@
 #define NdbBlobImpl_H
 
 class NdbBlobImpl {
- public:
-  static constexpr Uint32 BlobTableNameSize = 40;
+public:
+  STATIC_CONST( BlobTableNameSize = 40 );
   // "Invalid blob attributes or invalid blob parts table"
-  static constexpr Uint32 ErrTable = 4263;
-  // "Invalid usage of blob attribute"
-  static constexpr Uint32 ErrUsage = 4264;
+  STATIC_CONST( ErrTable = 4263 );
+  // "Invalid usage of blob attribute" 
+  STATIC_CONST( ErrUsage = 4264 );
   // "The blob method is not valid in current blob state"
-  static constexpr Uint32 ErrState = 4265;
+  STATIC_CONST( ErrState = 4265 );
   // "Invalid blob seek position"
-  static constexpr Uint32 ErrSeek = 4266;
+  STATIC_CONST( ErrSeek = 4266 );
   // "Corrupted blob value"
-  static constexpr Uint32 ErrCorrupt = 4267;
+  STATIC_CONST( ErrCorrupt = 4267 );
   // "Error in blob head update forced rollback of transaction"
-  static constexpr Uint32 ErrAbort = 4268;
+  STATIC_CONST( ErrAbort = 4268 );
   // "Unknown blob error"
-  static constexpr Uint32 ErrUnknown = 4270;
+  STATIC_CONST( ErrUnknown = 4270 );
   // "Corrupted main table PK in blob operation"
-  static constexpr Uint32 ErrCorruptPK = 4274;
+  STATIC_CONST( ErrCorruptPK = 4274 );
   // "The blob method is incompatible with operation type or lock mode"
-  static constexpr Uint32 ErrCompat = 4275;
+  STATIC_CONST( ErrCompat = 4275 );
 };
 
 #endif
