@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2004, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2004, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,14 +22,15 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
+
 #ifndef CLUSTER_CONNECTION_IMPL_HPP
 #define CLUSTER_CONNECTION_IMPL_HPP
 
-#include <NdbMutex.h>
-#include <NodeBitmask.hpp>
-#include <Vector.hpp>
 #include <cstdint>
 #include <ndb_cluster_connection.hpp>
+#include <Vector.hpp>
+#include <NdbMutex.h>
+#include <NodeBitmask.hpp>
 #include "DictCache.hpp"
 #include "kernel/ndb_limits.h"
 
@@ -43,29 +43,32 @@ struct ndb_mgm_configuration;
 class Ndb;
 
 extern "C" {
-void *run_ndb_cluster_connection_connect_thread(void *);
+  void* run_ndb_cluster_connection_connect_thread(void*);
 }
 
-struct NdbApiConfig {
-  NdbApiConfig()
-      : m_scan_batch_size(MAX_SCAN_BATCH_SIZE),
-        m_batch_byte_size(SCAN_BATCH_SIZE),
-        m_batch_size(DEF_BATCH_SIZE),
-        m_waitfor_timeout(120000),
-        m_default_queue_option(0),
-        m_default_hashmap_size(0),
-        m_verbose(0) {}
+struct NdbApiConfig
+{
+  NdbApiConfig() :
+    m_scan_batch_size(MAX_SCAN_BATCH_SIZE),
+    m_batch_byte_size(SCAN_BATCH_SIZE),
+    m_batch_size(DEF_BATCH_SIZE),
+    m_waitfor_timeout(120000),
+    m_default_queue_option(0),
+    m_default_hashmap_size(0),
+    m_verbose(0)
+    {}
 
   Uint32 m_scan_batch_size;
   Uint32 m_batch_byte_size;
   Uint32 m_batch_size;
-  Uint32 m_waitfor_timeout;  // in milli seconds...
+  Uint32 m_waitfor_timeout; // in milli seconds...
   Uint32 m_default_queue_option;
   Uint32 m_default_hashmap_size;
   Uint32 m_verbose;
 };
 
-class Ndb_cluster_connection_impl : public Ndb_cluster_connection {
+class Ndb_cluster_connection_impl : public Ndb_cluster_connection
+{
   Ndb_cluster_connection_impl(const char *connectstring,
                               Ndb_cluster_connection *main_connection,
                               int force_api_nodeid);
@@ -79,62 +82,57 @@ class Ndb_cluster_connection_impl : public Ndb_cluster_connection {
 
   inline unsigned get_connect_count() const;
   inline unsigned get_min_db_version() const;
-  inline unsigned get_min_api_version() const;
-
- public:
+public:
   inline Uint64 *get_latest_trans_gci() { return &m_latest_trans_gci; }
 
- private:
+private:
   friend class Ndb;
   friend class NdbImpl;
   friend class NdbWaitGroup;
-  friend void *run_ndb_cluster_connection_connect_thread(void *);
+  friend void* run_ndb_cluster_connection_connect_thread(void*);
   friend class Ndb_cluster_connection;
   friend class NdbEventBuffer;
   friend class SignalSender;
   friend class NDBT_Context;
-
+  
   static Int32 const MAX_PROXIMITY_GROUP = INT32_MAX;
   static Int32 const INVALID_PROXIMITY_GROUP = INT32_MIN;
   static Int32 const DATA_NODE_NEIGHBOUR_PROXIMITY_ADJUSTMENT = -30;
   static Uint32 const HINT_COUNT_BITS = 10;
   static Uint32 const HINT_COUNT_HALF = (1 << (HINT_COUNT_BITS - 1));
-  static Uint32 const HINT_COUNT_MASK =
-      (HINT_COUNT_HALF | (HINT_COUNT_HALF - 1));
+  static Uint32 const HINT_COUNT_MASK = (HINT_COUNT_HALF | (HINT_COUNT_HALF - 1));
 
-  struct Node {
-    Node(Uint32 _g = 0, Uint32 _id = 0)
-        : this_group_idx(0),
-          next_group_idx(0),
-          config_group(_g),  // between 0 and 200
-          adjusted_group(_g),
-          id(_id),
-          hint_count(0) {}
-    Uint32 this_group_idx;  // First index of node with same group
-    Uint32 next_group_idx;  // Next index of node not with same node, or 0.
-    Uint32 config_group;    // Proximity group from cluster connection config
-    Int32 adjusted_group;   // Proximity group adjusted via ndbapi calls
+  struct Node
+  {
+    Node(Uint32 _g= 0, Uint32 _id= 0) : this_group_idx(0),
+                                        next_group_idx(0),
+                                        config_group(_g), // between 0 and 200
+                                        adjusted_group(_g),
+                                        id(_id),
+                                        hint_count(0) {}
+    Uint32 this_group_idx; // First index of node with same group
+    Uint32 next_group_idx; // Next index of node not with same node, or 0.
+    Uint32 config_group; // Proximity group from cluster connection config
+    Int32 adjusted_group; // Proximity group adjusted via ndbapi calls
     Uint32 id;
     /**
-     * Counts how many times node was chosen for hint when
+     * Counts how many times node was choosen for hint when
      * more than one were possible.
      */
     Uint32 hint_count;
   };
 
   NdbNodeBitmask m_db_nodes;
-  NdbMutex *m_nodes_proximity_mutex;
+  NdbMutex* m_nodes_proximity_mutex;
   Vector<Node> m_nodes_proximity;
   Uint16 m_location_domain_id[MAX_NODES];
   Uint32 m_my_node_id;
   Uint32 m_max_api_nodeid;
   Uint32 m_my_location_domain_id;
-  int init_nodes_vector(Uint32 nodeid, const ndb_mgm_configuration *config);
-  int configure(Uint32 nodeid, const ndb_mgm_configuration *config);
+  int init_nodes_vector(Uint32 nodeid, const ndb_mgm_configuration &config);
+  int configure(Uint32 nodeid, const ndb_mgm_configuration &config);
   void connect_thread();
   void set_name(const char *name);
-  void configure_tls(const char *search_path, int mgm_tls_level);
-  const char *get_tls_certificate_path() const;
   int set_service_uri(const char *, const char *, int, const char *);
   void set_data_node_neighbour(Uint32 neighbour_node);
   void adjust_node_proximity(Uint32 node_id, Int32 adjustment);
@@ -144,12 +142,13 @@ class Ndb_cluster_connection_impl : public Ndb_cluster_connection {
   /**
    * Select the "closest" node
    */
-  Uint32 select_node(NdbImpl *impl_ndb, const Uint16 *nodes, Uint32 cnt);
+  Uint32 select_node(NdbImpl *impl_ndb, const Uint16* nodes, Uint32 cnt);
   /**
    * Select primary or if primary in other location domain
    * choose a node in the same location domain
    */
-  Uint32 select_location_based(NdbImpl *impl_ndb, const Uint16 *nodes,
+  Uint32 select_location_based(NdbImpl *impl_ndb,
+                               const Uint16* nodes,
                                Uint32 cnt);
   /**
    * Choose node in same location domain if one exists, otherwise
@@ -157,7 +156,9 @@ class Ndb_cluster_connection_impl : public Ndb_cluster_connection {
    */
   Uint32 select_any(NdbImpl *impl_ndb);
 
-  int connect(int no_retries, int retry_delay_in_seconds, int verbose);
+  int connect(int no_retries,
+              int retry_delay_in_seconds,
+              int verbose);
 
   Ndb_cluster_connection *m_main_connection;
   GlobalDictCache *m_globalDictCache;
@@ -171,17 +172,17 @@ class Ndb_cluster_connection_impl : public Ndb_cluster_connection {
   NdbMutex *m_event_add_drop_mutex;
   Uint64 m_latest_trans_gci;
 
-  NdbMutex *m_new_delete_ndb_mutex;
-  NdbCondition *m_new_delete_ndb_cond;
-  Ndb *m_first_ndb_object;
-  void link_ndb_object(Ndb *);
-  void unlink_ndb_object(Ndb *);
+  NdbMutex* m_new_delete_ndb_mutex;
+  NdbCondition* m_new_delete_ndb_cond;
+  Ndb* m_first_ndb_object;
+  void link_ndb_object(Ndb*);
+  void unlink_ndb_object(Ndb*);
 
   BaseString m_latest_error_msg;
   unsigned m_latest_error;
 
   // Scan batch configuration parameters
-  NdbApiConfig m_ndbapiconfig;
+  NdbApiConfig m_config;
 
   // Avoid transid reuse with Block ref reuse
   Vector<Uint32> m_next_transids;
@@ -191,9 +192,9 @@ class Ndb_cluster_connection_impl : public Ndb_cluster_connection {
   // Closest data node neighbour
   Uint32 m_data_node_neighbour;
 
-  // Base offset for stats, from Ndb objects that are no
+  // Base offset for stats, from Ndb objects that are no 
   // longer with us
-  Uint64 globalApiStatsBaseline[Ndb::NumClientStatistics];
+  Uint64 globalApiStatsBaseline[ Ndb::NumClientStatistics ];
 
   NdbWaitGroup *m_multi_wait_group;
 
@@ -203,15 +204,6 @@ class Ndb_cluster_connection_impl : public Ndb_cluster_connection {
 
   // system.name copied from configuration
   BaseString m_system_name;
-
-  // Config generation of used configuration
-  Uint32 m_config_generation{0};
-
-  // TLS Certificate Search Path
-  const char *m_tls_search_path{nullptr};
-
-  // Some connection requires TLS
-  bool m_tls_requirement{false};
 };
 
 #endif
