@@ -1,17 +1,16 @@
 /*
- Copyright (c) 2003, 2024, Oracle and/or its affiliates.
+ Copyright (c) 2003, 2019, Oracle and/or its affiliates.  All rights reserved
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2.0,
  as published by the Free Software Foundation.
 
- This program is designed to work with certain software (including
+ This program is also distributed with certain software (including
  but not limited to OpenSSL) that is licensed under separate terms,
  as designated in a particular file or component or in included license
  documentation.  The authors of MySQL hereby grant you an additional
  permission to link the program and your derivative works with the
- separately licensed software that they have either included with
- the program or referenced in the documentation.
+ separately licensed software that they have included with MySQL.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,35 +22,37 @@
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
-#include <HugoTransactions.hpp>
 #include <NDBT.hpp>
 #include <NDBT_Test.hpp>
-#include <NdbRestarter.hpp>
+#include <HugoTransactions.hpp>
 #include <UtilTransactions.hpp>
+#include <NdbRestarter.hpp>
+#include <NdbBackup.hpp>
+
+
+#define CHECK(b) if (!(b)) { \
+  g_err << "ERR: "<< step->getName() \
+         << " failed on line " << __LINE__ << endl; \
+  result = NDBT_FAILED; \
+  continue; } 
 
-#define CHECK(b)                                                          \
-  if (!(b)) {                                                             \
-    g_err << "ERR: " << step->getName() << " failed on line " << __LINE__ \
-          << endl;                                                        \
-    result = NDBT_FAILED;                                                 \
-    continue;                                                             \
-  }
 
 #include "Bank.hpp"
 
-const char *_database = "BANK";
+const char* _database = "BANK";
 
-int runCreateBank(NDBT_Context *ctx, NDBT_Step *step) {
+int runCreateBank(NDBT_Context* ctx, NDBT_Step* step){
   Bank bank(ctx->m_cluster_connection, _database);
   int overWriteExisting = true;
-  if (bank.createAndLoadBank(overWriteExisting) != NDBT_OK) return NDBT_FAILED;
+  if (bank.createAndLoadBank(overWriteExisting) != NDBT_OK)
+    return NDBT_FAILED;
   return NDBT_OK;
 }
 
-int runBankTimer(NDBT_Context *ctx, NDBT_Step *step) {
+int runBankTimer(NDBT_Context* ctx, NDBT_Step* step){
   Bank bank(ctx->m_cluster_connection, _database);
-  int wait = 30;  // Max seconds between each "day"
-  int yield = 1;  // Loops before bank returns
+  int wait = 30; // Max seconds between each "day"
+  int yield = 1; // Loops before bank returns 
 
   while (ctx->isTestStopped() == false) {
     bank.performIncreaseTime(wait, yield);
@@ -59,10 +60,10 @@ int runBankTimer(NDBT_Context *ctx, NDBT_Step *step) {
   return NDBT_OK;
 }
 
-int runBankTransactions(NDBT_Context *ctx, NDBT_Step *step) {
+int runBankTransactions(NDBT_Context* ctx, NDBT_Step* step){
   Bank bank(ctx->m_cluster_connection, _database);
-  int wait = 10;    // Max ms between each transaction
-  int yield = 100;  // Loops before bank returns
+  int wait = 10; // Max ms between each transaction
+  int yield = 100; // Loops before bank returns 
 
   while (ctx->isTestStopped() == false) {
     bank.performTransactions(wait, yield);
@@ -70,13 +71,13 @@ int runBankTransactions(NDBT_Context *ctx, NDBT_Step *step) {
   return NDBT_OK;
 }
 
-int runBankGL(NDBT_Context *ctx, NDBT_Step *step) {
+int runBankGL(NDBT_Context* ctx, NDBT_Step* step){
   Bank bank(ctx->m_cluster_connection, _database);
-  int yield = 20;  // Loops before bank returns
+  int yield = 20; // Loops before bank returns 
   int result = NDBT_OK;
 
   while (ctx->isTestStopped() == false) {
-    if (bank.performMakeGLs(yield) != NDBT_OK) {
+    if (bank.performMakeGLs(yield) != NDBT_OK){
       ndbout << "bank.performMakeGLs FAILED" << endl;
       result = NDBT_FAILED;
     }
@@ -84,47 +85,49 @@ int runBankGL(NDBT_Context *ctx, NDBT_Step *step) {
   return NDBT_OK;
 }
 
-int runBankSum(NDBT_Context *ctx, NDBT_Step *step) {
+int runBankSum(NDBT_Context* ctx, NDBT_Step* step){
   Bank bank(ctx->m_cluster_connection, _database);
-  int wait = 2000;  // Max ms between each sum of accounts
-  int yield = 1;    // Loops before bank returns
+  int wait = 2000; // Max ms between each sum of accounts
+  int yield = 1; // Loops before bank returns 
   int result = NDBT_OK;
 
   while (ctx->isTestStopped() == false) {
-    if (bank.performSumAccounts(wait, yield) != NDBT_OK) {
+    if (bank.performSumAccounts(wait, yield) != NDBT_OK){
       ndbout << "bank.performSumAccounts FAILED" << endl;
       result = NDBT_FAILED;
     }
   }
-  return result;
+  return result ;
 }
 
-int runDropBank(NDBT_Context *ctx, NDBT_Step *step) {
+int runDropBank(NDBT_Context* ctx, NDBT_Step* step){
   Bank bank(ctx->m_cluster_connection, _database);
-  if (bank.dropBank() != NDBT_OK) return NDBT_FAILED;
+  if (bank.dropBank() != NDBT_OK)
+    return NDBT_FAILED;
   return NDBT_OK;
 }
 
-int runBankController(NDBT_Context *ctx, NDBT_Step *step) {
-  Ndb *pNdb = GETNDB(step);
+int runBankController(NDBT_Context* ctx, NDBT_Step* step){
+  Ndb* pNdb = GETNDB(step);
   int loops = ctx->getNumLoops();
   int records = ctx->getNumRecords();
   int l = 0;
   int result = NDBT_OK;
 
-  while (l < loops && result != NDBT_FAILED) {
-    if (pNdb->waitUntilReady() != 0) {
+  while (l < loops && result != NDBT_FAILED){
+
+    if (pNdb->waitUntilReady() != 0){
       result = NDBT_FAILED;
       continue;
     }
 
     // Sleep for a while
     NdbSleep_SecSleep(records);
-
+    
     l++;
   }
 
-  if (pNdb->waitUntilReady() != 0) {
+  if (pNdb->waitUntilReady() != 0){
     result = NDBT_FAILED;
   }
 
@@ -134,7 +137,8 @@ int runBankController(NDBT_Context *ctx, NDBT_Step *step) {
 }
 
 NDBT_TESTSUITE(testBank);
-TESTCASE("Bank", "Run the bank\n") {
+TESTCASE("Bank", 
+	 "Run the bank\n"){
   INITIALIZER(runCreateBank);
   STEP(runBankTimer);
   STEP(runBankTransactions);
@@ -142,10 +146,11 @@ TESTCASE("Bank", "Run the bank\n") {
   // TODO  STEP(runBankSum);
   STEP(runBankController);
   FINALIZER(runDropBank);
+
 }
 NDBT_TESTSUITE_END(testBank)
 
-int main(int argc, const char **argv) {
+int main(int argc, const char** argv){
   ndb_init();
   // Tables should not be auto created
   NDBT_TESTSUITE_INSTANCE(testBank);
@@ -153,3 +158,5 @@ int main(int argc, const char **argv) {
 
   return testBank.execute(argc, argv);
 }
+
+
