@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2004, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2004, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -701,10 +701,8 @@ run_startHint_ordered_index(NDBT_Context* ctx, NDBT_Step* step)
     return NDBT_FAILED;
   }
 
-  const Uint32 errorInsert = ctx->getProperty("errorinsertion", (unsigned) 8050);
-
   NdbRestarter restarter;
-  if(restarter.insertErrorInAllNodes(errorInsert) != 0)
+  if(restarter.insertErrorInAllNodes(8050) != 0)
     return NDBT_FAILED;
   
   HugoCalculator dummy(*tab);
@@ -1096,7 +1094,7 @@ dist_scan_body(Ndb* pNdb, int records, int parts, PartInfo* partInfo, bool usePr
                                                             tabRow,
                                                             tab->getColumn(DistTabDKeyCol)->getAttrId()));
           dKeyVal= partValue;
-          // dKeyVal= partValue + 1; // Test failure case
+          // dKeyVal= partValue + 1; // Test failue case
           
           pSpec.type= Ndb::PartitionSpec::PS_DISTR_KEY_RECORD;
           pSpec.KeyRecord.keyRecord= tabRecord;
@@ -1409,20 +1407,6 @@ TESTCASE("startTransactionHint_orderedIndex_mrr_userDefined",
   INITIALIZER(run_dist_test);
   INITIALIZER(run_drop_dist_table);
 }
-TESTCASE("startTransactionHint_orderedIndex_MaxKey",
-         "Test startTransactionHint with max hash value via error insert")
-{
-  /* Special regression case */
-  TC_PROPERTY("distributionkey", (unsigned)0);
-  TC_PROPERTY("OrderedIndex", (unsigned)1);
-  TC_PROPERTY("errorinsertion", (unsigned) 8119);
-  INITIALIZER(run_drop_table);
-  INITIALIZER(run_create_table);
-  INITIALIZER(run_create_pk_index);
-  INITIALIZER(run_startHint_ordered_index);
-  INITIALIZER(run_create_pk_index_drop);
-  INITIALIZER(run_drop_table);
-}
 
 NDBT_TESTSUITE_END(testPartitioning)
 
