@@ -1,16 +1,15 @@
-/* Copyright (c) 2011, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2011, 2020, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -41,9 +40,9 @@
 
 class PFS_index_users : public PFS_engine_index {
  public:
-  explicit PFS_index_users(PFS_engine_key *key_1) : PFS_engine_index(key_1) {}
+  PFS_index_users(PFS_engine_key *key_1) : PFS_engine_index(key_1) {}
 
-  ~PFS_index_users() override = default;
+  ~PFS_index_users() {}
 
   virtual bool match(PFS_user *pfs) = 0;
 };
@@ -53,18 +52,18 @@ class cursor_by_user : public PFS_engine_table {
  public:
   static ha_rows get_row_count();
 
-  void reset_position() override;
+  virtual void reset_position(void);
 
-  int rnd_next() override;
-  int rnd_pos(const void *pos) override;
+  virtual int rnd_next();
+  virtual int rnd_pos(const void *pos);
 
-  int index_next() override;
+  virtual int index_next();
 
  protected:
-  explicit cursor_by_user(const PFS_engine_table_share *share);
+  cursor_by_user(const PFS_engine_table_share *share);
 
  public:
-  ~cursor_by_user() override = default;
+  ~cursor_by_user() {}
 
  protected:
   virtual int make_row(PFS_user *user) = 0;
