@@ -1,4 +1,4 @@
-/* Copyright (c) 2010, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2020, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -28,9 +28,8 @@
 
 #include "storage/perfschema/pfs_account.h"
 
-#include <assert.h>
 #include "my_compiler.h"
-
+#include "my_dbug.h"
 #include "my_sys.h"
 #include "sql/mysqld.h"  // global_status_var
 #include "storage/perfschema/pfs.h"
@@ -65,80 +64,36 @@ int init_account(const PFS_global_param *param) {
 }
 
 /** Cleanup all the account buffers. */
-void cleanup_account() { global_account_container.cleanup(); }
+void cleanup_account(void) { global_account_container.cleanup(); }
 
 static const uchar *account_hash_get_key(const uchar *entry, size_t *length) {
   const PFS_account *const *typed_entry;
   const PFS_account *account;
   const void *result;
   typed_entry = reinterpret_cast<const PFS_account *const *>(entry);
-  assert(typed_entry != nullptr);
+  DBUG_ASSERT(typed_entry != nullptr);
   account = *typed_entry;
-  assert(account != nullptr);
-  *length = sizeof(account->m_key);
-  result = &account->m_key;
+  DBUG_ASSERT(account != nullptr);
+  *length = account->m_key.m_key_length;
+  result = account->m_key.m_hash_key;
   return reinterpret_cast<const uchar *>(result);
 }
 
-static uint account_hash_func(const LF_HASH *, const uchar *key,
-                              size_t key_len [[maybe_unused]]) {
-  const PFS_account_key *account_key;
-  uint64 nr1;
-  uint64 nr2;
-
-  assert(key_len == sizeof(PFS_account_key));
-  account_key = reinterpret_cast<const PFS_account_key *>(key);
-  assert(account_key != nullptr);
-
-  nr1 = 0;
-  nr2 = 0;
-
-  account_key->m_user_name.hash(&nr1, &nr2);
-  account_key->m_host_name.hash(&nr1, &nr2);
-
-  return nr1;
-}
-
-static int account_hash_cmp_func(const uchar *key1,
-                                 size_t key_len1 [[maybe_unused]],
-                                 const uchar *key2,
-                                 size_t key_len2 [[maybe_unused]]) {
-  const PFS_account_key *account_key1;
-  const PFS_account_key *account_key2;
-  int cmp;
-
-  assert(key_len1 == sizeof(PFS_account_key));
-  assert(key_len2 == sizeof(PFS_account_key));
-  account_key1 = reinterpret_cast<const PFS_account_key *>(key1);
-  account_key2 = reinterpret_cast<const PFS_account_key *>(key2);
-  assert(account_key1 != nullptr);
-  assert(account_key2 != nullptr);
-
-  cmp = account_key1->m_user_name.sort(&account_key2->m_user_name);
-  if (cmp != 0) {
-    return cmp;
-  }
-  cmp = account_key1->m_host_name.sort(&account_key2->m_host_name);
-  return cmp;
-}
-
 /**
   Initialize the user hash.
   @return 0 on success
 */
 int init_account_hash(const PFS_global_param *param) {
   if ((!account_hash_inited) && (param->m_account_sizing != 0)) {
-    lf_hash_init3(&account_hash, sizeof(PFS_account *), LF_HASH_UNIQUE,
-                  account_hash_get_key, account_hash_func,
-                  account_hash_cmp_func, nullptr /* ctor */, nullptr /* dtor */,
-                  nullptr /* init */);
+    lf_hash_init(&account_hash, sizeof(PFS_account *), LF_HASH_UNIQUE, 0, 0,
+                 account_hash_get_key, &my_charset_bin);
     account_hash_inited = true;
   }
   return 0;
 }
 
 /** Cleanup the user hash. */
-void cleanup_account_hash() {
+void cleanup_account_hash(void) {
   if (account_hash_inited) {
     lf_hash_destroy(&account_hash);
     account_hash_inited = false;
@@ -155,15 +110,31 @@ static LF_PINS *get_account_hash_pins(PFS_thread *thread) {
   return thread->m_account_hash_pins;
 }
 
-static void set_account_key(PFS_account_key *key, const PFS_user_name *user,
-                            const PFS_host_name *host) {
-  key->m_user_name = *user;
-  key->m_host_name = *host;
+static void set_account_key(PFS_account_key *key, const char *user,
+                            uint user_length, const char *host,
+                            uint host_length) {
+  DBUG_ASSERT(user_length <= USERNAME_LENGTH);
+  DBUG_ASSERT(host_length <= HOSTNAME_LENGTH);
+
+  char *ptr = &key->m_hash_key[0];
+  if (user_length > 0) {
+    memcpy(ptr, user, user_length);
+    ptr += user_length;
+  }
+  ptr[0] = 0;
+  ptr++;
+  if (host_length > 0) {
+    memcpy(ptr, host, host_length);
+    ptr += host_length;
+  }
+  ptr[0] = 0;
+  ptr++;
+  key->m_key_length = ptr - &key->m_hash_key[0];
 }
 
-PFS_account *find_or_create_account(PFS_thread *thread,
-                                    const PFS_user_name *user,
-                                    const PFS_host_name *host) {
+PFS_account *find_or_create_account(PFS_thread *thread, const char *username,
+                                    uint username_length, const char *hostname,
+                                    uint hostname_length) {
   LF_PINS *pins = get_account_hash_pins(thread);
   if (unlikely(pins == nullptr)) {
     global_account_container.m_lost++;
@@ -171,7 +142,7 @@ PFS_account *find_or_create_account(PFS_thread *thread,
   }
 
   PFS_account_key key;
-  set_account_key(&key, user, host);
+  set_account_key(&key, username, username_length, hostname, hostname_length);
 
   PFS_account **entry;
   PFS_account *pfs;
@@ -181,7 +152,7 @@ PFS_account *find_or_create_account(PFS_thread *thread,
 
 search:
   entry = reinterpret_cast<PFS_account **>(
-      lf_hash_search(&account_hash, pins, &key, sizeof(key)));
+      lf_hash_search(&account_hash, pins, key.m_hash_key, key.m_key_length));
   if (entry && (entry != MY_LF_ERRPTR)) {
     pfs = *entry;
     pfs->inc_refcount();
@@ -194,26 +165,39 @@ search:
   pfs = global_account_container.allocate(&dirty_state);
   if (pfs != nullptr) {
     pfs->m_key = key;
+    if (username_length > 0) {
+      pfs->m_username = &pfs->m_key.m_hash_key[0];
+    } else {
+      pfs->m_username = nullptr;
+    }
+    pfs->m_username_length = username_length;
 
-    pfs->m_user = find_or_create_user(thread, &key.m_user_name);
-    pfs->m_host = find_or_create_host(thread, &key.m_host_name);
+    if (hostname_length > 0) {
+      pfs->m_hostname = &pfs->m_key.m_hash_key[username_length + 1];
+    } else {
+      pfs->m_hostname = nullptr;
+    }
+    pfs->m_hostname_length = hostname_length;
+
+    pfs->m_user = find_or_create_user(thread, username, username_length);
+    pfs->m_host = find_or_create_host(thread, hostname, hostname_length);
 
     pfs->init_refcount();
     pfs->reset_stats();
-    pfs->reset_connections_stats();
+    pfs->m_disconnected_count = 0;
 
-    if (user->length() > 0 && host->length() > 0) {
-      lookup_setup_actor(thread, &key.m_user_name, &key.m_host_name,
-                         &pfs->m_enabled, &pfs->m_history);
+    if (username_length > 0 && hostname_length > 0) {
+      lookup_setup_actor(thread, username, username_length, hostname,
+                         hostname_length, &pfs->m_enabled, &pfs->m_history);
     } else {
       pfs->m_enabled = true;
       pfs->m_history = true;
     }
 
     int res;
+    pfs->m_lock.dirty_to_allocated(&dirty_state);
     res = lf_hash_insert(&account_hash, pins, &pfs);
     if (likely(res == 0)) {
-      pfs->m_lock.dirty_to_allocated(&dirty_state);
       return pfs;
     }
 
@@ -226,7 +210,7 @@ search:
       pfs->m_host = nullptr;
     }
 
-    global_account_container.dirty_to_free(&dirty_state, pfs);
+    global_account_container.deallocate(pfs);
 
     if (res > 0) {
       if (++retry_count > retry_max) {
@@ -295,6 +279,7 @@ void PFS_account::aggregate_waits(PFS_user *safe_user, PFS_host *safe_host) {
 
   /* Orphan account, no parent to aggregate to. */
   reset_waits_stats();
+  return;
 }
 
 void PFS_account::aggregate_stages(PFS_user *safe_user, PFS_host *safe_host) {
@@ -344,6 +329,7 @@ void PFS_account::aggregate_stages(PFS_user *safe_user, PFS_host *safe_host) {
   */
   aggregate_all_stages(write_instr_class_stages_stats(),
                        global_instr_class_stages_array);
+  return;
 }
 
 void PFS_account::aggregate_statements(PFS_user *safe_user,
@@ -394,6 +380,7 @@ void PFS_account::aggregate_statements(PFS_user *safe_user,
   */
   aggregate_all_statements(write_instr_class_statements_stats(),
                            global_instr_class_statements_array);
+  return;
 }
 
 void PFS_account::aggregate_transactions(PFS_user *safe_user,
@@ -447,6 +434,7 @@ void PFS_account::aggregate_transactions(PFS_user *safe_user,
   */
   aggregate_all_transactions(write_instr_class_transactions_stats(),
                              &global_transaction_stat);
+  return;
 }
 
 void PFS_account::aggregate_errors(PFS_user *safe_user, PFS_host *safe_host) {
@@ -495,6 +483,7 @@ void PFS_account::aggregate_errors(PFS_user *safe_user, PFS_host *safe_host) {
     -  EVENTS_ERRORS_SUMMARY_GLOBAL_BY_ERROR
   */
   aggregate_all_errors(write_instr_class_errors_stats(), &global_error_stat);
+  return;
 }
 
 void PFS_account::aggregate_memory(bool alive, PFS_user *safe_user,
@@ -545,20 +534,32 @@ void PFS_account::aggregate_memory(bool alive, PFS_user *safe_user,
   */
   aggregate_all_memory(alive, write_instr_class_memory_stats(),
                        global_instr_class_memory_array);
+  return;
 }
 
 void PFS_account::aggregate_status(PFS_user *safe_user, PFS_host *safe_host) {
-  /*
-    Never aggregate to global_status_var,
-    because of the parallel THD -> global_status_var flow.
-  */
+  if (likely(safe_user != nullptr && safe_host != nullptr)) {
+    /*
+      Aggregate STATUS_BY_ACCOUNT to:
+      - STATUS_BY_USER
+      - STATUS_BY_HOST
+    */
+    safe_user->m_status_stats.aggregate(&m_status_stats);
+    safe_host->m_status_stats.aggregate(&m_status_stats);
+    m_status_stats.reset();
+    return;
+  }
 
   if (safe_user != nullptr) {
     /*
       Aggregate STATUS_BY_ACCOUNT to:
       - STATUS_BY_USER
+      - GLOBAL_STATUS
     */
     safe_user->m_status_stats.aggregate(&m_status_stats);
+    m_status_stats.aggregate_to(&global_status_var);
+    m_status_stats.reset();
+    return;
   }
 
   if (safe_host != nullptr) {
@@ -567,34 +568,41 @@ void PFS_account::aggregate_status(PFS_user *safe_user, PFS_host *safe_host) {
       - STATUS_BY_HOST
     */
     safe_host->m_status_stats.aggregate(&m_status_stats);
+    m_status_stats.reset();
+    return;
   }
 
+  /*
+    Aggregate STATUS_BY_ACCOUNT to:
+    - GLOBAL_STATUS
+  */
+  m_status_stats.aggregate_to(&global_status_var);
   m_status_stats.reset();
+  return;
 }
 
 void PFS_account::aggregate_stats(PFS_user *safe_user, PFS_host *safe_host) {
+  if (likely(safe_user != nullptr && safe_host != nullptr)) {
+    safe_user->m_disconnected_count += m_disconnected_count;
+    safe_host->m_disconnected_count += m_disconnected_count;
+    m_disconnected_count = 0;
+    return;
+  }
+
   if (safe_user != nullptr) {
-    safe_user->aggregate_stats_from(this);
+    safe_user->m_disconnected_count += m_disconnected_count;
+    m_disconnected_count = 0;
+    return;
   }
 
   if (safe_host != nullptr) {
-    safe_host->aggregate_stats_from(this);
+    safe_host->m_disconnected_count += m_disconnected_count;
+    m_disconnected_count = 0;
+    return;
   }
 
-  reset_connections_stats();
-}
-
-void PFS_account::aggregate_disconnect(ulonglong controlled_memory,
-                                       ulonglong total_memory) {
-  m_disconnected_count++;
-
-  if (m_max_controlled_memory < controlled_memory) {
-    m_max_controlled_memory = controlled_memory;
-  }
-
-  if (m_max_total_memory < total_memory) {
-    m_max_total_memory = total_memory;
-  }
+  m_disconnected_count = 0;
+  return;
 }
 
 void PFS_account::release() { dec_refcount(); }
@@ -607,60 +615,32 @@ void PFS_account::rebase_memory_stats() {
   }
 }
 
-void PFS_account::carry_memory_stat_alloc_delta(
-    PFS_memory_stat_alloc_delta *delta, uint index) {
+void PFS_account::carry_memory_stat_delta(PFS_memory_stat_delta *delta,
+                                          uint index) {
   PFS_memory_shared_stat *event_name_array;
   PFS_memory_shared_stat *stat;
-  PFS_memory_stat_alloc_delta delta_buffer;
-  PFS_memory_stat_alloc_delta *remaining_delta;
+  PFS_memory_stat_delta delta_buffer;
+  PFS_memory_stat_delta *remaining_delta;
 
   event_name_array = write_instr_class_memory_stats();
   stat = &event_name_array[index];
-  remaining_delta = stat->apply_alloc_delta(delta, &delta_buffer);
+  remaining_delta = stat->apply_delta(delta, &delta_buffer);
 
   if (remaining_delta == nullptr) {
     return;
   }
 
   if (m_user != nullptr) {
-    m_user->carry_memory_stat_alloc_delta(remaining_delta, index);
+    m_user->carry_memory_stat_delta(remaining_delta, index);
     /* do not return, need to process m_host below */
   }
 
   if (m_host != nullptr) {
-    m_host->carry_memory_stat_alloc_delta(remaining_delta, index);
+    m_host->carry_memory_stat_delta(remaining_delta, index);
     return;
   }
 
-  carry_global_memory_stat_alloc_delta(remaining_delta, index);
-}
-
-void PFS_account::carry_memory_stat_free_delta(
-    PFS_memory_stat_free_delta *delta, uint index) {
-  PFS_memory_shared_stat *event_name_array;
-  PFS_memory_shared_stat *stat;
-  PFS_memory_stat_free_delta delta_buffer;
-  PFS_memory_stat_free_delta *remaining_delta;
-
-  event_name_array = write_instr_class_memory_stats();
-  stat = &event_name_array[index];
-  remaining_delta = stat->apply_free_delta(delta, &delta_buffer);
-
-  if (remaining_delta == nullptr) {
-    return;
-  }
-
-  if (m_user != nullptr) {
-    m_user->carry_memory_stat_free_delta(remaining_delta, index);
-    /* do not return, need to process m_host below */
-  }
-
-  if (m_host != nullptr) {
-    m_host->carry_memory_stat_free_delta(remaining_delta, index);
-    return;
-  }
-
-  carry_global_memory_stat_free_delta(remaining_delta, index);
+  carry_global_memory_stat_delta(remaining_delta, index);
 }
 
 PFS_account *sanitize_account(PFS_account *unsafe) {
@@ -674,13 +654,14 @@ static void purge_account(PFS_thread *thread, PFS_account *account) {
   }
 
   PFS_account **entry;
-  entry = reinterpret_cast<PFS_account **>(lf_hash_search(
-      &account_hash, pins, &account->m_key, sizeof(account->m_key)));
+  entry = reinterpret_cast<PFS_account **>(
+      lf_hash_search(&account_hash, pins, account->m_key.m_hash_key,
+                     account->m_key.m_key_length));
   if (entry && (entry != MY_LF_ERRPTR)) {
-    assert(*entry == account);
+    DBUG_ASSERT(*entry == account);
     if (account->get_refcount() == 0) {
-      lf_hash_delete(&account_hash, pins, &account->m_key,
-                     sizeof(account->m_key));
+      lf_hash_delete(&account_hash, pins, account->m_key.m_hash_key,
+                     account->m_key.m_key_length);
       account->aggregate(false, account->m_user, account->m_host);
       if (account->m_user != nullptr) {
         account->m_user->release();
@@ -699,9 +680,9 @@ static void purge_account(PFS_thread *thread, PFS_account *account) {
 
 class Proc_purge_account : public PFS_buffer_processor<PFS_account> {
  public:
-  explicit Proc_purge_account(PFS_thread *thread) : m_thread(thread) {}
+  Proc_purge_account(PFS_thread *thread) : m_thread(thread) {}
 
-  void operator()(PFS_account *pfs) override {
+  virtual void operator()(PFS_account *pfs) {
     PFS_user *user = sanitize_user(pfs->m_user);
     PFS_host *host = sanitize_host(pfs->m_host);
     pfs->aggregate(true, user, host);
@@ -716,7 +697,7 @@ class Proc_purge_account : public PFS_buffer_processor<PFS_account> {
 };
 
 /** Purge non connected accounts, reset stats of connected account. */
-void purge_all_account() {
+void purge_all_account(void) {
   PFS_thread *thread = PFS_thread::get_current_thread();
   if (unlikely(thread == nullptr)) {
     return;
@@ -729,15 +710,13 @@ void purge_all_account() {
 class Proc_update_accounts_derived_flags
     : public PFS_buffer_processor<PFS_account> {
  public:
-  explicit Proc_update_accounts_derived_flags(PFS_thread *thread)
-      : m_thread(thread) {}
+  Proc_update_accounts_derived_flags(PFS_thread *thread) : m_thread(thread) {}
 
-  void operator()(PFS_account *pfs) override {
-    if (pfs->m_key.m_user_name.length() > 0 &&
-        pfs->m_key.m_host_name.length() > 0) {
-      lookup_setup_actor(m_thread, &pfs->m_key.m_user_name,
-                         &pfs->m_key.m_host_name, &pfs->m_enabled,
-                         &pfs->m_history);
+  virtual void operator()(PFS_account *pfs) {
+    if (pfs->m_username_length > 0 && pfs->m_hostname_length > 0) {
+      lookup_setup_actor(m_thread, pfs->m_username, pfs->m_username_length,
+                         pfs->m_hostname, pfs->m_hostname_length,
+                         &pfs->m_enabled, &pfs->m_history);
     } else {
       pfs->m_enabled = true;
       pfs->m_history = true;
