@@ -1,4 +1,4 @@
-/* Copyright (c) 2008, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2008, 2020, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -30,17 +30,16 @@
 #include <string.h>
 #include <algorithm>
 
+#include "m_ctype.h"
 #include "m_string.h"
 #include "my_bitmap.h"
 #include "my_byteorder.h"
 #include "my_dbug.h"
 #include "my_macros.h"
 #include "my_sqlcommand.h"
-#include "my_time.h"
 #include "myisampack.h"
-#include "mysql/components/services/bits/psi_bits.h"
-#include "mysql/components/services/bits/psi_mutex_bits.h"
-#include "mysql/strings/m_ctype.h"
+#include "mysql/components/services/psi_mutex_bits.h"
+#include "mysql/psi/psi_base.h"
 #include "sql/auth/auth_acls.h"
 #include "sql/current_thd.h"
 #include "sql/field.h"
@@ -62,7 +61,6 @@
 #include "storage/perfschema/table_ees_by_thread_by_error.h"
 #include "storage/perfschema/table_ees_by_user_by_error.h"
 #include "storage/perfschema/table_ees_global_by_error.h"
-#include "storage/perfschema/table_error_log.h"
 #include "storage/perfschema/table_esgs_by_account_by_event_name.h"
 #include "storage/perfschema/table_esgs_by_host_by_event_name.h"
 #include "storage/perfschema/table_esgs_by_thread_by_event_name.h"
@@ -99,7 +97,6 @@
 #include "storage/perfschema/table_global_variables.h"
 #include "storage/perfschema/table_host_cache.h"
 #include "storage/perfschema/table_hosts.h"
-#include "storage/perfschema/table_keyring_component_status.h"
 #include "storage/perfschema/table_keyring_keys.h"
 #include "storage/perfschema/table_md_locks.h"
 #include "storage/perfschema/table_mems_by_account_by_event_name.h"
@@ -111,7 +108,6 @@
 #include "storage/perfschema/table_performance_timers.h"
 #include "storage/perfschema/table_persisted_variables.h"
 #include "storage/perfschema/table_prepared_stmt_instances.h"
-#include "storage/perfschema/table_processlist.h"
 #include "storage/perfschema/table_replication_applier_configuration.h"
 #include "storage/perfschema/table_replication_applier_filters.h"
 #include "storage/perfschema/table_replication_applier_global_filters.h"
@@ -120,12 +116,10 @@
 #include "storage/perfschema/table_replication_applier_status_by_worker.h"
 /* For replication related perfschema tables. */
 #include "storage/perfschema/table_log_status.h"
-#include "storage/perfschema/table_replication_asynchronous_connection_failover.h"
 #include "storage/perfschema/table_replication_connection_configuration.h"
 #include "storage/perfschema/table_replication_connection_status.h"
 #include "storage/perfschema/table_replication_group_member_stats.h"
 #include "storage/perfschema/table_replication_group_members.h"
-#include "storage/perfschema/table_rpl_async_connection_failover_managed.h"
 #include "storage/perfschema/table_session_account_connect_attrs.h"
 #include "storage/perfschema/table_session_connect_attrs.h"
 #include "storage/perfschema/table_session_status.h"
@@ -133,8 +127,6 @@
 #include "storage/perfschema/table_setup_actors.h"
 #include "storage/perfschema/table_setup_consumers.h"
 #include "storage/perfschema/table_setup_instruments.h"
-#include "storage/perfschema/table_setup_meters.h"
-#include "storage/perfschema/table_setup_metrics.h"
 #include "storage/perfschema/table_setup_objects.h"
 #include "storage/perfschema/table_setup_threads.h"
 #include "storage/perfschema/table_socket_instances.h"
@@ -149,7 +141,6 @@
 #include "storage/perfschema/table_threads.h"
 #include "storage/perfschema/table_tiws_by_index_usage.h"
 #include "storage/perfschema/table_tiws_by_table.h"
-#include "storage/perfschema/table_tls_channel_status.h"
 #include "storage/perfschema/table_tlws_by_table.h"
 #include "storage/perfschema/table_user_defined_functions.h"
 #include "storage/perfschema/table_users.h"
@@ -480,9 +471,90 @@
   @{
 */
 
+bool PFS_table_context::initialize(void) {
+  if (m_restore) {
+    /* Restore context from TLS. */
+    PFS_table_context *context = THR_PFS_contexts[m_thr_key];
+    DBUG_ASSERT(context != nullptr);
+
+    if (context) {
+      m_last_version = context->m_current_version;
+      m_map = context->m_map;
+      DBUG_ASSERT(m_map_size == context->m_map_size);
+      m_map_size = context->m_map_size;
+      m_word_size = context->m_word_size;
+    }
+  } else {
+    /* Check that TLS is not in use. */
+    PFS_table_context *context = THR_PFS_contexts[m_thr_key];
+
+    context = this;
+
+    /* Initialize a new context, store in TLS. */
+    m_last_version = m_current_version;
+    m_map = nullptr;
+    m_word_size = sizeof(ulong) * 8;
+
+    /* Write to TLS. */
+    THR_PFS_contexts[m_thr_key] = context;
+  }
+
+  m_initialized = (m_map_size > 0) ? (m_map != nullptr) : true;
+
+  return m_initialized;
+}
+
+/* Constructor for global or single thread tables, map size = 0.  */
+PFS_table_context::PFS_table_context(ulonglong current_version, bool restore,
+                                     THR_PFS_key key)
+    : m_thr_key(key),
+      m_current_version(current_version),
+      m_last_version(0),
+      m_map(nullptr),
+      m_map_size(0),
+      m_word_size(sizeof(ulong)),
+      m_restore(restore),
+      m_initialized(false),
+      m_last_item(0) {
+  initialize();
+}
+
+/* Constructor for by-thread or aggregate tables, map size = max
+ * thread/user/host/account. */
+PFS_table_context::PFS_table_context(ulonglong current_version, ulong map_size,
+                                     bool restore, THR_PFS_key key)
+    : m_thr_key(key),
+      m_current_version(current_version),
+      m_last_version(0),
+      m_map(nullptr),
+      m_map_size(map_size),
+      m_word_size(sizeof(ulong)),
+      m_restore(restore),
+      m_initialized(false),
+      m_last_item(0) {
+  initialize();
+}
+
+PFS_table_context::~PFS_table_context(void) {}
+
+void PFS_table_context::set_item(ulong n) {
+  if (n == m_last_item) {
+    return;
+  }
+  ulong word = n / m_word_size;
+  ulong bit = n % m_word_size;
+  m_map[word] |= (1UL << bit);
+  m_last_item = n;
+}
+
+bool PFS_table_context::is_item_set(ulong n) {
+  ulong word = n / m_word_size;
+  ulong bit = n % m_word_size;
+  return (m_map[word] & (1UL << bit));
+}
+
 static PFS_engine_table_share *all_shares[] = {
     &table_cond_instances::m_share,
-    &table_error_log::m_share,
     &table_events_waits_current::m_share,
     &table_events_waits_history::m_share,
     &table_events_waits_history_long::m_share,
@@ -499,13 +571,10 @@ static PFS_engine_table_share *all_shares[] = {
     &table_mutex_instances::m_share,
     &table_os_global_by_type::m_share,
     &table_performance_timers::m_share,
-    &table_processlist::m_share,
     &table_rwlock_instances::m_share,
     &table_setup_actors::m_share,
     &table_setup_consumers::m_share,
     &table_setup_instruments::m_share,
-    &table_setup_meters::m_share,
-    &table_setup_metrics::m_share,
     &table_setup_objects::m_share,
     &table_setup_threads::m_share,
     &table_tiws_by_index_usage::m_share,
@@ -583,8 +652,6 @@ static PFS_engine_table_share *all_shares[] = {
     &table_replication_group_member_stats::m_share,
     &table_replication_applier_filters::m_share,
     &table_replication_applier_global_filters::m_share,
-    &table_replication_asynchronous_connection_failover::m_share,
-    &table_rpl_async_connection_failover_managed::m_share,
     &table_log_status::m_share,
 
     &table_prepared_stmt_instances::m_share,
@@ -604,8 +671,6 @@ static PFS_engine_table_share *all_shares[] = {
     &table_persisted_variables::m_share,
     &table_user_defined_functions::m_share,
     &table_binary_log_transaction_compression_stats::m_share,
-    &table_tls_channel_status::m_share,
-    &table_keyring_component_status::m_share,
     nullptr};
 
 static PSI_mutex_key key_LOCK_pfs_share_list;
@@ -640,7 +705,7 @@ void PFS_engine_table_share::get_all_tables(List<const Plugin_table> *tables) {
 }
 
 /** Initialize all the table share locks. */
-void PFS_engine_table_share::init_all_locks() {
+void PFS_engine_table_share::init_all_locks(void) {
   PFS_engine_table_share **current;
 
   for (current = &all_shares[0]; (*current) != nullptr; current++) {
@@ -649,7 +714,7 @@ void PFS_engine_table_share::init_all_locks() {
 }
 
 /** Delete all the table share locks. */
-void PFS_engine_table_share::delete_all_locks() {
+void PFS_engine_table_share::delete_all_locks(void) {
   PFS_engine_table_share **current;
 
   for (current = &all_shares[0]; (*current) != nullptr; current++) {
@@ -657,7 +722,7 @@ void PFS_engine_table_share::delete_all_locks() {
   }
 }
 
-ha_rows PFS_engine_table_share::get_row_count() const {
+ha_rows PFS_engine_table_share::get_row_count(void) const {
   return m_get_row_count();
 }
 
@@ -672,7 +737,7 @@ int PFS_engine_table_share::write_row(PFS_engine_table *pfs_table, TABLE *table,
 
   /* We internally read from Fields to support the write interface */
   org_bitmap = dbug_tmp_use_all_columns(table, table->read_set);
-  const int result = m_write_row(pfs_table, table, buf, fields);
+  int result = m_write_row(pfs_table, table, buf, fields);
   dbug_tmp_restore_column_map(table->read_set, org_bitmap);
 
   return result;
@@ -741,7 +806,7 @@ int PFS_engine_table::read_row(TABLE *table, unsigned char *buf,
   Field **fields_reset;
 
   /* We must read all columns in case a table is opened for update */
-  const bool read_all = !bitmap_is_clear_all(table->write_set);
+  bool read_all = !bitmap_is_clear_all(table->write_set);
 
   /* We internally write to Fields to support the read interface */
   org_bitmap = dbug_tmp_use_all_columns(table, table->write_set);
@@ -756,7 +821,7 @@ int PFS_engine_table::read_row(TABLE *table, unsigned char *buf,
     f->reset();
   }
 
-  const int result = read_row_values(table, buf, fields, read_all);
+  int result = read_row_values(table, buf, fields, read_all);
   dbug_tmp_restore_column_map(table->write_set, org_bitmap);
 
   return result;
@@ -776,7 +841,7 @@ int PFS_engine_table::update_row(TABLE *table, const unsigned char *old_buf,
 
   /* We internally read from Fields to support the write interface */
   org_bitmap = dbug_tmp_use_all_columns(table, table->read_set);
-  const int result = update_row_values(table, old_buf, new_buf, fields);
+  int result = update_row_values(table, old_buf, new_buf, fields);
   dbug_tmp_restore_column_map(table->read_set, org_bitmap);
 
   return result;
@@ -788,7 +853,7 @@ int PFS_engine_table::delete_row(TABLE *table, const unsigned char *buf,
 
   /* We internally read from Fields to support the delete interface */
   org_bitmap = dbug_tmp_use_all_columns(table, table->read_set);
-  const int result = delete_row_values(table, buf, fields);
+  int result = delete_row_values(table, buf, fields);
   dbug_tmp_restore_column_map(table->read_set, org_bitmap);
 
   return result;
@@ -828,22 +893,21 @@ int PFS_engine_table::update_row_values(TABLE *, const unsigned char *,
 int PFS_engine_table::index_read(KEY *key_infos, uint index, const uchar *key,
                                  uint key_len,
                                  enum ha_rkey_function find_flag) {
-  // assert(m_index != NULL);
+  // DBUG_ASSERT(m_index != NULL);
   if (m_index == nullptr) {
     return HA_ERR_END_OF_FILE;
   }
 
   // FIXME: Unclear what to do here
-  assert(find_flag != HA_READ_PREFIX_LAST);
-  assert(find_flag != HA_READ_PREFIX_LAST_OR_PREV);
+  DBUG_ASSERT(find_flag != HA_READ_PREFIX_LAST);
+  DBUG_ASSERT(find_flag != HA_READ_PREFIX_LAST_OR_PREV);
 
   // No GIS here
-  assert(find_flag != HA_READ_MBR_CONTAIN);
-  assert(find_flag != HA_READ_MBR_INTERSECT);
-  assert(find_flag != HA_READ_MBR_WITHIN);
-  assert(find_flag != HA_READ_MBR_DISJOINT);
-  assert(find_flag != HA_READ_MBR_EQUAL);
-  assert(find_flag != HA_READ_NEAREST_NEIGHBOR);
+  DBUG_ASSERT(find_flag != HA_READ_MBR_CONTAIN);
+  DBUG_ASSERT(find_flag != HA_READ_MBR_INTERSECT);
+  DBUG_ASSERT(find_flag != HA_READ_MBR_WITHIN);
+  DBUG_ASSERT(find_flag != HA_READ_MBR_DISJOINT);
+  DBUG_ASSERT(find_flag != HA_READ_MBR_EQUAL);
 
   KEY *key_info = key_infos + index;
   m_index->set_key_info(key_info);
@@ -873,9 +937,9 @@ PFS_engine_table_share *PFS_dynamic_table_shares::find_share(
     mysql_mutex_assert_owner(&LOCK_pfs_share_list);
   }
 
-  for (auto *it : shares_vector) {
+  for (auto it : shares_vector) {
     if ((compare_table_names(table_name, it->m_table_def->get_name()) == 0) &&
-        (!it->m_in_purgatory || is_dead_too)) {
+        (it->m_in_purgatory == false || is_dead_too)) {
       return it;
     }
   }
@@ -902,14 +966,13 @@ void PFS_dynamic_table_shares::remove_share(PFS_engine_table_share *share) {
 /** Implementation of internal ACL checks, for the performance schema. */
 class PFS_internal_schema_access : public ACL_internal_schema_access {
  public:
-  PFS_internal_schema_access() = default;
+  PFS_internal_schema_access() {}
 
-  ~PFS_internal_schema_access() override = default;
+  ~PFS_internal_schema_access() {}
 
-  ACL_internal_access_result check(ulong want_access, ulong *save_priv,
-                                   bool any_combination_will_do) const override;
+  ACL_internal_access_result check(ulong want_access, ulong *save_priv) const;
 
-  const ACL_internal_table_access *lookup(const char *name) const override;
+  const ACL_internal_table_access *lookup(const char *name) const;
 };
 
 static bool allow_drop_schema_privilege() {
@@ -918,18 +981,16 @@ static bool allow_drop_schema_privilege() {
     in particular, as a schema level privilege:
     - DROP SCHEMA
     - GRANT DROP on performance_schema.*
-    - REVOKE DROP ON performance_schema.*
     - DROP TABLE performance_schema.*
 
     As a table level privilege:
     - DROP TABLE performance_schema.foo
     - GRANT DROP on performance_schema.foo
-    - REVOKE DROP on performance_schema.foo
     - TRUNCATE TABLE performance_schema.foo
 
     Here, we want to:
     - always prevent DROP SCHEMA (SQLCOM_DROP_DB)
-    - allow GRANT/REVOKE to give the TRUNCATE on any tables
+    - allow GRANT to give the TRUNCATE on any tables
     - allow DROP TABLE checks to proceed further,
       in particular to drop unknown tables,
       see PFS_unknown_acl::check()
@@ -939,16 +1000,18 @@ static bool allow_drop_schema_privilege() {
     return false;
   }
 
-  assert(thd->lex != nullptr);
-  return ((thd->lex->sql_command == SQLCOM_TRUNCATE) ||
-          (thd->lex->sql_command == SQLCOM_GRANT) ||
-          (thd->lex->sql_command == SQLCOM_REVOKE) ||
-          (thd->lex->sql_command == SQLCOM_DROP_TABLE));
+  DBUG_ASSERT(thd->lex != nullptr);
+  if ((thd->lex->sql_command != SQLCOM_TRUNCATE) &&
+      (thd->lex->sql_command != SQLCOM_GRANT) &&
+      (thd->lex->sql_command != SQLCOM_DROP_TABLE)) {
+    return false;
+  }
+
+  return true;
 }
 
 ACL_internal_access_result PFS_internal_schema_access::check(ulong want_access,
-                                                             ulong *,
-                                                             bool) const {
+                                                             ulong *) const {
   const ulong always_forbidden =
       CREATE_ACL | REFERENCES_ACL | INDEX_ACL | ALTER_ACL | CREATE_TMP_ACL |
       EXECUTE_ACL | CREATE_VIEW_ACL | SHOW_VIEW_ACL | CREATE_PROC_ACL |
@@ -1022,37 +1085,25 @@ static bool allow_drop_table_privilege() {
     return false;
   }
 
-  assert(thd->lex != nullptr);
-  return ((thd->lex->sql_command == SQLCOM_TRUNCATE) ||
-          (thd->lex->sql_command == SQLCOM_GRANT));
+  DBUG_ASSERT(thd->lex != nullptr);
+  if ((thd->lex->sql_command != SQLCOM_TRUNCATE) &&
+      (thd->lex->sql_command != SQLCOM_GRANT)) {
+    return false;
+  }
+
+  return true;
 }
 
 PFS_readonly_acl pfs_readonly_acl;
 
-ACL_internal_access_result PFS_readonly_acl::check(
-    ulong want_access, ulong *granted_access,
-    bool any_combination_will_do) const {
+ACL_internal_access_result PFS_readonly_acl::check(ulong want_access,
+                                                   ulong *) const {
   const ulong always_forbidden = INSERT_ACL | UPDATE_ACL | DELETE_ACL |
                                  CREATE_ACL | DROP_ACL | REFERENCES_ACL |
                                  INDEX_ACL | ALTER_ACL | CREATE_VIEW_ACL |
                                  SHOW_VIEW_ACL | TRIGGER_ACL | LOCK_TABLES_ACL;
 
-  const ulong can_be_allowed = TABLE_ACLS & (~always_forbidden);
-
-  const ulong want_forbidden = want_access & always_forbidden;
-
-  const ulong want_allowable = want_access & can_be_allowed;
-
-  if (any_combination_will_do) {
-    if (want_allowable != 0) {
-      *granted_access = want_allowable;
-      return ACL_INTERNAL_ACCESS_CHECK_GRANT;
-    }
-
-    return ACL_INTERNAL_ACCESS_DENIED;
-  }
-
-  if (want_forbidden != 0) {
+  if (unlikely(want_access & always_forbidden)) {
     return ACL_INTERNAL_ACCESS_DENIED;
   }
 
@@ -1062,73 +1113,43 @@ ACL_internal_access_result PFS_readonly_acl::check(
 PFS_readonly_world_acl pfs_readonly_world_acl;
 
 ACL_internal_access_result PFS_readonly_world_acl::check(
-    ulong want_access, ulong *save_priv, bool any_combination_will_do) const {
+    ulong want_access, ulong *save_priv) const {
   ACL_internal_access_result res =
-      PFS_readonly_acl::check(want_access, save_priv, any_combination_will_do);
+      PFS_readonly_acl::check(want_access, save_priv);
   if (res == ACL_INTERNAL_ACCESS_CHECK_GRANT) {
     res = ACL_INTERNAL_ACCESS_GRANTED;
   }
   return res;
 }
 
-PFS_readonly_processlist_acl pfs_readonly_processlist_acl;
-
-ACL_internal_access_result PFS_readonly_processlist_acl::check(
-    ulong want_access, ulong *save_priv, bool any_combination_will_do) const {
-  const ACL_internal_access_result res =
-      PFS_readonly_acl::check(want_access, save_priv, any_combination_will_do);
-
-  if ((res == ACL_INTERNAL_ACCESS_CHECK_GRANT) && (want_access & SELECT_ACL)) {
-    return ACL_INTERNAL_ACCESS_GRANTED;
-  }
-  return res;
-}
-
 PFS_truncatable_acl pfs_truncatable_acl;
 
-ACL_internal_access_result PFS_truncatable_acl::check(
-    ulong want_access, ulong *granted_access,
-    bool any_combination_will_do) const {
+ACL_internal_access_result PFS_truncatable_acl::check(ulong want_access,
+                                                      ulong *) const {
   const ulong always_forbidden = INSERT_ACL | UPDATE_ACL | DELETE_ACL |
                                  CREATE_ACL | REFERENCES_ACL | INDEX_ACL |
                                  ALTER_ACL | CREATE_VIEW_ACL | SHOW_VIEW_ACL |
                                  TRIGGER_ACL | LOCK_TABLES_ACL;
 
-  const ulong can_be_allowed = TABLE_ACLS & (~always_forbidden);
-
-  ulong want_allowable = want_access & can_be_allowed;
-
-  ulong want_forbidden = want_access & always_forbidden;
+  if (unlikely(want_access & always_forbidden)) {
+    return ACL_INTERNAL_ACCESS_DENIED;
+  }
 
   if (want_access & DROP_ACL) {
     if (!allow_drop_table_privilege()) {
-      want_forbidden |= DROP_ACL;
-      want_allowable &= ~DROP_ACL;
+      return ACL_INTERNAL_ACCESS_DENIED;
     }
   }
 
-  if (any_combination_will_do) {
-    if (want_allowable != 0) {
-      *granted_access = want_allowable;
-      return ACL_INTERNAL_ACCESS_CHECK_GRANT;
-    }
-
-    return ACL_INTERNAL_ACCESS_DENIED;
-  }
-
-  if (want_forbidden != 0) {
-    return ACL_INTERNAL_ACCESS_DENIED;
-  }
-
   return ACL_INTERNAL_ACCESS_CHECK_GRANT;
 }
 
 PFS_truncatable_world_acl pfs_truncatable_world_acl;
 
 ACL_internal_access_result PFS_truncatable_world_acl::check(
-    ulong want_access, ulong *save_priv, bool any_combination_will_do) const {
-  ACL_internal_access_result res = PFS_truncatable_acl::check(
-      want_access, save_priv, any_combination_will_do);
+    ulong want_access, ulong *save_priv) const {
+  ACL_internal_access_result res =
+      PFS_truncatable_acl::check(want_access, save_priv);
   if (res == ACL_INTERNAL_ACCESS_CHECK_GRANT) {
     res = ACL_INTERNAL_ACCESS_GRANTED;
   }
@@ -1137,29 +1158,13 @@ ACL_internal_access_result PFS_truncatable_world_acl::check(
 
 PFS_updatable_acl pfs_updatable_acl;
 
-ACL_internal_access_result PFS_updatable_acl::check(
-    ulong want_access, ulong *granted_access,
-    bool any_combination_will_do) const {
+ACL_internal_access_result PFS_updatable_acl::check(ulong want_access,
+                                                    ulong *) const {
   const ulong always_forbidden =
       INSERT_ACL | DELETE_ACL | CREATE_ACL | DROP_ACL | REFERENCES_ACL |
       INDEX_ACL | ALTER_ACL | CREATE_VIEW_ACL | SHOW_VIEW_ACL | TRIGGER_ACL;
 
-  const ulong can_be_allowed = TABLE_ACLS & (~always_forbidden);
-
-  const ulong want_forbidden = want_access & always_forbidden;
-
-  const ulong want_allowable = want_access & can_be_allowed;
-
-  if (any_combination_will_do) {
-    if (want_allowable != 0) {
-      *granted_access = want_allowable;
-      return ACL_INTERNAL_ACCESS_CHECK_GRANT;
-    }
-
-    return ACL_INTERNAL_ACCESS_DENIED;
-  }
-
-  if (want_forbidden != 0) {
+  if (unlikely(want_access & always_forbidden)) {
     return ACL_INTERNAL_ACCESS_DENIED;
   }
 
@@ -1168,72 +1173,35 @@ ACL_internal_access_result PFS_updatable_acl::check(
 
 PFS_editable_acl pfs_editable_acl;
 
-ACL_internal_access_result PFS_editable_acl::check(
-    ulong want_access, ulong *granted_access,
-    bool any_combination_will_do) const {
+ACL_internal_access_result PFS_editable_acl::check(ulong want_access,
+                                                   ulong *) const {
   const ulong always_forbidden = CREATE_ACL | REFERENCES_ACL | INDEX_ACL |
                                  ALTER_ACL | CREATE_VIEW_ACL | SHOW_VIEW_ACL |
                                  TRIGGER_ACL;
 
-  const ulong can_be_allowed = TABLE_ACLS & (~always_forbidden);
-
-  ulong want_forbidden = want_access & always_forbidden;
-
-  ulong want_allowable = want_access & can_be_allowed;
+  if (unlikely(want_access & always_forbidden)) {
+    return ACL_INTERNAL_ACCESS_DENIED;
+  }
 
   if (want_access & DROP_ACL) {
     if (!allow_drop_table_privilege()) {
-      want_forbidden |= DROP_ACL;
-      want_allowable &= ~DROP_ACL;
+      return ACL_INTERNAL_ACCESS_DENIED;
     }
   }
 
-  if (any_combination_will_do) {
-    if (want_allowable != 0) {
-      *granted_access = want_allowable;
-      return ACL_INTERNAL_ACCESS_CHECK_GRANT;
-    }
-
-    return ACL_INTERNAL_ACCESS_DENIED;
-  }
-
-  if (want_forbidden != 0) {
-    return ACL_INTERNAL_ACCESS_DENIED;
-  }
-
   return ACL_INTERNAL_ACCESS_CHECK_GRANT;
 }
 
 PFS_unknown_acl pfs_unknown_acl;
 
-ACL_internal_access_result PFS_unknown_acl::check(
-    ulong want_access, ulong *granted_access,
-    bool any_combination_will_do) const {
-  /*
-    Only enforce ACL_INTERNAL_ACCESS_DENIED
-    for operations that can create unwanted SQL objects
-    in the performance schema,
-    relax error messages otherwise.
-  */
+ACL_internal_access_result PFS_unknown_acl::check(ulong want_access,
+                                                  ulong *) const {
   const ulong always_forbidden = CREATE_ACL | REFERENCES_ACL | INDEX_ACL |
-                                 ALTER_ACL | CREATE_VIEW_ACL | TRIGGER_ACL;
+                                 ALTER_ACL | CREATE_VIEW_ACL | TRIGGER_ACL |
+                                 INSERT_ACL | UPDATE_ACL | DELETE_ACL |
+                                 SHOW_VIEW_ACL | LOCK_TABLES_ACL;
 
-  const ulong can_be_allowed = TABLE_ACLS & (~always_forbidden);
-
-  const ulong want_forbidden = want_access & always_forbidden;
-
-  const ulong want_allowable = want_access & can_be_allowed;
-
-  if (any_combination_will_do) {
-    if (want_allowable != 0) {
-      *granted_access = want_allowable;
-      return ACL_INTERNAL_ACCESS_CHECK_GRANT;
-    }
-
-    return ACL_INTERNAL_ACCESS_DENIED;
-  }
-
-  if (want_forbidden != 0) {
+  if (unlikely(want_access & always_forbidden)) {
     return ACL_INTERNAL_ACCESS_DENIED;
   }
 
@@ -1264,9 +1232,9 @@ ACL_internal_access_result PFS_unknown_acl::check(
 */
 #define READ_INT_COMMON(DS, KT, DT, KORR)                                \
   if (m_remaining_key_part_info->store_length <= m_remaining_key_len) {  \
-    const size_t data_size = DS;                                         \
-    assert(m_remaining_key_part_info->type == KT);                       \
-    assert(m_remaining_key_part_info->store_length >= data_size);        \
+    size_t data_size = DS;                                               \
+    DBUG_ASSERT(m_remaining_key_part_info->type == KT);                  \
+    DBUG_ASSERT(m_remaining_key_part_info->store_length >= data_size);   \
     isnull = false;                                                      \
     if (m_remaining_key_part_info->field->is_nullable()) {               \
       if (m_remaining_key[0]) {                                          \
@@ -1275,7 +1243,7 @@ ACL_internal_access_result PFS_unknown_acl::check(
       m_remaining_key += HA_KEY_NULL_LENGTH;                             \
       m_remaining_key_len -= HA_KEY_NULL_LENGTH;                         \
     }                                                                    \
-    const DT data = KORR(m_remaining_key);                               \
+    DT data = KORR(m_remaining_key);                                     \
     m_remaining_key += data_size;                                        \
     m_remaining_key_len -= (uint)data_size;                              \
     m_parts_found++;                                                     \
@@ -1283,7 +1251,7 @@ ACL_internal_access_result PFS_unknown_acl::check(
     *value = data;                                                       \
     return ((m_remaining_key_len == 0) ? find_flag : HA_READ_KEY_EXACT); \
   }                                                                      \
-  assert(m_remaining_key_len == 0);                                      \
+  DBUG_ASSERT(m_remaining_key_len == 0);                                 \
   return HA_READ_INVALID
 
 enum ha_rkey_function PFS_key_reader::read_int8(enum ha_rkey_function find_flag,
@@ -1336,39 +1304,9 @@ enum ha_rkey_function PFS_key_reader::read_ulonglong(
   READ_INT_COMMON(8, HA_KEYTYPE_ULONGLONG, unsigned long long, uint8korr);
 }
 
-enum ha_rkey_function PFS_key_reader::read_timestamp(
-    enum ha_rkey_function find_flag, bool &isnull, ulonglong *value, uint dec) {
-  const size_t data_size = 4 + ((size_t)((dec + 1) / 2));
-  my_timeval tm;
-
-  if (m_remaining_key_part_info->store_length <= m_remaining_key_len) {
-    assert(m_remaining_key_part_info->type == HA_KEYTYPE_BINARY);
-    assert(m_remaining_key_part_info->store_length >= data_size);
-    isnull = false;
-    if (m_remaining_key_part_info->field->is_nullable()) {
-      if (m_remaining_key[0]) {
-        isnull = true;
-      }
-      m_remaining_key += HA_KEY_NULL_LENGTH;
-      m_remaining_key_len -= HA_KEY_NULL_LENGTH;
-    }
-    my_timestamp_from_binary(&tm, m_remaining_key, dec);
-    const ulonglong data =
-        static_cast<ulonglong>(tm.m_tv_sec) * 1000000ULL + tm.m_tv_usec;
-    m_remaining_key += data_size;
-    m_remaining_key_len -= (uint)data_size;
-    m_parts_found++;
-    m_remaining_key_part_info++;
-    *value = data;
-    return ((m_remaining_key_len == 0) ? find_flag : HA_READ_KEY_EXACT);
-  }
-  assert(m_remaining_key_len == 0);
-  return HA_READ_INVALID;
-}
-
 enum ha_rkey_function PFS_key_reader::read_varchar_utf8(
     enum ha_rkey_function find_flag, bool &isnull, char *buffer,
-    uint *buffer_length, uint buffer_capacity [[maybe_unused]]) {
+    uint *buffer_length, uint buffer_capacity) {
   if (m_remaining_key_part_info->store_length <= m_remaining_key_len) {
     /*
       Stored as:
@@ -1382,7 +1320,7 @@ enum ha_rkey_function PFS_key_reader::read_varchar_utf8(
     size_t data_offset = 2;
     isnull = false;
     if (m_remaining_key_part_info->field->is_nullable()) {
-      assert(HA_KEY_NULL_LENGTH <= m_remaining_key_len);
+      DBUG_ASSERT(HA_KEY_NULL_LENGTH <= m_remaining_key_len);
 
       length_offset++;
       data_offset++;
@@ -1391,19 +1329,24 @@ enum ha_rkey_function PFS_key_reader::read_varchar_utf8(
       }
     }
 
-    assert(m_remaining_key_part_info->type == HA_KEYTYPE_VARTEXT1 ||
-           m_remaining_key_part_info->type == HA_KEYTYPE_VARTEXT2);
+    DBUG_ASSERT(m_remaining_key_part_info->type == HA_KEYTYPE_VARTEXT1 ||
+                m_remaining_key_part_info->type == HA_KEYTYPE_VARTEXT2);
 
-    assert(data_offset <= m_remaining_key_len);
-    const size_t string_len = uint2korr(m_remaining_key + length_offset);
-    assert(data_offset + string_len <= m_remaining_key_part_info->store_length);
-    assert(data_offset + string_len <= m_remaining_key_len);
-    assert(string_len <= buffer_capacity);
+    DBUG_ASSERT(data_offset <= m_remaining_key_len);
+    size_t string_len = uint2korr(m_remaining_key + length_offset);
+    DBUG_ASSERT(data_offset + string_len <=
+                m_remaining_key_part_info->store_length);
+    DBUG_ASSERT(data_offset + string_len <= m_remaining_key_len);
+
+    // DBUG_ASSERT(string_len <= buffer_capacity);
+    if (string_len > buffer_capacity) {
+      string_len = buffer_capacity;
+    }
 
     memcpy(buffer, m_remaining_key + data_offset, string_len);
     *buffer_length = (uint)string_len;
 
-    auto *pos = (uchar *)buffer;
+    uchar *pos = (uchar *)buffer;
     const uchar *end = skip_trailing_space(pos, string_len);
     *buffer_length = (uint)(end - pos);
 
@@ -1414,13 +1357,13 @@ enum ha_rkey_function PFS_key_reader::read_varchar_utf8(
     return ((m_remaining_key_len == 0) ? find_flag : HA_READ_KEY_EXACT);
   }
 
-  assert(m_remaining_key_len == 0);
+  DBUG_ASSERT(m_remaining_key_len == 0);
   return HA_READ_INVALID;
 }
 
 enum ha_rkey_function PFS_key_reader::read_text_utf8(
     enum ha_rkey_function find_flag, bool &isnull, char *buffer,
-    uint *buffer_length, uint buffer_capacity [[maybe_unused]]) {
+    uint *buffer_length, uint buffer_capacity) {
   if (m_remaining_key_part_info->store_length <= m_remaining_key_len) {
     /*
       Stored as:
@@ -1428,30 +1371,38 @@ enum ha_rkey_function PFS_key_reader::read_text_utf8(
       - followed by data
       - Length determined by key definition
     */
-    assert(m_remaining_key_part_info->type == HA_KEYTYPE_TEXT);
+    DBUG_ASSERT(m_remaining_key_part_info->type == HA_KEYTYPE_TEXT);
 
+    size_t length_offset = 0;
     size_t data_offset = 0;
     isnull = false;
     if (m_remaining_key_part_info->field->is_nullable()) {
-      assert(HA_KEY_NULL_LENGTH <= m_remaining_key_len);
+      DBUG_ASSERT(HA_KEY_NULL_LENGTH <= m_remaining_key_len);
 
+      length_offset++;
       data_offset++;
       if (m_remaining_key[0]) {
         isnull = true;
       }
     }
 
-    assert(data_offset <= m_remaining_key_len);
+    DBUG_ASSERT(data_offset <= m_remaining_key_len);
     size_t string_len = m_remaining_key_part_info->length;
-    assert(data_offset + string_len <= m_remaining_key_part_info->store_length);
-    assert(data_offset + string_len <= m_remaining_key_len);
-    assert(string_len <= buffer_capacity);
+    DBUG_ASSERT(data_offset + string_len <=
+                m_remaining_key_part_info->store_length);
+    DBUG_ASSERT(data_offset + string_len <= m_remaining_key_len);
+
+    // DBUG_ASSERT(string_len <= buffer_capacity);
+    if (string_len > buffer_capacity)  // FIXME
+    {
+      string_len = buffer_capacity;
+    }
 
     memcpy(buffer, m_remaining_key + data_offset, string_len);
     *buffer_length = (uint)string_len;
 
     const CHARSET_INFO *cs = &my_charset_utf8mb4_bin;
-    auto *pos = (uchar *)buffer;
+    uchar *pos = (uchar *)buffer;
     if (cs->mbmaxlen > 1) {
       size_t char_length;
       char_length =
@@ -1468,7 +1419,7 @@ enum ha_rkey_function PFS_key_reader::read_text_utf8(
     return ((m_remaining_key_len == 0) ? find_flag : HA_READ_KEY_EXACT);
   }
 
-  assert(m_remaining_key_len == 0);
+  DBUG_ASSERT(m_remaining_key_len == 0);
   return HA_READ_INVALID;
 }
 
@@ -1477,35 +1428,29 @@ void PFS_engine_index::read_key(const uchar *key, uint key_len,
   PFS_key_reader reader(m_key_info, key, key_len);
 
   if (m_key_ptr_1 != nullptr) {
-    assert(native_strcasecmp(m_key_info->key_part[0].field->field_name,
-                             m_key_ptr_1->m_name) == 0);
+    DBUG_ASSERT(native_strcasecmp(m_key_info->key_part[0].field->field_name,
+                                  m_key_ptr_1->m_name) == 0);
     m_key_ptr_1->read(reader, find_flag);
   }
 
   if (m_key_ptr_2 != nullptr) {
-    assert(native_strcasecmp(m_key_info->key_part[1].field->field_name,
-                             m_key_ptr_2->m_name) == 0);
+    DBUG_ASSERT(native_strcasecmp(m_key_info->key_part[1].field->field_name,
+                                  m_key_ptr_2->m_name) == 0);
     m_key_ptr_2->read(reader, find_flag);
   }
 
   if (m_key_ptr_3 != nullptr) {
-    assert(native_strcasecmp(m_key_info->key_part[2].field->field_name,
-                             m_key_ptr_3->m_name) == 0);
+    DBUG_ASSERT(native_strcasecmp(m_key_info->key_part[2].field->field_name,
+                                  m_key_ptr_3->m_name) == 0);
     m_key_ptr_3->read(reader, find_flag);
   }
 
   if (m_key_ptr_4 != nullptr) {
-    assert(native_strcasecmp(m_key_info->key_part[3].field->field_name,
-                             m_key_ptr_4->m_name) == 0);
+    DBUG_ASSERT(native_strcasecmp(m_key_info->key_part[3].field->field_name,
+                                  m_key_ptr_4->m_name) == 0);
     m_key_ptr_4->read(reader, find_flag);
   }
 
-  if (m_key_ptr_5 != nullptr) {
-    assert(native_strcasecmp(m_key_info->key_part[4].field->field_name,
-                             m_key_ptr_5->m_name) == 0);
-    m_key_ptr_5->read(reader, find_flag);
-  }
-
   m_fields = reader.m_parts_found;
 }
 
