@@ -1,4 +1,4 @@
-/* Copyright (c) 2017, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2020, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -244,15 +244,16 @@ bool plugin_table_service_initialized = false;
  * @param fields    Array of fields in the table
  */
 static int write_row(PFS_engine_table *pfs_table, TABLE *table,
-                     unsigned char *buf [[maybe_unused]], Field **fields) {
+                     unsigned char *buf MY_ATTRIBUTE((unused)),
+                     Field **fields) {
   int result = 0;
   Field *f;
-  auto *temp = (table_plugin_table *)pfs_table;
+  table_plugin_table *temp = (table_plugin_table *)pfs_table;
 
   for (; (f = *fields); fields++) {
-    if (bitmap_is_set(table->write_set, f->field_index())) {
+    if (bitmap_is_set(table->write_set, f->field_index)) {
       result = temp->m_st_table->write_column_value(
-          temp->plugin_table_handle, (PSI_field *)f, f->field_index());
+          temp->plugin_table_handle, (PSI_field *)f, f->field_index);
       if (result) {
         return result;
       }
@@ -330,7 +331,7 @@ static int initialize_table_share(PFS_engine_table_share *share,
  * @param share share to be destroyed.
  */
 static void destroy_table_share(PFS_engine_table_share *share) {
-  assert(share);
+  DBUG_ASSERT(share);
 
   thr_lock_delete(share->m_thr_lock_ptr);
   delete share->m_table_def;
@@ -365,8 +366,8 @@ static int pfs_add_tables_v1(PFS_engine_table_share_proxy **st_share_list,
    */
   for (uint i = 0; i < share_list_count; i++) {
     temp_st_share = st_share_list[i];
-    assert(temp_st_share && temp_st_share->m_table_name &&
-           temp_st_share->m_table_name_length > 0);
+    DBUG_ASSERT(temp_st_share && temp_st_share->m_table_name &&
+                temp_st_share->m_table_name_length > 0);
 
     /* If table already exists either in
      * - Native PFS tables list
@@ -388,21 +389,22 @@ static int pfs_add_tables_v1(PFS_engine_table_share_proxy **st_share_list,
     temp_st_share = st_share_list[i];
 
     /* Create a new instance of table_share */
-    auto *temp_share = new PFS_engine_table_share();
+    PFS_engine_table_share *temp_share = new PFS_engine_table_share();
 
     /* Initialize table share for this new table */
     if (initialize_table_share(temp_share, temp_st_share)) {
       /* Delete all the initialized table shares till now */
-      for (auto *share : share_list) {
+      for (auto share : share_list) {
         pfs_external_table_shares.remove_share(share);
         destroy_table_share(share);
       }
       pfs_external_table_shares.unlock_share_list();
       return 1;
+    } else {
+      /* Add share to PFS shares list */
+      pfs_external_table_shares.add_share(temp_share);
+      share_list.push_back(temp_share);
     }
-    /* Add share to PFS shares list */
-    pfs_external_table_shares.add_share(temp_share);
-    share_list.push_back(temp_share);
   }
 
   /* Unlock mutex on PFS share list now because while creating table (by DD API)
@@ -417,15 +419,14 @@ static int pfs_add_tables_v1(PFS_engine_table_share_proxy **st_share_list,
   for (uint i = 0; i < share_list_count; i++) {
     temp_st_share = st_share_list[i];
 
-    const Plugin_table t(PERFORMANCE_SCHEMA_str.str,
-                         temp_st_share->m_table_name,
-                         temp_st_share->m_table_definition,
-                         "engine = 'performance_schema'", nullptr);
+    Plugin_table t(PERFORMANCE_SCHEMA_str.str, temp_st_share->m_table_name,
+                   temp_st_share->m_table_definition,
+                   "engine = 'performance_schema'", nullptr);
     if (create_native_table_for_pfs(&t)) {
       /* ============== CRITICAL SECTION 2 (begin) ================= */
       pfs_external_table_shares.lock_share_list();
       /* Delete all the initialized table share till now */
-      for (auto *share : share_list) {
+      for (auto share : share_list) {
         pfs_external_table_shares.remove_share(share);
         destroy_table_share(share);
       }
@@ -458,8 +459,8 @@ static int pfs_delete_tables_v1(PFS_engine_table_share_proxy **st_share_list,
   /* Check if any of the table, in the list, doesn't exist. */
   for (uint i = 0; i < share_list_count; i++) {
     PFS_engine_table_share_proxy *temp_st_share = st_share_list[i];
-    assert(temp_st_share && temp_st_share->m_table_name &&
-           temp_st_share->m_table_name_length > 0);
+    DBUG_ASSERT(temp_st_share && temp_st_share->m_table_name &&
+                temp_st_share->m_table_name_length > 0);
 
     /* Search table share for the table in other list (including purgatory) */
     PFS_engine_table_share *temp_share =
@@ -484,7 +485,7 @@ static int pfs_delete_tables_v1(PFS_engine_table_share_proxy **st_share_list,
    *
    * Now traverse share_list and drop tables using DD API.
    */
-  for (auto *share : share_list) {
+  for (auto share : share_list) {
     if (drop_native_table_for_pfs(PERFORMANCE_SCHEMA_str.str,
                                   share->m_table_def->get_name())) {
       return 1;
@@ -497,7 +498,7 @@ static int pfs_delete_tables_v1(PFS_engine_table_share_proxy **st_share_list,
   /* At this point tables have been dropped. So we can remove shares from
    * PFS shares list.
    */
-  for (auto *share : share_list) {
+  for (auto share : share_list) {
     pfs_external_table_shares.remove_share(share);
     destroy_table_share(share);
   }
@@ -514,7 +515,7 @@ static int pfs_delete_tables_v1(PFS_engine_table_share_proxy **st_share_list,
  * Type TINYINT                       *
  **************************************/
 void set_field_tinyint_v1(PSI_field *f, PSI_tinyint value) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+  Field *f_ptr = reinterpret_cast<Field *>(f);
   if (value.is_null) {
     f_ptr->set_null();
   } else {
@@ -523,7 +524,7 @@ void set_field_tinyint_v1(PSI_field *f, PSI_tinyint value) {
 }
 
 void set_field_utinyint_v1(PSI_field *f, PSI_utinyint value) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+  Field *f_ptr = reinterpret_cast<Field *>(f);
   if (value.is_null) {
     f_ptr->set_null();
   } else {
@@ -532,7 +533,7 @@ void set_field_utinyint_v1(PSI_field *f, PSI_utinyint value) {
 }
 
 void get_field_tinyint_v1(PSI_field *f, PSI_tinyint *value) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+  Field *f_ptr = reinterpret_cast<Field *>(f);
   if (f_ptr->is_null()) {
     value->is_null = true;
     return;
@@ -543,7 +544,7 @@ void get_field_tinyint_v1(PSI_field *f, PSI_tinyint *value) {
 }
 
 void get_field_utinyint_v1(PSI_field *f, PSI_utinyint *value) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+  Field *f_ptr = reinterpret_cast<Field *>(f);
   if (f_ptr->is_null()) {
     value->is_null = true;
     return;
@@ -555,10 +556,10 @@ void get_field_utinyint_v1(PSI_field *f, PSI_utinyint *value) {
 
 void read_key_tinyint_v1(PSI_key_reader *reader, PSI_plugin_key_tinyint *key,
                          int find_flag) {
-  auto *pfs_reader = (PFS_key_reader *)reader;
-  const auto e_find_flag = (enum ha_rkey_function)find_flag;
+  PFS_key_reader *pfs_reader = (PFS_key_reader *)reader;
+  enum ha_rkey_function e_find_flag = (enum ha_rkey_function)find_flag;
 
-  char temp_value{0};
+  char temp_value;
   key->m_find_flags =
       pfs_reader->read_int8(e_find_flag, key->m_is_null, &temp_value);
   key->m_value = temp_value;
@@ -566,10 +567,10 @@ void read_key_tinyint_v1(PSI_key_reader *reader, PSI_plugin_key_tinyint *key,
 
 void read_key_utinyint_v1(PSI_key_reader *reader, PSI_plugin_key_utinyint *key,
                           int find_flag) {
-  auto *pfs_reader = (PFS_key_reader *)reader;
-  const auto e_find_flag = (enum ha_rkey_function)find_flag;
+  PFS_key_reader *pfs_reader = (PFS_key_reader *)reader;
+  enum ha_rkey_function e_find_flag = (enum ha_rkey_function)find_flag;
 
-  unsigned char temp_value{0};
+  unsigned char temp_value;
   key->m_find_flags =
       pfs_reader->read_uint8(e_find_flag, key->m_is_null, &temp_value);
   key->m_value = temp_value;
@@ -593,7 +594,7 @@ bool match_key_utinyint_v1(bool record_null, unsigned long record_value,
  * Type SMALLINT                      *
  **************************************/
 void set_field_smallint_v1(PSI_field *f, PSI_smallint value) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+  Field *f_ptr = reinterpret_cast<Field *>(f);
   if (value.is_null) {
     f_ptr->set_null();
   } else {
@@ -602,7 +603,7 @@ void set_field_smallint_v1(PSI_field *f, PSI_smallint value) {
 }
 
 void set_field_usmallint_v1(PSI_field *f, PSI_usmallint value) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+  Field *f_ptr = reinterpret_cast<Field *>(f);
   if (value.is_null) {
     f_ptr->set_null();
   } else {
@@ -611,7 +612,7 @@ void set_field_usmallint_v1(PSI_field *f, PSI_usmallint value) {
 }
 
 void get_field_smallint_v1(PSI_field *f, PSI_smallint *value) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+  Field *f_ptr = reinterpret_cast<Field *>(f);
   if (f_ptr->is_null()) {
     value->is_null = true;
     return;
@@ -622,7 +623,7 @@ void get_field_smallint_v1(PSI_field *f, PSI_smallint *value) {
 }
 
 void get_field_usmallint_v1(PSI_field *f, PSI_usmallint *value) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+  Field *f_ptr = reinterpret_cast<Field *>(f);
   if (f_ptr->is_null()) {
     value->is_null = true;
     return;
@@ -634,10 +635,10 @@ void get_field_usmallint_v1(PSI_field *f, PSI_usmallint *value) {
 
 void read_key_smallint_v1(PSI_key_reader *reader, PSI_plugin_key_smallint *key,
                           int find_flag) {
-  auto *pfs_reader = (PFS_key_reader *)reader;
-  const auto e_find_flag = (enum ha_rkey_function)find_flag;
+  PFS_key_reader *pfs_reader = (PFS_key_reader *)reader;
+  enum ha_rkey_function e_find_flag = (enum ha_rkey_function)find_flag;
 
-  short temp_value{0};
+  short temp_value;
   key->m_find_flags =
       pfs_reader->read_int16(e_find_flag, key->m_is_null, &temp_value);
   key->m_value = temp_value;
@@ -645,10 +646,10 @@ void read_key_smallint_v1(PSI_key_reader *reader, PSI_plugin_key_smallint *key,
 
 void read_key_usmallint_v1(PSI_key_reader *reader,
                            PSI_plugin_key_usmallint *key, int find_flag) {
-  auto *pfs_reader = (PFS_key_reader *)reader;
-  const auto e_find_flag = (enum ha_rkey_function)find_flag;
+  PFS_key_reader *pfs_reader = (PFS_key_reader *)reader;
+  enum ha_rkey_function e_find_flag = (enum ha_rkey_function)find_flag;
 
-  unsigned short temp_value{0};
+  unsigned short temp_value;
   key->m_find_flags =
       pfs_reader->read_uint16(e_find_flag, key->m_is_null, &temp_value);
   key->m_value = temp_value;
@@ -672,7 +673,7 @@ bool match_key_usmallint_v1(bool record_null, unsigned long record_value,
  * Type MEDIUMINT                     *
  **************************************/
 void set_field_mediumint_v1(PSI_field *f, PSI_mediumint value) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+  Field *f_ptr = reinterpret_cast<Field *>(f);
   if (value.is_null) {
     f_ptr->set_null();
   } else {
@@ -681,7 +682,7 @@ void set_field_mediumint_v1(PSI_field *f, PSI_mediumint value) {
 }
 
 void set_field_umediumint_v1(PSI_field *f, PSI_umediumint value) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+  Field *f_ptr = reinterpret_cast<Field *>(f);
   if (value.is_null) {
     f_ptr->set_null();
   } else {
@@ -690,7 +691,7 @@ void set_field_umediumint_v1(PSI_field *f, PSI_umediumint value) {
 }
 
 void get_field_mediumint_v1(PSI_field *f, PSI_mediumint *value) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+  Field *f_ptr = reinterpret_cast<Field *>(f);
   if (f_ptr->is_null()) {
     value->is_null = true;
     return;
@@ -701,7 +702,7 @@ void get_field_mediumint_v1(PSI_field *f, PSI_mediumint *value) {
 }
 
 void get_field_umediumint_v1(PSI_field *f, PSI_umediumint *value) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+  Field *f_ptr = reinterpret_cast<Field *>(f);
   if (f_ptr->is_null()) {
     value->is_null = true;
     return;
@@ -713,10 +714,10 @@ void get_field_umediumint_v1(PSI_field *f, PSI_umediumint *value) {
 
 void read_key_mediumint_v1(PSI_key_reader *reader,
                            PSI_plugin_key_mediumint *key, int find_flag) {
-  auto *pfs_reader = (PFS_key_reader *)reader;
-  const auto e_find_flag = (enum ha_rkey_function)find_flag;
+  PFS_key_reader *pfs_reader = (PFS_key_reader *)reader;
+  enum ha_rkey_function e_find_flag = (enum ha_rkey_function)find_flag;
 
-  long temp_value{0};
+  long temp_value;
   key->m_find_flags =
       pfs_reader->read_int24(e_find_flag, key->m_is_null, &temp_value);
   key->m_value = temp_value;
@@ -724,10 +725,10 @@ void read_key_mediumint_v1(PSI_key_reader *reader,
 
 void read_key_umediumint_v1(PSI_key_reader *reader,
                             PSI_plugin_key_umediumint *key, int find_flag) {
-  auto *pfs_reader = (PFS_key_reader *)reader;
-  const auto e_find_flag = (enum ha_rkey_function)find_flag;
+  PFS_key_reader *pfs_reader = (PFS_key_reader *)reader;
+  enum ha_rkey_function e_find_flag = (enum ha_rkey_function)find_flag;
 
-  unsigned long temp_value{0};
+  unsigned long temp_value;
   key->m_find_flags =
       pfs_reader->read_uint24(e_find_flag, key->m_is_null, &temp_value);
   key->m_value = temp_value;
@@ -751,7 +752,7 @@ bool match_key_umediumint_v1(bool record_null, unsigned long record_value,
  * Type INTEGER (INT)                 *
  **************************************/
 void set_field_integer_v1(PSI_field *f, PSI_int value) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+  Field *f_ptr = reinterpret_cast<Field *>(f);
   if (value.is_null) {
     f_ptr->set_null();
   } else {
@@ -760,7 +761,7 @@ void set_field_integer_v1(PSI_field *f, PSI_int value) {
 }
 
 void set_field_uinteger_v1(PSI_field *f, PSI_uint value) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+  Field *f_ptr = reinterpret_cast<Field *>(f);
   if (value.is_null) {
     f_ptr->set_null();
   } else {
@@ -769,7 +770,7 @@ void set_field_uinteger_v1(PSI_field *f, PSI_uint value) {
 }
 
 void get_field_integer_v1(PSI_field *f, PSI_int *value) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+  Field *f_ptr = reinterpret_cast<Field *>(f);
   if (f_ptr->is_null()) {
     value->is_null = true;
     return;
@@ -780,7 +781,7 @@ void get_field_integer_v1(PSI_field *f, PSI_int *value) {
 }
 
 void get_field_uinteger_v1(PSI_field *f, PSI_int *value) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+  Field *f_ptr = reinterpret_cast<Field *>(f);
   if (f_ptr->is_null()) {
     value->is_null = true;
     return;
@@ -792,10 +793,10 @@ void get_field_uinteger_v1(PSI_field *f, PSI_int *value) {
 
 void read_key_integer_v1(PSI_key_reader *reader, PSI_plugin_key_integer *key,
                          int find_flag) {
-  auto *pfs_reader = (PFS_key_reader *)reader;
-  const auto e_find_flag = (enum ha_rkey_function)find_flag;
+  PFS_key_reader *pfs_reader = (PFS_key_reader *)reader;
+  enum ha_rkey_function e_find_flag = (enum ha_rkey_function)find_flag;
 
-  long temp_value{0};
+  long temp_value;
   key->m_find_flags =
       pfs_reader->read_long(e_find_flag, key->m_is_null, &temp_value);
   key->m_value = temp_value;
@@ -803,10 +804,10 @@ void read_key_integer_v1(PSI_key_reader *reader, PSI_plugin_key_integer *key,
 
 void read_key_uinteger_v1(PSI_key_reader *reader, PSI_plugin_key_uinteger *key,
                           int find_flag) {
-  auto *pfs_reader = (PFS_key_reader *)reader;
-  const auto e_find_flag = (enum ha_rkey_function)find_flag;
+  PFS_key_reader *pfs_reader = (PFS_key_reader *)reader;
+  enum ha_rkey_function e_find_flag = (enum ha_rkey_function)find_flag;
 
-  unsigned long temp_value{0};
+  unsigned long temp_value;
   key->m_find_flags =
       pfs_reader->read_ulong(e_find_flag, key->m_is_null, &temp_value);
   key->m_value = temp_value;
@@ -830,7 +831,7 @@ bool match_key_uinteger_v1(bool record_null, unsigned long record_value,
  * Type BIGINT                        *
  **************************************/
 void set_field_bigint_v1(PSI_field *f, PSI_bigint value) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+  Field *f_ptr = reinterpret_cast<Field *>(f);
   if (value.is_null) {
     f_ptr->set_null();
   } else {
@@ -839,7 +840,7 @@ void set_field_bigint_v1(PSI_field *f, PSI_bigint value) {
 }
 
 void set_field_ubigint_v1(PSI_field *f, PSI_ubigint value) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+  Field *f_ptr = reinterpret_cast<Field *>(f);
   if (value.is_null) {
     f_ptr->set_null();
   } else {
@@ -848,7 +849,7 @@ void set_field_ubigint_v1(PSI_field *f, PSI_ubigint value) {
 }
 
 void get_field_bigint_v1(PSI_field *f, PSI_bigint *value) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+  Field *f_ptr = reinterpret_cast<Field *>(f);
   if (f_ptr->is_null()) {
     value->is_null = true;
     return;
@@ -859,7 +860,7 @@ void get_field_bigint_v1(PSI_field *f, PSI_bigint *value) {
 }
 
 void get_field_ubigint_v1(PSI_field *f, PSI_ubigint *value) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+  Field *f_ptr = reinterpret_cast<Field *>(f);
   if (f_ptr->is_null()) {
     value->is_null = true;
     return;
@@ -871,10 +872,10 @@ void get_field_ubigint_v1(PSI_field *f, PSI_ubigint *value) {
 
 void read_key_bigint_v1(PSI_key_reader *reader, PSI_plugin_key_bigint *key,
                         int find_flag) {
-  auto *pfs_reader = (PFS_key_reader *)reader;
-  const auto e_find_flag = (enum ha_rkey_function)find_flag;
+  PFS_key_reader *pfs_reader = (PFS_key_reader *)reader;
+  enum ha_rkey_function e_find_flag = (enum ha_rkey_function)find_flag;
 
-  long long temp_value{0};
+  long long temp_value;
   key->m_find_flags =
       pfs_reader->read_longlong(e_find_flag, key->m_is_null, &temp_value);
   key->m_value = temp_value;
@@ -882,10 +883,10 @@ void read_key_bigint_v1(PSI_key_reader *reader, PSI_plugin_key_bigint *key,
 
 void read_key_ubigint_v1(PSI_key_reader *reader, PSI_plugin_key_ubigint *key,
                          int find_flag) {
-  auto *pfs_reader = (PFS_key_reader *)reader;
-  const auto e_find_flag = (enum ha_rkey_function)find_flag;
+  PFS_key_reader *pfs_reader = (PFS_key_reader *)reader;
+  enum ha_rkey_function e_find_flag = (enum ha_rkey_function)find_flag;
 
-  unsigned long long temp_value{0};
+  unsigned long long temp_value;
   key->m_find_flags =
       pfs_reader->read_ulonglong(e_find_flag, key->m_is_null, &temp_value);
   key->m_value = temp_value;
@@ -909,7 +910,7 @@ bool match_key_ubigint_v1(bool record_null, unsigned long long record_value,
  * Type DECIMAL                       *
  **************************************/
 void set_field_decimal_v1(PSI_field *f, PSI_double value) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+  Field *f_ptr = reinterpret_cast<Field *>(f);
   if (value.is_null) {
     f_ptr->set_null();
   } else {
@@ -918,7 +919,7 @@ void set_field_decimal_v1(PSI_field *f, PSI_double value) {
 }
 
 void get_field_decimal_v1(PSI_field *f, PSI_double *value) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+  Field *f_ptr = reinterpret_cast<Field *>(f);
   if (f_ptr->is_null()) {
     value->is_null = true;
     return;
@@ -932,7 +933,7 @@ void get_field_decimal_v1(PSI_field *f, PSI_double *value) {
  * Type FLOAT                         *
  **************************************/
 void set_field_float_v1(PSI_field *f, PSI_double value) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+  Field *f_ptr = reinterpret_cast<Field *>(f);
   if (value.is_null) {
     f_ptr->set_null();
   } else {
@@ -941,7 +942,7 @@ void set_field_float_v1(PSI_field *f, PSI_double value) {
 }
 
 void get_field_float_v1(PSI_field *f, PSI_double *value) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+  Field *f_ptr = reinterpret_cast<Field *>(f);
   if (f_ptr->is_null()) {
     value->is_null = true;
     return;
@@ -955,7 +956,7 @@ void get_field_float_v1(PSI_field *f, PSI_double *value) {
  * Type DOUBLE                        *
  **************************************/
 void set_field_double_v1(PSI_field *f, PSI_double value) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+  Field *f_ptr = reinterpret_cast<Field *>(f);
   if (value.is_null) {
     f_ptr->set_null();
   } else {
@@ -964,7 +965,7 @@ void set_field_double_v1(PSI_field *f, PSI_double value) {
 }
 
 void get_field_double_v1(PSI_field *f, PSI_double *value) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+  Field *f_ptr = reinterpret_cast<Field *>(f);
   if (f_ptr->is_null()) {
     value->is_null = true;
     return;
@@ -977,17 +978,17 @@ void get_field_double_v1(PSI_field *f, PSI_double *value) {
 /**************************************
  * Type CHAR                          *
  **************************************/
-void set_field_char_utf8mb4_v1(PSI_field *f, const char *value, uint len) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+void set_field_char_utf8_v1(PSI_field *f, const char *value, uint len) {
+  Field *f_ptr = reinterpret_cast<Field *>(f);
   if (len > 0) {
-    set_field_char_utf8mb4(f_ptr, value, len);
+    set_field_char_utf8(f_ptr, value, len);
   } else {
     f_ptr->set_null();
   }
 }
 
-void get_field_char_utf8mb4_v1(PSI_field *f, char *val, uint *len) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+void get_field_char_utf8_v1(PSI_field *f, char *val, uint *len) {
+  Field *f_ptr = reinterpret_cast<Field *>(f);
 
   /* If NULL is provided */
   if (f_ptr->is_null()) {
@@ -996,14 +997,23 @@ void get_field_char_utf8mb4_v1(PSI_field *f, char *val, uint *len) {
     return;
   }
 
-  get_field_char_utf8mb4(f_ptr, val, len);
+  val = get_field_char_utf8(f_ptr, val, len);
 }
 
 /**************************************
  * Type VARCAHAR                      *
  **************************************/
+void set_field_varchar_utf8_len_v1(PSI_field *f, const char *str, uint len) {
+  Field *f_ptr = reinterpret_cast<Field *>(f);
+  if (len > 0) {
+    set_field_varchar_utf8(f_ptr, str, len);
+  } else {
+    f_ptr->set_null();
+  }
+}
+
 void set_field_varchar_utf8mb4_len_v1(PSI_field *f, const char *str, uint len) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+  Field *f_ptr = reinterpret_cast<Field *>(f);
   if (len > 0) {
     set_field_varchar_utf8mb4(f_ptr, str, len);
   } else {
@@ -1011,17 +1021,26 @@ void set_field_varchar_utf8mb4_len_v1(PSI_field *f, const char *str, uint len) {
   }
 }
 
-void set_field_varchar_utf8mb4_v1(PSI_field *f, const char *str) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+void set_field_varchar_utf8_v1(PSI_field *f, const char *str) {
+  Field *f_ptr = reinterpret_cast<Field *>(f);
   if (str != nullptr) {
-    set_field_varchar_utf8mb4(f_ptr, str);
+    set_field_varchar_utf8(f_ptr, str);
   } else {
     f_ptr->set_null();
   }
 }
 
-void get_field_varchar_utf8mb4_v1(PSI_field *f, char *val, uint *len) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+void set_field_varchar_utf8mb4_v1(PSI_field *f, const char *str) {
+  Field *f_ptr = reinterpret_cast<Field *>(f);
+  if (str != nullptr) {
+    set_field_varchar_utf8(f_ptr, str);
+  } else {
+    f_ptr->set_null();
+  }
+}
+
+void get_field_varchar_utf8_v1(PSI_field *f, char *val, uint *len) {
+  Field *f_ptr = reinterpret_cast<Field *>(f);
 
   /* If NULL is provided */
   if (f_ptr->is_null()) {
@@ -1030,14 +1049,14 @@ void get_field_varchar_utf8mb4_v1(PSI_field *f, char *val, uint *len) {
     return;
   }
 
-  get_field_varchar_utf8mb4(f_ptr, val, len);
+  val = get_field_varchar_utf8(f_ptr, val, len);
 }
 
 /**************************************
  * Type BLOB/TEXT                     *
  **************************************/
 void set_field_blob_v1(PSI_field *f, const char *val, uint len) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+  Field *f_ptr = reinterpret_cast<Field *>(f);
   if (len > 0) {
     set_field_blob(f_ptr, val, len);
   } else {
@@ -1046,7 +1065,7 @@ void set_field_blob_v1(PSI_field *f, const char *val, uint len) {
 }
 
 void get_field_blob_v1(PSI_field *f, char *val, uint *len) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+  Field *f_ptr = reinterpret_cast<Field *>(f);
 
   /* If NULL is provided */
   if (f_ptr->is_null()) {
@@ -1055,30 +1074,14 @@ void get_field_blob_v1(PSI_field *f, char *val, uint *len) {
     return;
   }
 
-  get_field_blob(f_ptr, val, len);
-}
-
-/**************************************
- * Type TEXT                          *
- **************************************/
-void set_field_text_v1(PSI_field *f, const char *val, uint len) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
-  if (val != nullptr) {
-    set_field_text(f_ptr, val, len, &my_charset_utf8mb4_bin);
-  } else {
-    f_ptr->set_null();
-  }
-}
-
-void get_field_text_v1(PSI_field *f, char *val, uint *len) {
-  get_field_blob_v1(f, val, len);
+  val = get_field_blob(f_ptr, val, len);
 }
 
 /**************************************
  * Type ENUM                          *
  **************************************/
 void set_field_enum_v1(PSI_field *f, PSI_enum value) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+  Field *f_ptr = reinterpret_cast<Field *>(f);
   if (value.is_null) {
     f_ptr->set_null();
   } else {
@@ -1087,7 +1090,7 @@ void set_field_enum_v1(PSI_field *f, PSI_enum value) {
 }
 
 void get_field_enum_v1(PSI_field *f, PSI_enum *value) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+  Field *f_ptr = reinterpret_cast<Field *>(f);
   if (f_ptr->is_null()) {
     value->is_null = true;
     return;
@@ -1101,7 +1104,7 @@ void get_field_enum_v1(PSI_field *f, PSI_enum *value) {
  * Type DATE                          *
  **************************************/
 void set_field_date_v1(PSI_field *f, const char *value, uint len) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+  Field *f_ptr = reinterpret_cast<Field *>(f);
   if (len > 0) {
     set_field_date(f_ptr, value, len);
   } else {
@@ -1110,7 +1113,7 @@ void set_field_date_v1(PSI_field *f, const char *value, uint len) {
 }
 
 void get_field_date_v1(PSI_field *f, char *val, uint *len) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+  Field *f_ptr = reinterpret_cast<Field *>(f);
 
   /* If NULL is provided */
   if (f_ptr->is_null()) {
@@ -1119,14 +1122,14 @@ void get_field_date_v1(PSI_field *f, char *val, uint *len) {
     return;
   }
 
-  get_field_date(f_ptr, val, len);
+  val = get_field_date(f_ptr, val, len);
 }
 
 /**************************************
  * Type TIME                          *
  **************************************/
 void set_field_time_v1(PSI_field *f, const char *value, uint len) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+  Field *f_ptr = reinterpret_cast<Field *>(f);
   if (len > 0) {
     set_field_time(f_ptr, value, len);
   } else {
@@ -1135,7 +1138,7 @@ void set_field_time_v1(PSI_field *f, const char *value, uint len) {
 }
 
 void get_field_time_v1(PSI_field *f, char *val, uint *len) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+  Field *f_ptr = reinterpret_cast<Field *>(f);
 
   /* If NULL is provided */
   if (f_ptr->is_null()) {
@@ -1144,14 +1147,14 @@ void get_field_time_v1(PSI_field *f, char *val, uint *len) {
     return;
   }
 
-  get_field_time(f_ptr, val, len);
+  val = get_field_time(f_ptr, val, len);
 }
 
 /**************************************
  * Type DATETIME                      *
  **************************************/
 void set_field_datetime_v1(PSI_field *f, const char *value, uint len) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+  Field *f_ptr = reinterpret_cast<Field *>(f);
   if (len > 0) {
     set_field_datetime(f_ptr, value, len);
   } else {
@@ -1160,7 +1163,7 @@ void set_field_datetime_v1(PSI_field *f, const char *value, uint len) {
 }
 
 void get_field_datetime_v1(PSI_field *f, char *val, uint *len) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+  Field *f_ptr = reinterpret_cast<Field *>(f);
 
   /* If NULL is provided */
   if (f_ptr->is_null()) {
@@ -1169,14 +1172,14 @@ void get_field_datetime_v1(PSI_field *f, char *val, uint *len) {
     return;
   }
 
-  get_field_datetime(f_ptr, val, len);
+  val = get_field_datetime(f_ptr, val, len);
 }
 
 /**************************************
  * Type TIMESTAMP                     *
  **************************************/
 void set_field_timestamp_v1(PSI_field *f, const char *value, uint len) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+  Field *f_ptr = reinterpret_cast<Field *>(f);
   if (len > 0) {
     set_field_timestamp(f_ptr, value, len);
   } else {
@@ -1185,7 +1188,7 @@ void set_field_timestamp_v1(PSI_field *f, const char *value, uint len) {
 }
 
 void set_field_timestamp2_v1(PSI_field *f, ulonglong value) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+  Field *f_ptr = reinterpret_cast<Field *>(f);
   if (value > 0) {
     set_field_timestamp(f_ptr, value);
   } else {
@@ -1194,7 +1197,7 @@ void set_field_timestamp2_v1(PSI_field *f, ulonglong value) {
 }
 
 void get_field_timestamp_v1(PSI_field *f, char *val, uint *len) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+  Field *f_ptr = reinterpret_cast<Field *>(f);
 
   /* If NULL is provided */
   if (f_ptr->is_null()) {
@@ -1203,14 +1206,14 @@ void get_field_timestamp_v1(PSI_field *f, char *val, uint *len) {
     return;
   }
 
-  get_field_timestamp(f_ptr, val, len);
+  val = get_field_timestamp(f_ptr, val, len);
 }
 
 /**************************************
  * Type YEAR                          *
  **************************************/
 void set_field_year_v1(PSI_field *f, PSI_year value) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+  Field *f_ptr = reinterpret_cast<Field *>(f);
   if (value.is_null) {
     f_ptr->set_null();
   } else {
@@ -1219,7 +1222,7 @@ void set_field_year_v1(PSI_field *f, PSI_year value) {
 }
 
 void get_field_year_v1(PSI_field *f, PSI_year *value) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+  Field *f_ptr = reinterpret_cast<Field *>(f);
   if (f_ptr->is_null()) {
     value->is_null = true;
     return;
@@ -1233,19 +1236,19 @@ void get_field_year_v1(PSI_field *f, PSI_year *value) {
  * NULL                               *
  **************************************/
 void set_field_null_v1(PSI_field *f) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
+  Field *f_ptr = reinterpret_cast<Field *>(f);
   f_ptr->set_null();
 }
 
 unsigned int get_parts_found_v1(PSI_key_reader *reader) {
-  auto *pfs_reader = (PFS_key_reader *)reader;
+  PFS_key_reader *pfs_reader = (PFS_key_reader *)reader;
   return pfs_reader->m_parts_found;
 }
 
 void read_key_string_v1(PSI_key_reader *reader, PSI_plugin_key_string *key,
                         int find_flag) {
-  auto *pfs_reader = (PFS_key_reader *)reader;
-  const auto e_find_flag = (enum ha_rkey_function)find_flag;
+  PFS_key_reader *pfs_reader = (PFS_key_reader *)reader;
+  enum ha_rkey_function e_find_flag = (enum ha_rkey_function)find_flag;
 
   key->m_find_flags = PFS_key_pstring::stateless_read(
       *pfs_reader, e_find_flag, key->m_is_null, key->m_value_buffer,
@@ -1262,20 +1265,23 @@ bool match_key_string_v1(bool record_null, const char *record_string_value,
 }
 
 void init_pfs_plugin_table() {
-  assert(!plugin_table_service_initialized);
+  DBUG_ASSERT(!plugin_table_service_initialized);
 
   /* Asserts that ERRORS defined in pfs_plugin_table_service.h are in
      accordance with ERRORS defined in my_base.h
   */
-  static_assert((PFS_HA_ERR_WRONG_COMMAND == HA_ERR_WRONG_COMMAND) &&
-                (PFS_HA_ERR_RECORD_DELETED == HA_ERR_RECORD_DELETED) &&
-                (PFS_HA_ERR_END_OF_FILE == HA_ERR_END_OF_FILE) &&
-                (PFS_HA_ERR_NO_REFERENCED_ROW == HA_ERR_NO_REFERENCED_ROW) &&
-                (PFS_HA_ERR_FOUND_DUPP_KEY == HA_ERR_FOUND_DUPP_KEY) &&
-                (PFS_HA_ERR_RECORD_FILE_FULL == HA_ERR_RECORD_FILE_FULL));
+  static_assert(
+      (PFS_HA_ERR_WRONG_COMMAND == HA_ERR_WRONG_COMMAND) &&
+          (PFS_HA_ERR_RECORD_DELETED == HA_ERR_RECORD_DELETED) &&
+          (PFS_HA_ERR_END_OF_FILE == HA_ERR_END_OF_FILE) &&
+          (PFS_HA_ERR_NO_REFERENCED_ROW == HA_ERR_NO_REFERENCED_ROW) &&
+          (PFS_HA_ERR_FOUND_DUPP_KEY == HA_ERR_FOUND_DUPP_KEY) &&
+          (PFS_HA_ERR_RECORD_FILE_FULL == HA_ERR_RECORD_FILE_FULL),
+      "");
 
   pfs_external_table_shares.init_mutex();
   plugin_table_service_initialized = true;
+  return;
 }
 
 void cleanup_pfs_plugin_table() {
@@ -1287,6 +1293,85 @@ void cleanup_pfs_plugin_table() {
 
 /* clang-format off */
 
+/* Initialization of service methods to actual PFS implementation */
+/** @deprecated */
+BEGIN_SERVICE_IMPLEMENTATION(performance_schema, pfs_plugin_table)
+    pfs_add_tables_v1, pfs_delete_tables_v1,
+
+    set_field_tinyint_v1, set_field_utinyint_v1, get_field_tinyint_v1,
+    // read_key_,
+    // match_key_,
+
+    set_field_smallint_v1, set_field_usmallint_v1, get_field_smallint_v1,
+    // read_key_,
+    // match_key_,
+
+    set_field_mediumint_v1, set_field_umediumint_v1, get_field_mediumint_v1,
+    // read_key_,
+    // match_key_,
+
+    set_field_integer_v1, set_field_uinteger_v1, get_field_integer_v1,
+    read_key_integer_v1, match_key_integer_v1,
+
+    set_field_bigint_v1, set_field_ubigint_v1, get_field_bigint_v1,
+    // read_key_,
+    // match_key_,
+
+    set_field_decimal_v1, get_field_decimal_v1,
+    // read_key_,
+    // match_key_,
+
+    set_field_float_v1, get_field_float_v1,
+    // read_key_,
+    // match_key_,
+
+    set_field_double_v1, get_field_double_v1,
+    // read_key_,
+    // match_key_,
+
+    set_field_char_utf8_v1, get_field_char_utf8_v1, read_key_string_v1,
+    match_key_string_v1,
+
+    set_field_varchar_utf8_v1, set_field_varchar_utf8_len_v1,
+    get_field_varchar_utf8_v1,
+    // read_key_,
+    // match_key_,
+
+    set_field_varchar_utf8mb4_v1, set_field_varchar_utf8mb4_len_v1,
+    // read_key_,
+    // match_key_,
+
+    set_field_blob_v1, get_field_blob_v1,
+    // read_key_,
+    // match_key_,
+
+    set_field_enum_v1, get_field_enum_v1,
+    // read_key_,
+    // match_key_,
+
+    set_field_date_v1, get_field_date_v1,
+    // read_key_,
+    // match_key_,
+
+    set_field_time_v1, get_field_time_v1,
+    // read_key_,
+    // match_key_,
+
+    set_field_datetime_v1, get_field_datetime_v1,
+    // read_key_,
+    // match_key_,
+
+    set_field_timestamp_v1, get_field_timestamp_v1,
+    // read_key_,
+    // match_key_,
+
+    set_field_year_v1, get_field_year_v1,
+    // read_key_,
+    // match_key_,
+
+    set_field_null_v1
+END_SERVICE_IMPLEMENTATION();
+
 /* Initialization of service methods to actual PFS implementation */
 BEGIN_SERVICE_IMPLEMENTATION(performance_schema, pfs_plugin_table_v1)
     pfs_add_tables_v1, pfs_delete_tables_v1, get_parts_found_v1
@@ -1334,12 +1419,14 @@ BEGIN_SERVICE_IMPLEMENTATION(performance_schema, pfs_plugin_column_double_v1)
     set_field_double_v1, get_field_double_v1
 END_SERVICE_IMPLEMENTATION();
 
-BEGIN_SERVICE_IMPLEMENTATION(performance_schema, pfs_plugin_column_string_v2)
-    set_field_char_utf8mb4_v1,
-    get_field_char_utf8mb4_v1,
+BEGIN_SERVICE_IMPLEMENTATION(performance_schema, pfs_plugin_column_string_v1)
+    set_field_char_utf8_v1,
+    get_field_char_utf8_v1,
     read_key_string_v1,
     match_key_string_v1,
-    get_field_varchar_utf8mb4_v1,
+    set_field_varchar_utf8_v1,
+    set_field_varchar_utf8_len_v1,
+    get_field_varchar_utf8_v1,
     set_field_varchar_utf8mb4_v1,
     set_field_varchar_utf8mb4_len_v1
 END_SERVICE_IMPLEMENTATION();
@@ -1376,8 +1463,4 @@ BEGIN_SERVICE_IMPLEMENTATION(performance_schema, pfs_plugin_column_year_v1)
     set_field_year_v1, get_field_year_v1
 END_SERVICE_IMPLEMENTATION();
 
-BEGIN_SERVICE_IMPLEMENTATION(performance_schema, pfs_plugin_column_text_v1)
-    set_field_text_v1, get_field_text_v1
-END_SERVICE_IMPLEMENTATION();
-
 /* clang-format on */
