@@ -1,16 +1,15 @@
-/* Copyright (c) 2008, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2008, 2020, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -32,11 +31,7 @@
 #include "my_inttypes.h"
 #include "my_macros.h"
 #include "my_sys.h"
-#include "mysql/strings/m_ctype.h"
 #include "mysys_err.h"
-#include "sql/mysqld.h"
-#include "storage/perfschema/mysql_server_telemetry_metrics_service_imp.h"
-#include "storage/perfschema/mysql_server_telemetry_traces_service_imp.h"
 #include "storage/perfschema/pfs.h"
 #include "storage/perfschema/pfs_account.h"
 #include "storage/perfschema/pfs_builtin_memory.h"
@@ -52,14 +47,12 @@
 #include "storage/perfschema/pfs_host.h"
 #include "storage/perfschema/pfs_instr.h"
 #include "storage/perfschema/pfs_instr_class.h"
-#include "storage/perfschema/pfs_metrics_service_imp.h"
 #include "storage/perfschema/pfs_plugin_table.h"
 #include "storage/perfschema/pfs_prepared_stmt.h"
 #include "storage/perfschema/pfs_program.h"
 #include "storage/perfschema/pfs_setup_actor.h"
 #include "storage/perfschema/pfs_setup_object.h"
 #include "storage/perfschema/pfs_timer.h"
-#include "storage/perfschema/pfs_tls_channel.h"
 #include "storage/perfschema/pfs_user.h"
 #include "template_utils.h"
 
@@ -67,12 +60,10 @@ PFS_global_param pfs_param;
 
 PFS_table_stat PFS_table_stat::g_reset_template;
 
-static void cleanup_performance_schema();
-void cleanup_instrument_config();
+static void cleanup_performance_schema(void);
+void cleanup_instrument_config(void);
 
 void pre_initialize_performance_schema() {
-  record_main_thread_id();
-
   pfs_initialized = false;
 
   init_all_builtin_memory_class();
@@ -91,6 +82,9 @@ void pre_initialize_performance_schema() {
     in the instrumented code, to explicitly cleanup the instrumentation.
   */
   THR_PFS = nullptr;
+  for (int i = 0; i < THR_PFS_NUM_KEYS; ++i) {
+    THR_PFS_contexts[i] = nullptr;
+  }
 }
 
 int initialize_performance_schema(
@@ -106,8 +100,7 @@ int initialize_performance_schema(
     PSI_memory_bootstrap **memory_bootstrap,
     PSI_error_bootstrap **error_bootstrap,
     PSI_data_lock_bootstrap **data_lock_bootstrap,
-    PSI_system_bootstrap **system_bootstrap,
-    PSI_tls_channel_bootstrap **tls_channel_bootstrap) {
+    PSI_system_bootstrap **system_bootstrap) {
   bool init_failed = false;
 
   *thread_bootstrap = nullptr;
@@ -126,7 +119,6 @@ int initialize_performance_schema(
   *error_bootstrap = nullptr;
   *data_lock_bootstrap = nullptr;
   *system_bootstrap = nullptr;
-  *tls_channel_bootstrap = nullptr;
 
   pfs_enabled = param->m_enabled;
 
@@ -163,8 +155,7 @@ int initialize_performance_schema(
       init_account_hash(param) || init_digest(param) ||
       init_digest_hash(param) || init_program(param) ||
       init_program_hash(param) || init_prepared_stmt(param) ||
-      init_meter_class(param->m_meter_class_sizing) ||
-      init_metric_class(param->m_metric_class_sizing) || init_error(param)) {
+      init_error(param)) {
     /*
       The performance schema initialization failed.
       Free the memory used, and disable the instrumentation.
@@ -181,8 +172,6 @@ int initialize_performance_schema(
         param->m_consumer_events_stages_history_enabled;
     flag_events_stages_history_long =
         param->m_consumer_events_stages_history_long_enabled;
-    flag_events_statements_cpu =
-        param->m_consumer_events_statements_cpu_enabled;
     flag_events_statements_current =
         param->m_consumer_events_statements_current_enabled;
     flag_events_statements_history =
@@ -208,7 +197,6 @@ int initialize_performance_schema(
     flag_events_stages_current = false;
     flag_events_stages_history = false;
     flag_events_stages_history_long = false;
-    flag_events_statements_cpu = false;
     flag_events_statements_current = false;
     flag_events_statements_history = false;
     flag_events_statements_history_long = false;
@@ -244,7 +232,6 @@ int initialize_performance_schema(
       *error_bootstrap = &pfs_error_bootstrap;
       *data_lock_bootstrap = &pfs_data_lock_bootstrap;
       *system_bootstrap = &pfs_system_bootstrap;
-      *tls_channel_bootstrap = &pfs_tls_channel_bootstrap;
     }
   }
 
@@ -260,44 +247,6 @@ int initialize_performance_schema(
   */
   init_pfs_plugin_table();
 
-  /*
-     Initialize TLS channel instrumentation data structures
-    This must be done:
-    - after the memory allocation for rwlock instrumentation,
-      so that rwlock LOCK_pfs_tls_channel gets instrumented
-      (if the instrumentation is enabled),
-    - Even if the RWLOCK LOCK_pfs_tls_channels ends up not instrumented,
-       it still needs to be initialized.
-  */
-  init_pfs_tls_channels_instrumentation();
-
-  /*
-    Initialize telemetry metrics instrument service.
-    This must be done:
-    - after the memory allocation for rwlock instrumentation,
-      so that rwlock LOCK_pfs_metrics gets instrumented
-      (if the instrumentation is enabled),
-    - Even if the rwlock LOCK_pfs_metrics ends up not instrumented,
-       it still needs to be initialized.
-  */
-  initialize_mysql_server_metrics_instrument_service();
-
-  /*
-    Initialize telemetry metrics service.
-  */
-  initialize_mysql_server_telemetry_metrics_service();
-
-  /*
-    Initialize telemetry tracing service.
-    This must be done:
-    - after the memory allocation for mutex instrumentation,
-      so that mutex LOCK_pfs_tracing_callback gets instrumented
-      (if the instrumentation is enabled),
-    - Even if the mutex LOCK_pfs_tracing_callback ends up not instrumented,
-       it still needs to be initialized.
-  */
-  initialize_mysql_server_telemetry_traces_service();
-
   if (init_failed) {
     return 1;
   }
@@ -305,7 +254,7 @@ int initialize_performance_schema(
   return 0;
 }
 
-static void cleanup_performance_schema() {
+static void cleanup_performance_schema(void) {
   /*
     my.cnf options
   */
@@ -357,10 +306,6 @@ static void cleanup_performance_schema() {
     find_XXX_class(key)
     will return PSI_NOT_INSTRUMENTED
   */
-  cleanup_mysql_server_telemetry_metrics_service();
-  cleanup_mysql_server_metrics_instrument_service();
-  cleanup_mysql_server_telemetry_traces_service();
-  cleanup_pfs_tls_channels_instrumentation();
   cleanup_pfs_plugin_table();
   cleanup_error();
   cleanup_program();
@@ -375,20 +320,17 @@ static void cleanup_performance_schema() {
   cleanup_statement_class();
   cleanup_socket_class();
   cleanup_memory_class();
-  cleanup_meter_class();
-  cleanup_metric_class();
 
   cleanup_instruments();
 }
 
-void shutdown_performance_schema() {
+void shutdown_performance_schema(void) {
   pfs_initialized = false;
 
   /* disable everything, especially for this thread. */
   flag_events_stages_current = false;
   flag_events_stages_history = false;
   flag_events_stages_history_long = false;
-  flag_events_statements_cpu = false;
   flag_events_statements_current = false;
   flag_events_statements_history = false;
   flag_events_statements_history_long = false;
@@ -442,11 +384,11 @@ void cleanup_instrument_config() {
 */
 
 int add_pfs_instr_to_array(const char *name, const char *value) {
-  const size_t name_length = strlen(name);
-  const size_t value_length = strlen(value);
+  size_t name_length = strlen(name);
+  size_t value_length = strlen(value);
 
   /* Allocate structure plus string buffers plus null terminators */
-  auto *e = (PFS_instr_config *)my_malloc(
+  PFS_instr_config *e = (PFS_instr_config *)my_malloc(
       PSI_NOT_INSTRUMENTED,
       sizeof(PFS_instr_config) + name_length + 1 + value_length + 1,
       MYF(MY_WME));
