@@ -1,17 +1,16 @@
 /*
-  Copyright (c) 2019, 2024, Oracle and/or its affiliates.
+  Copyright (c) 2019, 2020, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,13 +28,12 @@
 
 #include "storage/perfschema/table_binary_log_transaction_compression_stats.h"
 
-#include <assert.h>
 #include <cmath>
 
 #include <stddef.h>
 
 #include "my_compiler.h"
-
+#include "my_dbug.h"
 #include "sql/binlog/global.h"
 #include "sql/binlog/monitoring/context.h"
 #include "sql/field.h"
@@ -50,7 +48,7 @@ struct st_binary_log_transaction_compression_stats {
   std::vector<binlog::monitoring::Compression_stats *> stats;
 
   void clear() {
-    for (auto *x : stats) delete x;
+    for (auto x : stats) delete x;
     stats.clear();
   }
 
@@ -134,7 +132,7 @@ PFS_engine_table_share table_binary_log_transaction_compression_stats::m_share =
     {
         &pfs_truncatable_acl,
         &table_binary_log_transaction_compression_stats::create,
-        nullptr, /* write_row */
+        NULL, /* write_row */
         table_binary_log_transaction_compression_stats::delete_all_rows,
         table_binary_log_transaction_compression_stats::get_row_count,
         sizeof(pos_t), /* ref length */
@@ -156,9 +154,9 @@ table_binary_log_transaction_compression_stats::
     : PFS_engine_table(&m_share, &m_pos), m_pos(0), m_next_pos(0) {}
 
 table_binary_log_transaction_compression_stats::
-    ~table_binary_log_transaction_compression_stats() = default;
+    ~table_binary_log_transaction_compression_stats() {}
 
-void table_binary_log_transaction_compression_stats::reset_position() {
+void table_binary_log_transaction_compression_stats::reset_position(void) {
   m_pos.m_index = 0;
   m_next_pos.m_index = 0;
 }
@@ -172,9 +170,9 @@ int table_binary_log_transaction_compression_stats::delete_all_rows() {
   return 0;
 }
 
-int table_binary_log_transaction_compression_stats::rnd_next() {
+int table_binary_log_transaction_compression_stats::rnd_next(void) {
   m_rows.update();
-  const uint row_count = get_row_count();
+  uint row_count = get_row_count();
   for (m_pos.set_at(&m_next_pos); m_pos.m_index < row_count; m_pos.next()) {
     m_next_pos.set_after(&m_pos);
     return 0;
@@ -212,8 +210,8 @@ int table_binary_log_transaction_compression_stats::read_row_values(
            last_trx_timestamp) = row->get_last_transaction_stats();
 
   for (; (f = *fields); fields++) {
-    if (read_all || bitmap_is_set(table->read_set, f->field_index())) {
-      switch (f->field_index()) {
+    if (read_all || bitmap_is_set(table->read_set, f->field_index)) {
+      switch (f->field_index) {
         case 0: /** LOG_TYPE */
         {
           set_field_enum(f, row->get_log_type());
@@ -221,10 +219,10 @@ int table_binary_log_transaction_compression_stats::read_row_values(
         }
         case 1: /** COMPRESSION_TYPE */
         {
-          const std::string s_type =
-              mysql::binlog::event::compression::type_to_string(
+          std::string s_type =
+              binary_log::transaction::compression::type_to_string(
                   row->get_type());
-          set_field_varchar_utf8mb4(f, s_type.c_str(), s_type.size());
+          set_field_varchar_utf8(f, s_type.c_str(), s_type.size());
           break;
         }
         case 2: /** TRANSACTION_COUNTER */
@@ -271,7 +269,7 @@ int table_binary_log_transaction_compression_stats::read_row_values(
           set_field_timestamp(f, last_trx_timestamp);
           break;
         default:
-          assert(false); /* purecov: inspected */
+          DBUG_ASSERT(false); /* purecov: inspected */
       }
     }
   }
