@@ -1,4 +1,4 @@
-/* Copyright (c) 2010, 2022, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2020, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -28,9 +28,9 @@
 
 #include "storage/perfschema/table_esgs_by_account_by_event_name.h"
 
-#include <assert.h>
 #include <stddef.h>
 
+#include "my_dbug.h"
 #include "my_thread.h"
 #include "sql/field.h"
 #include "sql/plugin_table.h"
@@ -167,10 +167,10 @@ int table_esgs_by_account_by_event_name::rnd_pos(const void *pos) {
   return HA_ERR_RECORD_DELETED;
 }
 
-int table_esgs_by_account_by_event_name::index_init(uint idx [[maybe_unused]],
-                                                    bool) {
+int table_esgs_by_account_by_event_name::index_init(
+    uint idx MY_ATTRIBUTE((unused)), bool) {
   PFS_index_esgs_by_account_by_event_name *result = nullptr;
-  assert(idx == 0);
+  DBUG_ASSERT(idx == 0);
   result = PFS_NEW(PFS_index_esgs_by_account_by_event_name);
   m_opened_index = result;
   m_index = result;
@@ -238,21 +238,21 @@ int table_esgs_by_account_by_event_name::read_row_values(TABLE *table,
   Field *f;
 
   /* Set the null bits */
-  assert(table->s->null_bytes == 1);
+  DBUG_ASSERT(table->s->null_bytes == 1);
   buf[0] = 0;
 
   for (; (f = *fields); fields++) {
-    if (read_all || bitmap_is_set(table->read_set, f->field_index())) {
-      switch (f->field_index()) {
+    if (read_all || bitmap_is_set(table->read_set, f->field_index)) {
+      switch (f->field_index) {
         case 0: /* USER */
         case 1: /* HOST */
-          m_row.m_account.set_nullable_field(f->field_index(), f);
+          m_row.m_account.set_field(f->field_index, f);
           break;
         case 2: /* EVENT_NAME */
           m_row.m_event_name.set_field(f);
           break;
         default: /* 3, ... COUNT/SUM/MIN/AVG/MAX */
-          m_row.m_stat.set_field(f->field_index() - 3, f);
+          m_row.m_stat.set_field(f->field_index - 3, f);
           break;
       }
     }
