@@ -1,16 +1,15 @@
-/* Copyright (c) 2010, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2020, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,9 +27,9 @@
 
 #include "storage/perfschema/table_esgs_global_by_event_name.h"
 
-#include <assert.h>
 #include <stddef.h>
 
+#include "my_dbug.h"
 #include "my_thread.h"
 #include "sql/field.h"
 #include "sql/plugin_table.h"
@@ -92,7 +91,7 @@ PFS_engine_table *table_esgs_global_by_event_name::create(
   return new table_esgs_global_by_event_name();
 }
 
-int table_esgs_global_by_event_name::delete_all_rows() {
+int table_esgs_global_by_event_name::delete_all_rows(void) {
   reset_events_stages_by_thread();
   reset_events_stages_by_account();
   reset_events_stages_by_user();
@@ -101,7 +100,7 @@ int table_esgs_global_by_event_name::delete_all_rows() {
   return 0;
 }
 
-ha_rows table_esgs_global_by_event_name::get_row_count() {
+ha_rows table_esgs_global_by_event_name::get_row_count(void) {
   return stage_class_max;
 }
 
@@ -110,14 +109,14 @@ table_esgs_global_by_event_name::table_esgs_global_by_event_name()
   m_normalizer = time_normalizer::get_stage();
 }
 
-void table_esgs_global_by_event_name::reset_position() {
+void table_esgs_global_by_event_name::reset_position(void) {
   m_pos = 1;
   m_next_pos = 1;
 }
 
 int table_esgs_global_by_event_name::rnd_init(bool) { return 0; }
 
-int table_esgs_global_by_event_name::rnd_next() {
+int table_esgs_global_by_event_name::rnd_next(void) {
   PFS_stage_class *stage_class;
 
   if (global_instr_class_stages_array == nullptr) {
@@ -152,17 +151,17 @@ int table_esgs_global_by_event_name::rnd_pos(const void *pos) {
   return HA_ERR_RECORD_DELETED;
 }
 
-int table_esgs_global_by_event_name::index_init(uint idx [[maybe_unused]],
+int table_esgs_global_by_event_name::index_init(uint idx MY_ATTRIBUTE((unused)),
                                                 bool) {
   PFS_index_esgs_global_by_event_name *result = nullptr;
-  assert(idx == 0);
+  DBUG_ASSERT(idx == 0);
   result = PFS_NEW(PFS_index_esgs_global_by_event_name);
   m_opened_index = result;
   m_index = result;
   return 0;
 }
 
-int table_esgs_global_by_event_name::index_next() {
+int table_esgs_global_by_event_name::index_next(void) {
   PFS_stage_class *stage_class;
 
   if (global_instr_class_stages_array == nullptr) {
@@ -210,16 +209,16 @@ int table_esgs_global_by_event_name::read_row_values(TABLE *table,
   Field *f;
 
   /* Set the null bits */
-  assert(table->s->null_bytes == 0);
+  DBUG_ASSERT(table->s->null_bytes == 0);
 
   for (; (f = *fields); fields++) {
-    if (read_all || bitmap_is_set(table->read_set, f->field_index())) {
-      switch (f->field_index()) {
+    if (read_all || bitmap_is_set(table->read_set, f->field_index)) {
+      switch (f->field_index) {
         case 0: /* NAME */
           m_row.m_event_name.set_field(f);
           break;
         default: /* 1, ... COUNT/SUM/MIN/AVG/MAX */
-          m_row.m_stat.set_field(f->field_index() - 1, f);
+          m_row.m_stat.set_field(f->field_index - 1, f);
           break;
       }
     }
