@@ -1,4 +1,4 @@
-/* Copyright (c) 2010, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2020, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -27,11 +27,10 @@
 
 #include "storage/perfschema/table_events_stages.h"
 
-#include <assert.h>
 #include <stddef.h>
 
 #include "my_compiler.h"
-
+#include "my_dbug.h"
 #include "my_thread.h"
 #include "sql/field.h"
 #include "sql/plugin_table.h"
@@ -198,7 +197,7 @@ table_events_stages_common::table_events_stages_common(
 int table_events_stages_common::make_row(PFS_events_stages *stage) {
   ulonglong timer_end;
 
-  auto *unsafe = (PFS_stage_class *)stage->m_class;
+  PFS_stage_class *unsafe = (PFS_stage_class *)stage->m_class;
   PFS_stage_class *klass = sanitize_stage_class(unsafe);
   if (unlikely(klass == nullptr)) {
     return HA_ERR_RECORD_DELETED;
@@ -219,8 +218,8 @@ int table_events_stages_common::make_row(PFS_events_stages *stage) {
   m_normalizer->to_pico(stage->m_timer_start, timer_end, &m_row.m_timer_start,
                         &m_row.m_timer_end, &m_row.m_timer_wait);
 
-  m_row.m_name = klass->m_name.str();
-  m_row.m_name_length = klass->m_name.length();
+  m_row.m_name = klass->m_name;
+  m_row.m_name_length = klass->m_name_length;
 
   make_source_column(stage->m_source_file, stage->m_source_line, m_row.m_source,
                      sizeof(m_row.m_source), m_row.m_source_length);
@@ -242,13 +241,13 @@ int table_events_stages_common::read_row_values(TABLE *table,
   Field *f;
 
   /* Set the null bits */
-  assert(table->s->null_bytes == 2);
+  DBUG_ASSERT(table->s->null_bytes == 2);
   buf[0] = 0;
   buf[1] = 0;
 
   for (; (f = *fields); fields++) {
-    if (read_all || bitmap_is_set(table->read_set, f->field_index())) {
-      switch (f->field_index()) {
+    if (read_all || bitmap_is_set(table->read_set, f->field_index)) {
+      switch (f->field_index) {
         case 0: /* THREAD_ID */
           set_field_ulonglong(f, m_row.m_thread_internal_id);
           break;
@@ -263,10 +262,10 @@ int table_events_stages_common::read_row_values(TABLE *table,
           }
           break;
         case 3: /* EVENT_NAME */
-          set_field_varchar_utf8mb4(f, m_row.m_name, m_row.m_name_length);
+          set_field_varchar_utf8(f, m_row.m_name, m_row.m_name_length);
           break;
         case 4: /* SOURCE */
-          set_field_varchar_utf8mb4(f, m_row.m_source, m_row.m_source_length);
+          set_field_varchar_utf8(f, m_row.m_source, m_row.m_source_length);
           break;
         case 5: /* TIMER_START */
           if (m_row.m_timer_start != 0) {
@@ -319,7 +318,7 @@ int table_events_stages_common::read_row_values(TABLE *table,
           }
           break;
         default:
-          assert(false);
+          DBUG_ASSERT(false);
       }
     }
   }
@@ -334,14 +333,14 @@ PFS_engine_table *table_events_stages_current::create(
 table_events_stages_current::table_events_stages_current()
     : table_events_stages_common(&m_share, &m_pos), m_pos(0), m_next_pos(0) {}
 
-void table_events_stages_current::reset_position() {
+void table_events_stages_current::reset_position(void) {
   m_pos.m_index = 0;
   m_next_pos.m_index = 0;
 }
 
 int table_events_stages_current::rnd_init(bool) { return 0; }
 
-int table_events_stages_current::rnd_next() {
+int table_events_stages_current::rnd_next(void) {
   PFS_thread *pfs_thread;
   PFS_events_stages *stage;
 
@@ -372,16 +371,17 @@ int table_events_stages_current::rnd_pos(const void *pos) {
   return HA_ERR_RECORD_DELETED;
 }
 
-int table_events_stages_current::index_init(uint idx [[maybe_unused]], bool) {
+int table_events_stages_current::index_init(uint idx MY_ATTRIBUTE((unused)),
+                                            bool) {
   PFS_index_events_stages *result;
-  assert(idx == 0);
+  DBUG_ASSERT(idx == 0);
   result = PFS_NEW(PFS_index_events_stages);
   m_opened_index = result;
   m_index = result;
   return 0;
 }
 
-int table_events_stages_current::index_next() {
+int table_events_stages_current::index_next(void) {
   PFS_thread *pfs_thread;
   PFS_events_stages *stage;
 
@@ -406,12 +406,12 @@ int table_events_stages_current::index_next() {
   return HA_ERR_END_OF_FILE;
 }
 
-int table_events_stages_current::delete_all_rows() {
+int table_events_stages_current::delete_all_rows(void) {
   reset_events_stages_current();
   return 0;
 }
 
-ha_rows table_events_stages_current::get_row_count() {
+ha_rows table_events_stages_current::get_row_count(void) {
   return global_thread_container.get_row_count();
 }
 
@@ -423,14 +423,14 @@ PFS_engine_table *table_events_stages_history::create(
 table_events_stages_history::table_events_stages_history()
     : table_events_stages_common(&m_share, &m_pos), m_pos(), m_next_pos() {}
 
-void table_events_stages_history::reset_position() {
+void table_events_stages_history::reset_position(void) {
   m_pos.reset();
   m_next_pos.reset();
 }
 
 int table_events_stages_history::rnd_init(bool) { return 0; }
 
-int table_events_stages_history::rnd_next() {
+int table_events_stages_history::rnd_next(void) {
   PFS_thread *pfs_thread;
   PFS_events_stages *stage;
   bool has_more_thread = true;
@@ -470,10 +470,10 @@ int table_events_stages_history::rnd_pos(const void *pos) {
   PFS_thread *pfs_thread;
   PFS_events_stages *stage;
 
-  assert(events_stages_history_per_thread != 0);
+  DBUG_ASSERT(events_stages_history_per_thread != 0);
   set_position(pos);
 
-  assert(m_pos.m_index_2 < events_stages_history_per_thread);
+  DBUG_ASSERT(m_pos.m_index_2 < events_stages_history_per_thread);
 
   pfs_thread = global_thread_container.get(m_pos.m_index_1);
   if (pfs_thread != nullptr) {
@@ -492,16 +492,17 @@ int table_events_stages_history::rnd_pos(const void *pos) {
   return HA_ERR_RECORD_DELETED;
 }
 
-int table_events_stages_history::index_init(uint idx [[maybe_unused]], bool) {
+int table_events_stages_history::index_init(uint idx MY_ATTRIBUTE((unused)),
+                                            bool) {
   PFS_index_events_stages *result;
-  assert(idx == 0);
+  DBUG_ASSERT(idx == 0);
   result = PFS_NEW(PFS_index_events_stages);
   m_opened_index = result;
   m_index = result;
   return 0;
 }
 
-int table_events_stages_history::index_next() {
+int table_events_stages_history::index_next(void) {
   PFS_thread *pfs_thread;
   PFS_events_stages *stage;
   bool has_more_thread = true;
@@ -543,12 +544,12 @@ int table_events_stages_history::index_next() {
   return HA_ERR_END_OF_FILE;
 }
 
-int table_events_stages_history::delete_all_rows() {
+int table_events_stages_history::delete_all_rows(void) {
   reset_events_stages_history();
   return 0;
 }
 
-ha_rows table_events_stages_history::get_row_count() {
+ha_rows table_events_stages_history::get_row_count(void) {
   return events_stages_history_per_thread *
          global_thread_container.get_row_count();
 }
@@ -561,14 +562,14 @@ PFS_engine_table *table_events_stages_history_long::create(
 table_events_stages_history_long::table_events_stages_history_long()
     : table_events_stages_common(&m_share, &m_pos), m_pos(0), m_next_pos(0) {}
 
-void table_events_stages_history_long::reset_position() {
+void table_events_stages_history_long::reset_position(void) {
   m_pos.m_index = 0;
   m_next_pos.m_index = 0;
 }
 
 int table_events_stages_history_long::rnd_init(bool) { return 0; }
 
-int table_events_stages_history_long::rnd_next() {
+int table_events_stages_history_long::rnd_next(void) {
   PFS_events_stages *stage;
   uint limit;
 
@@ -624,11 +625,11 @@ int table_events_stages_history_long::rnd_pos(const void *pos) {
   return make_row(stage);
 }
 
-int table_events_stages_history_long::delete_all_rows() {
+int table_events_stages_history_long::delete_all_rows(void) {
   reset_events_stages_history_long();
   return 0;
 }
 
-ha_rows table_events_stages_history_long::get_row_count() {
+ha_rows table_events_stages_history_long::get_row_count(void) {
   return events_stages_history_long_size;
 }
