@@ -1,4 +1,4 @@
-/* Copyright (c) 2010, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2020, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -27,11 +27,10 @@
 
 #include "storage/perfschema/table_events_transactions.h"
 
-#include <assert.h>
 #include <stddef.h>
 
 #include "my_compiler.h"
-
+#include "my_dbug.h"
 #include "my_thread.h"
 #include "sql/field.h"
 #include "sql/plugin_table.h"
@@ -58,7 +57,7 @@ Plugin_table table_events_transactions_current::m_table_def(
     "  EVENT_NAME VARCHAR(128) not null,\n"
     "  STATE ENUM('ACTIVE', 'COMMITTED', 'ROLLED BACK'),\n"
     "  TRX_ID BIGINT unsigned,\n"
-    "  GTID VARCHAR(90),\n"
+    "  GTID VARCHAR(64),\n"
     "  XID_FORMAT_ID INTEGER,\n"
     "  XID_GTRID VARCHAR(130),\n"
     "  XID_BQUAL VARCHAR(130),\n"
@@ -111,7 +110,7 @@ Plugin_table table_events_transactions_history::m_table_def(
     "  EVENT_NAME VARCHAR(128) not null,\n"
     "  STATE ENUM('ACTIVE', 'COMMITTED', 'ROLLED BACK'),\n"
     "  TRX_ID BIGINT unsigned,\n"
-    "  GTID VARCHAR(90),\n"
+    "  GTID VARCHAR(64),\n"
     "  XID_FORMAT_ID INTEGER,\n"
     "  XID_GTRID VARCHAR(130),\n"
     "  XID_BQUAL VARCHAR(130),\n"
@@ -164,7 +163,7 @@ Plugin_table table_events_transactions_history_long::m_table_def(
     "  EVENT_NAME VARCHAR(128) not null,\n"
     "  STATE ENUM('ACTIVE', 'COMMITTED', 'ROLLED BACK'),\n"
     "  TRX_ID BIGINT UNSIGNED,\n"
-    "  GTID VARCHAR(90),\n"
+    "  GTID VARCHAR(64),\n"
     "  XID_FORMAT_ID INTEGER,\n"
     "  XID_GTRID VARCHAR(130),\n"
     "  XID_BQUAL VARCHAR(130),\n"
@@ -234,7 +233,7 @@ int table_events_transactions_common::make_row(
     PFS_events_transactions *transaction) {
   ulonglong timer_end;
 
-  auto *unsafe = (PFS_transaction_class *)transaction->m_class;
+  PFS_transaction_class *unsafe = (PFS_transaction_class *)transaction->m_class;
   PFS_transaction_class *klass = sanitize_transaction_class(unsafe);
   if (unlikely(klass == nullptr)) {
     return HA_ERR_RECORD_DELETED;
@@ -255,18 +254,15 @@ int table_events_transactions_common::make_row(
   m_normalizer->to_pico(transaction->m_timer_start, timer_end,
                         &m_row.m_timer_start, &m_row.m_timer_end,
                         &m_row.m_timer_wait);
-  m_row.m_name = klass->m_name.str();
-  m_row.m_name_length = klass->m_name.length();
+  m_row.m_name = klass->m_name;
+  m_row.m_name_length = klass->m_name_length;
 
   make_source_column(transaction->m_source_file, transaction->m_source_line,
                      m_row.m_source, sizeof(m_row.m_source),
                      m_row.m_source_length);
 
-  /* A GTID consists of the TSID (transaction source id) and GNO
-     (transaction number).
-     The TSID consists of transaction source UUID and a user-defined tag
-     (empty by default).
-     The TSID is stored in transaction->m_tsid and the GNO is stored in
+  /* A GTID consists of the SID (source id) and GNO (transaction number).
+     The SID is stored in transaction->m_sid and the GNO is stored in
      transaction->m_gtid_spec.gno.
 
      On a master, the GTID is assigned when the transaction commit.
@@ -285,12 +281,13 @@ int table_events_transactions_common::make_row(
 
      The Gtid_specification contains the GNO, as well as a type code
      that specifies which of the three modes is currently in effect.
-     Given a TSID, it can generate the textual representation of the
+     Given a SID, it can generate the textual representation of the
      GTID.
   */
+  rpl_sid *sid = &transaction->m_sid;
   Gtid_specification *gtid_spec = &transaction->m_gtid_spec;
-  mysql::gtid::Tsid tsid(transaction->m_tsid);
-  m_row.m_gtid_length = gtid_spec->to_string(tsid, m_row.m_gtid);
+  m_row.m_gtid_length = gtid_spec->to_string(sid, m_row.m_gtid);
+
   m_row.m_xid = transaction->m_xid;
   m_row.m_isolation_level = transaction->m_isolation_level;
   m_row.m_read_only = transaction->m_read_only;
@@ -322,8 +319,8 @@ static const ulong XID_BUFFER_SIZE = XIDDATASIZE * 2 + 2 + 1;
 */
 static size_t xid_to_hex(char *buf, size_t buf_len, PSI_xid *xid, size_t offset,
                          size_t length) {
-  assert(buf_len >= XID_BUFFER_SIZE);
-  assert(offset + length <= XIDDATASIZE);
+  DBUG_ASSERT(buf_len >= XID_BUFFER_SIZE);
+  DBUG_ASSERT(offset + length <= XIDDATASIZE);
   *buf++ = '0';
   *buf++ = 'x';
   return bin_to_hex_str(buf, buf_len - 2, (char *)(xid->data + offset),
@@ -342,7 +339,7 @@ static size_t xid_to_hex(char *buf, size_t buf_len, PSI_xid *xid, size_t offset,
 */
 static void xid_store(Field *field, PSI_xid *xid, size_t offset,
                       size_t length) {
-  assert(!xid->is_null());
+  DBUG_ASSERT(!xid->is_null());
   if (xid_printable(xid, offset, length)) {
     field->store(xid->data + offset, length, &my_charset_bin);
   } else {
@@ -352,7 +349,7 @@ static void xid_store(Field *field, PSI_xid *xid, size_t offset,
     */
     char xid_buf[XID_BUFFER_SIZE];
 
-    const size_t xid_str_len =
+    size_t xid_str_len =
         xid_to_hex(xid_buf, sizeof(xid_buf), xid, offset, length);
     field->store(xid_buf, xid_str_len, &my_charset_bin);
   }
@@ -373,14 +370,14 @@ int table_events_transactions_common::read_row_values(TABLE *table,
   Field *f;
 
   /* Set the null bits */
-  assert(table->s->null_bytes == 3);
+  DBUG_ASSERT(table->s->null_bytes == 3);
   buf[0] = 0;
   buf[1] = 0;
   buf[2] = 0;
 
   for (; (f = *fields); fields++) {
-    if (read_all || bitmap_is_set(table->read_set, f->field_index())) {
-      switch (f->field_index()) {
+    if (read_all || bitmap_is_set(table->read_set, f->field_index)) {
+      switch (f->field_index) {
         case 0: /* THREAD_ID */
           set_field_ulonglong(f, m_row.m_thread_internal_id);
           break;
@@ -395,7 +392,7 @@ int table_events_transactions_common::read_row_values(TABLE *table,
           }
           break;
         case 3: /* EVENT_NAME */
-          set_field_varchar_utf8mb4(f, m_row.m_name, m_row.m_name_length);
+          set_field_varchar_utf8(f, m_row.m_name, m_row.m_name_length);
           break;
         case 4: /* STATE */
           set_field_enum(f, m_row.m_state);
@@ -408,7 +405,7 @@ int table_events_transactions_common::read_row_values(TABLE *table,
           }
           break;
         case 6: /* GTID */
-          set_field_varchar_utf8mb4(f, m_row.m_gtid, m_row.m_gtid_length);
+          set_field_varchar_utf8(f, m_row.m_gtid, m_row.m_gtid_length);
           break;
         case 7: /* XID_FORMAT_ID */
           if (!m_row.m_xa || m_row.m_xid.is_null()) {
@@ -441,7 +438,7 @@ int table_events_transactions_common::read_row_values(TABLE *table,
           }
           break;
         case 11: /* SOURCE */
-          set_field_varchar_utf8mb4(f, m_row.m_source, m_row.m_source_length);
+          set_field_varchar_utf8(f, m_row.m_source, m_row.m_source_length);
           break;
         case 12: /* TIMER_START */
           if (m_row.m_timer_start != 0) {
@@ -502,7 +499,7 @@ int table_events_transactions_common::read_row_values(TABLE *table,
           }
           break;
         default:
-          assert(false);
+          DBUG_ASSERT(false);
       }
     }
   }
@@ -519,14 +516,14 @@ table_events_transactions_current::table_events_transactions_current()
       m_pos(0),
       m_next_pos(0) {}
 
-void table_events_transactions_current::reset_position() {
+void table_events_transactions_current::reset_position(void) {
   m_pos.m_index = 0;
   m_next_pos.m_index = 0;
 }
 
 int table_events_transactions_current::rnd_init(bool) { return 0; }
 
-int table_events_transactions_current::rnd_next() {
+int table_events_transactions_current::rnd_next(void) {
   PFS_thread *pfs_thread;
   PFS_events_transactions *transaction;
   bool has_more_thread = true;
@@ -560,17 +557,17 @@ int table_events_transactions_current::rnd_pos(const void *pos) {
   return HA_ERR_RECORD_DELETED;
 }
 
-int table_events_transactions_current::index_init(uint idx [[maybe_unused]],
-                                                  bool) {
+int table_events_transactions_current::index_init(
+    uint idx MY_ATTRIBUTE((unused)), bool) {
   PFS_index_events_transactions *result;
-  assert(idx == 0);
+  DBUG_ASSERT(idx == 0);
   result = PFS_NEW(PFS_index_events_transactions);
   m_opened_index = result;
   m_index = result;
   return 0;
 }
 
-int table_events_transactions_current::index_next() {
+int table_events_transactions_current::index_next(void) {
   PFS_thread *pfs_thread;
   PFS_events_transactions *transaction;
   bool has_more_thread = true;
@@ -593,12 +590,12 @@ int table_events_transactions_current::index_next() {
   return HA_ERR_END_OF_FILE;
 }
 
-int table_events_transactions_current::delete_all_rows() {
+int table_events_transactions_current::delete_all_rows(void) {
   reset_events_transactions_current();
   return 0;
 }
 
-ha_rows table_events_transactions_current::get_row_count() {
+ha_rows table_events_transactions_current::get_row_count(void) {
   return global_thread_container.get_row_count();
 }
 
@@ -612,14 +609,14 @@ table_events_transactions_history::table_events_transactions_history()
       m_pos(),
       m_next_pos() {}
 
-void table_events_transactions_history::reset_position() {
+void table_events_transactions_history::reset_position(void) {
   m_pos.reset();
   m_next_pos.reset();
 }
 
 int table_events_transactions_history::rnd_init(bool) { return 0; }
 
-int table_events_transactions_history::rnd_next() {
+int table_events_transactions_history::rnd_next(void) {
   PFS_thread *pfs_thread;
   PFS_events_transactions *transaction;
   bool has_more_thread = true;
@@ -658,10 +655,10 @@ int table_events_transactions_history::rnd_pos(const void *pos) {
   PFS_thread *pfs_thread;
   PFS_events_transactions *transaction;
 
-  assert(events_transactions_history_per_thread != 0);
+  DBUG_ASSERT(events_transactions_history_per_thread != 0);
   set_position(pos);
 
-  assert(m_pos.m_index_2 < events_transactions_history_per_thread);
+  DBUG_ASSERT(m_pos.m_index_2 < events_transactions_history_per_thread);
 
   pfs_thread = global_thread_container.get(m_pos.m_index_1);
   if (pfs_thread != nullptr) {
@@ -679,17 +676,17 @@ int table_events_transactions_history::rnd_pos(const void *pos) {
   return HA_ERR_RECORD_DELETED;
 }
 
-int table_events_transactions_history::index_init(uint idx [[maybe_unused]],
-                                                  bool) {
+int table_events_transactions_history::index_init(
+    uint idx MY_ATTRIBUTE((unused)), bool) {
   PFS_index_events_transactions *result;
-  assert(idx == 0);
+  DBUG_ASSERT(idx == 0);
   result = PFS_NEW(PFS_index_events_transactions);
   m_opened_index = result;
   m_index = result;
   return 0;
 }
 
-int table_events_transactions_history::index_next() {
+int table_events_transactions_history::index_next(void) {
   PFS_thread *pfs_thread;
   PFS_events_transactions *transaction;
   bool has_more_thread = true;
@@ -732,12 +729,12 @@ int table_events_transactions_history::index_next() {
   return HA_ERR_END_OF_FILE;
 }
 
-int table_events_transactions_history::delete_all_rows() {
+int table_events_transactions_history::delete_all_rows(void) {
   reset_events_transactions_history();
   return 0;
 }
 
-ha_rows table_events_transactions_history::get_row_count() {
+ha_rows table_events_transactions_history::get_row_count(void) {
   return events_transactions_history_per_thread *
          global_thread_container.get_row_count();
 }
@@ -752,14 +749,14 @@ table_events_transactions_history_long::table_events_transactions_history_long()
       m_pos(0),
       m_next_pos(0) {}
 
-void table_events_transactions_history_long::reset_position() {
+void table_events_transactions_history_long::reset_position(void) {
   m_pos.m_index = 0;
   m_next_pos.m_index = 0;
 }
 
 int table_events_transactions_history_long::rnd_init(bool) { return 0; }
 
-int table_events_transactions_history_long::rnd_next() {
+int table_events_transactions_history_long::rnd_next(void) {
   PFS_events_transactions *transaction;
   uint limit;
 
@@ -815,11 +812,11 @@ int table_events_transactions_history_long::rnd_pos(const void *pos) {
   return make_row(transaction);
 }
 
-int table_events_transactions_history_long::delete_all_rows() {
+int table_events_transactions_history_long::delete_all_rows(void) {
   reset_events_transactions_history_long();
   return 0;
 }
 
-ha_rows table_events_transactions_history_long::get_row_count() {
+ha_rows table_events_transactions_history_long::get_row_count(void) {
   return events_transactions_history_long_size;
 }
