@@ -1,16 +1,15 @@
-/* Copyright (c) 2010, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2020, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,9 +27,9 @@
 
 #include "storage/perfschema/table_ews_global_by_event_name.h"
 
-#include <assert.h>
 #include <stddef.h>
 
+#include "my_dbug.h"
 #include "my_thread.h"
 #include "sql/field.h"
 #include "sql/plugin_table.h"
@@ -99,7 +98,7 @@ PFS_engine_table *table_ews_global_by_event_name::create(
   return new table_ews_global_by_event_name();
 }
 
-int table_ews_global_by_event_name::delete_all_rows() {
+int table_ews_global_by_event_name::delete_all_rows(void) {
   reset_events_waits_by_instance();
   reset_table_waits_by_table_handle();
   reset_table_waits_by_table();
@@ -107,7 +106,7 @@ int table_ews_global_by_event_name::delete_all_rows() {
   return 0;
 }
 
-ha_rows table_ews_global_by_event_name::get_row_count() {
+ha_rows table_ews_global_by_event_name::get_row_count(void) {
   return wait_class_max;
 }
 
@@ -117,12 +116,12 @@ table_ews_global_by_event_name::table_ews_global_by_event_name()
   m_normalizer = time_normalizer::get_wait();
 }
 
-void table_ews_global_by_event_name::reset_position() {
+void table_ews_global_by_event_name::reset_position(void) {
   m_pos.reset();
   m_next_pos.reset();
 }
 
-int table_ews_global_by_event_name::rnd_next() {
+int table_ews_global_by_event_name::rnd_next(void) {
   PFS_mutex_class *mutex_class;
   PFS_rwlock_class *rwlock_class;
   PFS_cond_class *cond_class;
@@ -235,12 +234,13 @@ int table_ews_global_by_event_name::rnd_pos(const void *pos) {
       }
       break;
     case pos_ews_global_by_event_name::VIEW_TABLE:
-      assert(m_pos.m_index_2 >= 1);
-      assert(m_pos.m_index_2 <= 2);
+      DBUG_ASSERT(m_pos.m_index_2 >= 1);
+      DBUG_ASSERT(m_pos.m_index_2 <= 2);
       if (m_pos.m_index_2 == 1) {
         return make_table_io_row(&global_table_io_class);
+      } else {
+        return make_table_lock_row(&global_table_lock_class);
       }
-      return make_table_lock_row(&global_table_lock_class);
       break;
     case pos_ews_global_by_event_name::VIEW_SOCKET:
       socket_class = find_socket_class(m_pos.m_index_2);
@@ -261,24 +261,24 @@ int table_ews_global_by_event_name::rnd_pos(const void *pos) {
       }
       break;
     default:
-      assert(false);
+      DBUG_ASSERT(false);
       break;
   }
 
   return HA_ERR_RECORD_DELETED;
 }
 
-int table_ews_global_by_event_name::index_init(uint idx [[maybe_unused]],
+int table_ews_global_by_event_name::index_init(uint idx MY_ATTRIBUTE((unused)),
                                                bool) {
   PFS_index_ews_global_by_event_name *result = nullptr;
-  assert(idx == 0);
+  DBUG_ASSERT(idx == 0);
   result = PFS_NEW(PFS_index_ews_global_by_event_name);
   m_opened_index = result;
   m_index = result;
   return 0;
 }
 
-int table_ews_global_by_event_name::index_next() {
+int table_ews_global_by_event_name::index_next(void) {
   PFS_mutex_class *mutex_class;
   PFS_rwlock_class *rwlock_class;
   PFS_cond_class *cond_class;
@@ -350,8 +350,9 @@ int table_ews_global_by_event_name::index_next() {
               m_next_pos.set_after(&m_pos);
               if (m_pos.m_index_2 == 1) {
                 return make_table_io_row(table_class);
+              } else {
+                return make_table_lock_row(table_class);
               }
-              return make_table_lock_row(table_class);
             }
             m_pos.set_after(&m_pos);
           }
@@ -509,16 +510,16 @@ int table_ews_global_by_event_name::read_row_values(TABLE *table,
   Field *f;
 
   /* Set the null bits */
-  assert(table->s->null_bytes == 0);
+  DBUG_ASSERT(table->s->null_bytes == 0);
 
   for (; (f = *fields); fields++) {
-    if (read_all || bitmap_is_set(table->read_set, f->field_index())) {
-      switch (f->field_index()) {
+    if (read_all || bitmap_is_set(table->read_set, f->field_index)) {
+      switch (f->field_index) {
         case 0: /* EVENT_NAME */
           m_row.m_event_name.set_field(f);
           break;
         default: /* 1, ... COUNT/SUM/MIN/AVG/MAX */
-          m_row.m_stat.set_field(f->field_index() - 1, f);
+          m_row.m_stat.set_field(f->field_index - 1, f);
           break;
       }
     }
