@@ -1,16 +1,15 @@
-/* Copyright (c) 2008, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2008, 2020, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -49,9 +48,13 @@ struct THR_LOCK;
 /** A row of PERFORMANCE_SCHEMA.FILE_SUMMARY_BY_INSTANCE. */
 struct row_file_summary_by_instance {
   /** Column FILE_NAME. */
-  PFS_file_name m_file_name;
+  const char *m_filename;
+  /** Length in bytes of @c m_filename. */
+  uint m_filename_length;
+
   /** Column EVENT_NAME. */
   PFS_event_name_row m_event_name;
+
   /** Column OBJECT_INSTANCE_BEGIN */
   const void *m_identity;
   /**
@@ -63,10 +66,10 @@ struct row_file_summary_by_instance {
 
 class PFS_index_file_summary_by_instance : public PFS_engine_index {
  public:
-  explicit PFS_index_file_summary_by_instance(PFS_engine_key *key_1)
+  PFS_index_file_summary_by_instance(PFS_engine_key *key_1)
       : PFS_engine_index(key_1) {}
 
-  ~PFS_index_file_summary_by_instance() override = default;
+  ~PFS_index_file_summary_by_instance() {}
 
   virtual bool match(const PFS_file *pfs) = 0;
 };
@@ -78,9 +81,9 @@ class PFS_index_file_summary_by_instance_by_instance
       : PFS_index_file_summary_by_instance(&m_key),
         m_key("OBJECT_INSTANCE_BEGIN") {}
 
-  ~PFS_index_file_summary_by_instance_by_instance() override = default;
+  ~PFS_index_file_summary_by_instance_by_instance() {}
 
-  bool match(const PFS_file *pfs) override;
+  bool match(const PFS_file *pfs);
 
  private:
   PFS_key_object_instance m_key;
@@ -92,9 +95,9 @@ class PFS_index_file_summary_by_instance_by_file_name
   PFS_index_file_summary_by_instance_by_file_name()
       : PFS_index_file_summary_by_instance(&m_key), m_key("FILE_NAME") {}
 
-  ~PFS_index_file_summary_by_instance_by_file_name() override = default;
+  ~PFS_index_file_summary_by_instance_by_file_name() {}
 
-  bool match(const PFS_file *pfs) override;
+  bool match(const PFS_file *pfs);
 
  private:
   PFS_key_file_name m_key;
@@ -106,9 +109,9 @@ class PFS_index_file_summary_by_instance_by_event_name
   PFS_index_file_summary_by_instance_by_event_name()
       : PFS_index_file_summary_by_instance(&m_key), m_key("EVENT_NAME") {}
 
-  ~PFS_index_file_summary_by_instance_by_event_name() override = default;
+  ~PFS_index_file_summary_by_instance_by_event_name() {}
 
-  bool match(const PFS_file *pfs) override;
+  bool match(const PFS_file *pfs);
 
  private:
   PFS_key_event_name m_key;
@@ -123,22 +126,22 @@ class table_file_summary_by_instance : public PFS_engine_table {
   static int delete_all_rows();
   static ha_rows get_row_count();
 
-  void reset_position() override;
+  virtual void reset_position(void);
 
-  int rnd_next() override;
-  int rnd_pos(const void *pos) override;
+  virtual int rnd_next();
+  virtual int rnd_pos(const void *pos);
 
-  int index_init(uint idx, bool sorted) override;
-  int index_next() override;
+  virtual int index_init(uint idx, bool sorted);
+  virtual int index_next();
 
  private:
-  int read_row_values(TABLE *table, unsigned char *buf, Field **fields,
-                      bool read_all) override;
+  virtual int read_row_values(TABLE *table, unsigned char *buf, Field **fields,
+                              bool read_all);
 
   table_file_summary_by_instance();
 
  public:
-  ~table_file_summary_by_instance() override = default;
+  ~table_file_summary_by_instance() {}
 
  private:
   int make_row(PFS_file *pfs);
