@@ -1,16 +1,15 @@
-/* Copyright (c) 2013, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2020, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,8 +27,7 @@
 
 #include "storage/perfschema/table_prepared_stmt_instances.h"
 
-#include <assert.h>
-
+#include "my_dbug.h"
 #include "my_thread.h"
 #include "sql/field.h"
 #include "sql/plugin_table.h"
@@ -62,7 +60,6 @@ Plugin_table table_prepared_stmt_instances::m_table_def(
     "                         'TRIGGER') DEFAULT NULL,\n"
     "  OWNER_OBJECT_SCHEMA varchar(64) DEFAULT NULL,\n"
     "  OWNER_OBJECT_NAME varchar(64) DEFAULT NULL,\n"
-    "  EXECUTION_ENGINE ENUM ('PRIMARY', 'SECONDARY'),\n"
     "  TIMER_PREPARE bigint(20) unsigned NOT NULL,\n"
     "  COUNT_REPREPARE bigint(20) unsigned NOT NULL,\n"
     "  COUNT_EXECUTE bigint(20) unsigned NOT NULL,\n"
@@ -89,10 +86,6 @@ Plugin_table table_prepared_stmt_instances::m_table_def(
     "  SUM_SORT_SCAN bigint(20) unsigned NOT NULL,\n"
     "  SUM_NO_INDEX_USED bigint(20) unsigned NOT NULL,\n"
     "  SUM_NO_GOOD_INDEX_USED bigint(20) unsigned NOT NULL,\n"
-    "  SUM_CPU_TIME BIGINT unsigned not null,\n"
-    "  MAX_CONTROLLED_MEMORY BIGINT unsigned not null,\n"
-    "  MAX_TOTAL_MEMORY BIGINT unsigned not null,\n"
-    "  COUNT_SECONDARY bigint(20) unsigned NOT NULL,\n"
     "  PRIMARY KEY (OBJECT_INSTANCE_BEGIN) USING HASH,\n"
     "  UNIQUE KEY (OWNER_THREAD_ID, OWNER_EVENT_ID) USING HASH,\n"
     "  KEY (STATEMENT_ID) USING HASH,\n"
@@ -192,12 +185,12 @@ PFS_engine_table *table_prepared_stmt_instances::create(
   return new table_prepared_stmt_instances();
 }
 
-int table_prepared_stmt_instances::delete_all_rows() {
+int table_prepared_stmt_instances::delete_all_rows(void) {
   reset_prepared_stmt_instances();
   return 0;
 }
 
-ha_rows table_prepared_stmt_instances::get_row_count() {
+ha_rows table_prepared_stmt_instances::get_row_count(void) {
   return global_prepared_stmt_container.get_row_count();
 }
 
@@ -206,12 +199,12 @@ table_prepared_stmt_instances::table_prepared_stmt_instances()
   m_normalizer = time_normalizer::get_statement();
 }
 
-void table_prepared_stmt_instances::reset_position() {
+void table_prepared_stmt_instances::reset_position(void) {
   m_pos = 0;
   m_next_pos = 0;
 }
 
-int table_prepared_stmt_instances::rnd_next() {
+int table_prepared_stmt_instances::rnd_next(void) {
   PFS_prepared_stmt *pfs;
 
   m_pos.set_at(&m_next_pos);
@@ -259,7 +252,7 @@ int table_prepared_stmt_instances::index_init(uint idx, bool) {
       result = PFS_NEW(PFS_index_prepared_stmt_instances_by_owner_object);
       break;
     default:
-      assert(false);
+      DBUG_ASSERT(false);
       break;
   }
 
@@ -268,7 +261,7 @@ int table_prepared_stmt_instances::index_init(uint idx, bool) {
   return 0;
 }
 
-int table_prepared_stmt_instances::index_next() {
+int table_prepared_stmt_instances::index_next(void) {
   PFS_prepared_stmt *pfs;
   bool has_more = true;
 
@@ -311,10 +304,17 @@ int table_prepared_stmt_instances::make_row(PFS_prepared_stmt *prepared_stmt) {
   }
 
   m_row.m_owner_object_type = prepared_stmt->m_owner_object_type;
-  m_row.m_owner_object_name = prepared_stmt->m_owner_object_name;
-  m_row.m_owner_object_schema = prepared_stmt->m_owner_object_schema;
 
-  m_row.m_secondary = prepared_stmt->m_secondary;
+  m_row.m_owner_object_name_length = prepared_stmt->m_owner_object_name_length;
+  if (m_row.m_owner_object_name_length > 0)
+    memcpy(m_row.m_owner_object_name, prepared_stmt->m_owner_object_name,
+           m_row.m_owner_object_name_length);
+
+  m_row.m_owner_object_schema_length =
+      prepared_stmt->m_owner_object_schema_length;
+  if (m_row.m_owner_object_schema_length > 0)
+    memcpy(m_row.m_owner_object_schema, prepared_stmt->m_owner_object_schema,
+           m_row.m_owner_object_schema_length);
 
   /* Get prepared statement prepare stats. */
   m_row.m_prepare_stat.set(m_normalizer, &prepared_stmt->m_prepare_stat);
@@ -339,12 +339,12 @@ int table_prepared_stmt_instances::read_row_values(TABLE *table,
   /*
     Set the null bits.
   */
-  assert(table->s->null_bytes == 1);
+  DBUG_ASSERT(table->s->null_bytes == 1);
   buf[0] = 0;
 
   for (; (f = *fields); fields++) {
-    if (read_all || bitmap_is_set(table->read_set, f->field_index())) {
-      switch (f->field_index()) {
+    if (read_all || bitmap_is_set(table->read_set, f->field_index)) {
+      switch (f->field_index) {
         case 0: /* OBJECT_INSTANCE_BEGIN */
           set_field_ulonglong(f, (intptr)m_row.m_identity);
           break;
@@ -353,8 +353,8 @@ int table_prepared_stmt_instances::read_row_values(TABLE *table,
           break;
         case 2: /* STATEMENT_NAME */
           if (m_row.m_stmt_name_length > 0)
-            set_field_varchar_utf8mb4(f, m_row.m_stmt_name,
-                                      m_row.m_stmt_name_length);
+            set_field_varchar_utf8(f, m_row.m_stmt_name,
+                                   m_row.m_stmt_name_length);
           else {
             f->set_null();
           }
@@ -384,22 +384,29 @@ int table_prepared_stmt_instances::read_row_values(TABLE *table,
           }
           break;
         case 7: /* OWNER_OBJECT_SCHEMA */
-          set_nullable_field_schema_name(f, &m_row.m_owner_object_schema);
+          if (m_row.m_owner_object_schema_length > 0)
+            set_field_varchar_utf8(f, m_row.m_owner_object_schema,
+                                   m_row.m_owner_object_schema_length);
+          else {
+            f->set_null();
+          }
           break;
         case 8: /* OWNER_OBJECT_NAME */
-          set_nullable_field_object_name(f, &m_row.m_owner_object_name);
+          if (m_row.m_owner_object_name_length > 0)
+            set_field_varchar_utf8(f, m_row.m_owner_object_name,
+                                   m_row.m_owner_object_name_length);
+          else {
+            f->set_null();
+          }
           break;
-        case 9: /* EXECUTION_ENGINE */
-          set_field_enum(f, m_row.m_secondary ? ENUM_SECONDARY : ENUM_PRIMARY);
-          break;
-        case 10: /* TIMER_PREPARE */
+        case 9: /* TIMER_PREPARE */
           m_row.m_prepare_stat.set_field(1, f);
           break;
-        case 11: /* COUNT_REPREPARE */
+        case 10: /* COUNT_REPREPARE */
           m_row.m_reprepare_stat.set_field(0, f);
           break;
-        default: /* 12, ... COUNT/SUM/MIN/AVG/MAX */
-          m_row.m_execute_stat.set_field(f->field_index() - 12, f);
+        default: /* 14, ... COUNT/SUM/MIN/AVG/MAX */
+          m_row.m_execute_stat.set_field(f->field_index - 11, f);
           break;
       }
     }
