@@ -1,16 +1,15 @@
-/* Copyright (c) 2008, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2008, 2020, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,9 +27,8 @@
 
 #include "storage/perfschema/table_setup_objects.h"
 
-#include <assert.h>
 #include "my_compiler.h"
-
+#include "my_dbug.h"
 #include "my_thread.h"
 #include "sql/field.h"
 #include "sql/plugin_table.h"
@@ -43,8 +41,6 @@
 #include "storage/perfschema/pfs_setup_object.h"
 #include "storage/perfschema/table_helper.h"
 
-struct CHARSET_INFO;
-
 THR_LOCK table_setup_objects::m_table_lock;
 
 Plugin_table table_setup_objects::m_table_def(
@@ -95,7 +91,7 @@ static int update_derived_flags() {
 
 bool PFS_index_setup_objects::match(PFS_setup_object *pfs) {
   if (m_fields >= 1) {
-    if (!m_key_1.match(pfs->m_key.m_object_type)) {
+    if (!m_key_1.match(pfs->get_object_type())) {
       return false;
     }
   }
@@ -115,6 +111,28 @@ bool PFS_index_setup_objects::match(PFS_setup_object *pfs) {
   return true;
 }
 
+bool PFS_index_setup_objects::match(row_setup_objects *row) {
+  if (m_fields >= 1) {
+    if (!m_key_1.match(row->m_object_type)) {
+      return false;
+    }
+  }
+
+  if (m_fields >= 2) {
+    if (!m_key_2.match(row->m_schema_name, row->m_schema_name_length)) {
+      return false;
+    }
+  }
+
+  if (m_fields >= 3) {
+    if (!m_key_3.match(row->m_object_name, row->m_object_name_length)) {
+      return false;
+    }
+  }
+
+  return true;
+}
+
 PFS_engine_table *table_setup_objects::create(PFS_engine_table_share *) {
   return new table_setup_objects();
 }
@@ -134,16 +152,16 @@ int table_setup_objects::write_row(PFS_engine_table *, TABLE *table,
   bool timed = true;
 
   for (; (f = *fields); fields++) {
-    if (bitmap_is_set(table->write_set, f->field_index())) {
-      switch (f->field_index()) {
+    if (bitmap_is_set(table->write_set, f->field_index)) {
+      switch (f->field_index) {
         case 0: /* OBJECT_TYPE */
           object_type = (enum_object_type)get_field_enum(f);
           break;
         case 1: /* OBJECT_SCHEMA */
-          object_schema = get_field_varchar_utf8mb4(f, &object_schema_data);
+          object_schema = get_field_varchar_utf8(f, &object_schema_data);
           break;
         case 2: /* OBJECT_NAME */
-          object_name = get_field_varchar_utf8mb4(f, &object_name_data);
+          object_name = get_field_varchar_utf8(f, &object_name_data);
           break;
         case 3: /* ENABLED */
           enabled_value = (enum_yes_no)get_field_enum(f);
@@ -152,7 +170,7 @@ int table_setup_objects::write_row(PFS_engine_table *, TABLE *table,
           timed_value = (enum_yes_no)get_field_enum(f);
           break;
         default:
-          assert(false);
+          DBUG_ASSERT(false);
       }
     }
   }
@@ -173,32 +191,18 @@ int table_setup_objects::write_row(PFS_engine_table *, TABLE *table,
     return HA_ERR_NO_REFERENCED_ROW;
   }
 
-  enabled = (enabled_value == ENUM_YES);
-  timed = (timed_value == ENUM_YES);
+  enabled = (enabled_value == ENUM_YES) ? true : false;
+  timed = (timed_value == ENUM_YES) ? true : false;
 
-  PFS_schema_name schema_value;
-  PFS_object_name object_value;
-  schema_value.set(object_schema->ptr(), object_schema->length());
-
-  /*
-    Collation rules for PFS_object_name depends on
-    what the object actually is.
-  */
-  if (object_type == OBJECT_TYPE_TABLE) {
-    object_value.set_as_table(object_name->ptr(), object_name->length());
-  } else {
-    object_value.set_as_routine(object_name->ptr(), object_name->length());
-  }
-
-  result = insert_setup_object(object_type, &schema_value, &object_value,
-                               enabled, timed);
+  result = insert_setup_object(object_type, object_schema, object_name, enabled,
+                               timed);
   if (result == 0) {
     result = update_derived_flags();
   }
   return result;
 }
 
-int table_setup_objects::delete_all_rows() {
+int table_setup_objects::delete_all_rows(void) {
   int result = reset_setup_object();
   if (result == 0) {
     result = update_derived_flags();
@@ -206,19 +210,19 @@ int table_setup_objects::delete_all_rows() {
   return result;
 }
 
-ha_rows table_setup_objects::get_row_count() {
+ha_rows table_setup_objects::get_row_count(void) {
   return global_setup_object_container.get_row_count();
 }
 
 table_setup_objects::table_setup_objects()
     : PFS_engine_table(&m_share, &m_pos), m_pos(0), m_next_pos(0) {}
 
-void table_setup_objects::reset_position() {
+void table_setup_objects::reset_position(void) {
   m_pos.m_index = 0;
   m_next_pos.m_index = 0;
 }
 
-int table_setup_objects::rnd_next() {
+int table_setup_objects::rnd_next(void) {
   PFS_setup_object *pfs;
 
   m_pos.set_at(&m_next_pos);
@@ -246,16 +250,16 @@ int table_setup_objects::rnd_pos(const void *pos) {
   return HA_ERR_RECORD_DELETED;
 }
 
-int table_setup_objects::index_init(uint idx [[maybe_unused]], bool) {
+int table_setup_objects::index_init(uint idx MY_ATTRIBUTE((unused)), bool) {
   PFS_index_setup_objects *result = nullptr;
-  assert(idx == 0);
+  DBUG_ASSERT(idx == 0);
   result = PFS_NEW(PFS_index_setup_objects);
   m_opened_index = result;
   m_index = result;
   return 0;
 }
 
-int table_setup_objects::index_next() {
+int table_setup_objects::index_next(void) {
   PFS_setup_object *pfs;
   bool has_more = true;
 
@@ -279,9 +283,11 @@ int table_setup_objects::make_row(PFS_setup_object *pfs) {
   pfs_optimistic_state lock;
   pfs->m_lock.begin_optimistic_lock(&lock);
 
-  m_row.m_object_type = pfs->m_key.m_object_type;
-  m_row.m_schema_name = pfs->m_key.m_schema_name;
-  m_row.m_object_name = pfs->m_key.m_object_name;
+  m_row.m_object_type = pfs->get_object_type();
+  memcpy(m_row.m_schema_name, pfs->m_schema_name, pfs->m_schema_name_length);
+  m_row.m_schema_name_length = pfs->m_schema_name_length;
+  memcpy(m_row.m_object_name, pfs->m_object_name, pfs->m_object_name_length);
+  m_row.m_object_name_length = pfs->m_object_name_length;
   m_row.m_enabled_ptr = &pfs->m_enabled;
   m_row.m_timed_ptr = &pfs->m_timed;
 
@@ -297,27 +303,27 @@ int table_setup_objects::read_row_values(TABLE *table, unsigned char *buf,
   Field *f;
 
   /* Set the null bits */
-  assert(table->s->null_bytes == 1);
+  DBUG_ASSERT(table->s->null_bytes == 1);
   buf[0] = 0;
 
   for (; (f = *fields); fields++) {
-    if (read_all || bitmap_is_set(table->read_set, f->field_index())) {
-      switch (f->field_index()) {
+    if (read_all || bitmap_is_set(table->read_set, f->field_index)) {
+      switch (f->field_index) {
         case 0: /* OBJECT_TYPE */
           set_field_enum(f, m_row.m_object_type);
           break;
         case 1: /* OBJECT_SCHEMA */
-          if (m_row.m_schema_name.length())
-            set_field_varchar_utf8mb4(f, m_row.m_schema_name.ptr(),
-                                      m_row.m_schema_name.length());
+          if (m_row.m_schema_name_length)
+            set_field_varchar_utf8(f, m_row.m_schema_name,
+                                   m_row.m_schema_name_length);
           else {
             f->set_null();
           }
           break;
         case 2: /* OBJECT_NAME */
-          if (m_row.m_object_name.length())
-            set_field_varchar_utf8mb4(f, m_row.m_object_name.ptr(),
-                                      m_row.m_object_name.length());
+          if (m_row.m_object_name_length)
+            set_field_varchar_utf8(f, m_row.m_object_name,
+                                   m_row.m_object_name_length);
           else {
             f->set_null();
           }
@@ -329,7 +335,7 @@ int table_setup_objects::read_row_values(TABLE *table, unsigned char *buf,
           set_field_enum(f, (*m_row.m_timed_ptr) ? ENUM_YES : ENUM_NO);
           break;
         default:
-          assert(false);
+          DBUG_ASSERT(false);
       }
     }
   }
@@ -344,15 +350,19 @@ int table_setup_objects::update_row_values(TABLE *table, const unsigned char *,
   enum_yes_no value;
 
   for (; (f = *fields); fields++) {
-    if (bitmap_is_set(table->write_set, f->field_index())) {
-      switch (f->field_index()) {
+    if (bitmap_is_set(table->write_set, f->field_index)) {
+      switch (f->field_index) {
+        case 0: /* OBJECT_TYPE */
+        case 1: /* OBJECT_SCHEMA */
+        case 2: /* OBJECT_NAME */
+          return HA_ERR_WRONG_COMMAND;
         case 3: /* ENABLED */
           value = (enum_yes_no)get_field_enum(f);
           /* Reject illegal enum values in ENABLED */
           if ((value != ENUM_YES) && (value != ENUM_NO)) {
             return HA_ERR_NO_REFERENCED_ROW;
           }
-          *m_row.m_enabled_ptr = (value == ENUM_YES);
+          *m_row.m_enabled_ptr = (value == ENUM_YES) ? true : false;
           break;
         case 4: /* TIMED */
           value = (enum_yes_no)get_field_enum(f);
@@ -360,10 +370,10 @@ int table_setup_objects::update_row_values(TABLE *table, const unsigned char *,
           if ((value != ENUM_YES) && (value != ENUM_NO)) {
             return HA_ERR_NO_REFERENCED_ROW;
           }
-          *m_row.m_timed_ptr = (value == ENUM_YES);
+          *m_row.m_timed_ptr = (value == ENUM_YES) ? true : false;
           break;
         default:
-          return HA_ERR_WRONG_COMMAND;
+          DBUG_ASSERT(false);
       }
     }
   }
@@ -374,8 +384,12 @@ int table_setup_objects::update_row_values(TABLE *table, const unsigned char *,
 
 int table_setup_objects::delete_row_values(TABLE *, const unsigned char *,
                                            Field **) {
-  int result = delete_setup_object(m_row.m_object_type, &m_row.m_schema_name,
-                                   &m_row.m_object_name);
+  CHARSET_INFO *cs = &my_charset_utf8mb4_bin;
+  enum_object_type object_type = m_row.m_object_type;
+  String object_schema(m_row.m_schema_name, m_row.m_schema_name_length, cs);
+  String object_name(m_row.m_object_name, m_row.m_object_name_length, cs);
+
+  int result = delete_setup_object(object_type, &object_schema, &object_name);
 
   if (result == 0) {
     result = update_derived_flags();
