@@ -1,16 +1,15 @@
-/* Copyright (c) 2017, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2020, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,9 +27,9 @@
 
 #include "storage/perfschema/table_setup_threads.h"
 
-#include <assert.h>
 #include <stddef.h>
 
+#include "my_dbug.h"
 #include "my_thread.h"
 #include "sql/field.h"
 #include "sql/plugin_table.h"
@@ -87,17 +86,17 @@ PFS_engine_table *table_setup_threads::create(PFS_engine_table_share *) {
   return new table_setup_threads();
 }
 
-ha_rows table_setup_threads::get_row_count() { return thread_class_max; }
+ha_rows table_setup_threads::get_row_count(void) { return thread_class_max; }
 
 table_setup_threads::table_setup_threads()
     : PFS_engine_table(&m_share, &m_pos), m_pos(1), m_next_pos(1) {}
 
-void table_setup_threads::reset_position() {
+void table_setup_threads::reset_position(void) {
   m_pos.m_index = 1;
   m_next_pos.m_index = 1;
 }
 
-int table_setup_threads::rnd_next() {
+int table_setup_threads::rnd_next(void) {
   PFS_thread_class *instr_class = nullptr;
 
   /* Do not advertise threads when disabled. */
@@ -138,10 +137,10 @@ int table_setup_threads::rnd_pos(const void *pos) {
   return HA_ERR_RECORD_DELETED;
 }
 
-int table_setup_threads::index_init(uint idx [[maybe_unused]], bool) {
+int table_setup_threads::index_init(uint idx MY_ATTRIBUTE((unused)), bool) {
   PFS_index_setup_threads *result;
 
-  assert(idx == 0);
+  DBUG_ASSERT(idx == 0);
   result = PFS_NEW(PFS_index_setup_threads);
 
   m_opened_index = result;
@@ -149,7 +148,7 @@ int table_setup_threads::index_init(uint idx [[maybe_unused]], bool) {
   return 0;
 }
 
-int table_setup_threads::index_next() {
+int table_setup_threads::index_next(void) {
   PFS_thread_class *instr_class = nullptr;
 
   /* Do not advertise threads when disabled. */
@@ -188,7 +187,7 @@ int table_setup_threads::read_row_values(TABLE *table, unsigned char *buf,
   int properties;
 
   /* Set the null bits */
-  assert(table->s->null_bytes == 1);
+  DBUG_ASSERT(table->s->null_bytes == 1);
   buf[0] = 0;
 
   /*
@@ -197,11 +196,11 @@ int table_setup_threads::read_row_values(TABLE *table, unsigned char *buf,
   */
 
   for (; (f = *fields); fields++) {
-    if (read_all || bitmap_is_set(table->read_set, f->field_index())) {
-      switch (f->field_index()) {
+    if (read_all || bitmap_is_set(table->read_set, f->field_index)) {
+      switch (f->field_index) {
         case 0: /* NAME */
-          set_field_varchar_utf8mb4(f, m_row.m_instr_class->m_name.str(),
-                                    m_row.m_instr_class->m_name.length());
+          set_field_varchar_utf8(f, m_row.m_instr_class->m_name,
+                                 m_row.m_instr_class->m_name_length);
           break;
         case 1: /* ENABLED */
           set_field_enum(f,
@@ -233,7 +232,7 @@ int table_setup_threads::read_row_values(TABLE *table, unsigned char *buf,
           }
           break;
         default:
-          assert(false);
+          DBUG_ASSERT(false);
       }
     }
   }
@@ -247,18 +246,24 @@ int table_setup_threads::update_row_values(TABLE *table, const unsigned char *,
   enum_yes_no value;
 
   for (; (f = *fields); fields++) {
-    if (bitmap_is_set(table->write_set, f->field_index())) {
-      switch (f->field_index()) {
+    if (bitmap_is_set(table->write_set, f->field_index)) {
+      switch (f->field_index) {
+        case 0: /* NAME */
+          return HA_ERR_WRONG_COMMAND;
         case 1: /* ENABLED */
           value = (enum_yes_no)get_field_enum(f);
-          m_row.m_instr_class->m_enabled = (value == ENUM_YES);
+          m_row.m_instr_class->m_enabled = (value == ENUM_YES) ? true : false;
           break;
         case 2: /* HISTORY */
           value = (enum_yes_no)get_field_enum(f);
-          m_row.m_instr_class->m_history = (value == ENUM_YES);
+          m_row.m_instr_class->m_history = (value == ENUM_YES) ? true : false;
           break;
-        default:
+        case 3: /* PROPERTIES */
+        case 4: /* VOLATILITY */
+        case 5: /* DOCUMENTATION */
           return HA_ERR_WRONG_COMMAND;
+        default:
+          DBUG_ASSERT(false);
       }
     }
   }
