@@ -1,16 +1,15 @@
-/* Copyright (c) 2015, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -72,16 +71,16 @@ struct row_status_by_user {
 struct pos_status_by_user : public PFS_double_index {
   pos_status_by_user() : PFS_double_index(0, 0) {}
 
-  inline void reset() {
+  inline void reset(void) {
     m_index_1 = 0;
     m_index_2 = 0;
   }
 
-  inline bool has_more_user() {
+  inline bool has_more_user(void) {
     return (m_index_1 < global_user_container.get_row_count());
   }
 
-  inline void next_user() {
+  inline void next_user(void) {
     m_index_1++;
     m_index_2 = 0;
   }
@@ -94,7 +93,7 @@ class PFS_index_status_by_user : public PFS_engine_index {
         m_key_1("USER"),
         m_key_2("VARIABLE_NAME") {}
 
-  ~PFS_index_status_by_user() override = default;
+  ~PFS_index_status_by_user() {}
 
   virtual bool match(PFS_user *pfs);
   virtual bool match(const Status_variable *pfs);
@@ -104,6 +103,18 @@ class PFS_index_status_by_user : public PFS_engine_index {
   PFS_key_variable_name m_key_2;
 };
 
+/**
+  Store and retrieve table state information for queries that reinstantiate
+  the table object.
+*/
+class table_status_by_user_context : public PFS_table_context {
+ public:
+  table_status_by_user_context(ulonglong current_version, bool restore)
+      : PFS_table_context(current_version,
+                          global_user_container.get_row_count(), restore,
+                          THR_PFS_SBU) {}
+};
+
 /** Table PERFORMANCE_SCHEMA.STATUS_BY_USER. */
 class table_status_by_user : public PFS_engine_table {
   typedef pos_status_by_user pos_t;
@@ -115,22 +126,22 @@ class table_status_by_user : public PFS_engine_table {
   static int delete_all_rows();
   static ha_rows get_row_count();
 
-  void reset_position() override;
+  virtual void reset_position(void);
 
-  int rnd_init(bool scan) override;
-  int rnd_next() override;
-  int rnd_pos(const void *pos) override;
+  virtual int rnd_init(bool scan);
+  virtual int rnd_next();
+  virtual int rnd_pos(const void *pos);
 
-  int index_init(uint idx, bool sorted) override;
-  int index_next() override;
+  virtual int index_init(uint idx, bool sorted);
+  virtual int index_next();
 
  protected:
-  int read_row_values(TABLE *table, unsigned char *buf, Field **fields,
-                      bool read_all) override;
+  virtual int read_row_values(TABLE *table, unsigned char *buf, Field **fields,
+                              bool read_all);
   table_status_by_user();
 
  public:
-  ~table_status_by_user() override = default;
+  ~table_status_by_user() {}
 
  protected:
   int make_row(PFS_user *user, const Status_variable *status_var);
@@ -151,6 +162,10 @@ class table_status_by_user : public PFS_engine_table {
   /** Next position. */
   pos_t m_next_pos;
 
+  /** Table context with global status array version and map of materialized
+   * threads. */
+  table_status_by_user_context *m_context;
+
   PFS_index_status_by_user *m_opened_index;
 };
 
