@@ -1,16 +1,15 @@
-/* Copyright (c) 2012, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2012, 2020, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,9 +27,9 @@
 
 #include "storage/perfschema/table_table_handles.h"
 
-#include <assert.h>
 #include <stddef.h>
 
+#include "my_dbug.h"
 #include "my_thread.h"
 #include "sql/field.h"
 #include "sql/plugin_table.h"
@@ -89,9 +88,6 @@ bool PFS_index_table_handles_by_object::match(PFS_table *pfs) {
   }
 
   PFS_table_share *share = sanitize_table_share(pfs->m_share);
-  if (share == nullptr) {
-    return false;
-  }
 
   if (m_fields >= 2) {
     if (!m_key_2.match(share)) {
@@ -138,21 +134,21 @@ PFS_engine_table *table_table_handles::create(PFS_engine_table_share *) {
   return new table_table_handles();
 }
 
-ha_rows table_table_handles::get_row_count() {
+ha_rows table_table_handles::get_row_count(void) {
   return global_table_container.get_row_count();
 }
 
 table_table_handles::table_table_handles()
     : PFS_engine_table(&m_share, &m_pos), m_pos(0), m_next_pos(0) {}
 
-void table_table_handles::reset_position() {
+void table_table_handles::reset_position(void) {
   m_pos.m_index = 0;
   m_next_pos.m_index = 0;
 }
 
 int table_table_handles::rnd_init(bool) { return 0; }
 
-int table_table_handles::rnd_next() {
+int table_table_handles::rnd_next(void) {
   PFS_table *pfs;
 
   m_pos.set_at(&m_next_pos);
@@ -193,7 +189,7 @@ int table_table_handles::index_init(uint idx, bool) {
       result = PFS_NEW(PFS_index_table_handles_by_owner);
       break;
     default:
-      assert(false);
+      DBUG_ASSERT(false);
       break;
   }
 
@@ -202,7 +198,7 @@ int table_table_handles::index_init(uint idx, bool) {
   return 0;
 }
 
-int table_table_handles::index_next() {
+int table_table_handles::index_next(void) {
   PFS_table *pfs;
 
   m_pos.set_at(&m_next_pos);
@@ -265,16 +261,16 @@ int table_table_handles::read_row_values(TABLE *table, unsigned char *buf,
   Field *f;
 
   /* Set the null bits */
-  assert(table->s->null_bytes == 1);
+  DBUG_ASSERT(table->s->null_bytes == 1);
   buf[0] = 0;
 
   for (; (f = *fields); fields++) {
-    if (read_all || bitmap_is_set(table->read_set, f->field_index())) {
-      switch (f->field_index()) {
+    if (read_all || bitmap_is_set(table->read_set, f->field_index)) {
+      switch (f->field_index) {
         case 0: /* OBJECT_TYPE */
         case 1: /* OBJECT_SCHEMA */
         case 2: /* OBJECT_NAME */
-          m_row.m_object.set_field(f->field_index(), f);
+          m_row.m_object.set_field(f->field_index, f);
           break;
         case 3: /* OBJECT_INSTANCE_BEGIN */
           set_field_ulonglong(f, (intptr)m_row.m_identity);
@@ -300,7 +296,7 @@ int table_table_handles::read_row_values(TABLE *table, unsigned char *buf,
           set_field_lock_type(f, m_row.m_external_lock);
           break;
         default:
-          assert(false);
+          DBUG_ASSERT(false);
       }
     }
   }
