@@ -1,16 +1,15 @@
-/* Copyright (c) 2010, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2020, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,9 +29,9 @@
 
 #include "my_config.h"
 
-#include <assert.h>
 #include <stddef.h>
 
+#include "my_dbug.h"
 #include "my_thread.h"
 #include "sql/field.h"
 #include "sql/plugin_table.h"
@@ -164,13 +163,13 @@ PFS_engine_table *table_tiws_by_index_usage::create(PFS_engine_table_share *) {
   return new table_tiws_by_index_usage();
 }
 
-int table_tiws_by_index_usage::delete_all_rows() {
+int table_tiws_by_index_usage::delete_all_rows(void) {
   reset_table_io_waits_by_table_handle();
   reset_table_io_waits_by_table();
   return 0;
 }
 
-ha_rows table_tiws_by_index_usage::get_row_count() {
+ha_rows table_tiws_by_index_usage::get_row_count(void) {
   return global_table_share_index_container.get_row_count();
 }
 
@@ -179,14 +178,14 @@ table_tiws_by_index_usage::table_tiws_by_index_usage()
   m_normalizer = time_normalizer::get_wait();
 }
 
-void table_tiws_by_index_usage::reset_position() {
+void table_tiws_by_index_usage::reset_position(void) {
   m_pos.reset();
   m_next_pos.reset();
 }
 
 int table_tiws_by_index_usage::rnd_init(bool) { return 0; }
 
-int table_tiws_by_index_usage::rnd_next() {
+int table_tiws_by_index_usage::rnd_next(void) {
   PFS_table_share *table_share;
   bool has_more_table = true;
 
@@ -194,8 +193,7 @@ int table_tiws_by_index_usage::rnd_next() {
     table_share =
         global_table_share_container.get(m_pos.m_index_1, &has_more_table);
     if (table_share != nullptr) {
-      const uint safe_key_count =
-          sanitize_index_count(table_share->m_key_count);
+      uint safe_key_count = sanitize_index_count(table_share->m_key_count);
       if (m_pos.m_index_2 < safe_key_count) {
         m_next_pos.set_after(&m_pos);
         return make_row(table_share, m_pos.m_index_2);
@@ -219,7 +217,7 @@ int table_tiws_by_index_usage::rnd_pos(const void *pos) {
 
   table_share = global_table_share_container.get(m_pos.m_index_1);
   if (table_share != nullptr) {
-    const uint safe_key_count = sanitize_index_count(table_share->m_key_count);
+    uint safe_key_count = sanitize_index_count(table_share->m_key_count);
     if (m_pos.m_index_2 < safe_key_count) {
       return make_row(table_share, m_pos.m_index_2);
     }
@@ -231,16 +229,17 @@ int table_tiws_by_index_usage::rnd_pos(const void *pos) {
   return HA_ERR_RECORD_DELETED;
 }
 
-int table_tiws_by_index_usage::index_init(uint idx [[maybe_unused]], bool) {
+int table_tiws_by_index_usage::index_init(uint idx MY_ATTRIBUTE((unused)),
+                                          bool) {
   PFS_index_tiws_by_index_usage *result = nullptr;
-  assert(idx == 0);
+  DBUG_ASSERT(idx == 0);
   result = PFS_NEW(PFS_index_tiws_by_index_usage);
   m_opened_index = result;
   m_index = result;
   return 0;
 }
 
-int table_tiws_by_index_usage::index_next() {
+int table_tiws_by_index_usage::index_next(void) {
   PFS_table_share *table_share;
   bool has_more_table = true;
 
@@ -249,8 +248,7 @@ int table_tiws_by_index_usage::index_next() {
         global_table_share_container.get(m_pos.m_index_1, &has_more_table);
     if (table_share != nullptr) {
       if (m_opened_index->match(table_share)) {
-        const uint safe_key_count =
-            sanitize_index_count(table_share->m_key_count);
+        uint safe_key_count = sanitize_index_count(table_share->m_key_count);
         for (; m_pos.m_index_2 <= MAX_INDEXES; m_pos.m_index_2++) {
           if (m_opened_index->match(table_share, m_pos.m_index_2)) {
             if (m_pos.m_index_2 < safe_key_count) {
@@ -281,7 +279,7 @@ int table_tiws_by_index_usage::make_row(PFS_table_share *pfs_share,
   PFS_table_share_index *pfs_index;
   pfs_optimistic_state lock;
 
-  assert(index <= MAX_INDEXES);
+  DBUG_ASSERT(index <= MAX_INDEXES);
 
   pfs_share->m_lock.begin_optimistic_lock(&lock);
 
@@ -315,17 +313,17 @@ int table_tiws_by_index_usage::read_row_values(TABLE *table, unsigned char *buf,
   Field *f;
 
   /* Set the null bits */
-  assert(table->s->null_bytes == 1);
+  DBUG_ASSERT(table->s->null_bytes == 1);
   buf[0] = 0;
 
   for (; (f = *fields); fields++) {
-    if (read_all || bitmap_is_set(table->read_set, f->field_index())) {
-      switch (f->field_index()) {
+    if (read_all || bitmap_is_set(table->read_set, f->field_index)) {
+      switch (f->field_index) {
         case 0: /* OBJECT_TYPE */
         case 1: /* SCHEMA_NAME */
         case 2: /* OBJECT_NAME */
         case 3: /* INDEX_NAME */
-          m_row.m_index.set_nullable_field(f->field_index(), f);
+          m_row.m_index.set_field(f->field_index, f);
           break;
         case 4: /* COUNT_STAR */
           set_field_ulonglong(f, m_row.m_stat.m_all.m_count);
@@ -433,7 +431,7 @@ int table_tiws_by_index_usage::read_row_values(TABLE *table, unsigned char *buf,
           set_field_ulonglong(f, m_row.m_stat.m_delete.m_max);
           break;
         default:
-          assert(false);
+          DBUG_ASSERT(false);
       }
     }
   }
