@@ -1,16 +1,15 @@
-/* Copyright (c) 2011, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2011, 2020, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -56,9 +55,9 @@ class PFS_index_users_by_user : public PFS_index_users {
  public:
   PFS_index_users_by_user() : PFS_index_users(&m_key), m_key("USER") {}
 
-  ~PFS_index_users_by_user() override = default;
+  ~PFS_index_users_by_user() {}
 
-  bool match(PFS_user *pfs) override;
+  virtual bool match(PFS_user *pfs);
 
  private:
   PFS_key_user m_key;
@@ -74,19 +73,19 @@ class table_users : public cursor_by_user {
   static int delete_all_rows();
 
  protected:
-  int read_row_values(TABLE *table, unsigned char *buf, Field **fields,
-                      bool read_all) override;
+  virtual int read_row_values(TABLE *table, unsigned char *buf, Field **fields,
+                              bool read_all);
 
  protected:
   table_users();
 
  public:
-  ~table_users() override = default;
+  ~table_users() {}
 
-  int index_init(uint idx, bool sorted) override;
+  int index_init(uint idx, bool sorted);
 
  private:
-  int make_row(PFS_user *pfs) override;
+  virtual int make_row(PFS_user *pfs);
 
   /** Table share lock. */
   static THR_LOCK m_table_lock;
