@@ -1,16 +1,15 @@
-/* Copyright (c) 2013, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2013, 2020, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,14 +28,13 @@
   Table USER_VARIABLES_BY_THREAD (declarations).
 */
 
-#include <assert.h>
 #include <stddef.h>
 #include <sys/types.h>
 
 #include "my_base.h"
-
+#include "my_dbug.h"
 #include "my_inttypes.h"
-#include "mysql/components/services/bits/psi_bits.h"
+#include "mysql/psi/psi_base.h"
 #include "prealloced_array.h"
 #include "storage/perfschema/pfs_engine_table.h"
 #include "storage/perfschema/pfs_instr.h"
@@ -55,11 +53,12 @@ struct THR_LOCK;
 
 struct User_variable {
  public:
-  User_variable() = default;
+  User_variable() {}
 
-  User_variable(const User_variable &uv) = default;
+  User_variable(const User_variable &uv)
+      : m_name(uv.m_name), m_value(uv.m_value) {}
 
-  ~User_variable() = default;
+  ~User_variable() {}
 
   PFS_variable_name_row m_name;
   PFS_user_variable_value_row m_value;
@@ -81,7 +80,7 @@ class User_variables {
   void materialize(PFS_thread *pfs, THD *thd);
 
   bool is_materialized(PFS_thread *pfs) {
-    assert(pfs != nullptr);
+    DBUG_ASSERT(pfs != nullptr);
     if (m_pfs != pfs) {
       return false;
     }
@@ -128,12 +127,12 @@ struct row_uvar_by_thread {
 struct pos_uvar_by_thread : public PFS_double_index {
   pos_uvar_by_thread() : PFS_double_index(0, 0) {}
 
-  inline void reset() {
+  inline void reset(void) {
     m_index_1 = 0;
     m_index_2 = 0;
   }
 
-  inline void next_thread() {
+  inline void next_thread(void) {
     m_index_1++;
     m_index_2 = 0;
   }
@@ -146,7 +145,7 @@ class PFS_index_uvar_by_thread : public PFS_engine_index {
         m_key_1("THREAD_ID"),
         m_key_2("VARIABLE_NAME") {}
 
-  ~PFS_index_uvar_by_thread() override = default;
+  ~PFS_index_uvar_by_thread() {}
 
   virtual bool match(PFS_thread *pfs);
   virtual bool match(const User_variable *pfs);
@@ -166,22 +165,22 @@ class table_uvar_by_thread : public PFS_engine_table {
   static PFS_engine_table *create(PFS_engine_table_share *);
   static ha_rows get_row_count();
 
-  void reset_position() override;
+  virtual void reset_position(void);
 
-  int rnd_next() override;
-  int rnd_pos(const void *pos) override;
+  virtual int rnd_next();
+  virtual int rnd_pos(const void *pos);
 
-  int index_init(uint idx, bool sorted) override;
-  int index_next() override;
+  virtual int index_init(uint idx, bool sorted);
+  virtual int index_next();
 
  protected:
-  int read_row_values(TABLE *table, unsigned char *buf, Field **fields,
-                      bool read_all) override;
+  virtual int read_row_values(TABLE *table, unsigned char *buf, Field **fields,
+                              bool read_all);
 
   table_uvar_by_thread();
 
  public:
-  ~table_uvar_by_thread() override { m_THD_cache.reset(); }
+  ~table_uvar_by_thread() { m_THD_cache.reset(); }
 
  protected:
   int materialize(PFS_thread *thread);
