@@ -1,16 +1,15 @@
-/* Copyright (c) 2011, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2011, 2020, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -31,12 +30,8 @@
 #include "storage/perfschema/pfs_instr.h"
 #include "storage/perfschema/pfs_stat.h"
 #include "storage/perfschema/pfs_user.h"
-#include "storage/perfschema/unittest/stub_digest.h"
 #include "storage/perfschema/unittest/stub_pfs_global.h"
 #include "storage/perfschema/unittest/stub_pfs_plugin_table.h"
-#include "storage/perfschema/unittest/stub_pfs_tls_channel.h"
-#include "storage/perfschema/unittest/stub_server_telemetry.h"
-#include "storage/perfschema/unittest/stub_telemetry_metrics.h"
 #include "unittest/mytap/tap.h"
 
 static void test_oom() {
@@ -59,7 +54,6 @@ static void test_oom() {
   PSI_error_bootstrap *error_boot;
   PSI_data_lock_bootstrap *data_lock_boot;
   PSI_system_bootstrap *system_boot;
-  PSI_tls_channel_bootstrap *tls_channel_boot;
 
   memset(&param, 0xFF, sizeof(param));
   param.m_enabled = true;
@@ -98,28 +92,10 @@ static void test_oom() {
   param.m_program_sizing = 0;
   param.m_statement_stack_sizing = 0;
   param.m_memory_class_sizing = 10;
-  param.m_meter_class_sizing = 5;
-  param.m_metric_class_sizing = 10;
   param.m_metadata_lock_sizing = 0;
   param.m_max_digest_length = 0;
   param.m_max_sql_text_length = 0;
   param.m_error_sizing = 0;
-  param.m_consumer_events_stages_current_enabled = false;
-  param.m_consumer_events_stages_history_enabled = false;
-  param.m_consumer_events_stages_history_long_enabled = false;
-  param.m_consumer_events_statements_cpu_enabled = false;
-  param.m_consumer_events_statements_current_enabled = false;
-  param.m_consumer_events_statements_history_enabled = false;
-  param.m_consumer_events_statements_history_long_enabled = false;
-  param.m_consumer_events_transactions_current_enabled = false;
-  param.m_consumer_events_transactions_history_enabled = false;
-  param.m_consumer_events_transactions_history_long_enabled = false;
-  param.m_consumer_events_waits_current_enabled = false;
-  param.m_consumer_events_waits_history_enabled = false;
-  param.m_consumer_events_waits_history_long_enabled = false;
-  param.m_consumer_global_instrumentation_enabled = false;
-  param.m_consumer_thread_instrumentation_enabled = false;
-  param.m_consumer_statement_digest_enabled = false;
 
   /* Setup */
 
@@ -131,17 +107,16 @@ static void test_oom() {
       &param, &thread_boot, &mutex_boot, &rwlock_boot, &cond_boot, &file_boot,
       &socket_boot, &table_boot, &mdl_boot, &idle_boot, &stage_boot,
       &statement_boot, &transaction_boot, &memory_boot, &error_boot,
-      &data_lock_boot, &system_boot, &tls_channel_boot);
+      &data_lock_boot, &system_boot);
   ok(rc == 0, "init ok");
   thread_service = (PSI_thread_service_t *)thread_boot->get_interface(
       PSI_CURRENT_THREAD_VERSION);
 
   PSI_thread_key thread_key_1;
-  PSI_thread_info all_thread[] = {{&thread_key_1, "T-1", "T-1", 0, 0, ""}};
+  PSI_thread_info all_thread[] = {{&thread_key_1, "T-1", 0, 0, ""}};
   thread_service->register_thread("test", all_thread, 1);
 
-  PSI_thread *thread_1 =
-      thread_service->new_thread(thread_key_1, 0, nullptr, 0);
+  PSI_thread *thread_1 = thread_service->new_thread(thread_key_1, nullptr, 0);
   thread_service->set_thread(thread_1);
 
   /* Tests */
