@@ -1,16 +1,15 @@
-/* Copyright (c) 2002, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2002, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    Without limiting anything contained in the foregoing, this file,
    which is part of C Driver for MySQL (Connector/C), is also subject to the
@@ -26,21 +25,23 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
-#include <algorithm>
-#include <cassert>
-#include <cstdint>
-#include <cstring>
+#include <string.h>
+#include <sys/types.h>
 
-#include "m_string.h"  // skip_trailing_space
+#include <algorithm>
+
+#include "m_ctype.h"
+#include "m_string.h"
 #include "my_compiler.h"
-#include "mysql/strings/m_ctype.h"
-#include "strings/m_ctype_internals.h"
+#include "my_dbug.h"
+#include "my_inttypes.h"
+#include "my_macros.h"
 #include "strings/str_uca_type.h"
 #include "template_utils.h"
 
 size_t my_caseup_str_mb(const CHARSET_INFO *cs, char *str) {
-  uint32_t l = 0;
-  const uint8_t *map = cs->to_upper;
+  uint32 l;
+  const uchar *map = cs->to_upper;
   char *str_orig = str;
 
   while (*str) {
@@ -48,7 +49,7 @@ size_t my_caseup_str_mb(const CHARSET_INFO *cs, char *str) {
     if ((l = my_ismbchar(cs, str, str + cs->mbmaxlen)))
       str += l;
     else {
-      *str = (char)map[(uint8_t)*str];
+      *str = (char)map[(uchar)*str];
       str++;
     }
   }
@@ -56,8 +57,8 @@ size_t my_caseup_str_mb(const CHARSET_INFO *cs, char *str) {
 }
 
 size_t my_casedn_str_mb(const CHARSET_INFO *cs, char *str) {
-  uint32_t l = 0;
-  const uint8_t *map = cs->to_lower;
+  uint32 l;
+  const uchar *map = cs->to_lower;
   char *str_orig = str;
 
   while (*str) {
@@ -65,7 +66,7 @@ size_t my_casedn_str_mb(const CHARSET_INFO *cs, char *str) {
     if ((l = my_ismbchar(cs, str, str + cs->mbmaxlen)))
       str += l;
     else {
-      *str = (char)map[(uint8_t)*str];
+      *str = (char)map[(uchar)*str];
       str++;
     }
   }
@@ -73,7 +74,7 @@ size_t my_casedn_str_mb(const CHARSET_INFO *cs, char *str) {
 }
 
 static inline const MY_UNICASE_CHARACTER *get_case_info_for_ch(
-    const CHARSET_INFO *cs, unsigned page, unsigned offs) {
+    const CHARSET_INFO *cs, uint page, uint offs) {
   const MY_UNICASE_CHARACTER *p;
   return cs->caseinfo ? ((p = cs->caseinfo->page[page]) ? &p[offs] : nullptr)
                       : nullptr;
@@ -83,26 +84,26 @@ static inline const MY_UNICASE_CHARACTER *get_case_info_for_ch(
   For character sets which don't change octet length in case conversion.
 */
 size_t my_caseup_mb(const CHARSET_INFO *cs, char *src, size_t srclen,
-                    char *dst [[maybe_unused]],
-                    size_t dstlen [[maybe_unused]]) {
-  uint32_t l = 0;
+                    char *dst MY_ATTRIBUTE((unused)),
+                    size_t dstlen MY_ATTRIBUTE((unused))) {
+  uint32 l;
   char *srcend = src + srclen;
-  const uint8_t *map = cs->to_upper;
+  const uchar *map = cs->to_upper;
 
-  assert(cs->caseup_multiply == 1);
-  assert(src == dst && srclen == dstlen);
-  assert(cs->mbmaxlen == 2);
+  DBUG_ASSERT(cs->caseup_multiply == 1);
+  DBUG_ASSERT(src == dst && srclen == dstlen);
+  DBUG_ASSERT(cs->mbmaxlen == 2);
 
   while (src < srcend) {
     if ((l = my_ismbchar(cs, src, srcend))) {
       const MY_UNICASE_CHARACTER *ch;
-      if ((ch = get_case_info_for_ch(cs, (uint8_t)src[0], (uint8_t)src[1]))) {
+      if ((ch = get_case_info_for_ch(cs, (uchar)src[0], (uchar)src[1]))) {
         *src++ = ch->toupper >> 8;
         *src++ = ch->toupper & 0xFF;
       } else
         src += l;
     } else {
-      *src = (char)map[(uint8_t)*src];
+      *src = (char)map[(uchar)*src];
       src++;
     }
   }
@@ -110,26 +111,26 @@ size_t my_caseup_mb(const CHARSET_INFO *cs, char *src, size_t srclen,
 }
 
 size_t my_casedn_mb(const CHARSET_INFO *cs, char *src, size_t srclen,
-                    char *dst [[maybe_unused]],
-                    size_t dstlen [[maybe_unused]]) {
-  uint32_t l = 0;
+                    char *dst MY_ATTRIBUTE((unused)),
+                    size_t dstlen MY_ATTRIBUTE((unused))) {
+  uint32 l;
   char *srcend = src + srclen;
-  const uint8_t *map = cs->to_lower;
+  const uchar *map = cs->to_lower;
 
-  assert(cs->casedn_multiply == 1);
-  assert(src == dst && srclen == dstlen);
-  assert(cs->mbmaxlen == 2);
+  DBUG_ASSERT(cs->casedn_multiply == 1);
+  DBUG_ASSERT(src == dst && srclen == dstlen);
+  DBUG_ASSERT(cs->mbmaxlen == 2);
 
   while (src < srcend) {
     if ((l = my_ismbchar(cs, src, srcend))) {
       const MY_UNICASE_CHARACTER *ch;
-      if ((ch = get_case_info_for_ch(cs, (uint8_t)src[0], (uint8_t)src[1]))) {
+      if ((ch = get_case_info_for_ch(cs, (uchar)src[0], (uchar)src[1]))) {
         *src++ = ch->tolower >> 8;
         *src++ = ch->tolower & 0xFF;
       } else
         src += l;
     } else {
-      *src = (char)map[(uint8_t)*src];
+      *src = (char)map[(uchar)*src];
       src++;
     }
   }
@@ -147,17 +148,17 @@ size_t my_casedn_mb(const CHARSET_INFO *cs, char *src, size_t srclen,
 */
 static size_t my_casefold_mb_varlen(const CHARSET_INFO *cs, char *src,
                                     size_t srclen, char *dst,
-                                    size_t dstlen [[maybe_unused]],
-                                    const uint8_t *map, size_t is_upper) {
+                                    size_t dstlen MY_ATTRIBUTE((unused)),
+                                    const uchar *map, size_t is_upper) {
   char *srcend = src + srclen, *dst0 = dst;
 
-  assert(cs->mbmaxlen == 2);
+  DBUG_ASSERT(cs->mbmaxlen == 2);
 
   while (src < srcend) {
     size_t mblen = my_ismbchar(cs, src, srcend);
     if (mblen) {
       const MY_UNICASE_CHARACTER *ch;
-      if ((ch = get_case_info_for_ch(cs, (uint8_t)src[0], (uint8_t)src[1]))) {
+      if ((ch = get_case_info_for_ch(cs, (uchar)src[0], (uchar)src[1]))) {
         int code = is_upper ? ch->toupper : ch->tolower;
         src += 2;
         if (code > 0xFF) *dst++ = code >> 8;
@@ -167,7 +168,7 @@ static size_t my_casefold_mb_varlen(const CHARSET_INFO *cs, char *src,
         *dst++ = *src++;
       }
     } else {
-      *dst++ = (char)map[(uint8_t)*src++];
+      *dst++ = (char)map[(uchar)*src++];
     }
   }
   return (size_t)(dst - dst0);
@@ -175,15 +176,15 @@ static size_t my_casefold_mb_varlen(const CHARSET_INFO *cs, char *src,
 
 size_t my_casedn_mb_varlen(const CHARSET_INFO *cs, char *src, size_t srclen,
                            char *dst, size_t dstlen) {
-  assert(dstlen >= srclen * cs->casedn_multiply);
-  assert(src != dst || cs->casedn_multiply == 1);
+  DBUG_ASSERT(dstlen >= srclen * cs->casedn_multiply);
+  DBUG_ASSERT(src != dst || cs->casedn_multiply == 1);
   return my_casefold_mb_varlen(cs, src, srclen, dst, dstlen, cs->to_lower, 0);
 }
 
 size_t my_caseup_mb_varlen(const CHARSET_INFO *cs, char *src, size_t srclen,
                            char *dst, size_t dstlen) {
-  assert(dstlen >= srclen * cs->caseup_multiply);
-  assert(src != dst || cs->caseup_multiply == 1);
+  DBUG_ASSERT(dstlen >= srclen * cs->caseup_multiply);
+  DBUG_ASSERT(src != dst || cs->caseup_multiply == 1);
   return my_casefold_mb_varlen(cs, src, srclen, dst, dstlen, cs->to_upper, 1);
 }
 
@@ -192,8 +193,8 @@ size_t my_caseup_mb_varlen(const CHARSET_INFO *cs, char *src, size_t srclen,
  */
 
 int my_strcasecmp_mb(const CHARSET_INFO *cs, const char *s, const char *t) {
-  uint32_t l = 0;
-  const uint8_t *map = cs->to_upper;
+  uint32 l;
+  const uchar *map = cs->to_upper;
 
   while (*s && *t) {
     /* Pointing after the '\0' is safe here. */
@@ -201,11 +202,11 @@ int my_strcasecmp_mb(const CHARSET_INFO *cs, const char *s, const char *t) {
       while (l--)
         if (*s++ != *t++) return 1;
     } else if (my_mbcharlen(cs, *t) != 1 ||
-               map[(uint8_t)*s++] != map[(uint8_t)*t++])
+               map[(uchar)*s++] != map[(uchar)*t++])
       return 1;
   }
   /* At least one of '*s' and '*t' is zero here. */
-  assert(!*t || !*s);
+  DBUG_ASSERT(!*t || !*s);
   return (*t != *s);
 }
 
@@ -219,17 +220,17 @@ int my_strcasecmp_mb(const CHARSET_INFO *cs, const char *s, const char *t) {
 #define INC_PTR(cs, A, B) \
   A += (my_ismbchar(cs, A, B) ? my_ismbchar(cs, A, B) : 1)
 
-#define likeconv(s, A) (uint8_t)(s)->sort_order[(uint8_t)(A)]
+#define likeconv(s, A) (uchar)(s)->sort_order[(uchar)(A)]
 
 static int my_wildcmp_mb_impl(const CHARSET_INFO *cs, const char *str,
                               const char *str_end, const char *wildstr_arg,
                               const char *wildend_arg, int escape, int w_one,
                               int w_many, int recurse_level) {
   int result = -1; /* Not found, using wildcards */
-  const uint8_t *wildstr = pointer_cast<const uint8_t *>(wildstr_arg);
-  const uint8_t *wildend = pointer_cast<const uint8_t *>(wildend_arg);
+  const uchar *wildstr = pointer_cast<const uchar *>(wildstr_arg);
+  const uchar *wildend = pointer_cast<const uchar *>(wildend_arg);
 
-  if (my_string_stack_guard && my_string_stack_guard(recurse_level)) return -1;
+  if (my_string_stack_guard && my_string_stack_guard(recurse_level)) return 1;
   while (wildstr != wildend) {
     while (*wildstr != w_many && *wildstr != w_one) {
       int l;
@@ -254,7 +255,9 @@ static int my_wildcmp_mb_impl(const CHARSET_INFO *cs, const char *str,
       if (wildstr == wildend) break;
     }
     if (*wildstr == w_many) { /* Found w_many */
-      uint8_t cmp = 0;
+      uchar cmp;
+      const uchar *mb = wildstr;
+      int mb_len = 0;
 
       wildstr++;
       /* Remove any '%' and '_' from the wild search string */
@@ -273,14 +276,14 @@ static int my_wildcmp_mb_impl(const CHARSET_INFO *cs, const char *str,
       if ((cmp = *wildstr) == escape && wildstr + 1 != wildend)
         cmp = *++wildstr;
 
-      const uint8_t *mb = wildstr;
-      unsigned mb_len = my_ismbchar(cs, wildstr, wildend);
-      INC_PTR(cs, wildstr, wildend); /* This is compared through cmp */
+      mb = wildstr;
+      mb_len = my_ismbchar(cs, wildstr, wildend);
+      INC_PTR(cs, wildstr, wildend); /* This is compared trough cmp */
       cmp = likeconv(cs, cmp);
       do {
         for (;;) {
           if (str >= str_end) return -1;
-          if (mb_len > 0) {
+          if (mb_len) {
             if (str + mb_len <= str_end && memcmp(str, mb, mb_len) == 0) {
               str += mb_len;
               break;
@@ -316,19 +319,19 @@ size_t my_numchars_mb(const CHARSET_INFO *cs, const char *pos,
                       const char *end) {
   size_t count = 0;
   while (pos < end) {
-    unsigned mb_len = 0;
+    uint mb_len;
     pos += (mb_len = my_ismbchar(cs, pos, end)) ? mb_len : 1;
     count++;
   }
   return count;
 }
 
-size_t my_charpos_mb3(const CHARSET_INFO *cs, const char *pos, const char *end,
-                      size_t length) {
+size_t my_charpos_mb(const CHARSET_INFO *cs, const char *pos, const char *end,
+                     size_t length) {
   const char *start = pos;
 
   while (length && pos < end) {
-    unsigned mb_len = 0;
+    uint mb_len;
     pos += (mb_len = my_ismbchar(cs, pos, end)) ? mb_len : 1;
     length--;
   }
@@ -343,8 +346,8 @@ size_t my_well_formed_len_mb(const CHARSET_INFO *cs, const char *b,
     my_wc_t wc;
     int mb_len;
 
-    if ((mb_len = cs->cset->mb_wc(cs, &wc, pointer_cast<const uint8_t *>(b),
-                                  pointer_cast<const uint8_t *>(e))) <= 0) {
+    if ((mb_len = cs->cset->mb_wc(cs, &wc, pointer_cast<const uchar *>(b),
+                                  pointer_cast<const uchar *>(e))) <= 0) {
       *error = b < e ? 1 : 0;
       break;
     }
@@ -354,9 +357,9 @@ size_t my_well_formed_len_mb(const CHARSET_INFO *cs, const char *b,
   return (size_t)(b - b_start);
 }
 
-unsigned my_instr_mb(const CHARSET_INFO *cs, const char *b, size_t b_length,
-                     const char *s, size_t s_length, my_match_t *match,
-                     unsigned nmatch) {
+uint my_instr_mb(const CHARSET_INFO *cs, const char *b, size_t b_length,
+                 const char *s, size_t s_length, my_match_t *match,
+                 uint nmatch) {
   const char *end, *b0;
   int res = 0;
 
@@ -376,16 +379,16 @@ unsigned my_instr_mb(const CHARSET_INFO *cs, const char *b, size_t b_length,
     while (b < end) {
       int mb_len;
 
-      if (!cs->coll->strnncoll(cs, pointer_cast<const uint8_t *>(b), s_length,
-                               pointer_cast<const uint8_t *>(s), s_length,
+      if (!cs->coll->strnncoll(cs, pointer_cast<const uchar *>(b), s_length,
+                               pointer_cast<const uchar *>(s), s_length,
                                false)) {
         if (nmatch) {
           match[0].beg = 0;
-          match[0].end = (unsigned)(b - b0);
+          match[0].end = (uint)(b - b0);
           match[0].mb_len = res;
           if (nmatch > 1) {
             match[1].beg = match[0].end;
-            match[1].end = match[0].end + (unsigned)s_length;
+            match[1].end = match[0].end + (uint)s_length;
             match[1].mb_len = 0; /* Not computed */
           }
         }
@@ -402,8 +405,8 @@ unsigned my_instr_mb(const CHARSET_INFO *cs, const char *b, size_t b_length,
 
 /* BINARY collations handlers for MB charsets */
 
-int my_strnncoll_mb_bin(const CHARSET_INFO *cs [[maybe_unused]],
-                        const uint8_t *s, size_t slen, const uint8_t *t,
+int my_strnncoll_mb_bin(const CHARSET_INFO *cs MY_ATTRIBUTE((unused)),
+                        const uchar *s, size_t slen, const uchar *t,
                         size_t tlen, bool t_is_prefix) {
   size_t len = std::min(slen, tlen);
   int cmp = len == 0 ? 0 : memcmp(s, t, len);
@@ -415,7 +418,7 @@ int my_strnncoll_mb_bin(const CHARSET_INFO *cs [[maybe_unused]],
 
   SYNOPSIS
     my_strnncollsp_mb_bin()
-    cs			Character set
+    cs			Chararacter set
     s			String to compare
     slen		Length of 's'
     t			String to compare
@@ -432,13 +435,14 @@ int my_strnncoll_mb_bin(const CHARSET_INFO *cs [[maybe_unused]],
     0 if strings are equal
 */
 
-int my_strnncollsp_mb_bin(const CHARSET_INFO *cs [[maybe_unused]],
-                          const uint8_t *a, size_t a_length, const uint8_t *b,
+int my_strnncollsp_mb_bin(const CHARSET_INFO *cs MY_ATTRIBUTE((unused)),
+                          const uchar *a, size_t a_length, const uchar *b,
                           size_t b_length) {
-  size_t length = 0;
-  int res = 0;
+  const uchar *end;
+  size_t length;
+  int res;
 
-  const uint8_t *end = a + (length = std::min(a_length, b_length));
+  end = a + (length = std::min(a_length, b_length));
   while (a < end) {
     if (*a++ != *b++) return ((int)a[-1] - (int)b[-1]);
   }
@@ -476,13 +480,13 @@ int my_strnncollsp_mb_bin(const CHARSET_INFO *cs [[maybe_unused]],
     switch (cs->cset->ismbchar(cs, (const char *)src, (const char *)se)) {   \
       case 4:                                                                \
         *dst++ = *src++;                                                     \
-        [[fallthrough]];                                                     \
+        /* fall through */                                                   \
       case 3:                                                                \
         *dst++ = *src++;                                                     \
-        [[fallthrough]];                                                     \
+        /* fall through */                                                   \
       case 2:                                                                \
         *dst++ = *src++;                                                     \
-        [[fallthrough]];                                                     \
+        /* fall through */                                                   \
       case 0:                                                                \
         *dst++ = *src++; /* byte in range 0x80..0xFF which is not MB head */ \
     }                                                                        \
@@ -493,15 +497,15 @@ int my_strnncollsp_mb_bin(const CHARSET_INFO *cs [[maybe_unused]],
   characters having multibyte weights *equal* to their codes:
   cp932, euckr, gb2312, sjis, eucjpms, ujis.
 */
-size_t my_strnxfrm_mb(const CHARSET_INFO *cs, uint8_t *dst, size_t dstlen,
-                      unsigned nweights, const uint8_t *src, size_t srclen,
-                      unsigned flags) {
-  uint8_t *d0 = dst;
-  uint8_t *de = dst + dstlen;
-  const uint8_t *se = src + srclen;
-  const uint8_t *sort_order = cs->sort_order;
+size_t my_strnxfrm_mb(const CHARSET_INFO *cs, uchar *dst, size_t dstlen,
+                      uint nweights, const uchar *src, size_t srclen,
+                      uint flags) {
+  uchar *d0 = dst;
+  uchar *de = dst + dstlen;
+  const uchar *se = src + srclen;
+  const uchar *sort_order = cs->sort_order;
 
-  assert(cs->mbmaxlen <= 4);
+  DBUG_ASSERT(cs->mbmaxlen <= 4);
 
   /*
     If "srclen" is smaller than both "dstlen" and "nweights"
@@ -530,7 +534,7 @@ size_t my_strnxfrm_mb(const CHARSET_INFO *cs, uint8_t *dst, size_t dstlen,
   }
 
   /*
-    A thorough loop, checking all possible limits:
+    A thourough loop, checking all possible limits:
     "se", "nweights" and "de".
   */
   for (; src < se && nweights && dst < de; nweights--) {
@@ -552,15 +556,15 @@ pad:
   return my_strxfrm_pad(cs, d0, dst, de, nweights, flags);
 }
 
-int my_strcasecmp_mb_bin(const CHARSET_INFO *cs [[maybe_unused]], const char *s,
-                         const char *t) {
+int my_strcasecmp_mb_bin(const CHARSET_INFO *cs MY_ATTRIBUTE((unused)),
+                         const char *s, const char *t) {
   return strcmp(s, t);
 }
 
-void my_hash_sort_mb_bin(const CHARSET_INFO *cs [[maybe_unused]],
-                         const uint8_t *key, size_t len, uint64_t *nr1,
-                         uint64_t *nr2) {
-  const uint8_t *pos = key;
+void my_hash_sort_mb_bin(const CHARSET_INFO *cs MY_ATTRIBUTE((unused)),
+                         const uchar *key, size_t len, uint64 *nr1,
+                         uint64 *nr2) {
+  const uchar *pos = key;
 
   /*
      Remove trailing spaces. We have to do this to be able to compare
@@ -570,8 +574,7 @@ void my_hash_sort_mb_bin(const CHARSET_INFO *cs [[maybe_unused]],
 
   for (; pos < key; pos++) {
     nr1[0] ^=
-        (uint64_t)((((unsigned)nr1[0] & 63) + nr2[0]) * ((unsigned)*pos)) +
-        (nr1[0] << 8);
+        (uint64)((((uint)nr1[0] & 63) + nr2[0]) * ((uint)*pos)) + (nr1[0] << 8);
     nr2[0] += 3;
   }
 }
@@ -596,7 +599,7 @@ void my_hash_sort_mb_bin(const CHARSET_INFO *cs [[maybe_unused]],
 */
 static void pad_max_char(const CHARSET_INFO *cs, char *str, char *end) {
   char buf[10];
-  int buflen = 0;
+  char buflen;
 
   if (!(cs->state & MY_CS_UNICODE)) {
     if (cs->max_sort_char <= 255) {
@@ -608,7 +611,7 @@ static void pad_max_char(const CHARSET_INFO *cs, char *str, char *end) {
       buflen = 2;
     } else {
       /* Currently, it's only for GB18030, so it must be a 4-byte char */
-      assert(cs->max_sort_char > 0xFFFFFF);
+      DBUG_ASSERT(cs->max_sort_char > 0xFFFFFF);
       buf[0] = cs->max_sort_char >> 24 & 0xFF;
       buf[1] = cs->max_sort_char >> 16 & 0xFF;
       buf[2] = cs->max_sort_char >> 8 & 0xFF;
@@ -616,15 +619,14 @@ static void pad_max_char(const CHARSET_INFO *cs, char *str, char *end) {
       buflen = 4;
     }
   } else {
-    buflen =
-        cs->cset->wc_mb(cs, cs->max_sort_char, pointer_cast<uint8_t *>(buf),
-                        pointer_cast<uint8_t *>(buf) + sizeof(buf));
+    buflen = cs->cset->wc_mb(cs, cs->max_sort_char, (uchar *)buf,
+                             (uchar *)buf + sizeof(buf));
   }
 
-  assert(buflen > 0);
+  DBUG_ASSERT(buflen > 0);
   do {
     if ((str + buflen) <= end) {
-      /* Enough space for the character */
+      /* Enough space for the characer */
       memcpy(str, buf, buflen);
       str += buflen;
     } else {
@@ -658,7 +660,7 @@ bool my_like_range_mb(const CHARSET_INFO *cs, const char *ptr,
                       size_t ptr_length, char escape, char w_one, char w_many,
                       size_t res_length, char *min_str, char *max_str,
                       size_t *min_length, size_t *max_length) {
-  unsigned mb_len = 0;
+  uint mb_len;
   const char *end = ptr + ptr_length;
   char *min_org = min_str;
   char *min_end = min_str + res_length;
@@ -730,10 +732,10 @@ bool my_like_range_mb(const CHARSET_INFO *cs, const char *ptr,
         'ab\min\min\min\min' and 'ab\max\max\max\max'.
 
       */
-      const MY_UCA_INFO::flags_type *contraction_flags = nullptr;
+      const char *contraction_flags = nullptr;
       if (cs->uca) contraction_flags = cs->uca->contraction_flags;
       if (contraction_flags && ptr + 1 < end &&
-          my_uca_can_be_contraction_head(contraction_flags, (uint8_t)*ptr)) {
+          my_uca_can_be_contraction_head(contraction_flags, (uchar)*ptr)) {
         /* Ptr[0] is a contraction head. */
 
         if (ptr[1] == w_one || ptr[1] == w_many) {
@@ -753,10 +755,9 @@ bool my_like_range_mb(const CHARSET_INFO *cs, const char *ptr,
           is not a contraction, then we put only ptr[0],
           and continue with ptr[1] on the next loop.
         */
-        if (my_uca_can_be_contraction_tail(contraction_flags,
-                                           (uint8_t)ptr[1]) &&
+        if (my_uca_can_be_contraction_tail(contraction_flags, (uchar)ptr[1]) &&
             my_uca_contraction2_weight(cs->uca->contraction_nodes,
-                                       (uint8_t)ptr[0], ptr[1])) {
+                                       (uchar)ptr[0], ptr[1])) {
           /* Contraction found */
           if (maxcharlen == 1 || min_str + 1 >= min_end) {
             /* Both contraction parts don't fit, quit */
@@ -819,8 +820,8 @@ bool my_like_range_generic(const CHARSET_INFO *cs, const char *ptr,
   for (; charlen > 0; charlen--) {
     my_wc_t wc, wc2;
     int res;
-    if ((res = cs->cset->mb_wc(cs, &wc, pointer_cast<const uint8_t *>(ptr),
-                               pointer_cast<const uint8_t *>(end))) <= 0) {
+    if ((res = cs->cset->mb_wc(cs, &wc, pointer_cast<const uchar *>(ptr),
+                               pointer_cast<const uchar *>(end))) <= 0) {
       if (res == MY_CS_ILSEQ) /* Bad sequence */
         return true;          /* min_length and max_length are not important */
       break;                  /* End of the string */
@@ -828,39 +829,37 @@ bool my_like_range_generic(const CHARSET_INFO *cs, const char *ptr,
     ptr += res;
 
     if (wc == (my_wc_t)escape) {
-      if ((res = cs->cset->mb_wc(cs, &wc, pointer_cast<const uint8_t *>(ptr),
-                                 pointer_cast<const uint8_t *>(end))) <= 0) {
+      if ((res = cs->cset->mb_wc(cs, &wc, pointer_cast<const uchar *>(ptr),
+                                 pointer_cast<const uchar *>(end))) <= 0) {
         if (res == MY_CS_ILSEQ)
           return true; /* min_length and max_length are not important */
-                       /*
-                          End of the string: Escape is the last character.
-                          Put escape as a normal character.
-                          We'll will leave the loop on the next iteration.
-                       */
+        /*
+           End of the string: Escape is the last character.
+           Put escape as a normal character.
+           We'll will leave the loop on the next iteration.
+        */
       } else
         ptr += res;
 
       /* Put escape character to min_str and max_str  */
-      if ((res = cs->cset->wc_mb(cs, wc, pointer_cast<uint8_t *>(min_str),
-                                 pointer_cast<uint8_t *>(min_end))) <= 0)
+      if ((res = cs->cset->wc_mb(cs, wc, (uchar *)min_str, (uchar *)min_end)) <=
+          0)
         goto pad_set_lengths; /* No space */
       min_str += res;
 
-      if ((res = cs->cset->wc_mb(cs, wc, pointer_cast<uint8_t *>(max_str),
-                                 pointer_cast<uint8_t *>(max_end))) <= 0)
+      if ((res = cs->cset->wc_mb(cs, wc, (uchar *)max_str, (uchar *)max_end)) <=
+          0)
         goto pad_set_lengths; /* No space */
       max_str += res;
       continue;
     } else if (wc == (my_wc_t)w_one) {
-      if ((res = cs->cset->wc_mb(cs, cs->min_sort_char,
-                                 pointer_cast<uint8_t *>(min_str),
-                                 pointer_cast<uint8_t *>(min_end))) <= 0)
+      if ((res = cs->cset->wc_mb(cs, cs->min_sort_char, (uchar *)min_str,
+                                 (uchar *)min_end)) <= 0)
         goto pad_set_lengths;
       min_str += res;
 
-      if ((res = cs->cset->wc_mb(cs, cs->max_sort_char,
-                                 pointer_cast<uint8_t *>(max_str),
-                                 pointer_cast<uint8_t *>(max_end))) <= 0)
+      if ((res = cs->cset->wc_mb(cs, cs->max_sort_char, (uchar *)max_str,
+                                 (uchar *)max_end)) <= 0)
         goto pad_set_lengths;
       max_str += res;
       continue;
@@ -876,13 +875,13 @@ bool my_like_range_generic(const CHARSET_INFO *cs, const char *ptr,
       goto pad_min_max;
     }
 
-    const MY_UCA_INFO::flags_type *contraction_flags = nullptr;
+    const char *contraction_flags = nullptr;
     if (cs->uca) contraction_flags = cs->uca->contraction_flags;
     if (contraction_flags &&
         my_uca_can_be_contraction_head(contraction_flags, wc) &&
-        (res = cs->cset->mb_wc(cs, &wc2, pointer_cast<const uint8_t *>(ptr),
-                               pointer_cast<const uint8_t *>(end))) > 0) {
-      const uint16_t *weight = nullptr;
+        (res = cs->cset->mb_wc(cs, &wc2, pointer_cast<const uchar *>(ptr),
+                               pointer_cast<const uchar *>(end))) > 0) {
+      const uint16 *weight;
       if ((wc2 == (my_wc_t)w_one || wc2 == (my_wc_t)w_many)) {
         /* Contraction head followed by a wildcard */
         *min_length = *max_length = res_length;
@@ -904,13 +903,13 @@ bool my_like_range_generic(const CHARSET_INFO *cs, const char *ptr,
         charlen--;
 
         /* Put contraction head */
-        if ((res = cs->cset->wc_mb(cs, wc, pointer_cast<uint8_t *>(min_str),
-                                   pointer_cast<uint8_t *>(min_end))) <= 0)
+        if ((res = cs->cset->wc_mb(cs, wc, (uchar *)min_str,
+                                   (uchar *)min_end)) <= 0)
           goto pad_set_lengths;
         min_str += res;
 
-        if ((res = cs->cset->wc_mb(cs, wc, pointer_cast<uint8_t *>(max_str),
-                                   pointer_cast<uint8_t *>(max_end))) <= 0)
+        if ((res = cs->cset->wc_mb(cs, wc, (uchar *)max_str,
+                                   (uchar *)max_end)) <= 0)
           goto pad_set_lengths;
         max_str += res;
         wc = wc2; /* Prepare to put contraction tail */
@@ -918,12 +917,12 @@ bool my_like_range_generic(const CHARSET_INFO *cs, const char *ptr,
     }
 
     /* Normal character, or contraction tail */
-    if ((res = cs->cset->wc_mb(cs, wc, pointer_cast<uint8_t *>(min_str),
-                               pointer_cast<uint8_t *>(min_end))) <= 0)
+    if ((res = cs->cset->wc_mb(cs, wc, (uchar *)min_str, (uchar *)min_end)) <=
+        0)
       goto pad_set_lengths;
     min_str += res;
-    if ((res = cs->cset->wc_mb(cs, wc, pointer_cast<uint8_t *>(max_str),
-                               pointer_cast<uint8_t *>(max_end))) <= 0)
+    if ((res = cs->cset->wc_mb(cs, wc, (uchar *)max_str, (uchar *)max_end)) <=
+        0)
       goto pad_set_lengths;
     max_str += res;
   }
@@ -959,10 +958,10 @@ static int my_wildcmp_mb_bin_impl(const CHARSET_INFO *cs, const char *str,
                                   const char *wildend_arg, int escape,
                                   int w_one, int w_many, int recurse_level) {
   int result = -1; /* Not found, using wildcards */
-  const uint8_t *wildstr = pointer_cast<const uint8_t *>(wildstr_arg);
-  const uint8_t *wildend = pointer_cast<const uint8_t *>(wildend_arg);
+  const uchar *wildstr = pointer_cast<const uchar *>(wildstr_arg);
+  const uchar *wildend = pointer_cast<const uchar *>(wildend_arg);
 
-  if (my_string_stack_guard && my_string_stack_guard(recurse_level)) return -1;
+  if (my_string_stack_guard && my_string_stack_guard(recurse_level)) return 1;
   while (wildstr != wildend) {
     while (*wildstr != w_many && *wildstr != w_one) {
       int l;
@@ -971,7 +970,7 @@ static int my_wildcmp_mb_bin_impl(const CHARSET_INFO *cs, const char *str,
         if (str + l > str_end || memcmp(str, wildstr, l) != 0) return 1;
         str += l;
         wildstr += l;
-      } else if (str == str_end || *wildstr++ != static_cast<uint8_t>(*str++))
+      } else if (str == str_end || *wildstr++ != static_cast<uchar>(*str++))
         return (1); /* No match */
       if (wildstr == wildend)
         return (str != str_end); /* Match if both are at end */
@@ -987,6 +986,7 @@ static int my_wildcmp_mb_bin_impl(const CHARSET_INFO *cs, const char *str,
     }
     if (*wildstr == w_many) { /* Found w_many */
       int cmp;
+      const uchar *mb = wildstr;
       int mb_len = 0;
 
       wildstr++;
@@ -1006,9 +1006,9 @@ static int my_wildcmp_mb_bin_impl(const CHARSET_INFO *cs, const char *str,
       if ((cmp = *wildstr) == escape && wildstr + 1 != wildend)
         cmp = *++wildstr;
 
-      const uint8_t *mb = wildstr;
+      mb = wildstr;
       mb_len = my_ismbchar(cs, wildstr, wildend);
-      INC_PTR(cs, wildstr, wildend); /* This is compared through cmp */
+      INC_PTR(cs, wildstr, wildend); /* This is compared trough cmp */
       do {
         for (;;) {
           if (str >= str_end) return -1;
@@ -1018,7 +1018,7 @@ static int my_wildcmp_mb_bin_impl(const CHARSET_INFO *cs, const char *str,
               break;
             }
           } else if (!my_ismbchar(cs, str, str_end) &&
-                     static_cast<uint8_t>(*str) == cmp) {
+                     static_cast<uchar>(*str) == cmp) {
             str++;
             break;
           }
@@ -1293,9 +1293,9 @@ size_t my_numcells_mb(const CHARSET_INFO *cs, const char *b, const char *e) {
 
   while (b < e) {
     int mb_len;
-    unsigned pg = 0;
-    if ((mb_len = cs->cset->mb_wc(cs, &wc, pointer_cast<const uint8_t *>(b),
-                                  pointer_cast<const uint8_t *>(e))) <= 0 ||
+    uint pg;
+    if ((mb_len = cs->cset->mb_wc(cs, &wc, pointer_cast<const uchar *>(b),
+                                  pointer_cast<const uchar *>(e))) <= 0 ||
         wc > 0xFFFF) {
       /*
         Let's think a wrong sequence takes 1 dysplay cell.
@@ -1319,8 +1319,8 @@ size_t my_numcells_mb(const CHARSET_INFO *cs, const char *b, const char *e) {
   return clen;
 }
 
-int my_mb_ctype_mb(const CHARSET_INFO *cs, int *ctype, const uint8_t *s,
-                   const uint8_t *e) {
+int my_mb_ctype_mb(const CHARSET_INFO *cs, int *ctype, const uchar *s,
+                   const uchar *e) {
   my_wc_t wc;
   int res = cs->cset->mb_wc(cs, &wc, s, e);
   if (res <= 0 || wc > 0xFFFF)
