@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -122,14 +122,14 @@ class CacheStorageTest : public ::testing::Test, public Test_MDL_context_owner {
     mdl_destroy();
   }
 
-  void SetUp() override {
+  virtual void SetUp() {
     m_init.SetUp();
     // Mark this as a dd system thread to skip MDL checks/asserts in the dd
     // cache.
     thd()->system_thread = SYSTEM_THREAD_DD_INITIALIZE;
-#ifndef NDEBUG
+#ifndef DBUG_OFF
     dd::cache::Storage_adapter::s_use_fake_storage = true;
-#endif /* !NDEBUG */
+#endif /* !DBUG_OFF */
     dd::cache::Dictionary_client::Auto_releaser releaser(thd()->dd_client());
     mysql = new dd::Schema_impl();
     mysql->set_name("mysql");
@@ -143,7 +143,7 @@ class CacheStorageTest : public ::testing::Test, public Test_MDL_context_owner {
     EXPECT_FALSE(m_mdl_context.has_locks());
   }
 
-  void TearDown() override {
+  virtual void TearDown() {
     /*
       Explicit scope + auto releaser to make sure acquired objects are
       released before teardown of the thd.
@@ -160,14 +160,14 @@ class CacheStorageTest : public ::testing::Test, public Test_MDL_context_owner {
     delete mysql;
     m_mdl_context.release_transactional_locks();
     m_mdl_context.destroy();
-#ifndef NDEBUG
+#ifndef DBUG_OFF
     dd::cache::Storage_adapter::s_use_fake_storage = false;
-#endif /* !NDEBUG */
+#endif /* !DBUG_OFF */
     m_init.TearDown();
   }
 
-  void notify_shared_lock(MDL_context_owner *in_use,
-                          bool needs_thr_lock_abort) override {
+  virtual void notify_shared_lock(MDL_context_owner *in_use,
+                                  bool needs_thr_lock_abort) {
     in_use->notify_shared_lock(nullptr, needs_thr_lock_abort);
   }
 
@@ -180,16 +180,15 @@ class CacheStorageTest : public ::testing::Test, public Test_MDL_context_owner {
   MDL_request m_request;
 
  private:
-  CacheStorageTest(CacheStorageTest const &) = delete;
-  CacheStorageTest &operator=(CacheStorageTest const &) = delete;
+  GTEST_DISALLOW_COPY_AND_ASSIGN_(CacheStorageTest);
 };
 
 template <typename T>
 class CacheTest : public ::testing::Test {
  protected:
-  void SetUp() override {}
+  virtual void SetUp() {}
 
-  void TearDown() override {}
+  virtual void TearDown() {}
 };
 
 typedef ::testing::Types<dd::Charset_impl, dd::Collation_impl,
@@ -197,7 +196,7 @@ typedef ::testing::Types<dd::Charset_impl, dd::Collation_impl,
                          dd::Table_impl, dd::Tablespace_impl, dd::View_impl,
                          dd::Event_impl, dd::Procedure_impl>
     DDTypes;
-TYPED_TEST_SUITE(CacheTest, DDTypes);
+TYPED_TEST_CASE(CacheTest, DDTypes);
 
 // Helper class to create objects and wrap them in elements.
 class CacheTestHelper {
@@ -357,7 +356,7 @@ TYPED_TEST(CacheTest, Element_map_aux_key) {
   // The aux key behavior is not uniform, and this test is therefore omitted.
 }
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
 template <typename Intrfc_type, typename Impl_type>
 void test_basic_store_and_get(CacheStorageTest *tst, THD *thd) {
   dd::cache::Dictionary_client *dc = thd->dd_client();
@@ -1274,7 +1273,7 @@ TEST_F(CacheStorageTest, TestTransactionMaxSePrivateId) {
 //   dd::cache::Dictionary_client::Auto_releaser releaser(&dc);
 
 //   // Create a new tablespace.
-//   dd::Object_id tablespace_id [[maybe_unused]];
+//   dd::Object_id tablespace_id MY_ATTRIBUTE((unused));
 //   {
 //     std::unique_ptr<dd::Tablespace> obj(dd::create_object<dd::Tablespace>());
 //     dd_unittest::set_attributes(obj.get(), "test_tablespace");
@@ -1394,7 +1393,7 @@ TEST_F(CacheStorageTest, TestTriggers) {
 
   dd::String_type obj_name =
       dd::Table::DD_table::instance().name() + dd::String_type("_trigs");
-  dd::Object_id id [[maybe_unused]];
+  dd::Object_id id MY_ATTRIBUTE((unused));
 
   //
   // Create table object
@@ -1613,5 +1612,5 @@ TEST_F(CacheStorageTest, CloneInternalPointersTest) {
   EXPECT_EQ(clone_indices[1], &clone_parts[1]->indexes().front()->index());
 }
 
-#endif /* !NDEBUG */
+#endif /* !DBUG_OFF */
 }  // namespace dd_cache_unittest
