@@ -1,16 +1,15 @@
-/* Copyright (c) 2013, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2013, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -41,11 +40,11 @@ namespace dd_properties_unittest {
 
 class PropertiesTest : public ::testing::Test {
  protected:
-  PropertiesTest() = default;
+  PropertiesTest() {}
 
-  void SetUp() override { m_props = new dd::Properties_impl(); }
+  virtual void SetUp() { m_props = new dd::Properties_impl(); }
 
-  void TearDown() override { delete m_props; }
+  virtual void TearDown() { delete m_props; }
 
   static void SetUpTestCase() {}
 
@@ -54,8 +53,8 @@ class PropertiesTest : public ::testing::Test {
   dd::Properties *m_props;
 
  private:
-  PropertiesTest(PropertiesTest const &) = delete;
-  PropertiesTest &operator=(PropertiesTest const &) = delete;
+  // Declares (but does not define) copy constructor and assignment operator.
+  GTEST_DISALLOW_COPY_AND_ASSIGN_(PropertiesTest);
 };
 
 static const dd::String_type value(const dd::Properties &p,
@@ -201,7 +200,7 @@ TEST_F(PropertiesTest, SetGetStrings) {
   // Key "" is illegal and will not be added
   p->set("", "");
 
-#if !defined(NDEBUG)
+#if !defined(DBUG_OFF)
   EXPECT_DEATH_IF_SUPPORTED(p->get("", &str), ".*UTC - mysqld got.*");
 #else
   EXPECT_TRUE(p->get("", &str));
@@ -401,7 +400,7 @@ TEST_F(PropertiesTest, ValidSetGetIntBool) {
   delete p;
 }
 
-#if !defined(NDEBUG)
+#if !defined(DBUG_OFF)
 
 // Tests invalid setting and getting of int and bool options
 typedef PropertiesTest PropertiesTestDeathTest;
@@ -489,7 +488,7 @@ TEST_F(PropertiesTestDeathTest, FailingSetGetIntBool) {
   delete p;
 }
 
-#endif  // NDEBUG
+#endif  // DBUG_OFF
 
 // Tests the exists function
 TEST_F(PropertiesTest, OptionsExist) {
@@ -628,7 +627,7 @@ TEST_F(PropertiesTest, IterationSize) {
 
   EXPECT_TRUE(p->size() == 0);
 
-  for (dd::Properties::iterator it [[maybe_unused]] = p->begin();
+  for (dd::Properties::iterator it MY_ATTRIBUTE((unused)) = p->begin();
        it != p->end(); ++it, ++i)
     EXPECT_TRUE(false);
 
@@ -962,7 +961,7 @@ TEST_F(PropertiesTest, ValidKeys) {
   dd::Properties_impl p({"a"});
 
   EXPECT_FALSE(p.set("a", "1"));
-#if defined(NDEBUG)
+#if defined(DBUG_OFF)
   EXPECT_TRUE(p.set("b", "2"));
 #else
   EXPECT_DEATH_IF_SUPPORTED(p.set("b", "2"), ".*UTC - mysqld got.*");
@@ -982,7 +981,7 @@ TEST_F(PropertiesTest, ValidKeys) {
   dd::String_type str;
   EXPECT_TRUE(!p.get("b", &str) && str == "2");
 
-#if !defined(NDEBUG)
+#if !defined(DBUG_OFF)
   // Then adding a valid key asserts in dbug, though, because
   // we might then hide keys present.
   EXPECT_DEATH_IF_SUPPORTED(p.add_valid_keys({"c"}), ".*UTC - mysqld got.*");
