@@ -1,4 +1,4 @@
-/* Copyright (c) 2011, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2011, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -35,9 +35,7 @@
 #include "m_string.h"
 #include "my_inttypes.h"
 #include "my_macros.h"
-#include "mysql/strings/int2str.h"
-#include "sql-common/my_decimal.h"
-#include "template_utils.h"
+#include "sql/my_decimal.h"
 #include "unittest/gunit/benchmark.h"
 
 namespace decimal_unittest {
@@ -1113,7 +1111,7 @@ static const DecimalToStringParam TO_STRING_OVERFLOW_TRUNCATE[] = {
     {"0.00000", 4, "0.0", E_DEC_TRUNCATED},
 };
 
-INSTANTIATE_TEST_SUITE_P(OverflowTruncate, DecimalToStringTest,
-                         testing::ValuesIn(TO_STRING_OVERFLOW_TRUNCATE));
+INSTANTIATE_TEST_CASE_P(OverflowTruncate, DecimalToStringTest,
+                        testing::ValuesIn(TO_STRING_OVERFLOW_TRUNCATE));
 
 }  // namespace decimal_unittest
