@@ -1,4 +1,4 @@
-/* Copyright (c) 2011, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2011, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -22,16 +22,13 @@
 
 #include <gtest/gtest.h>
 #include <sys/types.h>
-#include <memory>
 
 #include "my_inttypes.h"
-#include "mysql/strings/m_ctype.h"
-#include "sql-common/my_decimal.h"
 #include "sql/field.h"
+#include "sql/my_decimal.h"
 #include "sql/protocol.h"
 #include "sql/sql_class.h"
 #include "sql/sql_time.h"
-#include "strings/m_ctype_internals.h"
 #include "unittest/gunit/fake_table.h"
 #include "unittest/gunit/mysys_util.h"
 #include "unittest/gunit/test_utils.h"
@@ -101,7 +98,7 @@ class Mock_protocol : public Protocol {
   ulong get_client_capabilities() override { return 0; }
   bool has_client_capability(unsigned long) override { return false; }
   void end_partial_result_set() override {}
-  int shutdown(bool server_shutdown [[maybe_unused]] = false) override {
+  int shutdown(bool server_shutdown MY_ATTRIBUTE((unused)) = false) override {
     return 0;
   }
   SSL_handle get_ssl() { return nullptr; }
@@ -196,7 +193,7 @@ TEST_F(FieldTest, FieldTimef) {
   EXPECT_DOUBLE_EQ(field->val_real(), copy->val_real());
   EXPECT_EQ(field->val_int(), copy->val_int());
   EXPECT_EQ(field->val_time_temporal(), copy->val_time_temporal());
-  EXPECT_EQ(0, field->cmp(field->field_ptr(), copy->field_ptr()));
+  EXPECT_EQ(0, field->cmp(field->ptr, copy->ptr));
 
   // Test reset
   EXPECT_EQ(0, field->reset());
@@ -294,12 +291,12 @@ TEST_F(FieldTest, FieldTimef) {
 
   Mock_table m_table(thd());
   f->table = &m_table;
-  my_timeval tv;
+  struct timeval tv;
   int warnings = 0;
   EXPECT_FALSE(f->get_timestamp(&tv, &warnings));
-  EXPECT_EQ(123400, tv.m_tv_usec);
+  EXPECT_EQ(123400, tv.tv_usec);
 
-  ::destroy_at(field);
+  destroy(field);
 }
 
 TEST_F(FieldTest, FieldTimefCompare) {
@@ -336,8 +333,7 @@ TEST_F(FieldTest, FieldTimefCompare) {
             0, memcmp(sortStrings[i], sortStrings[j], fields[i]->pack_length()))
             << fields[i]->val_str(&tmp)->c_ptr() << " < "
             << fields[j]->val_str(&tmp)->c_ptr();
-        EXPECT_GT(
-            0, fields[i]->cmp(fields[i]->field_ptr(), fields[j]->field_ptr()))
+        EXPECT_GT(0, fields[i]->cmp(fields[i]->ptr, fields[j]->ptr))
             << fields[i]->val_str(&tmp)->c_ptr() << " < "
             << fields[j]->val_str(&tmp)->c_ptr();
       } else if (i > j) {
@@ -345,8 +341,7 @@ TEST_F(FieldTest, FieldTimefCompare) {
             0, memcmp(sortStrings[i], sortStrings[j], fields[i]->pack_length()))
             << fields[i]->val_str(&tmp)->c_ptr() << " > "
             << fields[j]->val_str(&tmp)->c_ptr();
-        EXPECT_LT(
-            0, fields[i]->cmp(fields[i]->field_ptr(), fields[j]->field_ptr()))
+        EXPECT_LT(0, fields[i]->cmp(fields[i]->ptr, fields[j]->ptr))
             << fields[i]->val_str(&tmp)->c_ptr() << " > "
             << fields[j]->val_str(&tmp)->c_ptr();
       } else {
@@ -354,8 +349,7 @@ TEST_F(FieldTest, FieldTimefCompare) {
             0, memcmp(sortStrings[i], sortStrings[j], fields[i]->pack_length()))
             << fields[i]->val_str(&tmp)->c_ptr() << " = "
             << fields[j]->val_str(&tmp)->c_ptr();
-        EXPECT_EQ(
-            0, fields[i]->cmp(fields[i]->field_ptr(), fields[j]->field_ptr()))
+        EXPECT_EQ(0, fields[i]->cmp(fields[i]->ptr, fields[j]->ptr))
             << fields[i]->val_str(&tmp)->c_ptr() << " = "
             << fields[j]->val_str(&tmp)->c_ptr();
       }
@@ -408,16 +402,16 @@ TEST_F(FieldTest, CopyFieldSet) {
   Field_set *f_to = create_field_set(&tl3);
   bitmap_set_all(f_to->table->write_set);
   uchar to_fieldval = 0;
-  f_to->set_field_ptr(&to_fieldval);
+  f_to->ptr = &to_fieldval;
 
   Field_set *f_from = create_field_set(&tl4);
   bitmap_set_all(f_from->table->write_set);
   bitmap_set_all(f_from->table->read_set);
   uchar from_fieldval = static_cast<uchar>(typeset);
-  f_from->set_field_ptr(&from_fieldval);
+  f_from->ptr = &from_fieldval;
 
   Copy_field *cf = new (thd()->mem_root) Copy_field;
-  cf->set(f_to, f_from);
+  cf->set(f_to, f_from, false);
   cf->invoke_do_copy();
 
   // Copy_field DTOR is not invoked in all contexts, so we may leak memory.
@@ -448,7 +442,7 @@ void test_make_sort_key(Field *field, uchar *from, const uchar *expected,
   const uint pack_length = field->pack_length();
   Fake_TABLE table(field);
   table.s->db_low_byte_first = false;
-  field->set_field_ptr(from);
+  field->ptr = from;
 
   for (uint key_length = min_key_length; key_length <= pack_length;
        ++key_length) {
@@ -508,8 +502,7 @@ class Mock_charset : public CHARSET_INFO {
  public:
   mutable bool strnxfrm_called;
   Mock_charset() : strnxfrm_called(false) {
-    CHARSET_INFO *ci = get_charset_by_name("latin1_general_ci", MYF(0));
-    cset = ci->cset;
+    cset = &my_charset_8bit_handler;
     coll = &mock_collation;
     mbmaxlen = 1;
     pad_attribute = PAD_SPACE;
