@@ -1,16 +1,15 @@
-/* Copyright (c) 2014, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -38,9 +37,9 @@ namespace gcs_member_info_unittest {
 
 class ClusterMemberInfoTest : public ::testing::Test {
  protected:
-  ClusterMemberInfoTest() = default;
+  ClusterMemberInfoTest() {}
 
-  void SetUp() override {
+  virtual void SetUp() {
     string hostname("pc_hostname");
     string uuid("781f947c-db4a-11e3-98d1-f01faf1a1c44");
     uint port = 4444;
@@ -55,9 +54,6 @@ class ClusterMemberInfoTest : public ::testing::Test {
     bool in_primary_mode = false;
     bool has_enforces_update_everywhere_checks = false;
     uint member_weight = 70;
-    std::string member_recovery_endpoints = "DEFAULT";
-    std::string member_view_change_uuid = "AUTOMATIC";
-    bool allow_single_leader = true;
 
     gcs_member_id = new Gcs_member_identifier("stuff");
 
@@ -70,13 +66,11 @@ class ClusterMemberInfoTest : public ::testing::Test {
         gcs_member_id->get_member_id(), status, local_member_plugin_version,
         gtid_assignment_block_size, Group_member_info::MEMBER_ROLE_PRIMARY,
         in_primary_mode, has_enforces_update_everywhere_checks, member_weight,
-        lower_case_table_names, default_table_encryption,
-        member_recovery_endpoints.c_str(), member_view_change_uuid.c_str(),
-        allow_single_leader, PSI_NOT_INSTRUMENTED);
+        lower_case_table_names, default_table_encryption, PSI_NOT_INSTRUMENTED);
     local_node->update_gtid_sets(executed_gtid, purged_gtid, retrieved_gtid);
   }
 
-  void TearDown() override {
+  virtual void TearDown() {
     delete gcs_member_id;
     delete local_node;
   }
@@ -114,18 +108,15 @@ TEST_F(ClusterMemberInfoTest, EncodeDecodeIdempotencyTest) {
   ASSERT_EQ(local_node->get_role(), decoded_local_node.get_role());
   ASSERT_EQ(local_node->get_member_weight(),
             decoded_local_node.get_member_weight());
-  ASSERT_EQ(local_node->get_view_change_uuid(),
-            decoded_local_node.get_view_change_uuid());
-  ASSERT_EQ(local_node->get_allow_single_leader(),
-            decoded_local_node.get_allow_single_leader());
+
   delete encoded;
 }
 
 class ClusterMemberInfoManagerTest : public ::testing::Test {
  protected:
-  ClusterMemberInfoManagerTest() = default;
+  ClusterMemberInfoManagerTest() {}
 
-  void SetUp() override {
+  virtual void SetUp() {
     string hostname("pc_hostname");
     string uuid("8d7r947c-dr4a-17i3-59d1-f01faf1kkc44");
     uint port = 4444;
@@ -138,10 +129,6 @@ class ClusterMemberInfoManagerTest : public ::testing::Test {
     bool in_primary_mode = false;
     bool has_enforces_update_everywhere_checks = false;
     uint member_weight = 80;
-    std::string member_recovery_endpoints = "DEFAULT";
-    std::string member_view_change_uuid =
-        "8896eb66-8684-11eb-8dcd-0242ac130003";
-    bool allow_single_leader = false;
 
     Group_member_info::Group_member_status status =
         Group_member_info::MEMBER_OFFLINE;
@@ -152,15 +139,13 @@ class ClusterMemberInfoManagerTest : public ::testing::Test {
         gcs_member_id->get_member_id(), status, local_member_plugin_version,
         gtid_assignment_block_size, Group_member_info::MEMBER_ROLE_SECONDARY,
         in_primary_mode, has_enforces_update_everywhere_checks, member_weight,
-        lower_case_table_names, default_table_encryption,
-        member_recovery_endpoints.c_str(), member_view_change_uuid.c_str(),
-        allow_single_leader, PSI_NOT_INSTRUMENTED);
+        lower_case_table_names, default_table_encryption, PSI_NOT_INSTRUMENTED);
 
     cluster_member_mgr =
         new Group_member_info_manager(local_node, PSI_NOT_INSTRUMENTED);
   }
 
-  void TearDown() override {
+  virtual void TearDown() {
     delete cluster_member_mgr;
     delete gcs_member_id;
     delete local_node;
@@ -188,9 +173,6 @@ TEST_F(ClusterMemberInfoManagerTest, GetLocalInfoByUUIDTest) {
   bool in_primary_mode = false;
   bool has_enforces_update_everywhere_checks = false;
   uint member_weight = 90;
-  std::string member_recovery_endpoints = "DEFAULT";
-  std::string member_view_change_uuid = "99f957fc-75c5-445d-98ae-7ea02e55c5be";
-  bool allow_single_leader = true;
 
   Group_member_info::Group_member_status status =
       Group_member_info::MEMBER_OFFLINE;
@@ -201,22 +183,20 @@ TEST_F(ClusterMemberInfoManagerTest, GetLocalInfoByUUIDTest) {
       gcs_member_id.get_member_id(), status, local_member_plugin_version,
       gtid_assignment_block_size, Group_member_info::MEMBER_ROLE_PRIMARY,
       in_primary_mode, has_enforces_update_everywhere_checks, member_weight,
-      lower_case_table_names, default_table_encryption,
-      member_recovery_endpoints.c_str(), member_view_change_uuid.c_str(),
-      allow_single_leader, PSI_NOT_INSTRUMENTED);
+      lower_case_table_names, default_table_encryption, PSI_NOT_INSTRUMENTED);
   new_member->update_gtid_sets(executed_gtid, purged_gtid, retrieved_gtid);
 
   cluster_member_mgr->add(new_member);
 
   string uuid_to_get("8d7r947c-dr4a-17i3-59d1-f01faf1kkc44");
 
-  Group_member_info retrieved_local_info;
-  const bool retrieved_local_info_not_found =
-      cluster_member_mgr->get_group_member_info(uuid_to_get,
-                                                retrieved_local_info);
+  Group_member_info *retrieved_local_info =
+      cluster_member_mgr->get_group_member_info(uuid_to_get);
 
-  ASSERT_TRUE(false == retrieved_local_info_not_found);
-  ASSERT_EQ(retrieved_local_info.get_uuid(), uuid_to_get);
+  ASSERT_TRUE(retrieved_local_info != nullptr);
+  ASSERT_EQ(retrieved_local_info->get_uuid(), uuid_to_get);
+
+  delete retrieved_local_info;
 }
 
 TEST_F(ClusterMemberInfoManagerTest, UpdateStatusOfLocalObjectTest) {
@@ -245,7 +225,7 @@ TEST_F(ClusterMemberInfoManagerTest, GetLocalInfoByUUIDAfterEncodingTest) {
   vector<uchar> *encoded = new vector<uchar>();
   cluster_member_mgr->encode(encoded);
 
-  Group_member_info_list *decoded_members =
+  vector<Group_member_info *> *decoded_members =
       cluster_member_mgr->decode(&encoded->front(), encoded->size());
 
   cluster_member_mgr->update(decoded_members);
@@ -255,37 +235,33 @@ TEST_F(ClusterMemberInfoManagerTest, GetLocalInfoByUUIDAfterEncodingTest) {
 
   string uuid_to_get("8d7r947c-dr4a-17i3-59d1-f01faf1kkc44");
 
-  Group_member_info retrieved_local_info;
-  const bool retrieved_local_info_not_found =
-      cluster_member_mgr->get_group_member_info(uuid_to_get,
-                                                retrieved_local_info);
+  Group_member_info *retrieved_local_info =
+      cluster_member_mgr->get_group_member_info(uuid_to_get);
 
-  ASSERT_TRUE(false == retrieved_local_info_not_found);
+  ASSERT_TRUE(retrieved_local_info != nullptr);
 
-  ASSERT_EQ(local_node->get_port(), retrieved_local_info.get_port());
-  ASSERT_EQ(local_node->get_hostname(), retrieved_local_info.get_hostname());
-  ASSERT_EQ(local_node->get_uuid(), retrieved_local_info.get_uuid());
+  ASSERT_EQ(local_node->get_port(), retrieved_local_info->get_port());
+  ASSERT_EQ(local_node->get_hostname(), retrieved_local_info->get_hostname());
+  ASSERT_EQ(local_node->get_uuid(), retrieved_local_info->get_uuid());
   ASSERT_EQ(local_node->get_gcs_member_id().get_member_id(),
-            retrieved_local_info.get_gcs_member_id().get_member_id());
+            retrieved_local_info->get_gcs_member_id().get_member_id());
   ASSERT_EQ(local_node->get_recovery_status(),
-            retrieved_local_info.get_recovery_status());
+            retrieved_local_info->get_recovery_status());
   ASSERT_EQ(local_node->get_write_set_extraction_algorithm(),
-            retrieved_local_info.get_write_set_extraction_algorithm());
+            retrieved_local_info->get_write_set_extraction_algorithm());
   ASSERT_EQ(local_node->get_gtid_executed(),
-            retrieved_local_info.get_gtid_executed());
+            retrieved_local_info->get_gtid_executed());
   ASSERT_EQ(local_node->get_gtid_purged(),
-            retrieved_local_info.get_gtid_purged());
+            retrieved_local_info->get_gtid_purged());
   ASSERT_EQ(local_node->get_gtid_retrieved(),
-            retrieved_local_info.get_gtid_retrieved());
+            retrieved_local_info->get_gtid_retrieved());
   ASSERT_EQ(local_node->get_gtid_assignment_block_size(),
-            retrieved_local_info.get_gtid_assignment_block_size());
-  ASSERT_EQ(local_node->get_role(), retrieved_local_info.get_role());
+            retrieved_local_info->get_gtid_assignment_block_size());
+  ASSERT_EQ(local_node->get_role(), retrieved_local_info->get_role());
   ASSERT_EQ(local_node->get_member_weight(),
-            retrieved_local_info.get_member_weight());
-  ASSERT_EQ(local_node->get_view_change_uuid(),
-            retrieved_local_info.get_view_change_uuid());
-  ASSERT_EQ(local_node->get_allow_single_leader(),
-            retrieved_local_info.get_allow_single_leader());
+            retrieved_local_info->get_member_weight());
+
+  delete retrieved_local_info;
 }
 
 TEST_F(ClusterMemberInfoManagerTest,
@@ -294,7 +270,7 @@ TEST_F(ClusterMemberInfoManagerTest,
   vector<uchar> *encoded = new vector<uchar>();
   cluster_member_mgr->encode(encoded);
 
-  Group_member_info_list *decoded_members =
+  vector<Group_member_info *> *decoded_members =
       cluster_member_mgr->decode(&encoded->front(), encoded->size());
 
   cluster_member_mgr->update(decoded_members);
@@ -318,17 +294,16 @@ TEST_F(ClusterMemberInfoManagerTest,
   ASSERT_EQ(executed_gtid, local_node->get_gtid_executed());
   ASSERT_EQ(retrieved_gtid, local_node->get_gtid_retrieved());
 
-  Group_member_info retrieved_local_info;
-  const bool retrieved_local_info_not_found =
-      cluster_member_mgr->get_group_member_info(local_node->get_uuid(),
-                                                retrieved_local_info);
+  Group_member_info *retrieved_local_info =
+      cluster_member_mgr->get_group_member_info(local_node->get_uuid());
 
-  ASSERT_TRUE(false == retrieved_local_info_not_found);
   ASSERT_EQ(Group_member_info::MEMBER_ONLINE,
-            retrieved_local_info.get_recovery_status());
+            retrieved_local_info->get_recovery_status());
 
-  ASSERT_EQ(executed_gtid, retrieved_local_info.get_gtid_executed());
-  ASSERT_EQ(retrieved_gtid, retrieved_local_info.get_gtid_retrieved());
+  ASSERT_EQ(executed_gtid, retrieved_local_info->get_gtid_executed());
+  ASSERT_EQ(retrieved_gtid, retrieved_local_info->get_gtid_retrieved());
+
+  delete retrieved_local_info;
 }
 
 TEST_F(ClusterMemberInfoManagerTest, EncodeDecodeLargeSets) {
@@ -349,9 +324,6 @@ TEST_F(ClusterMemberInfoManagerTest, EncodeDecodeLargeSets) {
   bool in_primary_mode = false;
   bool has_enforces_update_everywhere_checks = false;
   uint member_weight = 40;
-  std::string member_recovery_endpoints = "DEFAULT";
-  std::string member_view_change_uuid = "ba1c4c32-1887-4ccf-bc5a-8f6165d19ea3";
-  bool allow_single_leader = true;
 
   Group_member_info::Group_member_status status =
       Group_member_info::MEMBER_OFFLINE;
@@ -362,27 +334,23 @@ TEST_F(ClusterMemberInfoManagerTest, EncodeDecodeLargeSets) {
       gcs_member_id.get_member_id(), status, local_member_plugin_version,
       gtid_assignment_block_size, Group_member_info::MEMBER_ROLE_PRIMARY,
       in_primary_mode, has_enforces_update_everywhere_checks, member_weight,
-      lower_case_table_names, default_table_encryption,
-      member_recovery_endpoints.c_str(), member_view_change_uuid.c_str(),
-      allow_single_leader, PSI_NOT_INSTRUMENTED);
+      lower_case_table_names, default_table_encryption, PSI_NOT_INSTRUMENTED);
   new_member->update_gtid_sets(executed_gtid, purged_gtid, retrieved_gtid);
 
   cluster_member_mgr->add(new_member);
 
   string uuid_to_get("8d7r947c-dr4a-17i3-59d1-f01faf1kkc44");
 
-  Group_member_info retrieved_local_info;
-  bool retrieved_local_info_not_found =
-      cluster_member_mgr->get_group_member_info(uuid_to_get,
-                                                retrieved_local_info);
+  Group_member_info *retrieved_local_info =
+      cluster_member_mgr->get_group_member_info(uuid_to_get);
 
-  ASSERT_TRUE(false == retrieved_local_info_not_found);
-  ASSERT_EQ(retrieved_local_info.get_uuid(), uuid_to_get);
+  ASSERT_TRUE(retrieved_local_info != nullptr);
+  ASSERT_EQ(retrieved_local_info->get_uuid(), uuid_to_get);
 
   vector<uchar> *encoded = new vector<uchar>();
   cluster_member_mgr->encode(encoded);
 
-  Group_member_info_list *decoded_members =
+  vector<Group_member_info *> *decoded_members =
       cluster_member_mgr->decode(&encoded->front(), encoded->size());
   delete encoded;
 
@@ -392,51 +360,49 @@ TEST_F(ClusterMemberInfoManagerTest, EncodeDecodeLargeSets) {
 
   ASSERT_EQ(2U, cluster_member_mgr->get_number_of_members());
 
-  retrieved_local_info_not_found =
-      cluster_member_mgr->get_group_member_info(uuid, retrieved_local_info);
+  delete retrieved_local_info;
+  retrieved_local_info = cluster_member_mgr->get_group_member_info(uuid);
 
-  ASSERT_TRUE(false == retrieved_local_info_not_found);
+  ASSERT_TRUE(retrieved_local_info != nullptr);
 
-  ASSERT_EQ(port, retrieved_local_info.get_port());
-  ASSERT_EQ(hostname, retrieved_local_info.get_hostname());
-  ASSERT_EQ(executed_gtid, retrieved_local_info.get_gtid_executed());
-  ASSERT_EQ(retrieved_gtid, retrieved_local_info.get_gtid_retrieved());
+  ASSERT_EQ(port, retrieved_local_info->get_port());
+  ASSERT_EQ(hostname, retrieved_local_info->get_hostname());
+  ASSERT_EQ(executed_gtid, retrieved_local_info->get_gtid_executed());
+  ASSERT_EQ(retrieved_gtid, retrieved_local_info->get_gtid_retrieved());
   ASSERT_EQ(write_set_algorithm,
-            retrieved_local_info.get_write_set_extraction_algorithm());
+            retrieved_local_info->get_write_set_extraction_algorithm());
 
-  retrieved_local_info_not_found = cluster_member_mgr->get_group_member_info(
-      uuid_to_get, retrieved_local_info);
+  delete retrieved_local_info;
+  retrieved_local_info = cluster_member_mgr->get_group_member_info(uuid_to_get);
 
-  ASSERT_TRUE(false == retrieved_local_info_not_found);
+  ASSERT_TRUE(retrieved_local_info != nullptr);
 
-  ASSERT_EQ(local_node->get_port(), retrieved_local_info.get_port());
-  ASSERT_EQ(local_node->get_hostname(), retrieved_local_info.get_hostname());
-  ASSERT_EQ(local_node->get_uuid(), retrieved_local_info.get_uuid());
+  ASSERT_EQ(local_node->get_port(), retrieved_local_info->get_port());
+  ASSERT_EQ(local_node->get_hostname(), retrieved_local_info->get_hostname());
+  ASSERT_EQ(local_node->get_uuid(), retrieved_local_info->get_uuid());
   ASSERT_EQ(local_node->get_gcs_member_id().get_member_id(),
-            retrieved_local_info.get_gcs_member_id().get_member_id());
+            retrieved_local_info->get_gcs_member_id().get_member_id());
   ASSERT_EQ(local_node->get_recovery_status(),
-            retrieved_local_info.get_recovery_status());
+            retrieved_local_info->get_recovery_status());
   ASSERT_EQ(local_node->get_write_set_extraction_algorithm(),
-            retrieved_local_info.get_write_set_extraction_algorithm());
+            retrieved_local_info->get_write_set_extraction_algorithm());
   ASSERT_EQ(local_node->get_gtid_executed(),
-            retrieved_local_info.get_gtid_executed());
+            retrieved_local_info->get_gtid_executed());
   ASSERT_EQ(local_node->get_gtid_purged(),
-            retrieved_local_info.get_gtid_purged());
+            retrieved_local_info->get_gtid_purged());
   ASSERT_EQ(local_node->get_gtid_retrieved(),
-            retrieved_local_info.get_gtid_retrieved());
+            retrieved_local_info->get_gtid_retrieved());
   ASSERT_EQ(local_node->get_gtid_assignment_block_size(),
-            retrieved_local_info.get_gtid_assignment_block_size());
-  ASSERT_EQ(local_node->get_role(), retrieved_local_info.get_role());
+            retrieved_local_info->get_gtid_assignment_block_size());
+  ASSERT_EQ(local_node->get_role(), retrieved_local_info->get_role());
   ASSERT_EQ(local_node->get_member_weight(),
-            retrieved_local_info.get_member_weight());
+            retrieved_local_info->get_member_weight());
   ASSERT_EQ(local_node->get_lower_case_table_names(),
-            retrieved_local_info.get_lower_case_table_names());
+            retrieved_local_info->get_lower_case_table_names());
   ASSERT_EQ(local_node->get_default_table_encryption(),
-            retrieved_local_info.get_default_table_encryption());
-  ASSERT_EQ(local_node->get_view_change_uuid(),
-            retrieved_local_info.get_view_change_uuid());
-  ASSERT_EQ(local_node->get_allow_single_leader(),
-            retrieved_local_info.get_allow_single_leader());
+            retrieved_local_info->get_default_table_encryption());
+
+  delete retrieved_local_info;
 }
 
 }  // namespace gcs_member_info_unittest
