@@ -1,16 +1,15 @@
-/* Copyright (c) 2011, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2011, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,38 +25,32 @@
 #include <limits.h>
 #include <stddef.h>
 #include <sys/types.h>
-
-#include <array>
 #include <memory>
 #include <new>
-#include <utility>
 
 #include "decimal.h"
 #include "field_types.h"
 #include "lex_string.h"
+#include "m_ctype.h"
+#include "m_string.h"
 #include "my_alloc.h"
 #include "my_inttypes.h"
 #include "my_macros.h"
 #include "my_table_map.h"
-#include "mysql/strings/dtoa.h"
-#include "mysql/strings/m_ctype.h"
 #include "mysql_time.h"
 #include "mysys_err.h"
-#include "sql-common/json_dom.h"
-#include "sql-common/my_decimal.h"
 #include "sql/item.h"
 #include "sql/item_cmpfunc.h"
 #include "sql/item_create.h"
 #include "sql/item_strfunc.h"
-#include "sql/item_sum.h"
 #include "sql/item_timefunc.h"
+#include "sql/json_dom.h"
+#include "sql/my_decimal.h"
 #include "sql/sql_class.h"
 #include "sql/sql_lex.h"
 #include "sql/tztime.h"
 #include "sql_string.h"
-#include "string_with_len.h"
 #include "unittest/gunit/fake_table.h"
-#include "unittest/gunit/mock_field_long.h"
 #include "unittest/gunit/mock_field_timestamp.h"
 #include "unittest/gunit/mysys_util.h"
 #include "unittest/gunit/test_utils.h"
@@ -66,15 +59,12 @@ namespace item_unittest {
 
 using my_testing::Mock_error_handler;
 using my_testing::Server_initializer;
-using std::array;
-using std::make_pair;
-using std::pair;
 using ::testing::Return;
 
 class ItemTest : public ::testing::Test {
  protected:
-  void SetUp() override { initializer.SetUp(); }
-  void TearDown() override { initializer.TearDown(); }
+  virtual void SetUp() { initializer.SetUp(); }
+  virtual void TearDown() { initializer.TearDown(); }
 
   THD *thd() { return initializer.thd(); }
 
@@ -239,7 +229,7 @@ TEST_F(ItemTest, ItemInt) {
 
   item_int->neg();
   EXPECT_EQ(-val, item_int->val_int());
-  EXPECT_EQ(precision, item_int->decimal_precision());
+  EXPECT_EQ(precision - 1, item_int->decimal_precision());
 
   // Functions inherited from parent class(es).
   const table_map tmap = 0;
@@ -279,7 +269,7 @@ TEST_F(ItemTest, ItemString) {
   EXPECT_EQ(MYSQL_TYPE_STRING, field_string.type());
 
   // CHAR field for testing strings with illegal values
-  Mock_field_string field_string_utf8(20, &my_charset_utf8mb3_general_ci);
+  Mock_field_string field_string_utf8(20, &my_charset_utf8_general_ci);
   EXPECT_EQ(MYSQL_TYPE_STRING, field_string_utf8.type());
 
   /*
@@ -326,7 +316,7 @@ TEST_F(ItemTest, ItemString) {
 
   // VARCHAR field for testing strings with illegal values
   Mock_field_varstring field_varstring_utf8(20, &table_share,
-                                            &my_charset_utf8mb3_general_ci);
+                                            &my_charset_utf8_general_ci);
   EXPECT_EQ(MYSQL_TYPE_VARCHAR, field_varstring_utf8.type());
 
   /*
@@ -384,103 +374,6 @@ TEST_F(ItemTest, ItemEqual) {
   EXPECT_EQ(1, item_equal->val_int());
 }
 
-TEST_F(ItemTest, ItemViewRef) {
-  Mock_field_long f_field1(true);
-  Mock_field_long f_field2(true);
-  Item_field *field1 = new Item_field(&f_field1);
-  Item_field *field2 = new Item_field(&f_field2);
-  Item *field3 = new Item_int(123);
-
-  // Create a view reference over a constant expression from an inner
-  // table of an outer join.
-  Table_ref table;
-  table.outer_join = true;
-  table.set_tableno(2);
-  Item_view_ref *view_ref =
-      new Item_view_ref(&thd()->lex->current_query_block()->context, &field3,
-                        nullptr, nullptr, "t1", "f1", &table);
-
-  Item_func_eq *eq1 = new Item_func_eq(field1, field2);
-  Item_func_eq *eq2 = new Item_func_eq(field1, field3);
-  Item_func_eq *eq3 = new Item_func_eq(field1, view_ref);
-
-  // True because both arguments are fields.
-  EXPECT_TRUE(eq1->contains_only_equi_join_condition());
-  // False because the right side argument is a constant and
-  // therefore concluded as a filter.
-  EXPECT_FALSE(eq2->contains_only_equi_join_condition());
-  // Even though the used table information for the right side
-  // argument says it is not a constant expression, it examines
-  // the underlying expression in the view reference and concludes
-  // it as constant expression there by making it a filter and
-  // not a equi-join condition.
-  EXPECT_FALSE(eq3->contains_only_equi_join_condition());
-}
-
-TEST_F(ItemTest, ItemRollupSwitcher) {
-  Mock_field_long f_field1(true);
-  Item_field *field1 = new Item_field(&f_field1);
-  POS p;
-  Item_sum_sum *agg1 = new Item_sum_sum(p, field1, false, nullptr);
-  Item_sum_sum *agg2 = new Item_sum_sum(p, field1, false, nullptr);
-
-  List<Item> li1;
-  li1.push_back(agg1);
-  Item_rollup_sum_switcher *sw1 = new Item_rollup_sum_switcher(&li1);
-  List<Item> li2;
-  li2.push_back(agg2);
-  Item_rollup_sum_switcher *sw2 = new Item_rollup_sum_switcher(&li2);
-  EXPECT_TRUE(agg1->eq(agg2, false));
-  EXPECT_TRUE(agg1->eq(sw2, false));
-  EXPECT_TRUE(sw1->eq(sw2, false));
-  EXPECT_TRUE(sw1->eq(agg2, false));
-
-  EXPECT_FALSE(agg1->is_rollup_sum_wrapper());
-  EXPECT_TRUE(sw1->is_rollup_sum_wrapper());
-}
-
-TEST_F(ItemTest, ItemEqualEq) {
-  Mock_field_timestamp field1;
-  field1.field_name = "field1";
-  Mock_field_timestamp field2;
-  field2.field_name = "field2";
-
-  Item_equal *item_equal1 =
-      new Item_equal(new Item_field(&field1), new Item_field(&field2));
-  Item_equal *item_equal2 =
-      new Item_equal(new Item_field(&field2), new Item_field(&field1));
-  Item_equal *item_equal3 =
-      new Item_equal(new Item_int(123), new Item_field(&field1));
-  Item_equal *item_equal4 =
-      new Item_equal(new Item_int(123), new Item_field(&field1));
-  item_equal4->add(new Item_field(&field2));
-
-  Item_func_equal *spaceship =
-      new Item_func_equal(new Item_field(&field1), new Item_field(&field2));
-
-  // Create an array of <int, Item *> pairs. If the int members of two pairs are
-  // the same, the Item * members should be considered equal by Item::eq(). (The
-  // first two objects are equivalent. The rest of them are different.)
-  array<pair<int, Item *>, 5> items{
-      make_pair(1, item_equal1), make_pair(1, item_equal2),
-      make_pair(2, item_equal3), make_pair(3, item_equal4),
-      make_pair(4, spaceship),
-  };
-
-  // Now compare all items with each other.
-  for (bool binary_cmp : {true, false}) {
-    for (pair<int, Item *> it1 : items) {
-      for (pair<int, Item *> it2 : items) {
-        EXPECT_EQ(it1.first == it2.first,
-                  it1.second->eq(it2.second, binary_cmp))
-            << "Item 1: " << ItemToString(it1.second)
-            << ", Item 2: " << ItemToString(it2.second)
-            << ", binary_cmp = " << binary_cmp;
-      }
-    }
-  }
-}
-
 TEST_F(ItemTest, ItemFuncExportSet) {
   String str;
   Item *on_string = new Item_string(STRING_WITH_LEN("on"), &my_charset_bin);
@@ -491,7 +384,7 @@ TEST_F(ItemTest, ItemFuncExportSet) {
     Item *export_set =
         new Item_func_export_set(POS(), new Item_int(2), on_string, off_string,
                                  sep_string, new Item_int(4));
-    Parse_context pc(thd(), thd()->lex->current_query_block());
+    Parse_context pc(thd(), thd()->lex->current_select());
     EXPECT_FALSE(export_set->itemize(&pc, &export_set));
     EXPECT_FALSE(export_set->fix_fields(thd(), nullptr));
     EXPECT_EQ(&str, export_set->val_str(&str));
@@ -502,7 +395,7 @@ TEST_F(ItemTest, ItemFuncExportSet) {
     Item *export_set =
         new Item_func_export_set(POS(), new Item_int(2), on_string, off_string,
                                  sep_string, new Item_int(0));
-    Parse_context pc(thd(), thd()->lex->current_query_block());
+    Parse_context pc(thd(), thd()->lex->current_select());
     EXPECT_FALSE(export_set->itemize(&pc, &export_set));
     EXPECT_FALSE(export_set->fix_fields(thd(), nullptr));
     EXPECT_EQ(&str, export_set->val_str(&str));
@@ -526,7 +419,7 @@ TEST_F(ItemTest, ItemFuncExportSet) {
         POS(), new Item_int(0xff),
         new Item_func_repeat(POS(), string_x, item_int_repeat), string_x,
         sep_string);
-    Parse_context pc(thd(), thd()->lex->current_query_block());
+    Parse_context pc(thd(), thd()->lex->current_select());
     SCOPED_TRACE("");
     EXPECT_FALSE(export_set->itemize(&pc, &export_set));
     EXPECT_FALSE(export_set->fix_fields(thd(), nullptr));
@@ -540,7 +433,7 @@ TEST_F(ItemTest, ItemFuncExportSet) {
     Item *export_set = new Item_func_export_set(
         POS(), new Item_int(0xff), string_x,
         new Item_func_repeat(POS(), string_x, item_int_repeat), sep_string);
-    Parse_context pc(thd(), thd()->lex->current_query_block());
+    Parse_context pc(thd(), thd()->lex->current_select());
     SCOPED_TRACE("");
     EXPECT_FALSE(export_set->itemize(&pc, &export_set));
     EXPECT_FALSE(export_set->fix_fields(thd(), nullptr));
@@ -554,7 +447,7 @@ TEST_F(ItemTest, ItemFuncExportSet) {
     Item *export_set = new Item_func_export_set(
         POS(), new Item_int(0xff), string_x, string_x,
         new Item_func_repeat(POS(), string_x, item_int_repeat));
-    Parse_context pc(thd(), thd()->lex->current_query_block());
+    Parse_context pc(thd(), thd()->lex->current_select());
     SCOPED_TRACE("");
     EXPECT_FALSE(export_set->itemize(&pc, &export_set));
     EXPECT_FALSE(export_set->fix_fields(thd(), nullptr));
@@ -576,7 +469,7 @@ TEST_F(ItemTest, ItemFuncExportSet) {
     Item *export_set = new Item_func_export_set(
         POS(), new Item_string(STRING_WITH_LEN("1111111"), &my_charset_bin),
         lpad, new Item_int(1));
-    Parse_context pc(thd(), thd()->lex->current_query_block());
+    Parse_context pc(thd(), thd()->lex->current_select());
     SCOPED_TRACE("");
     EXPECT_FALSE(export_set->itemize(&pc, &export_set));
     EXPECT_FALSE(export_set->fix_fields(thd(), nullptr));
@@ -593,7 +486,7 @@ TEST_F(ItemTest, ItemFuncIntDivOverflow) {
   const char divisor_str[] = "0.5";
   Item_float *dividend = new Item_float(dividend_str, sizeof(dividend_str));
   Item_float *divisor = new Item_float(divisor_str, sizeof(divisor_str));
-  Item_func_div_int *quotient = new Item_func_div_int(dividend, divisor);
+  Item_func_int_div *quotient = new Item_func_int_div(dividend, divisor);
 
   Mock_error_handler error_handler(thd(), ER_TRUNCATED_WRONG_VALUE);
   EXPECT_FALSE(quotient->fix_fields(thd(), nullptr));
@@ -607,7 +500,7 @@ TEST_F(ItemTest, ItemFuncIntDivUnderflow) {
   const char divisor_str[] = "1.7976931348623157E+308";
   Item_float *dividend = new Item_float(dividend_str, sizeof(dividend_str));
   Item_float *divisor = new Item_float(divisor_str, sizeof(divisor_str));
-  Item_func_div_int *quotient = new Item_func_div_int(dividend, divisor);
+  Item_func_int_div *quotient = new Item_func_int_div(dividend, divisor);
 
   Mock_error_handler error_handler(thd(), ER_TRUNCATED_WRONG_VALUE);
   EXPECT_FALSE(quotient->fix_fields(thd(), nullptr));
@@ -635,7 +528,7 @@ TEST_F(ItemTest, ItemFuncSetUserVar) {
 
   LEX_CSTRING var_name = {STRING_WITH_LEN("a")};
   Item_func_set_user_var *user_var =
-      new Item_func_set_user_var(var_name, item_str);
+      new Item_func_set_user_var(var_name, item_str, false);
   EXPECT_FALSE(user_var->set_entry(thd(), true));
   EXPECT_FALSE(user_var->fix_fields(thd(), nullptr));
   EXPECT_EQ(val1, user_var->val_int());
@@ -655,7 +548,7 @@ TEST_F(ItemTest, OutOfMemory) {
   Item_int *item = new Item_int(42);
   EXPECT_NE(nullptr, item);
 
-#if !defined(NDEBUG)
+#if !defined(DBUG_OFF)
   // Setting debug flags triggers enter/exit trace, so redirect to /dev/null.
   DBUG_SET("o," IF_WIN("NUL", "/dev/null"));
 
@@ -727,7 +620,7 @@ TEST_F(ItemTest, MysqlTimeCache) {
   MysqlTime datetime6(2011, 11, 7, 10, 20, 30, 123456, false,
                       MYSQL_TIMESTAMP_DATETIME);
   MysqlTime time6(0, 0, 0, 10, 20, 30, 123456, false, MYSQL_TIMESTAMP_TIME);
-  my_timeval tv6 = {1320661230, 123456};
+  struct timeval tv6 = {1320661230, 123456};
   const MYSQL_TIME *ltime;
   MYSQL_TIME_cache cache;
 
@@ -806,8 +699,8 @@ TEST_F(ItemTest, MysqlTimeCache) {
   /*
     Testing DATETIME(5)
   */
-  MysqlTime datetime5 = {
-      2011, 11, 7, 10, 20, 30, 123450, false, MYSQL_TIMESTAMP_DATETIME};
+  MysqlTime datetime5(2011, 11, 7, 10, 20, 30, 123450, false,
+                      MYSQL_TIMESTAMP_DATETIME);
   cache.set_datetime(&datetime5, 5);
   EXPECT_EQ(1840440237558456890LL, cache.val_packed());
   EXPECT_EQ(5, cache.decimals());
@@ -875,7 +768,7 @@ TEST_F(ItemTest, ItemFuncConvIntMin) {
   Item *item_conv = new Item_func_conv(POS(), new Item_string("5", 1, &charset),
                                        new Item_int(INT_MIN),   // from_base
                                        new Item_int(INT_MIN));  // to_base
-  Parse_context pc(thd(), thd()->lex->current_query_block());
+  Parse_context pc(thd(), thd()->lex->current_select());
   EXPECT_FALSE(item_conv->itemize(&pc, &item_conv));
   EXPECT_FALSE(item_conv->fix_fields(thd(), nullptr));
   const String *null_string = nullptr;
@@ -887,15 +780,19 @@ TEST_F(ItemTest, ItemDecimalTypecast) {
   const char msg[] = "";
   POS pos;
   pos.cpp.start = pos.cpp.end = pos.raw.start = pos.raw.end = msg;
+  // Sun Studio needs this null_item,
+  // it fails to compile EXPECT_EQ(NULL, create_func_cast());
+  const Item *null_item = nullptr;
 
   Cast_type type;
   type.target = ITEM_CAST_DECIMAL;
+
   type.length = "123456789012345678901234567890";
   type.dec = nullptr;
 
   {
     initializer.set_expected_error(ER_TOO_BIG_PRECISION);
-    EXPECT_EQ(nullptr, create_func_cast(thd(), pos, nullptr, type, false));
+    EXPECT_EQ(null_item, create_func_cast(thd(), pos, nullptr, &type));
   }
 
   {
@@ -904,14 +801,14 @@ TEST_F(ItemTest, ItemDecimalTypecast) {
     type.length = buff;
     type.dec = nullptr;
     initializer.set_expected_error(ER_TOO_BIG_PRECISION);
-    EXPECT_EQ(nullptr, create_func_cast(thd(), pos, nullptr, type, false));
+    EXPECT_EQ(null_item, create_func_cast(thd(), pos, nullptr, &type));
   }
 
   {
     type.length = nullptr;
     type.dec = "123456789012345678901234567890";
     initializer.set_expected_error(ER_TOO_BIG_SCALE);
-    EXPECT_EQ(nullptr, create_func_cast(thd(), pos, nullptr, type, false));
+    EXPECT_EQ(null_item, create_func_cast(thd(), pos, nullptr, &type));
   }
 
   {
@@ -920,7 +817,7 @@ TEST_F(ItemTest, ItemDecimalTypecast) {
     type.length = buff;
     type.dec = buff;
     initializer.set_expected_error(ER_TOO_BIG_SCALE);
-    EXPECT_EQ(nullptr, create_func_cast(thd(), pos, nullptr, type, false));
+    EXPECT_EQ(null_item, create_func_cast(thd(), pos, nullptr, &type));
   }
 }
 
