@@ -1,16 +1,15 @@
-/* Copyright (c) 2014, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -21,14 +20,13 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
-#include <assert.h>
 #include <gtest/gtest.h>
 #include <stddef.h>
-
 #include <sstream>
 #include <string>
 #include <vector>
 
+#include "my_dbug.h"
 #include "my_inttypes.h"
 #include "my_table_map.h"
 #include "sql/item_cmpfunc.h"
@@ -58,16 +56,17 @@ class ItemFilterTest : public ::testing::Test {
  protected:
   ItemFilterTest() : rows_in_table(200) {}
 
-  void SetUp() override {
+  virtual void SetUp() {
     initializer.SetUp();
     init_sql_alloc(PSI_NOT_INSTRUMENTED, &m_alloc,
-                   thd()->variables.range_alloc_block_size);
+                   thd()->variables.range_alloc_block_size, 0);
   }
 
-  void TearDown() override {
+  virtual void TearDown() {
     delete m_table;
 
     initializer.TearDown();
+    free_root(&m_alloc, MYF(0));
   }
 
   THD *thd() { return initializer.thd(); }
@@ -173,15 +172,14 @@ class ItemFilterTest : public ::testing::Test {
 #define create_initem_check_filter(er, lst, ut, fti) \
   do_create_initem_check_filter(er, lst, ut, fti, TestFailLinePrinter(__LINE__))
   Item_func_in *do_create_initem_check_filter(
-      const float expected_result, const mem_root_deque<Item *> &lst,
-      const table_map used_tables, MY_BITMAP *fields_to_ignore,
-      TestFailLinePrinter called_from_line) {
+      const float expected_result, List<Item> &lst, const table_map used_tables,
+      MY_BITMAP *fields_to_ignore, TestFailLinePrinter called_from_line) {
     SCOPED_TRACE(called_from_line);
 
-    PT_item_list *list = new (thd()->mem_root) PT_item_list(POS());
+    PT_item_list *list = new (thd()->mem_root) PT_item_list;
     list->value = lst;
     Item_func_in *in_item = new Item_func_in(POS(), list, false);
-    Parse_context pc(thd(), thd()->lex->current_query_block());
+    Parse_context pc(thd(), thd()->lex->current_select());
     EXPECT_FALSE(in_item->itemize(&pc, (Item **)&in_item));
 
     Item *itm = static_cast<Item *>(in_item);
@@ -206,7 +204,7 @@ class ItemFilterTest : public ::testing::Test {
   MEM_ROOT m_alloc;
 
   Fake_TABLE *m_table;
-  Table_ref *m_table_list;
+  TABLE_LIST *m_table_list;
   /*
     Pointer to m_table->field. Only valid if the table was
     created by calling one of ItemFilterTest::create_table*()
@@ -243,7 +241,7 @@ Item_func *ItemFilterTest::create_item(Item_func::Functype type, Field *fld,
       result = new Item_func_isnotnull(new Item_field(fld));
       break;
     case Item_func::BETWEEN: {
-      Parse_context pc(thd(), thd()->lex->current_query_block());
+      Parse_context pc(thd(), thd()->lex->current_select());
       result =
           new Item_func_between(POS(), new Item_field(fld), new Item_int(val1),
                                 new Item_int(val2), false);
@@ -252,7 +250,7 @@ Item_func *ItemFilterTest::create_item(Item_func::Functype type, Field *fld,
     }
     default:
       result = nullptr;
-      assert(false);
+      DBUG_ASSERT(false);
       return result;
   }
   Item *itm = static_cast<Item *>(result);
@@ -297,7 +295,7 @@ TEST_F(ItemFilterTest, BasicDefaultRows) {
                            m_field[0], unused_int, unused_int, used_tables,
                            &no_ignore_flds);
   // Check filtering for predicate: field0 IS NOT NULL
-  create_item_check_filter(1.0F - COND_FILTER_EQUALITY,
+  create_item_check_filter(1.0 - COND_FILTER_EQUALITY,
                            Item_func::ISNOTNULL_FUNC, m_field[0], unused_int,
                            unused_int, used_tables, &no_ignore_flds);
   // Check filtering for predicate: field0 BETWEEN 10 AND 12
@@ -319,7 +317,7 @@ TEST_F(ItemFilterTest, BasicIgnoreField) {
   // Predicates on m_field[0] should be ignored
   MY_BITMAP ignore_fld0;
   bitmap_init(&ignore_fld0, nullptr, m_table->s->fields);
-  bitmap_set_bit(&ignore_fld0, m_field[0]->field_index());
+  bitmap_set_bit(&ignore_fld0, m_field[0]->field_index);
 
   // Check filtering for predicate on ignored field: field0 = 10
   create_item_check_filter(COND_FILTER_ALLPASS, Item_func::MULT_EQUAL_FUNC,
@@ -403,7 +401,7 @@ TEST_F(ItemFilterTest, BasicConstAnd) {
     Calculate filtering of (field0=10 AND field1=10 AND field2<99)
     while ignoring predicates on field0
   */
-  bitmap_set_bit(&ignore_flds, m_field[0]->field_index());
+  bitmap_set_bit(&ignore_flds, m_field[0]->field_index);
 
   float filter = and_it->get_filtering_effect(
       thd(), m_table_list->map(), used_tables, &ignore_flds, rows_in_table);
@@ -413,7 +411,7 @@ TEST_F(ItemFilterTest, BasicConstAnd) {
     Calculate filtering of (field0=10 AND field1=10 AND field2<99)
     while ignoring predicates on field0 and field1
   */
-  bitmap_set_bit(&ignore_flds, m_field[1]->field_index());
+  bitmap_set_bit(&ignore_flds, m_field[1]->field_index);
   filter = and_it->get_filtering_effect(thd(), m_table_list->map(), used_tables,
                                         &ignore_flds, rows_in_table);
   EXPECT_FLOAT_EQ(COND_FILTER_INEQUALITY, filter);
@@ -422,7 +420,7 @@ TEST_F(ItemFilterTest, BasicConstAnd) {
     Calculate filtering of (field0=10 AND field1=10 AND field2<99)
     while ignoring predicates on field0, field1 and field2
   */
-  bitmap_set_bit(&ignore_flds, m_field[2]->field_index());
+  bitmap_set_bit(&ignore_flds, m_field[2]->field_index);
   filter = and_it->get_filtering_effect(thd(), m_table_list->map(), used_tables,
                                         &ignore_flds, rows_in_table);
   EXPECT_FLOAT_EQ(COND_FILTER_ALLPASS, filter);
@@ -487,7 +485,7 @@ TEST_F(ItemFilterTest, BasicConstOr) {
     The result is always 1.0 if any of the ORed predicates is 1.0.
 
   */
-  bitmap_set_bit(&ignore_flds, m_field[0]->field_index());
+  bitmap_set_bit(&ignore_flds, m_field[0]->field_index);
   const float filt_ign0 = or_it->get_filtering_effect(
       thd(), m_table_list->map(), used_tables, &ignore_flds, rows_in_table);
   EXPECT_FLOAT_EQ(COND_FILTER_ALLPASS, filt_ign0);
@@ -496,7 +494,7 @@ TEST_F(ItemFilterTest, BasicConstOr) {
     Calculate filtering of (field0=10 OR field1=10 OR field2<99) while
     ignoring predicates on m_field[0] and m_field[1]
   */
-  bitmap_set_bit(&ignore_flds, m_field[1]->field_index());
+  bitmap_set_bit(&ignore_flds, m_field[1]->field_index);
   const float filt_ign1 = or_it->get_filtering_effect(
       thd(), m_table_list->map(), used_tables, &ignore_flds, rows_in_table);
   EXPECT_FLOAT_EQ(COND_FILTER_ALLPASS, filt_ign1);
@@ -506,7 +504,7 @@ TEST_F(ItemFilterTest, BasicConstOr) {
     ignoring predicates on m_field[2]
   */
   bitmap_clear_all(&ignore_flds);
-  bitmap_set_bit(&ignore_flds, m_field[2]->field_index());
+  bitmap_set_bit(&ignore_flds, m_field[2]->field_index);
   const float filt_ign2 = or_it->get_filtering_effect(
       thd(), m_table_list->map(), used_tables, &ignore_flds, rows_in_table);
   EXPECT_FLOAT_EQ(COND_FILTER_ALLPASS, filt_ign2);
@@ -527,7 +525,7 @@ TEST_F(ItemFilterTest, InPredicate) {
   bitmap_init(&ignore_flds, nullptr, m_table->s->fields);
 
   // Calculate filtering effect of "col IN (1)"
-  mem_root_deque<Item *> in_lst1(*THR_MALLOC);
+  List<Item> in_lst1;
   in_lst1.push_back(new Item_field(m_field[0]));
   in_lst1.push_back(new Item_int(1));
 
@@ -535,7 +533,7 @@ TEST_F(ItemFilterTest, InPredicate) {
                              &ignore_flds);
 
   // Calculate filtering effect of "col IN (1, ..., 4)"
-  mem_root_deque<Item *> in_lst2(*THR_MALLOC);
+  List<Item> in_lst2;
   in_lst2.push_back(new Item_field(m_field[0]));
   in_lst2.push_back(new Item_int(1));
   in_lst2.push_back(new Item_int(2));
@@ -553,7 +551,7 @@ TEST_F(ItemFilterTest, InPredicate) {
     110 * COND_FILTER_EQUALITY = 0.6, but the filtering effect
     of IN has an upper limit of 0.5.
   */
-  mem_root_deque<Item *> in_lst3(*THR_MALLOC);
+  List<Item> in_lst3;
   in_lst3.push_back(new Item_field(m_field[0]));
   for (int i = 1; i <= 110; i++) in_lst3.push_back(new Item_int(i));
 
@@ -564,7 +562,7 @@ TEST_F(ItemFilterTest, InPredicate) {
     There is no filtering effect for "col IN (1,..,6)" if 'col' is
     ignored.
   */
-  bitmap_set_bit(&ignore_flds, m_field[0]->field_index());
+  bitmap_set_bit(&ignore_flds, m_field[0]->field_index);
   const float filt_ign = in_it->get_filtering_effect(
       thd(), m_table_list->map(), used_tables, &ignore_flds, rows_in_table);
   EXPECT_FLOAT_EQ(COND_FILTER_ALLPASS, filt_ign);
