@@ -1,16 +1,15 @@
-/* Copyright (c) 2015, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -21,44 +20,20 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
-#include <atomic>
-#include <cassert>
+#include <gtest/gtest.h>
 #include <cstring>
-#include <initializer_list>
-#include <map>
 #include <memory>
-#include <new>
-#include <string>
-#include <utility>
-
-#include "gtest/gtest.h"
 
 #include "base64.h"
-#include "decimal.h"
-#include "field_types.h"
 #include "my_byteorder.h"
 #include "my_inttypes.h"
-#include "my_sys.h"
-#include "my_time.h"
-#include "mysql/components/services/bits/psi_bits.h"
-#include "mysql/mysql_lex_string.h"
-#include "mysql/strings/m_ctype.h"
-#include "mysql_time.h"
-#include "mysqld_error.h"
-#include "sql-common/json_binary.h"
-#include "sql-common/json_diff.h"
-#include "sql-common/json_dom.h"
-#include "sql-common/json_error_handler.h"
-#include "sql-common/json_path.h"
-#include "sql-common/my_decimal.h"
-#include "sql/current_thd.h"
-#include "sql/field.h"
-#include "sql/item_json_func.h"
-#include "sql/psi_memory_key.h"
+#include "sql/json_binary.h"
+#include "sql/json_diff.h"
+#include "sql/json_dom.h"
+#include "sql/json_path.h"
+#include "sql/my_decimal.h"
 #include "sql/sql_class.h"
-#include "sql/sql_const.h"
 #include "sql/sql_time.h"
-#include "sql/table.h"
 #include "sql_string.h"
 #include "template_utils.h"  // down_cast
 #include "unittest/gunit/base_mock_field.h"
@@ -75,16 +50,14 @@ namespace json_dom_unittest {
 class JsonDomTest : public ::testing::Test {
  protected:
   Base_mock_field_json m_field{};
-  Fake_TABLE *m_table;
+  Fake_TABLE m_table{&m_field};
   void SetUp() override {
     initializer.SetUp();
-    m_table = new Fake_TABLE(&m_field);
     m_field.make_writable();
-    m_table->in_use = thd();
+    m_table.in_use = thd();
   }
   void TearDown() override {
-    m_table->cleanup_partial_update();
-    delete m_table;
+    m_table.cleanup_partial_update();
     initializer.TearDown();
   }
   my_testing::Server_initializer initializer;
@@ -100,8 +73,7 @@ class JsonDomTest : public ::testing::Test {
 static std::string format(const Json_dom &d) {
   String buffer;
   Json_wrapper w(d.clone());
-  EXPECT_FALSE(
-      w.to_string(&buffer, true, "format", [] { ASSERT_TRUE(false); }));
+  EXPECT_FALSE(w.to_string(&buffer, true, "format"));
 
   return std::string(buffer.ptr(), buffer.length());
 }
@@ -116,9 +88,8 @@ static std::string format(const Json_dom_ptr &ptr) { return format(*ptr); }
   @return a DOM representing the JSON document
 */
 static Json_dom_ptr parse_json(const char *json_text) {
-  auto dom = Json_dom::parse(
-      json_text, std::strlen(json_text), [](const char *, size_t) {},
-      [] { ASSERT_TRUE(false); });
+  auto dom =
+      Json_dom::parse(json_text, std::strlen(json_text), nullptr, nullptr);
   EXPECT_FALSE(dom == nullptr);
   return dom;
 }
@@ -129,7 +100,7 @@ static Json_dom_ptr parse_json(const char *json_text) {
   @return the parsed path
 */
 static Json_path parse_path(const char *json_path) {
-  Json_path path(key_memory_JSON);
+  Json_path path;
   size_t bad_index;
   EXPECT_FALSE(parse_path(std::strlen(json_path), json_path, &path, &bad_index))
       << "bad index: " << bad_index;
@@ -425,15 +396,13 @@ TEST_F(JsonDomTest, BasicTest) {
      Included so we test error recovery
   */
   const char *half_object_item = "{\"label\": ";
-  dom = Json_dom::parse(
-      half_object_item, std::strlen(half_object_item),
-      [](const char *, size_t) {}, [] { ASSERT_TRUE(false); });
+  dom = Json_dom::parse(half_object_item, std::strlen(half_object_item),
+                        nullptr, nullptr);
   EXPECT_EQ(nullptr, dom);
 
   const char *half_array_item = "[1,";
-  dom = Json_dom::parse(
-      half_array_item, std::strlen(half_array_item),
-      [](const char *, size_t) {}, [] { ASSERT_TRUE(false); });
+  dom = Json_dom::parse(half_array_item, std::strlen(half_array_item), nullptr,
+                        nullptr);
   EXPECT_EQ(nullptr, dom);
 }
 
@@ -459,16 +428,16 @@ TEST_F(JsonDomTest, EscapeSpecialChars) {
   EXPECT_EQ(str.value(), str2->value());
 }
 
-void vet_wrapper_length(const char *text, size_t expected_length) {
+void vet_wrapper_length(const THD *thd, const char *text,
+                        size_t expected_length) {
   Json_wrapper dom_wrapper(parse_json(text));
 
   EXPECT_EQ(expected_length, dom_wrapper.length())
       << "Wrapped DOM: " << text << "\n";
 
   String serialized_form;
-  EXPECT_FALSE(json_binary::serialize(
-      dom_wrapper.to_dom(), JsonSerializationDefaultErrorHandler(current_thd),
-      &serialized_form));
+  EXPECT_FALSE(
+      json_binary::serialize(thd, dom_wrapper.to_dom(thd), &serialized_form));
   json_binary::Value binary = json_binary::parse_binary(
       serialized_form.ptr(), serialized_form.length());
   Json_wrapper binary_wrapper(binary);
@@ -488,30 +457,31 @@ TEST_F(JsonDomTest, WrapperTest) {
   // Constructors, assignment, copy constructors, aliasing
   Json_dom *d = new (std::nothrow) Json_null();
   Json_wrapper w(d);
-  EXPECT_EQ(w.to_dom(), d);
+  const THD *thd = this->thd();
+  EXPECT_EQ(w.to_dom(thd), d);
   Json_wrapper w_2(w);
-  EXPECT_NE(w.to_dom(), w_2.to_dom());  // deep copy
+  EXPECT_NE(w.to_dom(thd), w_2.to_dom(thd));  // deep copy
 
   Json_wrapper w_2b;
   EXPECT_TRUE(w_2b.empty());
   w_2b = w;
-  EXPECT_NE(w.to_dom(), w_2b.to_dom());  // deep copy
+  EXPECT_NE(w.to_dom(thd), w_2b.to_dom(thd));  // deep copy
 
   w.set_alias();  // d is now "free" again
   Json_wrapper w_3(w);
-  EXPECT_EQ(w.to_dom(), w_3.to_dom());  // alias copy
+  EXPECT_EQ(w.to_dom(thd), w_3.to_dom(thd));  // alias copy
   w_3 = w;
-  EXPECT_EQ(w.to_dom(), w_3.to_dom());  // alias copy
+  EXPECT_EQ(w.to_dom(thd), w_3.to_dom(thd));  // alias copy
 
   Json_wrapper w_4(d);  // give d a new owner
   Json_wrapper w_5;
   w_5 = std::move(w_4);  // takes over d
-  EXPECT_EQ(w_4.to_dom(), w_5.to_dom());
+  EXPECT_EQ(w_4.to_dom(thd), w_5.to_dom(thd));
 
   Json_wrapper w_6;
   EXPECT_EQ(enum_json_type::J_ERROR, w_6.type());
-  EXPECT_EQ(nullptr, w_6.to_dom());
-  EXPECT_EQ(nullptr, w_6.clone_dom());
+  EXPECT_EQ(nullptr, w_6.to_dom(thd));
+  EXPECT_EQ(nullptr, w_6.clone_dom(thd));
   EXPECT_EQ(0U, w_6.length());
 
   Json_dom *i = new (std::nothrow) Json_int(1);
@@ -519,27 +489,27 @@ TEST_F(JsonDomTest, WrapperTest) {
   w_5 = std::move(w_7);  // should deallocate w_5's original
 
   // scalars
-  vet_wrapper_length("false", 1);
-  vet_wrapper_length("true", 1);
-  vet_wrapper_length("null", 1);
-  vet_wrapper_length("1.1", 1);
-  vet_wrapper_length("\"hello world\"", 1);
+  vet_wrapper_length(thd, "false", 1);
+  vet_wrapper_length(thd, "true", 1);
+  vet_wrapper_length(thd, "null", 1);
+  vet_wrapper_length(thd, "1.1", 1);
+  vet_wrapper_length(thd, "\"hello world\"", 1);
 
   // objects
-  vet_wrapper_length("{}", 0);
-  vet_wrapper_length("{ \"a\" : 100 }", 1);
-  vet_wrapper_length("{ \"a\" : 100, \"b\" : 200 }", 2);
+  vet_wrapper_length(thd, "{}", 0);
+  vet_wrapper_length(thd, "{ \"a\" : 100 }", 1);
+  vet_wrapper_length(thd, "{ \"a\" : 100, \"b\" : 200 }", 2);
 
   // arrays
-  vet_wrapper_length("[]", 0);
-  vet_wrapper_length("[ 100 ]", 1);
-  vet_wrapper_length("[ 100, 200 ]", 2);
+  vet_wrapper_length(thd, "[]", 0);
+  vet_wrapper_length(thd, "[ 100 ]", 1);
+  vet_wrapper_length(thd, "[ 100, 200 ]", 2);
 
   // nested objects
-  vet_wrapper_length("{ \"a\" : 100, \"b\" : { \"c\" : 300 } }", 2);
+  vet_wrapper_length(thd, "{ \"a\" : 100, \"b\" : { \"c\" : 300 } }", 2);
 
   // nested arrays
-  vet_wrapper_length("[ 100, [ 200, 300 ] ]", 2);
+  vet_wrapper_length(thd, "[ 100, [ 200, 300 ] ]", 2);
 }
 
 void vet_merge(const char *left_text, const char *right_text,
@@ -679,7 +649,7 @@ TEST_F(JsonDomTest, MergeTest) {
 
 /// Create a JSON path for accessing an array element at the given position.
 static Json_path array_accessor(size_t idx) {
-  Json_path path(key_memory_JSON);
+  Json_path path;
   path.append(Json_path_leg(idx));
   return path;
 }
@@ -725,32 +695,31 @@ TEST_F(JsonDomTest, AttemptBinaryUpdate) {
   auto dom = parse_json("[\"abc\", 123, \"def\", -70000]");
 
   String buffer;
-  EXPECT_FALSE(json_binary::serialize(
-      dom.get(), JsonSerializationDefaultErrorHandler(thd()), &buffer));
+  EXPECT_FALSE(json_binary::serialize(thd(), dom.get(), &buffer));
 
   json_binary::Value binary =
       json_binary::parse_binary(buffer.ptr(), buffer.length());
 
-  EXPECT_FALSE(m_table->mark_column_for_partial_update(&m_field));
-  EXPECT_FALSE(m_table->setup_partial_update(true));
+  EXPECT_FALSE(m_table.mark_column_for_partial_update(&m_field));
+  EXPECT_FALSE(m_table.setup_partial_update(true));
 
   // Verify that the table interface for partial update works.
-  EXPECT_TRUE(m_table->has_binary_diff_columns());
-  EXPECT_TRUE(m_table->is_marked_for_partial_update(&m_field));
-  EXPECT_TRUE(m_table->is_binary_diff_enabled(&m_field));
-  EXPECT_NE(nullptr, m_table->get_binary_diffs(&m_field));
-  m_table->disable_binary_diffs_for_current_row(&m_field);
-  EXPECT_FALSE(m_table->has_binary_diff_columns());
-  EXPECT_TRUE(m_table->is_marked_for_partial_update(&m_field));
-  EXPECT_FALSE(m_table->is_binary_diff_enabled(&m_field));
-  EXPECT_EQ(nullptr, m_table->get_binary_diffs(&m_field));
-  m_table->clear_partial_update_diffs();
-  EXPECT_TRUE(m_table->has_binary_diff_columns());
-  EXPECT_TRUE(m_table->is_marked_for_partial_update(&m_field));
-  EXPECT_TRUE(m_table->is_binary_diff_enabled(&m_field));
-  EXPECT_NE(nullptr, m_table->get_binary_diffs(&m_field));
+  EXPECT_TRUE(m_table.has_binary_diff_columns());
+  EXPECT_TRUE(m_table.is_marked_for_partial_update(&m_field));
+  EXPECT_TRUE(m_table.is_binary_diff_enabled(&m_field));
+  EXPECT_NE(nullptr, m_table.get_binary_diffs(&m_field));
+  m_table.disable_binary_diffs_for_current_row(&m_field);
+  EXPECT_FALSE(m_table.has_binary_diff_columns());
+  EXPECT_TRUE(m_table.is_marked_for_partial_update(&m_field));
+  EXPECT_FALSE(m_table.is_binary_diff_enabled(&m_field));
+  EXPECT_EQ(nullptr, m_table.get_binary_diffs(&m_field));
+  m_table.clear_partial_update_diffs();
+  EXPECT_TRUE(m_table.has_binary_diff_columns());
+  EXPECT_TRUE(m_table.is_marked_for_partial_update(&m_field));
+  EXPECT_TRUE(m_table.is_binary_diff_enabled(&m_field));
+  EXPECT_NE(nullptr, m_table.get_binary_diffs(&m_field));
 
-  const Binary_diff_vector *diffs = m_table->get_binary_diffs(&m_field);
+  const Binary_diff_vector *diffs = m_table.get_binary_diffs(&m_field);
   EXPECT_TRUE(diffs != nullptr);
   EXPECT_EQ(0U, diffs->size());
 
@@ -760,7 +729,7 @@ TEST_F(JsonDomTest, AttemptBinaryUpdate) {
     Json_wrapper doc(binary);
     EXPECT_EQ(TYPE_OK, m_field.store_json(&doc));
     Json_wrapper jstr(new (std::nothrow) Json_string("abcd"));
-    m_table->clear_partial_update_diffs();
+    m_table.clear_partial_update_diffs();
     auto path = array_accessor(i);
     String shadow;
     bool success = true;
@@ -771,7 +740,7 @@ TEST_F(JsonDomTest, AttemptBinaryUpdate) {
     EXPECT_FALSE(replaced);
     EXPECT_EQ(0U, diffs->size());
     EXPECT_EQ(0, doc.compare(Json_wrapper(binary)));
-    EXPECT_EQ(0U, m_table->get_logical_diffs(&m_field)->size());
+    EXPECT_EQ(0U, m_table.get_logical_diffs(&m_field)->size());
   }
 
   // Enough space for an inlinable value anywhere in the array.
@@ -781,7 +750,7 @@ TEST_F(JsonDomTest, AttemptBinaryUpdate) {
     EXPECT_EQ(TYPE_OK, m_field.store_json(&doc));
     EXPECT_FALSE(m_field.val_json(&doc));
     Json_wrapper jint(new (std::nothrow) Json_int(456));
-    m_table->clear_partial_update_diffs();
+    m_table.clear_partial_update_diffs();
     auto path = array_accessor(i);
     String shadow;
     bool success = false;
@@ -794,7 +763,7 @@ TEST_F(JsonDomTest, AttemptBinaryUpdate) {
 
     Json_array_ptr array(down_cast<Json_array *>(dom->clone().release()));
     array->remove(i);
-    array->insert_clone(i, jint.to_dom());
+    array->insert_clone(i, jint.to_dom(thd()));
     EXPECT_EQ(0, doc.compare(Json_wrapper(std::move(array))));
 
     EXPECT_EQ(TYPE_OK, m_field.store_json(&doc));
@@ -808,7 +777,7 @@ TEST_F(JsonDomTest, AttemptBinaryUpdate) {
     EXPECT_EQ(TYPE_OK, m_field.store_json(&doc));
     EXPECT_FALSE(m_field.val_json(&doc));
     Json_wrapper jint(new (std::nothrow) Json_uint(80000));
-    m_table->clear_partial_update_diffs();
+    m_table.clear_partial_update_diffs();
     auto path = array_accessor(i);
     String shadow;
     bool success = false;
@@ -819,7 +788,7 @@ TEST_F(JsonDomTest, AttemptBinaryUpdate) {
     if (!success) {
       EXPECT_FALSE(replaced);
       EXPECT_EQ(0U, diffs->size());
-      EXPECT_EQ(0U, m_table->get_logical_diffs(&m_field)->size());
+      EXPECT_EQ(0U, m_table.get_logical_diffs(&m_field)->size());
       continue;
     }
     EXPECT_TRUE(replaced);
@@ -827,7 +796,7 @@ TEST_F(JsonDomTest, AttemptBinaryUpdate) {
 
     Json_array_ptr array(down_cast<Json_array *>(dom->clone().release()));
     array->remove(i);
-    array->insert_clone(i, jint.to_dom());
+    array->insert_clone(i, jint.to_dom(thd()));
     EXPECT_EQ(0, doc.compare(Json_wrapper(std::move(array))));
 
     EXPECT_EQ(TYPE_OK, m_field.store_json(&doc));
@@ -840,7 +809,7 @@ TEST_F(JsonDomTest, AttemptBinaryUpdate) {
     EXPECT_EQ(TYPE_OK, m_field.store_json(&doc));
     EXPECT_FALSE(m_field.val_json(&doc));
     Json_wrapper jint(new (std::nothrow) Json_int(456));
-    m_table->clear_partial_update_diffs();
+    m_table.clear_partial_update_diffs();
     String shadow;
     auto path = array_accessor(100);
 
@@ -855,7 +824,7 @@ TEST_F(JsonDomTest, AttemptBinaryUpdate) {
     EXPECT_FALSE(success);
     EXPECT_FALSE(replaced);
     EXPECT_EQ(0U, diffs->size());
-    EXPECT_EQ(0U, m_table->get_logical_diffs(&m_field)->size());
+    EXPECT_EQ(0U, m_table.get_logical_diffs(&m_field)->size());
 
     /*
       JSON_REPLACE is a no-op if a non-existing path is given, so expect
@@ -867,22 +836,21 @@ TEST_F(JsonDomTest, AttemptBinaryUpdate) {
     EXPECT_FALSE(replaced);
     EXPECT_EQ(0U, diffs->size());
     EXPECT_EQ(0, doc.compare(Json_wrapper(binary)));
-    EXPECT_EQ(0U, m_table->get_logical_diffs(&m_field)->size());
+    EXPECT_EQ(0U, m_table.get_logical_diffs(&m_field)->size());
 
     /*
       If we replace the top-level document (empty path), we do a full update.
       Expect the attempt to do partial update to fail.
     */
-    EXPECT_FALSE(
-        doc.attempt_binary_update(&m_field, Json_path(PSI_NOT_INSTRUMENTED),
-                                  &jint, false, &shadow, &success, &replaced));
+    EXPECT_FALSE(doc.attempt_binary_update(&m_field, Json_path(), &jint, false,
+                                           &shadow, &success, &replaced));
     EXPECT_FALSE(success);
     EXPECT_FALSE(replaced);
   }
 
   {
     SCOPED_TRACE("");
-    m_table->clear_partial_update_diffs();
+    m_table.clear_partial_update_diffs();
     Json_wrapper doc(binary);
     EXPECT_EQ(TYPE_OK, m_field.store_json(&doc));
     EXPECT_FALSE(m_field.val_json(&doc));
@@ -900,7 +868,7 @@ TEST_F(JsonDomTest, AttemptBinaryUpdate) {
       EXPECT_TRUE(success);
       EXPECT_TRUE(replaced);
       array->remove(i);
-      array->insert_clone(i, jstr.to_dom());
+      array->insert_clone(i, jstr.to_dom(thd()));
       EXPECT_EQ(0, doc.compare(array_wrapper));
 
       EXPECT_EQ(TYPE_OK, m_field.store_json(&doc));
@@ -925,8 +893,8 @@ TEST_F(JsonDomTest, AttemptBinaryUpdate) {
 */
 TEST_F(JsonDomTest, AttemptBinaryUpdate_AllTypes) {
   // Make the table ready for partial update.
-  EXPECT_FALSE(m_table->mark_column_for_partial_update(&m_field));
-  EXPECT_FALSE(m_table->setup_partial_update(true));
+  EXPECT_FALSE(m_table.mark_column_for_partial_update(&m_field));
+  EXPECT_FALSE(m_table.setup_partial_update(true));
 
   my_decimal decimal;
   EXPECT_FALSE(double2my_decimal(0, 3.14, &decimal));
@@ -970,7 +938,7 @@ TEST_F(JsonDomTest, AttemptBinaryUpdate_AllTypes) {
   };
 
   for (auto dom : doms) {
-    m_table->clear_partial_update_diffs();
+    m_table.clear_partial_update_diffs();
 
     Json_array_ptr original_dom(new (std::nothrow) Json_array);
     original_dom->append_alias(new (std::nothrow) Json_string(20, 'x'));
@@ -985,8 +953,7 @@ TEST_F(JsonDomTest, AttemptBinaryUpdate_AllTypes) {
     EXPECT_FALSE(m_field.val_json(&doc));
 
     StringBuffer<STRING_BUFFER_USUAL_SIZE> original;
-    EXPECT_FALSE(
-        doc.to_binary(JsonSerializationDefaultErrorHandler(thd()), &original));
+    EXPECT_FALSE(doc.to_binary(thd(), &original));
 
     Json_wrapper new_value(dom->clone());
 
@@ -1000,9 +967,9 @@ TEST_F(JsonDomTest, AttemptBinaryUpdate_AllTypes) {
                                            &replaced));
     EXPECT_TRUE(success);
     EXPECT_FALSE(replaced);
-    EXPECT_EQ(0U, m_table->get_binary_diffs(&m_field)->size());
-    EXPECT_EQ(0U, m_table->get_logical_diffs(&m_field)->size());
-    m_table->clear_partial_update_diffs();
+    EXPECT_EQ(0U, m_table.get_binary_diffs(&m_field)->size());
+    EXPECT_EQ(0U, m_table.get_logical_diffs(&m_field)->size());
+    m_table.clear_partial_update_diffs();
 
     // Try with non-existing path and JSON_SET logic. Requires full update.
     EXPECT_FALSE(doc.attempt_binary_update(&m_field, array_accessor(100),
@@ -1010,8 +977,8 @@ TEST_F(JsonDomTest, AttemptBinaryUpdate_AllTypes) {
                                            &replaced));
     EXPECT_FALSE(success);
     EXPECT_FALSE(replaced);
-    EXPECT_EQ(0U, m_table->get_binary_diffs(&m_field)->size());
-    m_table->clear_partial_update_diffs();
+    EXPECT_EQ(0U, m_table.get_binary_diffs(&m_field)->size());
+    m_table.clear_partial_update_diffs();
 
     // Try with a valid path. Expect success.
     EXPECT_FALSE(doc.attempt_binary_update(&m_field, array_accessor(0),
@@ -1019,10 +986,10 @@ TEST_F(JsonDomTest, AttemptBinaryUpdate_AllTypes) {
                                            &replaced));
     EXPECT_TRUE(success);
     EXPECT_TRUE(replaced);
-    EXPECT_NE(0U, m_table->get_binary_diffs(&m_field)->size());
+    EXPECT_NE(0U, m_table.get_binary_diffs(&m_field)->size());
 
     String str;
-    new_value.to_string(&str, true, "test", [] { ASSERT_TRUE(false); });
+    new_value.to_string(&str, true, "test");
     // Verify the updated document.
     {
       Json_array_ptr a(new (std::nothrow) Json_array);
@@ -1032,7 +999,7 @@ TEST_F(JsonDomTest, AttemptBinaryUpdate_AllTypes) {
 
     // Verify the binary diffs.
     EXPECT_EQ(TYPE_OK, m_field.store_json(&doc));
-    verify_binary_diffs(&m_field, m_table->get_binary_diffs(&m_field), original,
+    verify_binary_diffs(&m_field, m_table.get_binary_diffs(&m_field), original,
                         buffer);
 
     delete dom;
@@ -1043,8 +1010,8 @@ TEST_F(JsonDomTest, AttemptBinaryUpdate_AllTypes) {
   Test attempt_binary_update() with invalid input.
 */
 TEST_F(JsonDomTest, AttemptBinaryUpdate_Error) {
-  EXPECT_FALSE(m_table->mark_column_for_partial_update(&m_field));
-  EXPECT_FALSE(m_table->setup_partial_update(true));
+  EXPECT_FALSE(m_table.mark_column_for_partial_update(&m_field));
+  EXPECT_FALSE(m_table.setup_partial_update(true));
   Json_wrapper doc(parse_json("[1,2,3,4]"));
   EXPECT_EQ(TYPE_OK, m_field.store_json(&doc));
   EXPECT_FALSE(m_field.val_json(&doc));
@@ -1057,7 +1024,7 @@ TEST_F(JsonDomTest, AttemptBinaryUpdate_Error) {
   EXPECT_EQ(enum_json_type::J_ERROR, error.type());
 
   // Expect the calls to attempt_binary_update() to fail.
-  const ErrorHandlerFunctionPointer old_error_handler_hook = error_handler_hook;
+  const auto old_error_handler_hook = error_handler_hook;
   error_handler_hook = my_message_sql;
   String buffer;
   bool success;
@@ -1112,12 +1079,11 @@ void test_apply_json_diffs(Field_json *field, const Json_diff_vector &diffs,
 
   TABLE *table = field->table;
   if (table->is_binary_diff_enabled(field)) {
-    JsonSerializationDefaultErrorHandler error_handler{current_thd};
     StringBuffer<STRING_BUFFER_USUAL_SIZE> original;
-    EXPECT_FALSE(json_binary::serialize(parse_json(orig_json).get(),
-                                        error_handler, &original));
+    EXPECT_FALSE(json_binary::serialize(
+        table->in_use, parse_json(orig_json).get(), &original));
     StringBuffer<STRING_BUFFER_USUAL_SIZE> updated;
-    EXPECT_FALSE(doc.to_binary(error_handler, &updated));
+    EXPECT_FALSE(doc.to_binary(table->in_use, &updated));
     verify_binary_diffs(field, table->get_binary_diffs(field), original,
                         updated);
   }
@@ -1269,8 +1235,8 @@ static void do_apply_json_diffs_tests(Field_json *field) {
 TEST_F(JsonDomTest, ApplyJsonDiffs) { do_apply_json_diffs_tests(&m_field); }
 
 TEST_F(JsonDomTest, ApplyJsonDiffs_CollectBinaryDiffs) {
-  EXPECT_FALSE(m_table->mark_column_for_partial_update(&m_field));
-  EXPECT_FALSE(m_table->setup_partial_update(true));
+  EXPECT_FALSE(m_table.mark_column_for_partial_update(&m_field));
+  EXPECT_FALSE(m_table.setup_partial_update(true));
   do_apply_json_diffs_tests(&m_field);
 }
 
@@ -1332,63 +1298,6 @@ static void BM_JsonDomSearchKey(size_t num_iterations) {
 }
 BENCHMARK(BM_JsonDomSearchKey)
 
-/**
-  Microbenchmark for Json_dom::parse(const json_binary::Value &v).
-
-  @param num_iterations the number of iterations to run
-  @param json_text      the string to be parsed(after being converted to binary)
-*/
-static void benchmark_dom_parse(size_t num_iterations, const char *json_text) {
-  StopBenchmarkTiming();
-  my_testing::Server_initializer initializer;
-  initializer.SetUp();
-
-  auto dom = parse_json(json_text);
-
-  String buffer;
-  EXPECT_FALSE(json_binary::serialize(
-      dom.get(), JsonSerializationDefaultErrorHandler(initializer.thd()),
-      &buffer));
-
-  json_binary::Value binary =
-      json_binary::parse_binary(buffer.ptr(), buffer.length());
-
-  StartBenchmarkTiming();
-  for (size_t i = 0; i < num_iterations; ++i) {
-    Json_dom_ptr ptr = Json_dom::parse(binary);
-    assert(ptr);
-  }
-
-  StopBenchmarkTiming();
-}
-
-/**
-  Microbenchmark that tests Json_dom::parse(const json_binary::Value &v)
-  with a deep JSON value
-
-  @param num_iterations the number of iterations to run
-*/
-static void BM_JsonDomParseDeep(size_t num_iterations) {
-  benchmark_dom_parse(num_iterations,
-                      "[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[["
-                      "[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[["
-                      R"({"key":123})"
-                      "]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]"
-                      "]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]");
-}
-BENCHMARK(BM_JsonDomParseDeep)
-
-/**
-  Microbenchmark that tests Json_dom::parse(const json_binary::Value &v)
-  with a shallow JSON value
-
-  @param num_iterations the number of iterations to run
-*/
-static void BM_JsonDomParseShallow(size_t num_iterations) {
-  benchmark_dom_parse(num_iterations, R"([{"key":123}])");
-}
-BENCHMARK(BM_JsonDomParseShallow)
-
 /**
   Run a microbenchmarks that tests how fast Json_wrapper::seek() is on
   a wrapper that wraps a binary JSON value.
@@ -1410,8 +1319,7 @@ static void benchmark_binary_seek(size_t num_iterations, const Json_path &path,
   initializer.SetUp();
 
   String buffer;
-  EXPECT_FALSE(json_binary::serialize(
-      &o, JsonSerializationDefaultErrorHandler(initializer.thd()), &buffer));
+  EXPECT_FALSE(json_binary::serialize(initializer.thd(), &o, &buffer));
   json_binary::Value val =
       json_binary::parse_binary(buffer.ptr(), buffer.length());
 
@@ -1476,7 +1384,7 @@ static void BM_JsonStringToString_Plain(size_t num_iterations) {
 
   for (size_t i = 0; i < num_iterations; ++i) {
     String buf;
-    wr.to_string(&buf, true, "test", [] { ASSERT_TRUE(false); });
+    wr.to_string(&buf, true, "test");
     EXPECT_EQ(quoted_length, buf.length());
   }
 
@@ -1504,7 +1412,7 @@ static void BM_JsonStringToString_SpecialChars(size_t num_iterations) {
 
   for (size_t i = 0; i < num_iterations; ++i) {
     String buf;
-    wr.to_string(&buf, true, "test", [] { ASSERT_TRUE(false); });
+    wr.to_string(&buf, true, "test");
     EXPECT_LT(str.size(), buf.length());
   }
 
@@ -1534,7 +1442,7 @@ static void BM_JsonObjectToString(size_t num_iterations) {
 
   for (size_t i = 0; i < num_iterations; ++i) {
     String buf;
-    wr.to_string(&buf, true, "test", [] { ASSERT_TRUE(false); });
+    wr.to_string(&buf, true, "test");
     EXPECT_LT(0U, buf.length());
   }
 
@@ -1564,7 +1472,7 @@ static void BM_JsonBooleanArrayToString(size_t num_iterations) {
 
   for (size_t i = 0; i < num_iterations; ++i) {
     String buf;
-    wrapper.to_string(&buf, true, "test", [] { ASSERT_TRUE(false); });
+    wrapper.to_string(&buf, true, "test");
     EXPECT_LT(0U, buf.length());
   }
 
@@ -1594,7 +1502,7 @@ static void BM_JsonDoubleArrayToString(size_t num_iterations) {
 
   for (size_t i = 0; i < num_iterations; ++i) {
     String buf;
-    wrapper.to_string(&buf, true, "test", [] { ASSERT_TRUE(false); });
+    wrapper.to_string(&buf, true, "test");
     EXPECT_LT(0U, buf.length());
   }
 
@@ -1626,7 +1534,7 @@ static void BM_JsonDecimalArrayToString(size_t num_iterations) {
 
   for (size_t i = 0; i < num_iterations; ++i) {
     String buf;
-    wrapper.to_string(&buf, true, "test", [] { ASSERT_TRUE(false); });
+    wrapper.to_string(&buf, true, "test");
     EXPECT_LT(0U, buf.length());
   }
 
@@ -1657,7 +1565,7 @@ static void BM_JsonDateArrayToString(size_t num_iterations) {
 
   for (size_t i = 0; i < num_iterations; ++i) {
     String buf;
-    wrapper.to_string(&buf, true, "test", [] { ASSERT_TRUE(false); });
+    wrapper.to_string(&buf, true, "test");
     EXPECT_LT(0U, buf.length());
   }
 
@@ -1718,9 +1626,8 @@ static void BM_JsonWrapperObjectIteratorBinary(size_t num_iterations) {
   }
 
   String serialized_object;
-  EXPECT_FALSE(json_binary::serialize(
-      &dom_object, JsonSerializationDefaultErrorHandler(initializer.thd()),
-      &serialized_object));
+  EXPECT_FALSE(json_binary::serialize(initializer.thd(), &dom_object,
+                                      &serialized_object));
   Json_wrapper wrapper(json_binary::parse_binary(serialized_object.ptr(),
                                                  serialized_object.length()));
   EXPECT_EQ(enum_json_type::J_OBJECT, wrapper.type());
