@@ -1,4 +1,4 @@
-/* Copyright (c) 2014, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -20,26 +20,24 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
+// First include (the generated) my_config.h, to get correct platform defines.
+#include "my_config.h"
+
+#include <gmock/gmock.h>
+#include <gtest/gtest.h>
 #include <sys/types.h>
 #include <cstring>
-#include <memory>
-#include <new>
+#include <fstream>
+#include <iostream>
 #include <string>
 #include <utility>
 
-#include "gtest/gtest.h"
-
-#include "mysql/components/services/bits/psi_bits.h"
-#include "sql-common/json_binary.h"
-#include "sql-common/json_dom.h"
-#include "sql-common/json_error_handler.h"
-#include "sql-common/json_path.h"
-#include "sql/psi_memory_key.h"
+#include "m_ctype.h"
+#include "sql/json_dom.h"
+#include "sql/json_path.h"
 #include "sql_string.h"
 #include "unittest/gunit/test_utils.h"
 
-class THD;
-
 /**
  Test json path abstraction.
  */
@@ -47,8 +45,8 @@ namespace json_path_unittest {
 
 class JsonPathTest : public ::testing::Test {
  protected:
-  void SetUp() override { initializer.SetUp(); }
-  void TearDown() override { initializer.TearDown(); }
+  virtual void SetUp() { initializer.SetUp(); }
+  virtual void TearDown() { initializer.TearDown(); }
   THD *thd() const { return initializer.thd(); }
   my_testing::Server_initializer initializer;
   void vet_wrapper_seek(Json_wrapper *wrapper, const Json_path &path,
@@ -123,8 +121,8 @@ class JsonGoodLocationTestP : public ::testing::TestWithParam<Location_tuple> {
   arg of Json_wrapper.seek().
 */
 class JsonGoodOnoTestP : public ::testing::TestWithParam<Ono_tuple> {
-  void SetUp() override { initializer.SetUp(); }
-  void TearDown() override { initializer.TearDown(); }
+  virtual void SetUp() { initializer.SetUp(); }
+  virtual void TearDown() { initializer.TearDown(); }
   my_testing::Server_initializer initializer;
 
  protected:
@@ -135,8 +133,8 @@ class JsonGoodOnoTestP : public ::testing::TestWithParam<Ono_tuple> {
   Class that contains parameterized test cases for cloning tests.
 */
 class JsonGoodCloneTestP : public ::testing::TestWithParam<Clone_tuple> {
-  void SetUp() override { initializer.SetUp(); }
-  void TearDown() override { initializer.TearDown(); }
+  virtual void SetUp() { initializer.SetUp(); }
+  virtual void TearDown() { initializer.TearDown(); }
   my_testing::Server_initializer initializer;
 };
 
@@ -165,7 +163,7 @@ void good_path_common(const char *path_expression, Json_path *json_path) {
 /** Verify that a good path parses correctly */
 void good_path(bool check_path, const char *path_expression,
                std::string expected_path) {
-  Json_path json_path(key_memory_JSON);
+  Json_path json_path;
   good_path_common(path_expression, &json_path);
   if (check_path) {
     String str;
@@ -179,7 +177,7 @@ void good_path(bool check_path, const char *path_expression,
     EXPECT_EQ(expected_path, std::string(str.ptr(), str.length()));
 
     // Move-assign to a new path and verify that it's the same.
-    Json_path path3(key_memory_JSON);
+    Json_path path3;
     path3 = std::move(path2);
     str.length(0);
     EXPECT_FALSE(path3.to_string(&str));
@@ -198,7 +196,7 @@ void good_path(const char *path_expression) {
 
 /** Verify whether the path contains a wildcard, ellipsis or range token. */
 void contains_wildcard(const char *path_expression, bool expected_answer) {
-  Json_path json_path(key_memory_JSON);
+  Json_path json_path;
   good_path_common(path_expression, &json_path);
   EXPECT_EQ(expected_answer, json_path.can_match_many());
 }
@@ -207,7 +205,7 @@ void contains_wildcard(const char *path_expression, bool expected_answer) {
 void good_leg_at(const char *path_expression, int leg_index,
                  const std::string &expected_leg,
                  enum_json_path_leg_type expected_leg_type) {
-  Json_path json_path(key_memory_JSON);
+  Json_path json_path;
   good_path_common(path_expression, &json_path);
 
   const Json_path_leg *actual_leg = *(json_path.begin() + leg_index);
@@ -240,14 +238,14 @@ void compare_paths(Json_path &left, Json_path_clone &right) {
 /** Verify that clones look alike */
 void verify_clone(const char *path_expression_1,
                   const char *path_expression_2) {
-  Json_path real_path1(key_memory_JSON);
+  Json_path real_path1;
   good_path_common(path_expression_1, &real_path1);
 
-  Json_path_clone cloned_path(key_memory_JSON);
+  Json_path_clone cloned_path;
   for (const Json_path_leg *leg : real_path1) cloned_path.append(leg);
   compare_paths(real_path1, cloned_path);
 
-  Json_path real_path2(key_memory_JSON);
+  Json_path real_path2;
   good_path_common(path_expression_2, &real_path2);
   cloned_path.clear();
   for (const Json_path_leg *leg : real_path2) cloned_path.append(leg);
@@ -260,7 +258,7 @@ void verify_clone(const char *path_expression_1,
 void good_leg_types(const char *path_expression,
                     enum_json_path_leg_type *expected_leg_types,
                     size_t length) {
-  Json_path json_path(key_memory_JSON);
+  Json_path json_path;
   good_path_common(path_expression, &json_path);
 
   EXPECT_EQ(length, json_path.leg_count());
@@ -273,7 +271,7 @@ void good_leg_types(const char *path_expression,
 /** Verify that a bad path fails as expected */
 void bad_path(const char *path_expression, size_t expected_index) {
   size_t actual_index = 0;
-  Json_path json_path(key_memory_JSON);
+  Json_path json_path;
   EXPECT_TRUE(parse_path(strlen(path_expression), path_expression, &json_path,
                          &actual_index))
       << "Unexpectedly parsed " << path_expression;
@@ -308,16 +306,14 @@ void JsonPathTest::vet_wrapper_seek(Json_wrapper *wrapper,
   String result_buffer;
 
   if (hits.size() == 1) {
-    EXPECT_FALSE(hits[0].to_string(&result_buffer, true, "test",
-                                   [] { ASSERT_TRUE(false); }));
+    EXPECT_FALSE(hits[0].to_string(&result_buffer, true, "test"));
   } else {
     Json_array *a = new (std::nothrow) Json_array();
     for (uint i = 0; i < hits.size(); ++i) {
-      a->append_clone(hits[i].to_dom());
+      a->append_clone(hits[i].to_dom(thd()));
     }
     Json_wrapper w(a);
-    EXPECT_FALSE(
-        w.to_string(&result_buffer, true, "test", [] { ASSERT_TRUE(false); }));
+    EXPECT_FALSE(w.to_string(&result_buffer, true, "test"));
   }
 
   std::string actual = std::string(result_buffer.ptr(), result_buffer.length());
@@ -328,8 +324,7 @@ void JsonPathTest::vet_wrapper_seek(Json_wrapper *wrapper,
 
     if (hits.size() > 0) {
       String source_buffer;
-      EXPECT_FALSE(wrapper->to_string(&source_buffer, true, "test",
-                                      [] { ASSERT_TRUE(false); }));
+      EXPECT_FALSE(wrapper->to_string(&source_buffer, true, "test"));
       source_output = source_buffer.ptr();
       result_output = actual.c_str();
     }
@@ -345,31 +340,27 @@ void JsonPathTest::vet_wrapper_seek(const char *json_text,
                                     const char *path_text,
                                     const std::string &expected,
                                     bool expected_null) const {
-  Json_dom_ptr dom = Json_dom::parse(
-      json_text, std::strlen(json_text), [](const char *, size_t) {},
-      [] { ASSERT_TRUE(false); });
+  Json_dom_ptr dom =
+      Json_dom::parse(json_text, std::strlen(json_text), nullptr, nullptr);
 
   String serialized_form;
-  EXPECT_FALSE(json_binary::serialize(
-      dom.get(), JsonSerializationDefaultErrorHandler(thd()),
-      &serialized_form));
+  EXPECT_FALSE(json_binary::serialize(thd(), dom.get(), &serialized_form));
   json_binary::Value binary = json_binary::parse_binary(
       serialized_form.ptr(), serialized_form.length());
 
   Json_wrapper dom_wrapper(std::move(dom));
   Json_wrapper binary_wrapper(binary);
 
-  Json_path path(key_memory_JSON);
+  Json_path path;
   good_path_common(path_text, &path);
   vet_wrapper_seek(&dom_wrapper, path, expected, expected_null);
   vet_wrapper_seek(&binary_wrapper, path, expected, expected_null);
 }
 
 void vet_dom_location(const char *json_text, const char *path_text) {
-  Json_dom_ptr dom = Json_dom::parse(
-      json_text, std::strlen(json_text), [](const char *, size_t) {},
-      [] { ASSERT_TRUE(false); });
-  Json_path path(key_memory_JSON);
+  Json_dom_ptr dom =
+      Json_dom::parse(json_text, std::strlen(json_text), nullptr, nullptr);
+  Json_path path;
   good_path_common(path_text, &path);
   Json_dom_vector hits(PSI_NOT_INSTRUMENTED);
 
@@ -409,27 +400,25 @@ void vet_only_needs_one(Json_wrapper &wrapper, const Json_path &path,
   Vet the short-circuiting effects of the only_needs_one argument
   of Json_wrapper.seek().
 
-  @param[in] thd                    Session object.
   @param[in] json_text              Text of the json document to search.
   @param[in] path_text              Text of the path expression to use.
   @param[in] expected_hits          Total number of expected matches.
+  @param[in] thd                    THD handle
 */
-void vet_only_needs_one(const THD *thd, const char *json_text,
-                        const char *path_text, uint expected_hits) {
-  Json_dom_ptr dom = Json_dom::parse(
-      json_text, std::strlen(json_text), [](const char *, size_t) {},
-      [] { ASSERT_TRUE(false); });
+void vet_only_needs_one(const char *json_text, const char *path_text,
+                        uint expected_hits, const THD *thd) {
+  Json_dom_ptr dom =
+      Json_dom::parse(json_text, std::strlen(json_text), nullptr, nullptr);
 
   String serialized_form;
-  EXPECT_FALSE(json_binary::serialize(
-      dom.get(), JsonSerializationDefaultErrorHandler(thd), &serialized_form));
+  EXPECT_FALSE(json_binary::serialize(thd, dom.get(), &serialized_form));
   json_binary::Value binary = json_binary::parse_binary(
       serialized_form.ptr(), serialized_form.length());
 
   Json_wrapper dom_wrapper(std::move(dom));
   Json_wrapper binary_wrapper(binary);
 
-  Json_path path(key_memory_JSON);
+  Json_path path;
   good_path_common(path_text, &path);
   vet_only_needs_one(dom_wrapper, path, expected_hits);
   vet_only_needs_one(binary_wrapper, path, expected_hits);
@@ -450,8 +439,7 @@ void vet_only_needs_one(const THD *thd, const char *json_text,
 std::string format(Json_dom *dom) {
   String buffer;
   Json_wrapper wrapper(dom->clone());
-  EXPECT_FALSE(
-      wrapper.to_string(&buffer, true, "format", [] { ASSERT_TRUE(false); }));
+  EXPECT_FALSE(wrapper.to_string(&buffer, true, "format"));
 
   return std::string(buffer.ptr(), buffer.length());
 }
@@ -559,8 +547,8 @@ TEST_P(JsonGoodPathTestP, GoodPaths) {
   good_path(param.m_path_expression, param.m_expected_path);
 }
 
-INSTANTIATE_TEST_SUITE_P(PositiveNoColumnScope, JsonGoodPathTestP,
-                         ::testing::ValuesIn(good_paths_no_column_scope));
+INSTANTIATE_TEST_CASE_P(PositiveNoColumnScope, JsonGoodPathTestP,
+                        ::testing::ValuesIn(good_paths_no_column_scope));
 
 /** Test that path leg types look correct. */
 TEST_F(JsonPathTest, LegTypes) {
@@ -817,8 +805,8 @@ static const Bad_path bad_paths_no_column_scope[] = {
     {"$[-]", 2},
 };
 
-INSTANTIATE_TEST_SUITE_P(NegativeNoColumnScope, JsonBadPathTestP,
-                         ::testing::ValuesIn(bad_paths_no_column_scope));
+INSTANTIATE_TEST_CASE_P(NegativeNoColumnScope, JsonBadPathTestP,
+                        ::testing::ValuesIn(bad_paths_no_column_scope));
 
 /** Good paths with column scope not supported yet */
 TEST_F(JsonPathTest, PositiveColumnScope) {
@@ -864,8 +852,8 @@ static const Good_path good_quoted_key_names[] = {
     {"$.\"a b\".\" c d \"", "$.\"a b\".\" c d \""},
 };
 
-INSTANTIATE_TEST_SUITE_P(QuotedKeyNamesPositive, JsonGoodPathTestP,
-                         ::testing::ValuesIn(good_quoted_key_names));
+INSTANTIATE_TEST_CASE_P(QuotedKeyNamesPositive, JsonGoodPathTestP,
+                        ::testing::ValuesIn(good_quoted_key_names));
 
 /** Test bad quoted key names */
 static const Bad_path bad_quoted_key_names[] = {
@@ -889,8 +877,8 @@ static const Bad_path bad_quoted_key_names[] = {
     //{ false, "$.ghi\"abcd\"", 5 },
 };
 
-INSTANTIATE_TEST_SUITE_P(QuotedKeyNamesNegative, JsonBadPathTestP,
-                         ::testing::ValuesIn(bad_quoted_key_names));
+INSTANTIATE_TEST_CASE_P(QuotedKeyNamesNegative, JsonBadPathTestP,
+                        ::testing::ValuesIn(bad_quoted_key_names));
 
 /* Test that unquoted key names may not be ECMAScript identifiers */
 
@@ -930,8 +918,8 @@ static const Good_path good_ecmascript_identifiers[] = {
      "bc"},
 };
 
-INSTANTIATE_TEST_SUITE_P(GoodECMAScriptIdentifiers, JsonGoodPathTestP,
-                         ::testing::ValuesIn(good_ecmascript_identifiers));
+INSTANTIATE_TEST_CASE_P(GoodECMAScriptIdentifiers, JsonGoodPathTestP,
+                        ::testing::ValuesIn(good_ecmascript_identifiers));
 
 TEST_F(JsonPathTest, BadECMAScriptIdentifiers) {
   // key names may not contain embedded quotes
@@ -1265,9 +1253,8 @@ TEST_F(JsonPathTest, RemoveDomTest) {
   {
     SCOPED_TRACE("");
     std::string json_text = "[100, 200, 300]";
-    Json_dom_ptr dom = Json_dom::parse(
-        json_text.data(), json_text.length(), [](const char *, size_t) {},
-        [] { ASSERT_TRUE(false); });
+    Json_dom_ptr dom =
+        Json_dom::parse(json_text.data(), json_text.length(), nullptr, nullptr);
     auto array = static_cast<Json_array *>(dom.get());
     EXPECT_TRUE(array->remove(1));
     EXPECT_EQ("[100, 300]", format(array));
@@ -1275,9 +1262,8 @@ TEST_F(JsonPathTest, RemoveDomTest) {
     EXPECT_EQ("[100, 300]", format(array));
 
     json_text = "{\"a\": 100, \"b\": 200, \"c\": 300}";
-    dom = Json_dom::parse(
-        json_text.data(), json_text.length(), [](const char *, size_t) {},
-        [] { ASSERT_TRUE(false); });
+    dom =
+        Json_dom::parse(json_text.data(), json_text.length(), nullptr, nullptr);
     auto object = static_cast<Json_object *>(dom.get());
     EXPECT_TRUE(object->remove("b"));
     EXPECT_EQ("{\"a\": 100, \"c\": 300}", format(object));
@@ -1394,8 +1380,8 @@ TEST_P(JsonGoodLocationTestP, GoodLocations) {
   vet_dom_location(param.m_json_text, param.m_path_expression);
 }
 
-INSTANTIATE_TEST_SUITE_P(LocationTesting, JsonGoodLocationTestP,
-                         ::testing::ValuesIn(location_tuples));
+INSTANTIATE_TEST_CASE_P(LocationTesting, JsonGoodLocationTestP,
+                        ::testing::ValuesIn(location_tuples));
 
 // Tuples for the test of the only_needs_one arg of Json_wrapper.seek()
 static const Ono_tuple ono_tuples[] = {
@@ -1409,12 +1395,12 @@ static const Ono_tuple ono_tuples[] = {
 /** Test good paths without column scope */
 TEST_P(JsonGoodOnoTestP, GoodOno) {
   Ono_tuple param = GetParam();
-  vet_only_needs_one(thd(), param.m_json_text, param.m_path_expression,
-                     param.m_expected_hits);
+  vet_only_needs_one(param.m_json_text, param.m_path_expression,
+                     param.m_expected_hits, thd());
 }
 
-INSTANTIATE_TEST_SUITE_P(OnoTesting, JsonGoodOnoTestP,
-                         ::testing::ValuesIn(ono_tuples));
+INSTANTIATE_TEST_CASE_P(OnoTesting, JsonGoodOnoTestP,
+                        ::testing::ValuesIn(ono_tuples));
 
 // Tuples for tests of cloning
 static const Clone_tuple clone_tuples[] = {
@@ -1429,8 +1415,8 @@ TEST_P(JsonGoodCloneTestP, GoodClone) {
   verify_clone(param.m_path_expression_1, param.m_path_expression_2);
 }
 
-INSTANTIATE_TEST_SUITE_P(CloneTesting, JsonGoodCloneTestP,
-                         ::testing::ValuesIn(clone_tuples));
+INSTANTIATE_TEST_CASE_P(CloneTesting, JsonGoodCloneTestP,
+                        ::testing::ValuesIn(clone_tuples));
 
 /**
   A class used for parameterized test cases for the
@@ -1444,7 +1430,7 @@ TEST_P(JsonPathLegAutowrapP, Autowrap) {
   const std::string path_text = "$" + param.first + ".a";
   const bool expected_result = param.second;
 
-  Json_path path(key_memory_JSON);
+  Json_path path;
   size_t idx = 0;
   EXPECT_FALSE(parse_path(path_text.length(), path_text.data(), &path, &idx));
   EXPECT_EQ(0U, idx);
@@ -1487,7 +1473,7 @@ static const std::pair<std::string, bool> autowrap_tuples[] = {
     {"[last-2 to last-1]", false},
 };
 
-INSTANTIATE_TEST_SUITE_P(AutowrapTesting, JsonPathLegAutowrapP,
-                         ::testing::ValuesIn(autowrap_tuples));
+INSTANTIATE_TEST_CASE_P(AutowrapTesting, JsonPathLegAutowrapP,
+                        ::testing::ValuesIn(autowrap_tuples));
 
 }  // end namespace json_path_unittest
