@@ -1,4 +1,4 @@
-/* Copyright (c) 2016, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -36,14 +36,14 @@ using namespace keyring;
 
 class Keyring_api_test : public ::testing::Test {
  public:
-  Keyring_api_test() = default;
-  ~Keyring_api_test() override {
+  Keyring_api_test() {}
+  ~Keyring_api_test() {
     delete[] plugin_name;
     delete[] keyring_filename;
   }
 
  protected:
-  void SetUp() override {
+  virtual void SetUp() {
     plugin_name = new char[strlen("FakeKeyring") + 1];
     strcpy(plugin_name, "FakeKeyring");
     keyring_filename = new char[strlen("./keyring") + 1];
@@ -62,7 +62,7 @@ class Keyring_api_test : public ::testing::Test {
     key_LOCK_keyring = PSI_NOT_INSTRUMENTED;
     sample_key_data = "Robi";
   }
-  void TearDown() override {
+  virtual void TearDown() {
     keyring_deinit_with_mock_logger();
     remove(keyring_file_data_value);
     remove("./keyring.backup");
@@ -245,6 +245,23 @@ TEST_F(Keyring_api_test, StoreTwiceTheSameDifferentTypes) {
       1);
 }
 
+TEST_F(Keyring_api_test, KeyGenerate) {
+  EXPECT_EQ(mysql_key_generate("Robert_key", "AES", "Robert", 128), 0);
+  char *key_type;
+  size_t key_len;
+  void *key;
+  EXPECT_EQ(mysql_key_fetch("Robert_key", &key_type, "Robert", &key, &key_len),
+            0);
+  EXPECT_STREQ("AES", key_type);
+  EXPECT_EQ(key_len, (size_t)128);
+  // Try accessing the last byte of key
+  char ch = ((char *)key)[key_len - 1];
+  // Just to get rid of unused variable compiler error
+  (void)ch;
+  my_free(key);
+  my_free(key_type);
+}
+
 TEST_F(Keyring_api_test, InitWithDifferentKeyringFile) {
   EXPECT_EQ(
       mysql_key_store("Robert_key", "AES", "Robert", sample_key_data.c_str(),
