@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2022, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -37,8 +37,6 @@
 
 using std::string;
 
-using ::testing::NiceMock;
-
 namespace gcs_xcom_control_unittest {
 typedef enum { JJ = 0, LL = 1, JL = 2, LJ = 3 } InvocationOrder;
 
@@ -142,7 +140,7 @@ class mock_gcs_xcom_view_change_control_interface
     m_joining_leaving_mutex.init(PSI_NOT_INSTRUMENTED, nullptr);
   }
 
-  ~mock_gcs_xcom_view_change_control_interface() override {
+  ~mock_gcs_xcom_view_change_control_interface() {
     m_mutex_current_view.destroy();
     m_joining_leaving_mutex.destroy();
   }
@@ -152,10 +150,7 @@ class mock_gcs_xcom_view_change_control_interface
   MOCK_METHOD0(wait_for_view_change_end, void());
   MOCK_METHOD0(is_view_changing, bool());
 
-  MOCK_METHOD0(finalize, void());
-  MOCK_METHOD0(is_finalized, bool());
-
-  bool start_leave() override {
+  bool start_leave() {
     bool retval = false;
 
     m_joining_leaving_mutex.lock();
@@ -166,13 +161,13 @@ class mock_gcs_xcom_view_change_control_interface
     return !retval;
   }
 
-  void end_leave() override {
+  void end_leave() {
     m_joining_leaving_mutex.lock();
     m_leaving = false;
     m_joining_leaving_mutex.unlock();
   }
 
-  bool is_leaving() override {
+  bool is_leaving() {
     bool retval;
 
     m_joining_leaving_mutex.lock();
@@ -182,7 +177,7 @@ class mock_gcs_xcom_view_change_control_interface
     return retval;
   }
 
-  bool start_join() override {
+  bool start_join() {
     bool retval = false;
 
     m_joining_leaving_mutex.lock();
@@ -193,13 +188,13 @@ class mock_gcs_xcom_view_change_control_interface
     return !retval;
   }
 
-  void end_join() override {
+  void end_join() {
     m_joining_leaving_mutex.lock();
     m_joining = false;
     m_joining_leaving_mutex.unlock();
   }
 
-  bool is_joining() override {
+  bool is_joining() {
     bool retval;
 
     m_joining_leaving_mutex.lock();
@@ -209,18 +204,16 @@ class mock_gcs_xcom_view_change_control_interface
     return retval;
   }
 
-  void set_current_view(Gcs_view *view) override {
+  void set_current_view(Gcs_view *view) {
     m_mutex_current_view.lock();
     delete m_current_view;
     m_current_view = view;
     m_mutex_current_view.unlock();
   }
 
-  void set_unsafe_current_view(Gcs_view *view) override {
-    set_current_view(view);
-  }
+  void set_unsafe_current_view(Gcs_view *view) { set_current_view(view); }
 
-  Gcs_view *get_current_view() override {
+  Gcs_view *get_current_view() {
     Gcs_view *view = nullptr;
 
     m_mutex_current_view.lock();
@@ -230,13 +223,11 @@ class mock_gcs_xcom_view_change_control_interface
     return view;
   }
 
-  Gcs_view *get_unsafe_current_view() override { return m_current_view; }
+  Gcs_view *get_unsafe_current_view() { return m_current_view; }
 
-  bool belongs_to_group() override { return m_belongs_to_group; }
+  bool belongs_to_group() { return m_belongs_to_group; }
 
-  void set_belongs_to_group(bool belong) override {
-    m_belongs_to_group = belong;
-  }
+  void set_belongs_to_group(bool belong) { m_belongs_to_group = belong; }
 };
 
 // This typedef is needed because GMock can't deal with multiple template args.
@@ -343,8 +334,7 @@ class mock_gcs_xcom_proxy : public Gcs_xcom_proxy_base {
   }
 
   MOCK_METHOD3(new_node_address_uuid,
-               node_address *(unsigned int n, char const *names[],
-                              blob uuids[]));
+               node_address *(unsigned int n, char *names[], blob uuids[]));
   MOCK_METHOD2(delete_node_address, void(unsigned int n, node_address *na));
   MOCK_METHOD3(xcom_client_add_node, bool(connection_descriptor *con,
                                           node_list *nl, uint32_t group_id));
@@ -358,13 +348,6 @@ class mock_gcs_xcom_proxy : public Gcs_xcom_proxy_base {
                     xcom_event_horizon event_horizon));
   MOCK_METHOD2(xcom_client_set_event_horizon,
                bool(uint32_t group_id, xcom_event_horizon event_horizon));
-  MOCK_METHOD2(xcom_client_set_max_leaders,
-               bool(uint32_t group_id, node_no max_leaders));
-  MOCK_METHOD4(xcom_client_set_leaders,
-               bool(uint32_t group_id, u_int n, char const *names[],
-                    node_no max_nr_leaders));
-  MOCK_METHOD2(xcom_client_get_leaders,
-               bool(uint32_t gid, leader_info_data &leaders));
   MOCK_METHOD4(xcom_client_get_synode_app_data,
                bool(connection_descriptor *con, uint32_t group_id,
                     synode_no_array &synodes, synode_app_data_array &reply));
@@ -447,49 +430,22 @@ class mock_my_xp_socket_util : public My_xp_socket_util {
   MOCK_METHOD1(disable_nagle_in_socket, int(int fd));
 };
 
-class mock_gcs_network_provider_operations_interface
-    : public Network_provider_operations_interface {
- public:
-  mock_gcs_network_provider_operations_interface() {
-    ON_CALL(*this, start_active_network_provider())
-        .WillByDefault(Return(false));
-    ON_CALL(*this, stop_all_network_providers()).WillByDefault(Return(false));
-    ON_CALL(*this, stop_active_network_provider()).WillByDefault(Return(false));
-    ON_CALL(*this, configure_active_provider(_)).WillByDefault(Return(false));
-    ON_CALL(*this, configure_active_provider_secure_connections(_))
-        .WillByDefault(Return(false));
-  }
-
-  MOCK_METHOD0(start_active_network_provider, bool());
-  MOCK_METHOD0(stop_all_network_providers, bool());
-  MOCK_METHOD0(stop_active_network_provider, bool());
-  MOCK_METHOD1(configure_active_provider,
-               bool(Network_configuration_parameters &params));
-  MOCK_METHOD1(configure_active_provider_secure_connections,
-               bool(Network_configuration_parameters &params));
-  MOCK_METHOD0(remove_all_network_provider, void());
-  MOCK_METHOD1(remove_network_provider,
-               void(enum_transport_protocol provider_key));
-};
-
 class mock_gcs_xcom_control : public Gcs_xcom_control {
  public:
-  mock_gcs_xcom_control(
-      Gcs_xcom_node_address *xcom_node_address,
-      std::vector<Gcs_xcom_node_address *> &xcom_peers,
-      Gcs_group_identifier group_identifier, Gcs_xcom_proxy *xcom_proxy,
-      Gcs_xcom_group_management *xcom_group_management,
-      Gcs_xcom_engine *gcs_engine,
-      Gcs_xcom_state_exchange_interface *state_exchange,
-      Gcs_xcom_view_change_control_interface *view_control, bool boot,
-      My_xp_socket_util *socket_util,
-      std::unique_ptr<Network_provider_operations_interface> network_ops)
+  mock_gcs_xcom_control(Gcs_xcom_node_address *xcom_node_address,
+                        std::vector<Gcs_xcom_node_address *> &xcom_peers,
+                        Gcs_group_identifier group_identifier,
+                        Gcs_xcom_proxy *xcom_proxy,
+                        Gcs_xcom_group_management *xcom_group_management,
+                        Gcs_xcom_engine *gcs_engine,
+                        Gcs_xcom_state_exchange_interface *state_exchange,
+                        Gcs_xcom_view_change_control_interface *view_control,
+                        bool boot, My_xp_socket_util *socket_util)
       : Gcs_xcom_control(xcom_node_address, xcom_peers, group_identifier,
                          xcom_proxy, xcom_group_management, gcs_engine,
-                         state_exchange, view_control, boot, socket_util,
-                         std::move(network_ops)) {}
+                         state_exchange, view_control, boot, socket_util) {}
 
-  enum_gcs_error join() override { return join(nullptr); }
+  enum_gcs_error join() { return join(nullptr); }
 
   enum_gcs_error join(Gcs_view *view) {
     enum_gcs_error ret = GCS_NOK;
@@ -518,7 +474,7 @@ class mock_gcs_xcom_control : public Gcs_xcom_control {
     return ret;
   }
 
-  enum_gcs_error leave() override {
+  enum_gcs_error leave() {
     enum_gcs_error ret = GCS_NOK;
 
     if (!m_view_control->start_leave()) {
@@ -549,18 +505,18 @@ void finalize_xcom() { extern_xcom_control_if->do_leave(); }
 
 class XComControlTest : public GcsBaseTest {
  protected:
-  XComControlTest() = default;
+  XComControlTest() {}
 
-  ~XComControlTest() override = default;
+  virtual ~XComControlTest() {}
 
-  void SetUp() override {
+  virtual void SetUp() {
     m_wait_called = false;
     m_wait_called_mutex.init(PSI_NOT_INSTRUMENTED, nullptr);
     m_wait_called_cond.init(PSI_NOT_INSTRUMENTED);
 
-    mock_se = new NiceMock<mock_gcs_xcom_state_exchange_interface>();
+    mock_se = new mock_gcs_xcom_state_exchange_interface();
 
-    mock_vce = new NiceMock<mock_gcs_xcom_view_change_control_interface>();
+    mock_vce = new mock_gcs_xcom_view_change_control_interface();
 
     xcom_node_address = new Gcs_xcom_node_address("127.0.0.1:12345");
     peers.push_back(new Gcs_xcom_node_address("127.0.0.1:12345"));
@@ -570,24 +526,21 @@ class XComControlTest : public GcsBaseTest {
     string group_name("only_group");
     group_id = new Gcs_group_identifier(group_name);
 
-    mock_socket_util = new NiceMock<mock_my_xp_socket_util>();
+    mock_socket_util = new mock_my_xp_socket_util();
 
     gcs_engine.initialize(nullptr);
 
     xcom_group_mgm = new Gcs_xcom_group_management(&proxy, *group_id);
 
-    mock_net_ops_for_interface =
-        std::make_unique<mock_gcs_network_provider_operations_interface>();
-    mock_net_ops = mock_net_ops_for_interface.get();
-
-    xcom_control_if = new NiceMock<mock_gcs_xcom_control>(
+    xcom_control_if = new mock_gcs_xcom_control(
         xcom_node_address, peers, *group_id, &proxy, xcom_group_mgm,
-        &gcs_engine, mock_se, mock_vce, true, mock_socket_util,
-        std::move(mock_net_ops_for_interface));
+        &gcs_engine, mock_se, mock_vce, true, mock_socket_util);
     extern_xcom_control_if = xcom_control_if;
+
+    My_xp_util::init_time();
   }
 
-  void TearDown() override {
+  virtual void TearDown() {
     gcs_engine.finalize(finalize_xcom);
 
     delete mock_socket_util;
@@ -610,16 +563,13 @@ class XComControlTest : public GcsBaseTest {
 
   Gcs_group_identifier *group_id;
 
-  NiceMock<mock_gcs_xcom_proxy> proxy;
-  NiceMock<mock_gcs_control_event_listener> mock_ev_listener;
+  mock_gcs_xcom_proxy proxy;
+  mock_gcs_control_event_listener mock_ev_listener;
 
-  NiceMock<mock_gcs_xcom_control> *xcom_control_if;
-  NiceMock<mock_gcs_xcom_state_exchange_interface> *mock_se;
-  NiceMock<mock_gcs_xcom_view_change_control_interface> *mock_vce;
-  NiceMock<mock_my_xp_socket_util> *mock_socket_util;
-  mock_gcs_network_provider_operations_interface *mock_net_ops;
-  std::unique_ptr<mock_gcs_network_provider_operations_interface>
-      mock_net_ops_for_interface;
+  mock_gcs_xcom_control *xcom_control_if;
+  mock_gcs_xcom_state_exchange_interface *mock_se;
+  mock_gcs_xcom_view_change_control_interface *mock_vce;
+  mock_my_xp_socket_util *mock_socket_util;
 
   bool m_wait_called;
   My_xp_mutex_impl m_wait_called_mutex;
@@ -649,18 +599,14 @@ class XComControlTest : public GcsBaseTest {
   }
 };
 
-/**
- * @brief This test exercices a basic leave-join scenario.
- *
- */
 TEST_F(XComControlTest, JoinLeaveTest) {
   EXPECT_CALL(proxy, xcom_input_connect(_, _)).Times(1);
   EXPECT_CALL(proxy, test_xcom_tcp_connection(_, _)).Times(1);
   EXPECT_CALL(proxy, xcom_client_boot(_, _)).Times(1);
   EXPECT_CALL(proxy, xcom_client_remove_node(_, _)).Times(1);
-  /*EXPECT_CALL(proxy, xcom_wait_for_xcom_comms_status_change(_))
+  EXPECT_CALL(proxy, xcom_wait_for_xcom_comms_status_change(_))
       .Times(1)
-      .WillOnce(SetArgReferee<0>(GCS_OK));*/
+      .WillOnce(SetArgReferee<0>(GCS_OK));
   EXPECT_CALL(proxy, xcom_wait_ready()).Times(1);
   EXPECT_CALL(proxy, xcom_wait_exit()).Times(1);
   EXPECT_CALL(proxy, xcom_init(_)).Times(1);
@@ -675,16 +621,14 @@ TEST_F(XComControlTest, JoinLeaveTest) {
   ASSERT_FALSE(xcom_control_if->is_xcom_running());
 }
 
-/**
- * @brief This test exercises a join() after a successful join()
- *
- */
 TEST_F(XComControlTest, JoinTestFailedMultipleJoins) {
   EXPECT_CALL(proxy, xcom_input_connect(_, _)).Times(1);
   EXPECT_CALL(proxy, test_xcom_tcp_connection(_, _)).Times(1);
   EXPECT_CALL(proxy, xcom_client_boot(_, _)).Times(1);
   EXPECT_CALL(proxy, xcom_client_remove_node(_, _)).Times(1);
-
+  EXPECT_CALL(proxy, xcom_wait_for_xcom_comms_status_change(_))
+      .Times(1)
+      .WillOnce(SetArgReferee<0>(GCS_OK));
   EXPECT_CALL(proxy, xcom_wait_ready()).Times(1);
   EXPECT_CALL(proxy, xcom_wait_exit()).Times(1);
   EXPECT_CALL(proxy, xcom_init(_)).Times(1);
@@ -703,14 +647,11 @@ TEST_F(XComControlTest, JoinTestFailedMultipleJoins) {
   ASSERT_FALSE(xcom_control_if->is_xcom_running());
 }
 
-/**
- * @brief This test exercises a failed
- *
- */
 TEST_F(XComControlTest, JoinTestFailedToStartComms) {
   EXPECT_CALL(proxy, xcom_client_boot(_, _)).Times(0);
   EXPECT_CALL(proxy, xcom_wait_ready()).Times(0);
   EXPECT_CALL(proxy, xcom_init(_)).Times(1);
+  EXPECT_CALL(proxy, xcom_exit()).Times(1);
 
   /*
     The join is forced to wait until the XCOM's tread is running.
@@ -759,6 +700,28 @@ TEST_F(XComControlTest, JoinTestFailedToSendBootToXCom) {
   ASSERT_FALSE(xcom_control_if->is_xcom_running());
 }
 
+TEST_F(XComControlTest, JoinTestTimeoutStartingComms) {
+  Gcs_xcom_proxy *my_proxy = new Gcs_xcom_proxy_impl();
+
+  EXPECT_CALL(proxy, xcom_init(_)).Times(1);
+  EXPECT_CALL(proxy, xcom_exit()).Times(1);
+
+  /*
+    The join is forced to wait until the XCOM's tread is running.
+    In this test case though, we make the operation time out.
+  */
+  EXPECT_CALL(proxy, xcom_wait_for_xcom_comms_status_change(_))
+      .Times(1)
+      .WillOnce(Invoke(
+          my_proxy, &Gcs_xcom_proxy::xcom_wait_for_xcom_comms_status_change));
+
+  enum_gcs_error result = xcom_control_if->join();
+  ASSERT_EQ(GCS_NOK, result);
+  ASSERT_FALSE(xcom_control_if->is_xcom_running());
+
+  delete my_proxy;
+}
+
 TEST_F(XComControlTest, JoinTestFailedToStartXCom) {
   EXPECT_CALL(proxy, xcom_input_connect(_, _)).Times(1);
   EXPECT_CALL(proxy, test_xcom_tcp_connection(_, _)).Times(1);
@@ -766,6 +729,7 @@ TEST_F(XComControlTest, JoinTestFailedToStartXCom) {
   EXPECT_CALL(proxy, xcom_wait_ready()).Times(1).WillOnce(Return(GCS_NOK));
   EXPECT_CALL(proxy, xcom_init(_)).Times(1);
   EXPECT_CALL(proxy, xcom_exit()).Times(1);
+  EXPECT_CALL(proxy, xcom_wait_for_xcom_comms_status_change(_)).Times(1);
 
   enum_gcs_error result = xcom_control_if->join();
 
@@ -783,6 +747,7 @@ TEST_F(XComControlTest, JoinTestTimeoutStartingXCom) {
       .WillOnce(Invoke(my_proxy, &Gcs_xcom_proxy::xcom_wait_ready));
   EXPECT_CALL(proxy, xcom_init(_)).Times(1);
   EXPECT_CALL(proxy, xcom_exit()).Times(1);
+  EXPECT_CALL(proxy, xcom_wait_for_xcom_comms_status_change(_)).Times(1);
 
   enum_gcs_error result = xcom_control_if->join();
 
@@ -811,34 +776,8 @@ TEST_F(XComControlTest, JoinTestWithoutBootNorPeers) {
 
 TEST_F(XComControlTest, JoinTestSkipOwnNodeAndCycleThroughPeerNodes) {
   connection_descriptor *con =
-      (connection_descriptor *)malloc(sizeof(connection_descriptor));
-  con->fd = 0;
-
-  /*connection_descriptor *failed_con =
       (connection_descriptor *)malloc(sizeof(connection_descriptor *));
-  failed_con->fd = -1;*/
-
-  auto create_failed_con_lambda = [](std::string a, xcom_port b) {
-    (void)a;
-    (void)b;
-
-    connection_descriptor *failed_con =
-        (connection_descriptor *)malloc(sizeof(connection_descriptor));
-    failed_con->fd = -1;
-
-    return failed_con;
-  };
-
-  auto create_good_con_lambda = [](std::string a, xcom_port b) {
-    (void)a;
-    (void)b;
-
-    connection_descriptor *failed_con =
-        (connection_descriptor *)malloc(sizeof(connection_descriptor));
-    failed_con->fd = 0;
-
-    return failed_con;
-  };
+  con->fd = 0;
 
   EXPECT_CALL(proxy, xcom_input_connect(_, _)).Times(1);
   EXPECT_CALL(proxy, test_xcom_tcp_connection(_, _)).Times(1);
@@ -849,7 +788,7 @@ TEST_F(XComControlTest, JoinTestSkipOwnNodeAndCycleThroughPeerNodes) {
   // Fail to connect to the peer every time.
   EXPECT_CALL(proxy, xcom_client_open_connection(Eq("127.0.0.1"), Eq(12346)))
       .Times(3)
-      .WillRepeatedly(create_failed_con_lambda);
+      .WillRepeatedly(Return((connection_descriptor *)nullptr));
 
   /*
    Fail to connect on the first attempt.
@@ -858,8 +797,8 @@ TEST_F(XComControlTest, JoinTestSkipOwnNodeAndCycleThroughPeerNodes) {
   */
   EXPECT_CALL(proxy, xcom_client_open_connection(Eq("127.0.0.1"), Eq(12347)))
       .Times(3)
-      .WillOnce(create_failed_con_lambda)
-      .WillRepeatedly(create_good_con_lambda);
+      .WillOnce(Return((connection_descriptor *)nullptr))
+      .WillRepeatedly(Return((connection_descriptor *)con));
   EXPECT_CALL(*mock_socket_util, disable_nagle_in_socket(_))
       .Times(2)
       .WillOnce(Return(-1))
@@ -881,34 +820,24 @@ TEST_F(XComControlTest, JoinTestSkipOwnNodeAndCycleThroughPeerNodes) {
   result = xcom_control_if->leave();
   ASSERT_EQ(GCS_OK, result);
   ASSERT_FALSE(xcom_control_if->is_xcom_running());
+
+  free(con);
 }
 
 TEST_F(XComControlTest, JoinTestAllPeersUnavailable) {
-  auto create_failed_con_lambda = [](std::string a, xcom_port b) {
-    (void)a;
-    (void)b;
-
-    connection_descriptor *failed_con =
-        (connection_descriptor *)malloc(sizeof(connection_descriptor));
-    failed_con->fd = -1;
-
-    return failed_con;
-  };
-
   EXPECT_CALL(proxy, xcom_input_connect(_, _)).Times(1);
   EXPECT_CALL(proxy, test_xcom_tcp_connection(_, _)).Times(1);
   EXPECT_CALL(proxy, xcom_client_boot(_, _)).Times(0);
   EXPECT_CALL(proxy, xcom_wait_ready()).Times(0);
   EXPECT_CALL(proxy, xcom_init(_)).Times(1);
   EXPECT_CALL(proxy, xcom_exit()).Times(1);
-
   /*
    Fail to connect to all peers every time.
    peers.size() - 1 because we skip our own address.
   */
   EXPECT_CALL(proxy, xcom_client_open_connection(_, _))
       .Times((peers.size() - 1) * Gcs_xcom_control::CONNECTION_ATTEMPTS)
-      .WillRepeatedly(create_failed_con_lambda);
+      .WillRepeatedly(Return((connection_descriptor *)nullptr));
   EXPECT_CALL(*mock_socket_util, disable_nagle_in_socket(_)).Times(0);
   EXPECT_CALL(proxy, xcom_client_add_node(_, _, _)).Times(0);
   EXPECT_CALL(proxy, xcom_client_remove_node(_, _, _)).Times(0);
@@ -1084,14 +1013,9 @@ TEST_F(XComControlTest, ViewChangedJoiningTest) {
   uuid_2.encode(reinterpret_cast<uchar **>(&blob_2.data.data_val),
                 &blob_2.data.data_len);
 
-  node_address node_addrs[2] = {{const_cast<char *>("127.0.0.1:12345"),
-                                 blob_1,
-                                 {x_1_0, x_1_2},
-                                 P_PROP | P_ACC | P_LEARN},
-                                {const_cast<char *>("127.0.0.1:12346"),
-                                 blob_2,
-                                 {x_1_0, x_1_2},
-                                 P_PROP | P_ACC | P_LEARN}};
+  node_address node_addrs[2] = {
+      {const_cast<char *>("127.0.0.1:12345"), blob_1, {x_1_0, x_1_2}},
+      {const_cast<char *>("127.0.0.1:12346"), blob_2, {x_1_0, x_1_2}}};
 
   // Common unit test data
   Gcs_xcom_view_identifier *view_id = new Gcs_xcom_view_identifier(999999, 27);
@@ -1271,18 +1195,10 @@ TEST_F(XComControlTest, FailedNodeRemovalTest) {
   uuid_3.encode(reinterpret_cast<uchar **>(&blob_3.data.data_val),
                 &blob_3.data.data_len);
 
-  node_address node_addrs[3] = {{const_cast<char *>("127.0.0.1:12345"),
-                                 blob_1,
-                                 {x_1_0, x_1_2},
-                                 P_PROP | P_ACC | P_LEARN},
-                                {const_cast<char *>("127.0.0.1:12343"),
-                                 blob_2,
-                                 {x_1_0, x_1_2},
-                                 P_PROP | P_ACC | P_LEARN},
-                                {const_cast<char *>("127.0.0.1:12341"),
-                                 blob_3,
-                                 {x_1_0, x_1_2},
-                                 P_PROP | P_ACC | P_LEARN}};
+  node_address node_addrs[3] = {
+      {const_cast<char *>("127.0.0.1:12345"), blob_1, {x_1_0, x_1_2}},
+      {const_cast<char *>("127.0.0.1:12343"), blob_2, {x_1_0, x_1_2}},
+      {const_cast<char *>("127.0.0.1:12341"), blob_3, {x_1_0, x_1_2}}};
 
   site_def *site_config = new_site_def();
   init_site_def(3, node_addrs, site_config);
@@ -1403,14 +1319,9 @@ TEST_F(XComControlTest, FailedNodeGlobalViewTest) {
   uuid_2.encode(reinterpret_cast<uchar **>(&blob_2.data.data_val),
                 &blob_2.data.data_len);
 
-  node_address node_addrs[2] = {{const_cast<char *>("127.0.0.1:12345"),
-                                 blob_1,
-                                 {x_1_0, x_1_2},
-                                 P_PROP | P_ACC | P_LEARN},
-                                {const_cast<char *>("127.0.0.1:12343"),
-                                 blob_2,
-                                 {x_1_0, x_1_2},
-                                 P_PROP | P_ACC | P_LEARN}};
+  node_address node_addrs[2] = {
+      {const_cast<char *>("127.0.0.1:12345"), blob_1, {x_1_0, x_1_2}},
+      {const_cast<char *>("127.0.0.1:12343"), blob_2, {x_1_0, x_1_2}}};
 
   site_def *site_config = new_site_def();
   init_site_def(2, node_addrs, site_config);
@@ -2014,6 +1925,7 @@ TEST_F(XComControlTest, ParallellJoinsTest) {
   EXPECT_CALL(proxy, test_xcom_tcp_connection(_, _)).Times(1);
   EXPECT_CALL(proxy, xcom_client_boot(_, _)).Times(1);
   EXPECT_CALL(proxy, xcom_client_remove_node(_, _)).Times(1);
+  EXPECT_CALL(proxy, xcom_wait_for_xcom_comms_status_change(_)).Times(1);
   EXPECT_CALL(proxy, xcom_wait_ready()).Times(1);
   EXPECT_CALL(proxy, xcom_init(_)).Times(1);
   EXPECT_CALL(proxy, xcom_exit()).Times(0);
@@ -2256,10 +2168,8 @@ TEST_F(XComControlTest, LocalViewAfterExpel) {
   uuid.encode(reinterpret_cast<uchar **>(&blob.data.data_val),
               &blob.data.data_len);
 
-  node_address node_addrs[1] = {{const_cast<char *>(member_id_2.c_str()),
-                                 blob,
-                                 {x_1_0, x_1_2},
-                                 P_PROP | P_ACC | P_LEARN}};
+  node_address node_addrs[1] = {
+      {const_cast<char *>(member_id_2.c_str()), blob, {x_1_0, x_1_2}}};
 
   site_def *site_config = new_site_def();
   init_site_def(1, node_addrs, site_config);
@@ -2317,9 +2227,9 @@ TEST_F(XComControlTest, LocalViewAfterExpel) {
 
    Upon receiving V2, does NOT expel `suspect_1`. The only reason to expel
    `suspect_1` is because the view states that there is a majority of members
-   alive, `(me, suspect_2)` vs. `{suspect_1}`. But we have already ordered
-   the expel of `suspect_2`, so it should be counted as if it is failed. In
-   this case, there is no majority, `(me)` vs. `{suspect_1, suspect_2}`, so
+   alive, `(me, suspect_2)` vs. `{suspect_1}`. But we have already ordered the
+   expel of `suspect_2`, so it should be counted as if it is failed. In this
+   case, there is no majority, `(me)` vs. `{suspect_1, suspect_2}`, so
    `suspect_1` should not be expelled.
 
    Upon receiving V3, expels `me`, due to the `force_remove` flag in the
@@ -2352,11 +2262,12 @@ TEST_F(XComControlTest, DoNotDisbandEntireGroup) {
   EXPECT_CALL(proxy, test_xcom_tcp_connection(_, _)).Times(1);
   EXPECT_CALL(proxy, xcom_client_boot(_, _)).Times(1);
   EXPECT_CALL(proxy, xcom_client_remove_node(_, _))
-      .Times(2)  // once for expulsion, once for leave
+      .Times(3)  // twice for expulsion, once for leave
       .WillOnce(WithArgs<0>(Invoke(suspect_is_2)))
+      .WillOnce(WithArgs<0>(Invoke(suspect_is_me)))
       .WillOnce(WithArgs<0>(Invoke(suspect_is_me)));
   EXPECT_CALL(proxy, delete_node_address(_, _))
-      .Times(3);  // once for boot, once for expulsion, once for leave
+      .Times(4);  // once for boot, twice for expulsion, once for leave
 
   enum_gcs_error result = xcom_control_if->join(create_fake_view());
   ASSERT_EQ(GCS_OK, result);
@@ -2402,7 +2313,7 @@ TEST_F(XComControlTest, DoNotDisbandEntireGroup) {
   ASSERT_EQ(mgr->has_majority(), false);
 
   /* Test expulsion of me.
-     We should NOT issue the expel, as specified in the EXPECT_CALL(proxy,
+     We should issue the expel, as specified in the EXPECT_CALL(proxy,
      xcom_client_remove_node(_, _)) above. */
   // Build vector with suspect nodes
   member_suspect_nodes = std::vector<Gcs_member_identifier *>{me.get()};
