@@ -1,16 +1,15 @@
-/* Copyright (c) 2010, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -21,12 +20,12 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
-#include <assert.h>
 #include <gtest/gtest.h>
 
+#include "m_ctype.h"
+#include "my_dbug.h"
 #include "my_inttypes.h"
 #include "my_sys.h"
-#include "mysql/strings/m_ctype.h"
 
 namespace like_range_unittest {
 
@@ -75,21 +74,21 @@ static const char *charset_list[] = {
 
     "ujis_japanese_ci",   "ujis_bin",
 
-    "utf8mb3_general_ci", "utf8mb3_unicode_ci",
-    "utf8mb3_bin",
+    "utf8_general_ci",    "utf8_unicode_ci", "utf8_bin",
 };
 
 class LikeRangeTest : public ::testing::TestWithParam<const char *> {
  protected:
-  void SetUp() override {
-    m_charset = get_charset_by_name(GetParam(), MYF(0));
-    assert(m_charset);
+  virtual void SetUp() {
+    MY_CHARSET_LOADER loader;
+    my_charset_loader_init_mysys(&loader);
+    m_charset = my_collation_get_by_name(&loader, GetParam(), MYF(0));
+    DBUG_ASSERT(m_charset);
   }
   CHARSET_INFO *m_charset;
 };
 
-INSTANTIATE_TEST_SUITE_P(Foo1, LikeRangeTest,
-                         ::testing::ValuesIn(charset_list));
+INSTANTIATE_TEST_CASE_P(Foo1, LikeRangeTest, ::testing::ValuesIn(charset_list));
 
 TEST_P(LikeRangeTest, TestLikeRange) {
   test_like_range_for_charset(m_charset, "abc%", 4);
