@@ -1,16 +1,15 @@
-/* Copyright (c) 2012, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2012, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -58,8 +57,8 @@ class MakeSortKeyTest : public ::testing::Test {
     m_to = Bounds_checked_array<uchar>(&m_buff[8], sizeof(m_buff) - 8);
   }
 
-  void SetUp() override { initializer.SetUp(); }
-  void TearDown() override { initializer.TearDown(); }
+  virtual void SetUp() { initializer.SetUp(); }
+  virtual void TearDown() { initializer.TearDown(); }
 
   THD *thd() { return initializer.thd(); }
 
@@ -76,6 +75,7 @@ class MakeSortKeyTest : public ::testing::Test {
 
   Sort_param m_sort_param;
   st_sort_field m_sort_fields[2];  // sortlength() adds an end marker !!
+  uchar m_ref_buff[4];             // unused, but needed for make_sortkey()
   uchar m_buff[100];
   Bounds_checked_array<uchar> m_to;
 };
@@ -90,7 +90,7 @@ TEST_F(MakeSortKeyTest, IntResult) {
   EXPECT_EQ(INT_RESULT, m_sort_fields[0].result_type);
 
   size_t longest_addon_so_far = 0;  // Unused.
-  m_sort_param.make_sortkey(m_to, {}, &longest_addon_so_far);
+  m_sort_param.make_sortkey(m_to, m_ref_buff, &longest_addon_so_far);
   SCOPED_TRACE("");
   verify_buff(total_length);
 }
@@ -98,7 +98,7 @@ TEST_F(MakeSortKeyTest, IntResult) {
 TEST_F(MakeSortKeyTest, IntResultNull) {
   thd()->variables.max_sort_length = 4U;
   Item *int_item = m_sort_fields[0].item = new Item_int(42);
-  int_item->set_nullable(true);
+  int_item->maybe_null = true;
   int_item->null_value = true;
 
   const uint total_length = sortlength(thd(), m_sort_fields, 1);
@@ -107,7 +107,7 @@ TEST_F(MakeSortKeyTest, IntResultNull) {
   EXPECT_EQ(INT_RESULT, m_sort_fields[0].result_type);
 
   size_t longest_addon_so_far = 0;  // Unused.
-  m_sort_param.make_sortkey(m_to, {}, &longest_addon_so_far);
+  m_sort_param.make_sortkey(m_to, m_ref_buff, &longest_addon_so_far);
   SCOPED_TRACE("");
   verify_buff(total_length);
 }
@@ -117,7 +117,7 @@ TEST_F(MakeSortKeyTest, DecimalResult) {
   thd()->variables.max_sort_length = 4U;
   m_sort_fields[0].item =
       new Item_decimal(POS(), dec_str, strlen(dec_str), &my_charset_bin);
-  Parse_context pc(thd(), thd()->lex->current_query_block());
+  Parse_context pc(thd(), thd()->lex->current_select());
   EXPECT_FALSE(m_sort_fields[0].item->itemize(&pc, &m_sort_fields[0].item));
 
   const uint total_length = sortlength(thd(), m_sort_fields, 1);
@@ -126,7 +126,7 @@ TEST_F(MakeSortKeyTest, DecimalResult) {
   EXPECT_EQ(DECIMAL_RESULT, m_sort_fields[0].result_type);
 
   size_t longest_addon_so_far = 0;  // Unused.
-  m_sort_param.make_sortkey(m_to, {}, &longest_addon_so_far);
+  m_sort_param.make_sortkey(m_to, m_ref_buff, &longest_addon_so_far);
   SCOPED_TRACE("");
   verify_buff(total_length);
 }
@@ -142,7 +142,7 @@ TEST_F(MakeSortKeyTest, RealResult) {
   EXPECT_EQ(REAL_RESULT, m_sort_fields[0].result_type);
 
   size_t longest_addon_so_far = 0;  // Unused.
-  m_sort_param.make_sortkey(m_to, {}, &longest_addon_so_far);
+  m_sort_param.make_sortkey(m_to, m_ref_buff, &longest_addon_so_far);
   SCOPED_TRACE("");
   verify_buff(total_length);
 }
@@ -155,35 +155,34 @@ TEST_F(MakeSortKeyTest, AddonFields) {
   EXPECT_EQ(INT_RESULT, m_sort_fields[0].result_type);
 
   Sort_addon_field addon_field;
-  float val = static_cast<float>(M_PI);
+  float val = M_PI;
   Field_float field(nullptr, 0, nullptr, '\0', Field::NONE, "", 0, false,
                     false);
   Fake_TABLE table(&field);
   table.s->db_low_byte_first = false;
-  field.set_field_ptr(reinterpret_cast<unsigned char *>(&val));
+  field.ptr = reinterpret_cast<unsigned char *>(&val);
   addon_field.field = &field;
+  addon_field.offset = 4;  // Need room for the length bytes.
   addon_field.max_length = field.max_packed_col_length();
   Addon_fields addon_fields(make_array(&addon_field, 1));
-  addon_fields.set_first_addon_relative_offset(0);
   addon_fields.set_using_packed_addons(true);
   m_sort_param.addon_fields = &addon_fields;
 
   // Test regular packing.
   size_t longest_addon_so_far = 0;  // Unused.
-  size_t len = m_sort_param.make_sortkey(m_to, {}, &longest_addon_so_far);
-  EXPECT_EQ(total_length + sizeof(float) + addon_fields.first_addon_offset(),
-            len);
+  size_t len =
+      m_sort_param.make_sortkey(m_to, m_ref_buff, &longest_addon_so_far);
+  EXPECT_EQ(total_length + sizeof(float) + addon_field.offset, len);
   float unpacked_val;
   field.unpack(reinterpret_cast<uchar *>(&unpacked_val),
-               m_to.array() + m_sort_fields[0].length +
-                   addon_fields.first_addon_offset(),
-               /*param_data=*/0);
+               m_to.array() + m_sort_fields[0].length + addon_field.offset,
+               /*param_data=*/0, /*low_byte_first=*/false);
   EXPECT_EQ(unpacked_val, val);
 
   // Test truncation. (The actual contents don't matter in this case.)
   std::unique_ptr<uchar[]> trunc_buf(new uchar[len - 4]);
   size_t trunc_len = m_sort_param.make_sortkey(
-      make_array(trunc_buf.get(), len - 4), {}, &longest_addon_so_far);
+      make_array(trunc_buf.get(), len - 4), m_ref_buff, &longest_addon_so_far);
   EXPECT_GT(trunc_len, len - 4)
       << "make_sortkey() should report back that there was not enough room.";
 }
