@@ -1,16 +1,15 @@
-/* Copyright (c) 2011, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2011, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -61,8 +60,8 @@ class Mock_field_timestamp : public Field_timestamp {
     initialize();
   }
 
-  my_timeval to_timeval() {
-    my_timeval tm;
+  timeval to_timeval() {
+    timeval tm;
     int warnings = 0;
     get_timestamp(&tm, &warnings);
     EXPECT_EQ(0, warnings);
@@ -70,16 +69,16 @@ class Mock_field_timestamp : public Field_timestamp {
   }
 
   /* Averts ASSERT_COLUMN_MARKED_FOR_WRITE assertion. */
-  void make_writable() { bitmap_set_bit(table->write_set, field_index()); }
-  void make_readable() { bitmap_set_bit(table->read_set, field_index()); }
+  void make_writable() { bitmap_set_bit(table->write_set, field_index); }
+  void make_readable() { bitmap_set_bit(table->read_set, field_index); }
 
-  void store_timestamp(const my_timeval *tm) override {
+  void store_timestamp(const timeval *tm) {
     make_writable();
     Field_temporal_with_date_and_time::store_timestamp(tm);
     store_timestamp_called = true;
   }
 
-  ~Mock_field_timestamp() override { delete static_cast<Fake_TABLE *>(table); }
+  ~Mock_field_timestamp() { delete static_cast<Fake_TABLE *>(table); }
 };
 
 #endif  // MOCK_FIELD_TIMESTAMP_H
