@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2015, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,8 +24,6 @@
 #include <gtest/gtest.h>
 #include <stddef.h>
 #include <sys/types.h>
-#include <memory>
-#include <string>
 
 #include "my_alloc.h"
 #include "my_inttypes.h"
@@ -71,7 +68,7 @@ class Mock_global_error_handler {
   uint m_expected_error;
   int m_handle_called;
 
-  ErrorHandlerFunctionPointer m_old_error_handler_hook;
+  void (*m_old_error_handler_hook)(uint, const char *, myf);
 };
 
 Mock_global_error_handler *Mock_global_error_handler::current = nullptr;
@@ -90,61 +87,54 @@ const size_t num_iterations = 1ULL;
 
 class MyAllocTest : public ::testing::TestWithParam<size_t> {
  protected:
-  void SetUp() override {
-    ::new ((void *)&m_root) MEM_ROOT(PSI_NOT_INSTRUMENTED, 1024);
+  virtual void SetUp() {
+    init_alloc_root(PSI_NOT_INSTRUMENTED, &m_root, 1024, 0);
   }
-  void TearDown() override { m_root.Clear(); }
+  virtual void TearDown() { free_root(&m_root, MYF(0)); }
   size_t m_num_objects;
   MEM_ROOT m_root;
 };
 
 class MyPreAllocTest : public ::testing::Test {
  protected:
-  void SetUp() override {
-    ::new ((void *)&m_prealloc_root) MEM_ROOT(PSI_NOT_INSTRUMENTED, 1024);
+  virtual void SetUp() {
+    init_alloc_root(PSI_NOT_INSTRUMENTED, &m_prealloc_root, 1024, 2048);
   }
-  void TearDown() override { m_prealloc_root.Clear(); }
+  virtual void TearDown() { free_root(&m_prealloc_root, MYF(0)); }
   size_t m_num_objects;
   MEM_ROOT m_prealloc_root;
 };
 
 size_t test_values[] = {100, 1000, 10000, 100000};
 
-INSTANTIATE_TEST_SUITE_P(MyAlloc, MyAllocTest,
-                         ::testing::ValuesIn(test_values));
+INSTANTIATE_TEST_CASE_P(MyAlloc, MyAllocTest, ::testing::ValuesIn(test_values));
 
 TEST_P(MyAllocTest, NoMemoryLimit) {
   m_num_objects = GetParam();
   for (size_t ix = 0; ix < num_iterations; ++ix) {
     for (size_t objcount = 0; objcount < m_num_objects; ++objcount)
-      EXPECT_NE(nullptr, m_root.Alloc(8));
+      m_root.Alloc(100);
   }
-  // Normally larger, but with Valgrind/ASan, we'll get exact-sized blocks,
-  // so also allow equal.
-  EXPECT_GE(m_root.allocated_size(), num_iterations * m_num_objects * 8);
 }
 
 TEST_P(MyAllocTest, WithMemoryLimit) {
   m_num_objects = GetParam();
-  m_root.set_max_capacity(num_iterations * m_num_objects * 8);
+  m_root.set_max_capacity(num_iterations * m_num_objects * 100);
   for (size_t ix = 0; ix < num_iterations; ++ix) {
     for (size_t objcount = 0; objcount < m_num_objects; ++objcount)
-      EXPECT_NE(nullptr, m_root.Alloc(8));
+      m_root.Alloc(100);
   }
-  EXPECT_EQ(m_root.allocated_size(), num_iterations * m_num_objects * 8);
 }
 
 TEST_F(MyAllocTest, CheckErrorReporting) {
-  EXPECT_NE(nullptr, m_root.Alloc(1000));
+  const void *null_pointer = nullptr;
+  EXPECT_TRUE(m_root.Alloc(1000));
   m_root.set_max_capacity(100);
-  EXPECT_EQ(nullptr, m_root.Alloc(1000));
+  EXPECT_EQ(null_pointer, m_root.Alloc(1000));
   m_root.set_error_for_capacity_exceeded(true);
   Mock_global_error_handler error_handler(EE_CAPACITY_EXCEEDED);
-  EXPECT_NE(nullptr, m_root.Alloc(1000));
+  EXPECT_TRUE(m_root.Alloc(1000));
   EXPECT_EQ(1, error_handler.handle_called());
-
-  EXPECT_FALSE(m_root.ForceNewBlock(2048));
-  EXPECT_EQ(2, error_handler.handle_called());
 }
 
 TEST_F(MyAllocTest, MoveConstructorDoesNotLeak) {
@@ -175,86 +165,4 @@ TEST_F(MyAllocTest, ExceptionalBlocksAreNotReusedForLargerAllocations) {
   EXPECT_NE(ptr, ptr2);
 }
 
-TEST_F(MyAllocTest, RawInterface) {
-  MEM_ROOT alloc(PSI_NOT_INSTRUMENTED, 512);
-
-  // Nothing allocated yet.
-  std::pair<char *, char *> block = alloc.Peek();
-  EXPECT_EQ(0, block.second - block.first);
-
-  // Create a block.
-  alloc.ForceNewBlock(16);
-  block = alloc.Peek();
-  EXPECT_EQ(512, block.second - block.first);
-
-  // Write and commit some memory.
-  char *store_ptr = reinterpret_cast<char *>(block.first);
-  strcpy(store_ptr, "12345");
-  alloc.RawCommit(6);
-  block = alloc.Peek();
-  EXPECT_EQ(506, block.second - block.first);
-
-  // Get a new block.
-  alloc.ForceNewBlock(512);
-  block = alloc.Peek();
-#if defined(HAVE_VALGRIND) || defined(HAVE_ASAN)
-  EXPECT_EQ(512, block.second - block.first);
-#else
-  EXPECT_EQ(768, block.second - block.first);
-#endif
-
-  // The value should still be there.
-  EXPECT_STREQ("12345", store_ptr);
-
-  // Get a new block to satisfy more than the current block size (512 * 1.5^2).
-  EXPECT_FALSE(alloc.ForceNewBlock(2048));
-  block = alloc.Peek();
-  EXPECT_EQ(2048, block.second - block.first);
-}
-
-TEST_F(MyAllocTest, ArrayAllocInitialization) {
-  MEM_ROOT alloc(PSI_NOT_INSTRUMENTED, 512);
-
-  // No default value means each element is value-initialized. For int, it means
-  // they are set to 0.
-  int *int_array1 = alloc.ArrayAlloc<int>(100);
-  ASSERT_NE(nullptr, int_array1);
-  for (int i = 0; i < 100; ++i) {
-    EXPECT_EQ(0, int_array1[i]);
-  }
-
-  // Initialize to explicit value.
-  int *int_array2 = alloc.ArrayAlloc<int>(100, 123);
-  ASSERT_NE(nullptr, int_array2);
-  for (int i = 0; i < 100; ++i) {
-    EXPECT_EQ(123, int_array2[i]);
-  }
-
-  // Initialize from rvalue. (Verifies that a bug, which made it only initialize
-  // the first element correctly, is fixed.)
-  std::string *string_array1 = alloc.ArrayAlloc<std::string>(
-      10, std::string("abcdefghijklmnopqrstuvwxyz"));
-  ASSERT_NE(nullptr, string_array1);
-  for (int i = 0; i < 10; ++i) {
-    EXPECT_EQ("abcdefghijklmnopqrstuvwxyz", string_array1[i]);
-  }
-  std::destroy_n(string_array1, 10);
-
-  // Should be allowed to create an array of a class which is not
-  // copy-constructible.
-  auto uptr_array1 = alloc.ArrayAlloc<std::unique_ptr<int>>(10);
-  ASSERT_NE(nullptr, uptr_array1);
-  auto uptr_array2 = alloc.ArrayAlloc<std::unique_ptr<int>>(10, nullptr);
-  ASSERT_NE(nullptr, uptr_array2);
-  for (int i = 0; i < 10; ++i) {
-    EXPECT_EQ(nullptr, uptr_array1[i]);
-    EXPECT_EQ(nullptr, uptr_array2[i]);
-  }
-
-  // This should give a compiler error, because it attempts to copy a value of a
-  // non-copyable type:
-  //
-  // alloc.ArrayAlloc<std::unique_ptr<int>>(3, std::make_unique<int>(123));
-}
-
 }  // namespace my_alloc_unittest
