@@ -1,4 +1,4 @@
-/* Copyright (c) 2018, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -48,7 +48,7 @@ using ::testing::SetErrnoAndReturn;
 
 class MockRead {
  public:
-  virtual ~MockRead() = default;
+  virtual ~MockRead() {}
   MOCK_METHOD3(mockread, ssize_t(int, void *, size_t));
 };
 
@@ -59,11 +59,11 @@ ssize_t mockfs_read(int fd, void *buf, size_t count) {
 }
 
 class MysysMyReadTest : public ::testing::Test {
-  void SetUp() override {
+  virtual void SetUp() {
     mock_read = mockfs_read;
     mockfs = new MockRead;
   }
-  void TearDown() override {
+  virtual void TearDown() {
     mock_read = nullptr;
     delete mockfs;
     mockfs = nullptr;
