@@ -1,4 +1,4 @@
-/* Copyright (c) 2011, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2011, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -20,128 +20,59 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
-#include <gmock/gmock.h>
 #include <gtest/gtest.h>
 #include <limits.h>
-#include <thread>
-
-#include "my_config.h"
 
 #include "m_string.h"
 #include "my_inttypes.h"
 #include "my_stacktrace.h"
-#include "mysql/strings/int2str.h"
-#include "template_utils.h"
 #include "unittest/gunit/test_utils.h"
-#include "unittest/gunit/thread_utils.h"
 
 namespace segfault_unittest {
 
 using my_testing::Mock_error_handler;
 using my_testing::Server_initializer;
 
-size_t get_number_of_occurrences(std::string hay, std::string needle) {
-  size_t pos = hay.find(needle);
-  size_t count = 0;
-  if (pos != std::string::npos) {
-    for (;;) {
-      count++;
-      const auto new_pos = hay.substr(pos + 1).find(needle);
-      if (new_pos == std::string::npos) {
-        break;
-      }
-      pos += 1 + new_pos;
-    }
-  }
-  return count;
-}
-
-MATCHER_P3(ContainsRangeOfOccurrences, n, m, str, "") {
-  const auto count = get_number_of_occurrences(arg, str);
-  *result_listener << "where the actual count found is " << count;
-  return n <= static_cast<int>(count) && static_cast<int>(count) <= m;
-}
-static bool contains_cached_result;
-MATCHER_P3(ContainsRangeOfOccurrencesCached, n, m, str, "") {
-  const auto count = get_number_of_occurrences(arg, str);
-  std::cout << "ContainsRangeOfOccurrencesCached(" << n << ", " << m
-            << ") seen " << count << " occurrences and ";
-  if (n <= static_cast<int>(count) && static_cast<int>(count) <= m) {
-    contains_cached_result = true;
-    std::cout << "matched." << std::endl;
-  } else {
-    std::cout << "did not match." << std::endl;
-  }
-  return true;
-}
-
 class FatalSignalDeathTest : public ::testing::Test {
  protected:
-  void SetUp() override {
+  virtual void SetUp() {
     ::testing::FLAGS_gtest_death_test_style = "threadsafe";
     initializer.SetUp();
   }
-  void TearDown() override { initializer.TearDown(); }
+  virtual void TearDown() { initializer.TearDown(); }
 
   Server_initializer initializer;
-
-  const std::string expected_backtrace_string =
-      "Attempting backtrace. You can use the following "
-      "information to find out";
 };
 
 TEST_F(FatalSignalDeathTest, Abort) {
 #if defined(_WIN32)
-  EXPECT_DEATH_IF_SUPPORTED(
-      abort(), ContainsRangeOfOccurrences(1, 1, " UTC - mysqld got exception"));
+  EXPECT_DEATH_IF_SUPPORTED(abort(), ".* UTC - mysqld got exception.*");
 #else
-  EXPECT_DEATH_IF_SUPPORTED(
-      abort(), ContainsRangeOfOccurrences(1, 1, " UTC - mysqld got signal 6"));
+  EXPECT_DEATH_IF_SUPPORTED(abort(), ".* UTC - mysqld got signal 6.*");
 #endif
 }
 
-TEST_F(FatalSignalDeathTest, CrashOnMyAbort) {
-  EXPECT_DEATH_IF_SUPPORTED(
-      my_abort(), ContainsRangeOfOccurrences(1, 1, expected_backtrace_string));
-}
-TEST_F(FatalSignalDeathTest, CrashOnTerminate) {
-  EXPECT_DEATH_IF_SUPPORTED(
-      std::terminate(),
-      ContainsRangeOfOccurrences(1, 1, expected_backtrace_string));
-}
-
-static void test_parallel_crash() {
-  thread::Notification go;
-  thread::Notification ready[10];
-  auto test = [&ready, &go](int i) {
-    my_thread_init();
-    ready[i].notify();
-    go.wait_for_notification();
-    my_abort();
-    my_thread_end();
-  };
-  std::thread *t[10];
-  for (int i = 0; i < 10; ++i) {
-    t[i] = new std::thread{test, i};
-  }
-  for (int i = 0; i < 10; ++i) {
-    ready[i].wait_for_notification();
-  }
-  go.notify();
-  for (int i = 0; i < 10; ++i) {
-    t[i]->join();
-    delete t[i];
-  }
-}
-
-TEST_F(FatalSignalDeathTest, CrashOnParallelAbort) {
-  contains_cached_result = false;
-  for (size_t count = 0; count < 1000 && !contains_cached_result; ++count) {
-    EXPECT_DEATH_IF_SUPPORTED(
-        test_parallel_crash(),
-        ContainsRangeOfOccurrencesCached(2, 10, expected_backtrace_string));
-  }
-  EXPECT_TRUE(contains_cached_result);
+TEST_F(FatalSignalDeathTest, Segfault) {
+#if defined(_WIN32)
+  int *pint = NULL;
+  /*
+   After upgrading from gtest 1.5 to 1.6 this segfault is no longer
+   caught by handle_fatal_signal(). We get an empty error message from the
+   gtest library instead.
+  */
+  EXPECT_DEATH_IF_SUPPORTED(*pint = 42, "");
+#elif defined(__SANITIZE_ADDRESS__)
+/* gcc 4.8.1 with '-fsanitize=address -O1' */
+/* Newer versions of ASAN give other error message, disable it */
+// EXPECT_DEATH_IF_SUPPORTED(*pint= 42, ".*ASAN:SIGSEGV.*");
+#else
+  int *pint = nullptr;
+  /*
+   On most platforms we get SIGSEGV == 11, but SIGBUS == 10 is also possible.
+   And on Mac OsX we can get SIGILL == 4 (but only in optmized mode).
+  */
+  EXPECT_DEATH_IF_SUPPORTED(*pint = 42, ".* UTC - mysqld got signal .*");
+#endif
 }
 
 // Verifies that my_safe_utoa behaves like sprintf(_, "%llu", _)
