@@ -1,16 +1,15 @@
-/* Copyright (c) 2009, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2009, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -31,16 +30,13 @@
 #include <gtest/gtest.h>
 #include <stddef.h>
 
-#include <memory>
-
 #include "my_inttypes.h"
 #include "my_thread.h"
 #include "sql/sql_error.h"
 #include "sql/sql_list.h"
 #include "sql/thr_malloc.h"
 #include "sql_string.h"
-#include "template_utils.h"
-#include "unittest/gunit/gunit_test_main.h"
+#include "unittest/gunit/test_utils.h"
 
 namespace sql_list_unittest {
 
@@ -64,21 +60,26 @@ class SqlListTest : public ::testing::Test {
   SqlListTest()
       : m_mem_root_p(&m_mem_root), m_int_list(), m_int_list_iter(m_int_list) {}
 
-  void SetUp() override { THR_MALLOC = &m_mem_root_p; }
+  virtual void SetUp() {
+    init_sql_alloc(PSI_NOT_INSTRUMENTED, &m_mem_root, 1024, 0);
+    THR_MALLOC = &m_mem_root_p;
+  }
+
+  virtual void TearDown() { free_root(&m_mem_root, MYF(0)); }
 
   static void SetUpTestCase() {
     current_thd = nullptr;
     THR_MALLOC = nullptr;
   }
 
-  MEM_ROOT m_mem_root{PSI_NOT_INSTRUMENTED, 1024};
+  MEM_ROOT m_mem_root;
   MEM_ROOT *m_mem_root_p;
   List<int> m_int_list;
   List_iterator<int> m_int_list_iter;
 
  private:
-  SqlListTest(SqlListTest const &) = delete;
-  SqlListTest &operator=(SqlListTest const &) = delete;
+  // Declares (but does not define) copy constructor and assignment operator.
+  GTEST_DISALLOW_COPY_AND_ASSIGN_(SqlListTest);
 };
 
 // Tests that we can construct and destruct lists.
@@ -86,7 +87,7 @@ TEST_F(SqlListTest, ConstructAndDestruct) {
   EXPECT_TRUE(m_int_list.is_empty());
   List<int> *p_int_list = new (*THR_MALLOC) List<int>;
   EXPECT_TRUE(p_int_list->is_empty());
-  ::destroy_at(p_int_list);
+  destroy(p_int_list);
 }
 
 // Tests basic operations push and pop.
@@ -109,14 +110,15 @@ TEST_F(SqlListTest, BasicOperations) {
 TEST_F(SqlListTest, DeepCopy) {
   int values[] = {11, 22, 33, 42, 5};
   insert_values(values, &m_int_list);
-  MEM_ROOT mem_root(PSI_NOT_INSTRUMENTED, 4096);
+  MEM_ROOT mem_root;
+  init_alloc_root(PSI_NOT_INSTRUMENTED, &mem_root, 4096, 4096);
   List<int> list_copy(m_int_list, &mem_root);
   EXPECT_EQ(list_copy.elements, m_int_list.elements);
   while (!list_copy.is_empty()) {
     EXPECT_EQ(*m_int_list.pop(), *list_copy.pop());
   }
   EXPECT_TRUE(m_int_list.is_empty());
-  mem_root.Clear();
+  free_root(&mem_root, MYF(0));
 }
 
 // Tests that we can iterate over values.
