@@ -1,16 +1,15 @@
-/* Copyright (c) 2012, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2012, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -58,7 +57,7 @@ class TableCacheBasicTest : public ::testing::Test {
  protected:
   static const uint MAX_THREADS = 3;
 
-  void SetUp() override {
+  virtual void SetUp() {
     Global_THD_manager *thd_manager = Global_THD_manager::get_instance();
     thd_manager->set_unit_test();
     // Reset thread ID counter for each test.
@@ -69,7 +68,7 @@ class TableCacheBasicTest : public ::testing::Test {
 
     ::testing::FLAGS_gtest_death_test_style = "threadsafe";
   }
-  void TearDown() override {
+  virtual void TearDown() {
     for (uint i = 0; i < MAX_THREADS; ++i) initializer[i].TearDown();
   }
 
@@ -86,7 +85,7 @@ class TableCacheBasicTest : public ::testing::Test {
 class TableCacheSingleCacheTest : public TableCacheBasicTest {
  protected:
   virtual uint CachesNumber() { return 1; }
-  void SetUp() override {
+  virtual void SetUp() {
     TableCacheBasicTest::SetUp();
 
     /*
@@ -99,7 +98,7 @@ class TableCacheSingleCacheTest : public TableCacheBasicTest {
     table_cache_size_per_instance = 100;
     ASSERT_FALSE(table_def_init());
   }
-  void TearDown() override {
+  virtual void TearDown() {
     table_def_free();
     TableCacheBasicTest::TearDown();
   }
@@ -112,7 +111,7 @@ class TableCacheSingleCacheTest : public TableCacheBasicTest {
 
 class TableCacheDoubleCacheTest : public TableCacheSingleCacheTest {
  protected:
-  uint CachesNumber() override { return 2; }
+  virtual uint CachesNumber() { return 2; }
 };
 
 /**
@@ -128,9 +127,7 @@ class Mock_share : public TABLE_SHARE {
   Mock_share(const char *key)
       :  // Assertion in some of Table_cache methods check that the
          // version of the share is up-to-date, so make sure it's set.
-        TABLE_SHARE(refresh_version, false),
-        // MEM_ROOT is used for constructing ha_example() instances.
-        m_mem_root(PSI_NOT_INSTRUMENTED, 1024) {
+        TABLE_SHARE(refresh_version, false) {
     /*
       Both table_cache_key and cache_element array are used by
       Table_cache code.
@@ -139,11 +136,13 @@ class Mock_share : public TABLE_SHARE {
     table_cache_key.length = strlen(key);
     memset(cache_element_arr, 0, sizeof(cache_element_arr));
     cache_element = cache_element_arr;
+    // MEM_ROOT is used for constructing ha_example() instances.
+    init_alloc_root(PSI_NOT_INSTRUMENTED, &m_mem_root, 1024, 0);
     // Ensure that share is never destroyed.
     increment_ref_count();
   }
 
-  ~Mock_share() { m_mem_root.Clear(); }
+  ~Mock_share() { free_root(&m_mem_root, MYF(0)); }
 
   TABLE *create_table(THD *thd) {
     TABLE *result =
@@ -750,7 +749,7 @@ TEST_F(TableCacheDoubleCacheDeathTest, ManagerFreeTable) {
   // There should be assert failure since we are trying
   // to free all tables for share_1, while some tables
   // are in use.
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   EXPECT_DEATH_IF_SUPPORTED(
       table_cache_manager.free_table(thd_1, TDC_RT_REMOVE_ALL, &share_1),
       ".*Assertion.*is_empty.*");
@@ -788,7 +787,7 @@ TEST_F(TableCacheDoubleCacheDeathTest, ManagerFreeTable) {
   // There should be assert failure since we are trying
   // to free all not own TABLEs for share_1, while thd_2
   // has a TABLE object for it in used
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   EXPECT_DEATH_IF_SUPPORTED(
       table_cache_manager.free_table(thd_1, TDC_RT_REMOVE_NOT_OWN, &share_1),
       ".*Assertion.*0.*");
