@@ -1,16 +1,15 @@
-/* Copyright (c) 2017, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2.0,
  as published by the Free Software Foundation.
 
- This program is designed to work with certain software (including
+ This program is also distributed with certain software (including
  but not limited to OpenSSL) that is licensed under separate terms,
  as designated in a particular file or component or in included license
  documentation.  The authors of MySQL hereby grant you an additional
  permission to link the program and your derivative works with the
- separately licensed software that they have either included with
- the program or referenced in the documentation.
+ separately licensed software that they have included with MySQL.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -32,7 +31,7 @@ namespace test {
 
 class Auth_chaining_test_suite_base : public Xcl_session_impl_tests {
  public:
-  void SetUp() override {
+  void SetUp() {
     m_sut = prepare_session();
     EXPECT_CALL(m_mock_connection_state, is_connected())
         .WillRepeatedly(Return(false));
@@ -98,12 +97,7 @@ class Auth_chaining_test_suite_base : public Xcl_session_impl_tests {
 
 class Auth_chaining_test_suite
     : public Auth_chaining_test_suite_base,
-      public ::testing::WithParamInterface<std::vector<std::string>> {
-  void SetUp() override {
-    Auth_chaining_test_suite_base::SetUp();
-    EXPECT_CALL(*m_mock_protocol, reset_buffering());
-  }
-};
+      public ::testing::WithParamInterface<std::vector<std::string>> {};
 
 TEST_F(Auth_chaining_test_suite, cap_auth_method_server_supports_nothing) {
   set_ssl_state(true);
@@ -656,7 +650,7 @@ TEST_P(Auth_chaining_test_suite, custom_error_handling) {
             m_sut->connect("host", 1290, "user", "pass", "schema").error());
 }
 
-INSTANTIATE_TEST_SUITE_P(
+INSTANTIATE_TEST_CASE_P(
     Instantiation_custom_error_handling, Auth_chaining_test_suite,
     ::testing::Values(std::vector<std::string>{"PLAIN", "MYSQL41"},
                       std::vector<std::string>{"MYSQL41", "PLAIN"},
@@ -683,7 +677,7 @@ TEST_P(Auth_chaining_test_suite_sha256_fail_no_ssl, sha256_fail_with_no_ssl) {
             m_sut->connect("host", 1290, "user", "pass", "schema").error());
 }
 
-INSTANTIATE_TEST_SUITE_P(
+INSTANTIATE_TEST_CASE_P(
     Instantiation_sha256_fail, Auth_chaining_test_suite_sha256_fail_no_ssl,
     ::testing::Values(std::vector<std::string>{"SHA256_MEMORY"},
                       std::vector<std::string>{"MYSQL41", "SHA256_MEMORY"},
@@ -707,18 +701,17 @@ TEST_P(Auth_chaining_fatal_errors, break_chain_on_fatal_errors) {
   EXPECT_CALL(m_mock_connection_state, get_connection_type())
       .WillOnce(Return(Connection_type::Tcp));
   EXPECT_CALL(*m_mock_protocol, use_compression(Compression_algorithm::k_none));
-  EXPECT_CALL(*m_mock_protocol, reset_buffering());
 
   ASSERT_EQ(GetParam().error(),
             m_sut->connect("host", 1290, "user", "pass", "schema").error());
 }
 
-INSTANTIATE_TEST_SUITE_P(Instantiation_auth_chaining_fatal_errors,
-                         Auth_chaining_fatal_errors,
-                         ::testing::Values(xcl::XError{CR_X_READ_TIMEOUT, ""},
-                                           xcl::XError{CR_X_WRITE_TIMEOUT, ""},
-                                           xcl::XError{CR_SERVER_GONE_ERROR,
-                                                       ""}));
+INSTANTIATE_TEST_CASE_P(Instantiation_auth_chaining_fatal_errors,
+                        Auth_chaining_fatal_errors,
+                        ::testing::Values(xcl::XError{CR_X_READ_TIMEOUT, ""},
+                                          xcl::XError{CR_X_WRITE_TIMEOUT, ""},
+                                          xcl::XError{CR_SERVER_GONE_ERROR,
+                                                      ""}));
 
 }  // namespace test
 }  // namespace xcl
