@@ -1,17 +1,16 @@
 /*
- * Copyright (c) 2015, 2024, Oracle and/or its affiliates.
+ * Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is designed to work with certain software (including
+ * This program is also distributed with certain software (including
  * but not limited to OpenSSL) that is licensed under separate terms,
  * as designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have either included with
- * the program or referenced in the documentation.
+ * separately licensed software that they have included with MySQL.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -622,8 +621,8 @@ Param_docpath_arg docpath_arg_param[] = {
     {ER_X_SUCCESS, "$.`a\\`"},
 };
 
-INSTANTIATE_TEST_SUITE_P(docpath_arg, Admin_command_arguments_docpath_test,
-                         testing::ValuesIn(docpath_arg_param));
+INSTANTIATE_TEST_CASE_P(docpath_arg, Admin_command_arguments_docpath_test,
+                        testing::ValuesIn(docpath_arg_param));
 
 #define INVALID_VALUE_ERROR(arg_name)      \
   ngs::Error_code(ER_X_CMD_ARGUMENT_VALUE, \
