@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2000, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2000, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    Without limiting anything contained in the foregoing, this file,
    which is part of C Driver for MySQL (Connector/C), is also subject to the
@@ -46,15 +45,14 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/types.h>
-#include <fstream>
 #include <new>
 #include <set>
 #include <utility>
 #include <vector>
 
+#include "m_ctype.h"
 #include "m_string.h"
 #include "my_byteorder.h"
-#include "my_checksum.h"
 #include "my_compiler.h"
 #include "my_dbug.h"
 #include "my_dir.h"
@@ -63,12 +61,8 @@
 #include "my_sys.h"
 #include "mysql/components/services/log_shared.h"
 #include "mysql/service_mysql_alloc.h"
-#include "mysql/strings/m_ctype.h"
-#include "mysql/strings/my_strtoll10.h"
-#include "nulls.h"
 #include "prealloced_array.h"
 #include "print_version.h"
-#include "strxmov.h"
 #include "welcome_copyright_notice.h"
 
 #define MAX_ERROR_NAME_LENGTH 64
@@ -88,7 +82,7 @@ static const char *MSGS_TO_CLIENT = "../share/messages_to_clients.txt";
 static const char *DATADIRECTORY = "../share/";
 static const char *selftest_dir = nullptr;
 bool selftest_flag = false;
-#ifndef NDEBUG
+#ifndef DBUG_OFF
 static const char *default_dbug_option = "d:t:O,/tmp/comp_err.trace";
 #endif
 
@@ -100,6 +94,8 @@ static const char *default_dbug_option = "d:t:O,/tmp/comp_err.trace";
     from 2 bytes to 4 bytes.
 */
 uchar file_head[] = {254, 254, ERRMSG_VERSION, 1};
+/* Store positions to each error message row to store in errmsg.sys header */
+std::vector<uint> file_pos;
 
 const char *empty_string = ""; /* For empty states */
 /*
@@ -184,9 +180,9 @@ using err_range = std::pair<uint, uint>;
 std::set<err_range> reserved_sections;
 
 static struct my_option my_long_options[] = {
-#ifdef NDEBUG
-    {"debug", '#', "This is a non-debug version. Catch this and exit", nullptr,
-     nullptr, nullptr, GET_DISABLED, OPT_ARG, 0, 0, 0, nullptr, 0, nullptr},
+#ifdef DBUG_OFF
+    {"debug", '#', "This is a non-debug version. Catch this and exit", 0, 0, 0,
+     GET_DISABLED, OPT_ARG, 0, 0, 0, 0, 0, 0},
 #else
     {"debug", '#', "Output debug log", &default_dbug_option,
      &default_dbug_option, nullptr, GET_STR, OPT_ARG, 0, 0, 0, nullptr, 0,
@@ -221,10 +217,9 @@ static struct my_option my_long_options[] = {
      0, nullptr, 0, nullptr}};
 
 static struct languages *parse_charset_string(char *str);
-static struct errors *parse_error_string(char *str, int er_count,
+static struct errors *parse_error_string(char *ptr, int er_count,
                                          PFS_error_stat error_stat);
-static struct message *parse_message_string(const struct errors *current_error,
-                                            struct message *new_message,
+static struct message *parse_message_string(struct message *new_message,
                                             char *str);
 static struct languages *find_language(struct languages *languages,
                                        const char *short_name);
@@ -235,12 +230,11 @@ static int parse_input_file(const char *file_name, struct errors ***last_error,
                             struct languages **top_language,
                             int base_error_code, PFS_error_stat error_stat);
 static int get_options(int *argc, char ***argv);
-static void usage();
+static void usage(void);
 static bool get_one_option(int optid, const struct my_option *opt,
                            char *argument);
-static char *parse_text_line(char *pos, bool allow_newline_escape);
-static int copy_rows(FILE *to, char *row, long start_pos,
-                     std::vector<uint> *file_pos);
+static char *parse_text_line(char *pos);
+static int copy_rows(FILE *to, char *row, int row_nr, long start_pos);
 static char *parse_default_language(char *str);
 static uint parse_error_offset(char *str);
 static bool parse_reserved_error_section(char *str);
@@ -380,27 +374,6 @@ static void print_escaped_string(FILE *f, const char *str) {
   }
 }
 
-std::string get_file_contents(std::string filename) {
-  std::ifstream ifs(filename);
-  if (!ifs.good()) {
-    return "";
-  }
-  return std::string(std::istreambuf_iterator<char>(ifs),
-                     std::istreambuf_iterator<char>());
-}
-
-static void compare_and_move_file(std::string source_filename,
-                                  std::string destination_filename) {
-  const auto source_contents = get_file_contents(source_filename);
-  const auto destination_contents = get_file_contents(destination_filename);
-  if (source_contents == destination_contents) {
-    unlink(source_filename.c_str());
-  } else {
-    unlink(destination_filename.c_str());
-    rename(source_filename.c_str(), destination_filename.c_str());
-  }
-}
-
 static int create_header_files(struct errors *error_head) {
   FILE *er_definef, *er_namef;
   FILE *er_errmsg;
@@ -410,18 +383,14 @@ static int create_header_files(struct errors *error_head) {
 
   DBUG_TRACE;
 
-  std::string headerfile_tmp = std::string(HEADERFILE) + "_tmp";
-  std::string namefile_tmp = std::string(NAMEFILE) + "_tmp";
-  std::string msgfile_tmp = std::string(MSGFILE) + "_tmp";
-
-  if (!(er_definef = my_fopen(headerfile_tmp.c_str(), O_WRONLY, MYF(MY_WME)))) {
+  if (!(er_definef = my_fopen(HEADERFILE, O_WRONLY, MYF(MY_WME)))) {
     return 1;
   }
-  if (!(er_namef = my_fopen(namefile_tmp.c_str(), O_WRONLY, MYF(MY_WME)))) {
+  if (!(er_namef = my_fopen(NAMEFILE, O_WRONLY, MYF(MY_WME)))) {
     my_fclose(er_definef, MYF(0));
     return 1;
   }
-  if (!(er_errmsg = my_fopen(msgfile_tmp.c_str(), O_WRONLY, MYF(MY_WME)))) {
+  if (!(er_errmsg = my_fopen(MSGFILE, O_WRONLY, MYF(MY_WME)))) {
     my_fclose(er_definef, MYF(0));
     my_fclose(er_namef, MYF(0));
     return 1;
@@ -583,17 +552,13 @@ static int create_header_files(struct errors *error_head) {
   my_fclose(er_definef, MYF(0));
   my_fclose(er_namef, MYF(0));
   my_fclose(er_errmsg, MYF(0));
-
-  compare_and_move_file(headerfile_tmp, HEADERFILE);
-  compare_and_move_file(namefile_tmp, NAMEFILE);
-  compare_and_move_file(msgfile_tmp, MSGFILE);
   return 0;
 }
 
 static int create_sys_files(struct languages *lang_head,
                             struct errors *error_head, uint row_count) {
   FILE *to;
-  uint csnum = 0, length;
+  uint csnum = 0, length, i, row_nr;
   uchar head[32];
   char outfile[FN_REFLEN], *outfile_end;
   long start_pos;
@@ -632,8 +597,7 @@ static int create_sys_files(struct languages *lang_head,
     /* 4 is for 4 bytes to store row position / error message */
     start_pos = (long)(HEADER_LENGTH + row_count * 4);
     fseek(to, start_pos, 0);
-    // Store positions to each error message row to store in errmsg.sys header
-    std::vector<uint> file_pos;
+    row_nr = 0;
     for (tmp_error = error_head; tmp_error; tmp_error = tmp_error->next_error) {
       /* dealing with messages */
       tmp = find_message(tmp_error, tmp_lang->lang_short_name, false);
@@ -645,10 +609,11 @@ static int create_sys_files(struct languages *lang_head,
                 tmp_error->er_name, tmp_lang->lang_short_name);
         goto err;
       }
-      if (copy_rows(to, tmp->text, start_pos, &file_pos)) {
+      if (copy_rows(to, tmp->text, row_nr, start_pos)) {
         fprintf(stderr, "Failed to copy rows to %s\n", outfile);
         goto err;
       }
+      row_nr++;
     }
 
     /* continue with header of the errmsg.sys file */
@@ -664,8 +629,8 @@ static int create_sys_files(struct languages *lang_head,
     if (my_fwrite(to, (uchar *)head, HEADER_LENGTH, MYF(MY_WME | MY_FNABP)))
       goto err;
 
-    for (uint pos : file_pos) {
-      int4store(head, pos);
+    for (i = 0; i < row_count; i++) {
+      int4store(head, file_pos.at(i));
       if (my_fwrite(to, (uchar *)head, 4, MYF(MY_WME | MY_FNABP))) goto err;
     }
     my_fclose(to, MYF(0));
@@ -716,11 +681,10 @@ static int parse_input_file(const char *file_name, struct errors ***last_error,
                             struct languages **top_lang, int base_error_code,
                             PFS_error_stat error_stat) {
   FILE *file;
-  char *str, buff[2000];  // temporary fix for compilation failure of Wl14191 at
-                          // few platforms
+  char *str, buff[1000];
   const char *fail = nullptr;
   struct errors *current_error = nullptr, **tail_error = *last_error;
-  struct message current_message {};
+  struct message current_message;
   int rcount = 0; /* Number of error codes in current section. */
   int ecount = 0; /* Number of error codes in total. */
   DBUG_TRACE;
@@ -730,14 +694,6 @@ static int parse_input_file(const char *file_name, struct errors ***last_error,
   if (!(file = my_fopen(file_name, O_RDONLY, MYF(MY_WME)))) return 0;
 
   while ((str = fgets(buff, sizeof(buff), file))) {
-    if (strlen(buff) >= sizeof(buff) - 1) {
-      fprintf(stderr,
-              "A line longer than %zu characters is not supported. Increase "
-              "`buff` size in `parse_input_file()` in `comp_err.cc`.",
-              sizeof(buff));
-      fail = "\n";
-      goto done;
-    }
     if (is_prefix(str, "language")) {
       if (*top_lang != nullptr) {
         fail = "More than one languages directive found.\n";
@@ -803,7 +759,7 @@ static int parse_input_file(const char *file_name, struct errors ***last_error,
         fail = "Error in the input file format\n";
         goto done;
       }
-      if (!parse_message_string(current_error, &current_message, str)) {
+      if (!parse_message_string(&current_message, str)) {
         fprintf(stderr, "Failed to parse message string for error '%s'",
                 current_error->er_name);
         fail = "\n";
@@ -983,7 +939,9 @@ static bool parse_reserved_error_section(char *str) {
   }
 
   auto ret = reserved_sections.insert(err_range(sec_start, sec_end));
-  return (!ret.second);
+  if (!ret.second) return true;
+
+  return false;
 }
 
 /* Parsing of the default language line. e.g. "default-language eng" */
@@ -1063,6 +1021,7 @@ static struct message *find_message(struct errors *err, const char *lang,
 
     if (!strcmp(tmp->lang_short_name, lang)) return tmp;
     if (!strcmp(tmp->lang_short_name, default_language)) {
+      DBUG_ASSERT(tmp->text[0] != 0);
       return_val = tmp;
     }
   }
@@ -1127,7 +1086,7 @@ static ha_checksum checksum_format_specifier(const char *msg) {
             "Still inside formatspecifier after end of string"
             " in'%s'\n",
             msg);
-    assert(start == nullptr);
+    DBUG_ASSERT(start == nullptr);
   }
 
   /* Add number of format specifiers to checksum as extra safeguard */
@@ -1156,7 +1115,7 @@ static int check_message_format(struct errors *err, const char *mess) {
   if ((err->msg).empty()) return 0; /* No previous message to compare against */
 
   first = err->msg.begin();
-  assert(first != nullptr);
+  DBUG_ASSERT(first != nullptr);
 
   if (checksum_format_specifier(first->text) !=
       checksum_format_specifier(mess)) {
@@ -1209,8 +1168,7 @@ static char *get_word(char **str) {
   remember to which error does the text belong
 */
 
-static struct message *parse_message_string(const struct errors *current_error,
-                                            struct message *new_message,
+static struct message *parse_message_string(struct message *new_message,
                                             char *str) {
   char *start;
 
@@ -1240,21 +1198,18 @@ static struct message *parse_message_string(const struct errors *current_error,
   while (*str == ' ' || *str == '\t' || *str == '\n') str++;
 
   if (*str != '"') {
-    fprintf(stderr,
-            "Unexpected character at start of the message, expected '\"', "
-            "found '%c'\n",
-            *str);
+    fprintf(stderr, "Unexpected EOL.\n");
     DBUG_PRINT("info", ("str: %s", str));
     return nullptr;
   }
 
   /* reading the text */
   start = str + 1;
-  str = parse_text_line(start, current_error->d_code < BASE_ERROR_LOG);
+  str = parse_text_line(start);
 
-  if (!str || !(new_message->text =
-                    my_strndup(PSI_NOT_INSTRUMENTED, start, (uint)(str - start),
-                               MYF(MY_WME | MY_FAE))))
+  if (!(new_message->text =
+            my_strndup(PSI_NOT_INSTRUMENTED, start, (uint)(str - start),
+                       MYF(MY_WME | MY_FAE))))
     return nullptr; /* Fatal error */
   DBUG_PRINT("info", ("msg_text: %s", new_message->text));
 
@@ -1420,8 +1375,8 @@ static struct languages *parse_charset_string(char *str) {
 /* Read options */
 
 static bool get_one_option(int optid,
-                           const struct my_option *opt [[maybe_unused]],
-                           char *argument [[maybe_unused]]) {
+                           const struct my_option *opt MY_ATTRIBUTE((unused)),
+                           char *argument MY_ATTRIBUTE((unused))) {
   DBUG_TRACE;
   switch (optid) {
     case 'V':
@@ -1437,7 +1392,7 @@ static bool get_one_option(int optid,
   return false;
 }
 
-static void usage() {
+static void usage(void) {
   DBUG_TRACE;
   print_version();
   printf(
@@ -1464,32 +1419,23 @@ static int get_options(int *argc, char ***argv) {
   row as a C-compiler would convert a textstring
 */
 
-static char *parse_text_line(char *pos, bool allow_newline_escape) {
+static char *parse_text_line(char *pos) {
   int i, nr;
   char *row = pos;
   size_t len;
   DBUG_TRACE;
 
   len = strlen(pos);
-  while (*pos && *pos != '"') {
+  while (*pos) {
     if (*pos == '\\') {
       switch (*++pos) {
         case '\\':
         case '"':
-        case '\'':
           (void)memmove(pos - 1, pos, len - (row - pos));
           break;
         case 'n':
-          if (!allow_newline_escape) {
-            fprintf(stderr,
-                    "Found newline escaped character in a message to the Error "
-                    "Log.\nThis is highly discouraged, the newline characters "
-                    "in messages to Error Log are replaced with a space in "
-                    "`log_sink_trad.cc`.\n");
-            return nullptr;
-          }
           pos[-1] = '\n';
-          (void)memmove(pos, pos + 1, len - (row - (pos + 1)));
+          (void)memmove(pos, pos + 1, len - (row - pos));
           break;
         default:
           if (*pos >= '0' && *pos < '8') {
@@ -1499,39 +1445,24 @@ static char *parse_text_line(char *pos, bool allow_newline_escape) {
             pos -= i;
             pos[-1] = nr;
             (void)memmove(pos, pos + i, len - (row - pos));
-          } else if (*pos) {
-            fprintf(stderr, "Found incorrect escape character: %c\n", *pos);
-            return nullptr;
-          }
+          } else if (*pos)
+            (void)memmove(pos - 1, pos, len - (row - pos)); /* Remove '\' */
       }
     } else
       pos++;
   }
-  /* Remove endline and expect a closing '"' character. */
-  if (*pos != '"') {
-    fprintf(stderr, "Could not find a '\"' on the end of the message\n");
-    return nullptr;
-  }
-  for (auto end_of_line = pos + 1; *end_of_line; end_of_line++) {
-    if (*end_of_line != '\n' && *end_of_line != '\r') {
-      fprintf(stderr,
-              "Unexpected character at the end of the line, expected newline, "
-              "found  '%c' (0x%02x)\n",
-              *end_of_line < 32 ? '?' : *end_of_line, (int)*end_of_line);
-      return nullptr;
-    }
-  }
+  while (pos > row + 1 && *pos != '"') pos--;
   *pos = 0;
   return pos;
 }
 
 /* Copy rows from memory to file and remember position */
 
-static int copy_rows(FILE *to, char *row, long start_pos,
-                     std::vector<uint> *file_pos) {
+static int copy_rows(FILE *to, char *row, int row_nr, long start_pos) {
   DBUG_TRACE;
 
-  file_pos->push_back(static_cast<uint>(ftell(to) - start_pos));
+  file_pos.insert(file_pos.begin() + row_nr,
+                  static_cast<uint>(ftell(to) - start_pos));
   if (fputs(row, to) == EOF || fputc('\0', to) == EOF) {
     fprintf(stderr, "Can't write to outputfile\n");
     return 1;
