@@ -1,16 +1,15 @@
-/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    Without limiting anything contained in the foregoing, this file,
    which is part of C Driver for MySQL (Connector/C), is also subject to the
@@ -30,7 +29,7 @@
   Note that we can't have assertion on file descriptors;  The reason for
   this is that during mysql shutdown, another thread can close a file
   we are working on.  In this case we should just return read errors from
-  the file descriptor.
+  the file descriptior.
 */
 
 #include <sys/types.h>
@@ -45,7 +44,6 @@
 #include "mysql/psi/mysql_socket.h"
 #include "mysql/psi/psi_memory.h"  // IWYU pragma: keep
 #include "mysql/service_mysql_alloc.h"
-#include "string_with_len.h"
 #include "template_utils.h"
 #include "vio/vio_priv.h"
 
@@ -87,16 +85,16 @@ void internal_vio_delete(Vio *vio);
   @retval 1       The requested I/O event has occurred.
 */
 
-static int no_io_wait(Vio *vio [[maybe_unused]],
-                      enum enum_vio_io_event event [[maybe_unused]],
-                      int timeout [[maybe_unused]]) {
+static int no_io_wait(Vio *vio MY_ATTRIBUTE((unused)),
+                      enum enum_vio_io_event event MY_ATTRIBUTE((unused)),
+                      int timeout MY_ATTRIBUTE((unused))) {
   return 1;
 }
 
 #endif
 
 extern "C" {
-static bool has_no_data(Vio *vio [[maybe_unused]]) { return false; }
+static bool has_no_data(Vio *vio MY_ATTRIBUTE((unused))) { return false; }
 }  // extern "C"
 
 Vio::Vio(uint flags) {
@@ -211,11 +209,20 @@ Vio &Vio::operator=(Vio &&vio) {
 
 static bool vio_init(Vio *vio, enum enum_vio_type type, my_socket sd,
                      uint flags) {
-  DBUG_PRINT("enter vio_init",
-             ("type: %d sd: " MY_SOCKET_FMT " flags: %d", type, sd, flags));
+  DBUG_PRINT("enter vio_init", ("type: %d sd: %d  flags: %d", type, sd, flags));
 
   mysql_socket_setfd(&vio->mysql_socket, sd);
 
+#ifdef HAVE_KQUEUE
+  DBUG_ASSERT(type == VIO_TYPE_TCPIP || type == VIO_TYPE_SOCKET ||
+              type == VIO_TYPE_SSL);
+  vio->kq_fd = kqueue();
+  if (vio->kq_fd == -1) {
+    DBUG_PRINT("vio_init", ("kqueue failed with errno: %d", errno));
+    return true;
+  }
+#endif
+
   vio->localhost = flags & VIO_LOCALHOST;
   vio->type = type;
 
@@ -223,103 +230,85 @@ static bool vio_init(Vio *vio, enum enum_vio_type type, my_socket sd,
   vio->network_namespace[0] = '\0';
 #endif
 
-  switch (type) {
 #ifdef _WIN32
-    case VIO_TYPE_NAMEDPIPE:
-      vio->viodelete = vio_delete;
-      vio->vioerrno = vio_errno;
-      vio->read = vio_read_pipe;
-      vio->write = vio_write_pipe;
-      vio->fastsend = vio_fastsend;
-      vio->viokeepalive = vio_keepalive;
-      vio->should_retry = vio_should_retry;
-      vio->was_timeout = vio_was_timeout;
-      vio->vioshutdown = vio_shutdown_pipe;
-      vio->peer_addr = vio_peer_addr;
-      vio->io_wait = no_io_wait;
-      vio->is_connected = vio_is_connected_pipe;
-      vio->has_data = has_no_data;
-      vio->is_blocking = vio_is_blocking;
-      vio->set_blocking = vio_set_blocking;
-      vio->set_blocking_flag = vio_set_blocking_flag;
-      vio->is_blocking_flag = true;
-      break;
-
-    case VIO_TYPE_SHARED_MEMORY:
-      vio->viodelete = vio_delete_shared_memory;
-      vio->vioerrno = vio_errno;
-      vio->read = vio_read_shared_memory;
-      vio->write = vio_write_shared_memory;
-      vio->fastsend = vio_fastsend;
-      vio->viokeepalive = vio_keepalive;
-      vio->should_retry = vio_should_retry;
-      vio->was_timeout = vio_was_timeout;
-      vio->vioshutdown = vio_shutdown_shared_memory;
-      vio->peer_addr = vio_peer_addr;
-      vio->io_wait = no_io_wait;
-      vio->is_connected = vio_is_connected_shared_memory;
-      vio->has_data = has_no_data;
-      vio->is_blocking = vio_is_blocking;
-      vio->set_blocking = vio_set_blocking;
-      vio->set_blocking_flag = vio_set_blocking_flag;
-      vio->is_blocking_flag = true;
-      break;
+  if (type == VIO_TYPE_NAMEDPIPE) {
+    vio->viodelete = vio_delete;
+    vio->vioerrno = vio_errno;
+    vio->read = vio_read_pipe;
+    vio->write = vio_write_pipe;
+    vio->fastsend = vio_fastsend;
+    vio->viokeepalive = vio_keepalive;
+    vio->should_retry = vio_should_retry;
+    vio->was_timeout = vio_was_timeout;
+    vio->vioshutdown = vio_shutdown_pipe;
+    vio->peer_addr = vio_peer_addr;
+    vio->io_wait = no_io_wait;
+    vio->is_connected = vio_is_connected_pipe;
+    vio->has_data = has_no_data;
+    vio->is_blocking = vio_is_blocking;
+    vio->set_blocking = vio_set_blocking;
+    vio->is_blocking_flag = true;
+    return false;
+  }
+  if (type == VIO_TYPE_SHARED_MEMORY) {
+    vio->viodelete = vio_delete_shared_memory;
+    vio->vioerrno = vio_errno;
+    vio->read = vio_read_shared_memory;
+    vio->write = vio_write_shared_memory;
+    vio->fastsend = vio_fastsend;
+    vio->viokeepalive = vio_keepalive;
+    vio->should_retry = vio_should_retry;
+    vio->was_timeout = vio_was_timeout;
+    vio->vioshutdown = vio_shutdown_shared_memory;
+    vio->peer_addr = vio_peer_addr;
+    vio->io_wait = no_io_wait;
+    vio->is_connected = vio_is_connected_shared_memory;
+    vio->has_data = has_no_data;
+    vio->is_blocking = vio_is_blocking;
+    vio->set_blocking = vio_set_blocking;
+    vio->is_blocking_flag = true;
+    return false;
+  }
 #endif /* _WIN32 */
-
-    case VIO_TYPE_SSL:
-      vio->viodelete = vio_ssl_delete;
-      vio->vioerrno = vio_errno;
-      vio->read = vio_ssl_read;
-      vio->write = vio_ssl_write;
-      vio->fastsend = vio_fastsend;
-      vio->viokeepalive = vio_keepalive;
-      vio->should_retry = vio_should_retry;
-      vio->was_timeout = vio_was_timeout;
-      vio->vioshutdown = vio_ssl_shutdown;
-      vio->peer_addr = vio_peer_addr;
-      vio->io_wait = vio_io_wait;
-      vio->is_connected = vio_is_connected;
-      vio->has_data = vio_ssl_has_data;
-      vio->timeout = vio_socket_timeout;
-      vio->is_blocking = vio_is_blocking;
-      vio->set_blocking = vio_set_blocking;
-      vio->set_blocking_flag = vio_set_blocking_flag;
-      vio->is_blocking_flag = true;
-      break;
-
-    default:
-      vio->viodelete = vio_delete;
-      vio->vioerrno = vio_errno;
-      vio->read = vio->read_buffer ? vio_read_buff : vio_read;
-      vio->write = vio_write;
-      vio->fastsend = vio_fastsend;
-      vio->viokeepalive = vio_keepalive;
-      vio->should_retry = vio_should_retry;
-      vio->was_timeout = vio_was_timeout;
-      vio->vioshutdown = vio_shutdown;
-      vio->peer_addr = vio_peer_addr;
-      vio->io_wait = vio_io_wait;
-      vio->is_connected = vio_is_connected;
-      vio->timeout = vio_socket_timeout;
-      vio->has_data = vio->read_buffer ? vio_buff_has_data : has_no_data;
-      vio->is_blocking = vio_is_blocking;
-      vio->set_blocking = vio_set_blocking;
-      vio->set_blocking_flag = vio_set_blocking_flag;
-      vio->is_blocking_flag = true;
-      break;
+  if (type == VIO_TYPE_SSL) {
+    vio->viodelete = vio_ssl_delete;
+    vio->vioerrno = vio_errno;
+    vio->read = vio_ssl_read;
+    vio->write = vio_ssl_write;
+    vio->fastsend = vio_fastsend;
+    vio->viokeepalive = vio_keepalive;
+    vio->should_retry = vio_should_retry;
+    vio->was_timeout = vio_was_timeout;
+    vio->vioshutdown = vio_ssl_shutdown;
+    vio->peer_addr = vio_peer_addr;
+    vio->io_wait = vio_io_wait;
+    vio->is_connected = vio_is_connected;
+    vio->has_data = vio_ssl_has_data;
+    vio->timeout = vio_socket_timeout;
+    vio->is_blocking = vio_is_blocking;
+    vio->set_blocking = vio_set_blocking;
+    vio->set_blocking_flag = vio_set_blocking_flag;
+    vio->is_blocking_flag = true;
+    return false;
   }
-
-  DBUG_EXECUTE_IF("vio_init_returns_error", { return true; });
-
-#ifdef HAVE_KQUEUE
-  assert(type == VIO_TYPE_TCPIP || type == VIO_TYPE_SOCKET ||
-         type == VIO_TYPE_SSL);
-  vio->kq_fd = kqueue();
-  if (vio->kq_fd == -1) {
-    DBUG_PRINT("vio_init", ("kqueue failed with errno: %d", errno));
-    return true;
-  }
-#endif
+  vio->viodelete = vio_delete;
+  vio->vioerrno = vio_errno;
+  vio->read = vio->read_buffer ? vio_read_buff : vio_read;
+  vio->write = vio_write;
+  vio->fastsend = vio_fastsend;
+  vio->viokeepalive = vio_keepalive;
+  vio->should_retry = vio_should_retry;
+  vio->was_timeout = vio_was_timeout;
+  vio->vioshutdown = vio_shutdown;
+  vio->peer_addr = vio_peer_addr;
+  vio->io_wait = vio_io_wait;
+  vio->is_connected = vio_is_connected;
+  vio->timeout = vio_socket_timeout;
+  vio->has_data = vio->read_buffer ? vio_buff_has_data : has_no_data;
+  vio->is_blocking = vio_is_blocking;
+  vio->set_blocking = vio_set_blocking;
+  vio->set_blocking_flag = vio_set_blocking_flag;
+  vio->is_blocking_flag = true;
 
   return false;
 }
@@ -351,22 +340,16 @@ static bool vio_init(Vio *vio, enum enum_vio_type type, my_socket sd,
 */
 
 bool vio_reset(Vio *vio, enum enum_vio_type type, my_socket sd,
-               void *ssl [[maybe_unused]], uint flags) {
+               void *ssl MY_ATTRIBUTE((unused)), uint flags) {
   int ret = false;
   Vio new_vio(flags);
   DBUG_TRACE;
 
   /* The only supported rebind is from a socket-based transport type. */
-  assert(vio->type == VIO_TYPE_TCPIP || vio->type == VIO_TYPE_SOCKET);
+  DBUG_ASSERT(vio->type == VIO_TYPE_TCPIP || vio->type == VIO_TYPE_SOCKET);
 
   if (vio_init(&new_vio, type, sd, flags)) return true;
 
-#ifdef USE_PPOLL_IN_VIO
-  /* Preserve thread_id & signal_mask for this connection */
-  new_vio.thread_id = vio->thread_id;
-  new_vio.signal_mask = vio->signal_mask;
-#endif /* USE_PPOLL_IN_VIO */
-
   /* Preserve perfschema info for this connection */
   new_vio.mysql_socket.m_psi = vio->mysql_socket.m_psi;
 
@@ -428,7 +411,7 @@ Vio *mysql_socket_vio_new(MYSQL_SOCKET mysql_socket, enum_vio_type type,
   Vio *vio;
   my_socket sd = mysql_socket_getfd(mysql_socket);
   DBUG_TRACE;
-  DBUG_PRINT("enter", ("sd: " MY_SOCKET_FMT, sd));
+  DBUG_PRINT("enter", ("sd: %d", sd));
 
   if ((vio = internal_vio_create(flags))) {
     if (vio_init(vio, type, sd, flags)) {
@@ -446,7 +429,7 @@ Vio *vio_new(my_socket sd, enum enum_vio_type type, uint flags) {
   Vio *vio;
   MYSQL_SOCKET mysql_socket = MYSQL_INVALID_SOCKET;
   DBUG_TRACE;
-  DBUG_PRINT("enter", ("sd: " MY_SOCKET_FMT, sd));
+  DBUG_PRINT("enter", ("sd: %d", sd));
 
   mysql_socket_setfd(&mysql_socket, sd);
   vio = mysql_socket_vio_new(mysql_socket, type, flags);
@@ -466,10 +449,10 @@ Vio *vio_new_win32pipe(HANDLE hPipe) {
     }
 
     /* Create an object for event notification. */
-    vio->overlapped.hEvent = CreateEvent(nullptr, false, false, nullptr);
-    if (vio->overlapped.hEvent == nullptr) {
+    vio->overlapped.hEvent = CreateEvent(NULL, false, false, NULL);
+    if (vio->overlapped.hEvent == NULL) {
       internal_vio_delete(vio);
-      return nullptr;
+      return NULL;
     }
     vio->hPipe = hPipe;
   }
@@ -558,7 +541,7 @@ void vio_delete(Vio *vio) { internal_vio_delete(vio); }
   components below it when application finish
 
 */
-void vio_end() { vio_ssl_end(); }
+void vio_end(void) { vio_ssl_end(); }
 
 struct vio_string {
   const char *m_str;
@@ -590,4 +573,5 @@ void get_vio_type_name(enum enum_vio_type vio_type, const char **str,
   }
   *str = vio_type_names[index].m_str;
   *len = vio_type_names[index].m_len;
+  return;
 }
