@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2001, 2022, Oracle and/or its affiliates.
+   Copyright (c) 2001, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -30,7 +30,7 @@
   Note that we can't have assertion on file descriptors;  The reason for
   this is that during mysql shutdown, another thread can close a file
   we are working on.  In this case we should just return read errors from
-  the file descriptor.
+  the file descriptior.
 */
 
 #include "my_config.h"
@@ -70,7 +70,7 @@
 
 #include "mysql/psi/mysql_socket.h"
 
-int vio_errno(Vio *vio [[maybe_unused]]) {
+int vio_errno(Vio *vio MY_ATTRIBUTE((unused))) {
 /* These transport types are not Winsock based. */
 #ifdef _WIN32
   if (vio->type == VIO_TYPE_NAMEDPIPE || vio->type == VIO_TYPE_SHARED_MEMORY)
@@ -93,7 +93,7 @@ int vio_errno(Vio *vio [[maybe_unused]]) {
 int vio_socket_io_wait(Vio *vio, enum enum_vio_io_event event) {
   int timeout, ret;
 
-  assert(event == VIO_IO_EVENT_READ || event == VIO_IO_EVENT_WRITE);
+  DBUG_ASSERT(event == VIO_IO_EVENT_READ || event == VIO_IO_EVENT_WRITE);
 
   /* Choose an appropriate timeout. */
   if (event == VIO_IO_EVENT_READ)
@@ -138,7 +138,7 @@ size_t vio_read(Vio *vio, uchar *buf, size_t size) {
   DBUG_TRACE;
 
   /* Ensure nobody uses vio_read_buff and vio_read simultaneously. */
-  assert(vio->read_end == vio->read_pos);
+  DBUG_ASSERT(vio->read_end == vio->read_pos);
 
   /* If timeout is enabled, do not block if data is unavailable. */
   if (vio->read_timeout >= 0) flags = VIO_DONTWAIT;
@@ -158,7 +158,7 @@ size_t vio_read(Vio *vio, uchar *buf, size_t size) {
       Nonblocking with either EAGAIN or EWOULDBLOCK. Don't call
       io_wait. 0 bytes are available.
     */
-    assert(error == SOCKET_EAGAIN || error == SOCKET_EWOULDBLOCK);
+    DBUG_ASSERT(error == SOCKET_EAGAIN || error == SOCKET_EWOULDBLOCK);
     if (!vio_is_blocking(vio)) {
       DBUG_PRINT("info", ("vio_read on nonblocking socket read no bytes"));
       return -1;
@@ -292,11 +292,6 @@ int vio_set_blocking_flag(Vio *vio, bool status) {
       VIO_TYPE_SSL == vio->type) {
     vio->is_blocking_flag = status;
     ret = vio_set_blocking(vio, status);
-  } else {
-    DBUG_PRINT("warning", ("Connection type %d is not supported for "
-                           "asynchronous communication protocol",
-                           vio->type));
-    ret = -1;
   }
   return ret;
 }
@@ -306,12 +301,12 @@ bool vio_is_blocking(Vio *vio) {
   return vio->is_blocking_flag;
 }
 
-int vio_socket_timeout(Vio *vio, uint which [[maybe_unused]], bool old_mode) {
+int vio_socket_timeout(Vio *vio, uint which MY_ATTRIBUTE((unused)),
+                       bool old_mode) {
   int ret = 0;
   DBUG_TRACE;
 
 #if defined(_WIN32)
-  (void)old_mode;  // maybe_unused
   {
     int optname;
     DWORD timeout = 0;
@@ -327,8 +322,8 @@ int vio_socket_timeout(Vio *vio, uint which [[maybe_unused]], bool old_mode) {
 
       Assert that the VIO timeout is either positive or set to infinite.
     */
-    assert(which || vio->read_timeout);
-    assert(!which || vio->write_timeout);
+    DBUG_ASSERT(which || vio->read_timeout);
+    DBUG_ASSERT(!which || vio->write_timeout);
 
     if (which) {
       optname = SO_SNDTIMEO;
@@ -463,10 +458,10 @@ int vio_shutdown(Vio *vio) {
   DBUG_TRACE;
 
   if (vio->inactive == false) {
-    assert(vio->type == VIO_TYPE_TCPIP || vio->type == VIO_TYPE_SOCKET ||
-           vio->type == VIO_TYPE_SSL);
+    DBUG_ASSERT(vio->type == VIO_TYPE_TCPIP || vio->type == VIO_TYPE_SOCKET ||
+                vio->type == VIO_TYPE_SSL);
 
-    assert(mysql_socket_getfd(vio->mysql_socket) >= 0);
+    DBUG_ASSERT(mysql_socket_getfd(vio->mysql_socket) >= 0);
     if (mysql_socket_shutdown(vio->mysql_socket, SHUT_RDWR)) r = -1;
 
 #ifdef USE_PPOLL_IN_VIO
@@ -498,18 +493,11 @@ int vio_shutdown(Vio *vio) {
   return r;
 }
 
-#ifndef NDEBUG
-
-#ifdef _WIN32
-#define SOCKET_PRINTF_FORMAT "%llu"
-#else
-#define SOCKET_PRINTF_FORMAT "%d"
-#endif
-
+#ifndef DBUG_OFF
 void vio_description(Vio *vio, char *buf) {
   switch (vio->type) {
     case VIO_TYPE_SOCKET:
-      snprintf(buf, VIO_DESCRIPTION_SIZE, "socket (" SOCKET_PRINTF_FORMAT ")",
+      snprintf(buf, VIO_DESCRIPTION_SIZE, "socket (%d)",
                mysql_socket_getfd(vio->mysql_socket));
       break;
 #ifdef _WIN32
@@ -521,12 +509,12 @@ void vio_description(Vio *vio, char *buf) {
       break;
 #endif
     default:
-      snprintf(buf, VIO_DESCRIPTION_SIZE, "socket (" SOCKET_PRINTF_FORMAT ")",
+      snprintf(buf, VIO_DESCRIPTION_SIZE, "TCP/IP (%d)",
                mysql_socket_getfd(vio->mysql_socket));
       break;
   }
 }
-#endif  // NDEBUG
+#endif  // DBUG_OFF
 
 enum enum_vio_type vio_type(const Vio *vio) { return vio->type; }
 
@@ -564,13 +552,13 @@ static void vio_get_normalized_ip(const struct sockaddr *src, size_t src_length,
       break;
 
     case AF_INET6: {
-      const auto *src_addr6 = (const struct sockaddr_in6 *)src;
+      const struct sockaddr_in6 *src_addr6 = (const struct sockaddr_in6 *)src;
       const struct in6_addr *src_ip6 = &(src_addr6->sin6_addr);
       const uint32 *src_ip6_int32 =
           pointer_cast<const uint32 *>(src_ip6->s6_addr);
 
       if (IN6_IS_ADDR_V4MAPPED(src_ip6) || IN6_IS_ADDR_V4COMPAT(src_ip6)) {
-        auto *dst_ip4 = (struct sockaddr_in *)dst;
+        struct sockaddr_in *dst_ip4 = (struct sockaddr_in *)dst;
 
         /*
           This is an IPv4-mapped or IPv4-compatible IPv6 address. It should
@@ -625,8 +613,8 @@ static void vio_get_normalized_ip(const struct sockaddr *src, size_t src_length,
 bool vio_get_normalized_ip_string(const struct sockaddr *addr,
                                   size_t addr_length, char *ip_string,
                                   size_t ip_string_size) {
-  struct sockaddr_storage norm_addr_storage {};
-  auto *norm_addr = (struct sockaddr *)&norm_addr_storage;
+  struct sockaddr_storage norm_addr_storage;
+  struct sockaddr *norm_addr = (struct sockaddr *)&norm_addr_storage;
   size_t norm_addr_length;
   int err_code;
 
@@ -678,8 +666,8 @@ bool vio_peer_addr(Vio *vio, char *ip_buffer, uint16 *port,
     int err_code;
     char port_buffer[NI_MAXSERV];
 
-    struct sockaddr_storage addr_storage {};
-    auto *addr = (struct sockaddr *)&addr_storage;
+    struct sockaddr_storage addr_storage;
+    struct sockaddr *addr = (struct sockaddr *)&addr_storage;
     socket_len_t addr_length = sizeof(addr_storage);
 
     /* Get sockaddr by socked fd. */
@@ -791,7 +779,7 @@ static bool socket_peek_read(Vio *vio, uint *bytes) {
 int vio_io_wait(Vio *vio, enum enum_vio_io_event event, int timeout) {
   int ret;
   int retry_count = 0;
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   short revents = 0;
 #endif
   struct pollfd pfd;
@@ -810,14 +798,14 @@ int vio_io_wait(Vio *vio, enum enum_vio_io_event event, int timeout) {
   switch (event) {
     case VIO_IO_EVENT_READ:
       pfd.events = MY_POLL_SET_IN;
-#ifndef NDEBUG
+#ifndef DBUG_OFF
       revents = MY_POLL_SET_IN | MY_POLL_SET_ERR | POLLRDHUP;
 #endif
       break;
     case VIO_IO_EVENT_WRITE:
     case VIO_IO_EVENT_CONNECT:
       pfd.events = MY_POLL_SET_OUT;
-#ifndef NDEBUG
+#ifndef DBUG_OFF
       revents = MY_POLL_SET_OUT | MY_POLL_SET_ERR;
 #endif
       break;
@@ -875,7 +863,7 @@ int vio_io_wait(Vio *vio, enum enum_vio_io_event event, int timeout) {
       break;
     default:
       /* Ensure that the requested I/O event has completed. */
-      assert(pfd.revents & revents);
+      DBUG_ASSERT(pfd.revents & revents);
       break;
   }
 
@@ -955,7 +943,7 @@ int vio_io_wait(Vio *vio, enum enum_vio_io_event event, int timeout) {
   ret |= (FD_ISSET(fd, &exceptfds) ? 1 : 0);
 
   /* Not a timeout, ensure that a condition was met. */
-  assert(ret);
+  DBUG_ASSERT(ret);
 
   return ret;
 }
@@ -1013,8 +1001,9 @@ int vio_io_wait(Vio *vio, enum enum_vio_io_event event, int timeout) {
     for (int i = 0; i < nev; i++) {
       if (!(kev_event[i].flags & (EV_ERROR | EV_EOF))) {
         // Ensure that the requested I/O event has completed.
-        assert(event == VIO_IO_EVENT_READ ? kev_event[i].filter & EVFILT_READ
-                                          : kev_event[i].filter & EVFILT_WRITE);
+        DBUG_ASSERT(event == VIO_IO_EVENT_READ
+                        ? kev_event[i].filter & EVFILT_READ
+                        : kev_event[i].filter & EVFILT_WRITE);
       }
 
       // Shutdown or kill in progress, indicate error.
@@ -1054,7 +1043,7 @@ bool vio_socket_connect(Vio *vio, struct sockaddr *addr, socklen_t len,
   DBUG_TRACE;
 
   /* Only for socket-based transport types. */
-  assert(vio->type == VIO_TYPE_SOCKET || vio->type == VIO_TYPE_TCPIP);
+  DBUG_ASSERT(vio->type == VIO_TYPE_SOCKET || vio->type == VIO_TYPE_TCPIP);
 
   /* If timeout is not infinite, set socket to non-blocking mode. */
   if (((timeout > -1) || nonblocking) && vio_set_blocking(vio, false))
@@ -1123,8 +1112,9 @@ bool vio_socket_connect(Vio *vio, struct sockaddr *addr, socklen_t len,
   if (nonblocking && wait) {
     if (connect_done) *connect_done = false;
     return false;
+  } else {
+    return (ret != 0);
   }
-  return (ret != 0);
 }
 
 /**
@@ -1167,10 +1157,10 @@ bool vio_is_connected(Vio *vio) {
   if (!bytes && vio->type == VIO_TYPE_SSL)
     bytes = SSL_pending((SSL *)vio->ssl_arg);
 
-  return bytes != 0;
+  return bytes ? true : false;
 }
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
 
 /**
   Number of bytes in the read or socket buffer
