@@ -1,16 +1,15 @@
-/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    Without limiting anything contained in the foregoing, this file,
    which is part of C Driver for MySQL (Connector/C), is also subject to the
@@ -30,7 +29,7 @@
   Note that we can't have assertion on file descriptors;  The reason for
   this is that during mysql shutdown, another thread can close a file
   we are working on.  In this case we should just return read errors from
-  the file descriptor.
+  the file descriptior.
 */
 
 #include <errno.h>
@@ -59,7 +58,7 @@
   after the data is compressed right before the data is written to
   the network layer.
 
-  The TLS support is announced in
+  The TLS suppport is announced in
   @ref page_protocol_connection_phase_packets_protocol_handshake sent by the
   server via ::CLIENT_SSL and is enabled if the client returns the same
   capability.
@@ -94,7 +93,7 @@
   00 00 00 00                                           ....
   ~~~~~~~
 
-  Then the rest of the communication is switched to TLS:
+  Then the rest of the communication is swithed to TLS:
   ~~~~~~~
   16 03 01 00 5e 01 00 00    5a 03 01 4c a3 49 2e 7a    ....^...Z..L.I.z
   b5 06 75 68 5c 30 36 73    f1 82 79 70 58 4c 64 bb    ..uh\06s..ypXLd.
@@ -119,7 +118,7 @@
 */
 /* clang-format on */
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
 
 static void report_errors(SSL *ssl) {
   unsigned long l;
@@ -130,11 +129,7 @@ static void report_errors(SSL *ssl) {
 
   DBUG_TRACE;
 
-#if OPENSSL_VERSION_NUMBER >= 0x30000000L
-  while ((l = ERR_get_error_all(&file, &line, nullptr, &data, &flags))) {
-#else  /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
   while ((l = ERR_get_error_line_data(&file, &line, &data, &flags))) {
-#endif /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
     DBUG_PRINT("error", ("OpenSSL: %s:%s:%d:%s\n", ERR_error_string(l, buf),
                          file, line, (flags & ERR_TXT_STRING) ? data : ""));
   }
@@ -244,7 +239,7 @@ static bool ssl_should_retry(Vio *vio, int ret, enum enum_vio_io_event *event,
       /* first save the top ERR error */
       err_error = ERR_get_error();
       /* now report all remaining errors on and/or clear the error stack */
-#ifndef NDEBUG /* Debug build */
+#ifndef DBUG_OFF /* Debug build */
       /* Note: the OpenSSL error queue gets cleared in report_errors(). */
       report_errors(ssl);
 #else /* Release build */
@@ -275,11 +270,11 @@ size_t vio_ssl_read(Vio *vio, uchar *buf, size_t size) {
       SSL_read() returns an error from the error queue, when SSL_read() failed
       because it would block.
     */
-    assert(ERR_peek_error() == 0);
+    DBUG_ASSERT(ERR_peek_error() == 0);
 
     ret = SSL_read(ssl, buf, (int)size);
 
-    if (ret > 0) break;
+    if (ret >= 0) break;
 
     /* Process the SSL I/O error. */
     if (!ssl_should_retry(vio, ret, &event, &ssl_errno_not_used)) break;
@@ -317,11 +312,11 @@ size_t vio_ssl_write(Vio *vio, const uchar *buf, size_t size) {
       SSL_write() returns an error from the error queue, when SSL_write() failed
       because it would block.
     */
-    assert(ERR_peek_error() == 0);
+    DBUG_ASSERT(ERR_peek_error() == 0);
 
     ret = SSL_write(ssl, buf, (int)size);
 
-    if (ret > 0) break;
+    if (ret >= 0) break;
 
     /* Process the SSL I/O error. */
     if (!ssl_should_retry(vio, ret, &event, &ssl_errno_not_used)) break;
@@ -355,7 +350,7 @@ int vio_ssl_shutdown(Vio *vio) {
     alert on socket shutdown to avoid truncation attacks. However, this can
     cause problems since we often hold a lock during shutdown and this IO can
     take an unbounded amount of time to complete. Since our packets are self
-    describing with length, we aren't vulnerable to these attacks. Therefore,
+    describing with length, we aren't vunerable to these attacks. Therefore,
     we just shutdown by closing the socket (quiet shutdown).
     */
     SSL_set_quiet_shutdown(ssl, 1);
@@ -374,12 +369,6 @@ int vio_ssl_shutdown(Vio *vio) {
       default: /* Shutdown failed */
         DBUG_PRINT("vio_error",
                    ("SSL_shutdown() failed, error: %d", SSL_get_error(ssl, r)));
-#ifndef NDEBUG /* Debug build */
-        /* Note: the OpenSSL error queue gets cleared in report_errors(). */
-        report_errors(ssl);
-#else /* Release build */
-        ERR_clear_error();
-#endif
         break;
     }
   }
@@ -398,7 +387,7 @@ void vio_ssl_delete(Vio *vio) {
   }
 
 #if OPENSSL_VERSION_NUMBER < 0x10100000L
-  ERR_remove_thread_state(nullptr);
+  ERR_remove_thread_state(0);
 #endif /* OPENSSL_VERSION_NUMBER < 0x10100000L */
 
   vio_delete(vio);
@@ -436,7 +425,7 @@ static size_t ssl_handshake_loop(Vio *vio, SSL *ssl, ssl_handshake_func_t func,
       SSL-handshake-function returns an error from the error queue, when the
       function failed because it would block.
     */
-    assert(ERR_peek_error() == 0);
+    DBUG_ASSERT(ERR_peek_error() == 0);
 
     int handshake_ret;
     handshake_ret = func(ssl);
@@ -449,11 +438,6 @@ static size_t ssl_handshake_loop(Vio *vio, SSL *ssl, ssl_handshake_func_t func,
     /* Process the SSL I/O error. */
     if (!ssl_should_retry(vio, handshake_ret, &event, ssl_errno_holder)) break;
 
-    DBUG_EXECUTE_IF("bug32372038", {
-      DBUG_SET("+d,bug32372038_ssl_started");
-      return VIO_SOCKET_WANT_READ;
-    };);
-
     if (!vio->is_blocking_flag) {
       switch (event) {
         case VIO_IO_EVENT_READ:
@@ -509,7 +493,7 @@ long pfs_ssl_bio_callback_ex(BIO *b, int oper, const char * /* argp */,
   switch (oper) {
     case BIO_CB_READ:
       vio = reinterpret_cast<Vio *>(BIO_get_callback_arg(b));
-      assert(vio->m_psi_read_locker == nullptr);
+      DBUG_ASSERT(vio->m_psi_read_locker == nullptr);
       if (vio->mysql_socket.m_psi != nullptr) {
         vio->m_psi_read_locker = PSI_SOCKET_CALL(start_socket_wait)(
             &vio->m_psi_read_state, vio->mysql_socket.m_psi, PSI_SOCKET_RECV,
@@ -525,7 +509,7 @@ long pfs_ssl_bio_callback_ex(BIO *b, int oper, const char * /* argp */,
       break;
     case BIO_CB_WRITE:
       vio = reinterpret_cast<Vio *>(BIO_get_callback_arg(b));
-      assert(vio->m_psi_write_locker == nullptr);
+      DBUG_ASSERT(vio->m_psi_write_locker == nullptr);
       if (vio->mysql_socket.m_psi != nullptr) {
         vio->m_psi_write_locker = PSI_SOCKET_CALL(start_socket_wait)(
             &vio->m_psi_write_state, vio->mysql_socket.m_psi, PSI_SOCKET_SEND,
@@ -549,7 +533,7 @@ long pfs_ssl_bio_callback_ex(BIO *b, int oper, const char * /* argp */,
     case BIO_CB_GETS:
     case BIO_CB_GETS | BIO_CB_RETURN:
     default:
-      assert(false);
+      DBUG_ASSERT(false);
   }
 
   return ret;
@@ -583,15 +567,15 @@ long pfs_ssl_bio_callback(BIO *b, int oper, const char *argp, int argi,
 #ifdef HAVE_PSI_SOCKET_INTERFACE
 static void pfs_ssl_setup_instrumentation(Vio *vio, const SSL *ssl) {
   BIO *rbio = SSL_get_rbio(ssl);
-  assert(rbio != nullptr);
-  assert(BIO_method_type(rbio) == BIO_TYPE_SOCKET);
+  DBUG_ASSERT(rbio != nullptr);
+  DBUG_ASSERT(BIO_method_type(rbio) == BIO_TYPE_SOCKET);
 
   BIO *wbio = SSL_get_wbio(ssl);
-  assert(wbio != nullptr);
-  assert(BIO_method_type(wbio) == BIO_TYPE_SOCKET);
+  DBUG_ASSERT(wbio != nullptr);
+  DBUG_ASSERT(BIO_method_type(wbio) == BIO_TYPE_SOCKET);
 
   char *cb_arg = reinterpret_cast<char *>(vio);
-  assert(cb_arg != nullptr);
+  DBUG_ASSERT(cb_arg != nullptr);
 
   BIO_set_callback_arg(rbio, cb_arg);
 
@@ -613,39 +597,19 @@ static void pfs_ssl_setup_instrumentation(Vio *vio, const SSL *ssl) {
 }
 #endif /* HAVE_PSI_SOCKET_INTERFACE */
 
-#ifndef NDEBUG
-static void print_ssl_session_id(SSL_SESSION *sess, const char *action) {
-  unsigned int length;
-  const unsigned char *id = SSL_SESSION_get_id(sess, &length);
-  char tohex[1024], *w = &tohex[0];
-
-  if (id && length) {
-    for (unsigned inx = 0; inx < length && w < &tohex[1021]; inx++) {
-      sprintf(w, "%2.2X", id[inx]);
-      w += 2;
-    }
-    *w = 0;
-  } else
-    strcpy(tohex, "<empty>");
-  DBUG_PRINT("info", ("%sSSL session ID [%s]", action, tohex));
-}
-#endif  // !NDEBUG
-
 static int ssl_do(struct st_VioSSLFd *ptr, Vio *vio, long timeout,
-                  SSL_SESSION *ssl_session, ssl_handshake_func_t func,
-                  unsigned long *ssl_errno_holder, SSL **sslptr,
-                  const char *sni_servername) {
+                  ssl_handshake_func_t func, unsigned long *ssl_errno_holder,
+                  SSL **sslptr) {
   SSL *ssl = nullptr;
   my_socket sd = mysql_socket_getfd(vio->mysql_socket);
 
   /* Declared here to make compiler happy */
-#if !defined(NDEBUG)
+#if !defined(DBUG_OFF)
   int j, n;
 #endif
 
   DBUG_TRACE;
-  DBUG_PRINT("enter", ("ptr: %p, sd: " MY_SOCKET_FMT "  ctx: %p", ptr, sd,
-                       ptr->ssl_context));
+  DBUG_PRINT("enter", ("ptr: %p, sd: %d  ctx: %p", ptr, sd, ptr->ssl_context));
 
   if (!sslptr) {
     sslptr = &ssl;
@@ -658,35 +622,6 @@ static int ssl_do(struct st_VioSSLFd *ptr, Vio *vio, long timeout,
       return 1;
     }
 
-    if (ssl_session != nullptr) {
-#ifndef NDEBUG
-      print_ssl_session_id(ssl_session, "Setting for reuse ");
-#endif
-      if (!SSL_set_session(ssl, ssl_session)) {
-#ifndef NDEBUG
-        DBUG_PRINT("error", ("SSL_set_session failed"));
-        report_errors(ssl);
-#endif
-        ERR_clear_error();
-      } else {
-        DBUG_PRINT("info", ("reused existing session"));
-      }
-    }
-
-    if (sni_servername) {
-// Check if the openssl supports SNI, and only then set the tlsext_host_name
-#if !defined(OPENSSL_NO_TLSEXT)
-      if (!SSL_set_tlsext_host_name(ssl, const_cast<char *>(sni_servername))) {
-        DBUG_PRINT("error", ("SSL_set_tlsext_host_name failure"));
-        *ssl_errno_holder = ERR_get_error();
-        return 1;
-      }
-#else
-      *ssl_errno_holder = 0;
-      return 1;
-#endif /* !def(OPENSSL_NO_TLSEXT) */
-    }
-
     DBUG_PRINT("info", ("ssl: %p timeout: %ld", ssl, timeout));
     SSL_clear(ssl);
     SSL_SESSION_set_timeout(SSL_get_session(ssl), timeout);
@@ -697,7 +632,7 @@ static int ssl_do(struct st_VioSSLFd *ptr, Vio *vio, long timeout,
     sk_SSL_COMP_zero(SSL_COMP_get_compression_methods());
 #endif
 
-#if !defined(NDEBUG)
+#if !defined(DBUG_OFF)
     {
       STACK_OF(SSL_COMP) *ssl_comp_methods = nullptr;
       ssl_comp_methods = SSL_COMP_get_compression_methods();
@@ -727,7 +662,6 @@ static int ssl_do(struct st_VioSSLFd *ptr, Vio *vio, long timeout,
   } else {
     ssl = *sslptr;
   }
-  ERR_clear_error();
 
   size_t loop_ret;
   if ((loop_ret = ssl_handshake_loop(vio, ssl, func, ssl_errno_holder))) {
@@ -740,9 +674,6 @@ static int ssl_do(struct st_VioSSLFd *ptr, Vio *vio, long timeout,
     *sslptr = nullptr;
     return (int)VIO_SOCKET_ERROR;
   }
-#ifndef NDEBUG
-  print_ssl_session_id(SSL_get_session(ssl), "Connected ");
-#endif
 
   /*
     Connection succeeded. Install new function handlers,
@@ -754,7 +685,7 @@ static int ssl_do(struct st_VioSSLFd *ptr, Vio *vio, long timeout,
     *sslptr = nullptr;
   }
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   {
     /* Print some info about the peer */
     X509 *cert;
@@ -786,17 +717,14 @@ static int ssl_do(struct st_VioSSLFd *ptr, Vio *vio, long timeout,
 int sslaccept(struct st_VioSSLFd *ptr, Vio *vio, long timeout,
               unsigned long *ssl_errno_holder) {
   DBUG_TRACE;
-  int ret = ssl_do(ptr, vio, timeout, nullptr, SSL_accept, ssl_errno_holder,
-                   nullptr, nullptr);
+  int ret = ssl_do(ptr, vio, timeout, SSL_accept, ssl_errno_holder, nullptr);
   return ret;
 }
 
 int sslconnect(struct st_VioSSLFd *ptr, Vio *vio, long timeout,
-               SSL_SESSION *session, unsigned long *ssl_errno_holder, SSL **ssl,
-               const char *sni_servername) {
+               unsigned long *ssl_errno_holder, SSL **ssl) {
   DBUG_TRACE;
-  int ret = ssl_do(ptr, vio, timeout, session, SSL_connect, ssl_errno_holder,
-                   ssl, sni_servername);
+  int ret = ssl_do(ptr, vio, timeout, SSL_connect, ssl_errno_holder, ssl);
   return ret;
 }
 
